<?xml version="1.0" encoding="utf-8"?>

<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:fo="http://www.w3.org/1999/XSL/Format">
    <xsl:output method="xml" indent="yes"/>
    <xsl:param name="balance_history_empty"/>
    <xsl:param name="is_show_in_hours"/>
    <xsl:template match="BalanceHistory">
        <xsl:param name="page-count"/>
        <fo:root font-family="Charis">
            <fo:layout-master-set>
                <fo:simple-page-master master-name="Letter"
                                       page-height="279.4mm"
                                       page-width="215.9mm"
                                       keep-with-next="always"
                                       margin-top="1cm"
                                       margin-bottom="2cm"
                                       margin-left="2cm"
                                       margin-right="2cm">
                    <fo:region-body/>
                    <fo:region-after/>
                </fo:simple-page-master>
            </fo:layout-master-set>
            <fo:page-sequence master-reference="Letter" initial-page-number="1">
                <fo:static-content flow-name="xsl-region-after" text-align="right" font-size="8pt">
                    <fo:block>
                        <xsl:if test="$page-count &gt; 1">
                            Page
                            <fo:page-number/>
                            of
                            <xsl:value-of select="$page-count"/>
                        </xsl:if>
                    </fo:block>
                </fo:static-content>

                <fo:flow flow-name="xsl-region-body">
                    <fo:block padding-after="1pt" font-size="16pt" font-weight="bold">
                                    <xsl:apply-templates select="./@type"/> Balance History
                    </fo:block>
                    <xsl:variable name="leaderLength">
                        <xsl:apply-templates select="./@type" mode="leaderLength"/>
                    </xsl:variable>
                                    <fo:block padding-after="7pt" padding-left="10pt">
                                        <fo:leader leader-length="{$leaderLength}"
                                                   leader-pattern="rule"
                                                   alignment-baseline="middle"
                                                   rule-thickness="0.5pt" color="black"/>
                                    </fo:block>
                                    <fo:block font-size="11pt"  >
                                        <fo:inline font-weight="bold" > User: &#x00A0;</fo:inline>
                                        <xsl:apply-templates select="./user" mode="user-info"/>
                                    </fo:block>
                                    <fo:block font-size="11pt" padding-after="15pt">
                                        <fo:inline font-weight="bold" > Date: &#x00A0;</fo:inline>
                                        <xsl:value-of select="./@date"/>
                                    </fo:block>
                    <fo:table  table-layout="fixed" width="100%"
                            border-style="solid"
                            border-width="1pt"
                               border-color="#CECECE" >
                        <fo:table-column column-width="90pt"   border-style="solid"
                                         border-width="1pt"
                                         border-color="#CECECE" />
                        <fo:table-column column-width="220pt"  border-style="solid"
                                         border-width="1pt"
                                         border-color="#CECECE"/>
                        <fo:table-column column-width="90pt"  border-style="solid"
                                         border-width="1pt"
                                         border-color="#CECECE"/>
                        <fo:table-column column-width="90pt"  border-style="solid"
                                         border-width="1pt"
                                         border-color="#CECECE"/>
                        <fo:table-header>
                            <fo:table-row height="20pt">
                                <fo:table-cell
                                        font-size="9pt"  padding="5px"
                                        display-align="center"
                                        font-weight="bold"
                                        background-color="#EBEBEB">
                                    <fo:block margin-left="10pt">Effective Date</fo:block>
                                </fo:table-cell>
                                <fo:table-cell
                                        font-weight="bold"
                                               font-size="9pt" padding="5px"
                                               display-align="center"
                                               background-color="#EBEBEB">
                                    <fo:block  margin-left="10pt">Description</fo:block>
                                </fo:table-cell>
                                <fo:table-cell
                                        font-weight="bold"
                                               font-size="9pt"  padding="5px" display-align="center"
                                               text-align="center"
                                               background-color="#EBEBEB">
                                    <fo:block>Value</fo:block>
                                </fo:table-cell>
                                <fo:table-cell
                                        font-weight="bold"
                                               font-size="9pt" padding="5px" display-align="center"
                                               text-align="center"
                                               background-color="#EBEBEB">
                                    <fo:block>Ending Balance</fo:block>
                                </fo:table-cell>
                            </fo:table-row>
                        </fo:table-header>
                        <fo:table-body>
                            <xsl:choose>
                            <xsl:when test="./LeaveTimeCompletedRecord |
                                            ./ManualIncreaseRecord |
                                            ./ManualSetRecord |
                                            ./ManualDecreaseRecord |
                                            ./AutomaticBalanceResetRecord |
                                            ./AutomaticBalanceLimitRecord |
                                            ./AutomaticBalanceAccrueRecord">
                                <xsl:apply-templates  mode="table-row"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:apply-templates select="."  mode="balance_history_empty"/>
                            </xsl:otherwise>
                            </xsl:choose>
                        </fo:table-body>
                    </fo:table>
                </fo:flow>
            </fo:page-sequence>
        </fo:root>
    </xsl:template>

    <xsl:template match="BalanceHistory/@type[../@type='SICK']">
         Sick Days
    </xsl:template>

    <xsl:template match="BalanceHistory/@type[../@type='PTO']">
         PTO
    </xsl:template>

    <xsl:template match="BalanceHistory/@type[../@type='PTO']" mode="leaderLength">
        158pt
    </xsl:template>

    <xsl:template match="BalanceHistory/@type[../@type='SICK']" mode="leaderLength">
        198pt
    </xsl:template>

    <xsl:template match="AutomaticBalanceAccrueRecord | AutomaticBalanceResetRecord | AutomaticBalanceLimitRecord
                | ManualSetRecord | ManualDecreaseRecord | ManualIncreaseRecord | LeaveTimeCompletedRecord" mode="table-row">
        <fo:table-row  border-style="solid"
                       border-width="1pt"
                       border-color="#CECECE">
            <fo:table-cell
                    font-size="9pt"
                    padding="5px"
                    display-align="center">
                <fo:block margin-left="10pt">
                    <xsl:value-of select="effectiveDate"/>
                </fo:block>
            </fo:table-cell>
            <fo:table-cell
                    padding="5px"
                    display-align="center">
                <fo:block-container overflow="hidden">
                    <xsl:apply-templates select="." mode="getDescription"/>
                </fo:block-container>
            </fo:table-cell>
            <fo:table-cell font-size="9pt"
                           padding="5px"
                           text-align="right"
                           display-align="center">
                <fo:block-container overflow="hidden">
                    <xsl:apply-templates select="." mode="getDifferenceValue"/>
                </fo:block-container>
            </fo:table-cell>
            <fo:table-cell
                    font-size="9pt"
                    padding="5px"
                    text-align="right"
                    display-align="center">
                <fo:block-container overflow="hidden">
                            <fo:block >
                                    <xsl:apply-templates select="."  mode="getEndingBalance"/>
                            </fo:block>
                </fo:block-container>
            </fo:table-cell>
        </fo:table-row>
    </xsl:template>

    <xsl:template  match="BalanceHistory" mode="balance_history_empty">
        <fo:table-row  border-style="solid"
                       border-width="1pt"
                       border-color="#CECECE"
                       height="30pt">
            <fo:table-cell display-align="center"
                           text-align="center"
                           number-columns-spanned="4">
                <fo:block font-size="9pt"> <xsl:value-of select="$balance_history_empty"/> </fo:block>
            </fo:table-cell>
        </fo:table-row>
    </xsl:template>

    <xsl:template match="AutomaticBalanceLimitRecord | AutomaticBalanceResetRecord | ManualSetRecord " mode="getDifferenceValue" >
       <fo:block>
         =  <xsl:apply-templates select="." mode="formatDifferenceValue"/>
       </fo:block>
    </xsl:template>

    <xsl:template match="LeaveTimeCompletedRecord | ManualDecreaseRecord" mode="getDifferenceValue">
        <fo:block color="#E0401A">
            &#8722; <xsl:apply-templates select="." mode="formatDifferenceValue"/>
        </fo:block>
    </xsl:template>

    <xsl:template match="AutomaticBalanceAccrueRecord |  ManualIncreaseRecord" mode="getDifferenceValue">
        <fo:block color="#44916C">
            + <xsl:apply-templates select="." mode="formatDifferenceValue"/>
        </fo:block>
    </xsl:template>

    <xsl:template match="AutomaticBalanceAccrueRecord | AutomaticBalanceResetRecord | AutomaticBalanceLimitRecord
                | ManualSetRecord | ManualDecreaseRecord | ManualIncreaseRecord | LeaveTimeCompletedRecord" mode="formatDifferenceValue">
        <xsl:variable name="hours" select="translate(differenceValue/hours/string, '-', '&#8722;')"/>
        <xsl:variable name="days"  select="translate(differenceValue/days/string, '-', '&#8722;')"/>

        <xsl:if test="$is_show_in_hours ='true'">
            <xsl:value-of select="$hours"/>h (<xsl:value-of select="$days"/>d)
        </xsl:if>
        <xsl:if test="$is_show_in_hours ='false'">
            <xsl:value-of select="$days"/>d  (<xsl:value-of select="$hours"/>h)
        </xsl:if>
    </xsl:template>

    <xsl:template match="AutomaticBalanceAccrueRecord | AutomaticBalanceResetRecord | AutomaticBalanceLimitRecord
                | ManualSetRecord | ManualDecreaseRecord | ManualIncreaseRecord | LeaveTimeCompletedRecord" mode="getDescription" >
    <fo:table table-layout="fixed" width="100%">
        <fo:table-column column-width="25pt"/>
        <fo:table-column  column-width="170pt"/>
        <fo:table-body>
            <fo:table-row>
                <fo:table-cell display-align="center" padding-after="3pt">
                    <fo:block margin-left="10pt" font-size="10px" padding-before="3pt">
                        <xsl:apply-templates select="." mode="description-icon"/>  &#x00A0;
                    </fo:block>
                </fo:table-cell>
                <fo:table-cell display-align="center" padding-after="3pt">
                <fo:block font-size="10px" padding-before="3pt">
                    <xsl:apply-templates select="." mode="description-text"/>
                </fo:block>
                </fo:table-cell>

            </fo:table-row>
            <xsl:apply-templates select="." mode="comment-row"/>
        </fo:table-body>
    </fo:table>
</xsl:template>




    <xsl:template  match="AutomaticBalanceAccrueRecord | AutomaticBalanceResetRecord | AutomaticBalanceLimitRecord
                | ManualSetRecord | ManualDecreaseRecord | ManualIncreaseRecord | LeaveTimeCompletedRecord" mode="getEndingBalance">

        <xsl:if test="endingBalance/days/number > -0.1" >
        <fo:block>
                <xsl:apply-templates select="." mode="formatEndingBalance"/>
        </fo:block>
        </xsl:if>
        <xsl:if test="-0.1 >= endingBalance/days/number ">
            <fo:block color="#E0401A">
                <xsl:apply-templates select="." mode="formatEndingBalance"/>
            </fo:block>
        </xsl:if>
    </xsl:template>

    <xsl:template match="AutomaticBalanceAccrueRecord | AutomaticBalanceResetRecord | AutomaticBalanceLimitRecord
                | ManualSetRecord | ManualDecreaseRecord | ManualIncreaseRecord | LeaveTimeCompletedRecord" mode="formatEndingBalance">
        <xsl:variable name="hours" select="translate(endingBalance/hours/string, '-', '&#8722;')"/>
        <xsl:variable name="days"  select="translate(endingBalance/days/string, '-', '&#8722;')"/>

        <xsl:if test="$is_show_in_hours ='true'">
            <fo:inline font-weight="bold" > <xsl:value-of select="$hours"/>h </fo:inline> (<xsl:value-of select="$days"/>d)
        </xsl:if>
        <xsl:if test="$is_show_in_hours ='false'">
            <fo:inline font-weight="bold" > <xsl:value-of select="$days"/>d  </fo:inline> (<xsl:value-of select="$hours"/>h)
        </xsl:if>
    </xsl:template>


    <xsl:template match="AutomaticBalanceAccrueRecord" mode="description-text">
        Automatic Accrual
    </xsl:template>

    <xsl:template match="AutomaticBalanceLimitRecord" mode="description-text">
        Automatic <xsl:apply-templates select="../@type"/> Limit
    </xsl:template>

    <xsl:template match="AutomaticBalanceResetRecord" mode="description-text">
        Automatic <xsl:apply-templates select="../@type"/> Reset
    </xsl:template>

    <xsl:template match="LeaveTimeCompletedRecord" mode="description-text">
        <xsl:value-of select="LeaveType/@name"/> Completed
    </xsl:template>

    <xsl:template match="ManualDecreaseRecord" mode="description-text">
          Manual Adjustment: Decrease
    </xsl:template>

    <xsl:template match="ManualIncreaseRecord" mode="description-text">
        Manual Adjustment: Increase
    </xsl:template>

    <xsl:template match="ManualSetRecord" mode="description-text">
        Manual Adjustment: Hard Value
    </xsl:template>


    <xsl:template match="AutomaticBalanceLimitRecord | AutomaticBalanceAccrueRecord | AutomaticBalanceResetRecord" mode="description-icon">
        <fo:external-graphic content-width="8px" src="{'servlet-context:/img/default/auto-adjustment_print.png'}"/>
    </xsl:template>

    <xsl:template match="ManualIncreaseRecord | ManualDecreaseRecord | ManualSetRecord" mode="description-icon">
        <fo:external-graphic content-width="8px" src="{'servlet-context:/img/default/manual-adjustment_print.png'}"/>
    </xsl:template>

    <xsl:template match="LeaveTimeCompletedRecord" mode="description-icon">
        <fo:external-graphic content-width="8px" src="{concat('servlet-context:/img/default/leave_types/icons/leave_type_', LeaveType/@iconId, '_print.png')}"/>
    </xsl:template>


    <xsl:template match="ManualDecreaseRecord | ManualIncreaseRecord | ManualSetRecord" mode="comment-row">
       <xsl:call-template name="commentText">
           <xsl:with-param name="comment">
               by <xsl:apply-templates select="balanceManagerInfo/user" mode="user-info"/>
               <xsl:if test="./comment !=''">
               - <xsl:value-of select="comment"/>
               </xsl:if>
           </xsl:with-param>
       </xsl:call-template>
    </xsl:template>

    <xsl:template match="LeaveTimeCompletedRecord" mode="comment-row">
        <xsl:call-template name="commentText">
            <xsl:with-param name="comment">
              <xsl:value-of select="comment"/>
            </xsl:with-param>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="user" mode="user-info">
        <xsl:call-template name="formatUserName">
            <xsl:with-param name="firstName" select="@firstName"/>
            <xsl:with-param name="lastName" select="@lastName"/>
            <xsl:with-param name="middleInitial" select="@middleInitial"/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template name="commentText" match="ManualDecreaseRecord | ManualIncreaseRecord | ManualSetRecord | LeaveTimeCompletedRecord" mode="comment-text">
        <xsl:param name="comment"/>
        <xsl:if test="$comment != ''">
            <fo:table-row>
                <fo:table-cell><fo:block/></fo:table-cell>
                <fo:table-cell padding-left="6pt" font-size="8pt" padding-after="3pt" wrap-option="wrap">
                    <fo:block color="#717171">
                         <xsl:value-of select="$comment"/>
                    </fo:block>
                </fo:table-cell>
            </fo:table-row>
        </xsl:if>
    </xsl:template>

    <xsl:template name="formatUserName">
        <xsl:param name="firstName"/>
        <xsl:param name="lastName"/>
        <xsl:param name="middleInitial"/>
        <xsl:value-of select="$firstName"/>
        <xsl:if test="$middleInitial != ''">
            <xsl:value-of select="concat(' ', $middleInitial, '.')"/>
        </xsl:if>
        <xsl:value-of select="concat(' ', $lastName)"/>
    </xsl:template>

</xsl:stylesheet>