<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0"
      xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
      xmlns:fo="http://www.w3.org/1999/XSL/Format"
      xmlns:i="http://www.actitime.com/xsd/2.1/invoice.xsd">
  <xsl:output method="xml" indent="yes"/>
  <xsl:template match="i:invoice">
    <xsl:param name="page-count"/>
    <fo:root font-family="Charis">
      <fo:layout-master-set>
        <fo:simple-page-master master-name="Letter"
              page-height="279.4mm"
              page-width="215.9mm"
              margin-top="2cm"
              margin-bottom="2cm"
              margin-left="1.5cm"
              margin-right="1.5cm">
          <fo:region-body/>
          <fo:region-after/>
        </fo:simple-page-master>
      </fo:layout-master-set>
      <fo:page-sequence master-reference="Letter" initial-page-number="1">
        <fo:static-content flow-name="xsl-region-after" text-align="right" font-size="8pt">
            <fo:block>
                <xsl:if test="$page-count &gt; 1">
                    Page <fo:page-number/> of
                    <xsl:value-of select="$page-count"/>
                </xsl:if>
            </fo:block>
        </fo:static-content>

          <fo:flow flow-name="xsl-region-body" >
          <fo:table table-layout="fixed" width="100%">
              <fo:table-body>
                  <fo:table-row>
                      <fo:table-cell>

                          <xsl:if test="i:company/i:logoURI/text() != ''">
                            <fo:block padding-after="15pt">
                              <fo:external-graphic content-width="150px" src="{i:company/i:logoURI/text()}"/>
                            </fo:block>
                          </xsl:if>

                          <fo:block padding-after="15pt">
                              <fo:table table-layout="fixed" width="100%">
                                  <fo:table-column column-width="230pt" border="1pt solid white"/>
                                  <fo:table-column/>
                                  <fo:table-body>
                                      <fo:table-row>
                                          <fo:table-cell padding-left="6px">
                                              <fo:block font-weight="bold"
                                                        white-space-collapse="false"
                                                        white-space-treatment="preserve"
                                                        linefeed-treatment="preserve"
                                                        padding-after="10pt">
                                                  <xsl:value-of select="i:company/i:name/text()"/>
                                              </fo:block>
                                          </fo:table-cell>
                                          <fo:table-cell>
                                              <fo:block/>
                                          </fo:table-cell>
                                      </fo:table-row>
                                      <fo:table-row>
                                          <fo:table-cell padding-left="6px">
                                              <fo:block  white-space-collapse="false"
                                                         white-space-treatment="preserve"
                                                         linefeed-treatment="preserve"
                                                         font-size="9pt">
                                                  <xsl:value-of select="i:company/i:address/text()"/>
                                              </fo:block>
                                          </fo:table-cell>
                                          <fo:table-cell>
                                              <fo:block/>
                                          </fo:table-cell>
                                      </fo:table-row>
                                  </fo:table-body>
                              </fo:table>
                          </fo:block>

                      </fo:table-cell>
                      <fo:table-cell text-align="right">
                          <fo:block  padding-after="5pt" font-size="33pt"><xsl:value-of select="i:header/text()"/></fo:block>
                          <fo:block padding-after="8pt">
                              <fo:leader leader-length="181pt"
                                         leader-pattern="rule"
                                         alignment-baseline="middle"
                                         rule-thickness="0.5pt" color="black"/>
                          </fo:block>

                          <fo:table table-layout="fixed" width="100%" font-size="10pt">
                              <fo:table-column column-number="1"/>
                              <fo:table-column column-number="2" column-width="100pt"/>
                              <fo:table-body>
                                  <xsl:for-each select="i:invoiceFields/i:field">
                                      <fo:table-row>
                                          <fo:table-cell padding="2pt" font-weight="bold">
                                              <fo:block white-space-collapse="false"
                                                        white-space-treatment="preserve"
                                                        linefeed-treatment="preserve"><xsl:value-of select="@name"/>:</fo:block>
                                          </fo:table-cell>
                                          <fo:table-cell padding="2pt" text-align="left">
                                              <fo:block white-space-collapse="false"
                                                        white-space-treatment="preserve"
                                                        linefeed-treatment="preserve"><xsl:value-of select="text()"/></fo:block>
                                          </fo:table-cell>
                                      </fo:table-row>
                                  </xsl:for-each>
                              </fo:table-body>
                          </fo:table>


                      </fo:table-cell>
                  </fo:table-row>
                  <fo:table-row>
                    <fo:table-cell padding-top="15pt">
                        <fo:block>
                            <fo:table table-layout="fixed" width="100%" text-align="center">
                                <fo:table-column column-width="230pt"/>
                                <fo:table-column/>
                                <fo:table-body>
                                    <fo:table-row>
                                        <fo:table-cell border-top-style="solid" border-top-width="1pt"
                                                       border-left-style="solid" border-left-width="1pt"
                                                       border-right-style="solid" border-right-width="1pt"
                                                       border-bottom-style="solid" border-bottom-width="0.5pt"
                                                       font-size="10pt" font-weight="bold" padding="5px"
                                                       background-color="#DDDDDD" text-align="left">
                                            <fo:block>BILL TO</fo:block>
                                        </fo:table-cell>
                                        <fo:table-cell><fo:block> </fo:block></fo:table-cell>
                                    </fo:table-row>
                                    <fo:table-row>
                                        <fo:table-cell border-bottom-style="solid" border-bottom-width="1pt"
                                                       border-left-style="solid" border-left-width="1pt"
                                                       border-right-style="solid" border-right-width="1pt"
                                                       border-top-style="solid" border-top-width="0.5pt"
                                                       font-size="9pt" padding="6px"
                                                       text-align="left">
                                            <fo:block white-space-collapse="false"
                                                      white-space-treatment="preserve"
                                                      linefeed-treatment="preserve">
                                                <xsl:value-of select="i:billingSummary/i:billTo/text()"/>
                                            </fo:block>
                                        </fo:table-cell>
                                        <fo:table-cell><fo:block> </fo:block></fo:table-cell>
                                    </fo:table-row>
                                </fo:table-body>
                            </fo:table>
                        </fo:block>
                    </fo:table-cell>

                    <fo:table-cell padding-top="15pt">
                        <fo:table table-layout="fixed" width="100%" text-align="center">
                            <fo:table-column/>
                            <fo:table-column column-width="150pt"/>
                            <fo:table-body>
                                <fo:table-row>
                                    <fo:table-cell><fo:block> </fo:block></fo:table-cell>
                                    <fo:table-cell border-top-style="solid" border-top-width="1pt"
                                                   border-left-style="solid" border-left-width="1pt"
                                                   border-right-style="solid" border-right-width="1pt"
                                                   border-bottom-style="solid" border-bottom-width="0.5pt"
                                                   font-size="10pt" font-weight="bold" padding="5px"
                                                   background-color="#DDDDDD">
                                        <fo:block>AMOUNT DUE</fo:block>
                                    </fo:table-cell>
                                </fo:table-row>
                                <fo:table-row>
                                    <fo:table-cell><fo:block> </fo:block></fo:table-cell>
                                    <fo:table-cell border-bottom-style="solid" border-bottom-width="1pt"
                                                   border-left-style="solid" border-left-width="1pt"
                                                   border-right-style="solid" border-right-width="1pt"
                                                   border-top-style="solid" border-top-width="0.5pt"
                                                   font-size="11pt" font-weight="bold" padding="6px">
                                        <fo:block>
                                            <xsl:value-of select="i:currencySign/text()"/>&#160;
                                            <xsl:value-of select="i:billingSummary/i:total/text()"/>
                                        </fo:block>
                                    </fo:table-cell>
                                </fo:table-row>
                            </fo:table-body>
                        </fo:table>

                        <fo:block padding-after="8pt"> </fo:block>

                        <fo:table text-align="center" table-layout="fixed" width="100%">
                            <fo:table-column/>
                            <fo:table-column column-width="150pt"/>
                            <fo:table-body>
                                <fo:table-row>
                                    <fo:table-cell><fo:block> </fo:block></fo:table-cell>
                                    <fo:table-cell border-top-style="solid" border-top-width="1pt"
                                                   border-left-style="solid" border-left-width="1pt"
                                                   border-right-style="solid" border-right-width="1pt"
                                                   border-bottom-style="solid" border-bottom-width="0.5pt"
                                                   font-size="10pt" font-weight="bold" padding="5px"
                                                   background-color="#DDDDDD">
                                        <fo:block>TERMS</fo:block>
                                    </fo:table-cell>
                                </fo:table-row>
                                <fo:table-row>
                                    <fo:table-cell><fo:block> </fo:block></fo:table-cell>
                                    <fo:table-cell border-bottom-style="solid" border-bottom-width="1pt"
                                                   border-left-style="solid" border-left-width="1pt"
                                                   border-right-style="solid" border-right-width="1pt"
                                                   border-top-style="solid" border-top-width="0.5pt"
                                                   font-size="10pt" padding="6px">
                                        <fo:block>
                                            <xsl:value-of select="i:billingSummary/i:terms/text()"/>
                                        </fo:block>
                                    </fo:table-cell>
                                </fo:table-row>
                            </fo:table-body>
                        </fo:table>
                    </fo:table-cell>
                  </fo:table-row>
              </fo:table-body>
          </fo:table>

          <fo:block padding-before="20px">


              <fo:table table-layout="fixed" width="100%">
                  <fo:table-column column-width="220pt"/>
                  <fo:table-column/>
                  <fo:table-column column-width="70pt"/>
                  <fo:table-column column-width="70pt"/>
                  <fo:table-column column-width="80pt"/>
                  <fo:table-header>
                      <fo:table-row>
                          <fo:table-cell border-style="solid" border-top-width="1pt"
                                         border-left-width="1pt"
                                         border-right-width="0.5pt"
                                         border-bottom-width="0.5pt"
                                         font-size="10pt" font-weight="bold" padding="5px" display-align="center"
                                         background-color="#DDDDDD"><fo:block>DESCRIPTION</fo:block></fo:table-cell>
                          <fo:table-cell border-style="solid" border-top-width="1pt"
                                         border-left-width="0.5pt"
                                         border-right-width="0.5pt"
                                         border-bottom-width="0.5pt"
                                         font-size="10pt" font-weight="bold" padding="5px" display-align="center"
                                         background-color="#DDDDDD"><fo:block>ITEM</fo:block></fo:table-cell>
                          <fo:table-cell border-style="solid" border-top-width="1pt"
                                         border-left-width="0.5pt"
                                         border-right-width="0.5pt"
                                         border-bottom-width="0.5pt"
                                         font-size="10pt" font-weight="bold" padding="5px" display-align="center" text-align="center"
                                         background-color="#DDDDDD"><fo:block>QTY</fo:block></fo:table-cell>
                          <fo:table-cell border-style="solid" border-top-width="1pt"
                                         border-left-width="0.5pt"
                                         border-right-width="0.5pt"
                                         border-bottom-width="0.5pt"
                                         font-size="10pt" font-weight="bold" padding="5px" display-align="center" text-align="center"
                                         background-color="#DDDDDD"><fo:block>RATE</fo:block><fo:block font-size="9pt">(<xsl:value-of select="i:currencySign/text()"/>/mh)</fo:block></fo:table-cell>
                          <fo:table-cell border-style="solid" border-top-width="1pt"
                                         border-left-width="0.5pt"
                                         border-right-width="1pt"
                                         border-bottom-width="0.5pt"
                                         font-size="10pt" font-weight="bold" padding="5px" display-align="center" text-align="center"
                                         background-color="#DDDDDD"><fo:block>AMOUNT</fo:block><fo:block font-size="9pt">(<xsl:value-of select="i:currencySign/text()"/>)</fo:block></fo:table-cell>
                      </fo:table-row>
                  </fo:table-header>
                  <fo:table-body>
                      <xsl:for-each select="i:items/i:item">
                          <fo:table-row keep-together="auto">
                              <fo:table-cell border-style="solid"
                                             border-top-width="0.5pt"
                                             border-left-width="1pt"
                                             border-right-width="0.5pt"
                                             border-bottom-width="0.5pt"
                                             font-size="8pt"
                                             padding="5px">
                                  <fo:block white-space-collapse="false"
                                            white-space-treatment="preserve"
                                            linefeed-treatment="preserve"><xsl:value-of select="i:description/text()"/></fo:block>
                              </fo:table-cell>
                              <fo:table-cell border-style="solid"
                                             border-width="0.5pt"
                                             font-size="8pt"
                                             padding="5px">
                                  <fo:block><xsl:value-of select="i:name/text()"/></fo:block>
                              </fo:table-cell>
                              <fo:table-cell border-style="solid"
                                             border-width="0.5pt"
                                             font-size="8pt"
                                             padding="5px"
                                             text-align="right">
                                  <fo:block><xsl:value-of select="i:quantity/text()"/></fo:block>
                              </fo:table-cell>
                              <fo:table-cell border-style="solid"
                                             border-width="0.5pt"
                                             font-size="8pt"
                                             padding="5px"
                                             text-align="right">
                                  <fo:block><xsl:value-of select="i:rate/text()"/></fo:block>
                              </fo:table-cell>
                              <fo:table-cell border-style="solid"
                                             border-top-width="0.5pt"
                                             border-left-width="0.5pt"
                                             border-right-width="1pt"
                                             border-bottom-width="0.5pt"
                                             font-size="8pt"
                                             font-weight="bold"
                                             padding="5px"
                                             text-align="right">
                                  <fo:block><xsl:value-of select="i:amount/text()"/></fo:block>
                              </fo:table-cell>
                          </fo:table-row>
                      </xsl:for-each>
                          <fo:table-row keep-with-previous="always">
                              <fo:table-cell  number-columns-spanned="5">
                                  <fo:block>
                                      <fo:table table-layout="fixed" width="100%">
                                          <fo:table-column/>
                                          <fo:table-column column-width="140"/>
                                          <fo:table-column column-width="80"/>
                                          <fo:table-body>
                                              <xsl:choose>
                                                  <xsl:when test="count(i:discounts/i:discount) + count(i:taxes/i:tax) &gt; 0">
                                                      <fo:table-row>
                                                          <fo:table-cell><fo:block> </fo:block></fo:table-cell>
                                                          <fo:table-cell border-left="0.5pt solid"
                                                                         border-right="0.5pt solid"
                                                                         border-bottom="0.5pt solid"
                                                                         font-size="9pt"
                                                                         font-weight="bold"
                                                                         padding="5px"
                                                                         text-align="center"
                                                                         background-color="#DDDDDD">
                                                              <fo:block>Sub-Total (<xsl:value-of select="i:currencySign/text()"/>)</fo:block>
                                                          </fo:table-cell>
                                                          <fo:table-cell border-left="0.5pt solid"
                                                                         border-right="1pt solid"
                                                                         border-bottom="0.5pt solid"
                                                                         font-size="9pt"
                                                                         font-weight="bold"
                                                                         padding="5px"
                                                                         display-align="center"
                                                                         text-align="right"
                                                                         background-color="#DDDDDD">
                                                              <fo:block><xsl:value-of select="i:billingSummary/i:subtotal/text()"/></fo:block>
                                                          </fo:table-cell>
                                                      </fo:table-row>
                                                  </xsl:when>
                                                  <xsl:otherwise>
                                                    <fo:table-row>
                                                        <fo:table-cell>
                                                            <fo:block></fo:block>
                                                        </fo:table-cell>
                                                        <fo:table-cell border-top="0.5pt solid"
                                                                       border-left="1pt solid"
                                                                       border-right="0.5pt solid"
                                                                       border-bottom="1pt solid"
                                                                       font-size="10.5pt"
                                                                       font-weight="bold"
                                                                       padding="5px"
                                                                       text-align="center"
                                                                       background-color="#DDDDDD">
                                                            <fo:block>TOTAL (<xsl:value-of
                                                                    select="i:currencySign/text()"/>)
                                                            </fo:block>
                                                        </fo:table-cell>
                                                        <fo:table-cell border-top="0.5pt solid"
                                                                       border-left="0.5pt solid"
                                                                       border-right="1pt solid"
                                                                       border-bottom="1pt solid"
                                                                       font-size="10.5pt"
                                                                       font-weight="bold"
                                                                       padding="5px"
                                                                       display-align="center"
                                                                       text-align="right"
                                                                       background-color="#DDDDDD">
                                                            <fo:block>
                                                                <xsl:value-of
                                                                        select="i:billingSummary/i:total/text()"/>
                                                            </fo:block>
                                                        </fo:table-cell>
                                                    </fo:table-row>
                                                  </xsl:otherwise>
                                              </xsl:choose>
                                          </fo:table-body>
                                      </fo:table>
                                    </fo:block>
                              </fo:table-cell>
                          </fo:table-row>
                  </fo:table-body>
              </fo:table>

          </fo:block>

        <xsl:if test="count(i:discounts/i:discount) + count(i:taxes/i:tax) &gt; 0">
            <fo:block padding-before="10px">

                <fo:table table-layout="fixed" width="100%">
                    <fo:table-column/>
                    <fo:table-column column-width="70pt"/>
                    <fo:table-column column-width="80pt"/>
                    <fo:table-body>
                        <xsl:for-each select="i:discounts/i:discount">
                            <fo:table-row>
                                <fo:table-cell border-style="solid"
                                               border-top-width="0.5pt"
                                               border-left-width="1pt"
                                               border-right-width="0.5pt"
                                               border-bottom-width="0.5pt"
                                               font-size="8pt"
                                               padding="5px">
                                    <fo:block white-space-collapse="false"
                                              white-space-treatment="preserve"
                                              linefeed-treatment="preserve">
                                        <xsl:value-of select="i:name/text()"/>
                                    </fo:block>
                                </fo:table-cell>
                                <fo:table-cell border-style="solid"
                                               border-width="0.5pt"
                                               font-size="8pt"
                                               padding="5px"
                                               text-align="right">
                                    <fo:block>
                                        <xsl:value-of select="i:rate/text()"/>
                                    </fo:block>
                                </fo:table-cell>
                                <fo:table-cell border-style="solid"
                                               border-top-width="0.5pt"
                                               border-left-width="0.5pt"
                                               border-right-width="1pt"
                                               border-bottom-width="0.5pt"
                                               font-size="8pt"
                                               font-weight="bold"
                                               padding="5px"
                                               text-align="right">
                                    <fo:block>
                                        <xsl:value-of select="i:amount/text()"/>
                                    </fo:block>
                                </fo:table-cell>
                            </fo:table-row>
                        </xsl:for-each>
                        <xsl:for-each select="i:taxes/i:tax">
                            <fo:table-row>
                                <fo:table-cell border-style="solid"
                                               border-top-width="0.5pt"
                                               border-left-width="1pt"
                                               border-right-width="0.5pt"
                                               border-bottom-width="0.5pt"
                                               font-size="8pt"
                                               padding="5px">
                                    <fo:block white-space-collapse="false"
                                              white-space-treatment="preserve"
                                              linefeed-treatment="preserve">
                                        <xsl:value-of select="i:name/text()"/>
                                    </fo:block>
                                </fo:table-cell>
                                <fo:table-cell border-style="solid"
                                               border-width="0.5pt"
                                               font-size="8pt"
                                               padding="5px"
                                               text-align="right">
                                    <fo:block>
                                        <xsl:value-of select="i:rate/text()"/>
                                    </fo:block>
                                </fo:table-cell>
                                <fo:table-cell border-style="solid"
                                               border-top-width="0.5pt"
                                               border-left-width="0.5pt"
                                               border-right-width="1pt"
                                               border-bottom-width="0.5pt"
                                               font-size="8pt"
                                               font-weight="bold"
                                               padding="5px"
                                               text-align="right">
                                    <fo:block>
                                        <xsl:value-of select="i:amount/text()"/>
                                    </fo:block>
                                </fo:table-cell>
                            </fo:table-row>
                        </xsl:for-each>
                        <fo:table-row border-top-style="solid" border-top-width="1pt" keep-with-previous="always">
                            <fo:table-cell number-columns-spanned="3" border-right="1pt solid">
                                <fo:block>
                                    <fo:table table-layout="fixed" width="100%">
                                        <fo:table-column/>
                                        <fo:table-column column-width="139.5pt"/>
                                        <fo:table-column column-width="79.5pt"/>
                                        <fo:table-body>
                                            <fo:table-row>
                                                <fo:table-cell>
                                                    <fo:block></fo:block>
                                                </fo:table-cell>
                                                <fo:table-cell border-top="0.5pt solid"
                                                               border-left="1pt solid"
                                                               border-right="0.5pt solid"
                                                               border-bottom="1pt solid"
                                                               font-size="10.5pt"
                                                               font-weight="bold"
                                                               padding="5px"
                                                               text-align="center"
                                                               background-color="#DDDDDD">
                                                    <fo:block>TOTAL (<xsl:value-of
                                                            select="i:currencySign/text()"/>)
                                                    </fo:block>
                                                </fo:table-cell>
                                                <fo:table-cell border-top="0.5pt solid"
                                                               border-left="0.5pt solid"
                                                               border-right="0 none"
                                                               border-bottom="1pt solid"
                                                               font-size="10.5pt"
                                                               font-weight="bold"
                                                               padding="5px"
                                                               display-align="center"
                                                               text-align="right"
                                                               background-color="#DDDDDD">
                                                    <fo:block>
                                                        <xsl:value-of
                                                                select="i:billingSummary/i:total/text()"/>
                                                    </fo:block>
                                                </fo:table-cell>
                                            </fo:table-row>
                                        </fo:table-body>
                                    </fo:table>
                                </fo:block>
                            </fo:table-cell>
                        </fo:table-row>
                    </fo:table-body>
                </fo:table>

            </fo:block>
        </xsl:if>

          <xsl:if test="string-length(normalize-space( i:customerMessage/text() )) &gt; 0">
            <fo:block padding-before="25pt" padding-after="8pt"
                      font-size="10pt"
                      white-space-collapse="false"
                      linefeed-treatment="preserve">
              <xsl:value-of select="i:customerMessage/text()"/>
            </fo:block>


                <xsl:if test="string-length(normalize-space( i:additionalInfo/text() )) &gt; 0">
                    <fo:block>
                        <fo:leader leader-length="100%"
                                   leader-pattern="rule"
                                   alignment-baseline="middle"
                                   rule-thickness="0.5pt" color="black"/>
                    </fo:block>
                </xsl:if>
          </xsl:if>

          <xsl:if test="string-length(normalize-space( i:additionalInfo/text() )) &gt; 0">
            <fo:block font-size="8pt"
                      white-space-collapse="false"
                      linefeed-treatment="preserve"
                      keep-together="1">
                <xsl:value-of select="i:additionalInfo/text()"/>
            </fo:block>
          </xsl:if>

        </fo:flow>
      </fo:page-sequence>
    </fo:root>
  </xsl:template>
</xsl:stylesheet>