<?xml version="1.0" encoding="utf-8"?>

<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:fo="http://www.w3.org/1999/XSL/Format"
                xmlns:date="http://exslt.org/dates-and-times"
                extension-element-prefixes="date"
                xmlns:my="https://www.actitime.com/pdf-schema" exclude-result-prefixes="my">
    <xsl:output method="xml" indent="yes"/>

    <xsl:param name="is_sick_show_in_hours"/>
    <xsl:param name="is_pto_show_in_hours"/>
    <xsl:param name="first-day-week"/>
    <xsl:param name="ptoEnabled"/>
    <xsl:param name="sickEnabled"/>
    <xsl:param name="colorScheme"/>
    
    <xsl:variable name="namedays">
    <day><one>S</one><three>SUN</three></day>
    <day><one>M</one><three>MON</three></day>
    <day><one>T</one><three>TUE</three></day>
    <day><one>W</one><three>WEN</three></day>
    <day><one>T</one><three>THU</three></day>
    <day><one>F</one><three>FRI</three></day>
    <day><one>S</one><three>SAT</three></day>
    </xsl:variable>

    <my:colorSchemeMap>
        <entry key="default">#6697c0</entry>
        <entry key="ferngreen">#90ac77</entry>
        <entry key="azureblue">#6697c0</entry>
        <entry key="aquamarine">#62a9bb</entry>
        <entry key="yellowgreen">#8db258</entry>
        <entry key="charcoalgrey">#81909f</entry>
        <entry key="red">#c55858</entry>
        <entry key="orange">#cd844d</entry>
        <entry key="purple">#ad7bc3</entry>
    </my:colorSchemeMap>

    <xsl:variable name="fullNameMonths">
        <month>January</month>
        <month>February</month>
        <month>March</month>
        <month>April</month>
        <month>May</month>
        <month>June</month>
        <month>July</month>
        <month>August</month>
        <month>September</month>
        <month>October</month>
        <month>November</month>
        <month>December</month>
    </xsl:variable>

    <xsl:variable name="shortNameMonths">
        <month>Jan</month>
        <month>Feb</month>
        <month>Mar</month>
        <month>Apr</month>
        <month>May</month>
        <month>Jun</month>
        <month>Jul</month>
        <month>Aug</month>
        <month>Sep</month>
        <month>Oct</month>
        <month>Nov</month>
        <month>Dec</month>
    </xsl:variable>

  <xsl:param name="nameDaysStr" select="document('')/*/xsl:variable[@name='namedays']/*"/>
  <xsl:param name="fullNameMonthStr" select="document('')/*/xsl:variable[@name='fullNameMonths']/*"/>
  <xsl:param name="shortNameMonthStr" select="document('')/*/xsl:variable[@name='shortNameMonths']/*"/>
  <xsl:variable name="leaveTypes" select="LeaveTimeHistory/leaveTypes/leaveType"/>
  <xsl:variable name="nonWorkingDays" select="LeaveTimeHistory/nonWorkingDays"/>
  <xsl:variable name="countMonths" select="count(LeaveTimeHistory/month)"/>
  <xsl:variable name="today" select="LeaveTimeHistory/today/day"/>
    <xsl:variable name="dayWidth">
        <xsl:if test="$countMonths = 1">
            <xsl:value-of select="'35'"/>
        </xsl:if>
        <xsl:if test="$countMonths > 1">
            <xsl:value-of select="'20'"/>
        </xsl:if>
    </xsl:variable>
    <xsl:variable name="dayHeight">
        <xsl:if test="$countMonths = 1">
            <xsl:value-of select="'20'"/>
        </xsl:if>
        <xsl:if test="$countMonths > 1">
            <xsl:value-of select="'15'"/>
        </xsl:if>
    </xsl:variable>

    <xsl:variable name="daySize">
        <xsl:if test="$countMonths = 1">
            <xsl:value-of select="'9'"/>
        </xsl:if>
        <xsl:if test="$countMonths > 1">
            <xsl:value-of select="'8'"/>
        </xsl:if>
    </xsl:variable>



    <xsl:template match="LeaveTimeHistory">
        <xsl:param name="page-count"/>
        <fo:root font-family="Charis">
            <fo:layout-master-set>
                <fo:simple-page-master master-name="Letter"
                                       page-height="279.4mm"
                                       page-width="215.9mm"
                                       keep-with-next="always"
                                       margin-top="1cm"
                                       margin-bottom="2cm"
                                       margin-left="1.5cm"
                                       margin-right="1.5cm">
                    <fo:region-body/>
                    <fo:region-after/>
                </fo:simple-page-master>
            </fo:layout-master-set>
            <fo:page-sequence master-reference="Letter" initial-page-number="1">
                <fo:static-content flow-name="xsl-region-after" text-align="right" font-size="8pt">
                    <fo:block>
                        <xsl:if test="$page-count &gt; 1">
                            Page
                            <fo:page-number/>
                            of
                            <xsl:value-of select="$page-count"/>
                        </xsl:if>
                    </fo:block>
                </fo:static-content>

                <fo:flow flow-name="xsl-region-body">
                    <fo:block>
                      <xsl:apply-templates select="." mode="table"/>
                    </fo:block>
                </fo:flow>
            </fo:page-sequence>
        </fo:root>
    </xsl:template>


    <xsl:template match="LeaveTimeHistory" mode="table">
        <fo:table width="530pt" table-layout="fixed" border-bottom-width="0.5pt" border-bottom-style="solid">
            <fo:table-column column-width="530pt"/>
            <fo:table-body>
                <fo:table-row>
                    <fo:table-cell>
                        <fo:block font-size="16pt" font-weight="bold">Past Leaves Calendar for
                            <xsl:call-template name="formatUserName">
                                <xsl:with-param name="firstName" select="user/@firstName"/>
                                <xsl:with-param name="lastName" select="user/@lastName"/>
                                <xsl:with-param name="middleInitial" select="user/@middleInitial"/>
                            </xsl:call-template>
                        </fo:block>
                    </fo:table-cell>
                </fo:table-row>
            </fo:table-body>
        </fo:table>
        <fo:block padding-after="20pt"/>

        <xsl:call-template name="informationHeader"/>

        <fo:block padding-after="20pt"/>

        <xsl:apply-templates select="BalanceStats"/>

        <fo:block padding-after="20pt"/>
                <xsl:apply-templates select="./leaveTypes/leaveType[not(position() mod 2 = 0)]"
                                         mode="LeaveTypeTotals"/>
        <fo:block padding-after="20pt"/>


        <fo:table width="530pt" table-layout="fixed" border-bottom-style="solid" border-bottom-width="0.5pt">
            <fo:table-column column-width="530pt"/>
            <fo:table-body>
                <fo:table-row>
                    <fo:table-cell>
                        <fo:block padding-after="1pt" font-size="16pt" font-weight="bold">Calendar</fo:block>
                    </fo:table-cell>
                </fo:table-row>
            </fo:table-body>
        </fo:table>

        <fo:block padding-after="20pt"/>

        <xsl:apply-templates select="." mode="calendar"/>

        <fo:block padding-after="20pt"/>
        <fo:block >
            <fo:table table-layout="fixed" width="530pt" border-bottom-style="solid" border-bottom-width="0.5pt">
                <fo:table-column column-width="170pt"/>
                <fo:table-column column-width="360pt"/>
                <fo:table-body>
                    <fo:table-row keep-together.within-page="always">
                        <fo:table-cell>
                            <fo:block padding-after="1pt" font-size="16pt" font-weight="bold">Leave Time Details</fo:block>
                        </fo:table-cell>
                             <fo:table-cell text-align="right">
                                 <fo:block  padding-before="4pt" font-size="8pt">
                                     <xsl:apply-templates select="leaveRequests/@filterByBalance[. = 'PTO' or .='SICK']" mode="filterTitle"/>
                                 </fo:block>
                             </fo:table-cell>
                    </fo:table-row>
                </fo:table-body>
            </fo:table>
            <fo:block padding-after="20pt"/>
            <fo:block keep-together.within-page="auto">
                <xsl:apply-templates select="leaveRequests"/>
            </fo:block>
        </fo:block>
   </xsl:template>

    <xsl:template match="BalanceStats">
       <fo:table table-layout="fixed" width="100%" font-size="10pt" margin-left="22pt">
           <fo:table-body >
              <fo:table-row font-size="12pt" border-bottom-style="solid"  border-bottom-color="#eaeaea">
                <fo:table-cell margin-left="0pt" width="110pt" font-weight="bold">
                    <fo:block  padding-after="5pt">
                      Balance:
                    </fo:block>
                </fo:table-cell>
                  <xsl:apply-templates select="Date[@type='startPeriod']/Balance" mode="nameBalances"/>
                </fo:table-row>
                <xsl:apply-templates select="Date"/>

               <fo:table-row>
                   <fo:table-cell margin-left="0pt">
                       <fo:block  padding-before="5pt" font-weight="bold">
                            Spent
                       </fo:block>
                   </fo:table-cell>
                  <xsl:apply-templates select="spent/Balance"/>
               </fo:table-row>
          </fo:table-body>
        </fo:table>
    </xsl:template>

    <xsl:template match="BalanceStats/Date[@type = 'startPeriod']/Balance" mode="nameBalances">
        <fo:table-cell width="100pt" font-weight="bold" font-size="10pt">
            <fo:block>
                <xsl:apply-templates select="./@type"/>
            </fo:block>
        </fo:table-cell>
    </xsl:template>

    <xsl:template match="BalanceStats/Date">
        <fo:table-row>
            <fo:table-cell margin-left="0pt"  >
                <fo:block padding-before="5pt" padding-after="5pt">
                    <xsl:variable name="month" select="day/@month"/>
                    on   <xsl:value-of  select="concat($shortNameMonthStr[number($month)],' ',day/@day, ', ', day/@year)"/>
                </fo:block>
            </fo:table-cell>
            <xsl:apply-templates select="Balance[@type='PTO']"/>
            <xsl:apply-templates select="Balance[@type='SICK']"/>
        </fo:table-row>
    </xsl:template>

    <xsl:template match="BalanceStats/*/Balance">
        <fo:table-cell padding-before="5pt"   >
            <xsl:variable name="numberValue">
                <xsl:value-of select="./value/days/number"/>
            </xsl:variable>
            <xsl:variable name="color">
                <xsl:if test="-0.1 > $numberValue">
                    <xsl:value-of select="'#E0401A'"/>
                </xsl:if>
                <xsl:if test="$numberValue >= -0.1">
                    <xsl:value-of select="'#000000'"/>
                </xsl:if>
            </xsl:variable>

             <fo:block  color="{$color}">
                <xsl:variable name="showInHours">
                    <xsl:apply-templates select="./@type" mode="showInHours"/>
                </xsl:variable>
                <xsl:apply-templates select="value">
                    <xsl:with-param name="is_show_in_hours" select="$showInHours"/>
                </xsl:apply-templates>
            </fo:block>
        </fo:table-cell>
    </xsl:template>

    <xsl:template match="LeaveTimeHistory/leaveRequests/@filterByBalance[ .= 'PTO' or .='SICK']" mode="filterTitle">
        <fo:block>
                           <fo:external-graphic height="8pt"  content-width="12px" src="servlet-context:/img/default/filter-pdf.png"/>
                           <fo:inline  font-weight="bold">   Applied filter: </fo:inline>  leaves affecting <xsl:apply-templates select="."/>  Balance
        </fo:block>
    </xsl:template>

    <xsl:template match="LeaveTimeHistory[count(month) = 1]" mode="calendar" >
        <fo:block-container margin-left="47pt">
        <fo:table width="100%" table-layout="fixed">
            <fo:table-column column-width="100%"/>
            <fo:table-body >
              <fo:table-row>
                  <xsl:apply-templates select="month" mode="month"/>
              </fo:table-row>
            </fo:table-body>
        </fo:table>
        </fo:block-container>
    </xsl:template>



    <xsl:template match="LeaveTimeHistory[count(month) > 1]" mode="calendar">
        <fo:table table-layout="fixed" width="540pt">
            <fo:table-column column-width="180pt"/>
            <fo:table-column column-width="180pt"/>
            <fo:table-column column-width="180pt"/>
            <fo:table-body>
                <xsl:apply-templates select="month[position() mod 3 = 1]" mode="rowMonth"/>
            </fo:table-body>
        </fo:table>
    </xsl:template>

    <xsl:template match="month" mode="rowMonth" >
            <fo:table-row keep-together.within-page="always">
                <xsl:apply-templates select="." mode="month"/>
                <xsl:apply-templates select="(following-sibling::month)[1]" mode="month"/>
                <xsl:apply-templates select="(following-sibling::month)[2]" mode="month"/>
            </fo:table-row>
    </xsl:template>

    <xsl:template name="monthCalendar" match="month" mode="month">
        <fo:table-cell>
        <xsl:variable name="number-year"   select="./@year"/>
        <xsl:variable name="number-m"   select="./@number"/>
        <xsl:variable name="holidays" select="./holiday"/>
        <xsl:variable name="leavetimes" select="./leaveTime"/>
        <xsl:variable name="NumberOfDaysInMonth">
            <xsl:apply-templates select="." mode="DaysInMonth"/>
        </xsl:variable>
        <xsl:variable name="FirstDayInWeekForMonth">
            <xsl:choose>
                <xsl:when test="$number-m &lt; 10">
                    <xsl:value-of select="date:day-in-week(date:date(concat($number-year,'-0', $number-m, '-01')))"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="date:day-in-week(date:date(concat($number-year,'-', $number-m, '-01')))"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
            <fo:block padding-after="3pt"/>
            <fo:table  table-layout="fixed" width="100%" text-align="center">

            <fo:table-column column-width="{$dayWidth}pt"/>
            <fo:table-column column-width="{$dayWidth}pt"/>
            <fo:table-column column-width="{$dayWidth}pt"/>
            <fo:table-column column-width="{$dayWidth}pt"/>
            <fo:table-column column-width="{$dayWidth}pt"/>
            <fo:table-column column-width="{$dayWidth}pt"/>
            <fo:table-column column-width="{$dayWidth}pt"/>
            <fo:table-body>

                    <fo:table-row>
                        <fo:table-cell number-columns-spanned="7" text-align="center" font-size="10pt">
                            <fo:block-container margin-left="1pt" margin-right="1pt">
                            <fo:block>
                                <xsl:value-of select="concat($fullNameMonthStr[number($number-m)],', ', $number-year)"/>
                            </fo:block>
                            </fo:block-container>
                        </fo:table-cell>
                    </fo:table-row>

                    <fo:table-row height="7pt">
                        <fo:table-cell>
                            <fo:block/>
                        </fo:table-cell>
                    </fo:table-row>

                    <xsl:call-template name="daysRow">
                        <xsl:with-param name="nonWorkingdays" select="$nonWorkingDays"/>
                    </xsl:call-template>

                    <fo:table-row height="5pt">
                        <fo:table-cell>
                            <fo:block/>
                        </fo:table-cell>
                    </fo:table-row>

                <xsl:variable name="FirstDayInWeekForMonthWithStartDay">
                    <xsl:value-of select="(($FirstDayInWeekForMonth - $first-day-week) +7) mod 7 + 1"/>
                </xsl:variable>

                <xsl:variable name="WeeksInMonth">
                    <xsl:value-of select="($NumberOfDaysInMonth + $FirstDayInWeekForMonthWithStartDay) div 7"/>
                </xsl:variable>


                <xsl:call-template name="CalendarWeek">
                        <xsl:with-param name="WeeksInMonth" select="$WeeksInMonth"/>
                        <xsl:with-param name="FirstDayInWeekForMonth"
                                        select="$FirstDayInWeekForMonthWithStartDay"/>
                        <xsl:with-param name="NumberOfDaysInMonth" select="$NumberOfDaysInMonth"/>
                        <xsl:with-param name="year" select="$number-year"/>
                        <xsl:with-param name="month" select="$number-m"/>
                        <xsl:with-param name="holidays" select="$holidays"/>
                        <xsl:with-param name="leavetimes" select="$leavetimes"/>
                    </xsl:call-template>
             </fo:table-body>
        </fo:table>
       </fo:table-cell>
    </xsl:template>

    <xsl:template name="daysRow">
        <xsl:param name="nonWorkingdays"/>
        <fo:table-row  border-bottom-style="solid"  border-bottom-color="#eaeaea" 
                       border-top-style="solid" border-top-color="#eaeaea">
               <xsl:call-template name="nameDay">
                  <xsl:with-param name="countDay" select="1"/>
                  <xsl:with-param name="nonWorkingdays" select="$nonWorkingdays"/>
                  <xsl:with-param name="day" select="$first-day-week -1"/>
               </xsl:call-template>
               </fo:table-row>
    </xsl:template>

    <xsl:template name="nameDay">
          <xsl:param name="countDay"/>
          <xsl:param name="nonWorkingdays"/>
          <xsl:param name="day"/>
          <xsl:variable name="numberday">
            <xsl:value-of select="$day mod 7 + 1"/>
          </xsl:variable>
        <fo:table-cell display-align="center" padding-top="4px" padding-bottom="3px">
            <xsl:variable name="bgColor">
                <xsl:if test="$nonWorkingdays[./day=$numberday]">
                    <xsl:value-of select="document('')/*/my:colorSchemeMap/entry[@key=$colorScheme]"/>
                </xsl:if>
                <xsl:if test="not($nonWorkingdays[./day=$numberday])">
                    <xsl:value-of select="document('')/*/my:colorSchemeMap/entry[@key=$colorScheme]"/>
                </xsl:if>
            </xsl:variable>
            <fo:block-container margin-left="1pt" margin-right="1pt">
            <xsl:variable name="nameDay">
                <xsl:if test="$countMonths = 1">
                    <xsl:value-of select="$nameDaysStr[$day mod 7 + 1]/three"/>
                </xsl:if>
                <xsl:if test="$countMonths > 1">
                    <xsl:value-of select="$nameDaysStr[$day mod 7 + 1]/one"/>
                </xsl:if>
            </xsl:variable>

            <fo:block color="{$bgColor}" font-size="{$daySize + 1}">
                <xsl:value-of select="$nameDay"/>
            </fo:block>
            </fo:block-container>
        </fo:table-cell>
        <xsl:if test="7 > $countDay">
        <xsl:call-template name="nameDay">
            <xsl:with-param name="countDay" select="$countDay + 1"/>
            <xsl:with-param name="nonWorkingdays" select="$nonWorkingdays"/>
            <xsl:with-param name="day" select="$day+1"/>
        </xsl:call-template>
        </xsl:if>
    </xsl:template>

    <xsl:template match="month[@number = 1 or @number = 3 or @number = 5 or @number = 7 or @number = 8 or @number = 10 or @number = 12] |
                          day[@month = 1 or @month = 3 or @month = 5 or @month = 7 or @month = 8 or @month = 10 or @month = 12 ]" mode="DaysInMonth">
        31
    </xsl:template>

    <xsl:template match="month[@number = 4 or @number = 6 or @number = 9 or @number = 11] |
                          day[@month = 4 or @month = 6 or @month = 9 or @month = 11]" mode="DaysInMonth">
        30
    </xsl:template>

    <xsl:template match="month[@number = 2 and (@year mod 4 = 0 and (@year mod 100 > 0) or @year mod 400 = 0)  ] |
                          day[@month = 2 and (@year mod 4 = 0 and (@year mod 100 > 0) or @year mod 400 = 0)]" mode="DaysInMonth">
        29
    </xsl:template>

    <xsl:template match="month[@number = 2 and ((@year mod 4) > 0 or (@year mod 100 = 0 and @year mod 400 > 0))] | day[@month = 2 and ((@year mod 4) > 0 or (@year mod 100 = 0 and @year mod 400 > 0))]" mode="DaysInMonth">
        28
    </xsl:template>

    <xsl:template name="CalendarWeek">
        <xsl:param name="WeeksInMonth"/>
        <xsl:param name="FirstDayInWeekForMonth"/>
        <xsl:param name="NumberOfDaysInMonth" />
        <xsl:param name="year" />
        <xsl:param name="month" />
        <xsl:param name="holidays"/>
        <xsl:param name="leavetimes"/>

        <xsl:param name="week">1</xsl:param>
        <xsl:param name="day">1</xsl:param>
        <fo:table-row height="{$dayHeight}pt">
            <xsl:call-template name="CalendarDay">
                <xsl:with-param name="FirstDayInWeekForMonth" select="$FirstDayInWeekForMonth"/>
                <xsl:with-param name="NumberOfDaysInMonth" select="$NumberOfDaysInMonth" />
                <xsl:with-param name="day" select="$day"/>
                <xsl:with-param name="year" select="$year" />
                <xsl:with-param name="month" select="$month" />
                <xsl:with-param name="holidays" select="$holidays"/>
                <xsl:with-param name="leavetimes" select="$leavetimes"/>
            </xsl:call-template>
        </fo:table-row>
        <xsl:if test="$WeeksInMonth &gt; $week">
            <xsl:call-template name="CalendarWeek">
                <xsl:with-param name="WeeksInMonth" select="$WeeksInMonth"/>
                <xsl:with-param name="FirstDayInWeekForMonth" select="$FirstDayInWeekForMonth"/>
                <xsl:with-param name="NumberOfDaysInMonth" select="$NumberOfDaysInMonth" />
                <xsl:with-param name="year" select="$year" />
                <xsl:with-param name="month" select="$month" />
                <xsl:with-param name="week" select="$week + 1"/>
                <xsl:with-param name="day" select="$week * 7 - ($FirstDayInWeekForMonth - 2)"/>
                <xsl:with-param name="holidays" select="$holidays"/>
                <xsl:with-param name="leavetimes" select="$leavetimes"/>
            </xsl:call-template>
        </xsl:if>
    </xsl:template>

    <xsl:template name="CalendarDay">
        <xsl:param name="FirstDayInWeekForMonth" />
        <xsl:param name="NumberOfDaysInMonth" />
        <xsl:param name="count">1</xsl:param>
        <xsl:param name="day"/>
        <xsl:param name="year" />
        <xsl:param name="month" />
        <xsl:param name="holidays"/>
        <xsl:param name="leavetimes"/>
        <xsl:choose>
            <xsl:when test="($day = 1 and $count != $FirstDayInWeekForMonth) or $day &gt; $NumberOfDaysInMonth">
                <fo:table-cell font-size="{$daySize}pt">
                    <fo:block> &#x00A0;</fo:block>
                </fo:table-cell>
                <xsl:if test="$count &lt; 7">
                    <xsl:call-template name="CalendarDay">
                        <xsl:with-param name="FirstDayInWeekForMonth" select="$FirstDayInWeekForMonth"/>
                        <xsl:with-param name="NumberOfDaysInMonth" select="$NumberOfDaysInMonth" />
                        <xsl:with-param name="year" select="$year" />
                        <xsl:with-param name="month" select="$month" />
                        <xsl:with-param name="count" select="$count + 1"/>
                        <xsl:with-param name="day" select="$day"/>
                        <xsl:with-param name="holidays" select="$holidays"/>
                        <xsl:with-param name="leavetimes" select="$leavetimes"/>
                    </xsl:call-template>
                </xsl:if>
            </xsl:when>
            <xsl:otherwise>
                <xsl:variable name="isHoliday"  select="$holidays/@day=$day"/>
                <xsl:variable name="isLeaveType" select="$leavetimes/@day=$day"/>
                <xsl:variable name="isFutureDay">
                    <xsl:value-of select="string($year > $today/@year or
                              ($year = $today/@year) and  ($month > $today/@month) or
                              ($year = $today/@year) and  ($month = $today/@month) and   ($day >= $today/@day))"/>
                </xsl:variable>

                 <xsl:variable name="bgColor">
                     <xsl:if test="$isLeaveType">
                         <xsl:choose>
                             <xsl:when test="$isFutureDay = 'true'">
                                 <xsl:value-of select="'#e7e7e7'"/>
                             </xsl:when>
                             <xsl:otherwise> <xsl:call-template name="bgColorForLeave">
                                 <xsl:with-param name="iconId"  select="$leaveTypes[@id=$leavetimes[@day=$day]/@leaveType]/@iconId"/>
                                 <xsl:with-param name="isHoliday" select="$isHoliday"/>
                             </xsl:call-template>
                             </xsl:otherwise>
                         </xsl:choose>
                     </xsl:if>
                     <xsl:if test="not($isLeaveType)">
                         <xsl:value-of select="'#ffffff'"/>
                     </xsl:if>
                 </xsl:variable>
                <fo:table-cell display-align="center">
                    <fo:block-container height="{$dayHeight - 2}"   margin-left="1pt" margin-right="1pt" background-color="{$bgColor}" >
                        <xsl:call-template name="textDay">
                        <xsl:with-param name="day" select="$day"/>
                        <xsl:with-param name="isHoliday" select="$isHoliday"/>
                        <xsl:with-param name="isFutureDay" select="$isFutureDay"/>
                        <xsl:with-param name="iconId"  select="$leaveTypes[@id=$leavetimes[@day=$day]/@leaveType]/@iconId"/>
                    </xsl:call-template>
                    </fo:block-container>
                </fo:table-cell>
                <xsl:if test="$count &lt; 7">
                    <xsl:call-template name="CalendarDay">
                        <xsl:with-param name="FirstDayInWeekForMonth" select="$FirstDayInWeekForMonth"/>
                        <xsl:with-param name="NumberOfDaysInMonth" select="$NumberOfDaysInMonth" />
                        <xsl:with-param name="year" select="$year" />
                        <xsl:with-param name="month" select="$month" />
                        <xsl:with-param name="count" select="$count + 1"/>
                        <xsl:with-param name="day" select="$day + 1"/>
                        <xsl:with-param name="holidays" select="$holidays"/>
                        <xsl:with-param name="leavetimes" select="$leavetimes"/>
                    </xsl:call-template>
                </xsl:if>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>


    <xsl:template name="textDay">
        <xsl:param name="day"/>
        <xsl:param name="isHoliday"/>
        <xsl:param name="isFutureDay"/>
        <xsl:param name="iconId"/>
        <xsl:variable name="color">
            <xsl:choose>
                <xsl:when test="$isFutureDay = 'true'">
                    <xsl:value-of select="'#aaaaaa'"/>
                </xsl:when>
                <xsl:when test="$isHoliday">
                    <xsl:value-of select="'#8f1111'"/>
                </xsl:when>
                <xsl:when test="$iconId">
                    <xsl:call-template name="textColorForLeaveTimeDay">
                        <xsl:with-param name="iconId" select="$iconId"/>
                    </xsl:call-template>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="'#000000'"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
                <fo:block color="{$color}">
                    <xsl:call-template name="calendar-day">
                        <xsl:with-param name="day" select="$day" />
                    </xsl:call-template>
                </fo:block>
        </xsl:template>

    <xsl:template name="textColorForLeaveTimeDay">
        <xsl:param name="iconId"/>
        <xsl:choose>
            <xsl:when test="$iconId = 1 or  $iconId = 2  or  $iconId = 5  or  $iconId = 6  or  $iconId = 7
             or  $iconId = 8  or  $iconId = 9 or  $iconId = 13 or  $iconId = 14 or  $iconId = 15
              or  $iconId = 17  or  $iconId = 18">
                <xsl:value-of select="'#ffffff'"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="'#000000'"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="bgColorForLeave">
        <xsl:param name="isHoliday"/>
        <xsl:param name="iconId"/>
        <xsl:if test="not($isHoliday)">
            <xsl:choose>
                <xsl:when test="$iconId = 1"><xsl:value-of select="'#e97373'"/></xsl:when>
                <xsl:when test="$iconId = 2"><xsl:value-of select="'#82b1e7'"/></xsl:when>
                <xsl:when test="$iconId = 3"><xsl:value-of select="'#c1e873'"/></xsl:when>
                <xsl:when test="$iconId = 4"><xsl:value-of select="'#e7df72'"/></xsl:when>
                <xsl:when test="$iconId = 5"><xsl:value-of select="'#b186d6'"/></xsl:when>
                <xsl:when test="$iconId = 6"><xsl:value-of select="'#ad9c66'"/></xsl:when>
                <xsl:when test="$iconId = 7"><xsl:value-of select="'#c49b6f'"/></xsl:when>
                <xsl:when test="$iconId = 8"><xsl:value-of select="'#e36ac9'"/></xsl:when>
                <xsl:when test="$iconId = 9"><xsl:value-of select="'#646cc1'"/></xsl:when>
                <xsl:when test="$iconId = 10"><xsl:value-of select="'#d88f64'"/></xsl:when>
                <xsl:when test="$iconId = 11"><xsl:value-of select="'#adc7e8'"/></xsl:when>
                <xsl:when test="$iconId = 12"><xsl:value-of select="'#74ce39'"/></xsl:when>
                <xsl:when test="$iconId = 13"><xsl:value-of select="'#dc3030'"/></xsl:when>
                <xsl:when test="$iconId = 14"><xsl:value-of select="'#8238e5'"/></xsl:when>
                <xsl:when test="$iconId = 15"><xsl:value-of select="'#f8c909'"/></xsl:when>
                <xsl:when test="$iconId = 16"><xsl:value-of select="'#8b98a0'"/></xsl:when>
                <xsl:when test="$iconId = 17"><xsl:value-of select="'#8bdc59'"/></xsl:when>
                <xsl:when test="$iconId = 18"><xsl:value-of select="'#e84e4e'"/></xsl:when>
                <xsl:when test="$iconId = 19"><xsl:value-of select="'#a461ff'"/></xsl:when>
                <xsl:when test="$iconId = 20"><xsl:value-of select="'#ebe304'"/></xsl:when>
            </xsl:choose>
        </xsl:if>
        <xsl:if test="$isHoliday">
            <xsl:choose>
                <xsl:when test="$iconId = 1"><xsl:value-of select="'#f2abab'"/></xsl:when>
                <xsl:when test="$iconId = 2"><xsl:value-of select="'#b4d0f1'"/></xsl:when>
                <xsl:when test="$iconId = 3"><xsl:value-of select="'#daf1ab'"/></xsl:when>
                <xsl:when test="$iconId = 4"><xsl:value-of select="'#f1ecaa'"/></xsl:when>
                <xsl:when test="$iconId = 5"><xsl:value-of select="'#d0b6e6'"/></xsl:when>
                <xsl:when test="$iconId = 6"><xsl:value-of select="'#cec4a3'"/></xsl:when>
                <xsl:when test="$iconId = 7"><xsl:value-of select="'#dcc3a9'"/></xsl:when>
                <xsl:when test="$iconId = 8"><xsl:value-of select="'#eea6df'"/></xsl:when>
                <xsl:when test="$iconId = 9"><xsl:value-of select="'#a2a7da'"/></xsl:when>
                <xsl:when test="$iconId = 10"><xsl:value-of select="'#e8bca2'"/></xsl:when>
                <xsl:when test="$iconId = 11"><xsl:value-of select="'#ceddf1'"/></xsl:when>
                <xsl:when test="$iconId = 12"><xsl:value-of select="'#ace288'"/></xsl:when>
                <xsl:when test="$iconId = 13"><xsl:value-of select="'#ea8383'"/></xsl:when>
                <xsl:when test="$iconId = 14"><xsl:value-of select="'#b488ef'"/></xsl:when>
                <xsl:when test="$iconId = 15"><xsl:value-of select="'#fbdf6b'"/></xsl:when>
                <xsl:when test="$iconId = 16"><xsl:value-of select="'#b9c1c6'"/></xsl:when>
                <xsl:when test="$iconId = 17"><xsl:value-of select="'#b9ea9b'"/></xsl:when>
                <xsl:when test="$iconId = 18"><xsl:value-of select="'#f19595'"/></xsl:when>
                <xsl:when test="$iconId = 19"><xsl:value-of select="'#c8a0ff'"/></xsl:when>
                <xsl:when test="$iconId = 20"><xsl:value-of select="'#f3ee68'"/></xsl:when>
            </xsl:choose>
        </xsl:if>


    </xsl:template>
    <xsl:template name="calendar-day">
        <xsl:param name="day" />
     <fo:block font-weight="0.1pt" font-size="{$daySize}pt">
        <xsl:value-of select="$day"/>
     </fo:block>
    </xsl:template>


   <xsl:template name="informationHeader">
        <fo:table table-layout="fixed" background-color="#f8f8f8" width="525pt">
            <fo:table-column  column-width="140pt"/>
            <fo:table-column column-width="400pt" />
            <fo:table-body>
                <fo:table-row>
                    <fo:table-cell padding-left="20pt">
                        <fo:block padding-before="10pt" padding-after="10pt">
                            Period to View:
                        </fo:block>
                    </fo:table-cell>
                    <fo:table-cell>
                        <fo:block font-weight="bold"  padding-before="10pt">
                            <xsl:apply-templates select="period"/>
                        </fo:block>
                    </fo:table-cell>
                </fo:table-row>
                <fo:table-row>
                    <fo:table-cell padding-left="20pt">
                        <fo:block padding-after="10pt">
                            Leave Types:
                        </fo:block>
                    </fo:table-cell>
                    <fo:table-cell>
                        <fo:block-container overflow="hidden">
                            <fo:block  font-weight="bold" >
                                <xsl:apply-templates select="." mode="listLeavesNames"/>
                            </fo:block>
                        </fo:block-container>
                    </fo:table-cell>
                </fo:table-row>
                <fo:table-row>
                    <fo:table-cell  padding-left="20pt">
                        <fo:block padding-after="10pt">
                            Total Leave Time:
                        </fo:block>
                    </fo:table-cell>
                    <fo:table-cell>
                        <fo:block font-weight="bold">
                          <xsl:apply-templates select="./total"/>
                        </fo:block>
                    </fo:table-cell>
                </fo:table-row>
            </fo:table-body>
        </fo:table>
    </xsl:template>

    <xsl:template match="LeaveTimeHistory" mode="listLeavesNames">
        All Leave Types
    </xsl:template>

    <xsl:template match="LeaveTimeHistory[not(@isAllLeaveTypes)]" mode="listLeavesNames">
        <xsl:apply-templates select="./leaveTypes/leaveType"/>
    </xsl:template>

    <xsl:template match="LeaveTimeHistory/leaveTypes/leaveType">
            <xsl:value-of select="@name"/><xsl:text>, </xsl:text>
    </xsl:template>

    <xsl:template match="LeaveTimeHistory/leaveTypes/leaveType[position() = last()]">
        <xsl:value-of select="@name"/>
    </xsl:template>

    <xsl:template match="LeaveTimeHistory[count(month) > 1]/period">
        <xsl:variable name="countDayLastMonth">
           <xsl:apply-templates select="./end/day" mode="DaysInMonth"/>
        </xsl:variable>
        <fo:block>
                <xsl:call-template name="formatDate">
                     <xsl:with-param name="day" select="01"/>
                     <xsl:with-param name="month" select="./start/day/@month"/>
                     <xsl:with-param name="year" select="./start/day/@year"/>
                     <xsl:with-param name="isFullMonth" select="true()"/>
                </xsl:call-template>
                    -
                <xsl:call-template name="formatDate">
                     <xsl:with-param name="day" select="$countDayLastMonth"/>
                     <xsl:with-param name="month" select="./end/day/@month"/>
                     <xsl:with-param name="year" select="./end/day/@year"/>
                     <xsl:with-param name="isFullMonth" select="true()"/>
                </xsl:call-template>
                <xsl:text> (</xsl:text> <xsl:value-of select="$countMonths"/> <xsl:text> months) </xsl:text>
        </fo:block>
    </xsl:template>

    <xsl:template match="LeaveTimeHistory[count(month) = 1]/period">
        <xsl:variable name="startNumberMonth" select="./start/day/@month"/>
        <fo:block>
                    <xsl:value-of select="concat($fullNameMonthStr[number($startNumberMonth)], ', ', start/day/@year)"/>
        </fo:block>
    </xsl:template>

    <xsl:template match="total | value">
        <xsl:param name="is_show_in_hours"/>
        <xsl:if test="$is_show_in_hours ='true'">
            <xsl:value-of select="./hours/string"/>h (<xsl:value-of select="./days/string"/>d)
        </xsl:if>
        <xsl:if test="$is_show_in_hours ='false'">
            <xsl:value-of select="./days/string"/>d  (<xsl:value-of select="./hours/string"/>h)
        </xsl:if>
    </xsl:template>

    <xsl:template match="leaveType/total | LeaveTimeHistory/total">
            <fo:inline> <xsl:value-of select="./days/string"/>d  </fo:inline>  (<xsl:value-of select="./hours/string"/>h)
    </xsl:template>

    <xsl:template match="leaveType[@flag = 'empty']/total">-</xsl:template>

   <xsl:template match="leaveRequests">
      <fo:table table-layout="fixed" width="100%">
          <fo:table-column column-width="90pt"   />
          <fo:table-column column-width="100pt" />
          <fo:table-column column-width="90pt" />
          <xsl:if test="$ptoEnabled = 'true' or $sickEnabled = 'true' ">
            <fo:table-column column-width="90pt" />
            <fo:table-column column-width="160pt"/>
          </xsl:if>
          <xsl:if test="$ptoEnabled = 'false' and $sickEnabled = 'false' ">
              <fo:table-column column-width="250pt" />
          </xsl:if>

          <fo:table-header>
              <fo:table-row height="13pt"  keep-together.within-page="always">
                  <fo:table-cell
                          font-size="9pt"  padding="5px"
                          border-style="solid"
                          border-width="1pt"
                          border-color="#CECECE"
                          display-align="center"
                          background-color="#EBEBEB">
                      <fo:block margin-left="10pt">Leave Type
                      <xsl:if test="@sortBy='leaveType'">
                              <xsl:apply-templates select="@isAsc"/>
                      </xsl:if>
                      </fo:block>
                  </fo:table-cell>
                  <fo:table-cell
                          border-style="solid"
                          border-width="1pt"
                          border-color="#CECECE"
                          font-size="9pt" padding="5px"
                          display-align="center"
                          background-color="#EBEBEB">
                      <fo:block  margin-left="10pt">Period
                      <xsl:if test="@sortBy='period'">
                           <xsl:apply-templates select="@isAsc"/>
                      </xsl:if>
                      </fo:block>
                  </fo:table-cell>
                  <fo:table-cell
                          border-style="solid"
                          border-width="1pt"
                          border-color="#CECECE"
                          font-size="9pt"  padding="5px" display-align="center"
                          text-align="center"
                          background-color="#EBEBEB">
                      <fo:block>Time Requested</fo:block>
                  </fo:table-cell>
                  <xsl:if test="$ptoEnabled = 'true' or $sickEnabled = 'true' ">
                    <fo:table-cell
                            border-style="solid"
                            border-width="1pt"
                            border-color="#CECECE"
                            font-size="9pt" padding="1px"
                            text-align="center"
                            background-color="#EBEBEB">
                        <xsl:if test="$ptoEnabled = 'true' and $sickEnabled = 'true'">
                          <fo:block margin-left="5pt"  text-align="left"  font-size="9pt"><xsl:apply-templates select="./@filterByBalance"/> </fo:block>
                          <fo:block font-size="7pt">Balance before/after req.</fo:block>
                        </xsl:if>
                        <xsl:if test="$ptoEnabled = 'true' and $sickEnabled = 'false'">
                            <fo:block font-size="8pt" padding-before="3px">PTO before/after req. </fo:block>
                        </xsl:if>
                        <xsl:if test="$ptoEnabled = 'false' and $sickEnabled = 'true'">
                            <fo:block  font-size="8pt" padding-before="3px" >Sick Days before/after req. </fo:block>
                        </xsl:if>
                     </fo:table-cell>
                  </xsl:if>
                  <fo:table-cell
                          border-style="solid"
                          border-width="1pt"
                          border-color="#CECECE"
                          font-size="9pt" padding="5px"
                          background-color="#EBEBEB">
                      <fo:block>Status</fo:block>
                  </fo:table-cell>
              </fo:table-row>
          </fo:table-header>
          <fo:table-body>
                  <xsl:apply-templates select="." mode="leaveRequestTableBody" />
             </fo:table-body>
      </fo:table>
    </xsl:template>

    <xsl:template match="leaveRequests/@filterByBalance[../@filterByBalance='ALL']">
        All
    </xsl:template>

    <xsl:template match="Balance/@type[../@type='PTO'] | leaveRequests/@filterByBalance[../@filterByBalance='PTO'] ">
        PTO
    </xsl:template>

    <xsl:template match="Balance/@type[../@type='SICK'] | leaveRequests/@filterByBalance[../@filterByBalance='SICK']">
        Sick Days
    </xsl:template>

    <xsl:template match="leaveRequests" mode="leaveRequestTableBody">
        <xsl:apply-templates select="./leaveRequest"/>
    </xsl:template>

    <xsl:template match="leaveRequest">
        <fo:table-row height="13pt" border-bottom-style="solid"
                      border-bottom-width="0.5pt"
                      border-bottom-color="#CECECE" keep-together.within-page="always">
            <fo:table-cell font-size="8pt"
                           padding="5px"
                           text-align="left"
                           display-align="center">
                <fo:table width="100%" table-layout="fixed">
                    <fo:table-column column-width="16pt"/>
                    <fo:table-column column-width="50pt"/>
                    <fo:table-body>
                        <fo:table-row>
                            <xsl:variable name="leaveTypeId" select="LeaveType/@id"/>
                            <xsl:variable name="leaveType" select="$leaveTypes[@id=$leaveTypeId]"/>
                            <fo:table-cell display-align="center" text-align="center">
                                <fo:block margin-left="2pt" font-size="8pt" >
                                    <xsl:call-template name="leaveIcon">]
                                        <xsl:with-param name="iconId" select="$leaveType/@iconId"/>
                                    </xsl:call-template>
                                </fo:block>
                            </fo:table-cell>
                            <fo:table-cell >
                                <fo:block-container overflow="hidden">
                                    <fo:block margin-left="4pt" font-size="8pt">
                                        <xsl:apply-templates select="$leaveType" mode="name"/>
                                    </fo:block>
                                </fo:block-container>
                            </fo:table-cell>
                        </fo:table-row>
                    </fo:table-body>
                </fo:table>
             </fo:table-cell  >
            <fo:table-cell font-size="8pt"
                           padding="5px"
                           display-align="center">
                  <xsl:apply-templates select="./period" mode="periodForTable"/>
            </fo:table-cell>
            <fo:table-cell font-size="8pt"
                           padding="5px"
                           text-align="center"
                           display-align="center">
                  <xsl:apply-templates select="./TimeRequested"/>
            </fo:table-cell>
            <xsl:if test="$ptoEnabled = 'true' or $sickEnabled = 'true' ">
                <fo:table-cell font-size="8pt"
                               padding="5px"
                               display-align="center">
                    <xsl:apply-templates select="./Balance"/>
                </fo:table-cell>
            </xsl:if>
            <fo:table-cell font-size="8pt"
                           padding="5px"
                           display-align="center">
                <xsl:apply-templates select="./Status"/>
            </fo:table-cell>
           </fo:table-row>
    </xsl:template>

    <xsl:template match="leaveRequests[not(leaveRequest)]" mode="leaveRequestTableBody">
        <fo:table-row  height="30pt">
            <xsl:variable name="colspan">
                <xsl:if test="$ptoEnabled = 'true' or $sickEnabled = 'true' ">5</xsl:if>
                <xsl:if test="$ptoEnabled = 'false' and $sickEnabled = 'false' ">4</xsl:if>
            </xsl:variable>
            <fo:table-cell display-align="center"
                           text-align="center"
                           number-columns-spanned="{$colspan}">
                <fo:block font-size="9pt"> There are no leave time details available </fo:block>
            </fo:table-cell>
        </fo:table-row>
    </xsl:template>

    <xsl:template name="leaveIcon">
        <xsl:param name="iconId"/>
        <fo:external-graphic content-width="11px" src="{concat('servlet-context:/img/default/leave_types/icons/leave_type_', $iconId, '_24bit.png')}"/>
    </xsl:template>

    <xsl:template match="period[start/day/@day = end/day/@day and  start/day/@month = end/day/@month and start/day/@year = end/day/@year]" mode="periodForTable">
        <xsl:variable name="startDate">
            <xsl:call-template name="formatDate">
                <xsl:with-param name="day" select="start/day/@day"/>
                <xsl:with-param name="month" select="start/day/@month"/>
                <xsl:with-param name="year" select="start/day/@year"/>
                <xsl:with-param name="isFullMonth" select="false()"/>
            </xsl:call-template>
        </xsl:variable>
        <fo:block>
            <xsl:value-of select="$startDate"/>
        </fo:block>
    </xsl:template>

    <xsl:template match="period" mode="periodForTable">
        <fo:block>
            <xsl:variable name="formatDay">
                <xsl:call-template name="formatDay">
                    <xsl:with-param name="day" select="./start/day/@day"/>
                </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="startMonth" select="./start/day/@month"/>
            <xsl:value-of select="concat($formatDay, ' ', $shortNameMonthStr[number($startMonth) ])"/>
            -
            <xsl:call-template name="formatDate">
                <xsl:with-param name="day" select="./end/day/@day"/>
                <xsl:with-param name="month" select="./end/day/@month"/>
                <xsl:with-param name="year" select="./end/day/@year"/>
                <xsl:with-param name="isFullMonth" select="false()"/>
            </xsl:call-template>
        </fo:block>
    </xsl:template>

    <xsl:template name="formatDate">
        <xsl:param name="day"/>
        <xsl:param name="month"/>
        <xsl:param name="year"/>
        <xsl:param name="isFullMonth"/>
        <xsl:variable name="formatDay">
            <xsl:call-template name="formatDay">
                <xsl:with-param name="day" select="$day"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:if test="$isFullMonth">
            <xsl:value-of select="concat($fullNameMonthStr[number($month)],' ',$formatDay,', ',  $year)"/>
        </xsl:if>
        <xsl:if test="not($isFullMonth)">
            <xsl:value-of select="concat($shortNameMonthStr[number($month)],' ',$formatDay,', ',  $year)"/>
        </xsl:if>
    </xsl:template>

    <xsl:template name="formatDay">
        <xsl:param name="day"/>
        <xsl:if test="10 > $day " >
            <xsl:value-of select="concat('0', $day)"/>
        </xsl:if>
        <xsl:if test="$day >=10 " >
            <xsl:value-of select="$day"/>
        </xsl:if>
     </xsl:template>

    <xsl:template match="TimeRequested">
        <fo:block> <xsl:value-of select="./days/string"/>d  (<xsl:value-of select="./hours/string"/>h)
        </fo:block>
    </xsl:template>

    <xsl:template match="Balance">
        <xsl:if test="./before">
            <xsl:variable name="showInHours">
                <xsl:apply-templates select="./@type" mode="showInHours"/>
            </xsl:variable>
        <fo:table width="100%" table-layout="fixed">
            <fo:table-column column-width="40pt"/>
            <fo:table-column column-width="40pt"/>
            <fo:table-body>
                <fo:table-row>
                    <fo:table-cell text-align="center">
                        <fo:block>
                            <xsl:apply-templates select="./before">
                                   <xsl:with-param name="is_show_in_hours" select="$showInHours"/>
                            </xsl:apply-templates>
                        </fo:block>
                    </fo:table-cell>
                    <fo:table-cell text-align="center">
                        <fo:block>
                            <xsl:apply-templates select="./after">
                                <xsl:with-param name="is_show_in_hours" select="$showInHours"/>
                            </xsl:apply-templates>
                        </fo:block>
                    </fo:table-cell>
                </fo:table-row>
                <xsl:if test="./@type = 'SICK'">
                    <fo:table-row>
                        <fo:table-cell
                            number-columns-spanned="2"
                                      text-align="center"
                                      display-align="center">
                            <fo:block  color="#a0a0a0" font-size="7pt"><xsl:apply-templates select="./@type"/> Balance</fo:block>
                        </fo:table-cell>
                    </fo:table-row>
                </xsl:if>
            </fo:table-body>
        </fo:table>
        </xsl:if>
        <xsl:if test="not(./before)">
            <fo:table width="100%" table-layout="fixed">
                <fo:table-column column-width="40pt"/>
                <fo:table-column column-width="40pt"/>
                <fo:table-body>
                    <fo:table-row>
                        <fo:table-cell text-align="center">
                            <fo:block>  -  </fo:block>
                        </fo:table-cell>
                        <fo:table-cell text-align="center">
                            <fo:block>  -  </fo:block>
                        </fo:table-cell>
                    </fo:table-row>
                </fo:table-body>
            </fo:table>
        </xsl:if>
    </xsl:template>


    <xsl:template match="Balance/@type[../@type='PTO']" mode="showInHours">
        <xsl:value-of select="$is_pto_show_in_hours"/>
    </xsl:template>

    <xsl:template match="Balance/@type[../@type='SICK']" mode="showInHours">
        <xsl:value-of select="$is_sick_show_in_hours"/>
    </xsl:template>




    <xsl:template match="before | after">
            <xsl:param name="is_show_in_hours"/>
            <xsl:if test="$is_show_in_hours ='true'">
                <xsl:apply-templates select="./hours">
                        <xsl:with-param name="is_show_in_hours" select="'true'"/>
                </xsl:apply-templates>
            </xsl:if>
            <xsl:if test="$is_show_in_hours ='false'">
                <xsl:apply-templates select="./days">
                        <xsl:with-param name="is_show_in_hours" select="'false'"/>
                </xsl:apply-templates>
            </xsl:if>
    </xsl:template>

    <xsl:template match="hours | days">
        <xsl:param name="is_show_in_hours"/>
        <xsl:variable name="formatValue">
            <xsl:value-of select="./string"/>
        </xsl:variable>
        <xsl:variable name="value">
            <xsl:value-of select="./number"/>
        </xsl:variable>
        <xsl:variable name="color">
            <xsl:if test="-0.1 > $value">
                <xsl:value-of select="'#E0401A'"/>
            </xsl:if>
            <xsl:if test="$value >= -0.1">
                <xsl:value-of select="'#000000'"/>
            </xsl:if>
        </xsl:variable>

        <fo:block color="{$color}">
            <xsl:call-template name="formatPtoIndaysOrHOurs">
                <xsl:with-param name="is_show_in_hours" select="$is_show_in_hours"/>
                <xsl:with-param name="value" select="$formatValue"/>
            </xsl:call-template>
        </fo:block>
    </xsl:template>

    <xsl:template name="formatPtoIndaysOrHOurs">
        <xsl:param name="value"/>
        <xsl:param name="is_show_in_hours"/>
        <xsl:if test="$is_show_in_hours ='true'">
            <xsl:value-of select="$value"/>h
        </xsl:if>
        <xsl:if test="$is_show_in_hours ='false'">
            <xsl:value-of select="$value"/>d
        </xsl:if>
    </xsl:template>

    <xsl:template name="formatUserName">
        <xsl:param name="firstName"/>
        <xsl:param name="lastName"/>
        <xsl:param name="middleInitial"/>
        <xsl:value-of select="$firstName"/>
        <xsl:if test="$middleInitial != ''">
            <xsl:value-of select="concat(' ', $middleInitial, '.')"/>
        </xsl:if>
        <xsl:value-of select="concat(' ', $lastName)"/>
    </xsl:template>

    <xsl:template match="Status">
        <xsl:variable name="month" select="./day/@month"/>
        <xsl:variable name="day" select="./day/@day"/>
        <xsl:variable name="year" select="./day/@year"/>
        <xsl:variable name="date">
            <xsl:call-template name="formatDate">
                <xsl:with-param name="day" select="$day"/>
                <xsl:with-param name="month" select="$month"/>
                <xsl:with-param name="year" select="$year"/>
                <xsl:with-param name="isFullMonth" select="false()"/>
            </xsl:call-template>
        </xsl:variable>
        <fo:block>
            Approved <xsl:value-of select="$date"/>
        </fo:block>
        <fo:block padding-before="2pt" color="#a0a0a0" font-size="7pt">
            <xsl:apply-templates select="." mode="approverInfo"/>
        </fo:block>
    </xsl:template>

    <xsl:template match="Status" mode="approverInfo">
        by <xsl:call-template name="formatUserName">
            <xsl:with-param name="firstName" select="./approverInfo/user/@firstName"/>
            <xsl:with-param name="lastName" select="./approverInfo/user/@lastName"/>
            <xsl:with-param name="middleInitial" select="./approverInfo/user/@middleInitial"/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="Status[not(approverInfo)]" mode="approverInfo">
        Automatic approval
    </xsl:template>

    <xsl:template match="leaveType" mode="LeaveTypeTotals">
        <fo:table table-layout="fixed" width="560pt">
            <fo:table-column column-width="232pt"/>
            <fo:table-column column-width="280pt"/>
            <fo:table-body>
                <fo:table-row>
                    <xsl:apply-templates select="." mode="leaveTypeTotal" />
                    <xsl:apply-templates select="(following-sibling::leaveType)[1]" mode="leaveTypeTotal" />
                </fo:table-row>
            </fo:table-body>
        </fo:table>
    </xsl:template>

    <xsl:template match="leaveType" mode="leaveTypeTotal">

        <fo:table-cell padding-left="22pt">
            <fo:table table-layout="fixed" width="100pt">
                <fo:table-column column-width="133pt"/>
                <fo:table-column column-width="75pt"/>
                <fo:table-body>
                    <fo:table-row>
                        <fo:table-cell>
                            <fo:table table-layout="fixed" width="125pt">
                                <fo:table-column column-width="5pt"/>
                                <fo:table-column column-width="110pt"/>
                                <fo:table-body>
                                    <fo:table-row>
                                        <xsl:variable name="iconId">
                                            <xsl:value-of select="./@iconId"/>
                                        </xsl:variable>
                                        <xsl:variable name="bgcolor">
                                            <xsl:call-template name="bgColorForLeave">
                                                <xsl:with-param name="iconId"
                                                                select="$iconId"/>
                                                <xsl:with-param name="isHoliday" select="false()"/>
                                            </xsl:call-template>
                                        </xsl:variable>
                                        <fo:table-cell display-align="center">
                                            <fo:block-container background-color="{$bgcolor}" height="15pt" margin-top="2pt" margin-bottom="2pt">
                                                <fo:block/>
                                            </fo:block-container>
                                        </fo:table-cell>
                                        <fo:table-cell font-size="10pt" padding-left="5pt" display-align="center">
                                            <fo:block-container overflow="hidden">
                                                <fo:block>
                                                   <xsl:apply-templates select="." mode="name"/>
                                                </fo:block>
                                            </fo:block-container>
                                        </fo:table-cell>
                                    </fo:table-row>
                                </fo:table-body>
                            </fo:table>
                        </fo:table-cell>
                        <fo:table-cell font-size="10pt" display-align="center">
                            <fo:block>
                                <xsl:apply-templates select="./total"/>
                            </fo:block>
                        </fo:table-cell>
                    </fo:table-row>
                </fo:table-body>
            </fo:table>
        </fo:table-cell>
     </xsl:template>

    <xsl:template match="leaveRequests/@isAsc[. = 'true']">
           <fo:external-graphic content-width="5px" src="{'servlet-context:/actiplans/img/default/tables/sort-down.png'}"/>
    </xsl:template>

    <xsl:template match="leaveRequests/@isAsc[. = 'false']">
        <fo:external-graphic content-width="5px" src="{'servlet-context:/actiplans/img/default/tables/sort-up.png'}"/>
    </xsl:template>

    <xsl:template match="leaveType[@flag = 'archived'] | leaveType[@flag = 'empty']" mode="name">
        <fo:block color="#959595">
            <xsl:value-of select="@name"/>
        </fo:block>
    </xsl:template>

    <xsl:template match="leaveType" mode="name">
        <fo:block>
            <xsl:value-of select="@name"/>
        </fo:block>
    </xsl:template>

 </xsl:stylesheet>


