
function LeaveTimeRequestManager( request, dateFrom, dateTo, callback )
{
    RpcControlManager.call( this );
    this.request = request;
    this.dateFrom = dateFrom;
    this.dateTo = dateTo;
    this.callback = callback;
}

LeaveTimeRequestManager.prototype = new RpcControlManager();

LeaveTimeRequestManager.prototype.getTimeoutMsg = function()
{
    return messageResource.getMessage("leave_time_request_dialog.js.timeout");
};

LeaveTimeRequestManager.prototype.handleNoRightsError = function()
{
  return false;
};

LeaveTimeRequestManager.prototype.handleSpecificError = function( exception )
{
    var errorThrown = { exception: getExceptionNameFromTrace( exception.trace ) };

    return  new CorporateScheduleErrorHandler(this.onNoRightsToUpdateRequestException).defaultErrorHandler(null, null, errorThrown);
};

LeaveTimeRequestManager.prototype.onNoRightsToUpdateRequestException = function (){
    //abstract
};


LeaveTimeRequestManager.prototype.onDataSavingComplete = function( result )
{
    new InvocationResultHandler().handleResultCode( result.resultCode );

    if (result.warning == LeaveTimeRequestManager.WARNING_ARCHIVED_LEAVE_TYPE) {
        this.handleArchivedLeaveType( result );
    }

    this.callback( result );
};


var LeaveScheduleUtils = {};

LeaveScheduleUtils.createLegend = function(parentElementId, maxColumnNumber)
{
    var legendTable = document.getElementById(parentElementId);
    var ltTr;

    var leaveTypesAmount = ACTIVE_LEAVE_TYPES_ORDER.length;
    for( var i = 0; i < leaveTypesAmount; i++ )
    {
        var leaveType = LEAVE_TYPES[ACTIVE_LEAVE_TYPES_ORDER[i]];

        var columnNumber = i % maxColumnNumber;
        if (columnNumber == 0) {
            ltTr = document.createElement("tr");
            legendTable.appendChild(ltTr);
        }

        var ltBlock = document.createElement("td");
        ltBlock.className = "scheduleLegendLTBlock";
        var ltTextBlock = document.createElement("td");
        ltTextBlock.className = "scheduleLegendLTName";

        var ltTextDiv = document.createElement("div");

        var ltInnerTextDiv = document.createElement("div");
        ltInnerTextDiv.className = "scheduleLegendLTNameDiv";
        ltInnerTextDiv.innerHTML = escapeHtmlSymbols(leaveType.name);
        ltTextDiv.appendChild(ltInnerTextDiv);
        ltTextBlock.appendChild(ltTextDiv);

        var ltColorBlock = document.createElement("div");
        ltColorBlock.className = "ltColor leaveType"+ leaveType.iconId;
        ltBlock.appendChild(ltColorBlock);
        ltTr.appendChild(ltBlock);
        ltTr.appendChild(ltTextBlock);

        LeaveScheduleUtils._addPrintLeaveIcon(leaveType, ltColorBlock);
    }

    var remainingColumnNumber = maxColumnNumber - leaveTypesAmount % maxColumnNumber;
    if (remainingColumnNumber > 0 && remainingColumnNumber < maxColumnNumber) {
        var emptyTd = document.createElement("td");
        emptyTd.colSpan = remainingColumnNumber * 2;
        ltTr.appendChild(emptyTd);
    }

    $("#scheduleLegend").show();
};

LeaveScheduleUtils.createLegendWithFloat = function ( parentElementId )
{
    var legendContainer = document.getElementById( parentElementId );

    var leaveTypesAmount = ACTIVE_LEAVE_TYPES_ORDER.length;
    for( var i = 0; i < leaveTypesAmount; i++ )
    {
        var leaveType = LEAVE_TYPES[ACTIVE_LEAVE_TYPES_ORDER[i]];

        var ltBlock = document.createElement( "div" );
        ltBlock.className = "scheduleLegendLTBlock";

        var ltColorBlock = document.createElement( "div" );
        ltColorBlock.className = "ltColor leaveType" + leaveType.iconId;
        ltBlock.appendChild( ltColorBlock );

        var ltTextBlock = document.createElement( "div" );
        ltTextBlock.className = "scheduleLegendLTName";

        var ltTextDiv = document.createElement("div");
        ltTextDiv.className = "scheduleLegendLTNameDiv";
        ltTextDiv.innerHTML = escapeHtmlSymbols(leaveType.name);
        ltTextBlock.appendChild(ltTextDiv);

        ltBlock.appendChild( ltTextBlock );

        legendContainer.appendChild( ltBlock );

        LeaveScheduleUtils._addPrintLeaveIcon( leaveType, ltColorBlock );
    }

    $( "#scheduleLegend" ).show();
};

LeaveScheduleUtils._addPrintLeaveIcon = function ( leaveType, ltColorBlock )
{
    // add content for print
    var printLeaveDiv = document.createElement( "div" );
    printLeaveDiv.className = "for-print-only";
    var img = document.createElement( "img" );
    img.className = "printLeaveTypeImage";
    printLeaveDiv.appendChild( img );
    img.src = leaveType.getPrintIconPath();
    img.style.visibility = "visible";
    ltColorBlock.appendChild( printLeaveDiv );
};

function LeaveModificationsUtil(){}
LeaveModificationsUtil.prototype = _.create(Object.prototype, {
    getActualLeaveTypeId: function (request) 
    {
        return !request.requestChange ? request.leaveTypeId : request.requestChange.leaveTypeId;
    }
});

