function StateController()
{
}

StateController.prototype.init = function ( initialState )
{
    this._actualState = {};
    this._savedState = {};

    this._onLoadStateActions = [];
    this._onSaveStateActions = [];

    this.setProperties( initialState );
    this.saveState();

    return this;
};

/**
 * @param callback is function( actualState ), this functions will be called, when loadState calls.
 *                              actualState means state after load from store
 */
StateController.prototype.addOnLoadStateActions = function ( callback )
{
    this._onLoadStateActions.push( callback );
};

/**
 * @param callback is function( actualState ), this functions will be called, when loadState calls
 */
StateController.prototype.addOnSaveStateActions = function ( callback )
{
    this._onSaveStateActions.push( callback );
};

StateController.prototype.saveState = function ()
{
    this._savedState = jQuery.extend( true, {}, this._actualState );
    for( var i = 0; i < this._onSaveStateActions.length; i++ )
    {
        this._onSaveStateActions[i]( this._actualState );
    }
};

StateController.prototype.loadState = function ()
{
    this._actualState = jQuery.extend( true, {}, this._savedState );
    for( var i = 0; i < this._onLoadStateActions.length; i++ )
    {
        this._onLoadStateActions[i]( this._actualState );
    }
};

/**
 * Note: returns an mutable object!
 */
StateController.prototype.getState = function ()
{
    return this._actualState;
};

StateController.prototype.setProperties = function ( state )
{
    for( var key in state )
    {
        if( state.hasOwnProperty( key ) )
        {
            this._actualState[key] = state[key];
        }
    }
};

/**
 * Note: returns an object with mutable properties!
 */
StateController.prototype.getProperties = function ( keysArray )
{
    var result = {};
    for( var i = 0; i < keysArray.length; i++ )
    {
        var key = keysArray[i];
        result[key] = this._actualState[key];
    }
    return result;
};

StateController.prototype.setProperty = function ( key, value )
{
    this._actualState[key] = value;
};

/**
 * Note: returns an mutable object!
 */
StateController.prototype.getProperty = function ( key )
{
    return this._actualState[key];
};
