function CorporateScheduleLeaveTimeHistoryManager () {}

CorporateScheduleLeaveTimeHistoryManager.prototype = new LeaveTimeHistoryManager();

CorporateScheduleLeaveTimeHistoryManager.prototype.init = function ( initialData ) {
    this._initElements();
    this.initUserSelector(initialData.userGroupsEnabled, initialData.timeZoneGroupsEnabled, initialData.users, initialData.userGroups, initialData.timeZoneGroups, initialData.selectedUserId);

    this.nameService = initialData.nameService;

    return LeaveTimeHistoryManager.prototype.init.call(this, initialData);
};

CorporateScheduleLeaveTimeHistoryManager.prototype.makeInitRequest = function(initialData)
{
    var self = this;
    this._updateDataRequestFunction(
            function(userLeaveTimeHistoryData)
            {
                initialData.userLeaveTimeHistoryData = userLeaveTimeHistoryData;
                self.baseInit(initialData);
                $(document).trigger("apReports.leaveTimeHistory.dataUpdated");
            },
            initialData);
};

CorporateScheduleLeaveTimeHistoryManager.prototype.updateDataRequestFunction = function(params)
{
    var self = this;
    var scrollToCalendars = this.scrollToFunctionFactory();
    this._updateDataRequestFunction(
            function(userLeaveTimeHistoryData)
            {
                self.successResultCallback(userLeaveTimeHistoryData);
                scrollToCalendars();
                $(document).trigger("apReports.leaveTimeHistory.dataUpdated");
            },
            params
    );
};

CorporateScheduleLeaveTimeHistoryManager.prototype._updateDataRequestFunction = function(successResultCallback, params)
{
    var self = this;
    $(document).trigger("apReports.leaveTimeHistory.updateData");
    makeRpcCall(this.nameService + '.getLeaveTimeHistory', [params.selectedUserId, params.dateFrom, params.period, params.rangeType, params.leaveTypeIds],
                successResultCallback,
                function(jqXHR, textStatus, errorThrown)
                {
                    $(document).trigger("apReports.leaveTimeHistory.dataUpdated");
                    return self.defaultErrorHandler(jqXHR, textStatus, errorThrown);
                }
    );
};

CorporateScheduleLeaveTimeHistoryManager.prototype.makeUpdateRequest = function ( spinnerContainer ) {
    this.initialData.selectedUserId = this.getSelectedUserId();
    if (this.initialData.selectedUserId != 0)
        LeaveTimeHistoryManager.prototype.makeUpdateRequest.call(this, spinnerContainer);
};

CorporateScheduleLeaveTimeHistoryManager.prototype.updateData = function ( data ) {
    var noUserSelected = data.selectedUserId;
    this.leaveTimeHistoryBody.toggleClass("noUserSelected", !noUserSelected);
    if ( data.selectedUserId )
        LeaveTimeHistoryManager.prototype.updateData.call(this, data);
};

CorporateScheduleLeaveTimeHistoryManager.prototype.initUserSelector = function ( userGroupsEnabled, timeZoneGroupsEnabled, users, userGroups, timeZoneGroups, selectedUserId )
{
    var self = this;

    var userSelectorPanelConfig = {
        groupBySettingCookieName: "leaveTimeHistoryUserSelectorGroupBySetting",
        showDisabledSettingCookieName: "leaveTimeHistoryUserSelectorShowDisabledSetting",
        collapsedCookiesKey: "leaveTimeHistory"
    };
    var userSelectorPanel = userGroupsEnabled || timeZoneGroupsEnabled ? new AT.UserForLeaveHistorySelectorPanel(userSelectorPanelConfig) :
                            new AT.UserForLeaveHistorySelectorPanelWithInlineSearch(userSelectorPanelConfig);
    var userSelectorMenu = new AT.SharedUserButtonMenu( {   panel : userSelectorPanel,
                                                            additionalCls:  "ap-selector-menu " +
                                                                            "userForLeaveTimeHistorySelectorMenu " +
                                                                            (userGroupsEnabled || timeZoneGroupsEnabled ? "userSelectorWithGroupingModePanel " : ""),
                                                            shadow:false
                                                        });
    var chooseUserButton = new AT.HierarchySelectorDropdownButton({ renderTo: "leaveHistorySelector_chooseUserButton",
                                                                      menu: userSelectorMenu
                                                                  });

    userSelectorPanel.initialize(APReportsStore.getViewerId(), userGroupsEnabled, timeZoneGroupsEnabled, users, userGroups, timeZoneGroups, selectedUserId);
    userSelectorPanel.setSelectionChangedHandler( function () {
        if (self.stateController)
            self.stateController.setProperty('userSelectorState', userSelectorPanel.getState());
        APReportsPage.PastLeavesPage.Store.setSelectedUserId(userSelectorPanel.getSelectedUserId());
        userSelectorPanel.close();
        self.makeUpdateRequest( $("#userSelectorSpinner") ); });

    this.userSelector = userSelectorPanel;
};

CorporateScheduleLeaveTimeHistoryManager.prototype.getSelectedUserId = function () {
    return this.userSelector.getSelectedUserId();
};

CorporateScheduleLeaveTimeHistoryManager.prototype.getPdfElementForm = function(){
    return document.forms["LeaveTimeHistoryFormOnCorporateSchedule"];
};

CorporateScheduleLeaveTimeHistoryManager.prototype.initStateController = function ( initialData ) {
    LeaveTimeHistoryManager.prototype.initStateController.call(this, initialData);

    this.stateController.setProperty('userSelectorState', this.userSelector.getState());
    this.stateController.saveState();

    var self = this;
    this.stateController.addOnLoadStateActions( function ( state )
                                                {
                                                    self.userSelector.loadState(state.userSelectorState);
                                                } );
};
