/**
 * @author evdokimov
 */
function PastLeavesCalendar()
{}

PastLeavesCalendar.prototype = _.create(Object.prototype, {
    dayClassNames: ["sun", "mon", "tue", "wed", "thu", "fri", "sat"],

    /*
     * @config [isWideContainerFn] required
     * @config [isNarrowContainer] required
     * @config [calendarContainer] required
     *
     * */
    init: function(config)
    {
        this.templateCalendar = $("#templateCalendar");
        this.totalLeaveTime = $("#totalLeaveTime");
        this.totalLeaveTimeAdditional = $("#totalLeaveTimeAdditional");

        this.$scrollWrapper = $("#calendarLeaveTypesWrap");
        this.leaveTypesTableContent = this.$scrollWrapper.find("#leaveTypesTableBody");
        this.scroll = ScrollUtils.addVerticalScroll(this.$scrollWrapper, {isFaded: false});
        this.leaveTypesTemplateTable = $("#leaveTypesTemplateTable");
        this.templatePrintImg = $("#templatePrintImg");

        this.calendarContainer = config.calendarContainer;

        this.daysFormatter = new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 1, true, '', 'd');
        this.hoursFormatter = new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 2, true, '', 'h');

        this.isWideContainer = config.isWideContainerFn;
        this.isNarrowContainer = config.isNarrowContainer;

        this.isWide = !this.isWideContainer();
        this.updateCalendarContainerWidth();

        var self = this;
        $(window).resize(function() { self.updateCalendarContainerWidth(); });
        $(document).on("apReports.PastLeavesPage.openPage", function()
        {
            self.updateCalendarContainerWidth();
        });

        return this;
    },

    updateCalendarContainerWidth: function()
    {
        var isWideContainer = this.isWideContainer();
        this.calendarContainer.toggleClass("narrow", this.isNarrowContainer());

        if (this.isWide == isWideContainer)
            return;

        this.isWide = isWideContainer;
        this.calendarContainer.toggleClass("wide", isWideContainer);

        var dayNameText = isWideContainer ? Date.dayNamesShortCapital : Date.dayNamesAbbr;

        for (var i = 0; i < 7; i++)
        {
            this.calendarContainer.find("." + this.dayClassNames[i]).text(dayNameText[i]);
        }
    },

    /*
     * @calendarConfig [monthName] required
     * @calendarConfig [startAt] required
     * @calendarConfig [daysNumber] required
     * @calendarConfig [prevMonthDaysNumber] required
     * @calendarConfig [pastDaysNumber] required
     * @calendarConfig [leavesMap] required
     *
     * */
    createCalendar: function(wideCalendar, calendarConfig)
    {
        var weekStartDay = SystemSettings.weekStartDay;
        var calendarEl = this.templateCalendar.find(".calendar").clone();

        calendarEl.find(".monthName").text(calendarConfig.monthName);

        var dayNameText = wideCalendar ? Date.dayNamesShortCapital : Date.dayNamesAbbr;
        var weekDaysName = calendarEl.find(".weekDaysName");
        var daysTable = calendarEl.find(".daysTable");

        var skip = ( calendarConfig.startAt - weekStartDay + 8 ) % 7;

        for (var i = 1; i < 8; i++)
        {
            var currDay = ( i + weekStartDay - 2 ) % 7;
            var currDayEl = weekDaysName.find(".day" + i);
            currDayEl.text(dayNameText[currDay]);
            currDayEl.addClass(this.dayClassNames[currDay]).toggleClass('nonWorking', !!this.nonWorkingDays[currDay]);
        }

        var self = this;
        var appendDayFn = function(weekRowEl, weekDay, value)
        {
            var dayEl = weekRowEl.find(".day" + weekDay);

            self.templatePrintImg.find(".dayNumber").clone().appendTo(dayEl);
            dayEl.find(".dayNumber").text(value);

            return dayEl;
        };

        var dayCount = 1;
        for (i = 1; i < 7; i++)
        {
            var weekRow = daysTable.find(".row" + i);
            for (var j = 1; j < 8; j++)
            {
                if (dayCount <= skip)
                {
                    skip--;
                    /*var pastDayEl = appendDayFn(weekRow, j, calendarConfig.prevMonthDaysNumber - skip);
                    pastDayEl.addClass('prevMonth');*/
                }
                else if (dayCount > calendarConfig.daysNumber)
                {
                    /*var futureDayEl = appendDayFn(weekRow, j, dayCount - calendarConfig.daysNumber);
                    futureDayEl.addClass('prevMonth');*/
                    dayCount++;
                }
                else
                {
                    var dayEl = appendDayFn(weekRow, j, dayCount);
                    dayEl.toggleClass('nonWorking', calendarConfig.workdayDurations[dayCount - 1] == 0);

                    if (dayCount > calendarConfig.pastDaysNumber)
                        dayEl.addClass('future');

                    if (calendarConfig.leavesMap[dayCount - 1])
                    {
                        var requestLeaveType = calendarConfig.leavesMap[dayCount - 1];
                        var iconId = LEAVE_TYPES[requestLeaveType].iconId;
                        dayEl.addClass("leave").addClass("leave" + iconId);

                        this.templatePrintImg.find(".leaveTypeImg").clone().appendTo(dayEl);
                        dayEl.find(".leaveTypeImg > img").attr("src", getLeaveTypePrintIconPath(iconId));
                    }

                    dayCount++;
                }
            }
        }

        return calendarEl;
    },

    createCalendarConfig: function(year, month, requests, userYesterday)
    {
        var prevMonthStart = Date.create(year, month - 1, 1);
        var monthStart = Date.create(year, month, 1);
        var monthEnd = Date.create(year, month + 1, 1);

        var calendar = {};
        calendar.monthName = Date.monthNames[month] + ", " + year;
        calendar.startAt = monthStart.getDay();

        calendar.prevMonthDaysNumber = monthStart.offsetInDaysFrom(prevMonthStart);
        calendar.daysNumber = monthEnd.offsetInDaysFrom(monthStart);

        calendar.pastDaysNumber = userYesterday.offsetInDaysFrom(monthStart);
        calendar.leavesMap = this.getLeaveDaysMap(monthStart, monthEnd, requests);

        calendar.prevMonth = Date.create(year, month, 1);

        return calendar;
    },

    getLeaveDaysMap: function(monthStart, monthEnd, requests)
    {
        var result = {};

        var monthStartTime = monthStart.getTime();
        var monthEndTime = monthEnd.getTime();

        $.each(requests, function(i, leave)
        {
            var dateTime = leave.date.getTime();
            if (dateTime >= monthStartTime && dateTime < monthEndTime)
            {
                var dayNumber = leave.date.offsetInDaysFrom(monthStart);
                result[dayNumber] = leave.leaveTypeId;
            }
        });

        return result;
    },

    /*
     * @data required
     * @data [selectedUserId]
     * @data [leaveTimeTotal]
     * @data [leaveTypeIds] list of selected leave type ids.
     * @data [leaveTypeTotals] map from leave type to user's leave time requests time.
     * @data [dateFrom] first calendar to show
     * @data [period] number of displayed calendars
     * @data [requests] leave time map
     * @data [workdayDurations] users workdayDurations for all selected time
     *
     * */
    updateData: function(data)
    {
        var defaultWorkSchedule = APReportsPage.PastLeavesPage.Store.getDefaultWorkSchedule();
        this.nonWorkingDays = {};
        for (var i = 0; i < 7; i++)
        {
            if (defaultWorkSchedule[i] == 0)
                this.nonWorkingDays[i] = true;
        }

        var totalPtoStrings = this.formatPtoTimeTotal(data.leaveTimeTotal);
        this.totalLeaveTime.html(totalPtoStrings.mainValue);
        this.totalLeaveTimeAdditional.html("(" + totalPtoStrings.convertedValue + ")");

        this.updateBalanceTypesTable(data.dateFrom,
                                     data.dateTo,
                                     data.balancesOnStartDate,
                                     data.balancesOnFinishDate,
                                     data.spentBalanceInfo,
                                     data.ptoHoursPerDay);

        var sortedByPosLeaveTypes;
        var displayEmptyRows = false;
        if (data.leaveTypeIds && data.leaveTypeIds.length > 0)
        {
            sortedByPosLeaveTypes = $.grep(
                    LEAVE_TYPES_ORDERED,
                    function(leaveType)
                    {
                        return $.inArray(leaveType.id, data.leaveTypeIds) !== -1;
                    }
            );
            displayEmptyRows = true;
        }
        else
        {
            sortedByPosLeaveTypes = LEAVE_TYPES_ORDERED;
        }

        this.updateLeaveTypesTable(sortedByPosLeaveTypes, data.leaveTypeTotals, displayEmptyRows);

        var calendars = this._convertCalendars(data.dateFrom, data.period);
        var arrayLength = calendars.length;
        var isSingle = ( arrayLength == 1 );

        this.calendarContainer.toggleClass("singleCalendar", isSingle);
        this.calendarContainer.toggleClass("odd", (arrayLength % 2) == 1);
        this.calendarContainer.toggleClass("threeCalendar", (arrayLength == 3));

        this.deleteCalendars();
        var workdayDurationsCount = 0;
        for (i = 0; i < arrayLength; i++)
        {
            var calendarConfig = this.createCalendarConfig(calendars[i].year, calendars[i].month, data.requests, data.usersToday);
            calendarConfig.workdayDurations = data.workdayDurations.slice(workdayDurationsCount, workdayDurationsCount + calendarConfig.daysNumber);
            workdayDurationsCount += calendarConfig.daysNumber;

            var calendarEl = this.createCalendar(isSingle || this.isWideContainer(), calendarConfig);
            calendarEl.appendTo(this.calendarContainer);
        }
        this.calendarContainer.show();
    },

    deleteCalendars: function()
    {
        this.calendarContainer.hide();
        this.calendarContainer.html("");
    },

    _convertCalendars: function(dateFrom, period)
    {
        var result = [];

        var month = dateFrom.getMonth();
        var year = dateFrom.getFullYear();

        for (var i = 0; i < period; i++)
        {
            result[i] = {month: month, year: year};
            month = ( month + 1 ) % 12;
            year += (month == 0) ? 1 : 0;
        }

        return result;
    },

    createLeaveTypeRow: function()
    {
        return this.leaveTypesTemplateTable.find(".leaveTypeDescription").clone();
    },

    updateLeaveTypesTable: function(leaveTypes, leaveTypeTotals, displayEmptyRows)
    {
        var self = this;
        var content = this.leaveTypesTableContent;
        content.hide();
        content.find("tr").remove();
        $.each(leaveTypes, function(i, leaveType)
        {
            var leaveTypeId = leaveType.id;
            var totalTimeStr = "-";
            var convertedTimeStr = "";
            var empty = false;

            if (leaveTypeTotals[leaveTypeId] && leaveTypeTotals[leaveTypeId].hours > 0)
            {
                var totalPtoStrings = self.formatPtoTimeTotal(leaveTypeTotals[leaveTypeId]);
                totalTimeStr = totalPtoStrings.mainValue + " ";
                convertedTimeStr = "(" + totalPtoStrings.convertedValue + ")";
            }
            else
            {
                if (!displayEmptyRows)
                    return;
                empty = true;
            }

            var newRow = self.createLeaveTypeRow();
            newRow.toggleClass("empty", empty);
            newRow.toggleClass("archived", !leaveType.active);
            newRow.appendTo(content);
            newRow.find(".leaveTypeIcon").addClass("leaveType" + leaveType.iconId);
            newRow.find(".leaveTypeName").text(leaveType.name);;

            //newRow.find(".leaveTypeIcon .display").find("img").attr("src", getLeaveTypeIconPath(leaveType.iconId));
            newRow.find(".leaveTypeIcon .for-print-only").find("img").attr("src", getLeaveTypePrintIconPath(leaveType.iconId));

            var newRowEl = newRow.find(".leaveTypeTime");
            newRowEl.find(".mainValue").text(totalTimeStr);
            newRowEl.find(".additionalValue").text(convertedTimeStr);
        });

        content.show();
        ScrollUtils.refreshCustomScroll(this.scroll);
    },

    updateBalanceTypesTable: function(dateFrom, dateTo, balancesInfoOnFromDate, balancesInfoOnToDate, spentBalanceInfo, countWorkHoursPerDay)
    {
        new BalanceTypesTable().init("balanceTypesTable", {
            dateFrom: dateFrom,
            dateTo: dateTo,
            balancesInfoOnFromDate: balancesInfoOnFromDate,
            balancesInfoOnToDate: balancesInfoOnToDate,
            spentBalanceInfo: spentBalanceInfo,
            countWorkHoursPerDay: countWorkHoursPerDay
        });
    },

    formatPtoTimeTotal: function(ptoTotal)
    {
        var showInHours = false; // do not use SystemSettings.showPTOInHours since it is leave time not pto;
        var ptoInHours = this.hoursFormatter.format(ptoTotal.hours);
        var ptoInDays = this.daysFormatter.format(ptoTotal.days);

        var ptoString = {};
        ptoString.mainValue = showInHours ? ptoInHours : ptoInDays;
        ptoString.convertedValue = !showInHours ? ptoInHours : ptoInDays;

        return ptoString;
    }
});