function UserManagementResultHandler(row, updateRowFunction, customSuccessHandler, customErrorHandler, exceptionMapExtender) {

    var self = this;
    this._updateRow  = updateRowFunction;

    this._exceptionMap = $.extend(true,{},UserManagementResultHandler._defaultExceptionMap, exceptionMapExtender);

    this.successHandler = function( userManagementResult ) {

        if ( customSuccessHandler )
            customSuccessHandler(userManagementResult);

        self._successHandlerBase(userManagementResult, row);
    };

    this.errorHandler = function( jqXHR, textStatus, errorThrown ) {

        if ( customErrorHandler )
            if ( customErrorHandler( jqXHR, textStatus, errorThrown ) )
                return true;

        return self._errorHandlerBase(errorThrown, row);
    }

}

UserManagementResultHandler.prototype._handleLicenseError = function() {
    //do nothing
};

UserManagementResultHandler.prototype._successHandlerBase = function( userManagementResult, row ) {
    if( row )
    {
        var userInfo =  userManagementResult.userInfo;
        row.data().record = userInfo;
        row.data().userApprovers = userManagementResult.userApprovers;
        row.data().warnings = userInfo.warnings;
    }
    this._updateRow(row);
};

UserManagementResultHandler.prototype._errorHandlerBase = function( errorThrown, row ) {
    var errorHandlerFunction = this._exceptionMap[errorThrown.exception];
    return errorHandlerFunction ? errorHandlerFunction.call(this, row, errorThrown) : false;
};

UserManagementResultHandler._defaultExceptionMap = {};

UserManagementResultHandler._defaultExceptionMap['com.actimind.actitime.services.NoRightsForRemoteCallException'] = function( row ) {
    alert(messageResource.getMessage("error.js.json_rpc.no_rights"));
    document.location.reload();
    return true;
};

UserManagementResultHandler._defaultExceptionMap['com.actimind.actitime.services.NoRightsToManageAccountSettings'] = function( row ) {
    alert(messageResource.getMessage("actiplans.user_list.noRightsForUserAccount"));
    document.location.reload();
    return true;
};

UserManagementResultHandler._defaultExceptionMap['com.actimind.actitime.administration.NoSuchUserException'] = function( row ) {
    alert(messageResource.getMessage('actiplans.user_list.error_user_was_deleted'));
    UserListPage.removeRow( row );
    return true;
};

UserManagementResultHandler._defaultExceptionMap['com.actimind.actitime.administration.UserIsLockedException'] = function( row ) {
    alert(messageResource.getMessage('actiplans.user_list.user_is_locked'));
    return true;
};

UserManagementResultHandler._defaultExceptionMap['com.actimind.actitime.administration.NoSuchUserGroupException'] = function( row ) {
    alert(messageResource.getMessage('actiplans.user_list.error_user_group_was_deleted'));
    document.location.reload();
    return true;
};

UserManagementResultHandler._defaultExceptionMap['com.actimind.actitime.licensing.MaxNumOfActiveUsersReachedException'] = function( row ) {
    alert(messageResource.getMessage('actiplans.user_list.error_max_users'));
    this._handleLicenseError();
    return true;
};

UserManagementResultHandler._defaultExceptionMap['com.actimind.common.licensing.LicenseViolatedException'] = function( row ) {
    alert(messageResource.getMessage('actiplans.user_list.error_license_violated'));
    this._handleLicenseError();
    return true;
};

UserManagementResultHandler._defaultExceptionMap['com.actimind.common.administration.UserPTOControlTurnedOff'] = function( row ) {
    alert(messageResource.getMessage('actiplans.user_list.error_pto_control_disabled'));
    document.location.reload();
    return true;
};

UserManagementResultHandler._defaultExceptionMap['com.actimind.common.administration.UserSickBalanceControlTurnedOff'] = function( row ) {
    alert(messageResource.getMessage('actiplans.user_list.error_sick_days_control_disabled'));
    document.location.reload();
    return true;
};

UserManagementResultHandler._defaultExceptionMap['com.actimind.actiplans.services.UserManagementService$HireDateInFutureException'] = function( row ) {
    alert(messageResource.getMessage('actiplans.user_list.error_hire_date_in_future'));
    return true;
};