function LeaveTimeRequestsPage()
{

}

LeaveTimeRequestsPage.prototype = {
    init: function(userSettings, pageData, canEditPastRequests, wasAtIntegrated, minRecordsPerPage, userGroups, userGroupsMap, tzGroups, tzGroupsMap, maxVisibleDays )
    {
        this.userGroups = userGroups;
        this.timeZoneGroups = tzGroups;
        this.userGroupsMap = userGroupsMap;
        this.timeZoneGroupsMap = tzGroupsMap;
        this.canEditPastRequests = canEditPastRequests;
        this.wasAtIntegrated = wasAtIntegrated;
        this.currentUserActiveInAT = pageData.currentUserActiveInAT;
        this.allSelectedRequestIdsSet = {};
        this.ptoHoursPerDay = data.ptoHoursPerDay;

        this.rowClassesByStatus = _.create();
        this.rowClassesByStatus[LeaveTimeRequestStatus.REQUESTED] = "requested";
        this.rowClassesByStatus[LeaveTimeRequestStatus.APPROVED] = "approved";
        this.rowClassesByStatus[LeaveTimeRequestStatus.REJECTED] = "rejected";

        this.rowTextByStatus = _.create();
        this.rowTextByStatus[LeaveTimeRequestStatus.REQUESTED] = messageResource.getMessage('actiplans.request_status.waiting_for_approval');
        this.rowTextByStatus[LeaveTimeRequestStatus.APPROVED] = messageResource.getMessage('actiplans.request_status.approved');
        this.rowTextByStatus[LeaveTimeRequestStatus.REJECTED] = messageResource.getMessage('actiplans.request_status.rejected');

        this.initBalanceFormatters(this.ptoHoursPerDay);

        this.stateController = new StateController().init({
                                                              sortBy: userSettings["tableSorting"]["sortByStr"],
                                                              isAscending: userSettings["tableSorting"]["asc"],
                                                              statusFilter: userSettings["statusFilter"],
                                                              leaveTypeFilter: userSettings["leaveTypesFilter"],
                                                              groupFilter: userSettings["groupsFilter"],
                                                              groupFilterType: userSettings["groupsFilterType"],
                                                              balanceTypeFilter: userSettings["balanceTypeFilter"],
                                                              pageNum: 1,
                                                              recordsPerPage: userSettings["recordsPerPage"],
                                                              userNameFilterText: "",
                                                              displayOnlyAssignedToMeUsers: userSettings["displayOnlyAssignedToMeUsers"],
                                                              showPastApprovedRejectedRequests: userSettings["showPastApprovedRejectedRequests"]
                                                          });
        this.stateController.addOnLoadStateActions(_.bind(this.updateUIFromState, this));
        this.stateController.addOnSaveStateActions(_.bind(this.onSaveStateCallback, this));

        this.currentTotalRecords = pageData["totalRecords"];
        this.initPagerController(pageData["totalRecords"], minRecordsPerPage);
        this.spinner = new SpinnerController();

        // init buttons for process several selected requests
        this.initSelectedRequestsProcessButtons();

        this.initLeaveTypeSelector(userSettings["leaveTypesFilter"]);

        this.initUserNameFilter();

        this.initStatusFilter(userSettings.statusFilter);

        this.initBalanceTypeFilter(userSettings.balanceTypeFilter);

        this.initGroupsFilter(userGroups, timeZoneGroups, userSettings.groupsFilter, userSettings["groupsFilterType"]);

        this.daysFormatter =
                new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 1, true, '', 'd');
        this.hoursFormatter =
                new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 2, true, '', 'h');
        this.usernameHighlighter = new EmptyHighlighter();

        this.invocationResultHandler = new InvocationResultHandler();

        this.initHeaderControls(userSettings.statusFilter);

        this.notApplicableRequestTooltip = new LeaveTimeRequestsPageTooltip($("#notApplicableRequestTooltip"));
        this.requestAutoApprovalTooltip = new LeaveTimeRequestsPageTooltip($("#requestAutoApprovalTooltip"));
        this.rejectedRequestTooltip = new LeaveTimeRequestsPageTooltip($("#rejectedRequestTooltip"));
        this.waitingDeletionRequestTooltip = new LeaveTimeRequestsPageTooltip($("#waitingDeletionRequestTooltip"));
        this.wasAtIntegratedTooltip = new LeaveTimeRequestsPageTooltip($("#wasAtIntegratedTooltip"));
        this.cannotEditPastRequestTooltip = new LeaveTimeRequestsPageTooltip($("#cannotEditPastRequestTooltip"));

        // render page
        this.onGetRequestsComplete(data);

        $.each($("table.requests > thead > tr > td"), function()
        {
            disableTextSelection(this);
        });

        this.corporateScheduleWidthCalculator = new CorporateScheduleWidthCalculator(maxVisibleDays);
        $(window).resize(_.bind(this.fixTableWidth, this));
        this.fixTableWidth();
    },

    getSelectedRequestsData: function()
    {
        if (_.isUndefined(this.selectedRequestsData))
        {
            this.selectedRequestsData = {};
            this.selectedRequestsData["selected"] = [];
            this.selectedRequestsData["selectedWaitingForApproval"] = [];
            this.selectedRequestsData["selectedApproved"] = [];
            this.selectedRequestsData["selectedRejected"] = [];
            this.selectedRequestsData["selectedWaitingDeletion"] = [];
        }
        return this.selectedRequestsData;
    },

    setSelectedRequestsData: function(selectedData)
    {
        this.selectedRequestsData = selectedData;
    },

    fixTableWidth: function()
    {
        $('#corporateScheduleTable').css('width', this.corporateScheduleWidthCalculator.getContentWidth());
    },

    initPagerController: function(currentTotalRecords, minRecordsPerPage)
    {
        var pagerData = {
            totalRecords: currentTotalRecords,
            recordsPerPage: this.stateController.getState().recordsPerPage,
            pageNum: this.stateController.getState().pageNum
        };

        var self = this;
        this.pagerController = new PagerController().init(
                {
                    pagerData: pagerData,
                    handlersMap: {
                        pageChanged: function(pageNum)
                        {
                            self.stateController.getState().pageNum = pageNum;
                            self.makeGetRequestsRpcCall();
                        },
                        recordsPerPageChanged: function(recordsPerPage)
                        {
                            self.stateController.getState().recordsPerPage = recordsPerPage;
                            self.pagerController.loadPage(1);
                        }
                    },
                    pagerControlsElementId: "pagerPanel",
                    pagerElementId: "pager",
                    recordsPerPageSelectorId: "recordsPerPageSelector",
                    minRecordsPerPage: minRecordsPerPage
                }
        );
    },

    updatePagerController: function(currentTotalRecords)
    {
        this.pagerController.setPagerStateAndUpdateUI(
                currentTotalRecords,
                this.stateController.getState().pageNum,
                this.stateController.getState().recordsPerPage
        );
        this.currentTotalRecords = currentTotalRecords;
        $('#totalRecordsPlaceholder').text(currentTotalRecords);
    },

    onApproveOrRejectDialogOkClick: function(rpcMethod, selectedRequestsForChangeStatus, selectedRequestForChangeDeletionStatus, comment)
    {
        var self = this;
        var selectedUserIds = [];
        var usersRequestIds = [];

        _.each(_.clone(selectedRequestsForChangeStatus).concat(selectedRequestForChangeDeletionStatus), function(request)
        {
            if (_.indexOf(selectedUserIds, request.userId) == -1)
            {
                selectedUserIds.push(request.userId);
                usersRequestIds = usersRequestIds.concat(self.userIdRequestIdsMap[request.userId]);
            }
        });

        var onComplete = function(data)
        {
            self.onApproveOrRejectComplete(data, usersRequestIds);
            PageOverlayTipsManager.getInstance().refreshDynamicTips();
        };

        this.prepareInitialUpdateRequest(function()
                                         {
                                             return makeRpcCall(rpcMethod,
                                                                [self.getRequestIds(selectedRequestsForChangeStatus), self.getRequestIds(
                                                                        selectedRequestForChangeDeletionStatus), usersRequestIds, comment],
                                                                onComplete,
                                                                new CorporateScheduleErrorHandler(
                                                                        _.bind(self.makeGetRequestsRpcCall, self),
                                                                        null,
                                                                        _.bind(self.stopSpinner, self)
                                                                ).multipleUsersErrorHandler);
                                         });
    },

    //TODO get data for requests ???
    initSelectedRequestsProcessButtons: function()
    {
        var self = this;

        // -- Approve dialog --
        //TODO dialog not used ???
        /* var approveDialogTextarea = $("#approveDialogDiv").find("textarea" );
         var approveDialog = new ProcessSelectedRequestsDialog().init('#approveDialogDiv', 'processSelectedRequestsDialog approve-ui-dialog',
         '.buttonsPlaceholder', function(){});

         $("#approveDialogDiv textarea").attr('placeholder', approveRejectPlaceholderText).placeholder();
         var approveDialogHandler = function() {
         this.onApproveOrRejectDialogOkClick('CorporateRequestsService.approve', selectedWaitingForApproval, selectedWaitingDeletion, trim( approveDialogTextarea.pVal() ));
         approveDialog.close();
         return true;
         };

         var _approveTextareaManager = new TextAreaLengthManager();
         _approveTextareaManager.init( approveDialogTextarea.get(0), maxCommentLength, longCommentTitle );
         _approveTextareaManager.setDependentButton($("#approveDialogDiv").find(".okButton" ).get(0), approveDialogHandler);*/


        $('.acctionButton.approve').click(function()
                                          {
                                              if (!$(this).hasClass("disabled"))
                                              {
                                                  var selectedData = self.getSelectedRequestsData();
                                                  self.onApproveOrRejectDialogOkClick('CorporateRequestsService.approve',
                                                                                      selectedData.selectedWaitingForApproval,
                                                                                      selectedData.selectedWaitingDeletion,
                                                                                      "");
                                              }
                                          });

        // -- Reject dialog --
        var rejectDialog = new RejectSelectedRequestsDialog($("#rejectDialogDiv"), function()
        {
            var selectedData = self.getSelectedRequestsData();
            self.onApproveOrRejectDialogOkClick('CorporateRequestsService.reject',
                                                selectedData.selectedWaitingForApproval.concat(selectedData.selectedApproved),
                                                selectedData.selectedWaitingDeletion,
                                                rejectDialog.getComment());
            rejectDialog.close();
            return true;
        }).init();

        $('.acctionButton.reject').click(function()
                                         {
                                             if (!$(this).hasClass("disabled"))
                                                 rejectDialog.show($('.acctionButton.approve'), self.getSelectedRequestsData());
                                         });

        // -- Delete dialog --
        var deleteDialog = new DeleteSelectedRequestsDialog($('#deleteDialogDiv'), function()
        {
            self.prepareInitialUpdateRequest(function()
                                             {
                                                 var selectedData = self.getSelectedRequestsData();
                                                 var requestsForDeletion = _.filter(selectedData.selected,
                                                                                    function(request) {return !request.waitingDeletion});
                                                 return makeRpcCall('CorporateRequestsService.delete',
                                                                    [self.getRequestIds(requestsForDeletion)],
                                                                    _.bind(self.onBulkDeleteComplete, self),
                                                                    new CorporateScheduleErrorHandler(
                                                                            _.bind(self.makeGetRequestsRpcCall, self),
                                                                            null,
                                                                            _.bind(self.stopSpinner, self)).multipleUsersErrorHandler);
                                             });
            deleteDialog.close();
            return true;
        }).init();

        $('.acctionButton.delete').click(function()
                                         {
                                             if (!$(this).hasClass("disabled"))
                                                 deleteDialog.show($('.acctionButton.approve'), self.getSelectedRequestsData());
                                         });
    },

    /**
     * Init filter for leave types
     * @param leaveTypesFilter initial value
     */
    initLeaveTypeSelector: function(leaveTypesFilter)
    {
        var leaveTypes = $.extend(true, [], LEAVE_TYPES_ORDERED);
        var leaveTypesSelectorUtil = new LeaveTypesSelectorUtil().init(
                leaveTypes,
                SystemSettings.apPtoBalanceCalculationEnabled,
                SystemSettings.apSickBalanceCalculationEnabled,
                leaveTypesFilter
        );

        var ptoName = messageResource.getMessage("leaves_selector.group_label.pto_balance");
        var sdbName = messageResource.getMessage("leaves_selector.group_label.sick_days_balance");
        var bothName = SystemSettings.apPtoBalanceCalculationEnabled && SystemSettings.apSickBalanceCalculationEnabled ?
                       messageResource.getMessage("leaves_selector.group_label.both") : SystemSettings.apPtoBalanceCalculationEnabled ? ptoName : sdbName;

        leaveTypes = leaveTypesSelectorUtil.getDataForSelector(
                {
                    ptoLabel: messageResource.getMessage('leaves_selector.group_label.affects', [ptoName]),
                    sickDaysLabel: messageResource.getMessage('leaves_selector.group_label.affects', [sdbName]),
                    withoutBalancesLabel: messageResource.getMessage('leaves_selector.group_label.does_not_affect', [bothName])
                }
        );
        var leavePanel = AT.APLeaveTypesSelectorPanel;
        leaveTypesSelectorUtil.setSelectorTreeConstructorToPanel(leavePanel);

        var onLeaveTypeFilterChange = function(newValue)
        {
            this.stateController.getState().leaveTypeFilter = newValue;
            this.pagerController.loadPage(1);
        };

        this.leaveTypeFilter = new ItemSelector("table.requests > thead .leaveTypeFilter", _.bind(onLeaveTypeFilterChange, this),
                leavePanel, leaveTypes, leaveTypesSelectorUtil.isNeedToShowInactiveLeaves());

    },

    /**
     * Filter for user names
     */
    initUserNameFilter: function()
    {
        var $searchField = $('#requestsTableContainer_userNameFilter'); 
        var userNameFilterPlaceholder = messageResource.getMessage("actiplans.leavestable.start_typing_name");
        $searchField.attr('placeholder', userNameFilterPlaceholder).placeholder();

        var onUserNameFilterChanged = function(newValue)
        {
            this.abortPreparedAndActiveRequests();
            this.stateController.getState().userNameFilterText = newValue;
            this.pagerController.loadPage(1);
        };
        this.userNameFilter = new LeaveRequestsUserNameFilter($searchField, _.bind(onUserNameFilterChanged, this));
    },

    /**
     * Filter for request statuses (APPROVED, REJECTED, etc)
     * @param initStatusValue initial filter value
     */
    initStatusFilter: function(initStatusValue)
    {
        var self = this;
        var onStatusFilterChange = function(newValue)
        {
            var waitingApproval = !_.isNull(newValue) && newValue[0] == 0;
            if (waitingApproval)
            {
                self.$showPastApprovedRejectedRequestsChBox.prop("checked", false);
                self.stateController.getState().showPastApprovedRejectedRequests = false;
            }
            self.$showPastApprovedRejectedRequestsChBox.prop("disabled", waitingApproval);
            self.stateController.getState().statusFilter = newValue;
            self.pagerController.loadPage(1);
        };
        this.statusFilter = new LeaveRequestsStatusFilter("table.requests > thead .statusFilter", onStatusFilterChange, initStatusValue);
    },

    /**
     * Filter for balace type PTO or SDB
     * @param initialBalanceFilterValue initial value for filter
     */
    initBalanceTypeFilter: function(initialBalanceFilterValue)
    {
        var leaveBalanceEnabled = SystemSettings.apPtoBalanceCalculationEnabled || SystemSettings.apSickBalanceCalculationEnabled;
        if (leaveBalanceEnabled)
        {
            if (SystemSettings.apPtoBalanceCalculationEnabled && SystemSettings.apSickBalanceCalculationEnabled)
            {
                var onBalanceTypeFilterChange = function(newValue)
                {
                    this.stateController.getState().balanceTypeFilter = newValue;
                    this.pagerController.loadPage(1);
                };

                new LeaveRequestsBalanceTypeFilter("table.requests > thead .balanceTypeFilter", _.bind(onBalanceTypeFilterChange, this), initialBalanceFilterValue);
                $(".balanceHeaderContainer").text(messageResource.getMessage("actiplans.leave_time_requests.balances") + "\n")
                        .append($("<span class='beforeAfterReqSpan'>" + messageResource.getMessage("actiplans.leave_time_requests.before_after_req") + "</span>"));
                $(".balanceTypeFilterBlock").addClass("smallTopMargin");
            }
            else
            {
                var headerTextKey = SystemSettings.apPtoBalanceCalculationEnabled ? "actiplans.leave_time_requests.pto" : "actiplans.leave_time_requests.sick";
                $(".balanceHeaderContainer").text(messageResource.getMessage(headerTextKey))
                        .append($("<span class='smaller'>" + messageResource.getMessage("actiplans.leave_time_requests.before_after_req") + "</span>"));
                $(".balanceTypeFilterBlock").hide();
            }
        }
        else
        {
            $(".corporateRequestsPage").addClass("withoutBalancesColumn");
            $(".corporateRequestsPage table.requests col.balance.before").remove();
            $(".corporateRequestsPage table.requests col.balance.after").remove();
        }
    },

    /**
     * Init filter for user and time zone groups
     * @param userGroups user groups data or null if UGR feature off
     * @param timeZoneGroups time zone groups or null if TZG feature off
     * @param groupsFilterValue selected groups ids
     * @param groupsFilterTypeValue type of selected groups
     */
    initGroupsFilter: function(userGroups, timeZoneGroups, groupsFilterValue, groupsFilterTypeValue)
    {
        if (userGroups && groupsFilterTypeValue == GroupType.USER_GROUP)
        {
            _.each(userGroups, function(group)
            {
                if (_.indexOf(groupsFilterValue, group.id) != -1)
                    group.selected = true;
            });
        }
        if (timeZoneGroups)
        {
            _.each(timeZoneGroups, function(group)
            {
                if (group.id == DEFAULT_TIME_ZONE_GROUP_ID)
                {
                    group.isDefault = true;
                    return false;
                }
            });

            if (groupsFilterTypeValue == GroupType.TIME_ZONE_GROUP)
            {
                _.each(timeZoneGroups, function(group)
                {
                    if (_.indexOf(groupsFilterValue, group.id) != -1)
                        group.selected = true;
                });
            }
        }

        if (!userGroups && !timeZoneGroups)
        {
            $("#requestsTableContainer_userGroupFilter").hide();
        }

        if (!userGroups)
        {
            $("#sortByUserGroupLink").hide();
            $("#sortByUserOrUserGroupSeparator").hide();
        }
        if (!timeZoneGroups)
        {
            $("#sortByTimeZoneGroupLink").hide();
            $("#sortByUserGroupOrTimeZoneGroupSeparator").hide();
        }

        if (userGroups || timeZoneGroups)
        {
            var self = this;
            var onGroupFilterChange = function(newValue)
            {
                self.stateController.getState().groupFilter = newValue;
                self.stateController.getState().groupFilterType = self.groupsFilter.getPanel().getGroupType();
                self.pagerController.loadPage(1);
            };
            this.groupsFilter = new ItemSelector("table.requests > thead .userGroupFilter", _.bind(onGroupFilterChange, this),
                    AT.SmartGroupsSelectorPanel, null, false, {userGroupsData: userGroups, timeZoneGroupsData: timeZoneGroups, groupType: groupsFilterTypeValue}, true)
        }
    },

    onSaveStateCallback: function()
    {
        this.leaveTypeFilter.saveState();
        if (this.groupsFilter)
            this.groupsFilter.saveState();
    },

    getRequestIds: function(requests)
    {
        var ids = [];
        _.each(requests, function(req) { ids.push(req.id) });
        return ids;
    },

    updateUIFromState: function(savedState)
    {
        this.userNameFilter.setText(savedState.userNameFilterText);
        this.statusFilter.setActiveItemByValue(savedState.statusFilter == null ? -1 : savedState.statusFilter, true);
        this.leaveTypeFilter.loadState();
        if (this.groupsFilter)
            this.groupsFilter.loadState();
        this.updateSortingIndicators();
        this.updateDisplayOnlyAssignedToMeUsersChBox();
        this.updateShowPastApprovedRejectedRequestsCheckbox();
        this.pagerController.setPagerStateAndUpdateUI(
                this.currentTotalRecords,
                savedState.pageNum,
                savedState.recordsPerPage
        );
    },

    stopSpinner: function()
    {
        this.spinner.stop();
    },

    //TODO set values ???
    abortPreparedAndActiveRequests: function()
    {
        if (this.filterRpcCall)
            this.filterRpcCall.abort();

        if (this.updateRpcCall)
            this.updateRpcCall.abort();

        this.userNameFilter.abortTimer();
    },

    prepareInitialRequest: function()
    {
        this.abortPreparedAndActiveRequests();
        this.spinner.stop();
        this.spinner.spin("leavesManagementPageSpinner");
    },

    onApproveOrRejectComplete: function(data, requestIdsToUpdate, dontClearSelection)
    {
        this.stopSpinner();
        this.showAlertIfNeededOnApproveOrReject(data);
        this.updateRows(requestIdsToUpdate, data.records);
        if (!dontClearSelection)
            this.clearSelectionOnAllPages();
    },

    prepareInitialUpdateRequest: function(rpcRequest)
    {
        if (this.filterRpcCall)
            this.stateController.loadState();
        this.prepareInitialRequest();

        this.filterRpcCall = null;
        this.updateRpcCall = rpcRequest();
    },

    prepareInitialFilterRequest: function(rpcRequest)
    {
        this.prepareInitialRequest();
        this.filterRpcCall = rpcRequest();
        this.updateRpcCall = null;
    },

    onBulkDeleteComplete: function(data)
    {
        this.showAlertIfNeededOnDeleteRequest(data);
        this.makeGetRequestsRpcCall();
        this.clearSelectionOnAllPages();
    },

    showAlertIfNeededOnDeleteRequest: function(data)
    {
        var totalErrors = data["fullyExpiredCount"];
        var totalPartiallyProcessed = data["partiallyDeletedCount"];

        if (totalErrors === 0 && totalPartiallyProcessed === 0)
            return;

        var message = "";
        if (totalErrors !== 0)
        {
            var total = data["totalCount"];
            message = ( total === 1 ) ?
                      messageResource.getMessage("actiplans.leave_time_requests.delete_requests.expired_request") :
                      messageResource.getMessage("actiplans.leave_time_requests.delete_requests.expired_requests", [totalErrors, total]);
        }

        if (totalErrors !== 0 && totalPartiallyProcessed !== 0)
            message += "\n";

        if (totalPartiallyProcessed !== 0)
        {
            message += ( totalPartiallyProcessed === 1 ) ?
                       messageResource.getMessage("actiplans.leave_time_requests.process_selected_dialog.request_deleted_partially") :
                       messageResource.getMessage("actiplans.leave_time_requests.process_selected_dialog.requests_deleted_partially", [totalPartiallyProcessed]);
        }

        if (this.canEditPastRequests)
        {
            message += "\n" + messageResource.getMessage("actiplans.leave_time_requests.process_selected_dialog.concurrent_change_of_rights");
        }

        alert(message);
    },

    getMessageForErrorState: function(message, stateCount, stateKeySuffix, total, totalErrors, concurrentChangeOfRights)
    {
        if (total == 1 && stateCount == 1)
        {
            message = messageResource.getMessage("actiplans.leave_time_requests.process_selected_dialog.one_request_" + stateKeySuffix);
            if (concurrentChangeOfRights)
            {
                message += "\n" + messageResource.getMessage("actiplans.leave_time_requests.process_selected_dialog.concurrent_change_of_rights");
            }
        }
        else if (total > 1 && totalErrors > 0)
        {
            if (totalErrors - stateCount == 0)
            {
                if (stateCount == 1)
                    message = messageResource.getMessage("actiplans.leave_time_requests.process_selected_dialog.request_" + stateKeySuffix, [total]);
                else
                    message = messageResource.getMessage("actiplans.leave_time_requests.process_selected_dialog.requests_" + stateKeySuffix, [stateCount, total]);
            }
            else
            {
                if (message == "")
                    message = messageResource.getMessage("actiplans.leave_time_requests.process_selected_dialog.many_errors", [totalErrors, total]) + "\n";

                if (stateCount == 1)
                    message += "\n" + messageResource.getMessage("actiplans.leave_time_requests.process_selected_dialog.one_of_requests_" + stateKeySuffix, [stateCount]);
                else if (stateCount > 1)
                    message += "\n" + messageResource.getMessage("actiplans.leave_time_requests.process_selected_dialog.many_of_requests_" + stateKeySuffix, [stateCount, total]);
            }

            if (concurrentChangeOfRights)
            {
                message += "\n" + messageResource.getMessage("actiplans.leave_time_requests.process_selected_dialog.concurrent_change_of_rights");
            }
        }
        return message;
    },

    showAlertIfNeededOnApproveOrReject: function(data)
    {
        var total = data["totalCount"];
        var totalErrors = data["nonexistentCount"] + data["incompatibleStatusCount"] + data["alreadyProcessedCount"] + data["expiredCount"];

        var message = "";
        message = this.getMessageForErrorState(message, data["nonexistentCount"], "not_exists", total, totalErrors, false);
        message = this.getMessageForErrorState(message, data["incompatibleStatusCount"] + data["alreadyProcessedCount"], "already_processed", total, totalErrors, false);
        message = this.getMessageForErrorState(message, data["expiredCount"], "already_started", total, totalErrors, this.canEditPastRequests);
        
        if (message !== "")
            alert(message);
    },

    updateRows: function(requestIds, requestWithUserInfoRecords)
    {
        var requestIdRequestDataMap = {};

        _.each(requestWithUserInfoRecords, function(record)
        {
            requestIdRequestDataMap[record.request.id] = record;
        });

        var removedRequestIds = [];
        for (var i = 0; i < requestIds.length; i++)
        {
            var requestId = requestIds[i];
            var row = $(this.requestIdRowElementMap[requestId]);

            if (!requestIdRequestDataMap.hasOwnProperty(requestId))
            {
                delete this.allSelectedRequestIdsSet[requestId];
                row.remove();
                removedRequestIds.push(requestId);
            }
            else
            {
                this.setRowData(row, requestIdRequestDataMap[requestId]);
            }
        }

        if (removedRequestIds.length > 0)
            this.removeNotExistingRequestsAndRedrawTable(removedRequestIds, requestIds, requestIdRequestDataMap);

        this.onSelectionChanged();
    },

    removeNotExistingRequestsAndRedrawTable: function(deletedRequestIds, updatedRequests, requestsMap)
    {
        var records = this._data.records;
        var newRecords = [];
        for (var i = 0; i < records.length; i++)
        {
            var id = records[i].request.id;
            if ($.inArray(id, deletedRequestIds) == -1)
            {
                // if not deleted and updated
                if ($.inArray(id, updatedRequests) != -1)
                    records[i] = requestsMap[id];

                // if not deleted
                newRecords.push(records[i]);
            }
        }

        this._data.records = newRecords;
        this._data.totalRecords -= deletedRequestIds.length;

        this.onGetRequestsComplete(this._data);
    },

    onSelectionChanged: function()
    {
        var selected = [];

        $('table.requests > tbody.actualContent > tr').removeClass("selected");

        var self = this;
        $('table.requests > tbody.actualContent td.select input:checked').each(function()
                                                                               {
                                                                                   var row = self.getParentRequestRow(this);
                                                                                   selected.push(row.data());
                                                                                   row.addClass("selected");
                                                                               });

        var selectedWaitingForApproval = [];
        var selectedApproved = [];
        var selectedRejected = [];
        var selectedWaitingDeletion = [];
        var onlyEditSelection = {
            approved: []
        };

        _.each(selected, function(data)
        {
            var req = data.request;
            if (req.waitingDeletion)
            {
                selectedWaitingDeletion.push(req);
            }
            else if (req.status == LeaveTimeRequestStatus.REQUESTED || req["requestChange"])
            {
                selectedWaitingForApproval.push(req);
            }
            else if (req.status == LeaveTimeRequestStatus.APPROVED)
            {
                if(data.onlyEditableByManager)
                    onlyEditSelection.approved.push(req);
                else
                    selectedApproved.push(req);
            }
            else if (req.status == LeaveTimeRequestStatus.REJECTED)
            {
                selectedRejected.push(req);
            }
        });

        var selectedData = new LeaveTimeRequestsPageSelectionWithMasks(selectedWaitingForApproval, selectedApproved, selectedRejected, selectedWaitingDeletion, onlyEditSelection);
        this.setSelectedRequestsData(selectedData);

        this.setActionButtonStatus($('.acctionButton.approve'), selectedData.approvableMasked);
        this.setActionButtonStatus($('.acctionButton.reject'), selectedData.rejectableMasked);
        this.setActionButtonStatus($('.acctionButton.delete'), selectedData.deletableMasked);

        var tableRequests = $('table.requests');
        var headSelectionCheckbox = tableRequests.find('thead td.select input');
        var rowSelectionCheckboxes = tableRequests.find('> tbody.actualContent td.select input');
        var allSelected = rowSelectionCheckboxes.length > 0 && rowSelectionCheckboxes.filter(":not(:checked)").length == 0;
        headSelectionCheckbox.prop("checked", allSelected);
    },

    setActionButtonStatus: function(buttonEl, maskedSelection)
    {
        var empty = maskedSelection.isEmpty();
        buttonEl.toggleClass("disabled", empty);
        buttonEl.toggleClass("enabled", !empty);
    },

    /**
     * main function for handle new recieved data
     * TODO may be it is possible to split this method
     * @param data actual data
     */
    onGetRequestsComplete: function(data)
    {
        this._data = data;
        this.stopSpinner();
        var self = this;

        // detect that features state is changed
        var userGroupsEnabled = !!this.userGroups;
        var timeZoneGroupsEnabled = !!this.timeZoneGroups;
        var alertMessage = undefined;
        if (data.userGroupsEnabled != userGroupsEnabled)
        {
            alertMessage = data.userGroupsEnabled ?
                           messageResource.getMessage("actiplans.common.js.error.user_groups_enabled") :
                           messageResource.getMessage("actiplans.common.js.error.user_groups_disabled");
        }
        else if (data.timeZoneGroupsEnabled != timeZoneGroupsEnabled)
        {
            alertMessage = data.timeZoneGroupsEnabled ?
                           messageResource.getMessage("actiplans.common.js.error.time_zone_groups_enabled") :
                           messageResource.getMessage("actiplans.common.js.error.time_zone_groups_disabled");
        }
        if (!_.isUndefined(alertMessage))
        {
            alert(alertMessage);
            document.location.reload();
            return;
        }

        this.stateController.getState().pageNum = data.pageNum;
        this.ptoHoursPerDay = data.ptoHoursPerDay;
        this.initBalanceFormatters(data.ptoHoursPerDay);

        this.setAssignedToMeCheckBox(data["showAssignedToMeOption"]);
        this.updateShowPastApprovedRejectedRequestsCheckbox();

        this.stateController.saveState();

        $("table.requests > tbody.actualContent tr").remove();

        var groupingByUserGroupEnabled = !!userGroups && this.stateController.getState().sortBy == "USER_GROUP";
        var groupingByTimeZoneGroupEnabled = !!timeZoneGroups && this.stateController.getState().sortBy == "TIME_ZONE_GROUP";
        var groupingEnabled = groupingByUserGroupEnabled || groupingByTimeZoneGroupEnabled;
        $("table.requests col.group").toggle(groupingEnabled);
        $("table.requests td.user.combined").attr("colspan", groupingEnabled ? 2 : 1);

        if (isIE8 && groupingEnabled)
        {
            // IE8 cannot correctly render grouping column if it was previously hidden
            // but this fix may cause unnecessary scrolling to top, so we only apply it in IE8
            $("table.requests").css("display", "none").css("display", "table");
        }

        var group = {};

        var STYLE_CONSTANTS = {
            minRowHeight: 53,
            groupPaddings: 10
        };

        function setupGroupCell(groupsMap, isLast)
        {
            group.cell.attr("rowspan", group.rowsCount);
            group.cell.children(".groupName").css("max-height",
                                                  group.rowsCount * STYLE_CONSTANTS.minRowHeight
                                                  - STYLE_CONSTANTS.groupPaddings
                                                  + "px");
            group.cell.children(".groupName").text(self.formatGroupVertical(groupsMap, group.id));
            group.cell.attr("title", self.formatGroup(groupsMap, group.id));
            group.cell.toggleClass('last', isLast);
        }

        if (data.records.length == 0)
        {
            $("table.requests tbody.templates tr.emptyRequest").clone().appendTo("table.requests tbody.actualContent");
        }

        this.userIdRequestIdsMap = {};
        this.requestIdRowElementMap = {};

        _.each(data.records, function(record)
        {
            if ((groupingByUserGroupEnabled && !self.userGroupsMap[record.userInfo.userGroupId]) ||
                (groupingByTimeZoneGroupEnabled && !self.timeZoneGroupsMap[record.userInfo.timeZoneGroupId]))
            {
                document.location.reload();
                return false;
            }

            var newRow = $("table.requests tbody.templates tr.request").clone().appendTo("table.requests tbody.actualContent");

            self.setRowData(newRow, record);

            if (self.allSelectedRequestIdsSet[record.request.id]) //TODO change this value
                newRow.find("td.select input").prop('checked', true);

            if (!self.userIdRequestIdsMap[record.userInfo.id])
                self.userIdRequestIdsMap[record.userInfo.id] = [];
            self.userIdRequestIdsMap[record.userInfo.id].push(record.request.id);

            self.requestIdRowElementMap[record.request.id] = newRow[0];

            if (groupingEnabled)
            {
                var groupId = groupingByUserGroupEnabled ? record.userInfo.userGroupId : record.userInfo.timeZoneGroupId;
                if (group.id != groupId)
                {
                    if (group.id)
                        setupGroupCell(groupingByUserGroupEnabled ? self.userGroupsMap : self.timeZoneGroupsMap, false);

                    group.id = groupId;
                    group.cell = newRow.find("td.user.group");
                    group.rowsCount = 1;

                    newRow.addClass("firstInGroup");
                }
                else
                {
                    newRow.find("td.user.group").hide(); // do not remove, just hide
                                                         // (to simplify table corners styling CSS)
                    group.rowsCount++;
                }
            }
            else
            {
                newRow.find("td.user.group").remove();
            }
        });

        //TODO refactor
        if (groupingEnabled && group.id)
        {
            setupGroupCell(groupingByUserGroupEnabled ? this.userGroupsMap : this.timeZoneGroupsMap, true);
        }


        this.updatePagerController(data.totalRecords);

        this.onSelectionChanged();

        PageOverlayTipsManager.getInstance().refreshDynamicTips(false);

        $('div#contentDiv').show();

        this.filterRpcCall = null;
    },

    initBalanceFormatters: function(ptoHoursPerDay)
    {
        this.ptoFormatter = new PTOFormatter().init(ptoHoursPerDay);
        this.sickFormatter = new SickFormatter().init(ptoHoursPerDay);
    },

    setRowData: function(row, corporateRequestsRowData)
    {
        var request = corporateRequestsRowData.request;
        var userInfo = corporateRequestsRowData.userInfo;

        row.data().request = request;
        row.data().userInfo = userInfo;
        row.data().revocableApprovedRequest = corporateRequestsRowData.revocableApprovedRequest;
        row.data().approvableByManager = corporateRequestsRowData.approvableByManager;
        row.data().onlyEditableByManager = corporateRequestsRowData.onlyEditableByManager;

        var leaveTypeId = LeaveTimeRequestUtils.getLeaveTypeIdFromRequestWithChanges(request);
        var leaveType = LEAVE_TYPES[leaveTypeId]; // values from cache

        if (!leaveType)
        {
            document.location.reload();
            return;
        }

        this.setEditRequestFunctionsForRow(row, corporateRequestsRowData);

        row.find("td.user.name").html(this.formatUserFullName(userInfo));

        var status = LeaveTimeRequestUtils.getStatusFromRequestWithChanges(request);

        var periodCellHtml = "<span class='dateText'>" + this.formatRequestPeriod(request) + "</span>";
        if (request.past)
        {
            row.toggleClass("pastRequest", true).toggleClass("waitingDeletion", status.waitingDeletion);
            periodCellHtml = "<div class='pastText'>" + messageResource.getMessage("requests_table.period_cell.past_text") + "</div>" + periodCellHtml
        }
        row.find("td.period").html(periodCellHtml);

        row.find("td.leaveType img").attr("src", leaveType.getIconPath());
        row.find("td.leaveType .for-print-only img").attr("src", leaveType.getPrintIconPath());
        row.find("td.leaveType .leaveTypeName").text(leaveType.name);
        row.find("td.timeRequested").text(this.formatTimeRequested(request));
        var comment = LeaveTimeRequestUtils.getCommentFromRequestWithChanges(request);
        row.find("td.leaveType").toggleClass("has-comment", (comment != "") && (comment != null));

        var statusMainColumn = row.find("td.status");

        if (status.waitingDeletion)
        {
            statusMainColumn.addClass("waitingDeletion");
            $.each(this.rowClassesByStatus, function(status, className)
            {
                statusMainColumn.toggleClass(className, false);
            });
        }
        else
        {
            statusMainColumn.removeClass("waitingDeletion");
            $.each(this.rowClassesByStatus, function(statusCode, className)
            {
                statusMainColumn.toggleClass(className, status.code == statusCode);
            });
        }

        statusMainColumn.find(".statusText").text(status.waitingDeletion ?
                                                  messageResource.getMessage("actiplans.request_status.waiting_for_deletion_approval") :
                                                  this.rowTextByStatus[status.code]);
        statusMainColumn.find(".when").text(status.statusDate.parseJSONDate().format("M d, Y"));
        var byWhom = status.approverInfo ?
                     messageResource.getMessage('actiplans.request_status.by_user',
                                                [UserNameFormat.formatFML(status.approverInfo.firstName, status.approverInfo.lastName, status.approverInfo.middleInitial)]) :
                     status.code == LeaveTimeRequestStatus.APPROVED && !status.waitingDeletion ?
                     messageResource.getMessage('actiplans.request_status.automatic_approval') :
                     '';

        var $details = statusMainColumn.find(".details");

        if (!byWhom)
        {
            $details.hide();
        }
        else
        {
            $details.addClass("byWhom").text(byWhom).show();
        }


        if (status.statusMessage)
            statusMainColumn.find(".approvalComment").text('- ' + status.statusMessage);
        else
            statusMainColumn.find(".approvalComment").text("");

        row.find("td.status").toggleClass("has-comment", status.statusMessage != null);

        var paintAsNotApplicable = function(row)
        {
            row.find("td.balance.before").text("\u2013").addClass("na");
            row.find("td.balance.after").text("\u2013").addClass("na");
        };

        var addBalanceChangeData = function(row, balanceChangeData, formatter)
        {
            var start = balanceChangeData["startBalance"];
            var end = balanceChangeData["endBalance"];

            var hasStartValue = start != null;
            var hasEndValue = end != null;

            var beforeEl = row.find("td.balance.before").text(hasStartValue ? formatter.format(start) : "\u2013").removeClass("na", !hasStartValue);

            if (hasStartValue)
                beforeEl.toggleClass("negative", formatter.isNegativePTO(start));
            else
                beforeEl.addClass("beforeNA");

            row.find("td.balance.after").text(hasEndValue ? formatter.format(end) : "\u2013").toggleClass("negative", hasEndValue && formatter.isNegativePTO(end))
                    .removeClass("na", !hasEndValue);

        };

        var addBalancesOnStartEndData = function(row, balanceChangeData, balanceType, ptoFormatter, sickFormatter)
        {
            if (balanceChangeData)
            {
                switch (balanceType)
                {
                    case BalanceType.PTO:
                        if (SystemSettings.apPtoBalanceCalculationEnabled)
                        {
                            addBalanceChangeData(row, balanceChangeData, ptoFormatter);
                        }
                        else
                        {
                            paintAsNotApplicable(row);
                        }
                        break;
                    case BalanceType.SICK:
                        if (SystemSettings.apSickBalanceCalculationEnabled)
                        {
                            addBalanceChangeData(row, balanceChangeData, sickFormatter);

                            row.find("td.balance.before").append($("<span class='sickSpan'>" + messageResource.getMessage(
                                                                           "requests_table.balance_cell.sick_balance") + "</span>")).addClass(
                                    "sickBalanceCell");
                            row.find("td.balance.after").addClass("sickBalanceCell");
                        }
                        else
                        {
                            paintAsNotApplicable(row);
                        }
                        break;
                }
            }
            else
            {
                paintAsNotApplicable(row);
            }
        };

        var addChangedTodayBalanceData = function(row, changedTodayBalance, isPtoEnabled, isSickEnabled, ptoFormatter, sickFormatter)
        {

            row.find("td.balance.before")
                    .html(LeaveTimeRequestUtils.getChangeTodayBalanceCellContentForLeaveRequestTables(changedTodayBalance, isPtoEnabled, isSickEnabled,
                                                                                                      ptoFormatter, sickFormatter))
                    .addClass("changedValue")
                    .removeClass("na");

            row.find("td.balance.after").empty();
        };

        row.find("td.balance.before").removeClass("sickBalanceCell na negative beforeNA");
        row.find("td.balance.after").removeClass("sickBalanceCell na negative");

        var balanceValue = LeaveTimeRequestUtils.getBalanceValueFromRequestWithChanges(request, userInfo);
        var balanceView = LeaveTimeRequestUtils.viewBalanceAsType(request, userInfo.ptoEnabled, userInfo.sickEnabled, balanceValue.endBeforeHireDate);
        switch (balanceView)
        {
            case LeaveTimeRequestUtils.ViewBalanceAs.DOES_NOT_AFFECT_BALANCE:
            case LeaveTimeRequestUtils.ViewBalanceAs.BALANCE_ON_START_END:
            case LeaveTimeRequestUtils.ViewBalanceAs.DO_NOT_VIEW:
                addBalancesOnStartEndData(row, request["balanceData"] ? request.balanceData.balanceChangeData : null,
                                          leaveType.balanceType, this.ptoFormatter, this.sickFormatter);
                break;
            case LeaveTimeRequestUtils.ViewBalanceAs.CURRENT_BALANCE_BEFORE_AFTER_APPROVAL:
            case LeaveTimeRequestUtils.ViewBalanceAs.DOES_NOT_AFFECT_CURRENT_BALANCE:
                addChangedTodayBalanceData(row, balanceValue.changedTodayBalance,
                                           userInfo.ptoEnabled && SystemSettings.apPtoBalanceCalculationEnabled,
                                           userInfo.sickEnabled && SystemSettings.apSickBalanceCalculationEnabled,
                                           this.ptoFormatter, this.sickFormatter);
                break;
        }

        if (!corporateRequestsRowData.approvableByManager)
        {
            delete this.allSelectedRequestIdsSet[request.id];
        }

        var self = this;
        row.find("td.select label")
                .mouseover(function() { self.setParentRowHover(this); })
                .mouseout(function() { self.removeParentRowHover(this); });
        row.find("td.select input").click(function()
                                          {
                                              var requestId = self.getParentRequestRow(this).data().request.id;

                                              if (this.checked)
                                                  self.allSelectedRequestIdsSet[requestId] = true;
                                              else
                                                  delete self.allSelectedRequestIdsSet[requestId];

                                              self.onSelectionChanged();
                                          });
    },

    /**
     *  Add all events to request row
     * @param row current request row
     * @param corporateRequestsRowData
     */
    setEditRequestFunctionsForRow: function(row, corporateRequestsRowData)
    {
        var request = corporateRequestsRowData.request;
        var approvableByManager = corporateRequestsRowData.approvableByManager;
        var onlyEditableByManager = corporateRequestsRowData.onlyEditableByManager;

        var selectEl = row.find("td.select");
        var leaveTypeEl = row.find("td.leaveType");
        var periodEl = row.find("td.period");
        var statusEl = row.find("td.status");
        var commentButton = statusEl.find(".commentButton");
        var requestCommentButton = leaveTypeEl.find(".requestCommentButton");
        selectEl.unbind('mouseenter mouseleave mousemove');
        leaveTypeEl.unbind('mouseenter mouseleave mousemove mouseover mouseout');
        periodEl.unbind('mouseenter mouseleave mousemove mouseover mouseout');
        statusEl.unbind('mouseenter mouseleave mousemove mouseover mouseout');
        leaveTypeEl.unbind('click');
        periodEl.attr('onclick', '').unbind('click');
        statusEl.unbind('click');
        statusEl.unbind("contextmenu");

        var self = this;

        var setCellHover = function(element, hoverClass)
        {
            $(element).addClass(hoverClass);
        };
        var removeCellHover = function(element, hoverClass)
        {
            $(element).removeClass(hoverClass);
        };

        if (row.data().request.statusMessage != null)
        {
            var onCommentButtonMouseOver = function(jQueryEvent)
            {
                setCellHover($(this).parents(".status"), "commentHover");
                if ((!request.past || (self.canEditPastRequests)) && approvableByManager)
                {
                    setCellHover($(this).parents(".status"), "darkHover");
                }
                self.notApplicableRequestTooltip.hide();
                jQueryEvent.stopPropagation();
            };

            var onCommentButtonMouseOut = function(jQueryEvent)
            {
                removeCellHover($(this).parents(".status"), "darkHover commentHover");
                jQueryEvent.stopPropagation();
            };

            commentButton.click(
                    function(jQueryEvent)
                    {
                        self.onCommentButtonClick.call(self, jQueryEvent, commentButton, row.data().request.statusMessage)
                    })
                    .mouseover(onCommentButtonMouseOver)
                    .mouseout(onCommentButtonMouseOut);
        }
        else
        {
            commentButton.unbind('mouseover mouseout click');
        }

        if (LeaveTimeRequestUtils.getCommentFromRequestWithChanges(row.data().request) != null)
        {
            var onRequestCommentButtonMouseOver = function(jQueryEvent)
            {
                setCellHover($(this).parents(".leaveType"), "commentHover");
                if ((!request.past || (self.canEditPastRequests)) && (approvableByManager || onlyEditableByManager))
                {
                    setCellHover($(this).parents(".leaveType"), "darkHover");
                }
                jQueryEvent.stopPropagation();
            };

            var onRequestCommentButtonMouseOut = function(jQueryEvent)
            {
                removeCellHover($(this).parents(".leaveType"), "darkHover commentHover");
                jQueryEvent.stopPropagation();
            };
            requestCommentButton.click(
                    function(jQueryEvent)
                    {
                        self.onCommentButtonClick.call(self, jQueryEvent, requestCommentButton, LeaveTimeRequestUtils.getCommentFromRequestWithChanges(row.data().request))
                    })
                    .mouseover(onRequestCommentButtonMouseOver)
                    .mouseout(onRequestCommentButtonMouseOut);
        }
        else
        {
            requestCommentButton.unbind('mouseover mouseout click');
        }

        var onActiveElementMouseOver = function()
        {
            setCellHover(this, "darkHover");
        };

        var onActiveElementMouseOut = function()
        {
            removeCellHover(this, "darkHover");
        };

        var statusRejected = row.data().request.status == LeaveTimeRequestStatus.REJECTED;

        var notEditableTooltip = null;
        if (request.past && this.wasAtIntegrated && this.currentUserActiveInAT)
            notEditableTooltip = this.wasAtIntegratedTooltip;
        else if (request.past && !this.canEditPastRequests )
            notEditableTooltip = this.cannotEditPastRequestTooltip;
        else if (!approvableByManager && !onlyEditableByManager)
            notEditableTooltip = corporateRequestsRowData.userInfo.requiresLTRApproval ? this.notApplicableRequestTooltip : this.requestAutoApprovalTooltip;
        else if (request.waitingDeletion)
            notEditableTooltip = this.waitingDeletionRequestTooltip;
        else if (statusRejected)
            notEditableTooltip = this.rejectedRequestTooltip;

        if (notEditableTooltip)
            notEditableTooltip.bind([leaveTypeEl, periodEl]);
        else
        {
            leaveTypeEl.click(_.bind(this.onLeaveTypeClick, this))
                    .mouseover(onActiveElementMouseOver).mouseout(onActiveElementMouseOut);

            periodEl.click(_.bind(this.onPeriodClick, this))
                    .mouseover(onActiveElementMouseOver).mouseout(onActiveElementMouseOut);
        }

        var statusNotChangeableTooltip = null;
        if (request.past && this.wasAtIntegrated)
            statusNotChangeableTooltip = this.wasAtIntegratedTooltip;
        else if (request.past && !this.canEditPastRequests)
            statusNotChangeableTooltip = this.cannotEditPastRequestTooltip;
        else if (!approvableByManager && !onlyEditableByManager)
        {
            statusNotChangeableTooltip = corporateRequestsRowData.userInfo.requiresLTRApproval ? this.notApplicableRequestTooltip : this.requestAutoApprovalTooltip;
            row.addClass("notApprovable");
            row.addClass("notEditable");
        }

        if (statusNotChangeableTooltip)
            statusNotChangeableTooltip.bind([statusEl, selectEl]);
        else
        {
            row.removeClass("notApprovable");
            row.removeClass("notEditable");

            statusEl.click(_.bind(this.onStatusClick, this))
                    .mouseover(onActiveElementMouseOver).mouseout(onActiveElementMouseOut);
            statusEl.contextmenu(function(jQueryEvent)
                                 {
                                     self.onStatusClick(jQueryEvent);
                                     return false;
                                 });
        }
        
        var notSelectable = (request.past && !this.canEditPastRequests) || (!approvableByManager && !onlyEditableByManager);
        if (notSelectable)
            selectEl.empty();
    },

    formatTimeRequested: function(request)
    {
        var leaveTimeTotal = LeaveTimeRequestUtils.getLeaveTimeTotalValueFromRequestWithChanges(request);
        return this.daysFormatter.format(leaveTimeTotal.days)
               + ' (' + this.hoursFormatter.format(leaveTimeTotal.hours) + ')';
    },

    onCommentButtonClick: function(jQueryEvent, target, dialogText)
    {
        if (_.isUndefined(this.commentDialog))
        {
            var removeAllRowsClickedStyle = function()
            {
                $('table.requests > tbody > tr.request').removeClass('clicked');
            };

            this.commentDialog = new DropdownDialog({
                wrapperClass: 'commentPopup',
                dialogDiv: '#commentDialogDiv',
                hideButtons: true,
                onClose: removeAllRowsClickedStyle,
                isWithBeak: true,
                hideOnResize: true
            });
        }
        this.setParentRowClickedStyle(this);
        $('#commentDialogDiv').text(dialogText);
        this.commentDialog.showAt(target, {offset: "13px 12px", my: "right top", at: "right bottom", collision: "none"});
        // resize popup if it's left offset is negative
        var wrapper = $('.commentPopup');
        var offset = wrapper.offset();
        if (offset.left < 0)
        {
            var width = wrapper.outerWidth();
            wrapper.css("max-width", width + offset.left + "px");
            wrapper.css("left", 0);
        }
        jQueryEvent.stopPropagation();
    },

    setParentRowHover: function(element)
    {
        this.getParentRequestRow(element).addClass('hover');
    },

    removeParentRowHover: function(element)
    {
        this.getParentRequestRow(element).removeClass('hover');
    },

    setParentRowClickedStyle: function(element)
    {
        this.getParentRequestRow(element).addClass('clicked');
    },

    setParentRowProcessHover: function(element)
    {
        var row = this.getParentRequestRow(element);
        row.addClass('process_hover');
    },

    getParentRequestRow: function(element)
    {
        return $(element).parents('tr.request');
    },

    removeParentRowProcessHover: function(element)
    {
        var row = this.getParentRequestRow(element);
        row.removeClass('process_hover');
    },

    removeCellDialogSelectedStyle: function(element)
    {
        $(element).removeClass("dialogSelected");
    },

    requestIsDisabledForEditing: function(request)
    {
        return request.status == LeaveTimeRequestStatus.REJECTED || request.past && !this.canEditPastRequests || request.waitingDeletion;
    },

    setCellDialogSelectedStyle: function(element)
    {
        $(element).addClass("dialogSelected");
    },

    hl: function(str)
    {
        return this.usernameHighlighter.highlightTokens(str);
    },

    onLeaveTypeClick: function(jQueryEvent)
    {
        var requestData = $(jQueryEvent.target).closest("tr").data();
        var userInfo = requestData.userInfo;

        var self = this;
        if (this.requestIsDisabledForEditing(requestData.request))
            return;

        var cell = $(jQueryEvent.target).closest("td.leaveType");
        this.setCellDialogSelectedStyle(cell);

        var successor = {
            hide: function()
            {
                self.removeParentRowProcessHover(jQueryEvent.target);
                self.removeCellDialogSelectedStyle(cell, 'dialogSelected');
                self.leaveTypesListDialog = null;
            },

            leaveTypeSelected: function(leaveTypeId)
            {
                var request = requestData.request;
                request.leaveTypeId = leaveTypeId;

                var requestsToUpdate = self.userIdRequestIdsMap[userInfo.id];
                var updateCommand = {
                    method: 'CorporateRequestsService.updateRequestLeaveType',
                    params: [request.id,
                             request.userId,
                             request.leaveTypeId,
                             requestsToUpdate],
                    requestsToUpdate: requestsToUpdate
                };

                self.updateLeaveRequest(userInfo, jQueryEvent.target, updateCommand);
            }
        };

        this.leaveTypesListDialog = new LeaveTypesListDialog(successor, false);

        this.abortPreparedAndActiveRequests();
        this.setParentRowProcessHover(jQueryEvent.target);
        this.leaveTypesListDialog.showAt($(jQueryEvent.delegateTarget).find(".leaveTypeWithImg").get(0));
    },

    makeOnSingleRequestOperationSuccess: function(rowPeriodEl, successHandler)
    {
        var self = this;
        return function(data)
        {
            self.removeParentRowProcessHover(rowPeriodEl);
            var result = successHandler(data);
            PageOverlayTipsManager.getInstance().refreshDynamicTips();
            return result;
        }
    },

    makeOnSingleRequestOperationFailed: function(rowPeriodEl)
    {
        var self = this;
        return function()
        {
            self.stopSpinner();
            self.removeParentRowProcessHover(rowPeriodEl);
        }
    },

    getRequestsErrorHandler: function(jqXHR, textStatus)
    {
        this.stopSpinner();
        this.filterRpcCall = null;

        if (textStatus == "abort")
            return true;

        this.stateController.loadState();
        return false;  //delegate work to default error handler
    },

    makeGetRequestsRpcCall: function()
    {
        var actualState = this.stateController.getState();
        var punctuators = StringTokenizer.PUNCTUATORS_FOR_SEARCH;
        var usernameTokenizer = new StringTokenizer(punctuators);
        var userNameTokens = usernameTokenizer.getTokens(actualState.userNameFilterText);

        this.usernameHighlighter = userNameTokens.length > 0 ? new TokensHighlighter(userNameTokens, punctuators) : new EmptyHighlighter();

        var self = this;
        this.prepareInitialFilterRequest(function()
                                         {
                                             return makeRpcCall("CorporateRequestsService.getRequests",
                                                                [actualState.groupFilter, actualState.groupFilterType, actualState.leaveTypeFilter, actualState.statusFilter, actualState.balanceTypeFilter,
                                                                 actualState.sortBy, actualState.isAscending, actualState.displayOnlyAssignedToMeUsers, actualState.showPastApprovedRejectedRequests,
                                                                 userNameTokens, actualState.pageNum, actualState.recordsPerPage],
                                                                _.bind(self.onGetRequestsComplete, self), _.bind(self.getRequestsErrorHandler, self), null);
                                         });
    },

    updateLeaveRequest: function(userInfo, jQueryEventTarget, updateCommand)
    {
        var self = this;

        function onSaveRequestSuccess(data)
        {
            self.stopSpinner();
            self.invocationResultHandler.handleResultCode(data["resultCode"]);
            self.updateRows(updateCommand.requestsToUpdate, data["data"]);
        }

        var errorHandler = new CorporateScheduleErrorHandler(_.bind(this.makeGetRequestsRpcCall, this), userInfo,
                _.bind(this.makeOnSingleRequestOperationFailed(jQueryEventTarget), this));

        this.prepareInitialUpdateRequest(function()
                                         {
                                             return makeRpcCall(updateCommand.method, updateCommand.params,
                                                                _.bind(self.makeOnSingleRequestOperationSuccess(jQueryEventTarget, onSaveRequestSuccess), self),
                                                                errorHandler.singleUserErrorHandler);
                                         });
    },

    onPeriodClick: function(jQueryEvent)
    {
        var requestData = $(jQueryEvent.target).closest("tr").data();
        var leaveTimeRequest = requestData.request;
        var userInfo = requestData.userInfo;

        var cell = $(jQueryEvent.target).closest("td.period");
        this.setCellDialogSelectedStyle(cell);

        // copy the request object.
        leaveTimeRequest = jQuery.extend({}, leaveTimeRequest);

        var self = this;
        var editRequestDialog;
        var successor = {

            closed: function()
            {
                self.removeParentRowProcessHover(jQueryEvent.target);
                self.removeCellDialogSelectedStyle(cell);
            },

            cancelled: function()
            {
                editRequestDialog.close();
            }
        };

        if ((this.requestIsDisabledForEditing(leaveTimeRequest)) || !( requestData.approvableByManager || requestData.onlyEditableByManager ))
        {
            editRequestDialog = new ViewLeaveRequestPeriodDialog(LeaveTimeRequestUtils.mergeChangesToRequest(
                    leaveTimeRequest), userInfo["ptoEnabled"], userInfo["sickEnabled"], successor, self.ptoHoursPerDay, null).init();
        }
        else
        {
            successor.saveRequest = function()
            {
                var requestsToUpdate = self.userIdRequestIdsMap[userInfo.id];
                var updateCommand = {
                    method: 'CorporateRequestsService.updateRequestDuration',
                    params: [leaveTimeRequest.id,
                             leaveTimeRequest.userId,
                             leaveTimeRequest.firstDayDuration,
                             leaveTimeRequest.lastDayDuration,
                             leaveTimeRequest.middleDayDuration,
                             requestsToUpdate],
                    requestsToUpdate: requestsToUpdate
                };

                self.updateLeaveRequest(userInfo, jQueryEvent.target, updateCommand);
                editRequestDialog.close();
            };
            editRequestDialog = new EditLeaveRequestPeriodDialog(LeaveTimeRequestUtils.mergeChangesToRequest(
                    leaveTimeRequest), userInfo["ptoEnabled"], userInfo["sickEnabled"], userInfo, successor, self.ptoHoursPerDay, null).init();
        }

        this.abortPreparedAndActiveRequests();

        this.setParentRowProcessHover(cell);

        editRequestDialog.showAt(cell, "10px, -10px");
    },

    onStatusClick: function(jQueryEvent)
    {
        if (this.requestMenu != null)
        { // menu is shown already
            return;
        }
        var statusCell = $(jQueryEvent.target).closest("td.status")[0];
        var requestData = $(jQueryEvent.target).closest("tr.request").data();
        var leaveTimeRequest = requestData.request;

        // copy the request object.
        leaveTimeRequest = jQuery.extend({}, leaveTimeRequest);

        if (leaveTimeRequest.past && !this.canEditPastRequests)
        {
            return;
        }

        var itemsHandlers = this.getStatusMenuItemsHandlers();
        var items = [];
        var self = this;

        if (!requestData.onlyEditableByManager)
        {

            if (!leaveTimeRequest.waitingDeletion && !leaveTimeRequest["requestChange"] && leaveTimeRequest.status == LeaveTimeRequestStatus.APPROVED && requestData.revocableApprovedRequest)
            {
                items.push({
                               text: messageResource.getMessage("actiplans.leavestable.context_menu.revoke"),
                               listeners: {
                                   click: function()
                                   {
                                       itemsHandlers["revokeApprovedRequest"](statusCell);
                                   }
                               },
                               href: "javascript: void(0)",
                               cls: "revokeContextButton"
                           });
            }

            if (leaveTimeRequest.status == LeaveTimeRequestStatus.REJECTED)
            {
                items.push({
                               text: messageResource.getMessage("actiplans.leavestable.context_menu.rollback"),
                               listeners: {
                                   click: function()
                                   {
                                       itemsHandlers["rollbackRejectedRequest"](statusCell);
                                   }
                               },
                               href: "javascript: void(0)",
                               cls: "rollbackContextButton"
                           });
            }

            if (leaveTimeRequest.waitingDeletion || leaveTimeRequest["requestChange"] || (leaveTimeRequest.status != LeaveTimeRequestStatus.APPROVED && leaveTimeRequest.status != LeaveTimeRequestStatus.REJECTED))
            {
                var approveMessageKey = "actiplans.leavestable.context_menu.approve";
                if (leaveTimeRequest.waitingDeletion)
                {
                    approveMessageKey = "actiplans.leavestable.context_menu.approve_deletion";
                }
                else if (leaveTimeRequest["requestChange"])
                {
                    approveMessageKey = "actiplans.leavestable.context_menu.approve_changes";
                }
                else if (leaveTimeRequest.past)
                {
                    approveMessageKey = "actiplans.leavestable.context_menu.approve_past_leave";
                }
                items.push({
                               text: messageResource.getMessage(approveMessageKey),
                               listeners: {
                                   click: function()
                                   {
                                       itemsHandlers["approveRequest"](statusCell);
                                   }
                               },
                               href: "javascript: void(0)",
                               cls: "approveContextButton"
                           });

                if (!leaveTimeRequest.waitingDeletion)
                {
                    items.push({
                                   text: messageResource.getMessage("actiplans.leavestable.context_menu.approve_with_comment"),
                                   listeners: {
                                       click: function()
                                       {
                                           var position = self.requestMenu.position;
                                           self.requestMenu.hide();
                                           itemsHandlers["approveRequestWithComment"](statusCell, position);
                                       }
                                   },
                                   href: "javascript: void(0)",
                                   cls: "approveWithCommentContextButton"
                               });
                }
            }

            if (leaveTimeRequest.status != LeaveTimeRequestStatus.REJECTED)
            {
                var rejectMessageKey = "actiplans.leavestable.context_menu.reject";
                if (leaveTimeRequest.waitingDeletion)
                {
                    rejectMessageKey = "actiplans.leavestable.context_menu.reject_deletion";
                }
                else if (leaveTimeRequest["requestChange"])
                {
                    rejectMessageKey = "actiplans.leavestable.context_menu.reject_changes";
                }
                else if (leaveTimeRequest.past)
                {
                    rejectMessageKey = "actiplans.leavestable.context_menu.reject_past_leave";
                }
                items.push({
                               text: messageResource.getMessage(rejectMessageKey),
                               listeners: {
                                   click: function()
                                   {
                                       if (leaveTimeRequest.waitingDeletion || leaveTimeRequest["requestChange"])
                                       {
                                           itemsHandlers["rejectRequest"](statusCell);
                                       }
                                       else
                                       {
                                           var position = self.requestMenu.position;
                                           self.requestMenu.hide();
                                           itemsHandlers["rejectRequestWithComment"](statusCell, position);
                                       }
                                   }
                               },
                               href: "javascript: void(0)",
                               cls: "rejectContextButton"
                           });
            }
        }

        if (!leaveTimeRequest.waitingDeletion)
        {
            if (items.length > 0)
            {
                items.push(new Ext.menu.Separator());
            }
            items.push({
                           text: messageResource.getMessage(leaveTimeRequest.past ?
                                                            "actiplans.leavestable.context_menu.delete_past" :
                                                            "actiplans.leavestable.context_menu.delete"),
                           listeners: {
                               click: function()
                               {
                                   itemsHandlers["deleteRequest"](statusCell);
                               }
                           },
                           href: "javascript: void(0)",
                           cls: "deleteContextButton"
                       });
        }

        this.requestMenu = new Ext.menu.Menu({
            items: items,
            cls: "at-dropdown-list-btn-menu leaveRequestContextMenu"
        });

        this.setParentRowProcessHover(jQueryEvent.target);
        this.setCellDialogSelectedStyle(statusCell);

        this.requestMenu.position = [jQueryEvent.pageX + 15, jQueryEvent.pageY + 15];
        this.requestMenu.showAt([0, 0]);

        var requestMenuDiv = $(".leaveRequestContextMenu");
        var requestMenuSize = [requestMenuDiv.outerWidth() + 4, requestMenuDiv.outerHeight() + 3];

        this.requestMenu.hide();
        this.requestMenu.showAt(this.updateElementPosition(this.requestMenu.position, requestMenuSize));


        this.requestMenu.addListener("hide", function()
        {
            if (self.requestDialog == null) //TODO dialog ???
            {
                self.removeParentRowProcessHover(jQueryEvent.target);
                self.removeCellDialogSelectedStyle(statusCell);
            }
            Ext.destroy(self.requestMenu);
            self.requestMenu = null;
        });

    },

    /**
     * Lazy init status menu handlers for status menu item listeners
     * @returns {{}|*}
     */
    getStatusMenuItemsHandlers: function()
    {
        if (!_.isUndefined(this.statusMenuHandlers))
        {
            return this.statusMenuHandlers;
        }

        var self = this;
        var makeProcessRequest = function(params)
        {

            return function(rowElement)
            {
                if (!params.confirmationMessageKey || confirm(messageResource.getMessage(params.confirmationMessageKey)))
                {
                    var data = self.getParentRequestRow(rowElement).data();
                    var userRequestIds = self.userIdRequestIdsMap[data.userInfo.id];
                    var handler = new CorporateScheduleErrorHandler(_.bind(self.makeGetRequestsRpcCall, self), data.userInfo, _.bind(self.makeOnSingleRequestOperationFailed(rowElement), self));
                    var onComplete = params.makeOnComplete(data, userRequestIds);
                    var rpcMethodParams = params.getRpcMethodParams(data, userRequestIds);

                    self.prepareInitialUpdateRequest(function()
                                                     {
                                                         return makeRpcCall(params.rpcMethod,
                                                                            rpcMethodParams,
                                                                            self.makeOnSingleRequestOperationSuccess(rowElement, onComplete),
                                                                            handler.singleUserErrorHandler);
                                                     });
                }
            }
        };


        var deleteRequest = makeProcessRequest({
                                                   confirmationMessageKey: "actiplans.corporate_requests.confirmDelete",
                                                   rpcMethod: "CorporateRequestsService.delete",
                                                   makeOnComplete: function()
                                                   {
                                                       return function onDeleteComplete(data)
                                                       {
                                                           self.showAlertIfNeededOnDeleteRequest(data);
                                                           self.makeGetRequestsRpcCall();
                                                       };
                                                   },
                                                   getRpcMethodParams: function(data)
                                                   {
                                                       return [[data.request.id]];
                                                   }
                                               });

        var revokeApprovedRequest = makeProcessRequest({
                                                           rpcMethod: "CorporateRequestsService.revokeApproval",
                                                           makeOnComplete: function(data, userRequestIds)
                                                           {
                                                               return function(data)
                                                               {
                                                                   self.stopSpinner();
                                                                   self.invocationResultHandler.handleResultCode(data["resultCode"]);
                                                                   self.updateRows(userRequestIds, data["data"]);
                                                               };
                                                           },
                                                           getRpcMethodParams: function(data, userRequestIds)
                                                           {
                                                               return [data.request.id, userRequestIds];
                                                           }
                                                       });

        var rollbackRejectedRequest = makeProcessRequest({
                                                             rpcMethod: "CorporateRequestsService.rollbackRejection",
                                                             makeOnComplete: function(data, userRequestIds)
                                                             {
                                                                 return function(data)
                                                                 {
                                                                     self.stopSpinner();
                                                                     self.invocationResultHandler.handleResultCode(data["resultCode"]);
                                                                     self.updateRows(userRequestIds, data["data"]);
                                                                 };
                                                             },
                                                             getRpcMethodParams: function(data, userRequestIds)
                                                             {
                                                                 return [data.request.id, userRequestIds];
                                                             }
                                                         });

        var createApproveRejectRequestProcessor = function(rpcMethod, comment)
        {
            return makeProcessRequest({
                                          rpcMethod: rpcMethod,
                                          makeOnComplete: function(data, userRequestIds)
                                          {
                                              return function(data)
                                              {
                                                  self.onApproveOrRejectComplete(data, userRequestIds, true);
                                              };
                                          },
                                          getRpcMethodParams: function(data, userRequestIds)
                                          {
                                              var request = data.request;
                                              var idsForApproveStatus = request.waitingDeletion ? [] : [request.id];
                                              var idsForApproveDeletion = request.waitingDeletion ? [request.id] : [];
                                              comment = _.isUndefined(comment) ? "" : comment;
                                              return [idsForApproveStatus, idsForApproveDeletion, userRequestIds, comment];
                                          }
                                      });
        };

        var approveRequest = createApproveRejectRequestProcessor("CorporateRequestsService.approve");
        var rejectRequest = createApproveRejectRequestProcessor("CorporateRequestsService.reject");

        var showApproveOrRejectDialog = function(rowElement, rpcMethod, menuPosition, dialog)
        {
            var data = self.getParentRequestRow(rowElement).data();
            var requestDialog = dialog.init(data.userInfo, LeaveTimeRequestUtils.getLeaveTypeIdFromRequestWithChanges(data.request), data.request.past, {
                closed: function()
                {
                    self.removeParentRowProcessHover(rowElement);
                    self.removeCellDialogSelectedStyle(rowElement);
                    requestDialog = null;
                },
                cancelled: function()
                {

                },
                submitted: function(comment)
                {
                    createApproveRejectRequestProcessor(rpcMethod, comment)(rowElement);
                }
            }, 2000);
            requestDialog.show(function() { this.offset({top: 0, left: 0})});
            var newPosition = self.updateElementPosition(menuPosition, [requestDialog.outerWidth(), requestDialog.outerHeight()]);
            requestDialog.show(function() { this.offset({top: newPosition[1], left: newPosition[0]})});
        };

        var approveRequestWithComment = function(rowElement, menuPosition)
        {
            showApproveOrRejectDialog(rowElement, "CorporateRequestsService.approve", menuPosition, new RequestApproveDialog());
        };

        var rejectRequestWithComment = function(rowElement, menuPosition)
        {
            showApproveOrRejectDialog(rowElement, "CorporateRequestsService.reject", menuPosition, new RequestRejectDialog());
        };

        this.statusMenuHandlers = {};
        this.statusMenuHandlers["deleteRequest"] = deleteRequest;
        this.statusMenuHandlers["revokeApprovedRequest"] = revokeApprovedRequest;
        this.statusMenuHandlers["rollbackRejectedRequest"] = rollbackRejectedRequest;
        this.statusMenuHandlers["approveRequest"] = approveRequest;
        this.statusMenuHandlers["rejectRequest"] = rejectRequest;
        this.statusMenuHandlers["approveRequestWithComment"] = approveRequestWithComment;
        this.statusMenuHandlers["rejectRequestWithComment"] = rejectRequestWithComment;
        return this.statusMenuHandlers;
    },

    updateElementPosition: function(currentPosition, elementSize)
    {
        var currentLeft = currentPosition[0] - getBodyScrollLeft();
        var currentTop = currentPosition[1] - getBodyScrollTop();
        var width = elementSize[0];
        var height = elementSize[1];
        var windowWidth = $(window).width();
        var windowHeight = $(window).height();
        var newLeft = (currentLeft + width > windowWidth) ? windowWidth - width : currentLeft;
        var newTop = (currentTop + height > windowHeight) ? windowHeight - height : currentTop;
        return [newLeft + getBodyScrollLeft(), newTop + getBodyScrollTop()];
    },

    formatRequestPeriod: function(request)
    {
        var firstDate = request.firstDate.parseJSONDate();
        var lastDate = request.lastDate.parseJSONDate();

        if (firstDate.isSameDate(lastDate))
            return firstDate.format("M d, Y");
        else
            return firstDate.format("M d") + " &ndash; " + lastDate.format("M d, Y");
    },

    formatGroup: function(groupsMap, groupId)
    {
        return groupsMap[groupId].name;
    },

    formatGroupVertical: function(groupsMap, groupId)
    {
        return splitByUnicodeChar(groupsMap[groupId].name.toUpperCase()).join("\n");
    },

    formatUserFullName: function(userInfo)
    {
        return UserNameFormat.formatLFM(
                this.hl(userInfo.firstName),
                this.hl(userInfo.lastName),
                this.hl(userInfo.middleInitial));
    },

    clearSelectionOnAllPages: function()
    {
        $("table.requests > tbody.actualContent td.select input").prop("checked", false);
        this.allSelectedRequestIdsSet = {};

        this.onSelectionChanged();
    },

    updateSortingIndicators: function()
    {
        var sortIndicator = $("table.requests > thead [data-sortBy]");
        sortIndicator.removeClass("ascending");
        sortIndicator.removeClass("descending");
        $("table.requests > thead [data-sortBy=" + this.stateController.getState().sortBy + "]")
                .addClass(this.stateController.getState().isAscending ? "ascending" : "descending");
    },

    initHeaderControls: function(initStatusValue)
    {
        var self = this;

        this.$displayOnlyAssignedToMeUsersChBox = $("#displayOnlyAssignedToMeUsersChBox").on("change", function()
        {
            self.stateController.getState().displayOnlyAssignedToMeUsers = self.$displayOnlyAssignedToMeUsersChBox.is(":checked");
            self.pagerController.loadPage(1);

        });
        this.$showPastApprovedRejectedRequestsChBox = $("#showPastApprovedRejectedRequestsChBox").on("change", function()
        {
            self.stateController.getState().showPastApprovedRejectedRequests = self.$showPastApprovedRejectedRequestsChBox.is(":checked");
            self.pagerController.loadPage(1);
        });

        if (!_.isNull(initStatusValue) && initStatusValue[0] == 0) //if status filter is Waiting Approval the checkbox is disabled
        {
            this.$showPastApprovedRejectedRequestsChBox.prop("disabled", true);
        }


        // sort button click
        $("table.requests > thead .sortButton").click(function()
                                                      {
                                                          var sortControl = $(this).parent();
                                                          var sortBy = sortControl.attr("data-sortBy");

                                                          var actualState = self.stateController.getState();

                                                          if (actualState.sortBy == sortBy)
                                                          {
                                                              actualState.isAscending = !actualState.isAscending;
                                                          }
                                                          else
                                                          {
                                                              actualState.sortBy = sortBy;
                                                              actualState.isAscending = true;
                                                          }

                                                          self.updateSortingIndicators();
                                                          self.pagerController.loadPage(1);
                                                      });

        // select all checkbox
        $('table.requests > thead td.select input').click(
                function()
                {
                    if (this.checked)
                    {
                        $("table.requests > tbody.actualContent td.select input").each(function()
                                                                                       {
                                                                                           var parentRow = self.getParentRequestRow(this);
                                                                                           $(this).prop("checked", true);
                                                                                           self.allSelectedRequestIdsSet[parentRow.data().request.id] = true;
                                                                                       });
                        self.onSelectionChanged();
                    }
                    else
                    {
                        self.clearSelectionOnAllPages();
                    }
                }
        );

        // update indicators after init
        this.updateSortingIndicators();
    },

    setAssignedToMeCheckBox: function(showAssignedToMeOption)
    {
        if (!showAssignedToMeOption)
        {
            $('#displayOnlyAssignedToMeUsersBlock').hide();
            if (!userGroups && !timeZoneGroups)
            {
                $("td.select .spacer").hide();
                $(".user_filters").hide();
                $("#searchFieldContainer").addClass('onSecondLine');
                $(".requests .leaveType .leaveTypeFilter").addClass('onSecondLine');
                $(".requests .balance .balanceTypeFilterBlock").addClass('onSecondLine');
                $(".requests .status .statusFilterBlock").addClass('onSecondLine');
            }
        }
        else
        {
            this.updateDisplayOnlyAssignedToMeUsersChBox();
            $(".user_filters").show();
            $('#displayOnlyAssignedToMeUsersBlock').show();
            $("#searchFieldContainer").removeClass('onSecondLine');
            $(".requests .leaveType .leaveTypeFilter").removeClass('onSecondLine');
            $(".requests .balance .balanceTypeFilterBlock").removeClass('onSecondLine');
            $(".requests .status .statusFilterBlock").removeClass('onSecondLine');
            $("td.select .spacer").show();
        }
    },

    updateDisplayOnlyAssignedToMeUsersChBox: function()
    {
        this.$displayOnlyAssignedToMeUsersChBox.prop("checked", this.stateController.getState().displayOnlyAssignedToMeUsers);
    },

    updateShowPastApprovedRejectedRequestsCheckbox: function()
    {
        this.$showPastApprovedRejectedRequestsChBox.prop("checked", this.stateController.getState().showPastApprovedRejectedRequests);
    }
};