function LeaveTimeHistoryManager()
{
}

LeaveTimeHistoryManager.prototype.dayClassNames = ["sun", "mon", "tue", "wed", "thu", "fri", "sat"];

LeaveTimeHistoryManager.prototype._initElements = function () {
    if ( this.leaveTimeHistoryBody )
        return;

    this.apReportsContent = $("#apReportsContent");
    this.leaveTimeHistoryBody = $("#leaveTimeHistoryBody");
    this.errorPlaceHolder = $(".leaveTimeHistoryError");
    this.pdfErrorPlaceholder = $(".pdfActionError");
    if (hasServerSideErrors)
    {
        this.pdfErrorPlaceholder.append($("#apReport_generatePDFErrors"));
        hasServerSideErrors = false;
    }

    this.calendarContainer = $("#calendarContainer");
};

LeaveTimeHistoryManager.prototype.init = function(initialData)
{
    this.initialData = initialData;
    this.initMonthYearSelector(initialData);
    this.initLeaveTypeFilter(
            initialData.leaveTypeIds,
            SystemSettings.apPtoBalanceCalculationEnabled,
            SystemSettings.apSickBalanceCalculationEnabled
    );
    // to avoid of loading incorrect state from user settings
    this.initialData.dateFrom = this.getDateFrom();
    this.initialData.period = this.getPeriod();
    this.makeInitRequest(this.initialData);
    return this;
};

LeaveTimeHistoryManager.prototype.baseInit = function ( initialData ) {
    this._hideError();
    this.daysFormatter = new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 1, true, '', 'd');
    this.hoursFormatter = new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 2, true, '', 'h');

    this.calendar = new PastLeavesCalendar().init({
                                                      isWideContainerFn: _.bind(this.isWideContainer, this),
                                                      isNarrowContainer: _.bind(this.isNarrowContainer, this),
                                                      calendarContainer: this.calendarContainer
                                                  });

    if (initialData.ptoHoursPerDay){
        this.ptoHoursPerDay = initialData.ptoHoursPerDay;
    }

    this.initStateController(initialData);

    if (initialData.userLeaveTimeHistoryData)
        this._updateState(initialData.userLeaveTimeHistoryData, initialData);

    this.initialData = initialData;

    this.leaveTimeDetailsTable = new LeaveTimeDetailsTable().init(initialData.leaveTimeDetailsSettings, initialData.nameService);

    this.updateData(initialData);

    this.initCompleted = true;

};

LeaveTimeHistoryManager.prototype.initStateController = function ( initialData )
{
    var self = this;
    this.stateController = new StateController().init({ period: initialData.period,
                                                        dateFrom: initialData.dateFrom,
                                                        rangeType: initialData.rangeType
                                                      });
    this.stateController.addOnSaveStateActions( function ( )
                                                {
                                                    self.leaveTypeFilter.saveState();
                                                    APReportsPage.PastLeavesPage.Store.setPeriodState(self.stateController.getState());
                                                    APReportsPage.PastLeavesPage.Store.setLeaveTypeSelection(self.leaveTypeFilter.getSelection());
                                                } );

    this.stateController.addOnLoadStateActions( function ( state )
                                                {
                                                    self._hideError();
                                                    self.leaveTypeFilter.loadState();
                                                    var fromDate = new Date(state.dateFrom.getFullYear(), state.dateFrom.getMonth());
                                                    var dateTo = new Date(state.dateFrom.getFullYear(), state.dateFrom.getMonth() + state.period);
                                                    self.monthYearSelector.setSelectedDateRange(new DateRange(fromDate, dateTo), state.rangeType);
                                                } );
};

LeaveTimeHistoryManager.prototype._updateState = function ( userLeaveTimeHistoryData, state ) {
    state.usersToday = userLeaveTimeHistoryData.userToday.parseJSONDate();
    state.leaveTimeTotal = userLeaveTimeHistoryData.leaveTimeTotal;
    state.leaveTypeTotals = !!userLeaveTimeHistoryData.leaveTypeTotals ? userLeaveTimeHistoryData.leaveTypeTotals.map : {};
    state.workdayDurations = userLeaveTimeHistoryData.workdayDurations;
    state.requests = this._convertRequests(userLeaveTimeHistoryData.requests);
    state.requestsForTable = userLeaveTimeHistoryData.requests;
    state.dateTo =  Date.parseJSONDate(userLeaveTimeHistoryData.dateTo);
    state.balancesOnStartDate = userLeaveTimeHistoryData.balancesOnStartDate;
    state.balancesOnFinishDate = userLeaveTimeHistoryData.balancesOnFinishDate;
    state.spentBalanceInfo =userLeaveTimeHistoryData.spentBalanceInfo;

    if( userLeaveTimeHistoryData.ptoHoursPerDay != -1)
        state.ptoHoursPerDay = userLeaveTimeHistoryData.ptoHoursPerDay;
    else   state.ptoHoursPerDay = this.ptoHoursPerDay;

    state.dateFrom = userLeaveTimeHistoryData.periodParameters.dateFrom.parseJSONDate();
    state.period = userLeaveTimeHistoryData.periodParameters.period;
    this.monthYearSelector.setSelectedDateRange(new DateRange(state.dateFrom,
            new Date(state.dateFrom.getFullYear(), state.dateFrom.getMonth() + state.period - 1)), state.rangeType);
    this.stateController.setProperty('dateFrom', state.dateFrom);
    this.stateController.setProperty('period', state.period);

    this.stateController.saveState();
};

LeaveTimeHistoryManager.prototype._handleNewLeaveTypeEmerged = function() {
    alert(messageResource.getMessage("actiplans.leaves_management.new_leave_type"));
    document.location.reload();
};

LeaveTimeHistoryManager.prototype.initLeaveTypeFilter = function (leaveTypeIds, isPTOOn, isSickDaysOn ) {
    var leaveTypes = _.map(LEAVE_TYPES_ORDERED, _.clone);
    var leaveTypesSelectorUtil = new LeaveTypesSelectorUtil().init(
        leaveTypes,
        isPTOOn,
        isSickDaysOn,
        leaveTypeIds
    );

    var ptoName = messageResource.getMessage("leaves_selector.group_label.pto_balance");
    var sdbName = messageResource.getMessage("leaves_selector.group_label.sick_days_balance");
    var bothName = isPTOOn && isSickDaysOn ?
        messageResource.getMessage("leaves_selector.group_label.both") : isPTOOn ? ptoName : sdbName;

    leaveTypes = leaveTypesSelectorUtil.getDataForSelector(
        {
            ptoLabel : messageResource.getMessage('leaves_selector.group_label.affects', [ptoName]),
            sickDaysLabel : messageResource.getMessage('leaves_selector.group_label.affects', [sdbName]),
            withoutBalancesLabel : messageResource.getMessage('leaves_selector.group_label.does_not_affect', [bothName])
        }
    );
    var leavePanel = AT.APLeaveTypesSelectorPanel;
    leaveTypesSelectorUtil.setSelectorTreeConstructorToPanel(leavePanel);

    var self = this;
    this.leaveTypeFilter = new ItemSelector("#leaveTypeFilter", function(newValue) {self.onLeaveTypeFilterChange(newValue)},
            leavePanel, leaveTypes, leaveTypesSelectorUtil.isNeedToShowInactiveLeaves(),
            {allItemsOptionLabelKey: "leaves_selector.options.all_items_with_bold.reported"});
};


LeaveTimeHistoryManager.prototype.initMonthYearSelector = function (initialData)
{
    var self = this;
    this.monthYearSelector = new MonthYearSelector().init(    $(".monthRangeSelector"),
                                                              initialData.startSelectorDate, initialData.lastSelectorDate,
                                                              function()
                                                              {
                                                                  self.makeUpdateRequest();
                                                              });
    this.monthYearSelector.setSelectedDateRange(new DateRange(initialData.dateFrom,
            new Date(initialData.dateFrom.getFullYear(), initialData.dateFrom.getMonth() + initialData.period - 1)), initialData.rangeType);
};

/*
 * @data required
 * @data [selectedUserId]
 * @data [leaveTimeTotal]
 * @data [leaveTypeIds] list of selected leave type ids.
 * @data [leaveTypeTotals] map from leave type to user's leave time requests time.
 * @data [dateFrom] first calendar to show
 * @data [period] number of displayed calendars
 * @data [requests] leave time map
 * @data [workdayDurations] users workdayDurations for all selected time
 *
 * */
LeaveTimeHistoryManager.prototype.updateData = function ( data ) {
    this.calendar.updateData(data);
    var isPTO = data.balancesOnStartDate != null && data.balancesOnStartDate.PTOBalance != null;
    var isSick = data.balancesOnStartDate != null && data.balancesOnStartDate.sickBalance != null;
    this.leaveTimeDetailsTable.initAndUpdate(data.requestsForTable, data.ptoHoursPerDay, isPTO, isSick);
};

LeaveTimeHistoryManager.prototype.isWideContainer = function() {
    return this.apReportsContent.width() > 1550;
};

LeaveTimeHistoryManager.prototype.isNarrowContainer = function() {
    return this.apReportsContent.width() < 1200;
};

LeaveTimeHistoryManager.prototype.getSelectedUserId = function () {
    // abstract
};

LeaveTimeHistoryManager.prototype.getDateFrom = function () {
    return this.monthYearSelector.getSelectedDateRange().dateFrom;
};

LeaveTimeHistoryManager.prototype.getPeriod = function () {
    var selectedDateRange = this.monthYearSelector.getSelectedDateRange();
    return _.isNull(selectedDateRange.dateTo) ? 1 : (selectedDateRange.dateTo.getFullYear() - selectedDateRange.dateFrom.getFullYear()) * 12
                                                    + selectedDateRange.dateTo.getMonth() - selectedDateRange.dateFrom.getMonth() + 1;
};

LeaveTimeHistoryManager.prototype.getRangeType = function()
{
    return this.monthYearSelector.getRangeType();
};

LeaveTimeHistoryManager.prototype.getSelectedLeaveTypes = function () {
    return this.leaveTypeFilter.getSelection();
};


LeaveTimeHistoryManager.prototype.defaultErrorHandler = function(jqXHR, textStatus, errorThrown) {
    if ( errorThrown.exception == "com.actimind.actitime.services.NoRightsForRemoteCallException" ) {
        alert(messageResource.getMessage("actiplans.leaves_management.noRightsForPage"));
        window.location.reload();
        return true;
    }

    if( errorThrown.exception == 'com.actimind.actitime.administration.NoSuchUserException' )
    {
        alert(messageResource.getMessage("common.error.selected_user_was_deleted"));
        window.location.reload();
        return true;
    }

    if( errorThrown.exception == 'com.actimind.actiplans.services.NoRightsForViewingUserReportsData' )
    {
        alert(messageResource.getMessage("actiplans.leaves_management.error_no_rights_for_viewing_user_requests"));
        window.location.reload();
        return true;
    }

    this.returnToPreviousState();

    return false;
};

LeaveTimeHistoryManager.prototype.returnToPreviousState = function () {
    if (this.stateController)
    {
        this.stateController.loadState();
    } else {
        this._showError(messageResource.getMessage("report.js.report_loading_error.network_error", [PRODUCT_NAME]));
    }
};

LeaveTimeHistoryManager.prototype._showError = function(message)
{
    this.errorPlaceHolder.html(message);
    this.leaveTimeHistoryBody.addClass('withError');
},

LeaveTimeHistoryManager.prototype._hideError = function()
{
    this.leaveTimeHistoryBody.removeClass('withError');
},

LeaveTimeHistoryManager.prototype.makeInitRequest = function(initialData)
{
    // Abstract
},

LeaveTimeHistoryManager.prototype.makeUpdateRequest = function ( ) {
    if (this.initCompleted)
    {
        deleteErrors();

        this.initialData.dateFrom = this.getDateFrom();
        this.initialData.period = this.getPeriod();
        this.initialData.rangeType = this.getRangeType();
        this.initialData.leaveTypeIds = this.getSelectedLeaveTypes();

        this.updateDataRequestFunction( this.initialData );
    }
    else
    {
        this.makeInitRequest(this.initialData);
    }
};

LeaveTimeHistoryManager.prototype.successResultCallback = function ( userLeaveTimeHistoryData ) {
    this._hideError();
    this._updateState( userLeaveTimeHistoryData, this.initialData );

    this.updateData(this.initialData);
};

LeaveTimeHistoryManager.prototype.scrollToFunctionFactory = function () {
    var currPosition = $(window).scrollTop();

    return function () {
        $(window).scrollTop(currPosition);
    };
};

LeaveTimeHistoryManager.prototype.updateDataRequestFunction = function ( params ) {
    //abstract
};

LeaveTimeHistoryManager.prototype.onLeaveTypeFilterChange = function () {
    this.makeUpdateRequest( $("#leaveTypesSelectorSpinner") );
};

LeaveTimeHistoryManager.prototype._convertRequests = function ( requests ) {
    var result = {};
    var self = this;
    var leaveNumber = 0;

    $.each(requests, function (i, request) {
        if ( !LEAVE_TYPES[request.leaveTypeId] ) {
            self._handleNewLeaveTypeEmerged();
            return false;
        }

        var firstDate = request.firstDate.parseJSONDate();
        var lastDate = request.lastDate.parseJSONDate();

        for(var di = new DatesIterator( firstDate, lastDate ); di.hasNext(); ) {
            var d = di.next();
            result[leaveNumber++] = {date: d, leaveTypeId: request.leaveTypeId};
        }
    });

    return result;
};

LeaveTimeHistoryManager.prototype.getPdfForm = function(){
    var documentForm = this.getPdfElementForm();
    documentForm.userId.value = this.getSelectedUserId();
    var leaveTypeFilter = this.getSelectedLeaveTypes();
    if( !leaveTypeFilter )
    {
        leaveTypeFilter = "";
    }
    documentForm.leaveTypeFilter.value = leaveTypeFilter;
    documentForm.dateFrom.value = this.getDateFrom().format(Date.convertSmartToExtDateFormat("yyyy-MM-dd"));
    documentForm.period.value = this.getPeriod();
    documentForm.balanceTypeFilter.value = this.getBalanceTypeFilterValue();
    return documentForm;
};

LeaveTimeHistoryManager.prototype.getBalanceTypeFilterValue = function()
{
    return (_.isUndefined(this.leaveTimeDetailsTable)) ? LeaveBalanceFilter.ALL : this.leaveTimeDetailsTable.getBalanceTypeFilterValue();
};


