var LeaveTimePlanningPage = {};

LeaveTimePlanningPage.pageData = null;
LeaveTimePlanningPage.leaveTimePlanningTable = null;
LeaveTimePlanningPage.leaveTypesListDialog = null;
LeaveTimePlanningPage.requestDialog = null;
LeaveTimePlanningPage.requestMenu = null;

LeaveTimePlanningPage.scheduleTableController = null;
LeaveTimePlanningPage.leaveTableUpdateController = null;

LeaveTimePlanningPage.userGroupsSelector = null;

LeaveTimePlanningPage.pageStateController = null;

LeaveTimePlanningPage.dateFrom = function()
{
    return LeaveTimePlanningPage.scheduleTableController.getDateFrom();
};

LeaveTimePlanningPage.closeDialogs = function()
{
    if (LeaveTimePlanningPage.leaveTypesListDialog)
    {
        LeaveTimePlanningPage.leaveTypesListDialog.hide();
    }
};

LeaveTimePlanningPage.createRequest = function()
{
    LeaveTimePlanningPage.leaveTableUpdateController.abortUpdateSchedulesRequest();

    if (!LeaveType.hasActiveLeaveTypes())
    {
        alert("Cannot create request as there are no active leave types.\nPlease contact your system administrator for further assistance.");
        LeaveTimePlanningPage.scheduleTableController.clearSelection();
        return;
    }

    var selectionRegion = LeaveTimePlanningPage.scheduleTableController.getSelectionController().getSelection();
    var row = LeaveTimePlanningPage.scheduleTableController.getRow(selectionRegion.rowMin);
    var isPast = row.isPastDateCell(row.getCell(selectionRegion.colMin));
    var dateRange = LeaveTimePlanningPage.scheduleTableController.getDateRangeForRegion(selectionRegion);

    var successor = {
        hide: function()
        {
            if (LeaveTimePlanningPage.requestDialog == null)
                LeaveTimePlanningPage.scheduleTableController.clearSelection();
            LeaveTimePlanningPage.leaveTypesListDialog = null;
        },

        leaveTypeSelected: function(leaveTypeId)
        {
            var request = LeaveTimePlanningPage.createDefaultRequest(row.getUserId(), dateRange, isPast);
            request.leaveTypeId = leaveTypeId;

            LeaveTimePlanningPage.makeRequestAJAXCallAndUpdateData(request, selectionRegion, LeaveTimePlanningControlManager.CreateRequestManager);
        },

        customAddSelected: function()
        {
            var leaveTimeRequest = LeaveTimePlanningPage.createDefaultRequest(row.getUserId(), dateRange, isPast);

            LeaveTimePlanningPage.requestDialog = new CreateLeaveRequestDialog(leaveTimeRequest, row.isPTOUser(), row.isSickUser(), row.getUserInfo(), {
                        closed: function()
                        {
                            LeaveTimePlanningPage.scheduleTableController.clearSelection();
                            LeaveTimePlanningPage.requestDialog = null;
                        },
                        cancelled: function()
                        {
                            LeaveTimePlanningPage.requestDialog.close();
                        },
                        createRequest: function(request)
                        {
                            LeaveTimePlanningPage.makeRequestAJAXCallAndUpdateData(request, selectionRegion, LeaveTimePlanningControlManager.CreateRequestManager);
                        }
                    },
                    LeaveTimePlanningPage.createRequestRecalculationRPCManager,
                    LeaveTimePlanningPage.pageData.ptoHoursPerDay,
                    row.currentBalanceInfo,
                    true
            ).init();

            LeaveTimePlanningPage.leaveTypesListDialog.hide();
            LeaveTimePlanningPage.requestDialog.showAt(
                    row.getCell(selectionRegion.colMax).getCellEl()
            );

        }
    };

    LeaveTimePlanningPage.leaveTypesListDialog = new LeaveTypesListDialog(successor, true);
    LeaveTimePlanningPage.leaveTypesListDialog.showAt(row.getCell(selectionRegion.colMax).getCellEl());
};

LeaveTimePlanningPage.showApproveDialog = function(selectionRegion, row, leaveTimeRequest)
{
    // copy the request object.
    leaveTimeRequest = jQuery.extend({}, leaveTimeRequest);

    LeaveTimePlanningPage.requestDialog = new RequestApproveDialog().init(row.getUserInfo(),
                                                                          LeaveTimeRequestUtils.getLeaveTypeIdFromRequestWithChanges(leaveTimeRequest),
                                                                          leaveTimeRequest.past, {

        closed: function()
        {
            LeaveTimePlanningPage.scheduleTableController.clearSelection();
            LeaveTimePlanningPage.requestDialog = null;
        },

        cancelled: function()
        {

        },

        submitted: function(comment)
        {
            LeaveTimePlanningPage.approveRequestAndUpdateData(leaveTimeRequest, selectionRegion, comment);
        }

    }, 2000);

    if (LeaveTimePlanningPage.requestMenu)
        LeaveTimePlanningPage.requestMenu.hide();

    LeaveTimePlanningPage.requestDialog.showAt(
            row.getCell(selectionRegion.colMax).getCellEl()
    );
};

LeaveTimePlanningPage.showRejectDialog = function(selectionRegion, row, leaveTimeRequest)
{
    // copy the request object.
    leaveTimeRequest = jQuery.extend({}, leaveTimeRequest);

    LeaveTimePlanningPage.requestDialog = new RequestRejectDialog().init(row.getUserInfo(), leaveTimeRequest.leaveTypeId, leaveTimeRequest.past, {

        closed: function()
        {
            LeaveTimePlanningPage.scheduleTableController.clearSelection();
            LeaveTimePlanningPage.requestDialog = null;
        },

        cancelled: function()
        {

        },

        submitted: function(comment)
        {
            LeaveTimePlanningPage.rejectRequestAndUpdateData(leaveTimeRequest, selectionRegion, comment);
        }

    }, 2000);

    if (LeaveTimePlanningPage.requestMenu)
        LeaveTimePlanningPage.requestMenu.hide();

    LeaveTimePlanningPage.requestDialog.showAt(
            row.getCell(selectionRegion.colMax).getCellEl()
    );
};

LeaveTimePlanningPage.createRequestRecalculationRPCManager = function(request, requestDialogCallback)
{
    var dateFrom = LeaveTimePlanningPage.pageData.dateFrom; // must be saved before request

    var myCallback = function(invocationResult)
    {
        if (invocationResult.calculatedRequestData)
        {
            requestDialogCallback(invocationResult.calculatedRequestData);
        }
        else
        {
            if (LeaveTimePlanningPage.requestDialog)
                LeaveTimePlanningPage.requestDialog.close();
            LeaveTimePlanningPage.requestSaved(invocationResult, dateFrom);
        }
    };

    return new LeaveTimePlanningControlManager.RecalculateRequestManager(
            request,
            LeaveTimePlanningPage.pageData.dateFrom,
            LeaveTimePlanningPage.pageData.dateTo,
            myCallback
    );
};

LeaveTimePlanningPage.showEditRequestDialog = function()
{
    var selectionRegion = LeaveTimePlanningPage.scheduleTableController.getSelectionController().getSelection();
    var row = LeaveTimePlanningPage.scheduleTableController.getRow(selectionRegion.rowMin);
    var leaveTimeRequest = row.getCell(selectionRegion.colMin).getValue().requestData;

    // copy the request object.
    leaveTimeRequest = jQuery.extend({}, leaveTimeRequest);
    var currentBalanceInfo = row.currentBalanceInfo;

    if (leaveTimeRequest.status == LeaveTimeRequestStatus.APPROVED && leaveTimeRequest.past && !LeaveTimePlanningPage.canEditPastRequests)
    {
        // Started approved requests cannot be edited unless this is allowed by system settings / integration status
        LeaveTimePlanningPage.requestDialog = new ViewLeaveRequestDialog(leaveTimeRequest, row.isPTOUser(), row.isSickUser(), {

            closed: function()
            {
                LeaveTimePlanningPage.scheduleTableController.clearSelection();
                LeaveTimePlanningPage.requestDialog = null;
            },

            cancelled: function()
            {
                LeaveTimePlanningPage.requestDialog.close();
            }

        }, currentBalanceInfo).init();
    }
    else
    {
        LeaveTimePlanningPage.requestDialog = new EditLeaveRequestDialog(leaveTimeRequest, row.isPTOUser(), row.isSickUser(), row.getUserInfo(), {

            closed: function()
            {
                LeaveTimePlanningPage.scheduleTableController.clearSelection();
                LeaveTimePlanningPage.requestDialog = null;
            },

            cancelled: function()
            {
                LeaveTimePlanningPage.requestDialog.close();
            },

            saveRequest: function(request)
            {
                LeaveTimePlanningPage.saveRequestChangesAndUpdateData(request, selectionRegion);
            }

        }, LeaveTimePlanningPage.createRequestRecalculationRPCManager, LeaveTimePlanningPage.pageData.ptoHoursPerDay, currentBalanceInfo, true).init();
    }


    if (LeaveTimePlanningPage.requestMenu)
        LeaveTimePlanningPage.requestMenu.hide();

    LeaveTimePlanningPage.requestDialog.showAt(
            row.getCell(selectionRegion.colMax).getCellEl()
    );
};

LeaveTimePlanningPage.revokeApprovalAndUpdateData = function(request, selectionRegion)
{
    LeaveTimePlanningPage.makeRequestAJAXCallAndUpdateData(request, selectionRegion, LeaveTimePlanningControlManager.RevokeApprovalManager);
};

LeaveTimePlanningPage.saveRequestChangesAndUpdateData = function(request, selectionRegion)
{
    LeaveTimePlanningPage.makeRequestAJAXCallAndUpdateData(request, selectionRegion, LeaveTimePlanningControlManager.UpdateRequestManager);
};

LeaveTimePlanningPage.approveRequestAndUpdateData = function(request, selectionRegion, comment)
{
    var dateFrom = LeaveTimePlanningPage.pageData.dateFrom; // must be saved before request
    var manager = new LeaveTimePlanningControlManager.ApproveRequestManager(
            request,
            comment,
            LeaveTimePlanningPage.pageData.dateFrom,
            LeaveTimePlanningPage.pageData.dateTo,
            function(invocationResult) { LeaveTimePlanningPage.requestSaved(invocationResult, dateFrom); }
    );

    LeaveTimePlanningPage.makeRequestAJAXCallWithManagerAndUpdateData(request, selectionRegion, manager);
};

LeaveTimePlanningPage.rejectRequestAndUpdateData = function(request, selectionRegion, comment)
{
    var dateFrom = LeaveTimePlanningPage.pageData.dateFrom; // must be saved before request
    var manager = new LeaveTimePlanningControlManager.RejectRequestManager(
            request,
            comment,
            LeaveTimePlanningPage.pageData.dateFrom,
            LeaveTimePlanningPage.pageData.dateTo,
            function(invocationResult) { LeaveTimePlanningPage.requestSaved(invocationResult, dateFrom); }
    );

    LeaveTimePlanningPage.makeRequestAJAXCallWithManagerAndUpdateData(request, selectionRegion, manager);
};

LeaveTimePlanningPage.deleteRequestAndUpdateData = function(request, selectionRegion)
{
    LeaveTimePlanningPage.makeRequestAJAXCallAndUpdateData(request, selectionRegion, LeaveTimePlanningControlManager.DeleteRequestManager);
};


LeaveTimePlanningPage.showNotApplicableRequestTooltip = function()
{
    var selectionRegion = LeaveTimePlanningPage.scheduleTableController.getSelectionController().getSelection();
    var row = LeaveTimePlanningPage.scheduleTableController.getRow(selectionRegion.rowMin);
    var cell = row.getCell(selectionRegion.colMax).getCellEl();

    // user with auto approved requests can`t be in user list, so we add some additional information
    var dialog = row.userInfo.requiresLTRApproval ? LeaveTimePlanningPage._notApplicableRequestDialog : LeaveTimePlanningPage._autoApprovedUserDialog;

    dialog.showAt(cell, {my: "left top", at: "left top", offset: "16px 28px"});

    doOnceOnWindowResize(function() { if (LeaveTimePlanningPage._notApplicableRequestTooltip) LeaveTimePlanningPage._notApplicableRequestTooltip.hide(); });
};

LeaveTimePlanningPage.hideNotApplicableRequestTooltip = function(jQueryEvent)
{
    LeaveTimePlanningPage.scheduleTableController.clearSelection();
    LeaveTimePlanningPage.scheduleTableController.unhover();
};

LeaveTimePlanningPage.approveDeletionAndUpdateData = function(leaveTimeRequest, selectionRegion)
{
    if (LeaveTimePlanningPage.deletionConfirmation(leaveTimeRequest))
    {
        var dateFrom = LeaveTimePlanningPage.pageData.dateFrom; // must be saved before request
        var manager = new LeaveTimePlanningControlManager.ApproveRequestDeletionManager(
                leaveTimeRequest,
                LeaveTimePlanningPage.pageData.dateFrom,
                LeaveTimePlanningPage.pageData.dateTo,
                function(invocationResult) { LeaveTimePlanningPage.requestSaved(invocationResult, dateFrom); }
        );
        LeaveTimePlanningPage.makeRequestAJAXCallWithManagerAndUpdateData(leaveTimeRequest, selectionRegion, manager);
    }
};

LeaveTimePlanningPage.rejectDeletionAndUpdateData = function(leaveTimeRequest, selectionRegion)
{
    var dateFrom = LeaveTimePlanningPage.pageData.dateFrom; // must be saved before request
    var manager = new LeaveTimePlanningControlManager.RejectRequestDeletionManager(
            leaveTimeRequest,
            LeaveTimePlanningPage.pageData.dateFrom,
            LeaveTimePlanningPage.pageData.dateTo,
            function(invocationResult) { LeaveTimePlanningPage.requestSaved(invocationResult, dateFrom); }
    );
    LeaveTimePlanningPage.makeRequestAJAXCallWithManagerAndUpdateData(leaveTimeRequest, selectionRegion, manager);
};

LeaveTimePlanningPage.deleteRequestWithConfirmation = function(leaveTimeRequest, selectionRegion)
{
    if (LeaveTimePlanningPage.deletionConfirmation(leaveTimeRequest))
    {
        LeaveTimePlanningPage.deleteRequestAndUpdateData(leaveTimeRequest, selectionRegion);
    }
};

LeaveTimePlanningPage.deletionConfirmation = function(leaveTimeRequest)
{
    return confirm(messageResource.getMessage(leaveTimeRequest.past ? "actiplans.leave_time_requests.delete_request_confirmation.past" : "actiplans.leave_time_planning.delete_request_confirmation"));
};

LeaveTimePlanningPage.existingRequestSelected = function()
{
    LeaveTimePlanningPage.leaveTableUpdateController.abortUpdateSchedulesRequest();

    if (LeaveTimePlanningPage.requestMenu)
    {
        // this helps to switch between menus of different requests
        LeaveTimePlanningPage.requestMenu.suspendEvents();
        LeaveTimePlanningPage.requestMenu.hide();
    }

    var selectionRegion = LeaveTimePlanningPage.scheduleTableController.getSelectionController().getSelection();
    var row = LeaveTimePlanningPage.scheduleTableController.getRow(selectionRegion.rowMin);
    var leaveTimeRequest = row.getCell(selectionRegion.colMin).getValue().requestData;

    var statusData = row.getCell(selectionRegion.colMin).getValue().statusData;

    // copy the request object.
    leaveTimeRequest = jQuery.extend({}, leaveTimeRequest);
    var items = [];
    var isPast = leaveTimeRequest.past;

    if (!leaveTimeRequest.waitingDeletion &&  _.isNull(statusData))
    {
        var isApprovableRequest = LeaveTimePlanningPage.canEditPastRequests || !isPast; // check if possible to manage request (if there is no integration or if it future request)
        if (row.isApprovableUser && isApprovableRequest)
        {
            if (leaveTimeRequest.status == LeaveTimeRequestStatus.APPROVED && !leaveTimeRequest.requestChange)
            {
                if (leaveTimeRequest.revocableApprovedRequest)
                {
                    items.push({
                                   text: messageResource.getMessage("actiplans.leavestable.context_menu.revoke"),
                                   listeners: {click: function() { LeaveTimePlanningPage.revokeApprovalAndUpdateData(leaveTimeRequest, selectionRegion); }},
                                   href: "javascript: void(0)",
                                   cls: "revokeContextButton"
                               });
                }
            }
            else
            {
                var approveMessageKey = "actiplans.leavestable.context_menu.approve";
                if (leaveTimeRequest.requestChange)
                {
                    approveMessageKey = "actiplans.leavestable.context_menu.approve_changes";
                }
                else if (leaveTimeRequest.past)
                {
                    approveMessageKey = "actiplans.leavestable.context_menu.approve_past_leave";
                }

                items.push({
                               text: messageResource.getMessage(approveMessageKey),
                               listeners: {click: function() { LeaveTimePlanningPage.approveRequestAndUpdateData(leaveTimeRequest, selectionRegion, ""); }},
                               href: "javascript: void(0)",
                               cls: "approveContextButton"
                           });

                items.push({
                               text: messageResource.getMessage("actiplans.leavestable.context_menu.approve_with_comment"),
                               listeners: {click: function() {LeaveTimePlanningPage.showApproveDialog(selectionRegion, row, leaveTimeRequest); }},
                               href: "javascript: void(0)",
                               cls: "approveWithCommentContextButton"
                           });

            }
            var rejectMessageKey = "actiplans.leavestable.context_menu.reject";
            if (leaveTimeRequest.requestChange)
            {
                rejectMessageKey = "actiplans.leavestable.context_menu.reject_changes";
            }
            else if (isPast)
            {
                rejectMessageKey = "actiplans.leavestable.context_menu.reject_past_leave";
            }
            items.push({
                           text: messageResource.getMessage(rejectMessageKey),
                           listeners: {
                               click: function()
                               {
                                   if (!leaveTimeRequest.requestChange)
                                       LeaveTimePlanningPage.showRejectDialog(selectionRegion, row, leaveTimeRequest);
                                   else
                                       LeaveTimePlanningPage.rejectRequestAndUpdateData(leaveTimeRequest, selectionRegion, null)
                               }
                           },
                           href: "javascript: void(0)",
                           cls: "rejectContextButton"
                       });

            if (items.length > 0)
                items.push(new Ext.menu.Separator());
        }

        if ((row.isApprovableUser || row.editableNotApprovableUser) && isApprovableRequest)
        {
            items.push({
                           text: messageResource.getMessage(isPast ? "actiplans.leavestable.context_menu.edit_past" : "actiplans.leavestable.context_menu.edit"),
                           listeners: {click: LeaveTimePlanningPage.showEditRequestDialog},
                           href: "javascript: void(0)",
                           cls: "editContextButton"
                       });

            items.push(new Ext.menu.Separator());
        }

        items.push({
                       text: messageResource.getMessage(isPast ? "actiplans.leavestable.context_menu.delete_past" : "actiplans.leavestable.context_menu.delete"),
                       listeners: {click: function() {LeaveTimePlanningPage.deleteRequestWithConfirmation(leaveTimeRequest, selectionRegion)}},
                       href: "javascript: void(0)",
                       cls: "deleteContextButton"
                   });
    }
    else if (row.isApprovableUser && leaveTimeRequest.waitingDeletion)
    {
        items.push({
                       text: messageResource.getMessage("actiplans.leavestable.context_menu.approve_deletion"),
                       listeners: {click: function() {LeaveTimePlanningPage.approveDeletionAndUpdateData(leaveTimeRequest, selectionRegion)}},
                       href: "javascript: void(0)",
                       cls: "approveContextButton"
                   });

        items.push({
                       text: messageResource.getMessage("actiplans.leavestable.context_menu.reject_deletion"),
                       listeners: {click: function() {LeaveTimePlanningPage.rejectDeletionAndUpdateData(leaveTimeRequest, selectionRegion, null); }},
                       href: "javascript: void(0)",
                       cls: "rejectContextButton"
                   });
    }

    if (items.length > 0)
    {
        LeaveTimePlanningPage.requestMenu = new Ext.menu.Menu({
            items: items,
            cls: "at-dropdown-list-btn-menu leaveRequestContextMenu"
        });

        LeaveTimePlanningPage.requestMenu.addListener("hide", function()
        {
            if (LeaveTimePlanningPage.requestDialog == null)
                LeaveTimePlanningPage.scheduleTableController.clearSelection();
            LeaveTimePlanningPage.requestMenu = null;
        });

        LeaveTimePlanningPage.requestMenu.show(row.getCell(selectionRegion.colMax).getCellEl());

        doOnceOnWindowResize(function() { if (LeaveTimePlanningPage.requestMenu) LeaveTimePlanningPage.requestMenu.hide(); });
    }
};

LeaveTimePlanningPage.createDefaultRequest = function(userId, dateRange, isPast)
{
    if (!LeaveType.hasActiveLeaveTypes())
        return;

    return {
        id: -546,
        userId: userId,
        leaveTypeId: LEAVE_TYPES[ACTIVE_LEAVE_TYPES_ORDER[0]].id,
        status: LeaveTimeRequestStatus.REQUESTED,
        approverId: null,
        firstDate: dateRange.dateFrom.toJSON(),
        lastDate: dateRange.dateTo.toJSON(),
        length: dateRange.dateTo.offsetInDaysFrom(dateRange.dateFrom) + 1, // TODO: this can be calculated from selection range
        firstDayDuration: -1, // whole day
        lastDayDuration: -1, // whole day
        middleDayDuration: -1, // whole day
        comment: null,
        past: isPast
    };
};

LeaveTimePlanningPage.requestSaved = function(invocationResult, dateFrom)
{
    try
    {
        if (invocationResult.data)
            LeaveTimePlanningPage.scheduleTableController.updateUserSchedule(invocationResult.data, dateFrom);
    }
    catch (e)
    {
        alert(e);
    }
};

LeaveTimePlanningPage.makeRequestAJAXCallWithManagerAndUpdateData = function(request, tableRegion, manager)
{
    if (LeaveTimePlanningPage.requestDialog)
        LeaveTimePlanningPage.requestDialog.close();
    LeaveTimePlanningPage.requestDialog = null;

    LeaveTimePlanningPage.scheduleTableController.clearSelection();
    LeaveTimePlanningPage.scheduleTableController.showProgress(tableRegion);
    LeaveTimePlanningPage.scheduleTableController.disableUserActivities();

    RpcControlManager.cancelAllAJAXRequests();
    var updateTable = function()
    {
        LeaveTimePlanningPage.leaveTableUpdateController.simpleUpdateSchedules(
                LeaveTimePlanningPage.dateFrom(), 'leavesManagementPageSpinner');
    };
    manager.handleNoRightsError = function(exception)
    {
        var errorThrown = {exception: getExceptionNameFromTrace(exception.trace)};
        return new CorporateScheduleErrorHandler(updateTable).defaultErrorHandler(null, null, errorThrown);
    };
    RpcControlManager.saveRpcControlBoundData(manager, undefined);
};

LeaveTimePlanningPage.makeRequestAJAXCallAndUpdateData = function(request, tableRegion, rpcManagerConstructor)
{
    var dateFrom = LeaveTimePlanningPage.pageData.dateFrom; // must be saved before request
    var manager = new rpcManagerConstructor(
            request,
            LeaveTimePlanningPage.pageData.dateFrom,
            LeaveTimePlanningPage.pageData.dateTo,
            function(invocationResult) { LeaveTimePlanningPage.requestSaved(invocationResult, dateFrom); }
    );

    LeaveTimePlanningPage.makeRequestAJAXCallWithManagerAndUpdateData(request, tableRegion, manager);
};

LeaveTimePlanningPage.setAssignedToMeCheckBox = function(showAssignedToMeOption)
{
    var $assignedToMeFilterBlock = $('#displayOnlyAssignedToMeUsersBlock');
    if (!showAssignedToMeOption)
    {
        $assignedToMeFilterBlock.hide();
        $('#searchFieldContainer').toggleClass('onSecondLine', !allUserGroups && !allTimeZoneGroups);
    }
    else
    {
        var state = LeaveTimePlanningPage.pageStateController.getProperty('displayOnlyAssignedToMeUsers');
        $('#displayOnlyAssignedToMeUsersChBox').prop('checked', state);
        $assignedToMeFilterBlock.show();
        $('#searchFieldContainer').removeClass('onSecondLine');
    }
};

LeaveTimePlanningPage.init = function(dateFrom, pageData, allUserGroups, allTimeZoneGroups, selectedGroupIds, selectedGroupsType,
                                      displayOnlyAssignedToMeUsers, dateRangeCalculator, stateController, canEditPastRequests, wasAtIntegrated)
{
    LeaveTimePlanningPage.pageStateController = stateController;

    LeaveTimePlanningPage.pageData = pageData;

    LeaveTimePlanningPage.canEditPastRequests = canEditPastRequests;
    LeaveTimePlanningPage.wasAtIntegrated = wasAtIntegrated;
    LeaveTimePlanningPage.isCurrentUserActiveInAT = pageData.currentUserActiveInAT;

    LeaveTimePlanningPage.pageStateController.setProperty('displayOnlyAssignedToMeUsers', displayOnlyAssignedToMeUsers);

    LeaveTimePlanningPage.setAssignedToMeCheckBox(pageData.showAssignedToMeOption);

    LeaveTimePlanningPage._notApplicableRequestDialog = new DropdownDialog({
        dialogDiv: $('#notApplicableRequestLabel'),
        hideButtons: true,
        wrapperClass: 'notApplicableRequestTooltip',
        showEmptyTitleBar: false,
        onClose: LeaveTimePlanningPage.hideNotApplicableRequestTooltip,
        hideOnResize: true
    });

    LeaveTimePlanningPage._autoApprovedUserDialog = new DropdownDialog({
        dialogDiv: $('#requestAutoApprovalLabel'),
        hideButtons: true,
        wrapperClass: 'notApplicableRequestTooltip',
        showEmptyTitleBar: false,
        onClose: LeaveTimePlanningPage.hideNotApplicableRequestTooltip,
        hideOnResize: true
    });

    LeaveTimePlanningPage.leaveTableUpdateController = new LeaveTableUpdateController().init(
            dateRangeCalculator,
            LeaveTimePlanningPage.updatePageElements,
            "LeaveTimePlanningService.getData",
            function(dateFrom, dateRange)
            {
                return [
                    dateRange.dateFrom.toJSON(),
                    dateRange.dateTo.toJSON(),
                    LeaveTimePlanningPage.userGroupsSelector ? LeaveTimePlanningPage.userGroupsSelector.getSelection() : null,
                    LeaveTimePlanningPage.userGroupsSelector ? LeaveTimePlanningPage.userGroupsSelector.getPanel().getGroupType() : null,
                    LeaveTimePlanningPage.scheduleTableController.getSearchEmployeesTokens(),
                    LeaveTimePlanningPage.pageStateController.getProperty('pageNum'),
                    LeaveTimePlanningPage.pageStateController.getProperty('recordsPerPage'),
                    LeaveTimePlanningPage.pageStateController.getProperty('sortBy'),
                    LeaveTimePlanningPage.pageStateController.getProperty('isAscending'),
                    LeaveTimePlanningPage.pageStateController.getProperty('displayOnlyAssignedToMeUsers'),
                    dateFrom.toJSON()
                ];
            },
            function(pageData)
            {
                LeaveTimePlanningPage.pageData = pageData;
                LeaveTimePlanningPage.setAssignedToMeCheckBox(pageData.showAssignedToMeOption);
            },
            LeaveTimePlanningPage.pageStateController
    );

    LeaveTimePlanningPage.scheduleTableController = new LeaveTimePlanningPage.ScheduleTableController().init(allUserGroups != null, allTimeZoneGroups != null);
    LeaveTimePlanningPage.mouseUpHook = function(e)
    {
        var indicators = LeaveTimePlanningPage.userGroupsSelector._panel.customScroller.indicators;
        for (var idx = 0; idx < indicators.length; idx++)
            indicators[idx]._end(e);
        return true;
    };

    LeaveScheduleUtils.createLegendWithFloat("scheduleLegendSecondBlock");

    if (allUserGroups)
    {
        allUserGroups = [{id: SystemSettings.usersWithoutGroupId, name: messageResource.getMessage('common.users_without_group')}].concat(allUserGroups);

        if (selectedGroupsType == GroupType.USER_GROUP)
        {
            $.each(allUserGroups, function()
            {
                if ($.inArray(this.id, selectedGroupIds) != -1)
                    this.selected = true;
            });
        }
    }

    if (allTimeZoneGroups)
    {
        $.each(allTimeZoneGroups, function()
        {
            if (selectedGroupsType == GroupType.TIME_ZONE_GROUP && $.inArray(this.id, selectedGroupIds) != -1)
                this.selected = true;
            this.isDefault = this.id == DEFAULT_TIME_ZONE_GROUP_ID;
        });
    }

    if (allUserGroups || allTimeZoneGroups)
    {
        LeaveTimePlanningPage.userGroupsSelector = new ItemSelector(
                '.userGroupsFilterPlaceholder',
                function(selection)
                {
                    LeaveTimePlanningPage.leaveTableUpdateController.simpleUpdateSchedules(LeaveTimePlanningPage.dateFrom(), 'leavesManagementPageSpinner');
                },
                AT.SmartGroupsSelectorPanel,
                null,
                false,
                {userGroupsData: allUserGroups, timeZoneGroupsData: allTimeZoneGroups, groupType: selectedGroupsType},
                true,
                function()
                {
                    LeaveTimePlanningPage.scheduleTableController._leaveTimePlanningTable.mouseEventsHandler.mouseUpHook =
                            LeaveTimePlanningPage.mouseUpHook;
                },
                function()
                {
                    LeaveTimePlanningPage.scheduleTableController._leaveTimePlanningTable.mouseEventsHandler.mouseUpHook = null;
                }
        );
    }

    stateController.addOnSaveStateActions(function(state)
                                          {
                                              if (LeaveTimePlanningPage.userGroupsSelector) LeaveTimePlanningPage.userGroupsSelector.saveState();
                                              LeaveTimePlanningPage.scheduleTableController.updateControls(state, false);
                                          });

    stateController.addOnLoadStateActions(function(state)
                                          {
                                              if (LeaveTimePlanningPage.userGroupsSelector) LeaveTimePlanningPage.userGroupsSelector.loadState();
                                              LeaveTimePlanningPage.scheduleTableController.updateControls(state, true);
                                          });

    PageOverlayTipsManager.setInstance(new LeaveTimePlanningPageOverlayTipsManager());
    LeaveTimePlanningPage.updatePageElements(dateFrom);
    LeaveTimePlanningPage.scheduleTableController.updateControls(stateController.getState(), true);

    $(window).resize(function() { LeaveTimePlanningPage.updatePageElements(LeaveTimePlanningPage.dateFrom()); });
};

/**
 * @return boolean
 *         true if tables was rendered
 *              (only if visible date range (dateFrom, dateFrom + visibleDays)
 *               contained in (LeaveTimePlanningPage.pageData.dateFrom, LeaveTimePlanningPage.pageData.dateTo)),
 *         false otherwise
 */
LeaveTimePlanningPage.updatePageElements = function(dateFrom)
{
    var loadedDateFrom = LeaveTimePlanningPage.pageData.dateFrom.parseJSONDate();
    var loadedDateTo = LeaveTimePlanningPage.pageData.dateTo.parseJSONDate();

    var dateTo = dateFrom.getNDaysAfter(LeaveTimePlanningPage.corporateScheduleWidthCalculator.getVisibleDaysCount() - 1);

    if (compareDates(loadedDateFrom, dateFrom) <= 0 && compareDates(dateTo, loadedDateTo) <= 0)
    {
        if (!LeaveTimePlanningPage.scheduleTableController.update(dateFrom, dateTo))
            return false;

        $('#contentInnerDiv').css('width', LeaveTimePlanningPage.corporateScheduleWidthCalculator.getContentWidth());

        return true;
    }
    else
    {
        return false;
    }
};

LeaveTimePlanningPage.onSortButtonClick = function(sortBy)
{
    var state = LeaveTimePlanningPage.pageStateController.getProperties(['sortBy', 'isAscending']);
    if (state.sortBy == sortBy)
    {
        state.isAscending = !state.isAscending;
    }
    else
    {
        state.sortBy = sortBy;
        state.isAscending = true;
    }
    LeaveTimePlanningPage.pageStateController.setProperties(state);

    LeaveTimePlanningPage.leaveTableUpdateController.simpleUpdateSchedules(LeaveTimePlanningPage.dateFrom(), 'leavesManagementPageSpinner');
};

LeaveTimePlanningPage._reloadPage = function()
{
    RpcControlManager.cancelAllAJAXRequests();
    LeaveTimePlanningPage.leaveTableUpdateController.abortUpdateSchedulesRequest();

    document.location.reload();
};

LeaveTimePlanningPage.leaveTypeNotFoundCallback = function()
{
    LeaveTimePlanningPage._reloadPage();
};


function LeaveTimePlanningControlManager(request, dateFrom, dateTo, callback)
{
    LeaveTimeRequestManager.call(this, request, dateFrom, dateTo, callback);
}

LeaveTimePlanningControlManager.prototype = _.create(LeaveTimeRequestManager.prototype);

LeaveTimePlanningControlManager.prototype.endSavingData = function()
{
    LeaveTimePlanningPage.scheduleTableController.hideProgress();
    LeaveTimePlanningPage.scheduleTableController.enableUserActivities();
};

LeaveTimePlanningControlManager.prototype.getService = function()
{
    return jsonrpc.LeaveTimePlanningService;
};

LeaveTimePlanningControlManager.prototype.onNoRightsToUpdateRequestException = function()
{
    if (LeaveTimePlanningPage.requestDialog)
        LeaveTimePlanningPage.requestDialog.close();
    LeaveTimePlanningPage.leaveTableUpdateController.simpleUpdateSchedules(LeaveTimePlanningPage.dateFrom(), 'leavesManagementPageSpinner');
};

LeaveTimePlanningControlManager.prototype.getMethods = function()
{
    return [
        "LeaveTimePlanningService.createRequest",
        "LeaveTimePlanningService.updateRequest",
        "LeaveTimePlanningService.deleteRequest",
        "LeaveTimePlanningService.approveRequest",
        "LeaveTimePlanningService.rejectRequest",
        "LeaveTimePlanningService.revokeRequestApproval",
        "LeaveTimePlanningService.recalculateRequest",
        "LeaveTimePlanningService.approveRequestDeletion",
        "LeaveTimePlanningService.rejectRequestDeletion"
    ];
};


LeaveTimePlanningControlManager.CreateRequestManager = function (request, dateFrom, dateTo, callback)
{
    LeaveTimePlanningControlManager.call(this, request, dateFrom, dateTo, callback);
};

LeaveTimePlanningControlManager.CreateRequestManager.prototype = _.create(LeaveTimePlanningControlManager.prototype, {
    makeRemoteCall: function(callback)
    {
        return this.getService().createRequest(
                callback,
                this.request.userId,
                this.request.leaveTypeId,
                this.request.firstDate,
                this.request.lastDate,
                this.request.firstDayDuration,
                this.request.lastDayDuration,
                this.request.middleDayDuration,
                this.request.comment,
                this.dateFrom,
                this.dateTo
        );
    }
});


LeaveTimePlanningControlManager.UpdateRequestManager = function (request, dateFrom, dateTo, callback)
{
    LeaveTimePlanningControlManager.call(this, request, dateFrom, dateTo, callback);
};

LeaveTimePlanningControlManager.UpdateRequestManager.prototype = _.create(LeaveTimePlanningControlManager.prototype, {
    makeRemoteCall: function(callback)
    {
        return this.getService().updateRequest(
                callback,
                this.request.id,
                this.request.userId,
                this.request.leaveTypeId,
                this.request.firstDayDuration,
                this.request.lastDayDuration,
                this.request.middleDayDuration,
                this.request.comment,
                this.dateFrom,
                this.dateTo
        );
    }
});


LeaveTimePlanningControlManager.DeleteRequestManager = function(request, dateFrom, dateTo, callback)
{
    LeaveTimePlanningControlManager.call(this, request, dateFrom, dateTo, callback);
};

LeaveTimePlanningControlManager.DeleteRequestManager.prototype = _.create(LeaveTimePlanningControlManager.prototype, {
    makeRemoteCall: function(callback)
    {
        return this.getService().deleteRequest(
                callback,
                this.request.id,
                this.request.userId,
                this.dateFrom,
                this.dateTo
        );
    }
});

LeaveTimePlanningControlManager.ApproveRequestDeletionManager = function(request, dateFrom, dateTo, callback)
{
    LeaveTimePlanningControlManager.call(this, request, dateFrom, dateTo, callback);
};

LeaveTimePlanningControlManager.ApproveRequestDeletionManager.prototype = _.create(LeaveTimePlanningControlManager.prototype, {
    makeRemoteCall: function(cb)
    {
        return this.getService().approveRequestDeletion(
                cb,
                this.request.id,
                this.request.userId,
                this.dateFrom,
                this.dateTo
        );
    }
});

LeaveTimePlanningControlManager.RejectRequestDeletionManager = function(request, dateFrom, dateTo, callback)
{
    LeaveTimePlanningControlManager.call(this, request, dateFrom, dateTo, callback);
};

LeaveTimePlanningControlManager.RejectRequestDeletionManager.prototype = _.create(LeaveTimePlanningControlManager.prototype, {
    makeRemoteCall: function(cb)
    {
        return this.getService().rejectRequestDeletion(
                cb,
                this.request.id,
                this.request.userId,
                this.dateFrom,
                this.dateTo
        );
    }
});

LeaveTimePlanningControlManager.ApproveRequestManager = function(request, comment, dateFrom, dateTo, callback)
{
    LeaveTimePlanningControlManager.call(this, request, dateFrom, dateTo, callback);
    this.comment = comment;
};

LeaveTimePlanningControlManager.ApproveRequestManager.prototype = _.create(LeaveTimePlanningControlManager.prototype);

LeaveTimePlanningControlManager.ApproveRequestManager.prototype.makeRemoteCall = function(callback)
{
    return this.getService().approveRequest(
            callback,
            this.request.id,
            this.request.userId,
            this.comment,
            this.dateFrom,
            this.dateTo
    );
};


LeaveTimePlanningControlManager.RejectRequestManager = function(request, comment, dateFrom, dateTo, callback)
{
    LeaveTimePlanningControlManager.call(this, request, dateFrom, dateTo, callback);
    this.comment = comment;
};

LeaveTimePlanningControlManager.RejectRequestManager.prototype = _.create(LeaveTimePlanningControlManager.prototype, {

    makeRemoteCall: function(callback)
    {
        return this.getService().rejectRequest(
                callback,
                this.request.id,
                this.request.userId,
                this.comment,
                this.dateFrom,
                this.dateTo
        );
    }
});


LeaveTimePlanningControlManager.RevokeApprovalManager = function(request, dateFrom, dateTo, callback)
{
    LeaveTimePlanningControlManager.call(this, request, dateFrom, dateTo, callback);
};

LeaveTimePlanningControlManager.RevokeApprovalManager.prototype = _.create(LeaveTimePlanningControlManager.prototype, {

    makeRemoteCall: function(callback)
    {
        return this.getService().revokeRequestApproval(
                callback,
                this.request.id,
                this.request.userId,
                this.dateFrom,
                this.dateTo
        );
    }
});


LeaveTimePlanningControlManager.RecalculateRequestManager = function(request, dateFrom, dateTo, callback)
{
    LeaveTimePlanningControlManager.call(this, request, dateFrom, dateTo, callback);
};

LeaveTimePlanningControlManager.RecalculateRequestManager.prototype = _.create(LeaveTimePlanningControlManager.prototype, {

    endSavingData: function(success)
    {
        if (!success)
        {
            LeaveTimePlanningPage.scheduleTableController.enableUserActivities();
            LeaveTimePlanningPage.requestDialog.closeWithoutAborting();
        }
    },

    makeRemoteCall: function(callback)
    {
        return this.getService().recalculateRequest(
                callback,
                this.request.id,
                this.request.userId,
                this.request.firstDate,
                this.request.lastDate,
                this.request.leaveTypeId,
                this.request.firstDayDuration,
                this.request.middleDayDuration,
                this.request.lastDayDuration,
                this.request.comment,
                this.dateFrom,
                this.dateTo
        );
    }
});

LeaveTimePlanningPage.ScheduleTableController = function()
{
    /**
     * @return this
     */
    this.init = function(userGroupsEnabled, timeZoneGroupsEnabled)
    {
        var that = this;

        this._userGroupsEnabled = userGroupsEnabled;
        this._timeZoneGroupsEnabled = timeZoneGroupsEnabled;

        this._schedulesTableContainer = document.getElementById("leaveTimePlanningTablePlaceholder");
        this._schedulesTempHeaderContainer = document.getElementById("leaveTimePlanningHeaderContainer");
        this._schedulesTableHeaderRow = document.getElementById("leaveTimePlanningTableHeaderRow");
        this._leaveTimePlanningScrollPanel = document.getElementById("leaveTimePlanningScrollPanel");
        this.$displayOnlyAssignedToMeUsersChBox = $("#displayOnlyAssignedToMeUsersChBox");

        this._totalRecordsPlaceholderContainer = $('#totalRecordsPlaceholder');
        this._employeesSearchFieldContainer = $('#employeesSearchField');

        this._lastSchedulesData = null;
        this._lastDateFrom = Date.create(1970, 1, 1);
        this._lastDateTo = Date.create(1970, 1, 1);

        this._leaveTimePlanningTable = null;

        this.$displayOnlyAssignedToMeUsersChBox.on("change",
                                                   function()
                                                   {
                                                       LeaveTimePlanningPage.pageStateController.setProperty('displayOnlyAssignedToMeUsers', $(this).is(":checked"));
                                                       LeaveTimePlanningPage.leaveTableUpdateController.simpleUpdateSchedules(LeaveTimePlanningPage.dateFrom(), 'leavesManagementPageSpinner');
                                                   }
        );

        this._pagerController = new PagerController().init(
                {
                    pagerData: LeaveTimePlanningPage.pageStateController.getProperties(['pageNum', 'totalRecords', 'recordsPerPage']),
                    handlersMap: {
                        pageChanged: function(pageNum)
                        {
                            LeaveTimePlanningPage.pageStateController.setProperty('pageNum', pageNum);
                            LeaveTimePlanningPage.leaveTableUpdateController.simpleUpdateSchedules(that.getDateFrom(), 'spinnerPlaceholderPager');
                        },
                        recordsPerPageChanged: function(recordsPerPage)
                        {
                            LeaveTimePlanningPage.pageStateController.setProperty('recordsPerPage', recordsPerPage);
                            that._pagerController.loadPage(1);
                        }
                    },
                    pagerControlsElementId: "pagerPanel",
                    pagerElementId: "pager",
                    recordsPerPageSelectorId: "recordsPerPageSelector",
                    minRecordsPerPage: MIN_RECORDS_PER_PAGE
                }
        );

        this._pagerController.render();

        $(this._schedulesTableHeaderRow).find('.sortButton').click(
                function()
                {
                    LeaveTimePlanningPage.onSortButtonClick($(this).parent().attr("data-sortBy"));
                });

        this._employeesSearchFieldContainer.attr('placeholder', messageResource.getMessage("corporate_schedule.leave_time_planning.js.searchEmployees.placeholder")).placeholder();

        var _updateSearchFilter = function()
        {
                LeaveTimePlanningPage.pageStateController.setProperty('employeesSearchText', that._employeesSearchFieldContainer.pVal());
                LeaveTimePlanningPage.pageStateController.setProperty('pageNum', 1);
                LeaveTimePlanningPage.leaveTableUpdateController.updateSchedulesWithDelay(that.getDateFrom(), 'leavesManagementPageSpinner', 200);

        };

        this._employeesSearchFieldContainer.searchField(_updateSearchFilter);

        this._employeesSearchFieldContainer.keyup(function(event)
                                                  {
                                                      if (event.keyCode == KEY_CODE_ESC && this.value != "")
                                                      {
                                                          this.value = "";
                                                      }

                                                      that._employeesSearchFieldContainer.updateSearchField();
                                                  });

        this._scrollPanelController = new ScrollPanelController().init(
                "leaveTimePlanningScrollPanel",
                function() { LeaveTimePlanningPage.leaveTableUpdateController.loadPreviousWeek(that.getDateFrom()); },
                function() { LeaveTimePlanningPage.leaveTableUpdateController.loadNextWeek(that.getDateFrom()); },
                function(date) { LeaveTimePlanningPage.leaveTableUpdateController.loadCustomWeek(date); },
                that.getDateFrom(),
                function() { return LeaveTimePlanningPage.pageData.corporateSchedule.today.parseJSONDate(); },
                LeaveTimePlanningPage.scheduleCalculator,
                function() { return getMinTodayOfUsers(LeaveTimePlanningPage.pageData.corporateSchedule, LeaveTimePlanningPage.pageData.userSchedules); }
        );

        return this;
    };

    this._updateScrollPanel = function()
    {
        this._scrollPanelController.updateControls(this.getDateFrom());
    };

    this.getDateFrom = function()
    {
        return this._lastDateFrom;
    };

    this.updateControls = function(state, includeImmediatelyControls)
    {
        this._updateScrollPanel();

        this._pagerController.setPagerStateAndUpdateUI(
                state.totalRecords,
                state.pageNum,
                state.recordsPerPage
        );
        this._totalRecordsPlaceholderContainer.text(state.totalRecords);

        if (includeImmediatelyControls)
        {
            this._employeesSearchFieldContainer.pVal(state.employeesSearchText);
        }

        $(this._schedulesTableHeaderRow).find('[data-sortBy]').removeClass("ascending descending");
        $(this._schedulesTableHeaderRow).find('[data-sortBy=' + state.sortBy + ']')
                .addClass(state.isAscending ? "ascending" : "descending");
    };

    this.clearSelection = function() { this._leaveTimePlanningTable.clearSelection(); };
    this.unhover = function() { this._leaveTimePlanningTable.hoverController.unhover(); };
    this.getSelectionController = function() { return this._leaveTimePlanningTable.getSelectionController() };
    this.getRow = function(rowNum) { return this._leaveTimePlanningTable.getRow(rowNum); };
    this.getDateRangeForRegion = function(selectionRegion) { return this._leaveTimePlanningTable.getDateRangeForRegion(selectionRegion); };

    this.updateUserSchedule = function(userSchedule, dateFrom)
    {
        LeaveTimePlanningPage.fillRevocableFlagOnUserRequests(userSchedule);

        var scheduleDataOffset = this._lastDateFrom.offsetInDaysFrom(dateFrom.parseJSONDate());
        this._leaveTimePlanningTable.updateUserSchedule(userSchedule, scheduleDataOffset);

        var userSchedules = LeaveTimePlanningPage.pageData.userSchedules;
        for (var i = 0; i < userSchedules.length; i++)
        {
            var oldUserSchedule = userSchedules[i];
            if (oldUserSchedule.userInfo.id == userSchedule.userInfo.id)
            {
                userSchedules[i] = userSchedule;
                break;
            }
        }
        PageOverlayTipsManager.getInstance().refreshDynamicTips();
    };

    this.showProgress = function(tableRegion) { this._leaveTimePlanningTable.showProgress(tableRegion); };
    this.hideProgress = function()
    {
        if (this._leaveTimePlanningTable)
        {
            this._leaveTimePlanningTable.hideProgress();
        }
    };

    this.enableUserActivities = function() { this._leaveTimePlanningTable.enableUserActivities(); };
    this.disableUserActivities = function() { this._leaveTimePlanningTable.disableUserActivities(); };

    /**
     * must be guaranteed that (dateFrom, dateTo) in (LeaveTimePlanningPage.pageData.dateFrom, LeaveTimePlanningPage.pageData.dateTo)
     * @param dateFrom
     * @param dateTo
     */
    this.update = function(dateFrom, dateTo)
    {
        var schedulesData = LeaveTimePlanningPage.pageData;
        var userGroupsEnabledChanged = this._userGroupsEnabled != schedulesData.userGroupsEnabled;
        var timeZoneGroupsEnabledChanged = this._timeZoneGroupsEnabled != schedulesData.timeZoneGroupsEnabled;
        if (userGroupsEnabledChanged || timeZoneGroupsEnabledChanged)
        {
            var message;
            if (userGroupsEnabledChanged)
            {
                message = schedulesData.userGroupsEnabled ?
                          messageResource.getMessage("actiplans.common.js.error.user_groups_enabled") :
                          messageResource.getMessage("actiplans.common.js.error.user_groups_disabled");
            }
            else
            {
                message = schedulesData.timeZoneGroupsEnabled ?
                          messageResource.getMessage("actiplans.common.js.error.time_zone_groups_enabled") :
                          messageResource.getMessage("actiplans.common.js.error.time_zone_groups_disabled");
            }

            alert(message);

            LeaveTimePlanningPage._reloadPage();
            return false;
        }

        if (schedulesData != this._lastSchedulesData || !dateFrom.isSameDate(this._lastDateFrom) || !dateTo.isSameDate(this._lastDateTo))
        {
            this.hideProgress();
            this._lastSchedulesData = schedulesData;
            this._lastDateFrom = dateFrom;
            this._lastDateTo = dateTo;
            LeaveTimePlanningPage.pageStateController.setProperty('pageNum', schedulesData.pageNum);
            LeaveTimePlanningPage.pageStateController.setProperty('totalRecords', schedulesData.totalCount);

            this._schedulesTempHeaderContainer.appendChild(this._schedulesTableHeaderRow);

            var schedulesDataOffset = dateFrom.offsetInDaysFrom(schedulesData.dateFrom.parseJSONDate());

            LeaveTimePlanningPage.fillRevocableFlagOnUsersRequests(schedulesData);

            if (this._leaveTimePlanningTable)
                this._leaveTimePlanningTable.hideTooltip();
            this._leaveTimePlanningTable = new LeaveTimePlanningTable(
                    null,
                    dateFrom,
                    dateTo,
                    new ArraySlice(schedulesData.corporateSchedule.workdayDurations, schedulesDataOffset),
                    schedulesData.corporateSchedule.today.parseJSONDate(),
                    schedulesData.groupsData,
                    schedulesData.userSchedules,
                    schedulesDataOffset,
                    LeaveTimePlanningPage.leaveTypeNotFoundCallback,
                    this._schedulesTableHeaderRow,
                    this._getTokensHighlighter(),
                    schedulesData.hasUsers,
                    LeaveTimePlanningPage.pageStateController.getProperty('sortBy'),
                    LeaveTimePlanningPage.pageData.ptoHoursPerDay,
                    LeaveTimePlanningPage.canEditPastRequests,
                    LeaveTimePlanningPage.wasAtIntegrated,
                    schedulesData.approvableUsers,
                    schedulesData.editableNotApprovableUsers,
                    LeaveTimePlanningPage.isCurrentUserActiveInAT
            );

            // When we want to show one dialog instead of another with a single click.
            // We need to hide previous dialog before opening new one, but unfortunately it might call clearSelection on close.
            // And clearSelection will change our state to hoverState which is quite unexpected.
            // So we should use chain oldState(old dialog) -> hoverState(close dialogs) -> newState(new dialog)
            this._leaveTimePlanningTable.addNewHoverListener(LeaveTimePlanningPage.closeDialogs);
            this._leaveTimePlanningTable.addNewRequestSelectionListener(LeaveTimePlanningPage.createRequest);
            this._leaveTimePlanningTable.addExistingRequestSelectionListener(LeaveTimePlanningPage.existingRequestSelected);
            this._leaveTimePlanningTable.addNotApplicableCellSelectionListener(LeaveTimePlanningPage.showNotApplicableRequestTooltip);

            this._schedulesTableContainer.innerHTML = '';
            this._leaveTimePlanningTable.buildDOM(this._schedulesTableContainer);
            this.enableUserActivities();

            // defined in leave_time_planning.jsp
            ie8fixTable();

            this._leaveTimePlanningScrollPanel.colSpan = dateTo.offsetInDaysFrom(dateFrom) + 1;

            this._updateScrollPanel(); // if change date and scroll while request is pending scroll panel must be updated
        }

        PageOverlayTipsManager.getInstance().refreshDynamicTips(false);

        return true;
    };

    this._PUNCTUATORS = StringTokenizer.PUNCTUATORS_FOR_SEARCH;

    this.getSearchEmployeesTokens = function()
    {
        return new StringTokenizer(this._PUNCTUATORS).getTokens(LeaveTimePlanningPage.pageStateController.getProperty('employeesSearchText').toLowerCase());
    };

    this._getTokensHighlighter = function()
    {
        var tokens = this.getSearchEmployeesTokens();
        return (tokens && tokens.length > 0) ? new TokensHighlighter(tokens, this._PUNCTUATORS) : new EmptyHighlighter();
    };
};

LeaveTimePlanningPage.fillRevocableFlagOnUserRequests = function(userSchedule)
{
    var userRequests = userSchedule.requests;
    var revocableRequestsSet = makeSet(userSchedule.revocableApprovedRequestIds);
    for (var i = 0; i < userRequests.length; i++)
    {
        var request = userRequests[i];
        request.revocableApprovedRequest = revocableRequestsSet[request.id] === true;
    }
};

LeaveTimePlanningPage.fillRevocableFlagOnUsersRequests = function(schedulesData)
{
    var userSchedules = schedulesData.userSchedules;
    for (var i = 0; i < userSchedules.length; i++)
    {
        LeaveTimePlanningPage.fillRevocableFlagOnUserRequests(userSchedules[i])
    }
};
