function ItemSelector( renderTo, onChange, panelConstructor, data, showInactive, panelParams, isNoShadow,
                       onShowListener, onHideListener )
{
    var that = this;
    this._renderTo = $( renderTo );

    if (!panelParams)
        panelParams = {};

    panelParams.showInactive = showInactive;
    panelParams.data = data;
    panelParams.showResetButton = true;

    this._panel = new panelConstructor( panelParams );

    var button = new AT.SelectorDropdownButton( {
                                                    renderTo: this._renderTo[0],
                                                    selectorPanel: this._panel,
                                                    dropDownMenuAnchor: "td.contents",
                                                    dropUpMenuAnchor: "td.contents",
                                                    additionalMenuCls: "ap-selector-menu",
                                                    isNoShadow: isNoShadow
                                                } );
    
    if(onShowListener)
        button.on("menushow",onShowListener);

    if(onHideListener)
        button.on( "menuhide",onHideListener);

    button.on( "menuhide", function ()
    {
        if( that._panel.isStateChanged() )
        {
            onChange( that.getSelection() );
        }
    } );

    this._panel.on( "change", function ()
    {
        that._updateUI();
    }, this );

    this.saveState();
    this._updateUI();
}

ItemSelector.prototype.getSelection = function ()
{
    if( !this._panel )
        return null; // filter is not enabled

    if( !this._panel.isCustomOptionSelected() )
        return null; // 'all' option selected

    return jQuery.map( this._panel.getSelection(), function ( val ) { return val.id; } );
};

ItemSelector.prototype.getPanel = function ()
{
    return this._panel;
};

ItemSelector.prototype.saveState = function ()
{
    this._panel.setInitialValuesFromCurrent();
};

ItemSelector.prototype.loadState = function ()
{
    this._panel.resetState( true );
};

ItemSelector.prototype._updateUI = function ()
{
    var selectedIds = this.getSelection();
    this._onUpdateState( selectedIds && selectedIds.length != 0 );
};

ItemSelector.prototype._onUpdateState = function ( active )
{
    if( active )
    {
        this._renderTo.addClass( "active" );
    }
    else
    {
        this._renderTo.removeClass( "active" );
    }

    if( this._onUpdateStateFunction )
    {
        this._onUpdateStateFunction( active );
    }
};

/**
 * function will be called when filter state changed (or more often).
 *         func(active), where active is equals to true, if filter is active
 */
ItemSelector.prototype.setOnUpdateStateFunction = function ( func )
{
    this._onUpdateStateFunction = func;
};
