function MonthYearSelector () {}


MonthYearSelector.RangesTypes = {
    CURRENT_MONTH: "currentMonthRange",
    LAST_THREE_MONTHS: "threeMonthsRange",
    LAST_SIX_MONTHS: "sixMonthsRange",
    LAST_TWELVE_MONTHS: "twelveMonthsRange",
    CUSTOM: "customRange"
};

MonthYearSelector.RangesTypesLengths = {
    "currentMonthRange": 1,
    "threeMonthsRange": 3,
    "sixMonthsRange": 6,
    "twelveMonthsRange": 12
};

MonthYearSelector.prototype.init = function ( $buttonEl, startSelectorDate, lastSelectorDate, applyCallback)
{
    var self = this;

    // create and render selector button
    this.$selectorButton = $buttonEl;
    this.$rangeLabel = $buttonEl.find('.monthYearSelectorButton');
    this.$selectorButton.click( function () { self.showPanel(); } );

    // initialize elements and event handlers
    this.selectorPanel = new MonthYearSelectorPanel($(document)).init();
    this._maxPeriodLength = (lastSelectorDate.getFullYear() - startSelectorDate.getFullYear()) * 12
                            +  (lastSelectorDate.getMonth() - startSelectorDate.getMonth()) + 1;
    this._initRangeTypes(this.selectorPanel, this._maxPeriodLength);
    this._createYearBlocks(this.selectorPanel, startSelectorDate, lastSelectorDate);

    var restoreAtClose = true;
    // create panel as dropdown dialog
    this.panel = new DropdownDialog({
                                        wrapperClass: "monthYearSelectorPanelWrapper noprint",
                                        dialogDiv: this.selectorPanel.$el,
                                        hideButtons: false,
                                        hideOnResize: true,
                                        showEmptyTitleBar : false,
                                        onClose: function (event) {
                                            self.$selectorButton.removeClass("active");
                                            if (restoreAtClose)
                                            {
                                                self._selectDateRange(self._selectionBackup.selectedDateRange, self._selectionBackup.rangeType);
                                            }
                                            restoreAtClose = true;
                                        },
                                        okButtonText: messageResource.getMessage( "common.apply" ),
                                        onOkClick: function()
                                        {
                                            restoreAtClose = false;
                                            self.hidePanel();
                                            applyCallback();
                                        }
                                    });

    return this;
};

MonthYearSelector.prototype._initRangeTypes = function(panel, maxPeriodLength)
{
    var self = this;
    _.forEach(_.values(MonthYearSelector.RangesTypes), function(rangeType)
    {
        var rangeLength = MonthYearSelector.RangesTypesLengths[rangeType];
        if (!_.isUndefined(rangeLength) && rangeLength > maxPeriodLength)
        {
            panel.disableRangeType(rangeType)
        } else {
            panel.find('.' + rangeType).click(function() { self._selectDateRangeAndScroll(self._selectedDateRange, rangeType);  });
        }
    });
};

MonthYearSelector.prototype._createYearBlocks = function(panel, startSelectorDate, lastSelectorDate)
{
    panel.years = [];
    this._startSelectorDate = startSelectorDate;
    this._lastSelectorDate = lastSelectorDate;

    for (var year = startSelectorDate.getFullYear(); year <= lastSelectorDate.getFullYear(); year++)
    {
        var yearBlock = new MonthYearSelectorPanel.YearBlock(panel.$yearBlocksEl, year).init();
        this._createMonthBlocks(yearBlock);
        panel.years[year] = yearBlock;
    }

    panel.years[lastSelectorDate.getFullYear()].$el.addClass("last");
};

MonthYearSelector.prototype._createMonthBlocks = function(yearBlock)
{
    var self = this;
    yearBlock.months = [];

    for (var month = 0; month < 12; month++)
    {
        var beforeSelectorStartDate = yearBlock.getYear() == this._startSelectorDate.getFullYear() && month < this._startSelectorDate.getMonth();
        var afterSelectorLastDate = yearBlock.getYear() == this._lastSelectorDate.getFullYear() && month > this._lastSelectorDate.getMonth();
        var disabled = beforeSelectorStartDate || afterSelectorLastDate;
        var monthBlock = yearBlock.addMonthBlock(month, disabled);
        if (!disabled)
        {
            monthBlock.$el.click(_.bind(this.changeSelectedDateRange, this, new Date(yearBlock.getYear(), month)));
            var date = new Date(yearBlock.getYear(), month);
            monthBlock.$el.hover(_.bind(this._hoverDateRange, this, date), _.bind(this._unhoverDateRange, this, date));
        }
        else if (beforeSelectorStartDate)
        {
            MonthYearSelectorPanel.NoLeavesToolTip.getInstance().subscribeToMouseEvents(monthBlock.$el);
        }
        yearBlock.months[month] = monthBlock;
    }
};

/**
 * @param dateRange date range to iterate months
 * @param iteratee apply iteratee to each month
 * @return number period length in months
 */
MonthYearSelector.prototype._iterateMonthsAndApply = function(dateRange, iteratee)
{
    if (_.isUndefined(dateRange))
        return 0;

    if (_.isNull(dateRange.dateTo))
    {
        iteratee.call(this.selectorPanel.years[dateRange.dateFrom.getFullYear()].months[dateRange.dateFrom.getMonth()]);
        return 1;
    }

    var mi,d;
    var periodLength = 0;
    for ( mi = new MonthIterator(dateRange.dateFrom, dateRange.dateTo); mi.hasNext(); )
    {
        d = mi.next();
        iteratee.call(this.selectorPanel.years[d.getFullYear()].months[d.getMonth()]);
        periodLength++;
    }
    return periodLength;
};

MonthYearSelector.prototype._changePanelDateRange = function(newDate)
{
    var periodLength = 0;

    if (!_.isNull(this._selectedDateRange.dateTo))
    {
        this._iterateMonthsAndApply(this._selectedDateRange, MonthYearSelectorPanel.MonthBlock.prototype.unselectAndUnhover);
        this.selectorPanel.years[newDate.getFullYear()].months[newDate.getMonth()].select();
        this._selectedDateRange = new DateRange(newDate, null);
    }
    else
    {
        this._selectedDateRange = new DateRange(Date.min(this._selectedDateRange.dateFrom, newDate), Date.max(this._selectedDateRange.dateFrom, newDate));
        periodLength = this._iterateMonthsAndApply(this._selectedDateRange, MonthYearSelectorPanel.MonthBlock.prototype.select);
    }
    this._updatePeriodLength(periodLength);
};

MonthYearSelector.prototype._hoverDateRange = function(newDate)
{
    if (_.isNull(this._selectedDateRange.dateTo))
    {
        this._iterateMonthsAndApply(new DateRange(Date.min(this._selectedDateRange.dateFrom, newDate), Date.max(this._selectedDateRange.dateFrom, newDate)),
                                    MonthYearSelectorPanel.MonthBlock.prototype.hover);
    }
};

MonthYearSelector.prototype._unhoverDateRange = function(newDate)
{
    if (_.isNull(this._selectedDateRange.dateTo))
    {
        this._iterateMonthsAndApply(new DateRange(Date.min(this._selectedDateRange.dateFrom, newDate), Date.max(this._selectedDateRange.dateFrom, newDate)),
                                    MonthYearSelectorPanel.MonthBlock.prototype.unhover);
    }
};

MonthYearSelector.prototype.changeSelectedDateRange = function (newDate) {
    this._changePanelDateRange(newDate);
    this._rangeType = MonthYearSelector.RangesTypes.CUSTOM;
    this.selectorPanel.selectRangeType(this._rangeType);
    this.selectorPanel.$selectedPeriod.html("<span>" + this._getRangeLabel() + "</span>");
    this._updateSelectorButton();
};

MonthYearSelector.prototype._isDateRangeValid = function(selectedDateRange, rangeType)
{
    if (rangeType == MonthYearSelector.RangesTypes.CUSTOM)
    {
        var isFirstDateAvailable = (new Date(this._startSelectorDate.getFullYear(), this._startSelectorDate.getMonth())).getTime() <= selectedDateRange.dateFrom.getTime();
        var isLastDateAvailable = (new Date(this._lastSelectorDate.getFullYear(), this._lastSelectorDate.getMonth() + 1)).getTime() > selectedDateRange.dateTo.getTime();
        return isFirstDateAvailable && isLastDateAvailable;
    }

    var rangeLength = MonthYearSelector.RangesTypesLengths[rangeType];
    return _.isUndefined(rangeLength) || (rangeLength <= this._maxPeriodLength);
};

MonthYearSelector.prototype._selectDateRange = function (selectedDateRange, rangeType) {
    this._iterateMonthsAndApply(this._selectedDateRange, MonthYearSelectorPanel.MonthBlock.prototype.unselectAndUnhover);
    this._rangeType = (this._isDateRangeValid(selectedDateRange, rangeType)) ? rangeType : MonthYearSelector.RangesTypes.CURRENT_MONTH;

    switch (this._rangeType) {
        case MonthYearSelector.RangesTypes.CUSTOM:
            this._selectedDateRange = selectedDateRange;
            break;
        case MonthYearSelector.RangesTypes.LAST_THREE_MONTHS:
            this._selectedDateRange = new DateRange(new Date(this._lastSelectorDate.getFullYear(), this._lastSelectorDate.getMonth() - 2), this._lastSelectorDate);
            break;
        case MonthYearSelector.RangesTypes.LAST_SIX_MONTHS:
            this._selectedDateRange = new DateRange(new Date(this._lastSelectorDate.getFullYear(), this._lastSelectorDate.getMonth() - 5), this._lastSelectorDate);
            break;
        case MonthYearSelector.RangesTypes.LAST_TWELVE_MONTHS:
            this._selectedDateRange = new DateRange(new Date(this._lastSelectorDate.getFullYear(), this._lastSelectorDate.getMonth() - 11), this._lastSelectorDate);
            break;
        default:
            this._selectedDateRange = new DateRange(this._lastSelectorDate, this._lastSelectorDate);
            this._rangeType = MonthYearSelector.RangesTypes.CURRENT_MONTH;
    }

    var periodLength = this._iterateMonthsAndApply(this._selectedDateRange, MonthYearSelectorPanel.MonthBlock.prototype.select);

    this.selectorPanel.$selectedPeriod.html("<span>" + this._getRangeLabel() + "</span>");
    this.selectorPanel.selectRangeType(this._rangeType);
    this._updatePeriodLength(periodLength);
    this._updateSelectorButton();
};

MonthYearSelector.prototype._selectDateRangeAndScroll = function(selectedDateRange, rangeType)
{
    this._selectDateRange(selectedDateRange, rangeType);
    this._scrollToFirstSelectedDate();
};

MonthYearSelector.prototype.setSelectedDateRange = function(selectedDateRange, rangeType)
{
    this._selectionBackup =  { selectedDateRange: selectedDateRange, rangeType: rangeType};
    this._selectDateRange(selectedDateRange, rangeType);
};

MonthYearSelector.prototype.getSelectedDateRange = function () {
    return this._selectedDateRange;
};

MonthYearSelector.prototype.getRangeType = function()
{
    return this._rangeType;
};

MonthYearSelector.prototype._updateSelectorButton = function () {

    this.$rangeLabel.html("<span>" + this._getRangeLabel() + "</span>");
};

MonthYearSelector.prototype._makeSelectionLabel = function(date)
{
    return Date.monthNamesShort[date.getMonth()] + " " + date.getFullYear();
};

MonthYearSelector.prototype._getRangeLabel  = function()
{
    var _dateFrom = this._selectedDateRange.dateFrom;
    var _dateTo = this._selectedDateRange.dateTo;
    var label = this._makeSelectionLabel(_dateFrom);
    if (!_.isNull(_dateTo))
    {
        var isOneMonthPeriod = _dateFrom.getFullYear() == _dateTo.getFullYear() && _dateFrom.getMonth() == _dateTo.getMonth();
        label += (isOneMonthPeriod) ? "" : " - " + this._makeSelectionLabel(this._selectedDateRange.dateTo);
    }
    return label;
};

MonthYearSelector.prototype.showPanel = function () {
    this.$selectorButton.addClass("active");
    this.panel.showAt(this.$selectorButton, {     my: "left top",
                                            at: "left bottom",
                                            collision: "flip" }
    );

    ScrollUtils.refreshCustomScroll(this.selectorPanel.yearBlocksScroll);
    this._scrollToFirstSelectedDate();
};

MonthYearSelector.prototype.hidePanel = function () {
    this.panel.hide();
};

MonthYearSelector.prototype._updatePeriodLength = function(periodLength)
{
    this.selectorPanel.$periodLength.text((periodLength > 1) ? periodLength + ' ' + messageResource.getMessage('common.months')
                                                  : '1 ' + messageResource.getMessage('common.month'));
};

MonthYearSelector.prototype._scrollToFirstSelectedDate = function () {
    var dateFrom = this._selectedDateRange.dateFrom;
    var dateFromBlock = this.selectorPanel.years[dateFrom.getFullYear()].months[dateFrom.getMonth()];
    ScrollUtils.scrollIntoViewVertical(this.selectorPanel.yearBlocksScroll, dateFromBlock.$el[0]);
};
