function MonthYearSelectorPanel($containerEl)
{
    TemplateView.call(this, $containerEl);
}

MonthYearSelectorPanel.prototype = _.create(TemplateView.prototype, {
    html: "<div class='monthYearSelectorPanel'>" +
          " <div class='rangeSelectorContainer'>" +
          "     <div class='emptyRange'></div>" +
          "     <div class='currentMonthRange range'><span class='rangeLabel'><%=currentMonthLabel%></span></div>" +
          "     <div class='threeMonthsRange range'><span class='rangeLabel'><%=threeMonthsLabel%></span></div>" +
          "     <div class='sixMonthsRange range'><span class='rangeLabel'><%=sixMonthsLabel%></span></div>" +
          "     <div class='twelveMonthsRange range'><span class='rangeLabel'><%=twelveMonthsLabel%></span></div>" +
          "     <div class='customRange range'><span class='rangeLabel'><%=customRangeLabel%></span></div>" +
          " </div>" +
          " <div class='calendarSelectorContainer'>" +
          "   <div class='selectorHeader'>" +
          "     <div class='selectedPeriod'></div>" +
          "     <div class='periodLength'></div>" +
          "   </div>" +
          "   <div class='yearBlocksWrapper'>" +
          "     <div class='yearBlocks'></div>" +
          "   </div>" +
          " </div> " +
          "</div>",

    afterRender: function()
    {
        var self = this;
        this.$yearBlocksEl = this.find(".yearBlocks");
        this.$selectedPeriod = this.find(".selectedPeriod");
        this.$periodLength = this.find(".periodLength");

        this.yearBlocksScroll = ScrollUtils.addVerticalScroll(this.find(".yearBlocksWrapper"), {isFaded: false});
    },

    messageKeys: function()
    {
        return {
            "currentMonthLabel": "controls.month_year_period_selector.ranges.current_month",
            "threeMonthsLabel": "controls.month_year_period_selector.ranges.three_months",
            "sixMonthsLabel": "controls.month_year_period_selector.ranges.six_months",
            "twelveMonthsLabel": "controls.month_year_period_selector.ranges.twelve_months",
            "customRangeLabel": "controls.month_year_period_selector.ranges.custom_range"
        }
    },
    
    selectRangeType: function(rangeType)
    {
        var self = this;
        _.forEach(_.values(MonthYearSelector.RangesTypes), function(type)
        {
            self.find('.' + type).toggleClass('selected', type == rangeType);
        });
    },
    
    disableRangeType: function(rangeType)
    {
        this.find('.' + rangeType).addClass('disabled');
    }
});



MonthYearSelectorPanel.YearBlock = function($containerEl, year, panel)
{
    TemplateView.call(this, $containerEl);
    this.year = year;
};

MonthYearSelectorPanel.YearBlock.prototype = _.create(TemplateView.prototype,
{
    html: "<div class='yearBlock'>" +
          " <span class='yearLabel'><%=year%></span>" +
          " <table class='monthBlocks'>" +
          "     <tr class='firstMonthsRow'></tr>" +
          "     <tr class='secondMonthsRow'></tr>" +
          "     <tr class='thirdMonthsRow'></tr>" +
          "     <tr class='forthMonthsRow'></tr>" +
          " </table>" +
          "</div>",

    afterRender: function()
    {
        this.$firstRow = this.find(".firstMonthsRow");
        this.$secondRow = this.find(".secondMonthsRow");
        this.$thirdRow = this.find(".thirdMonthsRow");
        this.$forthRow = this.find(".forthMonthsRow");
    },

    templateModel: function()
    {
        return { year: this.year }
    },

    getYear: function()
    {
        return this.year;
    },
    
    addMonthBlock: function(month, disabled)
    {
        if (month < 3)
            return new MonthYearSelectorPanel.MonthBlock(this.$firstRow, month, disabled).init();
        if (month < 6)
            return new MonthYearSelectorPanel.MonthBlock(this.$secondRow, month, disabled).init();
        if (month < 9)
            return new MonthYearSelectorPanel.MonthBlock(this.$thirdRow, month, disabled).init();

        return new MonthYearSelectorPanel.MonthBlock(this.$forthRow, month, disabled).init();
    }

});

MonthYearSelectorPanel.MonthBlock = function($containerEl, monthIndex, isDisabled)
{
    TemplateView.call(this, $containerEl);
    this.monthIndex = monthIndex;
    this.isDisabled = isDisabled;
};

MonthYearSelectorPanel.MonthBlock.prototype = _.create(TemplateView.prototype, {
    html: "<td class='monthCell'><%=month%></td>",

    afterRender: function()
    {
        if (this.isDisabled)
        {
            this.$el.addClass("disabled");
        }
    },

    templateModel: function()
    {
        return {month: Date.monthNamesShort[this.monthIndex].toUpperCase()}
    },
    
    getMonthIndex: function()
    {
        return this.monthIndex;
    },

    select: function()
    {
        this.$el.addClass("selected");
    },

    hover: function()
    {
        this.$el.addClass("hovered");
    },

    unhover: function()
    {
        this.$el.removeClass("hovered");
    },

    unselect: function()
    {
        this.$el.removeClass("selected");
    },

    unselectAndUnhover: function()
    {
        this.unselect();
        this.unhover();
    }
});

MonthYearSelectorPanel.NoLeavesToolTip = function($containerEl)
{
    Tooltip.call(this, $containerEl);
};

MonthYearSelectorPanel.NoLeavesToolTip.prototype = _.create(Tooltip.prototype, {
    html: "<div id='noLeavesTooltip' class='noLeavesTooltip noprint'><%-labelText%></div>",

    messageKeys: function()
    {
        return {
            labelText: "controls.tooltip.no_leaves"
        };
    }
});


MonthYearSelectorPanel.NoLeavesToolTip.getInstance = function()
{
    if (!this._instance)
        this._instance = new this($("body")).init();
    return this._instance;
};