Ext.namespace("AT");

AT.UserForLeaveHistorySelectorPanel = Ext.extend(AT.ThreePositionUserSelectorPanel, {
    showInactiveLabelKey: "actiplans.leave_time_history.user_selector.show_disabled",
    labelNoItemsSelectedKey: "actiplans.leave_time_history.user_selector.choose_user.label",

    optionValue: true,
    allItemsOptionValue: false,
    customOption: null,

    selectionChangedOnShowInactive: false,

    resetOnHide: false,

    createCustomOption: function()
    {
        this.customOption = undefined;
    },

    createGenericOptions: function()
    {
        this.genericOptions = [];
    },

    getCustomOptionSection: function() {

        var customOptionControls = [];
        // from left to right
        customOptionControls.push(this.viewByPanel);
        customOptionControls.push(this.getCustomOptionButtonsSection());

        this.customControlsContainer = new Ext.Container({
            cls: "userSelectorCustomControlsContainer",
            autoEl: "div",
            items: customOptionControls
        });
        return [this.customControlsContainer];
    },

    getCustomOptionButtons : function() {
        var fields = [];
        if (this.userGroupsEnabled || this.timeZoneGroupsEnabled ) {
          //  fields.push(new Ext.Panel({ cls: "ie8hack" }));
            fields.push(this.filterButton);
            fields.push(this.makeVerticalSeparator());
            fields.push(this.collapseAllButton, this.expandAllButton);
        }
        return fields;
    },

    getCurrentOptionValue: function() {
        return this.customOptionValue;
    },

    isCustomOptionSelected: function() {
        return true;
    },

    createFooterButtons: function() {
        //do nothing
    },

    getFooterSection: function() {
        return [];
    },

    getSelectedUserId: function() {
        var selection = this.getSelection(false);
        return selection.getSecondLevel().length > 0 ? selection.getSecondLevel()[0] : 0;
    },

    initHideUserGroupsStrategy: function ()
    {
        return new HideUserGroupsStrategy({ pleaseSelectMessageKey: "user_selector.one_item.please_select" });
    },

    initShowUserGroupsStrategy: function ()
    {
        return new ShowUserGroupsStrategy({ pleaseSelectMessageKey: "user_selector.one_item.please_select" });
    },

    collectCurrentSettings: function ()
    {
        return {
            showInactiveCheckboxValue: this.getShowInactive(),
            selectedUserId: this.getSelectedUserId(),
            searchString: this.filterWordsField ? this.filterWordsField.getSearchString() : null,
            initialSelection: this.createUserSelection(this.getSelectedUserId())
        };
    },

    createTree: function() {
        var tree = AT.UserForLeaveHistorySelectorPanel.superclass.createTree.call(this);
        var selectionController = new SingleItemTreeSelectionController(tree, TreeNode.TYPE_USER);
        tree.setSelectionController(selectionController);
        return tree;
    },
    
    initialize: function (loggedInUserId, userGroupsEnabled, timeZoneGroupsEnabled, users, userGroups, timeZoneGroups, selectedUserId)
    {
        AT.UserForLeaveHistorySelectorPanel.superclass.initialize.call(this, userGroupsEnabled, timeZoneGroupsEnabled, users, userGroups, timeZoneGroups);

        this.loggedInUserId = loggedInUserId;
        
        var showDisabledSettingString = $.cookie( this.showDisabledSettingCookieName );
        var showDisabled = showDisabledSettingString == "showDisabled";                     /*false, if setting was not found*/

        /*todo : radzivillovich : may be check hidden user's selection later after building tree and use checkAndProcessHiddenNodeSelection?*/
        var isSelectedUserActive = this.usersMap[selectedUserId] && this.usersMap[selectedUserId].isActive();
        if (!showDisabled && !isSelectedUserActive)
            showDisabled = true;

        this.loadState({showInactive: showDisabled, selection: this.createUserSelection(selectedUserId)});
    },

    createGroupsUsersMaps: function()
    {
        this.usersMap = {};
        for (var i = 0; i < this.users.length; i++)
        {
            var user = this.users[i];
            this.usersMap[user.id] = user;
        }
    },

    setSettingsBeforeTreeRebuilding: function ( settings )
    {
        this.loadState({showInactive: settings.showInactiveCheckboxValue, selection: this.createUserSelection(settings.selectedUserId)}, true);
        this.selectorFilter.setShowInactive(settings.showInactiveCheckboxValue);
        if (settings.searchString)
            this.selectorFilter.setSearchString(settings.searchString);
    },

    getFilterWordsPopupCoordinates: function(){
        return {
            y: -3,
            x: -90
        }
    },

    createUserSelection: function( selectedUserId ) {
        return (!selectedUserId || selectedUserId == null) ? TwoLevelSelection.emptySelection() : new TwoLevelSelection([],[selectedUserId]);
    },

    getSelection: function(isFullFormed) {
        return this.userSelectorGroupsStrategy.makeSelectionFromNodeSet(this.getSelectedNodeSet(), isFullFormed, false);
    },

    getOnlyUsersSelection: function() {
        return this.userSelectorGroupsStrategy.makeOnlyUsersSelection(this.getSelectedNodeSet());
    },

    createSelectionFilters: function()
    {
        AT.UserForLeaveHistorySelectorPanel.superclass.createSelectionFilters.call(this);

        var context = this;
        this.showInactiveCheckbox.on("check", function(e, checked) {
            context.treeActionHandler(function() {
                context.selectorFilter.setShowInactiveAndReFilter(checked);
                $.cookie(context.showDisabledSettingCookieName, checked ? "showDisabled" : "hideDisabled", { expires: 365 });
            }, false, false);
        });
    },

    close: function() {
        this.fireEvent("close");
    },

    makeSelectionHelper: function() {
        this.selectionHelper = new AT.SimpleUsersAndGroupsSelectionHelper({
            viewerId: this.loggedInUserId,
            itemNounSingularKey: this.labelItemNounSingularKey,
            itemNounPluralKey: this.labelItemNounPluralKey,
            showInactiveCount: true,
            showSingleItemInactiveText: false,
            inactiveMultipleTextKey: this.labelDisabledMultipleKey,
            userSelectorGroupsStrategy: this.userSelectorGroupsStrategy,
            maxCharacters: this.labelMaxChars,
            pleaseSelectMessageKey: 'users_selector.user_not_selected'
        });
    },

    createEmptyMessageBuilder: function() {
        return new LeaveTimeHistoryUsersSelectorEmptyMessageBuilder(this.userSelectorGroupsStrategy);
    }

});