/**
 * Created by filatov on 29.12.2015.
 */

function BalanceTypesTable(){}

BalanceTypesTable.prototype.init = function(tableId, config)
{
    var table = $( "#" + tableId );
    table.empty();
    this.content = $(document.createElement( "tbody" ));
    table.append( this.content );

    table.hide();

    this.daysFormatter = new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 1, true, '', 'd', 'N/A');
    this.hoursFormatter = new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 2, true, '', 'h', 'N/A');

    this.balancesInfoOnFromDate = config.balancesInfoOnFromDate;
    this.balancesInfoOnToDate = config.balancesInfoOnToDate;
    this.spentBalanceInfo = config.spentBalanceInfo;
    this.dateFrom = config.dateFrom;
    this.dateTo = config.dateTo;
    this.countWorkHoursPerDay = config.countWorkHoursPerDay;
    this._updateUI();
    table.show();
};

BalanceTypesTable.prototype._updateUI = function()
{
    if( this.balancesInfoOnFromDate == null ) return;

    var isPTO = false;
    var isSick = false;
    if( this.balancesInfoOnFromDate.PTOBalance != null )
        isPTO = true;
    if( this.balancesInfoOnFromDate.sickBalance != null )
        isSick = true;
    this.content.append(this._createMainRow(isSick, isPTO));
    this.content.append(this._createRowForDate(isPTO, isSick, this.dateFrom, this.balancesInfoOnFromDate ));
    this.content.append(this._createRowForDate(isPTO, isSick, this.dateTo, this.balancesInfoOnToDate ));
    this.content.append(this._createSpentRow(isPTO, isSick, this.dateTo, this.spentBalanceInfo ));
    this.content.show();
};

BalanceTypesTable.prototype._createMainRow = function(isSick, isPTO)
{
    var newRow = document.createElement( "tr" );
    newRow.className = "mainRow";
    var balanceCell = this.createColumn();
    balanceCell.innerHTML = messageResource.getMessage("actiplans.leaves_management.balance");
    balanceCell.className = "balanceCell";
    newRow.appendChild( balanceCell );
    if(isPTO ^ isSick) {
        balanceCell.colSpan = 2;
        balanceCell.className += " onlyOneColumn";
    }
    if( isPTO )
    {
        var ptoCell = this.createColumn();
        ptoCell.innerHTML = messageResource.getMessage("actiplans.leaves_management.balance.pto");
        newRow.appendChild( ptoCell );
    }
    if( isSick )
    {
        var sickCell = this.createColumn();
        sickCell.innerHTML =  messageResource.getMessage("actiplans.leaves_management.balance.sick");
        newRow.appendChild( sickCell );
    }
    return newRow;
};


BalanceTypesTable.prototype._createRowForDate = function(isPTO, isSick, date, balanceInfoForDate){

        var dateRow = this.createRow();
        var dateCell = this.createColumn();

        dateCell.innerHTML = messageResource.getMessage("actiplans.leaves_management.balance.on")+ " " + 
                             Date.monthNamesShort[date.getMonth()] + " " + date.getDate() + ", " + date.getFullYear();
    
        dateCell.className = "dataInfo";
        dateRow.appendChild(dateCell);
        if(isPTO ^ isSick) {
            dateCell.colSpan = 2;
            dateCell.className += " onlyOneColumn";
        }

        if(isPTO){
            var ptoCell = this.createBalanceCell(balanceInfoForDate.PTOBalance, "PTO");
            dateRow.appendChild( ptoCell );
        }
        if(isSick){
            var sickCell = this.createBalanceCell(balanceInfoForDate.sickBalance, "Sick");
            dateRow.appendChild( sickCell );
        }
        return dateRow;
};

BalanceTypesTable.prototype._createSpentRow = function(isPTO, isSick, date,spentBalanceInfo){

    var dateRow = this.createRow();
    var dateCell = this.createColumn();

    dateCell.innerHTML = messageResource.getMessage("actiplans.leaves_management.balance.spent");
    dateCell.className = "dataInfo";

    dateRow.appendChild(dateCell);
    dateRow.className ="spentRow";
    if(isPTO ^ isSick) {
        dateCell.colSpan = 2;
        dateCell.className += " onlyOneColumn";
    }

    if(isPTO){
        var ptoCell = this.createBalanceCell(spentBalanceInfo.PTOBalance , "PTO");
        dateRow.appendChild( ptoCell );
    }
    if(isSick){
        var sickCell = this.createBalanceCell(spentBalanceInfo.sickBalance, "Sick");
        dateRow.appendChild( sickCell );
    }
    return dateRow;
};

BalanceTypesTable.prototype.createBalanceCell = function(balanceValue, namePrefix ) {
    var balanceCell = this.createColumn();
    var  totalPtoStrings = this.formatBalanceValue(balanceValue, SystemSettings["show" + namePrefix + "InHours"], this.countWorkHoursPerDay );
    var totalTimeStr = totalPtoStrings.mainValue + " ";
    balanceCell.className ="balanceValue";
    if(balanceValue < 0){
        balanceCell.className +=" negative";
    }
    balanceCell.innerHTML =totalTimeStr + "<span class='balanceInHours'>(" + totalPtoStrings.convertedValue + ")</span>";
    return balanceCell;
};


BalanceTypesTable.prototype.formatBalanceValue = function( balanceValue, showInHours, countWorkHoursPerDay ) {

    var balanceInHours = this.hoursFormatter.format(balanceValue * countWorkHoursPerDay);
    var balanceInDays = this.daysFormatter.format(balanceValue);

    var balanceString = {};
    balanceString.mainValue = showInHours ? balanceInHours : balanceInDays;
    balanceString.convertedValue = !showInHours ? balanceInHours : balanceInDays;

    return balanceString;
};


BalanceTypesTable.prototype.createColumn = function(){
    return document.createElement("td");
};

BalanceTypesTable.prototype.createRow = function(){
    return document.createElement("tr");
};