function LeaveTimeRequestDialog(request, isPTOUser, isSickUser, userInfo, successor, recalcManagerFactory, ptoHoursPerDay, currentBalanceInfo)
{
    this.request = request;
    this.isPTOUser = isPTOUser;
    this.isSickUser = isSickUser;
    this.userInfo = userInfo;
    this.successor = successor;
    this.recalcManagerFactory = recalcManagerFactory;
    this.ptoHoursPerDay = ptoHoursPerDay;
    this.currentBalanceInfo = currentBalanceInfo;
}

LeaveTimeRequestDialog.prototype = {
    init: function()
    {
        this.requestChangeOriginal = _.clone(this.request.requestChange);
        if (this.request.id > 0) this.request.leaveTypeIdOriginal = this.request.leaveTypeId;
        this.leaveTypeId = this.requestOrRequestChange().leaveTypeId;

        this.requestOrRequestChange().balanceType = LEAVE_TYPES[this.leaveTypeId].balanceType;

        this.dialogDivId = "#leaveTimeRequestDialog";
        var dialogDiv = $(this.dialogDivId);

        this.dialog = dialogDiv.dialog({
                                           autoOpen: false,
                                           draggable: false,
                                           resizable: false,
                                           modal: true, // this is needed for overlay to appear, so we can handle 'outside' clicks to close the dialog
                                           close: this.successor.closed,
                                           width: 368,
                                           dialogClass: "leaveTimeRequestDialog"
                                       });

        if (LeaveTimeRequestDialog.leaveTypeSelector)
            $("#" + this.getElementId("leaveTypeSelector")).children().remove();
        LeaveTimeRequestDialog.leaveTypeSelector = new AT.LeaveTypesSelector({
            renderTo: this.getElementId("leaveTypeSelector")
        });

        this.workdayDurations = [];

        this.titleDiv = this.getElementBySubId("titleDiv");
        this.formatter = new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 1, true);
        this.ptoFormatter = new PTOFormatter().init(this.ptoHoursPerDay);
        this.sickFormatter = new SickFormatter().init(this.ptoHoursPerDay);

        this.commentEl = this.getElementBySubId("userComment");

        this._textareaManager = new TextAreaLengthManager();
        this._textareaManager.init(
                this.commentEl,
                2000,
                messageResource.getMessage("actiplans.leave_time_requests.long_comment")
        );

        var requestBeforeHireDateMsgKey = (this.request.past) ? "actiplans.leavestable.dialog.before_hire_date.past" : "actiplans.leavestable.dialog.before_hire_date";

       if (this.userInfo && (this.request.firstDate < this.userInfo.hireDate)){
            this.getElementBySubId("requestBeforeHireDate").innerHTML = messageResource.getMessage(requestBeforeHireDateMsgKey, [Date.parseJSONDate(this.userInfo.hireDate).format("D, M d")]);
       }

        var requestNotAffectBalanceMsgKey;
        var requestNotAffectCurrentBalanceMsgKey;
        if (this.isPTOUser && this.isSickUser)
        {
            requestNotAffectBalanceMsgKey = "actiplans.leavestable.edit_leave_popup.request_not_affect_balances";
            requestNotAffectCurrentBalanceMsgKey = "actiplans.leavestable.edit_leave_popup.request_not_affect_current_balances";
        }
        else if (this.isPTOUser)
        {
            requestNotAffectBalanceMsgKey = "actiplans.leavestable.edit_leave_popup.request_not_affect_pto";
            requestNotAffectCurrentBalanceMsgKey = "actiplans.leavestable.edit_leave_popup.request_not_affect_current_pto";
        }
        else
        {
            requestNotAffectBalanceMsgKey = "actiplans.leavestable.edit_leave_popup.request_not_affect_sick";
            requestNotAffectCurrentBalanceMsgKey = "actiplans.leavestable.edit_leave_popup.request_not_affect_current_sick";
        }

        if (this.request.past)
        {
            requestNotAffectBalanceMsgKey += ".past";
        }

        this.getElementBySubId("requestNotAffectBalance").innerHTML = messageResource.getMessage(requestNotAffectBalanceMsgKey);
        this.getElementBySubId("requestNotAffectCurrentBalance").innerHTML = messageResource.getMessage(requestNotAffectCurrentBalanceMsgKey);

        doOnceOnWindowResize(_.bind(this.close, this));

        return this;
    },

    close: function()
    {
        this.abortAllOperations();
        this.dialog.dialog("close");
    },

    closeWithoutAborting: function()
    {
        this.dialog.dialog("close");
    },

    save: function()
    {
        this.successor.saveRequest(LeaveTimeRequestUtils.mergeChangesToRequest(this.request));
    },

    cancel: function()
    {
        if (this.requestChangeOriginal)
            _.extend(this.request.requestChange, this.requestChangeOriginal);
        this.request.recalculated = false;

        this.successor.cancelled();
    },

    requestOrRequestChange: function()
    {
        return this.request.requestChange || this.request
    },

    isTitleVisible: function()
    {
        return this.userInfo ? true : false;
    },

    abortAllOperations: function()
    {
        RpcControlManager.cancelAllAJAXRequests();
        clearTimeout(this.onPasteRecalculateTimeOut);
        clearTimeout(this.recalculateTimeoutId);
    },

    configureTitleVisibility: function()
    {
        var visible = this.isTitleVisible();
        this.titleDiv.style.display = visible ? "block" : "none";
        return visible;
    },

    getTitlePrefix: function()
    {
        return "";
    },

    configureTitle: function()
    {
        this.getElementBySubId("titlePrefix").innerHTML = this.getTitlePrefix();
        this.getElementBySubId("userName").innerHTML = this.getUserTitle();
    },

    getUserTitle: function(){
        return "";
    },

    getApprovalCommentDiv: function()
    {
        return this.getElementBySubId("approvalComment");
    },

    configureCommentVisibility: function()
    {
        this.getApprovalCommentDiv().style.display = "none";
        return false;
    },

    getLeaveTypeSelectorTitle: function()
    {
        return this.request.past ? messageResource.getMessage("actiplans.leavestable.dialog.past_leave_request_selector_title")
                : messageResource.getMessage("actiplans.leavestable.dialog.leave_time_request_selector_title");
    },

    configureLeaveTypeSelector: function()
    {
        var self = this;

        if (!LEAVE_TYPES[this.leaveTypeId].active)
            LeaveTimeRequestDialog.leaveTypeSelector.addLeaveType(LEAVE_TYPES[this.leaveTypeId]);

        LeaveTimeRequestDialog.leaveTypeSelector.selectLeaveType(this.leaveTypeId);
        LeaveTimeRequestDialog.leaveTypeSelector.setSelectionCallback(function(leaveTypeId) { self.leaveTypeSelected(leaveTypeId); });
        LeaveTimeRequestDialog.leaveTypeSelector.setEnabled(true);
    },

    leaveTypeSelected: function(leaveTypeId)
    {
        if (leaveTypeId != this.leaveTypeId)
        {
            this.leaveTypeId = leaveTypeId;
            this.requestOrRequestChange().leaveTypeId = leaveTypeId;
            this.recalculateRequestData();
        }
    },

    isPTODataVisible: function()
    {
        return (!!this.request.balanceData || this.request.changedTodayBalance) && this.request.balanceType == BalanceType.PTO;
    },

    isSickDataVisible: function()
    {
        return (!!this.request.balanceData || this.request.changedTodayBalance) && this.request.balanceType == BalanceType.SICK;
    },

    isRequestDataIsBeingRecalculated: function()
    {
        return this.requestDataIsBeingRecalculated;
    },

    isAutoApproved: function()
    {
        return this.userInfo && !this.userInfo.requiresLTRApproval;
    },

    doesRequestNotAffectPTO: function()
    {
        return !this.isPTODataVisible();
    },

    doesRequestNotAffectSick: function()
    {
        return !this.isSickDataVisible();
    },

    requestDataRecalculated: function(result)
    {
        this.request.recalculated = true;

        var requestToModify = this.requestOrRequestChange();
        requestToModify.leaveTimeTotal = result.leaveTimeTotal;
        requestToModify.balanceData = result.balanceData;
        requestToModify.balanceType = result.balanceType;

        if (result.changedTodayBalance && result.changedTodayBalance.PTOBalance == null && result.changedTodayBalance.sickBalance == null)
            result.changedTodayBalance = null;

        requestToModify.changedTodayBalance = result.changedTodayBalance;

        this.setRequestCalculatedData();
        this.requestDataIsBeingRecalculated = false;
        this.dialog.dialog("option", "dialogClass", this.getDialogClass());
    },

    recalculateRequestDataAsync: function()
    {
        var self = this;
        this.recalculateTimeoutId = setTimeout(function() { self.recalculateRequestData() }, 400);
    },

    recalculateRequestData: function()
    {
        this.abortAllOperations();

        var self = this;
        var timer = setTimeout(function()
                               {
                                   self.requestDataIsBeingRecalculated = true;
                                   self.dialog.dialog("option", "dialogClass", self.getDialogClass());
                               }, 1500);
        var manager = this.recalcManagerFactory(LeaveTimeRequestUtils.mergeChangesToRequest(this.request), function(result)
        {
            if (timer)
            {
                clearTimeout(timer);
                timer = 0;
            }
            self.requestDataRecalculated(result);
        });
        RpcControlManager.saveRpcControlBoundData(manager);
    },

    getMainDialogClasses: function()
    {
        return ["leaveTimeRequestDialog"];
    },

    getDialogClass: function()
    {
        var classes = [];

        var mainDialogClasses = this.getMainDialogClasses();
        for (var i = 0; i < mainDialogClasses.length; i++)
            classes.push(mainDialogClasses[i]);

        if (!this.isTitleVisible())
            classes.push("noTitle");

        if (this.request.past)
            classes.push("past");

        if (this.request.length == 1)
            classes.push("oneDayRequest");
        else if (this.request.length == 2)
            classes.push("twoDaysRequest");
        else if (this.request.length == 3)
            classes.push("threeDaysRequest");

        var startBeforeHireDate = this.userInfo && (this.request.firstDate < this.userInfo.hireDate);
        var endBeforeHireDate = this.userInfo && (this.request.lastDate < this.userInfo.hireDate);
        if (startBeforeHireDate)
            classes.push("beforeHireDate");

        if (this.isRequestDataIsBeingRecalculated())
            classes.push("balanceDataBeingRequested");

        var changedTodayBalance = LeaveTimeRequestUtils.changedTodayBalanceTuple(this.requestOrRequestChange());
        var changedTodayPTO = changedTodayBalance && _.isNumber(changedTodayBalance.PTOBalance);
        var changedTodaySick = changedTodayBalance && _.isNumber(changedTodayBalance.sickBalance);

        switch (LeaveTimeRequestUtils.viewBalanceAsType(this.request, this.isPTOUser, this.isSickUser, endBeforeHireDate))
        {
            case LeaveTimeRequestUtils.ViewBalanceAs.DO_NOT_VIEW:
                classes.push("doNotViewBalance");
                break;
            case LeaveTimeRequestUtils.ViewBalanceAs.BALANCE_ON_START_END:
                classes.push("viewBalanceOnStartEnd");
                break;
            case LeaveTimeRequestUtils.ViewBalanceAs.CURRENT_BALANCE_BEFORE_AFTER_APPROVAL:
                classes.push("viewCurrentBalanceBeforeAfter");
                break;
            case LeaveTimeRequestUtils.ViewBalanceAs.DOES_NOT_AFFECT_CURRENT_BALANCE:
                classes.push("doesNotAffectCurrentBalance");
                break;
            case LeaveTimeRequestUtils.ViewBalanceAs.DOES_NOT_AFFECT_BALANCE:
                classes.push("doesNotAffectBalance");
                break;
        }

        var leaveType = LEAVE_TYPES[this.leaveTypeId];
        if (LeaveTimeRequestUtils.leaveTypeAffectsBalance(leaveType, this.isPTOUser, this.isSickUser, endBeforeHireDate))
            classes.push("leaveTypeAffectsBalance");
        if (this.isPTOUser || this.isSickUser)
            classes.push("balanceTypeEnabled");

        if(this.isPTOUser || this.isSickUser)
        {
            if (!this.requestOrRequestChange().balanceData && !changedTodayPTO && !changedTodaySick )
                classes.push("requestNotAffectBalance");

            if (LeaveTimeRequestUtils.requestRequiresApprovalType(this.request, this.isAutoApproved()))
                classes.push("showModificationNote");
            if (changedTodayPTO)
                classes.push("changedTodayPTO");
            if (changedTodaySick)
                classes.push("changedTodaySick");
        }


        return classes.join(" ");
    },

    getElementId: function(subId)
    {
        return "leaveTimeRequestDialog_" + subId;
    },

    getElementBySubId: function(subId)
    {
        return document.getElementById(this.getElementId(subId));
    },

    setRequestCalculatedData: function()
    {
        var requestToShow = this.requestOrRequestChange();

        if (requestToShow.leaveTimeTotal)
        {
            this.getElementBySubId("leaveTimeTotalSubtitle").innerHTML = messageResource.getMessage(this._makeSubtitleMsgKeyForRequest("leave_time"));
            this.getElementBySubId("leaveTimeTotalDaysDiv").innerHTML = this.formatDays(requestToShow.leaveTimeTotal.days);
            this.getElementBySubId("leaveTimeTotalHoursDiv").innerHTML = "(" + this.formatHours(requestToShow.leaveTimeTotal.hours) + ")";
        }

        this._setBalanceData(requestToShow);
    },

    _setBalanceData: function(requestToShow)
    {
        var balanceData = requestToShow.balanceData;
        var balanceType = requestToShow.balanceType;
        var balanceFormatter = balanceType == BalanceType.PTO ? this.ptoFormatter : this.sickFormatter;

        var startBalanceMsgKey;
        var endBalanceMsgKey;
        var startBalance;
        var endBalance;
        var leaveType = LEAVE_TYPES[this.leaveTypeId];

        var requestChangeTodayBalance = LeaveTimeRequestUtils.changedTodayBalanceTuple(requestToShow);
        if (this.currentBalanceInfo && (_.isNumber(requestChangeTodayBalance.PTOBalance) || _.isNumber(requestChangeTodayBalance.sickBalance)))
        {
            if (_.isNumber(requestChangeTodayBalance.PTOBalance))
            {
                this.getElementBySubId("currentPTOBalance").innerHTML = messageResource.getMessage("actiplans.leavestable.tooltip.current_pto");
                this.getElementBySubId("currentPTOBalanceDiv").className = this._getBalanceValueClasses(this.currentBalanceInfo.PTOBalance, this.ptoFormatter);
                this.getElementBySubId("currentPTOBalanceDiv").innerHTML = this.ptoFormatter.format(this.currentBalanceInfo.PTOBalance);
                this.getElementBySubId("currentPTOBalanceAfter").innerHTML = messageResource.getMessage("actiplans.leavestable.tooltip.current_pto_after");
                this.getElementBySubId("currentPTOBalanceAfterDiv").className = this._getBalanceValueClasses(requestChangeTodayBalance.PTOBalance, this.ptoFormatter);
                this.getElementBySubId("currentPTOBalanceAfterDiv").innerHTML = this.ptoFormatter.format(requestChangeTodayBalance.PTOBalance);
            }

            if (_.isNumber(requestChangeTodayBalance.sickBalance))
            {
                this.getElementBySubId("currentSickBalance").innerHTML = messageResource.getMessage("actiplans.leavestable.tooltip.current_sick");
                this.getElementBySubId("currentSickBalanceDiv").className = this._getBalanceValueClasses(this.currentBalanceInfo.sickBalance, this.sickFormatter);
                this.getElementBySubId("currentSickBalanceDiv").innerHTML = this.sickFormatter.format(this.currentBalanceInfo.sickBalance);
                this.getElementBySubId("currentSickBalanceAfter").innerHTML = messageResource.getMessage("actiplans.leavestable.tooltip.current_sick_after");
                this.getElementBySubId("currentSickBalanceAfterDiv").className = this._getBalanceValueClasses(requestChangeTodayBalance.sickBalance, this.sickFormatter);
                this.getElementBySubId("currentSickBalanceAfterDiv").innerHTML = this.sickFormatter.format(requestChangeTodayBalance.sickBalance);
            }
        }
        else if (balanceData && balanceData.balanceChangeData)
        {
            if (balanceType == BalanceType.PTO)
            {
                startBalanceMsgKey = "leave_time_request_dialog.pto_balance_as_of";
                endBalanceMsgKey = "leave_time_request_dialog.pto_balance_as_of";
            }
            else
            {
                startBalanceMsgKey = "leave_time_request_dialog.sick_balance_as_of";
                endBalanceMsgKey = "leave_time_request_dialog.sick_balance_as_of";
            }
            startBalance = balanceData.balanceChangeData.startBalance;
            endBalance = balanceData.balanceChangeData.endBalance;
        }

        if (!_.isUndefined(startBalance) && !_.isUndefined(endBalance))
        {
            this.getElementBySubId("firstDayBalanceDiv").className = this._getBalanceValueClasses(startBalance, balanceFormatter);
            this.getElementBySubId("lastDayBalanceDiv").className = this._getBalanceValueClasses(endBalance, balanceFormatter);

            this.getElementBySubId("firstDayBalanceDiv").innerHTML = startBalance != null ? this.formatBalance(startBalance, balanceFormatter) : "N/A";
            this.getElementBySubId("lastDayBalanceDiv").innerHTML = endBalance != null ? this.formatBalance(endBalance, balanceFormatter) : "N/A";

            var firstDate = this.request.firstDate.parseJSONDate();
            var lastDate = this.request.lastDate.parseJSONDate();

            this.getElementBySubId("firstDayBalance").innerHTML = messageResource.getMessage(startBalanceMsgKey, [this.formatBalanceDate(firstDate)]);
            this.getElementBySubId("lastDayBalance").innerHTML = messageResource.getMessage(endBalanceMsgKey, [this.formatBalanceDate(lastDate)]);
        }


        var total = balanceData && balanceData.requestedPaidTime;
        if (!_.isNumber(total) && requestToShow.leaveTimeTotal)
            total = LeaveTimeRequestUtils.getRequestedBalanceFromLeaveTime(requestToShow.leaveTimeTotal, leaveType);
        if (_.isNumber(total))
            this.getElementBySubId("requestedDaysDiv").innerHTML = this.formatBalance(total, balanceFormatter);

        var requestedBalanceMsgKey = this._makeSubtitleMsgKeyForRequest(balanceType == BalanceType.PTO ? "pto" : "sick");
        if (requestedBalanceMsgKey)
            this.getElementBySubId("requestedBalanceTitle").innerHTML = messageResource.getMessage(requestedBalanceMsgKey);


        var modificationsNoteKey;
        switch (LeaveTimeRequestUtils.requestRequiresApprovalType(this.request, this.isAutoApproved()))
        {
            case LeaveTimeRequestUtils.RequiresApproval.REQUIRES_BECAUSE_OF_LEAVE_TYPE:
                modificationsNoteKey = this.request.id > 0 ? 'actiplans.leavestable.tooltip.modifications_note' : 'actiplans.leavestable.tooltip.creation_note';
                break;
            case LeaveTimeRequestUtils.RequiresApproval.REQUIRES_BECAUSE_CURRENT_BALANCE_CHANGES:
                modificationsNoteKey = this.request.id > 0 ? 'actiplans.leavestable.tooltip.modifications_note_affects_balance' : 'actiplans.leavestable.tooltip.creation_note_affects_balance';
                break;
        }

        this.getElementBySubId("balanceModificationsNote").innerHTML = modificationsNoteKey ? messageResource.getMessage(modificationsNoteKey) : "";

    },

    _getBalanceValueClasses: function(value, balanceFormatter)
    {
        return ( value && this.isNegativeBalance(value, balanceFormatter) ) ? "balanceValue negative" : "balanceValue";
    },

    workdayDurationChanged: function(durationName)
    {
        var input = this.getElementBySubId(durationName);
        var val = trim(input.value);
        var newValue = null;
        if (val.length == 0)
        {
            newValue = -1;
        }
        else
        {
            // val is in hours but parsedValue is in minutes.
            var parsedValue = UserTime.parseTimeWithSystemSettings(val);
            if (parsedValue != null && parsedValue > 0 && (parsedValue / 60) < 1000000)
                newValue = parsedValue;
        }

        if (newValue == null)
        {
            input.error = true;
        }
        else
        {
            if (this.requestOrRequestChange()[durationName] != newValue)
            {
                this.requestOrRequestChange()[durationName] = newValue;
                this.recalculateRequestDataAsync();
            }
            input.error = undefined;
        }
    },

    formatWorkdayDuration: function(wd)
    {
        return wd < 0 ? "" : UserTime.formatTime(wd);
    },

    workdayFieldBlur: function(durationName)
    {
        var input = this.getElementBySubId(durationName);
        var durationCell = this.getElementBySubId(durationName + "Cell");

        if (input.error)
        {
            this.showError($(durationCell));
        }
        else
        {
            input.value = this.formatWorkdayDuration(this.requestOrRequestChange()[durationName]);
            this.hideError($(durationCell));
        }
    },

    showError: function(jElement)
    {
        jElement.addClass("error");
        jElement.attr("title", messageResource.getMessage("leave_time_request_dialog.js.invalid_duration", [SystemSettings.decimalSeparator]));
    },

    hideError: function(jElement)
    {
        jElement.removeAttr("title");
        jElement.removeClass("error");
    },

    hasError: function(durationName)
    {
        var input = this.getElementBySubId(durationName);
        return !!input.error;
    },

    hasCommentError: function()
    {
        return this._textareaManager.isIncorrectLenght();
    },

    hasErrors: function()
    {
        return !(!this.hasError("firstDayDuration") && !this.hasError("middleDayDuration") && !this.hasError("lastDayDuration") && !this.hasCommentError());
    },

    blinkErrors: function()
    {
        var errorEls = $(this.dialogDivId + " .error");
        errorEls.removeClass("error");
        window.setTimeout(function() { errorEls.addClass("error"); }, 250);
    },

    initField: function(workdayDurationName)
    {
        var durationInput = this.getElementBySubId(workdayDurationName);
        durationInput.className = "";
        durationInput.value = this.formatWorkdayDuration(this.requestOrRequestChange()[workdayDurationName]);
        durationInput.disabled = false;
        durationInput.error = undefined;

        var self = this;

        durationInput.onkeyup = function()
        {
            self.abortAllOperations();
            self.workdayDurationChanged(workdayDurationName)
        };

        durationInput.onpaste = function()
        {
            var fn = function() { self.workdayDurationChanged(workdayDurationName); };
            self.abortAllOperations();
            this.onPasteRecalculateTimeOut = setTimeout(fn, 100);
        };

        durationInput.onblur = function() { self.workdayFieldBlur(workdayDurationName); };

        $(durationInput).placeholder();
    },

    initCommentField: function()
    {
        this.commentEl.placeholder = messageResource.getMessage("actiplans.leavestable.leave_time_request.enter_comment");
        $(this.commentEl).placeholder();
        if (this.requestOrRequestChange().comment)
            $(this.commentEl).pVal(this.requestOrRequestChange().comment);
        else
            $(this.commentEl).pVal("");
        this.commentEl.disabled = false;
    },

    showAt: function(element, offset)
    {
        this.getElementBySubId("leaveTypeSelectorTitle").innerHTML = this.getLeaveTypeSelectorTitle();
        this.configureLeaveTypeSelector();

        var firstDate = this.request.firstDate.parseJSONDate();
        var lastDate = this.request.lastDate.parseJSONDate();

        this.getElementBySubId("firstDayCellTitle").innerHTML = this.formatWDDate(firstDate);
        if (this.request.length > 1)
            this.getElementBySubId("lastDayCellTitle").innerHTML = this.formatWDDate(lastDate);
        if (this.request.length == 3)
        {
            this.getElementBySubId("middleDayCellTitle").innerHTML = this.formatWDDate(firstDate.nextDay());
        }
        else if (this.request.length > 3)
        {
            var from = firstDate.nextDay();
            var to = lastDate.prevDay();
            this.getElementBySubId("middleDayCellTitle").innerHTML = this.formatWDDate(from) + " - " + this.formatWDDate(to);
        }

        this.hideError($(this.dialogDivId + " .error"));

        this.initField("firstDayDuration");
        this.initField("middleDayDuration");
        this.initField("lastDayDuration");

        this.initCommentField();

        this.requestDataIsBeingRecalculated = !!(this.request.requestChange || !this.request.leaveTimeTotal);
        if (this.requestDataIsBeingRecalculated)
            this.recalculateRequestData();
        else
            this.setRequestCalculatedData();

        if (this.isTitleVisible())
            this.configureTitle();

        this.dialog.dialog("option", "buttons", [{text: "", "class": "hiddenButton"}]);
        this.dialog.dialog("option", "dialogClass", this.getDialogClass());

        var buttonsPane = this.dialog.parent().find(".ui-dialog-buttonpane");
        buttonsPane[0].innerHTML = '';
        buttonsPane.prepend("<textarea name='approvalComment' id='leaveTimeRequestDialog_approvalComment'></textarea>");

        this.createButtons(buttonsPane);

        if (this.request.status == LeaveTimeRequestStatus.APPROVED &&
            this.request.approverInfo &&
            this.request.approverInfo.id != this.request.userId && !this.request.requestChange)
        {
            var escapedApproverName = escapeHtmlSymbols(this.request.approverInfo.firstName) + " " + escapeHtmlSymbols(this.request.approverInfo.lastName);
            buttonsPane.prepend(
                    "<table class='summary approvalInfo' cellspacing='0' cellpadding='0'><tr><th class='titleText'>" +
                    "<span class='approvedBy'>Approved by</span>" +
                    "<span class='approverName'>" + escapedApproverName + "</span>" +
                    "</th></tr></table>" +
                    (this.request.statusMessage ? "<div class='statusMessage'>" + escapeHtmlSymbols(truncateStringByLength(this.request.statusMessage, 100)) + "</div>" : "")
            );
        }

        this.dialog.dialog("open");

        this._textareaManager.update();

        // when the dialog is created, it is wrapped into a container, that's why we use parent()
        this.dialog.parent().position({
                                          of: $(element),
                                          my: "left top",
                                          at: "left bottom",
                                          offset: offset,
                                          collision: "flip"
                                      });

        // close the dialog when user clicks outside
        $('.ui-widget-overlay').click(_.bind(this.cancel, this));
    },

    getCommitButtonTitle: function()
    {
        return messageResource.getMessage("common.save");
    },

    createButtons: function(buttonsPane)
    {
        return [];
    },

    _createAndSetButtonElement: function(buttonsPane, text, className, withIcon)
    {

        var buttonEl = document.createElement("div");
        var additionalClass = className ? className : "";

        if (withIcon)
        {
            var spanEl = document.createElement("span");
            spanEl.innerHTML = text;
            spanEl.className = "buttonIcon";
            additionalClass += " withIcon";
            buttonEl.appendChild(spanEl);
        }
        else
        {
            buttonEl.innerHTML = text;
            additionalClass += " noIcon";
        }

        buttonEl.className = "greyButton " + additionalClass;

        buttonsPane[0].appendChild(buttonEl);

        return buttonEl;
    },

    createButton: function(buttonsPane, text, clickCallback, className, checkForErrors, withIcon)
    {
        var self = this;
        var buttonElement = this._createAndSetButtonElement(buttonsPane, text, className, withIcon);

        $(buttonElement).click(function()
                               {
                                   self._textareaManager.update();
                                   var comment  = trim($(self.commentEl).pVal());
                                   if (_.isObject(self.request.requestChange))
                                   {
                                       self.request.requestChange.comment = comment;
                                   }
                                   else
                                   {
                                       self.request.comment = comment;
                                   }
                                   if (checkForErrors && self.hasErrors())
                                   {
                                       self.blinkErrors();
                                       return;
                                   }
                                   if (clickCallback && clickCallback())
                                       self.dialog.dialog('close');
                               });
    },

    createCancelButton: function(text, buttonsPane, clickCallBack)
    {
        var buttonElement = this._createAndSetButtonElement(buttonsPane, text, "cancelButton ", false);
        $(buttonElement).click(clickCallBack);
    },

    formatHours: function(hours)
    {
        return this.formatter.format(hours) + "h";
    },

    formatDays: function(days)
    {
        return this.formatter.format(days) + "d";
    },

    formatBalance: function(value, balanceFormatter)
    {
        return balanceFormatter.format(value);
    },

    isNegativeBalance: function(value, balanceFormatter)
    {
        return balanceFormatter.isNegativePTO(value);
    },

    formatWDDate: function(date)
    {
        return date.format("D, M d");
    },

    formatBalanceDate: function(date)
    {
        return date.format("M d");
    },

    formatUserName: function()
    {
        return escapeHtmlSymbols(UserNameFormat.formatFML(this.userInfo.firstName, this.userInfo.lastName, this.userInfo.middleInitial));
    },

    setReadOnly: function(readOnly)
    {
    },

    createMessage: function()
    {
        var key = this.request.past ? "actiplans.leavestable.dialog.create_past_request" : "actiplans.leavestable.dialog.submit";
        return messageResource.getMessage(key);
    },

    _makeSubtitleMsgKeyForRequest: function(leaveTypeStr)
    {
        return "leave_time_request_dialog." + ((this.request.past) ? "spent_" : "requested_") + leaveTypeStr;
    }
};

// ----------------------------------------------------------------------------------------

function BaseEditLeaveRequestDialog(request, isPTOUser, isSickUser, userInfo, successor, recalcManagerFactory, ptoHoursPerDay, currentBalanceInfo, openedByManager)
{
    LeaveTimeRequestDialog.call(this, request, isPTOUser, isSickUser, userInfo, successor, recalcManagerFactory, ptoHoursPerDay, currentBalanceInfo);
    this.openedByManager = openedByManager;
}

BaseEditLeaveRequestDialog.prototype = _.create(LeaveTimeRequestDialog.prototype, {
    isAutoApproved: function()
    {
        return LeaveTimeRequestDialog.prototype.isAutoApproved.call(this) ||
               this.openedByManager && this.request.status == LeaveTimeRequestStatus.APPROVED && !this.request.requestChange;
    }
});


// ----------------------------------------------------------------------------------------

function CreateLeaveRequestDialog(request, isPTOUser, isSickUser, userInfo, successor, recalcManagerFactory, ptoHoursPerDay, currentBalanceInfo, openedByManager)
{
    BaseEditLeaveRequestDialog.call(this, request, isPTOUser, isSickUser, userInfo, successor, recalcManagerFactory, ptoHoursPerDay, currentBalanceInfo, openedByManager);
}

CreateLeaveRequestDialog.prototype = _.create(BaseEditLeaveRequestDialog.prototype, {
    createButtons: function(buttonsPane)
    {
        var self = this;
        return [
            this.createButton(buttonsPane, this.createMessage(), function() { self.successor.createRequest(self.request); }, "createButton", true, false),
            this.createCancelButton(messageResource.getMessage("common.cancel"), buttonsPane, _.bind(this.cancel, this))
        ];
    },

    getTitlePrefix: function()
    {
        return this.request.past ? messageResource.getMessage("actiplans.leavestable.dialog.past_leave_title_prefix") :
               messageResource.getMessage("actiplans.leavestable.dialog.leave_time_request_title_prefix");
    },

    getUserTitle : function()
    {
        return  messageResource.getMessage("actiplans.leavestable.dialog.user", [this.formatUserName()]);
    }
});


// ----------------------------------------------------------------------------------------


function EditLeaveRequestDialog(request, isPTOUser, isSickUser, userInfo, successor, recalcManagerFactory, ptoHoursPerDay, currentBalanceInfo, openedByManager)
{
    BaseEditLeaveRequestDialog.call(this, request, isPTOUser, isSickUser, userInfo, successor, recalcManagerFactory, ptoHoursPerDay, currentBalanceInfo, openedByManager);
}

EditLeaveRequestDialog.prototype = _.create(BaseEditLeaveRequestDialog.prototype, {
    createButtons: function(buttonsPane)
    {
        var buttons = [];
        buttons.push(this.createButton(buttonsPane, messageResource.getMessage("actiplans.leavestable.dialog.save"), _.bind(this.save, this), "saveButton", true, false));
        buttons.push(this.createCancelButton(messageResource.getMessage("common.cancel"), buttonsPane, _.bind(this.cancel, this)));
        return buttons;
    },

    getTitlePrefix: function()
    {
        return this.request.past ? messageResource.getMessage("leave_time_request_dialog.js.past_leave") :
               messageResource.getMessage("leave_time_request_dialog.js.leave_request_by");
    },

    getUserTitle : function()
    {
        return  messageResource.getMessage("leave_time_request_dialog.js.user", [this.formatUserName()]);
    },

    getLeaveTypeSelectorTitle: function()
    {
        return messageResource.getMessage("actiplans.leavestable.dialog.leave_type_title");
    }
});

// ----------------------------------------------------------------------------------------

function EditLeaveRequestLeaveTypeDialog(request, isPTOUser, isSickUser, userInfo, successor, ptoHoursPerDay, currentBalanceInfo)
{
    LeaveTimeRequestDialog.call(this, request, isPTOUser, isSickUser, userInfo, successor, null, ptoHoursPerDay, currentBalanceInfo);
}

EditLeaveRequestLeaveTypeDialog.prototype = _.create(LeaveTimeRequestDialog.prototype, {
    createButtons: function(buttonsPane)
    {
        return [
            this.createButton(buttonsPane, messageResource.getMessage("actiplans.leavestable.dialog.save"), _.bind(this.save, this), "saveButton", true, false),
            this.createCancelButton(messageResource.getMessage("common.cancel"), buttonsPane, _.bind(this.cancel, this))
        ];
    },

    getMainDialogClasses: function()
    {
        return ["leaveTimeRequestDialog", "editLeaveTypeMode"];
    },

    recalculateRequestData: function()
    {
        //no need
    },

    recalculateRequestDataAsync: function()
    {
        //no need
    }
});

// ----------------------------------------------------------------------------------------

function EditLeaveRequestPeriodDialog(request, isPTOUser, isSickUser, userInfo, successor, ptoHoursPerDay, currentBalanceInfo)
{
    LeaveTimeRequestDialog.call(this, request, isPTOUser, isSickUser, userInfo, successor, null, ptoHoursPerDay, currentBalanceInfo);
}

EditLeaveRequestPeriodDialog.prototype = _.create(LeaveTimeRequestDialog.prototype, {
    createButtons: function(buttonsPane)
    {
        return [
            this.createButton(buttonsPane, messageResource.getMessage("actiplans.leavestable.dialog.save"), _.bind(this.save, this), "saveButton", true, false),
            this.createCancelButton(messageResource.getMessage("common.cancel"), buttonsPane, _.bind(this.cancel, this))
        ];
    },

    getMainDialogClasses: function()
    {
        return ["leaveTimeRequestDialog", "editPeriodMode"];
    },

    recalculateRequestData: function()
    {
        // no need
    },

    recalculateRequestDataAsync: function()
    {
        // no need
    },

    setRequestCalculatedData: function()
    {
        // no need
    }
});

// ----------------------------------------------------------------------------------------

function ReadOnlyRequestDialog(request, isPTOUser, isSickUser, userInfo, successor, ptoHoursPerDay, currentBalanceInfo)
{
    LeaveTimeRequestDialog.call(this, request, isPTOUser, isSickUser, userInfo, successor, undefined, ptoHoursPerDay, currentBalanceInfo);
}

ReadOnlyRequestDialog.prototype = _.create(LeaveTimeRequestDialog.prototype, {
    getMainDialogClasses: function()
    {
        return ["leaveTimeRequestDialog", "readOnlyMode"];
    },

    initField: function(workdayDurationName)
    {
        var durationInput = this.getElementBySubId(workdayDurationName);
        durationInput.value = this.formatWorkdayDuration(this.requestOrRequestChange()[workdayDurationName]);
        durationInput.disabled = true;

        if ($.browser.msie && $(durationInput).val().length === 0)
        {
            var placeholder = $(durationInput).attr("placeholder");
            $(durationInput).val(placeholder);
        }
    },

    initCommentField: function()
    {
        if (this.requestOrRequestChange().comment)
            this.commentEl.value = this.requestOrRequestChange().comment;
        else
            this.commentEl.value = "";
        this.commentEl.placeholder = "";
        this.commentEl.disabled = true;
    },

    configureLeaveTypeSelector: function()
    {
        LeaveTimeRequestDialog.leaveTypeSelector.selectLeaveType(this.leaveTypeId);
        LeaveTimeRequestDialog.leaveTypeSelector.setEnabled(false);
    }
});

// ----------------------------------------------------------------------------------------

function ViewLeaveRequestDialog(request, isPTOUser, isSickUser, successor, ptoHoursPerDay, currentBalanceInfo)
{
    ReadOnlyRequestDialog.call(this, request, isPTOUser, isSickUser, null, successor, ptoHoursPerDay, currentBalanceInfo);
}

ViewLeaveRequestDialog.prototype = _.create(ReadOnlyRequestDialog.prototype, {
    createButtons: function(buttonsPane)
    {
        return [
            this.createCancelButton(messageResource.getMessage("common.close"), buttonsPane, _.bind(this.cancel, this))
        ];
    }
});

// ----------------------------------------------------------------------------------------

function ViewLeaveRequestLeaveTypeDialog(request, isPTOUser, isSickUser, successor, ptoHoursPerDay, currentBalanceInfo)
{
    ReadOnlyRequestDialog.call(this, request, isPTOUser, isSickUser, null, successor, ptoHoursPerDay, currentBalanceInfo);
}

ViewLeaveRequestLeaveTypeDialog.prototype = _.create(ReadOnlyRequestDialog.prototype, {
    createButtons: function(buttonsPane)
    {
        return [this.createCancelButton(messageResource.getMessage("common.close"), buttonsPane, _.bind(this.cancel, this))];
    },

    getMainDialogClasses: function()
    {
        return ["leaveTimeRequestDialog", "readOnlyLeaveTypeMode"];
    }
});

// ----------------------------------------------------------------------------------------

function ViewLeaveRequestPeriodDialog(request, isPTOUser, isSickUser, successor, ptoHoursPerDay, currentBalanceInfo)
{
    ReadOnlyRequestDialog.call(this, request, isPTOUser, isSickUser, null, successor, ptoHoursPerDay, currentBalanceInfo);
}

ViewLeaveRequestPeriodDialog.prototype = _.create(ReadOnlyRequestDialog.prototype, {
    createButtons: function(buttonsPane)
    {
        return [this.createCancelButton(messageResource.getMessage("common.close"), buttonsPane, _.bind(this.cancel, this))];
    },

    getMainDialogClasses: function()
    {
        return ["leaveTimeRequestDialog", "readOnlyPeriodMode"];
    },

    setRequestCalculatedData: function()
    {
        // no need
    }
});
