Ext.namespace("AT");

AT.LeaveTypesMenu = function( selectionListener, customItems, showAllLeaveTypes )
{
    this.onLeaveTypeClick = function(item)
    {
        selectionListener.call( selectionListener, item.value );
    };

    var items = [];
    for( var i = 0; i < LEAVE_TYPES_ORDER.length; i++ )
    {
        var leaveType = LEAVE_TYPES[LEAVE_TYPES_ORDER[i]];
        if (showAllLeaveTypes || leaveType.active)
        {
            items.push({
                           text: escapeHtmlSymbols(leaveType.name),
                           value: leaveType.id,
                           icon: leaveType.getIconPath(),
                           listeners: { click: this.onLeaveTypeClick },
                           href: "javascript: void(0)"
                       });
        }
    }

    if( customItems )
    {
        items.push( new Ext.menu.Separator() );
        for( var j = 0; j < customItems.length; j++ )
            items.push( customItems[j] );
    }

    this.extMenu = new Ext.menu.Menu({
                                      items: items,
                                      cls: "leave-type-selector-menu at-dropdown-list-btn-menu"
                                  });
};

AT.LeaveTypesMenu.prototype.showAt = function( element )
{
    this.extMenu.show(element);
};

AT.LeaveTypesMenu.prototype.hide = function()
{
    this.extMenu.hide();
};

AT.LeaveTypesMenu.prototype.addLeaveType = function(leaveType)
{
    var item = {
        text: escapeHtmlSymbols(leaveType.name),
        value: leaveType.id,
        icon: leaveType.getIconPath(),
        listeners: { click: this.onLeaveTypeClick },
        href: "javascript: void(0)"
    };
    this.extMenu.add(item);
};

AT.LeaveTypesSelector = Ext.extend(AT.AbstractDropdownButton, {

    initComponent: function()
    {
        this.text = "test";
        this.cls = "leave-type-selector at-dropdown-list-btn";
        this.ctCls = "at-dropdown-list-btn-ct";

        var me = this;

        var listener = function(leaveTypeId)
        {
            me.selectLeaveType(leaveTypeId);
            if(me.selectionCallback)
                me.selectionCallback( leaveTypeId );
        };

        this.leaveTypesMenu = new AT.LeaveTypesMenu(listener);
        this.menu = this.leaveTypesMenu.extMenu;

        AT.AbstractDropdownButton.superclass.initComponent.call(this);
    },

    addLeaveType: function(leaveType)
    {
        this.leaveTypesMenu.addLeaveType(leaveType);
    },

    setEnabled: function( enabled )
    {
        this.menu = enabled? this.leaveTypesMenu.extMenu : null;
    },

    setIcon: function( url )
    {
        if( this.rendered )
        {
            this.getEl().child(".x-btn-center").setStyle('background-image', 'url(' +url+')');
        }
    },

    selectLeaveType: function( leaveTypeId )
    {
        this.value = leaveTypeId;
        this.setText(escapeHtmlSymbols(LEAVE_TYPES[leaveTypeId].name));
        this.setIcon(LEAVE_TYPES[leaveTypeId].getIconPath());
    },

    setSelectionCallback: function( selectionCallback )
    {
        this.selectionCallback = selectionCallback;
    }
});