function ProcessSelectedRequestsDialog(dialogEl, dialogClass)
{
    this.dialogEl = dialogEl;
    this.dialogClass = dialogClass;
}

ProcessSelectedRequestsDialog.elementsForRequestTypes = [
    {
        dataField: "selectedWaitingForApproval",
        singleTypeSelectedEl: ".allWaitingForApproval",
        detailsEl: ".detailsWaitingForApproval",
        countEl: ".waitingForApprovalCount"
    },
    {
        dataField: "selectedWaitingDeletion",
        singleTypeSelectedEl: ".allWaitingDeletion",
        detailsEl: ".detailsWaitingDeletion",
        countEl: ".waitingDeletionCount"
    },
    {
        dataField: "selectedApproved",
        singleTypeSelectedEl: ".allApproved",
        detailsEl: ".detailsApproved",
        countEl: ".approvedCount"
    },
    {
        dataField: "selectedRejected",
        singleTypeSelectedEl: ".allRejected",
        detailsEl: ".detailsRejected",
        countEl: ".rejectedCount"
    }
];

ProcessSelectedRequestsDialog.prototype = {
    init: function()
    {
        this.dialog = this.dialogEl.dialog({
                                               autoOpen: false,
                                               draggable: false,
                                               resizable: false,
                                               modal: true, // this is needed for overlay to appear, so we can handle 'outside' clicks to close the dialog
                                               width: this.dialogEl.width(), // this is why we need the dialog div to be in DOM already,
                                               minHeight: 0,
                                               dialogClass: this.dialogClass
                                           });

        this.wrapper = this.dialog.parent();
        this.wrapper.find(".ui-dialog-titlebar").hide();
        var buttonsPane = this.wrapper.find(".buttonsPlaceholder");
        this._createButtons(buttonsPane);

        return this;
    },

    show: function(showAt, selectedData)
    {
        this._fillSelectedData(selectedData);

        var self = this;

        this.dialog.dialog("open");

        // when the dialog is created, it is wrapped into a container, that's why we use wrapper
        this.wrapper.position({
                                  of: showAt,
                                  my: "left top",
                                  at: "left bottom",
                                  offset: "-114px 8px",
                                  collision: "none"
                              });

        // close the dialog when user clicks outside
        $('.ui-widget-overlay').click(function() { self.close(); });

        //to show placeholder text inside textarea
        this.dialog.focus();

        doOnceOnWindowResize(function() { self.close(); });
    },

    close: function()
    {
        this.dialog.dialog("close");
    },

    _createButtons: function(buttonsPane)
    {
        var self = this;
        this._createButton(buttonsPane, messageResource.getMessage("common.ok"), "greyButton okButton");
        this._createButton(buttonsPane, messageResource.getMessage("common.cancel"), "greyButton", function() { self.close(); });
    },

    _createButton: function(buttonsPane, text, className, callback)
    {
        var buttonEl = document.createElement("div");
        buttonEl.className = className;

        var spanEl = document.createElement("span");
        spanEl.innerHTML = text;

        buttonEl.appendChild(spanEl);

        buttonsPane[0].appendChild(buttonEl);

        if (callback)
            $(buttonEl).click(callback);

        return buttonEl;
    },

    _fillSelectedData: function(selection)
    {
        selection = this._getMaskedSelection(selection);

        var singleSelectedRequestType = null;
        var previousRequestTypesEmpty = true;
        _.each(ProcessSelectedRequestsDialog.elementsForRequestTypes, function(requestType)
        {
            // Get array of selected requests of current type
            var requests = selection[requestType.dataField];

            // Find out if there is only one selected type
            if (requests.length > 0)
            {
                if (previousRequestTypesEmpty)
                    singleSelectedRequestType = requestType;
                else
                    singleSelectedRequestType = null;
                previousRequestTypesEmpty = false;
            }

            // Fill per-request-type number info
            this.dialogEl.find(requestType.detailsEl).toggle(requests.length > 0);
            this.dialogEl.find(requestType.countEl).html(requests.length);
            this.dialogEl.find('.waitingDeletionCount').html(selection.selectedWaitingDeletion.length);
            this.dialogEl.find('.approvedCount').html(selection.selectedApproved.length);
            this.dialogEl.find('.rejectedCount').html(selection.selectedRejected.length);
        }, this);

        // Fill total number info
        var count = selection.getSize();
        this.dialogEl.find('.count').html(count);
        this.dialogEl.find('.singular').toggle(count == 1);
        this.dialogEl.find('.plural').toggle(count != 1);

        // Single/multiple types selected behavior
        _.each(ProcessSelectedRequestsDialog.elementsForRequestTypes, function(requestType)
        {
            this.dialogEl.find(requestType.singleTypeSelectedEl).toggle(requestType == singleSelectedRequestType);
        }, this);
        this.dialogEl.find('.details').toggle(singleSelectedRequestType == null);
        this.dialogEl.find('.differentSelected').toggle(singleSelectedRequestType == null);
    },

    _getMaskedSelection: function(selection)
    {
        return selection;
    }
};

function DeleteSelectedRequestsDialog(dialogEl, okClickHandler)
{
    ProcessSelectedRequestsDialog.call(this, dialogEl, "processSelectedRequestsDialog delete-ui-dialog");
    this.okClickHandler = okClickHandler;
}

DeleteSelectedRequestsDialog.prototype = _.create(ProcessSelectedRequestsDialog.prototype, {
    init: function()
    {
        ProcessSelectedRequestsDialog.prototype.init.call(this);
        this.dialogEl.find(".okButton").click(this.okClickHandler);
        return this;
    },

    _getMaskedSelection: function(selection)
    {
        return selection.deletableMasked;
    }
});

function RejectSelectedRequestsDialog(dialogEl, okClickHandler)
{
    ProcessSelectedRequestsDialog.call(this, dialogEl, "processSelectedRequestsDialog reject-ui-dialog", _.noop);
    this.okClickHandler = okClickHandler;
}

RejectSelectedRequestsDialog.prototype = _.create(ProcessSelectedRequestsDialog.prototype, {
    init: function()
    {
        ProcessSelectedRequestsDialog.prototype.init.call(this);

        var maxCommentLength = 2000;
        var longCommentTitle = messageResource.getMessage("actiplans.corporate_requests.long_comment");
        var approveRejectPlaceholderText = messageResource.getMessage("leave_time_requests.approve_reject_dialog.enter_approval_comment");

        this._textareaEl = this.dialogEl.find('textarea');
        this._textareaEl.attr('placeholder', approveRejectPlaceholderText).placeholder();

        this.rejectTextareaManager = new TextAreaLengthManager();
        this.rejectTextareaManager.init(this._textareaEl.get(0), maxCommentLength, longCommentTitle);
        this.rejectTextareaManager.setDependentButton(this.dialogEl.find(".okButton").get(0), this.okClickHandler);

        return this;
    },

    getComment: function()
    {
        return trim(this._textareaEl.pVal());
    },

    _fillSelectedData: function(selectedData)
    {
        ProcessSelectedRequestsDialog.prototype._fillSelectedData.call(this, selectedData);

        this.dialogEl.find("textarea").val("");
        this.rejectTextareaManager.update();
    },

    _getMaskedSelection: function(selection)
    {
        return selection.rejectableMasked;
    }
});

function LeaveTimeRequestsPageSelection(selectedWaitingForApproval, selectedApproved, selectedRejected, selectedWaitingDeletion)
{
    this.selectedWaitingForApproval = selectedWaitingForApproval;
    this.selectedApproved = selectedApproved;
    this.selectedRejected = selectedRejected;
    this.selectedWaitingDeletion = selectedWaitingDeletion;
    this.selected = Array.prototype.concat.call([], selectedWaitingForApproval, selectedApproved, selectedRejected, selectedWaitingDeletion);
}

LeaveTimeRequestsPageSelection.prototype = {
    getSize: function()
    {
        return this.selected.length;
    },

    isEmpty: function()
    {
        return this.selected.length == 0;
    }
};

function LeaveTimeRequestsPageSelectionWithMasks(selectedWaitingForApproval, selectedApproved, selectedRejected, selectedWaitingDeletion, onlyEditSelection)
{
    LeaveTimeRequestsPageSelection.call(this, selectedWaitingForApproval, selectedApproved, selectedRejected, selectedWaitingDeletion);
    this.approvableMasked = new LeaveTimeRequestsPageSelection(selectedWaitingForApproval, [], [], selectedWaitingDeletion);
    this.rejectableMasked = new LeaveTimeRequestsPageSelection(selectedWaitingForApproval, selectedApproved, [], selectedWaitingDeletion);
    this.deletableMasked = new LeaveTimeRequestsPageSelection(selectedWaitingForApproval, [].concat(selectedApproved, onlyEditSelection.approved), selectedRejected, []);
}

