function APReportsMediator() {}

APReportsMediator.prototype = {
    init: function(config)
    {
        this._subscribeForPageEvents();
        this._setInitialSettings(config);
        var tabChangeTrigger = function(state)
        {
            if (APReportsStore.getSelectedTabId() != state.selectedTabId)
            {
                $(document).trigger("apReports.selectedTabChange.routed", state.selectedTabId);
            }
        };
        this._initComponents();

        var selectedPage = _.findWhere(_.values(APReportsPageInfo),
                                   {tab: APReportsStore.getSelectedTabId()});
        Router.add(APReportsPageInfo.LEAVE_TIME_REPORT[APReportsPageUtils.ROUTE_PROP], tabChangeTrigger)
                .add(APReportsPageInfo.PAST_LEAVES_CALENDAR[APReportsPageUtils.ROUTE_PROP], tabChangeTrigger)
                .add(APReportsPageInfo.BALANCE_HISTORY[APReportsPageUtils.ROUTE_PROP], tabChangeTrigger)
                .base(selectedPage[APReportsPageUtils.ROUTE_PROP], {selectedTabId: selectedPage["tab"]}, document.title);

        $(document).trigger("apReports.init");

        return this;
    },

    _initComponents: function()
    {
        new APReportsPage($("body")).init();
        new APReportsPage.Header($("#apReportsHeaderPlaceholder")).init();
        new APReportsPage.TabsController().init();
        this.subPageMediators = {
            "leaveTimeReportTab": new APReportsPage.LeaveTimeBalancesReportPage.Mediator().init(),
            "pastLeavesCalendarTab": new APReportsPage.PastLeavesPage.Mediator().init(),
            "balanceHistoryTab": new APReportsPage.BalanceHistoryPage.Mediator().init()
        };

        this.subPageMediators[APReportsStore.getSelectedTabId()].openPage();
    },

    _subscribeForPageEvents: function()
    {
        var self = this;

        $(document).on("apReports.selectedTabChange.click", function(e, selectedTabId)
        {
            var page = _.findWhere(_.values(APReportsPageInfo), {tab: selectedTabId});
            Router.navigate(page[APReportsPageUtils.ROUTE_PROP], {selectedTabId: selectedTabId}, document.title);
        });

        $(document).on("apReports.selectedTabChange.routed", function(e, selectedTabId)
        {
            self.subPageMediators[APReportsStore.getSelectedTabId()].leavePage();
            APReportsStore.setSelectedTabId(selectedTabId);
            self.subPageMediators[selectedTabId].openPage();
            self._savePageSetting();
        });
    },

    _savePageSetting: function()
    {
        makeRpcCall("ReportsService.saveAPReportsPageSetting", [{tabId: APReportsStore.getSelectedTabId()}], _.noop, _.constant(true));
    },

    _setInitialSettings: function(config)
    {
        APReportsStore.setViewerId(config["viewerInfo"]["id"]);
        APReportsStore.setVisibleUsers(config["visibleUsers"]);
        APReportsStore.setIsLTM(config["isLTM"]);
        APReportsStore.setGroups(config["userGroups"], config["timeZoneGroups"]);
        APReportsStore.setSelectedTabId(this._initialTab(config["apReportsPageSetting"].tabId));
        APReportsPage.PastLeavesPage.Store.setDefaultWorkSchedule(config["defaultWorkSchedule"]);
        APReportsPage.PastLeavesPage.Store.setUserSettings(config["calendarUserSettings"]);
        APReportsPage.PastLeavesPage.Store.setSelectedUserId(config["calendarUserSettings"]["userId"]);
        APReportsPage.PastLeavesPage.Store.setLeaveTimeDetailsSettings(config["leaveTimeDetailsSettings"]);
        APReportsPage.PastLeavesPage.Store.setSelectorDateRange(config["leaveTimeHistorySelectorDateRange"].from.parseJSONDate(),
                                                                config["leaveTimeHistorySelectorDateRange"].to.parseJSONDate());
        APReportsPage.LeaveTimeBalancesReportPage.Store.setChartSectionConfig(config["leaveReportChartSettings"]);
        var leaveReportSettings = config["leaveReportSettings"];
        APReportsPage.LeaveTimeBalancesReportPage.Store.setReportParams(leaveReportSettings["leaveReportParameters"]);
        APReportsPage.LeaveTimeBalancesReportPage.Store.setShowAssignedOnly(leaveReportSettings["showOnlyAssignedUsers"]);
        APReportsPage.LeaveTimeBalancesReportPage.Store.setShowAssignedOnlyEnabled(config["showAssignedOnlyEnabled"]);
        APReportsPage.LeaveTimeBalancesReportPage.Store.setViewerInfo(config["viewerInfo"]);
        var balanceHistorySettings = config.balanceHistorySettings;
        APReportsPage.BalanceHistoryPage.Store.setFormatterSettings(balanceHistorySettings.formatterSettings);
        APReportsPage.BalanceHistoryPage.Store.setSickHistorySettings(balanceHistorySettings.sickSettings);
        APReportsPage.BalanceHistoryPage.Store.setPTOHistorySettings(balanceHistorySettings.ptoSettings);
        APReportsPage.BalanceHistoryPage.Store.setCommonSettings(balanceHistorySettings.commonSettings);
    },

    _initialTab: function(selectedTab)
    {
        var routedPath = Router.getPath();
        var pagesInfo = _.values(APReportsPageInfo);
        var initial = _.find(pagesInfo, function(page) { return page[APReportsPageUtils.ROUTE_PROP] == routedPath && page.confirm()});
        initial = initial || _.find(pagesInfo, function(page) { return page["tab"] == selectedTab && page.confirm()});
        initial = initial || _.find(pagesInfo, function(page){return page.confirm()});
        return initial["tab"];
    }
};