APReportsPage.BalanceHistoryPage.Manager = function() {};

APReportsPage.BalanceHistoryPage.Manager.prototype = {

    init: function()
    {
        this._ptoHistoryTable = null;
        this._sickHistoryTable = null;
        // Configuration panel initialization
        var configPanel = new APReportsPage.BalanceHistoryPage.ConfigPanel($("#balanceHistoryConfigurationPanel")).init();
        if (APReportsStore.getVisibleUsers().length > 1)
        {
            this.initUserSelector(APReportsPage.BalanceHistoryPage.Store.getSelectedUserId(), APReportsPage.BalanceHistoryPage.Store.getUserSelectorState());
        }
        this.initBalanceTypeSelector(APReportsPage.BalanceHistoryPage.Store.getBalanceType());
        new APReportsPage.BalanceHistoryPage.ConfigPanel.ExportSelector(configPanel.find('.export'), "apReports.balanceHistoryPage.exportPDF", "").init();

        this.balancesViewerController = new APReportsPage.BalanceHistoryPage.BalancesViewerController($("#balancesViewer")).init();

        var pageSelector = $(".balanceHistoryPageContent");
        new APReportsPage.BalanceHistoryPage.UserSickControlTurnedOffWarn(pageSelector).subscribeForUpdate();
        new APReportsPage.BalanceHistoryPage.UserPTOControlTurnedOffWarn(pageSelector).subscribeForUpdate();

        return this;
    },

    initBalanceTypeSelector: function(balanceType)
    {
        var balanceTypeItems = [];
        if (SystemSettings.apPtoBalanceCalculationEnabled)
            balanceTypeItems.push({
                                      text: messageResource.getMessage("common.balance_history.table.pto"),
                                      value: BalanceHistory.BalanceTypes.PTO
                                  });
        if (SystemSettings.apSickBalanceCalculationEnabled)
            balanceTypeItems.push({
                                      text: messageResource.getMessage("common.balance_history.table.sick"),
                                      value: BalanceHistory.BalanceTypes.SICK
                                  });

        this.balanceTypeSelector = new ListSelector({
            menuCls: "balanceHistoryTypeSelectorMenu",
            cls: "display-for-print",
            renderTo: document.getElementById("balanceTypeSelectorPlaceholder"),
            value: balanceType,
            listItems: balanceTypeItems
        });

        this.balanceTypeSelector.on("change", function(e, type)
        {
            $(document).trigger("apReports.balanceHistoryPage.changeBalanceType", type.value);
        });
    },

    initUserSelector: function(selectedUserId, state)
    {
        var users = APReportsStore.getVisibleUsers();
        var userGroupsEnabled = APReportsStore.getUserGroupsEnabled();
        var timeZoneGroupsEnabled = APReportsStore.getTimeZoneGroupsEnabled();
        var userGroups = APReportsStore.getUserGroups();
        var timeZoneGroups = APReportsStore.getTimeZoneGroups();

        var userSelectorPanelConfig = {
            groupBySettingCookieName: "balanceHistoryUserSelectorGroupBySetting",
            showDisabledSettingCookieName: "balanceHistoryUserSelectorShowDisabledSetting",
            collapsedCookiesKey: "balanceHistory"
        };
        var userSelectorPanel = userGroupsEnabled || timeZoneGroupsEnabled ? new AT.UserForLeaveHistorySelectorPanel(userSelectorPanelConfig) :
                                new AT.UserForLeaveHistorySelectorPanelWithInlineSearch(userSelectorPanelConfig);
        var userSelectorMenu = new AT.SharedUserButtonMenu({
            panel: userSelectorPanel,
            additionalCls: "ap-selector-menu " +
                           "userForLeaveTimeHistorySelectorMenu " +
                           (userGroupsEnabled || timeZoneGroupsEnabled ? "userSelectorWithGroupingModePanel " : ""),
            shadow: false
        });
        var chooseUserButton = new AT.HierarchySelectorDropdownButton({
            renderTo: "balanceHistory_chooseUserButton",
            menu: userSelectorMenu
        });

        userSelectorPanel.initialize(APReportsStore.getViewerId(), userGroupsEnabled, timeZoneGroupsEnabled, users, userGroups, timeZoneGroups, selectedUserId);
        userSelectorPanel.setSelectionChangedHandler(function()
                                                     {
                                                         $(document).trigger("apReports.balanceHistoryPage.changeSelectedUser",
                                                                             [userSelectorPanel.getSelectedUserId(), userSelectorPanel.getState()]);
                                                         userSelectorPanel.close();
                                                     });
        if (!_.isNull(state))
        {
            userSelectorPanel.loadState(state);
        }
        this.userSelector = userSelectorPanel;
    },

    getUserSelectorState: function()
    {
        return this.userSelector ? this.userSelector.getState() : null;
    },

    _createPTOTable: function()
    {
        return new APReportsPage.BalanceHistoryPage.PTOTable($("#ptoBalanceHistoryTablePlaceholder")).init();
    },

    _createSickTable: function()
    {
        return new APReportsPage.BalanceHistoryPage.SickTable($("#sickBalanceHistoryTablePlaceholder")).init();
    },

    updatePTOTable: function()
    {
        if (_.isNull(this._ptoHistoryTable))
        {
            this._ptoHistoryTable = this._createPTOTable();
        }
        else
        {
            this._ptoHistoryTable.update();
        }
    },

    updateSickTable: function()
    {
        if (_.isNull(this._sickHistoryTable))
        {
            this._sickHistoryTable = this._createSickTable();
        }
        else
        {
            this._sickHistoryTable.update();
        }
    },

    updateBalancesViewer: function()
    {
        this.balancesViewerController.updateBalances(APReportsPage.BalanceHistoryPage.Store.getBalancesViewerDate(),
                                                    APReportsPage.BalanceHistoryPage.Store.getBalancesViewerBalance());
    },

    updateSelectors: function()
    {
        var state = APReportsPage.BalanceHistoryPage.Store.getUserSelectorState();
        if (state)
            this.userSelector.loadState(state);
        this.balanceTypeSelector.setActiveItemByValue(APReportsPage.BalanceHistoryPage.Store.getBalanceType(), true);
    },

    spinBalancesViewer: function()
    {
        this.balancesViewerController.spin();
    }
};