APReportsPage.BalanceHistoryPage.Mediator = function() {};

APReportsPage.BalanceHistoryPage.Mediator.prototype = _.create(PageMediator.prototype, {
    getPage: function()
    {
        return this.page
    },

    beforeOpenPageFirstTime: function()
    {
        this.tipsManager = new BalanceHistoryPageOverlayTipsManager();
        this.page = new APReportsPage.BalanceHistoryPage($("#apBalanceHistoryPlaceholder")).init();
        this._balanceRpc = undefined;

        this.balanceHistoryManager = new APReportsPage.BalanceHistoryPage.Manager().init();

        APReportsPage.BalanceHistoryPage.Store.setUserSelectorState(this.balanceHistoryManager.getUserSelectorState());
        APReportsPage.BalanceHistoryPage.Store.backupSelectors();

        this._subscribeForPageEvents();
        this._loadBalanceHistory();
    },

    beforeOpenPage: function()
    {
        PageOverlayTipsManager.setInstance(this.tipsManager);
    },

    _subscribeForPageEvents: function()
    {
        var self = this;

        $(document).on("apReports.balanceHistoryPage.SickTable.changeHistorySettings", function(e, settings)
        {
            APReportsPage.BalanceHistoryPage.Store.setSickHistorySettings({balanceHistoryRecordsPerPage: settings.recordsPerPage});
            makeRpcCall("ReportsService.saveMySickBalanceHistorySettings", [settings.recordsPerPage]);

        });

        $(document).on("apReports.balanceHistoryPage.PTOTable.changeHistorySettings", function(e, settings)
        {
            APReportsPage.BalanceHistoryPage.Store.setPTOHistorySettings({balanceHistoryRecordsPerPage: settings.recordsPerPage});
            makeRpcCall("ReportsService.saveMyPtoBalanceHistorySettings", [settings.recordsPerPage]);
        });

        $(document).on("apReports.balanceHistoryPage.changeBalanceType", function(e, balanceType)
        {
            APReportsPage.BalanceHistoryPage.Store.setBalanceType(balanceType);
            self._loadBalanceHistory();
        });

        $(document).on("apReports.balanceHistoryPage.changeSelectedUser", function(e, selectedUserId, state)
        {
            APReportsPage.BalanceHistoryPage.Store.setSelectedUserId(selectedUserId);
            APReportsPage.BalanceHistoryPage.Store.setUserSelectorState(state);
            // don`t need requests for another user
            self._loadBalanceHistory()
        });

        $(document).on("apReports.balanceHistoryPage.balancesViewerDateChange", function(e, balanceViewerDate)
        {
            APReportsPage.BalanceHistoryPage.Store.setBalancesViewerDate(balanceViewerDate);
            self._loadBalance();
        });

        function updateCommon(updateSelectors)
        {
            if (updateSelectors)
                self.balanceHistoryManager.updateSelectors();
            self.balanceHistoryManager.updateBalancesViewer();
        }

        $(document).on("apReports.balanceHistoryPage.endLoadSickHistory", function(e, updateSelectors)
        {
            updateCommon(updateSelectors);
            self.balanceHistoryManager.updateSickTable();
        });

        $(document).on("apReports.balanceHistoryPage.endLoadPTOHistory", function(e, updateSelectors)
        {
            updateCommon(updateSelectors);
            self.balanceHistoryManager.updatePTOTable();
        });

        $(document).on("apReports.balanceHistoryPage.startLoadBalance", function()
        {
            self.balanceHistoryManager.spinBalancesViewer();
        });

        $(document).on("apReports.balanceHistoryPage.endLoadBalance", function()
        {
            self.balanceHistoryManager.updateBalancesViewer();
        });

        $(document).on("apReports.balanceHistoryPage.exportPDF", function()
        {
            var spinner = new SpinnerController();
            spinner.spin("balanceHistoryExportSelector");
            document.getElementById('balanceTypeFormId').value = (APReportsPage.BalanceHistoryPage.Store.getBalanceType() == BalanceHistory.BalanceTypes.PTO) ?
                                                                 BalanceType.PTO : BalanceType.SICK;
            document.getElementById('selectedUserFormId').value = APReportsPage.BalanceHistoryPage.Store.getSelectedUserId();
            document.getElementById('BalanceHistoryForm').submit();
            setTimeout(function() {spinner.stop();}, 2000);
        });

    },

    _loadBalanceHistory: function()
    {
        // we load balances with current selected date on this request
        this.abortRpcCall(this._balanceRpc);
        switch (APReportsPage.BalanceHistoryPage.Store.getBalanceType())
        {
            case BalanceHistory.BalanceTypes.PTO:
                this._loadPTOBalanceHistory();
                break;
            case BalanceHistory.BalanceTypes.SICK:
                this._loadSickBalanceHistory();
                break;
        }
    },

    _loadPTOBalanceHistory: function()
    {
        $(document).trigger("apReports.balanceHistoryPage.beginLoadPTOHistory");
        var self = this;
        var userId = APReportsPage.BalanceHistoryPage.Store.getSelectedUserId();
        var balanceDate = APReportsPage.BalanceHistoryPage.Store.getBalancesViewerDate();
        this.abortRpcCall(this._ptoHistoryRpc);
        this._ptoHistoryRpc = makeRpcCall("ReportsService.getPtoBalanceHistoryPageData", [balanceDate, userId],
                                          function(userBalanceHistoryPageData)
                                          {
                                              self._processBalanceViewerData(userBalanceHistoryPageData);
                                              APReportsPage.BalanceHistoryPage.Store.backupSelectors();

                                              if (!self._validateLeaveTypes(userBalanceHistoryPageData.balanceHistoryRecords.list))
                                                  return;

                                              APReportsPage.BalanceHistoryPage.Store.setPTOModels(userBalanceHistoryPageData.balanceHistoryRecords.list);
                                              APReportsPage.BalanceHistoryPage.Store.setBalanceControlOn(userBalanceHistoryPageData.userControlOn);
                                              $(document).trigger("apReports.balanceHistoryPage.endLoadPTOHistory");
                                          },
                                          function(jqXHR, textStatus, errorThrown)
                                          {
                                              if (textStatus == "rpc_sessionExpired" || textStatus == "rpc_noRights")
                                                  return false;

                                              if (textStatus == "abort")
                                                  return true;

                                              if (textStatus == "rpc_customException")
                                              {
                                                  if (errorThrown.exception == "com.actimind.actiplans.services.NoRightsForViewingUserReportsData")
                                                  {
                                                      return self.showAlertAndReloadPage("actiplans.reports.balance_history.no_right_for_viewing_user_balance_history");
                                                  }

                                                  if (errorThrown.exception === "com.actimind.actitime.administration.NoSuchUserException")
                                                  {
                                                      return self.showAlertAndReloadPage("common.error.selected_user_was_deleted");
                                                  }
                                              }

                                              APReportsPage.BalanceHistoryPage.Store.restoreSelectors();
                                              $(document).trigger("apReports.balanceHistoryPage.endLoadPTOHistory", true);

                                              return false;
                                          }
        );
    },

    _loadSickBalanceHistory: function()
    {
        $(document).trigger("apReports.balanceHistoryPage.beginLoadSickHistory");
        var self = this;
        var userId = APReportsPage.BalanceHistoryPage.Store.getSelectedUserId();
        var balanceDate = APReportsPage.BalanceHistoryPage.Store.getBalancesViewerDate();
        this.abortRpcCall(this._sickHistoryRpc);
        this._sickHistoryRpc = makeRpcCall("ReportsService.getSickBalanceHistoryPageData", [balanceDate, userId],
                                           function(userBalanceHistoryPageData)
                                           {
                                               self._processBalanceViewerData(userBalanceHistoryPageData);
                                               APReportsPage.BalanceHistoryPage.Store.backupSelectors();

                                               if (!self._validateLeaveTypes(userBalanceHistoryPageData.balanceHistoryRecords.list))
                                                   return;

                                               APReportsPage.BalanceHistoryPage.Store.setSickModels(userBalanceHistoryPageData.balanceHistoryRecords.list);
                                               APReportsPage.BalanceHistoryPage.Store.setBalanceControlOn(userBalanceHistoryPageData.userControlOn);
                                               $(document).trigger("apReports.balanceHistoryPage.endLoadSickHistory");

                                           },
                                           function(jqXHR, textStatus, errorThrown)
                                           {
                                               if (textStatus == "abort")
                                               {
                                                   return true;
                                               }

                                               if (textStatus == "rpc_customException")
                                               {
                                                   if (errorThrown.exception == "com.actimind.actiplans.services.NoRightsForViewingUserReportsData")
                                                   {
                                                       return self.showAlertAndReloadPage("actiplans.reports.balance_history.no_right_for_viewing_user_balance_history");
                                                   }

                                                   if (errorThrown.exception === "com.actimind.actitime.administration.NoSuchUserException")
                                                   {
                                                       return self.showAlertAndReloadPage("common.error.selected_user_was_deleted");
                                                   }
                                               }

                                               APReportsPage.BalanceHistoryPage.Store.restoreSelectors();
                                               $(document).trigger("apReports.balanceHistoryPage.endLoadSickHistory", true);

                                               return false;
                                           }
        );
    },

    _loadBalance: function()
    {
        $(document).trigger("apReports.balanceHistoryPage.startLoadBalance");
        var self = this;
        var store = APReportsPage.BalanceHistoryPage.Store;
        var balanceType = store.getBalanceType();
        var serviceMethod = balanceType == BalanceHistory.BalanceTypes.PTO ? "ReportsService.getPtoBalanceForDate" : "ReportsService.getSickBalanceForDate";
        this.abortRpcCall(this._balanceRpc);
        this._balanceRpc = makeRpcCall(
                serviceMethod,
                [store.getBalancesViewerDate(), store.getSelectedUserId()],
                function(balance)
                {
                    store.backupBalancesViewerDate();
                    store.setBalancesViewerBalance(balance);
                    $(document).trigger("apReports.balanceHistoryPage.endLoadBalance");
                },
                function(jqXHR, textStatus, errorThrown)
                {
                    if (textStatus == "abort")
                    {
                        return true;
                    }

                    if (textStatus == "rpc_customException")
                    {
                        if (errorThrown.exception == "com.actimind.actiplans.services.NoRightsForViewingUserReportsData")
                        {
                            var msgKey = (balanceType == BalanceHistory.BalanceTypes.PTO) ?
                                         "actiplans.reports.balance_history.no_right_for_viewing_user_sick_balance" :
                                         "actiplans.reports.balance_history.no_right_for_viewing_user_pto_balance";
                            return self.showAlertAndReloadPage(msgKey);
                        }
                        if (errorThrown.exception === "com.actimind.actitime.administration.NoSuchUserException")
                        {
                            return self.showAlertAndReloadPage("common.error.selected_user_was_deleted")
                        }
                    }

                    store.restoreBalancesViewerDate();
                    $(document).trigger("apReports.balanceHistoryPage.endLoadBalance");
                }
        );
    },

    _validateLeaveTypes: function(records)
    {
        return _.every(records, function(record)
        {
            return LeaveTypeUtil.validateLeaveTypeById(record.leaveTypeId);
        })
    },

    _processBalanceViewerData: function(userBalanceHistoryPageData)
    {
        // requested date can differ from response due to userId changes and not available date
        APReportsPage.BalanceHistoryPage.Store.setBalancesViewerDate(Date.parseJSONDate(userBalanceHistoryPageData.balanceDate));
        APReportsPage.BalanceHistoryPage.Store.setUserWorkScheduleData(userBalanceHistoryPageData.workSchedule);
        APReportsPage.BalanceHistoryPage.Store.setBalancesViewerBalance(userBalanceHistoryPageData.balance);
    }
});