APReportsPage.BalanceHistoryPage.SingleBalanceViewer = function($containerEl)
{
    TemplateView.call(this, $containerEl);
};

APReportsPage.BalanceHistoryPage.SingleBalanceViewer.prototype = _.create(BalancesViewer.prototype, {

    updateOnInit: false,
    useUnderscoreTemplate: false,

    html: '<div class="calendar">' +
          ' <span class="calendarSpan"></span>' +
          ' <span class="calendarPlaceholder"></span>' +
          ' <span>:</span>' +
          '</div>' +
          '<div class="balances">' +
          ' <span class="bold mainValue"></span>' +
          ' <span class="additionalValue"></span>' +
          '</div>',
    
    initComponents: function(config, onDateChangeCallback)
    {
        if (_.isUndefined(APReportsPage.BalanceHistoryPage.SingleBalanceViewer.messages))
        {
            APReportsPage.BalanceHistoryPage.SingleBalanceViewer.messages = {
                ptoLabel: messageResource.getMessage("common.balancesViewer.balanceLabel", [messageResource.getMessage("common.balance_history.table.pto")]),
                sickLabel: messageResource.getMessage("common.balancesViewer.balanceLabel", [messageResource.getMessage("common.balance_history.table.sick")])
            };
        }

        this.formatter = new BalanceFormatter().init(config.hoursPerDay);

        this.$calendarLabel = this.$containerEl.find('.calendarSpan');
        this.$calendar = this.$containerEl.find('.calendarPlaceholder');
        this.$balances = this.$containerEl.find('.balances');
        this.$mainValue = this.$balances.find('.mainValue');
        this.$additionalValue = this.$balances.find('.additionalValue');
        this.onDateChange = onDateChangeCallback;
        this._createDateSelector(config);
        this.updateBalanceType(config);
    },

    update: function(balance)
    {
        this._displayBalance(balance);
    },

    updateBalanceType: function(config)
    {
        this.updateCalendarLabel(config.displayPto);
        this.formatter.setShowInHours(config.showInHours);
    },

    updateCalendarLabel: function(displayPto)
    {
        var messages = APReportsPage.BalanceHistoryPage.SingleBalanceViewer.messages;
        this.$calendarLabel.text(displayPto ? messages.ptoLabel : messages.sickLabel);
    },

    _createDateSelector: function(config)
    {
        this.dateSelector = BalancesViewer.createDateSelector(this.$calendar[0], config.userInitialSchedule,
                                                              config.userSchedules, config.userHireDate, config.userReleaseDate);
        if (this.onDateChange)
            this.dateSelector.on("change", this.onDateChange);
        return this.dateSelector;
    },

    _displayBalance: function( balance )
    {
        var isNegative = !_.isNull(balance) && !!this.formatter.isNegativePTO(balance);
        this.$balances.toggleClass("negative", isNegative);
        this.$balances.toggleClass("empty", _.isNull(balance));

        this._displayMainBalance(balance);
        // may be additional value without sign looks prettier
        //this._displayAdditionalBalance(isNegative ? -balance : balance);
        this._displayAdditionalBalance(balance);
    },

    _displayMainBalance: function(balance)
    {
        this.$mainValue.text(this.formatter.format(balance));
    },

    _displayAdditionalBalance: function(balance)
    {
        this.$additionalValue.html('&nbsp;&nbsp;('+this.formatter.format(balance, true)+')');
    }

});