APReportsPage.ConfigPanel.EditConfigPanel = function($containerEl, reportParameters)
{
    TemplateView.call(this, $containerEl);
    this.$containerEl = $containerEl;
    this.reportParameters = reportParameters;
};

APReportsPage.ConfigPanel.EditConfigPanel.prototype = _.create(TemplateView.prototype, {
    html: "<div class='editConfiguration dropDownIcon'>" +
          "   <%-edit_button%>" +
          "</div>",

    messageKeys: function()
    {
        return {
            "edit_button": "report.config.edit_button.title"
        }
    },

    afterRender: function()
    {
        var self = this;
        var users = APReportsStore.getVisibleUsers();
        var userGroups = APReportsStore.getUserGroups();
        var timeZoneGroups = APReportsStore.getTimeZoneGroups();
        var leavesData = _.map(LeaveTypeStore.getLeaveTypesOrdered(), _.clone);
        this.editConfigController = new LeavesEditFormConfigController().init(
                {
                    leavesData: leavesData,
                    reportParameters: this.reportParameters,
                    users: users,
                    userGroups: userGroups,
                    timeZoneGroups: timeZoneGroups,
                    changeCallback: function() { self.changeParametersCallback()},
                    productSettingsValidator : new APProductSettingsValidator(),
                    reportRightsValidator: new APReportRightsValidator()
                });

        this.editConfigController.setConfigButtonHandlers(
                {
                    onApply: function() {self.editConfigApplied();},
                    onCancel: function() {self.editConfigCanceled();}
                });

        this.editConfigurationButton = this.$containerEl.parent();
        this.panel = new DropdownDialog(
                {
                    wrapperClass: "leaveEditConfigPanel editReportConfigurationWrapper noprint",
                    dialogDiv: $("#editReportConfigurationPanel"),
                    hideButtons: true,
                    hideOnResize: !isIE8(),
                    showEmptyTitleBar: false,
                    outsideClickCallback: function() {self.editConfigCanceled();},
                    hideOnResizeCallback: function() {self.editConfigCanceled();},
                    onClose: function(event)
                    {
                        if (event.keyCode === $.ui.keyCode.ESCAPE)
                            self.editConfigCanceled();

                        self.editConfigurationButton.removeClass("active");
                    }
                });

        this.editConfigurationButton.click(
                function(el)
                {
                    self.editConfigurationButton.addClass("active");

                    self.panel.showAt(this, {
                        offset: "0 10px",
                        my: "left top",
                        at: "left bottom",
                        collision: "none"
                    });

                    var panelTop = $("#editReportConfigurationPanel").offset().top;
                    var panelWrapper = $(".editReportConfigurationWrapper");
                    if (panelTop < $(el.target).offset().top)
                    {
                        panelWrapper.addClass("toTop");
                        panelWrapper.removeClass("toBottom");
                    }
                    else
                    {
                        panelWrapper.addClass("toBottom");
                        panelWrapper.removeClass("toTop");
                    }
                });

        $(document).trigger("apReports.leaveTimeBalancesReport.reportConfig.create",  this.editConfigController.getConfig());
    },

    editConfigApplied: function()
    {
        if (!this.validate())
            return;

        var config = this.editConfigController.getConfig();
        this.editConfigController.updateUI();
        this.editConfigController.hideErrorTooltip();
        this.panel.hide();
        this.editConfigController.setSelectorsInitStateFromCurrent();
        $(document).trigger("apReports.leaveTimeBalancesReport.reportConfig.change", config);
    },


    editConfigCanceled: function()
    {
        this.editConfigController.resetConfig();
        if (this.editConfigController.validate())
            this.editConfigController.updateUI();
        this.editConfigController.hideErrorTooltip();
        this.panel.hide();
    },

    changeParametersCallback: function()
    {
        if (this.editConfigController && !this.editConfigController.validate())
            this.editConfigController.updateUI(true);
    },

    validateWithUI: function()
    {
        if (!this.editConfigController.validate())
        {
            this.editConfigController.updateUI(true);
            return false;
        }
        return true;
    },

    validate: function()
    {
        return this.editConfigController.validate();
    }
});