APReportsPage.ChartController = function(){};

APReportsPage.ChartController.prototype = {

    init: function()
    {
        this._chartContainerId = "leaveTimeAndBalancesChartContainer";
        this._chartContainerIdForExport = "leaveTimeAndBalancesChartContainerHidden";

        //init chart section with controls and chart placeholder
        this._chartSection = new APReportsPage.ChartSection($("#leaveTimeAndBalancesChartSectionContainer")).init();
        this._renderChartMoire = new LoadingMoire(this._chartSection.find(".chartContainers"), APReportsPageUtils.SPINNER_SETTINGS).init();

        var defaultChartSize = {
            "width": 970,
            "height": 320
        };

        // init available chart generators
        this._leaveTimeChartGenerator = new APReportsPage.LeaveTimeChartGenerator().init(defaultChartSize);
        this._balancesChartGenerator = new APReportsPage.BalancesChartGenerator().init(defaultChartSize);

        var self = this;
        $(APReportsPage.LeaveTimeBalancesReportPage.Store).on("reportDataChange", function()
        {
            self._currentState["isNeedRebuildChart"] = true;
            self._currentState["isNeedRebuildTotals"] = true;

            self._validateAndChangeChartTypeIfNecessary();

            self._chartSection.update();
            self.showChart();
        });

        $(APReportsPage.LeaveTimeBalancesReportPage.Store).on("chartSectionConfigChange", function()
        {
            var chartSectionConfig = APReportsPage.LeaveTimeBalancesReportPage.Store.getChartSectionConfig();
            self._currentState["isChartVisible"] = chartSectionConfig["showChart"];
            var newChartType = chartSectionConfig["chartType"];
            var newChartSortOption = chartSectionConfig["chartSortOption"];
            var chartTypeChanged = newChartType != self._currentState["chartType"];
            if (chartTypeChanged || !_.isEqual(newChartSortOption, self._currentState["chartSortOption"]))
            {
                if (chartTypeChanged)
                {
                    self._currentState["isNeedRebuildTotals"] = true;
                }
                self._currentState["isNeedRebuildChart"] = true;
                self._currentState["chartType"] = newChartType;
                self._currentState["chartSortOption"] = _.clone(newChartSortOption);
            }
            self.showChart();
        });

        var chartSectionConfig = APReportsPage.LeaveTimeBalancesReportPage.Store.getChartSectionConfig();

        this._currentState = {
            "isNeedRebuildChart": true,
            "isChartVisible": chartSectionConfig["showChart"],
            "chartType": chartSectionConfig["chartType"],
            "chartSortOption": _.clone(chartSectionConfig["chartSortOption"])
        };

        return this;
    },

    _validateAndChangeChartTypeIfNecessary: function()
    {
        var reportParams = APReportsPage.LeaveTimeBalancesReportPage.Store.getReportParams();
        if ((this._currentState["chartType"] == APReportChartType.LEAVE_TIME && !LeavesReportUtils.reportParamsWithLeaves(reportParams)) ||
            (this._currentState["chartType"] == APReportChartType.PTO_BALANCE && !LeavesReportUtils.reportParamsWithPTO(reportParams)) ||
            (this._currentState["chartType"] == APReportChartType.SICK_BALANCE && !LeavesReportUtils.reportParamsWithSick(reportParams)))
        {
            var newChartType;
            if (LeavesReportUtils.reportParamsWithLeaves(reportParams))
            {
                newChartType = APReportChartType.LEAVE_TIME;
            }
            else if (LeavesReportUtils.reportParamsWithPTO(reportParams))
            {
                newChartType = APReportChartType.PTO_BALANCE;
            }
            else
            {
                newChartType = APReportChartType.SICK_BALANCE;
            }
            this._currentState["chartType"] = newChartType;
            var chartSectionConfig = APReportsPage.LeaveTimeBalancesReportPage.Store.getChartSectionConfig();
            APReportsPage.LeaveTimeBalancesReportPage.Store.setChartSectionConfig(_.extend(chartSectionConfig, {"chartType": newChartType}));
        }
    },

    showChart: function()
    {
        if (!this._currentState["isChartVisible"])
        {
            this._chartSection.hideChartContainer();
            return;
        }
        this._chartSection.showChartContainer();

        if (this._currentState["isNeedRebuildTotals"])
        {
            this._chartSection.updateTotals(this._getTotalsTableData());
            this._currentState["isNeedRebuildTotals"] = false;
        }

        if (!this._currentState["isNeedRebuildChart"])
        {
            return;
        }
        this._renderChartMoire.show();

        var self = this;
        this._getCurrentChartGeneratorBondedWithData().showChart(
                this._chartContainerId,
                {
                    "onRendered": function()
                    {
                        self._renderChartMoire.hide();
                    }
                });

        this._currentState["isNeedRebuildChart"] = false;
    },

    exportChart: function(chartSize, onExportFinished)
    {
        this._chartSection.updateSizesForHiddenChartContainer(chartSize["height"], chartSize["width"]);

        var self = this;
        this._getCurrentChartGeneratorBondedWithData().exportChart(
                this._chartContainerIdForExport,
                {
                    "onExported": function(img)
                    {
                        self._chartSection.clearHiddenChartContainer();
                        onExportFinished(img);
                    },
                    "chartSize": chartSize
                });
    },

    _getCurrentChartGeneratorBondedWithData: function()
    {
        var data = APReportsPage.LeaveTimeBalancesReportPage.Store.getReportData();
        var chartData;
        var chartGenerator;
        var chartConfig;
        if (this._currentState["chartType"] == APReportChartType.LEAVE_TIME)
        {
            chartData = this._prepareLeaveTimeChartData(data);
            chartGenerator = this._leaveTimeChartGenerator;
            chartConfig = this._prepareConfigForLeaveTimeChart(data);
        }
        else
        {
            var balanceType = this._currentState["chartType"] == APReportChartType.PTO_BALANCE ? BalanceType.PTO : BalanceType.SICK;
            chartData = this._prepareBalancesChartData(data, balanceType);
            chartGenerator = this._balancesChartGenerator;
            chartConfig = this._prepareConfigForBalancesChart(data, balanceType);
        }

        var self = this;
        var showChartFn;
        var exportChartFn;
        if (chartData.length == 0)
        {
            showChartFn = function(_, additionalConfig)
            {
                self._chartSection.switchToNoDataState();
                defer(function(){additionalConfig["onRendered"]()});
            };
            exportChartFn = function(_, additionalConfig)
            {
                additionalConfig["onExported"](null);
            }
        }
        else
        {
            showChartFn = function(container, additionalConfig)
            {
                self._chartSection.switchToHasDataState();
                chartConfig = $.extend(true, chartConfig, additionalConfig);
                chartGenerator.renderChart(container, chartData, chartConfig);
            };
            exportChartFn = function(container, additionalConfig)
            {
                var exportChartConfig = chartGenerator.getExportChartConfig();
                chartConfig = $.extend(true, chartConfig, additionalConfig, exportChartConfig);
                chartGenerator.exportChartToJPG(container, chartData, chartConfig);
            }
        }

        return {
            "showChart": showChartFn,
            "exportChart": exportChartFn
        }
    },

    _prepareBalancesChartData: function(data, balanceType)
    {
        var balanceFieldName = balanceType == BalanceType.PTO ? "ptoBalance" : "sickBalance";

        // excludes when balance data is null or both balances are null
        var filteredData = _.filter(data["items"], function(item)
        {
            var balanceData = item[balanceFieldName];
            return !_.isNull(balanceData) && (!_.isNull(balanceData["balanceForFirstColumn"]) || !_.isNull(balanceData["balanceForSecondColumn"]));
        });
        var chartData = this._prepareCategoryAndBalances(filteredData, balanceFieldName);
        // sorts data
        var comparator;
        var chartSortOption = this._currentState["chartSortOption"];
        if (chartSortOption.sortType == APReportChartSortOption.BY_VALUES)
        {
            comparator = this._getComparatorForBalancesByValue(chartSortOption.asc);
        }
        else
        {
            comparator = this._getComparatorByCategory(chartSortOption.asc);
        }
        chartData.sort(comparator);
        return chartData;
    },

    _getComparatorForBalancesByValue: function(asc)
    {
        var self = this;
        var secondaryComparator = this._getComparatorByCategory(asc);
        return function(x, y)
        {
            if (+x["balanceForFirstColumn"] == +y["balanceForFirstColumn"])
            {
                if (+x["balanceForSecondColumn"] == +y["balanceForSecondColumn"])
                {
                    return secondaryComparator(x, y);
                }
                return self._compareValues(+x["balanceForSecondColumn"], +y["balanceForSecondColumn"], asc);
            }
            return self._compareValues(+x["balanceForFirstColumn"], +y["balanceForFirstColumn"], asc);
        }
    },
    
    _getComparatorByCategory: function(asc)
    {
        var comparator = ReportChartUtils.getSortCategoryFunctionForChart(this._getReportParams().firstGroupingLevel);
        return function(x, y)
        {
            return asc ? comparator(x.category, y.category) : -comparator(x.category, y.category);
        }
    },

    _compareValues: function(x, y, asc)
    {
        return x < y ? (asc ? -1 : 1) : (asc ? 1 : -1);
    },

    _getComparatorForLeaveTimeByValue: function(asc)
    {
        var self = this;
        var secondaryComparator = this._getComparatorByCategory(asc);
        return function(x, y)
        {
            if (x["totalLeaveSpent"] == y["totalLeaveSpent"])
            {
                return secondaryComparator(x, y);
            }
            return self._compareValues(x["totalLeaveSpent"], y["totalLeaveSpent"], asc);
        }
    },

    _prepareCategoryAndBalances: function(data, balanceFieldName)
    {
        var preparedData = this._prepareCategory(data);
        for (var i = 0; i < data.length; i++)
        {
            preparedData[i]["balanceForFirstColumn"] = data[i][balanceFieldName]["balanceForFirstColumn"];
            preparedData[i]["balanceForSecondColumn"] = data[i][balanceFieldName]["balanceForSecondColumn"];
        }
        return preparedData;
    },

    _prepareCategoryAndLeaveTime: function(data)
    {
        var preparedData = this._prepareCategory(data);
        for (var i = 0; i < data.length; i++)
        {
            preparedData[i]["leaveSpent"] = data[i]["leaveSpent"];
            preparedData[i]["totalLeaveSpent"] = data[i]["totalLeaveSpent"];
        }
        return preparedData;
    },

    _prepareCategory: function(data)
    {
        var groupingLevel = this._getReportParams().firstGroupingLevel;
        var store = APReportsPage.LeaveTimeBalancesReportPage.Store;
        var preparedData = [];
        var categoryPrepareFunction;
        switch (groupingLevel)
        {
            case ReportGroupingLevel.USER:
                categoryPrepareFunction = function(obj)
                {
                    return store.getUser(obj.id)
                };
                break;
            case ReportGroupingLevel.USER_GROUP:
                categoryPrepareFunction = function(obj)
                {
                    return store.getUserGroup(obj.id)
                };
                break;
            case ReportGroupingLevel.TIME_ZONE_GROUP:
                categoryPrepareFunction = function(obj)
                {
                    return store.getTimeZoneGroup(obj.id)
                };
                break;
            case ReportGroupingLevel.DATE:
            case ReportGroupingLevel.MONTH:
                categoryPrepareFunction = function(obj)
                {
                    return obj.id["date"];
                };
                break;
            case ReportGroupingLevel.WEEK:
                categoryPrepareFunction = function(obj)
                {
                    return obj.id;
                };
                break;
        }
        _.each(data, function(val)
        {
            preparedData.push({"category": categoryPrepareFunction(val)});
        });
        return preparedData;
    },

    _prepareLeaveTimeChartData: function(data)
    {
        var filteredData = _.filter(data["items"], function(item)
        {
            return item["totalLeaveSpent"] != 0;
        });
        var chartData = this._prepareCategoryAndLeaveTime(filteredData);

        var comparator;
        var chartSortOption = this._currentState["chartSortOption"];
        if (chartSortOption.sortType == APReportChartSortOption.BY_VALUES)
        {
            comparator = this._getComparatorForLeaveTimeByValue(chartSortOption.asc);
        }
        else
        {
            comparator = this._getComparatorByCategory(chartSortOption.asc);
        }
        chartData.sort(comparator);

        return chartData;
    },
    
    _getLeaveTypesIdsOrdered: function(data)
    {
        var leaveTypesIds = _.map(data["leaveSpent"], function(obj) {return obj.leaveTypeId;});
        return _.chain(LeaveTypeStore.getLeaveTypesOrder())
                .filter(function(id) { return _.contains(leaveTypesIds, id) })
                .reverse().value();
    },

    _prepareConfigForLeaveTimeChart: function(data)
    {
        var reportParams = this._getReportParams();
        return {
            "colorScheme": this._getLeaveTimeChartColorScheme(),
            "valueAxisLabelFunction": this._getFormatUtils().getTimeFormatFunction(reportParams.timeFormat),
            "categoryAxisLabelFunction": this._getFormatUtils().getCategoryAxisLabelFunction(reportParams.firstGroupingLevel),
            "categoryFormatFunctionForBalloon": this._getFormatUtils().getCategoryFormatFunctionForBalloon(reportParams.firstGroupingLevel),
            "leaveTypesIdsOrdered": this._getLeaveTypesIdsOrdered(data),
            "showScrollbar": true
        }
    },

    _prepareConfigForBalancesChart: function(data, balanceType)
    {
        var reportParams = this._getReportParams();
        var dateFormatFunction = this._getFormatUtils().getDateFormatFunction();
        return {
            "colorScheme": this._getBalancesChartColorScheme(balanceType),
            "balanceFormatFunction": this._getFormatUtils().getBalanceNumberFormatFunction(balanceType),
            "categoryAxisLabelFunction": this._getFormatUtils().getCategoryAxisLabelFunction(reportParams.firstGroupingLevel),
            "showScrollbar": true,
            "balloonConfig": {
                "dateBefore": dateFormatFunction(reportParams["dateRangeSelection"]["fromDate"]),
                "dateAfter": dateFormatFunction(reportParams["dateRangeSelection"]["toDate"]),
                "categoryFormatFunction": this._getFormatUtils().getCategoryFormatFunctionForBalloon(reportParams.firstGroupingLevel),
                "balloonBalanceLabel": messageResource.getMessage(balanceType == BalanceType.PTO ?
                                                                  "actiplans.reports.chart.pto_tooltip.balance" :
                                                                  "actiplans.reports.chart.sick_tooltip.balance")
            }
        }
    },

    _getLeaveTimeChartColorScheme: function()
    {
        return this.COLOR_SCHEMES["LEAVE_TIME"];
    },

    _getBalancesChartColorScheme: function(balanceType)
    {
        if (balanceType == BalanceType.PTO)
        {
            return this.COLOR_SCHEMES["PTO_BALANCE"];
        }
        else
        {
            return this.COLOR_SCHEMES["SICK_BALANCE"];
        }
    },

    COLOR_SCHEMES: {
        "PTO_BALANCE": {
            "balanceForFirstColumn": "#6898D3",
            "balanceForSecondColumn": "#86C087"
        },
        "SICK_BALANCE": {
            "balanceForFirstColumn": "#D97B79",
            "balanceForSecondColumn": "#E3B463"
        },
        "LEAVE_TIME": {
            "1": "#e97373",
            "2": "#82b1e7",
            "3": "#c1e873",
            "4": "#e7df72",
            "5": "#b186d6",
            "6": "#ad9c66",
            "7": "#c49b6f",
            "8": "#e36ac9",
            "9": "#646cc1",
            "10": "#d88f64",
            "11": "#adc7e8",
            "12": "#74ce39",
            "13": "#dc3030",
            "14": "#8238e5",
            "15": "#f8c909",
            "16": "#8b98a0",
            "17": "#8bdc59",
            "18": "#e84e4e",
            "19": "#a461ff",
            "20": "#ebe304"
        }
    },

    _getTotalsTableData: function()
    {
        var data = APReportsPage.LeaveTimeBalancesReportPage.Store.getReportData();
        switch (this._currentState["chartType"])
        {
            case APReportChartType.LEAVE_TIME:
                return this._createLeaveTimeTotalsTableData(data);
            case APReportChartType.PTO_BALANCE:
                return this._createBalancesTotalsTableData(data, BalanceType.PTO);
            case APReportChartType.SICK_BALANCE:
                return this._createBalancesTotalsTableData(data, BalanceType.SICK);
        }
    },

    _createLeaveTimeTotalsTableData: function(data)
    {
        if (data["totalLeaveSpent"] == 0)
        {
            //when leave time chart does not contain any data
            return {
                "headerData": {
                    "label": "",
                    "value": ""
                },
                "rowsData": []
            }
        }

        var rowsData = [];
        var firstRow = true;
        var timeFormatFunction = this._getFormatUtils().getTimeFormatFunction(this._getReportParams().timeFormat);

        _.each(data["leaveSpent"], function(datum)
        {
            var rowData = {};
            var leaveTypeId = datum.leaveTypeId;
            var leaveTypeBean = LeaveTypeStore.getLeaveType(leaveTypeId);
            rowData["iconClass"] = "leaveType" + leaveTypeBean.iconId;
            rowData["rowLabel"] = escapeHtmlSymbols(leaveTypeBean.name);
            rowData["rowValue"] = datum["leaveSpent"] == 0 ? "-" : timeFormatFunction(datum["leaveSpent"]);

            rowData["additionalClasses"] = "";
            if (!leaveTypeBean.active)
            {
                rowData["additionalClasses"] += "archived";
            }
            if (datum["leaveSpent"] == 0)
            {
                rowData["additionalClasses"] += " gray";
            }
            if (firstRow)
            {
                rowData["additionalClasses"] += " firstRow";
                firstRow = false;
            }
            rowsData.push(rowData);
        });
        return {
            "headerData": {
                "label": messageResource.getMessage("actiplans.reports.chart.total_leave_time"),
                "value": timeFormatFunction(data["totalLeaveSpent"])
            },
            "rowsData": rowsData
        };
    },

    _formatBalanceValueForTotalsTable: function(balanceObj, balanceColumnFieldName, balanceType)
    {
        return this._isBalanceNull(balanceObj, balanceColumnFieldName) ?
               "-" :
               this._getFormatUtils().getBalanceNumberFormatFunction(balanceType)(balanceObj[balanceColumnFieldName]);
    },

    _isBalanceNull: function(balanceObj, balanceColumnFieldName)
    {
        return _.isNull(balanceObj) || _.isNull(balanceObj[balanceColumnFieldName]);
    },

    _createBalancesTotalsTableData: function(data, balanceType)
    {
        var isPTOBalanceType = balanceType == BalanceType.PTO;
        var balanceObj = data[isPTOBalanceType ? "ptoBalance" : "sickBalance"];
        var dateFormatFunction = this._getFormatUtils().getDateFormatFunction();

        var valueBefore = this._formatBalanceValueForTotalsTable(balanceObj, "balanceForFirstColumn", balanceType);
        var valueAfter = this._formatBalanceValueForTotalsTable(balanceObj, "balanceForSecondColumn", balanceType);

        var reportParams = this._getReportParams();
        var rowLabelKey = isPTOBalanceType ? "actiplans.reports.chart.total_pto_as_of" : "actiplans.reports.chart.total_sick_as_of";

        var rowBalanceBefore = {
            "iconClass": isPTOBalanceType ? "ptoBefore" : "sickBefore",
            "rowLabel": messageResource.getMessage(rowLabelKey, [dateFormatFunction(reportParams["dateRangeSelection"]["fromDate"])]),
            "rowValue": valueBefore,
            "additionalClasses": "firstRow" + (!this._isBalanceNull(balanceObj, "balanceForFirstColumn") &&
                                               balanceObj["balanceForFirstColumn"] < 0 ? " negative" : "")
        };
        var rowBalanceAfter = {
            "iconClass": isPTOBalanceType ? "ptoAfter" : "sickAfter",
            "rowLabel": messageResource.getMessage(rowLabelKey, [dateFormatFunction(reportParams["dateRangeSelection"]["toDate"])]),
            "rowValue": valueAfter,
            "additionalClasses": !this._isBalanceNull(balanceObj, "balanceForSecondColumn") && balanceObj["balanceForSecondColumn"] < 0 ? "negative" : ""
        };

        var totalMessageKey = isPTOBalanceType ? "actiplans.reports.chart.total_pto" : "actiplans.reports.chart.total_sick";
        var balanceMeasureKey = (isPTOBalanceType && SystemSettings.showPTOInHours) || (!isPTOBalanceType && SystemSettings.showSickInHours) ?
                                "common.pto_units.hours" : "common.pto_units.days";

        return {
            "headerData": {
                "label": messageResource.getMessage(totalMessageKey, [messageResource.getMessage(balanceMeasureKey)]),
                "value": ""
            },
            "rowsData": [rowBalanceBefore, rowBalanceAfter]
        };
    },

    _getReportParams: function()
    {
        return APReportsPage.LeaveTimeBalancesReportPage.Store.getReportParams();
    },

    _getFormatUtils: function()
    {
        return APReportsPage.ChartController.FormatUtils;
    }
};