APReportsPage.ChartSection = function($containerElement)
{
    TemplateView.call(this, $containerElement);
};

APReportsPage.ChartSection.prototype = _.create(TemplateView.prototype, {
    html: "<div class='leaveTimeAndBalancesChartSection'>" +
          "  <div class='containerHeader'>" +
          "    <div class='collapseButton noprint'></div>" +
          "    <span class='selectorLabel'><%=viewChartLabel%></span><span class='selectorLink'><span class='selectorLinkText'></span></span>" +
          "    <span class='insteadOfSelector'></span>" +
          "    <span class='byLabel'></span>" +
          "    <div class='sortContainer'>" +
          "      <div><%=sortLabel%></div>" +
          "      <div class='sortButton alphabetically'>" +
          "        <span class='sortLink'><%=alphabetically%></span>" +
          "        <div class='sortIndicatorContainer'>" +
          "          <img class='ascSortIndicator' src='<%-ascSortImgSrc%>'/>" +
          "          <img class='descSortIndicator' src='<%-descSortImgSrc%>'/>" +
          "        </div>" +
          "      </div>" +
          "      <div class='sortButton chronologically'>" +
          "        <span class='sortLink'><%=chronologically%></span>" +
          "        <div class='sortIndicatorContainer'>" +
          "          <img class='ascSortIndicator' src='<%-ascSortImgSrc%>'/>" +
          "          <img class='descSortIndicator' src='<%-descSortImgSrc%>'/>" +
          "        </div>" +
          "      </div>" +
          "      <div>" +
          "        <span class='separator'></span>" +
          "      </div>" +
          "      <div class='sortButton value'>" +
          "        <span class='sortLink'><%=byValues%></span>" +
          "        <div class='sortIndicatorContainer'>" +
          "          <img class='ascSortIndicator' src='<%-ascSortImgSrc%>'/>" +
          "          <img class='descSortIndicator' src='<%-descSortImgSrc%>'/>" +
          "        </div>" +
          "      </div>" +
          "    </div>" +
          "  </div>" +
          "  <div class='content'>" +
          "    <div class='totalsContainer'></div>" +
          "    <div class='chartContainers'>" +
          "      <div id='leaveTimeAndBalancesChartContainer' class='chartContainer'></div>" +
          "      <div id='leaveTimeAndBalancesChartContainerHidden' class='hiddenChartContainer'></div>" +
          "    </div>" +
          "</div>",

    afterRender: function()
    {
        this.config = APReportsPage.LeaveTimeBalancesReportPage.Store.getChartSectionConfig();
        this.$selectorLink = this.find(".selectorLink");
        this.$collapseBtn = this.find(".collapseButton");
        this.$chartContainer = this.find(".chartContainer");
        this.$hiddenChartContainer = this.find(".hiddenChartContainer");
        this.$sortContainer = this.find(".sortContainer");
        this.$totalsContainer = this.find(".totalsContainer");
        this.$byLabel = this.find(".byLabel");
        this.$selectorLinkText = this.find(".selectorLinkText");
        this.$containerHeader = this.find(".containerHeader");

        this._initChartSelector(this.config["chartType"]);

        this.$collapseBtn.on("click", _.bind(this._onCollapseExpandButtonClick, this));

        this._addOnSortButtonClickCb("alphabetically",  APReportChartSortOption.BY_CATEGORY);
        this._addOnSortButtonClickCb("chronologically",  APReportChartSortOption.BY_CATEGORY);
        this._addOnSortButtonClickCb("value",  APReportChartSortOption.BY_VALUES);

        this.totalsTable = new APReportsPage.ChartSection.TotalsTable(this.$totalsContainer).init();
    },

    update: function()
    {
        var reportParams = APReportsPage.LeaveTimeBalancesReportPage.Store.getReportParams();
        var groupingLevel = reportParams.firstGroupingLevel;

        this.config = APReportsPage.LeaveTimeBalancesReportPage.Store.getChartSectionConfig();
        this._updateChartSelector(this.config["chartType"], reportParams);
        this._updateCollapseExpandButton(this.config["showChart"]);
        this._updateSortButtons(this.config["chartSortOption"], groupingLevel);
        this._updateByLabelText(groupingLevel);
        this.$containerEl.toggleClass("hidden", !this.config["showChart"]);
    },

    updateTotals: function(totalsData)
    {
        this.totalsTable.setData(totalsData);
        this.totalsTable.update();
    },
    
    templateModel: function()
    {
        return {
            ascSortImgSrc: atDefImgPath("report_configs/charts/sort_down.png"),
            descSortImgSrc: atDefImgPath("report_configs/charts/sort_up.png")
        }
    },

    messageKeys: function()
    {
        return {
            "viewChartLabel": "actiplans.reports.chart.view_label",
            "sortLabel": "actiplans.reports.chart.sort",
            "alphabetically": "actiplans.reports.chart.sort.alphabetically",
            "chronologically": "actiplans.reports.chart.sort.chronologically",
            "byValues": "actiplans.reports.chart.sort.by_values"
        };
    },

    hideChartContainer: function()
    {
        this.$sortContainer.hide();
        this.$totalsContainer.hide();
        this.$chartContainer.hide();
        this.$chartContainer.atAnimation().hideAnimated("hideAnimated");
    },

    showChartContainer: function()
    {
        this.$sortContainer.show();
        this.$totalsContainer.show();
        this.$chartContainer.show();
        this.$chartContainer.atAnimation().showAnimated("showAnimated");
    },

    updateSizesForHiddenChartContainer: function(height, width)
    {
        this.$hiddenChartContainer.css("height", height).css("width", width);
    },

    clearHiddenChartContainer: function()
    {
        this.$hiddenChartContainer.empty();
    },

    switchToNoDataState: function()
    {
        this.$chartContainer.empty();
        this.$chartContainer.append("<div class='noDataText'>" + messageResource.getMessage("actiplans.reports.chart.no_data") + "</div>");
        this.$el.addClass("noData");
    },

    switchToHasDataState: function()
    {
        this.$el.removeClass("noData");
    },

    _initChartSelector: function(chartType)
    {
        var self = this;
        this.chartTypeSelector = new APReportsPage.ChartSection.ChartTypeSelector($("body"), chartType).init();
        $(this.chartTypeSelector).on("chartTypeChanged", _.bind(this._onChartTypeChanged, this));
        this.$selectorLink.on("click", function()
        {
            self.chartTypeSelector.showAt($(this));
        });
    },

    _updateChartSelector: function(chartType, reportParams)
    {
        var chartTypesVisibilityData = {};

        chartTypesVisibilityData[APReportChartType.LEAVE_TIME] = LeavesReportUtils.reportParamsWithLeaves(reportParams);
        chartTypesVisibilityData[APReportChartType.PTO_BALANCE] = LeavesReportUtils.reportParamsWithPTO(reportParams);
        chartTypesVisibilityData[APReportChartType.SICK_BALANCE] = LeavesReportUtils.reportParamsWithSick(reportParams);

        var chartTypeMessage = LeavesReport.FormatUtils.formatChartType(chartType);

        var numberOfVisibleRows = _.reduce(chartTypesVisibilityData, function(memo, value) { return value ? memo + 1 : memo; }, 0);
        if (numberOfVisibleRows > 1)
        {
            this.chartTypeSelector.setChartTypesVisibilityDataAndCurrentChartType(chartTypesVisibilityData, chartType);
            this.chartTypeSelector.update();
            this.$selectorLinkText.text(chartTypeMessage);
        }
        else
        {
            this.find(".insteadOfSelector").text(chartTypeMessage);
        }
        this.$containerHeader.toggleClass("selectorVisible", numberOfVisibleRows > 1);
    },

    _onCollapseExpandButtonClick: function()
    {
        this.config["showChart"] = !this.config["showChart"];
        this._onChartSectionConfigChanged();
    },

    _addOnSortButtonClickCb: function(className, sortType)
    {
        var self = this;
        this.find("." + className + " .sortLink").on("click", function()
        {
            var asc = self.$sortContainer.hasClass(sortType) ?
                      self.$sortContainer.hasClass("desc") :
                      true;

            self.config["chartSortOption"] = {
                "sortType": sortType,
                "asc": asc
            };
            self._onChartSectionConfigChanged();
        });
    },

    _onChartTypeChanged: function(e, chartType)
    {
        if (this.config["chartType"] != chartType) // if chart type changed - need to show chart if hidden
        {
            this.config["showChart"] = true;
        }
        this.config["chartType"] = chartType;
        this._onChartSectionConfigChanged();
    },

    _updateCollapseExpandButton: function(showChart)
    {
        this.$collapseBtn.toggleClass("collapsed", !showChart);
    },

    _updateSortButtons: function(chartSortOption, groupingLevel)
    {
        var chronologicallySortVisible = groupingLevel == ReportGroupingLevel.DATE ||
                                         groupingLevel == ReportGroupingLevel.WEEK ||
                                         groupingLevel == ReportGroupingLevel.MONTH;
        this.$sortContainer[0].className = "sortContainer " + chartSortOption.sortType + (chartSortOption.asc ? " asc" : " desc") +
                                           (chronologicallySortVisible ? " chronologicallySortVisible" : "");
    },

    _updateByLabelText: function(groupingLevel)
    {
        var messageKey;
        switch (groupingLevel)
        {
            case ReportGroupingLevel.USER:
                messageKey = "common.Users";
                break;
            case ReportGroupingLevel.DATE:
                messageKey = "common.Days";
                break;
            case ReportGroupingLevel.WEEK:
                messageKey = "common.Weeks";
                break;
            case ReportGroupingLevel.MONTH:
                messageKey = "common.Months";
                break;
            case ReportGroupingLevel.USER_GROUP:
                messageKey = "common.User_Groups";
                break;
            case ReportGroupingLevel.TIME_ZONE_GROUP:
                messageKey = "common.Time_Zone_Groups";
                break;
        }
        this.$byLabel.text(messageResource.getMessage("actiplans.reports.chart.by_label", [messageResource.getMessage(messageKey)]));
    },

    _onChartSectionConfigChanged: function()
    {
        this.update();
        $(document).trigger("apReports.leaveTimeBalancesReport.chartConfig.change", this.config);
    }
});
