APReportsPage.ChartController.FormatUtils = {
    _balanceNumberFormatter: new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 2, true),
    _dateFormat: new DateFormat(SystemSettings.dateFormat),
    _timeNumberFormatter: new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 2, false),

    getBalanceNumberFormatFunction: function(balanceType)
    {
        var self = this;
        var showInHours = balanceType == BalanceType.PTO ? SystemSettings.showPTOInHours : SystemSettings.showSickInHours;
        return function(balance)
        {
            if (showInHours)
            {
                balance *= APReportsPage.LeaveTimeBalancesReportPage.Store.getDefaultWorkDayDuration() / 60;
            }
            return self._balanceNumberFormatter.format(balance);
        }
    },

    getDateFormatFunction: function()
    {
        var self = this;
        return function(date)
        {
            return self._dateFormat.formatDate(date);
        }
    },

    getTimeFormatFunction: function(timeFormat)
    {
        var self = this;
        if (timeFormat == TimeFormat.TIME)
        {
            return function(time)
            {
                return UserTime.formatTimeWithDigitSeparator(time);
            }
        }
        else
        {
            return function(time)
            {
                return self._timeNumberFormatter.format(time / 60);
            }
        }
    },

    getCategoryFormatFunctionForBalloon: function(groupingLevel)
    {
        var self = this;
        switch (groupingLevel)
        {
            case ReportGroupingLevel.USER:
                return function(user)
                {
                    return UserNameFormat.formatLFM(user["firstName"], user["lastName"], user["middleInitial"]);
                };
            case ReportGroupingLevel.USER_GROUP:
                return function(group)
                {
                    return group.id != USERS_WITHOUT_GROUP_ID ? group.name : messageResource.getMessage("report.user_groups.default_group_name");
                };
            case ReportGroupingLevel.TIME_ZONE_GROUP:
                return function(group)
                {
                    return group.id != DEFAULT_TIME_ZONE_GROUP_ID ? group.name : group.name + " " + messageResource.getMessage("common.default_group");
                };
            case ReportGroupingLevel.DATE:
                return function(date)
                {
                    return self._dateFormat.formatDate(date.parseJSONDate());
                };
            case ReportGroupingLevel.WEEK:
                return function(week)
                {
                    var from = week.from.parseJSONDate();
                    var to = week.to.parseJSONDate();
                    var firstPart = week.from == week.to ? "" : new DateFormat('MMM dd').formatDate(from) + " - ";
                    return firstPart + new DateFormat('MMM dd, yyyy').formatDate(to);
                };
            case ReportGroupingLevel.MONTH:
                return function(month)
                {
                    return new DateFormat('MMMM , yyyy').formatDate(month.parseJSONDate());
                };
        }
    },

    getCategoryAxisLabelFunction: function(groupingLevel)
    {
        return ReportChartUtils.getFormatCategoryFunctionForChart(groupingLevel, SystemSettings.dateFormat, false, 16, false);
    }
};