function CSVGenerator() {}
// TODO: rename CSVDataConverter
CSVGenerator.prototype = {
    init: function(usersMap, usersGroupMap, timeZoneGroupsMap)
    {
        this.usersMap = usersMap;
        this.usersGroupsMap = usersGroupMap;
        this.timeZoneGroupingMap = timeZoneGroupsMap;
        this.numberFormater = new NumberFormatter(
                // TODO: ask about separator
                SystemSettings.decimalSeparator,
                SystemSettings.digitGroupingSymbol,
                4,
                false
        );
        this.formatMonth = new DateFormat('MMMM yyyy');
        this.formatDay = new DateFormat('MMM dd, yyyy');
        this.formatWeekBegin = new DateFormat('MMM dd');
        this.formatWeekEnd = new DateFormat('MMM dd, yyyy');
        return this;
    },

    getGroupingLevels: function(config)
    {
        var first = config.firstGroupingLevel;
        var second = config.secondGroupingLevel;
        var third = config.thirdGroupingLevel;
        var fourth = config.fourthGroupingLevel;
        var grouping = [first, second, third, fourth];
        var result = [];
        for (var i = 0; i < grouping.length; ++i)
        {
            var group = grouping[i];
            if (group != null && group != ReportGroupingLevel.NONE)
            {
                result.push(group);
            }
        }
        return result;
    },

    timeGrouping: function(group)
    {
        return group == ReportGroupingLevel.DATE ||
               group == ReportGroupingLevel.WEEK ||
               group == ReportGroupingLevel.MONTH;
    },

    searchLastGroupTime: function(groups)
    {
        var result = ReportGroupingLevel.NONE;
        for (var i = 0; i < groups.length; i++)
        {
            var group = groups[i];
            if (this.timeGrouping(group))
            {
                result = group;
            }
        }
        return result;
    },

    process: function(grouping, item, line)
    {
        switch (grouping)
        {
            case "USER":
                line["Last Name"] = this.usersMap[item.id].lastName;
                line["First Name"] = this.usersMap[item.id].firstName;
                line["MI"] = this.usersMap[item.id].middleInitial;
                line["username"] = this.usersMap[item.id].username;
                break;
            case "USER_GROUP":
                line["Department"] = this.usersGroupsMap[item.id].name != null ?
                                     this.usersGroupsMap[item.id].name :
                                     messageResource.getMessage("user_groups_selector.without_department.label");
                break;
            case "TIME_ZONE_GROUP":
                line["Time Zone"] = this.timeZoneGroupingMap[item.id].name;
                break;
            case "DATE":
                var day = item.date.date;
                line["Day"] = this.formatDay.formatDate(day.parseJSONDate());
                break;
            case "WEEK":
                var from = item.dateRange.from;
                var to = item.dateRange.to;
                line["Week"] = this.formatWeekBegin.formatDate(from.parseJSONDate()) + " - " + this.formatWeekEnd.formatDate(to.parseJSONDate());
                break;
            case "MONTH":
                var month = item.date.date;
                line["Month"] = this.formatMonth.formatDate(month.parseJSONDate());
                break;
        }
    },


    processColumnNames: function(groupingList, columnsMap, columns, leaveTypes)
    {
        for (var i = 0; i < groupingList.length; ++i)
        {
            var grouping = groupingList[i];
            switch (grouping)
            {
                case ReportGroupingLevel.USER:
                    columnsMap["Last Name"] = "Last Name";
                    columnsMap["First Name"] = "First Name";
                    columnsMap["MI"] = "MI";
                    columnsMap["username"] = "username";
                    columns.push("Last Name");
                    columns.push("First Name");
                    columns.push("MI");
                    columns.push("username");
                    break;
                case ReportGroupingLevel.USER_GROUP:
                    columnsMap["Department"] = "Department";
                    columns.push("Department");
                    break;
                case ReportGroupingLevel.TIME_ZONE_GROUP:
                    columnsMap["Time Zone"] = "Time Zone";
                    columns.push("Time Zone");
                    break;
                case ReportGroupingLevel.DATE:
                    columnsMap["Day"] = "Day";
                    columns.push("Day");
                    break;
                case ReportGroupingLevel.WEEK:
                    columnsMap["Week"] = "Week";
                    columns.push("Week");
                    break;
                case ReportGroupingLevel.MONTH:
                    columnsMap["Month"] = "Month";
                    columns.push("Month");
                    break;
            }
        }
        if (this.showLeaveTime)
        {
            columns.push("Total");
            columnsMap["Total"] = "Total Leave Time";
            for (var j = 0; j < leaveTypes.length; ++j)
            {
                var columnName = this.getLeaveTypeColumnName(leaveTypes[j]);
                columnsMap[columnName] = LeaveTypeStore.getLeaveType(leaveTypes[j].leaveTypeId).name;
                columns.push(columnName);
            }
        }
        if (this.showBalances)
        {
            if (this.pto)
            {
                columns.push("PTO begin");
                columnsMap["PTO begin"] = "PTO Balance as of " + this.balanceColumnNameBegin();
                columns.push("PTO end");
                columnsMap["PTO end"] = "PTO Balance as of " + this.balanceColumnNameEnd();
            }
            if (this.sick)
            {
                columns.push("Sick begin");
                columnsMap["Sick begin"] = "Sick Balance as of " + this.balanceColumnNameBegin();
                columns.push("Sick end");
                columnsMap["Sick end"] = "Sick Balance as of " + this.balanceColumnNameEnd();
            }
        }
    },

    getLeaveTypeColumnName: function(leaveType)
    {
        return "leaveType" + leaveType.leaveTypeId.toString();
    },

    balanceColumnNameBegin: function()
    {
        var timeGrouping = this.searchLastGroupTime(this.groupingLevels);
        var postfix = "";
        var dateRange = this.dateRange;
        switch (timeGrouping)
        {
            case ReportGroupingLevel.NONE:
                postfix = this.formatDay.formatDate(dateRange.fromDate) + " (days)";
                break;
            case ReportGroupingLevel.DATE:
                postfix = " the start of the day (days)";
                break;
            case ReportGroupingLevel.WEEK:
                postfix = " the first day of week (days)";
                break;
            case  ReportGroupingLevel.MONTH:
                postfix = " the first day of month (days)";
                break;
        }
        return postfix;
    },

    balanceColumnNameEnd: function()
    {
        var timeGrouping = this.searchLastGroupTime(this.groupingLevels);
        var postfix = "";
        var dateRange = this.dateRange;
        switch (timeGrouping)
        {
            case ReportGroupingLevel.NONE:
                postfix = this.formatDay.formatDate(dateRange.toDate) + " (days)";
                break;
            case ReportGroupingLevel.DATE:
                postfix = " the end of the day (days)";
                break;
            case ReportGroupingLevel.WEEK:
                postfix = " the last day of week (days)";
                break;
            case ReportGroupingLevel.MONTH:
                postfix = " the last day of month (days)";
                break;
        }
        return postfix;
    },

    formatLeaveTime: function(time)
    {
        return this.numberFormater.format(time / 60);
    },

    generate: function(layer, depth, data, line)
    {
        this.process(this.groupingLevels[layer], data, line);

        if (layer + 1 == depth)
        {
            if (this.showLeaveTime)
            {
                line["Total"] = this.formatLeaveTime(data.totalLeaveSpent);
                var leaveSpends = data.leaveSpent;
                for (var l = 0; l < leaveSpends.length; ++l)
                {
                    var columnName = this.getLeaveTypeColumnName(leaveSpends[l]);
                    line[columnName] = this.formatLeaveTime(leaveSpends[l].leaveSpent);
                }
            }
            if (this.showBalances)
            {
                if (this.pto)
                {
                    var pto = data.ptoBalance;
                    line["PTO begin"] = (pto != null && pto.balanceForFirstColumn != null) ?
                                        this.numberFormater.format(pto.balanceForFirstColumn) :
                                        "";
                    line["PTO end"] = (pto != null && pto.balanceForSecondColumn != null) ?
                                      this.numberFormater.format(pto.balanceForSecondColumn) :
                                      "";
                }
                if (this.sick)
                {
                    var sick = data.sickBalance;
                    line["Sick begin"] = (sick != null && sick.balanceForFirstColumn != null) ?
                                         this.numberFormater.format(sick.balanceForFirstColumn) :
                                         "";
                    line["Sick end"] = (sick != null && sick.balanceForSecondColumn != null) ?
                                       this.numberFormater.format(sick.balanceForSecondColumn) :
                                       "";
                }
            }
            this.lines.push(_.clone(line));
            return;
        }

        for (var i = 0; i < data.items.length; ++i)
        {
            this.generate(layer + 1, depth, data.items[i], line);
        }
    },

    initCSVConfiguration: function(params)
    {
        this.showBalances = (params.columnsConfig == LeaveTypeColumnConfig.LEAVE_TIME_AND_BALANCES ||
                             params.columnsConfig == LeaveTypeColumnConfig.BALANCES_ONLY);
        this.showLeaveTime = (params.columnsConfig == LeaveTypeColumnConfig.LEAVE_TIME_AND_BALANCES ||
                              params.columnsConfig == LeaveTypeColumnConfig.LEAVE_TIME_ONLY);

        this.pto = (params.balanceQualifier == LeaveTypeBalanceQualifier.PTO_AND_SICK_DAYS ||
                    params.balanceQualifier == LeaveTypeBalanceQualifier.PTO_ONLY);
        this.sick = (params.balanceQualifier == LeaveTypeBalanceQualifier.PTO_AND_SICK_DAYS ||
                     params.balanceQualifier == LeaveTypeBalanceQualifier.SICK_DAYS_ONLY);
    },

    generateTitle: function(params, data)
    {

        return {
            columns: ["title", "description"],
            table: [
                {title: "Date Range:", description: "From A to B"},
                {title: "Staff:", description: "All staff"},
                {title: "Leave Types:", description: "All leave types"},
                {title: "", description: ""}] // empty line
        };
    },
    generateCSV: function(params, data)
    {
        this.groupingLevels = this.getGroupingLevels(params);
        this.initCSVConfiguration(params);
        this.dateRange = params.dateRangeSelection;
        var columnNames = {};
        var columns = [];
        this.processColumnNames(this.groupingLevels, columnNames, columns, data.leaveSpent);

        this.lines = [];
        for (var i = 0; i < data.items.length; ++i)
            this.generate(0, this.groupingLevels.length, data.items[i], {});

        return {
            columnNames: columnNames,
            columns: columns,
            table: this.lines
        };
    }
};
