function LeaveRequestCellRenderer()
{
    LeaveTableDayCellRenderer.call(this);
}

LeaveRequestCellRenderer.prototype = _.create(LeaveTableDayCellRenderer.prototype, {
    setLeaveTypeNotFoundCallback: function(leaveTypeNotFoundCallback)
    {
        LeaveRequestCellRenderer.prototype.leaveTypeNotFoundCallback = leaveTypeNotFoundCallback;
    },

    checkIfLeaveTypeNotFound: function(cell)
    {
        if (cell.getValue() != null && cell.getValue().requestData)
        {
            var leaveTypeBean = LEAVE_TYPES[cell.getValue().requestData.leaveTypeId];
            if (!leaveTypeBean)
            {
                cell.setValue(null);
                if (this.leaveTypeNotFoundCallback)
                {
                    this.leaveTypeNotFoundCallback();
                }
            }
        }
    },

    generateContent: function(cell)
    {
        this.checkIfLeaveTypeNotFound(cell);
        LeaveTableDayCellRenderer.prototype.generateContent.call(this, cell);
        this.addTooltip(cell);
    },

    generateContentForPrinting: function(cell)
    {
        if (cell.getValue() != null)
        {

            if(cell.getValue().requestData)
            {
                var printLeaveDiv = document.createElement("div");
                cell.printLeaveDiv = printLeaveDiv;
                printLeaveDiv.className = "for-print-only";
                var img = document.createElement("img");
                img.className = "printLeaveImage";
                cell.printImgElement = img;
                printLeaveDiv.appendChild(img);
                var img2 = document.createElement("img");
                img2.className = "printLeaveLevelImage";
                cell.printImgLevelElement = img2;
                printLeaveDiv.appendChild(img2);
                cell.getCellEl().appendChild(printLeaveDiv);
                this.updateLeaveInformation(cell);
            }

            if( cell.getValue().statusData){
                var printStatusDiv = document.createElement("div");
                cell.printStatusDiv = printStatusDiv;
                printStatusDiv.className = "for-print-only";
                var imgStatus = document.createElement("img");
                imgStatus.className = "printStatusImage";
                imgStatus.src = atDefImgPath("leave_types/chart_icons/leave-status.png");
                imgStatus.style.visibility = "visible";
                printStatusDiv.appendChild(imgStatus);
                cell.getCellEl().appendChild(printStatusDiv);
            }
        }
    },

    updateLeaveInformation: function(cell)
    {
        if (cell.getValue() != null && cell.getValue().requestData)
        {
            var leaveTime = cell.getValue().leavePercent;

            var requestData = cell.getValue().requestData;
            if (requestData.status == LeaveTimeRequestStatus.APPROVED && !requestData.requestChange)
            {
                if (leaveTime < 0.25)
                    cell.printImgLevelElement.src = atImgPath("leave_types/chart_icons/ten_percent_leave.gif");
                else if (leaveTime < 0.5)
                    cell.printImgLevelElement.src = atImgPath("leave_types/chart_icons/twentyfive_percent_leave.gif");
                else if (leaveTime < 0.75)
                    cell.printImgLevelElement.src = atImgPath("leave_types/chart_icons/half_percent_leave.gif");
                else if (leaveTime < 1)
                    cell.printImgLevelElement.src = atImgPath("leave_types/chart_icons/seventyfive_percent_leave.gif");
                else
                    cell.printImgLevelElement.src = atImgPath("leave_types/chart_icons/full_day_leave.gif");
            }
            else
            {
                if (leaveTime < 0.25)
                    cell.printImgLevelElement.src = atImgPath("leave_types/chart_icons/ten_percent_leave_requested.gif");
                else if (leaveTime < 0.5)
                    cell.printImgLevelElement.src = atImgPath("leave_types/chart_icons/twentyfive_percent_leave_requested.gif");
                else if (leaveTime < 0.75)
                    cell.printImgLevelElement.src = atImgPath("leave_types/chart_icons/half_percent_leave_requested.gif");
                else if (leaveTime < 1)
                    cell.printImgLevelElement.src = atImgPath("leave_types/chart_icons/seventyfive_percent_leave_requested.gif");
                else
                    cell.printImgLevelElement.src = atImgPath("leave_types/chart_icons/full_day_leave_requested.gif");
            }

            var leaveTypeBean = LEAVE_TYPES[cell.getValue().requestData.leaveTypeId];
            var printImg = cell.printImgElement;
            printImg.src = leaveTypeBean.getPrintIconPath();
            printImg.style.visibility = "visible";
        }
    },

    getCellClassName: function(cell)
    {
        var className = LeaveTableDayCellRenderer.prototype.getCellClassName.call(this, cell);

        if (cell.getValue() != null)
        {
            var requestData = cell.getValue().requestData;
            var statusData = cell.getValue().statusData;

            if (requestData)
            {
                var leaveTypeId = new LeaveModificationsUtil().getActualLeaveTypeId(requestData);
                var leaveTypeBean = LEAVE_TYPES[leaveTypeId];
                className.push("leaveType" + leaveTypeBean.iconId);

                if (requestData.waitingDeletion)
                    className.push("waitingDeletion");
                if (requestData.status == LeaveTimeRequestStatus.APPROVED && !requestData.requestChange)
                    className.push("approved");

                var leaveTime = cell.getValue().leavePercent;
                if (leaveTime < 0.25)
                    className.push("tenPercentLeave");
                else if (leaveTime < 0.5)
                    className.push("twentyFivePercentLeave");
                else if (leaveTime < 0.75)
                    className.push("fiftyPercentLeave");
                else if (leaveTime < 1)
                    className.push("seventyFivePercentLeave");
                else
                    className.push("fullLeave");
            }
            else if (statusData)
            {
                // TODO: PRO-256 do something with this class.
                className.push("leaveStatus");
            }
            else
                className.push("noLeave");
        }
        else
            className.push("noLeave");

        return className;
    },

    appendHoverClassesIfNeeded: function(cell, classesArray)
    {
        if (this.isHovered(cell))
        {
            classesArray.push("hover");
        }
        else if (cell.getValue() != null && cell.getValue().requestData)
        {
            var hoverRegion = this.getHoverController(cell).getHoverRegion();
            if (hoverRegion == NullTableRegion.instance)
                return;

            var hoveredCell = cell.getRow().getTable().getCell(hoverRegion.rowMin, hoverRegion.colMin);
            if (hoveredCell.getValue() == null)
                return;

            if (hoveredCell.getValue().requestData && hoveredCell.getValue().requestData.id == cell.getValue().requestData.id)
                classesArray.push("requestHover");
        }
    },

    addTooltip: function(cell)
    {
        /*
         cell.getRow().getTable().getTooltipController().addDayTooltip( cell.getCellElem(),
         cell.getRow().getCell(0).getValue().id,
         cell.getRow().getTable().getDatesRow().getCell( cell.getColIdx() ).getValue().date,
         cell.getValue() );
         */
    }
});

