/**
 *
 * @param containerEl Container element in DOM
 * @param dateFrom
 * @param dateTo
 * @param corporateScheduleSlice ArraySlice object, where corporateScheduleSlice.get(0) matches to dateFrom day
 * @param today
 * @param monthsRowEl
 * @param weeksRowEl
 * @param headerRowEl added to tbody
 * @param ptoHoursPerDay used to convert days to hours when displaying PTO balance
 * @param canEditPastRequests Indicates if editing past requests is allowed
 * @param wasAtIntegrated Indicates if actiTIME was ever integrated
 * @param todayBalance balance for today
 * @constructor
 */
function LeaveTable(containerEl,
                    dateFrom,
                    dateTo,
                    corporateScheduleSlice,
                    today,
                    monthsRowEl,
                    weeksRowEl,
                    headerRowEl,
                    ptoHoursPerDay,
                    canEditPastRequests,
                    wasAtIntegrated,
                    todayBalance,
                    isCurrentUserActiveInAT)
{
    this.dateFrom = dateFrom;
    this.dateTo = dateTo;
    this.monthsRowEl = monthsRowEl;
    this.weeksRowEl = weeksRowEl;
    this.headerRowEl = headerRowEl;
    this.corporateScheduleSlice = corporateScheduleSlice;
    this.today = today;
    this.dayHeaderRow = null;
    this.ptoHoursPerDay = ptoHoursPerDay;
    this.canEditPastRequests = canEditPastRequests;
    this.wasAtIntegrated = wasAtIntegrated;
    this.todayBalance = todayBalance;
    this.isCurrentUserActiveInAT = isCurrentUserActiveInAT;

    Table.call(this, null, containerEl);
    this.tableEl.className = "leaveTable";

    if (dateFrom)
    {
        this.dayHeaderRow = this.createDayHeaderRow();
        this.addRow(this.dayHeaderRow);

        this.createModelRows();

        this.createBottomRow();

        this.progressDiv = document.createElement("div");
        this.progressDiv.className = "leaveTable progressDiv";
    }

    this.preloadImagesForPrinting();
}

LeaveTable.prototype = _.create(Table.prototype, {
    clickable: true,

    addNewHoverListener: function(listener)
    {
        this.mouseEventsHandler.addNewHoverListener(listener);
    },

    addNewRequestSelectionListener: function(listener)
    {
        this.mouseEventsHandler.addNewRequestSelectionListener(listener);
    },

    addExistingRequestSelectionListener: function(listener)
    {
        this.mouseEventsHandler.addExistingRequestSelectionListener(listener);
    },

    isActiveInAT : function(){
        return this.isCurrentUserActiveInAT;
    },

    addNotApplicableCellSelectionListener: function(listener)
    {
        this.mouseEventsHandler.addNotApplicableCellSelectionListener(listener);
    },

    showProgress: function(tableRegion)
    {
        var fromCell = this.getRow(tableRegion.rowMin).getCell(tableRegion.colMin);
        var toCell = this.getRow(tableRegion.rowMax).getCell(tableRegion.colMax);

        var fromCellJQ = $(fromCell.getCellEl());
        var toCellJQ = $(toCell.getCellEl());

        var fromCellOffset = fromCellJQ.offset();
        var toCellOffset = toCellJQ.offset();

        this.progressDiv.style.left = fromCellOffset.left + "px";
        this.progressDiv.style.top = fromCellOffset.top + "px";
        this.progressDiv.style.width = (toCellOffset.left - fromCellOffset.left + toCellJQ.width()) + "px";
        this.progressDiv.style.height = (toCellOffset.top - fromCellOffset.top + toCellJQ.height()) + "px";

        document.body.appendChild(this.progressDiv);
        this.progressDiv.style.display = "block";
    },

    hideProgress: function()
    {
        this.progressDiv.style.display = "none";
        try
        { document.body.removeChild(this.progressDiv); }
        catch (ignore)
        {}
    },

    setProgressVisibility: function(isVisible)
    {
        this.progressDiv.style.visibility = isVisible ? "visible" : "hidden";
    },

    disableUserActivities: function()
    {
        if (this.mouseEventsHandler.isEnabled())
        {
            this.mouseEventsHandler.disable();
        }
    },

    enableUserActivities: function()
    {
        this.mouseEventsHandler.enable();
    },


    clearSelection: function()
    {
        this.selectionController.clearSelection();
        this.mouseEventsHandler.setHoverState();
    },

    getDateRangeForRegion: function(tableRegion)
    {
        return {
            dateFrom: this.getDayHeaderRow().getCell(tableRegion.colMin).getValue().date,
            dateTo: this.getDayHeaderRow().getCell(tableRegion.colMax).getValue().date
        };
    },

    getRowByUserId: function(userId)
    {
        return null;
    },

    createMouseEventsHandler: function()
    {
        this.tooltipController = new LeaveTableTooltipController(this.createTooltip());
        return new LeaveTableMouseEventsHandler(this, this.selectionController, this.hoverController, this.tooltipController);
    },

    createTooltip: function()
    {
        return new LeaveTableTooltip(this.ptoHoursPerDay).init();
    },

    hideTooltip: function()
    {
        this.tooltipController.hideTooltip();
    },

    getDateForCell: function(cell)
    {
        var dayHeaderCell = this.dayHeaderRow.getCell(cell.colIdx);
        return dayHeaderCell.getValue().date;
    },

    isPastDateCell: function(cell)
    {
        return this.getDateForCell(cell) < this.today;
    },

    isTodayCell: function(cell)
    {
        return this.getDateForCell(cell) - this.today == 0;
    },

    setMonthCellContent: function(cell, date)
    {
        var format = " ";
        if (cell.colSpan > 10)
            format = "F Y";
        else if (cell.colSpan > 9)
            format = "F 'y";
        else if (cell.colSpan > 5)
            format = "M Y";
        else if (cell.colSpan > 3)
            format = "M 'y";
        else if (cell.colSpan > 1)
            format = "M";

        $(cell).append($("<div>").addClass("monthCellTitle").html(date.format(format)));
    },


    buildMonthsRowDOM: function(rowEl)
    {
        // TODO: PRO-253 Make this work under jQuery control and compare the performance

        var i = this.createDatesIterator();
        var m = -1;
        var cell = null;
        var pastDaysRow = null;
        var firstInMonth = true;
        var first = true;
        var prevDate = null;
        var nextDate = i.next();

        while (nextDate <= this.dateTo)
        {
            if (m != nextDate.getMonth())
            {
                if (cell && prevDate)
                    this.setMonthCellContent(cell, prevDate);

                cell = document.createElement("td");
                cell.colSpan = 1;
                cell.className = "monthCell";
                if (first)
                    cell.className += " first";
                first = false;
                if (nextDate < this.today)
                {
                    cell.className += " past";

                    var pastDaysTable = $("<table>").attr("cellspacing", 0).attr("cellpadding", 0).addClass("pastDaysTable");
                    pastDaysRow = $("<tr>");
                    pastDaysTable.append(pastDaysRow);
                    $(cell).append(pastDaysTable);
                }
                else
                {
                    pastDaysRow = null;
                }

                firstInMonth = true;

                rowEl.appendChild(cell);
                m = nextDate.getMonth();
            }
            else
            {
                cell.colSpan++;
            }

            if (pastDaysRow)
            {
                var td = $("<td>");
                if (nextDate < this.today)
                    td.addClass("past");
                else if (nextDate.getTime() == this.today.getTime())
                    td.addClass("today");

                if (firstInMonth)
                {
                    td.addClass("first");
                    firstInMonth = false;
                }

                pastDaysRow.append(td);
            }

            prevDate = nextDate;
            nextDate = i.next();
        }

        this.setMonthCellContent(cell, prevDate);
        cell.className += " last";
    },

    buildWeeksRowDOM: function(rowEl)
    {
        var startingDay = this.dateFrom.getDay();
        var c = startingDay - SystemSettings.weekStartDay + 1;
        if (c < 0) c += 7;

        var cell = null;
        var firstCell = null;
        var lastCell = null;
        var divStart = null;
        var divEnd = null;
        var pastDaysRow = null;
        var firstInWeek = true;

        for (var i = this.createDatesIterator(); i.hasNext();)
        {
            var nextDate = i.next();
            if (cell == null || c == 0)
            {
                if (cell == null)
                {
                    firstCell = document.createElement("td");
                    cell = firstCell
                }
                else
                {
                    cell = document.createElement("td");
                }
                cell.className = "weekCell";
                cell.colSpan = 1;
                divStart = document.createElement("div");
                divStart.className = "startDate";
                divStart.innerHTML = nextDate.getDate();
                divEnd = document.createElement("div");
                divEnd.className = "endDate";

                if (nextDate < this.today)
                {
                    cell.className += " past";

                    var pastDaysTable = $("<table>").attr("cellspacing", 0).attr("cellpadding", 0).addClass("pastDaysTable");
                    pastDaysRow = $("<tr>");
                    pastDaysTable.append(pastDaysRow);
                    $(cell).append(pastDaysTable);
                }
                else
                {
                    pastDaysRow = null;
                }

                firstInWeek = true;

                cell.appendChild(divStart);
                cell.appendChild(divEnd);
                rowEl.appendChild(cell);
                lastCell = cell;
            }
            else
            {
                cell.colSpan++;
            }

            if (pastDaysRow)
            {
                var td = $("<td>");
                if (nextDate < this.today)
                    td.addClass("past");
                else if (nextDate.getTime() == this.today.getTime())
                    td.addClass("today");

                if (firstInWeek)
                {
                    td.addClass("first");
                    firstInWeek = false;
                }

                pastDaysRow.append(td);
            }

            divEnd.innerHTML = nextDate.getDate();
            c = (c == 6) ? 0 : c + 1;
        }

        firstCell.className += " first";

        if (firstCell.colSpan < 4)
            firstCell.firstChild.style.display = "none";

        if (firstCell.colSpan == 1)
        {
            firstCell.firstChild.nextSibling.style.fontSize = "8px";
            firstCell.firstChild.nextSibling.style.paddingLeft = "0px";
            firstCell.firstChild.nextSibling.style.paddingRight = "2px";
        }

        lastCell.className += " last";

        if (lastCell.colSpan < 4)
            lastCell.firstChild.nextSibling.style.display = "none";

        if (lastCell.colSpan == 1)
        {
            lastCell.firstChild.style.fontSize = "8px";
            lastCell.firstChild.style.paddingRight = "0px";
            lastCell.firstChild.style.paddingLeft = "1px";
        }
    },


    buildDOM: function(containerEl)
    {
        if (this.headerRowEl)
        {
            this.tbody.appendChild(this.headerRowEl);
        }

        this.tbody.appendChild(this.monthsRowEl);
        this.buildMonthsRowDOM(this.monthsRowEl);

        this.tbody.appendChild(this.weeksRowEl);
        this.buildWeeksRowDOM(this.weeksRowEl);

        Table.prototype.buildDOM.call(this, containerEl);
    },

    getBaseRowClasses: function(rowIdx)
    {
        var rowElClasses = [];

        if (rowIdx == 0)
        {
            rowElClasses.push("dayHeaderRow");
        }
        else
        {
            if (rowIdx == 1)
                rowElClasses.push("firstRow");
            if (rowIdx == this.getLastRow().getRowIdx())
                rowElClasses.push("lastRow");
        }

        rowElClasses.push(this.getRow(rowIdx).getRowClasses());

        return rowElClasses;
    },

    createDatesIterator: function()
    {
        //noinspection JSUnresolvedFunction
        return new DatesIterator(this.dateFrom, this.dateTo);
    },

    createDayHeaderRow: function()
    {
        var weekDaysRow = new DayHeaderRow();
        var d = 0;

        for (var i = this.createDatesIterator(); i.hasNext();)
        {
            var nextDate = i.next();
            weekDaysRow.addCell({date: nextDate, working: this.corporateScheduleSlice.get(d) > 0});
            d++;
        }

        return weekDaysRow;
    },

    getCorporateScheduleSlice: function()
    {
        return this.corporateScheduleSlice;
    },

    hoverChanged: function(hoverChangeEvent, mouseEvent)
    {
        var updateRegion = this.computeChangedRegion(hoverChangeEvent);
        var oldRequestRegion = this.getRequestRegionByHover(hoverChangeEvent.oldValue);
        var newRequestRegion = this.getRequestRegionByHover(hoverChangeEvent.newValue);
        this.updateCellRegionUI(updateRegion.union(oldRequestRegion).union(newRequestRegion));
    },

    getRequestRegionForCell: function(cell)
    {
        var value = cell.getValue();
        if (value == null || !value.requestData)
            return NullTableRegion.instance;

        var row = cell.getRow().getRowIdx();
        var col = cell.getColIdx();
        var date = this.getDateForCell(cell);

        var firstDate = value.requestData.firstDate.parseJSONDate();

        var numberOfCellsOnTheLeft = date.offsetInDaysFrom(firstDate);
        var numberOfCellsOnTheRight = value.requestData.lastDate.parseJSONDate().offsetInDaysFrom(date);

        return new TableRegion(
                row,
                Math.max(0, col - numberOfCellsOnTheLeft),
                row,
                Math.min(this.getRow(row).getLastCell().getColIdx(), col + numberOfCellsOnTheRight)
        );
    },

    getRequestRegionByHover: function(hoverRegion)
    {
        if (hoverRegion == NullTableRegion.instance)
            return NullTableRegion.instance;

        if (hoverRegion.rowMin == this.dayHeaderRow.getRowIdx())
            return NullTableRegion.instance;

        return this.getRequestRegionForCell(this.getCell(hoverRegion.rowMin, hoverRegion.colMin));
    },

    selectionChanged: function(selectionChangeEvent)
    {
        var updateRegion = this.computeChangedRegion(selectionChangeEvent);
        this.updateSelectionChangeRegionUI(updateRegion);
        this.updateDayHeaderRowCellsUI(updateRegion);
    },

    updateDayHeaderRowCellsUI: function(updateRegion)
    {
        var dayHeaderRowIdx = this.dayHeaderRow.getRowIdx();
        this.updateCellRegionUI(new TableRegion(dayHeaderRowIdx, updateRegion.colMin, dayHeaderRowIdx, updateRegion.colMax));
    },

    createModelRows: function()
    {
        // abstract
    },

    getDayHeaderRow: function()
    {
        return this.dayHeaderRow;
    },

    getBottomRowHeaderCellsNumber: function()
    {
        return 0;
    },

    createBottomRow: function()
    {
        // Bottom row is dummy, which is needed mostly for having its top border displayed. This border is then used for displaying colored frames etc.

        var bottomRowEl = document.createElement("tr");
        bottomRowEl.className = "bottomRow";
        var bottomRowHeaderCellsNumber = this.getBottomRowHeaderCellsNumber();
        for (var i = 0; i < bottomRowHeaderCellsNumber; i++)
        {
            var bottomTd = document.createElement("td");
            bottomTd.className = "bottomHeaderCell" + ((i + 1 == bottomRowHeaderCellsNumber) ? " lastBottomHeaderCell" : "");
            bottomRowEl.appendChild(bottomTd);
        }
        var bottomRow = new LeaveTableBottomRow(bottomRowEl);

        for (var d = this.createDatesIterator(); d.hasNext(); d.next())
            bottomRow.addCell("");
        this.addRow(bottomRow);
    },

    getToday: function()
    {
        return this.today;
    },

    preloadImagesForPrinting: function()
    {
        var images = [atImgPath("leave_types/chart_icons/ten_percent_leave.gif"),
                      atImgPath("leave_types/chart_icons/twentyfive_percent_leave.gif"),
                      atImgPath("leave_types/chart_icons/half_percent_leave.gif"),
                      atImgPath("leave_types/chart_icons/seventyfive_percent_leave.gif"),
                      atImgPath("leave_types/chart_icons/full_day_leave.gif"),
                      atImgPath("leave_types/chart_icons/ten_percent_leave_requested.gif"),
                      atImgPath("leave_types/chart_icons/twentyfive_percent_leave_requested.gif"),
                      atImgPath("leave_types/chart_icons/half_percent_leave_requested.gif"),
                      atImgPath("leave_types/chart_icons/seventyfive_percent_leave_requested.gif"),
                      atImgPath("leave_types/chart_icons/full_day_leave_requested.gif"),
                      atDefImgPath("leave_types/chart_icons/leave-status.png")];
        var preloader = new ImagePreloader(images);
    },

    preDestroy: function()
    {
        this.tooltipController.removeTooltip();
    }
});
