function LeaveTableMouseEventsHandler( table, selectionController, hoverController, tooltipController )
{
    TableMouseEventsHandler.apply(this, arguments);

    this.tooltipController = tooltipController;

    this.hoverState = new HoverState( this );
    this.cellSelectionState = new CellSelectionState( this );
    this.leaveRequestSelectedState = new LeaveRequestSelectedState( this );
    this.rangeSelectedState = new RangeSelectedState( this );
    this.notApplicableCellSelectionState = new NotApplicableCellSelectionState( this );
    this.colleaguesHoverState = new ColleaguesHoverState(this);

    this.state = this.hoverState;
    this.beforeDisableState = null;

    //should handle mouseUp which occurs beyond table borders
    var self = this;
    var prevMouseUp = document.onmouseup;
    $( document ).mouseup( function ( evt )
                           {
                               if( prevMouseUp )
                                   prevMouseUp.call( this, evt );
                               self.mouseUp( evt, null );
                           } );

    this.newHoverListeners = [];
    this.newRequestSelectionListeners = [];
    this.exisingRequestSelectionListeners = [];
    this.notApplicableCellSelectionListeners = [];
}

LeaveTableMouseEventsHandler.prototype = _.create(TableMouseEventsHandler.prototype);

LeaveTableMouseEventsHandler.prototype.checkMouseUpHook = function(evt) {
    return this.mouseUpHook?this.mouseUpHook(evt):true;
};

LeaveTableMouseEventsHandler.prototype.contextMenu = function( evt, cell) { this.state.mouseDown(evt, cell); this.state.mouseUp( evt, cell);};
LeaveTableMouseEventsHandler.prototype.mouseOver = function( evt, cell ) { this.state.mouseOver(evt, cell); };
LeaveTableMouseEventsHandler.prototype.mouseOut = function( evt, cell )  { this.state.mouseOut(evt, cell);  };
LeaveTableMouseEventsHandler.prototype.mouseMove = function( evt, cell ) { this.state.mouseMove(evt, cell); };
LeaveTableMouseEventsHandler.prototype.mouseDown = function( evt, cell ) { if( this.isLeftButton( evt ) ) this.state.mouseDown( evt, cell ); };
LeaveTableMouseEventsHandler.prototype.mouseUp = function( evt, cell )
{
    if(this.checkMouseUpHook(evt))
    {
        if (this.isLeftButton(evt))
            this.state.mouseUp(evt, cell);
        if (evt.stopPropagation)
            evt.stopPropagation();
        if (evt.cancelBubble != null)
            evt.cancelBubble = true;
    }
};

LeaveTableMouseEventsHandler.prototype.addNewHoverListener = function(listener)
{
    this.newHoverListeners.push(listener);
};

LeaveTableMouseEventsHandler.prototype.addNewRequestSelectionListener = function(listener)
{
    this.newRequestSelectionListeners.push(listener);
};

LeaveTableMouseEventsHandler.prototype.addExistingRequestSelectionListener = function(listener)
{
    this.exisingRequestSelectionListeners.push(listener);
};

LeaveTableMouseEventsHandler.prototype.addNotApplicableCellSelectionListener = function(listener)
{
    this.notApplicableCellSelectionListeners.push(listener);
};

LeaveTableMouseEventsHandler.prototype.fireHoverSelected = function()
{
    for( var i = 0; i < this.newHoverListeners.length; i++ )
        this.newHoverListeners[i].call();
};

LeaveTableMouseEventsHandler.prototype.fireNewRequestSelected = function()
{
    for( var i = 0; i < this.newRequestSelectionListeners.length; i++ )
        this.newRequestSelectionListeners[i].call();
};

LeaveTableMouseEventsHandler.prototype.fireExistingRequestSelected = function()
{
    for( var i = 0; i < this.exisingRequestSelectionListeners.length; i++ )
        this.exisingRequestSelectionListeners[i].call();
};

LeaveTableMouseEventsHandler.prototype.fireNotApplicableCellSelectionListeners = function()
{
    for( var i = 0; i < this.notApplicableCellSelectionListeners.length; i++ )
        this.notApplicableCellSelectionListeners[i].call();
};

LeaveTableMouseEventsHandler.prototype.isLeftButton = function( evt )
{
    return whichButtonPressed( getEventObj( evt ) ) == "left";
};

LeaveTableMouseEventsHandler.prototype.setCellSelectionState = function( anchorCell, mouseEvent )
{
    this.getSelectionController().changeSelectionTo( anchorCell );
    this.state = this.cellSelectionState;
    disableTextSelection(document.documentElement);
};

LeaveTableMouseEventsHandler.prototype.setLeaveRequestSelectedState = function( cell, mouseEvent )
{
    var table = cell.getRow().getTable();
    var requestRegion = table.getRequestRegionForCell( cell );
    this.getSelectionController().changeSelection(
        table.getCell( requestRegion.rowMin, requestRegion.colMin ),
        table.getCell( requestRegion.rowMax, requestRegion.colMax )
    );
    this.state = this.leaveRequestSelectedState;
    this.fireExistingRequestSelected();
};

LeaveTableMouseEventsHandler.prototype.setRangeSelectedState = function(mouseEvent)
{
    enableTextSelectionBack(document.documentElement);
    this.state = this.rangeSelectedState;
    this.fireNewRequestSelected();
};

LeaveTableMouseEventsHandler.prototype.setNotApplicableCellSelectedState = function( anchorCell, mouseEvent )
{
    this.getSelectionController().changeSelectionTo( anchorCell );
    this.state = this.notApplicableCellSelectionState;
    disableTextSelection(document.documentElement);
    this.fireNotApplicableCellSelectionListeners();
};

LeaveTableMouseEventsHandler.prototype.setHoverState = function(cell, mouseEvent)
{
    enableTextSelectionBack(document.documentElement);
    if (this.state != this.hoverState) {
        this.state = this.hoverState;
        this.fireHoverSelected();
    }
};

LeaveTableMouseEventsHandler.prototype.setColleaguesHoverState = function()
{
    enableTextSelectionBack(document.documentElement);
    this.state = this.colleaguesHoverState;
};

LeaveTableMouseEventsHandler.prototype.disable = function()
{
    this.beforeDisableState = this.state;
    this.state = new DisabledState( this );
};

LeaveTableMouseEventsHandler.prototype.enable = function()
{
    if(this.beforeDisableState)
        this.state = this.beforeDisableState;
    this.beforeDisableState = null;
};

LeaveTableMouseEventsHandler.prototype.isEnabled = function()
{
    return this.beforeDisableState == null;
};


// ---------------------------------------------------------------------------------------------------------------------

function TableMouseEventsHandlerState( mouseEventsHandler )
{
    this.mouseEventsHandler = mouseEventsHandler;
}
TableMouseEventsHandlerState.prototype.mouseOver = function( evt, cell ) {};
TableMouseEventsHandlerState.prototype.mouseOut = function( evt, cell ) {};
TableMouseEventsHandlerState.prototype.mouseDown = function( evt, cell ) {};
TableMouseEventsHandlerState.prototype.mouseUp = function( evt, cell ) {};
TableMouseEventsHandlerState.prototype.mouseMove = function( evt, cell ) {};

TableMouseEventsHandlerState.prototype.isTableClickable = function()
{
    return this.mouseEventsHandler.table.clickable;
};

TableMouseEventsHandlerState.prototype.isPastAndForbidden = function( cell )
{
    return !this.mouseEventsHandler.table.canEditPastRequests && cell.getRow().isPastDateCell(cell);
};

TableMouseEventsHandlerState.prototype.isBeforeHireDate = function( cell )
{
    return cell.getRow().isBeforeHireDateCell(cell);
};


// ---------------------------------------------------------------------------------------------------------------------


function HoverState( mouseEventsHandler )
{
    TableMouseEventsHandlerState.apply( this, arguments );
}

HoverState.prototype = new TableMouseEventsHandlerState( undefined );

HoverState.prototype.mouseOver = function( evt, cell )
{
    var thisDeref = this;
    cell.getRow().accept(
            {
                visitLeaveRow: function( cell )
                {
                        if (thisDeref.isTableClickable() && !thisDeref.isPastAndForbidden(cell) && !thisDeref.isBeforeHireDate(cell))
                            thisDeref.mouseEventsHandler.hoverController.hover(cell, cell, evt);
                        thisDeref.mouseEventsHandler.tooltipController.showTooltip(cell, evt);
                },

                visitDayHeaderRow: function( cell )
                {
                    thisDeref.mouseEventsHandler.tooltipController.hideTooltip();
                },
                visitOtherRow: function( cell )
                {
                    thisDeref.mouseEventsHandler.tooltipController.hideTooltip();
                }
            },
            cell
    );
};

HoverState.prototype.mouseOut = function( evt, cell )
{
    var thisDeref = this;
    cell.getRow().accept(
            {
                visitLeaveRow: function( cell )
                {
                    if(thisDeref.isTableClickable() && !thisDeref.isPastAndForbidden( cell ) && !thisDeref.isBeforeHireDate(cell))
                        thisDeref.mouseEventsHandler.hoverController.unhover( evt );

                    var target = getTargetElement( evt );
                    var targetCell = target ? target.parentNode.cellObj : undefined;
                    if( !targetCell )
                        thisDeref.mouseEventsHandler.tooltipController.hideTooltip();
                }
            },
            cell
    );
};

HoverState.prototype.mouseMove = function( evt, cell )
{
    var thisDeref = this;
    cell.getRow().accept({
            visitLeaveRow: function( cell )
            {
                thisDeref.mouseEventsHandler.tooltipController.showTooltip( cell, evt );
            }
    }, cell);
};

HoverState.prototype.mouseDown = function(evt, cell)
{
    var thisDeref = this;
    thisDeref.mouseEventsHandler.tooltipController.hideTooltip();
    cell.getRow().accept(
            {
                visitLeaveRow: function(cell)
                {
                    if (!thisDeref.isTableClickable() || thisDeref.isPastAndForbidden(cell) || thisDeref.isBeforeHireDate(cell))
                    {
                        return;
                    }

                    var isApprovableUser = cell.getRow().isApprovableUser == null || cell.getRow().isApprovableUser;
                    var editableUser = isApprovableUser || cell.getRow().editableNotApprovableUser;

                    if (editableUser)
                    {
                        thisDeref.mouseEventsHandler.getHoverController().unhover(evt);
                        if (cell.getValue() == null || !cell.getValue().requestData)
                        {
                            thisDeref.mouseEventsHandler.setCellSelectionState(cell, evt);
                        }
                        else
                        {
                            thisDeref.mouseEventsHandler.setLeaveRequestSelectedState(cell, evt);
                        }
                    }
                    else
                    {
                        thisDeref.mouseEventsHandler.setNotApplicableCellSelectedState(cell, evt);
                    }
                }
            },
            cell
    );

};


// ---------------------------------------------------------------------------------------------------------------------


function NotApplicableCellSelectionState( mouseEventsHandler )
{
    TableMouseEventsHandlerState.apply( this, arguments );
}

NotApplicableCellSelectionState.prototype = new TableMouseEventsHandlerState( undefined );

NotApplicableCellSelectionState.prototype.mouseDown = function( evt, cell )
{
    this.mouseEventsHandler.getSelectionController().clearSelection();
    this.mouseEventsHandler.setHoverState( cell, evt );
};


// ---------------------------------------------------------------------------------------------------------------------


function CellSelectionState( mouseEventsHandler )
{
    TableMouseEventsHandlerState.apply( this, arguments );
}

CellSelectionState.prototype = new TableMouseEventsHandlerState( undefined );

CellSelectionState.prototype.mouseOver = function( evt, cell )
{
    var selectionController = this.mouseEventsHandler.getSelectionController();
    var selection = selectionController.getSelection();
    var anchorCell = selectionController.getSelectionAnchor();
    var row = anchorCell.getRow();
    var targetCell = row.getCell( cell.getColIdx() );

    // if selection is shrunk, then we can safely update it
    // as the previous selection was already tested
    if( selection.containsCell( targetCell ) )
    {
        selectionController.changeSelectionTo( targetCell );
        this.updateTooltipVisibility(anchorCell, targetCell, evt);
        return;
    }

    var lastCheckedCell = null;
    var increment = targetCell.getColIdx() < selection.colMin ? -1 : 1;
    var idx = increment < 0 ? selection.colMin: selection.colMax;
    do
    {
        idx += increment;
        var iCell = row.getCell( idx );
        if( iCell.getValue() != null && iCell.getValue().requestData )
            break;
        if(this.isPastAndForbidden(iCell) || this.isBeforeHireDate(iCell))
            break;
        lastCheckedCell = iCell;
    }
    while( idx != targetCell.getColIdx() );

    if( lastCheckedCell != null )
    {
        selectionController.changeSelectionTo( lastCheckedCell );
        this.updateTooltipVisibility(anchorCell, lastCheckedCell, evt);
    }
};

CellSelectionState.prototype.mouseUp = function( evt, cell )
{
    this.mouseEventsHandler.tooltipController.hideTooltip(evt);
    this.mouseEventsHandler.setRangeSelectedState( evt );
};

CellSelectionState.prototype.mouseMove = function( evt, cell)
{
    this.mouseEventsHandler.tooltipController.moveTooltip(cell, evt);
};

CellSelectionState.prototype.updateTooltipVisibility = function(anchorCell, lastChosenCell, evt)
{
    var tooltipController = this.mouseEventsHandler.tooltipController;
    if (anchorCell == lastChosenCell)
        tooltipController.hideTooltip(evt);
    else
        tooltipController.showTooltip(lastChosenCell, evt, anchorCell);
};

// ---------------------------------------------------------------------------------------------------------------------


function LeaveRequestSelectedState( mouseEventsHandler )
{
    TableMouseEventsHandlerState.apply( this, arguments );
}

LeaveRequestSelectedState.prototype = new TableMouseEventsHandlerState( undefined );

LeaveRequestSelectedState.prototype.mouseDown = function( evt, cell )
{
    this.mouseEventsHandler.getSelectionController().clearSelection();
    this.mouseEventsHandler.setHoverState( cell, evt );
    this.mouseEventsHandler.state.mouseDown( evt, cell );
};


// ---------------------------------------------------------------------------------------------------------------------


function RangeSelectedState( mouseEventsHandler )
{
    TableMouseEventsHandlerState.apply( this, arguments );
}

RangeSelectedState.prototype = new TableMouseEventsHandlerState( undefined );

RangeSelectedState.prototype.mouseDown = function( evt, cell )
{
    this.mouseEventsHandler.getSelectionController().clearSelection();
    this.mouseEventsHandler.setHoverState( cell, evt );
    this.mouseEventsHandler.state.mouseDown( evt, cell );
};


// ---------------------------------------------------------------------------------------------------------------------


function DisabledState( mouseEventsHandler )
{
    TableMouseEventsHandlerState.apply( this, arguments );
}

DisabledState.prototype = new TableMouseEventsHandlerState( undefined );


// ---------------------------------------------------------------------------------------------------------------------

function ColleaguesHoverState(mouseEventsHandler)
{
    TableMouseEventsHandlerState.call(this, mouseEventsHandler);
}

ColleaguesHoverState.prototype = _.create(TableMouseEventsHandlerState.prototype, {
    mouseOver: function(evt, cell)
    {
        var self = this;
        cell.getRow().accept(
                {
                    visitLeaveRow: function(cell)
                    {
                        if (self._cellHasLeaveStatus(cell))
                            self.mouseEventsHandler.tooltipController.showTooltip(cell, evt);
                    },
                    visitDayHeaderRow: function()
                    {
                        self.mouseEventsHandler.tooltipController.hideTooltip();
                    },
                    visitOtherRow: function()
                    {
                        self.mouseEventsHandler.tooltipController.hideTooltip();
                    }
                },
                cell
        );
    },

    mouseOut: function(evt, cell)
    {
        var self = this;
        cell.getRow().accept(
                {
                    visitLeaveRow: function()
                    {
                        var target = getTargetElement(evt);
                        var targetCell = target ? target.parentNode.cellObj : undefined;
                        if (!targetCell)
                            self.mouseEventsHandler.tooltipController.hideTooltip();
                    }
                },
                cell
        );
    },

    mouseMove: function(evt, cell)
    {
        var self = this;
        cell.getRow().accept({
                                 visitLeaveRow: function(cell)
                                 {
                                     if (self._cellHasLeaveStatus(cell))
                                        self.mouseEventsHandler.tooltipController.showTooltip(cell, evt);
                                 }
                             }, cell);
    },

    _cellHasLeaveStatus: function(cell) {
        return cell.getValue() && cell.getValue().statusData;
    }
});