/**
 *
 * @param containerEl Container element in DOM
 * @param dateFrom
 * @param dateTo
 * @param corporateScheduleSlice ArraySlice object, where corporateScheduleSlice.get(0) matches to dateFrom day
 * @param today
 * @param userSchedules
 * @param hasColleagues
 * @param tokensHighlighter
 * @param schedulesDataOffset offset, where userSchedules[i].workdayDurations[schedulesDataOffset] matches to dateFrom day
 * @param leaveTypeNotFoundCallback
 * @param ptoHoursPerDay used to convert days to hours when displaying PTO balance
 * @constructor
 */
function ColleaguesScheduleTable(
        containerEl,
        dateFrom,
        dateTo,
        corporateScheduleSlice,
        today,
        userSchedules,
        hasColleagues,
        tokensHighlighter,
        schedulesDataOffset,
        leaveTypeNotFoundCallback,
        ptoHoursPerDay
        )
{
    this.userSchedules = userSchedules;
    this.hasColleagues = hasColleagues;
    this.tokensHighlighter = tokensHighlighter;
    this._schedulesDataOffset = schedulesDataOffset;
    this.leaveTypeNotFoundCallback = leaveTypeNotFoundCallback;

    var dateFromStr = dateFrom.format( dateFrom.getYear() == dateTo.getYear() ? "M d": "M d, Y" );
    var dateToStr   = dateTo.format( "M d, Y" );

    var monthsRowEl = document.createElement("tr");
    var searchCellEl = document.createElement("td");
    monthsRowEl.appendChild( searchCellEl );
    searchCellEl.rowSpan = 3;
    searchCellEl.className = "tableHeaderCell";
    searchCellEl.innerHTML = '&nbsp;';

    var spacerCell = document.createElement("td");
    monthsRowEl.appendChild( spacerCell );
    spacerCell.className = "spacerCell";
    spacerCell.rowSpan = 3;
    searchCellEl.className = "tableHeaderCell";

    var weeksRow = document.createElement("tr");

    LeaveTable.call( this, containerEl, dateFrom, dateTo, corporateScheduleSlice, today, monthsRowEl, weeksRow, undefined, ptoHoursPerDay, null, null, null );
    this.tableEl.id = "colleaguesScheduleTable";
    this.tableEl.className += " colleaguesScheduleTable";

    // this.mouseEventsHandler.setColleaguesHoverState();
}

//noinspection JSCheckFunctionSignatures
ColleaguesScheduleTable.prototype = _.create(LeaveTable.prototype);

ColleaguesScheduleTable.prototype.clickable = false;

ColleaguesScheduleTable.prototype.createTooltip = function()
{
    return new ColleaguesScheduleTooltip(this.ptoHoursPerDay).init();
};

ColleaguesScheduleTable.prototype.createModelRows = function ()
{
    if( this.userSchedules.length == 0 )
    {
        this.addRow( new NoColleaguesRow( this.today, this.dateFrom, this.dateTo, this.getCorporateScheduleSlice(), this.hasColleagues ) );
    }
    else
    {
        for( var i = 0; i < this.userSchedules.length; i++ )
        {
            var aRow = new ColleaguesScheduleRow(
                    this.userSchedules[i].userInfo,
                    this.userSchedules[i].requests,
                    this.userSchedules[i].statuses,
                    this.userSchedules[i].today.parseJSONDate(),
                    new ArraySlice( this.userSchedules[i].workdayDurations, this._schedulesDataOffset ),
                    this.tokensHighlighter
            );
            aRow.setLeaveTypeNotFoundCallback(this.leaveTypeNotFoundCallback);
            aRow.addLeaveCells( this.dateFrom, this.dateTo, this.getCorporateScheduleSlice(), this.ptoHoursPerDay );
            this.addRow( aRow );
        }
    }
};

ColleaguesScheduleTable.prototype.getBottomRowHeaderCellsNumber = function()
{
    return 2;
};