/**
 *
 * @param userInfo
 * @param requests
 * @param totalLeaveTime
 * @param today
 * @param workdayDurationsSlice ArraySlice object, where workdayDurationsSlice.get(0) matches to dateFrom day
 * @param dailyPTOSlice ArraySlice object, where dailyPTOSlice.get(0) matches to dateFrom day
 * @param dailySickSlice ArraySlice object, where dailySickSlice.get(0) matches to dateFrom day
 * @param groupedBy
 * @param groupRowsCount count of rows for user group, started from this row (0, if it is not a first user in row)
 * @param groupInfo
 * @param tokensHighlighter
 * @constructor
 */
function LeaveTimePlanningUserRow( userInfo, requests, statuses, totalLeaveTime, today, currentBalanceInfo, workdayDurationsSlice, dailyPTOSlice, dailySickSlice,
                                   groupedBy, groupRowsCount, groupInfo, tokensHighlighter, isApprovableUser, editableNotApprovableUser )
{
    this._tokensHighlighter = tokensHighlighter;
    this.groupRowsCount = groupRowsCount;
    this.groupInfo = groupInfo;
    this.groupVisible = groupedBy == LeaveTimePlanningPageConstants.SORT_BY_USER_GROUP ||
                        groupedBy == LeaveTimePlanningPageConstants.SORT_BY_TIME_ZONE_GROUP;
    this.isApprovableUser = isApprovableUser;
    this.editableNotApprovableUser = editableNotApprovableUser;
    UserLeaveRow.call(this, userInfo, requests, statuses, today, workdayDurationsSlice, dailyPTOSlice, dailySickSlice, currentBalanceInfo );

    this.userNameCell = document.createElement( "td" );
    this.userNameCell.className = "userNameCell";

    var groupCell;
    if (this.groupVisible)
    {
        if( groupRowsCount > 0 )
        {
            groupCell = document.createElement("td");
            groupCell.className = "groupCell firstInRow";
            groupCell.rowSpan = groupRowsCount;
            groupCell.title = groupInfo.name;

            var groupDiv = document.createElement("div");
            groupDiv.className = "groupName";
            groupDiv.innerHTML = escapeHtmlSymbols(splitByUnicodeChar(groupInfo.name.toUpperCase()).join("\n"));
            groupDiv.style.maxHeight = groupRowsCount * this.groupStyleConstants.rowHeight - this.groupStyleConstants.groupPaddings + "px";

            groupCell.appendChild(groupDiv);
        }
    }
    else
    {
        this.userNameCell.colSpan = 2;
        this.userNameCell.className += " firstInRow";
    }

    this.leaveTimeCell = document.createElement( "td" );
    this.leaveTimeCell.className = "timeTotalCell";

    this.totalLeaveTime = totalLeaveTime;

    this.updateContent( userInfo, totalLeaveTime );

    var rowEl = this.getRowEl();
    if (groupCell)
    {
        rowEl.appendChild(groupCell);
    }
    rowEl.appendChild( this.userNameCell );
    rowEl.appendChild( this.leaveTimeCell );
    rowEl.appendChild( this._createSpacerCell() );
}

LeaveTimePlanningUserRow.prototype = _.create(UserLeaveRow.prototype);

LeaveTimePlanningUserRow.prototype.formatUserName = function( userInfo )
{
    return UserNameFormat.formatLFM(
            userInfo.firstName,
            userInfo.lastName,
            userInfo.middleInitial
    );
};

LeaveTimePlanningUserRow.prototype.getRowClasses = function()
{
    if( this.groupVisible && this.groupRowsCount > 0 && this.getRowIdx() != 1 )
    {
        return ['firstRowInGroup'];
    }
    return [];
};

LeaveTimePlanningUserRow.prototype.updateContent = function( userInfo, totalLeaveTime )
{
    this.userNameCell.innerHTML = '<div class="userNameContainer">' +
                                  this._tokensHighlighter.highlightTokens( this.formatUserName( userInfo ) ) +
                                  '</div>';
    this.leaveTimeCell.innerHTML = LeaveTimePlanningTable.numberFormatter.format( totalLeaveTime / 60 );
};

LeaveTimePlanningUserRow.prototype._createSpacerCell = function()
{
    var spacerEl = document.createElement("td");
    spacerEl.className = "employeesSpacerCell";
    return spacerEl;
};

LeaveTimePlanningUserRow.prototype.getGroupRowsCount = function ()
{
    return this.groupRowsCount;
};

LeaveTimePlanningUserRow.prototype.getGroupInfo = function ()
{
    return this.groupInfo;
};

LeaveTimePlanningUserRow.prototype.groupStyleConstants = {
    rowHeight: 29,
    groupPaddings: 10
};