function MyRequestsControlManager(request, dateFrom, dateTo, callback)
{
    LeaveTimeRequestManager.call(this, request, dateFrom, dateTo, callback);
}

MyRequestsControlManager.prototype = _.create(LeaveTimeRequestManager.prototype,
                                              {
                                                  getService: function()
                                                  {
                                                      return jsonrpc.MyScheduleService;
                                                  },

                                                  getMethods: function()
                                                  {
                                                      return [
                                                          "MyScheduleService.createRequest",
                                                          "MyScheduleService.updateRequest",
                                                          "MyScheduleService.updateRequestDuration",
                                                          "MyScheduleService.updateRequestLeaveTypeAndComment",
                                                          "MyScheduleService.deleteRequest",
                                                          "MyScheduleService.deleteStatus",
                                                          "MyScheduleService.recalculateRequest",
                                                          "MyScheduleService.revokeRequestDeletion",
                                                          "MyScheduleService.cancelRequestModification"
                                                      ];
                                                  }
                                              });

// TODO: This file added to minification, so we need to add namespace at least to inner classes and functions with common names like this
function CreateRequestManager(request, dateFrom, dateTo, callback)
{
    MyRequestsControlManager.call(this, request, dateFrom, dateTo, callback);
}

CreateRequestManager.prototype = _.create(MyRequestsControlManager.prototype,
                                          {
                                              makeRemoteCall: function(callback)
                                              {
                                                  return this.getService().createRequest(
                                                          callback,
                                                          this.request.leaveTypeId,
                                                          this.request.firstDate,
                                                          this.request.lastDate,
                                                          this.request.firstDayDuration,
                                                          this.request.lastDayDuration,
                                                          this.request.middleDayDuration,
                                                          this.request.comment,
                                                          this.dateFrom,
                                                          this.dateTo
                                                  );
                                              }
                                          });


function UpdateRequestManager(request, dateFrom, dateTo, callback)
{
    MyRequestsControlManager.call(this, request, dateFrom, dateTo, callback);
}

UpdateRequestManager.prototype = _.create(MyRequestsControlManager.prototype,
                                          {
                                              makeRemoteCall: function(callback)
                                              {
                                                  return this.getService().updateRequest(
                                                          callback,
                                                          this.request.id,
                                                          this.request.leaveTypeId,
                                                          this.request.firstDayDuration,
                                                          this.request.lastDayDuration,
                                                          this.request.middleDayDuration,
                                                          this.request.comment,
                                                          this.dateFrom,
                                                          this.dateTo
                                                  );
                                              }
                                          });

function UpdateRequestDurationManager(request, dateFrom, dateTo, callback)
{
    MyRequestsControlManager.call(this, request, dateFrom, dateTo, callback);
}

UpdateRequestDurationManager.prototype = _.create(MyRequestsControlManager.prototype,
                                                  {
                                                      makeRemoteCall: function(callback)
                                                      {
                                                          return this.getService().updateRequestDuration(
                                                                  callback,
                                                                  this.request.id,
                                                                  this.request.firstDayDuration,
                                                                  this.request.lastDayDuration,
                                                                  this.request.middleDayDuration,
                                                                  this.dateFrom,
                                                                  this.dateTo
                                                          );
                                                      }
                                                  });

function UpdateRequestLeaveTypeAndCommentManager(request, dateFrom, dateTo, callback)
{
    MyRequestsControlManager.call(this, request, dateFrom, dateTo, callback);
}

UpdateRequestLeaveTypeAndCommentManager.prototype = _.create(MyRequestsControlManager.prototype,
                                                             {
                                                                 makeRemoteCall: function(callback)
                                                                 {
                                                                     return this.getService().updateRequestLeaveTypeAndComment(
                                                                             callback,
                                                                             this.request.id,
                                                                             this.request.leaveTypeId,
                                                                             this.request.comment,
                                                                             this.dateFrom,
                                                                             this.dateTo
                                                                     );
                                                                 }
                                                             });

function DeleteRequestManager(request, dateFrom, dateTo, callback)
{
    MyRequestsControlManager.call(this, request, dateFrom, dateTo, callback);
}

DeleteRequestManager.prototype = _.create(MyRequestsControlManager.prototype,
                                          {
                                              makeRemoteCall: function(callback)
                                              {
                                                  return this.getService().deleteRequest(
                                                          callback,
                                                          this.request.id,
                                                          this.dateFrom,
                                                          this.dateTo
                                                  );
                                              }
                                          });

function DeleteStatusManager(date, dateFrom, dateTo, callback)
{
    MyRequestsControlManager.call(this, null, dateFrom, dateTo, callback);
    this.date = date;
}

DeleteStatusManager.prototype = _.create(MyRequestsControlManager.prototype,
                                         {
                                             makeRemoteCall: function(callback)
                                             {
                                                 return this.getService().deleteStatus(
                                                         callback,
                                                         this.date,
                                                         this.dateFrom,
                                                         this.dateTo
                                                 );
                                             }
                                         });

function RecalculateRequestManager(request, dateFrom, dateTo, callback)
{
    MyRequestsControlManager.call(this, request, dateFrom, dateTo, callback);
}

RecalculateRequestManager.prototype = _.create(MyRequestsControlManager.prototype,
                                               {
                                                   makeRemoteCall: function(callback)
                                                   {
                                                       return this.getService().recalculateRequest(
                                                               callback,
                                                               this.request.id,
                                                               this.request.firstDate,
                                                               this.request.lastDate,
                                                               this.request.leaveTypeId,
                                                               this.request.firstDayDuration,
                                                               this.request.middleDayDuration,
                                                               this.request.lastDayDuration,
                                                               this.request.comment,
                                                               this.dateFrom,
                                                               this.dateTo
                                                       );
                                                   }
                                               });

RecalculateRequestManager.prototype.handleSpecificError = function(exception){
    var errorThrown = { exception: getExceptionNameFromTrace( exception.trace ) };
    if(errorThrown.exception == "com.actimind.actiplans.services.BaseLeaveRequestManagementService$DateBeforeHireDateException"){
        alert(messageResource.getMessage("actiplans.leave_time_requests.date_before_hire_date"));
        this.endSavingData();
        return true;
    }
    MyRequestsControlManager.prototype.handleSpecificError.call(this, exception);
};

function RevokeRequestDeletionManager(request, dateFrom, dateTo, callback)
{
    MyRequestsControlManager.call(this, request, dateFrom, dateTo, callback);
}

RevokeRequestDeletionManager.prototype = _.create(MyRequestsControlManager.prototype,
                                                  {
                                                      makeRemoteCall: function(callback)
                                                      {
                                                          return this.getService().revokeRequestDeletion(
                                                                  callback,
                                                                  this.request.id,
                                                                  this.dateFrom,
                                                                  this.dateTo
                                                          );
                                                      }
                                                  });


function CancelRequestModificationManager(request, dateFrom, dateTo, callback)
{
    MyRequestsControlManager.call(this, request, dateFrom, dateTo, callback);
}

CancelRequestModificationManager.prototype = _.create(MyRequestsControlManager.prototype,
                                                  {
                                                      makeRemoteCall: function(callback)
                                                      {
                                                          return this.getService().cancelRequestModification(
                                                                  callback,
                                                                  this.request.id,
                                                                  this.dateFrom,
                                                                  this.dateTo
                                                          );
                                                      }
                                                  });