function MySchedulePageController()
{
}

MySchedulePageController.TAB_NUM_STORAGE_KEY = "myScheduleTabNum";

MySchedulePageController.prototype = {
    init: function(dateFrom,
                   dateRangeCalculator,
                   mySchedulePageData,
                   userInitialSchedule,
                   userSchedules,
                   scheduleTabSettings,
                   wasAtIntegrated,
                   dataForSelector)
    {

        this.mySchedulePageData = mySchedulePageData; // init data
        this.pageSpinnerController = new SpinnerController();
        this.pageTabs = [];
        var scheduleTab = new ScheduleTab().init(
                dateFrom,
                mySchedulePageData,
                dateRangeCalculator,
                userInitialSchedule,
                userSchedules,
                scheduleTabSettings,
                wasAtIntegrated,
                dataForSelector
        );
        this.pageTabs.push(scheduleTab);

        var requestsTab = new RequestsTab().init(
                mySchedulePageData,
                this.pageSpinnerController,
                wasAtIntegrated
        );
        this.pageTabs.push(requestsTab);

        // var ptoHistoryTab = new PTOHistoryTab().init(historyTabSettings, mySchedulePageData.ptoHoursPerDay, this.pageSpinnerController);
        // this.pageTabs.push(ptoHistoryTab);
        //
        // var sickHistoryTab = new SickHistoryTab().init(historyTabSettings, mySchedulePageData.ptoHoursPerDay, this.pageSpinnerController);
        // this.pageTabs.push(sickHistoryTab);

        this._renderSwitcherTabs();
        this._setUpTabsCallbacks();
        this._updateTabsVisibility(this.mySchedulePageData);

        this._switchTab(this._getInitialTab(this.pageTabs, scheduleTab)); // by default selected schedule tab;

        this.balancesViewer = new BalancesViewer($("#balancesViewerPlaceholder")).init();
        this.balancesViewer.initComponents(
                    {
                        userInitialSchedule: userInitialSchedule,
                        userSchedules: userSchedules,
                        hoursPerDay: this.mySchedulePageData.ptoHoursPerDay,
                        userHireDate: this.mySchedulePageData.myScheduleData.userSchedule.userInfo.hireDate,
                        userReleaseDate: this.mySchedulePageData.userReleaseDate,
                        displayPto: this.mySchedulePageData.myScheduleData.ptoCalculationEnabled && this.mySchedulePageData.myScheduleData.userSchedule.userInfo.ptoEnabled,
                        displaySick: this.mySchedulePageData.myScheduleData.sickCalculationEnabled && this.mySchedulePageData.myScheduleData.userSchedule.userInfo.sickEnabled
                    },
                    _.bind(this._onBalancesViewerDateChange, this)
        );
        this._onBalancesViewerDateChange();
        PageOverlayTipsManager.getInstance().refreshDynamicTips(); //for add tip on balance viewer

        return this;
    },

    _renderSwitcherTabs: function()
    {
        var switcherEl = $("#pageTabsSwitcher");
        switcherEl.addClass("customHeaderTabs");
        switcherEl.empty();

        var self = this;

        for (var i = 0; i < this.pageTabs.length; i++)
        {
            var tabEl = this._getTabDomElement(this.pageTabs[i].getTabName(), i + 1);
            tabEl.on("click",
                     function(tab)
                     {
                         return function(event)
                         {
                             $(event.target).addClass("selected");
                             self._switchTab(tab);
                         }
                     }(this.pageTabs[i])
            );

            this.pageTabs[i].tabButton = tabEl;
            switcherEl.append(tabEl);
        }
    },

    /**
     * Update tabs visibility depends on page data
     * @param pageData
     * @private
     */
    _updateTabsVisibility: function(pageData)
    {
        $.each(this.pageTabs, function(ind, tab)
               {
                   var disabledTab = tab.isNeedToHideTab(pageData);
                   tab.tabButton.toggleClass("disabled", disabledTab);
                   tab.disabled = disabledTab;
               }
        );
    },
    
    /**
     * Method switch tabs. Current tab - hided and new selected
     * @param selectedTab
     * @private
     */
    _switchTab: function(selectedTab)
    {
        if (selectedTab.isActive)
        {
            return;
        }
        if (!this.currentTab)
        {
            this.currentTab = selectedTab;
        }
        else
        {
            this.currentTab.hideTab();
            this.currentTab.isActive = false;
            this.currentTab.tabButton.removeClass("selected");
        }

        selectedTab.updateTabAndShow(this.mySchedulePageData, this.tabChangingIsPerforming);

        selectedTab.tabButton.addClass("selected");
        selectedTab.isActive = true;

        //save tab num into session storage
        try {
            window.sessionStorage.setItem(MySchedulePageController.TAB_NUM_STORAGE_KEY, this.pageTabs.indexOf(selectedTab));
        } catch (e) {}

        this.currentTab = selectedTab;

        PageOverlayTipsManager.getInstance().refreshDynamicTips();
    },

    _setUpTabsCallbacks: function()
    {
        var self = this;
        $.each(this.pageTabs, function(id, tab)
        {
            tab.setOnChangeTabStateAction(
                    function(tab) {
                        return function(updateData)
                        {
                            self.mySchedulePageData = tab.mySchedulePageDataUpdater(self.mySchedulePageData, updateData);
                            self._updateTabsVisibility(self.mySchedulePageData);
                            self.onUpdatePageData(self.mySchedulePageData);

                            self.tabChangingIsPerforming = false;
                            // call update current tab for synchronize data between tabs if current tab is switched while ajax call performing
                            if(tab != self.currentTab)
                            {
                                self.currentTab.updateTabAndShow(self.mySchedulePageData, false);
                            }
                        }
                    }(tab)
            );

            tab.setOnStartChangingStateOfTabAction(
                    function()
                    {
                        self.tabChangingIsPerforming = true;
                    }
            );

            tab.setOnErrorChangingStartOfTabAction(
                    function()
                    {
                        self.tabChangingIsPerforming = false;
                        self.currentTab.updateTabAndShow(self.mySchedulePageData, false);
                    }
            );
        })
    },


    onUpdatePageData: function(actualPageData)
    {
        // calls when data on tab is changed
        this._updateBalancesViewerData(actualPageData.myScheduleData.ptoCalculationEnabled && actualPageData.myScheduleData.userSchedule.userInfo.ptoEnabled,
                                       actualPageData.myScheduleData.sickCalculationEnabled && actualPageData.myScheduleData.userSchedule.userInfo.sickEnabled);
    },

    _getTabDomElement: function(name, position)
    {
        var tabEl = $(document.createElement("span"));
        tabEl.attr("id", position);
        tabEl.addClass("tab");

        var titleEl = $(document.createElement("span"));
        titleEl.addClass("tabContainer");

        var textTab = $(document.createElement("span"));
        textTab.addClass("textTab");
        textTab.text(name);

        if (position > 1)
        {
            var titleSeparator = document.createElement("span");
            $(titleSeparator).addClass("titleSeparator");
            $(titleSeparator).html("&nbsp");
            $(tabEl).append(titleSeparator);
        }
        titleEl.append(textTab);
        tabEl.append(titleEl);
        return tabEl;
    },

    _getBalances: function(date, displayPTO, displaySick)
    {
        var balances = {};
        var offset = date.offsetInDaysFrom(this.mySchedulePageData.dateFrom.parseJSONDate());
        if (displayPTO)
        {
            var dailyPTO = this.mySchedulePageData.myScheduleData.userSchedule.dailyPTOBalance;
            if (dailyPTO == null || offset < 0 || offset >= dailyPTO.length) return null;
            balances.PTOBalance = dailyPTO[offset];
        }

        if (displaySick)
        {
            var dailySick = this.mySchedulePageData.myScheduleData.userSchedule.dailySickBalance;
            if (dailySick == null || offset < 0 || offset >= dailySick.length) return null;
            balances.sickBalance = dailySick[offset];
        }

        return balances;
    },

    _updateBalancesViewerData: function(newDisplayPto, newDisplaySick)
    {
        var date = this.balancesViewer.getDate();
        var balances = this._getBalances(date, newDisplayPto, newDisplaySick);
        var requestDate = this.balancesViewer.getDate();
        if (balances == null)
        {
            var self = this;
            makeRpcCall(
                    "MyScheduleService.getBalancesForDate",
                    [requestDate, newDisplayPto, newDisplaySick],
                    function(result)
                    {
                        self.balancesViewer.update({
                                                       displayPto: newDisplayPto,
                                                       displaySick: newDisplaySick,
                                                       balances: {PTOBalance: result ? result.PTOBalance : 0, sickBalance: result ? result.sickBalance : 0}
                                                   });
                        self._backupBalancesData(requestDate);
                    },
                    function()
                    {
                        self._restoreBalancesData();
                    }
            );
        }
        else
        {
            this.balancesViewer.update({
                                           displayPto: newDisplayPto,
                                           displaySick: newDisplaySick,
                                           balances: balances
                                       });
            this._backupBalancesData(requestDate);
        }
    },

    _backupBalancesData: function(requestDate)
    {
        this.backupBalancesDate = requestDate;
    },

    _restoreBalancesData: function()
    {
        if (this.backupBalancesDate)
        {
            this.balancesViewer.setDate(this.backupBalancesDate);
        }
    },

    _onBalancesViewerDateChange: function()
    {
        // keeps pto and sick display parameter the same
        this._updateBalancesViewerData(this.balancesViewer.isDisplayedPto(), this.balancesViewer.isDisplayedSick());
    },

    /**
     * Get ininitial tab num from session storage if exists
     * @param pageTabs - list of all tabs
     * @param defaultTab - tab by default
     * @returns {*} - tab that would be set initially
     * @private
     */
    _getInitialTab: function(pageTabs, defaultTab)
    {
        try {
            var tabNum = window.sessionStorage.getItem("myScheduleTabNum");
            if (tabNum && !_.isUndefined(pageTabs[tabNum]) && !pageTabs[tabNum].disabled)
            {
                return pageTabs[tabNum];
            }
        } catch (e) {}
        return defaultTab;
    }
};