/**
 * Controller for schedule of collegues
 */
function ColleaguesScheduleTableController()
{
}

ColleaguesScheduleTableController._PUNCTUATORS = StringTokenizer.PUNCTUATORS_FOR_SEARCH;

ColleaguesScheduleTableController.prototype = _.create(Object.prototype, {
    init: function(stateController, onPagerChangesCallback, onSearchFieldChangesCallback, leaveTypeNotFoundCallBack, dataForSelector, userSelection )
    {
        var that = this;
        this.leaveTypeNotFoundCallBack = leaveTypeNotFoundCallBack;

        this.stateController = stateController;

        this._colleaguesScheduleTableContainer = document.getElementById("colleaguesTable");
        this._colleaguesSearchFieldContainer = $('#colleaguesSearchField');
        this._totalRecordsPlaceholderContainer = $('#totalRecordsPlaceholder');

        this._lastMyScheduleData = null;
        this._lastColleaguesScheduleData = null;
        this._lastDateFrom = Date.create(1970, 1, 1);
        this._lastDateTo = Date.create(1970, 1, 1);

        this._pagerController = new PagerController().init(
                {
                    pagerData: stateController.getProperties(['pageNum', 'totalRecords', 'recordsPerPage']),
                    handlersMap: {
                        pageChanged: function(pageNum)
                        {
                            stateController.setProperty('pageNum', pageNum);
                            onPagerChangesCallback();
                        },
                        recordsPerPageChanged: function(recordsPerPage)
                        {
                            stateController.setProperty('recordsPerPage', recordsPerPage);
                            that._pagerController.loadPage(1);
                        }
                    },
                    pagerControlsElementId: "pagerPanel",
                    pagerElementId: "pager",
                    recordsPerPageSelectorId: "recordsPerPageSelector",
                    minRecordsPerPage: $("#recordsPerPageSelector").find("option:eq(0)").val(),
                    locationHashString: "colleaguesschedulepage"
                }
        );

        var _updateSearchFilter = function()
        {
            stateController.setProperty('usersSearchText', that._colleaguesSearchFieldContainer.pVal());
            stateController.setProperty('pageNum', 1);
            onSearchFieldChangesCallback();
        };

        this._colleaguesSearchFieldContainer.attr('placeholder', messageResource.getMessage("my_schedule.js.searchColleagues.placeholder")).placeholder();
        this._colleaguesSearchFieldContainer.searchField(_updateSearchFilter);
        this._colleaguesSearchFieldContainer.keyup(function(event)
                                                  {
                                                      if (event.keyCode == KEY_CODE_ESC && this.value != "")
                                                      {
                                                          this.value = "";
                                                      }

                                                      that._colleaguesSearchFieldContainer.updateSearchField();
                                                  });
        var self = this;

        this.userPanel = new AT.UsersForMyScheduleSelectorPanel();
        var chooseUserButton = new AT.HierarchySelectorDropdownButton({ renderTo: "usersSelectorId",
            menu: new AT.SharedUserButtonMenu( {panel : self.userPanel,  additionalCls: "ap-selector-menu usersForMySchedulePageSelectorMenu" })

         });

        this.userPanel.initialize( SystemSettings.apUGREnabled,
                                  SystemSettings.apTZGEnabled,
                                  dataForSelector.users,
                                  dataForSelector.userGroups,
                                  dataForSelector.timeZoneGroups,
                                  userSelection);


        chooseUserButton.setMenuHideCallback(
                function() {
                    self.userPanel.setAllUsersIfNoneSelected();
                    stateController.setProperty('userSelection', self.userPanel.getUserSelection());
                    if( self.userPanel.isStateChanged() )
                    {
                        onPagerChangesCallback();
                    }
                }
        );
        this._pagerController.render();

        return this;
    },

    getUserSelection: function(){
        return this.userPanel.getUserSelection();
    },

    updateControls: function(state, includeImmediatelyControls)
    {
        this._pagerController.setPagerStateAndUpdateUI(
                state.totalRecords,
                state.pageNum,
                state.recordsPerPage
        );
        this._totalRecordsPlaceholderContainer.text(state.totalRecords);

        if (includeImmediatelyControls)
        {
            this._colleaguesSearchFieldContainer.pVal(state.usersSearchText);
        }
        this.userPanel.updateSelector(state.userSelection);
    },

    /**
     * @param dateFrom
     * @param dateTo
     * @param pageData new data
     */
    update: function(dateFrom, dateTo, pageData)
    {
        var myScheduleData = pageData.myScheduleData;
        var colleaguesScheduleData = pageData.colleaguesScheduleData;

        if (myScheduleData != this._lastMyScheduleData ||
            colleaguesScheduleData != this._lastColleaguesScheduleData || !dateFrom.isSameDate(this._lastDateFrom) || !dateTo.isSameDate(this._lastDateTo))
        {
            // If we load empty data with non-standard user selection and empty search term, reload the interface
            if (colleaguesScheduleData.totalRecords === 0 && !this.stateController.getProperty("usersSearchText") && !this.getUserSelection().allUsers)
            {
                alert(messageResource.getMessage("my_schedule.js.no_users_loaded"));
                document.location.reload();
                return;
            }

            this._lastMyScheduleData = myScheduleData;
            this._lastColleaguesScheduleData = colleaguesScheduleData;
            this._lastDateFrom = dateFrom;
            this._lastDateTo = dateTo;

            this.stateController.setProperty('pageNum', colleaguesScheduleData.pageNum);
            this.stateController.setProperty('totalRecords', colleaguesScheduleData.totalRecords);

            var schedulesDataOffset = dateFrom.offsetInDaysFrom(pageData.dateFrom.parseJSONDate());

            if (this._colleaguesScheduleTable)
            {
                this._colleaguesScheduleTable.preDestroy();
            }

            var colleaguesScheduleTable = new ColleaguesScheduleTable(
                    $(this._colleaguesScheduleTableContainer),
                    dateFrom,
                    dateTo,
                    new ArraySlice(myScheduleData.corporateSchedule.workdayDurations, schedulesDataOffset),
                    myScheduleData.corporateSchedule.today.parseJSONDate(),
                    colleaguesScheduleData.records,
                    colleaguesScheduleData.hasColleagues,
                    this._getTokensHighlighter(),
                    schedulesDataOffset,
                    this.leaveTypeNotFoundCallback,
                    pageData.ptoHoursPerDay
            );

            this._colleaguesScheduleTableContainer.innerHTML = '';
            colleaguesScheduleTable.buildDOM(this._colleaguesScheduleTableContainer);

            this._colleaguesScheduleTable = colleaguesScheduleTable;
        }
    },

    getSearchColleaguesTokens: function()
    {
        return new StringTokenizer(ColleaguesScheduleTableController._PUNCTUATORS).getTokens(this.stateController.getProperty('usersSearchText').toLowerCase());
    },

    _getTokensHighlighter: function()
    {
        var tokens = this.getSearchColleaguesTokens();
        return (tokens && tokens.length > 0) ? new TokensHighlighter(tokens, ColleaguesScheduleTableController._PUNCTUATORS) : new EmptyHighlighter();
    }
});
