/**
 * Controller for schedule of current user
 */
function MyScheduleTableController()
{
}

MyScheduleTableController.prototype = {
    init: function(leaveTableUpdateController, pageData, userInitialSchedule, userSchedules, newRequestSelectionListener, existingRequestSelectionListener, leaveTypeNotFoundCallBack, wasAtIntegrated)
    {
        var that = this;

        this.leaveTypeNotFoundCallBack = leaveTypeNotFoundCallBack;
        this.newRequestSelectionListener = newRequestSelectionListener;
        this.existingRequestSelectionListener = existingRequestSelectionListener;

        this._wasAtIntegrated = wasAtIntegrated;

        this._mainCellContainer = document.getElementById("myScheduleTableContainer");

        this._lastMyScheduleData = null;
        this._lastDateFrom = Date.create(1970, 1, 1);
        this._lastDateTo = Date.create(1970, 1, 1);

        this._myScheduleTable = null;

        var todayDate = pageData.myScheduleData.userSchedule.today.parseJSONDate();

        this._scrollPanelController = new ScrollPanelController().init(
                "myScheduleScrollPanel",
                function() { leaveTableUpdateController.loadPreviousWeek(that.getDateFrom()); },
                function() { leaveTableUpdateController.loadNextWeek(that.getDateFrom()); },
                function(date) { leaveTableUpdateController.loadCustomWeek(date); },
                that.getDateFrom(),
                function() { return todayDate },
                new UserScheduleCalculator().init(invertedDaysDateMap, userInitialSchedule, userSchedules), //  invertedDaysDateMap - from invertedDaysServlet
                function()
                {
                    return getMinTodayOfUsers(pageData.myScheduleData.userSchedule, pageData.colleaguesScheduleData.records);
                }
        );

        return this;
    },

    getSelection: function() { return this._myScheduleTable.getSelectionController().getSelection(); },
    
    getDateRangeForRegion: function(selectionRegion) { return this._myScheduleTable.getDateRangeForRegion(selectionRegion); },
    
    clearSelection: function() { this._myScheduleTable.clearSelection(); },
    
    getRow: function(rowNum) { return this._myScheduleTable.getRow(rowNum); },
    
    getCell: function(rowNum, colNum) { return this._myScheduleTable.getCell(rowNum, colNum); },
    
    enableUserActivities: function() { this._myScheduleTable.enableUserActivities(); },
    
    disableUserActivities: function() { this._myScheduleTable.disableUserActivities(); },
    
    showProgress: function(tableRegion) { this._myScheduleTable.showProgress(tableRegion); },

    setProgressVisibility: function(isVisible)
    {
        this._myScheduleTable.setProgressVisibility(isVisible);
    },

    _hideProgress: function(table)
    {
        if (table)
        {
            table.hideProgress();
        }
    },

    getDateFrom: function()
    {
        return this._lastDateFrom;
    },

    _updateScrollPanel: function()
    {
        this._scrollPanelController.updateControls(this.getDateFrom());
    },

    updateControls: function(state)
    {
        this._updateScrollPanel();
    },

    hideProgress: function()
    {
        this._hideProgress(this._myScheduleTable);
    },

    /**
     * must be guaranteed that (dateFrom, dateTo) in (MySchedulePage.pageData.dateFrom, MySchedulePage.pageData.dateTo)
     * @param dateFrom
     * @param dateTo
     * @param pageData new page data
     */
    update: function(dateFrom, dateTo, pageData)
    {
        var myScheduleData = pageData.myScheduleData;

        if (myScheduleData != this._lastMyScheduleData || !dateFrom.isSameDate(this._lastDateFrom) || !dateTo.isSameDate(this._lastDateTo))
        {
            this._hideProgress(this._myScheduleTable);

            if (this._myScheduleTable)
            {
                this._myScheduleTable.hideTooltip(); //hide tooltip after rerender if exist
            }

            this._lastMyScheduleData = myScheduleData;
            this._lastDateFrom = dateFrom;
            this._lastDateTo = dateTo;

            var schedulesDataOffset = dateFrom.offsetInDaysFrom(pageData.dateFrom.parseJSONDate());
            var todayDate = myScheduleData.userSchedule.today.parseJSONDate();

            if (this._myScheduleTable)
            {
                this._myScheduleTable.preDestroy();
            }

            this._myScheduleTable = new MyScheduleTable(
                    $(this._mainCellContainer),
                    dateFrom,
                    dateTo,
                    new ArraySlice(myScheduleData.corporateSchedule.workdayDurations, schedulesDataOffset),
                    new ArraySlice(myScheduleData.userSchedule.workdayDurations, schedulesDataOffset),
                    todayDate,
                    myScheduleData.userSchedule.requests,
                    myScheduleData.userSchedule.statuses,
                    myScheduleData.userSchedule.userInfo,
                    new ArraySlice(myScheduleData.userSchedule.dailyPTOBalance, schedulesDataOffset),
                    new ArraySlice(myScheduleData.userSchedule.dailySickBalance, schedulesDataOffset),
                    this.leaveTypeNotFoundCallback,
                    pageData.ptoHoursPerDay,
                    pageData.myScheduleData.canManagePastRequests,
                    this._wasAtIntegrated,
                    myScheduleData.userSchedule.currentBalanceInfo
            );

            this._myScheduleTable.addNewRequestSelectionListener(this.newRequestSelectionListener);
            this._myScheduleTable.addExistingRequestSelectionListener(this.existingRequestSelectionListener);

            this._mainCellContainer.innerHTML = '';

            this._myScheduleTable.buildDOM(this._mainCellContainer);

            this._updateScrollPanel(); // if change date and scroll while request is pending scroll panel must be updated
        }
    }
};
