function RequestsTab(){}

RequestsTab.prototype = _.create(MySchedulePageTab.prototype);
RequestsTab.prototype.superclass = MySchedulePageTab;
RequestsTab.prototype.constructor = RequestsTab;

RequestsTab.prototype.init = function(mySchedulePageData, spinnerController, wasATIntegrated){

    this.pageData = mySchedulePageData;
    this.spinnerController = spinnerController;
    this.myRequestsTableController = new MyRequestsTableController().init(
        $.proxy(this.editRequestLeaveTypeFromRequestsTable, this),
        $.proxy(this.editRequestPeriodFromRequestsTable, this),
        $.proxy(this.deleteOrRevokeRequestFromRequestsTable, this),
        wasATIntegrated
    );

    this._updateTab();

    var self = this;
    $( window ).resize( function () { self._updateTab( ); } );

    return this;
};

RequestsTab.prototype.updateTabAndShow = function(updatedData, disableActivities){
    this.pageData = updatedData;
    this._updateTab();
    $("#requestsTableTab").show();

    if(disableActivities){
        this.myRequestsTableController.lockTable(true);
    } else {
        this.myRequestsTableController.lockTable(false);
    }
};

RequestsTab.prototype.hideTab = function() {
    $("#requestsTableTab").hide();
};

RequestsTab.prototype.getTabName = function() {
    return messageResource.getMessage("my_schedule.page.tabs.requests_tab.title");
};

RequestsTab.prototype.mySchedulePageDataUpdater = function( oldPageData, updatedData ) {
    oldPageData.myScheduleData = updatedData;
    return oldPageData; // this tab updates only myScheduleData
};


RequestsTab.prototype._updateTab = function(){

    this.myRequestsTableController.update(this.pageData);

};

RequestsTab.prototype.editRequestLeaveTypeFromRequestsTable = function ( request, button, closeDialogCallback )
{
    this.editRequest( RequestsTab.editLeaveTypeDialogStrategy, request, button, "10px, -10px", closeDialogCallback );
};

RequestsTab.prototype.editRequestPeriodFromRequestsTable = function ( request, button, closeDialogCallback )
{
    this.editRequest( RequestsTab.editPeriodDialogStrategy, request, button, "10px, -10px", closeDialogCallback );
};

RequestsTab.prototype.deleteOrRevokeRequestFromRequestsTable = function ( request, button )
{
    var requestRow = $(button).parent();
    if(request.requestChange){
        this.myRequestsTableController.lockTable(true);
        this.makeRequestAJAXCallAndUpdateData(request, CancelRequestModificationManager);
        return;
    }
    if (request.waitingDeletion)
    {
         this.myRequestsTableController.lockTable(true);
         this.makeRequestAJAXCallAndUpdateData(request, RevokeRequestDeletionManager);
         return;
    }

    requestRow.addClass("requestToDelete");
    if (confirm(messageResource.getMessage(request.past ? "actiplans.leave_time_requests.delete_request_confirmation.past" : "my_schedule.js.delete_request_confirmation")))
    {
        this.myRequestsTableController.lockTable(true);
        this.deleteRequestAndUpdateData(request);
    }
    else
    {
        this.myRequestsTableController.lockTable(false);
        this.myRequestsTableController.cancelRowAction();
    }
    requestRow.removeClass("requestToDelete");
};

RequestsTab.prototype.editRequest = function( createDialogStrategy, leaveTimeRequest, showAt, offset, closeDialogCallback )
{
    // copy the request object.
    leaveTimeRequest = jQuery.extend({},leaveTimeRequest);
    var self = this;

    var successor = {

        closed: function()
        {

            self.myRequestsTableController.cancelRowAction();
            self.requestDialog = null;
            if (closeDialogCallback)
                closeDialogCallback();
        },

        cancelled: function() {
            self.requestDialog.close();
        },

        deleteRequest: function( request ) {
            if( confirm( messageResource.getMessage(request.past ? "actiplans.leave_time_requests.delete_request_confirmation.past" : "my_schedule.js.delete_request_confirmation" ) ) )
                self.deleteRequestAndUpdateData( request );
        }

    };

    var userIsPTO = this.pageData.myScheduleData.userSchedule.userInfo.ptoEnabled;
    var userIsSick = this.pageData.myScheduleData.userSchedule.userInfo.sickEnabled;
    
    if((leaveTimeRequest.past && !this.pageData.myScheduleData.canManagePastRequests) || leaveTimeRequest.status == LeaveTimeRequestStatus.REJECTED || leaveTimeRequest.waitingDeletion)
    {
        self.requestDialog = createDialogStrategy.createViewLeaveRequestDialog(
                leaveTimeRequest,
                userIsPTO,
                userIsSick,
                successor,
                this.pageData.ptoHoursPerDay,
                this.pageData.myScheduleData.userSchedule.currentBalanceInfo
        );
    }
    else
    {
        successor.saveRequest = function( request ) {
            self.saveRequestChangesAndUpdateData( request, createDialogStrategy.updateRequestRPCManager );
        };

        self.requestDialog = createDialogStrategy.createEditLeaveRequestDialog(
                leaveTimeRequest,
                userIsPTO,
                userIsSick,
                successor,
                this.pageData.ptoHoursPerDay,
                this.pageData.myScheduleData.userSchedule.currentBalanceInfo
        );
    }

    self.requestDialog.showAt( showAt, offset );
};

RequestsTab.prototype.deleteRequestAndUpdateData = function( request )
{
    this.makeRequestAJAXCallAndUpdateData( request,DeleteRequestManager );
};

RequestsTab.prototype.saveRequestChangesAndUpdateData = function( request, RequestManager )
{
    this.makeRequestAJAXCallAndUpdateData( request, RequestManager );
};

RequestsTab.prototype.makeRequestAJAXCallAndUpdateData = function( request, rpcManagerConstructor )
{
    var self = this;

    this.onStartChangingStateOfTabAction(); // say page controller that we wont to change state of this tab

    this.myRequestsTableController.lockTable(true);

    if(this.requestDialog)
        this.requestDialog.close();
    this.requestDialog = null;

    this.spinnerController.spin("editRequestSpinner");

    rpcManagerConstructor.prototype.endSavingData = function(success){
        if(!success){
            self.onErrorChangingStartOfTabAction();
        }
        self.spinnerController.stop();
        self.myRequestsTableController.lockTable(false);
        self.myRequestsTableController.cancelRowAction();
    };

    rpcManagerConstructor.prototype.onNoRightsToUpdateRequestException = function(){
        if( self.requestDialog)
            self.requestDialog.close();
    };

    var manager = new rpcManagerConstructor(
        request,
        self.pageData.dateFrom,
        self.pageData.dateTo,
        $.proxy(self.requestSaved, self)
    );

    RpcControlManager.cancelAllAJAXRequests();
    RpcControlManager.saveRpcControlBoundData( manager, undefined );
};

RequestsTab.prototype.requestSaved = function ( invocationResult )
{
    if( invocationResult.data )
    {
        this.pageData.myScheduleData = invocationResult.data;
        this._updateTab();
        this.onChangeTabStateAction(invocationResult.data);
    }
    this.myRequestsTableController.lockTable(false);
    this.myRequestsTableController.cancelRowAction();
    this.spinnerController.stop();
};

RequestsTab.editLeaveTypeDialogStrategy = {

    createEditLeaveRequestDialog: function ( request, isPTOUser, isSickUser, successor, ptoHoursPerDay, currentBalanceInfo ) {
        return new EditLeaveRequestLeaveTypeDialog( request, isPTOUser, isSickUser, null, successor, ptoHoursPerDay, currentBalanceInfo ).init();
    },

    createViewLeaveRequestDialog: function ( request, isPTOUser, isSickUser, successor, ptoHoursPerDay, currentBalanceInfo ) {
        return new ViewLeaveRequestLeaveTypeDialog( request, isPTOUser, isSickUser, successor, ptoHoursPerDay, currentBalanceInfo ).init();
    },

    updateRequestRPCManager: UpdateRequestLeaveTypeAndCommentManager

};

RequestsTab.editPeriodDialogStrategy = {

    createEditLeaveRequestDialog: function ( request, isPTOUser, isSickUser, successor, ptoHoursPerDay, currentBalanceInfo ) {
        return new EditLeaveRequestPeriodDialog( request, isPTOUser, isSickUser, null, successor, ptoHoursPerDay, currentBalanceInfo ).init();
    },

    createViewLeaveRequestDialog: function ( request, isPTOUser, isSickUser, successor, ptoHoursPerDay, currentBalanceInfo ) {
        return new ViewLeaveRequestPeriodDialog( request, isPTOUser, isSickUser, successor, ptoHoursPerDay, currentBalanceInfo ).init();
    },

    updateRequestRPCManager: UpdateRequestDurationManager

};

//----------------------------------------
//Tab for requests table
function MyRequestsTableController(){}

MyRequestsTableController.prototype.init = function (onRequestLeaveTypeEditClick, onRequestPeriodEditClick, onRequestDeleteClick, wasATIntegrated )
{
    this._lastMyScheduleData = null;
    this._myRequestsTable = null;
    this._myRequestsTableBalanceTypeFilter = null;
    this._wasATIntegrated = wasATIntegrated;

    this.onRequestLeaveTypeEditClick = onRequestLeaveTypeEditClick;
    this.onRequestPeriodEditClick = onRequestPeriodEditClick;
    this.onRequestDeleteClick = onRequestDeleteClick;

    return this;
};

MyRequestsTableController.prototype.cancelRowAction = function ()
{
    if( this._myRequestsTable != null )
    {
        this._myRequestsTable.cancelRowAction();
    }
};

MyRequestsTableController.prototype.lockTable = function ( isLocked )
{
    if( this._myRequestsTable != null )
    {
        this._myRequestsTable.lockTable(isLocked);
    }
};

MyRequestsTableController.prototype.update = function (pageData)
{
    var myScheduleData = pageData.myScheduleData;
    var self = this;
    if( myScheduleData != this._lastMyScheduleData )
    {
        this._lastMyScheduleData = myScheduleData;


        var requestsToShow = [];

        // Filter out requests which are totally in the past.
        // This will affect approved requests only, because other requests are already filtered on the server side.
        $.each( myScheduleData.userSchedule.requests, function ()
        {
           // if( this.lastDate >= myScheduleData.userSchedule.today )//TODO ant add filter for request for past 30 days
                requestsToShow.push( this );
        } );

        if( this._myRequestsTable != null )
            this._myRequestsTable.destroy();

        this._myRequestsTable = new LeaveRequestTable( {
            renderTo: '#myRequestsTable>table',
            requests: requestsToShow,
            showEmployeeColumn: false,
            showDeleteColumn: true,
            showSelectColumn: false,
            showFullPeriod: true,
            paginate: false,
            onRequestLeaveTypeEditClick: self.onRequestLeaveTypeEditClick,
            onRequestPeriodEditClick: self.onRequestPeriodEditClick,
            onRequestDeleteClick: self.onRequestDeleteClick,
            ptoHoursPerDay: pageData.ptoHoursPerDay,
            isUserPTOControlOn: myScheduleData.userSchedule.userInfo.ptoEnabled,
            isUserSickControlOn: myScheduleData.userSchedule.userInfo.sickEnabled,
            balanceTypeFilter: this._myRequestsTableBalanceTypeFilter,
            ptoCalculationEnabled: myScheduleData.ptoCalculationEnabled,
            sickCalculationEnabled: myScheduleData.sickCalculationEnabled,
            leftDateToShowPastFilteredRequests: myScheduleData.userSchedule.today.parseJSONDate().getNDaysBefore(N_DAYS_IN_PAST_FOR_SHOW_APPROVED_REJECTED_REQUESTS),
            canEditPastRequests: myScheduleData.canManagePastRequests,
            userInfo: myScheduleData.userSchedule.userInfo,
            wasATIntegrated: self._wasATIntegrated
        } );

        if ( !this._myRequestsTableBalanceTypeFilter )
            this._myRequestsTableBalanceTypeFilter = this._myRequestsTable.getBalanceTypeFilter();
    }
};
