function ScheduleTab()
{
}

ScheduleTab.prototype = _.create(MySchedulePageTab.prototype, {
    init: function(dateFrom, mySchedulePageData, dateRangeCalculator, userInitialSchedule, userSchedules, schedulePageSettings, wasAtIntegrated,  dataForSelector)
    {

        var self = this;
        var stateController = new StateController().init({
                                                             totalRecords: mySchedulePageData.colleaguesScheduleData.totalRecords,
                                                             recordsPerPage: schedulePageSettings.recordsPerPage,
                                                             pageNum: mySchedulePageData.colleaguesScheduleData.pageNum,
                                                             usersSearchText: "",
                                                             userSelection: schedulePageSettings.userSelection
                                                         });

        this.pageData = mySchedulePageData;
        this.userInitialSchedule = userInitialSchedule;
        this.userSchedules = userSchedules;
        this.leaveTableUpdateController = new LeaveTableUpdateController().init(
                dateRangeCalculator,
                function(dateFrom)
                {
                    return self._updateTab(dateFrom)
                },
                "MyScheduleService.getMySchedulePageData",
                function(dateFrom, dateRange)
                {
                    return [
                        dateRange.dateFrom.toJSON(),
                        dateRange.dateTo.toJSON(),
                        self.colleaguesScheduleTableController.getSearchColleaguesTokens(),
                        stateController.getProperty('pageNum'),
                        stateController.getProperty('recordsPerPage'),
                        dateFrom.toJSON(),
                        self.colleaguesScheduleTableController.getUserSelection()
                    ];
                },
                function(pageData)
                {
                    // while data loaded only synchronize it
                    self.pageData = pageData;
                    self.onChangeTabStateAction(pageData);
                },
                stateController
        );

        var leaveTypeNotFoundCallback = function()
        {
            RpcControlManager.cancelAllAJAXRequests();
            self.leaveTableUpdateController.abortUpdateSchedulesRequest();
            document.location.reload();
        };

        this.myScheduleTableController = new MyScheduleTableController().init(
                this.leaveTableUpdateController,
                mySchedulePageData,
                userInitialSchedule,
                userSchedules,
                $.proxy(self.createRequest, self),
                $.proxy(self.editRequestFromSchedule, self),
                leaveTypeNotFoundCallback,
                wasAtIntegrated
        );

        LeaveScheduleUtils.createLegend("scheduleLegendSecondBlockTable", 5);
        this.colleaguesScheduleTableController = new ColleaguesScheduleTableController().init(
                stateController,
                function() { self.leaveTableUpdateController.simpleUpdateSchedules(self.myScheduleTableController.getDateFrom(), 'spinnerPlaceholderPager');},
                function() { self.leaveTableUpdateController.updateSchedulesWithDelay(self.myScheduleTableController.getDateFrom(), 'spinnerPlaceholderSearchField', 200);},
                leaveTypeNotFoundCallback,
                dataForSelector,
                schedulePageSettings.userSelection
        );

        this._updateTab(dateFrom);

        var updateControlsFunc = function(state, includeImmediatelyControls)
        {
            self.myScheduleTableController.updateControls(state);
            self.colleaguesScheduleTableController.updateControls(state, includeImmediatelyControls);

        };
        stateController.addOnSaveStateActions(function(state) { updateControlsFunc(state, false); });
        stateController.addOnLoadStateActions(function(state) { updateControlsFunc(state, true); });

        updateControlsFunc(stateController.getState(), true);

        $(window).resize(function()
                         {
                             if (self.isVisible)
                             {
                                 self._updateTab(self.myScheduleTableController.getDateFrom());
                                 self._updateWidthsOfFlexibleElements();
                             }
                         }
        );

        return this;
    },

    /**
     * Method updates widths of my_schedule calendar placeholder and width of leave types lengend
     */
    _updateWidthsOfFlexibleElements: function()
    {
        this._updateMyScheduleTableWidth();
        this._updateColleaguesTableWidth();
    },

    _updateRightSectionWidth: function(section, container)
    {
        var separatorWidth = 5;
        var dayWidth = 12;
        section.outerWidth(container.width() - separatorWidth - (this._getVisibleDaysCount() * dayWidth + 1));
    },

    _updateMyScheduleTableWidth: function()
    {
        this._updateLegendLength();
        this._updateRightSectionWidth($("#legendCell"), $("#myScheduleSectionTable"));
        var widthOfChartTable = $("#mainContentTable").width();
        $("#scheduleLegendTable").width(widthOfChartTable);
    },

    _updateColleaguesTableWidth: function()
    {
        var $colleaguesScheduleTable = $("#colleaguesScheduleTable");
        this._updateRightSectionWidth($colleaguesScheduleTable.find(".tableHeaderCell"), $colleaguesScheduleTable);
        PageOverlayTipsManager.getInstance().refreshDynamicTips(false);
    },

    _getTabContainerId: function()
    {
        return "scheduleTab";
    },

    getTabName: function()
    {
        return messageResource.getMessage("my_schedule.page.tabs.schedule_tab.title");
    },

    updateTabAndShow: function(newData, disableActivities)
    {
        this.pageData = newData;
        this.isVisible = true;
        this._updateTab();

        $("#" + this._getTabContainerId()).show();
        this.myScheduleTableController.setProgressVisibility(true);
        this._updateWidthsOfFlexibleElements();

        if (disableActivities)
        {
            this.myScheduleTableController.disableUserActivities();
        }
        else
        {
            this.myScheduleTableController.enableUserActivities();
        }
    },

    hideTab: function(data)
    {
        this.leaveTableUpdateController.abortUpdateSchedulesRequest();
        if (self.requestDialog)
            self.requestDialog.close();
        this.isVisible = false;
        this.myScheduleTableController.setProgressVisibility(false);
        $("#" + this._getTabContainerId()).hide();
    },

    mySchedulePageDataUpdater: function(oldPageData, updatedData)
    {
        return updatedData; // this tab updates all data
    },

    /**
     * This method updates content in schedule tab
     * @param dateFrom
     * @returns {boolean}
     * @private
     */
    _updateTab: function(dateFrom)
    {
        if (!dateFrom)
        {
            dateFrom = this.myScheduleTableController.getDateFrom();
        }
        var loadedDateFrom = this.pageData.dateFrom.parseJSONDate();
        var loadedDateTo = this.pageData.dateTo.parseJSONDate();

        var dateTo = dateFrom.getNDaysAfter(this._getVisibleDaysCount() - 1);

        if (compareDates(loadedDateFrom, dateFrom) <= 0 && compareDates(dateTo, loadedDateTo) <= 0)
        {
            this.myScheduleTableController.update(dateFrom, dateTo, this.pageData);

            if (!this.hideColleagues)
            {
                this.colleaguesScheduleTableController.update(dateFrom, dateTo, this.pageData);
                this._updateColleaguesTableWidth();
            }

            return true;
        }
        else
        {
            return false;
        }
    },

    _getVisibleDaysCount: function()
    {
        return 7/*days in week*/ * Math.max(
                        8/*weeks*/,
                        Math.floor(Math.min(MySchedulePageConstants.MAX_VISIBLE_DAYS, (document.body.clientWidth - 335) / (12/*day cell width*/)) / 7)
                );
    },

    _updateLegendLength: function()
    {
        var maxWidth = Math.max(80, ($('#myScheduleTable').width() - 140 ) / 5 - 40);
        var scheduleLegendLTNameDiv = $('.scheduleLegendLTNameDiv');
        scheduleLegendLTNameDiv.css('max-width', maxWidth);

        // fix for long leave type names in IE8
        var ua = navigator.userAgent.toLowerCase();
        var isIE8 = ua.indexOf("msie 8") > -1;
        if (isIE8)
        {
            $.each(scheduleLegendLTNameDiv, function()
            {
                if ($(this).parent().width() >= maxWidth)
                    $(this).parent().width($(this).width());
                else
                    $(this).parent().css('width', 'auto');
            });
        }
    },

    createRequest: function()
    {
        var self = this;

        if (!LeaveType.hasActiveLeaveTypes())
        {
            alert(messageResource.getMessage("my_schedule.js.create_request_no_leave_types"));
            this.myScheduleTableController.clearSelection();
            return;
        }

        this.leaveTableUpdateController.abortUpdateSchedulesRequest();

        var selectionRegion = this.myScheduleTableController.getSelection();
        var dateRange = this.myScheduleTableController.getDateRangeForRegion(selectionRegion);

        var userInfo = this.pageData.myScheduleData.userSchedule.userInfo;
        var firstDate = dateRange.dateFrom.toJSON();
        var lastDate = dateRange.dateTo.toJSON();
        var cell = this.myScheduleTableController.getRow(selectionRegion.rowMin).getCell(selectionRegion.colMin);
        var showAt = cell.getCellEl();

        // Actually create new request
        var leaveTimeRequest = {
            id: -546,
            userId: userInfo.id,
            leaveTypeId: LEAVE_TYPES[ACTIVE_LEAVE_TYPES_ORDER[0]].id,
            status: LeaveTimeRequestStatus.REQUESTED,
            approverId: null,
            firstDate: firstDate,
            lastDate: lastDate,
            length: dateRange.dateTo.offsetInDaysFrom(dateRange.dateFrom) + 1, // TODO: this can be calculated from selection range
            firstDayDuration: -1, // whole day
            lastDayDuration: -1, // whole day
            middleDayDuration: -1, // whole day
            comment: null,
            requestChange: null,
            past: cell.getRow().isPastDateCell(cell)
        };

        this.requestDialog = new CreateLeaveRequestDialog(
                leaveTimeRequest,
                userInfo.ptoEnabled && this.pageData.myScheduleData.ptoCalculationEnabled,
                userInfo.sickEnabled && this.pageData.myScheduleData.sickCalculationEnabled,
                userInfo,
                {

                    closed: function()
                    {
                        self.myScheduleTableController.clearSelection();
                        self.requestDialog = null;
                    },

                    cancelled: function()
                    {
                        self.requestDialog.close();
                    },

                    createRequest: function(request)
                    {
                        self.createRequestAndUpdateData(request, selectionRegion);
                    }

                },
                $.proxy(self.createRequestRecalculationRPCManager, self),
                this.pageData.ptoHoursPerDay,
                this.pageData.myScheduleData.userSchedule.currentBalanceInfo,
                false
        ).init();

        this.requestDialog.showAt(showAt);
    },

    onNoRightsToUpdateRequestAction: function()
    {
        if (this.requestDialog)
            this.requestDialog.close();
        this.leaveTableUpdateController.simpleUpdateSchedules(this.myScheduleTableController.getDateFrom(), 'spinnerPlaceholderNextWeek');
    },

    createRequestRecalculationRPCManager: function(request, requestDialogCallback)
    {
        var self = this;
        var myCallback = function(invocationResult)
        {
            if (invocationResult.calculatedRequestData)
            {
                requestDialogCallback(invocationResult.calculatedRequestData);
            }
            else
            {
                if (self.requestDialog)
                    self.requestDialog.close();
                self.requestSaved(invocationResult);
            }
        };

        var recalculateRequestManager = RecalculateRequestManager;
        recalculateRequestManager.prototype.endSavingData = function(success)
        {
            if (!success)
            {
                if (self.requestDialog)
                {
                    self.requestDialog.closeWithoutAborting();
                }
                self.myScheduleTableController.enableUserActivities();
            }
        };

        recalculateRequestManager.prototype.onNoRightsToUpdateRequestException = $.proxy(this.onNoRightsToUpdateRequestAction, this);

        return new recalculateRequestManager(
                request,
                this.pageData.dateFrom,
                this.pageData.dateTo,
                myCallback
        );
    },

    requestSaved: function(invocationResult)
    {
        if (invocationResult.data)
        {
            this.pageData.myScheduleData = invocationResult.data;
            this._updateTab(this.myScheduleTableController.getDateFrom());
            this.onChangeTabStateAction(this.pageData);
        }
    },

    createRequestAndUpdateData: function(request, tableRegion)
    {
        this.makeRequestWithConstructorAndUpdateData(tableRegion, request, CreateRequestManager);
    },

    deleteRequestAndUpdateData: function(request, tableRegion)
    {
        this.makeRequestWithConstructorAndUpdateData(tableRegion, request, DeleteRequestManager);
    },

    deleteStatusAndUpdateData: function(date, tableRegion)
    {
        this.makeRequestWithManagerAndUpdateData(tableRegion, new DeleteStatusManager(date, this.pageData.dateFrom, this.pageData.dateTo, $.proxy(this.requestSaved, this)));
    },

    saveRequestChangesAndUpdateData: function(request, tableRegion)
    {
        this.makeRequestWithConstructorAndUpdateData(tableRegion, request, UpdateRequestManager);
    },

    revokeModificationOrDeletionRequestAndUpdateData: function(request, tableRegion)
    {
        var manager;
        if (request.requestChange)
            manager = CancelRequestModificationManager;
        else if (request.waitingDeletion)
            manager = RevokeRequestDeletionManager;

        if (manager)
            this.makeRequestWithConstructorAndUpdateData(tableRegion, request, manager)
    },

    makeRequestWithManagerAndUpdateData: function(tableRegion, manager)
    {
        var self = this;
        if (this.requestDialog)
            this.requestDialog.close();
        this.requestDialog = null;

        this.onStartChangingStateOfTabAction();

        this.myScheduleTableController.showProgress(tableRegion);
        this.myScheduleTableController.disableUserActivities();

        manager.endSavingData = function(success)
        {
            if (!success)
            {
                self.onErrorChangingStartOfTabAction();
            }
            self.myScheduleTableController.hideProgress();
            self.myScheduleTableController.enableUserActivities();
        };
        manager.onNoRightsToUpdateRequestException = $.proxy(this.onNoRightsToUpdateRequestAction, this);


        RpcControlManager.cancelAllAJAXRequests();
        RpcControlManager.saveRpcControlBoundData(manager, undefined);
    },

    makeRequestWithConstructorAndUpdateData: function(tableRegion, request, rpcManagerConstructor)
    {
        this.makeRequestWithManagerAndUpdateData(tableRegion, new rpcManagerConstructor(
                request,
                this.pageData.dateFrom,
                this.pageData.dateTo,
                $.proxy(this.requestSaved, this)
        ));
    },

    editRequestFromSchedule: function(tableRange)
    {
        var self = this;

        this.leaveTableUpdateController.abortUpdateSchedulesRequest();

        var selectionRegion = this.myScheduleTableController.getSelection();
        var leaveTimeRequest = this.myScheduleTableController.getCell(selectionRegion.rowMin, selectionRegion.colMin).getValue().requestData;
        var showAt = this.myScheduleTableController.getRow(selectionRegion.rowMin).getCell(selectionRegion.colMin).getCellEl();

        var revokeModificationOrDeletionCallback = function()
        {
            self.revokeModificationOrDeletionRequestAndUpdateData(leaveTimeRequest, self.myScheduleTableController.getSelection());
        };

        var editRequestCallback = function()
        {
            // Keydown (except esc) is not working in edit dialog without timeout
            defer(function() { self.editRequest(leaveTimeRequest, selectionRegion, showAt, null); })
        };

        // add needed items for menu
        var menuItems = [];
        var isPastRequest = leaveTimeRequest.past;
        if (leaveTimeRequest.waitingDeletion)
        {
            menuItems.push(
                    {
                        text: messageResource.getMessage("actiplans.leavestable.context_menu.cancel_deletion"),
                        listeners: {click: revokeModificationOrDeletionCallback},
                        href: "javascript: void(0)",
                        cls: "cancelDeletionContextButton"
                    }
            );
        }
        else if (leaveTimeRequest.requestChange)
        {

            if (this.pageData.myScheduleData.canManagePastRequests)
            {
                menuItems.push(
                        {
                            text: messageResource.getMessage("actiplans.leavestable.context_menu.edit_past"),
                            listeners: {click: editRequestCallback},
                            href: "javascript: void(0)",
                            cls: "editContextButton"
                        },
                        new Ext.menu.Separator()
                );
            }
            menuItems.push(
                    {
                        text: messageResource.getMessage("actiplans.leavestable.context_menu.cancel_modification"),
                        listeners: {click: revokeModificationOrDeletionCallback},
                        href: "javascript: void(0)",
                        cls: "cancelModificationContextButton"
                    }
            );
        }
        else
        {
            var editRequestMessageKey = isPastRequest ? "actiplans.leavestable.context_menu.edit_past" : "actiplans.leavestable.context_menu.edit";
            var deleteRequestMessageKey = isPastRequest ? "actiplans.leavestable.context_menu.delete_past" : "actiplans.leavestable.context_menu.delete";
            var deleteRequestCallback = function()
            {
                if (confirm(messageResource.getMessage(isPastRequest ?
                                                       "actiplans.leave_time_requests.delete_request_confirmation.past" :
                                                       "my_schedule.js.delete_request_confirmation")))
                {
                    self.deleteRequestAndUpdateData(leaveTimeRequest, selectionRegion);
                }
            };
            // if there is no integration with aT - show edit button for past requests
            var isUserCanEditRequest = this.pageData.myScheduleData.canManagePastRequests || !isPastRequest;
            if (isUserCanEditRequest)
            {
                menuItems.push(
                        {
                            text: messageResource.getMessage(editRequestMessageKey),
                            listeners: {click: editRequestCallback},
                            href: "javascript: void(0)",
                            cls: "editContextButton"
                        }
                );
                menuItems.push(new Ext.menu.Separator());
            }
            menuItems.push(
                    {
                        text: messageResource.getMessage(deleteRequestMessageKey),
                        listeners: {click: deleteRequestCallback},
                        href: "javascript: void(0)",
                        cls: "deleteContextButton"
                    }
            );
        }
        this.showContextMenu(showAt, menuItems);
    },

    editRequest: function(leaveTimeRequest, selectionRegion, showAt, offset, closeDialogCallback)
    {
        if (this.requestMenu)
        {
            // this helps to switch between menus of different requests
            this.requestMenu.hide();
        }

        // copy the request object.
        leaveTimeRequest = jQuery.extend({}, leaveTimeRequest);
        var self = this;
        var successor = {

            closed: function()
            {
                self.myScheduleTableController.clearSelection();
                self.requestDialog = null;
                if (closeDialogCallback)
                    closeDialogCallback();
            },

            cancelled: function()
            {
                self.requestDialog.close();
            }
        };

        var userIsPTO = this.pageData.myScheduleData.userSchedule.userInfo.ptoEnabled;
        var userIsSick = this.pageData.myScheduleData.userSchedule.userInfo.sickEnabled;
        var ptoEnabled = this.pageData.myScheduleData.ptoCalculationEnabled;
        var sickEnabled = this.pageData.myScheduleData.sickCalculationEnabled;

        if ((!leaveTimeRequest.past || this.pageData.myScheduleData.canManagePastRequests) && leaveTimeRequest.status != LeaveTimeRequestStatus.REJECTED)
        {
            successor.saveRequest = function(request)
            {
                self.saveRequestChangesAndUpdateData(request, selectionRegion);
            };

            this.requestDialog = new EditLeaveRequestDialog(
                    leaveTimeRequest,
                    userIsPTO && ptoEnabled,
                    userIsSick && sickEnabled,
                    this.pageData.myScheduleData.userSchedule.userInfo,
                    successor,
                    $.proxy(self.createRequestRecalculationRPCManager, self),
                    self.pageData.ptoHoursPerDay,
                    this.pageData.myScheduleData.userSchedule.currentBalanceInfo,
                    false
            ).init();

        }
        else
        {
            this.requestDialog = new ViewLeaveRequestDialog(
                    leaveTimeRequest,
                    userIsPTO && ptoEnabled,
                    userIsSick && sickEnabled,
                    successor,
                    this.pageData.ptoHoursPerDay,
                    this.pageData.myScheduleData.userSchedule.currentBalanceInfo
            ).init();
        }

        this.requestDialog.showAt(showAt, offset);
    },

    showContextMenu: function(showAt, items)
    {
        if (this.requestMenu)
        {
            this.requestMenu.hide();
        }
        else
        {
            this.requestMenu = new Ext.menu.Menu({
                items: items,
                cls: "at-dropdown-list-btn-menu leaveRequestContextMenu"
            });

            var self = this;
            this.requestMenu.addListener("hide", function()
            {
                self.myScheduleTableController.clearSelection();
                self.requestMenu = null;
            });
            doOnceOnWindowResize(function()
                                 {
                                     if (self.requestMenu)
                                         self.requestMenu.hide();
                                 });

            this.requestMenu.show(showAt);
        }
    }
});