/**
 * @author filatov
 */
var LeaveTimeRequestUtils = {
    getLeaveTypeIdFromRequestWithChanges: function(request)
    {
        return request.requestChange ? request.requestChange.leaveTypeId : request.leaveTypeId;
    },

    getCommentFromRequestWithChanges: function(request)
    {
        return request.requestChange ? request.requestChange.comment : request.comment;
    },

    getStatusFromRequestWithChanges: function(request)
    {
        var status = {};
        if (request.requestChange)
        {
            status.code = LeaveTimeRequestStatus.REQUESTED;
            status.statusDate = request.requestChange.timestamp;
            status.waitingDeletion = false;
        }
        else
        {
            status.code = request.status;
            status.approverInfo = request.approverInfo;
            status.statusMessage = request.statusMessage;
            status.waitingDeletion = request.waitingDeletion;
            status.statusDate = request.statusDate;
        }
        return status;
    },

    changedTodayBalanceTuple: function(request)
    {
        var changedTodayBalance = request.changedTodayBalance;
        var leaveType = LEAVE_TYPES[request.leaveTypeId];

        if (_.isNumber(changedTodayBalance))
        {
            return {
                PTOBalance: leaveType.balanceType == BalanceType.PTO ? changedTodayBalance : null,
                sickBalance: leaveType.balanceType == BalanceType.SICK ? changedTodayBalance : null
            }
        }
        else if (_.has(changedTodayBalance, "PTOBalance") && _.has(changedTodayBalance, "sickBalance"))
        {
            return changedTodayBalance;
        }
        return { PTOBalance: null, sickBalance: null };
    },

    getBalanceValueFromRequestWithChanges: function(request, userInfo)
    {
        var balanceValue = {};

        balanceValue.changedTodayBalance = this.changedTodayBalanceTuple(request);

        if (request.requestChange && request.requestChange.changedTodayBalance)
        {
            balanceValue.changedTodayBalance = request.requestChange.changedTodayBalance;
        }

        var balanceData = request.balanceData;
        if (balanceData && balanceData.balanceChangeData)
        {
            balanceValue.before = balanceData.balanceChangeData.startBalance;
            balanceValue.after = balanceData.balanceChangeData.endBalance;
        }

        if (balanceData && balanceData.balanceChangeData || balanceValue.changedTodayBalance)
        {
            balanceValue.applicable = true;
        }

        balanceValue.startBeforeHireDate = Date.parseJSONDate(request.firstDate) < Date.parseJSONDate(userInfo.hireDate);
        balanceValue.endBeforeHireDate = Date.parseJSONDate(request.lastDate) < Date.parseJSONDate(userInfo.hireDate);

        return balanceValue;
    },

    getLeaveTimeTotalValueFromRequestWithChanges: function(request)
    {
        return request.requestChange ? request.requestChange.leaveTimeTotal : request.leaveTimeTotal;
    },

    mergeChangesToRequest: function(request)
    {
        if (request.requestChange)
        {
            request.leaveTypeId = request.requestChange.leaveTypeId;
            request.firstDayDuration = request.requestChange.firstDayDuration;
            request.lastDayDuration = request.requestChange.lastDayDuration;
            request.middleDayDuration = request.requestChange.middleDayDuration;
            request.comment = request.requestChange.comment;
        }
        return request;
    },

    getChangeTodayBalanceCellContentForLeaveRequestTables: function(changedTodayBalance, isPtoEnabled, isSickEnabled, ptoFormatter, sickFormatter)
    {
        var content = "";

        function getValueClasses(formatter, value)
        {
            return "changedValue" + (formatter.isNegativePTO(value) ? " negative" : "");
        }

        if (!_.isNumber(changedTodayBalance.PTOBalance) && !_.isNumber(changedTodayBalance.sickBalance))
        {
            var messageKey;
            if (isPtoEnabled && isSickEnabled)
            {
                messageKey = "requests_table.balance_cell.doesnt_change_balances";
            }
            else
            {
                messageKey = "requests_table.balance_cell.doesnt_change_one_balance";
            }
            content = createDiv("changedBalance", createSpan("label", messageResource.getMessage(messageKey)));
            if (!isPtoEnabled && isSickEnabled) content += createDiv("sickSpan", "Sick Days Balance");
        }
        else if (_.isNumber(changedTodayBalance.PTOBalance) && _.isNumber(changedTodayBalance.sickBalance))
        {
            content = createDiv("changedBalance",
                                createSpan("label", createSpan("label", messageResource.getMessage("requests_table.balance_cell.changes_balances"))) +
                                createDiv("balances",
                                createSpan("balance PTO",
                                createSpan("label", messageResource.getMessage("requests_table.balance_cell.changes_balances.pto")) +
                                createSpan(getValueClasses(ptoFormatter, changedTodayBalance.PTOBalance), ptoFormatter.format(changedTodayBalance.PTOBalance))) +
                                createSpan("balance Sick",
                                createSpan("label", messageResource.getMessage("requests_table.balance_cell.changes_balances.sick")) +
                                createSpan(getValueClasses(sickFormatter, changedTodayBalance.sickBalance),
                                           sickFormatter.format(changedTodayBalance.sickBalance)))));
        }
        else
        {
            var formatter = _.isNumber(changedTodayBalance.PTOBalance) ? ptoFormatter : sickFormatter;
            var balanceValue = _.isNumber(changedTodayBalance.PTOBalance) ? changedTodayBalance.PTOBalance : changedTodayBalance.sickBalance;
            content = createDiv("changedBalance",
                      createSpan("label", messageResource.getMessage("requests_table.balance_cell.changes_balance")) +
                      createSpan(getValueClasses(formatter, balanceValue), formatter.format(balanceValue))) + (_.isNumber(
                            changedTodayBalance.sickBalance) ? createDiv("sickSpan",
                                                                         "Sick Days Balance") : "");

        }

        return content;
    },

    getRequestedBalanceFromLeaveTime: function(leaveTimeTotal, leaveType)
    {
        var requestedBalance = leaveTimeTotal.days;
        if (leaveType && leaveType.pto)
            requestedBalance *= leaveType.ptoCoefficient;
        return requestedBalance;
    },

    isPastApproved: function(request)
    {
        return request && request.past && request.status == LeaveTimeRequestStatus.APPROVED && !request.requestChange && !request.waitingDeletion && !request.recalculated;
    },

    viewBalanceAsType: function(request, isPTOUser, isSickUser, isBeforeHireDate)
    {
        if (isBeforeHireDate)
            return this.ViewBalanceAs.DOES_NOT_AFFECT_BALANCE;
        if (this.isPastApproved(request))
            return this.ViewBalanceAs.DO_NOT_VIEW;

        var changesCurrentBalance = _.isNumber(request.changedTodayBalance) || this.changedTodayBalanceObjectNotEmpty(request.changedTodayBalance) ||
                                    request.requestChange && this.changedTodayBalanceObjectNotEmpty(request.requestChange.changedTodayBalance);
        if (changesCurrentBalance)
            return this.ViewBalanceAs.CURRENT_BALANCE_BEFORE_AFTER_APPROVAL;
        else
        {
            var leaveType = LEAVE_TYPES[request.leaveTypeId];
            var leaveTypeAffectsBalance = this.leaveTypeAffectsBalance(leaveType, isPTOUser, isSickUser);
            if (request.requestChange || request.waitingDeletion || request.past)
            {
                if (request.requestChange)
                {
                    var newLeaveType = LEAVE_TYPES[request.requestChange.leaveTypeId];
                    leaveTypeAffectsBalance |= this.leaveTypeAffectsBalance(newLeaveType, isPTOUser, isSickUser);
                }
                if (request.recalculated)
                {
                    var oldLeaveType = LEAVE_TYPES[request.leaveTypeIdOriginal];
                    leaveTypeAffectsBalance |= this.leaveTypeAffectsBalance(oldLeaveType, isPTOUser, isSickUser);
                }

                if (leaveTypeAffectsBalance)
                    return this.ViewBalanceAs.DOES_NOT_AFFECT_CURRENT_BALANCE;
                else
                    return this.ViewBalanceAs.DOES_NOT_AFFECT_BALANCE;
            }
            else
            {
                if (leaveTypeAffectsBalance)
                    return this.ViewBalanceAs.BALANCE_ON_START_END;
                else
                    return this.ViewBalanceAs.DOES_NOT_AFFECT_BALANCE;
            }
        }
    },

    requestRequiresApprovalType: function(request, autoApproved)
    {
        if (autoApproved)
            return this.RequiresApproval.DOES_NOT_REQUIRE;

        if (this.isPastApproved(request))
            return this.RequiresApproval.DOES_NOT_REQUIRE;

        var leaveType = LEAVE_TYPES[request.leaveTypeId];
        var leaveTypeRequiresApproval = _.isObject(leaveType) && leaveType.requiresApproval;

        var requestChangesCurrentBalance = _.isNumber(request.changedTodayBalance) || this.changedTodayBalanceObjectNotEmpty(request.changedTodayBalance);
        if (request.requestChange || request.waitingDeletion || request.past)
        {
            var currentBalanceChanges;
            var newLeaveType;
            var oldLeaveType;
            var oldLeaveTypeRequiresApproval;
            var newLeaveTypeRequiresApproval;

            if (request.requestChange)
            {
                currentBalanceChanges = this.changedTodayBalanceObjectNotEmpty(request.requestChange.changedTodayBalance);
                oldLeaveTypeRequiresApproval = leaveTypeRequiresApproval;
                newLeaveType = LEAVE_TYPES[request.requestChange.leaveTypeId];
                newLeaveTypeRequiresApproval = _.isObject(newLeaveType) && newLeaveType.requiresApproval;
            }
            else
            {
                currentBalanceChanges = requestChangesCurrentBalance;
                if (request.recalculated)
                {
                    oldLeaveType = LEAVE_TYPES[request.leaveTypeIdOriginal];
                    oldLeaveTypeRequiresApproval = _.isObject(oldLeaveType) && oldLeaveType.requiresApproval;
                }
                newLeaveTypeRequiresApproval = leaveTypeRequiresApproval;
            }

            if (currentBalanceChanges)
                return this.RequiresApproval.REQUIRES_BECAUSE_CURRENT_BALANCE_CHANGES;
            if (request.status == LeaveTimeRequestStatus.APPROVED && oldLeaveTypeRequiresApproval || newLeaveTypeRequiresApproval)
                return this.RequiresApproval.REQUIRES_BECAUSE_OF_LEAVE_TYPE;
        }

        return this.RequiresApproval.DOES_NOT_REQUIRE;
    },

    changedTodayBalanceObjectNotEmpty: function(changedTodayBalance)
    {
        return _.isObject(changedTodayBalance) && (_.isNumber(changedTodayBalance.PTOBalance) || _.isNumber(changedTodayBalance.sickBalance));
    },

    leaveTypeAffectsBalance: function(leaveType, isPTOUser, isSickUser, beforeHireDate)
    {
        return !beforeHireDate && _.isObject(leaveType) && (leaveType.balanceType == BalanceType.PTO && isPTOUser || leaveType.balanceType == BalanceType.SICK && isSickUser);
    }
};

LeaveTimeRequestUtils.ViewBalanceAs = {
    DO_NOT_VIEW: 0,
    BALANCE_ON_START_END: 1,
    CURRENT_BALANCE_BEFORE_AFTER_APPROVAL: 2,
    DOES_NOT_AFFECT_BALANCE: 3,
    DOES_NOT_AFFECT_CURRENT_BALANCE: 4
};

LeaveTimeRequestUtils.RequiresApproval = {
    DOES_NOT_REQUIRE: 0,
    REQUIRES_BECAUSE_CURRENT_BALANCE_CHANGES: 1,
    REQUIRES_BECAUSE_OF_LEAVE_TYPE: 2
};