create table at_user
(
    id counter not null,
    username varchar(32) not null,
    username_lower varchar(32) not null,
    md5_password varchar(32) not null,
    first_name varchar(32) not null,
    middle_name varchar(32),
    last_name varchar(32) not null,
    email varchar(255) not null,
    phone varchar(32),
    fax varchar(32),
    mobile varchar(32),
    other_contact varchar(32),
    is_root bit not null,
    is_enabled bit not null,

    constraint pk_at_user primary key(id),
    constraint nl_username unique(username_lower)
);

insert into at_user
(
    id,
    username,
    username_lower,
    md5_password,
    first_name,
    middle_name,
    last_name,
    email,
    phone,
    fax,
    mobile,
    other_contact,
    is_root,
    is_enabled
)
values
(
    '1',
    'admin',
    'admin',
    '1d0258c2440a8d19e716292b231e3190',
    'Administrator',
    '',
    'System',
    '',
    '',
    '',
    '',
    '',
    1,
    1
);

create table access_right
(
    id counter not null,
    name varchar(50) not null,
    description varchar(255) not null,
    constraint pk_access_right primary key(id)
);

insert into access_right (id, name, description) values (1, 'reporting.generate_reports', 'Generate Reports');
insert into access_right (id, name, description) values (2, 'tasks.manage_customers', 'Manage Customers');
insert into access_right (id, name, description) values (3, 'tasks.manage_projects', 'Manage Projects');
insert into access_right (id, name, description) values (4, 'tasks.manage_tasks', 'Manage Tasks');
insert into access_right (id, name, description) values (5, 'system.manage_users', 'Manage Users');
-- insert into access_right (id, name, description) values (6, 'system.manage_preferences', 'Manage System Preferences');
insert into access_right (id, name, description) values (7, 'system.manage_billing_types', 'Manage Billing Types');
insert into access_right (id, name, description) values (8, 'reporting.prepare_invoice_data', 'Prepare Invoice Data');

create table user_access_right
(
    user_id int not null,
    access_right_id int not null,
    constraint pk_user_right primary key(user_id, access_right_id),
    constraint fk_ur_user foreign key(user_id) references at_user(id),
    constraint fk_ur_right foreign key(access_right_id) references access_right(id)
);


create table billing_type
(
  id counter not null,
  name varchar(255),
  name_lower varchar(255),
  constraint pk_billing_type primary key(id),
  constraint nl_billing_type unique(name_lower)
);

insert
  into billing_type (id, name, name_lower)
  values (1, 'Billable', 'billable');

create table customer
(
  id counter not null,
  create_timestamp datetime not null,
  name varchar(209) not null,
  name_lower varchar(209) not null,
  description longtext,
  billing_status bit not null,
  constraint pk_customer primary key(id),
  constraint nl_customer unique(name_lower)
);

create table project
(
  id counter not null,
  customer_id int not null,
  create_timestamp datetime not null,
  name varchar(255) not null,
  name_lower varchar(255) not null,
  description longtext,
  billing_status bit not null,
  default_billing_type_id int,
  constraint pk_project primary key(id),
  constraint cn_project unique(customer_id, name_lower),
  constraint fk_project_billing_type foreign key(default_billing_type_id) references billing_type(id),
  constraint fk_project_customer foreign key(customer_id) references customer(id)
);

create table task
(
  id counter not null,
  project_id int not null,
  create_timestamp datetime not null,
  completion_date date,
  last_tt_date date,
  name varchar(255) not null,
  name_lower varchar(255) not null,
  description longtext,
  deadline_date date,
  billing_status bit not null,
  billing_type_id int,
  constraint pk_task primary key(id),
  constraint pn_task unique(project_id, name_lower),
  constraint fk_task_billing_type foreign key(billing_type_id) references billing_type(id),
  constraint fk_task_project foreign key(project_id) references project(id)
);

create table user_task
(
  user_id int not null,
  task_id int not null,
  constraint pk_user_task primary key(user_id, task_id),
  constraint fk_ut_user foreign key(user_id) references at_user(id),
  constraint fk_ut_task foreign key(task_id) references task(id)
);

create table tt_record
(
  user_id int not null,
  task_id int not null,
  record_date date not null,
  actuals int not null,
  constraint pk_tt_record primary key(user_id, task_id, record_date),
  constraint fk_tt_user foreign key(user_id) references at_user(id),
  constraint fk_tt_task foreign key(task_id) references task(id)
);