-- drop table if exists access_right_group;
-- create table access_right_group (
--     id int(3) unsigned not null auto_increment,
--     name varchar(50) not null default '0',
--     description varchar(255) not null default '0',
--     is_enabled bit default '1',
--     primary key (id),
--     index (is_enabled)
-- ) ENGINE=InnoDB comment='Access rights can be grouped';

ALTER DATABASE DEFAULT CHARACTER SET utf8 DEFAULT COLLATE utf8_unicode_ci;

drop table if exists access_right;
create table access_right (
    id int(3) unsigned not null auto_increment,
--     group_id int(3) unsigned not null default '0',
    name varchar(50) not null default '',
    description varchar(255) not null default '0',
--     is_enabled bit not null default '1',
--     foreign key group_id (group_id) references access_right_group(id),
    primary key  (id)
) ENGINE=InnoDB comment='A dictionary of all access rights used by system';

insert into access_right (id, name, description) values (1, 'reporting.generate_reports', 'Generate Reports');
insert into access_right (id, name, description) values (2, 'tasks.manage_customers', 'Manage Customers');
insert into access_right (id, name, description) values (3, 'tasks.manage_projects', 'Manage Projects');
insert into access_right (id, name, description) values (4, 'tasks.manage_tasks', 'Manage Tasks');
insert into access_right (id, name, description) values (5, 'system.manage_users', 'Manage Users');
-- insert into access_right (id, name, description) values (6, 'system.manage_preferences', 'Manage System Preferences');
insert into access_right (id, name, description) values (7, 'system.manage_billing_types', 'Manage Billing Types');
insert into access_right (id, name, description) values (8, 'reporting.prepare_invoice_data', 'Prepare Invoice Data');

drop table if exists at_user;
create table at_user (
    id int(3) unsigned not null auto_increment,
    username varchar(32) not null default '',
    username_lower varchar(32) not null default '',
    md5_password varchar(32) not null default '',
    first_name varchar(32) not null default '',
    middle_name varchar(32) default null,
    last_name varchar(32) not null default '',
    email varchar(255) not null default '',
    phone varchar(32) default null,
    fax varchar(32) default null,
    mobile varchar(32) default null,
    other_contact varchar(32) default null,
    is_root bit not null default 0,
    is_enabled bit not null default 1,
    primary key (id),
    index (is_root),
    index (is_enabled),
    unique key username (username_lower)
) ENGINE=InnoDB comment='User';

insert into at_user 
(
    id,
    username,
    username_lower,
    md5_password,
    first_name,
    middle_name,
    last_name,
    email,
    phone,
    fax,
    mobile,
    other_contact,
    is_root,
    is_enabled
)
values
(
    '1',
    'admin',
    'admin',
    md5('manager'),
    'Administrator',
    '',
    'System',
    '',
    '',
    '',
    '',
    '',
    1,
    1
);

drop table if exists user_access_right;
create table user_access_right (
    user_id int(3) unsigned not null default '0',
    access_right_id int(3) unsigned not null default '0',
    primary key (user_id,access_right_id),
    index (user_id),
    foreign key (user_id) references at_user(id),
    index (access_right_id),
    foreign key (access_right_id) references access_right(id)
) ENGINE=InnoDB comment='A list of access rights users have';

drop table if exists tt_record;

drop table if exists user_task;

drop table if exists task;

drop table if exists project;

drop table if exists customer;

drop table if exists billing_type;

create table billing_type
(
  id int(3) null,
  name varchar(255),
  name_lower varchar(255),
  unique(id),
  unique(name_lower)
) ENGINE=InnoDB;

insert
  into billing_type (id, name, name_lower)
  values (null, null, null);

insert
  into billing_type (id, name, name_lower)
  values (1, 'Billable', 'billable');

create table customer
(
  id int(3) not null auto_increment,
  create_timestamp datetime not null,
  name varchar(209) not null,
  name_lower varchar(209) not null,
  description text,
  billing_status bit not null,
  primary key(id),
  unique(name_lower)
) ENGINE=InnoDB;

create table project
(
  id int(3) not null auto_increment,
  customer_id int not null,
  create_timestamp datetime not null,
  name varchar(255) not null,
  name_lower varchar(255) not null,
  description text,
  billing_status bit not null,
  default_billing_type_id int,
  primary key(id),
  unique(customer_id, name_lower),
  index(default_billing_type_id),
  foreign key(default_billing_type_id) references billing_type(id),
  index(customer_id),
  foreign key(customer_id) references customer(id)
) ENGINE=InnoDB;

create table task
(
  id int(5) not null auto_increment,
  project_id int not null,
  create_timestamp datetime not null,
  completion_date date,
  last_tt_date date,
  name varchar(255) not null,
  name_lower varchar(255) not null,
  description text,
  deadline_date date,
  billing_status bit,
  billing_type_id int,
  primary key(id),
  unique(project_id, name_lower),
  index(billing_type_id),
  foreign key(billing_type_id) references billing_type(id),
  index(project_id),
  foreign key(project_id) references project(id)
) ENGINE=InnoDB;

create table user_task
(
  user_id int(3) unsigned not null,
  task_id int(5) not null,
  primary key(user_id, task_id),
  index(user_id),
  foreign key(user_id) references at_user(id),
  index(task_id),
  foreign key(task_id) references task(id)
) ENGINE=InnoDB;

create table tt_record
(
  user_id int(3) unsigned not null,
  task_id int(5) not null,
  record_date date not null,
  actuals int not null,
  primary key(user_id, task_id, record_date),
  index(user_id),
  foreign key(user_id) references at_user(id),
  index(task_id),
  foreign key(task_id) references task(id)
) ENGINE=InnoDB;

commit;
