function ChangeOtherWeekApprovalStatusMediator ()
{
    this.constructor = ChangeOtherWeekApprovalStatusMediator;
}

ChangeOtherWeekApprovalStatusMediator.prototype = _.create(ChangeWeekApprovalStatusMediator.prototype, {

    init: function(userData, dateFrom, dateTo, weekStatus, disabledControl, loggedInUserId, lastChangesTimestampStr)
    {
        this.userData = userData;
        this.spinnerController = new SpinnerController();
        this.lastChangesTimestampStr = lastChangesTimestampStr;

        var self = this;
        new ChangeOtherWeekApprovalStatusControl().init(
                userData, dateFrom, dateTo, weekStatus, disabledControl, loggedInUserId,
                function() { self.processApproval.apply(self, arguments) },
                function() { self.processRejection.apply(self, arguments) },
                function() { self.processRevoke.apply(self, arguments) },
                function() { self.getWeekStatusHistory.apply(self, arguments) }
        );

        return this;
    },

    processApproval: function(userId, dateFrom, spinnerPl, successCallback, errorCallback, disableControlCallback, enableControlCallback)
    {
        if (!this.timeEntriesChanged)
        {
            if (disableControlCallback())
                this._approve(userId, dateFrom, this.lastChangesTimestampStr, spinnerPl, successCallback, errorCallback);
        }
        else
            this._openApproveRejectConfirmDialog("change_week_approval_status.confirm_dialog.approve.header",
                                                 "change_week_approval_status.confirm_dialog.approve_save.caption",
                                                 function()
                                                 {
                                                     if (disableControlCallback() && !submitSubmitTTFormWithApprovalStatus(
                                                                     EffectiveWeekApprovalStatus.APPROVED))
                                                         enableControlCallback();
                                                 });
    },

    processRejection: function(userId, dateFrom, comment, spinnerPl, successCallback, errorCallback, disableControlCallback, enableControlCallback)
    {
        if (!this.timeEntriesChanged)
        {
            if (disableControlCallback())
                this._reject(userId, dateFrom, comment, this.lastChangesTimestampStr, spinnerPl, successCallback, errorCallback);
        }
        else
            this._openApproveRejectConfirmDialog("change_week_approval_status.confirm_dialog.reject.header",
                                                 "change_week_approval_status.confirm_dialog.reject_save.caption",
                                                 function()
                                                 {
                                                     if (disableControlCallback() && !submitSubmitTTFormWithApprovalStatusAndComment(
                                                                     EffectiveWeekApprovalStatus.REJECTED, comment))
                                                         enableControlCallback();
                                                 });
    },

    processRevoke: function(userId, dateFrom, visibleStatus, spinnerPl, successCallback, errorCallback)
    {
        this.callChangeStatusServiceMethod("EnterTTService.revokeUserWeekStatus",
                                           [userId, dateFrom, visibleStatus],
                                           spinnerPl,
                                           successCallback, errorCallback
        );
    },

    _approve: function(userId, dateFrom, lastChangesTimestampStr, spinnerPl, successCallback, errorCallback)
    {
        this.callChangeStatusServiceMethod("EnterTTService.approveUserWeek",
                                           [userId, dateFrom, lastChangesTimestampStr],
                                           spinnerPl,
                                           successCallback, errorCallback
        );
    },

    _reject: function(userId, dateFrom, comment, lastChangesTimestampStr, spinnerPl, successCallback, errorCallback)
    {
        this.callChangeStatusServiceMethod("EnterTTService.rejectUserWeek",
                                           [userId, dateFrom, comment, lastChangesTimestampStr],
                                           spinnerPl,
                                           successCallback, errorCallback
        );
    },

    setTimeEntriesChanged: function()
    {
        this.timeEntriesChanged = true;
    },

    commonSuccessHandler: function(result)
    {
        setApprovalStatusToSubmitTTForm(result.effectiveWeekStatus);

        var warnings = result.warnings;
        var needToReloadPage = false;
        if (warnings && warnings.length > 0)
        {
            var alertText = "";
            for (var i = 0; i < warnings.length; i++)
            {
                if (warnings[i] == SubmitTTConstants.WARNING_STATUS_HAS_ALREADY_BEEN_SET)
                    alertText += messageResource.getMessage("change_week_approval_status.status_already_set.warning",
                                                            [messageResource.getMessage(result.effectiveWeekStatus == EffectiveWeekApprovalStatus.APPROVED ?
                                                                                        "change_week_approval_status.week_approved" :
                                                                                        "change_week_approval_status.week_rejected")]);
                else if (warnings[i] == SubmitTTConstants.WARNING_STATUS_HAS_BEEN_CHANGED_CONCURRENTLY)
                    alertText += messageResource.getMessage("change_week_approval_status.status_has_been_changed.warning");
                else if (warnings[i] == SubmitTTConstants.WARNING_TTA_FEATURE_IS_DISABLED)
                {
                    alertText += messageResource.getMessage("userlist.js.error_tta_feature_disabled");
                    needToReloadPage = true;
                }

                alertText += "\n\n";
            }
            alert(alertText);
        }

        if (needToReloadPage)
            reloadSubmitTTPageWithoutFormReloading();
    },

    callChangeStatusServiceMethod: function(service, params, spinnerPl, successCallback, errorCallback)
    {
        var self = this;

        this._startChangeStatusSpinner(spinnerPl);
        makeRpcCall(service,
                    params,
                    function(result)
                    {
                        self._stopChangeStatusSpinner();

                        if (successCallback)
                            successCallback(result.effectiveWeekStatus);

                        self.commonSuccessHandler(result);
                    },
                    function(jqXHR, textStatus, errorThrown)
                    {
                        self._stopChangeStatusSpinner();

                        if (errorCallback && errorCallback())
                            return true;

                        return self.commonErrorHandler(jqXHR, textStatus, errorThrown);
                    }
        );
    },

    _openApproveRejectConfirmDialog: function(headerTextKey, deleteButtonCaptionKey, onApplyCallback)
    {
        operationConfirmDlg.initWithData(messageResource.getMessage(headerTextKey),
                                         [messageResource.getMessage(deleteButtonCaptionKey)],
                                         messageResource.getEscapedMessage("change_week_approval_status.confirm_dialog.text", [this.formatUserInfo()]),
                                         messageResource.getMessage("change_week_approval_status.confirm_dialog.do_nothing.caption"),
                                         {onConfirm: onApplyCallback},
                                         true, true);
        operationConfirmDlg.open(false);
    },

    formatUserInfo: function()
    {
        return UserNameFormat.formatFML(this.userData.firstName, this.userData.lastName, this.userData.middleName);
    },

    _startChangeStatusSpinner: function(spinnerPl)
    {
        this.spinnerController.spinInElement(spinnerPl);
    },

    _stopChangeStatusSpinner: function()
    {
        this.spinnerController.stop();
    }
});