function CodePhoneSelector() {}

CodePhoneSelector.prototype.init = function( config )
{
    this.containerId = config.containerId;
    this.mainSelectorClass = "countryCodeSelector";
    this.settings = {
        showDropdownToTop: config.showToTop,
        isFixedDropDown: config.isFixedDropDown,
        preSelection: false,
        hideDropdownOnEmptyInput: false,
        selectedClass: this.mainSelectorClass + "_selected",
        dropdownClass: this.mainSelectorClass +"_dropdown " + (config.showToTop ? " toTop" : " toBottom") + (config.isFixedDropDown ? " fixed" : "")
    };
    this.select = null;
    this.input = null;
    this.dropdown = null;
    this.dropdownList = null;
    this.cache = [];
    this.results = [];
    this.picked = false;
    this.valueBeforeFocus = null;
    this.selectedIndex = 0;
    this.allowMouseMove = true;
    this.options = config.options;
    this.createTextFunction = config.createTextFunction;
    this.createNameFunction = config.createNameFunction;
    this.setInputByValueFunction = config.setInputByValueFunction;
    this.getTokenFunction = config.getTokenFunction;
    this.searchIndex = new StringIndex( new StringTokenizer( StringTokenizer.PUNCTUATORS_FOR_SEARCH ) );
    this.predefinedValue = config.predefinedValue;
    this.reloadCache();
    this.renderControls();
    this.wire();
    return this;
};

CodePhoneSelector.prototype.reloadCache = function()
{
    var self = this;

    this.cache = $.map( this.options, function( value, index )
    {
        self.searchIndex.add( self.getTokenFunction( value ), index );
        return {
            name: self.createNameFunction( value ),
            value: value
        }
    } );
};

CodePhoneSelector.prototype.renderControls = function()
{
    this.input = $("<input type='text' autocomplete='off' />")
            .addClass(this.mainSelectorClass + "Input" + "_selector")
            .css({
                     visibility: 'visible'
                 });

    this.input.on( 'input', function()
    {
        this.value = this.value.replace( /[^/+0-9]/g, '' );
        this.value = this.value.replace( /(\d)\+/g, '$1' );
        this.value = this.value.replace( /\+\+/g, '+' );
    } );
    this.input.on("selectstart", function(e){stopEventPropagation(e);});

    this.input.focus(function(){self.container.addClass("inFocus");});
    this.input.blur(function()
                    {
                        self.container.removeClass("inFocus");
                    });

    this.dropdown = $("<div></div>").addClass(this.settings.dropdownClass);
    this.dropdownListEl = "<ul class='codePhoneSelectorDropdown'></ul>";
    this.dropdownList = $( this.dropdownListEl );
    this.dropdown.append( this.dropdownList );
    this.dropdown.css( 'max-height', '200px');

    this.container = $( "#" + this.containerId );
    this.container.append( this.input );

    var charFilter = function( charCode )
    {
        var sym = String.fromCharCode( charCode );
        return sym >= '0' && sym <= '9'
               || sym == '+'
               || sym == '-'
    };
    new SmartField( this.input, charFilter, function() {}, null, false, 10 );

    this.arrow = $("<div></div>")
            .addClass(this.mainSelectorClass + "Arrow" + "_selector")
            .css({
                     visibility: 'visible'
                 });

    this.container.append( this.arrow );
    $( "body" ).append( this.dropdown );
    if(this.predefinedValue){
        this.input.val(this.predefinedValue);
        this.initialSelect = true;
    }
    var self = this;
    $( window ).resize(function(){self.hide()});
};

CodePhoneSelector.prototype.wire = function()
{
    var self = this;

    this.input.click( function( event )
                      {
                          self.focus();
                          event.stopPropagation();
                      } );

    this.input.mouseup( function( event )
                        {
                            // This is so Safari selection actually occurs.
                            event.preventDefault();
                        } );

    this.input.focus( function( event )
                      {
                          self.valueBeforeFocus = self.input.val();
                          if(!self.picked) self.filterResults();
                          event.stopPropagation();
                      } );

    this.input.blur( function()
                     {
                         var value = self.input.val();
                         if (value[0] != '+' && value != "") self.input.val("+" + value);
                     } );

    this.arrow.mousedown( function( event )
                      {
                          if(self.container.hasClass("showAll")){
                              self.hide();
                          }
                          else
                          {
                              self.input.focus();
                              self.hide();
                              self.filterResults( true );
                              self.container.addClass( "showAll" );
                              event.stopPropagation();
                          }
                      } );

    this.dropdownList.mouseover( function( event )
                                 {
                                     if( !self.allowMouseMove )
                                     {
                                         self.allowMouseMove = true;
                                         return;
                                     }

                                     if( event.target.tagName == "LI" )
                                     {
                                         var rows = self.dropdown.find( "li" );
                                         self.markSelected( rows.index( $( event.target ) ) );
                                     }
                                 } );
    this.dropdownList.mouseleave( function()
                                  {
                                      self.markSelected( -1 );
                                  } );
    this.dropdownList.click(function()
                            {
                                stopEventPropagation(event);
                                cancelBubble(event);
                                self.pickSelected();
                                self.focusAndHide();
                            });

    this.dropdown.mouseover( function() {self.dropdownMouseover = true; } );
    this.dropdown.mouseleave( function()
                              {
                                  self.dropdownMouseover = false;
                              } );
    $(document).on("mousedown.codePhoneSelector", function(event)
    {
        // hide if click not under input and dropdown
       if(event.target != self.dropdown[0] && event.target != self.input[0] && $(event.target).parents().index(self.dropdown) == -1)
       {
           self.hide();
       }
    });

    this.input.keyup( function( event )
                      {
                          switch( event.keyCode )
                          {
                              case KEY_CODE_ENTER: // return
                                  event.preventDefault();
                                  self.pickSelected();
                                  self.focusAndHide();
                                  break;
                              case KEY_CODE_ESC: // esc
                                  event.preventDefault();
                                  self.reset();
                                  self.focusAndHide();
                                  break;
                              default:
                                  self.filterResults();
                                  break;
                          }
                          if( self.settings.hideDropdownOnEmptyInput )
                          {
                              if( self.input.val() == "" )
                                  self.dropdownList.hide();
                              else
                                  self.dropdownList.show();
                          }
                      } );

    this.input.keydown( function( event )
                        {
                            switch( event.keyCode )
                            {
                                case KEY_CODE_TAB:  // tab
                                    self.hide();
                                    break;
                                case KEY_CODE_PAGE_UP: // pgup
                                    event.preventDefault();
                                    self.markFirst();
                                    break;
                                case KEY_CODE_PAGE_DOWN: // pgedown
                                    event.preventDefault();
                                    self.markLast();
                                    break;
                                case KEY_CODE_UP: // up
                                    event.preventDefault();
                                    if(self.showed)
                                    {
                                        self.moveSelected( -1 );
                                    }
                                    else
                                    {
                                        self.dropdownList.show();
                                    }
                                    break;
                                case KEY_CODE_DOWN: // down
                                    event.preventDefault();
                                    self.moveSelected( 1 );
                                    break;
                                case KEY_CODE_ESC: // esc
                                    // no prevention when element is hidden
                                    if (!self.isDropDownShown())
                                        break;
                                case 13: // return
                                    event.preventDefault();
                                    event.stopPropagation();
                                    break;
                            }
                        } );

    this.dropdown
            .off(ScrollUtils.getWheelEventName())
            .on(
                    ScrollUtils.getWheelEventName(),
                    function(e)
                    {
                        if(self.showed)
                        {
                            _.bind(ScrollUtils.getWheelEventListenerForPreventScrollParents(), this)(e);
                        }
                    }
            )
};


CodePhoneSelector.prototype.filterResults = function( isAllResults )
{
    var value = this.input.val();
    if( value == this.lastValue ) return;
    this.results = [];
    if( value == "" || value == "+" || isAllResults  || this.initialSelect)
    {
        $.extend(this.results, this.cache);
        this.initialSelect = false;
        if(value=="+"){
            this.hideEmptyError();
        }
    }
    else
    {
        this.hideEmptyError();
        var ids = this.searchIndex.search( this.searchIndex.getTokenizer().getTokens( value ) );
        for( var prop  in ids )
        {
            this.results.push( this.cache[ prop ] );
        }
    }
    this.results.sort( function( a, b ) { return a.name.toLowerCase() < b.name.toLowerCase() ? -1 : (a.name.toLowerCase() > b.name.toLowerCase() ? 1 : 0); } );
    this.results.length > 0 ? this.renderDropdown() : this.hide();
    this.markFirst();
    this.picked = false;
    this.lastValue = value;
};


CodePhoneSelector.prototype.renderDropdown = function()
{

    this.dropdown.css( {
                           width: 200 + "px",
                           maxHeight: ''
                       } );

    this.dropdownList.empty();
    var self = this;
    _.each( this.results, function(val)
    {
        $("<li>" + val.name + "</li>")
                .appendTo(self.dropdownList)
    } );

    this.dropdown.show();

    var inputOffset = this.input.offset();
    var dropDownOffset = {};
    dropDownOffset.left = inputOffset.left;
    if(this.settings.showDropdownToTop)
    {
        dropDownOffset.top = inputOffset.top - this.dropdown.outerHeight(true);
    }
    else
    {
        dropDownOffset.top = (inputOffset.top + this.input.outerHeight());
    }
    this.dropdown.offset(dropDownOffset);
    this.showed = true;
};

CodePhoneSelector.prototype.markSelected = function( n )
{
    if( n < 0 || n >= this.results.length ) return;

    var rows = this.dropdown.find( "li" );
    rows.removeClass( this.settings.selectedClass );

    var row = $( rows[ n ] );
    this.selectedIndex = n;
    row.addClass( this.settings.selectedClass );
    var top = row.position().top;
    var delta = top + row.outerHeight() - this.dropdown.height();
    if( delta > 0 )
    {
        this.allowMouseMove = false;
        this.dropdown.scrollTop( this.dropdown.scrollTop() + delta );
    }
    else if( top < 0 )
    {
        this.allowMouseMove = false;
        this.dropdown.scrollTop( Math.max( 0, this.dropdown.scrollTop() + top ) );
    }
};

CodePhoneSelector.prototype.pickSelected = function()
{
    var selected = this.results[ this.selectedIndex ];
    if( selected )
    {
        this.picked = true;
         this.input.val( this.setInputByValueFunction( selected.value ) );
         this.hideEmptyError();
    }
    else
    {
        this.reset();
    }
};


CodePhoneSelector.prototype.getValue = function( )
{
    return this.input.val();
};


CodePhoneSelector.prototype.hide = function()
{
    this.dropdown.hide();
    this.container.removeClass("showAll");
    this.lastValue = null;
    this.showed = false;
};

CodePhoneSelector.prototype.isDropDownShown = function()
{
    return this.dropdownList.is(":visible");
};

CodePhoneSelector.prototype.addEmptyError = function(){
    this.container.addClass("emptyError");
};

CodePhoneSelector.prototype.hideEmptyError = function(){
    this.container.removeClass("emptyError");
};

CodePhoneSelector.prototype.moveSelected = function( n ) { this.markSelected( this.selectedIndex + n ); };
CodePhoneSelector.prototype.markFirst = function() { this.markSelected( 0 ); };
CodePhoneSelector.prototype.markLast = function() { this.markSelected( this.results.length - 1 ); };
CodePhoneSelector.prototype.reset = function() { this.input.val( this.valueBeforeFocus ); };
CodePhoneSelector.prototype.focus = function() { this.input.focus(); };
CodePhoneSelector.prototype.focusAndHide = function()
{
    this.focus();
    this.hide();
};

