var Router = {
    _routes: {},
    _config: {},
    _root: '/',
    _mode: !!(window.history && history.pushState) ? "history" : "anchors",

    config: function(config)
    {
        _.extend(this._config, config);
        return this;
    },

    init: function(context)
    {
        this._root = context;
        var handler = _.bind(Router.check, Router, null);
        History.Adapter.bind(window, 'statechange', handler);
        return this;
    },

    _clearPath: function(path)
    {
        return path.toString().replace(/\/$/,'').replace(/\?(.*)$/,'').replace(/;(.*)$/, '');
    },

    getPath: function(state)
    {
        var _state = state || this.getState();
        var path = this._clearPath(decodeURI(_state.hash));
        if (this._mode === "history")
        {
            return (this._root != '/') ? path.replace(this._root, '') : path;
        } else
        {
            var match = path.match(/#(.*)$/);
            return match ? match[1] : path;
        }
    },

    add: function(path, handler)
    {
        this._routes[path] = {handler: handler};
        return this;
    },

    remove: function(path)
    {
        delete this._routes[path];
        return this;
    },

    base: function(path, state, stateName)
    {
        var _path = (this._mode === "history") ? this._root + path : path;
        _path += this.getJSessionFromUrl();
        History.replaceState(state, stateName, _path);
        return this;
    },

    navigate: function(path, state, title)
    {
        var _path = (this._mode === "history") ? this._root + path : path;
        _path += this.getJSessionFromUrl();
        History.pushState(state, title, _path);
        return this;
    },

    /**
     * If browser cookies are off return jsessionid string or empty string either
     * @return {String} string with jsessionid (ex: ';jsessionid=6FB6AAC69D409E56FE13975123D6706A')
     */
    getJSessionFromUrl: function()
    {
        var _url = decodeURI(this.getState().hash);
        var match = _url.match(/jsessionid=(.*)$/);
        return (!_.isNull(match)) ? (';' + match[0]).replace(/\/$/,'').replace(/\?.*$/,'').replace(/&.*$/,'') : '';
    },

    getState: function()
    {
        return History.getState(true, true);
    },

    flush: function() {
        this._routes = [];
        this._config = {};
        this._root = '/';
        return this;
    },

    check: function(_path)
    {
        var state = this.getState();
        var path = _path || this.getPath(state);
        if (!_.isUndefined(this._routes[path]))
        {
            this._routes[path].handler.call({}, state.data, state.title);
        }
        return this;
    },

    supportHistoryAPI: function()
    {
        return this._mode === 'history';
    }
};