function SimpleListMenu($containerEl, itemHtmlCallback)
{
    DropdownTemplateView.call(this, $containerEl);
    this.items = [];
    this.itemHtmlCallback = itemHtmlCallback || SimpleListMenu.defaultItemHtmlCallback;
}

SimpleListMenu.defaultItemHtmlCallback = function(item)
{
    return escapeHtmlSymbols(item.name)
};

SimpleListMenu.prototype = _.create(DropdownTemplateView.prototype, {
    html: "<div class='dropdownContainer simpleListMenu'></div>",

    afterRender: function()
    {
        this.scrollView = new ScrollView(this.$el, 20).init();
        var self = this;
        this.bindClick(function(clickEvent) {self.trigger('click', clickEvent.originalEvent);});
    },

    getItems: function()
    {
        return this.items
    },

    setItems: function(items)
    {
        var self = this;

        this.items = items;

        function itemClickHandler(item)
        {
            $(self).trigger("itemClick", item);
            self.hide();
        }

        this.scrollView.setItems(_.map(items, function(item)
        {
            return new SimpleListMenu.Item(item, self.itemHtmlCallback, itemClickHandler);
        }));

        // _.each(items, function(item)
        // {
        //     var $itemEl = $("<div class='item'></div>");
        //     $itemEl.text(item.name);
        //     $itemEl[0].simpleListItem = item;
        //     $itemEl.click(itemClickHandler);
        //     self.$el.append($itemEl)
        // });
    },

    onShow: function()
    {
        DropdownTemplateView.prototype.onShow.call(this);
        this.scrollView.refreshScroll()
    }
});

SimpleListMenu.Item = function(item, itemHtmlCallback, clickHandler)
{
    TemplateView.call(this);
    this.item = item;
    this.itemHtmlCallback = itemHtmlCallback;
    this.clickHandler = clickHandler;
};

SimpleListMenu.Item.prototype = _.create(TemplateView.prototype, {
    html: "<div class='item'></div>",

    templateKey: "SimpleListMenu.Item",

    afterRender: function()
    {
        var self = this;
        this.$el.text(this.itemHtmlCallback(this.item));
        this.$el.click(function(e)
                       {
                           self.clickHandler(self.item);
                           stopEventPropagation(e);
                       });
    }
});