function SubmitTTCommentsManager () {}

SubmitTTCommentsManager.prototype.init = function ( config ) {
    this.VIEW_DESCRIPTION_POPUP_CAPTION = messageResource.getMessage('submit_tt.comments.view_description_popup_caption');
    this.EDIT_DESCRIPTION_POPUP_CAPTION = messageResource.getMessage('submit_tt.comments.edit_description_popup_caption');

    this.Messages = {
        MODIFICATIONS_NOT_SAVED : messageResource.getMessage("submit_tt.comments.modifications_not_saved"),
        TIMEOUT_DESCRIPTION_SAVING : messageResource.getMessage("submit_tt.comments.timeout_description_saving"),
        TIMEOUT_DESCRIPTION_LOADING : messageResource.getMessage("submit_tt.comments.timeout_description_loading"),
        ERROR_DESCRIPTION_SAVING : messageResource.getMessage("submit_tt.comments.error_description_saving"),
        ERROR_DESCRIPTION_LOADING : messageResource.getMessage("submit_tt.comments.error_description_loading"),
        TEXT_AREA_NOTE_NAME : messageResource.getMessage("submit_tt.comments.text_area_note_name"),
        TOO_LONG_DESCRIPTION : function(len) {
            return messageResource.getMessage('submit_tt.comments.too_long_description', [len]);
        }
    };
    
    this.showCommentsUtil = this; 
    this.imgEntered = imgPath("dialogs/comments/comment_entered.png");
    this.imgBlank = imgPath("dialogs/comments/comment_blank.png");
    
    this.state = {};
    this.state.userId = config.userId;
    this.reloadPageFunction = config.reloadPageFunction;
    this.updateImgFunction = config.updateImgFunction;
    this.updateFormFunction = config.updateFormFunction;

    // for cashing unmodifiable comments. see realization in "view_description.js" cacheDescription()
    this.entityType = "comment";
    
    return this;
};

// required interface for showSavingDescriptionWithDefaultOffset and showDescriptionWithDefaultOffset methods
SubmitTTCommentsManager.prototype.doLoadDescription = function ( hiddenName, descriptionLoadedHandler/*, errorHandler*/ ) {
    this.loadComment(descriptionLoadedHandler);
};

// required interface for showSavingDescriptionWithDefaultOffset and showDescriptionWithDefaultOffset methods
SubmitTTCommentsManager.prototype.doSaveDescription = function ( hiddenName, description, descriptionSavedHandler/*, errorHandler*/ ) {
    this.saveComment(description, descriptionSavedHandler);
};

SubmitTTCommentsManager.prototype.saveState = function ( taskId, day, date ) {
    this.state.taskId = taskId;
    this.state.day = day;
    this.state.date = parseDate(date, "yyyy-MM-dd");
};


SubmitTTCommentsManager.prototype.showEditableComment = function ( taskId, day, date, cellId, link ) {
    var self = this;
    this.saveState(taskId, day, date);
    showSavingDescriptionWithDefaultOffset("", link, this.showCommentsUtil, document.getElementById(cellId), false, 
                                           this.imgEntered, this.imgBlank, undefined, {onCloseTabMessage: self.Messages.MODIFICATIONS_NOT_SAVED});
};

SubmitTTCommentsManager.prototype.showReadOnlyComment = function ( taskId, day, date, cellId, link ) {
    this.saveState(taskId, day, date);
    // some unique id
    var objectId = taskId + "_" + day;
    showDescriptionWithDefaultOffset(objectId, link, this.showCommentsUtil, document.getElementById(cellId));
};

SubmitTTCommentsManager.prototype.loadComment = function ( descriptionLoadedHandler ) {
    var self = this;
    makeRpcCall( "EnterTTService.loadComment", [this.state.userId, this.state.taskId, this.state.date],
                 function ( comment ) { 
                     descriptionLoadedHandler( !!comment ? comment : "" );
                     
                     self.updateImg(comment);
                 },
                 function ( jqXHR, textStatus, errorThrown ) { return self.errorHandler(errorThrown); } );
};

SubmitTTCommentsManager.prototype.saveComment = function ( comment, descriptionSavedHandler ) {
    var self = this;
    makeRpcCall( "EnterTTService.saveComment", [this.state.userId, this.state.taskId, this.state.date, comment],
                 function () {
                     if ( !!descriptionSavedHandler )
                        descriptionSavedHandler();
                     
                     self.updateForm(comment);
                 }, 
                 function ( jqXHR, textStatus, errorThrown ) { return self.errorHandler(errorThrown); } );
};

SubmitTTCommentsManager.prototype.updateImg = function ( comment ) {
    this.updateImgFunction();
    this.updateForm(comment);
};

SubmitTTCommentsManager.prototype.updateForm = function ( comment ) {
    this.updateFormFunction(this.state.taskId, this.state.day, comment);
};

SubmitTTCommentsManager.prototype.reloadPage = function () {
    this.reloadPageFunction();
};

SubmitTTCommentsManager.prototype.errorsMap = {
    'com.actimind.actitime.administration.NoSuchUserException' : 'submit_tt.js.comments.error_user_was_deleted',
    'com.actimind.actitime.task.NoSuchTaskException' : 'submit_tt.js.comments.error_task_was_deleted',
    'com.actimind.actitime.services.NoRightsForRemoteCallException' : 'submit_tt.js.comments.error_no_rights_for_this_operation',
    'com.actimind.actitime.services.NoRightsForManagingOtherUsersException' : 'submit_tt.js.comments.error_no_rights_for_managing_other_users',
    'com.actimind.actitime.services.NoRightsToSubmitTTException' : 'submit_tt.js.comments.error_no_rights_for_submitting_tt',
    'com.actimind.actitime.administration.NoRightsToViewUsersTimeTrack' : 'submit_tt.js.comments.error_no_rights_to_manage_selected_user_tt',
    'com.actimind.actitime.task.TaskIsNotAssignedToUserException' : 'submit_tt.js.comments.error_task_is_not_assigned',
    'com.actimind.actitime.task.DayIsLockedException' : 'submit_tt.js.comments.error_day_is_locked',
    'com.actimind.actitime.task.TaskCompletedException' : 'submit_tt.js.comments.error_task_completed',
    'com.actimind.actitime.services.DateBeforeHireDateException' : 'submit_tt.js.comments.error_day_before_hire_date'
};

/* 
    Must be refactored. Much better to pass correct(view/edit) cleanup method down to errorHandler. 
    But descriptionState seems to be obsolete and i hope would be refactored in future so it is not very good idea to strengthen coupling.
*/
SubmitTTCommentsManager.prototype.cleanupRetryController = function () {
    descriptionCleanup(); // hack for view_comment reload controller
    descriptionSavingCleanup(true); // hack for edit_description reload controller    
};

SubmitTTCommentsManager.prototype.errorHandler = function ( errorThrown ) {
    var message = this.errorsMap[errorThrown.exception];
    if( !!message ) {
        this.cleanupRetryController();
        alert( messageResource.getMessage(message) );
        this.reloadPage();
        return true;
    }
    
    if( errorThrown.exception == 'com.actimind.actitime.administration.FutureDatesTimeTrackingIsRestrictedException' ) {
        this.cleanupRetryController();
        alert( messageResource.getMessage('submit_tt.js.comments.error_day_is_in_future') );
        // no reload!
        return true;
    }
    
    return false;
};