function Tooltip($containerEl)
{
    TemplateView.call(this, $containerEl);
}

Tooltip.prototype = _.create(TemplateView.prototype, {
    autoFollowMouseCursor: true,

    moveAfterCursor: function(event)
    {
        if (this._disabled || !this._visible)
            return;

        var tooltipWidth = this.$el.outerWidth();
        var tooltipHeight = this.$el.outerHeight();
        var xOffset = 15;
        var yOffset = 15;
        var minDistanceToBorderX = 15;
        var minDistanceToBorderY = 15;

        var x = event.clientX + xOffset;
        if (x + tooltipWidth + minDistanceToBorderX > $(window).width())
            x -= tooltipWidth + xOffset;

        var y = event.clientY + yOffset;
        if (y + tooltipHeight + minDistanceToBorderY > $(window).height())
            y -= tooltipHeight + yOffset;

        this.$el.offset({top: y + getBodyScrollTop(), left: x + getBodyScrollLeft()});
    },

    afterRender: function()
    {
        this.$el.hide();
        this.$el.addClass("yellowTooltip tooltipTex noprint");
    },

    show: function()
    {
        if (!this._visible)
        {
            this._visible = true;
            if (!this._disabled)
                this.$el.show()
        }
    },

    hide: function()
    {
        if (this._visible)
        {
            this._visible = false;
            if (!this._disabled)
                this.$el.hide()
        }
    },

    enable: function()
    {
        this._disabled = false;
        if (this._visible)
            this.$el.show();
    },

    disable: function()
    {
        this._disabled = true;
        if (this._visible)
            this.$el.hide();
    },

    setEnabled: function(enabled)
    {
        if (enabled)
            this.enable();
        else
            this.disable();
    },

    subscribeToMouseEvents: function($el, beforeShowCallback)
    {
        var self = this;
        $el.mouseenter(function()
                       {
                           if (beforeShowCallback)
                               beforeShowCallback();
                           self.show();
                       });
        $el.mouseleave(this.bind(this.hide));
        if (this.autoFollowMouseCursor)
            $el.mousemove(this.bind(this.moveAfterCursor));
    }
});

