function CreateNewTasksComponent()
{
}

CreateNewTasksComponent.prototype.init = function( containerId, billingTypes, defaultBillingTypeId,
                                                   canAddTasksToUser, weekStartDay, scheduleCalculator, estEnabled,
                                                   projectsWithTasksExist, defaultRowsNumber, isOpenedFromOpenTasks,
                                                   changesCallback, validateCallback, scrollableContainer )
{
    this.containerId = containerId;
    this.containerEl = $( "#" + this.containerId );

    this.billingTypes = billingTypes;
    this.defaultBillingTypeId = defaultBillingTypeId;

    this.canAddTasksToUser = canAddTasksToUser;
    this.projectsWithTasksExist = projectsWithTasksExist;

    this.estEnabled = estEnabled;

    this.weekStartDay = weekStartDay;
    this.scheduleCalculator = scheduleCalculator;

    this.defaultRowsNumber = defaultRowsNumber;
    this.isOpenedFromOpenTasks = isOpenedFromOpenTasks;

    this.changesCallback = changesCallback;
    this.validateCallback = validateCallback;

    this.scrollableContainer = scrollableContainer;

    this.spinner = new Spinner({ length: 3, width: 2, radius: 3, lines: 8, color: '#aaaaaa', trail: 60 });
    this.importSpinnerPlaceholder = this.containerEl.find( ".importSpinnerPlaceholder");

    this.oldGetOnPageDescription = window.getOnPageDescription;

    var self = this;

    this.createCreateTasksTable();

    this.addRowButton = this.getElementBySubId( "addNewRow" );
    $( this.addRowButton ).click( function() {
        self.createTasksTable.addEmptyRow();
        return false;
    });

    if ( this.isOpenedFromOpenTasks )
    {
        this.addFiveRowsButton = this.getElementBySubId( "addFiveRows" );
        this.addTenRowsButton = this.getElementBySubId( "addTenRows" );
        $( this.addFiveRowsButton ).click( function() {
            self.createTasksTable.addEmptyRows( 5 );
            return false;
        });
        $( this.addTenRowsButton ).click( function() {
            self.createTasksTable.addEmptyRows( 10 );
            return false;
        });
    }
    else
    {
        $( this.getElementBySubId( "addRowsContainer" ) ).addClass( "addOneRowOnly" );
    }

    if ( !this.projectsWithTasksExist )
        $( this.getElementBySubId( "importTasksContainer" ) ).hide();


    this.importTasksTooltip = this.createTooltip( "importTasksTooltipWrapper", this.getElementIdBySubId( "importTasksTooltip" ) );
    $( this.getElementBySubId( "importTasksContainer" ) ).find( ".infoIconImg" ).click( function() { self.showImportTasksTooltip(); } );

    return this;
};


CreateNewTasksComponent.prototype.getElementBySubId = function( id )
{
    return document.getElementById( this.getElementIdBySubId( id ) );
};

CreateNewTasksComponent.prototype.getElementIdBySubId = function( id )
{
    return this.containerId + "_" + id;
};

CreateNewTasksComponent.prototype.validate = function()
{
    return this.createTasksTable.validate();
};

CreateNewTasksComponent.prototype.isEmpty = function()
{
    return this.createTasksTable.isEmpty();
};

CreateNewTasksComponent.prototype.hasErrors = function()
{
    return this.createTasksTable.hasErrors();
};

CreateNewTasksComponent.prototype.createCreateTasksTable = function()
{
    this.createTooltips();

    this.createTasksTable = new CreateNewTasksTable().init( this.getElementBySubId( "createTasksTableTHead" ),
                                                            this.getElementBySubId( "createTasksTableColgroup" ),
                                                            this.defaultRowsNumber, null, this.billingTypes, this.defaultBillingTypeId,
                                                            this.weekStartDay, this.scheduleCalculator,
                                                            this.canAddTasksToUser && this.isOpenedFromOpenTasks,
                                                            this.canAddTasksToUser, this.estEnabled,
                                                            this.validateCallback, this.changesCallback,
                                                            this._self( this.showEstimateTooltip ),
                                                            this._self( this.showAddToTTTooltip ),
                                                            this._self( this.showDescriptionPopup ),
                                                            this._self( this.hideDescriptionPopup ),
                                                            this.scrollableContainer );
    this.getElementBySubId( "createTasksTableContainer" ).innerHTML = "";
    this.createTasksTable.buildDOM( this.getElementBySubId( "createTasksTableContainer" ) );
};

CreateNewTasksComponent.prototype.createTooltips = function()
{
    this.estimateTooltip = this.createTooltip( "estimateTooltipWrapper", this.getElementIdBySubId( "estimateTooltip" ) );

    if ( this.canAddTasksToUser && this.isOpenedFromOpenTasks )
    {
        this.addToTimeTrackTooltip = this.createTooltip( "addToTTTooltipWrapper",
                                                         this.getElementIdBySubId( "addToTTTooltip" ) );
    }
};

CreateNewTasksComponent.prototype.createTooltip = function( wrapperClass, dialogDivId )
{
    return new DropdownDialog({
                                  wrapperClass: wrapperClass,
                                  dialogDiv: $( "#" + dialogDivId ),
                                  hideButtons: true,
                                  hideOnResize: true,
                                  showEmptyTitleBar: false,
                                  isWithBeak: true
                              });
};

CreateNewTasksComponent.prototype.showEstimateTooltip = function ( )
{
    this.estimateTooltip.showAt( $( this.getElementBySubId( "createTasksTableTHead" ) ).find(".estimateCell"),
                                 {my: "left bottom", at: "left top", offset: "53px -3px", collision: "none" } );
    this.increaseOverlayZIndex();
};

CreateNewTasksComponent.prototype.showAddToTTTooltip = function ( )
{
    this.addToTimeTrackTooltip.showAt( $( this.getElementBySubId( "createTasksTableTHead" ) ).find(".addToTTCell div.infoIcon"),
                                       {my: "left bottom", at: "left top", offset: "-181px -13px", collision: "none" } );
    this.increaseOverlayZIndex();
};

CreateNewTasksComponent.prototype.showImportTasksTooltip = function ( )
{
    this.importTasksTooltip.showAt( $( this.getElementBySubId( "importTasksContainer" ) ).find(".infoIconImg"),
                                    {my: "left bottom", at: "left top", offset: "-14px -10px", collision: "none" } );
    this.increaseOverlayZIndex();
};

CreateNewTasksComponent.prototype.increaseOverlayZIndex = function ( )
{
    $(".ui-widget-overlay" ).zIndex(this.containerEl.zIndex() + 1);
};

CreateNewTasksComponent.prototype.getTasks = function()
{
   return this.createTasksTable.getTasks();
};

CreateNewTasksComponent.prototype.getCreateTasksTable = function()
{
    return this.createTasksTable;
};

CreateNewTasksComponent.prototype._self = function ( func ){
    var self = this;

    return function (){
        return func.apply(self, arguments);
    }
};

CreateNewTasksComponent.prototype.showDescriptionPopup = function( description, descriptionCellId, descriptionTd, descriptionChangedCallback )
{
    if (editDescriptionState)
    {
        if (editDescriptionState.objectId == descriptionCellId || !saveOnPageDescription())
            return;
    }

    var offsetX = -100;
    var offsetY = 25 - $( this.scrollableContainer ).scrollTop();

    var self = this;
    $( this.scrollableContainer ).on( 'scroll', function( e ) {
        self.hideDescriptionPopup( false );
    });

    showSavingDescriptionWithOffset( descriptionCellId, descriptionTd.firstChild,
                                     new CreateTaskDescriptionUtil( description, descriptionChangedCallback ),
                                     descriptionTd, true, offsetX, offsetY );
};

CreateNewTasksComponent.prototype.hideDescriptionPopup = function( isConfirmNeeded )
{
    if ( isConfirmNeeded )
    {
        return editDescriptionPopup.hide();
    }
    else
    {
        saveOnPageDescription( true );
        return true;
    }
};

CreateNewTasksComponent.prototype.restoreGetOnPageDescriptionFunction = function ()
{
    window.getOnPageDescription = this.oldGetOnPageDescription;
};

CreateNewTasksComponent.prototype.startImportSpin = function()
{
    this.spinner.spin( this.importSpinnerPlaceholder[0] );
};

CreateNewTasksComponent.prototype.stopImportSpin = function()
{
    this.spinner.stop();
};

CreateNewTasksComponent.prototype.importTasksRequest = function( taskIds, needToLoadDescriptions, customerId, projectId, isInActiveProjects )
{
    var self = this;
    var requestedTaskCount = (customerId == 0 && projectId == 0) ? taskIds.length : -1;

    var errorHandler = function( jqXHR, textStatus, errorThrown )
    {
        self.stopImportSpin();
        return false;
    };

    var successHandler = function( tasks )
    {
        self.createTasksTable.addTasks( tasks.list );
        self.stopImportSpin();
        var retrievedTaskCount = tasks.list.length;
        if( retrievedTaskCount == 0 && requestedTaskCount == -1 || retrievedTaskCount < requestedTaskCount )
        {
            // TODO: change texts!
            if( retrievedTaskCount == 0 )
                if( customerId == 0 )
                    alert( messageResource.getMessage( 'add_new_tasks.import.err.all_tasks_deleted_unavailable' ) );
                else
                    if( isInActiveProjects )
                        alert( messageResource.getMessage( 'add_new_tasks.import.err.all_tasks_deleted_unavailable_archived' ) );
                    else
                        alert( messageResource.getMessage( 'add_new_tasks.import.err.all_tasks_deleted_unavailable_restored' ) );
            else
                alert( messageResource.getMessage( 'add_new_tasks.import.err.some_tasks_deleted_unavailable' ) );
        }
    };

    if( requestedTaskCount == -1 || requestedTaskCount > 0 )
    {
        self.startImportSpin();
        makeRpcCall( "ImportTasksService.importTasks",
                     [taskIds, needToLoadDescriptions, customerId, projectId, isInActiveProjects],
                     successHandler,
                     errorHandler );
    }
};


function CreateTaskDescriptionUtil( description, descriptionChangedCallback ){

    this.descriptionChangedCallback = descriptionChangedCallback;

    window.getOnPageDescription = function( idx ){
        return description;
    }
}

CreateTaskDescriptionUtil.prototype = new DescriptionUtil( TASK_ENTITY );

CreateTaskDescriptionUtil.prototype.doSaveDescription = function (idx, description, descriptionSavedHandler, errorHandler)
{
    this.descriptionChangedCallback( description );

    if( descriptionSavedHandler != undefined )
        descriptionSavedHandler();
};


CreateNewTasksComponent.prototype.handleTaskError = function( taskError, taskErrorRowNumber )
{
    var self = this;
    switch ( taskError )
    {
        case TABLE_TASKS_CONSTANTS.OPEN_TASK_EXISTS_ERROR:
            alert( messageResource.getMessage( 'add_new_tasks.err_duplicate_detected.open_task' ) );
            self.getCreateTasksTable().showTaskNameExistsError( taskErrorRowNumber, false );
            break;

        case TABLE_TASKS_CONSTANTS.COMPLETED_TASK_EXISTS_ERROR:
            alert( messageResource.getMessage( 'add_new_tasks.err_duplicate_detected.completed_task' ) );
            self.getCreateTasksTable().showTaskNameExistsError( taskErrorRowNumber, true );
            break;

        case TABLE_TASKS_CONSTANTS.UNKNOWN_TASK_EXISTS_ERROR:
            alert( messageResource.getMessage( "add_new_tasks.js.unexpected_error" ) );
            break;

        case TABLE_TASKS_CONSTANTS.BILLING_TYPE_DELETED_OR_ARCHIVED_ERROR:
            alert( messageResource.getMessage( "add_new_tasks.billing_types_deleted_or_archived" ) );
            self.getCreateTasksTable().updateBillingTypes( billingTypes );
            break;
    }
};

CreateNewTasksComponent.prototype.handleBillingTypeError = function( billingTypeError, billingTypes )
{
    var self = this;
    switch ( billingTypeError )
    {
        case TABLE_TASKS_CONSTANTS.BILLING_TYPE_DELETED_OR_ARCHIVED_ERROR:
            alert( messageResource.getMessage( "add_new_tasks.billing_types_deleted_or_archived" ) );
            self.getCreateTasksTable().updateBillingTypes( billingTypes );
            break;
    }
};

