function CreateNewTasksTable()
{
    this.constructor = CreateNewTasksTable;
}

CreateNewTasksTable.prototype = _.create(Table.prototype);

CreateNewTasksTable.prototype.init = function ( theadEl, colgroupEl, defaultRowsNumber, records, billingTypes, defaultBillingTypeId,
                                                weekStartDay, scheduleCalculator, showAddToTTColumn, defaultAddToTTValue, estEnabled,
                                                validateCallback, changesCallback, showEstimateTooltipCallback, showAddToTTTooltipCallback,
                                                showDescriptionPopupCallback, hideDescriptionPopupCallback, scrollableContainer )
{
    Table.call( this, theadEl );

    this.theadEl = theadEl;

    this.billingTypes = billingTypes;
    this.defaultBillingTypeId = defaultBillingTypeId;

    this.weekStartDay = weekStartDay;
    this.scheduleCalculator = scheduleCalculator;

    this.defaultAddToTTValue = defaultAddToTTValue;
    this.showAddToTTColumn = showAddToTTColumn;
    this.estEnabled = estEnabled;

    this.scrollableContainer = scrollableContainer;

    this.changesCallback = function( cell ) {
        if ( cell )
            cell.getRow().updateClearLineCell();
        changesCallback();
    };

    var self = this;
    this.nameCellRenderer = new TaskNameCellRenderer().init( this.changesCallback, function() { self.markRowsWithEqualNamesAsInvalid() } );
    this.descriptionCellRenderer =  new TaskDescriptionCellRenderer().init( showDescriptionPopupCallback, this.changesCallback );
    if (this.estEnabled)
        this.estimateCellRenderer =  new TaskEstimateCellRenderer().init( this.changesCallback );
    this.deadlineCellRenderer =  new TaskDeadlineCellRenderer().init( this.weekStartDay, this.scheduleCalculator, this.changesCallback, this.scrollableContainer,
                                                                      messageResource.getMessage( "add_new_tasks.set_deadline" ),
                                                                      messageResource.getMessage( "add_new_tasks.js.deadline_is_in_past" ), true );
    this.billingTypeCellRenderer = this.billingTypes ? new TaskBillingTypeCellRenderer().init( this.billingTypes, this.defaultBillingTypeId, this.changesCallback, this.scrollableContainer ) : null;
    this.clearLineCellRenderer = new TaskClearLineCellRenderer().init();
    this.addToTTCellRenderer = this.showAddToTTColumn ? new TaskAddToTTCellRenderer().init( function() { self.updateAddAllToTTCheckboxValue(); } ) : null;

    this.nameCellValidator = new TaskNameCellValidator();
    this.descriptionCellValidator = new TaskDescriptionCellValidator();
    if (this.estEnabled)
        this.estimateCellValidator = new TaskEstimateCellValidator();
    this.billingTypeCellValidator = new TaskBillingTypeCellValidator();

    this.validateCallback = validateCallback;
    this.showEstimateTooltipCallback = showEstimateTooltipCallback;
    this.showAddToTTTooltipCallback = showAddToTTTooltipCallback;
    this.hideDescriptionPopupCallback = hideDescriptionPopupCallback;

    insertBefore( colgroupEl, theadEl );

    this.createBodyContent( records, defaultRowsNumber );

    this.setupTableHeaderEventHandlers();

    return this;
};

CreateNewTasksTable.prototype.setupTableHeaderEventHandlers = function ( )
{
    $( this.theadEl ).find( ".estimateCell .infoIcon" ).click( this.showEstimateTooltipCallback );
    $( this.theadEl ).find( ".addToTTCell .infoIcon" ).click( this.showAddToTTTooltipCallback );

    var self = this;
    this.addAllToTTCheckbox = $( this.theadEl ).find( ".addToTTCell input" );
    this.addAllToTTCheckbox.click( function() {
        for( var r = 0; r < self.rows.length; r++ )
            self.rows[r].setAddToTTValue( this.checked == true );
    });
};

CreateNewTasksTable.prototype.updateAddAllToTTCheckboxValue = function ()
{
    if ( !this.showAddToTTColumn )
        return;

    var allChecked = true;
    for( var r = 0; r < this.rows.length; r++ )
    {
        if ( !this.rows[r].addToTTCell.getValue() )
            allChecked = false;
    }

    this.addAllToTTCheckbox.attr( 'checked', allChecked );
};

CreateNewTasksTable.prototype.createBodyContent = function ( records, defaultRowsNumber )
{
    var emptyRowsNumber = defaultRowsNumber;

    if ( records && records.length > 0 )
    {
        for( var i = 0; i < records; i++ )
            this.addRow( this.createRow( records[i] ) );

        emptyRowsNumber -= records.length;
    }

    for( var j = 0; j < emptyRowsNumber; j++ )
        this.addRow( this.createEmptyRow() );
};

CreateNewTasksTable.prototype.createRow = function ( record )
{
    return new CreateNewTasksTableRow().init( record,
                                              this.nameCellRenderer,
                                              this.nameCellValidator,
                                              this.descriptionCellRenderer,
                                              this.descriptionCellValidator,
                                              this.estimateCellRenderer,
                                              this.estimateCellValidator,
                                              this.deadlineCellRenderer,
                                              this.billingTypeCellRenderer,
                                              this.billingTypeCellValidator,
                                              this.addToTTCellRenderer,
                                              this.clearLineCellRenderer );
};

CreateNewTasksTable.prototype.createEmptyRow = function ()
{
    var record = {
        name: "",
        description: "",
        estimate: "",
        deadline: null
    };
    return this.createRow( record );
};

CreateNewTasksTable.prototype.buildDOM = function( containerEl )
{
    Table.prototype.buildDOM.call(this, containerEl);
    this.updateTableClasses();
};

CreateNewTasksTable.prototype.addNonEmptyRow = function( record )
{
    var row = this.createRow( record );
    this.addRow( row );
    row.buildDOM( this.getTBody() );
    row.writeCellsContent();
    row.getRowEl().className = this.getRowElClasses( row.getRowIdx() ).join(" ");

    this.updateAddAllToTTCheckboxValue();
};

/**
 * Adds tasks in empty rows after last non-empty row.
 * Creates new rows if needed
 * @param records - records of tasks to add in the table.
 */
CreateNewTasksTable.prototype.addTasks = function( records )
{
    var insertRowIdx = this.rows.length;
    while( insertRowIdx > 0 && this.rows[insertRowIdx - 1].isEmpty() ) insertRowIdx--;

    for( var i = 0; i < records.length; i++ )
    {
        var record = records[i];
        record.estimate = record.budgetedTime;
        if( insertRowIdx < this.rows.length )
            this.replaceRowAndUpdateDOM( insertRowIdx, this.createRow( records[i] ) );
        else
            this.addNonEmptyRow( records[i] );
        insertRowIdx++;
    }

    if ( records.length > 0 )
        this.changesCallback();
    this.markRowsWithEqualNamesAsInvalid();
};

CreateNewTasksTable.prototype.addEmptyRow = function(  )
{
    var row = this.createEmptyRow();
    this.addRow( row );
    row.buildDOM( this.getTBody() );
    row.writeCellsContent();
    row.getRowEl().className = this.getRowElClasses( row.getRowIdx() ).join(" ");

    this.updateAddAllToTTCheckboxValue();
};

CreateNewTasksTable.prototype.addEmptyRows = function( rowsNumber )
{
    for( var i = 0; i < rowsNumber; i++ )
        this.addEmptyRow();
};

CreateNewTasksTable.prototype.replaceRowWithNewEmptyRow = function( rowIdx )
{
    this.replaceRowAndUpdateDOM( rowIdx, this.createEmptyRow() );
    this.markRowsWithEqualNamesAsInvalid();
};

CreateNewTasksTable.prototype.updateTableClasses = function()
{
    var classes = this.getBaseTableClasses();
    this.tableEl.className = classes.join(" ");
};

CreateNewTasksTable.prototype.getBaseTableClasses = function()
{
    var classes = ["createTasksTable"];
    if ( !this.billingTypes )
        classes.push( "hideBillingType" );
    if ( !this.estEnabled )
        classes.push( "hideEstimate" );
    if ( !this.showAddToTTColumn )
        classes.push( "hideAddToTT" );
    return classes;
};

CreateNewTasksTable.prototype.getTasks = function()
{
    var tasks = [];
    for( var r = 0; r < this.rows.length; r++ )
    {
        if ( !this.rows[r].isEmpty() )
        {
            var task = this.rows[r].getTask();
            tasks.push( task );
        }
    }
    return tasks;
};

CreateNewTasksTable.prototype.getNotEmptyRowsNumbers = function()
{
    var notEmptyRowsNumbers = [];
    for( var r = 0; r < this.rows.length; r++ )
    {
        if ( !this.rows[r].isEmpty() )
            notEmptyRowsNumbers.push( r );
    }
    return notEmptyRowsNumbers;
};

CreateNewTasksTable.prototype._validationDetected = function()
{
    if ( this.validateCallback )
        this.validateCallback();
};

CreateNewTasksTable.prototype.validate = function()
{
    var allRowsAreEmpty = this.isEmpty();

    var valid = !allRowsAreEmpty;
    for( var i = 0; i < this.rows.length; i++ )
    {
        var row = this.rows[i];
        var rowValid = row.validate();
        valid = valid && rowValid;
    }

    if ( allRowsAreEmpty )
    {
        var firstNameCell = this.rows[0].cells[0];
        firstNameCell.setInvalid();
        firstNameCell.tooltip = '';
        firstNameCell.updateUI();
    }

    var hasRowsWithEqualNames = this.markRowsWithEqualNamesAsInvalid();

    return valid && !hasRowsWithEqualNames;
};

CreateNewTasksTable.prototype.hasErrors = function()
{
    for( var i = 0; i < this.rows.length; i++ )
    {
        if ( this.rows[i].hasErrors() )
            return true;
    }
    return false;
};

CreateNewTasksTable.prototype.isEmpty = function()
{
    for( var r = 0; r < this.rows.length; r++ )
    {
        if ( !this.rows[r].isEmpty() )
            return false;
    }
    return true;
};

CreateNewTasksTable.prototype.markRowsWithEqualNamesAsInvalid = function()
{
    var rowsArray = this.getRowsWithEqualNames();
    var hasInvalid = false;
    for( var i = 0; i < this.rows.length; i++ )
    {
        var cell = this.rows[i].cells[0];
        if( rowsArray[i] )
        {
            cell.setInvalid();
            cell.tooltip = messageResource.getMessage( "add_new_tasks.err_duplicate_task_name" );
            hasInvalid = true;
        }
        else if( !cell.isEmpty() )
        {
            cell.setValid();
        }
        cell.updateUI();
    }
    return hasInvalid;
};

CreateNewTasksTable.prototype.getRowsWithEqualNames = function()
{
    var namesSet = {};
    var rowsWithEqualNames = [false];

    for( var i = 0; i < this.rows.length; i++ )
    {
        var name = this.rows[i].cells[0].getValue().toLowerCase();
        if( name != "" )
        {
            if( namesSet.hasOwnProperty( name ) )
            {
                rowsWithEqualNames[i] = true;
                continue;
            }
            namesSet[name] = i;
        }
    }

    return rowsWithEqualNames;
};

CreateNewTasksTable.prototype.updateUI = function()
{
    this.updateTableClasses();
    for( var i = 0; i < this.rows.length; i++ )
        this.rows[i].updateUI();
    this.updateAddAllToTTCheckboxValue();
};

CreateNewTasksTable.prototype.showTaskNameExistsError = function( rowIdx, isTaskCompleted )
{
    var nameCell = this.rows[rowIdx].cells[0];
    nameCell.setInvalid();
    if ( isTaskCompleted )
        nameCell.tooltip = messageResource.getMessage( "add_new_tasks.err_duplicate_detected.completed_task" );
    else
        nameCell.tooltip = messageResource.getMessage( "add_new_tasks.err_duplicate_detected.open_task" );
    nameCell.updateUI();
};

CreateNewTasksTable.prototype.updateBillingTypes = function( billingTypes )
{
    for( var i = 0; i < this.rows.length; i++ )
        this.rows[i].updateBillingTypes( billingTypes );
    this.validate();
};

CreateNewTasksTable.prototype.hideDescriptionPopup = function( isConfirmNeeded )
{
    this.hideDescriptionPopupCallback( isConfirmNeeded );
};

CreateNewTasksTable.prototype.disableTextSelection = function()
{
    // do not disable selection;
};

