function CreateNewTasksTableRow()
{
}

CreateNewTasksTableRow.prototype = _.create(TableRow.prototype);

CreateNewTasksTableRow.prototype.init = function( record, nameCellRenderer, nameCellValidator, descriptionCellRenderer, descriptionCellValidator,
                                                  estimateCellRenderer, estimateCellValidator, deadlineCellRenderer,
                                                  billingTypeCellRenderer, billingTypeCellValidator, addToTTCellRenderer,
                                                  clearLineCellRenderer )
{
    TableRow.call( this );

    this.nameCell = this.createNameCell( record.name, nameCellRenderer, nameCellValidator );
    this.addCellObj( this.nameCell );

    this.descriptionCell = this.createDescriptionCell( record.description, descriptionCellRenderer, descriptionCellValidator );
    this.addCellObj( this.descriptionCell );

    if ( estimateCellRenderer )
    {
        this.estimateCell = this.createEstimateCell( record.estimate, estimateCellRenderer, estimateCellValidator );
        this.addCellObj( this.estimateCell );
    }

    this.deadlineCell = this.createDeadlineCell( record.deadline, deadlineCellRenderer );
    this.addCellObj( this.deadlineCell );

    if ( billingTypeCellRenderer )
    {
        this.billingTypeCell = this.createBillingTypeCell( record.billingType, billingTypeCellRenderer, billingTypeCellValidator );
        this.addCellObj( this.billingTypeCell );
    }

    if ( addToTTCellRenderer )
    {
        this.addToTTCell = this.createAddToTTCell( record.addToTT, addToTTCellRenderer );
        this.addCellObj( this.addToTTCell );
    }

    this.clearLineCell = this.createClearLineCell( clearLineCellRenderer );
    this.addCellObj( this.clearLineCell );

    return this;
};

CreateNewTasksTableRow.prototype.writeCellsContent = function ()
{
    var cells = this.getCells();
    for( var i = 0; i < cells.length; i++ )
        cells[i].writeContent();
};

CreateNewTasksTableRow.prototype.updateUI = function()
{
    for( var i = 0; i < this.cells.length; i++ )
        this.updateCellUI( i );
};

CreateNewTasksTableRow.prototype.updateClearLineCell = function()
{
    this.clearLineCell.updateUI();
};

CreateNewTasksTableRow.prototype.isEmpty = function()
{
    var isEmpty = this.nameCell.isEmpty() &&
                  this.descriptionCell.isEmpty() &&
                  this.deadlineCell.getValue() == null;
    if ( this.estimateCell )
        isEmpty = isEmpty && this.estimateCell.isEmpty();
    return isEmpty;
};

CreateNewTasksTableRow.prototype.isModified = function()
{
    var isModified = !this.isEmpty();
    if ( this.billingTypeCell )
        isModified = isModified || this.billingTypeCell.getValue() != this.getTable().defaultBillingTypeId;
    return isModified;
};

CreateNewTasksTableRow.prototype.validate = function()
{
    this.nameCell.validate( false );
    this.descriptionCell.validate( true );
    if ( this.estimateCell )
        this.estimateCell.validate( true );
    if ( this.billingTypeCell )
        this.billingTypeCell.validate( false );

    if( this.isEmpty() )
    {
        this.nameCell.setValid();
        if( this.estimateCell )
            this.estimateCell.setValid();
    }

    if ( !this.nameCell.isValid() )
        this.nameCell.tooltip = messageResource.getMessage( "add_new_tasks.err_task_name_empty" );

    this.updateUI();

    return !this.hasErrors();
};

CreateNewTasksTableRow.prototype.hasErrors = function()
{
    if( !this.nameCell.isValid() )
        return true;

    if( !this.descriptionCell.isValid() )
        return true;

    if( this.estimateCell && !this.estimateCell.isValid() )
        return true;

    //noinspection RedundantIfStatementJS
    if( this.billingTypeCell && this.billingTypeCell.isEmpty() )
        return true;

    return false;
};

CreateNewTasksTableRow.prototype.clearLine = function()
{
    if ( this.isModified() )
    {
        if ( confirm( messageResource.getMessage( "add_new_tasks.js.modification_will_lost_confirm" ) ) )
        {
            this.getTable().replaceRowWithNewEmptyRow( this.rowIdx );
            this.getTable().hideDescriptionPopup();
        }
    }
    else if ( this.addToTTCell )
    {
        this.addToTTCell.setValue( false );
        this.addToTTCell.updateUI();
    }

    this.getTable().updateAddAllToTTCheckboxValue();
};

CreateNewTasksTableRow.prototype.getTask = function()
{
    var markedToBeAddedToUserTasks = this.addToTTCell ? this.addToTTCell.getValue() : ( this.getTable().defaultAddToTTValue ? true : false );
    var task = {
        name: this.nameCell.getValue(),
        description: this.descriptionCell.getValue(),
        deadline: this.deadlineCell.getValue(),
        markedToBeAddedToUserTasks: markedToBeAddedToUserTasks
    };
    if ( this.estimateCell )
        task.budgetedTime = this.estimateCell.getValue() ? UserTime.parseTimeWithSystemSettings( this.estimateCell.getValue() ) : null;
    task.billingType = this.billingTypeCell ? this.billingTypeCell.getValue() : this.getTable().defaultBillingTypeId;
    return task;
};

CreateNewTasksTableRow.prototype.setAddToTTValue = function( value )
{
    if ( this.addToTTCell )
    {
        this.addToTTCell.setValue( value );
        this.updateUI();
    }
};

CreateNewTasksTableRow.prototype.createNameCell = function( name, nameCellRenderer, nameCellValidator )
{
    var taskNameCell = new TabularDataTableCell().init( name, nameCellValidator, this );
    taskNameCell.setRenderer( nameCellRenderer );
    return taskNameCell;
};

CreateNewTasksTableRow.prototype.createDescriptionCell = function( description, descriptionCellRenderer, descriptionCellValidator )
{
    var taskDescriptionCell = new TabularDataTableCell().init( description, descriptionCellValidator, this );
    taskDescriptionCell.setRenderer( descriptionCellRenderer );
    return taskDescriptionCell;
};

CreateNewTasksTableRow.prototype.createEstimateCell = function ( estimate, estimateCellRenderer, estimateCellValidator )
{
    var estimateStr = ( estimate || estimate === 0 ) ? UserTime.formatTime( estimate ) : "";
    var taskEstimateCell = new TabularDataTableCell().init( estimateStr, estimateCellValidator, this );
    taskEstimateCell.setRenderer( estimateCellRenderer );
    return taskEstimateCell;
};

CreateNewTasksTableRow.prototype.createDeadlineCell = function ( deadline, deadlineCellRenderer )
{
    var taskDeadlineCell = new TableCell( deadline, this );
    taskDeadlineCell.setRenderer( deadlineCellRenderer );
    return taskDeadlineCell;
};

CreateNewTasksTableRow.prototype.createBillingTypeCell = function ( billingType, billingTypeCellRenderer, billingTypeCellValidator )
{
    var taskBillingTypeCell = new TabularDataTableCell().init( billingType, billingTypeCellValidator, this );
    taskBillingTypeCell.setRenderer( billingTypeCellRenderer );
    return taskBillingTypeCell;
};

CreateNewTasksTableRow.prototype.createAddToTTCell = function ( addToTT, addToTTCellRenderer )
{
    var taskAddToTTCell = new TableCell( addToTT, this );
    taskAddToTTCell.setRenderer( addToTTCellRenderer );
    return taskAddToTTCell;
};

CreateNewTasksTableRow.prototype.createClearLineCell = function( clearLineCellRenderer )
{
    var taskClearLineCell = new TableCell( null, this );
    taskClearLineCell.setRenderer( clearLineCellRenderer );
    return taskClearLineCell;
};

CreateNewTasksTableRow.prototype.updateBillingTypes = function( billingTypes )
{
    this.billingTypeCell.getRenderer().updateBillingTypes( billingTypes );

    $( this.billingTypeCell.getCellEl() ).empty();
    this.billingTypeCell.writeContent();
    this.updateUI();
};

CreateNewTasksTableRow.prototype.highlight = function ()
{
    $( this.getRowEl() ).addClass( "highlighted" );
};

CreateNewTasksTableRow.prototype.unhighlight = function ()
{
    $( this.getRowEl() ).removeClass( "highlighted" );
};

