function CreateTasksCustomerProjectSelector()
{

}

CreateTasksCustomerProjectSelector.MAX_PROJECT_LABEL_LENGTH = 55;

var CreateTasksCustomerProjectSelectorErrors = {};
CreateTasksCustomerProjectSelectorErrors.CUSTOMER_EMPTY = "customerEmpty";
CreateTasksCustomerProjectSelectorErrors.PROJECT_EMPTY = "projectEmpty";
CreateTasksCustomerProjectSelectorErrors.NO_CUSTOMERS = "noActiveCustomer";
CreateTasksCustomerProjectSelectorErrors.NO_PROJECTS = "noActiveProject";
CreateTasksCustomerProjectSelectorErrors.PROJECTS_NOT_SELECTED = "projectsNotSelected";
CreateTasksCustomerProjectSelectorErrors.CUSTOMER_NOT_SELECTED = "customerNotSelected";
CreateTasksCustomerProjectSelectorErrors.PROJECT_NOT_SELECTED = "projectNotSelected";
CreateTasksCustomerProjectSelectorErrors.PROJECT_EXISTS = "projectExists";
CreateTasksCustomerProjectSelectorErrors.CUSTOMER_EXISTS = "customerExists";
var CreateTasksCustomerProjectSelectorWarnings = {};
CreateTasksCustomerProjectSelectorWarnings.NO_CUSTOMERS = "noActiveCustomerWarning";
CreateTasksCustomerProjectSelectorWarnings.NO_PROJECTS = "noActiveProjectWarning";

CreateTasksCustomerProjectSelector.prototype = _.create(CustomerProjectSelectorForTaskPopup.prototype);

CreateTasksCustomerProjectSelector.prototype.init = function( containerId, customerSelectorContainerId,
                                                              projectSelectorContainerId, projectListContainerId,
                                                              newCustomerFieldId, newProjectFieldId,
                                                              customers, projects, canCreateCustomerProject,
                                                              selectedCustomerId, selectedProjectId,
                                                              changesCallback, validationCallback, scrollableContainer )
{
    this.projectListContainer = document.getElementById( projectListContainerId );
    this.canCreateCustomerProject = canCreateCustomerProject;

    this.validationCallback = validationCallback;
    CustomerProjectSelectorForTaskPopup.prototype.init.call(this, containerId, customerSelectorContainerId, projectSelectorContainerId,
                                                            customers, projects, selectedCustomerId, selectedProjectId,
                                                            changesCallback, scrollableContainer, false,  "createTasksCustomerProjectSelector" );
    var self = this;

    this.newCustomerField = document.getElementById( newCustomerFieldId );
    this.newProjectField = document.getElementById( newProjectFieldId );
    $( this.newCustomerField ).placeholder();
    $( this.newProjectField ).placeholder();
    this.addFieldHandlers( this.newCustomerField,
                           [CreateTasksCustomerProjectSelectorErrors.CUSTOMER_EMPTY, CreateTasksCustomerProjectSelectorErrors.CUSTOMER_EXISTS] );
    this.addFieldHandlers( this.newProjectField,
                           [CreateTasksCustomerProjectSelectorErrors.PROJECT_EMPTY, CreateTasksCustomerProjectSelectorErrors.PROJECT_EXISTS] );

    this.errors = {};
    this.warnings = {};

    this.createProjectList( selectedCustomerId ? this.getProjectsOfCustomer( selectedCustomerId ) : this.projects );

    if ( !this.customersExist() )
    {
        this.customerButton.setActiveItemByValue( CREATE_TASKS_CONSTANTS.NEW_CUSTOMER_OPTION_INDEX, true );
        this.projectButton.setActiveItemByValue( CREATE_TASKS_CONSTANTS.NEW_PROJECT_OPTION_INDEX, true );
        $( this.container ).addClass( "newCustomer newProject" );
        setFocusIntoFieldWithPlaceholder( this.newCustomerField );
    }

    return this;
};

CreateTasksCustomerProjectSelector.prototype.addCustomCustomerItems = function( items )
{
    items.push( {
                    text: messageResource.getEscapedMessage( "add_new_tasks.option.select_customer" ),
                    value: CREATE_TASKS_CONSTANTS.SELECT_CUSTOMER_OPTION_INDEX
                } );
    if ( this.canCreateCustomerProject )
    {
        items.push( {
                        text: messageResource.getEscapedMessage( "add_new_tasks.option.new_customer" ),
                        value: CREATE_TASKS_CONSTANTS.NEW_CUSTOMER_OPTION_INDEX
                    } );
    }
    var allActiveCustomersMsgKey = "add_new_tasks.option.all" + ( this.canCreateCustomerProject ? "" : "_available" ) + "_active_customers";
    items.push( {
                    text: messageResource.getEscapedMessage( allActiveCustomersMsgKey ),
                    value: CREATE_TASKS_CONSTANTS.ALL_ACTIVE_CUSTOMERS_OPTION_INDEX
                } );
};

CreateTasksCustomerProjectSelector.prototype.addCustomProjectItems = function( items )
{
    items.push( {
                    text: messageResource.getEscapedMessage( "add_new_tasks.option.select_project" ),
                    value: CREATE_TASKS_CONSTANTS.SELECT_PROJECT_OPTION_INDEX
                } );

    if ( this.canCreateCustomerProject )
    {
        items.push( {
                        text: messageResource.getEscapedMessage( "add_new_tasks.option.new_project" ),
                        value: CREATE_TASKS_CONSTANTS.NEW_PROJECT_OPTION_INDEX
                    } );
    }
    var allActiveProjectsMsgKey = "add_new_tasks.option.all" + ( this.canCreateCustomerProject ? "" : "_available" ) + "_active_projects";
    items.push( {
                    text: messageResource.getEscapedMessage( allActiveProjectsMsgKey ),
                    value: CREATE_TASKS_CONSTANTS.ALL_ACTIVE_PROJECTS_OPTION_INDEX
                },
                {
                    text: messageResource.getEscapedMessage( "add_new_tasks.option.selected_active_projects" ),
                    value: CREATE_TASKS_CONSTANTS.SELECTED_PROJECTS_OPTION_INDEX
                } );
};

CreateTasksCustomerProjectSelector.prototype.createListSelector = function( items, containerEl, selectorCls, changeCallback, selectedValue )
{
    var listSelector = CustomerProjectSelectorForTaskPopup.prototype.createListSelector.call(this, items, containerEl,
                                                                                             selectorCls, changeCallback, selectedValue );

    var handler = function()
    {
        var value = listSelector.getSelectedItem().value;
        if ( value == CREATE_TASKS_CONSTANTS.SELECT_CUSTOMER_OPTION_INDEX ||
             value == CREATE_TASKS_CONSTANTS.SELECT_PROJECT_OPTION_INDEX )
            $( containerEl ).addClass("notSelected");
        else
            $( containerEl ).removeClass("notSelected");
    };

    listSelector.on( "change", function () {
        handler();
    });

    handler();

    return listSelector;
};

CreateTasksCustomerProjectSelector.prototype.createProjectList = function( projects, selectedProjectNamesSet )
{
    if ( !projects || projects.length == 0 )
        return;

    if ( !selectedProjectNamesSet )
        selectedProjectNamesSet = {};

    var projectNamesSet = {};

    var table = document.createElement( "table");
    for ( var i=0; i < projects.length; i++ )
    {
        var project = projects[i];

        if( projectNamesSet.hasOwnProperty( project.name ) )
            continue;

        var row = document.createElement( "tr");
        var checkboxCell = document.createElement( "td");
        var checkBoxId = 'projectName' + project.id;
        var checkbox = createElementWithProperties( "input" , {
            type: 'checkbox',
            id: checkBoxId,
            name: 'selectedProjects',
            projectName: project.name
        });
        checkboxCell.appendChild( checkbox );
        row.appendChild( checkboxCell );

        var labelCell = document.createElement( "td");
        $( labelCell ).html( "<div class='projectListItem'><label for='" + checkBoxId + "'>" + escapeHtmlSymbols( project.name ) + "</label></div>" );
        if ( project.name.length > CreateTasksCustomerProjectSelector.MAX_PROJECT_LABEL_LENGTH )
            $( labelCell ).attr( 'title', escapeHtmlSymbols( project.name ) );

        row.appendChild( labelCell );
        table.appendChild( row );

        this.addCheckboxChangeHandler( checkbox, labelCell );

        if ( selectedProjectNamesSet.hasOwnProperty( project.name ) )
        {
            checkbox.checked = true;
            $( labelCell ).addClass("projectNameChecked");
        }

        projectNamesSet[project.name] = true;
    }
    this.projectListContainer.appendChild( table );
};

CreateTasksCustomerProjectSelector.prototype.addCheckboxChangeHandler = function( checkbox, labelCell )
{
    var self = this;
    $( checkbox ).change( function() {
        self.removeError( CreateTasksCustomerProjectSelectorErrors.PROJECTS_NOT_SELECTED );
        if( checkbox.checked )
            $( labelCell ).addClass("projectNameChecked");
        else
            $( labelCell ).removeClass("projectNameChecked");
    });
};

CreateTasksCustomerProjectSelector.prototype.customerChanged = function()
{
    this.removeAllErrors();
    this.removeWarning( CreateTasksCustomerProjectSelectorWarnings.NO_CUSTOMERS );
    this.removeWarning( CreateTasksCustomerProjectSelectorWarnings.NO_PROJECTS );

    $( this.container ).removeClass( "newCustomer" );

    var customerValue = this.getSelectedCustomerId();
    var projectValue = this.getSelectedProjectId();

    switch ( parseInt( customerValue ) )
    {
        case CREATE_TASKS_CONSTANTS.SELECT_CUSTOMER_OPTION_INDEX:
        {
            this.projectButton.setActiveItemByValue( CREATE_TASKS_CONSTANTS.SELECT_PROJECT_OPTION_INDEX );
            this.loadCustomerProjects();
            break;
        }
        case CREATE_TASKS_CONSTANTS.ALL_ACTIVE_CUSTOMERS_OPTION_INDEX:
        {
            if ( projectValue != CREATE_TASKS_CONSTANTS.ALL_ACTIVE_PROJECTS_OPTION_INDEX &&
                 projectValue != CREATE_TASKS_CONSTANTS.NEW_PROJECT_OPTION_INDEX &&
                 projectValue != CREATE_TASKS_CONSTANTS.SELECTED_PROJECTS_OPTION_INDEX &&
                 projectValue != CREATE_TASKS_CONSTANTS.SELECT_PROJECT_OPTION_INDEX ) {
                this.projectButton.setActiveItemByValue( CREATE_TASKS_CONSTANTS.SELECT_PROJECT_OPTION_INDEX );
            }

            this.loadCustomerProjects();
            if( !this.customersExist() )
            {
                this.addWarning( CreateTasksCustomerProjectSelectorWarnings.NO_CUSTOMERS );
            }
            if( !this.projectsExist() &&
                ( projectValue == CREATE_TASKS_CONSTANTS.ALL_ACTIVE_PROJECTS_OPTION_INDEX ||
                  projectValue == CREATE_TASKS_CONSTANTS.SELECTED_PROJECTS_OPTION_INDEX ) )
            {
                this.addWarning( CreateTasksCustomerProjectSelectorWarnings.NO_PROJECTS );
            }
            break;
        }
        case CREATE_TASKS_CONSTANTS.NEW_CUSTOMER_OPTION_INDEX:
        {
            if ( this.canCreateCustomerProject )
            {
                $( this.container ).addClass( "newCustomer" );
                this.projectButton.setActiveItemByValue( CREATE_TASKS_CONSTANTS.NEW_PROJECT_OPTION_INDEX );
                setFocusIntoFieldWithPlaceholder( this.newCustomerField );
            }
            break;
        }
        default:
        {
            if ( this.customerHasProjects( customerValue ) ) {
                this.loadCustomerProjects();
            } else {
                this.projectButton.setActiveItemByValue( CREATE_TASKS_CONSTANTS.NEW_PROJECT_OPTION_INDEX );
            }
        }
    }
};

CreateTasksCustomerProjectSelector.prototype.projectChanged = function()
{
    this.removeAllErrors();
    this.removeWarning( CreateTasksCustomerProjectSelectorWarnings.NO_PROJECTS );

    $( this.container ).removeClass( "newProject selectProjectsFromList" );

    var customerValue = this.getSelectedCustomerId();
    var projectValue = this.getSelectedProjectId();

    if ( projectValue == CREATE_TASKS_CONSTANTS.ALL_ACTIVE_PROJECTS_OPTION_INDEX )
    {
        if ( !this.projectsExist() ||
            ( this.isConcreteCustomerId( customerValue ) &&
              !this.customerHasProjects( customerValue ) ) )
        {
            this.addWarning( CreateTasksCustomerProjectSelectorWarnings.NO_PROJECTS );
        }
    }
    else if ( projectValue == CREATE_TASKS_CONSTANTS.SELECTED_PROJECTS_OPTION_INDEX )
    {
        $( this.container ).addClass( "selectProjectsFromList" );
        if( !this.projectsExist() || (this.isConcreteCustomerId( customerValue ) && !this.customerHasProjects( customerValue )) )
        {
            this.addWarning( CreateTasksCustomerProjectSelectorWarnings.NO_PROJECTS );
        }
        if( customerValue != CREATE_TASKS_CONSTANTS.ALL_ACTIVE_CUSTOMERS_OPTION_INDEX && !this.isConcreteCustomerId( customerValue ))
        {
            this.customerButton.setActiveItemByValue( CREATE_TASKS_CONSTANTS.ALL_ACTIVE_CUSTOMERS_OPTION_INDEX );
        }
    }
    else if ( this.isConcreteProjectId( projectValue ) )
    {
        var customerId = this.findProjectCustomerId( projectValue );
        if( customerId != customerValue ){
            this.customerButton.setActiveItemByValue( customerId );
            this.loadCustomerProjects();
        }
    }
    else if ( projectValue == CREATE_TASKS_CONSTANTS.NEW_PROJECT_OPTION_INDEX && this.canCreateCustomerProject )
    {
        $( this.container ).addClass( "newProject" );
        this.loadCustomerProjects();
        setFocusIntoFieldWithPlaceholder( this.newProjectField );
    }
};

CreateTasksCustomerProjectSelector.prototype.loadCustomerProjects = function()
{
    var projects = CustomerProjectSelectorForTaskPopup.prototype.loadCustomerProjects.call(this );

    this.updateProjectList( projects );
};

CreateTasksCustomerProjectSelector.prototype.updateProjectList = function( projects )
{
    var selectedProjectsNamesSet = this.getSelectedProjectsNamesSet();

    $( this.projectListContainer ).empty();
    this.createProjectList( this.getSelectedCustomerId() != CREATE_TASKS_CONSTANTS.ALL_ACTIVE_CUSTOMERS_OPTION_INDEX ? projects : this.projects,
                            selectedProjectsNamesSet );
};

CreateTasksCustomerProjectSelector.prototype.hasSelectedProjects = function()
{
    var checkboxes = document.getElementsByName("selectedProjects");
    for( var i=0; i<checkboxes.length; i++ )
    {
        if( checkboxes[i].checked )
            return true;
    }
    return false;
};

CreateTasksCustomerProjectSelector.prototype.getSelectedProjectsNames = function()
{
    var selectedProjects = [];
    var checkboxes = document.getElementsByName("selectedProjects");
    for( var i=0; i<checkboxes.length; i++ )
    {
        if ( checkboxes[i].checked )
            selectedProjects.push( checkboxes[i].projectName );
    }
    return selectedProjects;
};

CreateTasksCustomerProjectSelector.prototype.getSelectedProjectsNamesSet = function()
{
    var selectedProjectsNamesSet = {};
    var checkboxes = document.getElementsByName("selectedProjects");
    for( var i=0; i<checkboxes.length; i++ )
    {
        if ( checkboxes[i].checked )
            selectedProjectsNamesSet[checkboxes[i].projectName] = true;
    }
    return selectedProjectsNamesSet;
};

CreateTasksCustomerProjectSelector.prototype.isConcreteCustomerId = function(selected)
{
    return !isArrayContains([CREATE_TASKS_CONSTANTS.SELECT_CUSTOMER_OPTION_INDEX,
                             CREATE_TASKS_CONSTANTS.NEW_CUSTOMER_OPTION_INDEX,
                             CREATE_TASKS_CONSTANTS.ALL_ACTIVE_CUSTOMERS_OPTION_INDEX], selected);
};

CreateTasksCustomerProjectSelector.prototype.isConcreteProjectId = function(selected)
{
    return !isArrayContains([CREATE_TASKS_CONSTANTS.SELECT_PROJECT_OPTION_INDEX,
                             CREATE_TASKS_CONSTANTS.NEW_PROJECT_OPTION_INDEX,
                             CREATE_TASKS_CONSTANTS.ALL_ACTIVE_PROJECTS_OPTION_INDEX,
                             CREATE_TASKS_CONSTANTS.SELECTED_PROJECTS_OPTION_INDEX], selected);
};

CreateTasksCustomerProjectSelector.prototype.validate = function()
{
    this.removeErrors( [CreateTasksCustomerProjectSelectorErrors.CUSTOMER_EMPTY,
                        CreateTasksCustomerProjectSelectorErrors.PROJECT_EMPTY,
                        CreateTasksCustomerProjectSelectorErrors.CUSTOMER_NOT_SELECTED,
                        CreateTasksCustomerProjectSelectorErrors.PROJECT_NOT_SELECTED,
                        CreateTasksCustomerProjectSelectorErrors.PROJECTS_NOT_SELECTED] );

    var customerValue = this.getSelectedCustomerId();
    var projectValue = this.getSelectedProjectId();

    var projectsWarningShowed = this.hasWarning( CreateTasksCustomerProjectSelectorWarnings.NO_PROJECTS );

    if( customerValue == CREATE_TASKS_CONSTANTS.SELECT_CUSTOMER_OPTION_INDEX )
        this.addError( CreateTasksCustomerProjectSelectorErrors.CUSTOMER_NOT_SELECTED );

    if( projectValue == CREATE_TASKS_CONSTANTS.SELECT_PROJECT_OPTION_INDEX )
        this.addError( CreateTasksCustomerProjectSelectorErrors.PROJECT_NOT_SELECTED );

    if( projectsWarningShowed )
        this.addError( CreateTasksCustomerProjectSelectorErrors.NO_PROJECTS );

    if( this.hasWarning( CreateTasksCustomerProjectSelectorWarnings.NO_CUSTOMERS ) )
        this.addError( CreateTasksCustomerProjectSelectorErrors.NO_CUSTOMERS );

    if( projectValue == CREATE_TASKS_CONSTANTS.SELECTED_PROJECTS_OPTION_INDEX && !projectsWarningShowed && !this.hasSelectedProjects() )
        this.addError( CreateTasksCustomerProjectSelectorErrors.PROJECTS_NOT_SELECTED );

    if( customerValue == CREATE_TASKS_CONSTANTS.NEW_CUSTOMER_OPTION_INDEX && $( this.newCustomerField ).pVal() == "" )
        this.addError( CreateTasksCustomerProjectSelectorErrors.CUSTOMER_EMPTY );

    if( projectValue == CREATE_TASKS_CONSTANTS.NEW_PROJECT_OPTION_INDEX && $( this.newProjectField ).pVal() == "" )
        this.addError( CreateTasksCustomerProjectSelectorErrors.PROJECT_EMPTY );

    return !this.hasErrors();
};

CreateTasksCustomerProjectSelector.prototype.hasErrors = function()
{
    //noinspection LoopStatementThatDoesntLoopJS
    for( var prop in this.errors ) {
        if( this.errors[prop] == true )
            return true;
    }
    return false;
};

CreateTasksCustomerProjectSelector.prototype.addFieldHandlers = function( field, errorsToRemove )
{
    var self = this;
    var keyUpChangesHandler = function() {
        this.keyPressed = false;
        if( (!this.prevValue && !!this.value) || this.prevValue != this.value )
        {
            self.removeErrors( errorsToRemove );
            if ( self.changesCallback )
                self.changesCallback();
            this.prevValue = this.value;
        }
    };

    $( field ).keypress( function() {
        this.keyPressed = true;
    } );

    $( field ).keyup( function() { keyUpChangesHandler.call( this ); } );

    $( field ).blur( function() {
        $(this).pVal( trim( $(this).pVal() ) );
        //see Bug 48539
        if ( this.keyPressed ) keyUpChangesHandler.call( this );
    });
};

CreateTasksCustomerProjectSelector.prototype.getValues = function()
{
    var customerValue = this.getSelectedCustomerId();
    var projectValue = this.getSelectedProjectId();

    var newCustomerName = customerValue == CREATE_TASKS_CONSTANTS.NEW_CUSTOMER_OPTION_INDEX ?
                          $( this.newCustomerField ).pVal() :
                          null;
    var newProjectName = projectValue == CREATE_TASKS_CONSTANTS.NEW_PROJECT_OPTION_INDEX ?
                         $( this.newProjectField ).pVal() :
                         null;

    var selectedProjectNames = projectValue == CREATE_TASKS_CONSTANTS.SELECTED_PROJECTS_OPTION_INDEX ?
                               this.getSelectedProjectsNames() : null;

    return {
        customerId: customerValue,
        projectId: projectValue,
        newCustomerName: newCustomerName,
        newProjectName: newProjectName,
        selectedProjectNames: selectedProjectNames
    };
};

CreateTasksCustomerProjectSelector.prototype.addError = function( error )
{
    this.errors[error] = true;
    $( this.container ).addClass( error );
};

CreateTasksCustomerProjectSelector.prototype.removeError = function( error )
{
    this.errors[error] = false;
    $( this.container ).removeClass( error );

    if ( this.validationCallback )
        this.validationCallback();
};

CreateTasksCustomerProjectSelector.prototype.removeErrors = function( errors )
{
    for ( var i=0; i<errors.length; i++ )
        this.removeError( errors[i] );
};

CreateTasksCustomerProjectSelector.prototype.removeAllErrors = function()
{
    this.removeErrors( [CreateTasksCustomerProjectSelectorErrors.CUSTOMER_EMPTY,
                        CreateTasksCustomerProjectSelectorErrors.PROJECT_EMPTY,
                        CreateTasksCustomerProjectSelectorErrors.CUSTOMER_NOT_SELECTED,
                        CreateTasksCustomerProjectSelectorErrors.PROJECT_NOT_SELECTED,
                        CreateTasksCustomerProjectSelectorErrors.NO_CUSTOMERS,
                        CreateTasksCustomerProjectSelectorErrors.NO_PROJECTS,
                        CreateTasksCustomerProjectSelectorErrors.CUSTOMER_EXISTS,
                        CreateTasksCustomerProjectSelectorErrors.PROJECT_EXISTS,
                        CreateTasksCustomerProjectSelectorErrors.PROJECTS_NOT_SELECTED] );
};

CreateTasksCustomerProjectSelector.prototype.addCustomerNotSelectedError = function()
{
    this.addError( CreateTasksCustomerProjectSelectorErrors.CUSTOMER_NOT_SELECTED );
};

CreateTasksCustomerProjectSelector.prototype.addProjectNotSelectedError = function()
{
    this.addError( CreateTasksCustomerProjectSelectorErrors.PROJECT_NOT_SELECTED );
};

CreateTasksCustomerProjectSelector.prototype.addNoActiveCustomersError = function()
{
    this.addError( CreateTasksCustomerProjectSelectorErrors.NO_CUSTOMERS );
};

CreateTasksCustomerProjectSelector.prototype.addNoActiveProjectsError = function()
{
    this.addError( CreateTasksCustomerProjectSelectorErrors.NO_PROJECTS );
};

CreateTasksCustomerProjectSelector.prototype.addCustomerExistsError = function()
{
    this.addError( CreateTasksCustomerProjectSelectorErrors.CUSTOMER_EXISTS );
};

CreateTasksCustomerProjectSelector.prototype.addProjectExistsError = function()
{
    this.addError( CreateTasksCustomerProjectSelectorErrors.PROJECT_EXISTS );
};

CreateTasksCustomerProjectSelector.prototype.addWarning = function( warning )
{
    this.warnings[ warning ] = true;
    $( this.container ).addClass( warning );
};

CreateTasksCustomerProjectSelector.prototype.removeWarning = function( warning )
{
    this.warnings[ warning ] = false;
    $( this.container ).removeClass( warning );
};

CreateTasksCustomerProjectSelector.prototype.hasWarning = function( warning )
{
    return this.warnings[ warning ];
};

CreateTasksCustomerProjectSelector.prototype.removeProject = function( projectId )
{
    for ( var i=0; i < this.projects.length; i++ )
    {
        if ( this.projects[i].id == projectId )
        {
            this.projects.splice( i, 1 );
            break;
        }
    }

    this.updateProjectSelector();
};

CreateTasksCustomerProjectSelector.prototype.setCanCreateCustomerProject = function( canCreateCustomerProject )
{
    this.canCreateCustomerProject = canCreateCustomerProject;

    this.createCustomerSelector( this.getSelectedCustomerId() );
    this.createProjectSelector( this.projects, this.getSelectedProjectId() );
    this.customerChanged();
};

CreateTasksCustomerProjectSelector.prototype.removeAllProjectsOfSelectedCustomer = function()
{
    var customerValue = this.getSelectedCustomerId();
    if ( !this.isConcreteCustomerId( customerValue ) )
    {
        this.setProjectsAndUpdateSelector( null );
    }
    else
    {
        for ( var i=0; i < this.projects.length; i++ )
        {
            if ( this.projects[i].customerId == customerValue )
                this.projects.splice( i, 1 );
        }

        this.createProjectSelector( this.projects );
    }
};

CreateTasksCustomerProjectSelector.prototype.removeProjectsFromList = function( selectedProjectNames, customerId )
{
    for ( var i=0; i < this.projects.length; i++ )
    {
        if ( $.inArray( this.projects[i].name, selectedProjectNames ) != -1 &&
             ( !this.isConcreteCustomerId( customerId ) || this.projects[i].customerId == customerId ) )
        {
            this.projects.splice( i, 1 );
            i--;
        }
    }

    $( this.container ).removeClass( "selectProjectsFromList" );
    this.updateProjectSelector();
};

CreateTasksCustomerProjectSelector.prototype.updateProjectSelector = function( saveSelectedProjectId )
{
    var visibleProjects = CustomerProjectSelectorForTaskPopup.prototype.updateProjectSelector.call( this, saveSelectedProjectId );

    this.updateProjectList( visibleProjects );
};
