function CustomerProjectSelectorForTaskPopup()
{

}

CustomerProjectSelectorForTaskPopup.prototype.init = function( containerId, customerSelectorContainerId, projectSelectorContainerId,
                                                               customers, projects, selectedCustomer, selectedProject,
                                                               changesCallback, scrollableContainer, includeInactiveProjects, containerClass )
{
    this.container = document.getElementById( containerId );
    this.customerSelectorContainer = document.getElementById( customerSelectorContainerId );
    this.projectSelectorContainer = document.getElementById( projectSelectorContainerId );

    this.changesCallback = changesCallback;
    this.scrollableContainer = scrollableContainer;

    this.customers = customers ? customers : [];
    this.projects = projects ? projects : [];

    this.withInactive = includeInactiveProjects;

    this.containerClass = containerClass;
    this.createCustomerSelector( selectedCustomer );
    this.createProjectSelector( selectedCustomer ? this.getProjectsOfCustomer( selectedCustomer ) : this.projects, selectedProject );

    return this;
};

CustomerProjectSelectorForTaskPopup.prototype.createCustomerListItems = function()
{
    var items = [];

    this.addCustomCustomerItems( items );

    for (var i = 0; i < this.customers.length; i++) {
        var customer = this.customers[i];
        items.push( {   text: escapeHtmlSymbols( customer.name.concat(customer.activeCustomer ? "" : " "+messageResource.getMessage("common.archived")) ),
                        value: customer.id } );
    }

    return items;
};

CustomerProjectSelectorForTaskPopup.prototype.addCustomCustomerItems = function( items )
{
    // abstract
};

CustomerProjectSelectorForTaskPopup.prototype.createProjectListItems = function( projects )
{
    var items = [];

    this.addCustomProjectItems( items );

    for (var i = 0; i < projects.length; i++) {
        items.push( {   text: escapeHtmlSymbols( projects[i].name.concat(projects[i].activeProject ? "" : " "+messageResource.getMessage("common.archived")) ),
                        value: projects[i].id } );
    }

    return items;
};

CustomerProjectSelectorForTaskPopup.prototype.addCustomProjectItems = function( items )
{
    // abstract
};

CustomerProjectSelectorForTaskPopup.prototype.createCustomerSelector = function( selectedValue )
{
    var self = this;
    this.customerButton = this.createListSelector( this.createCustomerListItems(),
                                                   this.customerSelectorContainer,
                                                   "customerListSelector",
                                                   function() { self.customerChanged(); },
                                                   selectedValue );
};

CustomerProjectSelectorForTaskPopup.prototype.createProjectSelector = function( projects, selectedValue )
{
    var self = this;
    this.projectButton = this.createListSelector( this.createProjectListItems( projects ),
                                                  this.projectSelectorContainer,
                                                  "projectListSelector",
                                                  function() { self.projectChanged(); },
                                                  selectedValue );
};

CustomerProjectSelectorForTaskPopup.prototype.createListSelector = function( items, containerEl, selectorCls, changeCallback, selectedValue )
{
    $( containerEl ).empty();

    var listSelector = new ListSelector({
                                            renderTo: containerEl,
                                            listItems: items,
                                            menuCls: "customerProjectListSelector " + selectorCls + " " + this.containerClass,
                                            scrollableContainer: this.scrollableContainer
                                        });

    var self = this;
    listSelector.on( "change", function () {
        if ( changeCallback )
            changeCallback( listSelector.getSelectedItem().value );
        if ( self.changesCallback )
            self.changesCallback();
    });

    if ( selectedValue )
        listSelector.setActiveItemByValue( selectedValue, true );

    return listSelector;
};

CustomerProjectSelectorForTaskPopup.prototype.customerChanged = function()
{
    this.loadCustomerProjects();
};

CustomerProjectSelectorForTaskPopup.prototype.projectChanged = function()
{
    // abstract
};

CustomerProjectSelectorForTaskPopup.prototype.loadCustomerProjects = function()
{
    var customerValue = this.getSelectedCustomerId();
    var projectValue = this.getSelectedProjectId();

    var projects = this.getProjectsOfCustomer( customerValue, this.withInactive );

    this.createProjectSelector( projects, projectValue );

    return projects;
};

CustomerProjectSelectorForTaskPopup.prototype.getSelectedCustomerId = function()
{
    return this.customerButton.getValue();
};

CustomerProjectSelectorForTaskPopup.prototype.getSelectedProjectId = function()
{
    return this.projectButton.getValue();
};

CustomerProjectSelectorForTaskPopup.prototype.customerHasProjects = function(customerId)
{
    for (var i=0; i<this.projects.length; i++) {
        if (this.projects[i].customerId == customerId)
            return true;
    }
    return false;
};

CustomerProjectSelectorForTaskPopup.prototype.findProjectCustomerId = function(projectId)
{
    for (var i=0; i<this.projects.length; i++) {
        if (this.projects[i].id == projectId)
            return this.projects[i].customerId;
    }
    return -1;
};

CustomerProjectSelectorForTaskPopup.prototype.getProjectsOfCustomer = function(customerId)
{
    var result = [];
    for (var i=0; i<this.projects.length; i++) {
        var project = this.projects[i];
        if ((customerId <= 0 || project.customerId == customerId) && ( this.withInactive || project.activeProject ))
            result[result.length] = project;
    }
    return result;
};

CustomerProjectSelectorForTaskPopup.prototype.setCustomersAndProjectsAndUpdateSelector = function( customers, projects )
{
    this.customers = customers ? customers : [];
    this.createCustomerSelector();

    this.setProjectsAndUpdateSelector( projects );
};

CustomerProjectSelectorForTaskPopup.prototype.setProjectsAndUpdateSelector = function( projects )
{
    this.projects = projects ? projects : [];
    this.updateProjectSelector( true );
};

CustomerProjectSelectorForTaskPopup.prototype.updateProjectSelector = function( saveSelectedProjectId )
{
    var selectedCustomerId = this.getSelectedCustomerId();
    var visibleProjects = this.isConcreteCustomerId( selectedCustomerId ) ? this.getProjectsOfCustomer( selectedCustomerId ) : this.projects;
    this.createProjectSelector( visibleProjects, saveSelectedProjectId ? this.getSelectedProjectId(): null );

    return visibleProjects;
};

CustomerProjectSelectorForTaskPopup.prototype.customersExist = function()
{
    return this.customers.length != 0;
};

CustomerProjectSelectorForTaskPopup.prototype.projectsExist = function()
{
    return this.projects.length != 0;
};
