function TaskBillingTypeCellRenderer()
{}

TaskBillingTypeCellRenderer.prototype = _.create(TableCellRenderer.prototype);

TaskBillingTypeCellRenderer.BILLING_TYPE_NOT_SELECTED_VALUE = -1;

TaskBillingTypeCellRenderer.prototype.init = function( billingTypes, defaultBillingTypeId, changesCallback, scrollableContainer )
{
    TableCellRenderer.call( this );
    this.billingTypes = billingTypes;
    this.defaultBillingTypeId = defaultBillingTypeId;
    this.changesCallback = changesCallback;
    this.scrollableContainer = scrollableContainer;
    return this;
};

TaskBillingTypeCellRenderer.prototype.generateContent = function( cell )
{
    var self = this;

    var div = document.createElement( "div" );

    cell.getCellEl().appendChild( div );

    if ( cell.getValue() == null )
        cell.setValue( self.defaultBillingTypeId );

    var listItems = [];

    var selectedBillingTypeExists = false;

    for (var i = 0; i < this.billingTypes.length; i++)
    {
        var billingType = this.billingTypes[i];
        listItems.push( { text: escapeHtmlSymbols(billingType.name), value: billingType.id } );

        if ( cell.getValue() == billingType.id )
            selectedBillingTypeExists = true;
    }

    if ( !selectedBillingTypeExists )
    {
        listItems.unshift( {
                               text: messageResource.getMessage( 'add_new_tasks.not_selected' ),
                               value: TaskBillingTypeCellRenderer.BILLING_TYPE_NOT_SELECTED_VALUE
                           } );
        cell.setValue( TaskBillingTypeCellRenderer.BILLING_TYPE_NOT_SELECTED_VALUE );
    }

    var billingTypeSelector = new ListSelector({
                                               menuCls: "billingTypeSelectorMenu addNewTasks",
                                               renderTo: div,
                                               value: cell.getValue(),
                                               listItems: listItems,
                                               scrollableContainer: this.scrollableContainer
                                           });
    billingTypeSelector.on( "change", function ()
    {
        cell.setValue( billingTypeSelector.getSelectedItem().value );
        cell.updateUI();
        self.changesCallback( cell );
    } );
    cell.billingTypeSelector = billingTypeSelector;

    if ( !selectedBillingTypeExists )
    {
        cell.updateUI();
    }
};

TaskBillingTypeCellRenderer.prototype.updateUI = function( cell )
{
    TableCellRenderer.prototype.updateUI.call( this, cell );

    cell.billingTypeSelector.setActiveItemByValue( cell.getValue(), true );

    if ( cell.isValid() )
        $( cell.getCellEl() ).removeClass( "error" );
    else
        $( cell.getCellEl() ).addClass( "error" );
};

TaskBillingTypeCellRenderer.prototype.updateBillingTypes = function( billingTypes )
{
    this.billingTypes = billingTypes ? billingTypes : [];
};

TaskBillingTypeCellRenderer.prototype.getCellClassName = function( cell )
{
    return ["billingTypeCell"];
};
