function TaskDeadlineCellRenderer()
{}

TaskDeadlineCellRenderer.prototype = _.create(TableCellRenderer.prototype);

TaskDeadlineCellRenderer.prototype.init = function( weekStartDay, scheduleCalculator, changesCallback, scrollableContainer,
                                                    noValueMessage, deadlineInThePastMessage, checkPastDate )
{
    TableCellRenderer.call( this );
    this.weekStartDay = weekStartDay;
    this.scheduleCalculator = scheduleCalculator;
    this.changesCallback = changesCallback;
    this.scrollableContainer = scrollableContainer;
    this.noValueMessage = noValueMessage;
    this.deadlineInThePastMessage = deadlineInThePastMessage;
    this.checkPastDate = checkPastDate;
    return this;
};

TaskDeadlineCellRenderer.prototype.generateContent = function( cell, createClearDateAfterSelector )
{
    cell.getCellEl().innerHTML = "<div></div>";

    var self = this;

    var clearDateBtn = document.createElement( "div" );
    $( clearDateBtn ).addClass( "clearDate" );
    $( clearDateBtn ).click( function() {
        cell.setValue( null );
        cell.updateUI();
        self.changesCallback( cell );
    });

    var clearDateContainer = document.createElement( "div" );
    $( clearDateContainer ).addClass( "clearDateContainer" );

    clearDateContainer.appendChild( clearDateBtn );
    if ( !createClearDateAfterSelector )
        cell.getCellEl().firstChild.appendChild( clearDateContainer );

    var dateSelector = new DateSelector({
                                            minValue: new Date(1970, 0, 1),
                                            weekStartDay: this.weekStartDay,
                                            renderTo: cell.getCellEl().firstChild,
                                            scheduleCalculator: this.scheduleCalculator,
                                            date: cell.getValue(),
                                            format: Date.convertSmartToExtDateFormat( SystemSettings.dateFormat ).replace(/F/, "M"),
                                            scrollableContainer: this.scrollableContainer,
                                            displayValue: function ( date )
                                            {
                                                this.setText( date ? this.formatDate( date ) : self.noValueMessage );
                                            }
                                        });
    dateSelector.on("change", function() {
        cell.setValue( dateSelector.getValue() );
        cell.updateUI();
        self.changesCallback( cell );
    });
    cell.dateSelector = dateSelector;

    if ( createClearDateAfterSelector )
        cell.getCellEl().firstChild.appendChild( clearDateContainer );
};

TaskDeadlineCellRenderer.prototype.updateUI = function( cell )
{
    TableCellRenderer.prototype.updateUI.call( this, cell );
    this.updateDeadlineCell(cell);

};

TaskDeadlineCellRenderer.prototype.updateDeadlineCell = function( cell )
{
    var date = cell.getValue();

    if ( date )
    {
        var today = currentUserTime().clearTime().getTime();
        if (this.checkPastDate && today > date.getTime())
        {
            $( cell.getCellEl() ).addClass( "deadlineInPast" );
            cell.getCellEl().title = this.deadlineInThePastMessage;
        }
        else
        {
            $( cell.getCellEl() ).removeClass( "deadlineInPast" );
            cell.getCellEl().title = "";
        }

        $( cell.getCellEl() ).removeClass( "deadlineNotSet" );
        cell.dateSelector.setValue( date );
    }
    else
    {
        $( cell.getCellEl() ).addClass( "deadlineNotSet" );
        $( cell.getCellEl() ).removeClass( "deadlineInPast" );
        cell.getCellEl().title = "";
        cell.dateSelector.unsetValue();
    }
};

TaskDeadlineCellRenderer.prototype.getCellClassName = function( cell )
{
    return ["deadlineCell"];
};

