function TaskDescriptionCellRenderer()
{}

TaskDescriptionCellRenderer.prototype = _.create(TableCellRenderer.prototype);

TaskDescriptionCellRenderer.prototype.init = function( showTaskDescriptionPopupCallback, changesCallback )
{
    TableCellRenderer.call( this );

    this.descriptionTitle = messageResource.getMessage('add_new_tasks_tooltip_enable_description.task');
    this.showTaskDescriptionPopupCallback = showTaskDescriptionPopupCallback;
    this.changesCallback = changesCallback;

    return this;
};

TaskDescriptionCellRenderer.prototype.generateContent = function( cell )
{
    var table = document.createElement( "table" );
    table.className = "descriptionCellTable";
    var tbody = document.createElement( "tbody" );
    table.appendChild(tbody);
    var tr = document.createElement( "tr" );
    tbody.appendChild(tr);

    var descriptionCellId = "descriptionCell" + cell.getRow().getRowIdx();
    var descriptionTd = createElementWithProperties("td", {
        id: descriptionCellId
    });

    var self = this;
    var showTaskDescriptionPopup = function()
    {
        self.showTaskDescriptionPopupCallback( cell.getValue(), descriptionCellId, descriptionTd,
                                               function( value ) {
                                                   cell.setValue( value );
                                                   cell.updateUI();
                                                   self.changesCallback( cell );
                                               });
    };

    descriptionTd.appendChild( this.buildDescriptionElement( cell, cell.getValue(), showTaskDescriptionPopup ) );
    tr.appendChild(descriptionTd);

    cell.getCellEl().appendChild( table );
};

TaskDescriptionCellRenderer.prototype.buildDescriptionElement = function( cell, description, showTaskDescriptionPopupCallback ) {
    var that = this;
    var a = createElementWithProperties("a", {
        id: "descriptionElement",
        href : "#",
        title : this.descriptionTitle,
        onclick : function() {
            showTaskDescriptionPopupCallback();
        }
    });
    var img = document.createElement( "img" );
    a.appendChild(img);
    this.setDescriptionImage( img, trim( description ) == '' );
    cell.descriptionImg = img;
    return a;
};

TaskDescriptionCellRenderer.prototype.setDescriptionImage = function( img, isEmptyDescription )
{
    if ( isEmptyDescription ) {
        img.src = imgPath("dialogs/comments/comment_blank.png");
    } else {
        img.src = imgPath("dialogs/comments/comment_entered.png");
    }
};

TaskDescriptionCellRenderer.prototype.updateUI = function( cell )
{
    TableCellRenderer.prototype.updateUI.call( this, cell );

    this.setDescriptionImage( cell.descriptionImg, trim( cell.getValue()) == '' );

    if ( cell.isValid() )
    {
        $( cell.getCellEl() ).removeClass( "error" );
    }
    else
    {
        $( cell.getCellEl() ).addClass( "error" );
    }
};

TaskDescriptionCellRenderer.prototype.getCellClassName = function( cell )
{
    return ["descriptionCell"];
};
