function CreateTasksPopup()
{
}

CreateTasksPopup.prototype = _.create(BasicLightbox.prototype);

CreateTasksPopup.INITIAL_NUMBER_OF_ROWS = 3;
CreateTasksPopup.INITIAL_NUMBER_OF_ROWS_WHEN_OPENED_FROM_OPEN_TASKS = 5;
CreateTasksPopup.ERROR_IN_FOOTER_KEY = "add_new_tasks.js.footer_error.invalid_fields_exist";

CreateTasksPopup.prototype.close_tab_confirmation_message_key = "add_new_tasks.exit_confirmation";
CreateTasksPopup.prototype.exist_invalid_fields_message_key = "add_new_tasks.js.footer_error.invalid_fields_exist";

CreateTasksPopup.prototype.init = function( serviceName, customers, projects, billingTypes, defaultBillingTypeId,
                                            canCreateCustomerProject, canAddTasksToUser,
                                            weekStartDay, scheduleCalculator, estEnabled,
                                            projectsWithTasksExist, isOpenedFromOpenTasks,
                                            selectedCustomerId, selectedProjectId,
                                            successTaskCreationCallback, userId, dayOnWeek )
{
    this.popupId = "createTasksPopup";
    this.popupEl = $( "#" + this.popupId );
    this.serviceName = serviceName;

    this.cancel_confirmation_message_key = "add_new_tasks.js.cancel_confirmation";
    this.commitButtonTitlePlaceholder = $(this.getElementBySubId("commitBtn")).find(".buttonTitle");
    this.customers = customers;
    this.projects = projects;
    this.billingTypes = billingTypes;
    this.defaultBillingTypeId = defaultBillingTypeId;

    this.canCreateCustomerProject = canCreateCustomerProject;
    this.canAddTasksToUser = canAddTasksToUser;
    this.projectsWithTasksExist = projectsWithTasksExist;

    this.estEnabled = estEnabled;

    this.weekStartDay = weekStartDay;
    this.scheduleCalculator = scheduleCalculator;

    this.isOpenedFromOpenTasks = isOpenedFromOpenTasks;
    this.userId = userId;
    this.dayOnWeek = dayOnWeek;

    this.successTaskCreationCallback = successTaskCreationCallback;

    BasicLightbox.prototype.init.call( this, this.popupId, "closeCreateTasksPopupButton" );

    var self = this;

    this.setCancelCallback( function() { self.hide(); } );
    this.setCommitButtonTitle(messageResource.getMessage("add_new_tasks.create_tasks_btn_title"),
                              messageResource.getMessage("add_new_tasks.default_create_tasks_btn_title"), 17);

    this._changesCallback = function()
    {
        self.dataChanged = true;
    };

    this.customerProjectSelector = new CreateTasksCustomerProjectSelector().init( this.getElementIdBySubId( "selectCustomerAndProject" ),
                                                                                  this.getElementIdBySubId( "customerSelector" ),
                                                                                  this.getElementIdBySubId( "projectSelector" ),
                                                                                  this.getElementIdBySubId( "projectList" ),
                                                                                  this.getElementIdBySubId( "newCustomer" ),
                                                                                  this.getElementIdBySubId( "newProject" ),
                                                                                  this.customers, this.projects, this.canCreateCustomerProject,
                                                                                  selectedCustomerId, selectedProjectId,
                                                                                  this._changesCallback, this._validateCallback,
                                                                                  this.getScrollableContainer() );

    this.initCreateTasksComponent();

    this.setSuccessCallback(
        function( customerId, newCustomerName, projectId, newProjectName, selectedProjectNames, tasks ) {
            self.sendCreateTasksRequest( customerId, newCustomerName, projectId, newProjectName, selectedProjectNames, tasks );
        }
    );

    return this;
};

CreateTasksPopup.prototype.getElementBySubId = function( id )
{
    return document.getElementById( this.getElementIdBySubId( id ) );
};

CreateTasksPopup.prototype.getElementIdBySubId = function( id )
{
    return this.popupId + "_" + id;
};

CreateTasksPopup.prototype.hide = function()
{
    this.createTasksComponent.hideDescriptionPopup();
    this.createTasksComponent.restoreGetOnPageDescriptionFunction();
    BasicLightbox.prototype.hide.call( this );
};

CreateTasksPopup.prototype.getScrollableContainer = function()
{
    return this.popupEl.find( ".innerContent" )[0];
};

/* offset of the innerContent in the popup content
*  used when scrolling to section with error*/
CreateTasksPopup.prototype.getContentOffset = function() {
    return -60;
};

CreateTasksPopup.prototype.scrollToCustomerProjectSelector = function() {
    this.scrollToSection( $(this.getElementBySubId("selectCustomerAndProject")) );
};

CreateTasksPopup.prototype.scrollToTasksTable = function() {
    this.scrollToSection( $(this.getElementBySubId("importTasksContainer")) );
};

CreateTasksPopup.prototype.scrollToFirstTaskWithError = function() {
    var taskWithErr = this.popupEl.find(".createTasksTable .nameCell .error");
    if ( taskWithErr.get(0) )
        this.scrollToSection( taskWithErr );
    else
        this.scrollToTasksTable();
};

CreateTasksPopup.prototype.validate = function()
{
    var errorMsgKey = CreateTasksPopup.ERROR_IN_FOOTER_KEY;

    if ( this.customerProjectSelector.validate() )
    {
        if ( this.createTasksComponent.validate() )
        {
            return true;
        }

        if ( this.createTasksComponent.isEmpty() ) {
            errorMsgKey = "add_new_tasks.there_are_no_tasks_to_create";
            this.scrollToTasksTable();
        } else {
            this.scrollToFirstTaskWithError();
        }
    }
    else
        this.scrollToCustomerProjectSelector();

    this._printErrorInFooter( messageResource.getMessage( errorMsgKey ) );
    this._blinkErrorInFooter();
    return false;
};

CreateTasksPopup.prototype.hasErrors = function()
{
    return (this.customerProjectSelector && this.customerProjectSelector.hasErrors() || (this.createTasksComponent && this.createTasksComponent.hasErrors()));
};

CreateTasksPopup.prototype.initCreateTasksComponent = function()
{
    var defaultRowsNumber = this.isOpenedFromOpenTasks ?
                            CreateTasksPopup.INITIAL_NUMBER_OF_ROWS_WHEN_OPENED_FROM_OPEN_TASKS :
                            CreateTasksPopup.INITIAL_NUMBER_OF_ROWS;

    this.createTasksComponent = new CreateNewTasksComponent().init( this.popupId, this.billingTypes,
                                                                    this.defaultBillingTypeId, this.canAddTasksToUser,
                                                                    this.weekStartDay, this.scheduleCalculator,
                                                                    this.estEnabled, this.projectsWithTasksExist,
                                                                    defaultRowsNumber, this.isOpenedFromOpenTasks,
                                                                    this._changesCallback, this._validateCallback,
                                                                    this.getScrollableContainer() );
};

CreateTasksPopup.prototype.commit = function()
{
    if ( this.createTasksComponent.hideDescriptionPopup( true ) )
        BasicLightbox.prototype.commit.call( this );
};

CreateTasksPopup.prototype.getDataToCommit = function()
{
    var cpValues = this.customerProjectSelector.getValues();
    var tasks = this.createTasksComponent.getTasks();

    return [ cpValues.customerId, cpValues.newCustomerName,
             cpValues.projectId, cpValues.newProjectName,
             cpValues.selectedProjectNames,
             tasks ];
};

CreateTasksPopup.prototype.sendCreateTasksRequest = function( customerId, newCustomerName, projectId, newProjectName, selectedProjectNames, tasks )
{
    this.disableUserInputWhileSaving();
    var self = this;

    var notEmptyTaskRowsNumbers = this.createTasksComponent.getCreateTasksTable().getNotEmptyRowsNumbers();

    var errorHandler = function( jqXHR, textStatus, errorThrown )
    {
        self.enableUserInputWhileSaving();

        if( errorThrown.exception == 'com.actimind.actitime.services.BaseNewTasksService$NoRightsToManageTasksException' )
        {
            alert( messageResource.getMessage( 'add_new_tasks.no_right_create_tasks' ) );
            self.dataChanged = false;
            document.location.reload();
            return true;
        }
        if( errorThrown.exception == 'com.actimind.actitime.services.BaseNewTasksService$NoRightsToManageCustomersAndProjectException' )
        {
            alert( messageResource.getMessage( 'add_new_tasks.no_right_create_project_customer' ) );
            self.customerProjectSelector.setCanCreateCustomerProject( false );
            if ( newCustomerName )
                self.customerProjectSelector.addCustomerNotSelectedError();
            if ( newProjectName )
                self.customerProjectSelector.addProjectNotSelectedError();
            return true;
        }
        if( errorThrown.exception == 'com.actimind.actitime.services.BaseNewTasksService$NoRightsToManageSelectedProjectException' )
        {
            alert( messageResource.getMessage( 'add_new_tasks.project_unavailable' ) );
            self.customerProjectSelector.removeProject( projectId );
            self.customerProjectSelector.addProjectNotSelectedError();
            return true;
        }
        return false;
    };

    var successHandler = function( result )
    {
        self.enableUserInputWhileSaving();

        if ( result.customerProjectError )
        {
            self.handleCustomerProjectError( result.customerProjectError,
                                             customerId,
                                             projectId,
                                             selectedProjectNames,
                                             result.customers ? result.customers.list : null,
                                             result.projects ? result.projects.list : null );
        }
        else if ( result.taskError )
        {
            var rowNumber = notEmptyTaskRowsNumbers[ result.taskErrorRowNumber ];
            self.createTasksComponent.handleTaskError( result.taskError, rowNumber );
        }else if(result.billingTypeError){
            self.createTasksComponent.handleBillingTypeError( result.billingTypeError, result.billingTypes ? result.billingTypes.list : null );
        }
        else
        {
            self.hide();
            if ( self.successTaskCreationCallback )
            {
                self.successTaskCreationCallback( result );
            }
            else
                document.location.reload();
        }

        //todo: task list is empty case : alert( messageResource.getMessage( "add_new_tasks.js.unexpected_error" ) );

        if( result.customerProjectError || result.taskError ){
            self._printErrorInFooter(messageResource.getMessage(CreateTasksPopup.ERROR_IN_FOOTER_KEY));
        }

        if ( result.customerProjectError )
            self.scrollToCustomerProjectSelector();
        else
            self.scrollToFirstTaskWithError();
    };

    var data = [customerId, newCustomerName, projectId, newProjectName, selectedProjectNames, tasks];

    if ( !this.isOpenedFromOpenTasks && this.userId && this.dayOnWeek )
    {
        data.push( this.userId );
        data.push( this.dayOnWeek );
    }

    makeRpcCall( this.serviceName + ".createNewTasks", data, successHandler, errorHandler );
};

CreateTasksPopup.prototype.handleCustomerProjectError = function( customerProjectError, customerId, projectId, selectedProjectNames, customers, projects )
{
    var msgKey;

    switch ( customerProjectError )
    {
        case CREATE_TASKS_CONSTANTS.CUSTOMER_IS_NOT_EXIST_ERROR:
            alert( messageResource.getMessage( 'add_new_tasks.customer_deleted' ) );
            this.customerProjectSelector.setCustomersAndProjectsAndUpdateSelector( customers, projects );
            this.customerProjectSelector.addCustomerNotSelectedError();
            break;

        case CREATE_TASKS_CONSTANTS.CUSTOMER_EXISTS_ERROR:
            this.customerProjectSelector.addCustomerExistsError();
            break;

        case CREATE_TASKS_CONSTANTS.CUSTOMER_IS_ARCHIVED_ERROR:
            var customerName = this.getCustomerName( customerId );
            msgKey = this.canCreateCustomerProject ? 'add_new_tasks.customer_archived' : 'add_new_tasks.customer_archived.cant_manage_projects';
            alert( messageResource.getMessage( msgKey, [customerName] ) );
            this.customerProjectSelector.setCustomersAndProjectsAndUpdateSelector( customers, projects );
            this.customerProjectSelector.addCustomerNotSelectedError();
            break;

        case CREATE_TASKS_CONSTANTS.NO_ACTIVE_CUSTOMERS_ERROR:
            this.customerProjectSelector.setCustomersAndProjectsAndUpdateSelector( null, null );
            this.customerProjectSelector.addNoActiveCustomersError();
            break;

        case CREATE_TASKS_CONSTANTS.PROJECT_IS_NOT_EXIST_ERROR:
            alert( messageResource.getMessage( 'add_new_tasks.project_unavailable' ) );
            this.customerProjectSelector.setProjectsAndUpdateSelector( projects );
            this.customerProjectSelector.addProjectNotSelectedError();
            break;

        case CREATE_TASKS_CONSTANTS.PROJECT_EXISTS_ERROR:
            this.customerProjectSelector.addProjectExistsError();
            break;

        case CREATE_TASKS_CONSTANTS.PROJECT_IS_ARCHIVED_ERROR:
            var projectName = this.getProjectName( projectId );
            msgKey = this.canCreateCustomerProject ? 'add_new_tasks.project_archived' : 'add_new_tasks.project_archived.cant_manage_projects';
            alert( messageResource.getMessage( msgKey, [projectName] ) );
            this.customerProjectSelector.setProjectsAndUpdateSelector( projects );
            this.customerProjectSelector.addProjectNotSelectedError();
            break;

        case CREATE_TASKS_CONSTANTS.NO_ACTIVE_PROJECTS_ERROR:
            if ( selectedProjectNames )
            {
                this.customerProjectSelector.removeProjectsFromList( selectedProjectNames, customerId );
                if ( selectedProjectNames.length > 1 )
                    alert( messageResource.getMessage( 'add_new_tasks.projects_unavailable' ) );
                else
                    alert( messageResource.getMessage( 'add_new_tasks.project_unavailable' ) );
            }
            else
            {
                this.customerProjectSelector.removeAllProjectsOfSelectedCustomer();
                this.customerProjectSelector.addNoActiveProjectsError();
            }
            break;
    }
};

CreateTasksPopup.prototype.getCustomerName = function( customerId )
{
    for (var c=0; c<this.customers.length; c++) {
        if (this.customers[c].id == customerId)
            return this.customers[c].name;
    }
    return "";
};

CreateTasksPopup.prototype.getProjectName = function( projectId )
{
    for (var j=0; j<this.projects.length; j++) {
        if (this.projects[j].id == projectId)
            return this.projects[j].name;
    }
    return "";
};

// functions to check if we can ignore projectId  and customerId and use them from local storage

CreateTasksPopup.isIdValid = function(beans, id)
{
    if (_.isNull(id))
        return false;

    for (var j = 0; j < beans.length; j++)
    {
        if (beans[j].id == id)
            return true;
    }
    return false;
};

CreateTasksPopup.isCustomerIdValid = function(customers, id)
{
    if (id == CREATE_TASKS_CONSTANTS.ALL_ACTIVE_CUSTOMERS_OPTION_INDEX)
        return true;
    return CreateTasksPopup.isIdValid(customers, id);
};

CreateTasksPopup.isProjectIdValid = function(projects, id)
{
    if (id == CREATE_TASKS_CONSTANTS.ALL_ACTIVE_PROJECTS_OPTION_INDEX)
        return true;
    return CreateTasksPopup.isIdValid(projects, id);
};