function AMIHistoryPopup()
{
    this.constructor = AMIHistoryPopup;
}

AMIHistoryPopup.prototype.init = function (historyLightboxDiv, loadDataPortionCallback, closeCallback)
{
    this.historyLightboxDiv = historyLightboxDiv;
    this.loadDataPortionCallback = loadDataPortionCallback;

    var self = this;
    this.tableRenderer = new TTAHistoryTableRenderer().init(
        this.historyLightboxDiv.find(".tableContentContainer table"),
        {
            showTooltipCallback: function(event, comments) {
                self.showCommentsTooltip(event, comments);
            },
            hideTooltipCallback: function() {
                self.hideCommentsTooltip();
            },
            moveTooltipCallback: function(event) {
                self.moveCommentsTooltip(event);
            }
        }
    );

    this.historyPopup = new DropdownDialog( {
        wrapperClass: 'extendedTTAHistoryLightBox',
        dialogDiv: this.historyLightboxDiv,
        hideButtons: true,
        isWithBeak: true,
        hideOnResize: true,
        onClose: closeCallback
    } );

    this.moreRecordsButtonContainer = this.historyLightboxDiv.find(".moreRecordsButtonContainer");

    this.showMoreButton = this.historyLightboxDiv.find("#showMoreHistoryRecordsButton");
    this.showMoreButton.click( function() {
        self.loadDataPortionCallback(self.lastLoadedRecordId);
    });

    this.commentsTooltip = new TTAHistoryCommentTooltip().init("extendedTTAHistoryCommentsTooltip");

    return this;
};

AMIHistoryPopup.prototype.show = function(historyRecords, hasMoreRecords, timestampOfLastReadRecord, pressedButton)
{
    this.timestampOfLastReadRecord = timestampOfLastReadRecord;

    this.tableRenderer.render(historyRecords, timestampOfLastReadRecord);
    this.updateLastLoadedRecordId(historyRecords);
    this.loadedRecordsNumber = historyRecords.length;

    this.historyPopup.showAt(pressedButton, {
        offset: "-136px 15px",
        my:  "left top",
        at: "left bottom",
        collision: "none"
    });

    this.updateMoreRecordsContainer(hasMoreRecords);
    if (historyRecords.length == 0)
        this.moreRecordsButtonContainer.hide();
};

AMIHistoryPopup.prototype.addRecords = function(historyRecords, hasMoreRecords)
{
    this.tableRenderer.addRecords(historyRecords, this.timestampOfLastReadRecord);
    this.updateLastLoadedRecordId(historyRecords);
    this.loadedRecordsNumber += historyRecords.length;

    this.updateMoreRecordsContainer(hasMoreRecords);
};

AMIHistoryPopup.prototype.updateLastLoadedRecordId = function(historyRecords)
{
    if (historyRecords.length > 0)
        this.lastLoadedRecordId = historyRecords[historyRecords.length - 1].id;
};

AMIHistoryPopup.prototype.showCommentsTooltip = function(event, comments)
{
    this.commentsTooltip.updateUIAndShow( {comments: comments}, calculateTooltipOffset( this.commentsTooltip, event, ApprovalsTableController.BOTTOM_BAR_HEIGHT ));
};

AMIHistoryPopup.prototype.moveCommentsTooltip = function(event)
{
    this.commentsTooltip.move( calculateTooltipOffset( this.commentsTooltip, event, ApprovalsTableController.BOTTOM_BAR_HEIGHT ) );
};

AMIHistoryPopup.prototype.hideCommentsTooltip = function()
{
    this.commentsTooltip.hide();
};

AMIHistoryPopup.prototype.updateMoreRecordsContainer = function(hasMoreRecords)
{
    if ( hasMoreRecords )
        this.showMoreButton.html(messageResource.getMessage("approve_tt.tta_history.show_more_records", [this.loadedRecordsNumber]));
    else
        this.moreRecordsButtonContainer.find(".noMoreHistoryRecords").html(messageResource.getMessage("approve_tt.tta_history.no_more_records", [this.loadedRecordsNumber]));

    this.moreRecordsButtonContainer.toggleClass("noMoreRecords", !hasMoreRecords);
};

