function ApprovalsManagementMediator ()
{
    this.constructor = ApprovalsManagementMediator;
}

ApprovalsManagementMediator.prototype.init = function ( config )
{
    var self = this;
    this.leaveTypeMap = {};

    this.currentState = {};
    this.currentState.approveTTFilter = config.approveTTFilter;
    this.currentState.sortBy = config.sortBy;
    this.currentState.searchText = "";
    this.currentState.enabledFeatures = config.enabledFeatures;
    this.currentState.groupingStrategy = this.createInitGroupingStrategy(this.currentState.sortBy.getGroupingType());

    this.topLevelApprovalStatus = null;
    this.topLevelStatusTimestamp = null;

    this.approvalsTableController = new ApprovalsTableController().init("approveTTTable",
                                                                        this._isFeatureEnabled(ApprovalPageConstants.OVRFeature),
                                                                        this._isFeatureEnabled(ApprovalPageConstants.LVTFeature),
                                                                        this._isFeatureEnabled(ApprovalPageConstants.UGRFeature),
                                                                        this._isFeatureEnabled(ApprovalPageConstants.TZGFeature),
                                                                        function (groupingRow) { return self.getMoreChildRows(groupingRow) },
                                                                        function (userInfo){return self._getGroupName(userInfo)},
                                                                        function (leaveTypeId){return self._getLeaveType(leaveTypeId)},
                                                                        function (groupRow){return self.revokeGroupingRow(groupRow)},
                                                                        function (childRow){return self.revokeChildRow(childRow)},
                                                                        function (newApproveButtonEnabled, newRejectButtonEnable) {
                                                                            self.updateButtonsState(newApproveButtonEnabled, newRejectButtonEnable)
                                                                        });
    this.updateTableController();
    this.spinnerController = new SpinnerController();

    this.newState = $.extend( true, {}, this.currentState );

    this.monthSelector = this.initMonthSelector();

    var placeHolderTextKey = "approve_tt.user_input.placeholder";
    var onChangeCallback = function(){self._searchTermsChangedCallback()};
    this.inputSearchField = new FilterWordsField().initAndRender(onChangeCallback, document.getElementById("userNameFilter"), placeHolderTextKey, onChangeCallback);

    this.approvalStatusfilter =  new ApprovalStatusSelector().init("#approvalStatusSelectorPanel", "#statusFilterButton", this.currentState.approveTTFilter.statuses,function(){ self.changeFilterValueCallback()});

    this.initUsersSelector( config.userGroupsEnabled, config.users, config.userGroups, config.timeZoneGroups, config.approveTTFilter.allUsers, config.approveTTFilter.userIds );
    this.initSortButtons();
    this.initExpandCollapseButtons();
    this.initRejectCommentDialog();

    this.groupingRowToLoadOnStartCount = Math.ceil($(window).height() / 80) + 2; //80 is one collapsed groupingRow's height
    this.groupingRowToLoadOnScrollCount = 10;
    this.childRowToLoadOnStartMoreCount = 15;
    this.childRowToLoadOnShowMoreCount = 15;

    $(window).scroll($.proxy(self.checkScrollPositionAndLoadMoreGroupingRows, self));
    $(window).resize($.proxy(self.checkScrollPositionAndLoadMoreGroupingRows, self));

    this.initApproveButtons();
    this.updateFilters();
    this.loadPageData(false);
    this.initHistoryPopup(config.numberOfUnreadTTAHistoryRecords);

    this.namesOperationsKeys = {};
    this.namesOperationsKeys[WeekApprovalStatus.APPROVED] = "approve_tt.approve_operation";
    this.namesOperationsKeys[WeekApprovalStatus.REJECTED] = "approve_tt.reject_operation";

    return this;
};

ApprovalsManagementMediator.prototype.disableLoadDataOnScrollAndResize = function()
{
    this.canLoadMoreGroupingRows = false;
};

ApprovalsManagementMediator.prototype.enableLoadDataOnScrollAndResize = function()
{
    this.canLoadMoreGroupingRows = true;
};


ApprovalsManagementMediator.prototype.checkScrollPositionAndLoadMoreGroupingRows = function()
{
    var self = this;
    var heightDocument = $( document ).height();
    var scrollPosition =$( window ).scrollTop() + $( window ).height();

    // -3 and +3 for fixed bug 52338 (zoom browser != 100 %
    if( (scrollPosition > heightDocument - 3) && (scrollPosition < heightDocument + 3) && !self.allGroupingRowsLoaded && self.canLoadMoreGroupingRows )
    {
        self.disableLoadDataOnScrollAndResize();
        self.loadMoreGroupingRows();
    }
};

ApprovalsManagementMediator.prototype.loadMoreGroupingRows = function()
{
    var self = this;
    if ($( ".loadMoreSpinnerContainer" ).length)
    {
        this.approvalsTableController.changeButtonToSpinner();
        this.spinnerController.spin( "loadMoreContent" );
    }
    this.getMoreGroupingRows(self.approvalsTableController.hasNoRecords() ? null : this.getLastGroupingRowId(), false, this.groupingRowToLoadOnScrollCount);
};

ApprovalsManagementMediator.prototype.initMonthSelector = function()
{
    var self = this;

    return  new ApprovalMonthSelector().init(
            {
                panelElId: "#approvalMonthSelectorPanel",
                selectorCellId: "#approvalMonthSelector",
                onChangeCallback: function(){ self.changeFilterValueCallback() },
                onShowCallBack : function(renderCalendars)
                {
                    var yearsCount = 10; //to show in month selector
                    var startYear = new Date().getFullYear() - yearsCount + 1;

                    makeRpcCall(
                            "ApprovalManagementService.getMonthsInfo",
                            [startYear],
                            function( data ) {
                                renderCalendars(startYear, yearsCount, data.monthsData, data.today.parseJSONDate().getMonth());
                            },
                            function ( jqXHR, textStatus, errorThrown ) {
                                return false;
                            }
                    );
                },
                selectedDate: self.currentState.approveTTFilter.monthDate,
                spinnerController: self.spinnerController
            }
    );
};

ApprovalsManagementMediator.prototype.updateNumberOfUnreadHistoryRecords = function(numberOfUnreadRecords)
{
    var numberOfRecordsEl = $('#ttaHistoryRecordsCount');
    numberOfRecordsEl.html(numberOfUnreadRecords);
    numberOfRecordsEl.toggle(numberOfUnreadRecords != 0);
};

ApprovalsManagementMediator.prototype.initHistoryPopup = function(numberOfUnreadTTAHistoryRecords)
{
    this.updateNumberOfUnreadHistoryRecords(numberOfUnreadTTAHistoryRecords);

    var self = this;
    this.historyPopup = new AMIHistoryPopup().init($("#extendedTTAHistoryLightBoxDiv"),
        function(lastLoadedRecordId) {
            self.loadTTAHistoryRecords(
                [lastLoadedRecordId, ApprovalPageConstants.HISTORY_TABLE_PORTION_SIZE],
                function( historyData ) {
                    self.historyPopup.addRecords(historyData.historyRecords.list, historyData.hasMoreRecords);
                },
                "spinnerPlaceholderTTAHistoryLightBox"
            );
        },
        function() {
            self.updateNumberOfUnreadHistoryRecords(0);
        }
    );

    var ttaHistoryButton = $("#ttaHistoryButton");
    ttaHistoryButton.click(function() {
        self.spinnerController.spin("spinnerPlaceholderTitle");
        self.loadTTAHistoryRecords(
            [ApprovalPageConstants.HISTORY_TABLE_INITIAL_PORTION_SIZE],
            function( historyData ) {
                self.historyPopup.show(historyData.historyPortion.historyRecords.list, historyData.historyPortion.hasMoreRecords,
                    historyData.timestampOfLastReadRecord, ttaHistoryButton);
                self.updateNumberOfUnreadHistoryRecords(historyData.numberOfUnreadRecords);
            },
            "spinnerPlaceholderTitle"
        );
    });
};

ApprovalsManagementMediator.prototype.loadTTAHistoryRecords = function(data, successHandler, spinnerPlaceholderId)
{
    var self = this;
    this.abortLoadHistoryData();
    this.spinnerController.spin(spinnerPlaceholderId);
    this.loadHistoryDataRequest = makeRpcCall(
        "ApprovalManagementService.getTTAHistoryPortion",
        data,
        function( historyData ) {
            self.spinnerController.stop();
            successHandler(historyData);
        },
        function ( jqXHR, textStatus, errorThrown ) {
            self.spinnerController.stop();
            if( textStatus == "abort" )
            {
                return true;
            }
            if( errorThrown.exception == 'com.actimind.actitime.services.FeatureIsDisabledException' )
            {
                alert( messageResource.getMessage('approve_tt.tta_history.tta_feature_disabled'));
                document.location.reload();
                return true;
            }
        }
    );
};

ApprovalsManagementMediator.prototype.getLastGroupingRowId = function()
{
    return this.approvalsTableController.getLastGroupingRowModel();
};

ApprovalsManagementMediator.prototype.getLastChildRowId = function( groupingRow )
{
    return this.approvalsTableController.getLastChildRowModel( groupingRow );
};

ApprovalsManagementMediator.prototype.getGroupingRowId = function( groupingRow )
{
    return this.approvalsTableController.getRowModel( groupingRow );
};

ApprovalsManagementMediator.prototype.loadPageData = function(isUpdateControllerRendererAfterLoad)
{
    this.spinnerController.spin("spinnerPlaceholderTitle");
    
    this.topLevelApprovalStatus = null;
    this.topLevelStatusTimestamp = null;

    this.getMoreGroupingRows(null, isUpdateControllerRendererAfterLoad, this.groupingRowToLoadOnStartCount);
    this.approvalsTableController.deselectAllRows();
};

ApprovalsManagementMediator.prototype.updateTableController = function(){
    var self = this;
    this.approvalsTableController.updateRenderer(self.currentState.groupingStrategy);
};

ApprovalsManagementMediator.prototype.getMoreGroupingRows = function ( lastGroupingRowId, isUpdateControllerRendererAfterLoad, limit)
{
    var self = this;
    var successHandler = function(approvalManagementData){
                    self.initUserGroupsMap(approvalManagementData.userGroups);
                    self.initTimeZoneGroupsMap(approvalManagementData.timeZoneGroups);
                    if (isUpdateControllerRendererAfterLoad){
                        self.updateTableController();
                    }

                    if(lastGroupingRowId == null){
                        self.approvalsTableController.removeTable();
                    }

                    self.approvalsTableController.updateControllerStateAndUI({tokens : self.tokens, sortOrder: self.currentState.sortBy});
                    //self.lastGroupingRowId = approvalManagementData.rowsData.rows

                    if (self.approvalsTableController.isEmptyTable() && approvalManagementData.rowsData == null){
                        self.approvalsTableController.addNoRecordsRow();
                    }else
                    {
                        if (approvalManagementData.rowsData != null)
                        {
                            self.addGroupingRows( approvalManagementData.rowsData );
                            self.allGroupingRowsLoaded = !approvalManagementData.rowsData.hasMoreRows;
                        }else{
                            self.allGroupingRowsLoaded = true;
                        }
                        if(!self.allGroupingRowsLoaded)
                            self.approvalsTableController.addSpinnerRowToBottom(function() { self.loadMoreGroupingRows() });

                    }
                    self.spinnerController.stop();
                    self.enableLoadDataOnScrollAndResize();

                    PageOverlayTipsManager.getInstance().refreshDynamicTips();
     };

    var errorHandler = function (result, textStatus, errorThrown) {
        self.spinnerController.stop();

        if( errorThrown.exception == 'com.actimind.actitime.services.UserIsNotAssignedToManagerException' ) {
            alert( messageResource.getMessage("approve_tt.approve_table.users_is_not_assigned"));
            document.location.reload();
            return true;
        }
        
        self._revertState();
    };

    this.abortLoadPageData();
    this.abortUpdateStatus();

    this._loadPageDataRequestObject = this.newState.groupingStrategy.getMoreGroupingRows(lastGroupingRowId,
                                                    limit,
                                                    self.childRowToLoadOnStartMoreCount,
                                                    self.getParams(),
                                                    successHandler,
                                                    errorHandler,
                                                    function (approvalManagementData ) {
                                                        self.successLoadPageData(approvalManagementData, successHandler);
                                                        return true;
                                                    },
                                                    function (result, textStatus, jqXHR){
                                                        if( textStatus == "abort" )
                                                        {
                                                            return true;
                                                        }
                                                        if (errorHandler)
                                                            return errorHandler(result, textStatus, jqXHR);
                                                    });
};

ApprovalsManagementMediator.prototype.getMoreChildRows = function ( groupingRow )
{
    var self = this;
    var successHandler = function(approvalManagementData){
        self.spinnerController.stop();
        if (approvalManagementData.rows != null)
            self.addChildRows(groupingRow, approvalManagementData.rows);    // todo : fix this part
        else
            self.showNoMoreChildRowsToLoad(groupingRow);
    };

    var errorHandler = function (result, textStatus, errorThrown){
        self.spinnerController.stop();
        if( textStatus == "abort" )
        {
            self._revertState();
            return true;
        }
    };

    this.abortLoadPageData();
    this.abortUpdateStatus();
    this.abortRevoke();

    var filter = self.getParams();
    var grInfo = groupingRow.data("rowInfo");
    
    this.spinnerController.spin("spinnerPlaceholderTitle");
    this._loadPageDataRequestObject = this.newState.groupingStrategy.getMoreChildRows(self.getGroupingRowId(groupingRow),
                                                                                      self.getLastChildRowId( groupingRow ),
                                                                                      {   
                                                                                          filter: filter, 
                                                                                          groupTimestamp: grInfo.statusTimestamp, 
                                                                                          groupStatus: grInfo.weekApprovalStatus 
                                                                                      },
                                                                                      this.childRowToLoadOnShowMoreCount,
                                                                                      function(approvalManagementData){self.successLoadPageData(approvalManagementData,successHandler)},
                                                                                      errorHandler)
};

ApprovalsManagementMediator.prototype.revokeGroupingRow = function ( groupingRow )
{
    var self = this;
    var groupingRowInfo = groupingRow.data("rowInfo");
    var timestamp = groupingRowInfo.statusTimestamp;
    var status = groupingRowInfo.weekApprovalStatus;
    var selection = this.approvalsTableController.getRevokeRowsSelectionForGroupingRow(groupingRow);

    var successHandler = function(){
        self.approvalsTableController.updateGroupRowAfterRevoke(selection.getGroupingRowId(), {});
        PageOverlayTipsManager.getInstance().refreshDynamicTips();
    };

    this.makeRevokeCall(this.currentState.groupingStrategy.getServiceMethodForGroupingRevoke(), [selection.getGroupingRowId(), selection.getChildRowsIds(), status, timestamp ], false, successHandler );
};

ApprovalsManagementMediator.prototype.revokeChildRow = function ( childRow )
{
    var self = this;

    var childRowId = this.approvalsTableController.getUserWeekForChildRow($(childRow));
    var childRowInfo = $(childRow).data("rowInfo");
    var timestamp = childRowInfo.statusTimestamp;
    var status = childRowInfo.weekApprovalStatus;

    var additionalSuccessHandler = function () {
        self.approvalsTableController.onChildRowRevoked(childRowId);
    };
    this.makeRevokeCall("ApprovalManagementService.revokeUserWeek",[childRowId, status, timestamp ], true,  additionalSuccessHandler );
};

ApprovalsManagementMediator.prototype.makeRevokeCall = function(nameService, params, isChildRow, successHandler){
    var self = this;
    this.abortLoadPageData();
    this.spinnerController.spin("spinnerPlaceholderTitle");
    this.approvalsTableController.disableAllShowMoreRows();
    var successRevokeHandler = function(revokeWeeksStatusResult){
        var warnings = revokeWeeksStatusResult.warnings;
        self.approvalsTableController.enableAllShowMoreRows();
        var errorHandlerIfNoSuchUserOrNotAssigned = function(messageKeyIfSomeUser, messageKeyIfOneUser){
           return isChildRow? self.errorHandlerIfNotSuchUserOrNotAssigned([],[params[0]],  messageKeyIfSomeUser, messageKeyIfOneUser) :
                            self.errorHandlerIfNotSuchUserOrNotAssigned([params[0]],params[1], messageKeyIfSomeUser, messageKeyIfOneUser);
        };

        if ($.inArray(ApprovalPageConstants.WARNING_SOME_USERS_NOT_EXIST, warnings) != -1){
            errorHandlerIfNoSuchUserOrNotAssigned("approve_tt.approve_table.revoke_some_users_deleted","approve_tt.approve_table.user_deleted");
            window.location.reload();
            return;
        }

        if ($.inArray(ApprovalPageConstants.WARNING_NO_RIGHTS_TO_APPROVE_SOME_USERS, warnings) != -1){
            errorHandlerIfNoSuchUserOrNotAssigned("approve_tt.approve_table.revoke_users_not_assigned","approve_tt.approve_table.user_was_not_assigned");
            window.location.reload();
            return;
        }

        self.updateChildRowsStatus(revokeWeeksStatusResult.userWeeks);

        if (successHandler){
            successHandler();
        }
    };

    var errorHandler = function (result, textStatus, errorThrown){
        self.spinnerController.stop();
        self.approvalsTableController.enableAllShowMoreRows();
        if( textStatus == "abort" )
        {
            return true;
        }
    };

    this.revokeRequest = makeRpcCall(nameService, params,
                                            function(revokeWeeksStatusResult){self.successOperationHandler(revokeWeeksStatusResult, successRevokeHandler)},
                                            errorHandler, null, LONG_RPC_TIMEOUT
    );

};

ApprovalsManagementMediator.prototype.getParams = function()
{
    this.tokens = this.newState.searchText == null ? [] : new StringTokenizer(StringTokenizer.PUNCTUATORS_FOR_SEARCH).getTokens( this.newState.searchText.toLowerCase() );
    var sortBy = {asc: this.newState.sortBy.getAsc(), groupType:this.newState.sortBy.getGroupingType() };

    var params = [];
    params.push(this.newState.approveTTFilter);
    params.push(this.tokens);
    params.push(sortBy);

    return params;
};

ApprovalsManagementMediator.prototype.abortLoadPageData = function(){
    this._clearLoadPageDataTimer();
    if (this._loadPageDataRequestObject)
    {
        this._loadPageDataRequestObject.abort();
    }
};

ApprovalsManagementMediator.prototype.abortUpdateStatus = function(){
    if (this.updateStatuesRequest)
    {
        this.updateStatuesRequest.abort();
    }
};

ApprovalsManagementMediator.prototype.abortRevoke = function(){
    if (this.revokeRequest)
    {
        this.revokeRequest.abort();
    }
};

ApprovalsManagementMediator.prototype.abortLoadHistoryData = function(){
    if (this.loadHistoryDataRequest)
    {
        this.loadHistoryDataRequest.abort();
    }
};

ApprovalsManagementMediator.prototype._getGroupName = function(userInfo){
    if (this.userGroupMap != null)
        return this.userGroupMap[userInfo.userGroupId];

    if (this.timeZoneGroupMap == null) return null;
    return this.timeZoneGroupMap[userInfo.timeZoneGroupId];
};

ApprovalsManagementMediator.prototype.successLoadPageData = function(approvalManagementData, successHandler){

    var featuresDiff = new FeatureDiffStateController().init(this.currentState.enabledFeatures, approvalManagementData.enabledFeatures);

    this.updateCurrentState(approvalManagementData);
    this.usersSelectorPanel.saveCurrentState();

    if (this.validateIsChangedFeatures(featuresDiff))
        return;

    this.initLeaveMap(approvalManagementData.leaveTypes);

    if(successHandler){
        successHandler(approvalManagementData);
    }
};

ApprovalsManagementMediator.prototype.updateCurrentState = function(approvalManagementData){
    this.currentState.approveTTFilter.allUsers = this.newState.approveTTFilter.allUsers;
    this.currentState.approveTTFilter.userIds = this.newState.approveTTFilter.userIds;
    this.currentState.approveTTFilter.statuses = this.newState.approveTTFilter.statuses;
    this.currentState.approveTTFilter.monthDate = this.newState.approveTTFilter.monthDate;
    this.currentState.sortBy = this.newState.sortBy;
    this.currentState.searchText = this.newState.searchText;
    this.currentState.enabledFeatures = approvalManagementData.enabledFeatures;
    this.currentState.groupingStrategy = this.newState.groupingStrategy;
};

ApprovalsManagementMediator.prototype.addGroupingRows = function ( rowsData )
{
    this.approvalsTableController.removeSpinner();

    if ( this.topLevelApprovalStatus ) {
        var length = rowsData.rows.length;
        for ( var i = 0; i < length; i++ ) {
            rowsData.rows[i].weekApprovalStatus = this.topLevelApprovalStatus;
            rowsData.rows[i].statusTimestamp = this.topLevelStatusTimestamp;
        }
    }
    
    this.approvalsTableController.addGroupingRowsToBottom(rowsData);
};

ApprovalsManagementMediator.prototype.addChildRows = function ( groupingRow, groupingRowData )
{
    this.approvalsTableController.addChildRowsToGroupingRow( groupingRow, groupingRowData);
};

ApprovalsManagementMediator.prototype.showNoMoreChildRowsToLoad = function ( groupingRow )
{
    this.approvalsTableController.showNoMoreChildRowsToLoad( groupingRow );
};

ApprovalsManagementMediator.prototype.createInitGroupingStrategy = function ( groupType )
{
    return groupType == ApprovalPageConstants.sortByUser ? new UserGroupingStrategy() : new WeekGroupingStrategy();
};

ApprovalsManagementMediator.prototype.getRowsSelection = function ()
{
    return this.approvalsTableController.getRowsSelection();
};

ApprovalsManagementMediator.prototype.initUserGroupsMap = function (userGroupList)
{
    this.userGroupMap = this.createGroupsMap(userGroupList, true);
};

ApprovalsManagementMediator.prototype.initTimeZoneGroupsMap = function (timeZoneGroupList)
{
    this.timeZoneGroupMap = this.createGroupsMap(timeZoneGroupList, false);
};

ApprovalsManagementMediator.prototype.createGroupsMap = function (groupList, includeNullGroup)
{
    var groupMap = null;

    if( groupList != null )
    {
        groupMap={};
        if (includeNullGroup)
            groupMap[ApprovalPageConstants.USER_WITHOUT_GROUP] = messageResource.getEscapedMessage("approve_tt.table.username_cell.user_without_group");

        for( var i = 0; i < groupList.length; i++ )
        {
            var group = groupList[i];
            groupMap[ group.id ] = group.name;
        }
    }

    return groupMap;
};

ApprovalsManagementMediator.prototype._getLeaveType = function(leaveTypeId){
    if (this.leaveTypeMap == null) return null;
    return this.leaveTypeMap[leaveTypeId];
};


ApprovalsManagementMediator.prototype.initLeaveMap = function ( leaveList)
{
    if (leaveList)
    {
        for( var i = 0; i < leaveList.length; i++ )
        {
            var leaveTypeBean = leaveList[i];
            leaveTypeBean.sortingPosition = i;
            this.leaveTypeMap[leaveTypeBean.id] = leaveList[i];
        }
    }
};

ApprovalsManagementMediator.prototype.changeFilterValueCallback = function(){
    this.updateFilters();
    this.loadPageData(false);
};

ApprovalsManagementMediator.prototype.updateFilters = function()
{
    var usersFilterSelection = this.usersSelectorPanel.getSelectedUsers();
    this.newState.approveTTFilter.allUsers = usersFilterSelection.allUsers;
    this.newState.approveTTFilter.userIds = usersFilterSelection.selectedUserIds;
    this.newState.approveTTFilter.statuses = this._getSelectedStatuses();
    this.newState.approveTTFilter.monthDate = this._getSelectedDate();
};

ApprovalsManagementMediator.prototype._getSelectedStatuses= function() {
    return this.approvalStatusfilter.getSelection();
};

ApprovalsManagementMediator.prototype._getSelectedDate= function() {
    return this.monthSelector.getSelectedDate();
};

ApprovalsManagementMediator.prototype._searchTermsChangedCallback = function()
{
    this.newState.searchText = $.trim( this.getSearchValue() );
    this._clearLoadPageDataTimer();
    if(this.currentState.searchText != this.newState.searchText)
        this._loadDataWithDelay( 500, true );
};

ApprovalsManagementMediator.prototype._clearLoadPageDataTimer = function(){
    if( this._updateLoadPageDataDelayTimer != null )
    {
        clearTimeout( this._updateLoadPageDataDelayTimer );
        this._updateLoadPageDataDelayTimer = null;
    }
};

ApprovalsManagementMediator.prototype.initUsersSelector = function ( userGroupsEnabled, users, userGroups, timeZoneGroups, allUsers, selectedUserIds )
{
    var self = this;
    this.usersSelectorPanel = new AT.UsersForApproveTTPageSelectorPanel(
    {
        customOnChangeHandler: function () {
            var selectedUserIds = self.getUserIdsFromUsersSelectorIfSpecificAreSelected();
            $("#approveTTUsersSelectorButton").toggleClass("activeFilter", selectedUserIds != null );
        }
    });
    var timeZoneGroupsEnabled = this._isFeatureEnabled(ApprovalPageConstants.TZGFeature);
    var userSelectorMenu = new AT.SharedUserButtonMenu( {
                                                            panel : this.usersSelectorPanel,
                                                            additionalCls: "usersForApproveTTPageSelectorMenu " +
                                                                           (userGroupsEnabled || timeZoneGroupsEnabled ? "userSelectorWithGroupingModePanel " : "")
                                                        });
    var chooseUserButton = new AT.HierarchySelectorDropdownButton({ renderTo: "approveTTUsersSelectorButton",
                                                                      menu: userSelectorMenu
                                                                  });
    chooseUserButton.setMenuHideCallback(
            function() {
                self.usersSelectorPanel.setAllUsersIfNoneSelected();

                if( self.usersSelectorPanel.isStateChanged() )
                {
                    self.changeFilterValueCallback();
                }
            }
    );

    this.usersSelectorPanel.initialize(userGroupsEnabled, timeZoneGroupsEnabled, users, userGroups, timeZoneGroups,
                                       allUsers, selectedUserIds);
};

ApprovalsManagementMediator.prototype.getSearchValue = function(){
    return this.inputSearchField.getSearchString();
};

ApprovalsManagementMediator.prototype.initSortButtons = function(){
    var self = this;
    this.sortUserButton = $('#sortByNameLink');
    this.sortWeekButton = $('#sortByWeekLink');
    var onSortButtonClick = function(newSortByGrouping){
       self.changeSort(newSortByGrouping);
       self.loadPageData(true);
    };

    this.sortUserButton.on("click", function(){onSortButtonClick(ApprovalPageConstants.sortByUser)});
    this.sortWeekButton.on("click", function(){onSortButtonClick(ApprovalPageConstants.sortByWeek)});
};
ApprovalsManagementMediator.prototype.initApproveButtons = function(){
    var self = this;

    this.approveButton = $("#approveButton");
    this.rejectButton = $("#rejectButton");
    this.rejectButton.on("click", function(){
        if (!self.rejectButtonEnabled) return;

        var selection = self.getRowsSelection();

        if (!selection.getAllRowsSelected() && selection.getSelectedGroupingRowsIds().length == 0 && selection.getVisibleSelectedChildRowsIds().length==1)
        {
            var userWeek = selection.getVisibleSelectedChildRowsIds()[ 0 ];
            var user = self.approvalsTableController.getUserForChildRow( self.approvalsTableController.findChildRow( userWeek ) );
            var fullUserName = UserNameFormat.formatFML( user.firstName, user.lastName, user.middleName );
            self.rejectCommentDialog._open( self.rejectButton, {userInfo: fullUserName}, function(){return self.reject(selection, true)} );
        }
        else
            self.reject(selection, false);
    });

    this.approveButton.on("click", function(){
        if (!self.approveButtonEnabled) return;
        var selection = self.getRowsSelection();
        self.approve(selection);
    });

    this.updateButtonsState(false, false);
};

ApprovalsManagementMediator.prototype.updateButtonsState = function ( newApproveButtonEnabled, newRejectButtonEnabled )
{
    this.approveButtonEnabled = newApproveButtonEnabled;
    this.approveButton.toggleClass("disabled", !newApproveButtonEnabled);

    this.rejectButtonEnabled = newRejectButtonEnabled;
    this.rejectButton.toggleClass("disabled", !newRejectButtonEnabled);
};

ApprovalsManagementMediator.prototype.reject = function(selection, isNeededComment){
    var comment = null;
    if ( isNeededComment){
        if (!this.rejectCommentDialog.isValid())
        {
            alert(messageResource.getMessage("approve_tt.approve_table.reject_very_long_comment"));
            return false;
        }
        comment = this.rejectCommentDialog.getComment();
    }
    var configIfReject = {};
    configIfReject.comment = comment;
    selection = this.getRowsSelection();
    this.onApproveOrRejectData(this.currentState.groupingStrategy.getServiceMethodForReject(selection), selection, WeekApprovalStatus.REJECTED, configIfReject);

    return true;
};


ApprovalsManagementMediator.prototype.approve = function(selection){
    this.onApproveOrRejectData(this.currentState.groupingStrategy.getServiceMethodForApprove(selection), selection, WeekApprovalStatus.APPROVED)
};


ApprovalsManagementMediator.prototype.getFilterParams = function()
{
    var params = [];
    params.push(this.currentState.approveTTFilter.statuses);
    params.push(this.getUserIdsFromUsersSelectorIfSpecificAreSelected());
    params.push(this.tokens);
    params.push(this.currentState.approveTTFilter.monthDate);

    return params;
};

// returns an array with selected user ids, if "selected users" option is chosen.
// otherwise, returns null.
// that's what server expects when approving/rejecting
ApprovalsManagementMediator.prototype.getUserIdsFromUsersSelectorIfSpecificAreSelected = function ()
{
    var usersSelection = this.usersSelectorPanel.getSelectedUsers();
    return usersSelection.allUsers ? null : usersSelection.selectedUserIds;
};

ApprovalsManagementMediator.prototype.getParamsForApproveReject = function(selection){
    var self = this;

    var params = [];
    if (!selection.getAllRowsSelected())
    {
        var selectedGroups = selection.getSelectedGroupingRowsIds();
        var selectedRows = selection.getVisibleSelectedChildRowsIds();

        var timestamp = null;
        // now only single row operations are verified on concurrent modifications of corresponding TT
        if ( selectedGroups.length == 0 && selectedRows.length == 1) {
            var row = this.approvalsTableController.findChildRow( selectedRows[0] );
            timestamp = row.data("rowInfo").lastTTTimestamp;
        }
        
        params = [
            selectedGroups,
            selectedRows,
            selection.getExcludedChildRowsIds()
        ];
        params = params.concat(this.currentState.groupingStrategy.getFilterParamsForOperation(self.getFilterParams()));

        params.push(timestamp);
    }
    else
    {
        params = [
            selection.getSelectedGroupingRowsIds(),
            selection.getExcludedGroupingRowsIds(),
            selection.getVisibleSelectedChildRowsIds(),
            selection.getExcludedChildRowsIds()
        ];
        params = params.concat(this.getFilterParams());
    }
    return params;
};

ApprovalsManagementMediator.prototype.onApproveOrRejectData = function(rpcMethod, selection, approvalStatus, configIfReject){
    var self = this;

    var allRowsSelected = selection.getAllRowsSelected();
    if ( allRowsSelected ) {
       var confirmMessageKEy =  this.isAllFilterNotSelected() ?
                                "approve_tt.approve_table.approve_reject_confirmation_filters_not_selected"
                                        : "approve_tt.approve_table.approve_reject_confirmation_filters_selected";
       if ( !confirm(messageResource.getMessage(confirmMessageKEy,
                           [messageResource.getMessage(this.namesOperationsKeys[approvalStatus])])))
           return;
    }
    if (this._loadPageDataRequestObject)
    {
        this.abortLoadPageData();
        this._revertState();
    }

    var params = this.getParamsForApproveReject(selection);

    if (configIfReject){
        params.push(configIfReject.comment);
    }
    var approveButtonEnabledBeforeRequest = this.approveButtonEnabled;
    var rejectButtonEnabledBeforeRequest = this.rejectButtonEnabled;
    this.updateButtonsState(false, false);
    this.approvalsTableController.disableAllShowMoreRows();
    this.spinnerController.spin("spinnerPlaceholderTitle");
    this.updateStatuesRequest =
            makeRpcCall( rpcMethod, params,
                 function (changeWeekStatusResult) {
                     self.successOperationHandler(changeWeekStatusResult, function(){self.successUpdateStatus(changeWeekStatusResult, selection, approvalStatus )});
                 },
                 function (result, textStatus, errorThrown){
                     self.spinnerController.stop();
                     self.updateButtonsState(approveButtonEnabledBeforeRequest, rejectButtonEnabledBeforeRequest);
                     self.approvalsTableController.enableAllShowMoreRows();
                     if( textStatus == "abort" )
                     {
                         return true;
                     }
                     if( errorThrown.exception == 'com.actimind.actitime.administration.NoRightsToApproveUser' )
                     {
                         self.errorHandlerIfNotSuchUserOrNotAssigned(selection.getSelectedGroupingRowsIds(), selection.getVisibleSelectedChildRowsIds(),
                                                                     'approve_tt.approve_table.approve_reject_some_users_was_not_assigned',
                                                                     'approve_tt.approve_table.user_was_not_assigned');
                         document.location.reload();
                         return true;
                     }
                     if( errorThrown.exception == 'com.actimind.actitime.administration.NoSuchUserException' )
                     {
                         self.errorHandlerIfNotSuchUserOrNotAssigned(selection.getSelectedGroupingRowsIds(), selection.getVisibleSelectedChildRowsIds(),
                                                                     'approve_tt.approve_table.approve_reject_some_users_deleted',
                                                                     'approve_tt.approve_table.user_deleted');
                         document.location.reload();
                         return true;
                     }
                     if( errorThrown.exception == 'com.actimind.actitime.administration.TimeTrackChangedException' )
                     {
                         alert( messageResource.getMessage('approve_tt.approve_table.approve_reject_one_row.concurrent_modification_of_tt'));
                         document.location.reload();
                         return true;
                     }
                 },
                null,
                LONG_RPC_TIMEOUT);
};

ApprovalsManagementMediator.prototype.errorHandlerIfNotSuchUserOrNotAssigned = function(selectedGroupRows, selectedChildRows, messageKeyIfSomeUsers,messageKeyIfOneUser ){
    var user = this.getUserIfSelectOne(selectedGroupRows, selectedChildRows );
    if(!user)
        alert( messageResource.getMessage(messageKeyIfSomeUsers));
    else
    {
        var fullUserName = UserNameFormat.formatFML( user.firstName, user.lastName, user.middleName );
        alert( messageResource.getMessage( messageKeyIfOneUser, [fullUserName] ) );
    }
};

ApprovalsManagementMediator.prototype.getUserIfSelectOne = function(selectedGroupRows, selectedChildRows ){
    if (selectedGroupRows.length == 1 && this.currentState.sortBy.getGroupingType() == ApprovalPageConstants.sortByUser){
        var oneUser = true;
        for(var i = 0; i < selectedChildRows.length; ++i){
          if ( selectedChildRows[i].userId != selectedGroupRows[0])
            oneUser = false;
        }
        if (oneUser) return this.approvalsTableController.getGroupingRowModelById(selectedGroupRows[0]);
    }
    if(selectedGroupRows.length == 0 && selectedChildRows.length == 1){
        var userWeek = selectedChildRows[ 0 ];
        return this.approvalsTableController.getUserForChildRow( this.approvalsTableController.findChildRow( userWeek ) );
    }
};

ApprovalsManagementMediator.prototype.successUpdateStatus = function(changeWeekStatusResult, selection, approvalStatus) {
    var self = this;
    var allRowsSelected = selection.getAllRowsSelected();

    this.updateButtonsState(true, true);
    this.approvalsTableController.enableAllShowMoreRows();
    if ( allRowsSelected ) {
        this.topLevelApprovalStatus = approvalStatus;
        this.topLevelStatusTimestamp = changeWeekStatusResult.operationTimestamp;
    }
    
    var operationWasExecuted = changeWeekStatusResult.operationTimestamp != null;
    var groupRowsIds = changeWeekStatusResult.changedGroups;

   $.each( groupRowsIds, function ( i, id )
   {
        self.approvalsTableController.updateGroupRow( id, {
             status: approvalStatus,
             statusTimestamp: changeWeekStatusResult.operationTimestamp
        } );
    } );

    var allRowsWasChangedLoggedUser= this.updateChildRowsStatus(changeWeekStatusResult.userWeeks, changeWeekStatusResult.operationTimestamp, selection);

    if (!operationWasExecuted && selection.getVisibleSelectedChildRowsIds().length == 0 ){
        alert(messageResource.getMessage("approve_tt.approve_table.all_status_was_changed"));
    }
    if (!operationWasExecuted && selection.getVisibleSelectedChildRowsIds().length != 0 ){
        alert(messageResource.getMessage("approve_tt.approve_table.all_status_coincides_operation"));
    }
    if (operationWasExecuted && !allRowsWasChangedLoggedUser){
        alert(messageResource.getMessage("approve_tt.approve_table.some_status_coincides_operation"));
    }

    this.approvalsTableController.deselectAllRows();
};

ApprovalsManagementMediator.prototype.successOperationHandler = function(result, successHandler){
    var featureDiff =  new FeatureDiffStateController().init(this.currentState.enabledFeatures, result.enabledFeatures);
    this.currentState.enabledFeatures = result.enabledFeatures;
    if (featureDiff.wasFeatureChanged(ApprovalPageConstants.TTAFeature)){
        this._onFeatureChanged(ApprovalPageConstants.TTAFeature);
        return true;
    }

    this.spinnerController.stop();

    if (successHandler){
        successHandler(result);
        PageOverlayTipsManager.getInstance().refreshDynamicTips();
    }
};

ApprovalsManagementMediator.prototype.updateChildRowsStatus =function(userWeeks, operationTimeStamp){
    var self = this;
    var allRowsWasChangedLoggedUser = true;
    $.each( userWeeks, function ( i,  userWeekStatusChangeInfo )
    {
        if ( !userWeekStatusChangeInfo.statusChanged )
            allRowsWasChangedLoggedUser = false;
        
        self.approvalsTableController.updateChildRow(userWeekStatusChangeInfo.userWeekInfo, operationTimeStamp);
    } );
    return allRowsWasChangedLoggedUser;
};

ApprovalsManagementMediator.prototype.changeSort = function(newSortByGrouping){
    if (this.newState.sortBy.getGroupingType() != newSortByGrouping){
        this.newState.sortBy = new SortBy(newSortByGrouping, true );
        this.newState.groupingStrategy = this.createInitGroupingStrategy(this.newState.sortBy.getGroupingType());
    }
    else
        this.newState.sortBy.setAsc(!this.newState.sortBy.getAsc());
};


ApprovalsManagementMediator.prototype._revertState = function(){
    this._revertUI();
    this.newState = $.extend(true,{},this.currentState);
};

ApprovalsManagementMediator.prototype.setSearchText = function( text )
{
    this.inputSearchField.setInputValue(text);
};

ApprovalsManagementMediator.prototype._revertUI = function(  )
{
    this.approvalStatusfilter.initCheckboxes(this.currentState.approveTTFilter.statuses, true);
    this.setSearchText(this.currentState.searchText);
    this.usersSelectorPanel.fullResetState(true, false);
    this.monthSelector.updateStatus(this.currentState.approveTTFilter.monthDate);
};

ApprovalsManagementMediator.prototype._loadDataWithDelay = function ( delay, isUpdateControllerRendererAfterLoad )
{
    var self = this;
    // Note: if timer will be stopped, then it must be set to null
    this._updateLoadPageDataDelayTimer = setTimeout( function ()
                                                 {
                                                     self.loadPageData(isUpdateControllerRendererAfterLoad);
                                                     self._updateUserListDelayTimer = null;
                                                 }, delay );
};

ApprovalsManagementMediator.prototype.validateIsChangedFeatures =function(featureDiff){
    //This check should be in the beginning because then it will seem "Not Found"
    if (featureDiff.wasFeatureChanged(ApprovalPageConstants.TTAFeature))
    {
        this._onFeatureChanged( ApprovalPageConstants.TTAFeature );
        return true;
    }
    if (featureDiff.wasFeatureChanged(ApprovalPageConstants.UGRFeature))
    {
        this._onFeatureChanged( ApprovalPageConstants.UGRFeature );
        return true;
    }
    if (featureDiff.wasFeatureChanged(ApprovalPageConstants.OVRFeature))
    {
        this._onFeatureChanged( ApprovalPageConstants.OVRFeature );
        return true;
    }
     if (featureDiff.wasFeatureChanged( ApprovalPageConstants.LVTFeature))
    {
        this._onFeatureChanged( ApprovalPageConstants.LVTFeature );
        return true;
    }
    return false;
};

ApprovalsManagementMediator.prototype._onFeatureChanged = function(nameFeature){
    var messageKey = this._isFeatureEnabled(nameFeature) ? "approve_tt.js.feature_enabled" : "approve_tt.js.feature_disabled" ;
    alert(messageResource.getMessage(messageKey, [messageResource.getMessage("features."+ nameFeature +".title")]));    //TODO change alert;
    location.reload();
};

ApprovalsManagementMediator.prototype._isFeatureEnabled = function(nameFeature){
   return this.currentState.enabledFeatures.hasOwnProperty(nameFeature);
};

ApprovalsManagementMediator.prototype.initExpandCollapseButtons = function(){
    var self = this;
    this.expandAllButton = $("#expandButton");
    this.expandAllButton.click(function(){
        self.approvalsTableController.expandAll();
    });

    this.collapseAllButton = $("#collapseButton");
    this.collapseAllButton.click(function(){
        self.approvalsTableController.collapseAll();
    })
};

ApprovalsManagementMediator.prototype.initRejectCommentDialog = function(){
    this.rejectCommentDialog = new RejectCommentDialog().init("rejectWeekCommentDialog");
};

ApprovalsManagementMediator.prototype.isAllFilterNotSelected = function(){
    return this.monthSelector.isNotSelectedDate()
           && this.usersSelectorPanel.getSelectedUsers().allUsers
            && this.approvalStatusfilter.isAllSelected() && this.newState.searchText == "";
};
