function ApprovalsTableController ()
{
    this.constructor = ApprovalsTableController;
}

ApprovalsTableController.BOTTOM_BAR_HEIGHT = 30;

ApprovalsTableController.prototype.init = function ( tableId, isOvertimeEnabled, isLeaveTimeEnabled, isUGREnabled, isTZGEnabled,
                                                     showMoreCallback, getGroupNameCallBack,getLeaveTypeCallBack,
                                                     revokeGroupingRowCallback, revokeChildRowCallback,
                                                     updateButtonsStateCallback )
{
    this.table = $("#"+tableId);
    this.isOvertimeEnabled = isOvertimeEnabled;
    this.isLeaveTimeEnabled = isLeaveTimeEnabled;

    if(isOvertimeEnabled && !isLeaveTimeEnabled){
        this.table.find("#leaveTimeCol" ).attr("width","0%");
        this.table.find("#overtimeCol" ).attr("width","30%");
    }

    this.isUGREnabled = isUGREnabled;
    this.isTZGEnabled = isTZGEnabled;
    
    this.collapsed = false;

    this.table.toggleClass("hideGroups", !this.isUGREnabled && !this.isTZGEnabled);
    this.table.toggleClass("hideOVT", !this.isOvertimeEnabled);
    this.table.toggleClass("hideLVT", !this.isLeaveTimeEnabled);

    this.leaveTimeToolip = new LeaveTimeTooltipForApproveTT().init("leaveTimeTooltipForApproveTT");
    this.submitTTTooltip = new TableTooltip().init("timeTrackLinkTooltip");

    this._getGroupNameCallback = getGroupNameCallBack;
    this._getLeaveTypeCallbaack = getLeaveTypeCallBack;
    this.showMoreCallback = showMoreCallback;
    this.revokeGroupingRowCallback = revokeGroupingRowCallback;
    this.revokeChildRowCallback = revokeChildRowCallback;
    this.updateButtonsStateCallback = updateButtonsStateCallback;

    this.initSelectAllDeselectAllButtons();

    this.handlers = {};
    this.handlers._showLeaveTypeTooltipCallback  = this.updateAndShowTooltipCallback(this.leaveTimeToolip);
    this.handlers._hideLeaveTypeTooltipCallback  =  this.getHideTooltipCallback(this.leaveTimeToolip);
    this.handlers._moveLeaveTypeTooltipCallback =  this.getMoveTooltipCallback(this.leaveTimeToolip);
    this.handlers._showTimeTrackLinkTooltipCallback = this.showTooltipCallback(this.submitTTTooltip);
    this.handlers._hideTimeTrackLinkTooltipCallback = this.getHideTooltipCallback(this.submitTTTooltip);
    this.handlers._moveTimeTrackLinkTooltipCallback =  this.getMoveTooltipCallback(this.submitTTTooltip);

    return this;
};

ApprovalsTableController.prototype.updateControllerStateAndUI = function(conf){

    this.tokensHighlighter = conf.tokens && conf.tokens.length > 0 ? new TokensHighlighter( conf.tokens, StringTokenizer.PUNCTUATORS_FOR_SEARCH ) : new EmptyHighlighter();
    this.sortOrder = conf.sortOrder;
    this.updateTableHeader();
    return this;
};

ApprovalsTableController.prototype.updateRenderer = function(groupingStrategy){
    var self = this;

    this.groupingStrategy =groupingStrategy;
    var configuration = {};
    configuration.tokensHighlightCallback = function(str){ return  self.hl(str)};
    configuration._getGroupNameCallback = this._getGroupNameCallback;
    configuration._getLeaveTypeCallbaack = this._getLeaveTypeCallbaack;

    this.groupingRowRenderer = this.groupingStrategy.createGroupingRowRenderer(configuration);
    this.childRowRenderer = this.groupingStrategy.createChildRowRenderer(configuration);
    this.showMoreRowRenderer = this.groupingStrategy.createShowMoreRowRenderer();

    this.rowController = this.groupingStrategy.createRowController();

    this.selectionController = new ApprovalsTableSelectionController().init(this.rowController, this.groupingRowRenderer, this.childRowRenderer, this.showMoreRowRenderer,
                                                                            this, this.updateButtonsStateCallback);
};

ApprovalsTableController.prototype.removeTable = function(){
    this.table.find("> tbody:not(.templates)").remove();
    this.table.removeClass("emptyTable");
    this.groupingRowsMap = {};
};

ApprovalsTableController.prototype.hl = function ( str )
{
    return this.tokensHighlighter.highlightTokens( str );
};

ApprovalsTableController.prototype.addGroupingRowsToBottom = function ( rowsData )
{
    var self = this;

    $.each( rowsData.rows, function ()
    {
        var newGroupingSection = self._createGroupingSection(this);
        self.table.append(newGroupingSection);
    } );
};

ApprovalsTableController.prototype._addChildRowsToGroupingRow = function ( groupingRow, groupingRowData, childRows )
{
    var self = this;

    var tBody = groupingRow.parent();
    var childRowsMap = {};

    $.each( childRows, function ()
    {
        var childRow = self._createChildRow( this, groupingRow );
        tBody.append( childRow );
        childRowsMap[self._getRowIdForChildRow( childRow )] = childRow;
    } );

    this._updateChildRowsMapInGroupingRow( groupingRow, childRowsMap );
    var numberOfChildRows = self._getNumberOfChildRows( groupingRow );

    if( groupingRowData.recordsLeft )
    {
        var newShowMoreRows = self._createShowMoreRow( numberOfChildRows,
                                                      groupingRowData.recordsLeft,
                                                      function ()
                                                      {
                                                          self.showMoreCallback( $(newShowMoreRows).parent().find(".groupingRow" ).first());
                                                      } );
        $.each(newShowMoreRows, function() {
            tBody.append( this )
        });
    }

    this.selectionController.onMoreChildRowsAddedToGroupingRow( groupingRow );
};

ApprovalsTableController.prototype._createGroupingSection = function ( groupingRowData )
{
    var tbody = $( document.createElement( "tbody" ) );
    tbody.addClass( 'data' );

    var childRows = groupingRowData.childRows;
    groupingRowData.childRows = null;

    var groupingRow = this._createGroupingRow(groupingRowData);
    tbody.append( groupingRow );
    this._addChildRowsToGroupingRow( groupingRow, groupingRowData, childRows );

    this.groupingRowsMap[this._getRowIdForGroupingRow(groupingRow)] = groupingRow;

    this.selectionController.onGroupingRowAdded(groupingRow);

    if (this.collapsed) 
        tbody.addClass("hiddenChild");
    
    return tbody;
};

ApprovalsTableController.prototype.addChildRowsToGroupingRow = function ( groupingRow, groupingRowData )
{
    this.removeShowMoreRow( groupingRow );

    var childRows = groupingRowData.childRows;
    groupingRowData.childRows = null;

    this._addChildRowsToGroupingRow( groupingRow, groupingRowData, childRows );
};

ApprovalsTableController.prototype.removeShowMoreRow = function ( groupingRow )
{
    var showMoreRow = groupingRow.parent().find( ".showMoreRow" );
    $( showMoreRow ).remove();
    var showMoreRowTopSpacer = groupingRow.parent().find( ".messageRow_spacer_top" );
    $( showMoreRowTopSpacer ).remove();
    var showMoreRowBottomSpacer = groupingRow.parent().find( ".messageRow_spacer_bottom" );
    $( showMoreRowBottomSpacer ).remove();
};


ApprovalsTableController.prototype.showNoMoreChildRowsToLoad = function ( groupingRow )
{
    var showMoreRow = groupingRow.parent().find( ".showMoreRow" );

    this.showMoreRowRenderer.transformToMessageRow( showMoreRow, messageResource.getMessage('approve_tt.approve_table.no_more_data'))
};

ApprovalsTableController.prototype.addSpinnerRowToBottom = function ( loadMoreCallback )
{
    var self = this;

    var tbody = $( document.createElement( "tbody" ) );
    tbody.addClass( "loadMoreSpinnerContainer" );

    tbody.addClass( "buttonState" );
    var spinnerTr = $(document.createElement( "tr" ));
    var spinnerTd = document.createElement( "td" );
    spinnerTd.id = "groupingRowsLoadingSpinnerPlaceholder";

    var span = document.createElement( "span" );
    span.id = "loadMoreContent";
    span.innerHTML = messageResource.getMessage('approve_tt.approve_table.load_more');

    $(spinnerTd ).append(span);
    $(spinnerTd ).attr("colspan", "8");
    $(spinnerTd ).attr("align", "center");
    spinnerTr.append(spinnerTd);

    tbody.append(spinnerTr);
    tbody.click(loadMoreCallback);

    self.table.append(tbody);
};

ApprovalsTableController.prototype.changeButtonToSpinner = function (  )
{
    var tbody = $( ".loadMoreSpinnerContainer" );
    tbody.removeClass("buttonState");
    tbody.addClass( "spinnerState" );
    tbody.off('click');
    $("#loadMoreContent" ).empty();
};

ApprovalsTableController.prototype.getLastGroupingRowModel = function ()
{
    return this.getRowModel(this.getLastGroupingRow());
};

ApprovalsTableController.prototype.getLastChildRowModel = function ( groupingRow )
{
    return this.getRowModel(this.rowController.getLastChildRow( groupingRow ));
};

ApprovalsTableController.prototype.getRowModel = function ( row )
{
    return this.rowController.getModel( row );
};

ApprovalsTableController.prototype.removeSpinner = function (  )
{
    var spinner = $( ".loadMoreSpinnerContainer" );
    if (spinner) spinner.remove();
};

ApprovalsTableController.prototype._updateChildRowsMapInGroupingRow = function ( groupingRow, childRowsMap )
{
    this.groupingRowRenderer.updateChildRowsMapInGroupingRow(groupingRow, childRowsMap);
};

ApprovalsTableController.prototype._getNumberOfChildRows = function ( groupingRow )
{
    return this.rowController.getNumberOfChildRows( groupingRow );
};

ApprovalsTableController.prototype._createGroupingRow = function (groupingRowData)
{
    var self = this;
    return this.groupingRowRenderer.createRow(
        groupingRowData,
        function (row, selected)
        {
            self.selectionController.onGroupingRowSelectionChanged(row, selected);
        },
        self.revokeGroupingRowCallback
    );
};

ApprovalsTableController.prototype._getRowIdForGroupingRow = function ( groupingRow )
{
    return this.rowController.getModelIdForGroupingRow( groupingRow );
};

ApprovalsTableController.prototype._getRowIdForChildRow = function ( childRow )
{
    return this.rowController.getModelIdForChildRow( childRow );
};

ApprovalsTableController.prototype.getUserWeekForChildRow = function ( childRow )
{
    return this.rowController.getUserWeekForChildRow( childRow );
};

ApprovalsTableController.prototype._createChildRow = function ( childRowData, groupingRow )
{
    var self = this;
    return this.childRowRenderer.createRow(
            childRowData,
            groupingRow,
            function ( row, selected )
            {
                self.selectionController.onChildRowSelectionChanged(row, selected);
            },
            self.revokeChildRowCallback,
            self.handlers
    );
};

ApprovalsTableController.prototype._createShowMoreRow = function ( shownCount, totalCount, onClickCallback )
{
    var self = this;
    return this.showMoreRowRenderer.createRow(
            shownCount,
            totalCount,
            onClickCallback,
            function ( row ) {
                self.selectionController.onShowMoreRowDeselected( row );
            }
    );
};

ApprovalsTableController.prototype.getRowsSelection = function ()
{
    return this.selectionController.getSelection();
};

ApprovalsTableController.prototype.getRevokeRowsSelectionForGroupingRow  = function ( groupingRow )
{
    return this.selectionController.getRevokeSelectionForGroupingRow(groupingRow);
};

ApprovalsTableController.prototype.onChildRowRevoked = function ( childRowId )
{
    this.selectionController.onChildRowRevoked( this.findChildRow(childRowId) );
};

ApprovalsTableController.prototype.getGroupingRows = function ()
{
    var groupingRows = [];
    for (var name in this.groupingRowsMap)
    {
        if( this.groupingRowsMap.hasOwnProperty( name ) )
            groupingRows.push(this.groupingRowsMap[name]);
    }
    return groupingRows;
};

ApprovalsTableController.prototype.getLastGroupingRow = function ()
{
    return this.table.find( ".data" ).last().find( ".groupingRow" );
};

ApprovalsTableController.prototype.updateTableHeader = function(){
    this.table.toggleClass("sortByUser", this.sortOrder.getGroupingType() == ApprovalPageConstants.sortByUser);
    this.table.toggleClass("sortByWeek", this.sortOrder.getGroupingType() == ApprovalPageConstants.sortByWeek);
    this.table.toggleClass("asc", this.sortOrder.getAsc() == true);
};

ApprovalsTableController.prototype.expandAll = function(){
    this.table.find("tbody.data").removeClass("hiddenChild");
    this.collapsed = false;
};

ApprovalsTableController.prototype.collapseAll = function(){
    this.table.find("tbody.data" ).addClass("hiddenChild");
    this.collapsed = true;
};

ApprovalsTableController.prototype.isEmptyTable = function(){
    return this.table.children("tbody.data" ).length == 0;
};

ApprovalsTableController.prototype.initSelectAllDeselectAllButtons = function(){
    var self = this;

    this.selectAllButton = $("#selectAllButton");
    this.selectAllButton.click( function () {
        self.selectionController.onSelectAllPressed();
    });

    this.deselectAllButton = $("#deselectAllButton");
    this.deselectAllButton.click( function () {
        self.selectionController.onDeselectAllPressed();
    });
};

ApprovalsTableController.prototype.deselectAllRows = function ()
{
    this.selectionController.onDeselectAllPressed();
};

ApprovalsTableController.prototype.getRowInfo = function(childRow){
    return this.rowController.getRowInfo(childRow);
};

ApprovalsTableController.prototype.updateGroupRowAfterRevoke = function ( groupingRowId, groupData )
{
    this.updateGroupRow(groupingRowId, groupData);
    this.selectionController.onGroupingRowRevoked(this.findGroupRow(groupingRowId));
};

ApprovalsTableController.prototype.getGroupingRowModelById = function(groupingRowId){
    var groupingRow = this.findGroupRow(groupingRowId);
    return this.rowController.getModel(groupingRow);
};

ApprovalsTableController.prototype.updateGroupRow = function(groupingRowId, groupData){
    var self = this;
    var groupingRow = this.findGroupRow(groupingRowId);
    var rowModel = this.getRowInfo(groupingRow);
    rowModel.weekApprovalStatus = groupData.status;
    rowModel.statusTimestamp = groupData.statusTimestamp;
    
    var newGroupingRow = this._createGroupingRow(rowModel);
    this.groupingRowsMap[self._getRowIdForGroupingRow(newGroupingRow)] = newGroupingRow;
    this.groupingRowRenderer.updateChildRowsMapInGroupingRow(newGroupingRow, this.rowController.getChildRowsMap(groupingRow));

    groupingRow.after(newGroupingRow);
    groupingRow.remove();
};

ApprovalsTableController.prototype.updateChildRow = function( childRowInfo, operationTimestamp ){
    var self = this;
    var childRow = this.findChildRow(childRowInfo.userWeek);
    var rowModel =  this.getRowInfo(childRow);
    
    rowModel.weekApprovalStatus = childRowInfo.status;
    rowModel.statusTimestamp = childRowInfo.statusTimestamp;
    rowModel.changedDirectly = operationTimestamp == childRowInfo.statusTimestamp;
    
    var groupingRow = childRow.parent().find(".groupingRow" ).first();
    var childRowMap = this.rowController.getChildRowsMap(groupingRow);
    var newChildRow = this._createChildRow(rowModel, groupingRow);

    childRowMap[self._getRowIdForChildRow(newChildRow)] = newChildRow;

    childRow.after(newChildRow);
    childRow.remove();
};

ApprovalsTableController.prototype.findGroupRow = function(groupRowId){
   return this.groupingRowsMap[groupRowId];
};

ApprovalsTableController.prototype.findChildRow = function(userWeek){
    var self = this;
    var groupRow = this.groupingRowsMap[self.groupingStrategy.getGroupingRowIdFromChildRowId(userWeek)];
    return  this.rowController.getChildRowsMap(groupRow )[self.groupingStrategy.getChildRowIdFromChildRowId(userWeek)];
};

ApprovalsTableController.prototype.updateAndShowTooltipCallback = function(tooltip)
{

    return function( config, event ) {
        tooltip.updateUIAndShow( config, calculateTooltipOffset( tooltip, event, ApprovalsTableController.BOTTOM_BAR_HEIGHT ));
    };
};

ApprovalsTableController.prototype.showTooltipCallback  = function(tooltip)
{
    return function( event ) {
        tooltip.show();
        tooltip.move( calculateTooltipOffset( tooltip, event, ApprovalsTableController.BOTTOM_BAR_HEIGHT ) );
    };
};

ApprovalsTableController.prototype. getMoveTooltipCallback= function( tooltip )
{
    return function( event ) {
        tooltip.move( calculateTooltipOffset( tooltip, event, ApprovalsTableController.BOTTOM_BAR_HEIGHT ) );
    };
};

ApprovalsTableController.prototype.getHideTooltipCallback = function( tooltip )
{
    return function() {
        tooltip.hide();
    }
};

ApprovalsTableController.prototype.getGroupingRowForChildRow = function ( childRow )
{
    return childRow.parent().children().first();
};

ApprovalsTableController.prototype.getGroupingRowForShowMoreRow = function ( showMoreRow )
{
    return this.getGroupingRowForChildRow(showMoreRow);
};

ApprovalsTableController.prototype.getChildRowsForGroupingRow = function ( groupingRow )
{
    return groupingRow.parent().children(".childRow");
};

ApprovalsTableController.prototype.getShowMoreRowForGroupingRow = function ( groupingRow )
{
    var showMoreRow = groupingRow.parent().children(".showMoreRow");
    return showMoreRow.length != 0 ? showMoreRow : null;
};

ApprovalsTableController.prototype.getWeekApprovalStatus = function(row){
    return this.rowController.getWeekApprovalStatus(row);
};

ApprovalsTableController.prototype.getWeekApprovalStatusTimeStamp = function(row){
    return this.rowController.getWeekApprovalStatusTimeStamp(row);
};

ApprovalsTableController.prototype.getUserWeekForChildRow = function(row){
    return this.rowController.getUserWeekForChildRow(row);
};

ApprovalsTableController.prototype.getUserForChildRow = function( row ) {
    return this.rowController.getUserForChildRow( row );
};

ApprovalsTableController.prototype.createDataTBody = function(){
    var tbody = $( document.createElement( "tbody" ) );
    tbody.addClass( 'data' );
    return tbody;
};

ApprovalsTableController.prototype.addNoRecordsRow = function(){
    var tBody = this.createDataTBody();
    var row = $("#approveTTTable" ).find("tbody.templates").find("tr.noRecordsRow" ).clone();
    tBody.append( row );
    this.table.addClass("emptyTable");
    this.table.append(tBody);
};

ApprovalsTableController.prototype.hasNoRecords = function(){
 return this.table.hasClass("emptyTable");
};

ApprovalsTableController.prototype.disableAllShowMoreRows = function(){
    this.showMoreRowRenderer.disableWhenOperation(this.table.find( ".showMoreRow" ));
};

ApprovalsTableController.prototype.enableAllShowMoreRows = function(){
    this.showMoreRowRenderer.enableWhenOperation(this.table.find( ".showMoreRow"));
};


