function ApprovalsTableSelectionController ( )
{
    this.constructor = ApprovalsTableSelectionController;
}

ApprovalsTableSelectionController.prototype.init = function ( rowController, groupingRowRenderer, childRowRenderer, showMoreRowRenderer, tableController, updateButtonsStateCallback )
{
    this.allRowsSelected = false;
    this.rowController = rowController;
    this.groupingRowRenderer = groupingRowRenderer;
    this.childRowRenderer = childRowRenderer;
    this.showMoreRowRenderer = showMoreRowRenderer;
    this.tableController = tableController;
    this.updateButtonsStateCallback = updateButtonsStateCallback;

    this.rowSelectedHandlingEnabled = true;

    return this;
};

ApprovalsTableSelectionController.prototype.getSelection = function ( )
{
    var self = this;

    var rowsSelection;
    var groupingRows = this.tableController.getGroupingRows();

    if (this.allRowsSelected)
    {
        rowsSelection = new RowsSelection().init( true );
        $.each( groupingRows, function ()
        {
            var groupingRowId = self.rowController.getModelIdForGroupingRow(this);
            var childRows = self.tableController.getChildRowsForGroupingRow(this);

            if (self.rowController.isRowStatusNewlyChanged(this))
            {
                rowsSelection.addExcludedGroupingRowsId( groupingRowId );
                $.each(childRows, function ()
                {
                    var childRow = $(this);
                    if (!self.rowController.isRowStatusNewlyChanged(childRow))
                        rowsSelection.addChildRowId(self._getChildRowSelectionId(childRow));
                });
            }
            else
            {
                rowsSelection.addGroupingRowId(groupingRowId);
                $.each(childRows, function ()
                {
                    var childRow = $(this);
                    var childRowId = self._getChildRowSelectionId(childRow);
                    if (self.rowController.isRowStatusNewlyChanged(childRow))
                        rowsSelection.addExcludedChildRowsIds(childRowId);
                    else
                        rowsSelection.addChildRowId(childRowId);
                });
            }
        });
        return rowsSelection;
    }
    else
    {
        rowsSelection = new RowsSelection().init( false );
        $.each( groupingRows, function ()
        {
            var groupingRowId = self.rowController.getModelIdForGroupingRow(this);
            var childRows = self.tableController.getChildRowsForGroupingRow(this);

            if (self.groupingRowRenderer.isRowSelected(this))
            {
                rowsSelection.addGroupingRowId(groupingRowId);
                $.each(childRows, function ()
                {
                    var childRow = $(this);
                    var childRowId = self._getChildRowSelectionId(childRow);
                    if (self.rowController.isRowStatusNewlyChanged(childRow))
                        rowsSelection.addExcludedChildRowsIds(childRowId);
                    else
                        rowsSelection.addChildRowId(childRowId);
                });
            }
            else
            {
                $.each(childRows, function ()
                {
                    var childRow = $(this);
                    if (self.childRowRenderer.isRowSelected(childRow))
                        rowsSelection.addChildRowId(self._getChildRowSelectionId(childRow));
                });
            }
        });
        return rowsSelection;
    }
};

ApprovalsTableSelectionController.prototype._defineButtonStateAndUpdateThem = function ()
{
    var newApproveButtonEnabled = false, newRejectButtonEnabled = false;
    var self = this;
    var groupingRows = this.tableController.getGroupingRows();

    $.each( groupingRows, function () {
        if (self.groupingRowRenderer.isRowSelected(this))
        {
            newApproveButtonEnabled = true;
            newRejectButtonEnabled = true;
            return false;
        }
        else
        {
            var childRows = self.tableController.getChildRowsForGroupingRow(this);
            $.each( childRows, function () {
                var childRow = $(this);
                if (self.childRowRenderer.isRowSelected(childRow))
                {
                    if (!self.rowController.isChildRowApproved(childRow))
                        newApproveButtonEnabled = true;
                    if (!self.rowController.isChildRowRejected(childRow))
                        newRejectButtonEnabled = true;
                }
                if (newApproveButtonEnabled && newRejectButtonEnabled)
                    return false;
            });
        }
    });

    this._updateButtonsState(newApproveButtonEnabled, newRejectButtonEnabled);
};

ApprovalsTableSelectionController.prototype._updateButtonsState = function (newApproveButtonEnabled, newRejectButtonEnabled)
{
    this.updateButtonsStateCallback(newApproveButtonEnabled, newRejectButtonEnabled);
};

ApprovalsTableSelectionController.prototype._getChildRowSelectionId = function ( childRow )
{
    return this.rowController.getUserWeekForChildRow(childRow);
};

ApprovalsTableSelectionController.prototype._selectAllChildRowsIfNotNewlyChangedForGroupingRow = function ( groupingRow, alsoDisableSelection )
{
    var self = this;
    var childRows = this.tableController.getChildRowsForGroupingRow(groupingRow);

    $.each( childRows, function () {
        var childRow = $(this);
        if (!self.rowController.isRowStatusNewlyChanged(childRow))
            self.childRowRenderer.selectRow(childRow, alsoDisableSelection);
    });
};

ApprovalsTableSelectionController.prototype._deselectShowMoreRowIfExistsForGroupingRow = function ( groupingRow, alsoEnableSelection )
{
    var showMoreRow = this.tableController.getShowMoreRowForGroupingRow(groupingRow);
    if (showMoreRow != null)
        this.showMoreRowRenderer.deselectRow(showMoreRow, alsoEnableSelection);
};

ApprovalsTableSelectionController.prototype.onGroupingRowSelectionChanged = function ( groupingRow, checked )
{
    if (!this.rowSelectedHandlingEnabled)
        return;

    this._disableRowSelectedHandling();
    if ( checked )
    {
        this._selectChildAndShowMoreRowsForGroupingRow( groupingRow );
        this._updateButtonsState(true, true);
    }
    else
    {
        this._deselectChildAndShowMoreRowsForGroupingRow( groupingRow );
        this._defineButtonStateAndUpdateThem();
    }

    this._enableRowSelectedHandling();
};

ApprovalsTableSelectionController.prototype.onChildRowSelectionChanged = function ( childRow, checked )
{
    if (!this.rowSelectedHandlingEnabled)
        return;

    this._disableRowSelectedHandling();

    if ( !checked )
        this._onChildRowDeselected( childRow );

    this._defineButtonStateAndUpdateThem();

    this._enableRowSelectedHandling();
};

ApprovalsTableSelectionController.prototype._enableRowSelectedHandling = function ( )
{
    this.rowSelectedHandlingEnabled = true;
};

ApprovalsTableSelectionController.prototype._disableRowSelectedHandling = function ( )
{
    this.rowSelectedHandlingEnabled = false;
};

ApprovalsTableSelectionController.prototype.onSelectAllPressed = function ( )
{
    var self = this;

    if (this.allRowsSelected)
        return;

    this._disableRowSelectedHandling();
    var groupingRows = this.tableController.getGroupingRows();
    if (groupingRows.length == 0)
        return;
    $.each( groupingRows, function ()
    {
        if (!self.rowController.isRowStatusNewlyChanged(this))
        {
            self._selectGroupingRowAndItsChildRowsAndShowMoreRowAndDisableSelection(this);
        }
        else
        {
            self._selectAllChildRowsIfNotNewlyChangedForGroupingRow(this, true);
        }
    });
    this.allRowsSelected = true;

    this._updateButtonsState(true, true);

    this._enableRowSelectedHandling();
};

ApprovalsTableSelectionController.prototype.onDeselectAllPressed = function ( )
{
    var self = this;

    this._disableRowSelectedHandling();
    var groupingRows = this.tableController.getGroupingRows();

    $.each( groupingRows, function ()
    {
        if (self.groupingRowRenderer.isRowSelected(this))
            self.groupingRowRenderer.deselectRow(this, true);
        self._deselectChildAndShowMoreRowsForGroupingRow(this, true);
    });
    this.allRowsSelected = false;

    this._updateButtonsState(false, false);

    this._enableRowSelectedHandling();
};

ApprovalsTableSelectionController.prototype._selectChildAndShowMoreRowsForGroupingRow = function ( groupingRow, alsoDisableSelection )
{
    this._selectAllChildRowsIfNotNewlyChangedForGroupingRow(groupingRow, alsoDisableSelection);

    var showMoreRow = this.tableController.getShowMoreRowForGroupingRow(groupingRow);
    if (showMoreRow != null)
        this.showMoreRowRenderer.selectRow(showMoreRow, alsoDisableSelection);
};

ApprovalsTableSelectionController.prototype._deselectChildAndShowMoreRowsForGroupingRow = function ( groupingRow, alsoEnableSelection )
{
    var self = this;
    var childRows  = this.tableController.getChildRowsForGroupingRow(groupingRow);

    $.each( childRows, function () {
        var childRow = $(this);
        if (!self.rowController.isRowStatusNewlyChanged(childRow))
            self.childRowRenderer.deselectRow(childRow, alsoEnableSelection);
    });

    this._deselectShowMoreRowIfExistsForGroupingRow(groupingRow, alsoEnableSelection);
};

ApprovalsTableSelectionController.prototype._onChildRowDeselected = function ( childRow )
{
    var self = this;
    var groupingRow = this.tableController.getGroupingRowForChildRow(childRow);

    if (this.groupingRowRenderer.isRowSelected(groupingRow))
    {
        self.groupingRowRenderer.deselectRow(groupingRow);
        this._deselectShowMoreRowIfExistsForGroupingRow(groupingRow);
    }
};

ApprovalsTableSelectionController.prototype.onShowMoreRowDeselected = function ( showMoreRow )
{
    this._disableRowSelectedHandling();

    var groupingRow = this.tableController.getGroupingRowForShowMoreRow(showMoreRow);

    if (this.groupingRowRenderer.isRowSelected(groupingRow))
        this.groupingRowRenderer.deselectRow(groupingRow);

    this._defineButtonStateAndUpdateThem();

    this._enableRowSelectedHandling();
};

ApprovalsTableSelectionController.prototype.getRevokeSelectionForGroupingRow = function ( groupingRow )
{
    var self = this;

    var revokeRowsSelection = new RevokeGroupingRowSelection().init();
    revokeRowsSelection.setGroupingRowId(this.rowController.getModelIdForGroupingRow(groupingRow));

    var childRows = this.tableController.getChildRowsForGroupingRow(groupingRow);
    $.each( childRows, function () {
        var childRow = $(this);
        if (self.rowController.isChildRowStatusChangedTogetherWithGroupingRow(groupingRow, childRow))
            revokeRowsSelection.addChildRowId(self._getChildRowSelectionId(childRow));
    });

    return revokeRowsSelection;
};

ApprovalsTableSelectionController.prototype.onMoreChildRowsAddedToGroupingRow = function ( groupingRow )
{
    if ( this.allRowsSelected || this.groupingRowRenderer.isRowSelected(groupingRow) )
    {
        this._disableRowSelectedHandling();
        this._selectChildAndShowMoreRowsForGroupingRow( groupingRow, this.allRowsSelected );
        this._enableRowSelectedHandling();
    }
};

ApprovalsTableSelectionController.prototype.onGroupingRowAdded = function ( groupingRow )
{
    if (this.allRowsSelected)
    {
        this._disableRowSelectedHandling();
        this._selectGroupingRowAndItsChildRowsAndShowMoreRowAndDisableSelection(groupingRow);
        this._enableRowSelectedHandling();
    }
};

ApprovalsTableSelectionController.prototype.onChildRowRevoked = function ( childRow )
{
    var groupingRow = this.tableController.getGroupingRowForChildRow(childRow);
    if ( this.allRowsSelected || this.groupingRowRenderer.isRowSelected(groupingRow) )
    {
        this._disableRowSelectedHandling();
        this.childRowRenderer.selectRow( childRow, this.allRowsSelected );
        this._enableRowSelectedHandling();
    }
};

ApprovalsTableSelectionController.prototype.onGroupingRowRevoked = function ( groupingRow )
{
    if ( this.allRowsSelected )
    {
        this._disableRowSelectedHandling();
        this._selectGroupingRowAndItsChildRowsAndShowMoreRowAndDisableSelection(groupingRow);
        this._enableRowSelectedHandling();
    }
};

ApprovalsTableSelectionController.prototype._selectGroupingRowAndItsChildRowsAndShowMoreRowAndDisableSelection = function ( groupingRow )
{
    this.groupingRowRenderer.selectRow( groupingRow, true );
    this._selectChildAndShowMoreRowsForGroupingRow( groupingRow, true );
};