function ChildRowRenderer () {
    this.constructor = ChildRowRenderer;
}

ChildRowRenderer.prototype = new RowRenderer();

ChildRowRenderer.weekApprovalStatusCaption = {};
ChildRowRenderer.weekApprovalStatusCaption[WeekApprovalStatus.NOT_READY]   = messageResource.getMessage("approve_tt.approve_table.status.not_ready");
ChildRowRenderer.weekApprovalStatusCaption[WeekApprovalStatus.READY]       = messageResource.getMessage("approve_tt.approve_table.status.ready");
ChildRowRenderer.weekApprovalStatusCaption[WeekApprovalStatus.APPROVED]    = messageResource.getMessage("approve_tt.approve_table.status.approved");
ChildRowRenderer.weekApprovalStatusCaption[WeekApprovalStatus.REJECTED]    = messageResource.getMessage("approve_tt.approve_table.status.rejected");

ChildRowRenderer.prototype.init = function ( conf )
{
    this.childRowTemplate = $("#approveTTTable" ).find("tbody.templates").find("tr.childRow");
    this._getLeaveTypeCallback = conf._getLeaveTypeCallbaack;
    return this;
};

ChildRowRenderer.prototype.createRow = function ( childRowInfo, groupingRow, onCheckCallback, revokeCallback, handlers )
{
    var newRow = this.childRowTemplate.clone();

    var groupingRowInfo = groupingRow.data("rowInfo");
    this.initSubmitTTLinkCell(newRow, childRowInfo, groupingRowInfo.approvalRowInfo, handlers);
    this.initWeekStatusCell(newRow, childRowInfo);
    this.initWorkingTimeCell(newRow, childRowInfo);
    this.initLeaveTimeCell(newRow, childRowInfo, handlers);
    this.initOvertimeCell(newRow, childRowInfo);
    this.initSelectionCell(newRow, onCheckCallback);
    
    var groupOperation =
            !!groupingRowInfo.statusTimestamp && !!groupingRowInfo.weekApprovalStatus ?
              childRowInfo.statusTimestamp == groupingRowInfo.statusTimestamp &&
              childRowInfo.weekApprovalStatus == groupingRowInfo.weekApprovalStatus
            : false;

    if( childRowInfo.changedDirectly || groupOperation ) {
        newRow.addClass("operationWasApplied");
        this.initRevokeCell(newRow, revokeCallback);
    }

    newRow.data("rowInfo", childRowInfo);
    newRow.addClass(this.weekApprovalStatusCaptionClass[childRowInfo.weekApprovalStatus]);

    return newRow;
};



ChildRowRenderer.prototype.initSubmitTTLinkCell = function ( row, childRowInfo, approvalRowInfo, handlers )
{
    var link = row.find(".submitTTLinkCell a");
    link.html( this.formatSubmitTTLinkText(childRowInfo) );
    link.attr('href', this.formatSubmitTTLink(link, childRowInfo, approvalRowInfo));

    var delayHover = null;
    link.hover(function(event){ delayHover =  setTimeout(function(){ handlers._showTimeTrackLinkTooltipCallback(event)}, 1000)},
                              function(){  clearTimeout( delayHover );  handlers._hideTimeTrackLinkTooltipCallback()} );
    link.mousemove(function(event){ handlers._moveTimeTrackLinkTooltipCallback(event) });
};

ChildRowRenderer.prototype.initWeekStatusCell = function ( row, childRowInfo )
{
    row.find(".statusCell span").text(ChildRowRenderer.weekApprovalStatusCaption[childRowInfo.weekApprovalStatus]);
};

ChildRowRenderer.prototype.initWorkingTimeCell = function ( row, childRowInfo )
{
    var time = !!childRowInfo.ttInfo ? childRowInfo.ttInfo.workingTime : 0;
    if (time == 0){
        row.find(".workingTimeCell" ).addClass("timeIsZero");
    }
    row.find(".workingTimeCell .duration").text(this.formatTime( time));
};

ChildRowRenderer.prototype.initLeaveTimeCell = function ( row, childRowInfo, handlers )
{
        var leavesInfo =  !!childRowInfo.ttInfo ? childRowInfo.ttInfo.leavesInfo : null;
        var totalLeaveTime = leavesInfo == null? 0 : leavesInfo.totalLeaveTime;
        var leaveTimeInfos =  leavesInfo == null? [] : leavesInfo.leaveTimeInfos;

        var self = this;

        if (totalLeaveTime == 0 || leaveTimeInfos.length > 1)
        {
            row.find(".leaveTimeCell .timeCellLabel").text(messageResource.getMessage("approve_tt.approve_table.leave_time"));

            var textDurationElement = row.find(".leaveTimeCell .duration");
            textDurationElement.text(this.formatTime(totalLeaveTime));

            var delayHover = null;
            if (leaveTimeInfos.length > 1){
                textDurationElement.addClass("moreOneLeaveType");
                textDurationElement.hover(function(event){delayHover = setTimeout(function(){
                                              handlers._showLeaveTypeTooltipCallback(
                                                      {leaveTypeInfos: leaveTimeInfos, getLeaveTypeCallback: self._getLeaveTypeCallback},event)},1000)},
                                          function(){ clearTimeout( delayHover );
                                              handlers._hideLeaveTypeTooltipCallback()} );
                textDurationElement.mousemove(function(event){handlers._moveLeaveTypeTooltipCallback(event) });
                textDurationElement.click(function(event) {
                                                    handlers._showLeaveTypeTooltipCallback(
                                                      {leaveTypeInfos: leaveTimeInfos, getLeaveTypeCallback: self._getLeaveTypeCallback},event);
                event.stopPropagation();
                });
            }
            if (totalLeaveTime == 0){
                row.find(".leaveTimeCell" ).addClass("timeIsZero");
            }
        }
        else
        {
            var firstLeaveTimeInfo =  leaveTimeInfos[0];
            row.find(".leaveTimeCell .timeCellLabel").text(this._getLeaveTypeCallback(firstLeaveTimeInfo.leaveTypeId ).name+": "); // todo : pass leave types info here
            row.find(".leaveTimeCell .duration").text(this.formatTime(firstLeaveTimeInfo.leaveTimeDuration));
        }
};

ChildRowRenderer.prototype.initOvertimeCell = function ( row, childRowInfo )
{
    var overtime = !!childRowInfo.ttInfo ? childRowInfo.ttInfo.overtime : null;
    if ( overtime != null && overtime != 0 )
    {
            row.find(".overtimeCell .timeCellLabel").text(messageResource.getMessage("approve_tt.approve_table.overtime"));
            row.find(".overtimeCell .duration").text(this.formatTime(overtime));
    }
};

ChildRowRenderer.prototype.formatSubmitTTLinkText = function ( childRowInfo )
{
    // abstract
};

ChildRowRenderer.prototype.formatSubmitTTLink = function ( link, childRowInfo, approvalRowInfo )
{
    // abstract
};
