function RowRenderer ()
{
    this.constructor = RowRenderer;
}

RowRenderer.prototype.weekApprovalStatusCaptionClass = {};
RowRenderer.prototype.weekApprovalStatusCaptionClass[WeekApprovalStatus.NOT_READY]   = "notReady";
RowRenderer.prototype.weekApprovalStatusCaptionClass[WeekApprovalStatus.READY]       = "ready";
RowRenderer.prototype.weekApprovalStatusCaptionClass[WeekApprovalStatus.APPROVED]    = "approved";
RowRenderer.prototype.weekApprovalStatusCaptionClass[WeekApprovalStatus.REJECTED]    ="rejected";

RowRenderer.prototype.init = function ()
{
    return this;
};

RowRenderer.prototype.createRow = function ()
{
    // abstract
};

RowRenderer.prototype.getRowId = function ()
{
    // abstract
};

RowRenderer.prototype.isRowSelected = function ( row )
{
    return row.data("selected");
};

RowRenderer.prototype.isRowDisabled = function ( row )
{
    return row.data("disabled");
};

RowRenderer.prototype.selectRow = function ( row, alsoDisableSelection )
{
    row.data("selected", true);
    if(alsoDisableSelection)
        row.data( "disabled", true );
    this.updateSelectionUI(row);
};

RowRenderer.prototype.deselectRow = function ( row, alsoEnableSelection )
{
    row.data("selected", false);
    if (alsoEnableSelection)
        row.data( "disabled", false );
    this.updateSelectionUI(row);
};

RowRenderer.prototype.formatTime = function ( time )
{
    return UserTime.formatTime(time);
};

RowRenderer.prototype.initSelectionCell = function( row, onCheckCallback)
{
    var self = this;
    var selectionCheckbox = row.find(".selectionCell input:checkbox");
    row.data("selectionCheckbox", selectionCheckbox);
    selectionCheckbox.change( function () {
        var checked= $(this).prop('checked');
        row.toggleClass("selected", checked);
        onCheckCallback(row, checked )});
     var selectionCell = row.find(".selectionCell");

    var selectionCellClickHandler = function(event){
        event.stopPropagation();
        if (!self.isRowDisabled(row)){
            self.isRowSelected(row) ? self.deselectRow(row) : self.selectRow(row);
            selectionCheckbox.change();
        }
    };
    selectionCheckbox.click(selectionCellClickHandler);
    selectionCell.click(selectionCellClickHandler);
};

RowRenderer.prototype.initRevokeCell =  function(row, revokeCallback){
    var revokeLink =  row.find(".revokeCell .revokeLink" );
    var revoke = function(event){
        event.stopPropagation();
        revokeCallback(row);
    };
    row.find(".revokeCell").on("click", revoke);
};

RowRenderer.prototype.updateSelectionUI = function(row){
    var selection =  row.data("selected");
    var disabled =  row.data("disabled") == true;
    row.data("selectionCheckbox").prop('checked',  selection);
    row.toggleClass("selected",selection);
    row.data("selectionCheckbox").prop('disabled', disabled);
    row.find(".selectionCell").toggleClass("disabled", disabled);
};
