function TTAHistoryTableRenderer()
{
    this.constructor = TTAHistoryTableRenderer;
}

TTAHistoryTableRenderer.prototype.init = function(tableEl, handlers)
{
    this.tableEl = tableEl;
    this.tbody = this.tableEl.find("tbody").first();
    this.rowTemplate = this.tableEl.find("tbody.templates").find("tr.ttaHistoryRow");

    this.handlers = handlers;

    return this;
};

TTAHistoryTableRenderer.prototype.initDateTimeCell = function(row, dateStr)
{
    return row.find(".dateCell").append(Date.parseDate( dateStr.substring( 0, dateStr.length - 6 ), "Y-m-d H:i:s.u", true ).format( "M d, H:i" ));
};

TTAHistoryTableRenderer.prototype.initStatusCell = function(row, statusValue, approverInfo, bySystem)
{
    var statusCell = row.find(".statusCell");
    if (statusValue == WeekApprovalStatus.READY)
        statusCell.append(messageResource.getMessage("approve_tt.tta_history.ready_for_approval"));
    else if (statusValue == WeekApprovalStatus.APPROVED)
        statusCell.append(
            bySystem ?
                messageResource.getMessage("approve_tt.tta_history.auto_approved") :
                messageResource.getMessage("approve_tt.tta_history.approved_by")
        ).addClass("approved");
    else if (statusValue == WeekApprovalStatus.REJECTED)
        statusCell.append(messageResource.getMessage("approve_tt.tta_history.rejected_by")).addClass("rejected");
    else
        throw new Error("Wrong history response format");

    if (statusValue == WeekApprovalStatus.APPROVED || statusValue == WeekApprovalStatus.REJECTED)
    {
        statusCell.append(" " + escapeHtmlSymbols(UserNameFormat.formatFML(approverInfo.firstName, approverInfo.lastName, approverInfo.middleName)));
    }

    return statusCell;
};

TTAHistoryTableRenderer.prototype.initUserCell = function(row, user)
{
    var formattedName = escapeHtmlSymbols(UserNameFormat.formatFML(user.firstName, user.lastName, user.middleName));
    var suffix = formattedName.slice(-1) == "s" ? "'" : "'s";
    row.find(".userCell").append(formattedName + suffix);
};

TTAHistoryTableRenderer.prototype.initCommentCell = function(row, comment)
{
    if ( comment == null || comment == '')
        return;

    var self = this;
    var commentsCell = row.find(".commentCell");
    commentsCell.append("<div class='commentIcon'></div><div class='commentText'>" + escapeHtmlSymbols(comment) +"</div>");

    row.find(".commentCell").hover(function (event) {self.handlers.showTooltipCallback(event, comment)}, this.handlers.hideTooltipCallback);
    row.find(".commentCell").mousemove(function(event) {self.handlers.moveTooltipCallback(event) });
};

TTAHistoryTableRenderer.prototype.initWeekCell = function(row, weekInfo, userInfo)
{
    var link = row.find(".weekCell a");
    var approveTTUtil = new ApproveTTUtil();
    link.html( approveTTUtil.formatDateRange(weekInfo.weekFromDate, weekInfo.weekToDate) );
    link.attr('href', approveTTUtil.generateSubmitTTLink(link, userInfo, weekInfo));
    return row.find(".weekCell").html(link);
};

TTAHistoryTableRenderer.prototype.render = function(historyList, timestampOfLastReadRecord)
{
    this.tbody.empty();
    if (!historyList || historyList.length == 0) {
        var noRecordsRowTemplate = this.tableEl.find("tbody.templates tr.noRecordsRow");
        this.tbody.append(noRecordsRowTemplate.clone());
    }

    this.addRecords(historyList, timestampOfLastReadRecord);
};

TTAHistoryTableRenderer.prototype.addRecords = function(historyList, timestampOfLastReadRecord)
{
    var historyListLength = historyList.length;
    if (historyListLength > 0)
        this.tbody.find(".lastRow").removeClass("lastRow");

    for (var i=0; i < historyListLength; i++)
    {
        var entry = historyList[i];
        var auto = entry.approverNameInfo == null;

        var row = this.rowTemplate.clone();
        this.initDateTimeCell(row, entry.statusTimestampInViewerTimezone);
        this.initUserCell(row, entry.userInfo);
        this.initWeekCell(row, entry.weekInfo, entry.userInfo);
        this.initStatusCell(row, entry.status, entry.approverNameInfo, auto);
        this.initCommentCell(row, entry.comment);

        if (i == historyListLength - 1)
            row.addClass("lastRow");

        if (timestampOfLastReadRecord && entry.date.parseJSONTime() > timestampOfLastReadRecord.parseJSONTime())
            row.addClass("unreadRow");

        this.tbody.append(row);
    }
};