function UserGroupingStrategy ()
{
    this.constructor = UserGroupingStrategy;
}

UserGroupingStrategy.prototype = new TableGroupingStrategy();

UserGroupingStrategy.prototype.init = function ( conf )
{
    TableGroupingStrategy.prototype.init.call(this, conf);
    return this;
};

UserGroupingStrategy.prototype.createGroupingRowRenderer = function ( conf )
{
    return new GroupUserRowRenderer().init( conf );
};

UserGroupingStrategy.prototype.createRowController = function ()
{
    return new UserGroupingRowController().init();
};

UserGroupingStrategy.prototype.createChildRowRenderer = function (conf)
{
    return new ChildWeekRowRenderer().init(conf);
};

UserGroupingStrategy.prototype.createShowMoreRowRenderer = function ()
{
    return new ShowMoreWeeksRowRenderer().init();
};

UserGroupingStrategy.prototype.getMoreGroupingRows = function ( lastGroupingRowId,
                                                                limit,
                                                                rowsPerUserLimit,
                                                                params,
                                                                successHandler,
                                                                errorHandler,
                                                                successCallback,
                                                                errorCallback )
{
    return makeRpcCall( "ApprovalManagementService.getMoreUserGroupingRows",
                        [lastGroupingRowId].concat( params ).concat( [limit, rowsPerUserLimit] ),
                        successCallback,
                        errorCallback
    );
};

//getMoreChildRowsForUser( HttpServletRequest request, UserInfoImpl groupRowId, WeekInfoImpl lastChildRowId, ApproveTTFilter filter, 
//                         SortBy sortBy, int limit, Date groupTimestamp, WeekApprovalStatus groupStatus )
UserGroupingStrategy.prototype.getMoreChildRows = function ( groupingRowId, lastChildRowId, params, limit, successHandler, commonErrorHandler )
{
    var errorHandler =  function (result, textStatus, errorThrown){
        if( errorThrown.exception == 'com.actimind.actitime.administration.NoRightsToApproveUser' )
        {
            alert( messageResource.getMessage( 'approve_tt.approve_table.show_more_user_was_not_assigned',  [groupingRowId.lastName, groupingRowId.firstName]));
            document.location.reload();
            return true;
        }
        if( errorThrown.exception == 'com.actimind.actitime.administration.NoSuchUserException')
        {
            alert( messageResource.getMessage( 'approve_tt.approve_table.show_more_user_deleted',  [groupingRowId.lastName, groupingRowId.firstName] ));
            document.location.reload();
            return true;
        }
        return commonErrorHandler(result, textStatus, errorThrown)
    };
    
    var filter = params.filter;
    return makeRpcCall( "ApprovalManagementService.getMoreChildRowsForUser",
                        [groupingRowId, lastChildRowId, filter[0], filter[2], limit, params.groupTimestamp, params.groupStatus],
                        successHandler,
                        errorHandler
    );
};

UserGroupingStrategy.prototype.getGroupingRowIdFromChildRowId=function(userWeek){
    return userWeek.userId;
};


UserGroupingStrategy.prototype.getChildRowIdFromChildRowId = function(userWeek){
    return userWeek.weekIdDate;
};

UserGroupingStrategy.prototype.getServiceMethodForApprove = function(selection){
    if (selection.getAllRowsSelected())
        return  "ApprovalManagementService.approveAllWeeksForUserGrouping";
   return "ApprovalManagementService.approveWeeksForUserGrouping"
};

UserGroupingStrategy.prototype.getServiceMethodForReject = function(selection){
    if (selection.getAllRowsSelected())
        return  "ApprovalManagementService.rejectAllWeeksForUserGrouping";
    return "ApprovalManagementService.rejectWeeksForUserGrouping"
};

UserGroupingStrategy.prototype.getFilterParamsForOperation = function(params){
   var paramsOperation = [];
   paramsOperation.push(params[0]);
   paramsOperation.push(params[3]);
   return paramsOperation;
};

UserGroupingStrategy.prototype.getServiceMethodForGroupingRevoke = function(){
    return "ApprovalManagementService.revokeWeeksForUserGrouping";
};