function LeaveTypeAffectedBalanceCellRenderer ()
{
    this.constructor = LeaveTypeAffectedBalanceCellRenderer;
}

LeaveTypeAffectedBalanceCellRenderer.prototype = _.create(TableCellRenderer.prototype);

LeaveTypeAffectedBalanceCellRenderer.prototype.init = function ( onEditCallback, numberSettings, coefficientsFormatter, isPtoEnabled, isSdbEnabled )
{
    TableCellRenderer.call( this );
    this.onEditCallback = onEditCallback;
    this.coefficientsFormatter = coefficientsFormatter;
    this.numberSettings = numberSettings;
    this.isPtoEnabled = isPtoEnabled;
    this.isSickEnabled = isSdbEnabled;
    return this;
};

LeaveTypeAffectedBalanceCellRenderer.prototype.getCellClassName = function ( cell )
{
    return ["leaveTypeAffectedBalanceCell"];
};

LeaveTypeAffectedBalanceCellRenderer.prototype.generateContent = function ( cell )
{
    var that = this;

    if ( this.isPtoEnabled && cell.getValue().balanceType == BalanceType.PTO )
        this._appendPtoInfoToCell( cell );
    else if ( this.isSickEnabled && cell.getValue().balanceType == BalanceType.SICK )
        this._appendTextToCell( cell, messageResource.getMessage( "leave_type.list.sick_days_balance" ) );
    else
        this._appendTextToCell( cell, "\u2013" );

    cell.getCellEl().onclick = function(){
        var table = cell.getTable();
        if( !table.affectedBalancePopup )
        {
            var container = $( "#AffectedBalancePopup" );
            table.affectedBalancePopup = new AffectedBalancePopup().init( container, that.onEditCallback, that._createAndInitBalancesEditPanelController( container ));
        }
        table.affectedBalancePopup.setValue( cell.value );
        table.affectedBalancePopup.show(
                $( cell.getCellEl() ),
                function()
                {
                    cell.getRow().addOpenPopupClass();
                    $( cell.getCellEl() ).addClass( "affectedBalancePopupOpened" );
                },
                function()
                {
                    cell.getRow().removeOpenPopupClass();
                    $( cell.getCellEl() ).removeClass( "affectedBalancePopupOpened" );
                }
        );
    };
};

LeaveTypeAffectedBalanceCellRenderer.prototype._appendTextToCell = function( cell, text )
{
    var affectedBalanceElem = $( "<span>" );
    affectedBalanceElem.addClass( "leaveTypeInfoText" );
    affectedBalanceElem.text( text );
    cell.getCellEl().appendChild( affectedBalanceElem[0] );
};

LeaveTypeAffectedBalanceCellRenderer.prototype._appendTdWithTextToTr = function( text, trElem, textStyle )
{
    var tdElem = $( "<td>" );
    if ( textStyle )
        tdElem.addClass( textStyle );
    tdElem.append( $( "<span>" ).text( text ) );
    tdElem.appendTo( trElem );
};

LeaveTypeAffectedBalanceCellRenderer.prototype._appendPtoInfoToCell = function( cell )
{
    var affectedBalanceElem = $( "<table>" );
    affectedBalanceElem.addClass( "leaveTypeInfoText" );
    var trElem = $( "<tr>" );
    affectedBalanceElem.append( trElem );
    this._appendTdWithTextToTr( messageResource.getMessage( "leave_type.list.is_pto" ), trElem );
    this._appendTdWithTextToTr(
            messageResource.getMessage( "leave_type.list.coeff" ) + " "
            + this.coefficientsFormatter.format( cell.getValue().balanceCoefficient ),
            trElem,
            "coefficientTd"
    );
    cell.getCellEl().appendChild( affectedBalanceElem[0] );
};

LeaveTypeAffectedBalanceCellRenderer.prototype._createAndInitBalancesEditPanelController = function( container )
{
    var balancesEditPanelControllerConstructor = NoBalancesEditPanelController;
    if ( this.isPtoEnabled && this.isSickEnabled )
        balancesEditPanelControllerConstructor = LeaveTypeBalancesEditPanelController;
    else if ( this.isPtoEnabled )
        balancesEditPanelControllerConstructor = PtoBalanceEditPanelController;
    else if ( this.isSickEnabled )
        balancesEditPanelControllerConstructor = SickDaysBalanceEditPanelController;

    return new balancesEditPanelControllerConstructor().init( {
                                                                container: container,
                                                                numberSettings: this.numberSettings,
                                                                elementSuffix: "affectedBalancePopup"
                                                               } );
};
