function NoBalancesEditPanelController()
{
    this.constructor = NoBalancesEditPanelController;
}

NoBalancesEditPanelController.DEFAULT_BALANCE_COEFF = 1;

NoBalancesEditPanelController.WRONG_PTO_COEFF_ERROR = "wrongPtoCoeffError";
NoBalancesEditPanelController.EMPTY_PTO_COEFF_ERROR = "emptyPtoCoeffError";

NoBalancesEditPanelController.prototype.init = function( config )
{
    this.container = config.container;
    this.numberSettings = config.numberSettings;
    this.elementSuffix = config.elementSuffix;
    this.errors = [];

    this._initElements();
    this._hideElements();

    return this;
};

NoBalancesEditPanelController.prototype._initElements = function ()
{
    this.processLeaveTimeCheckbox = this._getElementBySubId( "processLeaveTimeCheckbox" );
    this.asPtoRadioButton = this._getElementBySubId( "asPtoRadioButton" );
    this.asPtoDiv = this.container.find( ".asPtoDiv" );
    this.ptoCoefficientInput = this._getElementBySubId( "ptoCoefficientInput" );
    this.ptoCoeffErrorPlaceholder = this.container.find( ".ptoCoeffErrorPlaceholder" );
    this.ptoCoefficientField = new DecimalField( this.ptoCoefficientInput, true, 5, 2, this.numberSettings.decimalSeparator, false, this.numberSettings.digitGroupingSymbol, undefined, true, this.numberSettings.ptoMaxNumber );
    this.asSickDaysRadioButton = this._getElementBySubId( "asSickDaysRadioButton" );
    this.asSickDaysDiv = this.container.find( ".asSickDaysDiv" );

    var errorSpan = $("<span>");
    errorSpan.addClass("emptyPtoCoeffErrorText");
    errorSpan.text( messageResource.getMessage("leave_type.error.pto_coefficient_required") );
    this.ptoCoeffErrorPlaceholder.append( errorSpan );
    errorSpan = $("<span>");
    errorSpan.addClass("wrongPtoCoeffErrorText");
    errorSpan.text( messageResource.getMessage("leave_type.error.invalid_pto_coefficient", [this.numberSettings.decimalSeparator, this.numberSettings.ptoMaxNumber]) );
    this.ptoCoeffErrorPlaceholder.append( errorSpan );
};

NoBalancesEditPanelController.prototype.isProcessLeaveTypeChecked = function()
{
    return !!this.processLeaveTimeCheckbox.attr( "checked" );
};

NoBalancesEditPanelController.prototype._hideElements = function ()
{
    this.container.addClass( "hidden" );
};

NoBalancesEditPanelController.prototype.hideErrors = function()
{
    this.errors = [];
    this.container.removeClass( NoBalancesEditPanelController.WRONG_PTO_COEFF_ERROR );
};

NoBalancesEditPanelController.prototype.checkEmpty = function()
{
    return false;
};

NoBalancesEditPanelController.prototype._checkEmptyAsPto = function()
{
    return this.ptoCoefficientField.isEmpty();
};

NoBalancesEditPanelController.prototype.hideEmptyError = function()
{
    this.container.removeClass(NoBalancesEditPanelController.EMPTY_PTO_COEFF_ERROR);
};

NoBalancesEditPanelController.prototype.showEmptyPtoErrorMessage = function()
{
    this.container.addClass(NoBalancesEditPanelController.EMPTY_PTO_COEFF_ERROR);
};

NoBalancesEditPanelController.prototype.showAllErrors = function()
{
    if ( this.checkEmpty() )
        this.showEmptyPtoErrorMessage();
    this.showErrors();
};

NoBalancesEditPanelController.prototype.hideAllErrors = function()
{
    this.hideEmptyError();
    this.hideErrors();
};

NoBalancesEditPanelController.prototype.populate = function( data )
{
    //do nothing
};

NoBalancesEditPanelController.prototype.validate = function()
{
    return true;
};

NoBalancesEditPanelController.prototype.getBalanceType = function()
{
    return null;
};

NoBalancesEditPanelController.prototype.getBalanceCoefficient = function()
{
    return this.isAsPTOChecked() ? this.ptoCoefficientField.getValueAsNumber() : NoBalancesEditPanelController.DEFAULT_BALANCE_COEFF;
};

NoBalancesEditPanelController.prototype._setPtoCoefficientIntoField = function( ptoCoefficient )
{
    this.ptoCoefficientField.setValueAsNumber( "number" == typeof ptoCoefficient ? ptoCoefficient : NoBalancesEditPanelController.DEFAULT_BALANCE_COEFF );
};

NoBalancesEditPanelController.prototype._getElementBySubId = function( subId )
{
    return $( "#" + subId + "_" + this.elementSuffix );
};

NoBalancesEditPanelController.prototype.showErrors = function()
{
    for( var i = 0; i < this.errors.length; i++ )
    {
        this.container.addClass( this.errors[i] );
    }
};

NoBalancesEditPanelController.prototype._validateAsPto = function()
{
    if ( !this.isAsPTOChecked() )
        return true;
    this.errors = [];
    if ( !this.ptoCoefficientField.isEmpty() && !this.ptoCoefficientField.isValid() )
    {
        this.errors.push( NoBalancesEditPanelController.WRONG_PTO_COEFF_ERROR );
        return false;
    }
    return true;
};

NoBalancesEditPanelController.prototype.setOnblurPtoCoefficientInputElement = function( onblurCallback )
{
    this.ptoCoefficientInput.on( "blur", onblurCallback );
};

NoBalancesEditPanelController.prototype.isAsPTOChecked = function()
{
    return this.isProcessLeaveTypeChecked() && this.asPtoRadioButton.attr( "checked" );
};

//-----------------------------------------------------------------------------------------------------------------------


function LeaveTypeBalancesEditPanelController()
{
    this.constructor = LeaveTypeBalancesEditPanelController;
}

LeaveTypeBalancesEditPanelController.prototype = new NoBalancesEditPanelController();

LeaveTypeBalancesEditPanelController.prototype.init = function( config )
{
    NoBalancesEditPanelController.prototype.init.call( this, config );

    var self = this;
    var innerErrorsChangeCallback = config.innerErrorsChangeCallback;

    this.processLeaveTimeCheckbox.click( function(event)
                                         {
                                             self._changeDisableValueOfCheckboxes( event.target.checked );
                                             if ( self.asPtoRadioButton.attr( "checked" ) && event.target.checked )
                                                 self.ptoCoefficientInput.removeAttr( "disabled" );
                                             else
                                                 self.ptoCoefficientInput.attr( "disabled", true );
                                             if ( innerErrorsChangeCallback )
                                                 innerErrorsChangeCallback();
                                         }
    );

    this.container.find( "input[name='processLeaveTimeRadio']" ).click( function(event)
                                                                        {
                                                                            if ( event.target != self.asPtoRadioButton[0] )
                                                                            {
                                                                                self.hideAllErrors();
                                                                                self.ptoCoefficientInput.attr( "disabled", true );
                                                                                if ( innerErrorsChangeCallback )
                                                                                    innerErrorsChangeCallback();
                                                                            }
                                                                            else
                                                                            {
                                                                                self.ptoCoefficientInput.removeAttr( "disabled" );
                                                                            }
                                                                        }
    );

    return this;
};

LeaveTypeBalancesEditPanelController.prototype._hideElements = function ()
{
    // do nothing
};

LeaveTypeBalancesEditPanelController.prototype.populate = function( data )
{
    if ( !data ) return;

    switch ( data.balanceType ) {
        case BalanceType.PTO:
            this.processLeaveTimeCheckbox.attr( "checked", "checked" );
            this.asPtoRadioButton.attr( "checked", "checked" );
            break;
        case BalanceType.SICK:
            this.processLeaveTimeCheckbox.attr( "checked", "checked" );
            this.asSickDaysRadioButton.attr( "checked", "checked" );
            break;
        default:
            this.asSickDaysRadioButton.removeAttr( "checked" );
            this.asPtoRadioButton.attr( "checked", "checked" );
            this.processLeaveTimeCheckbox.removeAttr( "checked" );
    }
    if ( data.balanceType == BalanceType.PTO )
    {
        this._setPtoCoefficientIntoField( data.ptoCoefficient );
        this.ptoCoefficientInput.removeAttr( "disabled" );
    }
    else
    {
        this._setPtoCoefficientIntoField(NoBalancesEditPanelController.DEFAULT_BALANCE_COEFF);
        this.ptoCoefficientInput.attr( "disabled", true );
    }
    this._changeDisableValueOfCheckboxes( this.processLeaveTimeCheckbox.attr( "checked" ) );
};

LeaveTypeBalancesEditPanelController.prototype._changeDisableValueOfCheckboxes = function( value )
{
    if ( value )
    {
        this.asPtoRadioButton.removeAttr( "disabled" );
        this.asSickDaysRadioButton.removeAttr( "disabled" );
    }
    else
    {
        this.asPtoRadioButton.attr( "disabled", true );
        this.asSickDaysRadioButton.attr( "disabled", true );
        this.hideAllErrors();
    }
};

LeaveTypeBalancesEditPanelController.prototype.validate = function()
{
    return this._validateAsPto();
};

LeaveTypeBalancesEditPanelController.prototype.getBalanceType = function()
{
    var balanceType = null;
    if ( this.isProcessLeaveTypeChecked() )
    {
        balanceType = this.asPtoRadioButton.attr( "checked" ) ? BalanceType.PTO : BalanceType.SICK;
    }
    return balanceType;
};

LeaveTypeBalancesEditPanelController.prototype.checkEmpty = function()
{
    return this._checkEmptyAsPto();
};


//-----------------------------------------------------------------------------------------------------------------------


function PtoBalanceEditPanelController()
{
    this.constructor = PtoBalanceEditPanelController;
}

PtoBalanceEditPanelController.prototype = new NoBalancesEditPanelController();

PtoBalanceEditPanelController.prototype.init = function( config )
{
    NoBalancesEditPanelController.prototype.init.call( this, config );

    var self = this;
    var innerErrorsChangeCallback = config.innerErrorsChangeCallback;

    this.processLeaveTimeCheckbox.click( function(event)
                                         {
                                             self._changeDisableValueOfPtoInput( event.target.checked );
                                             if ( !event.target.checked && innerErrorsChangeCallback )
                                                 innerErrorsChangeCallback();
                                         }
    );

    return this;
};

PtoBalanceEditPanelController.prototype._changeDisableValueOfPtoInput = function( value )
{
    if ( value )
    {
        this.ptoCoefficientInput.removeAttr( "disabled" );
    }
    else
    {
        this.ptoCoefficientInput.attr( "disabled", true );
        this.hideAllErrors();
    }
};

PtoBalanceEditPanelController.prototype._hideElements = function ()
{
    this.container.addClass( "withoutRadioButtons" );
    this.asSickDaysDiv.addClass( "hidden" );
};

PtoBalanceEditPanelController.prototype.validate = function()
{
    return this._validateAsPto();
};

PtoBalanceEditPanelController.prototype.populate = function( data )
{
    if ( !data ) return;

    if ( data.balanceType == BalanceType.PTO )
    {
        this.processLeaveTimeCheckbox.attr( "checked", "checked" );
        this._setPtoCoefficientIntoField( data.ptoCoefficient );
    }
    else
    {
        this.processLeaveTimeCheckbox.removeAttr( "checked" );
        this._setPtoCoefficientIntoField( NoBalancesEditPanelController.DEFAULT_BALANCE_COEFF );
    }
    this._changeDisableValueOfPtoInput( data.balanceType == BalanceType.PTO );

    this.asPtoRadioButton.attr( "checked", "checked" );
};

PtoBalanceEditPanelController.prototype.getBalanceType = function()
{
    return this.processLeaveTimeCheckbox.attr( "checked" ) ? BalanceType.PTO : null ;
};

PtoBalanceEditPanelController.prototype.checkEmpty = function()
{
    return this._checkEmptyAsPto();
};


//-----------------------------------------------------------------------------------------------------------------------


function SickDaysBalanceEditPanelController()
{
    this.constructor = PtoBalanceEditPanelController;
}

SickDaysBalanceEditPanelController.prototype = new NoBalancesEditPanelController();

SickDaysBalanceEditPanelController.prototype._hideElements = function ()
{
    this.container.addClass( "withoutRadioButtons" );
    this.asPtoDiv.addClass( "hidden" );
};

SickDaysBalanceEditPanelController.prototype.populate = function( data )
{
    if ( !data ) return;

    if ( data.balanceType == BalanceType.SICK )
        this.processLeaveTimeCheckbox.attr( "checked", "checked" );
    else
        this.processLeaveTimeCheckbox.removeAttr( "checked" );

    this.asSickDaysRadioButton.attr( "checked", "checked" );
};

SickDaysBalanceEditPanelController.prototype.getBalanceType = function()
{
    return this.processLeaveTimeCheckbox.attr( "checked" ) ? BalanceType.SICK : null;
};
