function LeaveTypeLightBox() {
}

LeaveTypeLightBox.NAME_EMPTY = "nameEmptyError";
LeaveTypeLightBox.NAME_DUPLICATE = "nameDuplicateError";
LeaveTypeLightBox.PTO_COEFF_EMPTY = "ptoCoeffEmptyError";
LeaveTypeLightBox.PTO_COEFF_INVALID = "ptoCoeffInvalidError";
LeaveTypeLightBox.RATE_COEFF_EMPTY = "rateCoeffEmptyError";
LeaveTypeLightBox.RATE_COEFF_INVALID = "rateCoeffInvalidError";

LeaveTypeLightBox.CREATE_POPUP_DEFAULT_LEAVE_TYPE_ID = 0;           // no leave type id yet
LeaveTypeLightBox.CREATE_POPUP_DEFAULT_LEAVE_TYPE_NAME = "";        // empty field
LeaveTypeLightBox.CREATE_POPUP_DEFAULT_LEAVE_TYPE_STATUS = true;    // active
LeaveTypeLightBox.CREATE_POPUP_DEFAULT_IS_PTO = false;              // does not affect pto
LeaveTypeLightBox.CREATE_POPUP_DEFAULT_PTO_COEFF = 1;               // as default in SystemSettings


LeaveTypeLightBox.prototype = _.create(BasicLightbox.prototype);
LeaveTypeLightBox.prototype.exist_invalid_fields_message_key = "leave_type.edit.correct_errors";
LeaveTypeLightBox.prototype.close_tab_confirmation_message_key = "leave_type.create.close_tab_confirmation";

LeaveTypeLightBox.prototype.init = function( numberSettings, iconSelectorPanel, balancesEditPanelControllerConstructor )
{
    var self = this;

    this.balancesEditPanel = new balancesEditPanelControllerConstructor().init( {
            container: $( ".balancesEditPanelSection" ),
            numberSettings: numberSettings,
            elementSuffix: "createPopup",
            innerErrorsChangeCallback: function() { self.updateErrors(false); }
    } );

    BasicLightbox.prototype.init.call( this, "leaveTypeLightBox", "closeLeaveTypeLightBoxBtn" );

    this.numberSettings = numberSettings;
    this.cancelCallback = self.hide;

    this.titlePlaceholder = this.getElementBySubId("titlePlaceholder");
    this.leaveTypeNamePlaceholder = this.getElementBySubId("leaveTypeNamePlaceholder");
    this.leaveTypeInfoTable = this.getElementBySubId("leaveTypeInfoTable");

    this.commitButtonTitlePlaceholder = $(document.getElementById( this.lightBoxEl.id + "_commitBtn" )).find(".buttonTitle");

    this.nameField = $(this.getElementBySubId("nameField"));

    this.iconSelector = new LeaveTypeIconSelector().init(iconSelectorPanel, function() { self.changesCallback(); } );
    this.iconSelectorPl = this.getElementBySubId("iconSelectorPlaceholder");
    this.iconSelectorPl.appendChild(this.iconSelector.getSelectorEl());
    this.newLeaveTypeDefaultIconId = 1;

    this.balancesEditPanel.setOnblurPtoCoefficientInputElement( function() {
        if ( self.balancesEditPanel.checkEmpty() )
        {
            self.balancesEditPanel.hideErrors();
            return;
        }
        self.balancesEditPanel.hideEmptyError();
        if ( self.balancesEditPanel.validate() )
            self.balancesEditPanel.hideErrors();
        else
            self.balancesEditPanel.showErrors();
        self.updateErrors(false);
    });

    this.statusSelectorPlaceHolderEl = this.getElementBySubId("statusSelectorPlaceholder");

    this.footer = $(this.getElementBySubId("footer"));

    return this;
};

// function to set value for input text fields with placeholder
LeaveTypeLightBox.prototype.setValue = function( field, value )
{
    field.removeClass('placeholder'); // IE8 doesn't switch placeholder class when field value is set
    field[0].value = value;
    field[0].prevValue = field[0].value;
};

/* Popup Title */
LeaveTypeLightBox.prototype.setTitle = function( title )
{
    this.titlePlaceholder.innerHTML = escapeHtmlSymbols( title );
};
LeaveTypeLightBox.prototype.setNameInTitle = function( name )
{
    this.leaveTypeNamePlaceholder.innerHTML = escapeHtmlSymbols( name );
};
/* End Popup Title */

/* Leave Type Name Field */
LeaveTypeLightBox.prototype.getLeaveTypeName = function()
{
    return this.nameField.pVal();
};
LeaveTypeLightBox.prototype.setNameFieldHandlersForCreatePopup = function () {
    var self = this;
    this.nameField.off();
    this.nameField.blur( this.basicOnBlurFieldHandler( function () { self.checkNameOnChange(); self.setNameInTitle(self.getLeaveTypeName()); } ) );
    this.nameField.keyup( this.basicKeyUpChangesHandler( function () { self.setNameInTitle(self.getLeaveTypeName()) } ) );
    this.nameField.change( function () { self.hideNameDuplicateError() } );
    this.nameField.keydown( this.basicOnKeyPressedHandler );
};
LeaveTypeLightBox.prototype.setNameFieldHandlersForEditPopup = function () {
    var self = this;
    this.nameField.off();
    this.nameField.blur( this.basicOnBlurFieldHandler( function () { self.checkNameOnChange(); } ) );
    this.nameField.keyup( this.basicKeyUpChangesHandler() );
    this.nameField.change( function () { self.hideNameDuplicateError() } );
    this.nameField.keydown( this.basicOnKeyPressedHandler );
};
LeaveTypeLightBox.prototype.checkNameOnChange = function ()
{
    this.checkEmpty( this.nameField, LeaveTypeLightBox.NAME_EMPTY, true, true );
    this.updateErrors(false);
};
LeaveTypeLightBox.prototype.isNameValid = function( onlyRemoveEmptyError )
{
    return !this.checkEmpty( this.nameField, LeaveTypeLightBox.NAME_EMPTY, onlyRemoveEmptyError, true );
};
LeaveTypeLightBox.prototype.displayNameDuplicateError = function ()
{
    this.errors[LeaveTypeLightBox.NAME_DUPLICATE] = true;
    this.updateErrors(true);
};

LeaveTypeLightBox.prototype.hideNameDuplicateError = function ()
{
    this.errors[LeaveTypeLightBox.NAME_DUPLICATE] = false;
    this.updateErrors();
};

LeaveTypeLightBox.prototype.setFocusIntoNameField = function ()
{
    setFocusIntoFieldWithPlaceholder( this.nameField );
};
/* End Leave Type Name Field */

/* Status Selector */
LeaveTypeLightBox.prototype.initStatusSelector = function ( value )
{
    var self = this;
    var statusSelectorItems = [
        {
            text: messageResource.getMessage("leave_type.active"),
            value: true
        },
        {
            text: messageResource.getMessage("leave_type.archived"),
            value: false
        }
    ];

    this.statusSelectorPlaceHolderEl.innerHTML = "";
    this.statusSelector = new ListSelector({
                                               menuCls: "leaveTypeStatusSelectorMenu",
                                               renderTo: this.statusSelectorPlaceHolderEl,
                                               value: value,
                                               listItems: statusSelectorItems
                                           });
    var handler = function (value) {
        $(self.statusSelectorPlaceHolderEl).removeClass("active archived");
        $(self.statusSelectorPlaceHolderEl ).addClass(value ? "active" : "archived");
    };
    handler(value);
    this.statusSelector.on( "change", function () { self.changesCallback(); handler(self.getStatus()); } );
};
LeaveTypeLightBox.prototype.getStatus = function ()
{
    return this.statusSelector.getSelectedItem().value;
};
/* End Status Selector */

/* Some Utility functions */
LeaveTypeLightBox.prototype.getElementBySubId = function( subId )
{
    return document.getElementById( this.lightBoxEl.id + "_" + subId );
};
LeaveTypeLightBox.prototype.setNewLeaveTypeDefaultIcon = function ( iconId ) {
    this.newLeaveTypeDefaultIconId = iconId;
};
/* End Some Utility functions */

/* Common Validating */
LeaveTypeLightBox.prototype.isCoeffValid = function( coeffField, coeffFieldEl, emptyErrorCode, invalidErrorCode, checkIfNotEmpty )
{
    this.errors[ invalidErrorCode ] = false;
    if (checkIfNotEmpty)
    {
        if( !this.checkEmpty( coeffFieldEl, emptyErrorCode, true ) )
            return this.errors[ invalidErrorCode ] = !coeffField.isValid();
    }
    else
    {
        var valid = coeffField.isValid();
        this.errors[ invalidErrorCode ] = !valid;
        return !(this.checkEmpty( coeffFieldEl, emptyErrorCode, false ) || !valid);
    }
};

LeaveTypeLightBox.prototype.getErrorsMap = function()
{
    return this.errors;
};

LeaveTypeLightBox.prototype.validate = function ()
{
    var result = this.performCommonCheck() & this.performSpecificCheck();
    this.updateErrors(true);
    return result;
};
LeaveTypeLightBox.prototype.performCommonCheck = function ()
{
    return this.isNameValid(false) & !this.errors[LeaveTypeLightBox.NAME_DUPLICATE];
};
LeaveTypeLightBox.prototype.performSpecificCheck = function () {
    return true;
};
/* End Common Validating */

/* Initialization before rendering */
LeaveTypeLightBox.prototype.initBeforeShow = function () {
    this.dataChanged = false;
    this.footer.removeClass("changesDetected");
    this.resetErrors();
};
LeaveTypeLightBox.prototype.initAsCreatePopup = function ( ) {
    this.initBeforeShow();

    this.editMode = false;
    $(this.lightBoxEl ).removeClass("editMode");
    this.setTitle(messageResource.getEscapedMessage("leave_type.create.title"));
    this.setNameInTitle(LeaveTypeLightBox.CREATE_POPUP_DEFAULT_LEAVE_TYPE_NAME);
    this.setCommitButtonTitle(messageResource.getEscapedMessage("leave_type.create.commit_button"));

    this.leaveTypeId = LeaveTypeLightBox.CREATE_POPUP_DEFAULT_LEAVE_TYPE_ID;

    this.setValue(this.nameField, LeaveTypeLightBox.CREATE_POPUP_DEFAULT_LEAVE_TYPE_NAME);
    this.setNameFieldHandlersForCreatePopup();
    this.nameField.placeholder();

    this.initStatusSelector(LeaveTypeLightBox.CREATE_POPUP_DEFAULT_LEAVE_TYPE_STATUS);
    this.iconSelector.setIcon(this.newLeaveTypeDefaultIconId);

    this.cancel_confirmation_message_key = "leave_type.create.exit_confirmation";
    this.close_tab_confirmation_message_key = "leave_type.create.close_tab_confirmation";

    this.balancesEditPanel.hideAllErrors();
    this.balancesEditPanel.populate( {
        balanceType: null,
        ptoCoefficient: LeaveTypeLightBox.CREATE_POPUP_DEFAULT_PTO_COEFF
    } );
};
/* End Initialization before rendering */

/* Changes handling */
LeaveTypeLightBox.prototype.changesDetected = function()
{
    if (!this.dataChanged)
    {
        this.dataChanged = true;
        if( this.editMode )
            this.footer.addClass( "changesDetected" );
    }
};
/* End Changes handling */

/* Errors Processing */
LeaveTypeLightBox.prototype.resetErrors = function () {
    this.errors = {};
    this.updateErrors();
};

LeaveTypeLightBox.prototype.setErrorClasses = function( classes )
{
    this.leaveTypeInfoTable.className = classes.join( " " );
};

LeaveTypeLightBox.prototype.updateErrors = function ( showErrorInFooter )
{
    if (this.hasErrors())
    {
        if( showErrorInFooter )
            this.displayErrorInFooter();
    }
    else
        this._removeErrorInFooter();
};
LeaveTypeLightBox.prototype.updateUI = function ()
{
    this.updateErrors(true);
};
LeaveTypeLightBox.prototype.displayErrorInFooter = function ()
{
    this._errorMessageBox.removeClass( "disabled" );
    this.balancesEditPanel.showAllErrors();
};

LeaveTypeLightBox.prototype._removeErrorInFooter = function ()
{
    BasicLightbox.prototype._removeErrorInFooter.call( this );
    this.balancesEditPanel.hideAllErrors();
};

LeaveTypeLightBox.prototype.hasErrors = function ()
{
    return BasicLightbox.prototype.hasErrors.call( this ) || this.balancesEditPanel.checkEmpty() || !this.balancesEditPanel.validate();
};
/* End Errors Processing */
