function LeaveTypeListPageController() {
    this.constructor = LeaveTypeListPageController;
}

LeaveTypeListPageController.RESULT_SUCCESSFULL = 200;
LeaveTypeListPageController.ERROR_LEAVE_TYPE_ALREADY_IN_USE = 400;
LeaveTypeListPageController.ERROR_NAME_DUPLICATE = 401;
LeaveTypeListPageController.ERROR_COMPETITIVE_DELETED = 402;

LeaveTypeListPageController.prototype.init = function (  leaveTypesInfo, numberSettings, newLeaveTypeDefaultIconId ) {
    var that = this;

    this.tHeadEl = document.getElementById("leaveTypesListTableHeader");
    this.containerEl = document.getElementById("leaveTypeListTableContainer");

    this.numberSettings = numberSettings;
    this.affectedBalanceHeaderTooltip = new DropdownDialog({
                                                         wrapperClass: "AffectedBalanceHeaderTooltipWrapper",
                                                         dialogDiv: $("#AffectedBalanceHeaderTooltip"),
                                                         hideButtons: true,
                                                         hideOnResize: true,
                                                         showEmptyTitleBar: false,
                                                         isWithBeak: true
                                                     });

    this.leaveTypeInUseTooltip = new DelayTooltip().init("cantDeleteLeaveTypeTooltip");

    this.newLeaveTypeButton = new GraphicButton( GraphicButton.getRoundedBox('createLeaveTypeDiv', messageResource.getMessage("leave_type.list.add_button_text")));
    this.newLeaveTypeButton.defineStandardEventHandlers(true, true, true, true);
    this.newLeaveTypeButton.button.el.on("click", function (leaveTypeInfo) { that.showCreateLeaveTypePopup(leaveTypeInfo) });

    this.iconSelectorPanel = this.createAndInitIconSelectorPanel();

    this.leaveTypeLightBox = this.createAndInitLeaveTypeLightBox();
    this.leaveTypeLightBox.setNewLeaveTypeDefaultIcon(newLeaveTypeDefaultIconId);

    this.errorMessages = {};
    this.errorMessages[LeaveTypeListPageController.ERROR_COMPETITIVE_DELETED] = messageResource.getMessage('leave_type.edit.error.competitive_deleted');
    this.errorMessages[LeaveTypeListPageController.ERROR_LEAVE_TYPE_ALREADY_IN_USE] = messageResource.getMessage('leave_type.list.cannot_delete.already_in_use');

    this.initializeHandlersMap();

    this.spinnerController = new SpinnerController();

    return this;
};

LeaveTypeListPageController.prototype.initializeHandlersMap = function ()
{
    // abstract
};

LeaveTypeListPageController.prototype.createBaseHandlersMap = function ()
{
    var that = this;
    return {
        showAffectedBalanceHeaderTooltip: function () { that.showAffectedBalanceHeaderTooltip() },
        hideAffectedBalanceHeaderTooltip: function () { that.hideAffectedBalanceHeaderTooltip() },
        deleteCellMouseOn: that.getShowTooltip(that.leaveTypeInUseTooltip),
        deleteCellMouseMove: that.getMoveTooltip(that.leaveTypeInUseTooltip),
        deleteCellMouseOut: that.getHideTooltip(that.leaveTypeInUseTooltip),
        deleteLeaveType: function (leaveTypeInfo) { that.deleteLeaveType( leaveTypeInfo ) },
        moveLeaveType: function (srcLeaveTypeId, targetLeaveTypeI) { that.moveLeaveType(srcLeaveTypeId, targetLeaveTypeI) },
        setLeaveTypeIcon : function (leaveTypeId, newIconId, callback) { that.setLeaveTypeIcon(leaveTypeId, newIconId, callback) },
        changeLeaveTypeStatus: function (leaveTypeId, currentStatus) { that.changeLeaveTypeStatus(leaveTypeId, currentStatus) },
        changeLeaveName: function(leaveTypeId, newName, customSuccessCallback, customErrorCallback){that.changeLeaveTypeName(leaveTypeId,newName, customSuccessCallback,  customErrorCallback )},
        changeAffectedBalance: function(leaveTypeId, balanceType, balanceCoefficient) {that.changeAffectedBalance(leaveTypeId, balanceType, balanceCoefficient)}
    };
};

LeaveTypeListPageController.prototype.render = function ( )
{
    this.leaveTypeListTableController.render();
};

LeaveTypeListPageController.prototype.showAffectedBalanceHeaderTooltip = function ( )
{
    this.affectedBalanceHeaderTooltip.showAt( document.getElementById("affectedBalanceInfoIcon"), {my: "left top", at: "left bottom", offset: this.tableHeaderTooltipsOffsetParams, collision: "none"} );
};
LeaveTypeListPageController.prototype.hideAffectedBalanceHeaderTooltip = function ( )
{
    this.affectedBalanceHeaderTooltip.hide();
};

LeaveTypeListPageController.prototype.getShowTooltip = function( tooltip )
{
    return function( config, event ) {
        tooltip.updateUIAndShow( config, calculateTooltipOffset( tooltip, event ) );
    };
};

LeaveTypeListPageController.prototype.getMoveTooltip = function( tooltip )
{
    return function( event ) {
        tooltip.move( calculateTooltipOffset( tooltip, event ) );
    };
};

LeaveTypeListPageController.prototype.getHideTooltip = function( tooltip )
{
    return function() {
        tooltip.hide();
    }
};

LeaveTypeListPageController.prototype.deleteLeaveType = function ( leaveTypeInfo )
{
    if (confirm(messageResource.getMessage('leave_type.confirm_delete')))
        this.callLeaveTypeServiceMethod( "LeaveTypeService.deleteLeaveType", [leaveTypeInfo.id], true );
};

LeaveTypeListPageController.prototype.moveLeaveType = function ( srcLeaveTypeId, targetLeaveTypeId )
{
    var self =this;
    var customSuccessHandler = function(result){
        self.leaveTypeListTableController.temporaryMarkRow(result.changedLeaveTypeId);
    };
    this.callLeaveTypeServiceMethod( "LeaveTypeService.moveLeaveType", [ srcLeaveTypeId, targetLeaveTypeId], true, customSuccessHandler );
};

LeaveTypeListPageController.prototype.showCreateLeaveTypePopup = function ( )
{
    var self = this;
    this.leaveTypeLightBox.initAsCreatePopup();
    this.leaveTypeLightBox.setSuccessCallback(
            function (leaveTypeInfo) {
                self.createLeaveType(leaveTypeInfo);
            } );
    this.leaveTypeLightBox.show();
    this.leaveTypeLightBox.setFocusIntoNameField();
};

LeaveTypeListPageController.prototype.createLeaveType = function ( leaveTypeInfo )
{
    this.callPopupLeaveTypeServiceMethod("LeaveTypeService.createLeaveType", [leaveTypeInfo]);
};

LeaveTypeListPageController.prototype.callPopupLeaveTypeServiceMethod = function ( method, params ) {
    var self = this;
    this.leaveTypeLightBox.disableUserInputWhileSaving();
    this.callLeaveTypeServiceMethod(method, params, false,
                                    function (result) {
                                        self.leaveTypeLightBox.enableUserInputWhileSaving();
                                        if (result.resultCode == LeaveTypeListPageController.ERROR_NAME_DUPLICATE)
                                        {
                                            self.leaveTypeLightBox.displayNameDuplicateError();
                                            return true;
                                        }
                                        else
                                            self.leaveTypeLightBox.hide();
                                    },
                                    function (jqXHR, textStatus, errorThrown) {
                                        self.leaveTypeLightBox.enableUserInputWhileSaving();
                                    } );
};

LeaveTypeListPageController.prototype.setLeaveTypeIcon = function ( leaveTypeId, iconId, callback)
{
    this.callLeaveTypeServiceMethod("LeaveTypeService.setIconId", [leaveTypeId, iconId], true, callback);
};

LeaveTypeListPageController.prototype.changeLeaveTypeStatus = function ( leaveTypeId, currentStatus )
{
    this.callLeaveTypeServiceMethod("LeaveTypeService.setLeaveTypeStatus", [leaveTypeId, !currentStatus], true);
};

LeaveTypeListPageController.prototype.changeLeaveTypeName = function ( leaveTypeId, newName, customSuccessCallback )
{
    this.callLeaveTypeServiceMethod("LeaveTypeService.setLeaveTypeName", [leaveTypeId, newName], true, customSuccessCallback);
};

LeaveTypeListPageController.prototype.changeAffectedBalance = function ( leaveTypeId, balanceType, balanceCoefficient )
{
    this.callLeaveTypeServiceMethod("LeaveTypeService.setAffectedBalance", [leaveTypeId, balanceType, balanceCoefficient], true);
};

LeaveTypeListPageController.prototype.leaveTypeServiceResultHandler = function ( result, customSuccessHandler )
{
    if (this.errorMessages[result.resultCode])
        alert(this.errorMessages[result.resultCode]);

    this.leaveTypeListTableController.redrawTableBody(result.leaveTypesInfo.list);

    if (result.changedLeaveTypeId > 0)
        this.leaveTypeListTableController.temporaryHighlightRow(result.changedLeaveTypeId);

    this.leaveTypeLightBox.setNewLeaveTypeDefaultIcon(result.newLeaveTypeIconId);

    if (customSuccessHandler)
        return customSuccessHandler(result);
};

LeaveTypeListPageController.prototype.leaveTypeServiceErrorHandler = function ( jqXHR, textStatus, errorThrown, customErrorHandler ) {
    if (customErrorHandler)
        return customErrorHandler(jqXHR, textStatus, errorThrown);
};

LeaveTypeListPageController.prototype.callLeaveTypeServiceMethod = function ( method, params, showSpinner, customSuccessHandler, customErrorHandler )
{
    var self = this;
    if (showSpinner)
        this.spinnerController.spin("spinnerPlaceholder");
    makeRpcCall(    method, params,
                    function (result)
                    {
                        self.spinnerController.stop();
                        self.leaveTypeServiceResultHandler(result, customSuccessHandler);
                    },
                    function( jqXHR, textStatus, errorThrown )
                    {
                        self.spinnerController.stop();
                        return self.leaveTypeServiceErrorHandler(jqXHR, textStatus, errorThrown, customErrorHandler);
                    }
    );
};
