function LeaveTypeListTableController()
{
    this.constructor = LeaveTypeListTableController;
}

LeaveTypeListTableController.prototype = _.create(Table.prototype);

LeaveTypeListTableController.prototype.init = function ( conf )
{
    var that = this;

    Table.call( this, conf.tHeadEl );

    this.containerEl = conf.containerEl;
    this.leaveTypeIdRowMap = {};

    this.handlersMap = conf.handlersMap;

    this.arrowImagesPaths = conf.arrowImagesPaths;

    this.isPtoEnabled = conf.isPtoEnabled;
    this.isSickEnabled = conf.isSickEnabled;

    this.leaveTypeListTableRenderer = this.getLeaveTypeListTableRenderer().init( this );
    this.rowRenderer = new LeaveTypeListRowRenderer().init();
    this.numberSettings = conf.numberSettings;
    this.coefficientsFormatter = new NumberFormatter(this.numberSettings.decimalSeparator, this.numberSettings.digitGroupingSymbol, 2, true);

    this.iconSelectorPanel = conf.iconSelectorPanel;
    this.iconSelectorPanel.setAfterCloseCallback( function()
                                                  {
                                                      $.each( that.rows, function( i, row ){
                                                          row.removeOpenPopupClass();
                                                          $( row.rowEl ).find( ".leaveTypeImageCell" ).removeClass( "iconSelectorOpened" );
                                                      });
                                                  }
    );

    this.createBodyContent(conf.leaveTypeRecords);

    return this;
};

LeaveTypeListTableController.prototype.getLeaveTypeListTableRenderer = function () {};

LeaveTypeListTableController.prototype.createBodyContent = function ( leaveTypeRecords )
{
    if ( leaveTypeRecords.length == 0 )
        this.insertEmptyTableRow();
    else
    {
        for( var i = 0; i < leaveTypeRecords.length; i++ )
        {
            var leaveTypeRecord = leaveTypeRecords[i];
            leaveTypeRecord.position = i + 1;
            this.addRow( this.createRow( leaveTypeRecord ) );
        }
        $(this.tbody).toggleClass("oneRow", leaveTypeRecords.length == 1);
    }
};

LeaveTypeListTableController.prototype.insertEmptyTableRow = function () {
    var emptyTableRow = document.createElement("tr");

    var emptyTableRowTd = document.createElement("td");
    emptyTableRowTd.colSpan = 10;
    $(emptyTableRowTd).text(messageResource.getMessage("leave_type.list.no_leave_types"));
    emptyTableRow.appendChild(emptyTableRowTd);

    emptyTableRow.id = "emptyLeaveTypesTableRow";

    this.tbody.appendChild(emptyTableRow);
};

LeaveTypeListTableController.prototype.removeEmptyTableRow = function () {
    $("#emptyLeaveTypesTableRow").remove();
};

LeaveTypeListTableController.prototype.createRow = function ( leaveTypeRecord )
{
    var row = new LeaveTypeListTableRow().init( leaveTypeRecord );
    this.leaveTypeIdRowMap[leaveTypeRecord.id] = row;
    this.addCellsToRow(row, leaveTypeRecord);
    return row;

};

LeaveTypeListTableController.prototype.createLeaveTypeNameCell = function (leaveTypeRecord, row)
{
    var leaveTypeNameCell = new TableCell( leaveTypeRecord, row );
    leaveTypeNameCell.setRenderer( new LeaveTypeNameCellRenderer().init(this.handlersMap.changeLeaveName) );
    return leaveTypeNameCell;
};

LeaveTypeListTableController.prototype.createLeaveTypeImageCell = function (leaveTypeRecord, row)
{
    var leaveTypeImageCell = new TableCell(leaveTypeRecord, row );
    leaveTypeImageCell.setRenderer( new LeaveTypeImageCellRenderer().init(this.handlersMap.setLeaveTypeIcon, this.iconSelectorPanel) );
    return leaveTypeImageCell;
};

LeaveTypeListTableController.prototype.createLeaveTypeAffectedBalanceCell = function (leaveTypeRecord, row)
{
    var leaveTypeAffectedBalanceCell = new TableCell( leaveTypeRecord, row );
    leaveTypeAffectedBalanceCell.setRenderer( new LeaveTypeAffectedBalanceCellRenderer().init(this.handlersMap.changeAffectedBalance, this.numberSettings, this.coefficientsFormatter, this.isPtoEnabled, this.isSickEnabled ) );
    return leaveTypeAffectedBalanceCell;
};

LeaveTypeListTableController.prototype.createLeaveTypeChangeOrderCell = function (leaveTypeRecord, row)
{
    var leaveTypeChangeOrderCell = new TableCell( leaveTypeRecord, row );
    leaveTypeChangeOrderCell.setRenderer( new LeaveTypeChangeOrderCellRenderer().init(this.arrowImagesPaths, this.handlersMap.moveLeaveType) );
    return leaveTypeChangeOrderCell;
};

LeaveTypeListTableController.prototype.createLeaveTypeStatusCell = function (leaveTypeRecord, row)
{
    var leaveTypeStatusCell = new TableCell( leaveTypeRecord, row );
    leaveTypeStatusCell.setRenderer( new LeaveTypeStatusCellRenderer().init(this.handlersMap.changeLeaveTypeStatus) );
    return leaveTypeStatusCell;
};

LeaveTypeListTableController.prototype.createLeaveTypeDeleteCell = function (leaveTypeRecord, row)
{
    var leaveTypeDeleteCell = new TableCell( leaveTypeRecord, row );
    leaveTypeDeleteCell.setRenderer( new LeaveTypeDeleteCellRenderer().init( this.handlersMap.deleteLeaveType,
                                                                             this.handlersMap.deleteCellMouseOn,
                                                                             this.handlersMap.deleteCellMouseMove,
                                                                             this.handlersMap.deleteCellMouseOut ) );
    return leaveTypeDeleteCell;
};

LeaveTypeListTableController.prototype.addCellsToRow = function ( row, leaveTypeRecord )
{
};

LeaveTypeListTableController.prototype.render = function ( )
{
    this.buildDOM( this.containerEl );
    this.leaveTypeListTableRenderer.updateUI( );
};

LeaveTypeListTableController.prototype.setupTableHeaderEventHandlers = function ( )
{
    $("#affectedBalanceInfoIcon" ).click(this.handlersMap.showAffectedBalanceHeaderTooltip);
};

LeaveTypeListTableController.prototype.getBaseRowClasses = function ( rowIdx )
{
    var leaveTypeInfo = this.getRow( rowIdx ).getLeaveTypeInfo();
    return this.rowRenderer.getRowClasses( leaveTypeInfo );
};

LeaveTypeListTableController.prototype.redrawTableBody = function ( leaveTypesInfo )
{
    this.removeEmptyTableRow();
    this.deleteAllRowsFromModelAndDom();
    this.createBodyContent(leaveTypesInfo);
    this.buildModelRowsDOM(this.tbody);
    this.leaveTypeListTableRenderer.updateUI();
};

LeaveTypeListTableController.prototype.temporaryHighlightRow = function ( leaveTypeId )
{
    var row = this.leaveTypeIdRowMap[leaveTypeId];
    row.highlight();
    setTimeout( function () { row.unhighlight() }, 1000 );
};

LeaveTypeListTableController.prototype.temporaryMarkRow = function ( leaveTypeId )
{
    var row = this.leaveTypeIdRowMap[leaveTypeId];
    if(!row){
        return;
    }
    var changedRowIdx = row.getRowIdx();
    var previousRow;
    if(changedRowIdx > 0)
    {
        previousRow = this.getRow( changedRowIdx - 1 );
        previousRow.addClass( "previous" );
    }

    var self = this;
    this.setDisabledHover(true);
    row.markMovedRow();
    row.addClass("step1");

    setTimeout( function ()
                {
                    row.removeClass("step1");
                    row.addClass("step2");
                    if(previousRow)
                        previousRow.removeClass("previous");
                    $( document ).on( "mousemove", function ()
                    {
                        self.setDisabledHover(false);
                        row.unMarkMovedRow();
                        row.removeClass("step2");
                        $( document ).off( "mousemove" )
                    } );
                }, 1000 );
};

LeaveTypeListTableController.prototype.isDisabledHover = function(){
  return this.disabledHover;
};

LeaveTypeListTableController.prototype.setDisabledHover = function(disabledHover){
    return this.disabledHover = disabledHover;
};

