/**
 * Created by evdokimov on 29.06.2015.
 */

function TTAConfirmDialog() {}

TTAConfirmDialog.prototype = new DHTMLPopupWithMoire( undefined );

TTAConfirmDialog.prototype.init = function ( config )
{
    DHTMLPopupWithMoire.call( this, config.popupId );

    var self = this;

    this.popupId = config.popupId;
    this.popupEl = $('#' + this.popupId);
    this.optionsDiv = this.popupEl.find("div.options");
    this.today = currentUserTime().clearTime();
    this.lastDisablingTimestamp = config.lastDisablingTimestamp;
    this.spinnerController = new SpinnerController();

    this.buttonsEnabled = true;

    var hideAfterSubmitting = function () {
        self.spinnerController.stop();
        self.popupEl.removeClass("savingState");
        self.buttonsEnabled = true;
        self.hide();
    };
    var processCloseClick = function () {
        if ( self.buttonsEnabled )
            self.hide();
    };
    
    this.findElementBySubId("_closeTTConfirmDialogButton").click( processCloseClick );
    this.findElementBySubId("_cancelBtn").click( processCloseClick );
    this.findElementBySubId("_submitBtn").click(
        function () {
            if ( self.buttonsEnabled )
            {
                self.spinnerController.spinInElement( self.popupEl.find( ".spinnerPlaceholder" ) );
                self.popupEl.addClass( "savingState" );
                self.buttonsEnabled = false;
                config.submitFunction( self.isApproveAllTT(), self.getTuesdayOfWeek(), hideAfterSubmitting );
            }
        }
    );
    
    this.hideOnMouseDown = function() { return false; };
    this.hideOnResize = function() { return false; };

    if ( !this.lastDisablingTimestamp )
        this.initFirstEnabling();
    else
        this.initOptions();
    
    return this;
};

TTAConfirmDialog.prototype.findElementBySubId = function( element )
{
    return $('#' + this.popupId + element);
};

TTAConfirmDialog.prototype.initOptions = function() {
    var self = this;
    
    this.approveTTOptionsName = "input[name='" + this.popupId + "_ApproveTTOption" + "']:checked";

    this.radioContainer = [];
    this.radioContainer[0] = this.findElementBySubId("_optionContainer1");
    this.radioContainer[1] = this.findElementBySubId("_optionContainer2");

    this.findElementBySubId("_ApproveAllTTOption").change( function() { self.selectRadioContainer(0); self.optionsDiv.removeClass("keepCurrentOptionSelected"); } );
    this.findElementBySubId("_KeepApprovalsOption").change( function() { self.selectRadioContainer(1); self.optionsDiv.addClass("keepCurrentOptionSelected"); } );

    this.documentCheckbox1 = document.getElementById(this.popupId + "_ApproveAllTTOption");
    this.documentCheckbox2 = document.getElementById(this.popupId + "_KeepApprovalsOption");

    this.radioContainer[0].click( function() {self.documentCheckbox1.click();} );
    this.radioContainer[1].click( function() {self.documentCheckbox2.click();} );

    this.fromDateSelector = document.getElementById(this.popupId + "_fromDateSelector2");
    this.initCalendar();
};

TTAConfirmDialog.prototype.initFirstEnabling = function() {
    this.popupEl.addClass("noOptions");
    this.fromDateSelector = document.getElementById(this.popupId + "_fromDateSelector1");
    this.initCalendar();
};

TTAConfirmDialog.prototype.initCalendar = function()
{
    $(this.fromDateSelector).html("");
    
    disableTextSelection(this.fromDateSelector);
    
    var weekStartDay = SystemSettings.weekStartDay - 1;
    var scheduleCalculator = new CorporateScheduleCalculator().init(invertedDaysDateMap, SystemSettings.defaultWorkSchedule);
    var currentWeek = this.today.getWeekStartDate( weekStartDay );
    var minDate = new Date(1970, 0, 5);
    
    this.dateSelector = new WeekSelectorWithRange( {
                                                       minValue: minDate,
                                                       minText: messageResource.getMessage("tta_feature_confirm_dialog.min_date_default_description"), 
                                                       maxValue: currentWeek,
                                                       maxText: messageResource.getMessage("tta_feature_confirm_dialog.max_date_default_description"),
                                                       weekStartDay: weekStartDay,
                                                       renderTo: this.fromDateSelector, 
                                                       scheduleCalculator: scheduleCalculator, 
                                                       date: currentWeek,
                                                       scrollableContainer: window, 
                                                       fromDateFormat: new DateFormat('MMM dd, yyyy')
                                                  } );
};

TTAConfirmDialog.prototype.getTuesdayOfWeek = function()
{
    var date = this.dateSelector.getValue();
    var dayOnWeek = date.getDay();
    var diff = ( dayOnWeek <= 2 ) ? 2 - dayOnWeek : 7 + 2 - dayOnWeek;
    
    return ( diff == 0 ) ? date : date.getNDaysAfter(diff);
};

TTAConfirmDialog.prototype.isApproveAllTT = function()
{
    var self = this;
    if ( !this.lastDisablingTimestamp )
        return true;
    
    return $(self.approveTTOptionsName).val();
};

TTAConfirmDialog.prototype.selectRadioContainer = function( id )
{
    for ( var i = 0; i < this.radioContainer.length; i++ ) {
        this.radioContainer[i].removeClass("checked");
    }
    this.radioContainer[id].addClass("checked");
};

TTAConfirmDialog.prototype.show = function ()
{
    DHTMLPopupWithMoire.prototype.show.call( this );
};

TTAConfirmDialog.prototype.hide = function ()
{
    DHTMLPopupWithMoire.prototype.hide.call( this );
    if( this._requestObject )
        this._requestObject.abort();
};