/*
    requires /js/at.js to be included
*/

function AccountInformationSection()
{
}

AccountInformationSection.NEW_USER_GROUP_EMPTY = "newUserGroupEmpty";
AccountInformationSection.NEW_TIME_ZONE_GROUP_EMPTY = "newTimeZoneGroupEmpty";
AccountInformationSection.DUPLICATE_TIME_ZONE_GROUP = "duplicateTimeZoneGroupError";
AccountInformationSection.CITY_DATA_EMPTY = "cityDataEmpty";
AccountInformationSection.HIRE_DATE_EMPTY = "hireDateEmpty";
AccountInformationSection.HIRE_DATE_IN_FUTURE_FOR_SELF = "hireDateInFutureForSelf";
AccountInformationSection.GROUP_NEW_OR_NOT_KNOWN = -2;

AccountInformationSection.prototype = new BaseUserInformationSection();

AccountInformationSection.prototype.init = function( lightBoxEl, userManagementData, currentTime, userId, accountSettings, changesCallback, validateCallback,
                                                     checkUsernameErrorHandler, corporateSchedule, userGroupsEnabled, timeZoneGroupsEnabled, firstTTDate,
                                                     firstLeaveRequestDate, editedUserIsSelf, handlerIfNoSuchUserAfterAjax, specificVersionDrawerConstructor )
{
    this.isUGREnabled = userGroupsEnabled;
    this.isTZGEnabled = timeZoneGroupsEnabled;
    var self =this;
    var sectionId = "accountInformationSection";
    var userData = {
        firstName: accountSettings.firstName,
        lastName: accountSettings.lastName,
        middleName: accountSettings.middleInitial,
        email: accountSettings.email,
        passwordData: accountSettings.passwordData,
        currentTime: currentTime
    };

    BaseUserInformationSection.prototype.init.call(this, lightBoxEl, sectionId, userData, changesCallback, validateCallback, false );

    this.specificFieldsDrawerController = specificVersionDrawerConstructor.init(
               {
                    getElementByIdFunction: function ( subId ) {return self.getElementBySubId( subId )},
                    changeDetectedFunction: function ( notRequireConfirmationOnCopySettings ) {self.changesDetected( notRequireConfirmationOnCopySettings )},
                    updateUICallback: function (errors) {$.extend(self.errors, errors); self.updateUI()},
                    validationDetectedCallback: function () {self.validationDetected()},
                    handlerIfNoSuchUserAfterAjax: handlerIfNoSuchUserAfterAjax
                }
    );
    this.specificFieldsDrawerController.initFields({accountSettings : accountSettings,
                                         currentTime: currentTime, editedUserIsSelf: editedUserIsSelf} );
    this.userId = userId;
    this.corporateSchedule = corporateSchedule;

    this.newUserGroupNameField = this.$subId("newUserGroupNameField");
    this.newTimeZoneGroupNameField = this.$subId("newTimeZoneGroupNameField");

    this.newUserGroupNameField.off();
    this.setValue( this.newUserGroupNameField, "" );

    this.newTimeZoneGroupNameField.off();
    this.setValue( this.newTimeZoneGroupNameField, "" );

    this.savedPasswordData = accountSettings.passwordData;

    this.newUserGroupNameField.attr("placeholder", messageResource.getMessage("user.add.js.user_group_selector.new_user_group_place_holder")).placeholder();
    this.newTimeZoneGroupNameField.attr("placeholder", messageResource.getMessage("user.add.js.time_zone_group_selector.new_time_zone_group_placeholder")).placeholder();

    this.checkUsernameErrorHandler = checkUsernameErrorHandler;

    this.newUserGroupNameField.blur( this.onBlurFieldHandler( function () { self._checkNewUserGroupNameValid( true ); } ) );
    this.newTimeZoneGroupNameField.blur( this.onBlurFieldHandler( function () { self._checkNewTimeZoneGroupNameValid( true ); } ) );
    this.newTimeZoneGroupNameField.on("change",
                                      function()
                                      {
                                          var value = trim(this.value);
                                          if (this.prevValue != value)
                                          {
                                              self.setDuplicateTimeZoneGroupError(false);
                                          }
                                          this.prevValue = value;
                                      });

    this.userGroupSelectorPlaceholderEl = this.getElementBySubId("userGroupSelectorPlaceholder");
    this.userGroupSelectorPlaceholderEl.innerHTML = "";

    this.timeZoneGroupSelectorPlaceholderEl = this.getElementBySubId("timeZoneGroupSelectorPlaceholder");
    this.timeZoneGroupSelectorPlaceholderEl.innerHTML = "";

    this.timeZonesSelectorPlaceholderEl = this.getElementBySubId("timeZonesSelectorPlaceholder");
    this.timeZonesSelectorPlaceholderEl.innerHTML = "";

    this.newUserGroupNameDiv = this.getElementBySubId("newUserGroupNameDiv");

    this.userGroupSelector = null;
    this.timeZoneGroupSelector = null;
    this.timeZonesSelector = null;

    this.updateUserGroupsVisibility(this.isUGREnabled);
    this.updateTimeZoneGroupsVisibility(this.isTZGEnabled);

    if (this.isUGREnabled && userManagementData.allUserGroups)
    {
        this.userGroupSelector = this.createUserGroupSelector( userManagementData.allUserGroups, accountSettings.userGroupId, this.userGroupSelectorPlaceholderEl );
        this.updateNewUserGroupFieldVisibility( accountSettings.userGroupId );
    }

    if (this.isTZGEnabled && userManagementData.allTimeZoneGroups)
    {
        this.timeZoneGroupSelector = this.createTimeZoneGroupSelector( userManagementData.allTimeZoneGroups, accountSettings.timeZoneGroupId, this.timeZoneGroupSelectorPlaceholderEl );
        this.updateTimeZoneSelectorVisibility( accountSettings.timeZoneGroupId );
        this.timeZonesSelector = this.createTimeZonesSelector(this.lightBoxEl.id + "_timeZonesSelectorPlaceholder");

        $(lightBoxEl).find(".innerContent")
                .off("scroll.userPopupTZSelector")
                .on("scroll.userPopupTZSelector", function()
                {
                    self.timeZonesSelector.cancelSearch();
                });
    }

    this.accessSelector = this.createAccessSelector( accountSettings.enabled,
                                                     this.getElementBySubId( "accessSelectorPlaceholder" ) );

    this.accessBoxRow = this.getElementBySubId("accessBoxRow");
    
    if( accountSettings.enabledInOtherProduct != null )
    {
        this.accessToOtherProductSelector = this.createAccessSelector(accountSettings.enabledInOtherProduct,
                                                                      this.getElementBySubId("accessToOtherProductSelectorPlaceholder") );
        this.getElementBySubId("accessToOtherProductSelectorLabel").style.display = "";
        $(this.accessBoxRow).removeClass("oneProduct");
    }
    else
    {
        this.accessToOtherProductSelector = null;
        this.getElementBySubId("accessToOtherProductSelectorPlaceholder").innerHTML = "";
        this.getElementBySubId("accessToOtherProductSelectorLabel").style.display = "none";
        $(this.accessBoxRow).addClass("oneProduct");
    }

    this.delReleaseDateBtn = this.getElementBySubId("delReleaseDateBtn");

    this.hireDatePlaceholderEl = this.getElementBySubId("hireDatePlaceholder");
    this.hireDatePlaceholderEl.innerHTML = "";
    this.hireDateChangedWarningEl = this.getElementBySubId("hireDateChangedWarningId");
    this.hireDateSelector = this.createDateSelector(
            userManagementData,
            "hireDate",
            this.hireDatePlaceholderEl,
            accountSettings.hireDate,
            this.corporateSchedule,
            function() {
                if (self._checkHireDateIsValid())
                {
                    self._warnAboutHireDateChanging();
                }
                self.updateUI();
            }
    );

    this.firstTTDate = firstTTDate;
    this.firstLeaveRequestDate = firstLeaveRequestDate;
    this.editedUserIsSelf = editedUserIsSelf;
    if ( !!this.editedUserIsSelf )
        $(lightBoxEl).addClass("selfEdit");
    else
        $(lightBoxEl).removeClass("selfEdit");
    this._setAccessBoxVisibility( !this.editedUserIsSelf );
    this.updateUI();
    return this;
};

AccountInformationSection.prototype.getSpecificValues = function(){
   return  this.specificFieldsDrawerController ?  this.specificFieldsDrawerController.getSpecificValues() : {};
};

AccountInformationSection.prototype.getSpecificNotCopyValues = function(){
    return  this.specificFieldsDrawerController ?  this.specificFieldsDrawerController.getNotCopyValues() : {};
};

AccountInformationSection.prototype.validationDetected = function ()
{
    if (this.specificFieldsDrawerController)  $.extend(this.errors, this.specificFieldsDrawerController.getErrors());
    BaseUserDataSection.prototype.validationDetected.call(this);
};

AccountInformationSection.prototype.createDateSelector = function( userManagementData, name, renderTo, strDate, corporateSchedule,
                                                                   onChange )
{
    var date = strDate && strDate.length > 0 ? strDate.parseJSONDate() : null;
    var scheduleCalculator = new CorporateScheduleCalculator().init( userManagementData.invertedDaysMap, corporateSchedule );

    var selector = new DateSelector({
        minValue: new Date(1970, 0, 1),
        weekStartDay: userManagementData.weekStartDay - 1,
        name: name,
        renderTo: renderTo,
        scheduleCalculator: scheduleCalculator,
        date: date,
        scrollableContainer: this.sectionEl.parentNode
    });
    var self = this;
    selector.serverHireDate = date;
    selector.previousDate = date;
    selector.on("change", function() {
        if( onChange )
            onChange();
        self.changesDetected( true ) });
    return selector;
};

AccountInformationSection.prototype.sectionWasLoaded = function()
{
    this._nameChanged();
};

AccountInformationSection.prototype.createAccessSelector = function( currentUserAccess, renderTo )
{
    renderTo.innerHTML = "";

    var self = this;
    var selector = new TumblerSwitch( currentUserAccess, function(enabled) {
            self.changesDetected( true );
        }, true);

    selector.appendAsChildTo( renderTo );

    return selector;
};

AccountInformationSection.prototype.getHireDateChangedWarning = function()
{
    if (_.isUndefined(this.hireDateChangedWarning))
    {
        var self = this;
        this.hireDateChangedWarning = new HireDateWarningDlg(
                $(this.lightBoxEl),
                $(this.lightBoxEl).find(".innerContent"),
                $(self.getElementBySubId("hireDateChangedWarningId")),
                function(isConfirmed)
                {
                    if (isConfirmed)
                    {
                        self.setPreviousHireDate(self.getHireDate());
                    } else {
                        self.setHireDate(self.getPreviousHireDate());
                    }
                });
    }
    return this.hireDateChangedWarning;
};

AccountInformationSection.prototype.createUserGroupSelector = function( userGroups, selectedGroupId, renderTo )
{
    var userGroupNotAssignedGroup = {
        name: messageResource.getMessage("user.add.js.user_group_selector.user_group_not_selected_item_title"),
        id: UserManagementConstants.USER_GROUP_NOT_ASSIGNED_ID
    };

    var newUserGroupGroup = {
        name: messageResource.getMessage("user.add.js.user_group_selector.new_user_group_item_title"),
        id: AccountInformationSection.GROUP_NEW_OR_NOT_KNOWN
    };

    var listItems = [userGroupNotAssignedGroup, newUserGroupGroup].concat( userGroups );
    var escapedListItems = [];
    $.each(listItems, function() {
                          var item = {id: this.id, name: escapeHtmlSymbols(this.name)};
                          escapedListItems.push(item);
                      });
    var selector = new ListSelector({
                         menuCls: "userGroupSelectorMenu",
                         renderTo: renderTo,
                         value: selectedGroupId,
                         listItems: escapedListItems,
                         textPropName: "name",
                         valuePropName: "id",
                         scrollableContainer: this.sectionEl.parentNode
                     });

    var self = this;

    selector.on("change", function( context, item ) {
        self.updateNewUserGroupFieldVisibility( item.id );
        self.changesDetected();
        self.validationDetected();
    });

    return selector;
};

AccountInformationSection.prototype.createTimeZoneGroupSelector = function( timeZoneGroups, selectedGroupId, renderTo )
{
    var newTimeZoneGroupGroup = {
        name: messageResource.getMessage("user.add.js.time_zone_group_selector.new_time_zone_group_item_title"),
        id: AccountInformationSection.GROUP_NEW_OR_NOT_KNOWN
    };

    var listItems = [newTimeZoneGroupGroup].concat( timeZoneGroups );
    var escapedListItems = [];
    $.each(listItems, function() {
        var item = {id: this.id, name: escapeHtmlSymbols(this.name), rawOffset: this.rawOffset};
        escapedListItems.push(item);
    });
    var selector = new TimeZoneGroupsListSelector({
                                        menuCls: "timeZoneGroupSelectorMenu",
                                        renderTo: renderTo,
                                        value: selectedGroupId,
                                        listItems: escapedListItems,
                                        textPropName: "name",
                                        valuePropName: "id",
                                        scrollableContainer: this.sectionEl.parentNode
                                    });

    var self = this;

    selector.on("change", function( context, item ) {
        self.setDuplicateTimeZoneGroupError(false);
        self.updateTimeZoneSelectorVisibility( item.id );
        self.changesDetected();
        self.validationDetected();
    });

    return selector;
};

AccountInformationSection.prototype.createTimeZonesSelector = function(renderTo)
{
    var self = this;
    return new TimeZonesSelector().init(
            renderTo,
            null,
            function(data) { self._cityChanged(data); },
            messageResource.getMessage('user.add.js.time_zone_group_selector.select_city_placeholder'),
            false,
            "userPopupSearch"
    );
};

AccountInformationSection.prototype._cityChanged = function(cityData)
{
    this.cityData = cityData;
    this.updateTimeZoneSelectorVisibility(this.timeZoneGroupSelector.getValue());
    if (this.newTimeZoneGroupNameField.pVal() == '')
        this.newTimeZoneGroupNameField.pVal(this.cityData.name.substring(0, 40));
    this._checkCityDataValid();
    this._checkNewTimeZoneGroupNameValid(true);
    this.updateUI();
    this.validationDetected();
};

AccountInformationSection.prototype._checkNewUserGroupNameValid = function( onlyRemoveEmptyError )
{
    if (this._isNewUserGroupFieldActive())
    {
        this._checkEmpty( this.newUserGroupNameField, AccountInformationSection.NEW_USER_GROUP_EMPTY, onlyRemoveEmptyError );
    }
};

AccountInformationSection.prototype.setDuplicateTimeZoneGroupError = function(value)
{
    this.errors[AccountInformationSection.DUPLICATE_TIME_ZONE_GROUP] = value;
    this.updateUI();
};

AccountInformationSection.prototype.updateTimeZoneGroups = function(timeZoneGroups)
{
    this.timeZoneGroupSelectorPlaceholderEl.innerHTML = "";
    this.timeZoneGroupSelector = this.createTimeZoneGroupSelector(
            timeZoneGroups, AccountInformationSection.GROUP_NEW_OR_NOT_KNOWN, this.timeZoneGroupSelectorPlaceholderEl );
};

AccountInformationSection.prototype._checkNewTimeZoneGroupNameValid = function( onlyRemoveEmptyError )
{
    if (this._isNewTimeZoneGroupFieldActive())
        this._checkEmpty( this.newTimeZoneGroupNameField, AccountInformationSection.NEW_TIME_ZONE_GROUP_EMPTY, onlyRemoveEmptyError );
};

AccountInformationSection.prototype._checkCityDataValid = function()
{
    this.errors[AccountInformationSection.CITY_DATA_EMPTY] = false;
    this.errors[AccountInformationSection.DUPLICATE_TIME_ZONE_GROUP] = false;

    if (this.timeZoneGroupSelector != null && this.timeZoneGroupSelector.getValue() == AccountInformationSection.GROUP_NEW_OR_NOT_KNOWN && this.cityData == null)
        this.errors[AccountInformationSection.CITY_DATA_EMPTY] = true;
};

AccountInformationSection.prototype._checkHireDateIsValid = function()
{
    this.errors[AccountInformationSection.HIRE_DATE_EMPTY] = false;
    this.errors[AccountInformationSection.HIRE_DATE_IN_FUTURE_FOR_SELF] = false;

    if( this.getHireDate() == null )
    {
        this.errors[AccountInformationSection.HIRE_DATE_EMPTY] = true;
        return false;
    }

    if( this.editedUserIsSelf && compareDates( this.getHireDate(), currentUserTime() ) > 0 )
    {
        this.errors[AccountInformationSection.HIRE_DATE_IN_FUTURE_FOR_SELF] = true;
        return false;
    }

    return true;
};

AccountInformationSection.prototype._warnAboutHireDateChanging = function()
{
    var atDataBeforeNewHireDate = this.firstTTDate && compareDates(this.firstTTDate, this.getHireDate()) < 0;
    var apDataBeforeNewHireDate = this.firstLeaveRequestDate && compareDates(this.firstLeaveRequestDate, this.getHireDate()) < 0;
    if (atDataBeforeNewHireDate || apDataBeforeNewHireDate)
    {
        var warningMsgKey;
        if (atDataBeforeNewHireDate && apDataBeforeNewHireDate)
        {
            warningMsgKey = "userlist.user_popup.hire_date_changed.integrated";
        } else if (atDataBeforeNewHireDate) {
            warningMsgKey = "userlist.user_popup.hire_date_changed.at";
        } else {
            warningMsgKey = "userlist.user_popup.hire_date_changed.ap";
        }
        this.getHireDateChangedWarning().showAt($(this.hireDatePlaceholderEl), warningMsgKey, this.getServerHireDate(), this.getHireDate());
    } else {
        this.setPreviousHireDate(this.getHireDate());
    }
};


AccountInformationSection.prototype.customValidate = function(oldErrors)
{
    $.extend( this.errors, this.specificFieldsDrawerController.validateFields( oldErrors, this.checkEmpty ) );
    this._checkNewUserGroupNameValid();
    this._checkCityDataValid();
    this._checkNewTimeZoneGroupNameValid();
    this._checkHireDateIsValid();
};

AccountInformationSection.prototype.updateUserGroupsVisibility = function(isUGREnabled)
{
    this.getElementBySubId("userGroupRow").style.display = isUGREnabled ? "block" : "none";
};

AccountInformationSection.prototype.updateTimeZoneGroupsVisibility = function(isTZGEnabled)
{
    this.getElementBySubId("timeZoneGroupRow").style.display = isTZGEnabled ? "block" : "none";
};

AccountInformationSection.prototype.updateNewUserGroupFieldVisibility = function( selectedGroupId )
{
    var showNewGroupField = this.userGroupSelector != null && selectedGroupId == AccountInformationSection.GROUP_NEW_OR_NOT_KNOWN;
    this.newUserGroupNameDiv.style.display = showNewGroupField ? "inline-block" : "none";
    $(this.getElementBySubId("userGroupRow")).toggleClass("showNewGroupField", showNewGroupField);
};

AccountInformationSection.prototype.updateTimeZoneSelectorVisibility = function( selectedGroupId )
{
    var showTimeZonesSelector = this.timeZoneGroupSelector != null && selectedGroupId == AccountInformationSection.GROUP_NEW_OR_NOT_KNOWN;
    $(this.getElementBySubId("timeZoneGroupRow")).toggleClass("showTimeZonesSelector", showTimeZonesSelector);
    this.updateNewTimeZoneGroupFieldVisibility(selectedGroupId);
};

AccountInformationSection.prototype.updateNewTimeZoneGroupFieldVisibility = function( selectedGroupId )
{
    var showTimeZonesSelector = this.timeZoneGroupSelector != null && selectedGroupId == AccountInformationSection.GROUP_NEW_OR_NOT_KNOWN;
    var showNewTimeZoneGroupField = showTimeZonesSelector && this.cityData != null;
    $(this.getElementBySubId("timeZoneGroupRow")).toggleClass("showNewTimeZoneGroupField", showNewTimeZoneGroupField);
};

AccountInformationSection.prototype._isNewUserGroupFieldActive = function()
{
    return this.userGroupSelector != null && this.newUserGroupNameDiv.style.display != "none";
};

AccountInformationSection.prototype._isNewTimeZoneDataActive = function()
{
    return this.timeZoneGroupSelector != null && this.timeZoneGroupSelector.getValue() == AccountInformationSection.GROUP_NEW_OR_NOT_KNOWN;
};

AccountInformationSection.prototype._isNewTimeZoneGroupFieldActive = function()
{
    return this._isNewTimeZoneDataActive() && this.cityData != null;
};

AccountInformationSection.prototype._setAccessBoxVisibility = function( visible )
{
    if ( visible )
        $(this.accessBoxRow).addClass("visible");
};

AccountInformationSection.prototype.getNewUserGroupName = function()
{
    if( !this.userGroupSelector )
        return null;
    if( this.userGroupSelector.getValue() != AccountInformationSection.GROUP_NEW_OR_NOT_KNOWN )
        return null;
    return this.newUserGroupNameField.pVal();
};

AccountInformationSection.prototype.getNewTimeZoneGroup = function()
{
    if (!this.timeZoneGroupSelector)
        return null;
    if( this.timeZoneGroupSelector.getValue() != AccountInformationSection.GROUP_NEW_OR_NOT_KNOWN )
        return null;
    if (!this.cityData)
        return null;
    return {
        id: -1,
        name: this.newTimeZoneGroupNameField.pVal(),
        cityPlaceId: this.cityData.placeId,
        cityName: this.cityData.name,
        timeZoneId: this.cityData.timeZoneId
    };
};

AccountInformationSection.prototype.getUserId = function()
{
    return this.userId;
};

AccountInformationSection.prototype.isEnabled = function()
{
    return this.accessSelector.isEnabled();
};

AccountInformationSection.prototype.isEnabledInOtherProduct = function()
{
    if( this.accessToOtherProductSelector )
        return this.accessToOtherProductSelector.isEnabled();
    else
        return null;
};

AccountInformationSection.prototype.getUserGroupId = function()
{
    return this.userGroupSelector ? this.userGroupSelector.getValue() : AccountInformationSection.GROUP_NEW_OR_NOT_KNOWN;
};

AccountInformationSection.prototype.getTimeZoneGroupId = function()
{
    return this.timeZoneGroupSelector ? this.timeZoneGroupSelector.getValue() : AccountInformationSection.GROUP_NEW_OR_NOT_KNOWN;
};

AccountInformationSection.prototype.getHireDate = function()
{
    return this.hireDateSelector.getValue();
};

AccountInformationSection.prototype.setHireDate = function(date)
{
    return this.hireDateSelector.setValue(date);
};


AccountInformationSection.prototype.getPreviousHireDate = function()
{
    return this.hireDateSelector.previousDate;
};

AccountInformationSection.prototype.getServerHireDate = function()
{
    return this.hireDateSelector.serverHireDate;
};

AccountInformationSection.prototype.setPreviousHireDate = function(date)
{
    this.hireDateSelector.previousDate = date;
};

AccountInformationSection.prototype.getReleaseDate = function()
{
   return null;
};

AccountInformationSection.prototype.hasErrors = function ()
{
    var ignoredErrors = [];
    if (!this._isNewUserGroupFieldActive())
    {
        ignoredErrors.push(AccountInformationSection.NEW_USER_GROUP_EMPTY);
    }
    if (!this._isNewTimeZoneDataActive())
    {
        ignoredErrors.push(AccountInformationSection.NEW_TIME_ZONE_GROUP_EMPTY);
        ignoredErrors.push(AccountInformationSection.CITY_DATA_EMPTY);
    }
    //noinspection LoopStatementThatDoesntLoopJS
    for( var prop in this.errors )
    {
        if( this.errors[prop] && ignoredErrors.indexOf(prop) < 0 )
        {
            return true;
        }
    }
    return false;
};

AccountInformationSection.prototype.getErrorState = function(  )
{
    var errors = jQuery.extend({}, this.errors);
    errors[AccountInformationSection.NEW_USER_GROUP_EMPTY] = false;
    errors[AccountInformationSection.NEW_TIME_ZONE_GROUP_EMPTY] = false;
    errors[AccountInformationSection.CITY_DATA_EMPTY] = false;
    return errors;
};

AccountInformationSection.prototype.setErrorState = function( errors )
{
    this.errors = errors;
    this.updateUI();
};

AccountInformationSection.prototype.setFocusIntoFirstNameField = function()
{
    setFocusIntoFieldWithPlaceholder( this.firstNameField );
};

AccountInformationSection.prototype.setToDuplicateUsernameState = function(){
    if(this.specificFieldsDrawerController)
    {
        this.specificFieldsDrawerController.userNameDrawer._checkUsernameDuplicatesCallback( true );
    }
};