function BaseUserInformationSection()
{
}

BaseUserInformationSection.FIRST_NAME_EMPTY = "firstNameEmpty";
BaseUserInformationSection.LAST_NAME_EMPTY = "lastNameEmpty";
BaseUserInformationSection.EMAIL_EMPTY = "emailEmpty";
BaseUserInformationSection.EMAIL_INVALID = "emailInvalid";

//if email not valid in Java
BaseUserInformationSection.JAVA_EMAIL_INVALID = "javaEmailInvalid";

BaseUserInformationSection.prototype = new BaseUserDataSection();

BaseUserInformationSection.prototype.init = function(lightBoxEl, sectionId, userData, changesCallback, validateCallback, isInstantValidation)
{
    BaseUserDataSection.prototype.init.call(this, lightBoxEl, changesCallback, validateCallback);

    this._isInstantValidation = isInstantValidation;

    this.sectionEl = this.getElementBySubId( sectionId );
    this.firstNameField = this.$subId("firstNameField");
    this.lastNameField = this.$subId("lastNameField");
    this.middleNameField = this.$subId("middleNameField");
    this.emailField = this.$subId("emailField");

    this.firstNameField.off();
    this.lastNameField.off();
    this.middleNameField.off();
    this.emailField.off();

    this.setValue( this.firstNameField, userData.firstName );
    this.setValue( this.lastNameField, userData.lastName );
    this.setValue( this.middleNameField, userData.middleName );
    this.setValue( this.emailField, userData.email );


    this.firstNameField.placeholder();
    this.lastNameField.placeholder();
    this.middleNameField.placeholder();
    this.emailField.placeholder();

    this.errors = {};

    var self = this;

    var trimOnBlurHandler = function()
    {
        $(this).pVal( trim(this.value) );
        //see Bug 48539
        if ( this.keyPressed ) keyUpChangesHandler.call(this);
    };

    var keyUpChangesHandler = function()
    {
        this.keyPressed = false;
        if( (!this.prevValue && !!this.value) || this.prevValue != this.value )
            self.changesDetected( true );
    };

    this.onKeyPressedHandler = function () {
        this.keyPressed = true;
    };

    this.onBlurFieldHandler = function(validator){
        return function()
        {
            $(this).pVal( trim(this.value) );
            //see Bug 48539
            if ( this.keyPressed ) keyUpChangesHandler.call(this);

            validator();
            self.updateUI();
            self.validationDetected();
        };
    };



    this.firstNameField.keydown( this.onKeyPressedHandler );
    this.firstNameField.blur( this.onBlurFieldHandler( function () { self._checkFirstNameValid( true ); } ) );
    this.firstNameField.keyup( keyUpChangesHandler );

    this.lastNameField.keydown( this.onKeyPressedHandler );
    this.lastNameField.blur( this.onBlurFieldHandler( function () { self._checkLastNameValid( true ); } ) );
    this.lastNameField.keyup( keyUpChangesHandler );

    this.middleNameField.keydown( this.onKeyPressedHandler );
    this.middleNameField.blur( trimOnBlurHandler );
    this.middleNameField.keyup( keyUpChangesHandler );

    this.emailField.keydown( this.onKeyPressedHandler );
    this.emailField.blur( this.onBlurFieldHandler( function () { self._checkEmailValid( true, self.errors ); } ) );
    this.emailField.keyup( keyUpChangesHandler );
    this.emailField.keyup( function(){ self.errors[BaseUserInformationSection.EMAIL_INVALID] = false;} );

    this.updateUI();
    return this;
};

BaseUserInformationSection.prototype.$subId = function( subId )
{
    return $( this.getElementBySubId( subId ) );
};

BaseUserInformationSection.prototype.setValue = function( field, value )
{
    field.removeClass('placeholder'); // IE8 doesn't switch placeholder class when field value is set
    field[0].value = value;
    field[0].prevValue = field[0].value;
};

BaseUserInformationSection.prototype.setUpNameCallback = function( nameCallback )
{
    if( !nameCallback )
        return;

    this._nameChangedCallback = nameCallback;
    var self = this;
    var keyUpHandler = function() { this.keyPressed = false; self.nameKeyUpHandler( this ); };

    this.firstNameField.keyup( keyUpHandler );
    this.firstNameField.blur( keyUpHandler );

    this.lastNameField.keyup( keyUpHandler );
    this.lastNameField.blur( keyUpHandler );

    this.middleNameField.keyup( keyUpHandler );
    this.middleNameField.blur( keyUpHandler );
};


BaseUserInformationSection.prototype.nameKeyUpHandler = function( field )
{
    var $field = $(field);
    if( !field.prevValue || field.prevValue != $field.pVal() )
    {
        field.prevValue = $field.pVal();
        this._nameChanged();
    }
};

BaseUserInformationSection.prototype._nameChanged = function()
{
    this._nameChangedCallback( this.firstNameField.pVal(), this.lastNameField.pVal(), this.middleNameField.pVal() );
};

BaseUserInformationSection.prototype._isEmailValid = function( email )
{
    return email.length == 0 || isEmailValid ( email );
};

BaseUserInformationSection.prototype._checkEmailValid = function(onlyRemoveEmptyError, oldErrors )
{
    this.errors[BaseUserInformationSection.EMAIL_INVALID] = false;

    if ( !this._checkEmpty( this.emailField, BaseUserInformationSection.EMAIL_EMPTY, onlyRemoveEmptyError ) )
        this.errors[ BaseUserInformationSection.EMAIL_INVALID ] = !this._isEmailValid( this.getEmail() ) || oldErrors[BaseUserInformationSection.EMAIL_INVALID];

    return !this.errors[BaseUserInformationSection.EMAIL_INVALID];
};

BaseUserInformationSection.prototype.setEmailInvalid = function(){
    this.errors[ BaseUserInformationSection.EMAIL_INVALID ] = true;
    this.validate();
    this.updateUI();
};

BaseUserInformationSection.prototype._checkFirstNameValid = function( onlyRemoveEmptyError )
{
    this._checkEmpty( this.firstNameField, BaseUserInformationSection.FIRST_NAME_EMPTY, onlyRemoveEmptyError );
};

BaseUserInformationSection.prototype._checkLastNameValid = function( onlyRemoveEmptyError )
{
    this._checkEmpty( this.lastNameField, BaseUserInformationSection.LAST_NAME_EMPTY, onlyRemoveEmptyError );
};

BaseUserInformationSection.prototype._checkEmpty = function ( field, errorCode, onlyRemoveEmptyError )
{
    if( field.pVal().length == 0 )
    {
        if( !onlyRemoveEmptyError || this._isInstantValidation )
        {
            this.errors[ errorCode ] = true;
        }
    }
    else
    {
        this.errors[ errorCode ] = false;
    }
    return this.errors[ errorCode ];
};



BaseUserInformationSection.prototype.validate = function()
{
    var oldErrors = this.errors;
    this.errors = {};
    this._checkFirstNameValid();
    this._checkLastNameValid();
    this._checkEmailValid(false,oldErrors);
    this.customValidate(oldErrors);

    this.updateUI();
    return !this.hasErrors();
};

BaseUserInformationSection.prototype.customValidate = function( oldErrors )
{
};

BaseUserInformationSection.prototype.updateUI = function()
{
    var classes = [];
    for( var p in this.errors )
        if( this.errors[p] )
            classes.push( p );
    this.sectionEl.className = classes.join(" ");
};

BaseUserInformationSection.prototype.getFirstName = function()
{
    return this.firstNameField.pVal();
};

BaseUserInformationSection.prototype.getLastName = function()
{
    return this.lastNameField.pVal();
};

BaseUserInformationSection.prototype.getMiddleInitial = function()
{
    return this.middleNameField.pVal();
};

BaseUserInformationSection.prototype.getEmail = function()
{
    return this.emailField.pVal();
};

BaseUserInformationSection.prototype.getSectionElement = function(  )
{
    return this.sectionEl;
};


