function HireDateWarningDlg($lightBoxEl, $scrollableLightboxContent, $warningDiv, onConfirmCallback)
{
    this.$lightBoxEl = $lightBoxEl;
    this.$scrollableLightboxContent = $scrollableLightboxContent;
    this.$warningDiv = $warningDiv;
    this.onConfirmCallback = onConfirmCallback;
    this.isIE8 = isIE8();
    this.$overlay = this._createOverlayIfNeededAndGet($lightBoxEl);
}



HireDateWarningDlg.prototype = {
    _createOverlayIfNeededAndGet: function($element)
    {
        var $overlay = $element.find(".hireDateChangedOverlay");
        if ($overlay.length == 0)
        {
            $overlay = $("<div>").addClass("hireDateChangedOverlay");
            $overlay.css({"display": "none", "position": "absolute", "top": 0, "height" : "100%"});
            $overlay.width($element.width());
            $element.append($overlay);
        }
        return $overlay;
    },

    close: function(isConfirmed)
    {
        this.onConfirmCallback(isConfirmed);
        this.hide();
    },

    showAt: function(selector, warningMsgKey, dateFrom, dateTo)
    {
        var self = this;
        KeyDownHandler.setOnKeyDownHandler(_.bind(this._keydownHandler, this));
        this.$overlay.css("display", "");

        this.$lightBoxEl.off();
        this.$lightBoxEl.on("wheel", function(event) {preventDefault(event)}); // modern standard
        this.$lightBoxEl.on("mousewheel", function(event) {preventDefault(event)}); // older browsers, IE
        this.$lightBoxEl.on("touchmove", function(event) {preventDefault(event)}); // mobile
        this.$lightBoxEl.on("mousedown", function(event) {return self._preventMouseMiddleButtonPress(event)});

        this._renderWarningContent(this.$warningDiv, warningMsgKey, dateFrom, dateTo);

        this.$warningDiv.focus();
        this.$warningDiv.show();

        if (this.isIE8)
        {
            this.$scrollableLightboxContent.css("overflow-y", "hidden");
        }
    },
    
    hide: function()
    {
        KeyDownHandler.restoreOnKeyDownHandler();
        this.$overlay.css("display", "none");
        this.$lightBoxEl.off();
        this.$warningDiv.hide();

        if (this.isIE8)
        {
            this.$scrollableLightboxContent.css("overflow-y", "");
        }
        
    },

    _preventMouseMiddleButtonPress: function(event)
    {
        if (whichButtonPressed(event) == "middle")
        {
            this.preventDefault(event);
            return false;
        }
        return true;
    },

    _renderWarningContent: function(warningDiv, warningMsgKey, fromDate, toDate)
    {
        warningDiv.empty();

        var balancesChangedMsg = (EnabledFeatures.PTO && EnabledFeatures.SDB) ? messageResource.getMessage("userlist.user_popup.hire_date_changed.balances") : "";

        warningDiv.append("<div class='warningMsgContainer'>" +
                          "<div class='msgContent'><span class='warningMsg'>" + messageResource.getMessage( warningMsgKey, [
                              new DateFormat(SystemSettings.dateFormat).formatDate( fromDate ),
                              new DateFormat(SystemSettings.dateFormat).formatDate( toDate )]).concat(balancesChangedMsg) +
                          "</span></div> </div>");
        var footer = $("<div class='warningFooter'></div>");
        var confirmBtn = $("<button class='hireDateConfirmBtn'>" + messageResource.getMessage("common.ok") + "</button>")
        var cancelBtn = $("<button class='hireDateCancelBtn'>" + messageResource.getMessage("common.cancel") + "</button>")
        footer.append(confirmBtn);
        footer.append(cancelBtn);
        warningDiv.append(footer);

        confirmBtn.click(_.bind(HireDateWarningDlg.prototype.close, this, true));
        cancelBtn.click(_.bind(HireDateWarningDlg.prototype.close, this, false));
    },

    _keydownHandler: function(event)
    {
        switch (event.keyCode)
        {
            case KEY_CODE_ESC:
                this.close(false);
                break;
            case KEY_CODE_ENTER:
                this.close(true);
                break;
        }
        cancelBubble(event);
        return false;
    }
};