function PermissionsSection()
{
}

PermissionsSection.prototype = new BaseUserDataSection();

PermissionsSection.sectionShownFirstTime = true;

PermissionsSection.prototype.init = function(lightBoxEl, userManagementData, initialAccessRights, changesCallback, editedUserIsLoggedIn, validateCallback)
{
    BaseUserDataSection.prototype.init.call( this, lightBoxEl, changesCallback, validateCallback );

    var newFeatures = userManagementData.viewerSettings.enabledFeatures;

    this._findSectionElements();
    this._createRightToChBoxMap();
    this._setEventOnChBoxes();
    this._setGenerateTimeAndMoneyChBoxesText( newFeatures );
    this._setAvailableReportsNote( newFeatures );
    this.manageGeneralSettingsNote.title=this._createManageGeneralSettingsToolTip( newFeatures );

    if( PermissionsSection.sectionShownFirstTime ){
        this._makeUnselectableChBoxLabels();
        PermissionsSection.sectionShownFirstTime=false;
    }

    var rightMap = this.createRightsMapFromArray(initialAccessRights);
    this._preselectChBoxes(rightMap);

    this._hideChBoxesAccordingToFeatures(newFeatures);

    this.editedUserIsLoggedIn = editedUserIsLoggedIn;

    if(editedUserIsLoggedIn){
        this.sectionEl.className="permissionsTemplatesEditPopup editedUserIsLoggedIn";
        this.manageAccountsChBox.disabled=true;
    }
    else{
        this.sectionEl.className="permissionsTemplatesEditPopup";
        this.manageAccountsChBox.disabled=false;
    }

    return this;
};

PermissionsSection.prototype.updateCheckboxesByArrayOfPermissions = function(permissions, withoutChangeDetecting)
{
    var permMap = this.createRightsMapFromArray(permissions);

    var rightToChBoxMap = this.rightToChBoxMap;
    for (var rightCode in rightToChBoxMap)
    {
        var isChecked = permMap.hasOwnProperty(rightCode);
        rightToChBoxMap[rightCode].checked = isChecked;
        this._onRightChanged(rightCode, isChecked, withoutChangeDetecting);
    }
};


PermissionsSection.prototype._createManageGeneralSettingsToolTip=function (newFeaturesMap){

    var hasBillingTypesFeature = newFeaturesMap.hasOwnProperty(UserManagementConstants.BTYFeature);
    var hasLeaveTypesFeature = newFeaturesMap.hasOwnProperty(UserManagementConstants.LVTFeature);
    var isPROVersion = newFeaturesMap.hasOwnProperty(UserManagementConstants.CLTFeature);

    var billingTypesText = hasBillingTypesFeature ? messageResource.getMessage("userlist.userdata.settings.billingType") + " " : "";
    var leaveTypesText = hasLeaveTypesFeature && isPROVersion ? messageResource.getMessage("userlist.userdata.settings.leaveType") + " " : "";
    var notificationsText = isPROVersion ? messageResource.getMessage("userlist.userdata.settings.notifications") + " " : "";

    return messageResource.getMessage("userlist.userdata.settings.note",[billingTypesText,leaveTypesText,notificationsText]);
};

PermissionsSection.prototype._hideChBoxesAccordingToFeatures=function (newFeaturesMap){

    if(newFeaturesMap.hasOwnProperty(UserManagementConstants.URAFeature) || newFeaturesMap.hasOwnProperty(UserManagementConstants.BTYFeature))
        this.generateMoneyReportRightContainer.style.visibility="visible";
    else
        this.generateMoneyReportRightContainer.style.visibility="hidden";

    if (newFeaturesMap.hasOwnProperty(UserManagementConstants.PTOFeature) || newFeaturesMap.hasOwnProperty(UserManagementConstants.SDBFeature) )
        this.ptoRightContainer.style.display="table-row";
    else
        this.ptoRightContainer.style.display="none";

    if (newFeaturesMap.hasOwnProperty(UserManagementConstants.LTTFeature))
        this.lockTimeTrackContainer.style.display="table-row";
    else
        this.lockTimeTrackContainer.style.display="none";
};

PermissionsSection.prototype._findSectionElements=function (){
    this.sectionEl = this.getElementBySubId( "permissionsSection" );

    this.manageGeneralSettingsNote = this.getElementBySubId( "manageGeneralSettingsNote" );

    this.submitTimeTrackChBox = this.getElementBySubId( "submitTimeTrackChBox" );
    this.submitOtherUserTimeTrackChBox = this.getElementBySubId( "submitOtherUserTimeTrackChBox" );
    this.lockTimeTrackChBox = this.getElementBySubId( "lockTimeTrackChBox" );
    this.lockTimeTrackContainer = this.getElementBySubId( "lockTTRightContainer" );

    this.manageCustomerProjectChBox = this.getElementBySubId( "manageCustomersProjectsChBox" );
    this.manageTaskChBox = this.getElementBySubId( "manageTasksChBox" );

    this.generateTimeReportsChBox = this.getElementBySubId( "generateTimeReportsChBox" );
    this.generateTimeReportsLabel = this.getElementBySubId( "generateTimeReportsLabel" );
    this.generateMoneyReportsChBox = this.getElementBySubId( "generateMoneyReportsChBox" );
    this.generateMoneyReportsLabel = this.getElementBySubId( "generateMoneyReportsLabel" );

    this.managePTOAndSickChBox = this.getElementBySubId( "ptoSettingsChBox" );
    this.manageAccountsChBox = this.getElementBySubId( "manageAccountsChBox" );
    this.manageGeneralSettingsChBox = this.getElementBySubId( "manageGeneralSettingsChBox" );

    this.availableTimeReportsNote = this.getElementBySubId( "availableTimeReportsNote" );
    this.availableMoneyReportsNote = this.getElementBySubId( "availableMoneyReportsNote" );
    this.generateMoneyReportRightContainer = this.getElementBySubId( "generateMoneyReportRightContainer" );
    this.cantRevokeManageUsersRightNote = this.getElementBySubId( "cantRevokeManageUsersRightNote" );
    this.ptoRightContainer = this.getElementBySubId( "ptoRightContainer" );
};

PermissionsSection.prototype._createRightToChBoxMap=function (){

    this.rightToChBoxMap={};

    this.rightToChBoxMap[AccessRight.SUBMIT_TIME_TRACK]               =     this.submitTimeTrackChBox;
    this.rightToChBoxMap[AccessRight.SUBMIT_OTHER_USER_TIME_TRACK]    =     this.submitOtherUserTimeTrackChBox;
    this.rightToChBoxMap[AccessRight.LOCK_TIME_TRACK]                 =     this.lockTimeTrackChBox;

    this.rightToChBoxMap[AccessRight.MANAGE_CUSTOMERS_PROJECTS]       =     this.manageCustomerProjectChBox;
    this.rightToChBoxMap[AccessRight.MANAGE_TASKS]                    =     this.manageTaskChBox;

    this.rightToChBoxMap[AccessRight.GENERATE_TIME_REPORTS]           =     this.generateTimeReportsChBox;
    this.rightToChBoxMap[AccessRight.GENERATE_MONEY_REPORTS]          =     this.generateMoneyReportsChBox;

    this.rightToChBoxMap[AccessRight.MANAGE_PTO_AND_SICK]             =     this.managePTOAndSickChBox;
    this.rightToChBoxMap[AccessRight.MANAGE_USERS]                    =     this.manageAccountsChBox;
    this.rightToChBoxMap[AccessRight.MANAGE_GENERAL_SETTINGS]         =     this.manageGeneralSettingsChBox;
};

PermissionsSection.prototype._setEventOnChBoxes=function (){
    var rightToChBoxMap = this.rightToChBoxMap;
    var self = this;
    for(var rightCode in rightToChBoxMap){
        (function ( rightCode )
        {
            var chBoxElement = rightToChBoxMap[rightCode];
            //don't use onchange , IE8 not stable
            chBoxElement.onclick = function () {
                self._onRightChanged(rightCode, rightToChBoxMap[rightCode].checked, false);
            };
        }( rightCode ));
    }
};

PermissionsSection.prototype.removeChangeDetectingAndEnableManageUsersChBox = function()
{
    var self = this;
    var rightToChBoxMap = this.rightToChBoxMap;
    for (var rightCode in rightToChBoxMap)
    {
        (function(context, rightCode)
        {
            rightToChBoxMap[rightCode].onclick = function () {
                self._onRightChanged(rightCode, rightToChBoxMap[rightCode].checked, true);
            };
            if(rightCode == AccessRight.MANAGE_USERS && context.editedUserIsLoggedIn)
            {
                $(context.sectionEl).removeClass("editedUserIsLoggedIn");
                context.manageAccountsChBox.disabled = false;
            }
        }(this, rightCode));
    }
};


PermissionsSection.prototype._makeUnselectableChBoxLabels=function (){
    var rightToChBoxMap = this.rightToChBoxMap;

    for(var rightCode in rightToChBoxMap){
        var chBoxElement = rightToChBoxMap[ rightCode ];
        this.makeUnselectable( chBoxElement.parentNode.parentNode );
    }
};

PermissionsSection.prototype._setGenerateTimeAndMoneyChBoxesText=function ( newFeatures ){
    var generateTimeReportsChBoxText = "";
    var generateMoneyReportsChBoxText = "";

    if(newFeatures.hasOwnProperty(UserManagementConstants.BTYFeature) || newFeatures.hasOwnProperty(UserManagementConstants.URAFeature))
        generateTimeReportsChBoxText = messageResource.getMessage("access_rights.generate_time_reports");
    else
        generateTimeReportsChBoxText = messageResource.getMessage("access_rights.generate_reports");

    if( newFeatures.hasOwnProperty( UserManagementConstants.BTYFeature )  && newFeatures.hasOwnProperty( UserManagementConstants.URAFeature ) )
        generateMoneyReportsChBoxText = messageResource.getMessage( "access_rights.generate_money_reports" );
    else if( newFeatures.hasOwnProperty( UserManagementConstants.BTYFeature ) )
        generateMoneyReportsChBoxText =  messageResource.getMessage( "access_rights.generate_money_reports_only_billing_plural" );
    else if( newFeatures.hasOwnProperty( UserManagementConstants.URAFeature ) )
        generateMoneyReportsChBoxText = messageResource.getMessage( "access_rights.generate_money_reports_only_cost" );

    this.generateTimeReportsLabel.innerHTML =  generateTimeReportsChBoxText;
    this.generateMoneyReportsLabel.innerHTML =  generateMoneyReportsChBoxText;

};

PermissionsSection.prototype._setAvailableReportsNote=function ( newFeatures ){

    this.availableTimeReportsNote.title= this.createAvailableByFeatureTimeReportsMsg( newFeatures );
    this.availableMoneyReportsNote.title= this.createAvailableByFeatureMoneyReportsMsg( newFeatures );
};

PermissionsSection.prototype.createAvailableByFeatureTimeReportsMsg = function ( newFeatures ){

    msg = "";

    msg+=UserManagementConstants.REPORT_TO_REPORT_NAME_MAP.STAFF_PERFORMANCE + ", " +
         UserManagementConstants.REPORT_TO_REPORT_NAME_MAP.OVERTIME + ", ";
    if( newFeatures.hasOwnProperty( UserManagementConstants.LVTFeature ) &&
        newFeatures.hasOwnProperty( UserManagementConstants.CLTFeature ) )
        msg+=UserManagementConstants.REPORT_TO_REPORT_NAME_MAP.LEAVE_TIME + ", ";
    if(newFeatures.hasOwnProperty(UserManagementConstants.TTCFeature))
        msg+=UserManagementConstants.REPORT_TO_REPORT_NAME_MAP.TIME_TRACK_IN_DETAIL + ", ";
    if(newFeatures.hasOwnProperty(UserManagementConstants.ESTFeature))
        msg+=UserManagementConstants.REPORT_TO_REPORT_NAME_MAP.ESTIMATED_VS_ACTUAL + ", ";

   return msg.substring(0, msg.length-2);
};

PermissionsSection.prototype.createAvailableByFeatureMoneyReportsMsg = function ( newFeatures ){
    msg = "";

    if(newFeatures.hasOwnProperty(UserManagementConstants.BTYFeature))
        msg+=UserManagementConstants.REPORT_TO_REPORT_NAME_MAP.BILLING_SUMMARY + ", ";
    if(newFeatures.hasOwnProperty(UserManagementConstants.BRAFeature))
        msg+=UserManagementConstants.REPORT_TO_REPORT_NAME_MAP.INVOICING + ", ";
    if(newFeatures.hasOwnProperty(UserManagementConstants.URAFeature))
        msg+=UserManagementConstants.REPORT_TO_REPORT_NAME_MAP.COST_OF_WORK + ", ";
    if(newFeatures.hasOwnProperty(UserManagementConstants.URAFeature) && newFeatures.hasOwnProperty(UserManagementConstants.BRAFeature))
        msg+=UserManagementConstants.REPORT_TO_REPORT_NAME_MAP.PROFIT_LOSS + ", ";

    return msg.substring(0, msg.length-2);
};

PermissionsSection.prototype._onRightChanged = function (rightCode, checked, withoutChangeDetecting){

    var dependent = AccessRightsUtil.getDependentRight(rightCode, checked);

    if (dependent) {
        var dependentChBox =   this.rightToChBoxMap[dependent.code];
        dependentChBox.checked = dependent.isGranted;
    }
    if (!withoutChangeDetecting)
    {
        this.changesDetected();
    }
};

PermissionsSection.prototype.getSelectedRights = function (){

    var rights = [];

    var rightToChBoxMap = this.rightToChBoxMap;

    for(var rightCode in rightToChBoxMap)
        if(rightToChBoxMap[rightCode].checked)
            rights.push( rightCode );

    return rights;
};

PermissionsSection.prototype._preselectChBoxes = function (rightMap)
{
    var rightToChBoxMap = this.rightToChBoxMap;
    for(var rightCode in rightToChBoxMap)
           rightToChBoxMap[ rightCode ].checked=rightMap.hasOwnProperty( rightCode );
};

PermissionsSection.prototype.createRightsMapFromArray = function (rightsValuesArray){

    var map = {};

    for( var i = 0; i < rightsValuesArray.length; i++ )
    {
        var rightValue = rightsValuesArray[i];
        map[rightValue]=true;
    }

    return map;
};

PermissionsSection.prototype.getFirstChBox = function()
{
    return this.rightToChBoxMap[AccessRight.SUBMIT_TIME_TRACK];
};

PermissionsSection.prototype.getLastChBox = function()
{
    return this.rightToChBoxMap[AccessRight.MANAGE_GENERAL_SETTINGS];
};

PermissionsSection.prototype.getSectionEl = function()
{
    return this.sectionEl;
};

PermissionsSection.prototype.disableCheckboxes = function()
{
    for (var rightCode in this.rightToChBoxMap)
    {
        if (this.rightToChBoxMap.hasOwnProperty(rightCode))
        {
            this.rightToChBoxMap[rightCode].disabled = true;
        }
    }
};

PermissionsSection.prototype.enableCheckboxes = function(checkAndDisableManageUsersRightIfEditedUserIsLoggedIn)
{
    for (var rightCode in this.rightToChBoxMap)
    {
        if (this.rightToChBoxMap.hasOwnProperty(rightCode))
        {
            if (checkAndDisableManageUsersRightIfEditedUserIsLoggedIn && rightCode == AccessRight.MANAGE_USERS && this.editedUserIsLoggedIn)
            {
                this.rightToChBoxMap[rightCode].disabled = true;
                this.rightToChBoxMap[rightCode].checked = true;
                $(this.sectionEl).addClass("editedUserIsLoggedIn");
            }
            else
            {
                this.rightToChBoxMap[rightCode].disabled = false;
            }
        }
    }
};