function PermissionsSectionMediator(){}

PermissionsSectionMediator.DUPLICATE_NAME_ERROR = "duplicateNameError";

PermissionsSectionMediator.prototype.init = function(config)
{
    var self = this;

    this.$lightBoxEl = $(config.lightBoxEl);
    this.permissionsSection = new PermissionsSection().init(config.lightBoxEl, config.userManagementData, this._getInitialAccessRights(config),
                                                            config.changesCallback, config.editedUserIsLoggedIn, config.validateCallback);

    this.$permissionsSectionEl = $(this.permissionsSection.getSectionEl());

    this.createAndInitPermissionTemplatesMenu(config.userManagementData.permissionsTemplates, config.forceCancelCb, config.checkLightBoxIsShown);
    this.permissionsSectionPopupFrame = new PermissionsTemplatesPopupFrame().init(this.$lightBoxEl,
                                                                                  this.$lightBoxEl.find(".innerContent"),
                                                                                  this.$permissionsSectionEl,
                                                                                  function(event) {self._keyDownCallback(event)},
                                                                                  function(event) {self._preventDefault(event)},
                                                                                  function()
                                                                                  {
                                                                                      self._cancelClickCallback();
                                                                                  });

    var $controlBlock = this.$lightBoxEl.find(".permissionTemplateControl");
    this.$savePermissionTemplateButton = $controlBlock.find(".savePermissionTemplateButton");

    this.$permissionTemplateNameInput = $controlBlock.find(".permissionTemplateNameInput");
    this.$permissionTemplateNameInput.off();
    this.$permissionTemplateNameInput.placeholder();

    var isOldIE = navigator.userAgent.toLowerCase().indexOf("msie 8") > -1 ||
                  navigator.userAgent.toLowerCase().indexOf("msie 9") > -1;

    var onTextChange = function()
    {
        self.$savePermissionTemplateButton.toggleClass("disabled", self.$permissionTemplateNameInput.pVal().trim() == "");
    };

    if (isOldIE)
    {
        this.$permissionTemplateNameInput.keyup(onTextChange);
        this.$permissionTemplateNameInput.on("propertychange", function(event)
                                                               {
                                                                   if (event.propertyName == "value")
                                                                   {
                                                                       onTextChange();
                                                                   }
                                                               });
        this.$permissionTemplateNameInput.on("cut", function()
                                                    {
                                                        setTimeout(onTextChange, 0);
                                                    });
    }
    this.$permissionTemplateNameInput.on("input", onTextChange);
    this.$permissionTemplateNameInput.on("change", function()
                                                   {
                                                       self.$permissionsSectionEl.removeClass(PermissionsSectionMediator.DUPLICATE_NAME_ERROR);
                                                   });

    this.$cancelCurrentChangesButton = $controlBlock.find(".cancelCurrentChangesButton");

    var $firstChBox = $(this.permissionsSection.getFirstChBox());
    $firstChBox.off();
    var $lastChBox = $(this.permissionsSection.getLastChBox());
    $lastChBox.off();

    this.tabCycle = new TabCycle().init();
    this.tabCycle.createSequence($lastChBox, this.$permissionTemplateNameInput, $firstChBox);

    this.savingSpinner = new SpinnerController();

    return this;
};

PermissionsSectionMediator.prototype._getInitialAccessRights = function(config)
{
    return config.accountSettings.accessRights;
};

PermissionsSectionMediator.prototype.updateTemplates = function(templates)
{
    this.permissionTemplatesMenu.updateTemplatesData(templates);
};

PermissionsSectionMediator.prototype.createAndInitPermissionTemplatesMenu = function(permissionsTemplates, forceCancelCb, checkLightBoxIsShown)
{
    var context = this;
    this.permissionTemplatesMenu = new PermissionsTemplatesMenu().init(
            "permissionTemplatesMenuId",
            permissionsTemplates,
            function() {context._showCreatePopup();},
            function(templateData) {context._showEditPopup(templateData);},
            function(templateData)
            {
                context.permissionsSection.updateCheckboxesByArrayOfPermissions(templateData.permissions, false);
                context.permissionsSection.enableCheckboxes(true);
            },
            function()
            {
                if (forceCancelCb)
                {
                    forceCancelCb();
                }
            },
            null,
            checkLightBoxIsShown
    );
    var permissionTemplatesMenuButton = $(this.permissionsSection.getElementBySubId("permissionsTemplatesMenuButton"));
    var showMenuCallback = function(event)
    {
        context.permissionTemplatesMenu.showAt(permissionTemplatesMenuButton);
        event.preventDefault();
    };
    permissionTemplatesMenuButton.off("click").on("click", showMenuCallback);
    var hideMenuOnScrollContentCallback = function()
    {
        if (context.permissionTemplatesMenu.isShown())
        {
            context.permissionTemplatesMenu.hide();
        }
    };
    this.$lightBoxEl.find(".innerContent").off("scroll", hideMenuOnScrollContentCallback).on("scroll", hideMenuOnScrollContentCallback);
};

PermissionsSectionMediator.prototype._startSaving = function()
{
    this.savingStarted = true;
    this.$permissionsSectionEl.addClass("saving");
    this.savingSpinner.spin(this.$lightBoxEl[0].id + "_savingPermissionsTemplatesSpinner");
    this.$permissionTemplateNameInput.attr("disabled", true);
    this.permissionsSection.disableCheckboxes();
};

PermissionsSectionMediator.prototype._stopSaving = function(checkAndDisableManageUsersRightIfEditedUserIsLoggedIn)
{
    this.permissionsSection.enableCheckboxes(checkAndDisableManageUsersRightIfEditedUserIsLoggedIn);

    if (!this.savingStarted)
    {
        return;
    }

    if(this.rpcCall)
    {
        this.rpcCall.abort();
    }

    this.$permissionsSectionEl.removeClass("saving");
    this.savingSpinner.stop();
    this.$permissionTemplateNameInput.removeAttr("disabled");
    this.savingStarted = false;
};


PermissionsSectionMediator.prototype._showCreatePopup = function()
{
    this._savePermissionSectionState();

    var self = this;
    this.$permissionTemplateNameInput.pVal("");
    this.$permissionTemplateNameInput.prop('disabled', false);
    this.$savePermissionTemplateButton.addClass("disabled");
    this.$cancelCurrentChangesButton.hide();
    this.$savePermissionTemplateButton.off();
    this.$savePermissionTemplateButton.click(function()
                                             {
                                                 var templateName = self.$permissionTemplateNameInput.pVal().trim();
                                                 if (templateName.length > 0)
                                                 {
                                                     self._saveAndApplyButtonCallback("PermissionsTemplatesService.createTemplate",
                                                             [templateName, self.permissionsSection.getSelectedRights()]);
                                                 }
                                                 else
                                                 {
                                                     self.$savePermissionTemplateButton.addClass("disabled");
                                                 }
                                             });
    this._showPopup();
};

PermissionsSectionMediator.prototype._showEditPopup = function(templateData)
{
    this._savePermissionSectionState();

    var self = this;
    var isDefaultTemplate = templateData.id == DEFAULT_PERMISSIONS_TEMPLATE_ID;
    var initTemplateName = templateData.name;
    var visibleTemplateName = initTemplateName +
                              (isDefaultTemplate ? " " + messageResource.getMessage('userlist.userdata.permissions_templates.js.menu.row.default_template') : '');

    this.$permissionTemplateNameInput.pVal(visibleTemplateName);
    this.$permissionTemplateNameInput.prop('disabled', isDefaultTemplate);

    this.permissionsSection.updateCheckboxesByArrayOfPermissions(templateData.permissions, true);
    this.$savePermissionTemplateButton.removeClass("disabled");

    this.$cancelCurrentChangesButton.show();
    this.$cancelCurrentChangesButton.off();
    this.$cancelCurrentChangesButton.click(function()
                                           {
                                               if (!self.savingStarted)
                                               {
                                                   self.$permissionsSectionEl.removeClass(PermissionsSectionMediator.DUPLICATE_NAME_ERROR);
                                                   self.$savePermissionTemplateButton.removeClass("disabled");
                                                   self.$permissionTemplateNameInput.pVal(visibleTemplateName);
                                                   self.permissionsSection.updateCheckboxesByArrayOfPermissions(templateData.permissions, true);
                                               }
                                           });

    this.$savePermissionTemplateButton.off();
    this.$savePermissionTemplateButton.click(function()
                                             {
                                                 var templateName = isDefaultTemplate ? initTemplateName : self.$permissionTemplateNameInput.pVal().trim();
                                                 if (templateName.length > 0)
                                                 {
                                                     self._saveAndApplyButtonCallback("PermissionsTemplatesService.updateTemplate",
                                                             [
                                                                 templateData.id,
                                                                 templateName,
                                                                 self.permissionsSection.getSelectedRights()
                                                             ]);
                                                 }
                                                 else
                                                 {
                                                     self.$savePermissionTemplateButton.addClass("disabled");
                                                 }
                                             });
    this._showPopup();
};

PermissionsSectionMediator.prototype.getPermissionsSection = function()
{
    return this.permissionsSection;
};

PermissionsSectionMediator.prototype._showPopup = function()
{
    this.$permissionsSectionEl.scrollintoview();
    this.$permissionsSectionEl.removeClass(PermissionsSectionMediator.DUPLICATE_NAME_ERROR);
    this.tabCycle.enable();
    this.permissionsSectionPopupFrame.show();
    this.permissionsSection.removeChangeDetectingAndEnableManageUsersChBox();
};

PermissionsSectionMediator.prototype._hidePopup = function()
{
    this.tabCycle.disable();
    this.permissionsSectionPopupFrame.hide();
    this.permissionsSection._setEventOnChBoxes();
};

PermissionsSectionMediator.prototype._savePermissionSectionState = function()
{
    this.prevPermissionRights = this.permissionsSection.getSelectedRights();
};

PermissionsSectionMediator.prototype._restorePermissionSectionState = function()
{
    this.permissionsSection.updateCheckboxesByArrayOfPermissions(this.prevPermissionRights, true);
};

PermissionsSectionMediator.prototype._cancelClickCallback = function()
{
    this._restorePermissionSectionState();
    this._stopSaving(true);
    this._hidePopup();
};

PermissionsSectionMediator.prototype._preventDefault = function(event)
{
    event = event || window.event;
    event.preventDefault ? event.preventDefault() : (event.returnValue = false);
};

PermissionsSectionMediator.prototype._keyDownCallback = function(event)
{
    event = event || window.event;
    if (event.keyCode == KEY_CODE_TAB)
    {
        if (_.every(this.$permissionsSectionEl.find("input"), function(input) {return !$(input).is(":focus")})) // if click is on overlay and after press TAB, focus will jump from popup
        {
            this.$permissionTemplateNameInput.focus();
            this._preventDefault(event);
            stopEventPropagation(event);
        }
    }
    else if (event.keyCode == KEY_CODE_ESC)
    {
        this._cancelClickCallback();
        this._preventDefault(event);
        stopEventPropagation(event);
    }
    else
    {
        var inputInFocus = this.$permissionTemplateNameInput.is(":focus");
        if ((!inputInFocus && event.keyCode >= KEY_CODE_SPACE && event.keyCode <= KEY_CODE_DOWN) ||
            (inputInFocus && (event.keyCode == KEY_CODE_PAGE_UP || event.keyCode == KEY_CODE_PAGE_DOWN)))
        {
            stopEventPropagation(event);
            this._preventDefault(event);
        }
    }
};

PermissionsSectionMediator.prototype._saveAndApplyButtonCallback = function(methodName, params)
{
    if (!this.$savePermissionTemplateButton.hasClass("disabled") && !this.savingStarted)
    {
        var self = this;
        this._startSaving();

        this.rpcCall = makeRpcCall(methodName, params,
                    function(result)
                    {
                        self.updateTemplates(result);
                        self._stopSaving(true);
                        self.permissionsSection.changesDetected();
                        self._hidePopup();
                    },
                    function(jqXHR, textStatus, errorThrown)
                    {
                        if(textStatus == "abort")
                        {
                            return true;
                        }
                        if (errorThrown.exception == 'com.actimind.actitime.services.PermissionsTemplateDuplicateNameException')
                        {
                            self.$permissionsSectionEl.addClass(PermissionsSectionMediator.DUPLICATE_NAME_ERROR);
                            self.updateTemplates(errorThrown.templates);
                            self._stopSaving(false);
                            return true;
                        }
                        if (errorThrown.exception == 'com.actimind.actitime.services.PermissionsTemplateNotExistException')
                        {
                            self.updateTemplates(errorThrown.templates);
                            if (confirm(messageResource.getMessage('userlist.userdata.permissions_templates.js.no_such_template_confirmation')))
                            {
                                self._stopSaving(false);
                                self._saveAndApplyButtonCallback("PermissionsTemplatesService.createTemplate", [params[1], params[2]]);
                            }
                            else
                            {
                                self._cancelClickCallback();
                            }
                            return true;
                        }
                        if (errorThrown.exception == 'com.actimind.actitime.services.NoRightsToManageAccountSettings')
                        {
                            alert(messageResource.getMessage("userlist.js.error_cant_manage_user_account"));
                            self._stopSaving(true);
                            self._hidePopup();
                            document.location.reload();
                            return true;
                        }
                        self._stopSaving(false);
                    });
    }
};