/**
 * One-off class: after #copyParams method call becomes invalid
 * @constructor
 */
CommonUserManagementSectionsBuilderParams = function () {
    this._preparatoryStrategy = null;
    this._workdayDurationsTableBuilder = null;
    this._userManagementData = null;
    this._currentTime = null;
    this._userId = null;
    this._accountSettings = null;
    this._defaultWorkSchedule = null;
    this._showPermissionsSection = false;
    this._editedUserIsLoggedIn = null;
    this._allowedToManageAccountSettings = null;
    this._firstTTDate = null;
    this._firstRequestDate = null;
    this._ratesTableStrategy = null;
};

CommonUserManagementSectionsBuilderParams.prototype.setPreparatoryStrategy = function(preparatoryStrategy) {
    this._preparatoryStrategy = preparatoryStrategy;
    return this;
}
CommonUserManagementSectionsBuilderParams.prototype.setWorkdayDurationsTableBuilder = function(workdayDurationsTableBuilder) {
    this._workdayDurationsTableBuilder = workdayDurationsTableBuilder;
    return this;
}
CommonUserManagementSectionsBuilderParams.prototype.setUserManagementData = function(userManagementData) {
    this._userManagementData = userManagementData;
    this._defaultWorkSchedule = userManagementData.defaultWorkSchedule;
    this._allowedToManageAccountSettings =  userManagementData.viewerSettings.allowedToManageAccountSettings;
    return this;
}
CommonUserManagementSectionsBuilderParams.prototype.setCurrentTime = function(currentTime) {
    this._currentTime = currentTime;
    return this;
}
CommonUserManagementSectionsBuilderParams.prototype.setUserId = function(userId) {
    this._userId = userId;
    return this;
}
CommonUserManagementSectionsBuilderParams.prototype.setAccountSettings = function(accountSettings) {
    this._accountSettings = accountSettings;
    this._showPermissionsSection = accountSettings ? true : false;
    return this;
}
CommonUserManagementSectionsBuilderParams.prototype.setEditedUserIsLoggedIn = function(editedUserIsLoggedIn) {
    this._editedUserIsLoggedIn = editedUserIsLoggedIn;
    return this;
}
CommonUserManagementSectionsBuilderParams.prototype.setFirstTTDate = function(firstTTDateStr) {
    this._firstTTDate = firstTTDateStr ? Date.parseJSONDate( firstTTDateStr ) : null;
    return this;
}
CommonUserManagementSectionsBuilderParams.prototype.setFirstRequestDate = function(firstRequestDateStr) {
    this._firstRequestDate = firstRequestDateStr ? Date.parseJSONDate( firstRequestDateStr ) : null;
    return this;
}
CommonUserManagementSectionsBuilderParams.prototype.setRatesTableStrategy = function( strategy )
{
    this._ratesTableStrategy = strategy;
    return this;
}

/**
 * This method must be called once for particular params object - can causes its invalid state.
 * @param actitimeUserManagementSectionsBuilder
 */
CommonUserManagementSectionsBuilderParams.prototype.copyParams = function(userManagementSectionsBuilder) {
    if (this.invalid) {
        throw new Error("UserManagementSectionsBuilderParams is invalid");
    }
    for(var k in this) {
        if(this.hasOwnProperty(k))
            userManagementSectionsBuilder[k] = this[k];
    }
    this.invalid = true;
}

ActitimeUserManagementSectionsBuilderParams = function() {
    this._ptoHoursPerDay = null;
}

ActitimeUserManagementSectionsBuilderParams.prototype = new CommonUserManagementSectionsBuilderParams();

ActitimeUserManagementSectionsBuilderParams.prototype.setUserManagementSectionsBuilderParams = function (commonUserManagementSectionsBuilderParams) {
    this.commonUserManagementSectionsBuilderParams = commonUserManagementSectionsBuilderParams;
    return this;
}

ActitimeUserManagementSectionsBuilderParams.prototype.setPtoHoursPerDay = function(ptoHoursPerDay) {
    this._ptoHoursPerDay = ptoHoursPerDay;
    return this;
}

ActitimeUserManagementSectionsBuilderParams.prototype.copyParams = function(actitimeUserManagementSectionsBuilder) {
    this.commonUserManagementSectionsBuilderParams.copyParams(actitimeUserManagementSectionsBuilder);
    CommonUserManagementSectionsBuilderParams.prototype.copyParams.call(this, actitimeUserManagementSectionsBuilder);
}

ActiplansUserManagementSectionsBuilderParams = function() {}

ActiplansUserManagementSectionsBuilderParams.prototype = new CommonUserManagementSectionsBuilderParams();

ActiplansUserManagementSectionsBuilderParams.prototype.setUserManagementSectionsBuilderParams = function (commonUserManagementSectionsBuilderParams) {
    this.commonUserManagementSectionsBuilderParams = commonUserManagementSectionsBuilderParams;
    return this;
}

ActiplansUserManagementSectionsBuilderParams.prototype.copyParams = function(actitimeUserManagementSectionsBuilder) {
    this.commonUserManagementSectionsBuilderParams.copyParams(actitimeUserManagementSectionsBuilder);
    delete this.commonUserManagementSectionsBuilderParams;
    CommonUserManagementSectionsBuilderParams.prototype.copyParams.call(this, actitimeUserManagementSectionsBuilder);
}

/*BASE USER MANAGEMENT SECTIONS BUILDER*/

function UserManagementSectionsBuilder (){
    this.constructor = UserManagementSectionsBuilder;
}
UserManagementSectionsBuilder.prototype.init = function (userManagementSectionsBuilderParams) {
    userManagementSectionsBuilderParams.copyParams(this);
    return this;
}

UserManagementSectionsBuilder.prototype.createSections = function (){
    //abstract
};

UserManagementSectionsBuilder.prototype.getSectionsErrorsHandler = function (){
    //abstract
};

UserManagementSectionsBuilder.prototype.getUserInputDataFetcher = function (){
    //abstract
};

UserManagementSectionsBuilder.prototype.initAfterShowing = function ( isCreateLightbox ){
    //abstract
};


/*ACTITIME USER MANAGEMENT SECTIONS BUILDER*/

function ActitimeUserManagementSectionsBuilder () {
    this.constructor = ActitimeUserManagementSectionsBuilder;
}

ActitimeUserManagementSectionsBuilder.prototype = new UserManagementSectionsBuilder();

ActitimeUserManagementSectionsBuilder.prototype.createSections = function (changesCallback, validateCallback, lightBoxEl, hideBoxBeforeReloadErrorHandler, setupNameCallBack, handlerIfNoSuchUserAfterAjax, showOvertime, forceCancelCb, checkLightBoxIsShown){
    var sections = [];
    var self = this;
    if (this._allowedToManageAccountSettings)
    {

        this._accountInformationSection =  this._preparatoryStrategy.getAccountInformationSectionConstructor().init(  lightBoxEl,
                                                                               this._userManagementData,
                                                                               this._currentTime,
                                                                               this._userId,
                                                                               this._accountSettings,
                                                                               changesCallback,
                                                                               validateCallback,
                                                                               hideBoxBeforeReloadErrorHandler,
                                                                               this._defaultWorkSchedule,
                                                                               this._userManagementData.viewerSettings.enabledFeatures.hasOwnProperty(UserManagementConstants.UGRFeature),
                                                                               this._userManagementData.viewerSettings.enabledFeatures.hasOwnProperty(UserManagementConstants.TZGFeature),
                                                                               this._firstTTDate,
                                                                               this._firstRequestDate,
                                                                               this._editedUserIsLoggedIn,
                                                                               handlerIfNoSuchUserAfterAjax,
                                                                               self._preparatoryStrategy.getVersionSpecificDrawer());

        this._workScheduleAndOvertimeSection = new WorkScheduleAndOvertimeSection().init(this._workdayDurationsTableBuilder,
                                                                                         lightBoxEl,
                                                                                         this._userManagementData,
                                                                                         this._accountSettings,
                                                                                         changesCallback,
                                                                                         validateCallback,
                                                                                         showOvertime);

        $(lightBoxEl).toggleClass("overtimeSectionOff", !showOvertime);

        sections.push(  this._accountInformationSection );
        sections.push(  this._workScheduleAndOvertimeSection );

        if( this._accountSettings.rates )
        {
            this._userRatesSection = new UserRatesSection().init(lightBoxEl,
                                                                 this._accountSettings,
                                                                 this._userManagementData,
                                                                 changesCallback,
                                                                 validateCallback,
                                                                 this._defaultWorkSchedule,
                                                                 this._currentTime,
                                                                 this._ratesTableStrategy,
                                                                 showOvertime);

            var selfUserRatesSection =  this._userRatesSection;

            this._workScheduleAndOvertimeSection.setOvertimeTrackingLevelCallback(function ( overtimeLevel ){ selfUserRatesSection.setOvertimeLevel( overtimeLevel ) });
            sections.push( this._userRatesSection );
        }
        else
        {
            UserRatesSection.hideSectionBlock( lightBoxEl );
        }

        if( this._showPermissionsSection )
        {
            this._permissionsSection = new PermissionsSectionMediator().init({
                                                                                lightBoxEl: lightBoxEl,
                                                                                userManagementData: this._userManagementData,
                                                                                accountSettings: this._accountSettings,
                                                                                changesCallback: changesCallback,
                                                                                editedUserIsLoggedIn: this._editedUserIsLoggedIn,
                                                                                validateCallback: validateCallback,
                                                                                forceCancelCb: forceCancelCb,
                                                                                checkLightBoxIsShown: checkLightBoxIsShown
                                                                            }).getPermissionsSection();

            sections.push( this._permissionsSection );
        }

        this._preparatoryStrategy.prepareSections( this._accountInformationSection, this._permissionsSection, setupNameCallBack );

    }

    var accountInformationSectionPlaceholderEl = document.getElementById( lightBoxEl.id + "_accountInformationSection" );
    var workScheduleAndOvertimeSectionPlaceholderEl = document.getElementById( lightBoxEl.id + "_workScheduleAndOvertimeSection" );
    var userRatesSectionPlaceholderEl = document.getElementById( lightBoxEl.id + "_userRatesSection" );
    var permissionsSectionPlaceholderEl = document.getElementById( lightBoxEl.id + "_permissionsSection" );

    $(accountInformationSectionPlaceholderEl).toggle(this._allowedToManageAccountSettings);
    $(workScheduleAndOvertimeSectionPlaceholderEl).toggle(this._allowedToManageAccountSettings);
    $(userRatesSectionPlaceholderEl).toggle(Boolean(this._allowedToManageAccountSettings && this._accountSettings && this._accountSettings.rates));
    $(permissionsSectionPlaceholderEl).toggle(this._allowedToManageAccountSettings && this._showPermissionsSection);

    return sections;
};

ActitimeUserManagementSectionsBuilder.prototype.getSectionsErrorsHandler = function (){
    return new SectionsErrorsHandler().init( this._accountInformationSection );
};

ActitimeUserManagementSectionsBuilder.prototype.getUserInputDataFetcher = function (){
    return new  ActitimeUserInputDataFetcher().init(this._accountInformationSection, this._workScheduleAndOvertimeSection,
                                                    this._userRatesSection, this._permissionsSection);
};

ActitimeUserManagementSectionsBuilder.prototype.initAfterShowing = function ( isCreateLightbox ){
    if ( isCreateLightbox )
        this._accountInformationSection.setFocusIntoFirstNameField();
    if ( this._userRatesSection )
        this._userRatesSection.updateAfterShowing();
};



/*ACTIPLANS USER MANAGEMENT SECTIONS BUILDER*/

function ActiplansUserManagementSectionsBuilder () {

}

ActiplansUserManagementSectionsBuilder.prototype = new UserManagementSectionsBuilder();

ActiplansUserManagementSectionsBuilder.prototype.init = function(preparatoryStrategy, workdayDurationsTableBuilder, userManagementData, currentTime, userId,
                                                                 accountSettings, editedUserIsLoggedIn, firstTTDate, firstRequestDate)
{

    UserManagementSectionsBuilder.prototype.init.call(this, preparatoryStrategy, workdayDurationsTableBuilder, userManagementData, currentTime, userId,
                                                      accountSettings, null, editedUserIsLoggedIn, null, null, firstTTDate, firstRequestDate, null);
    return this;

};

ActiplansUserManagementSectionsBuilder.prototype.createSections = function ( changesCallback, validateCallback, lightBoxEl, hideBoxBeforeReloadErrorHandler, setupNameCallBack, handlerIfNoSuchUserAfterAjax, showOvertime){
    var sections = [];
    var self = this;
    this._accountInformationSection = this._preparatoryStrategy.getAccountInformationSectionConstructor().init(  lightBoxEl,
                                                                           this._userManagementData,
                                                                           this._currentTime,
                                                                           this._userId,
                                                                           this._accountSettings,
                                                                           changesCallback,
                                                                           validateCallback,
                                                                           hideBoxBeforeReloadErrorHandler,
                                                                           this._defaultWorkSchedule,
                                                                           this._userManagementData.viewerSettings.userGroupsEnabled,
                                                                           this._userManagementData.viewerSettings.timeZoneGroupsEnabled,
                                                                           this._firstTTDate,
                                                                           this._firstRequestDate,
                                                                           this._editedUserIsLoggedIn,
                                                                           handlerIfNoSuchUserAfterAjax,
                                                                           self._preparatoryStrategy.getVersionSpecificDrawer()
    );

    this._workScheduleSection = new WorkScheduleSection().init(  this._workdayDurationsTableBuilder,
                                                                                     lightBoxEl,
                                                                                     this._userManagementData,
                                                                                     this._accountSettings,
                                                                                     changesCallback,
                                                                                     validateCallback, showOvertime );
    $(lightBoxEl).toggleClass("overtimeSectionOff", !showOvertime);

    sections.push( this._accountInformationSection );
    sections.push( this._workScheduleSection );

    this._preparatoryStrategy.prepareSections( this._accountInformationSection, null, setupNameCallBack );

    return sections;
};

ActiplansUserManagementSectionsBuilder.prototype.getSectionsErrorsHandler = function (){
    return new SectionsErrorsHandler().init( this._accountInformationSection );
};

ActiplansUserManagementSectionsBuilder.prototype.getUserInputDataFetcher = function (){
    return new  ActiplansUserInputDataFetcher().init(this._accountInformationSection, this._workScheduleSection);
};

ActiplansUserManagementSectionsBuilder.prototype.initAfterShowing = function ( isCreateLightbox ){
    if ( isCreateLightbox )
        this._accountInformationSection.setFocusIntoFirstNameField();
};

