function LeaveRatesCellRenderer()
{}

LeaveRatesCellRenderer.prototype = new TableCellRenderer();

LeaveRatesCellRenderer.prototype.init = function( leaveRatesMenu, changesCallback )
{
    TableCellRenderer.call( this );
    this.leaveRatesMenu = leaveRatesMenu;
    this.changesCallback = changesCallback;
    return this;
};

LeaveRatesCellRenderer.prototype.generateContent = function( cell )
{
    var leaveRatesEditor = new LeaveRatesEditor( {
         menu: this.leaveRatesMenu,
         renderTo: cell.getCellEl(),
         value: cell.getValue(),
         menuAlign: "tr-br",
         scrollableContainer: cell.getTable().getSectionContainer()
    } );

    leaveRatesEditor.on("menushow", function() {
        var val = cell.getValue();
        this.value = val;
        this.menu.setUp( val, cell.getRow().getRegularRateValue() );
    });

    var crossDiv = $("<div>" ).addClass("crossDiv").toggleClass("active", !this._hasNoLeaveRateValue(this.leaveRatesMenu));

    var self = this;
    leaveRatesEditor.on("change", function( leaveRates ) {
        cell.setValue( leaveRates );
        cell.validate( cell.getRow().getRegularRateValue() == null );
        self.changesCallback();
        cell.updateUI();
    } );

    cell.leaveRatesEditor = leaveRatesEditor;

    cell.getCellEl().appendChild(crossDiv[0]);

    crossDiv.click(function() {
        if (cell.getRow().isMarkedToRemove() || !crossDiv.hasClass("active"))
            return;
        cell.setValue(null);
        cell.validate(cell.getRow().getRegularRateValue() == null);
        self.changesCallback();
        cell.updateUI();
    });
};

LeaveRatesCellRenderer.prototype.updateUI = function( cell )
{
    TableCellRenderer.prototype.updateUI.call( this, cell );
    var isEmpty = cell.isEmpty();
    if( isEmpty )
    {
        cell.leaveRatesEditor.setText( messageResource.getMessage("user.add_edit.js.rates_editor.leave_rates.not_defined") );
    }
    else if( cell.isValid() )
    {
        var leaveRates = cell.getValue();
        var floatNumberValidator = cell.getTable().getFloatNumberValidator();
        var numberFormatter = cell.getTable().getNumberFormatter();
        var minRate = floatNumberValidator.parseFloat( leaveRates[0].rate );
        var maxRate = minRate;
        for( var i = 1; i < leaveRates.length; i++ )
        {
            var leaveRatesObj = leaveRates[i];
            var rateValue = floatNumberValidator.parseFloat( leaveRatesObj.rate );
            minRate = Math.min( minRate, rateValue );
            maxRate = Math.max( maxRate, rateValue );
        }

        if( minRate == maxRate )
            cell.leaveRatesEditor.setText( numberFormatter.format( minRate ) );
        else
            cell.leaveRatesEditor.setText( numberFormatter.format( minRate ) + " - " + numberFormatter.format( maxRate ) );
    }
    else
    {
        cell.leaveRatesEditor.setText( messageResource.getMessage("user.add_edit.js.rates_editor.leave_rates.invalid") );
    }

    cell.leaveRatesEditor.setDisabled( cell.getRow().isMarkedToRemove() );

    $(cell.getCellEl().lastChild ).toggleClass("active", !isEmpty);
    this.correctPositionOfCross(cell);
};


LeaveRatesCellRenderer.prototype.getCellClassName = function( cell )
{
    var classes = [cell.leaveRatesEditor.ctCls, "leaveRatesCell"];
    if( !cell.isValid() )
        classes.push("invalid");
    if ( cell.isEmpty() )
        classes.push("empty");
    return classes;
};

LeaveRatesCellRenderer.prototype._hasNoLeaveRateValue = function(leaveRates)
{
    var isEmpty = true;
    for (var i = 0; i < leaveRates.length; i++)
    {
        if (leaveRates[i].rate != "")
        {
            isEmpty = false;
            break;
        }
    }
    return isEmpty;
};

LeaveRatesCellRenderer.prototype.correctPositionOfCross = function(cell)
{
    $(cell.getCellEl().lastChild).css("left", cell.getCellEl().firstChild.offsetWidth + "px");
};
