function LeaveRatesEditorComponent( config )
{
    LeaveRatesEditorComponent.superclass.constructor.call(this, config);
    this.numberFormatter = config.numberFormatter;
    this.floatNumberValidator = config.floatNumberValidator;
    this.maxRate = config.maxRate;
    this.addEvents("okclick");
    this.addEvents("cancelclick");
}


Ext.extend( LeaveRatesEditorComponent, Ext.Component, {

    activeLeaveTypes: [],
    inputMap: {},
    defValTdMap: {},

    onRender: function( container, position )
    {
        var self = this;

        var mainDiv = document.createElement("div");

        this.el = Ext.get(mainDiv);

        var tableEl = document.createElement("table");
        tableEl.cellSpacing = 0;
        var tHeadEl = document.createElement("thead");
        var tBodyEl = document.createElement("tbody");
        mainDiv.appendChild( tableEl );
        tableEl.appendChild( tHeadEl );

        var c = $("<tr><th class='leaveTypeName'>" +
                messageResource.getMessage("user.add_edit.js.rates_editor.leave_rates.header.leave_type") +
                "</th><th class='icon'>&nbsp;</th><th class='rate'>" +
                messageResource.getMessage("user.add_edit.js.rates_editor.leave_rates.header.rate") +
                "</th><th class='default'>" +
                messageResource.getMessage("user.add_edit.js.rates_editor.leave_rates.header.default") +
                "</th></tr>");

        tHeadEl.appendChild( c[0] );
        tHeadEl.appendChild( $("<tr class='spacerRow'><td colspan=4></td></tr>")[0] );
        tableEl.appendChild( tBodyEl );

        this.inputMap = {};

        for( var i = 0; i < this.activeLeaveTypes.length; i++ )
        {
            var leaveType = this.activeLeaveTypes[i];
            var row = document.createElement("tr");
            var nameTd = document.createElement("td");
            nameTd.className="leaveTypeName";
            nameTd.innerHTML = escapeHtmlSymbols( leaveType.name );
            var iconTd = document.createElement("td");
            iconTd.className="icon";
            var iconImg = document.createElement("img");
            iconImg.src = imgPath("leave_types/icons/leave_type_" + leaveType.iconId + "_24bit.png");
            iconTd.appendChild( iconImg );
            var rateTd = document.createElement("td");
            rateTd.className="rate";
            var rateInput = document.createElement("input");
            rateInput.setAttribute("maxlength", "8");
            this.initRateInput( rateInput );
            this.inputMap[leaveType.id] = rateInput;
            rateTd.appendChild( rateInput );
            var defValTd = document.createElement("td");
            defValTd.className = "default";
            this.defValTdMap[leaveType.id] = defValTd;
            row.appendChild( nameTd );
            row.appendChild( iconTd );
            row.appendChild( rateTd );
            row.appendChild( defValTd );
            tBodyEl.appendChild( row );
        }

        var spacerRow = document.createElement( "tr" );
        spacerRow.className = "spacerRow";
        spacerRow.appendChild( $("<td colspan=4></td>")[0] );
        tBodyEl.appendChild(spacerRow);

        var tFoot = document.createElement("tfoot");
        var tr = document.createElement("tr");
        tr.className = "buttons";
        var td = document.createElement("td");
        td.colSpan = 4;
        var okButton = document.createElement("button");
        okButton.innerHTML = messageResource.getMessage("common.ok");
        okButton.onclick = function() { self.fireEvent("okclick"); };
        var cancelButton = document.createElement("button");
        cancelButton.innerHTML = messageResource.getMessage("common.cancel");
        cancelButton.onclick = function() { self.fireEvent("cancelclick"); };

        tableEl.appendChild(tFoot);
        tFoot.appendChild( tr );
        tr.appendChild( td );
        td.appendChild( okButton );
        td.appendChild( cancelButton );

        container.dom.insertBefore( mainDiv, position );
    },

    initRateInput: function( input )
    {
        var self = this;
        input.onblur = function()
        {
            this.value = trim( this.value );
            if( this.prevValue != this.value )
            {
                self.validate( this );
            }
        }
    },

    setUp: function( leaveRatesArray, regularRate )
    {
        for( var p in this.inputMap )
            this.inputMap[p].value = "";

        var hasNotEmptyValue = false;

        if( leaveRatesArray )
        {
            for( var i = 0; i < leaveRatesArray.length; i++ )
            {
                var leaveRate = leaveRatesArray[i];
                var input = this.inputMap[ leaveRate.leaveTypeId ];
                if( input )
                {
                    if( leaveRate.rate !== undefined && leaveRate.rate != null )
                        input.value = trim(String(leaveRate.rate));
                    hasNotEmptyValue = hasNotEmptyValue || input.value.length > 0;
                    input.prevValue = input.value;
                }
            }
        }

        var regularRateSpecified = _.isNumber(regularRate);

        var self = this;
        for( var l = 0; l < this.activeLeaveTypes.length; l++ )
        {
            var leaveType = this.activeLeaveTypes[l];
            var defTd = this.defValTdMap[ leaveType.id ];
            if(regularRateSpecified)
            {
                var defaultValue = this.numberFormatter.format( regularRate * leaveType.rateCoefficient );
                var a = document.createElement("a");
                a.innerHTML = defaultValue;
                a.val = defaultValue;
                a.ltId = leaveType.id;
                a.onclick = function()
                {
                    var input = self.inputMap[ this.ltId ];
                    input.value = this.val;
                    self.validate( input, true );
                };
                defTd.innerHTML = "";
                defTd.appendChild( document.createTextNode("( ") );
                defTd.appendChild( a );
                defTd.appendChild( document.createTextNode(" )" ) );
            }
            else
            {
                defTd.innerHTML = "( " + messageResource.getMessage("common.not_available_lower") + " )";
            }
        }

        for( var t in this.inputMap )
            this.validate( this.inputMap[t], hasNotEmptyValue || regularRateSpecified );
    },

    validate: function( input, emptyIsNotValid )
    {
        if( input.value.length == 0 )
        {
            if( emptyIsNotValid )
            {
                input.className = "invalid";
                input.title = messageResource.getMessage("user.add_edit.js.rates_editor.empty_rate");
            }
            else
            {
                input.className = null;
                input.removeAttribute("title");
            }
        }
        else if (this.floatNumberValidator.isValidNumber(input.value) && this.floatNumberValidator.parseFloat(input.value) < this.maxRate)
        {
            var val = this.floatNumberValidator.parseFloat( input.value );
            input.value = this.numberFormatter.format( val );
            input.className = null;
            input.removeAttribute("title");
        }
        else
        {
            input.className = "invalid";
            input.title = messageResource.getMessage("user.add_edit.js.rates_editor.invalid_rate", [this.maxRate]);
        }
    },

    getLeaveRates: function()
    {
        var leaveRates = [];
        for( var leaveTypeId in this.inputMap )
            leaveRates.push( { leaveTypeId: leaveTypeId, rate: this.inputMap[leaveTypeId].value } );
        return leaveRates;
    }

});
