function RateCellRenderer(  )
{
}

RateCellRenderer.prototype = new TableCellRenderer();

RateCellRenderer.prototype.init = function(changesCallback, baseCssClass, decimalSeparator)
{
    TableCellRenderer.call( this );
    this.changesCallback = changesCallback;
    this.baseCssClass = baseCssClass || "";
    this.decimalSeparator = decimalSeparator || ".";
    return this;
};

RateCellRenderer.prototype.generateContent = function( cell )
{
    var input = document.createElement("input");
    input.type = "text";
    input.setAttribute("maxlength", "8");
    input.setAttribute("placeholder", " 0" + this.decimalSeparator + "00"); // space is needed to distinguish placeholder from zero value in ie8/9
    this.setInputValue( input, input.value );
    $(input).placeholder();
    cell.getCellEl().appendChild( input );

    var crossSpan = $("<span>").addClass("crossSpan").toggleClass("active", cell.value != undefined);

    input.onblur = function() {
        this.value = trim(this.value);
        crossSpan.toggleClass("active", this.value != "");
        if( this.blurValue != this.value )
        {
            cell.setValue( $(this).pVal() );
            cell.updateUI();
            this.blurValue = this.value;
        }
    };

    var self = this;
    input.onkeyup = function()
    {
        if( this.keyUpValue != this.value )
        {
            self.changesCallback();
            crossSpan.toggleClass("active", trim(this.value) != "");
            this.keyUpValue = this.value;
        }
    };

    cell.getCellEl().appendChild(crossSpan[0]);

    crossSpan.click(function()
    {
        if (input.value != "" && !cell.getRow().isMarkedToRemove())
        {
            input.value = "";
            cell.setValue("");
            self.changesCallback();
            cell.updateUI();
        }
    });
};

RateCellRenderer.prototype.setInputValue = function( input, value )
{
    $(input).pVal(value);
    input.keyUpValue = input.value;
    input.blurValue = input.value;
};

RateCellRenderer.prototype.updateUI = function( cell )
{
    TableCellRenderer.prototype.updateUI.call( this, cell );
    var input = cell.getCellEl().firstChild;

    if( cell.isEmpty() )
    {
        this.setInputValue( input, "" );
    }
    else if( cell.isValid() )
    {
        var floatValue = cell.getTable().getFloatNumberValidator().parseFloat( cell.getValue() );
        this.setInputValue( input, cell.getTable().getNumberFormatter().format( floatValue ) );
    }

    switch( cell.getValidationResult() )
    {
        case TabularDataTableCell.VALUE_EMPTY: input.title = messageResource.getMessage("user.add_edit.js.rates_editor.empty_rate"); break;
        case TabularDataTableCell.VALUE_INVALID: input.title = messageResource.getMessage("user.add_edit.js.rates_editor.invalid_rate", [MAX_RATE]); break;
        default: input.removeAttribute("title");
    }

    input.disabled = cell.getRow().isMarkedToRemove();

    $(cell.getCellEl().lastChild).toggleClass("active", $(input).pVal() != "");
};

RateCellRenderer.prototype.getCellClassName = function( cell )
{
    var classes = [this.baseCssClass];
    if( !cell.isValid() )
        classes.push( "invalid" );
    return classes;
};


