function RatesTable() {}

RatesTable.prototype = new TabularDataWithEffectiveDatesTable();

RatesTable.prototype.init = function( rates, userManagementData, changesCallback, sectionContainer, validateCallback, corporateSchedule, currentTimeInUserTimeZone, ratesTableStrategy, showOvertime )
{
    var showLeaveRates = userManagementData.activeLeaveTypes && userManagementData.activeLeaveTypes.length > 0;
    this.overtimeLevel = UserManagementConstants.OVERTIME_OFF;
    this.showOvertime = showOvertime;
    this.sectionContainer = sectionContainer;

    this.changesCallback = changesCallback;
    this.ratesTableStrategy = ratesTableStrategy;

    TabularDataWithEffectiveDatesTable.prototype.init.call( this, this.buildHeader( showLeaveRates ), changesCallback, validateCallback, currentTimeInUserTimeZone );

    this.effectiveDateCellRenderer = new EffectiveDateCellRenderer().init(
            userManagementData.invertedDaysMap,
            userManagementData.weekStartDay,
            changesCallback,
            messageResource.getMessage(this.ratesTableStrategy.getEffectiveTextKey()),
            sectionContainer,
            corporateSchedule
    );
    this.effectiveDateCellValidator = new NotNullCellValidator().init();

    this.floatNumberValidator = new FloatNumberValidator( userManagementData.decimalSeparator );
    this.numberFormatter = new NumberFormatter( userManagementData.decimalSeparator, null, 2, false );

    this.regularRateCellRenderer = new RateCellRenderer().init(changesCallback, "regularRateColumn", userManagementData.decimalSeparator);
    this.rateCellValidator = new RateCellValidator().init();

    this.overtimeRateCellRenderer = null;
    if( userManagementData.viewerSettings.enabledFeatures[ UserManagementConstants.OVRFeature ] )
        this.overtimeRateCellRenderer = new RateCellRenderer().init(changesCallback, "overtimeColumn", userManagementData.decimalSeparator);

    this.leaveratesMenu = null;
    this.leaveRatesCellRenderer = null;
    this.leaveRatesCellValidator = null;
    if( showLeaveRates )
    {
        this.leaveratesMenu = new LeaveRatesMenu({
            activeLeaveTypes: userManagementData.activeLeaveTypes,
            numberFormatter: this.numberFormatter,
            floatNumberValidator: this.floatNumberValidator,
            maxRate: MAX_RATE,
            offsetXY: [0, 0]
        });
        this.leaveRatesCellRenderer = new LeaveRatesCellRenderer().init( this.leaveratesMenu, changesCallback );
        this.leaveRatesCellValidator = new LeaveRatesCellValidator().init();
    }

    this.removeRowCellRenderer = new RemoveRowCellRenderer().init();
    this.removeEmptyRowCellRenderer = new EmptyRemoveRowCellRenderer().init();

    this.userManagementData = userManagementData;

    if ( rates.length != 0 )
    {
        this.addRow( this.createFirstRow( rates[0] ) );
        for( var i = 1; i < rates.length; i++ )
            this.addRow( this.createRow( rates[i] ) );
    }
    else
    {
        this.addRow( this.createFirstEmptyRow() );
    }

    var self = this;
    this.ratesTableStrategy.buildAddRecordButtonIfNeeded( this.getTableEl(), function() { self.addEmptyRow(); } );

    return this;
};

RatesTable.prototype._createEmptyRatesObj = function()
{
    var ratesObj = {rateId: -1, leaveRates: []};
    var currentDateStr = this.userManagementData.todayInViewerTimeZone.parseJSONDate().truncateTime().toJSON();
    for( var i = 0; i < this.rows.length; i++ )
    {
        if( this.rows[i].isMarkedToRemove() )
            continue;
        var date = this.rows[i].cells[0].getValue();
        if( date && date.toJSON() == currentDateStr )
        {
            currentDateStr = null;
            break;
        }
    }
    if( currentDateStr )
        ratesObj.effectiveDate = currentDateStr;
    return ratesObj;
};

RatesTable.prototype.createEmptyRow = function()
{
    return this.createRow( this._createEmptyRatesObj() );
};

RatesTable.prototype.createFirstEmptyRow = function()
{
    return this.createFirstRow( this._createEmptyRatesObj() );
};

RatesTable.prototype._createRow = function( ratesObj, removeRowCellRenderer )
{
    return new RatesTableRow().init(
            ratesObj,
            this.effectiveDateCellRenderer,
            this.regularRateCellRenderer,
            this.overtimeRateCellRenderer,
            this.leaveRatesCellRenderer,
            removeRowCellRenderer,
            this.effectiveDateCellValidator,
            this.rateCellValidator,
            this.leaveRatesCellValidator,
            this.userManagementData.overtimeRateCoefficient,
            this.userManagementData.activeLeaveTypes
    );
};

RatesTable.prototype.createFirstRow = function( ratesObj )
{
    return this._createRow( ratesObj, this.removeEmptyRowCellRenderer );
};

RatesTable.prototype.createRow = function( ratesObj )
{
    return this._createRow( ratesObj, this.removeRowCellRenderer );
};

RatesTable.prototype.getBaseTableClasses = function()
{
    var classes = TabularDataTable.prototype.getBaseTableClasses.call( this );
    classes.push("userRatesTable");
    return classes;
};


RatesTable.prototype.buildHeader = function( showLeaveRates )
{
    var tHead = document.createElement("thead");
    var leaveRatesHeader = "<th class='leaveRatesCell'>" + (showLeaveRates ? messageResource.getEscapedMessage("user.add_edit.js.rates_editor.leave_rates_column_title") : "") + "</th>";
    var headerRow =
    $("<tr><th>&nbsp;</th>" +
    "<th class='regularRateColumn'>" + messageResource.getEscapedMessage("user.add_edit.js.rates_editor.regular_rate_column_title") + "</th>" +
    "<th class='overtimeColumn'>" + messageResource.getEscapedMessage("user.add_edit.js.rates_editor.overtime_rate_column_title") + "</th>"
    + leaveRatesHeader +
    "<th>&nbsp;</th></tr>")[0];
    tHead.appendChild( headerRow );
    return tHead;
};

RatesTable.prototype.updateAfterShowing = function()
{
    if (this.leaveRatesCellRenderer)
    {
        var row;
        for (var i = 0; i < this.rows.length; i++)
        {
            row = this.rows[i];
            for (var j = 0; j < row.cells.length; j++)
                if (row.cells[j].renderer instanceof LeaveRatesCellRenderer)
                    this.leaveRatesCellRenderer.correctPositionOfCross(row.cells[j]);
        }
    }
};

RatesTable.prototype.addEmptyRow = function()
{
    TabularDataWithEffectiveDatesTable.prototype.addEmptyRow.call( this );
    this.changesCallback();
};

RatesTable.prototype.setOvertimeLevel = function( overtimeLevel )
{
    this.overtimeLevel = overtimeLevel;
};

RatesTable.prototype.isOvertimeEnabled = function()
{
    return this.showOvertime && this.overtimeLevel != UserManagementConstants.OVERTIME_OFF;
};

RatesTable.prototype.getRates = function()
{
    var rates = [];
    for( var r = 0; r < this.rows.length; r++ )
    {
        var ratesObj = this.rows[r].getRates();
        if( ratesObj )
            rates.push( ratesObj );
    }
    return rates;
};

RatesTable.prototype.getFloatNumberValidator = function()
{
    return this.floatNumberValidator;
};

RatesTable.prototype.getNumberFormatter = function()
{
    return this.numberFormatter;
};

RatesTable.prototype.getSectionContainer = function()
{
    return this.sectionContainer;
};

//-------------------------------------

function RatesTableStrategy(){}

RatesTableStrategy.prototype.buildAddRecordButtonIfNeeded = function( callback ){};
RatesTableStrategy.prototype.getEffectiveTextKey = function(){};






//-------------------------------------

function CreateUserPopupRatesTableStrategy(){}

CreateUserPopupRatesTableStrategy.prototype = new RatesTableStrategy();

CreateUserPopupRatesTableStrategy.prototype.getEffectiveTextKey = function(){
    return "user.add_edit.js.rates_editor.effective_form.create";
};

//-------------------------------------

function EditUserPopupRatesTableStrategy(){}

EditUserPopupRatesTableStrategy.prototype = new RatesTableStrategy();

EditUserPopupRatesTableStrategy.prototype.buildAddRecordButtonIfNeeded = function( tableElem, callback )
{
    var tFoot = document.createElement( "tfoot" );
    var tr = document.createElement( "tr" );
    tr.className = "addRecord";
    var addRecTd = document.createElement( "td" );
    var a1 = document.createElement("a");
    var a2 = document.createElement("a");
    a1.className = "img";
    a2.className = "text";
    a1.innerHTML = "&nbsp;";
    a2.innerHTML = messageResource.getMessage( "user.add_edit.js.rates_editor.add_record" );
    if ( callback )
    {
        var callbackFn = function() { callback(); };
        a1.onclick = callbackFn;
        a2.onclick = callbackFn;
    }

    tFoot.appendChild( tr );
    tr.appendChild( addRecTd );
    addRecTd.appendChild( a1 );
    addRecTd.appendChild( a2 );
    tableElem.appendChild( tFoot );
};

EditUserPopupRatesTableStrategy.prototype.getEffectiveTextKey = function(){
    return "user.add_edit.js.rates_editor.effective_form.edit";
};
