function ManageRightsCellRenderer ()
{
    this.constructor = ManageRightsCellRenderer;
}

ManageRightsCellRenderer.prototype = _.create(TableCellRenderer.prototype, {

    init: function(isEditingAllowed, handlers)
    {
        TableCellRenderer.call(this);
        this.handlers = handlers;
        this.isEditingAllowed = isEditingAllowed;
        return this;
    },

    getCellClassName: function(cell)
    {
        return [];
    },

    getIconClassName: function(userListUserInfo)
    {
        return "";
    },

    _isPtoManage: function() {},

    _isActive: function(userListUserInfo) {},

    generateContent: function(cell)
    {
        var userListUserInfo = cell.value;

        var manageRightIcon = document.createElement("span");
        manageRightIcon.className = this.getIconClassName(userListUserInfo);

        var cellEl = cell.getCellEl();
        var jQcell = $(cellEl);
        cellEl.appendChild(manageRightIcon);

        if (this.isEditingAllowed)
        {
            if (cell.getValue().locked)
                cellEl.title = messageResource.getMessage("user.edit.js.user_is_locked");
            else
            {
                var self = this;

                $(manageRightIcon).addClass("editing");
                cellEl.onclick = function()
                {
                    self.handlers.onClick(cell);
                };

                jQcell.hover(
                        function(event)
                        {
                            self.handlers.onMouseIn(
                                    {
                                        isPto: self._isPtoManage(),
                                        isActive: self._isActive(userListUserInfo)
                                    },
                                    event);

                            jQcell.addClass("hovered")
                        },
                        function()
                        {
                            self.handlers.onMouseOut();
                            jQcell.removeClass("hovered")
                        }
                );
                jQcell.mousemove(
                        function(event)
                        {
                            self.handlers.onMouseMove(event);
                        }
                );
                jQcell.click(
                        function(event)
                        {
                            self.handlers.onMouseOut();
                        }
                );
            }
        }
    }
});

function ManageLeaveBalancesCellRenderer ()
{
    this.constructor = ManageLeaveBalancesCellRenderer;
}

ManageLeaveBalancesCellRenderer.prototype = _.create(ManageRightsCellRenderer.prototype, {

    getCellClassName: function()
    {
        return ["manageBalanceCell"];
    },

    getIconClassName: function(userListUserInfo)
    {
        var result = userListUserInfo.hasManagePTOAndSickRight ? 'manageBalanceEnabled' : 'manageBalanceDisabled';
        result += ' manageBalance';

        return result;
    },

    _isActive: function(userListUserInfo)
    {
        return userListUserInfo.hasManagePTOAndSickRight;
    },

    _isPtoManage: function()
    {
        return true;
    }
});

// ----------------------------------------

function ManageTTApproverCellRenderer ()
{
    this.constructor = ManageTTApproverCellRenderer;
}

ManageTTApproverCellRenderer.prototype = _.create(ManageRightsCellRenderer.prototype, {

    getCellClassName: function(cell)
    {
        var userListUserInfo = cell.value;
        var result = ['manageApproveCell', 'approverControlCell'];
        result.push(userListUserInfo.hasManageOtherTTRight ? 'enabled' : 'disabled');

        return result;
    },

    getIconClassName: function(userListUserInfo)
    {
        return "approveTTIcon";
    },

    _isActive: function(userListUserInfo)
    {
        return userListUserInfo.hasManageOtherTTRight;
    },

    _isPtoManage: function()
    {
        return false;
    }
});