function NameCellRenderer ()
{
    this.constructor = NameCellRenderer;
}

NameCellRenderer.prototype = _.create(TableCellRenderer.prototype, {

    init: function(tokensHighlighter, isEditingAllowed, onEditCallBack, warningsController)
    {
        TableCellRenderer.call(this);
        this.tokensHighlighter = tokensHighlighter;
        this.isEditingAllowed = isEditingAllowed;
        this.onEditCallBack = onEditCallBack;

        this.warningsController = warningsController;

        return this;
    },

    generateContent: function(cell)
    {
        var fullName = UserNameFormat.formatLFM(
                this.hl(cell.getValue().firstName),
                this.hl(cell.getValue().lastName),
                this.hl(cell.getValue().middleInitial)
        );

        var groupName = cell.getGroupName();
        var timeZoneGroupName = cell.getTimeZoneGroupName();

        var inactiveText = messageResource.getMessage("userlist.inactive");
        var enabledInAp = messageResource.getMessage("userlist.userdata.enabled_in_ap");

        var cellEl = cell.getCellEl();
        var jQcell = $(cellEl);

        var cellEl_innerHTML = "<div class='name'>" + "<span class='userInOtherProduct' title='" + enabledInAp + "'>" + "</span>" +
                               "<span class='userNameSpan'>" + fullName + "</span>" +
                               "<span class='inactive'>" + inactiveText + "</span></div>" +
                               "<div class='userGroups'>" +
                               "<div class='groupName'>" + groupName + "</div>" +
                               "<div class='timeZoneGroupName'>" + timeZoneGroupName + "</div>" +
                               "</div>" +
                               "<div class='warningsInRow'></div>";

        cellEl.innerHTML = "<table class='userNameContainer'><tr><td>" + cellEl_innerHTML + "</td></tr></table>";

        var hasInfo = cell.infos && cell.infos.length > 0;
        var hasWarnings = cell.warnings && cell.warnings.length > 0;

        if (hasWarnings)
        {
            var warningSection = jQcell.find(".warningsInRow");
            this._createWarningsEl(warningSection, cell.warnings);
        }

        if (hasInfo)
        {
            this._createInfoEl(cell, cell.getValue().username);
        }

        if (this.isEditingAllowed)
        {
            if (cell.getValue().locked)
                cellEl.title = messageResource.getMessage("user.edit.js.user_is_locked");
            else
            {
                var self = this;

                jQcell.click(function()
                             {
                                 self.onEditCallBack(cell.value);
                             });

                jQcell.hover(function()
                             {
                                 jQcell.addClass("hovered");
                             },
                             function()
                             {
                                 jQcell.removeClass("hovered");
                             });
            }
        }

    },

    _createInfoEl: function(cell, username)
    {
        var noteJQEl = this._createNoteEl(cell, username, 'infoSpan', cell.infos);
        this.warningsController.setInfoHoverEvents(noteJQEl, cell.infos);
    },

    _createWarningsEl: function(warningSection, warnings)
    {
        if (!warnings)
        {
            warningSection.hide();
            return;
        }
        this.warningsController.setWarnings(warningSection, warnings);
    },

    _createNoteEl: function(cell, username, className, notes)
    {
        var cellEl = cell.getCellEl();
        var jQcell = $(cellEl);

        var noteEl = document.createElement("span");
        noteEl.className = className;
        noteEl.id = escapeHtmlSymbols(username) + '_className';

        var noteJQEl = $(noteEl);
        this.warningsController.setClickEvents(noteJQEl, notes);

        jQcell.find(".userNameSpan").after(noteJQEl);

        return noteJQEl;
    },

    hl: function(str)
    {
        return this.tokensHighlighter.highlightTokens(str);
    },

    getCellClassName: function()
    {
        return ["userNameCell"];
    },

    updateGroupName: function(cell)
    {
        $(cell.getCellEl()).find(".groupName")[0].innerHTML = cell.groupName;
    },

    updateTimeZoneGroupName: function(cell)
    {
        $(cell.getCellEl()).find(".timeZoneGroupName")[0].innerHTML = cell.timeZoneGroupName;
    }
});