function TaskHierarchyAssignmentsCellRenderer ()
{
    this.constructor = TaskHierarchyAssignmentsCellRenderer;
}

TaskHierarchyAssignmentsCellRenderer.prototype = _.create(EditableCellRenderer.prototype);

TaskHierarchyAssignmentsCellRenderer.prototype.init = function ( isEditingAllowed, handlers )
{
    TableCellRenderer.call( this );

    this.isEditingAllowed = isEditingAllowed;
    this.handlers = handlers;

    return this;

};

TaskHierarchyAssignmentsCellRenderer.prototype.getCellClassName = function ( cell )
{
    var aInfo = this._getAssignmentsInfo( cell.getValue() );
    return ["taskHierarchyAssignmentsCell", aInfo.allProjectsByRights ? "disabled" : "enabled" ];
};

TaskHierarchyAssignmentsCellRenderer.prototype.generateContent = function ( cell )
{
    this.updateContent( cell );
};

TaskHierarchyAssignmentsCellRenderer.prototype._getAssignmentsInfo = function( userInfo )
{
    return {
        allProjectsByRights: userInfo.hasManageCustomersProjectsRight || userInfo.hasManageUsersRight,
        allProjectsFlag: userInfo.allProjectsAssigned,
        numberOfAssignedProjects: userInfo.numberOfAssignedProjects
    }
};

TaskHierarchyAssignmentsCellRenderer.prototype._generateInnerSpan = function( isEditingAllowed, aInfo )
{
    var span = document.createElement( "span" );
    var classes = [ "numberOfProjects" ];
    if( aInfo.allProjectsByRights || aInfo.allProjectsFlag )
    {
        if( aInfo.allProjectsByRights ) classes.push( "allProjectsNotClickable" );
        text = messageResource.getEscapedMessage( "userlist.js.project_assignments.all_projects" );
    }
    else if( !aInfo.numberOfAssignedProjects || aInfo.numberOfAssignedProjects == 0  )
    {
        if( isEditingAllowed ) classes.push( "noProjects" );
        text = messageResource.getEscapedMessage( "userlist.js.project_assignments.no_projects" );
    }
    else if( aInfo.numberOfAssignedProjects == 1 )
    {
        text = messageResource.getEscapedMessage( "userlist.js.project_assignments.one_project" );
    }
    else
    {
        text = messageResource.getEscapedMessage( "userlist.js.project_assignments.some_projects", [ aInfo.numberOfAssignedProjects ] );
    }
    span.className = classes.join( " " );
    span.innerHTML = text;
    return span;
};

TaskHierarchyAssignmentsCellRenderer.prototype.getJQCellContent = function(jQCell) {
    var jqContent = jQCell.find('.taskHierarchyAssignmentsContent');
    if(jqContent.length === 0) {
        // Add additional wrapping to not change tips on cell update
        var contentDiv = document.createElement('div');
        contentDiv.className = 'taskHierarchyAssignmentsContent';
        jqContent = $(contentDiv);
        jQCell.append(jqContent);
    }
    return jqContent;
};

TaskHierarchyAssignmentsCellRenderer.prototype.updateContent = function( cell )
{
    var self = this;
    var userInfo = cell.getValue();
    var cellEl = cell.getCellEl();
    var jQCell = $(cellEl);
    var aInfo = this._getAssignmentsInfo( userInfo );

    var qqDiv = document.createElement( "div" );
    qqDiv.className = "numberOfProjectsDiv";
    qqDiv.appendChild( self._generateInnerSpan( self.isEditingAllowed, aInfo ) );

    this.getJQCellContent( jQCell ).empty().append( qqDiv );

    jQCell.off( "mouseenter mouseleave" ); // remove tooltip handlers
    if( self.isEditingAllowed && !aInfo.allProjectsByRights ) // add tooltip handlers
    {
        if ( aInfo.numberOfAssignedProjects == 0 && !aInfo.allProjectsFlag) {
            jQCell.hover(   function ( event )
                {
                    self.handlers.onMouseIn( {
                        isHelp: false,
                        fullusername: UserNameFormat.formatFML( userInfo.firstName, userInfo.lastName, userInfo.middleInitial )
                    }, event );
                },
                function ()
                {
                    self.handlers.onMouseOut();
                }
            );
        } else {
            jQCell.hover(   function ( event )
                {
                    self.handlers.onMouseIn( {
                        isHelp: true
                    }, event );
                },
                function ()
                {
                    self.handlers.onMouseOut();
                }
            );
        }

        jQCell.mousemove(   function ( event )
                            {
                                self.handlers.onMouseMove( event );
                            }
        );
    }

    if( self.isEditingAllowed && !aInfo.allProjectsByRights )
    {
        cellEl.onclick = function ()
        {
            self.handlers.onMouseOut(); // force onMouseOut event for ie, safari to hide tooltip
            self.handlers.onClick(
                    {   userId: cell.getValue().id,
                        offset: "0px 0px",
                        cellObj: cell,
                        cellEl: cellEl
                    }
            );
        };
    }
    else
    {
        cellEl.onclick = function() {};
    }
};