function TaskHierarchyManageCellRenderer ()
{
    this.constructor = TaskHierarchyManageCellRenderer;
}

TaskHierarchyManageCellRenderer.prototype = _.create(TableCellRenderer.prototype, {

    init: function(isEditingAllowed, handlers)
    {
        TableCellRenderer.call(this);

        this.isEditingAllowed = isEditingAllowed;
        this.handlers = handlers;

        this.isOpenMenu = false; // flag indicates if menu is opened

        return this;

    },

    getCellClassName: function()
    {
        return ["taskHierarchyManageCell"];
    },

    generateContent: function(cell)
    {
        var userListUserInfo = cell.value;
        var self = this;
        var cellEl = cell.getCellEl();

        var manageElement = document.createElement("div");
        manageElement.className = "taskHierarchySelector";
        var jQcell = $(manageElement);
        cellEl.appendChild(manageElement);

        var permissionLevel;
        if (userListUserInfo.hasManageCustomersProjectsRight)
        {
            permissionLevel = UserManagementConstants.TASK_HIERARCHY_CP_PERMISSION;
            jQcell.addClass("customersPermission");
        }
        else if (userListUserInfo.hasManageTasksRight)
        {
            permissionLevel = UserManagementConstants.TASK_HIERARCHY_TASKS_PERMISSION;
            jQcell.addClass("tasksPermission");
        }
        else
        {
            permissionLevel = UserManagementConstants.TASK_HIERARCHY_NO_PERMISSION;
            jQcell.addClass("noPermissions");
        }

        if (this.isEditingAllowed)
        {
            cell.taskHierarchySelector = new ListSelector(
                    {
                        menuCls: "manageTaskHierarchyMenu",
                        renderTo: manageElement,
                        value: permissionLevel,
                        listItems: [
                            {
                                text: messageResource.getEscapedMessage("userlist.js.table.task_hierarchy.menu.customers"),
                                value: UserManagementConstants.TASK_HIERARCHY_CP_PERMISSION,
                                styleCls: "customersMenuItem"
                            },
                            {
                                text: messageResource.getEscapedMessage("userlist.js.table.task_hierarchy.menu.tasks"),
                                value: UserManagementConstants.TASK_HIERARCHY_TASKS_PERMISSION,
                                styleCls: "tasksMenuItem"
                            },
                            {
                                text: messageResource.getEscapedMessage("userlist.js.table.task_hierarchy.menu.no_permissions"),
                                value: UserManagementConstants.TASK_HIERARCHY_NO_PERMISSION,
                                styleCls: "noPermissionsMenuItem"
                            }
                        ],
                        onMenuShow: function()
                        {
                            ListSelector.superclass.onMenuShow.call(this);
                            self.isOpenMenu = true;
                        },
                        onMenuHide: function()
                        {
                            ListSelector.superclass.onMenuHide.call(this);
                            self.isOpenMenu = false;
                        }
                    }
            );
            cell.taskHierarchySelector.on("change", function()
            {
                self.handlers.onChange(cell);
            });

            // add handlers for tooltip
            jQcell.hover(
                    function(event)
                    {
                        if (!self.isOpenMenu)
                        { // if menu is opened do not show tooltip on current element
                            self.handlers.onMouseIn({}, event);
                        }
                    },
                    function()
                    {
                        self.handlers.onMouseOut();
                    }
            );
            jQcell.mousemove(
                    function(event)
                    {
                        self.handlers.onMouseMove(event);
                    }
            );
            jQcell.click(
                    function(event)
                    {
                        self.handlers.onMouseOut();
                    }
            );
        }
        else
        {
            var readOnlyImageElement = document.createElement("span");
            readOnlyImageElement.className = "permissionsImage";
            manageElement.appendChild(readOnlyImageElement);
        }
    }
});




