/**
 * Created by evdokimov on 12.03.2015.
 */

function PlannedLeaveRenderer () {}

PlannedLeaveRenderer.prototype = _.create(TableCellRenderer.prototype);

PlannedLeaveRenderer.prototype.init = function ( formatter ) {
    TableCellRenderer.call(this);
    this.formatter = formatter;
    return this;
};

PlannedLeaveRenderer.prototype.hasValue = function(cell)
{
    return false;
};

PlannedLeaveRenderer.prototype.getCellClassName = function ( cell ) {
    var value = cell.getValue();
    return [this.cellGeneralClassName, 'plannedLeaveCell', this.hasValue(cell) ? "on" : "off"];
};

PlannedLeaveRenderer.prototype.generateContent = function ( cell ) {
    //abstract
};

function PlannedPTOLeaveRenderer () {}
PlannedPTOLeaveRenderer.prototype = _.create(PlannedLeaveRenderer.prototype);
PlannedPTOLeaveRenderer.prototype.cellGeneralClassName = "ptoCell";
PlannedPTOLeaveRenderer.prototype.hasValue = function(cell)
{
    return cell.getValue().pto;
};
PlannedPTOLeaveRenderer.prototype.generateContent = function ( cell ) {
    var value = cell.getValue();
    cell.getCellEl().innerHTML = !this.hasValue(cell) ? "&ndash;" : this.formatter(value.currentPTOBalance.plannedBalanceApproved);
};

function PlannedSickLeaveRenderer () {}
PlannedSickLeaveRenderer.prototype = _.create(PlannedLeaveRenderer.prototype);
PlannedSickLeaveRenderer.prototype.cellGeneralClassName = "sickDaysCell";
PlannedSickLeaveRenderer.prototype.hasValue = function(cell)
{
    return cell.getValue().sick;
};
PlannedSickLeaveRenderer.prototype.generateContent = function ( cell ) {
    var value = cell.getValue();
    cell.getCellEl().innerHTML = !this.hasValue(cell) ? "&ndash;" : this.formatter(value.currentSickDaysBalance.plannedBalanceApproved);
};

// ----------------------------------------

function BalanceControlRenderer () {}

BalanceControlRenderer.prototype = _.create(TableCellRenderer.prototype);

BalanceControlRenderer.prototype.init = function ( onClickHandler ) {
    TableCellRenderer.call(this);
    this.onClickHandler = onClickHandler;
    return this;
};

BalanceControlRenderer.prototype.getCellClassName = function ( cell ) {
    var value = cell.getValue();
    return this.getFlagValue(value) ? [this.cellGeneralClassName, 'on'] : [this.cellGeneralClassName, 'off'];
};

BalanceControlRenderer.prototype.generateContent = function ( cell ) {
    var value = cell.getValue();
    cell.getCellEl().innerHTML = !this.getFlagValue(value) ? messageResource.getMessage('common.off') : messageResource.getMessage('common.on');

    var self = this;
    cell.getCellEl().onclick = function() {
        self.onClickHandler( cell, !self.getFlagValue(value));
    };
};

function PtoControlRenderer () {}
PtoControlRenderer.prototype = _.create(BalanceControlRenderer.prototype);
PtoControlRenderer.prototype.cellGeneralClassName = "ptoControlCell";
PtoControlRenderer.prototype.getFlagValue = function( value ) {
    return value.pto;
};

function SickDaysControlRenderer () {}
SickDaysControlRenderer.prototype = _.create(BalanceControlRenderer.prototype);
SickDaysControlRenderer.prototype.cellGeneralClassName = "sickDaysControlCell";
SickDaysControlRenderer.prototype.getFlagValue = function( value ) {
    return value.sick;
};

// ----------------------------------------

function BalanceRulesRenderer () {}

BalanceRulesRenderer.prototype = _.create(TableCellRenderer.prototype);

BalanceRulesRenderer.prototype.init = function ( formatter, handlers ) {
    TableCellRenderer.call(this);
    this.formatter = formatter;
    this.onClickHandler = handlers.onClickHandler;
    this.onMouseOverHandler = handlers.onMouseOverHandler;
    this.onMouseOutHandler = !handlers.onMouseOutHandler ? function(){} : handlers.onMouseOutHandler;
    this.onMouseMoveHandler = handlers.onMouseMoveHandler;

    return this;
};

BalanceRulesRenderer.prototype.getCellClassName = function ( cell ) {
    var data = this.getBalanceFromCellValue(cell.getValue());
    var result = [this.getCellGeneralClassName()];

    if ( !data )
        result.push("noBalance");
    else if ( data.defaultRules)
        result.push("defaultRules");

    result.push(this._customClassName());

    return result;
};

BalanceRulesRenderer.prototype.getCellGeneralClassName = function () {
    return this.mixin.cellGeneralClassName;
};
BalanceRulesRenderer.prototype.getBalanceFromCellValue = function (value) {
    var balanceData = this.mixin.getBalanceFromCellValue(value);
    if(balanceData){
        balanceData.hireDate = value.hireDate;
    }
    return balanceData;
};
BalanceRulesRenderer.prototype.generateContent = function ( cell ) {
    var data = this.getBalanceFromCellValue(cell.getValue());

    if ( !data )
        return;

    var cellEl = cell.getCellEl();

    if ( !data.defaultRules || this._handleDefaultPTORules() )
        cellEl.innerHTML = !data ? "" : this._getRuleValue(data);

    var self = this;

    if ( this.onClickHandler )
        cellEl.onclick = function( e ) { self.onClickHandler(e, data, cell); };

    if ( this.onMouseOverHandler )
        $(cellEl).hover( function( e ){ self.onMouseOverHandler(e, data); }, this.onMouseOutHandler );

    if ( this.onMouseMoveHandler )
        $(cellEl).mousemove( this.onMouseMoveHandler );

};

BalanceRulesRenderer.prototype._getRuleValue = function ( ptoData ) {
    // abstract
};

BalanceRulesRenderer.prototype._customClassName = function ( ptoData ) {
    // abstract
};

BalanceRulesRenderer.prototype._handleDefaultPTORules = function ( ptoData ) {
    return false;
};

// ----------------------------------------

function AccrualRulesCell () {}

AccrualRulesCell.prototype = _.create(BalanceRulesRenderer.prototype);

AccrualRulesCell.prototype._getRuleValue = function ( ptoData ) {
    return ptoData.accrualRule ? this.formatter(ptoData.accrualRule.value) + " " + CalendarFrequencyType.descriptionFor[ptoData.accrualRule.frequency.type] : "&ndash;";
};

AccrualRulesCell.prototype._customClassName = function ( ptoData ) {
    return "accrualRule"
};


function PtoRulesMixin () {}
PtoRulesMixin.prototype.cellGeneralClassName = "ptoRulesCell";
PtoRulesMixin.prototype.getBalanceFromCellValue = function( value ) {
    return value.currentPTOBalance;
};

function SickDaysRulesMixin () {}
SickDaysRulesMixin.prototype.cellGeneralClassName = "sickDaysRulesCell";
SickDaysRulesMixin.prototype.getBalanceFromCellValue = function( value ) {
    return value.currentSickDaysBalance;
};

function PtoAccrualRulesCell () {}
PtoAccrualRulesCell.prototype = _.create(AccrualRulesCell.prototype);
PtoAccrualRulesCell.prototype.mixin = new PtoRulesMixin();

function SickDaysAccrualRulesCell () {}
SickDaysAccrualRulesCell.prototype = _.create(AccrualRulesCell.prototype);
SickDaysAccrualRulesCell.prototype.mixin = new SickDaysRulesMixin();

// ----------------------------------------

function ResetRulesCell () {}

ResetRulesCell.prototype = _.create(BalanceRulesRenderer.prototype);

ResetRulesCell.prototype._getRuleValue = function ( data )
{
    return data.resetRule ?
           this.formatter( data.resetRule.value ) + " " + (data.resetRule.frequency.type == CalendarFrequencyType.ON_HIRE_DATE ?
           CalendarFrequencyType.descriptionFor[data.resetRule.frequency.type] :
           "") + " " + CalendarFrequencyType.parametersFormatterFor[data.resetRule.frequency.type]( data.resetRule.frequency.params ) :
           "&ndash;";
};

ResetRulesCell.prototype._customClassName = function ( data ) {
    return "resetRule"
};

function PtoResetRulesCell () {}
PtoResetRulesCell.prototype = _.create(ResetRulesCell.prototype);
PtoResetRulesCell.prototype.mixin = new PtoRulesMixin();

function SickDaysResetRulesCell () {}
SickDaysResetRulesCell.prototype = _.create(ResetRulesCell.prototype);
SickDaysResetRulesCell.prototype.mixin = new SickDaysRulesMixin();

// ----------------------------------------

function CapRulesCell () {}

CapRulesCell.prototype = _.create(BalanceRulesRenderer.prototype);

CapRulesCell.prototype._getRuleValue = function ( ptoData ) {
    return ptoData.balanceCap !== null ? this.formatter(ptoData.balanceCap) : "&ndash;";
};

CapRulesCell.prototype._customClassName = function ( ptoData ) {
    return "capRule";
};

function PtoCapRulesCell () {}
PtoCapRulesCell.prototype = _.create(CapRulesCell.prototype);
PtoCapRulesCell.prototype.mixin = new PtoRulesMixin();

function SickDaysCapRulesCell () {}
SickDaysCapRulesCell.prototype = _.create(CapRulesCell.prototype);
SickDaysCapRulesCell.prototype.mixin = new SickDaysRulesMixin();


// ----------------------------------------

function DefaultRulesCell () {}

DefaultRulesCell.prototype = _.create(BalanceRulesRenderer.prototype);
DefaultRulesCell.prototype.defaultMessage = null;
DefaultRulesCell.prototype._getRuleValue = function ( ptoData ) {
    return this.defaultMessage;
};

DefaultRulesCell.prototype._customClassName = function ( ptoData ) {
    return "defaultRule";
};

DefaultRulesCell.prototype._handleDefaultPTORules = function ( ptoData ) {
    return true;
};


function PtoDefaultRulesCell () {}
PtoDefaultRulesCell.prototype = _.create(DefaultRulesCell.prototype);
PtoDefaultRulesCell.prototype.defaultMessage = messageResource.getMessage("pto_rules_editor.default_pto_rules");
PtoDefaultRulesCell.prototype.mixin = new PtoRulesMixin();

function SickDaysDefaultRulesCell () {}
SickDaysDefaultRulesCell.prototype = _.create(DefaultRulesCell.prototype);
SickDaysDefaultRulesCell.prototype.defaultMessage = messageResource.getMessage("sick_days_rules_editor.default_sick_days_rules");
SickDaysDefaultRulesCell.prototype.mixin = new SickDaysRulesMixin();


// ----------------------------------------

