function UserTableController()
{
    this.constructor = UserTableController;
}

UserTableController.PUNCTUATORS = StringTokenizer.PUNCTUATORS_FOR_SEARCH;

UserTableController.prototype = _.create(Table.prototype, {
    init: function ( conf )
    {
        Table.call( this, conf.tHeadEl );

        this.userBeans = conf.userBeans;

        this.canManageLeaveBalances = conf.canManageLeaveBalances;
        this.canManageAccounts = conf.canManageAccounts;
        this.isPTOEnabled = conf.isPTOEnabled;
        this.isSickEnabled = conf.isSickEnabled;
        this.isTTAEnabled = conf.isTTAEnabled;
        this.isPASEnabled = conf.isPASEnabled;
        this.isAccountSettingsVisible = conf.isAccountSettingsVisible;
        this.sortOrder = conf.sortOrder;
        this.handlersMap = conf.handlersMap;
        this.warningsController = new WarningsController();

        this.tableRenderer = new UserTableRenderer();
        this.tableRenderer.init();
        this.containerEl = conf.containerEl;
        this.rowRenderer = new UserRowRenderer().init();
        this.userIdRowMap = {};

        this.ptoHoursPerDay = conf.ptoHoursPerDay;

        var self = this;
        if ( this.isPTOColumnAvailable() ) {
            this.ptoRulesManager = new BalanceRulesManager().init("pto", conf.ptoDefaults, this.ptoHoursPerDay, {
                defaultMessageNoteKey: "pto_rules_editor.info.default",
                titleNoteKey: "pto_rules_editor.use_accrual_rules",
                customMessageNoteKey: "pto_rules_editor.info.custom",
                customTitleTooltip: "pto_rules_editor.custom_rules",
                defaultTitleTooltip: "pto_rules_editor.default_rules"
            } );
            var ptoPrecision = SystemSettings.showPTOInHours? 2 : 1;
            var postfixPTOSymbol =  SystemSettings.showPTOInHours ? messageResource.getMessage('common.pto_units.hours.short') :  messageResource.getMessage('common.pto_units.days.short');

            this.ptoFormatter = new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol,ptoPrecision,
                    true, '',postfixPTOSymbol);

            this.rulesFormatter = new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, ptoPrecision,
                    true, '', postfixPTOSymbol);


            this.ptoValueFormatter = function ( value ) {
                var val = SystemSettings.showPTOInHours ? value * self.ptoHoursPerDay : value;
                return  self.ptoFormatter.format(val);
            };

            this.ptoRulesFormatter = function ( value ) {
                var val = SystemSettings.showPTOInHours ? value * self.ptoHoursPerDay : value;
                return  self.rulesFormatter.format(val);
            };
        }

        if ( this.isSickDaysColumnAvailable() ) {
            this.sickRulesManager = new BalanceRulesManager().init("sick", conf.sickDefaults, this.ptoHoursPerDay,  {
                defaultMessageNoteKey : "sick_rules_editor.info.default",
                titleNoteKey: "sick_rules_editor.use_accrual_rules",
                customMessageNoteKey: "pto_rules_editor.info.custom",
                customTitleTooltip: "sick_rules_editor.custom_rules",
                defaultTitleTooltip: "sick_rules_editor.default_rules"});
            var sickPrecision = SystemSettings.showSickInHours ? 2 : 1;
            var postfixSickSymbol =  SystemSettings.showSickInHours ? messageResource.getMessage('common.pto_units.hours.short') :  messageResource.getMessage('common.pto_units.days.short');

            this.sickFormatter = new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, sickPrecision,
                    true, '', postfixSickSymbol);

            this.sickDaysRulesFormatter = new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, sickPrecision,
                    true, '', postfixSickSymbol);

            this.sickValueFormatter = function ( value ) {
                var val = SystemSettings.showSickInHours ? value * self.ptoHoursPerDay : value;
                return  self.sickFormatter.format(val);
            };

            this.sickRulesFormatter = function ( value ) {
                var val = SystemSettings.showSickInHours ? value * self.ptoHoursPerDay : value;
                return  self.sickDaysRulesFormatter.format(val);
            };
        }

        this.tokensHighlighter = new EmptyHighlighter();
        this.createBodyContent(conf.userBeans, conf.groupMap, conf.timeZoneGroupMap, this.tokensHighlighter, conf.allApprovers);

        return this;
    },

    setSearchText: function( text )
    {
        this.tableRenderer.setSearchText( text );
    },

    makeUserIdGroupIdMapForVisibleUser: function ()
    {
        var map = {};
        for( var i = 0; i < this.rows.length; i++ )
        {
            var userListUserInfo = this.rows[i].getUserListUserInfo();
            map[ userListUserInfo.id ] = userListUserInfo.userGroupId;
        }
        return map;
    },

    makeTimeZoneIdGroupIdMapForVisibleUser: function ()
    {
        var map = {};
        for( var i = 0; i < this.rows.length; i++ )
        {
            var userListUserInfo = this.rows[i].getUserListUserInfo();
            map[ userListUserInfo.id ] = userListUserInfo.timeZoneGroupId;
        }
        return map;
    },

    getBaseRowClasses: function ( rowIdx )
    {
        // TODO: When we get rid of the UserTableRow, this table has to keep array of UserListUserInfo structures.
        return this.rowRenderer.getRowClasses( this.getRow( rowIdx ).getUserListUserInfo() );
    },

    render: function ()
    {
        this.buildDOM( this.containerEl );
    },

    createBodyContent: function ( userListUserInfoRecords, groupsMap, timeZoneGroupsMap, tokensHighlighter, allApprovers )
    {
        var approversSortingProcessor = this.isTTAEnabled ? createApproversSortingProcessor( allApprovers ) : function () {};
        for( var i = 0; i < userListUserInfoRecords.length; i++ )
        {
            var userListUserInfo = userListUserInfoRecords[i];
            var groupName = groupsMap == null ? " " : groupsMap[userListUserInfo.userGroupId];
            var timeZoneGroupName = timeZoneGroupsMap == null ? " " : timeZoneGroupsMap[userListUserInfo.timeZoneGroupId];

            this.addRow( this.createRow( userListUserInfo, groupName, timeZoneGroupName, tokensHighlighter, [], approversSortingProcessor ) );
        }
    },

    disableTextSelection: function(  )
    {
      //do not disable selection;
    },

    buildDOM: function ( containerEl )
    {
        Table.prototype.buildDOM.call( this, containerEl );
        this.updateUI();
        this.tableRenderer.setupEventHandlers( this.handlersMap );
        this.tableRenderer.initUserFilterField( this.handlersMap );
    },

    _findRow: function ( e ) {
        //this two rows for fix bug  51665 IE8
        var event = e || window.event;
        var targetElement = event.target || event.srcElement;
        return $(targetElement).closest('tr');
    },

    toggleHoverClassInRulesCell: function ( e, hovered ) {
        this._findRow(e).toggleClass('ptoRulesHovered', hovered);
    },

    _modifyPTOAccrualRules: function ( cell ) {
        var dialogValues = this.ptoRulesManager.getDialogValues();

        var self = this;
        this.handlersMap.modifyPTOAccrualRules(cell, dialogValues, function() { self.ptoRulesManager.hideDialog(); });
    },

    _modifySickAccrualRules: function ( cell ) {
        var dialogValues = this.sickRulesManager.getDialogValues();

        var self = this;
        this.handlersMap.modifySickAccrualRules(cell, dialogValues, function() { self.sickRulesManager.hideDialog(); });
    },

    _onEditRulesClicked: function ( jQueryEvent, rulesManager, balanceData, cell, cellSelector, onClickCallback ) {
        if( !balanceData )
            return;

        rulesManager.hideAccrualRulesTooltip();
        rulesManager.updateDialog(balanceData);

        var self = this;
        var row = this._findRow(jQueryEvent);

        rulesManager.setModifyBalanceAccrualRulesOkClick( onClickCallback );

        rulesManager.setOnCloseCallback( function() {
            row.removeClass("ptoEditorSelected");
            self.clearRowSelection(row);
        });

        var event = jQueryEvent || window.event;
        var targetCell = event.target || event.srcElement;
        targetCell = !balanceData.defaultRules ? row.find(".resetRule:visible") : row.find(".defaultRule:visible");



        rulesManager.showDialogAt(targetCell, {offset: "5px -10px"});

        row.addClass("ptoEditorSelected");
        this.addRowSelection(row);
    },

    _onEditPTORulesClicked: function ( jQueryEvent, balanceData, cell ) {
        var self = this;
        self._onEditRulesClicked(jQueryEvent, this.ptoRulesManager, balanceData, cell, ".ptoRulesCell", function() { self._modifyPTOAccrualRules(cell); });
    },

    _onEditSickRulesClicked: function ( jQueryEvent, balanceData, cell ) {
        var self = this;
        self._onEditRulesClicked(jQueryEvent, this.sickRulesManager, balanceData, cell, ".sickRulesCell", function() { self._modifySickAccrualRules(cell); });
    },

    createRow: function ( userListUserInfo, groupName, timeZoneGroupName, tokensHighlighter, warningsData, approversSortingProcessor )
    {
        var self = this;
        var row = new UserTableRow().init( userListUserInfo );

        this.userIdRowMap[userListUserInfo.id]=row;

        var nameCell = new UserNameCell().init( userListUserInfo, groupName, timeZoneGroupName, row, warningsData.warningMsgs, warningsData.infoMsgs );
        nameCell.setRenderer( new NameCellRenderer().init( tokensHighlighter,
                                                           this.isEditingUsersAllowed(),
                                                           this.handlersMap.editUser,
                                                           this.warningsController) );
        row.addCellObj( nameCell );

        if (this.isManageHierarchyColumnAvailable ())
        {
            var taskHierarchyManageCell = new TableCell( userListUserInfo );
            taskHierarchyManageCell.setRenderer(
                new TaskHierarchyManageCellRenderer().init(
                    this.canManageAccounts,
                    {
                        onChange: this.handlersMap.manageTaskHierarchy,
                        onMouseIn: this.handlersMap.showTaskHierarchyTooltip,
                        onMouseMove: this.handlersMap.moveTaskHierarchyTooltip,
                        onMouseOut: this.handlersMap.hideTaskHierarchyTooltip
                    }
                )
            );
            row.addCellObj( taskHierarchyManageCell );
        }

        if( this.isAssignmentsColumnAvailable() )
        {
            var taskHierarchyAssignmentsCell = new EditableCell().init( userListUserInfo, row );
            taskHierarchyAssignmentsCell.setRenderer(
                    new TaskHierarchyAssignmentsCellRenderer().init(
                            this.canManageAccounts,
                            {
                                onClick: this.handlersMap.editProjectAssignments,
                                onMouseIn: this.handlersMap.showAssignmentsTooltip,
                                onMouseMove: this.handlersMap.moveAssignmentsTooltip,
                                onMouseOut: this.handlersMap.hideAssignmentsTooltip
                            }
                    )
            );
            row.addCellObj( taskHierarchyAssignmentsCell );
        }

        if( this.isPTOColumnAvailable() )
        {
            var ptoCell = new TableCell( userListUserInfo );
            ptoCell.setRenderer( new PTOCellRenderer().init( this.handlersMap.editPTOBalance, this.ptoHoursPerDay ) );
            row.addCellObj( ptoCell );
        }

        if( this.isSickDaysColumnAvailable() )
        {
            var sickDaysCell = new TableCell( userListUserInfo );
            sickDaysCell.setRenderer( new SickDaysCellRenderer().init( this.handlersMap.editSickDaysBalance, this.ptoHoursPerDay ) );
            row.addCellObj( sickDaysCell );
        }

        if( this.isShowHistoryAvailable() )
        {
            var showPtoHistoryCell = new TableCell( userListUserInfo );
            showPtoHistoryCell.setRenderer( new ShowPtoHistoryCellRenderer().init( this.handlersMap.showPtoHistory ) );
            row.addCellObj( showPtoHistoryCell );

            var showSickDaysHistoryCell = new TableCell( userListUserInfo );
            showSickDaysHistoryCell.setRenderer( new ShowSickDaysHistoryCellRenderer().init( this.handlersMap.showSickHistory ) );
            row.addCellObj( showSickDaysHistoryCell );
        }

        var manageTTApproverCell = new TableCell( userListUserInfo );
        manageTTApproverCell.setRenderer(
            new ManageTTApproverCellRenderer().init(
                this.canManageAccounts,
                {
                    onClick: this.handlersMap.toggleIsTTApprover,
                    onMouseIn: this.handlersMap.showManageRightsTooltip,
                    onMouseMove: this.handlersMap.moveManageRightsTooltip,
                    onMouseOut: this.handlersMap.hideManageRightsTooltip
                }
            )
        );
        row.addCellObj( manageTTApproverCell );

        var usersForTTApproverCell = new EditableCell().init( userListUserInfo, row );
        usersForTTApproverCell.setRenderer(
            new UsersForTTApproverCellRenderer().init(
                {
                    onClick: this.handlersMap.showAssignUsersForTTApproverDialog,
                    onMouseIn: this.handlersMap.showAssignUsersForTTApproverTooltip,
                    onMouseMove: this.handlersMap.moveAssignUsersForTTApproverTooltip,
                    onMouseOut: this.handlersMap.hideAssignUsersForTTApproverTooltip
                },
                this.canManageAccounts
            )
        );

        row.addCellObj( usersForTTApproverCell );

        if ( this.isTTAColumnAvailable() )
        {
            var assignTTApproversCell = new AssignTTApproversCell().init( userListUserInfo, approversSortingProcessor(userListUserInfo.approversIds), row );
            assignTTApproversCell.setRenderer( new AssignTTApproversCellRenderer().init( {  onClick: this.handlersMap.showAssignTTApproversDialog,
                                                                                            onMouseIn: this.handlersMap.showAssignedApproversTooltip,
                                                                                            onMouseMove: this.handlersMap.moveAssignedApproversTooltip,
                                                                                            onMouseOut: this.handlersMap.hideAssignedApproversTooltip },
                                                                                            this.canManageAccounts ) );
            row.addCellObj( assignTTApproversCell );
        }

        if ( this.isManageLeaveBalancesColumnAvailable() )
        {
            var manageLeaveBalancesCell = new TableCell(userListUserInfo);
            manageLeaveBalancesCell.setRenderer(
                new ManageLeaveBalancesCellRenderer().init(
                    this.canManageAccounts,
                    {
                        onClick:this.handlersMap.switchManagePTOAndSickRight,
                        onMouseIn: this.handlersMap.showManageRightsTooltip,
                        onMouseMove: this.handlersMap.moveManageRightsTooltip,
                        onMouseOut: this.handlersMap.hideManageRightsTooltip
                    }
                )
            );
            row.addCellObj(manageLeaveBalancesCell);
        }

        if ( this.isPTOColumnAvailable() ) {
            var handlers = {
                onMouseOverHandler: function ( e, ptoData ) {
                    self.toggleHoverClassInRulesCell(e, true);
                    self.ptoRulesManager.showAccrualRulesTooltip(e, ptoData);
                },

                onMouseOutHandler: function ( e ) {
                    self.toggleHoverClassInRulesCell(e, false);
                    self.ptoRulesManager.hideAccrualRulesTooltip();
                },

                onMouseMoveHandler: function ( e ) { self.ptoRulesManager.moveAccrualRulesTooltip(e); },

                onClickHandler: function( e, ptoData, cell ){ self._onEditPTORulesClicked(e, ptoData, cell); }
            };

            var plannedPTOLeavesCell = new TableCell( userListUserInfo );
            plannedPTOLeavesCell.setRenderer( new PlannedPTOLeaveRenderer().init( this.ptoValueFormatter ) );
            row.addCellObj( plannedPTOLeavesCell );

            var ptoResetRulesCell = new TableCell( userListUserInfo );
            ptoResetRulesCell.setRenderer( new PtoResetRulesCell().init( this.ptoRulesFormatter, handlers ) );
            row.addCellObj( ptoResetRulesCell );

            var ptoAccrualRulesCell = new TableCell( userListUserInfo );
            ptoAccrualRulesCell.setRenderer( new PtoAccrualRulesCell().init( this.ptoRulesFormatter, handlers ) );
            row.addCellObj( ptoAccrualRulesCell );

            var ptoCapRulesCell = new TableCell( userListUserInfo );
            ptoCapRulesCell.setRenderer( new PtoCapRulesCell().init( this.ptoRulesFormatter, handlers ) );
            row.addCellObj( ptoCapRulesCell );

            var defaultRulesCell = new TableCell( userListUserInfo );
            defaultRulesCell.setRenderer( new PtoDefaultRulesCell().init( null, handlers ) );
            defaultRulesCell.setColspan(3);
            row.addCellObj( defaultRulesCell );

            var ptoControl = new TableCell( userListUserInfo );
            ptoControl.setRenderer( new PtoControlRenderer().init( this.handlersMap.togglePTOControlStatus ) );
            row.addCellObj( ptoControl );
        }

        if ( this.isSickDaysColumnAvailable() ) {
            handlers = {
                onMouseOverHandler: function ( e, ptoData ) {
                    self.toggleHoverClassInRulesCell(e, true);
                    self.sickRulesManager.showAccrualRulesTooltip(e, ptoData);
                },

                onMouseOutHandler: function ( e ) {
                    self.toggleHoverClassInRulesCell(e, false);
                    self.sickRulesManager.hideAccrualRulesTooltip();
                },

                onMouseMoveHandler: function ( e ) { self.sickRulesManager.moveAccrualRulesTooltip(e); },

                onClickHandler: function( e, sickData, cell ){ self._onEditSickRulesClicked(e, sickData, cell); }
            };

            var plannedSickLeavesCell = new TableCell( userListUserInfo );
            plannedSickLeavesCell.setRenderer( new PlannedSickLeaveRenderer().init( this.sickValueFormatter ) );
            row.addCellObj( plannedSickLeavesCell );

            var sickDaysResetRulesCell = new TableCell( userListUserInfo );
            sickDaysResetRulesCell.setRenderer( new SickDaysResetRulesCell().init( this.sickRulesFormatter, handlers ) );
            row.addCellObj( sickDaysResetRulesCell );

            var sickDaysAccrualRulesCell = new TableCell( userListUserInfo );
            sickDaysAccrualRulesCell.setRenderer( new SickDaysAccrualRulesCell().init( this.sickRulesFormatter, handlers ) );
            row.addCellObj( sickDaysAccrualRulesCell );

            var sickDaysCapRulesCell = new TableCell( userListUserInfo );
            sickDaysCapRulesCell.setRenderer( new SickDaysCapRulesCell().init( this.sickRulesFormatter, handlers ) );
            row.addCellObj( sickDaysCapRulesCell );

            var defaultSickDaysRulesCell = new TableCell( userListUserInfo );
            defaultSickDaysRulesCell.setRenderer( new SickDaysDefaultRulesCell().init( null, handlers ) );
            defaultSickDaysRulesCell.setColspan(3);
            row.addCellObj( defaultSickDaysRulesCell );

            var sickDaysControl = new TableCell( userListUserInfo );
            sickDaysControl.setRenderer( new SickDaysControlRenderer().init( this.handlersMap.toggleSickDaysControlStatus ) );
            row.addCellObj( sickDaysControl );
        }

        return row;

    },

    setUserTableStateAndUpdateUI: function ( userListBeans, groupsMap, timeZoneGroupMap, viewerSettings, sortOrder,
                                                                        tokens, isPTOEnabled, ptoHoursPerDay, allApprovers )
    {
        this.usersData = userListBeans;
        this.canManageLeaveBalances = viewerSettings.allowedToManageLeaveBalanceSettings;
        this.canManageAccounts = viewerSettings.allowedToManageAccountSettings;
        this.isPTOEnabled = isPTOEnabled;
        this.ptoHoursPerDay = ptoHoursPerDay;
        this.isAccountSettingsVisible = viewerSettings.showAccountSettings;

        this.deleteAllRowsFromModelAndDom();

        this.removeTrSpacers();

        this.tokensHighlighter = tokens && tokens.length > 0 ? new TokensHighlighter( tokens, UserTableController.PUNCTUATORS ) : new EmptyHighlighter();
        if ( this.ptoRulesManager )
            this.ptoRulesManager.reinitAccrualRulesControls( this.ptoHoursPerDay );

        if ( this.sickRulesManager )
            this.sickRulesManager.reinitAccrualRulesControls( this.ptoHoursPerDay );
        this.createBodyContent( this.usersData, groupsMap, timeZoneGroupMap, this.tokensHighlighter, allApprovers );

        this.sortOrder = sortOrder;

        this.buildModelRowsDOM( this.tbody );
        this.updateUI();
    },

    addCreatedUser: function(userListUserInfo, groupName, timeZoneGroupName, warnings, approversSortingProcessor) {
        var row = this.createRow(userListUserInfo, groupName, timeZoneGroupName, this.tokensHighlighter, warnings, approversSortingProcessor );
        this._insertRowIntoFirstPosition(row);
    },

    updateUser: function(userListUserInfo, groupName, timeZoneGroupName, warnings, isEditMyself, noHighlight, approversSortingProcessor)
    {
        var row = this.userIdRowMap[userListUserInfo.id];
        this._updateRow( userListUserInfo, groupName, timeZoneGroupName, row, warnings, noHighlight, approversSortingProcessor);
        if (isEditMyself)
        {
            var canManageLeaveBalancesChanged = this.canManageLeaveBalances != userListUserInfo.hasManagePTOAndSickRight;
            this.canManageLeaveBalances = userListUserInfo.hasManagePTOAndSickRight;
            if ( canManageLeaveBalancesChanged )
                this.updateUI();
        }

        PageOverlayTipsManager.getInstance().refreshDynamicTips();
    },

    deleteUser: function(userId){
        var row = $(this.userIdRowMap[userId].rowEl);
        var resetFirst = row.hasClass("first") && row.hasClass("newUser");

        this.removeRow(this.userIdRowMap[userId].getRowIdx());

        var newUsers = $(".newUser");
        if( !newUsers.length )
            this.removeTrSpacers();
        else
            newUsers.first().addClass("first");

        PageOverlayTipsManager.getInstance().refreshDynamicTips();
    },

    highlightRow: function ( row ) {
        $(row.getRowEl()).addClass("updatedUser");
        setTimeout( function() {
            $(row.getRowEl()).removeClass('updatedUser');
        }, 1500);
    },

    _updateRow: function(userListUserInfo, groupName, timeZoneGroupName, row, warnings, noHighlight, approversSortingProcessor)
    {
        var newRow = this.createRow(userListUserInfo, groupName, timeZoneGroupName, this.tokensHighlighter, warnings, approversSortingProcessor);
        this.replaceRowAndUpdateDOM(row.getRowIdx(), newRow);

        var classesToCopy = ["newUser", "first", "last", "selected"];
        for (var i=0; i<classesToCopy.length; i++)
        {
            if ($(row.getRowEl()).hasClass(classesToCopy[i]))
                $(newRow.getRowEl()).addClass(classesToCopy[i]);
        }

        if ( !noHighlight ) {
            this.highlightRow(newRow);
        }
    },

    removeTrSpacers: function() {
        $(".topSpacer, .bottomSpacer" ).remove();
    },

    _createSpacer: function(position)
    {
        var emptyRow = document.createElement("tr");
        emptyRow.className = position+"Spacer";

        var cellClasses = ["userNameCell"];

        cellClasses.push( "taskHierarchyManageCell" );
        cellClasses.push( "taskHierarchyAssignmentsCell" );

        if( this.isPTOColumnAvailable() )
        {
            cellClasses.push( "ptoCell" );
        }

        if( this.isShowHistoryAvailable() )
        {
            cellClasses.push( "showPtoHistoryCell" );
        }

        if ( this.isTTAEnabled )
        {
            cellClasses.push( "approverControlCell" );
            cellClasses.push( "usersForTTApproverCell" );
            cellClasses.push( "assignTTApproversCell" );
        }

        if ( this.isManageLeaveBalancesColumnAvailable() )
        {
            cellClasses.push( "manageBalanceCell" );
        }

        if ( this.isPTOColumnAvailable() ) {
            cellClasses.push( "plannedLeaveCell" );
            cellClasses.push( "ptoRulesCell resetRule" );
            cellClasses.push( "ptoRulesCell accrualRule" );
            cellClasses.push( "ptoRulesCell capRule" );
            cellClasses.push( "ptoControlCell" );
        }

        if ( this.isSickDaysColumnAvailable() ) {
            cellClasses.push( "sickDaysPlannedLeaveCell" );
            cellClasses.push( "sickDaysRulesCell resetRule" );
            cellClasses.push( "sickDaysRulesCell accrualRule" );
            cellClasses.push( "sickDaysRulesCell capRule" );
            cellClasses.push( "sickDaysControlCell" );
        }

        for (var i=0; i<cellClasses.length; i++)
        {
            var emptyRowTd = document.createElement("td");
            emptyRowTd.className = cellClasses[i];
            var emptyTextNode = document.createTextNode(" ");
            emptyRowTd.appendChild(emptyTextNode);
            emptyRow.appendChild(emptyRowTd);
        }

        return emptyRow;
    },

    _insertRowIntoFirstPosition: function(row)
    {
        this.setupRow( 0, row );
        this.rows.unshift(row);
        for( var i = 1; i < this.rows.length; i++ )
        {
            var nextRow = this.rows[i];
            nextRow.setRowIdx(i);
        }

        var rowElClasses = this.getRowElClasses( 0 );

        if(!$('.newUser').length) {
            var topEmptyRow = this._createSpacer("top");
            var bottomEmptyRow = this._createSpacer("bottom");

            this.tbody.insertBefore(bottomEmptyRow, this.tbody.firstChild);
            this.tbody.insertBefore(row.getRowEl(),this.tbody.firstChild);
            this.tbody.insertBefore(topEmptyRow, row.getRowEl());

            rowElClasses.push("last");
        } else {
            var firstNewUserEl = $(".newUser.first" )[0];
            this.tbody.insertBefore(row.getRowEl(), firstNewUserEl);
            $(".newUser.first" ).removeClass("first");
        }

        row.writeCellsContent();
        rowElClasses.push("newUser");
        rowElClasses.push("first");
        row.getRowEl().className = rowElClasses.join(" ");
    },

    isPTOColumnVisible: function ()
    {
        return this.isPTOEnabled && this.canManageLeaveBalances;
    },

    isTTAColumnVisible: function ()
    {
        return this.isTTAEnabled  && this.isAccountSettingsVisible;
    },

    isManageHierarchyColumnAvailable: function()
    {
        return this.isAccountSettingsVisible;
    },

    isAssignmentsColumnAvailable: function()
    {
        return this.isPASEnabled && this.isAccountSettingsVisible;
    },

    isPTOColumnAvailable: function ()
    {
        return this.isPTOEnabled && this.canManageLeaveBalances;
    },

    isSickDaysColumnAvailable: function ()
    {
        return this.isSickEnabled && this.canManageLeaveBalances;
    },

    isManageLeaveBalancesColumnAvailable: function ()
    {
        return (this.isPTOEnabled || this.isSickEnabled ) && this.isAccountSettingsVisible;
    },

    isTTAColumnAvailable: function ()
    {
        return this.isTTAEnabled && this.isAccountSettingsVisible;
    },

    isShowHistoryAvailable: function ()
    {
        return this.canManageLeaveBalances;
    },

    isEditingUsersAllowed: function ()
    {
        return this.canManageAccounts;
    },

    updateUserGroups: function (userIdGroupIdMap, groupMap)
    {
        for( var i = 0; i < this.rows.length; i++ )
        {
            var row = this.rows[i];

            row.userListUserInfo.userGroupId = userIdGroupIdMap[row.userListUserInfo.id];

            var cell = row.getCell(0);
            var name = groupMap[row.userListUserInfo.userGroupId];

            //if user was concurrently deleted he isn't in userGroupMap but we do not react
            if(name)
                cell.updateGroupName(name);
        }
    },

    updateTimeZoneGroups: function (timeZoneIdGroupIdMap, timeZoneGroupsMap)
    {
        for(var i = 0; i < this.rows.length; i++)
        {
            var row = this.rows[i];

            row.userListUserInfo.timeZoneGroupId = timeZoneIdGroupIdMap[row.userListUserInfo.id];

            var cell = row.getCell(0);
            var name = timeZoneGroupsMap[row.userListUserInfo.timeZoneGroupId];

            if(name)
                cell.updateTimeZoneGroupName(name);
        }
    },

    addRowSelection: function (row)
    {
        row.addClass("selected");
    },

    clearRowSelection: function (row)
    {
        row.removeClass("selected");
    },

    showLoadingHistoryProgress: function ( cell, spinner )
    {
        cell.addClass("loading");
        spinner.spinInElement( $(cell.find(".ptoHistorySpinnerPlaceholder" ).get( 0 )) );
    },

    hideLoadingHistoryProgress: function ( cell, spinner )
    {
        cell.removeClass("loading");
        spinner.stop();
    },

    updateTaskHierarchyAssignmentsCellAfterDialogOpened: function( cellObj, data )
    {
        var allProjectsAssigned = data.allProjectsAssigned;
        var numberOfAssignedProjects = data.numberOfActiveAssignedProjects;
        var userInfo = cellObj.getRow().getUserListUserInfo();
        userInfo.allProjectsAssigned = allProjectsAssigned;
        userInfo.numberOfAssignedProjects = numberOfAssignedProjects;
        this.updateCell(cellObj);
    },

    updateUsersForTTApproverCellAfterDialogOpened: function (cellObj, assignedUsersData)
    {
        cellObj.getRow().getUserListUserInfo().canManageAllUsers = assignedUsersData.canManageAllUsers;
        cellObj.getRow().getUserListUserInfo().numberOfAssignedUsers = assignedUsersData.numberOfAssignedUsers;
        this.updateCell(cellObj);
    },

    updateAssignTTApproversCellAfterDialogOpened: function (cellObj, manageApproversData, userApprovers)
    {
        cellObj.getRow().getUserListUserInfo().requiresTTApproval = manageApproversData.userRequiresApproval;
        cellObj.getRow().getUserListUserInfo().userApproverIds = manageApproversData.userApproverIds;
        cellObj.setUserApprovers( userApprovers );
        this.updateCell(cellObj);
    },

    updateAssignTTApproversCellWithActualApprovers: function (cellObj, actualApprovers)
    {
        cellObj.setUserApprovers( actualApprovers );
        this.updateCell(cellObj);
    },

    getVisibleUsersIds: function () {
        var ids = [];
        for ( var i=0; i < this.rows.length; i++ )
            ids.push(this.rows[i].getUserListUserInfo().id);
        return ids;
    },

    updateUI: function ()
    {
        this.tableRenderer.updateUI(
                this,
                this.isEditingUsersAllowed(),
                (this.isPTOEnabled || this.isSickEnabled ) && this.canManageLeaveBalances,
                this.isPTOEnabled,
                this.isSickEnabled,
                this.isTTAColumnVisible(),
                this.isAssignmentsColumnAvailable(),
                this.isAccountSettingsVisible,
                this.sortOrder.order,
                this.sortOrder.asc
        );
    }
});