function UsersForTTApproverCellRenderer ()
{
    this.constructor = UsersForTTApproverCellRenderer;
}
UsersForTTApproverCellRenderer.prototype = _.create(EditableCellRenderer.prototype);

UsersForTTApproverCellRenderer.prototype.init = function ( handlers, viewerCanManageAccounts )
{
    EditableCellRenderer.prototype.init.call( this );

    this.handlers = handlers;
    this.canManageAccounts = viewerCanManageAccounts;

    return this;
};

UsersForTTApproverCellRenderer.prototype.getCellClassName = function ( cell )
{
    var classes = ["usersForTTApproverCell", "manageApproveCell"];
    var userInfo = cell.getValue();

    if (!userInfo.hasManageOtherTTRight)
        classes.push("disabled");
    else
    {
        classes.push("enabled");
        if (!userInfo.canManageAllUsers)
        {
            var numberOfAssignedUsers = userInfo.numberOfAssignedUsers;
            if( !numberOfAssignedUsers || numberOfAssignedUsers == 0 )
                classes.push( "noUsers" );
        }
    }
    return classes;
};

UsersForTTApproverCellRenderer.prototype.generateContent = function ( cell )
{
    var self = this;

    var cellEl = cell.getCellEl();
    var userInfo = cell.getValue();
    var jQcell = $( cellEl );

    this.updateContent(cell);

    if ( this.canManageAccounts && userInfo.enabled && userInfo.hasManageOtherTTRight )
    {
        cellEl.onclick = function ()
        {
            self.handlers.onClick({  userId: cell.getValue().id,
                            offset: "0px 0px",
                            cellObj: cell,
                            cellEl: cellEl
                         });
        };

        jQcell.hover(
            function ( event ) {
                self.handlers.onMouseIn(
                    {},
                    event );
            },
            function () {
                self.handlers.onMouseOut();
            }

        );
        jQcell.mousemove(
            function( event ) {
                self.handlers.onMouseMove( event );
            }
        );
        jQcell.click (
            function( event ) {
                self.handlers.onMouseOut();
            }
        );
    }
};

UsersForTTApproverCellRenderer.prototype.formatInnerText = function ( cell )
{
    var userInfo = cell.getValue();
    var numberOfAssignedUsers = userInfo.numberOfAssignedUsers;

    var text;

    if (!userInfo.hasManageOtherTTRight)
        text = "-";
    else if (userInfo.canManageAllUsers)
        text = messageResource.getMessage("user_list.manager.approves.all_users");
    else if (!numberOfAssignedUsers || numberOfAssignedUsers == 0)
        text = messageResource.getMessage("user_list.manager.approves.no_users");
    else if (numberOfAssignedUsers == 1)
        text = messageResource.getMessage("user_list.manager.approves.one_user");
    else
        text = messageResource.getMessage("user_list.manager.approves.n_users", [numberOfAssignedUsers]);

    return text;
};

UsersForTTApproverCellRenderer.prototype.updateContent = function ( cell )
{
    cell.getCellEl().innerHTML = this.formatInnerText(cell);
};