function CreateUserWorkdayDurationsTable() {}

CreateUserWorkdayDurationsTable.prototype = _.create(TabularDataTable.prototype);

CreateUserWorkdayDurationsTable.prototype.init = function( userManagementData, accountSettings, changesCallback, validationCallback )
{
    this.userManagementData = userManagementData;

    TabularDataTable.prototype.init.call( this, this.buildHeader(), changesCallback, validationCallback );

    var durationsCellRenderer = new DurationsCellRenderer().init( changesCallback, userManagementData.weekStartDay, false, userManagementData.decimalSeparator );
    var durationsCellValidator = new DurationsCellValidator().init();
    var totalCellRenderer = new TotalCellRenderer().init();
    var setAsDefaultCellRenderer = new SetAsDefaultCellRenderer().init();

    var initialScheduleRow = new WorkdayDurationsTableRow().init( userManagementData.defaultWorkSchedule, 0, false, changesCallback );
    initialScheduleRow.setMarkedAsDefault( accountSettings.defaultInitialSchedule );
    initialScheduleRow.setInPast( false );

    var initialScheduleCellRenderer = new WorkingScheduleCellRenderer().init();
    var initialScheduleHeaderCell = new TabularDataTableCell().init( null, new NotNullCellValidator().init(), this );
    initialScheduleHeaderCell.setRenderer( initialScheduleCellRenderer );
    initialScheduleRow.addCellObj( initialScheduleHeaderCell, false );
    
    this.durationsCell = initialScheduleRow.createDurationsCell( accountSettings.initialSchedule, durationsCellValidator, durationsCellRenderer );
    initialScheduleRow.addCellObj( this.durationsCell, true );
    var totalCell = initialScheduleRow.createTotalCell( WorkdayDurationsEditor.sumDurations( accountSettings.initialSchedule ), totalCellRenderer );
    initialScheduleRow.addCellObj( totalCell, false );
    var setAsDefaultCell = initialScheduleRow.createSetAsDefaultCell( accountSettings.defaultInitialSchedule, setAsDefaultCellRenderer );
    initialScheduleRow.addCellObj( setAsDefaultCell, false );

    initialScheduleRow.setupTotalAndDefaultCellUpdates( totalCell, setAsDefaultCell, this.durationsCell );

    this.addRow( initialScheduleRow );

    return this;
};

CreateUserWorkdayDurationsTable.prototype.buildHeader = function()
{
    return WorkdayDurationsEditor.buildTableHeader( this.userManagementData.weekStartDay );
};

CreateUserWorkdayDurationsTable.prototype.getBaseTableClasses = function()
{
    return TabularDataTable.prototype.getBaseTableClasses.call( this ).concat(["wdTable", "createMode"]);
};

CreateUserWorkdayDurationsTable.prototype.getInitialSchedule = function()
{
    return this.durationsCell.getValue();
};

CreateUserWorkdayDurationsTable.prototype.isDefaultInitialSchedule = function()
{
    return this.durationsCell.getRow().isMarkedAsDefault();
};

CreateUserWorkdayDurationsTable.prototype.getSchedules = function()
{
    return null;
};
