function EditUserWorkdayDurationsTable() {}

EditUserWorkdayDurationsTable.prototype = _.create(TabularDataWithEffectiveDatesTable.prototype);

EditUserWorkdayDurationsTable.prototype.init = function( userManagementData, accountSettings, changesCallback, validationCallback, sectionContainer, corporateSchedule, currentTimeInUserTimeZone )
{
    this.currentDate = userManagementData.todayInViewerTimeZone.parseJSONDate().truncateTime();
    this.userManagementData = userManagementData;
    this.changesCallback = changesCallback;

    TabularDataWithEffectiveDatesTable.prototype.init.call( this, this.buildHeader(), changesCallback, validationCallback, currentTimeInUserTimeZone );

    this.durationsCellRenderer = new DurationsCellRenderer().init( changesCallback, userManagementData.weekStartDay, false, userManagementData.decimalSeparator );
    this.durationsCellValidator = new DurationsCellValidator().init();
    this.totalCellRenderer = new TotalCellRenderer().init();
    this.effectiveDateCellRenderer = new EffectiveDateCellRenderer().init(
            userManagementData.invertedDaysMap,
            userManagementData.weekStartDay,
            changesCallback,
            messageResource.getMessage("user.add_edit.js.wd_editor.effective_date_prefix"),
            sectionContainer,
            corporateSchedule
    );
    this.effectiveDateCellValidator = new NotNullCellValidator().init();
    this.setAsDefaultCellRenderer = new SetAsDefaultCellRenderer().init();
    this.removeRowCellRenderer = new RemoveRowCellRenderer().init();

    var initialScheduleCellRenderer = new InitialScheduleCellRenderer().init();

    var initialScheduleRow = new WorkdayDurationsTableRow().init( userManagementData.defaultWorkSchedule, 1, false, changesCallback );
    initialScheduleRow.setMarkedAsDefault( accountSettings.defaultInitialSchedule );
    initialScheduleRow.setInPast( false );

    var initialScheduleHeaderCell = new TabularDataTableCell().init( null, new NotNullCellValidator().init(), this );
    initialScheduleHeaderCell.setRenderer( initialScheduleCellRenderer );
    initialScheduleRow.addCellObj( initialScheduleHeaderCell, false );

    this.initialScheduleDurationsCell = initialScheduleRow.createDurationsCell( accountSettings.initialSchedule, this.durationsCellValidator, this.durationsCellRenderer );
    initialScheduleRow.addCellObj( this.initialScheduleDurationsCell, true );

    var totalCell = initialScheduleRow.createTotalCell( WorkdayDurationsEditor.sumDurations( accountSettings.initialSchedule ), this.totalCellRenderer );
    initialScheduleRow.addCellObj( totalCell, false );

    var setAsDefaultCell = initialScheduleRow.createSetAsDefaultCell( accountSettings.defaultInitialSchedule, this.setAsDefaultCellRenderer );
    initialScheduleRow.addCellObj( setAsDefaultCell, false );

    initialScheduleRow.setupTotalAndDefaultCellUpdates( totalCell, setAsDefaultCell, this.initialScheduleDurationsCell );

    var emptyCell = new TableCell( "", this );
    var emptyRenderer = new TableCellRenderer();
    emptyCell.setRenderer( emptyRenderer );
    initialScheduleRow.addCellObj( emptyCell, false );

    this.addRow( initialScheduleRow );

    for( var i = 0; i < accountSettings.workSchedule.length; i++ )
        this.addRow( this.createScheduleRow( accountSettings.workSchedule[i], false ) );

    this.getTableEl().appendChild( this.buildFooter() );

    return this;
};

EditUserWorkdayDurationsTable.prototype.getInitialSchedule = function()
{
    return this.initialScheduleDurationsCell.getValue();
};

EditUserWorkdayDurationsTable.prototype.isDefaultInitialSchedule = function()
{
    return this.initialScheduleDurationsCell.getRow().isMarkedAsDefault();
};

EditUserWorkdayDurationsTable.prototype.getSchedules = function()
{
    var schedules = [];
    for( var i = 1; i < this.rows.length; i++ )
    {
        var aRow = this.rows[i];
        if( !aRow.isMarkedToRemove() )
        schedules.push({
            effectiveDate: aRow.cells[0].getValue().toJSON(),
            durations: aRow.cells[1].getValue(),
            sameAsDefault: aRow.isMarkedAsDefault()
        });
    }
    return schedules;
};

EditUserWorkdayDurationsTable.prototype.getEffectiveRowIdx = function()
{
    var effectiveRowIdx = TabularDataWithEffectiveDatesTable.prototype.getEffectiveRowIdx.call( this );
    return effectiveRowIdx == null ? 0 : effectiveRowIdx;
};

EditUserWorkdayDurationsTable.prototype.resetDefaultIfDateInThePastAndUpdatePastFlag = function( effectiveDateCell )
{
    var inPast = effectiveDateCell.getValue() && effectiveDateCell.getValue().truncateTime() < this.currentDate;
    var row = effectiveDateCell.getRow();

    if( inPast )
    {
        var setAsDefaultCell = row.cells[ 3 ];
        if( setAsDefaultCell.getValue() == true )
        {
            setAsDefaultCell.setValue( false );
            setAsDefaultCell.updateUI();
        }
    }

    if( row.isInPast() != inPast )
    {
        row.setInPast( inPast );
        row.updateRowClasses();
    }
};

EditUserWorkdayDurationsTable.prototype.getBaseRowClasses = function( rowIdx )
{
    var classes = TabularDataWithEffectiveDatesTable.prototype.getBaseRowClasses.call( this, rowIdx );
    return classes.concat( this.getRow( rowIdx ).getRowClasses() );
};

EditUserWorkdayDurationsTable.prototype.updateEffectiveRow = function()
{
    var effectiveRowIdx = TabularDataWithEffectiveDatesTable.prototype.updateEffectiveRow.call( this );
    var initialScheduleRow = this.getRow(0);
    initialScheduleRow.setInPast( effectiveRowIdx != 0 );
    if( initialScheduleRow.isInPast() )
    {
        initialScheduleRow.cells[3].setValueNoValidate( false );
        initialScheduleRow.setMarkedAsDefault( false );
    }
    initialScheduleRow.updateUI();
};

EditUserWorkdayDurationsTable.prototype.createScheduleRow = function( schedule, deletable )
{
    var row = new WorkdayDurationsTableRow().init( this.userManagementData.defaultWorkSchedule, 1, deletable, this.changesCallback );
    var inPast = schedule.effectiveDate && schedule.effectiveDate.parseJSONDate() < this.currentDate;
    row.setMarkedAsDefault( !inPast && schedule.sameAsDefault );
    row.setInPast( inPast );

    var self = this;
    var effectiveDateCell = new TabularDataTableCell().init(
            schedule.effectiveDate ? schedule.effectiveDate.parseJSONDate() : null,
            this.effectiveDateCellValidator,
            this
    );
    effectiveDateCell.setRenderer( this.effectiveDateCellRenderer );
    effectiveDateCell.onChange( function() {
        self.markRowsWithEqualEffectiveDatesAsInvalid();
        self.updateEffectiveRow();
        self.resetDefaultIfDateInThePastAndUpdatePastFlag(effectiveDateCell);
    } );
    row.addCellObj( effectiveDateCell, true );



    var durationsCell = row.createDurationsCell( schedule.durations, this.durationsCellValidator, this.durationsCellRenderer );
    row.addCellObj( durationsCell, true );

    var totalCell = row.createTotalCell( WorkdayDurationsEditor.sumDurations( schedule.durations ), this.totalCellRenderer );
    row.addCellObj( totalCell, false );

    var setAsDefaultCell = row.createSetAsDefaultCell( row.isMarkedAsDefault(), this.setAsDefaultCellRenderer );
    row.addCellObj( setAsDefaultCell, false );

    row.setupTotalAndDefaultCellUpdates( totalCell, setAsDefaultCell, durationsCell );

    var removeRowCell = new TableCell( null, this );
    removeRowCell.setRenderer( this.removeRowCellRenderer );
    row.addCellObj( removeRowCell, false );

    return row;
};

EditUserWorkdayDurationsTable.prototype.buildHeader = function()
{
    return WorkdayDurationsEditor.buildTableHeader( this.userManagementData.weekStartDay )
};

EditUserWorkdayDurationsTable.prototype.buildFooter = function()
{
    var tFoot = document.createElement("tfoot");
    var tr = document.createElement("tr");
    tr.className = "addRecord";
    var addRecTd = document.createElement("td");
    addRecTd.className = "addRecordCell";
    var a1 = document.createElement("a");
    var a2 = document.createElement("a");
    a1.className = "img";
    a2.className = "text";
    a1.innerHTML = "&nbsp;";
    a2.innerHTML = messageResource.getMessage("user.add_edit.js.wd_editor.new_schedule");
    
    var self = this;
    var addEmptyRowFn = function() { self.addEmptyRow(); };
    a1.onclick = addEmptyRowFn; 
    a2.onclick = addEmptyRowFn;
    
    tFoot.appendChild( tr );
    tr.appendChild(addRecTd);
    addRecTd.appendChild(a1);
    addRecTd.appendChild(a2);
    return tFoot;
};


EditUserWorkdayDurationsTable.prototype.addEmptyRow = function(  )
{
    TabularDataWithEffectiveDatesTable.prototype.addEmptyRow.call( this );
    this.changesCallback();
};

EditUserWorkdayDurationsTable.prototype.createEmptyRow = function()
{
    var currentDateStr = this.currentDate.toJSON();
    for( var i = 1; i < this.rows.length; i++ )
    {
        if( this.rows[i].isMarkedToRemove() )
            continue;

        var date = this.rows[i].cells[0].getValue();
        if( date && date.toJSON() == currentDateStr )
        {
            currentDateStr = null;
            break;
        }
    }

    var emptySchedule = {
        effectiveDate: currentDateStr,
        durations: this.userManagementData.defaultWorkSchedule.slice(0 ),
        sameAsDefault: true
    };
    return this.createScheduleRow( emptySchedule, true );
};

EditUserWorkdayDurationsTable.prototype.getBaseTableClasses = function()
{
    return TabularDataTable.prototype.getBaseTableClasses.call( this ).concat(["wdTable", "editMode"]);
};
