var KEY_CODE_UP = 38;
var KEY_CODE_DOWN = 40;
var KEY_CODE_ESC = 27;
var KEY_CODE_ENTER = 13;
var KEY_CODE_TAB = 9;
var KEY_CODE_DEL = 46;
var KEY_CODE_SPACE = 32;
var KEY_CODE_PAGE_UP = 33;
var KEY_CODE_PAGE_DOWN = 34;

var canwork = 1;
var opened_popups = [];

/**
 * This function is wrapper for window.popup. Use it to check if popups are blocked in a browser
 * and show warning message. After popup window is focused.
 *
 * Parameters are equal original window.popup function.
 *
 */
function windowOpen(URL, name, specs, replace)
{
    var popupWindow = window.open(URL, name, specs, replace);

    if (popupWindow == null)
        alert( messageResource.getMessage("common.popups_blocked") );
    else
        popupWindow.focus();
}

function popup(url, name, width, height, menubar, toolbar, returnWindow, toAddToOpenedPopups)
{
    if (canwork)
    {
        canwork = 0;

        if (!name)      name    ="windowedPopup";
        if (!height)    height  = 500;
        if (!width)     width   = 830;
        if (!menubar)   menubar = "no";
        if (!toolbar)   toolbar = "no";

        if (name != 'windowedPopup')
        {
            if(window[name] && !window[name].closed)
                window[name].close();
        }

        var w = null;
        try {
            w = window.open(url, name,
                            "directories=no,height=" + height +
                            ",width=" + width +
                            ",location=no,menubar=" +  menubar +
                            ",resizable=yes,scrollbars=yes,status=no,toolbar=" + toolbar);

            if (w == null)
            {
                alert( messageResource.getMessage("common.popups_blocked") );
                return;
            }


            w.focus();

            if (typeof toAddToOpenedPopups == typeof undefined || // if parameter is not specified
                toAddToOpenedPopups) { // or specified and has value 'true'
                addPopup(w);
            }
        } catch (e) {
            // Can be thrown when user works offline.
        }

        canwork = 1;

        if (returnWindow && w != null) return w;
    }
}

/*
 * Ads the specified pop-up window to opened pop-up windows collection to close them by closeAllOpenedPopups() function
 * on document unloading.
 *
 * @param popupWindow - the pop-up to add.
 */
function addPopup(popupWindow) {
    opened_popups[opened_popups.length] = popupWindow;
}

/*
 * Closes all opened pop-up windows added to opened_popups collection by addPopup(popupWindow) function. This function
 * should be called by onload handler of the document body.
 */
function closeAllOpenedPopups() {
    for (var i = 0; i < opened_popups.length; i++) {
        var popupWindow = opened_popups[i];
        if (!popupWindow.closed) {
            popupWindow.close();
        }
    }
}

function show_in_main_window( messageKey )
{
    if (window.opener && window.name.indexOf('windowedPopup') >= 0)
    {
        var location = window.location.href;
        if (messageKey != null)
            location = addRequestParams(window.location.href, "e=".concat( messageKey ));
        window.opener.location.href = location;
        window.opener.focus();
        window.close();
    }
}

/*
 * Adds the specified params (string like 'param1=value1&param2=value2... etc.') to the specified URL.
 */
function addRequestParams(url, paramsStr) {
    var result = url;

    if (url.indexOf("?") >= 0) {
        result = result.concat("&");
    } else {
        result = result.concat("?");
    }

    return result.concat(paramsStr);
}

function notEmpty(obj, msg)
{
    if ( trim(obj.value).length > 0 )
    {
        if ( msg && msg != '')
        {
            alert(msg)
            obj.focus();
        }
        return false;
    }
    return true;
}

// Select/unselect checkboxes list
// form - form object
// name - checkboxes name
// state - true/false
function setCheckboxesState(form, name, state)
{
    for (var i = 0; i < form.length; i++) {
        if (form.elements[i].type == "checkbox" &&
            form.elements[i].name.search(name) != -1) {
            form.elements[i].checked = state;
        }
    }
}

// Select/unselect options in multiple select
// s - select object
// state - true/false
function setOptionsState(s, state)
{
    for (var i = 0; i < s.options.length; i++) {
        s.options[i].selected = state;
    }
}

// Get selected value from a selector object
function getVal(el)
{
    if (el.type == "select-one")
        if(el.selectedIndex != undefined)
            return parseInt(el.options[el.selectedIndex].value, 10);
        else
            // Opera 6.x
            return parseInt(el.options[el.value].value, 10);
    else
        return parseInt(el.value, 10);
}

function isEmailValid(email)
{
    var str = email.trim();
    return /.+@.+/i.test(str);
}

function isNotEmpty(str)
{
    return trim(str).length > 0;
}

/***
 * Function trims specified value
 * @param str - if str converted to boolean is false then empty string will be returned
 * @returns {String}
 */
function trim(str)
{
    str = String(str);

    var whitespace = ' \n\r\t\f\x0b\xa0\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200a\u200b\u2028\u2029\u3000';

    for (var lUk = 0; lUk < str.length; lUk++) {
        if (whitespace.indexOf(str.charAt(lUk)) === -1)
            break;
    }

    if(lUk==str.length) return '';

    for (var rUk = str.length - 1;; rUk--) {
        if (whitespace.indexOf(str.charAt(rUk)) === -1)
            break;
    }

    return str.substring(lUk,rUk+1);
}

// Function normalizes specified string
function normalize(str)
{
    var re = /\s{2,}/;

    while (re.test(str))
    {
        str = str.replace(re, " ");
    }

    return str;
}

// Function trims all text form fields
function trimAllTextFields(f)
{
    for (var i = 0; i < f.length; i++) {
        if (f.elements[i].type == 'text') {
            f.elements[i].value = trim(f.elements[i].value);
        }
    }
}

// Function normalizes all text form fields
function normalizeAllTextFields(f)
{
    for (var i = 0; i < f.length; i++) {
        if (f.elements[i].type == 'text') {
            f.elements[i].value = normalize(f.elements[i].value);
        }
    }
}

// Function normalizes specified string
function toUnixLineFeeds(str)
{
    return str.replace(/\r/g, '');
}

function shiftTo(obj, x, y, isOriginAtBottom) {
    var units = (typeof obj.style.left == "string") ? "px" : 0;

    obj.style.left = x + units;

    if (isOriginAtBottom)
    {
        obj.style.top = y - getObjectHeight(obj.id) + units;
    }
    else
        obj.style.top = y + units;
}

function isMSIE() {
    return navigator.userAgent.indexOf("Trident") != -1;
}

function isAndroid() {
    return navigator.userAgent.indexOf("Android") != -1;
}

function isIE8() {
    return navigator.userAgent.toLowerCase().indexOf("msie 8") > -1;
}

/**
 * Returns true when the browser is IE 8 or IE 9
 * @returns {boolean} true when the browser is IE 8 or IE 9
 */
function isOldIE()
{
    return isIE8() || navigator.userAgent.toLowerCase().indexOf("msie 9") > -1;
}

function isMobile() {
    return /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent);
}

function calculateCoords(event) {
    var targetEl = event.srcElement;
    if (!targetEl)
        targetEl = event.target;
    return calculateObjectCoords(targetEl);
}

function calculateObjectCoords(obj) {
    return {x:findPosX(obj), y:findPosY(obj)};
}

function findPosX(obj)
{
    var curleft = 0;
    if (obj.offsetParent)
    {
        while (obj.offsetParent)
        {
            curleft += obj.offsetLeft;
            obj = obj.offsetParent;
        }
    }
    else if (obj.x)
        curleft += obj.x;
    return curleft;
}

function findPosY(obj)
{
    var curtop = 0;
    if (obj.offsetParent)
    {
        while (obj.offsetParent)
        {
            curtop += obj.offsetTop;
            obj = obj.offsetParent;
        }
    }
    else if (obj.y)
        curtop += obj.y;
    return curtop;
}

function getBodyScrollLeft()
{
    var scrollX = 0;
    if( document.body && typeof document.body.scrollLeft != undefined )
    {
        scrollX += document.body.scrollLeft;
        if( document.body.parentNode && document.body.parentNode.scrollLeft && document.body.scrollLeft != document.body.parentNode.scrollLeft )
        {
            scrollX += document.body.parentNode.scrollLeft;
        }
    }
    else if( window.pageXOffset )
    {
        scrollX += window.pageXOffset;
    }
    return scrollX;
}

function getBodyScrollTop()
{
    var scrollY = 0;
    if( document.body && typeof document.body.scrollTop != undefined )
    {
        scrollY += document.body.scrollTop;
        if( document.body.parentNode && document.body.parentNode.scrollTop && document.body.scrollTop != document.body.parentNode.scrollTop )
        {
            scrollY += document.body.parentNode.scrollTop;
        }
    }
    else if( window.pageYOffset )
    {
        scrollY += window.pageYOffset;
    }
    return scrollY;
}

function getViewport()
{
    var scrollLeft = getBodyScrollLeft();
    var scrollTop = getBodyScrollTop();
    return {
        leftUpperCorner : { x:scrollLeft, y:scrollTop },
        rightBottomCorner : { x:scrollLeft + getInsideWindowWidth(), y:scrollTop + getInsideWindowHeight() }
    }
}

function scrollToElementIfNeeded ( element )
{
    var elementOffsetLeft = findPosX(element);
    var elementOffsetTop = findPosY(element);
    var leftUpperCorner = { x:elementOffsetLeft, y:elementOffsetTop };
    var rightBottomCorner = { x:elementOffsetLeft + element.clientWidth, y:elementOffsetTop + element.clientHeight };
    var viewport = getViewport();
    var viewportLeftUpperCorner = viewport.leftUpperCorner;
    var viewportRightBottomCorner = viewport.rightBottomCorner;

    var spacing = document.body.clientHeight/3; //to create a space between element and window border
    var scrollY = 0;
    if ( rightBottomCorner.y > viewportRightBottomCorner.y )
        scrollY = rightBottomCorner.y + spacing - ( viewportRightBottomCorner.y );
    else if ( leftUpperCorner.y < viewportLeftUpperCorner.y )
        scrollY = leftUpperCorner.y - (viewportLeftUpperCorner.y + spacing);
    if ( scrollY != 0 )
        window.scrollBy( 0, scrollY );
}

function scrollWindowIfNeeded(object, alignRight) {
    var $object = $(object);
    var objectOffsetLeft = findPosX(object);
    var objectOffsetTop = findPosY(object);
    var leftUpperCorner = { x:objectOffsetLeft, y:objectOffsetTop };
    var rightBottomCorner = { x:objectOffsetLeft + $object.outerWidth(true), y:objectOffsetTop + $object.outerHeight(true) };
    var viewport = getViewport();
    var viewportLeftUpperCorner = viewport.leftUpperCorner;
    var viewportRightBottomCorner = viewport.rightBottomCorner;

    var scrollX = 0;
    if (leftUpperCorner.x < viewportLeftUpperCorner.x) {
        scrollX = leftUpperCorner.x - viewportLeftUpperCorner.x;
    } else if (alignRight && rightBottomCorner.x > viewportRightBottomCorner.x) {
        scrollX = rightBottomCorner.x - viewportRightBottomCorner.x
    }

    var scrollY = 0;
    if (rightBottomCorner.y > viewportRightBottomCorner.y)
        scrollY = rightBottomCorner.y - viewportRightBottomCorner.y;

    if (scrollX != 0 || scrollY != 0) {
        /*alert("VIEWPORT\ntop-left.x = " + viewportLeftUpperCorner.x + ", top-left.y = " + viewportLeftUpperCorner.y +
              "; bottom-right.x = " + viewportRightBottomCorner.x + ", bottom-right.y = " + viewportRightBottomCorner.y + "\n\n" +
              "object\ntop-left.x = " + leftUpperCorner.x + ", top-left.y = " + leftUpperCorner.y +
              "; bottom-right.x = " + rightBottomCorner.x + ", bottom-right.y = " + rightBottomCorner.y + "\n\n" +
              "SCROLL BY\nscrollX = " + scrollX + ", scrollY = " + scrollY);*/
        window.scrollBy(scrollX, scrollY);
    }
}

/**
 * Smoothly scrolls the window down so that the needed element is fully visible.
 * Only scrolls down, doesn't scroll up even if the element is not visible.
 * Requires jQuery.
 *
 * @param element element to scroll to (either an element, selector or jQuery object)
 * @param [offset] how many pixels should be between the element bottom and the window bottom. Default is 0.
 * @param [duration] how long the animation will take. If not specified, then jQuery default is used.
 */
function smoothlyScrollDownToElement( element, offset, duration )
{
    element = $(element);

    if( offset == undefined )
        offset = 0;

    var initialScrollTop = $(document).scrollTop();
    var scrollTopChange =
            Math.max(0, -initialScrollTop + element.offset().top + element.outerHeight() - $(window).height() + offset);

    $("body,html").animate( { scrollTop: initialScrollTop + scrollTopChange }, duration );

    // Although we can use scrollTop() method on $(document), animate() doesn't work on it.
    // Instead, we use animate() on body and html elements (some browsers require body and some html).
}

/**
 * http://stackoverflow.com/a/7557433
 */
function isElementInViewPort(el)
{
    var rect = el.getBoundingClientRect();

    return (
            rect.top >= 0 &&
            rect.left >= 0 &&
            rect.bottom <= (window.innerHeight || document.documentElement.clientHeight) &&
            rect.right <= (window.innerWidth || document.documentElement.clientWidth)
            );
}


/*
 * Unifies new line symbol in the specified string. It is needed for unified string length determining in various
 * browsers, because in some browser (e.g. IE) new line symbol is '\r\n', but in others (e.g. Firefox) new line symbol
 * is '\n'. Because of these differences length of a string can vary in various browsers. To avoid differences this
 * function should be called before a string length determining.
 *
 * @param str a string where new line symbol should be unified.
 * @return string with unified new line symbol.
 */
function getStringWithUnifiedNewLineSymbol(str) {
    return str.replace(/(\r?\n)|(\r\n?)/g, '\r\n');
}

function showElement(element) {
    element.style.display = "block";
}

function hideElement(element) {
    element.style.display = "none";
}

function showElementById(id) {
    showElement(document.getElementById(id));
}

function hideElementById(id) {
    hideElement(document.getElementById(id));
}


/*
 * Shows moire.
 */
function showMoire(moireDivId, containerDivId) {
    var moire = document.getElementById(moireDivId);
    if (moire) {
        shiftTo(moire, 0, 0);

        var muarImg = moire.getElementsByTagName("img")[0];

        showElement(moire);
        showElement(muarImg);

        var windowResizeHandler = function() {
            moireHelper.extinguishEnterTTCalendarWeek();
        };
        moireHelper.setWindowResizeHandler(windowResizeHandler);
        windowResizeHandler();
    }
}

/*
 * Hides moire having the specified moireDivId.
 */
function hideMoire(moireDivId) {
    var moire = document.getElementById(moireDivId);
    if (moire) {
        hideElement(moire);
    }

    var muarImg = moire.getElementsByTagName("img")[0];
    if (muarImg)
        hideElement(muarImg);

    moireHelper.restoreWindowResizeHandler();
    moireHelper.extinguishEnterTTCalendarWeek();
}

/*
 * Determines whether keyboard button having the specified keyCode has been pressed or not.
 *
 * @return true if keyboard button having the specified keyCode has been pressed and false otherwise.
 */
function isKeyPressed(keyCode, evt) {
    var eventObj = getEventObj(evt);

    if (eventObj != null && eventObj.keyCode == keyCode) {
        if (document.all && navigator.userAgent.indexOf("Opera") == -1) {
            eventObj.returnValue = false;
        } else {
            eventObj.preventDefault();
        }

        return true;
    }

    return false;
}

/*
 * Determines whether TAB keyboard button has been pressed or not.
 *
 * @return true if TAB has been pressed and false otherwise.
 */
function isTabPressed(evt) {
    return isKeyPressed(9, evt);
}

/*
 * Calls the specified handler if TAB keyboard button has been pressed.
 */
function handleTab(tabHandler, evt) {
    if (isTabPressed(evt)) {
        tabHandler();
    }
}

/*
 * Determines whether ESC keyboard button has been pressed or not.
 *
 * @return true if ESC has been pressed and false otherwise.
 */
function isEscapePressed(evt) {
    return isKeyPressed(27, evt);
}

/*
 * Calls the specified handler if ESC keyboard button has been pressed.
 */
function handleEscape(escHandler, evt) {
    if (isEscapePressed(evt)) {
        escHandler(evt);
    }
}

/*
 * Determines whether ENTER keyboard button has been pressed or not.
 *
 * @return true if ENTER has been pressed and false otherwise.
 */
function isEnterPressed(evt) {
    return isKeyPressed(13, evt);
}

/*
 * Calls the specified handler if ENTER keyboard button has been pressed.
 */
function handleEnter(enterHandler, evt) {
    if (isEnterPressed(evt)) {
        enterHandler();
    }
}

// return client size
function getClientSize() {
    return [getInsideWindowWidth() + getBodyScrollLeft(), getInsideWindowHeight() + getBodyScrollTop()];
}

// Retrieve the rendered width of an element
function getObjectWidth(objId)  {
    var elem = document.getElementById(objId);
    var result = 0;
    if (elem.offsetWidth) {
        result = elem.offsetWidth;
    } else if (elem.clip && elem.clip.width) {
        result = elem.clip.width;
    } else if (elem.style && elem.style.pixelWidth) {
        result = elem.style.pixelWidth;
    }
    return parseInt(result, 10);
}

// Retrieve the rendered height of an element
function getObjectHeight(obj_id)  {
    var elem = document.getElementById(obj_id);
    var result = 0;
    if (elem.offsetHeight) {
        result = elem.offsetHeight;
    } else if (elem.clip && elem.clip.height) {
        result = elem.clip.height;
    } else if (elem.style && elem.style.pixelHeight) {
        result = elem.style.pixelHeight;
    }
    return parseInt(result, 10);
}

// Return the available content width space in browser window
// Note: under Firefox, Safari & Opera the scroller width will be included.
function getInsideWindowWidth( ) {
    if (window.innerWidth) {
        return window.innerWidth;
    } else if( document.documentElement && ( document.documentElement.clientWidth ) ) {
      //IE 6+ in 'standards compliant mode'
      return document.documentElement.clientWidth;
    } else if (document.body && document.body.clientWidth) {
        return document.body.clientWidth;
    } else if (document.body && document.body.parentElement) {
        return document.body.parentElement.clientWidth;
    }
    return 0;
}

// Return the available content height space in browser window
// Note: under Safari & Opera the scroller height will be included.
function getInsideWindowHeight( ) {
    if (window.innerHeight) {
        return window.innerHeight;
    } else if( document.documentElement && ( document.documentElement.clientHeight ) ) {
      //IE 6+ in 'standards compliant mode'
      return document.documentElement.clientHeight;
    } else if (document.body && document.body.clientHeight) {
        return document.body.clientHeight;
    } else if (document.body && document.body.parentElement) {
        // measure the html element's clientHeight
        return document.body.parentElement.clientHeight;
    }
    return 0;
}

function getEventObj(evt) {
    var eventObj = null;

    if (typeof evt != typeof undefined && evt != null) {
        eventObj = evt;
    } else if (typeof event != typeof undefined && event != null) {
        eventObj = event;
    }

    return eventObj;
}

/*
 * MoireHelper is intended for doing some preparatory actions before moire showing.
 */
function MoireHelper() {
    /****************************************** BEGIN PROPERTIES ******************************************/
    /*
     * Window resize handler set before replacing by {@link #setWindowResizeHandler}.
     */
    this.previousWindowResizeHandler = window.onresize;
    /*
     * Flag to determine whether window resize handler was modified or not.
     */
    this.windowResizeHandlerChanged = false;
    /****************************************** END PROPERTIES ******************************************/

    /****************************************** BEGIN PUBLIC METHODS ******************************************/

    /*
     * Replaces window resize handler by the specified one. Old handler can be restored later by calling
     * {@link #restoreWindowResizeHandler}.
     */
    this.setWindowResizeHandler = function(handler) {
        if (typeof handler != typeof undefined && handler != null) {
            this.previousWindowResizeHandler = window.onresize;
            window.onresize = handler;

            this.windowResizeHandlerChanged = true;
        }
    };

    /*
     * Restores window resize handler replaced by {@link #setWindowResizeHandler}. If {@link #setWindowResizeHandler}
     * has not been called before nothing happens.
     */
    this.restoreWindowResizeHandler = function() {
        if (this.windowResizeHandlerChanged) {
            window.onresize = this.previousWindowResizeHandler;

            this.windowResizeHandlerChanged = false;
        }
    };

    /*
     * Hides border around a week in the calendar on the Enter TT page (if do not do it the border floats when window is
     * resized). Also this method prevents the border showing after moire hiding.
     */
    this.extinguishEnterTTCalendarWeek = function() {
        if (typeof Calendar_calendarTableOut != typeof undefined) {
            Calendar_calendarTableOut();
        }
    };
    /****************************************** END PUBLIC METHODS ******************************************/
}

/*
 * ImagePreloader is intended for the specified images preloading. It is needed for seeing them without visible waiting.
 * After images loading callback function is called (if it is specified).
 *
 * @param images - array of image file names.
 * @param callback - JavaScript function to call after images loading.
 * @see - http://www.webreference.com/programming/javascript/gr/column3/
 */
function ImagePreloader(images, callback) {
    // store the callback
    this.callback = callback;

    // initialize internal state.
    this.nLoaded = 0;
    this.nProcessed = 0;
    this.aImages = [];

    // record the number of images.
    this.nImages = images.length;

    // for each image, call preload()
    for (var i = 0; i < images.length; i++) {
        this.preload(images[i]);
    }
}

ImagePreloader.prototype.preload = function(image) {
    // create new Image object and add to array
    var oImage = new Image();
    this.aImages.push(oImage);

    // set up event handlers for the Image object
    oImage.onload = ImagePreloader.prototype.onload;
    oImage.onerror = ImagePreloader.prototype.onerror;
    oImage.onabort = ImagePreloader.prototype.onabort;

    // assign pointer back to this.
    oImage.oImagePreloader = this;
    oImage.bLoaded = false;

    // assign the .src property of the Image object
    oImage.src = image;
};


ImagePreloader.prototype.onComplete = function() {
    this.nProcessed++;

    if (this.nProcessed == this.nImages &&
        typeof this.callback != typeof undefined && this.callback != null) {

        this.callback(this.aImages, this.nLoaded);
    }
};

ImagePreloader.prototype.onload = function() {
    this.bLoaded = true;
    this.oImagePreloader.nLoaded++;
    this.oImagePreloader.onComplete();
};

ImagePreloader.prototype.onerror = function() {
    this.bError = true;
    this.oImagePreloader.onComplete();
};

ImagePreloader.prototype.onabort = function() {
    this.bAbort = true;
    this.oImagePreloader.onComplete();
};

// disables text selection in the specified element in IE and FF
function disableTextSelection(element, dontChangeCursor)
{
    if( !element.aTDisabledSel )
    {
        element.prevOnselectstart = element.onselectstart;
        element.prevOndragstart = element.ondragstart;
        element.prevUnselectable = element.unselectable;
        element.prevMozUserSelect = element.style.MozUserSelect;
        element.prevCursor = element.style.cursor;

        element.onselectstart = function() { return false; };
        element.ondragstart = function() { return false; };
        element.unselectable = "on";
        element.style.MozUserSelect = "none";
        if(!dontChangeCursor)
            element.style.cursor = "default";

        element.aTDisabledSel = true;
    }
}

// disables text selection in the specified element in IE and FF
function enableTextSelectionBack(element)
{
    if( element.aTDisabledSel )
    {
        element.onselectstart = element.prevOnselectstart;
        element.ondragstart = element.prevOndragstart;
        element.unselectable = element.prevUnselectable;
        element.style.MozUserSelect = element.prevMozUserSelect;
        element.style.cursor = element.prevCursor;
        element.aTDisabledSel = false;
    }
}

function spentTimeIsInvalid(spentTime)
{
    return spentTime == null || spentTime < 0;
}

function leadZero(n)
{
    if(n < 10)
        return '0' + n;
    else
        return '' + n;
}

function areCookiesEnabled()
{
    document.cookie="cookies=true";
    return document.cookie ? true : false;
}

/**
 * Determines which mouse button was pressed.
 * @param event - the event object from the onmousedown event handler (it may work in some other events, too)
 */
function whichButtonPressed(event)
{
    if (event.which == null)
       /* IE case */
       return (event.button < 2) ? "left" :
                 ((event.button == 4) ? "middle" : "right");
    else
       /* All others */
       return (event.which < 2) ? "left" :
                 ((event.which == 2) ? "middle" : "right");
}

function setElementInnerHTML(elementId, html)
{
    document.getElementById(elementId).innerHTML = html;
}


/**
 * @param buttonGroup - radio button group
 * @return the array number of the selected radio button or -1 if no button is selected
 */
function getSelectedRadio(buttonGroup) {

   if (buttonGroup[0]) { // if the button group is an array (one button is not an array)
      for (var i=0; i<buttonGroup.length; i++) {
         if (buttonGroup[i].checked) {
            return i
         }
      }
   } else {
      if (buttonGroup.checked) { return 0; } // if the one button is checked, return zero
   }
   // if we get to this point, no radio button is selected
   return -1;
}


/**
 * @param buttonGroup - radio button group
 * @return the value of the selected radio button or null if no button is selected
 */
function getSelectedRadioValue(buttonGroup) {
   //
   var i = getSelectedRadio(buttonGroup);
   if (i == -1) {
      return null;
   } else {
      if (buttonGroup[i]) { // Make sure the button group is an array (not just one button)
         return buttonGroup[i].value;
      } else { // The button group is just the one button, and it is checked
         return buttonGroup.value
      }
   }
}
/**
 * Finds element with specified name at form.
 * @param form - js object form to search element
 * @param name of the element
 * @return element object
 */
function findFormElementByName(form, name) {
    return form.elements[name];
}

/**
 * function to added listener to specified element
 */
 function addListener(element, event, listener)
{
    if (element.addEventListener)
        element.addEventListener(event, listener, false);
    else
        element.attachEvent('on' + event, listener);
}

/**
 * function to remove listener from specified element
 */
 function removeListener(element, event, listener)
{
    if (element.removeEventListener)
        element.removeEventListener(event, listener, false);
    else
        element.detachEvent('on' + event, listener);
}

function stopEventPropagation(event)
{
    if (event.stopPropagation)
    {
        event.stopPropagation();
    }
    else
    {
        event.cancelBubble = true;
    }
}

/**
 * Creates Date object.
 * Use this function to avoid "smart" date conversion for DST.
 *
 * @param year - year of date
 * @param month - month of date
 * @param day - day of date
 * @return Date object
 */
function newDate(year, month, day)
{
    //12:00 is specified to avoid date conversion for DST
    return new Date(year, month, day, 12, 0);
}

/**
 * Compare two dates only by "date" part, excluding "time" part
 * @param date1
 * @param date2
 */
function areDatesEqual(date1, date2) {
    return date1.getFullYear() === date2.getFullYear() &&
           date1.getMonth() === date2.getMonth() &&
           date1.getDate() === date2.getDate();
}

/**
 * Compare two dates only by "date" part, excluding "time" part
 *
 * @param date1
 * @param date2
 * @return Number : negative if date1<date2, 0 if date1 == date2, positive if date1>date2 (only date, excluding time)
 */
function compareDates( date1, date2 )
{
    return date1.getFullYear() - date2.getFullYear() ||
           date1.getMonth() - date2.getMonth() ||
           date1.getDate() - date2.getDate();
}

/**
 * Gets target DOM element of specified event.
 *
 * @param event - event
 * @return object DOM element
 */
function getTargetElement( event )
{
    if (event.toElement) { //IE
        return event.toElement;
    } else if (event.relatedTarget) { //other browsers
        return event.relatedTarget;
    }
}

/**
 * Gets cookie value.
 * Note: return null if cookie not found or cookies disabled.
 *
 * @param name - cookie name
 * @return cookie value
 */
function getCookie(name)
{
    var cookies = document.cookie.split('; ');
    for (var index = 0; index < cookies.length; index++)
    {
        var nameValPair = cookies[index].split('=');
        if (name == nameValPair[0])
            return nameValPair[1];
    }
    return null;
}

/**
 * Saves value with specified cookie name.
 * Delete cookie if value is null.
 * Note: do nothing if cookies disabled.
 *
 * @param name - cookie name
 * @param value - saving value
 */
function setCookie( name, value, path )
{
    //current time
    var expireDate = new Date();
    if( value != null )
    {
        //current time + ~10 years
        expireDate = new Date(expireDate.getTime() + 310000000000);
    }
    document.cookie = [ name, "=", value, ";expires=", expireDate.toGMTString(), ( path ) ? ";path=" + path : "" ].join("");
}

function applyToSingleOrEach(singleOrCollection, func)
{
    if (!singleOrCollection)
        return;

    if (singleOrCollection.length != undefined)
    {
        for (var i = 0; i < singleOrCollection.length; i++)
            func(singleOrCollection[i], i);
    }
    else
    {
        func(singleOrCollection, 0);
    }
}

function applyToSingleOrIndexed(singleOrCollection, index, func)
{
    if (singleOrCollection.length != undefined)
    {
        func(singleOrCollection[index]);
    }
    else
    {
        if (index == 0)
            func(singleOrCollection);
        else
            throw "Not a collection, but non-zero index was given";
    }
}

function applyToSingleOrLast(singleOrCollection, func)
{
    if (singleOrCollection.length != undefined)
    {
        func(singleOrCollection[singleOrCollection.length - 1]);
    }
    else
    {
        func(singleOrCollection);
    }
}


/**
 * Returns actual width of the window by the width of the content to fit inside
 * when the browser supports this (IE does NOT so the result will be equal to the passed value).
 * May be used for window resizing
 */
function getCorrectedWidth(width)
{
    if( typeof( window.innerWidth ) == 'number' )
        //FF
        return width + window.outerWidth - window.innerWidth;
    return width;
}

/**
 * Returns actual height of the window by the height of the content to fit inside
 * when the browser supports this (IE does NOT so the result will be equal to the passed value).
 * May be used for window resizing
 */
function getCorrectedHeight(height)
{
    if( typeof( window.innerHeight ) == 'number' || typeof( window.outerHeight ) == 'number')
        //FF
        return height + window.outerHeight - window.innerHeight;
    return height;
}

/**
 * Resizes the browser window to fit specified size of the content
 * width and height parameters may be null in this case corresponding
 * dimension will not be affected
 * Note: actual size of the window depends on browser and its settings
 * Note: window will not be resized if it's bigger than needed
 */
function adaptiveWindowResize(width, height)
{
    var innerWidth  = getInsideWindowWidth();
    var innerHeight = getInsideWindowHeight();
    width = (width == null) ? innerWidth : width;
    height = (height == null) ? innerHeight : height;

    //we don't need to shrink the popups so resizing won't be applied to the bigger ones
    if (width > innerWidth || height > innerHeight)
        resizeWindow(width, height);
}

/**
 * Resizes the browser window to fit specified size of the content
 * Note: actual size of the window depends on browser and its settings
 */
function resizeWindow(width, height)
{
    //when possible it's better to calculate real window size then to resize twice
    window.resizeTo(getCorrectedWidth(width), getCorrectedHeight(height));

    //if we didn't get the expected result after the first resizing
    //we can calculate the difference and resize again (IE)
    var newWidth  = getInsideWindowWidth();
    var newHeight = getInsideWindowHeight();
    if (width != newWidth || height != newHeight)
        window.resizeTo(width + (width - newWidth), height + (height - newHeight));
}


/**
 * Resizes the browser window using {@link #adaptiveWindowResize} function
 * after the page is loaded. For the parameters description see {@link #adaptiveWindowResize}
 */
function adaptiveResizeAfterLoad(width, height)
{
    var callback = function()
    {
        adaptiveWindowResize(width, height);
    };

    addListener(window, "load", callback);
}

function ExitConfirmationHandlersManager () {}

/**
 * Makes browser show a confirmation message before any navigation when
 * isConfirmationOnUnloadRequired() == true
 * @param isPopupMode if isPopupMode is true, then exit confirmation will be disabled, when window.opener close
 * @param message confirmation message that will be shown when user would try to close the popup or navigate to another page
 * @param isConfirmationOnUnloadRequired a callback method that must always return true when we need to show confirmation before any navigation and false otherwise
 * */
ExitConfirmationHandlersManager.setUpExitConfirmation = function (message, isConfirmationOnUnloadRequired, isPopupMode, mustNotWarnUserIfIsNotCurrent)
{
    if (!window["exitConfirmationHandlers"])
        window["exitConfirmationHandlers"] = [];
    else if (window.onbeforeunload != null)
        window["exitConfirmationHandlers"].push( window.onbeforeunload );

    var confirmationEnabled = true;
    var confirmExit = function()
    {
        var confMessage = "";
        if ( confirmationEnabled && isConfirmationOnUnloadRequired() )
            confMessage += message;

        if ( window["exitConfirmationHandlers"] && window["exitConfirmationHandlers"].length > 0 )
        {
            for( var i = window["exitConfirmationHandlers"].length - 1; i >= 0; i-- )
            {
                var handler = window["exitConfirmationHandlers"][i];
                if( handler.mustWarnUser() )
                    confMessage += ( isNotEmpty(confMessage) ? "\n\n" : "" ) + handler.getConfirmationMessage();
            }
        }

        if (isNotEmpty(confMessage))
            return confMessage;
    };
    confirmExit.mustWarnUser = function () { return !mustNotWarnUserIfIsNotCurrent && confirmationEnabled && isConfirmationOnUnloadRequired() };
    confirmExit.getConfirmationMessage = function () { return message };

    var disableConfirmation= function()
    {
        confirmationEnabled = false;
    };

    window.onbeforeunload = confirmExit;

    //after a navigation in the parent window it would try to close all the popups
    //opened from it. In this case popups shall be closed without any confirmations
    /* TODO:
     * the call of the EventListener causes an error in FF7 if parent window needs to be reload after popup is closed.
     * confirmationEnabled and disableConfirmation should be moved outside of this function in order to allow to
     * remove EventListener from parent window before popup closes. This is required, because to remove EventListener
     * you need to specify the same function you setup as a listener
     * For more information refers to Bug 33983: FF7: JS error when Edit Open Task or Create New Tasks pop-up is closed
     */
    if (isPopupMode)
        addListener(window.opener, "beforeunload", disableConfirmation);
};

ExitConfirmationHandlersManager.restoreExitConfirmation = function ()
{
    var oldHandler;
    if ( window["exitConfirmationHandlers"] && (oldHandler = window["exitConfirmationHandlers"].pop()) )
        window.onbeforeunload = oldHandler;
    else
        window.onbeforeunload = null;
};

// if we set window.onbeforeunload = undefined, it will cause errors in IE
ExitConfirmationHandlersManager.resetExitConfirmation = function ()
{
    window.onbeforeunload  = null;
    window["exitConfirmationHandlers"] = null;
};


/**
 * Unlike just calling submit() method of a form, it notifies listeners
 */
function notifyAndSubmit(form)
{
    if (form.dispatchEvent) //other browsers
    {
        var event = form.ownerDocument.createEvent('HTMLEvents');
        event.initEvent("submit", false, true);
        form.dispatchEvent(event);
        if (!Event.SUBMIT) //Safari & Chrome
		    form.submit();
    }
    else if (form.fireEvent) //IE
    {
        form.fireEvent("onsubmit", event);
        form.submit();
    }
}


/**
 * Makes DOM node invisible and then after timeout removes it
 */
function removeElementWithTimeout(domElement, timeout)
{
    addClass(domElement, 'invisible');
    setTimeout(function()
    {
        domElement.parentNode.removeChild(domElement);
    }, timeout)
}

 /**
  * Sets focus to the passed DOM element
  */
function focusElement(elem)
{
    setTimeout(function()
    {
        elem.focus();
    })
}

function isSameOrParentOf(elem, checkElem)
{
    var currentElem = elem;
    while (currentElem)
    {
        if (currentElem == checkElem)
            return true;
        currentElem = currentElem.parentNode;
    }
    return false;
}

function hideFocusForLinks( elem ) {
    var links = elem.getElementsByTagName("a");
    var length = links.length;
    for( var i = 0; i < length; i++ )
        links[i].hideFocus = true;
}

/**
 * Sets the handler for clicking outside an element.
 * Requires jQuery.
 *
 * TODO: there must be a way to unset the handler. This function should store the actual handler (not the given one,
 *       but the one which is dynamically created and attached to the document mousedown) in the given elements data()
 *       property. This would allow the creation of the 'unsetMouseDownOutsideHandler' function, which would get the
 *       actual handler from the element's data() and unset it from the document.
 *
 * TODO: seems that this handler is actually for clicking outside the _children_ of the element.
 *       So if I click inside the element but outside any of its children, the handler won't be executed.
 *
 * @param element - an element, selector or jQuery object
 * @param handler - function
 */
function setMouseDownOutsideHandler( element, handler )
{
    element = $(element);

    $(document).mousedown( function(event)
                       {
                           if( $(event.target).parents().index(element) == -1 )
                           {
                               handler(event);
                           }
                       });
}


function addStrutToPageBottom()
{
document.body.appendChild(document.createElement("br"));
document.body.appendChild(document.createElement("br"));
}

/**
 * Holds a boolean value which can be switched to another one, but will automatically reset after the specified time.
 *
 * @param {boolean} defaultValue initial (default) value
 * @param {number} time time in milliseconds to hold the new value before reset
 * @constructor
 */
function BoolTimer( defaultValue, time )
{
    var value = defaultValue;
    var timeout;

    /**
     * Switches the value to the opposite of the default and sets the timer to reset it back to default.
     */
    this.switchValue = function()
    {
        value = !defaultValue;

        clearTimeout( timeout );

        timeout = setTimeout( function() {
            value = defaultValue;
        }, time);
    };

    /**
     * @return {Boolean} whether the current value is true
     */
    this.isTrue = function()
    {
        return value == true;
    };

    /**
     * @return {Boolean} whether the current value is false
     */
    this.isFalse = function()
    {
        return value == false;
    };
}

function doOnceOn( element, actionName, callback )
{
    var handler = function ()
    {
        callback();
        $( element ).off( actionName, handler );
    };
    $( element ).on( actionName, handler );
    return function() {$( element ).off( actionName, handler )};
}

function doOnceOnWindowResize( callback )
{
    return doOnceOn( window, "resize", callback );
}

function getLeaveTypeIconPath ( iconId )
{
    return atImgPath( 'leave_types/icons/leave_type_' + iconId + '_24bit.png' );
}

function getLeaveTypeColorIconPath ( iconId )
{
    return atImgPath( 'leave_types/chart_icons/leave_icon_' + iconId + '.gif' );
}

function getLeaveTypePrintIconPath ( iconId )
{
    return atImgPath( 'leave_types/icons/leave_type_' + iconId + '_print.png' );
}

/**
 * Show jQuery element at position with positionParams.
 * If part of the element is shown outside left or top border of the document,
 * try to fix it by changing collision parameter
*/
function showElementAt( element, positionParams )
{
    pos = {};
    pos.of = positionParams.of;
    pos.my = positionParams.my ? positionParams.my : "left top";
    pos.at = positionParams.at ? positionParams.at : "left bottom";
    pos.offset = positionParams.offset;
    col = ( positionParams.collision ? positionParams.collision : "flip" ).split( " " );
    pos.horCollision = col[0];
    pos.vertCollision = col[col.length - 1];

    pos.collision = pos.horCollision + " " + pos.vertCollision;
    element.position( pos );

    offset = element.offset();

    if( ( offset.left < 0 && pos.horCollision != "none" ) || ( offset.top < 0 && pos.vertCollision != "none" ) )
    {
        if( offset.left < 0 ) pos.horCollision = "none";
        if( offset.top < 0 ) pos.vertCollision = "none";

        pos.collision = pos.horCollision + " " + pos.vertCollision;
        element.position( pos );
    }
}


function setFocusIntoFieldWithPlaceholder( field ) {
    if( !isMSIE() ) // IE hides placeholder on focus, so we don't autofocus in IE
        $( field ).focus();
}

function atImgPath( relativePath )
{
    return AT_IMAGE_PATH + CustomizationSettings.colorScheme + "/" + relativePath + "?hash=" + HASH;
}

function atDefImgPath( relativePath )
{
    return AT_IMAGE_PATH + "default/" + relativePath + "?hash=" + HASH;
}

function imgPath( relativePath )
{
    return IMAGE_PATH + CustomizationSettings.colorScheme + "/" + relativePath + "?hash=" + HASH;
}

function defImgPath( relativePath )
{
    return IMAGE_PATH + "default/" + relativePath + "?hash=" + HASH;
}

/**
 * Defers function execution, calling it after 10 ms (not 0 ms, necessary for certain browsers).
 * @param f Function to call.
 * @param context Function context (this).
 */
function defer(f, context)
{
    if (context)
        return setTimeout(_.bind(f, context), 10);
    else
        return setTimeout(f, 10);
}

/**
 * Converts size to CSS pixel size string (for example, 15 to "15px")
 * @param size Size in pixels.
 */
function px(size)
{
    return size + "px";
}

/**
 * Converts array of objects, each of them having "id" field, to a map.
 * @param array Array of objects to convert.
 */
function arrayToMapById(array)
{
    var map = {};
    _.each(array, function(item) { map[item["id"]] = item; });
    return map;
}

/**
 * Converts array of objects to set ([1, 5, 7] -> {1: true, 5: true, 7: true}).
 * @param array Array of objects to convert.
 */
function arrayToSet(array)
{
    var map = {};
    _.each(array, function(item) { map[item] = true; });
    return map;
}

/**
 * Converts string to bytes encoded in UTF-16LE
 * @param str String to convert.
 * @returns {ArrayBuffer}
 */
function stringToUTF16(str)
{
    var buffer = new ArrayBuffer(str.length * 2);
    var bufferProxy = new Uint16Array(buffer);
    for (var i = 0; i < str.length; i++)
        bufferProxy[i] = str.charCodeAt(i);
    return buffer;
}

/**
 * Detects if browser supports saving files from client code.
 * @returns {boolean}
 */
function saveFileSupported()
{
    return !!(window.Blob || document.execCommand);
}

/**
 * Saves file to local file system.
 * @param content String or byte buffer with file contents.
 * @param fileName Desired file name.
 * @param blobOptions Default implementation uses Blob for saving, here you can pass blob options such as encoding, type, etc.
 */
function saveFile(content, fileName, blobOptions)
{
    if (window.Blob)
    {
        var utf16 = blobOptions.encoding.toLowerCase() == "utf-16le";
        var blobData = [];
        if (utf16)
        {
            blobData.push(new Uint8Array([0xFF, 0xFE]));
            if (_.isString(content))
                content = stringToUTF16(content);
        }
        blobData.push(content);

        var blob = new Blob(blobData, blobOptions);
        // Use FileSaver.js library
        saveAs(blob, fileName);
    }
    else
    {
        // IE8, IE9 special implementation
        if (document.execCommand)
        {
            // This way tabs and line breaks will save normally in the text file
            if (isIE8())
                content = "<pre>" + escapeHtmlSymbols(content) + "</pre>";

            var oWin = window.open("about:blank", "_blank");

            oWin.document.writeln(content);
            oWin.document.close();

            // Need to specify the filename that we are going to set here
            oWin.document.execCommand('SaveAs', true, fileName + ".txt");
            oWin.close();
        }
    }
}

/**
 * Subscribes element to outside click event.
 * @param $el Element.
 * @param eventNamespace Click event namespace to prevent unwanted unsubscribing.
 * @param clickHandler Click handler.
 * @param ignoreEventPredicate Custom ignore predicate, when it returns true, click handler doesn't trigger.
 * @param $outsideContainer outside element, where outside clicks are handled. Default value - document body
 */
function onOutsideClick($el, eventNamespace, clickHandler, ignoreEventPredicate, $outsideContainer)
{
    var eventName = "click." + eventNamespace;
    if (!$outsideContainer)
        $outsideContainer = $(document);
    $outsideContainer.off(eventName).on(eventName, function(e)
    {
        if (ignoreEventPredicate(e) ||
            e.target === document.body ||
            $el.is(e.target) ||
            $el.has(e.target).length > 0)
            return;

        clickHandler(e);
    });
}

/**
 * Fixes wheel event for elements, which are located inside iScroll containers.
 * @param $el Inner element with native scroll.
 */
function fixIScrollInnerWheelEvent($el)
{
    $el.on("wheel mousewheel DOMMouseScroll", function(e)
    {
        var el = $el[0];
        if (el.clientHeight < el.scrollHeight)
            e.stopPropagation();
    });
}