function BaseTaskPopup()
{
}

BaseTaskPopup.prototype = _.create(BasicLightbox.prototype);

BaseTaskPopup.prototype.init = function( popupId, viewTaskPopupData, estEnabled, btyEnabled, sortConstants,
                                         taskNotAssignedCallback, taskWasDeletedCallback )
{
    BasicLightbox.prototype.init.call( this, popupId, "closeTaskEditPopupButton" );

    this.statusOpenMessage = messageResource.getMessage("task.edit_popup.title.open");
    this.statusCompletedMessage = messageResource.getMessage("task.edit_popup.title.competed");

    var self = this;

    this.setCancelCallback( function() { self.hide() });

    this.popupId = popupId;
    this.popupEl = $( "#" + this.popupId );
    this.footer = this.findElementBySubId("footer");
    this.scrollableContainer = this.getElementBySubId("innerContent");
    this.taskInfoDiv = this.getElementBySubId("taskInfoDiv");

    this.estEnabled = estEnabled;
    this.btyEnabled = btyEnabled;

    this.taskNotAssignedCallback = taskNotAssignedCallback;
    this.taskWasDeletedCallback = taskWasDeletedCallback;

    this.task = viewTaskPopupData.task;
    this.project = this.task.project;
    this.customer = this.project.customer;
    this.billingType = this.task.billingType;

    this.dateFormat = Date.convertSmartToExtDateFormat( SystemSettings.dateFormat ).replace(/F/, "M");

    var statusMessage = this.task.completed ? this.statusCompletedMessage : this.statusOpenMessage;
    this.findElementBySubId("taskTitle").text(messageResource.getMessage(this.titleMessageKey, [statusMessage]));

    this.initCustomerProjectSelector();
    if ( this.btyEnabled )
        this.initBillingTypeSelector();

    this.taskComletionDateRowEl = this.popupEl.find("#completionDateRow");

    this.setTaskName(this.task.name);
    this.setTaskStatus(!this.task.completed);
    this.descriptionContainer = self.findElementBySubId("descriptionContainer");
    this.description = this.findElementBySubId("taskDescription").val(this.task.description);

    this.initDeadlineDate();

    this.setCompletionDate( !this.task.completed, true );

    this.descriptionBlock = this.findElementBySubId("descriptionBlock");
    this.descriptionVisible = false;
    this.toggleDescription = function() {
        self.descriptionBlock.toggleClass('expanded');
        self.descriptionContainer.toggleClass('expanded');
        self.slideTogglefunction(self.getElementBySubId("taskDescription"));
        self.descriptionVisible = !self.descriptionVisible;
        //fixed bug 50806
        var counter = self.getElementBySubId("taskDescriptionCounter")
        if(counter)
            self.getElementBySubId("taskDescriptionCounter" ).style.display = self.getElementBySubId("taskDescription" ).style.display;
    };

    this.descriptionButton =  this.getElementBySubId("descriptionButton");
    this.descriptionButton.title = messageResource.getMessage(this.description_tooltip_message_key);
    $(this.descriptionButton).click(function(){ if ( !self.animationInProccess) self.toggleDescription()});
    this.initDescriptionField();
    this.updateDescriptionImage();

    this.detailsTemplateTable = this.popupEl.find("#taskDetailsTemplateTable");
    this.reportedTimeTemplateTable = this.popupEl.find("#reportedTimeTemplateTable");
    this.executorsSummaryTable = this.findElementBySubId("executorsTable");
    this.executorsSummaryTableContent = this.findElementBySubId("executorsTableContent");

    var executorsSummary = viewTaskPopupData.executorsSummary;
    if ( executorsSummary ) {
        this.totalSummary = executorsSummary.total;

        if ( executorsSummary.executorsAvailable ) {
            var records = executorsSummary.records.list;
            $.each(records, function(i, record) {
                var newRow = self.createUserDetailRow();
                newRow.appendTo( self.executorsSummaryTableContent );
                newRow.find(".userDetailName").text(record.formattedName);
                newRow.find(".userDetailValue").text( self.formatTime(record.spentTime) );
            });

            var separator = this.createSeparatorRow();
            separator.appendTo( this.executorsSummaryTableContent );

            var newRow = this.createTotalDetailRow();
            newRow.appendTo( this.executorsSummaryTableContent );
            newRow.find(".totalDetailValue").text( self.formatTime(executorsSummary.total) );

            this.executorsSummaryTable.show();
        } else {
            this.findElementBySubId("executorsNotAvailableMsg").show();
        }

        this.executorsSummaryTableContent = this.findElementBySubId("taskEstimateSummary");
        if ( estEnabled ) {
            this.initEstimatedHours();

            this.findElementBySubId("spentHours").text(this.formatTime(executorsSummary.total));
            this.updateEstimateContent();

            this.executorsSummaryTableContent.show();
            this.findElementBySubId("executorsSummaryBlock").addClass("withEstimates");
        } else {
            this.executorsSummaryTableContent.hide();
        }
    } else
        this.findElementBySubId("executorsSummaryBlock").hide();


    this.slideTogglefunction = function(element){
        var maxHeight = 80;
        var interval = 0;
        var height;
        var toogle;
        var self = this;
        this.animationInProccess = true;
        if(element.style.display!='block'){
            element.style.height=0;
            height= 0;
            toogle =setInterval(function(){
                element.style.height=height+'px';
                height=height+interval;interval++;
                if(height>maxHeight){
                    element.style.height=maxHeight;
                    clearInterval(toogle);
                    self.animationInProccess = false;
                }
            },20);
            element.style.display='block';
        } else {
            height=$(element).height();
            toogle=setInterval(function(){
                element.style.height=height+'px';
                height=height-interval; interval++;
                if(height<1){
                    clearInterval(toogle);
                    element.style.display='none';
                    self.animationInProccess = false;
                }
            },20);
        }
    } ;


    this.settings = viewTaskPopupData.settings;

    this.reportedTimeTableContent = this.findElementBySubId("reportedTimeTableBody");
    if ( viewTaskPopupData.timeCommentsSet ) {
        var timeComments = viewTaskPopupData.reportedTimeComments.list;

        this.taskDateCell = this.findElementBySubId("taskDateCell");
        this.taskUserCell = this.findElementBySubId("taskUserCell");

        this.sortConstants = sortConstants;
        this.sorting = {
            sortBy: viewTaskPopupData.settings.sortBy,
            sortOrder: viewTaskPopupData.settings.sortOrder
        };

        this.updateSortingInfo();

        this.spinner = new SpinnerController();

        this.pagerData = {
            totalRecords: viewTaskPopupData.recordCount,
            recordsPerPage: this.settings.recordsPerPage,
            pageNum: 1
        };

        var totalRecordsPlaceholderContainer = $( '#viewTasktotalRecordsPlaceholder' ).text(this.pagerData.totalRecords);

        this.pagerSuccessHandler = function( result, scrollToPager ) {

            self.pagerData.totalRecords = result.recordCount;
            self.createTimeCommentsTable(result.reportedTimeComments.list);

            self.pagerController.setPagerStateAndUpdateUI(
                    self.pagerData.totalRecords,
                    self.pagerData.pageNum,
                    self.pagerData.recordsPerPage
            );

            totalRecordsPlaceholderContainer.text(self.pagerData.totalRecords);
            self.updateSortingInfo();
            self.spinner.stop();

            if ( scrollToPager )
                self.scrollToPager();
        };

        this.taskDateCell.find(".taskDateCell").click( function() {
            if ( self.sorting.sortBy == self.sortConstants.byDate )
                self.toggleSortOrder();
            else {
                self.sorting.sortBy = self.sortConstants.byDate;
                self.sorting.sortOrder = self.sortConstants.asc;
            }

            self.pagerData.pageNum = 1;
            self.getTimeComments( false );
        });

        this.taskUserCell.find(".taskUserCell").click( function() {
            if ( self.sorting.sortBy == self.sortConstants.byUser )
                self.toggleSortOrder();
            else {
                self.sorting.sortBy = self.sortConstants.byUser;
                self.sorting.sortOrder = self.sortConstants.asc;
            }

            self.pagerData.pageNum = 1;
            self.getTimeComments( false );
        });

        this.pagerController = new PagerController().init(
                {
                    pagerData: this.pagerData,
                    handlersMap: {
                        pageChanged: function ( pageNum )
                        {
                            self.pagerData.pageNum = pageNum;
                            self.getTimeComments( true );
                        },
                        recordsPerPageChanged: function ( recordsPerPage )
                        {
                            self.pagerData.pageNum = 1;
                            self.pagerData.recordsPerPage = recordsPerPage;
                            self.getTimeComments( true );
                        }
                    },
                    ignoreHashChange: true,
                    pagerControlsElementId: "viewTaskPagerPanel",
                    pagerElementId: "viewTaskpager",
                    recordsPerPageSelectorId: "viewTaskrecordsPerPageSelector",
                    minRecordsPerPage: MIN_RECORDS_PER_PAGE //CUSTOM_PAGES_SET_2
                }
        );
        this.pagerController.renderThisController();

        this.createTimeCommentsTable(timeComments);
    } else {
        this.popupEl.find("#viewTaskPagerPanel").hide();
        var emptyRow = this.createEmptyTableRow();
        emptyRow.appendTo( this.reportedTimeTableContent );
    }


    return this;
};

BaseTaskPopup.prototype.getElementBySubId = function( id )
{
    return document.getElementById( this.getElementIdBySubId( id ) );
};

BaseTaskPopup.prototype.getElementIdBySubId = function( id )
{
    return this.popupId + "_" + id;
};

BaseTaskPopup.prototype.findElementBySubId = function( id )
{
    return this.popupEl.find( '#' + this.getElementIdBySubId(id) );
};

BaseTaskPopup.prototype.initEstimatedHours = function()
{
    this.taskEstimateValue = this.task.budgetedTime;
    this.estimatedHoursEl = this.findElementBySubId("estimatedHours");
    this.setEstimatedHours();
};

BaseTaskPopup.prototype.updateEstimateContent = function()
{
    if ( this.taskEstimateValue != null )
    {
        this.findElementBySubId("variance").text(this.formatTime( this.taskEstimateValue - this.totalSummary ));
        if ( !this.estimateSlider )
            this.estimateSlider = new EstimateSlider();

        this.estimateSlider.update( this.totalSummary, this.taskEstimateValue, this.task.completed );
        this.findElementBySubId("varianceLine").show();
    }
    else {
        this.findElementBySubId("varianceLine").hide();
        this.executorsSummaryTableContent.find( "#estimateSlider" ).hide();
    }
};

BaseTaskPopup.prototype.setEstimatedHours = function()
{
    var estimateStr = ( this.taskEstimateValue != null ) ? this.formatTime(this.taskEstimateValue) :
                      messageResource.getMessage("task.edit_popup.not_needed");

    this.estimatedHoursEl.toggleClass("notNeeded", this.taskEstimateValue == null);
    this.findElementBySubId("estimatedHoursValue").text(estimateStr);
};

BaseTaskPopup.prototype.updateSortingInfo = function()
{
    var isAsc = this.sorting.sortOrder == this.sortConstants.asc;
    var isByDate = this.sorting.sortBy == this.sortConstants.byDate;

    this.taskDateCell.toggleClass("ascending",  isByDate && isAsc);
    this.taskDateCell.toggleClass("descending", isByDate && !isAsc);

    this.taskUserCell.toggleClass("ascending",  !isByDate && isAsc);
    this.taskUserCell.toggleClass("descending", !isByDate && !isAsc);
};

BaseTaskPopup.prototype.toggleSortOrder = function() {
    this.sorting.sortOrder = ( this.sorting.sortOrder == this.sortConstants.asc ) ? this.sortConstants.desc : this.sortConstants.asc;
};

BaseTaskPopup.prototype.getTimeComments = function( scrollToPager ) {
    var self = this;
    this.spinner.spin( this.getElementIdBySubId("reportedTimeTableSpinner") );
    makeRpcCall( "EditTaskService.getTimeComments", [this.task.id, this.sorting.sortBy, this.sorting.sortOrder, this.pagerData.recordsPerPage, this.pagerData.pageNum],
                 function( data ) {
                     return self.pagerSuccessHandler( data, scrollToPager );
                 },
                 function(jqXHR, textStatus, errorThrown) {
                     if( errorThrown.exception == 'com.actimind.actitime.task.TaskIsNotAssignedToUserException' )
                     {
                         alert( messageResource.getMessage("common.errors.task_not_assined") );
                         self.dataChanged = false;
                         self.taskNotAssignedCallback();
                         return true;
                     }
                     if( errorThrown.exception == 'com.actimind.actitime.task.NoSuchTaskException' )
                     {
                         alert( messageResource.getMessage('task.edit_popup.error.concurrently_deleted'));
                         self.dataChanged = false;
                         self.hide();
                         self.taskWasDeletedCallback();
                         return true;
                     }
                     return false;
                 });
};

BaseTaskPopup.prototype.updateDescriptionImage = function()
{
    var isEmptyDescription = trim( this.getTaskDescription() ) == '';
    $(this.descriptionBlock).toggleClass("emptyDescription", isEmptyDescription);
};

BaseTaskPopup.prototype.getTaskDescription = function()
{
    return this.task.description;
};

BaseTaskPopup.prototype.setCompletionDate = function( isOpenTask, initSelector )
{
    this.taskComletionDateValueEl = this.findElementBySubId("taskCompletionDate");
    if ( isOpenTask ) {
        this.hideCompletionDate();
    } else {
        if ( initSelector )
            this.initCompletionDateSelector();
        this.showCompletionDate();
    }
};

BaseTaskPopup.prototype.showCompletionDate = function()
{
    if (this.btyEnabled)
        this.taskComletionDateRowEl.show();
    else {
        this.findElementBySubId("completionDateText").show();
        this.findElementBySubId("taskCompletionDate").show();
    }
};

BaseTaskPopup.prototype.hideCompletionDate = function()
{
    if (this.btyEnabled)
        this.taskComletionDateRowEl.hide();
    else {
        this.findElementBySubId("completionDateText").hide();
        this.findElementBySubId("taskCompletionDate").hide();
    }
};

BaseTaskPopup.prototype.createTimeCommentsTable = function( timeComments ) {
    var self = this;
    var content = this.reportedTimeTableContent;

    content.hide();
    content.find("tr").remove();
    if( timeComments.length == 0 )
    {
        var emptyRow = self.createEmptyTableRow();
        emptyRow.appendTo( this.reportedTimeTableContent );
    }
    else
    {
        $.each( timeComments, function ( i, record )
        {
            var newRow = self.createReportedTimeRow();
            newRow.appendTo( content );
            newRow.find( ".reportedTimeDate" ).text( record.eventDate.parseJSONDate().format( self.dateFormat ) );
            newRow.find( ".reportedTimeUser" ).text( record.formattedName );
            newRow.find( ".reportedTimeValue" ).text( self.formatTime( record.actuals ) );
            if( record.commentsPresent )
                newRow.find( ".reportedTimeComment" ).text( record.comments );
        } );
    }
    content.show();
};

BaseTaskPopup.prototype.formatTime = function( minutes ) {
    return UserTime.formatTime(minutes);
};

BaseTaskPopup.prototype.createUserDetailRow = function() {
    return this.detailsTemplateTable.find(".userDetail").clone();
};

BaseTaskPopup.prototype.createSeparatorRow = function() {
    return this.detailsTemplateTable.find(".tableRowSeparator").clone();
};

BaseTaskPopup.prototype.createTotalDetailRow = function() {
    return this.detailsTemplateTable.find(".totalDetail").clone();
};

BaseTaskPopup.prototype.createReportedTimeRow = function() {
    return this.reportedTimeTemplateTable.find(".reportedTimeRow").clone();
};

BaseTaskPopup.prototype.createEmptyTableRow = function() {
    return this.reportedTimeTemplateTable.find(".emptyTableRow").clone();
};

BaseTaskPopup.prototype.scrollToPager = function() {
    this.innerContent.scrollTop( this.innerContent.scrollTop() + this.innerContent.find(".pager").position().top );
};

