/*
 * This file contains cell managers for Billing Type List interface.
 * Each manager contains logic for specific cell content manipulation.
 */

RpcControlManager.prototype.messagePrefix = "billing_type";

RpcControlManager.prototype.getService = function()
{
    return jsonrpc.BillingTypeService;
};

RpcControlManager.prototype.getMethods = function()
{
    return [ 'BillingTypeService.activateType',
             'BillingTypeService.archiveType',
             'BillingTypeService.setBillable',
             'BillingTypeService.setDefault',
             'BillingTypeService.setNonBillable'
           ];
};

function TypeCellManager(cell, typeId)
{
    this.base = TextCellManager;
    this.base(cell,
              messageResource.getMessage("billing_type.list.billable_tooltip"),
              messageResource.getMessage("billing_type.list.non_billable_tooltip"),
              messageResource.getMessage("billing_type.list.error_set_billable"),
              messageResource.getMessage("billing_type.list.error_set_non_billable"),
              messageResource.getMessage("billing_type.billable"),
              messageResource.getMessage("billing_type.non_billable"));
    this.typeId = typeId;
}

TypeCellManager.prototype = new TextCellManager();

TypeCellManager.prototype.getTimeoutMsg = function()
{
    return messageResource.getMessage("billing_type.list.timeout_update_status");
};

TypeCellManager.prototype.makeRemoteCall = function(callback)
{
    var reqId;
    if (this.initialState)
        reqId = this.getService().setNonBillable(callback, this.typeId);
    else
        reqId = this.getService().setBillable(callback, this.typeId);

    return reqId;
};

/*
 * Provides methods to manage a billing type 'is default' cell.
 * cell - status cell object
 * typeId - id of the billing type that the cell contains
 */
function IsDefaultCellManager(cell, typeId) {
    this.base = SingleChoiceLinkCellManager;
    this.base(cell, '', '',
            messageResource.getMessage(this.messagePrefix + ".list.cannot_be_default_tooltip"),
            messageResource.getMessage(this.messagePrefix + ".list.error_set_default"),
            '',
            messageResource.getMessage(this.messagePrefix + ".list.is_default"),
            messageResource.getMessage(this.messagePrefix + ".list.set_as_default"),
            messageResource.getMessage(this.messagePrefix + ".list.is_archived"),
            "setDefault(this.parentNode, " + typeId + ")");
    this.typeId = typeId;
}

IsDefaultCellManager.prototype = new SingleChoiceLinkCellManager();

IsDefaultCellManager.prototype.getTimeoutMsg = function() {
    return messageResource.getMessage(this.messagePrefix + ".list.timeout_update_default");
};

IsDefaultCellManager.prototype.makeRemoteCall = function(callback) {
    return this.getService().setDefault(callback, this.typeId);
};

(function() { // hides the variable from the page context
    var currentDefaultCellManager = null;

    IsDefaultCellManager.prototype.getCurrentChoiceManager = function() {
        return currentDefaultCellManager;
    };

    IsDefaultCellManager.prototype.setCurrentChoiceManager = function(manager) {
        currentDefaultCellManager = manager;
    };
})();

function setCurrentDefaultCell(cell, typeId) {
    IsDefaultCellManager.prototype
            .setCurrentChoiceManager(new IsDefaultCellManager(cell, typeId));
}

function getCurrentDefaultCell() {
    var manager = IsDefaultCellManager.prototype.getCurrentChoiceManager();
    if (manager) {
        return manager.cell;
    } else {
        return null;
    }
}