/**
* Calendar
*
* To get the selected value you have to define callback function
* with only one argument - string representing a date. You can use
* strToDate() to get Date object from this string.
*
* Based on showCalendar() function by Denis Gritcyuk <denis@softcomplex.com>
*
* @author       Dmitry Prokudin <dmitry@actimind.com>
*
* @access       public
* @version      1.0
*/

function createLayoutCalendar(strDateTime, today, startFrameDate, endFrameDate, callbackFunc, weekStartDay,
                              scheduleCalculator, headerDayDurations)
{
    return new Calendar(strDateTime, today, "", "", startFrameDate, endFrameDate, callbackFunc, false, weekStartDay,
                        scheduleCalculator, headerDayDurations);
}

var Calendar_calendarTableOut = function (table) {
 //default handler do nothing
};

var Calendar_calendarTableOver = function (table) {
 //default handler do nothing
};
//////////////////////////////////////////////////////////////////////////
////////////////////////////   Calendar class   //////////////////////////
//////////////////////////////////////////////////////////////////////////

// Note: "weekStartDay" parameter can take values starting from 1 (sunday) to 7 (saturday) in order to comply
// with java.util.Calendar class. But "weekStartDay" property of the object is assigned to the parameter - 1 value
// since all calculations in class methods rely on value between 0 and 6
function Calendar(strDateTime, today, startDate, endDate, startFrameDate, endFrameDate, callbackFunc, inPopup, weekStartDay,
        scheduleCalculator, headerDayDurations)
{
    // init properties
    this.weekDays  = [];

    this.scheduleCalculator = scheduleCalculator;
    this.headerDayDurations = headerDayDurations;

    this.callbackFunc = callbackFunc;
    this.inPopup = inPopup;
    this.onmouseOverHandler = "Calendar_highlightCell(this, event)";
    this.onmouseOutHandler = "Calendar_unhighlightCell(this, event)";

    // TODO: Use constants from date_js.jsp
    this.months = ["January", "February", "March",     "April",   "May",      "June",
                            "July",    "August",   "September", "October", "November", "December"];

    this.weekStartDay = (weekStartDay - 1); // Property value should be 0 based (see note above)
    this.weekDays = ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"];

    this.today = strToDate(today);

    this.baseDate = strDateTime == null || strDateTime == "" ?  this.today : strToDate(strDateTime);
    this.startDate = null;
    this.endDate = null;
    if(startDate != null && startDate != "" && endDate != null && endDate != "")
    {
        this.startDate = strToDate(startDate);
        this.endDate = strToDate(endDate);
    }
    this.startFrameDate = null;
    this.endFrameDate = null;
    if(startFrameDate != null && startFrameDate != "" && endFrameDate != null && endFrameDate != "")
    {
        this.startFrameDate = strToDate(startFrameDate);
        this.endFrameDate = strToDate(endFrameDate);
    }

    this.prevMonth = new Date(this.baseDate);
    this.prevMonth.setMonth(this.baseDate.getMonth() - 1);

    this.nextMonth = new Date(this.baseDate);
    this.nextMonth.setMonth(this.baseDate.getMonth() + 1);

    this.firstDay = new Date(this.baseDate);
    this.firstDay.setDate(1);
    this.firstDay.setDate(1 - (7 + this.firstDay.getDay() - this.weekStartDay) % 7);

    this.linkCurrentDay = true;
    this.header = false;

    this.setStyleNames("calendarWorkingDay", "calendarWeekendDay");

    this.changeMonthIfDateOutsideCurrentMonth = true;
    this.displayDaysOfOtherMonths = true; // If "true" displays days those do not belong to base month
}

Calendar.DAY_BORDER_COLOR = "#ffffff";
Calendar.BORDER_COLOR = "#b5b5b5";
Calendar.CURSOR_BG_COLOR = "#face00";

Calendar.prototype.setMouseOverHandler =
function Calendar_setMouseOverHandler(func) {
    this.onmouseOverHandler = func;
};

Calendar.prototype.setMouseOutHandler =
function Calendar_setMouseOutHandler(func) {
    this.onmouseOutHandler = func;
};


function Calendar_isWeekend(cell)
{
    var span = cell.firstChild;
    if(span && span.nodeName == 'SPAN')
    {
        //noinspection JSConstructorReturnsPrimitive
        return span.className.indexOf('Weekend') != -1;
    }
    //noinspection JSConstructorReturnsPrimitive
    return cell.className == 'calendarWeekendDay';
}

Calendar.prototype.isWeekend = Calendar_isWeekend;

var Calendar_highlightCell = function(cell)
{
    cell.style.backgroundColor = Calendar.CURSOR_BG_COLOR;
    try { cell.style.cursor = 'pointer'; } catch (e) {}
};

var Calendar_unhighlightCell = function(cell)
{
    try { cell.style.cursor = ''; } catch (e) {}
    if (Calendar_isWeekend(cell))
        cell.style.backgroundColor = "";
    else
        cell.style.backgroundColor = "";
    cell.style.borderColor = '';
};

function Calendar_setCellStyle(cell, isHoliday)
{
    if (isHoliday)
        replaceClass(cell, 'calendarWorkingDay', 'calendarWeekendDay');
    else
        replaceClass(cell, 'calendarWeekendDay', 'calendarWorkingDay');

    var span = cell.firstChild;
    if(span && span.nodeName == 'SPAN')
    {
        if(span.className.indexOf('CurrentMonth') != -1)
            span.className = isHoliday ? "calendarCurrentMonthWeekendDay" : "calendarCurrentMonthWorkingDay";
        else
            span.className = isHoliday ? "calendarOtherMonthWeekendDay" : "calendarOtherMonthWorkingDay";
    }
}

Calendar.prototype.openPopup =
function Calendar_openPopup(title, params, calendarHtml)
{
    var calendarWnd = window.open("", title, params);
    calendarWnd.opener = self;
    calendarWnd.document.write(calendarHtml);
    calendarWnd.document.close();
};

function TableElement(text, style, textStyle, link, hasFrame)
{
  this.text = text;
  this.style = style;
  this.textStyle = textStyle;
  this.link = link;
  this.hasFrame = hasFrame;
}

/**
* Styles you can use:
*
*  - calendarWeekDay
*  - calendarCurrentDay
*  - calendarWorkingDay
*  - calendarWeekendDay
*  - calendarCurrentMonthDay
*  - calendarOtherMonthDay
*
*/
Calendar.prototype.getCalendarHTML =
function Calendar_getCalendarHTML()
{
    var calendarTable = [];

    calendarTable[0] = [];
    for (var n = 0; n < 7; n++)
    {
        wd = (this.weekStartDay + n) % 7;
        style = this.headerDayDurations[wd] == 0 ? "calendarWeekendDayHeader" : "calendarWorkingDayHeader";
        calendarTable[0][n] = new TableElement(this.weekDays[wd], style, null, null, false);
    }

    var lastDayToGetDuration = new Date( this.nextMonth );
    lastDayToGetDuration.setDate( 6 );  // six days is max possible number of days shown of next month

    var schedule = this.scheduleCalculator.getEffectiveSchedule( this.firstDay, lastDayToGetDuration );
    var dayNum = 0;

    var curWeek = 1;
    var curDay = new Date(this.firstDay);
    while (curDay.getMonth() == this.baseDate.getMonth() ||
           curDay.getMonth() == this.firstDay.getMonth())
    {
        calendarTable[curWeek] = [];
        for (var currentWeekDay = 0; currentWeekDay < 7; currentWeekDay++)
        {
            var text;
            var style;
            var textStyle;
            var link;
            var hasFrame = this.startFrameDate != null && this.endFrameDate != null
                && curDay.getTime() >= this.startFrameDate.getTime() && curDay.getTime() <= this.endFrameDate.getTime();
            var currentDayNow = curDay.getFullYear() == this.today.getFullYear() &&
                                curDay.getMonth()    == this.today.getMonth() &&
                                curDay.getDate()     == this.today.getDate();
            var dayIsHoliday = schedule[dayNum++] == false;

            // If other month days should not be displayed and day currently being processed is not form base month
            if(!this.displayDaysOfOtherMonths && curDay.getMonth() != this.baseDate.getMonth()){

                // Create empty cell with no link, add it to calendar table and continue with next day
                text = "&nbsp;";
                style = "calendarEmptyDayCell";
                textStyle = dayIsHoliday ?  "calendarOtherMonthWeekendDay" : "calendarOtherMonthWorkingDay";
                calendarTable[curWeek][currentWeekDay] = new TableElement(text, style, textStyle, null, hasFrame);
                curDay.setDate(curDay.getDate() + 1);
                continue;   // Conitnue with next day
            }

            style = dayIsHoliday ? this.nonWorkDayStyle : this.workDayStyle;

            if (currentDayNow)
            {
                style += " calendarCurrentDay";
            }

            if (curDay.getMonth() == this.baseDate.getMonth())
            {
                textStyle = dayIsHoliday ?  "calendarCurrentMonthWeekendDay" : "calendarCurrentMonthWorkingDay";
            }
            else
            {
                textStyle = dayIsHoliday ?  "calendarOtherMonthWeekendDay" : "calendarOtherMonthWorkingDay";
            }

            text = curDay.getDate();

            if(currentDayNow != true || (currentDayNow == true && this.linkCurrentDay == true))
            {
                var goToDate = dateToStr(this.validDay(curDay));
                if(this.inPopup)
                    link="javascript: window.opener." + this.callbackFunc + "('" + goToDate + "', this);" +
                        "window.close();";
                else
                    link="javascript: if(typeof " + this.callbackFunc + " != typeof undefined){" + this.callbackFunc + "('" + goToDate + "', this);}";
            } else link = null;

            calendarTable[curWeek][currentWeekDay] = new TableElement(text, style, textStyle, link, hasFrame);
            curDay.setDate(curDay.getDate() + 1);
        }
        curWeek++;
    }

    var calendarHtml = "";


    calendarHtml += "<table id='calendar_table' width=\"100%\" border=0 bgcolor=" + Calendar.BORDER_COLOR + " cellspacing=1 cellpadding=0 class=calendarTable";
    if(this.highliteFrame){
        calendarHtml += " onmouseout='if (typeof Calendar_calendarTableOut != typeof undefined) Calendar_calendarTableOut(event)' onmouseover='if (typeof Calendar_calendarTableOver != typeof undefined) Calendar_calendarTableOver(event, this)'"
    }
    calendarHtml += ">";
    if(this.header)
       calendarHtml += "<tr><td class=\"currentMonthHeader\" colspan=\"7\" align=\"center\">" +
            this.getMonthLabel() + ", " + this.getYearLabel() + "</td>";

    this.weeksCount = calendarTable.length;
    for (var r = 0; r < calendarTable.length; r++)
        calendarHtml += this.getCalendarRowHTML(calendarTable, r);

    calendarHtml += "</table>";
    var frame = "<div id='calendar_frame' style='padding:0 0 0 0; margin:0 0 0 0; cursor:pointer;display:none;position:absolute;z-index:1001;border-style:solid;border-color:"+ Calendar.CURSOR_BG_COLOR +";' ></div>";
    calendarHtml += frame;
    return calendarHtml;
};

Calendar.prototype.validDay = function(day) {
    if (this.changeMonthIfDateOutsideCurrentMonth) return day;

    if(
        day.getMonth() < this.baseDate.getMonth() && day.getFullYear() == this.baseDate.getFullYear() ||
        day.getFullYear() < this.baseDate.getFullYear()
    )
        return new Date(this.baseDate.getFullYear(), this.baseDate.getMonth(), 1);

    if(
        day.getMonth() > this.baseDate.getMonth() && day.getFullYear() == this.baseDate.getFullYear() ||
        day.getFullYear() > this.baseDate.getFullYear()
    )
        return new Date(this.baseDate.getFullYear(), this.baseDate.getMonth(), this.getMaxDayOfMonth(this.baseDate.getFullYear(), this.baseDate.getMonth()));

    return day;
};

// Returns maximum day of month by specified month and year.
Calendar.prototype.getMaxDayOfMonth = function(year, month) {
    switch( month )
    {
        case 0: case 2: case 4: case 6: case 7: case 9: case 11: return 31;
        case 3: case 5: case 8: case 10: case 7: case 9: case 11: return 30;
        case 1: return (year % 4 == 0 && year % 100 != 0 || year % 400 == 0) ? 29 : 28;
    }
};


Calendar.prototype.getCalendarRowHTML =
function Calendar_getCalendarRowHTML(calendarTable, r)
{
    if(this.highliteFrame && r != 0)
        var html = "<tr id='calendar_row_" + r +"' onmouseover='" + this.onmouseOverHandler + "' onmouseout='" + this.onmouseOutHandler + "'" + ">";
    else
        var html = "<tr id='calendar_row_0'>";
    for (var day = 0; day < 7; day++)
    {
        var elem = calendarTable[r][day];
        var content = elem.text;
        var style = " class='" + elem.style + "'";

        html += "<td " + style;
        if (elem.link)
            html += " onclick=\"" + elem.link + "\"";
            //html += " onclick=\"alert('"+escape(elem.link)+"')\"";
        var borderStyle;
        if (calendarTable[r][day].hasFrame) {
            var color = Calendar.CURSOR_BG_COLOR;
            if (day == 0)
                borderStyle = 'padding:1px 2px 1px 1px; border-width: 2px 0px 2px 2px; border-color:' + color + ' ' + Calendar.DAY_BORDER_COLOR + ' ' + color + ' ' + color + ';';
            else if (day == 6)
                borderStyle = 'padding:1px 1px 1px 2px; border-width: 2px 2px 2px 0px; border-color:' + color + ' ' + color + ' ' + color + ' ' + Calendar.DAY_BORDER_COLOR;
            else
                borderStyle = 'padding:1px 2px 1px 2px; border-width: 2px 0px 2px 0px; border-color:' + color + ' ' + Calendar.DAY_BORDER_COLOR + ' ' + color + ' ' + Calendar.DAY_BORDER_COLOR;
            html += " style='" + borderStyle + "'" + "id='Calendar_frameElement'";
        }

        if (elem.link && !this.highliteFrame)
            html += " onmouseover='" + this.onmouseOverHandler + "' onmouseout='" + this.onmouseOutHandler + "'";

        html += ">";

        if (elem.textStyle)
            html += "<span class='" + elem.textStyle + "'>" + content + "</span></td>";
        else
            html += content + "</td>";
    }
    html += "</tr>";
    return html;
};

Calendar.prototype.setMonths =
function Calendar_setMonths(months)
{
    this.months = months;
};

Calendar.prototype.setHeader =
function Calendar_setHeader(header)
{
    this.header = header;
};

Calendar.prototype.setLinkCurrentDay =
function Calendar_setLinkCurrentDay(linkCurDay)
{
   this.linkCurrentDay = linkCurDay;
};

Calendar.prototype.setWeekDays =
function Calendar_setWeekDays(weekDays)
{
    this.weekDays = weekDays;
};

Calendar.prototype.setDisplayDaysOfOtherMonths =
function Calendar_setDisplayDaysOfOtherMonths(displayDaysOfOtherMonths){

    this.displayDaysOfOtherMonths = displayDaysOfOtherMonths;
};

Calendar.prototype.getCallback =
function Calendar_getCallback()
{
    return this.callbackFunc;
};

Calendar.prototype.getMonthLabel =
function Calendar_getMonthLabel()
{
    return this.months[this.baseDate.getMonth()];
};

Calendar.prototype.getYearLabel =
function Calendar_getYearLabel()
{
    return this.baseDate.getFullYear();
};

Calendar.prototype.setStyleNames =
function Calendar_setStyleNames(workDayStyle, nonWorkDayStyle)
{
    this.workDayStyle = workDayStyle;
    this.nonWorkDayStyle = nonWorkDayStyle;
};

Calendar.prototype.getTodayDay =
function Calendar_getTodayDay()
{
    return this.today.getDate();
};

Calendar.prototype.getTodayMonth =
function Calendar_getTodayMonth()
{
    return this.months[this.today.getMonth()];
};

Calendar.prototype.getTodayMonthNumber =
function Calendar_getTodayMonthNumber()
{
    return this.today.getMonth();
};

Calendar.prototype.getTodayYear =
function Calendar_getTodayYear()
{
    return this.today.getFullYear();
};

Calendar.prototype.getCurrentDay =
function Calendar_getCurrentDay()
{
    return this.baseDate.getDate();
};

Calendar.prototype.getCurrentMonth =
function Calendar_getCurrentMonth()
{
    return this.months[this.baseDate.getMonth()];
};

Calendar.prototype.getCurrentMonthNumber =
function Calendar_getCurrentMonthNumber()
{
    return this.baseDate.getMonth();
};

Calendar.prototype.getCurrentYear =
function Calendar_getCurrentYear()
{
    return this.baseDate.getFullYear();
};

Calendar.prototype.getNextMonth =
function Calendar_getNextMonth()
{
    return this.months[this.prevMonth];
};

Calendar.prototype.getPrevMonth =
function Calendar_getPrevMonth()
{
    return this.months[this.nextMonth];
};

Calendar.prototype.getNextMonthNumber =
function Calendar_getNextMonthNumber()
{
    return this.months[this.prevMonth];
};

Calendar.prototype.getPrevMonthNumber =
function Calendar_getPrevMonthNumber()
{
    return this.months[this.nextMonth];
};

Calendar.prototype.getCustomDateURL =
function Calendar_getCustomDateURL(year, month, day)
{
    return "javascript:window.opener.showCalendar('" + year + "-" + month + "-" + day + "', '" + dateToStr(this.today) + "', " + this.startDate.getFullYear() + ", " + this.endDate.getFullYear() + ", '" + this.callbackFunc + "');"
};

Calendar.prototype.getCustomDateJS =
function Calendar_getCustomDateJS(year, month, day)
{
    return "window.opener.showCalendar('" + year + "-" + month + "-" + day + "', '" + dateToStr(this.today) + "', " + this.startDate.getFullYear() + ", " + this.endDate.getFullYear() + ", '" + this.callbackFunc + "');"
};

Calendar.prototype.getEndDate =
function Calendar_getEndDate()
{
   return this.endDate;
};

Calendar.prototype.getStartDate =
function Calendar_getStartDate()
{
   return this.startDate;
};


// YYYY-MM-DD
function strToDate(strDateTime)
{
    var re = /^(\d+)\-(\d+)\-(\d+)$/;

    if (!re.exec(strDateTime))
        return alert("Invalid Datetime format: "+ strDateTime);

    // TODO: migrate to Date.create() defined in date_js.jsp
    return newDate(RegExp.$1, RegExp.$2 - 1, RegExp.$3);
}

// YYYY-MM-DD
function dateToStr(date)
{
    return String(date.getFullYear() + "-" + (date.getMonth() + 1) + "-" + date.getDate());
}

function getSelectedValue(selector)
{
    if (selector && selector.options)
        return selector.options[selector.selectedIndex].value;
}
