function BalancesViewer($containerEl)
{
    TemplateView.call(this, $containerEl);
}

BalancesViewer.prototype = _.create(TemplateView.prototype, {

    html: '<div class="balancesViewer">' +
          '  <div class="calendar">' +
          '   <span class="calendarSpan"><%=calendarLabel%></span>' +
          '   <span class="calendarPlaceholder"></span>' +
          '  </div>' +
          '  <span class="separator"></span>' +
          '  <div class="balances">' +
          '   <div class="ptoBalance">' +
          '     <span class="ptoLabel"><%=ptoLabel%></span>' +
          '     <span class="ptoValue"></span>' +
          '   </div>' +
          '   <div class="sickBalance">' +
          '     <span class="sickLabel"><%=sickLabel%></span>' +
          '     <span class="sickValue"></span>' +
          '  </div></div>' +
          '</div>',

    updateOnInit: false,

    messageKeys: function()
    {
        return {
            calendarLabel: "common.balancesViewer.label",
            ptoLabel: "common.balancesViewer.ptoLabel",
            sickLabel: "common.balancesViewer.sickLabel"
        }
    },

    initComponents: function(config, onDateChangeCallback)
    {
        this.ptoFormatter = new PTOFormatter().init(config.hoursPerDay);
        this.sickFormatter = new SickFormatter().init(config.hoursPerDay);

        this.displayPTOBalance = config.displayPto || false;
        this.displaySickBalance = config.displaySick || false;
        this.onDateChange = onDateChangeCallback;

        this.$calendarLabel = this.$containerEl.find('.calendarSpan');
        this.$calendar = this.$containerEl.find('.calendarPlaceholder');
        this.$balances = this.$containerEl.find('.balances');
        this.$ptoValue = this.$balances.find('.ptoValue');
        this.$sickValue = this.$balances.find('.sickValue');
        this._createDateSelector(config);
    },

    update: function(data)
    {
        this.displayPTOBalance = data.displayPto;
        this.displaySickBalance = data.displaySick;
        this._displayBalances(data.balances);
        this._updateMainClasses();
    },

    updateUserSchedule: function(config)
    {
        // nobody register any tips, but menu picker contains tips and this method called
        // can check if Ext.QuickTips.isEnabled(), then all works fine without substitution,
        // but no 100% guarantees, we can`t check if they disabled or not initialized
        var quickTipsUnregister = Ext.QuickTips.unregister;
        Ext.QuickTips.unregister = $.noop;
        this.dateSelector.destroy();
        Ext.QuickTips.unregister = quickTipsUnregister;

        this._createDateSelector(config);
    },

    _createDateSelector: function(config)
    {
        this.dateSelector = BalancesViewer.createDateSelector(this.$calendar[0], config.userInitialSchedule,
                                                              config.userSchedules, config.userHireDate, config.userReleaseDate);
        if (this.onDateChange)
            this.dateSelector.on("change", this.onDateChange);
        return this.dateSelector;
    },

    getDate: function()
    {
        return this.dateSelector.getValue();
    },

    setDate: function(date)
    {
        this.dateSelector.setValue(date);
    },

    _displayBalances: function( balances )
    {
        this._displayBalance(this.$ptoValue, this.ptoFormatter, balances.PTOBalance);
        this._displayBalance(this.$sickValue, this.sickFormatter, balances.sickBalance);
    },

    _displayBalance: function( $valueEl, formatter, balance )
    {
        var isNegative = !_.isNull(balance) && !!formatter.isNegativePTO(balance);
        $valueEl.toggleClass("negative", isNegative);
        $valueEl.toggleClass("empty", _.isNull(balance));
        $valueEl.html(formatter.format( balance ));
    },

    _updateMainClasses: function()
    {
        this.$el.toggleClass("showPTO", this.displayPTOBalance);
        this.$el.toggleClass("showSick", this.displaySickBalance);
    },

    isDisplayedPto: function()
    {
        return this.displayPTOBalance;
    },

    isDisplayedSick: function()
    {
        return this.displaySickBalance;
    }
});

BalancesViewer.createDateSelector = function(calendarSpan, userInitialSchedule, userSchedules, userHireDate, userReleaseDate)
{
    var minValue = new Date( 1970, 0, 1 );
    var minText = undefined;
    var maxValue = MAX_ALLOWED_DATE_FOR_TIME_TRACK;
    var maxText = undefined;

    if( userHireDate )
    {
        minValue = userHireDate.parseJSONDate();
        minText = messageResource.getMessage("common.balancesViewer.calendar.minText");
    }

    if( userReleaseDate )
    {
        maxValue = userReleaseDate.parseJSONDate();
        maxText = messageResource.getMessage("common.balancesViewer.calendar.maxText");
    }

    var defaultDate = currentUserTime().clearTime(); // today
    if( defaultDate > maxValue ) defaultDate = maxValue;

    return new DateSelector({
        minValue: minValue,
        maxValue: maxValue,
        minText: minText,
        maxText: maxText,
        showOkCancelButtons: false,
        weekStartDay: SystemSettings.weekStartDay - 1,
        name: "dateSelector",
        renderTo: calendarSpan,
        scheduleCalculator: new UserScheduleCalculator().init( invertedDaysDateMap, userInitialSchedule, userSchedules ),
        date: defaultDate,
        format: Date.convertSmartToExtDateFormat( SystemSettings.dateFormat.replace(/MMMM/, 'MMM') ), // short month format
        displayValue: function ( date )
        {
            this.setText( date
                                  ? areDatesEqual( date, currentUserTime() ) ? messageResource.getMessage( "common.datepicker.today" ) : this.formatDate( date )
                                  : messageResource.getMessage( "common.date_selector.select_date" ) );
        }
    });
};