// abstract class
BaseBalancesViewerController = function($containerEl) { this.$containerEl = $containerEl};

BaseBalancesViewerController.prototype = _.create(Object.prototype, {

    init: function()
    {
        this.spinner = new SpinnerController();

        this.balancesViewer = this.createBalanceViewer(this.$containerEl);

        var userWorkScheduleData = this.getUserWorkScheduleData();
        if (_.isNull(userWorkScheduleData))
            return false;

        this.userWorkScheduleData = userWorkScheduleData;
        this.balancesViewer.init().initComponents(this._getConfig(), this.onDateChange);

        // // called on first time update, then changed
        // this._reInit = _.bind(function()
        //                          {
        //                              var userWorkScheduleData = this.getUserWorkScheduleData();
        //                              if (_.isNull(userWorkScheduleData))
        //                                  return false;
        //
        //                              this.userWorkScheduleData = userWorkScheduleData;
        //                              this.balancesViewer.init().initComponents(this._getConfig(), this.onDateChange);
        //                              this._reInit = this._reInitBalanceViewer;
        //                              return true;
        //                          }, this);

        return this;
    },

    createBalanceViewer: function($containerEl)
    {
        // abstract
    },

    getUserWorkScheduleData: function()
    {
        // abstract
    },

    getFormatterSettings: function()
    {
        // abstract
    },

    onDateChange: function()
    {
        // abstract
    },

    _getConfig: function()
    {
        var workScheduleData = this.getUserWorkScheduleData();
        var formatterSettings = this.getFormatterSettings();

        return {
            userHireDate: workScheduleData.hireDate,
            userReleaseDate: workScheduleData.releaseDate,
            userInitialSchedule: workScheduleData.initialSchedule,
            userSchedules: workScheduleData.schedules,
            hoursPerDay: formatterSettings.workHoursPerDay
        };
    },

    _reInitBalanceViewer: function(config)
    {
        // if object changed then we need to remake dateSelector
        var userWorkScheduleData = this.getUserWorkScheduleData();
        if (this.userWorkScheduleData !== userWorkScheduleData)
        {
            this.userWorkScheduleData = userWorkScheduleData;
            this.balancesViewer.updateUserSchedule(config);
        }

        this.balancesViewer.showInHours = config.showInHours;
    },

    updateBalances: function(date, balanceData )
    {
        var userWorkScheduleData = this.getUserWorkScheduleData();
        if (_.isNull(userWorkScheduleData))
        {
            this.spinner.stop();
            return;
        }
        this._reInitBalanceViewer(this._getConfig());

        if (!_.isNull(date) && !areDatesEqual(this.getViewerDate(), date))
            this.balancesViewer.setDate(date);

        this.balancesViewer.update(balanceData);
        this.spinner.stop();
    },

    getViewerDate: function()
    {
        return this.balancesViewer.getDate();
    },

    spin: function()
    {
        this.spinner.spin("balancesViewerSpinner");
    }
});
