function ChangeMyWeekApprovalStatusControl ()
{
    this.constructor = ChangeMyWeekApprovalStatusControl;
}

ChangeMyWeekApprovalStatusControl.prototype.init = function(userId, weekStatus, disabled, dateFrom, dateTo, onChangeCallback, getWeekStatusHistoryCallback)
{
    var self = this;

    this.disabled = disabled;

    this.onChangeCallback = onChangeCallback;
    this.getWeekStatusHistoryCallback = getWeekStatusHistoryCallback;

    this.controlBlock = $(".changeWeekApprovalStatusControl");
    this.pageContainer = $(".mainContentPadding");
    this.statusMessage = this.controlBlock.find(".statusText");
    this.notReadyButton = this.controlBlock.find(".notReadyLabelCell");
    this.readyButton = this.controlBlock.find(".readyLabelCell");
    this.switcher = this.controlBlock.find(".switcherImageCell");

    this.currentState = null;
    this.controlBlock.show();
    this.update(weekStatus);
    this.defaultValue = weekStatus;
    this.value = weekStatus;

    this.userId = userId;
    this.dateFrom = dateFrom;
    this.dateTo = dateTo;

    this.controlBlock.show();

    if (!this.disabled)
    {
        // if the approval status control isn't disabled all needed elements to show history will be initialized

        this.historyButton = this.controlBlock.find(".ttaHistoryButton");

        this.historyButton.click(function() { self.showWeekStatusHistory(this); });

        this.historyLightboxDiv = $("#ttaHistoryLightBoxDiv");
        this.historyPopup = new DropdownDialog({
                                                    wrapperClass: 'ttaHistoryLightBox',
                                                    dialogDiv: this.historyLightboxDiv,
                                                    hideButtons: true,
                                                    isWithBeak: true,
                                                    hideOnResize: true
                                              });

        this.historySpinnerController = new SpinnerController();
    }

    return this;
};

ChangeMyWeekApprovalStatusControl.prototype.states =
{
    NOT_READY_DISABLED: {
        getClassName: function() { return "notReady" },
        updateUI: function(control)
        {
            control.controlBlock.removeClass("showStatus").addClass("showSwitcher disabledMyWeekApprovalStatusControl");
            PageOverlayTipsManager.getInstance().refreshDynamicTips();
        },
        getCurrentStatus: function() { return EffectiveWeekApprovalStatus.NOT_READY }
    },
    NOT_READY: {
        getClassName: function() { return "notReady" },
        updateUI: function ( control ) {
            control.controlBlock.removeClass( "showStatus" ).addClass( "showSwitcher" );
            control.notReadyButton.off( "click" );
            control.readyButton.off( "click" ).click( function() { control.update( EffectiveWeekApprovalStatus.READY ) } );
            control.switcher.off( "click" ).click( function() { control.update( EffectiveWeekApprovalStatus.READY ) } );
            PageOverlayTipsManager.getInstance().refreshDynamicTips();
        },
        getCurrentStatus: function () { return EffectiveWeekApprovalStatus.NOT_READY }
    },
    READY: {
        getClassName: function() { return "ready" },
        updateUI: function ( control ) {
            control.controlBlock.removeClass( "showStatus" ).addClass( "showSwitcher" );
            control.notReadyButton.off( "click" ).click( function() { control.update( EffectiveWeekApprovalStatus.NOT_READY ) } );
            control.readyButton.off( "click" );
            control.switcher.off( "click" ).click( function() { control.update( EffectiveWeekApprovalStatus.NOT_READY ) } );
            PageOverlayTipsManager.getInstance().refreshDynamicTips();
        },
        getCurrentStatus: function () { return EffectiveWeekApprovalStatus.READY }
    },
    APPROVED: {
        getClassName: function() { return "approved" },
        updateUI: function ( control ) {
            control.controlBlock.removeClass( "showSwitcher" ).addClass( "showStatus" );
            control.statusMessage.text(messageResource.getMessage("change_week_approval_status.status.approved"));
        },
        getCurrentStatus: function () { return EffectiveWeekApprovalStatus.APPROVED }
    },
    REJECTED: {
        getClassName: function() { return "rejected" },
        updateUI: function ( control ) {
            control.controlBlock.removeClass( "showSwitcher" ).addClass( "showStatus" );
            control.statusMessage.text(messageResource.getMessage("change_week_approval_status.status.rejected"));
        },
        getCurrentStatus: function () { return EffectiveWeekApprovalStatus.REJECTED }
    },
    AUTO_APPROVED: {
        getClassName: function() { return "autoApproved" },
        updateUI: function ( control ) {
            control.controlBlock.removeClass( "showSwitcher" ).addClass( "showStatus" );
            control.statusMessage.text(messageResource.getMessage("change_week_approval_status.status.auto_approved"));
        },
        getCurrentStatus: function () { return EffectiveWeekApprovalStatus.AUTO_APPROVED }
    },
    AUTO_APPROVED_DISABLED: {
        getClassName: function() { return "autoApproved" },
        updateUI: function(control)
        {
            control.controlBlock.removeClass("showSwitcher").addClass("showStatus disabledMyWeekApprovalStatusControl");
            control.statusMessage.text(messageResource.getMessage("change_week_approval_status.status.auto_approved"));
        },
        getCurrentStatus: function () { return EffectiveWeekApprovalStatus.AUTO_APPROVED }
    }
};

ChangeMyWeekApprovalStatusControl.prototype.updateState = function ( weekStatus )
{
    if (this.disabled)
    {
        this.currentState = weekStatus == EffectiveWeekApprovalStatus.AUTO_APPROVED ?
                            this.states.AUTO_APPROVED_DISABLED :
                            this.states.NOT_READY_DISABLED;
    }
    else
    {
        var newState;
        switch (weekStatus)
        {
            case EffectiveWeekApprovalStatus.NOT_READY:
                newState = this.states.NOT_READY;
                break;
            case EffectiveWeekApprovalStatus.READY:
                newState = this.states.READY;
                break;
            case EffectiveWeekApprovalStatus.APPROVED:
                newState = this.states.APPROVED;
                break;
            case EffectiveWeekApprovalStatus.REJECTED:
                newState = this.states.REJECTED;
                break;
            case EffectiveWeekApprovalStatus.AUTO_APPROVED:
                newState = this.states.AUTO_APPROVED;
        }
        this.currentState = newState;
    }
};

ChangeMyWeekApprovalStatusControl.prototype.updateUI = function ()
{
    this.currentState.updateUI(this);
};

ChangeMyWeekApprovalStatusControl.prototype.getCurrentStatus = function ()
{
    return this.currentState.getCurrentStatus();
};

ChangeMyWeekApprovalStatusControl.prototype.update = function ( newWeekStatus )
{
    if( this.currentState != null && this.getCurrentStatus() == newWeekStatus )
        return;
    var changed = this.currentState != null;
    var switcherWasShown = this.switcherIsShown();
    var oldState = this.currentState;
    this.updateState(newWeekStatus);
    this.controlBlock.addClass( this.currentState.getClassName() );
    this.pageContainer.addClass( this.currentState.getClassName() );
    if( oldState != null )
    {
        this.controlBlock.removeClass( oldState.getClassName() );
        this.pageContainer.removeClass( oldState.getClassName() );
    }
    this.updateUI();
    this.value = newWeekStatus;
    if( changed )
        if( this.onChangeCallback ) this.onChangeCallback();
};

ChangeMyWeekApprovalStatusControl.prototype.showSwitcherIfNeeded = function()
{
    if( this.getCurrentStatus() == EffectiveWeekApprovalStatus.APPROVED ||
        this.getCurrentStatus() == EffectiveWeekApprovalStatus.REJECTED )
    {
        this.update( EffectiveWeekApprovalStatus.NOT_READY );
    }
};

ChangeMyWeekApprovalStatusControl.prototype.switcherIsShown = function()
{
    return this.currentState != null
           && ( this.getCurrentStatus() == EffectiveWeekApprovalStatus.NOT_READY
                || this.getCurrentStatus() == EffectiveWeekApprovalStatus.READY );
};

ChangeMyWeekApprovalStatusControl.prototype._startSpinHistory = function(pressedButton){
    $(pressedButton).addClass("loading");
    this.historySpinnerController.spinInElement($(pressedButton));
};

ChangeMyWeekApprovalStatusControl.prototype._stopSpinHistory = function(pressedButton) {
    $(pressedButton).removeClass("loading");
    this.historySpinnerController.stop();
};

ChangeMyWeekApprovalStatusControl.prototype._showHistoryPopup = function(pressedButton) {

    var bottom = this.isElementInBottomControl(pressedButton);
    this.historyPopup.setBeak(bottom);

    var popupConfig;
    if (bottom)
        popupConfig = {
            offset: "30px -12px",
            my:  "right bottom",
            at: "right top",
            collision: "none"
        };
    else
        popupConfig = {
            offset: "30px 12px",
            my:  "right top",
            at: "right bottom",
            collision: "none"
        };

    this.historyPopup.showAt( pressedButton, popupConfig );
};

ChangeMyWeekApprovalStatusControl.prototype.isElementInBottomControl = function ( element )
{
    return $(element).closest(".weekApprovalStatusControlOuterContainer" ).hasClass("bottom");
};

ChangeMyWeekApprovalStatusControl.prototype.showWeekStatusHistory = function (pressedButton)
{
    this._startSpinHistory(pressedButton);
    var self = this;

    this.getWeekStatusHistoryCallback(this.userId, this.dateFrom, this.dateTo,
        function (historyRecords) {
            self._stopSpinHistory(pressedButton);
            renderTTAHistoryTable(self.historyLightboxDiv, self.userId, historyRecords);
            self._showHistoryPopup(pressedButton);
        },
        function ( ) {
            self._stopSpinHistory(pressedButton);
        }
    );
};

