function ChangeOtherWeekApprovalStatusControl ()
{
    this.constructor = ChangeOtherWeekApprovalStatusControl;
}

ChangeOtherWeekApprovalStatusControl.prototype.init = function(userData, dateFrom, dateTo, weekStatus, disabledControl, managerId,
                                                               approveCallback, rejectCallback, revokeCallback, getWeekStatusHistoryCallback)
{
    var self = this;

    this.disabledControl = disabledControl;

    this.pageContainer = $(".mainContentPadding");

    this.controlBlocks = $(".changeWeekApprovalStatusControl");
    this.currentStatusCells = this.controlBlocks.find(".currentStatusCell");
    this.approveButtons = this.controlBlocks.find(".approveButton");
    this.rejectButtons = this.controlBlocks.find(".rejectButton");
    this.revokeLinks = this.controlBlocks.find(".revokeLink");
    this.historyButtons = this.controlBlocks.find(".ttaHistoryButton");

    this.approveCallback = approveCallback;
    this.rejectCallback = rejectCallback;
    this.revokeCallback = revokeCallback;
    this.getWeekStatusHistoryCallback = getWeekStatusHistoryCallback;

    this.approveButtons.click( function ()
                               {
                                   if( !self._isDisabled() )
                                       self.approve( this );
                               }
    );

    this.rejectButtons.click( function ()
                              {
                                  if ( !self._isDisabled() )
                                    self.reject(this);
                              }
    );

    if (!this.disabledControl)
    {
        // initialize tooltip of buttons
        this.buttonsTooltip = $("#changeWeekApprovalStatusButtonsTooltip");
        this.approveButtons.hover(function() { self.showButtonTooltip(this) }, function() { self.hideButtonTooltip() });
        this.rejectButtons.hover(function() { self.showButtonTooltip(this) }, function() { self.hideButtonTooltip() })
    }

    this.revokeLinks.click( function ()
                            {
                                if( self._disableControls() )
                                    self.revoke(this);
                            }
    );
    this.historyButtons.click( function () { self.showWeekStatusHistory(this); } );

    this.userId = userData.userId;
    this.userData = userData;
    this.dateFrom = dateFrom;
    this.dateTo = dateTo;
    this.managerId = managerId;

    this.update(weekStatus);
    this.controlBlocks.show();

    this.rejectCommentDialog = new RejectCommentDialog().init("rejectWeekCommentDialog",
                                                              function(){return self._reject()});
    this.historyLightboxDiv = $("#ttaHistoryLightBoxDiv");
    this.historyPopup = new DropdownDialog( {
                                             wrapperClass: 'ttaHistoryLightBox',
                                             dialogDiv: this.historyLightboxDiv,
                                             hideButtons: true,
                                             isWithBeak: true,
                                             hideOnResize: true
                                         } );

    this.historySpinnerController = new SpinnerController();

    return this;
};

ChangeOtherWeekApprovalStatusControl.prototype.states =
{
    NOT_READY_DISABLED: {
        approve: function() {},
        reject: function() {},
        revoke: function() {},
        updateUI: function(control)
        {
            control.controlBlocks.addClass("disabledOtherWeekApprovalStatusControl");
            control._updateUI(this.getClassName(), "change_week_approval_status.status.not_ready");
        },
        getCurrentStatus: function() { return EffectiveWeekApprovalStatus.NOT_READY },
        showWeekStatusHistory: function() {},
        getClassName: function() { return "notReady"; }
    },
    NOT_READY: {
        approve: function ( control, pressedButton ) { control._approve(pressedButton); },
        reject: function ( control, pressedButton ) { control._openRejectCommentDialog(pressedButton); },
        revoke: function () {},
        updateUI: function ( control ) {
            control._updateUI( this.getClassName(), "change_week_approval_status.status.not_ready");
        },
        getCurrentStatus: function () { return EffectiveWeekApprovalStatus.NOT_READY },
        showWeekStatusHistory: function ( control, pressedButton ) { control._showWeekStatusHistory(pressedButton); },
        getClassName: function () { return "notReady"; }
    },
    READY: {
        approve: function ( control, pressedButton ) { control._approve(pressedButton); },
        reject: function ( control, pressedButton ) { control._openRejectCommentDialog(pressedButton); },
        revoke: function () {},
        updateUI: function ( control ) {
            control._updateUI( this.getClassName(), "change_week_approval_status.status.ready");
        },
        getCurrentStatus: function () { return EffectiveWeekApprovalStatus.READY },
        showWeekStatusHistory: function ( control, pressedButton ) { control._showWeekStatusHistory(pressedButton); },
        getClassName: function () { return "ready"; }
    },
    APPROVED: {
        approve: function () {},
        reject: function ( control, pressedButton ) { control._openRejectCommentDialog(pressedButton); },
        revoke: function ( control, pressedButton ) { control._revoke(pressedButton); },
        updateUI: function ( control ) {
            control._updateUI( this.getClassName(), "change_week_approval_status.status.approved");
        },
        getCurrentStatus: function () { return EffectiveWeekApprovalStatus.APPROVED },
        showWeekStatusHistory: function ( control, pressedButton ) { control._showWeekStatusHistory(pressedButton); },
        getClassName: function () { return "approved"; }
    },
    REJECTED: {
        approve: function ( control, pressedButton ) { control._approve(pressedButton); },
        reject: function () {},
        revoke: function ( control, pressedButton ) { control._revoke(pressedButton); },
        updateUI: function ( control ) {
            control._updateUI( this.getClassName(), "change_week_approval_status.status.rejected");
        },
        getCurrentStatus: function () { return EffectiveWeekApprovalStatus.REJECTED },
        showWeekStatusHistory: function ( control, pressedButton ) { control._showWeekStatusHistory(pressedButton); },
        getClassName: function () { return "rejected"; }
    },
    AUTO_APPROVED: {
        approve: function () {},
        reject: function () {},
        revoke: function () {},
        updateUI: function ( control ) {
            control._updateUI( this.getClassName(), "change_week_approval_status.status.auto_approved");
        },
        getCurrentStatus: function () { return EffectiveWeekApprovalStatus.AUTO_APPROVED },
        showWeekStatusHistory: function ( control, pressedButton ) { control._showWeekStatusHistory(pressedButton); },
        getClassName: function () { return "autoApproved"; }
    }
};

ChangeOtherWeekApprovalStatusControl.prototype._isDisabled = function ( )
{
    return this.controlBlocks.hasClass( "disabled" );
};

ChangeOtherWeekApprovalStatusControl.prototype._disableControls = function ( )
{
    if( !this._isDisabled() )
    {
        this.controlBlocks.addClass( "disabled" );
        return true;
    }
    return false;
};

ChangeOtherWeekApprovalStatusControl.prototype._enableControls = function ( )
{
    this.controlBlocks.removeClass( "disabled" );
};

ChangeOtherWeekApprovalStatusControl.prototype._updateUI = function ( currentStateClassName, currentStatusCaptionKey )
{
    for (var stateName in this.states)
    {
        var stateClassName = this.states[stateName].getClassName();
        this.controlBlocks.removeClass( stateClassName );
        this.pageContainer.removeClass( stateClassName );
    }
    this._enableControls();
    this.controlBlocks.addClass(currentStateClassName);
    this.pageContainer.addClass(currentStateClassName);
    this.currentStatusCells.text(messageResource.getMessage(currentStatusCaptionKey));
};

ChangeOtherWeekApprovalStatusControl.prototype.updateState = function ( weekStatus )
{
    var newState;
    if (this.disabledControl)
    {
        this.currentState = this.states.NOT_READY_DISABLED;
    }
    else
    {
        switch (weekStatus)
        {
            case EffectiveWeekApprovalStatus.NOT_READY:
                newState = this.states.NOT_READY;
                break;
            case EffectiveWeekApprovalStatus.READY:
                newState = this.states.READY;
                break;
            case EffectiveWeekApprovalStatus.APPROVED:
                newState = this.states.APPROVED;
                break;
            case EffectiveWeekApprovalStatus.REJECTED:
                newState = this.states.REJECTED;
                break;
            case EffectiveWeekApprovalStatus.AUTO_APPROVED:
                newState = this.states.AUTO_APPROVED;
        }
        this.currentState = newState;
    }
};

ChangeOtherWeekApprovalStatusControl.prototype._findNearestChangeStatusSpinnerPl = function (pressedButton)
{
    return $(pressedButton).closest(".weekApprovalStatusControlOuterContainer" ).find(".spinnerPlaceholder");
};

ChangeOtherWeekApprovalStatusControl.prototype._approve = function (pressedButton)
{
    var self = this;
    this.hideButtonTooltip();
    this.approveCallback(   this.userId, this.dateFrom,
                            this._findNearestChangeStatusSpinnerPl(pressedButton),
                            function (effectiveWeekStatus) {
                                self.update(effectiveWeekStatus);
                            },
                            function(){},
                            function() { return self._disableControls(); },
                            function() { return self._enableControls(); }
    );
};

ChangeOtherWeekApprovalStatusControl.prototype._reject = function ()
{
    if (!this.rejectCommentDialog.isValid())
    {
        alert(messageResource.getMessage("change_week_approval_status.reject_comment.too_long.alert"));
        return false;
    }

    var self = this;
    var comment = this.rejectCommentDialog.getComment();
    this.rejectCallback( this.userId, this.dateFrom, isNotEmpty( comment ) ? comment : null,
                         this._findNearestChangeStatusSpinnerPl( this.pressedRejectButton ),
                         function ( effectiveWeekStatus )
                         {
                             self.update( effectiveWeekStatus );
                         },
                         function(){},
                         function() { return self._disableControls(); },
                         function() { return self._enableControls(); }
    );
    return true;
};

ChangeOtherWeekApprovalStatusControl.prototype._revoke = function (pressedButton)
{
    var self = this;
    this.revokeCallback( this.userId, this.dateFrom, this.getCurrentStatus(),
                         this._findNearestChangeStatusSpinnerPl(pressedButton),
                         function (effectiveWeekStatus) {
                             self.update(effectiveWeekStatus);
                         }
    );
};

ChangeOtherWeekApprovalStatusControl.prototype._openRejectCommentDialog = function (pressedButton)
{
    // we can't pass parameter to onOkClick function (see initialization of this.rejectCommentDialog),
    //  so we save pressed reject button inside our object and then use it from _reject() method
    //  which is called inside onOkClick function.
    // in other case (_approve, _revoke) we use directly passed pressedButton element
    this.pressedRejectButton = pressedButton;

    this.hideButtonTooltip();

    var bottom = this.isElementInBottomControl(pressedButton);

    var popupConfig = {};
    popupConfig.userInfo = this.formatUserInfo();
    popupConfig.isBottom = bottom;
    this.rejectCommentDialog._open( pressedButton, popupConfig );
};

ChangeOtherWeekApprovalStatusControl.prototype._startSpinHistory = function(pressedButton){
    $(pressedButton).addClass("loading");
    this.historySpinnerController.spinInElement($(pressedButton));
};

ChangeOtherWeekApprovalStatusControl.prototype._stopSpinHistory = function(pressedButton) {
    $(pressedButton).removeClass("loading");
    this.historySpinnerController.stop();
};

ChangeOtherWeekApprovalStatusControl.prototype._showHistoryPopup = function(pressedButton) {

    var bottom = this.isElementInBottomControl(pressedButton);
    this.historyPopup.setBeak(bottom);

    var popupConfig;
    if (bottom)
        popupConfig = {
            offset: "30px -12px",
            my:  "right bottom",
            at: "right top",
            collision: "none"
        };
    else
        popupConfig = {
            offset: "30px 12px",
            my:  "right top",
            at: "right bottom",
            collision: "none"
        };

    this.historyPopup.showAt( pressedButton, popupConfig );
};

ChangeOtherWeekApprovalStatusControl.prototype.isElementInBottomControl = function ( element )
{
    return $(element).closest(".weekApprovalStatusControlOuterContainer" ).hasClass("bottom");
};

ChangeOtherWeekApprovalStatusControl.prototype._showWeekStatusHistory = function (pressedButton)
{
    this._startSpinHistory(pressedButton);
    var self = this;

    this.getWeekStatusHistoryCallback(this.userId, this.dateFrom, this.dateTo,
        function (historyRecords) {
            self._stopSpinHistory(pressedButton);
            renderTTAHistoryTable(self.historyLightboxDiv, self.managerId, historyRecords);
            self._showHistoryPopup(pressedButton);
        },
        function ( ) {
            self._stopSpinHistory(pressedButton);
        }
    );
};

ChangeOtherWeekApprovalStatusControl.prototype.updateUI = function ()
{
    this.currentState.updateUI(this);
};

ChangeOtherWeekApprovalStatusControl.prototype.approve = function (pressedButton)
{
    this.currentState.approve(this, pressedButton);
};

ChangeOtherWeekApprovalStatusControl.prototype.reject = function (pressedButton)
{
    this.currentState.reject(this, pressedButton);
};

ChangeOtherWeekApprovalStatusControl.prototype.revoke = function (pressedButton)
{
    this.currentState.revoke(this, pressedButton);
};

ChangeOtherWeekApprovalStatusControl.prototype.showWeekStatusHistory = function (pressedButton)
{
    this.currentState.showWeekStatusHistory(this, pressedButton);
};

ChangeOtherWeekApprovalStatusControl.prototype.getCurrentStatus = function ()
{
    return this.currentState.getCurrentStatus();
};

ChangeOtherWeekApprovalStatusControl.prototype.showButtonTooltip = function (hoveredButton)
{
    var $hoveredButton = $(hoveredButton);
    var tooltipText = messageResource.getMessage( $hoveredButton.hasClass("approveButton") ?
                                                     "change_week_approval_status.approve_button.tooltip" :
                                                     "change_week_approval_status.reject_button.tooltip");
    tooltipText +=" ";
    tooltipText += messageResource.getEscapedMessage("change_week_approval_status.user_of.tooltip", [this.formatUserInfo()]);

    this.buttonsTooltip.html(tooltipText);

    var rightPlace = $(window).scrollLeft() + $(window).width() - ( $hoveredButton.offset().left + 40 );

    this.buttonsTooltip.show();
    var tooltipWidth = this.buttonsTooltip.width()+1;
    this.buttonsTooltip.css("width", tooltipWidth);

    var offsetLeft = rightPlace > tooltipWidth + 30  ? $hoveredButton.offset().left + 40 : $hoveredButton.offset().left + rightPlace - tooltipWidth +10;

    this.buttonsTooltip.offset( {left: offsetLeft, top: $hoveredButton.offset().top+24} );
};

ChangeOtherWeekApprovalStatusControl.prototype.hideButtonTooltip = function ()
{
    this.buttonsTooltip.css("width", "");
    this.buttonsTooltip.css("top", "");
    this.buttonsTooltip.css("left", "");
    this.buttonsTooltip.hide();
};

ChangeOtherWeekApprovalStatusControl.prototype.update = function ( newWeekStatus )
{
    this.updateState(newWeekStatus);
    this.updateUI();
};

ChangeOtherWeekApprovalStatusControl.prototype.formatUserInfo = function ()
{
    return UserNameFormat.formatFML(this.userData.firstName, this.userData.lastName, this.userData.middleName);
};