/**
 *   require /js/controls/ListController.js
 *   require /js/util/AtMap.js
 */
/**
 * This class represent duallist selector
 * functionality:
 * - move items from left to right listbox and right to left listbox.
 * - also see ListController functionality.
 * @param allObjectsArray all Available object for this dual Controller.(class Array)
 * @param rightListObjectsKeys array  of special key(created by key generator)
 * for objects presented in right list box (class Array)
 * @param leftListController left list box controller (Class ListController)
 * @param rightListController right list box controller (Class ListController)
 */
function DualListController(allObjectsArray,
                            rightListObjectsKeys,
                            leftListController,
                            rightListController)
{
    this.allObjectsArray = allObjectsArray;
    this.rightListObjectsKeys = rightListObjectsKeys;
    this.leftList = leftListController;
    this.rightList = rightListController;

    this.allObjectsMap = new AtMap();
    this.allObjectsMap.addArrayToMap(this.allObjectsArray, this.rightList.itemKeyGenerator);

    this.leftList.setObjectsMap(this.generateListObjectsMapUseSubtractKeys(this.rightListObjectsKeys));

    this.rightList.setObjectsMap(this.generateListObjectsMapUseKeys(this.rightListObjectsKeys));
}

/**
 * update dual List Control.
 */
DualListController.prototype.update = function()
{
    this.leftList.updateList();
    this.rightList.updateList();
};

/**
 *  move all selected elements in right list to left list.
 * @return true if number of moved elements are more than 0;
 */
DualListController.prototype.moveLeft = function()
{
    return this.move(this.rightList, this.leftList);
};

/**
 * move all selected elements in left list to right list.
 * @return true if number of moved elements are more than 0;
 */
DualListController.prototype.moveRight = function()
{
    return this.move(this.leftList, this.rightList);
};

/**
 *
 * @param fromList list where selected elements are removed
 * @param toList list where selected elements are inserted
 * @return boolean true if number of moved elements are more than 0;
 */
DualListController.prototype.move = function(fromList, toList)
{
    var movingObjectsCounter = 0;
    var objectsToMove = fromList.getSelectedObjects();

    if (objectsToMove.length === 0)
        return false;

    toList.unselectObjects();
    for (var  i = 0; i < objectsToMove.length; i++)
    {
        var object = fromList.remove(objectsToMove[i]);
        if (object)
        {
            fromList.lazyUpdateList(object);
            toList.add(object);
            movingObjectsCounter++;
        }
    }
    fromList.scrollToVisibleArea();
    if (movingObjectsCounter > 0)
    {
        fromList.showEmptyListIfNeeded();
        toList.updateList();
    }
    return movingObjectsCounter > 0;
};

/**
 * This method create map use keys.
 * @param keys array of keys
 * @return map of objects associated with keys.
 */
DualListController.prototype.generateListObjectsMapUseKeys = function(keys)
{
    var tempArray = new Array();
    for (var i = 0; i < keys.length; i++)
    {
        var object = this.allObjectsMap.get(keys[i]);
        tempArray.push(object);
    }
    var map = new AtMap();
    map.addArrayToMap(tempArray, this.rightList.itemKeyGenerator);
    return map;
};

/**
 * This method create map with all avalable object except
 * objects associated with KeysToSubtract  object
 * @param keysToSubtract array of keys
 * @retrun map.
 */
DualListController.prototype.generateListObjectsMapUseSubtractKeys = function(keysToSubtract)
{
    var array = this.allObjectsMap.getValues();
    var tempArray = new Array();
    for (var i = 0; i < array.length; i++)
    {
        var object = array[i];
        var isObjectIncluded = true;
        for(var j = 0; j < keysToSubtract.length; j++ )
        {
            if (this.rightList.itemKeyGenerator.createKey(object) == keysToSubtract[j])
            {
                isObjectIncluded = false;
            }
        }
        if (isObjectIncluded)
        {
            tempArray.push(object);
        }
    }
    var map = new AtMap();
    map.addArrayToMap(tempArray, this.rightList.itemKeyGenerator);
    return map;
};
