function FieldController()
{

}

FieldController.LABEL_ERROR_CLASS = "field-label-error";

FieldController.prototype.init = function(elementId)
{
    this.initField(elementId, true);
};

FieldController.prototype.initField = function(elementId, grayText)
{
    this.fieldElement = document.getElementById( elementId );
    this.defaultMaxlength = this.fieldElement.getAttribute("maxlength");
    this.defaultValue = this.fieldElement.value;
    this.emptyCaptionShown = false;
    this.fieldElement.controller = this;
    var controller = this;

    if (grayText) {
        this.addListener("focus", function(){controller.hideGrayTextIfShown();});
        this.addListener("blur", function()
        {
            controller.fireBlur();
            if(controller.fieldElement.value != controller.defaultValue)
            {

                controller.fireChange();
            }
            controller.showGreyTextIfEmpty();
        });
    }

    this.addListener("keydown", function(){controller.fireChange();});
    this.addListener("change", function()
    {
        if(controller.fieldElement.value != controller.defaultValue)
            controller.fireChange();
        if (grayText)
            controller.showGreyTextIfEmpty();
    });
    if (grayText)
        this.showGreyTextIfEmpty();
};

FieldController.prototype.setDefaultValueIfNeeded = function()
{
    if ( this.isEmpty() )
        this.fieldElement.defaultValue = this.emptyCaption;
};

FieldController.prototype.onChangeHandler = undefined;
FieldController.prototype.onBlurHandler = undefined;
FieldController.prototype.emptyCaption = "";
FieldController.prototype.emptyCaptionStyleClass = "emptyCaption";

FieldController.prototype.hideGrayText = function()
{
    this.fieldElement.value = "";
    if (this.defaultMaxlength != null)
        this.fieldElement.setAttribute("maxlength", this.defaultMaxlength)
    this.fieldElement.className = "";
    this.fieldElement.spellcheck=true;
    this.emptyCaptionShown = false;
};

FieldController.prototype.hideGrayTextIfShown = function()
{
    if (this.emptyCaptionShown)
        this.hideGrayText();
};

FieldController.prototype.showGreyText = function( textToShow )
{
    this.fieldElement.className = this.emptyCaptionStyleClass;
    if (this.defaultMaxlength != null)
        this.fieldElement.setAttribute("maxlength", this.emptyCaption.length)
    this.fieldElement.value = textToShow ? textToShow : this.emptyCaption;
    this.fieldElement.spellcheck=false;
    this.emptyCaptionShown = true;
};

FieldController.prototype.showGreyTextIfEmpty = function()
{
    if (this.isEmpty())
        this.showGreyText();
};

FieldController.prototype.isEmpty = function()
{
    return this.emptyCaptionShown || trim(this.fieldElement.value).length == 0;
};

FieldController.prototype.addListener = function (event, listener)
{
    if (this.fieldElement.addEventListener)
        this.fieldElement.addEventListener(event, listener, false);
    else
        this.fieldElement.attachEvent('on' + event, listener);
};

FieldController.prototype.fireChange = function()
{
    if(this.onChangeHandler)
    {
        this.onChangeHandler(this);
    }
};

FieldController.prototype.fireBlur = function()
{
    if(this.onBlurHandler)
    {
        this.onBlurHandler(this);
    }
};

FieldController.prototype.getValue = function()
{
    return this.isEmpty() ? "" : getStringWithUnifiedNewLineSymbol(this.fieldElement.value);
};

FieldController.prototype.setValue = function( value )
{
    this.fieldElement.value = value;
};

FieldController.prototype.getElement = function()
{
    return this.fieldElement;
};

FieldController.prototype.setDisabled = function( isDisabled )
{
    this.fieldElement.disabled = isDisabled;
};

FieldController.prototype.isModified = function( )
{
    return trim(this.fieldElement.value) != trim(this.fieldElement.defaultValue);
};

