ListSelector = Ext.extend(AT.AbstractDropdownButton, {

    menuCls : "",
    listItems: [],
    textPropName: "text",
    valuePropName: "value",
    clsPropName: "styleCls",
    value: null,
    maxNonScrollableItems: 10,

    // private
    currentItem: null,

    initComponent: function()
    {
        var self = this;

        this.addEvents("change");

        this.itemClickHandler = function(item)
        {
            var wasActuallySet = self.setItem( item );
            if( wasActuallySet )
                self.fireEvent("change", self, item.origObj);
        };

        var mCls = "at-dropdown-list-btn-menu ";
        if( this.listItems.length > this.maxNonScrollableItems )
            mCls += "scrollable ";
        mCls += this.menuCls;


        this.menu =  new Ext.menu.Menu({ cls: mCls });

        if (this.listItems.length != 0)
        {
            this.addItems( this.listItems );

            if( !this.currentItem )
                this.setItem( this.menu.items.items[0]);
        }
        this.cls = "at-dropdown-list-btn " + (this.cls ? this.cls : "");
        this.ctCls = "at-dropdown-list-btn-ct " + (this.ctCls ? this.ctCls : "");

        AT.DropdownListButton.superclass.initComponent.call(this);
    },

    setItem: function( item )
    {
        if( this.currentItem )
        {
            if( this.currentItem == item )
                return false;
            this.currentItem.setIconClass("");
        }
        this.currentItem = item;
        this.currentItem.setIconClass("selected");
        this.value = item.value;
        this.setText( item.text );
        return true;
    },

    setActiveItemByValue: function( value, suppressChangeEvent )
    {
        var item = this.menu.items.find( function(item) { return item.value == value; });
        if ( item )
        {
            this.setItem( item );
            if ( !suppressChangeEvent )
                this.fireEvent("change", this, item.origObj);
        }
    },

    /**
     * @deprecated
     */
    getValue: function()
    {
        return this.value;
    },

    getSelectedItem: function()
    {
        return this.currentItem == null ? null : this.currentItem.origObj;
    },

    createItem: function( listItem )
    {
        return {
            text: listItem[ this.textPropName ],
            value: listItem[ this.valuePropName ],
            cls: listItem[ this.clsPropName ],
            origObj: listItem
        };
    },

    addItems: function(listItems){
        for( var i = 0; i < listItems.length; i++ )
        {
            var listItem = listItems[i];

            var itemConfig = this.createItem( listItem );
            itemConfig.listeners = { click: this.itemClickHandler };

            var item = this.menu.addMenuItem(itemConfig);

            if( this.value == item.value )
                this.setItem( item );
        }
    },

    addSeparator: function (){
        this.menu.addSeparator();
    },

    getItemByValue: function(value)
    {
        return this.menu.items.find(
                function(item) { return item.value == value; });
    }
});