MenuButton = function($containerEl, editable)
{
    TemplateView.call(this, $containerEl);
    this.editable = editable;
};

MenuButton.prototype = _.create(TemplateView.prototype, {

    updateOnInit: false,

    positionParams: {offset: "0px 1px"},

    afterRender: function()
    {
        this.disabled = false;
        this.$el.toggleClass('editable', this.editable);
        this.bindClick(function()
                       {
                           if (!this.editable || this.disabled || this.isLoading())
                               return;

                           if (!this.getMenu().ignoreNextShowing)
                               this.$el.addClass("pressed");
                           this.getMenu().showAt(this.$el, this.positionParams, _.bind(this._onSelect, this));
                       });
    },

    _onSelect: function(selectedValue)
    {
        this.$el.removeClass('pressed');
        if (this._change(selectedValue))
        {
            this.update();
            this.trigger("change", selectedValue);
        }
    },

    /**
     * Get button actual menu instance
     * @returns menu {DropdownTemplateView}
     */
    getMenu: function()
    {
    //    abstract
    },

    /**
     * Change button data value
     * @param  selectedValue: new selected value in menu
     * @returns true if data was changed
     *          false otherwise
     */
    _change:function(selectedValue)
    {
    //    abstract
    },

    isDisabled: function()
    {
        return this.disabled;
    },

    setDisabled: function(disabled)
    {
        this.disabled = disabled;
        this.update();
    },

    setEditable: function(editable)
    {
        this.editable = editable;
    }
});

_.extend(MenuButton.prototype, LoadingControlMixin);