/**
 * Created by filatov on 12.08.2015.
 */
function ReportUserSelectorButton()
{}

ReportUserSelectorButton.prototype.init = function ( config )
{
    var self = this;
    this.containerId = config.selectorId;

    this.predefinedUserSelection = {};
    var reportConfigUserSelection = config.userSelection;

    this.settedWithPredefinedValues = !!reportConfigUserSelection;
    this.predefinedUserSelection.selectedUsers = reportConfigUserSelection &&  reportConfigUserSelection.selectedUsers ? reportConfigUserSelection.selectedUsers : [];
    this.predefinedUserSelection.selectedGroups = reportConfigUserSelection && reportConfigUserSelection.selectedGroups ? reportConfigUserSelection.selectedGroups : [];
    this.predefinedUserSelection.allUsers = reportConfigUserSelection ? !!reportConfigUserSelection.allUsers : true;
    this.predefinedUserSelection.showDisabled = reportConfigUserSelection ?  !!reportConfigUserSelection.showDisabled : true;
    this.predefinedUserSelection.includeUsersWithoutData = reportConfigUserSelection ? !!reportConfigUserSelection.includeUsersWithoutData : false;
    this.predefinedUserSelection.groupType = reportConfigUserSelection ? reportConfigUserSelection.groupType : null;

    var params = config.report_name ? {collapsedCookiesKey: config.report_name }:{};
    var panelConstructor = !config.specificConstructorPanel ? AT.ReportsSelectorUsersPanel : config.specificConstructorPanel;
    if ( config.withApplyButton )
        Ext.apply( params, SelectorPanelWithApplyButtonHelper.getApplyButtonExtension( panelConstructor ) );

    this.needAdditionalCheckbox = config.needAdditionalCheckbox;
    this.includeUsersWithoutDataDefaultValue = _.isUndefined(config.includeUsersWithoutDataDefaultValue) ? false : config.includeUsersWithoutDataDefaultValue;

    var selectorPanelHeight = 450;
    if( this.needAdditionalCheckbox ) {
        Ext.apply(params, new GenericAdditionalCheckboxExtension().getExtension(panelConstructor, config.additionalCheckboxLabel));
        selectorPanelHeight += 20;
    }

    this.panel = new panelConstructor(params);
    this.panel.setPredefinedState(this.convertPredefinedStateFromSelection(this.predefinedUserSelection));

    this.userSelectorMenu = new AT.SharedUserButtonMenu( {panel: self.panel} );
    this._createUserButton(this.containerId, this.userSelectorMenu, selectorPanelHeight, config);
    return this;
};

ReportUserSelectorButton.prototype._createUserButton = function(containerId, userSelectorMenu, selectorPanelHeight, config)
{
    var self = this;
    this.chooseUserButton = new AT.HierarchySelectorDropdownButton({
        labelCls: "hierarchySelectorButton",
        renderTo: containerId,
        menu: userSelectorMenu,
        selectorPanelHeight: selectorPanelHeight
    });

    this.wasChanged = false;
    this.chooseUserButton.setMenuHideCallback(function()
                                              {
                                                  if (config.changeCallback && self.wasChanged)
                                                      self.wasChanged = false;
                                                  config.changeCallback();
                                              });
    this.chooseUserButton.setText(messageResource.getMessage("users_selector.reports.loading"));
    this.chooseUserButton.setEnabled(false);
};

ReportUserSelectorButton.prototype.setInitialValueFromCurrent = function(){
    this.panel.setInitialValueFromCurrent();
};

ReportUserSelectorButton.prototype.updateSelector = function(users, userGroups, timeZoneGroups){
    var convertedUserGroups = (userGroups == null) ? null : this.convertUserGroupBeansToJsGroups(userGroups);
    var convertedTimeZoneGroups = (timeZoneGroups == null) ? null : this.convertGroupBeansToJsGroups(timeZoneGroups);
    this.initializeData(userGroups != null, timeZoneGroups != null, this.convertUserInfoToJsUsers(users), convertedUserGroups, convertedTimeZoneGroups);
};

ReportUserSelectorButton.prototype.initializeData = function(userGroupsEnabled, timeZoneGroupsEnabled, users, userGroups, timeZoneGroups){
    this.panelWasConstructing = true;
    this.panel.initialize(userGroupsEnabled, timeZoneGroupsEnabled, users, userGroups, timeZoneGroups);
    this.chooseUserButton.setEnabled(true);
};

ReportUserSelectorButton.prototype.getSelection = function(){
    if(this.panelWasConstructing)
    {
            var result = {};
            result.allUsers = !this.panel.isCustomOptionSelected();
            var selection = this.panel.getSelection();
            result.selectedGroups = selection.getFirstLevel();
            result.selectedUsers = selection.getSecondLevel();
            result.showDisabled = this.panel.getShowInactive();
            result.includeUsersWithoutData = this.needAdditionalCheckbox ? this.panel.getAdditionalCheckboxValue() : 
                                             (result.allUsers ? false : this.includeUsersWithoutDataDefaultValue);
            result.groupType = this.panel.getGroupType();
            return result;
    }
    return this.predefinedUserSelection;
};

ReportUserSelectorButton.prototype.setSelection = function(userSelection){
    var predefinedState = this.convertPredefinedStateFromSelection( userSelection );
    this.panel.setPredefinedState( predefinedState );
    if(this.panelWasConstructing)
    {
        this.panel.loadState( predefinedState );
    }
};

ReportUserSelectorButton.prototype.convertUserInfoToJsUsers = function ( userbeans )
{
    var users = [];
    for( var i = 0; i < userbeans.length; ++i )
    {
        var u = userbeans[i];
        users.push( new User( u.id, u.firstName, u.lastName, u.middleName, u.userName, u.enabled, undefined, u.userGroupId, u.timeZoneGroupId ) );
    }
    return users;
};

ReportUserSelectorButton.prototype.convertUserGroupBeansToJsGroups = function (groupbeans)
{
    var groups = this.convertGroupBeansToJsGroups(groupbeans);
    groups.unshift(new Group(SystemSettings.usersWithoutGroupId, messageResource.getMessage('common.users_without_group')));
    return groups;
};

ReportUserSelectorButton.prototype.convertGroupBeansToJsGroups = function (groupbeans)
{
    var groups = [];
    for (var i = 0; i < groupbeans.length; ++i)
    {
        var g = groupbeans[i];
        groups.push(new Group(g.id, g.name));
    }
    return groups;
};

//need invoke when drop state on initial
ReportUserSelectorButton.prototype.resetValue = function(){
    if(this.panel.isAlreadyShown())
    {
        this.panel.resetState( true );
    }
};

ReportUserSelectorButton.prototype.validateIsUserSelected = function(){
    return  ( this.settedWithPredefinedValues || this.panel.isAlreadyShown() ) ? this.panel.validateIsUsersSelected() :
            this.predefinedUserSelection.allUsers ||
            this.predefinedUserSelection.selectedUsers != [] ||
            this.predefinedUserSelection.selectedGroups != []
};

ReportUserSelectorButton.prototype.setWasChanged = function(){
    this.wasChanged = true;
};

ReportUserSelectorButton.prototype.convertPredefinedStateFromSelection = function(selection){
    return {
        selection: selection.allUsers ? TwoLevelSelection.emptySelection() : new TwoLevelSelection( selection.selectedGroups, selection.selectedUsers ),
        optionValue: selection.allUsers,
        showInactive: selection.showDisabled,
        additionalCheckBoxValue: selection.includeUsersWithoutData,
        groupType: selection.groupType
    }
};

ReportUserSelectorButton.prototype.isStateChanged = function() {
    return this.panel.isStateChanged();
};