
function ApprovalStatusSelector(){}

ApprovalStatusSelector.prototype.init = function ( panelElId, buttonPlaceholderId, statuses, onChangeCallback ){
    var self = this;
    this.onChangeCallback = onChangeCallback;

    // create and render selector button
    var buttonPlaceholder = $(buttonPlaceholderId);
    var selectorButton = document.createElement("div");
    selectorButton.className = "statusApprovalSelectorButton";

    var selectorButtonTitle = document.createElement("span");
    selectorButtonTitle.className = "statusApprovalSelectorButtonTitle";
    selectorButton.appendChild(selectorButtonTitle);
    buttonPlaceholder.append(selectorButton);
    this.selectorButton = $(selectorButton);
    this.selectorButtonTitle = $(selectorButtonTitle);
    this.selectorButton.click( function () { self.showPanel(); } );

    this.panelEl = $(panelElId);
    this.panelEl.width(210);
    // create panel as dropdown dialog
    this.panel = new DropdownDialog({
                                        wrapperClass: "approvalStatusSelectorWrapper",
                                        dialogDiv: this.panelEl,
                                        hideButtons: false,
                                        hideOnResize: true,
                                        showEmptyTitleBar : false,
                                        okButtonText: messageResource.getMessage("approve_tt.selector_status.apply"),
                                        cancelButtonText: messageResource.getMessage("approve_tt.selector_status.cancel"),
                                        onOkClick: function () {
                                            return self.onChange();
                                        },
                                        onClose: function () {
                                            self.selectorButton.removeClass("panelOpened");
                                        }
                                });
    this.checkboxMap ={};

    this.addCheckBox("readyForApprovalCbx", "approve_tt.selector_status.ready");
    this.addCheckBox("notReadyForApprovalCbx", "approve_tt.selector_status.not_ready_title");
    this.addCheckBox("rejectedCbx", "approve_tt.selector_status.rejected");

    this.lastState = statuses;
    this.initCheckboxes(statuses);
    this.updateText(statuses);
    return this;
};

ApprovalStatusSelector.prototype.addCheckBox = function(checkBox, titleTextKey){
    var checkBoxEl =  document.getElementById(checkBox);
    if ( checkBoxEl){
        this.checkboxMap[checkBox] = {};
        this.checkboxMap[checkBox].checkBoxEl = checkBoxEl;
        this.checkboxMap[checkBox].titleTextKey = titleTextKey;
    }
};

ApprovalStatusSelector.prototype.showPanel = function () {
    this.initCheckboxes(this.lastState);
    this.selectorButton.addClass("panelOpened");
    this.panel.showAt(this.selectorButton, {    my: "left top",
                          at: "left bottom",
                          collision: "flip" }
    );
};

ApprovalStatusSelector.prototype.initCheckboxes = function(data, isUpdateText){
    var self = this;
    $.each(this.checkboxMap, function(key){
        self.setChecked(key, $.inArray(self.getValue(key), data) >= 0);
    });
    if (isUpdateText){
        this.updateText();
    }
    this.lastState = this.getSelection();
};

ApprovalStatusSelector.prototype.onChange = function () {
    if (this.isAllUnselected()){
        this.selectAll();
        this.onChange();
        return;
    }
    this.lastState = this.getSelection();
    this.updateText();
    this.panel.hide();
    if (this.onChangeCallback){
        this.onChangeCallback();
    }
};

ApprovalStatusSelector.prototype.isAllUnselected = function(){
    var self = this;
    var flag = true;
    $.each(this.checkboxMap, function(key){
        if (self.isChecked(key))flag = false;
    });
    return flag;
};

ApprovalStatusSelector.prototype.isAllSelected = function(){
    var self = this;
    var flag = true;
    $.each(this.checkboxMap, function(key){
        if (!self.isChecked(key))flag = false;
    });
    return flag;
};

ApprovalStatusSelector.prototype.selectAll = function(){
    var self = this;
    $.each(this.checkboxMap, function(key){
        self.setChecked(key, true);
    });
};


ApprovalStatusSelector.prototype.updateText = function () {
    var haveLast = false;
    var text ="";
    var self = this;

    $.each(this.checkboxMap, function(key, value){
        if (self.isChecked(key)){
            text += haveLast ? ", " : "";
            text += messageResource.getMessage( value.titleTextKey );
            haveLast = true;
        }
    });
    this.selectorButtonTitle.text(text);
};

ApprovalStatusSelector.prototype.isChecked =function(checkbox){
   return this.getCheckBoxEl(checkbox).checked;
};

ApprovalStatusSelector.prototype.setChecked =function(checkbox, value){
    document.getElementById(checkbox).checked = value;
};

ApprovalStatusSelector.prototype.getValue =function(checkbox){
    return this.getCheckBoxEl(checkbox).value;
};

ApprovalStatusSelector.prototype.getCheckBoxEl =function(checkbox){
    return this.checkboxMap[checkbox].checkBoxEl;
};

ApprovalStatusSelector.prototype.getSelection = function(){
    var result = [];
    var self = this;
    $.each(this.checkboxMap, function(key){
        if (self.isChecked(key)){
            result.push( self.getValue(key));
        }
    });
    return result;
};
