Ext.namespace("AT");

/**
 * Container that renders a rounded border around its contents.
 * @cfg {String} cls style class for the container (required).
 * @cfg {String} straightBorder pass "left", "right", "top" or "bottom" to render the corresponding border as straight,
 *     renders all borders rounded by default.
 * @see RoundedBoxTag.java
 */
AT.RoundedBox = Ext.extend(Ext.Container, {
    straightBorder: null,

    initComponent: function() {
        AT.RoundedBox.superclass.initComponent.call(this);

        if (this.straightBorder == "right") {
            this.template = this.templateRoundLeft;
        } else if (this.straightBorder == "left") {
            this.template = this.templateRoundRight;
        } else {
            this.template = this.templateRoundBoth;
        }
    },

    onRender: function(ct, pos) {
        var arguments = [ this.cls ];
        if (pos) {
            this.el = this.template.insertBefore(pos, arguments, true);
        } else {
            this.el = this.template.append(ct, arguments, true);
        }

        if (!this.el.id) {
            this.el.id = this.getId();
        }
    },

    getLayoutTarget: function() {
        return this.el.child(".i");
    },

    templateRoundBoth: new Ext.Template(
            '<div class="roundedBoxTag {0}" ><div class="i"></div></div>'),
    templateRoundLeft: new Ext.Template(
            '<div class="roundedBoxTag  straightBorder-right {0}" ><div class="i"></div></div>'),
    templateRoundRight: new Ext.Template(
            '<div class="roundedBoxTag  straightBorder-left {0}" ><div class="i"></div></div>')
});

Ext.reg("at_rounded_box", AT.RoundedBox);