/**
 * @author kostenko
 *
 * @deprecated DO NOT USE
 *
 * Extensions for DataTables component for making it editable.
 *
 * Note that when calling these functions, the first argument (oSettings) must be omitted,
 * because it is passed automatically.
 * Although IDEA will curse you for it, this is the way the extensions mechanism is implemented in DataTables.
 */

/**
 * @param oSettings (passed automatically)
 * @param cell TD element
 * @return array data of the row containing the given cell cell
 */
$.fn.dataTableExt.oApi.fnGetRowDataForCell = function (oSettings, cell)
{
    var aPos = this.fnGetPosition( cell );
    return this.fnGetData(aPos[0]);
};

/**
 * Updates the data for the given cell and re-renders the corresponding row.
 *
 * Doesn't cause the table to re-sort or re-filter.
 *
 * The backup of the data is stored in this.cellDataBackup field.
 *
 * If newData is not given (is undefined), the row data is not changed and no backup is stored,
 * but the row is re-rendered.
 *
 * @param oSettings (passed automatically)
 * @param cell TD element
 * @param newData cell data (optional)
 */
$.fn.dataTableExt.oApi.fnUpdateRowForCell = function (oSettings, cell, newData)
{
    var aPos = this.fnGetPosition( cell );
    var rowData = this.fnGetRowDataForCell(cell);

    if (newData !== undefined)
    {
        this.cellDataBackup = rowData[aPos[2]];
        rowData[aPos[2]] = newData;
    }

    this.fnUpdateRow(aPos[0], rowData);
};

/**
 * Updates the data in the cell with the given column index within the active row.
 *
 * Doesn't cause the table to re-sort or re-filter.
 *
 * @param oSettings (passed automatically)
 * @param columnIndex column index
 * @param newData new cell data
 */
$.fn.dataTableExt.oApi.fnUpdateActiveRowForColumn = function (oSettings, columnIndex, newData)
{
    var row = this.fnGetActiveRow();
    var rowData = this.fnGetData(row);
    rowData[columnIndex] = newData;

    this.fnUpdateRow(row, rowData);
};

/**
 * Updates the data in the given row.
 * Doesn't cause the table to re-sort or re-filter.
 *
 * @param oSettings (passed automatically)
 * @param row TR element or row index
 * @param newRowData new row data
 */
$.fn.dataTableExt.oApi.fnUpdateRow = function (oSettings, row, newRowData)
{
    // There's a bug in DataTables documentation: we need to pass undefined as the 3rd parameter to update the whole row.
    this.fnUpdate(newRowData, row, undefined, false, false); // update without re-draw
    this.fnDraw(false); // re-draw without re-sort and re-filter
};

/**
 * @param oSettings (passed automatically)
 * @param cell TD element, jQuery object or selector
 * @param clickHandler function
 */
$.fn.dataTableExt.oApi.fnMakeCellEditable = function (oSettings, cell, clickHandler)
{
    cell = $(cell);

    cell.addClass('editable');

    var self = this;

    cell.unbind('click'); // only one handler is supported, so let's unbind the previous one

    cell.click( function() {
        if (self.fnIsBusy() || self.fnGetActiveCell())
            return;

        cell.addClass('active');
        cell.parent().addClass('active');
        clickHandler.call(cell[0]);
    } );
};

/**
 * @param oSettings (passed automatically)
 * @param cell TD element, jQuery object or selector
 */
$.fn.dataTableExt.oApi.fnMakeCellNotEditable = function (oSettings, cell)
{
    $(cell).removeClass('editable');
    $(cell).unbind('click');
};

/**
 * @param oSettings (passed automatically)
 * @return object TD element
 */
$.fn.dataTableExt.oApi.fnGetActiveCell = function (oSettings)
{
    return this.fnGetActiveCellJQ()[0];
};

/**
 * @param oSettings (passed automatically)
 * @return object TD element wrapped in jQuery
 */
$.fn.dataTableExt.oApi.fnGetActiveCellJQ = function (oSettings)
{
    return $('td.active', this);
};

/**
 * @param oSettings (passed automatically)
 * @return object TR element
 */
$.fn.dataTableExt.oApi.fnGetActiveRow = function (oSettings)
{
    return $('td.active', this).parent()[0];
};

/**
 * @param oSettings (passed automatically)
 * @param newValue object new value for the active cell
 */
$.fn.dataTableExt.oApi.fnStartActiveCellSaving = function (oSettings, newValue)
{
    var cell = this.fnGetActiveCellJQ();
    cell.parent().removeClass('active');
    this.addClass('busy');
    this.fnUpdateRowForCell( cell[0], newValue );
};

/**
 * @param oSettings (passed automatically)
 */
$.fn.dataTableExt.oApi.fnIsBusy = function (oSettings)
{
    return this.hasClass('busy');
};

/**
 * @param oSettings (passed automatically)
 * @param withDelay boolean
 */
$.fn.dataTableExt.oApi.fnFinishCellEditing = function (oSettings, withDelay)
{
    this.cellDataBackup = undefined;

    var cell = this.fnGetActiveCellJQ();
    var self = this;

    var remove = function() {
        cell.removeClass('active');
        cell.parent().removeClass('active');
        self.removeClass('busy');
    };

    if (withDelay)
        setTimeout( remove, 350);
    else
        remove();
};

/**
 * @param oSettings (passed automatically)
 */
$.fn.dataTableExt.oApi.fnCompleteCellEditing = function (oSettings)
{
    this.fnFinishCellEditing(true);
};

/**
 * @param oSettings (passed automatically)
 */
$.fn.dataTableExt.oApi.fnCancelCellEditing = function (oSettings)
{
    if (this.cellDataBackup !== undefined)
        this.fnUpdateRowForCell( this.fnGetActiveCell(), this.cellDataBackup );

    this.fnFinishCellEditing(false);
};

/**
 * @param oSettings (passed automatically)
 */
$.fn.dataTableExt.oApi.fnCancelCellEditingIfNotBusy = function (oSettings)
{
    if (!this.fnIsBusy())
        this.fnCancelCellEditing();
};