function DateSelectorController() {
    this.constructor = DateSelectorController;
}

DateSelectorController.prototype.dateFormat = Date.convertSmartToExtDateFormat(SystemSettings.dateFormat);
//TODO: evdokimov java constants!
DateSelectorController.prototype.rangeMinDays = 1;
DateSelectorController.prototype.rangeMaxDays = 366 * 5;

DateSelectorController.prototype.rangeTooSmallMessageKey = "date_range_selector.range_error.shorter_than_7_days";
DateSelectorController.prototype.rangeTooLongMessageKey = "date_range_selector.range_error.longer_than_5_years";
DateSelectorController.prototype.rangeReverseMessageKey = "date_range_selector.range_error.reverse";
DateSelectorController.prototype.dateTooOldMessageKey = "date_range_selector.range_error.too_old";

DateSelectorController.prototype.init = function ( config ) {
    this.menuStates = config.menuStates ? config.menuStates : this._getDefaultMenuItems();
    
    if (!_.isUndefined(config.rangeMaxDays))
        this.rangeMaxDays = config.rangeMaxDays;
    
    if (!_.isUndefined(config.rangeTooLongMessageKey))
        this.rangeTooLongMessageKey = config.rangeTooLongMessageKey;
    
    this.dateRangeInitState = config.dateRangeParsed ? this.getParsedDateRange(config.dateRangeSelection) : this.parseJSON(config.dateRangeSelection);

    var renderTo = config.renderTo ? config.renderTo : "dateSelector";
    this.dateSelector = $("#" + renderTo);

    var buttonTextFormat = config.shortDateFormat ? this.dateFormat.replace(/F/, 'M') : this.dateFormat;

    // Determine if current range type can be found in menu states. If not, select the first suitable
    var rangeType = this.dateRangeInitState.qualifier;
    var rangeTypeIsLegal = false;
    var defaultRangeType = null;
    _.each(this.menuStates, function(menuState)
    {
        if (menuState.value)
        {
            if (!defaultRangeType)
                defaultRangeType = menuState.value;
            if (rangeType === menuState.value)
                rangeTypeIsLegal = true;
        }
    });
    if (!rangeTypeIsLegal)
        rangeType = defaultRangeType;

    var selectorConfig = {
        fromDate: this.dateRangeInitState.fromDate,
        toDate: this.dateRangeInitState.toDate,
        rangeTypeValue: rangeType,
        buttonTextFormat: buttonTextFormat,
        menuStates: this.menuStates,
        renderTo: renderTo,
        cls: "customDateRangeSelector",
        scheduleCalculator: this._createScheduleCalculator(config),
        layout: config.layout ? config.layout : "table"
    };

    this.dateRangeSelector = this._createDateRangeSelector(selectorConfig);

    var self = this;
    this.dateRangeSelector.on("change", function() {
        var isRangeValid = self._verifyRangeSize();
        return config.callOnChangeCallbackIfInvalid ? true : isRangeValid;
    } , this);

    Ext.fly(renderTo).dom.style.visibility = "visible";
    
    if ( config.rangeMinDays ) 
        this.rangeMinDays = config.rangeMinDays;

    if ( config.rangeMaxDays )
        this.rangeMaxDays = config.rangeMaxDays;

    this.errorTooltip = new TableTooltip().init("incorrectDateTooltip");
    this.errorTooltipTextPlaceholder = $("#incorrectDateTooltip_textPlaceholder");
    this._initErrorTooltip();
    this.setOnResizeHandler();
    
    return this;
};

DateSelectorController.prototype.setOnResizeHandler = function () {
    var self = this;
    $( window ).resize(function () { self.dateRangeSelector.hideMenu(); });
};

DateSelectorController.prototype.setInitStateFromCurrent = function () {
    // abstract
};

DateSelectorController.prototype._createDateRangeSelector = function () {
    // abstract
};

DateSelectorController.prototype._createScheduleCalculator = function ( config ) {
    return !config.scheduleCalculator ? 
                new CorporateScheduleCalculator().init(invertedDaysDateMap, SystemSettings.defaultWorkSchedule) :
                config.scheduleCalculator;
};

DateSelectorController.prototype.setOnChangeCallback = function ( onChangeCallBack ) {
    this.dateRangeSelector.on("change", onChangeCallBack, this);
};

DateSelectorController.prototype._initErrorTooltip = function () {
    // abstract
};

DateSelectorController.prototype._setErrorTooltipHandler = function ( dateCalendar ) {
    var self = this;

    dateCalendar.hover(function (event) {self._showTooltip(event);}, function () {self._hideTooltip();});
    dateCalendar.mousemove(function (event) {self._moveTooltip(event);});
};

DateSelectorController.prototype.getRangeTypeId = function () {
    return this.dateRangeSelector.getRangeTypeId();
};

DateSelectorController.prototype.getSelectedRange = function () {
    // abstract
};

DateSelectorController.prototype.getSelectedRangeJSON = function () {
    // abstract
};

DateSelectorController.prototype.setSelectedRangeJSON = function () {
    // abstract
};

DateSelectorController.prototype.setSelectedRange = function(parsedConfig) {
    // abstract
};

DateSelectorController.prototype.parseJSON = function ( config ) {
    // abstract
};

DateSelectorController.prototype.getParsedDateRange = function ( config ) {
    // abstract
};

DateSelectorController.prototype.hasErrors = function () {
    return !!this.errorMessageKey;
};

DateSelectorController.prototype.getErrorMessageKey = function () {
    return this.errorMessageKey;
};

DateSelectorController.prototype._calendarOpened = function () {
    // abstract
};

DateSelectorController.prototype._showTooltip = function( event ) {
    if ( !this.errorMessageKey || this._calendarOpened() ) 
        return;
    
    this.errorTooltip.show();
    this.errorTooltip.move( calculateTooltipOffset( this.errorTooltip, event ) );
};

DateSelectorController.prototype._moveTooltip = function( event ) {
    if ( !this.errorMessageKey || this._calendarOpened()) {
        this._hideTooltip();
        return;
    }
    
    this.errorTooltip.move( calculateTooltipOffset( this.errorTooltip, event ) );
};

DateSelectorController.prototype._hideTooltip = function() {
    this.errorTooltip.hide();
};

DateSelectorController.prototype._errorState = function( messageKey ) {
    this.dateSelector.addClass("incorrectDateRange");

    this.errorMessageKey = messageKey;
    this.errorTooltipTextPlaceholder.html(messageResource.getMessage(messageKey));
};

DateSelectorController.prototype._flushErrorState = function() {
    this.dateSelector.removeClass("incorrectDateRange");
    this.errorMessageKey = null;
};

// --------------------------------

function DateRangeSelectorController() {}

DateRangeSelectorController.prototype = new DateSelectorController();

DateRangeSelectorController.prototype.init = function ( config ) {
    DateSelectorController.prototype.init.call(this, config);
    return this;
};

DateRangeSelectorController.prototype._createDateRangeSelector = function ( selectorConfig ) {
    return new AT.CustomDateRangeSelector(selectorConfig);
};

DateRangeSelectorController.prototype._initErrorTooltip = function () {
    this.fromDateButton = this.dateSelector.find(".calendarButton.fromDate");
    var fromDateCalendar = this.fromDateButton.find(".at-date-button");
    this._setErrorTooltipHandler(fromDateCalendar);

    this.toDateButton = this.dateSelector.find(".calendarButton.toDate");
    var toDateCalendar = this.toDateButton.find(".at-date-button");
    this._setErrorTooltipHandler(toDateCalendar);
};

DateRangeSelectorController.prototype._calendarOpened = function () {
    return this.fromDateButton.hasClass("x-btn-menu-active") || this.toDateButton.hasClass("x-btn-menu-active");
};

DateRangeSelectorController.prototype._verifyRangeSize = function() {
    var fromDate = this.getFromDate().clearTime();
    var toDate = this.getToDate().clearTime();

    if ( fromDate > toDate ) {
        this._errorState(this.rangeReverseMessageKey);
        return false;
    }

    if ( fromDate && fromDate.getFullYear() < 1970 || toDate && toDate.getFullYear() < 1970 ) {
        this._errorState(this.dateTooOldMessageKey);
        return false;
    }

    if ( this.rangeMinDays && fromDate.add(Date.DAY, this.rangeMinDays - 1).clearTime() > toDate ) {
        this._errorState(this.rangeTooSmallMessageKey);
        return false;
    }

    if ( this.rangeMaxDays && fromDate.add(Date.DAY, this.rangeMaxDays - 1).clearTime() < toDate ) {
        this._errorState(this.rangeTooLongMessageKey);
        return false;
    }

    this._flushErrorState();
    return true;
};

DateRangeSelectorController.prototype.getFromDate = function () {
    return this.dateRangeSelector.getFromDate();
};

DateRangeSelectorController.prototype.getToDate = function () {
    return this.dateRangeSelector.getToDate();
};

DateRangeSelectorController.prototype.setInitStateFromCurrent = function () {
    this.dateRangeInitState.qualifier = this.dateRangeSelector.getRangeTypeValue();
    this.dateRangeInitState.fromDate = this.getFromDate();
    this.dateRangeInitState.toDate = this.getToDate();
};

DateRangeSelectorController.prototype.getSelectedRange = function () {
    return {
        qualifier: this.dateRangeSelector.getRangeTypeValue(),
        fromDate: this.getFromDate(),
        toDate: this.getToDate()
    };
};

DateRangeSelectorController.prototype.getSelectedRangeJSON = function () {
    return {
        qualifier: this.dateRangeSelector.getRangeTypeValue(),
        fromDate: this.getFromDate().toJSON(),
        toDate: this.getToDate().toJSON()
    };
};

DateRangeSelectorController.prototype.setSelectedRangeJSON = function ( config ) {
    this.setSelectedRange(this.parseJSON(config.dateRangeSelection));
};

DateRangeSelectorController.prototype.setSelectedRange = function(parsedConfig) {
    this.dateRangeSelector.setValues(parsedConfig.qualifier, parsedConfig.fromDate, parsedConfig.toDate);
};

DateRangeSelectorController.prototype.parseJSON = function ( config ) {
    return {
        qualifier: config.qualifier,
        fromDate: !config.fromDate ? null : Date.parseJSONDate(config.fromDate),
        toDate: !config.toDate ? null : Date.parseJSONDate(config.toDate)
    }
};

DateRangeSelectorController.prototype.getParsedDateRange = function ( config ) {
    return config;
};

DateRangeSelectorController.prototype.isDateRangeChanged = function() {
    return !areDatesEqual(this.dateRangeInitState.toDate, this.getToDate()) ||
           !areDatesEqual(this.dateRangeInitState.fromDate, this.getFromDate());
};

// --------------------------------

function SingleDateSelectorController() {}

SingleDateSelectorController.prototype = new DateSelectorController();

SingleDateSelectorController.prototype.init = function ( config ) {
    DateSelectorController.prototype.init.call(this, config);
    return this;
};

SingleDateSelectorController.prototype._createDateRangeSelector = function ( selectorConfig ) {
    return new AT.SingleDateSelector(selectorConfig);
};

SingleDateSelectorController.prototype._verifyRangeSize = function() {
    return true;
};

SingleDateSelectorController.prototype._initErrorTooltip = function () {
    this.toDateButton = this.dateSelector.find(".calendarButton.toDate");
    var toDateCalendar = this.toDateButton.find(".at-date-button");
    this._setErrorTooltipHandler(toDateCalendar);
};

SingleDateSelectorController.prototype._calendarOpened = function () {
    return this.toDateButton.hasClass("x-btn-menu-active");
};


SingleDateSelectorController.prototype.getDate = function () {
    return this.dateRangeSelector.getToDate();
};

SingleDateSelectorController.prototype.setInitStateFromCurrent = function () {
    this.dateRangeInitState.qualifier = this.dateRangeSelector.getRangeTypeValue();
    this.dateRangeInitState.date = this.getDate();
};

SingleDateSelectorController.prototype.getSelectedRange = function () {
    return {
        qualifier: this.dateRangeSelector.getRangeTypeValue(),
        date: this.getDate()
    };
};

//NOTE: at_selectors get "toDate" as date param, but returns "date" as config
SingleDateSelectorController.prototype.getSelectedRangeJSON = function () {
    return {
        qualifier: this.dateRangeSelector.getRangeTypeValue(),
        date: this.getDate().toJSON()
    };
};

SingleDateSelectorController.prototype.setSelectedRangeJSON = function ( config ) {
    var parsedConfig = this.parseJSON(config.dateSelection);
    this.dateRangeSelector.setValues(parsedConfig.qualifier, parsedConfig.toDate);
};

SingleDateSelectorController.prototype.setSelectedRange = function(parsedConfig) {
    this.dateRangeSelector.setValues(parsedConfig.qualifier, parsedConfig.toDate);
};

//NOTE: at_selectors get "toDate" as date param, but returns "date" as config
SingleDateSelectorController.prototype.parseJSON = function ( config ) {
    return {
        qualifier: config.qualifier,
        toDate: !config.date ? null : Date.parseJSONDate(config.date)
    }
};

SingleDateSelectorController.prototype.getParsedDateRange = function ( config ) {
    return {
        qualifier: config.qualifier,
        toDate: config.date
    }
};

SingleDateSelectorController.prototype.isDateRangeChanged = function() {
    return !areDatesEqual(this.dateRangeInitState.toDate, this.getDate());
};