function ApprovalMonthSelector() {}

ApprovalMonthSelector.prototype.init = function ( config )
{
    var self = this;

    this._spinnerController = config.spinnerController;
    this._dialogDiv = $( "#approvalMonthSelector_loadingTreeContainer" );
    this.selectorCell = $( config.selectorCellId );
    this.selectorButton = this.selectorCell.find("#approvalMonthSelectorButton");
    this.resetButton = this.selectorCell.find(".monthSelectorResetButton");
    this.statusTextSpan = this.selectorButton.find( ".monthSelectorStatusText" );
    this.panelEl = $( config.panelElId );
    this.onChangeCallback = config.onChangeCallback;

    this.selectedDate = config.selectedDate ? new Date( config.selectedDate ) : null;
    this.updateStatus( this.selectedDate );


    this.selectorButton.click( function ()
                               {
                                   self.showPanel();
                                   config.onShowCallBack( $.proxy( self.renderCalendars, self ) );
                               } );

    this.resetButton.click(function(){ self.resetSelectedDate( true ) });

    //create panel as dropdown dialog
    this.panel = new DropdownDialog( {
                                         wrapperClass: "approvalMonthSelectorPanelWrapper",
                                         dialogDiv: this.panelEl,
                                         hideButtons: true,
                                         hideOnResize: true,
                                         showEmptyTitleBar: false,
                                         onClose: function ()
                                         {
                                             self.selectorButton.removeClass( "panelOpened" );
                                         }
                                     } );
    return this;
};

ApprovalMonthSelector.prototype.updateStatus = function( selectedDate )
{
    var self = this;
    if (selectedDate)
    {
        self.selectedDate = selectedDate;
        self.selectorButton.addClass("active");
        self.statusTextSpan.html(Date.monthNamesShort[self.selectedDate.getMonth()] + ", " + self.selectedDate.getFullYear());
        self.resetButton.show();
    }
    else
        this.resetSelectedDate( false );
};

ApprovalMonthSelector.prototype.renderCalendars = function ( startYear, yearsCount, monthsData, currentMonth )
{
    var container = this.panelEl.find( "table tbody" );
    var self = this;

    var onMonthDivClickCallback = function ( el )
    {
        self.updateStatus( el.data( "selectedDate" ) )
    };

    for( var i = startYear; i < startYear + yearsCount; i++ )
    {
        var yearHeader = document.createElement( "tr" );
        var yearHeaderTd = document.createElement( "td" );
        yearHeaderTd.innerHTML += i;
        yearHeader.appendChild( yearHeaderTd );
        container.append( $( yearHeader ) );
        $( yearHeader ).addClass( "yearHeader" );
        $( yearHeaderTd ).attr( "colspan", "3" );


        for( var row = 0; row < 4; row++ )
        {
            var monthRow = document.createElement( "tr" );
            monthRow.className = "monthsRow";
            if( row == 0 )
                monthRow.className += " firstRow";

            if( row == 3 )
                monthRow.className += " lastRow";

            for( var m = 0; m < 3; m++ )
            {
                var monthNumber = row * 3 + m;

                var td = document.createElement( "td" );
                if( m == 0 )
                    td.className = "firstCell";
                if( m == 2 )
                    td.className = "lastCell";

                var monthDiv = document.createElement( "div" );
                monthDiv.className = "monthDiv";
                monthDiv.id = "month" + monthNumber + "-" + i;

                var monthValueSpan = document.createElement( "span" );
                monthValueSpan.className = "monthValueSpan";
                monthValueSpan.innerHTML = Date.monthNamesShort[monthNumber];

                monthDiv.appendChild( monthValueSpan );

                td.appendChild( monthDiv );

                if( self.selectedDate && i == self.selectedDate.getFullYear() && monthNumber == self.selectedDate.getMonth() )
                {
                    $( monthDiv ).addClass( "selected" );
                }

                if( i == startYear + yearsCount - 1 && currentMonth < monthNumber )
                {
                    $( monthDiv ).addClass( "futureMonth" );
                }
                else
                {
                    if( monthsData[i - startYear][monthNumber] )
                    {
                        $( monthDiv ).data( "selectedDate", Date.create( i, monthNumber, 1 ) );
                        $( monthValueSpan ).data( "selectedDate", Date.create( i, monthNumber, 1 ) );
                        $( monthDiv ).click( function ( el )
                                             {
                                                 onMonthDivClickCallback( $( el.target ) );
                                                 self.onChangeCallback();
                                                 self.hidePanel();
                                             } );
                        $( monthDiv ).addClass( "notApproved" );
                    }
                }
                monthRow.appendChild( td );
            }
            container.append( monthRow );
        }
    }
    this.switchDialogInLoadedState();
    this.panelEl.scrollTop( this.panelEl.get( 0 ).scrollHeight );
    var oneCalendarHeight = ((this.panelEl.scrollTop() + this.panelEl.height()) / yearsCount) - 1;
    var selectedYearNumber = this.selectedDate != null ? this.selectedDate.getFullYear() - startYear : yearsCount - 1;
    this.panelEl.scrollTop( oneCalendarHeight * selectedYearNumber );
};

ApprovalMonthSelector.prototype.getSelectedDate = function ()
{
    return this.selectedDate;
};

ApprovalMonthSelector.prototype.isNotSelectedDate = function ()
{
    return this.selectedDate == null;
};

ApprovalMonthSelector.prototype.resetSelectedDate = function ( needToReloadData )
{
    this.selectedDate = null;
    this.selectorButton.removeClass("active");
    this.statusTextSpan.html("Select month");
    this.resetButton.hide();
    if (needToReloadData)
        this.onChangeCallback();
};

ApprovalMonthSelector.prototype.switchDialogInLoadingState = function ()
{
    this._dialogDiv.removeClass( 'loadedState' ).addClass( 'loadingState' );
    this._spinnerController.spin( "approvalMonthSelector_spinnerPlaceholder" );
};

ApprovalMonthSelector.prototype.switchDialogInLoadedState = function ()
{
    this._dialogDiv.removeClass( 'loadingState' ).addClass( 'loadedState' );
    this._spinnerController.stop();
};

ApprovalMonthSelector.prototype.showPanel = function ()
{

    var container = this.panelEl.find( "table tbody" );
    container.empty();

    this.selectorButton.addClass( "panelOpened" );
    this.panel.showAt( this.statusTextSpan.parent(), {
                           my: "left top",
                           at: "left bottom",
                           collision: "flip"
                       }
    );
    this.switchDialogInLoadingState();
};
ApprovalMonthSelector.prototype.hidePanel = function ()
{
    this.panel.hide();
};
