Ext.namespace("AT");

/**
 * Corresponds to Java interface <code>com.actimind.actitime.util.daterange.DateRangeState</code>
 */
AT.DateRangeState = Ext.extend(Object,
{
    constructor   : function(config){Ext.apply(this, config);},
    isHasFromDate : function(){return true;},
    isHasToDate   : function(){return true;},
    getFromDate   : Ext.emptyFn,
    getToDate     : Ext.emptyFn,
    equals        : function(fromDate, toDate)
    {
        return this.isDatesEquals(this.isHasFromDate(), this.getFromDate(), fromDate) &&
               this.isDatesEquals(this.isHasToDate(), this.getToDate(), toDate);
    },

    //private
    isDatesEquals : function(compareAsDates, date1, date2)
    {
        if (compareAsDates && Ext.isDate(date1) && Ext.isDate(date2))
        {
            return date1.clearTime().getTime() == date2.clearTime().getTime();
        }
        else
        {
            return date1 == date2;
        }
    }
});

/**
 * Corresponds to Java class <code>com.actimind.actitime.util.daterange.CurrentMonthDateRangeState</code>
 */
AT.CurrentMonthDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
        return currentUserTime().getFirstDateOfMonth().clearTime();
    },
    getToDate     : function() {
        return currentUserTime().getLastDateOfMonth().clearTime();
    }
});

AT.CurrentMonthTillTodayDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
        return currentUserTime().getFirstDateOfMonth().clearTime();
    },
    getToDate     : function() {
        return currentUserTime().clearTime();
    }
});

/**
 * Corresponds to Java class <code>com.actimind.actitime.util.daterange.PreviousMonthDateRangeState</code>
 */
AT.PreviousMonthDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
        return currentUserTime().add(Date.MONTH, -1).getFirstDateOfMonth().clearTime();
    },
    getToDate     : function() {
        return currentUserTime().add(Date.MONTH, -1).getLastDateOfMonth().clearTime();
    }
});

AT.NextMonthDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
        return currentUserTime().add(Date.MONTH, 1).getFirstDateOfMonth().clearTime();
    },
    getToDate     : function() {
        return currentUserTime().add(Date.MONTH, 1).getLastDateOfMonth().clearTime();
    }
});

/**
 * Corresponds to Java class <code>com.actimind.actitime.util.daterange.CurrentWeekDateRangeState</code>
 */
AT.CurrentWeekDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate         : function() {
        return currentUserTime().getWeekStartDate().clearTime();
    },
    getToDate           : function() {
        return currentUserTime().getWeekStartDate().add(Date.DAY, 6).clearTime();
    }
});

AT.CurrentWeekTillTodayDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate         : function() {
        return currentUserTime().getWeekStartDate().clearTime();
    },
    getToDate           : function() {
        return currentUserTime().clearTime();
    }
});

AT.NextWeekDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate         : function() {
        return currentUserTime().getWeekStartDate().add(Date.DAY, 7).clearTime();
    },
    getToDate           : function() {
        return currentUserTime().getWeekStartDate().add(Date.DAY, 13).clearTime();
    }
});

/**
 * Corresponds to Java class <code>com.actimind.actitime.util.daterange.PreviousWeekDateRangeState</code>
 */
AT.PreviousWeekDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
        return currentUserTime().getWeekStartDate().add(Date.DAY, -7).clearTime();
    },
    getToDate     : function() {
        return currentUserTime().getWeekStartDate().add(Date.DAY, -1).clearTime();
    }
});

/**
 * Corresponds to Java class <code>com.actimind.actitime.util.daterange.CurrentAndPreviousWeeksDateRangeState</code>
 */
AT.CurrentAndPreviousWeeksDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
        return currentUserTime().getWeekStartDate().add(Date.DAY, -7).clearTime();
    },
    getToDate     : function() {
        return currentUserTime().getWeekStartDate().add(Date.DAY, 6).clearTime();
    }
});

/**
 * Corresponds to Java class <code>com.actimind.actitime.util.daterange.CurrentAndTwoPreviousWeeksDateRangeState</code>
 */
AT.CurrentAndTwoPreviousWeeksDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
        return currentUserTime().getWeekStartDate().add(Date.DAY, -14).clearTime();
    },
    getToDate     : function() {
        return currentUserTime().getWeekStartDate().add(Date.DAY, 6).clearTime();
    }
});

/**
 * Corresponds to Java class <code>com.actimind.actitime.util.daterange.FirstHalfOfCurrentMonthState</code>
 */
AT.FirstHalfOfCurrentMonthState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
        return currentUserTime().getFirstDateOfMonth().clearTime();
    },
    getToDate     : function() {
        return currentUserTime().getFirstDateOfMonth().add(Date.DAY, 14).clearTime();
    }
});

/**
 * Corresponds to Java class <code>com.actimind.actitime.util.daterange.SecondHalfOfCurrentMonthState</code>
 */
AT.SecondHalfOfCurrentMonthState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
        return currentUserTime().getFirstDateOfMonth().add(Date.DAY, 15).clearTime();
    },
    getToDate     : function() {
        return currentUserTime().getLastDateOfMonth().clearTime();
    }
});

/**
 * Corresponds to Java class <code>com.actimind.actitime.util.daterange.SecondHalfOfPreviousMonthState</code>
 */
AT.SecondHalfOfPreviousMonthState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
        return currentUserTime().add(Date.MONTH, -1).getFirstDateOfMonth().add(Date.DAY, 15).clearTime();
    },
    getToDate     : function() {
        return currentUserTime().add(Date.MONTH, -1).getLastDateOfMonth().clearTime();
    }
});

/**
 * Corresponds to Java class <code>com.actimind.actitime.util.daterange.Last7DaysDateRangeState</code>
 */
AT.Last7DaysDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
        return currentUserTime().add(Date.DAY, -6).clearTime();
    },
    getToDate     : function() {
        return currentUserTime().clearTime();
    }
});

/**
 * Corresponds to Java class <code>com.actimind.actitime.util.daterange.Last14DaysDateRangeState</code>
 */
AT.Last14DaysDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
        return currentUserTime().add(Date.DAY, -13).clearTime();
    },
    getToDate     : function() {
        return currentUserTime().clearTime();
    }
});

AT.Last21DaysDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
        return currentUserTime().add(Date.DAY, -20).clearTime();
    },
    getToDate     : function() {
        return currentUserTime().clearTime();
    }
});

AT.CurrentYearDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
        return new Date(currentUserTime().getFullYear(), 0, 1);
    },
    getToDate     : function() {
        return (new Date(currentUserTime().getFullYear() + 1, 0, 1)).add(Date.DAY, -1);
    }
});

AT.CurrentYearTillTodayDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
        return new Date(currentUserTime().getFullYear(), 0, 1);
    },
    getToDate     : function() {
        return currentUserTime().clearTime();
    }
});

AT.PreviousYearDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
        return new Date(currentUserTime().getFullYear() - 1, 0, 1);
    },
    getToDate     : function() {
        return (new Date(currentUserTime().getFullYear(), 0, 1)).add(Date.DAY, -1);
    }
});

/**
 *  Single date ranges.
 *  Following date ranges have the same fromDate value which is new Date(1970, 0, 1)
 */

AT.DayDateState = Ext.extend(AT.DateRangeState, {
    getFromDate   : function() {
        return new Date(1970, 0, 1);
    }
});

AT.TodayDateState = Ext.extend(AT.DayDateState,
{
    getToDate : function() {
        return currentUserTime().clearTime();
    }
});

AT.YesterdayDateState = Ext.extend(AT.DayDateState,
{
    getToDate       : function() {
        return currentUserTime().add(Date.DAY, -1).clearTime();
    }
});

AT.LastWeekDayDateState = Ext.extend(AT.DayDateState,
{
    dayOfWeek     : -1,
    getToDate       : function() {

        var userTime = currentUserTime();
        var daysDiff = this.dayOfWeek - userTime.getDay();
        if (daysDiff >= 0)
            daysDiff -= 7;
        return userTime.add(Date.DAY, daysDiff).clearTime();
    }
});

AT.LastSundayDateState = Ext.extend(AT.LastWeekDayDateState,
{
    dayOfWeek     : 0
});

AT.LastMondayDateState = Ext.extend(AT.LastWeekDayDateState,
{
    dayOfWeek     : 1
});

AT.LastTuesdayDateState = Ext.extend(AT.LastWeekDayDateState,
{
    dayOfWeek     : 2
});

AT.LastWednesdayDateState = Ext.extend(AT.LastWeekDayDateState,
{
    dayOfWeek     : 3
});

AT.LastThursdayDateState = Ext.extend(AT.LastWeekDayDateState,
{
    dayOfWeek     : 4
});

AT.LastFridayDateState = Ext.extend(AT.LastWeekDayDateState,
{
    dayOfWeek     : 5
});

AT.LastSaturdayDateState = Ext.extend(AT.LastWeekDayDateState,
{
    dayOfWeek     : 6
});


AT.EndOfThePreviousMonthDateState = Ext.extend(AT.DayDateState,
{
    getToDate       : function() {
        return currentUserTime().getFirstDateOfMonth().add(Date.DAY, -1).clearTime();
    }
});

AT.BeginningOfTheCurrentMonthDateState = Ext.extend(AT.DayDateState,
{
    getToDate       : function() {
        return currentUserTime().getFirstDateOfMonth().clearTime();
    }
});

//------------------------- actiPLANS states -----------------------------

AT.CurrentAndNextMonthDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate   : function() {
                        return currentUserTime().getFirstDateOfMonth().clearTime();
                    },
    getToDate     : function() {
                        return currentUserTime().add(Date.MONTH, 1).getLastDateOfMonth().clearTime();
                    }
});

AT.NextTwoMonthsDateRangeState = Ext.extend(AT.DateRangeState,
{
    getFromDate:function ()
    {
        return currentUserTime().add( Date.MONTH, 1 ).getFirstDateOfMonth().clearTime();
    },
    getToDate:function ()
    {
        return currentUserTime().add( Date.MONTH, 2 ).getLastDateOfMonth().clearTime();
    }
});

AT.SomeWeeksAheadDateRangeState = Ext.extend(AT.DateRangeState,
{
    countOfWeeks: 0,
    getFromDate: function ()
    {
        return currentUserTime().clearTime();
    },
    getToDate: function ()
    {
        return currentUserTime().add( Date.DAY, this.countOfWeeks * 7 - 1 ).clearTime();
    }
});

AT.getQuarter = function(month)
{
    return month / 3 >> 0;
};

AT.getQuarterFirstMonth = function(month)
{
    return AT.getQuarter(month) * 3;
};

AT.getPreviousQuarterFirstMonth = function(month)
{
    var prevQuarterMonth = month - 3;
    return prevQuarterMonth < 0 ? 9 : AT.getQuarterFirstMonth(prevQuarterMonth);
};

AT.CurrentQuarterDateRangeState = Ext.extend(
        AT.DateRangeState,
        {
            getFromDate: function()
            {
                var date = currentUserTime();
                date.setMonth(AT.getQuarterFirstMonth(date.getMonth()), 1);
                return date.clearTime();
            },
            getToDate: function()
            {
                var date = currentUserTime();
                date.setMonth(AT.getQuarterFirstMonth(date.getMonth())+2, 1);
                return date.getLastDateOfMonth().clearTime();
            }
        });

AT.PreviousQuarterDateRangeState = Ext.extend(
        AT.DateRangeState,
        {
            getFromDate: function()
            {
                var date = currentUserTime();
                date.setMonth(AT.getPreviousQuarterFirstMonth(date.getMonth()), 1);
                return date.clearTime();
            },
            getToDate: function()
            {
                var date = currentUserTime();
                date.setMonth(AT.getPreviousQuarterFirstMonth(date.getMonth())+2, 1);
                return date.getLastDateOfMonth().clearTime();
            }
        });