Ext.namespace("AT");

/**
 * Depends on:
 * <code>/js/at.js</code> specifically on areDatesEqual() function
 */

//noinspection JSUnusedLocalSymbols
AT.DateButton = Ext.extend(Ext.Button,
{
    handleMouseEvents : false,

    icoSrc : "",

    buttonSelector: "a:first",

    showToday       : true,

    menuAlign       : "tl-bl?",

    overCls         : "x-date-over",

    scheduleCalculator : null,

    menuListeners :
    {
        //noinspection JSUnusedLocalSymbols
        select : function (m, d)
        {
            this.setValue(d);
        },

        datePickerUpdate : function(date)
        {
            // display the given date, but don't actually change the date
            this.displayValue(date);
        },

        datePickerCancel : function()
        {
            // display the stored value
            this.displayValue(this.getValue());
        },

        hide : function ()
        {
            // focus the field, but only if the user didn't click outside the date picker
            if (! this.menu.wasSelectionOnHide)
                this.focus();

            this.restoreWindowKeyDownHandler();
        },

        show : function ()
        {
            this.setWindowKeyDownHandler();
        }
    },

    internalFormat: new DateFormat("yyyy-MM-dd"),

    initComponent: function()
    {
        AT.DateButton.superclass.initComponent.call(this);

        this.internalText = new Ext.form.Hidden(
        {
            renderTo : this.renderTo,
            name     : this.name + ".internalText"
        }
                );

        this.userText = new Ext.form.Hidden(
        {
            renderTo : this.renderTo,
            name     : this.name + ".userText"
        }
                );

        this.menu = new AT.menu.DatePickerMenu({
            minDate : this.minValue,
            maxDate : this.maxValue,
            format : this.format,
            showToday : this.showToday,
            scheduleCalculator: this.scheduleCalculator,
            startDay    : this.weekStartDay
        });
        this.addEvents("change");
        this.menu.on(Ext.apply({}, this.menuListeners, { scope:this }));
    },

    setWindowKeyDownHandler : function ()
    {
        var thisButton = this;
        if ( !this.escKeyMap )
            this.escKeyMap = new Ext.KeyMap(Ext.getBody().parent(), {
                key: Ext.EventObject.ESC,
                fn: function(key, ev){
                    if(this.menu){
                        this.menu.hide();
                    }
                },
                scope: thisButton,
                stopEvent: true
            });
        this.escKeyMap.enable();
    },

    restoreWindowKeyDownHandler : function ()
    {
        if (this.escKeyMap)
            this.escKeyMap.disable();
    },

    formatDate: function(date)
    {
        return Ext.isDate(date) ? date.dateFormat(this.format) : date;
    },

    showMenu : function(){
        if (this.menu)
        {
            this.menu.picker.setValue(this.getValue() || currentUserTime());
            this.menu.show(this.el.child(this.buttonSelector).child('.x-btn-with-menu'), this.menuAlign, null, 'dynamic');
        }
        return this;
    },

    setValue : function(date)
    {
        this.setText(this.formatDate(date));
        if (!this.date || !areDatesEqual(this.date, date))
        {
            this.date = date;
            this.fireEvent("change", this, date);
            this.internalText.setValue(this.internalFormat.formatDate(date));
            this.userText.setValue(this.formatDate(date));
        }
    },

    /**
     * Displays the given date, but doesn't actually change the date (the user didn't submit the date picker yet).
     * @param date
     */
    displayValue : function(date)
    {
        this.setText(this.formatDate(date));
    },

    getValue : function()
    {
        return this.date;
    },

    setText : function(text)
    {
        this.text = text;
        if(this.el)
        {
            this.el.child("span.date-text:first-child").update(text);
        }
        this.autoWidth();
    },

    onRender : function(ct, position) {
        if (!this.template) {
            if (!AT.DateButton.Template) {
                // hideous table template
                AT.DateButton.Template = new Ext.Template(
                        '<span><a href="#" style="text-decoration: none; cursor: pointer; display: inline-block;" class="at-date-button">',
                        '<table border="0" cellpadding="0" cellspacing="0"><tbody><tr><td class="date-text-td">',
                        '    <span class="date-text">{0}</span>',
                        '</td><td style="padding-left:2px">      ',
                        '<img src="{2}" class="date-button-image"/></td></tr></tbody></table></a></span>'
                );
            }
            this.template = AT.DateButton.Template;
        }
        var btn, targs = [this.text || '&#160;', this.type, this.icoSrc];

        if (position) {
            btn = this.template.insertBefore(position, targs, true);
        } else {
            btn = this.template.append(ct, targs, true);
        }
        var btnEl = btn.child(this.buttonSelector);
        btnEl.on('focus', this.onFocus, this);
        btnEl.on('blur', this.onBlur, this);

        this.initButtonEl(btn, btnEl);

        if (this.menu) {
            this.el.child(this.menuClassTarget).addClass("x-btn-with-menu");
        }
        Ext.ButtonToggleMgr.register(this);
    }

});

Ext.reg('at_datebutton', AT.DateButton);