Ext.namespace("AT.form");

    AT.form.DateField = Ext.extend(Ext.form.DateField,
    {
        isTriggerClicked : false,

        datepreparser : parseDate,

        rawFormat : '',

        weekStartDay : 1,

        scheduleCalculator: null,

        onTriggerClick : function()
        {
            if (this.disabled)
            {
                return;
            }

            if (this.isTriggerClicked)
            {
                this.isTriggerClicked = false;

                // date picker is being closed by clicking on its trigger, so we must focus the field
                this.focus();

                return;
            }

            if (this.menu == null)
            {
                this.menu = new AT.menu.DatePickerMenu(
                        {
                            minDate : this.minValue,
                            maxDate : this.maxValue,
                            disabledDatesRE : this.ddMatch,
                            disabledDatesText : this.disabledDatesText,
                            disabledDays : this.disabledDays,
                            disabledDaysText : this.disabledDaysText,
                            format : this.format,
                            showToday : this.showToday,
                            minText : String.format(this.minText, this.formatDate(this.minValue)),
                            maxText : String.format(this.maxText, this.formatDate(this.maxValue)),
                            scheduleCalculator: this.scheduleCalculator,
                            startDay: this.weekStartDay

                });
            }

            this.menu.on(Ext.apply({}, this.menuListeners,
            {
                scope:this
            }));
            this.valueBeforePickerShown = this.getValue();
            this.menu.picker.setValue(this.getValue() || currentUserTime());
            this.menu.show(this.el, "tl-bl?", null, this.getValue() == '' ? 'static' : 'dynamic');
        },

        // private
        onDocumentMouseDown : function(e)
        {
            if (!e.within(this.menu.picker.getEl().dom))
            {
                this.removeTriggerHandlers();
            }

            if (this.trigger.dom == e.target)
            {
                this.isTriggerClicked = true;
            }
        },

        onEscapePress :function(e)
        {
            if(Ext.EventObject.ESC == e.getKey())
            {
                this.removeTriggerHandlers();
            }
        },

        // private
        beforeBlur : function()
        {
            var v = this.prepareDate(this.getRawValue());
            if (v)
            {
                this.setValue(v);
            }
        },

        prepareDate : function(str)
        {
             return this.datepreparser(str, this.rawFormat);
        },

        removeTriggerHandlers : function()
        {
            Ext.get(Ext.isIE ? document.body : document).un("mousedown", this.onDocumentMouseDown, this);
            Ext.get(Ext.isIE ? document.body : document).un("keypress", this.onEscapePress, this);
        },

        addTriggerHandlers : function()
        {
           Ext.get(Ext.isIE ? document.body : document).on("mousedown", this.onDocumentMouseDown, this);
           Ext.get(Ext.isIE ? document.body : document).on("keypress", this.onEscapePress, this);
        },

        menuListeners :
        {
            select: function(m, d)
            {
                this.setValue(d);

                // Only remove trigger handlers if the user explicitly selected the date.
                // Because in case of 'selection on hide' the handler onDocumentMouseDown still needs to be called.
                if (! this.menu.wasSelectionOnHide)
                    this.removeTriggerHandlers();

                this.fireEvent("select", d);
            },
            show : function()
            { // retain focus styling
                this.addTriggerHandlers();
                this.onFocus();
            },
            hide : function()
            {
                var ml = this.menuListeners;
                this.menu.un("select", ml.select,  this);
                this.menu.un("show", ml.show,  this);
                this.menu.un("hide", ml.hide,  this);
                
                // focus the field, but only if the user didn't click outside the date picker
                if (! this.menu.wasSelectionOnHide)
                    this.focus();
            },
            beforeshow : function()
            {
                return this.fireEvent("beforepickershow", this);
            },

            datePickerUpdate : function(date)
            {
                if (this.getValue() != '')
                {
                    // use setRawValue because we don't want to fire any events
                    this.setRawValue(this.formatDate(date));
                }
            },

            datePickerCancel : function()
            {
                // use setRawValue because we don't want to fire any events
                this.setRawValue(this.formatDate(this.valueBeforePickerShown));
                this.removeTriggerHandlers();
            }
        }

    });

    Ext.reg('atdatefield', AT.form.DateField);

