Ext.namespace("AT");

AT.DateRangeDropdownSelector = Ext.extend(Ext.Container, {
    autoEl: "div",
    layout: "table",
    rangeTypeValue: null,
    menuStates: [],
    radioGroup: "dateRange",

    initComponent: function() {
        AT.DateRangeDropdownSelector.superclass.initComponent.call(this);

        this.dateRangeInitState = {
            dateRangeStateValue : this.rangeTypeValue
        };
        
        this.createDropdown();
        this.setItemsMap();
        this.addEvents("change");
        this.setMenuItems();
    },

    createDropdown: function() {
        this.dropdownCtn = new AT.CustomDropdownButton();
        this.add(this.dropdownCtn);
    },

    defineDropdownEvents: function() {
        this.dropdownCtn.on("render", function() { // deferring until the component is rendered
            this.dropdownCtn.el.on("mousedown", this.toggleMenu, this);
            this.dropdownCtn.el.dom.style.cursor = "pointer";
        }, this);
    },

    setMenuItems: function () {
        for (var i = 0; i < this.menuStates.length; ++i) {
            var state = this.menuStates[i];
            state.radioGroup = this.radioGroup;
            state.clickHandler = this.itemClicked;
            state.handlerScope = this;

            if ( state.custom )
                this.customState = state;

            if ( this.rangeTypeValue == state.value )
                this.updateState(state);
        }
    },
    
    afterRender: function() {
        this.defineDropdownEvents();
        this.menuShown = false;
    },

    fireChangeEvent: function() {
        this.fireEvent("change", this);
    },

    setItemsMap: function () {
        this.itemsMap = {};
        for ( var i = 0; i < this.menuStates.length; ++i ) {
            var state = this.menuStates[i];
            this.itemsMap[state.value] = state;
        }
    },
    
    refreshMenu: function() {
        var menuItems = [];
        for (var i = 0; i < this.menuStates.length; ++i) {
            var state = this.menuStates[i];
            menuItems.push(state.createItem(this.currentState));
        }

        this.menu = new Ext.menu.Menu({
                                          items: menuItems,
                                          cls: "at-dropdown-list-btn-menu rangesListMenu"
                                      });
        this.menu.on("show", this.showMenuHandler, this);
        this.menu.on("hide", this.hideMenuHandler, this);
    },

    setCustomState: function() {
        this.updateState(this.customState);
    },
    
    updateState: function(state) {
        this.currentState = state;
        this.dropdownCtn.setText(state.title);
    },
    
    /**
     * Dropdown menu item click handler.
     * @private
     */
    itemClicked: function (item) {
        if ( item.state !== this.currentState ) {
            this.updateState(item.state);
            this.fireChangeEvent();
        }
        
        // to speed up items menu hiding
        this.hideMenu();
    },
    
    hideMenu: function () {
        if ( this.menuShown )
            this.menu.hide();
    },

    mouseOver: function() {
        this.addClass("active");
    },

    mouseOut: function() {
        if (!this.menuShown)
            this.removeClass("active");
    },

    getMenuAlign: function() {
        return "tl-bl?";
    },

    getElementToShowMenu: function() {
        return this.el.child("table:first");
    },

    /**
     * Dropdown button click handler.
     * @private
     */
    toggleMenu: function( event ) {
        if ( event.button !== 0 )
            return;
        
        this.menuShown = !this.menuShown;
        
        if ( this.menuShown ) {
            this.refreshMenu(); // updating dates in the descriptions in case they changed
            this.menu.show(this.getElementToShowMenu(), this.getMenuAlign(), null);
            this.addClass("menuShown");
        }
    },

    /**
     * Dropdown menu hide handler.
     * @private
     */
    hideMenuHandler: function() {
        this.menuShown = false;
        this.removeClass("menuShown");
        this.restoreWindowKeyDownHandler();
    },

    showMenuHandler: function() {
        this.setWindowKeyDownHandler();
    },

    /**
     * @return {String} range type's title
     */
    getRangeType: function() {
        return this.currentState;
    },
    
    setRangeType: function( value ) {
        this.updateState(this.itemsMap[value]);
    },

    setWindowKeyDownHandler : function ()
    {
        var self = this;
        if ( !this.escKeyMap )
            this.escKeyMap = new Ext.KeyMap(Ext.getBody().parent(), {
                key: Ext.EventObject.ESC,
                fn: function(key, ev){
                    if(this.menu){
                        this.menu.hide();
                    }
                },
                scope: self,
                stopEvent: true
            });
        this.escKeyMap.enable();
    },

    restoreWindowKeyDownHandler : function ()
    {
        if (this.escKeyMap)
            this.escKeyMap.disable();
    }
    
});

Ext.reg("date_range_dropdown_selector", AT.DateRangeDropdownSelector);