Ext.namespace("AT");

AT.CustomDateRangeSelector = Ext.extend(AT.DropdownDateRangeSelector, {
    fromDate: null,
    toDate: null,

    initComponent: function() {
        AT.CustomDateRangeSelector.superclass.initComponent.call(this);
        
        this.fromDateButton = this.createDateButton("fromDate", this.scheduleCalculator);
        this.toDateButton = this.createDateButton("toDate", this.scheduleCalculator);

        this.add(this.fromDateButton);
        // "-" symbol
        this.add({
                     xtype: "box",
                     autoEl: {
                         tag: "span",
                         html: "-",
                         cls: "calendarsSeparator"
                     }
                 });

        this.add(this.toDateButton);         
        this.initButtons();
    },
    
    /**
     * Date button creation helper.
     * @private
     */
    createDateButton: function(name, scheduleCalculator) {
        return new AT.DateButton({
                                     format: this.buttonTextFormat,
                                     minValue: new Date(1970, 0, 1),
                                     weekStartDay: SystemSettings.weekStartDay - 1,
                                     scheduleCalculator: scheduleCalculator,
                                     name: name,
                                     cls: "calendarButton " + name
                                 });
    },
    
    afterRender: function() {
        this.registerCustomPicker(this.fromDateButton);
        this.registerCustomPicker(this.toDateButton);
    },

    initButtons: function() {
        var state = this.getRangeType();
        if ( !state.custom ) {
            this.fromDate = state.range.getFromDate();
            this.toDate = state.range.getToDate();
        }

        this.updateButtons( this.fromDate, this.toDate );
    },

    /**
     * Dropdown menu item changed handler.
     * @private
     */
    menuItemChangedCallback: function () {
        var state = this.getRangeType();
        
        if ( !state.custom )
            this.updateButtons(state.range.getFromDate(), state.range.getToDate());
        else {
            this.animateCustom(this.fromDateButton);
            this.animateCustom(this.toDateButton);
        }
    },
    
    /**
     * Updates the date controls on page.
     * @private
     * @param {Date} fromDate 'from' date
     * @param {Date} toDate 'to' date
     */
    updateButtons: function( fromDate, toDate ) {
        this.fromDateButton.setValue(fromDate);
        this.toDateButton.setValue(toDate);
    },

    /**
     * @return {Date} selected 'from' date.
     */
    getFromDate: function() {
        return this.fromDateButton.getValue();
    },

    setFromDate: function( date ) {
        this.fromDateButton.setValue(date);
    },

    setValues: function ( qualifier, fromDate, toDate ) {
        this.setRangeTypeValue(qualifier);
        this.fromDate = fromDate;
        this.toDate = toDate;
        
        this.initButtons();
    },
    
    /**
     * @return {Date} selected 'to' date.
     */
    getToDate: function() {
        return this.toDateButton.getValue();
    },
    
    setToDate: function( date ) {
        this.toDateButton.setValue(date);
    }
    
});

Ext.reg("date_range_selector", AT.CustomDateRangeSelector);