/**
 * This method wrap input field as datefield on onLoad browser event.
 * @param containerId id of span element which wrap as extjs datefield container
 * @param inputFieldId input field for wrap
 * @param dateFormat current dateformat
 * @param handlers all needed handlers for datefield events
 *
 * datefield events:
 *
 *        @event autosize : ( Ext.form.Field this, Number width )
 *        Fires when the autosize function is triggered. The field may or may not have actually changed size according to the ...
 *
 *        @event beforedestroy : ( Ext.Component this )
 *        Fires before the component is destroyed. Return false to stop the destroy.
 *
 *        @event beforehide : ( Ext.Component this )
 *        Fires before the component is hidden. Return false to stop the hide.
 *
 *        @event beforerender : ( Ext.Component this )
 *        Fires before the component is rendered. Return false to stop the render.
 *
 *        @event beforeshow : ( Ext.Component this )
 *        Fires before the component is shown. Return false to stop the show.
 *
 *        @event beforestaterestore : ( Ext.Component this, Object state )
 *        Fires before the state of the component is restored. Return false to stop the restore.
 *
 *        @event beforestatesave : ( Ext.Component this, Object state )
 *        Fires before the state of the component is saved to the configured state provider. Return false to stop the save.
 *
 *        @event blur : ( Ext.form.Field this )
 *        Fires when this field loses input focus.
 *
 *        @event change : ( Ext.form.Field this, Mixed newValue, Mixed oldValue )
 *        Fires just before the field blurs if the field value has changed.
 *
 *        @event destroy : ( Ext.Component this )
 *        Fires after the component is destroyed.
 *
 *        @event disable : ( Ext.Component this )
 *        Fires after the component is disabled.
 *
 *        @event enable : ( Ext.Component this )
 *        Fires after the component is enabled.
 *
 *        @event focus : ( Ext.form.Field this )
 *        Fires when this field receives input focus.
 *
 *        @event hide : ( Ext.Component this )
 *        Fires after the component is hidden.
 *
 *        @event invalid : ( Ext.form.Field this, String msg )
 *        Fires after the field has been marked as invalid.
 *
 *        @event keydown : ( Ext.form.TextField this, Ext.EventObject e )
 *        Keydown input field event. This event only fires if enableKeyEvents is set to true.
 *
 *        @event keypress : ( Ext.form.TextField this, Ext.EventObject e )
 *        Keypress input field event. This event only fires if enableKeyEvents is set to true.
 *
 *        @event keyup : ( Ext.form.TextField this, Ext.EventObject e )
 *        Keyup input field event. This event only fires if enableKeyEvents is set to true.
 *
 *        @event move : ( Ext.Component this, Number x, Number y )
 *        Fires after the component is moved.
 *
 *        @event render : ( Ext.Component this )
 *        Fires after the component is rendered.
 *
 *        @event resize : ( Ext.Component this, Number adjWidth, Number adjHeight, Number rawWidth, Number rawHeight )
 *        Fires after the component is resized.
 *
 *        @event show : ( Ext.Component this )
 *        Fires after the component is shown.
 *
 *        @event specialkey : ( Ext.form.Field this, Ext.EventObject e )
 *        Fires when any key related to navigation (arrows, tab, enter, esc, etc.) is pressed. You can check Ext.EventObject.g...
 *
 *        @event staterestore : ( Ext.Component this, Object state )
 *        Fires after the state of the component is restored.
 *
 *        @event statesave : ( Ext.Component this, Object state )
 *        Fires after the state of the component is saved to the configured state provider.
 *
 *        @event valid : ( Ext.form.Field this )
 *        Fires after the field has been validated with no errors.
 *
 * see extjs documentation for more information
 * <b>custom events:<b>
 *         @event select fires after date select from datepicker.(used for changeable controller)
 *                 you should added {@see form_changes.js} <b>changeableHandler<b>
 *                 for this event for propertly work.
 *                 Also see bug 25140 DHTML Cal: different behaviour on date change
 *         @event beforepickershow fires before the datePicker is shown.Return false to stop the show.
 *
 *
 * @param maxDate maximum allowed date.
 * @param maxText tooltip text for dates after maxDate.
 *
 **/
function makeDateField(containerId, inputFieldId, dateFormat, handlers, maxDate, maxText,  weekStartDay, withoutRender, scheduleCalculator)
{
Ext.onReady(function()
{
    var inputField = Ext.get(inputFieldId);
    var value = inputField.dom.value;
    var dateField = new AT.form.DateField(
    {
        el              : inputField.dom,
        format          : Date.convertSmartToExtDateFormat(dateFormat),
        rawFormat       : dateFormat,
        altFormats      : "",
        validationDelay : 60000, // give the user 1 minute to enter data before we automatically correct it
        invalidClass    : "",
        id              : containerId,
        maxValue        : maxDate,
        maxText         : maxText ? maxText : "",
        weekStartDay    : weekStartDay,
        scheduleCalculator : scheduleCalculator
        //readOnly: true
    });
    dateField.enableKeyEvents = true;
    dateField.on(handlers);
    if (!withoutRender) dateField.render(containerId);
    // DateField does not allow entering incorrect values using setValue(value)
    // method, so we put the value directly into the input field.
    inputField.dom.value = value;
});
}

