Ext.namespace("AT");

AT.DropdownDateRangeSelector = Ext.extend(Ext.Container, {
    autoEl: "div",
    layout: "table",
    
    menuItemChangedCallback: Ext.emptyFn,
    
    buttonTextFormat: Date.convertSmartToExtDateFormat(SystemSettings.dateFormat),
    
    rangeTypeValue: null,
    menuStates: [],
    radioGroup: "dateRange",
    scheduleCalculator: null,

    initComponent: function() {
        AT.DropdownDateRangeSelector.superclass.initComponent.call(this);

        this.dropdownCtn = new AT.DateRangeDropdownSelector(
                {
                    rangeTypeValue: this.rangeTypeValue,
                    menuStates: this.menuStates,
                    cls: "dateRangeDropdownSelector"
                });
        
        this.dropdownCtn.on("change", this.menuItemChanged, this);
        this.add(this.dropdownCtn);

        this.menuList = [this.dropdownCtn];
        
        this.addEvents("change");
    },

    hideMenu: function () {
        for ( var i = 0; i < this.menuList.length; i++ ) {
            this.menuList[i].hideMenu();
        }
    },
    
    /**
     * @private
     */
    fireChangeEvent: function() {
        this.fireEvent("change", this);
    },
    
    /**
     * @private
     */
    onCustomPickerChanged: function() {
        this.dropdownCtn.setCustomState();
        this.fireChangeEvent();
    },

    /**
     * @protected
     */
    registerCustomPicker: function( datePicker ) {

        var self = this;

        function pickerOpened( button ) {
            button.on("change", self.onCustomPickerChanged, self);
        }

        function pickerClosed( button ) {
            button.un("change", self.onCustomPickerChanged, self);
        }

        function prepareButton( button ) {
            button.on("menushow", pickerOpened);
            button.on("menuhide", pickerClosed);
            button.show();
        }

        this.menuList.push(datePicker);
        prepareButton(datePicker);
    },

    /**
     * Flashes date controls with custom background color.
     */
    animateCustom: function( customPicker ) {
        var customPickerEl = customPicker.el;

        var cls = "buttonAnimated";
        (function() {
            customPickerEl.toggleClass(cls);
        }).defer(500);

        (function() {
            customPickerEl.toggleClass(cls);
        }).defer(1250);
    },

    /**
     * Dropdown menu item changed handler.
     * @private
     */
    menuItemChanged: function () {
        this.menuItemChangedCallback();
        this.fireChangeEvent();
    },
    
    getRangeType: function() {
        return this.dropdownCtn.getRangeType();
    },
    
    getRangeTypeValue: function() {
        return this.getRangeType().value;
    },
    
    setRangeTypeValue: function( value ) {
        return this.dropdownCtn.setRangeType(value);
    }
    
});

Ext.reg("dropdown_date_range_selector", AT.DropdownDateRangeSelector);