Ext.namespace("AT");

AT.SingleDateSelector = Ext.extend(AT.DropdownDateRangeSelector, {
    toDate: null,

    initComponent: function() {
        AT.SingleDateSelector.superclass.initComponent.call(this);
        
        this.toDateButton = this.createDateButton("toDate", this.scheduleCalculator);
        this.add(this.toDateButton);

        this.initButton();
    },

    /**
     * Date button creation helper.
     * @private
     */
    createDateButton: function(name, scheduleCalculator) {
        return new AT.DateButton({
                                     format: this.buttonTextFormat,
                                     minValue: new Date(1970, 0, 1),
                                     weekStartDay: SystemSettings.weekStartDay - 1,
                                     scheduleCalculator: scheduleCalculator,
                                     name: name,
                                     cls: "calendarButton " + name
                                 });
    },

    afterRender: function() {
        this.registerCustomPicker(this.toDateButton);
    },

    initButton: function() {
        var state = this.getRangeType();
        if ( !state.custom )
            this.toDate = state.range.getToDate();

        this.updateButton(this.toDate);
    },
    
    /**
     * Dropdown menu item changed handler.
     * @private
     */
    menuItemChangedCallback: function () {
        var state = this.getRangeType();
        
        if ( !state.custom )
            this.updateButton(state.range.getToDate());
        else
            this.animateCustom(this.toDateButton);
    },

    /**
     * Updates the date control on page.
     * @private
     * @param {Date} toDate 'to' date
     */
    updateButton: function( toDate ) {
        this.toDateButton.setValue(toDate);
    },

    /**
     * @return {Date} selected 'to' date.
     */
    getToDate: function() {
        return this.toDateButton.getValue();
    },
    
    setToDate: function( date ) {
        this.toDateButton.setValue(date);
    },

    setValues: function ( qualifier, date ) {
        this.setRangeTypeValue(qualifier);
        this.toDate = date;

        this.initButton();
    }

});

Ext.reg("single_date_selector", AT.SingleDateSelector);