function DelayTooltip(){}

DelayTooltip.DEFAULT_DELAY = 500; // 0.5 sec
DelayTooltip.prototype.constructor = DelayTooltip;
DelayTooltip.superproto = TableTooltip.prototype;
DelayTooltip.prototype = new TableTooltip();

/**
 *
 * @param tooltipId id of tooltip element
 * @param delay delay in ms before tooltip will be showed (0.5 sec by default)
 * @returns {DelayTooltip}
 */
DelayTooltip.prototype.init = function( tooltipId, delay) {
    DelayTooltip.superproto.init.call(this, tooltipId);
    if (delay) {
        this.delay = delay;
    } else {
        this.delay = DelayTooltip.DEFAULT_DELAY;
    }
    this.isDelayed = true;

    this.hide(); // hide tooltip after init
    return this;
};

DelayTooltip.prototype.showWithDelay = function(){
    var self = this;
    this.tooltipDelayTimer = setTimeout(
        function() {
            self._setVisible(true); // show tooltip
        },
        this.delay
    )
};

DelayTooltip.prototype.updateUIAndShow = function( config, offset ) {
    DelayTooltip.superproto.updateUIAndShow.call(this, config, offset);
    if (this.isDelayed) {
        this.showWithDelay(); // show after delay
    } else {
        this._setVisible(true);
    }
};

DelayTooltip.prototype.hide = function(){
    if(this.tooltipDelayTimer && this.isDelayed ) {
        clearTimeout(this.tooltipDelayTimer);
    }
    this._setVisible(false); // hide tooltip
    DelayTooltip.superproto.hide.call(this);
};


DelayTooltip.prototype._setVisible = function( isVisible ) {
    if (isVisible) {
        this._tooltipEl.css("visibility", "visible");
    }  else {
        this._tooltipEl.css("visibility", "hidden");
    }
};

/**
 * set value of delay
 * @param newDelay
 */
DelayTooltip.prototype.setDelay = function( newDelay ) {
    this.delay = newDelay;
};

/**
 * on/off delay
 * @param isDelayed
 */
DelayTooltip.prototype.needDelay = function( isDelayed ) {
    this.isDelayed = isDelayed;
};