/**
 * Wrapper for AT.RoundedBox to provide graphic button functions(different kinds of highlighting)
 * @param button AT.RoundedBox element
 */

function GraphicButton(button)
{
    this.button = button;
    this.hoverClassName = "hoveredGraphicButton";
    this.activeClassName = "activeGraphicButton";
}

GraphicButton.prototype.hover = function()
{
    addClass(this.button.el.dom, this.hoverClassName);
};

GraphicButton.prototype.unhover = function()
{
    removeClass(this.button.el.dom, this.hoverClassName);
};

GraphicButton.prototype.activate = function()
{
    addClass(this.button.el.dom, this.activeClassName);
};

GraphicButton.prototype.deactivate = function()
{
    removeClass(this.button.el.dom, this.activeClassName);
};

GraphicButton.prototype.reset = function()
{
    this.unhover();
    this.deactivate();
};

/**
 * Define event handlers, which are standard for graphic button. If event is needed pass true in corresponding parameter and false otherwise.
 * @param isMouseOverStandard is mouse over event needed
 * @param isMouseOutStandard  is mouse out event needed
 * @param isMouseDownStandard is mouse down event needed
 * @param isMouseUpStandard   is mouse up event needed
 */
GraphicButton.prototype.defineStandardEventHandlers = function(isMouseOverStandard, isMouseOutStandard, isMouseDownStandard, isMouseUpStandard)
{
    if (isMouseOverStandard)
    {
        this.button.el.on("mouseover", this.hover, this);
    }
    if (isMouseOutStandard)
    {
        this.button.el.on("mouseout", this.reset, this);
    }
    if (isMouseDownStandard)
    {
        this.button.el.on("mousedown", this.activate, this);
    }
    if (isMouseUpStandard)
    {
        this.button.el.on("mouseup", this.deactivate, this);
    }
};

GraphicButton.getRoundedBox = function(renderToId, buttonText, className)
{
    return new AT.RoundedBox({
                                 cls: "graphicButton button " + (_.isUndefined(className) ? "" : className),
                                 renderTo: renderToId,
                                 items: [{
                                             xtype: "box",
                                             autoEl: {
                                                 tag: "div",
                                                 html: "<span unselectable='on'>"+ buttonText +"</span>"
                                             }
                                         }]
                             });
};


