/**
 * This button works with OkCancelPopup. It drops down and align to bottom right conner popup and sets ok/cancel event
 * handlers.
 *
 * Needs cell_button.js
 */

OkCancelPopupDropdownButton.prototype = new CellButton();
OkCancelPopupDropdownButton.prototype.parentClass = CellButton;

/**
 * Initializes OkCancelPopupDropdownButton instance
 *
 * <i> for other params description see CellButton constructor at cell_button.js
 *
 * @param popup is OkCancelPopup object.
 *
 */
function OkCancelPopupDropdownButton(buttonTable, buttonExternalCell, className, outerClassName, isDisabled, titleElement, hint,
                                     popup)
{
    this.parentClass(buttonTable, buttonExternalCell, className, outerClassName, isDisabled, titleElement, hint)
    
    this.popup = popup;

    this.onOkHandler = null;
    this.onCancelHandler = null;

    this.beforePopupShowHandler = null;
    this.afterPopupShowHandler = null;
}

/**
 * This event must be called for 'onClick' event.
 */
OkCancelPopupDropdownButton.prototype.click = function()
{
    if (this.isDisabled())
        return;

    if ( this.isClickProhibited && this.isClickProhibited()  )
        return;

    if (this.popup.isVisible())
    {
        this.popup.clickOk();

        if ( (this.popup.isVisible()) || (this.popup.getOwner() == this) )
            return;
    }

    this.press();

    var offsetX = 1 - 15 /* for arrow */;
    var offsetY = getObjectHeight( this.buttonExternalCell.id ) - 3 ;

    var me = this;

    this.popup.setOnOkClickHandler(function()
    {
        if ( ((me.onOkHandler) && (me.onOkHandler())) || (!me.onOkHandler) )
        {
            me.unpress();
            return true;
        }
        return false;
    } );
    this.popup.setOnCancelClickHandler(function()
    {
        if ( ((me.onCancelHandler) && (me.onCancelHandler())) || (!me.onCancelHandler) )
        {
            me.unpress();
            return true;
        }
        return false;
    } );
    this.popup.setBeforeShowHandler(this.beforePopupShowHandler);

    this.popup.setPopupPositionNeededHandler(function()
    {
        offsetY = getObjectHeight( me.buttonExternalCell.id ) - 3 ;
        me.popup.positionPopup(offsetX, offsetY, me.popup.popupWidth, null, false);
    });

    this.popup.setPositioningStrategy( new ElementRelativeOnTheRightEdgePositioning(this.getTableId(),
            this.popup.popupTableId) );

    this.popup.setOwner(this);

    this.popup.show(offsetX, offsetY, this.popup.popupWidth, null, false);

    if (this.afterPopupShowHandler)
        this.afterPopupShowHandler(this);
}

/**
 * Sets disable status for button, If it is disabled, close popup by 'cancel' button.
 * @param isDisabled true when button disabled else false.
 */
OkCancelPopupDropdownButton.prototype.setDisabled = function(isDisabled)
{
    if (isDisabled && this.pressed)
        this.popup.clickCancel();

    CellButton.prototype.setDisabled.call(this, isDisabled);    
}

/**
 * Sets handler for event after clicking 'ok' button in the popup
 * @param handler function(), not undefined, can be null. This function returns true if all is ok and we can close popup
 * or false in other cases.
 */
OkCancelPopupDropdownButton.prototype.setOnOkHandler = function(handler)
{
    this.onOkHandler = handler;
}

/**
 * Sets handler for event after clicking 'cancel' button in the popup
 * @param handler function(), not undefined, can be null. This function returns true if all is ok and we can close popup
 * or false in other cases.
 */
OkCancelPopupDropdownButton.prototype.setOnCancelHandler = function(handler)
{
    this.onCancelHandler = handler;
}

/**
 * Sets handler, executed when popup is shown.
 * @param handler function(), not undefined, can be null.
 */
OkCancelPopupDropdownButton.prototype.setAfterPopupShowHandler = function(handler)
{
    this.afterPopupShowHandler = handler;
}

/**
 * Sets handler, executed before popup is shown.
 * @param handler function(), not undefined, can be null.
 */
OkCancelPopupDropdownButton.prototype.setBeforePopupShowHandler = function(handler)
{
    this.beforePopupShowHandler = handler;
}

/**
 * Sets function, to be called in button click handler and checks if handler procedure can be performed.
 */
OkCancelPopupDropdownButton.prototype.setIsClickProhibited = function( fnc )
{
    this.isClickProhibited = fnc;
}

