function LeaveTypesSelectorUtil(){}

LeaveTypesSelectorUtil.prototype.init = function(leaveTypes, isPTOOn, isSickDaysOn, leavesSelection){
    this.leaveTypes = leaveTypes;

    this.allBalancesOff = !(isPTOOn || isSickDaysOn);

    var groups = [];
    this.sickLeaves = [];
    this.ptoLeaves = [];
    this.withoutBalanceLeaves = [];

    var selectedLeaves = {};
    if(leavesSelection){
        for(var i = 0; i < leavesSelection.length; i++) {
            selectedLeaves[leavesSelection[i]] = true;
        }
    }

    this.showInactive = false;

    for ( i = 0; i < leaveTypes.length; i++) {
        var leave  = leaveTypes[i];

        // split leaves by type
        if(isPTOOn && leave.balanceType == BalanceType.PTO){
            this.ptoLeaves.push(leave);
        } else if(isSickDaysOn && leave.balanceType == BalanceType.SICK) {
            this.sickLeaves.push(leave);
        } else {
            this.withoutBalanceLeaves.push(leave);
        }

        if ( selectedLeaves[leave.id] ) {
            leave.selected = true;
            if (!leave.active)
                this.showInactive = true;
        }
    }

    return this;
};

/**
 * Check if possible group leaves by balance type (or in group without balance(es))
 * if all balances is off in system than group by balance type is impossible
 * @returns {boolean}
 * @private
 */
LeaveTypesSelectorUtil.prototype._isPossibleToGroupLeaves = function(){
    return !this.allBalancesOff && (this.sickLeaves.length != 0 || this.ptoLeaves.length != 0 || this.withoutBalanceLeaves.length != 0);
};

/**
 * Return grouping leaves if possible or return all leaves as list
 * @param groupLabels
 * @returns {Array}
 */
LeaveTypesSelectorUtil.prototype.getDataForSelector = function(groupLabels) {

    if(this._isPossibleToGroupLeaves()) {
        var groups = [];
        groups.push({label: groupLabels.ptoLabel, children: this.ptoLeaves});
        groups.push({label: groupLabels.sickDaysLabel, children: this.sickLeaves});
        groups.push({label: groupLabels.withoutBalancesLabel, children: this.withoutBalanceLeaves});
        return groups;
    }
    return this.withoutBalanceLeaves;
};

/**
 * Set tree constructor for selector panel
 */
LeaveTypesSelectorUtil.prototype.setSelectorTreeConstructorToPanel = function(leavePanel){
    leavePanel.prototype.selectorTreeCtor =  this._isPossibleToGroupLeaves() ? AT.LeavesGroupingSelectorTree : AT.LeavesSelectorTree;
};

/**
 * Shows if there are archived leaves in selection
 * @returns {boolean}
 */
LeaveTypesSelectorUtil.prototype.isNeedToShowInactiveLeaves = function() {
    return this.showInactive;
};
