AT.GroupSelectorSwitcher = Ext.extend(Ext.Panel, {

    cls: "viewByPanel",

    init: function(userGroupsEnabled, timeZoneGroupsEnabled, groupByUserGroupCallback, groupByTimeZoneGroupCallback)
    {
        this.userGroupsEnabled = userGroupsEnabled;
        this.timeZoneGroupsEnabled = timeZoneGroupsEnabled;

        this.groupByUserGroupCallback = groupByUserGroupCallback;
        this.groupByTimeZoneGroupCallback = groupByTimeZoneGroupCallback;

        this.addComponents();

        if (!userGroupsEnabled)
            this.addClass("uzgDisabled");

        if (!timeZoneGroupsEnabled)
            this.addClass("tzgDisabled");
    },

    addComponents: function()
    {
        if (this.userGroupsEnabled && this.timeZoneGroupsEnabled)
        {
            this.addSortByUserGroupLink();
            this.addSortByTimeZoneGroupLink();
        }
        else if (this.userGroupsEnabled)
        {
            this.addTextPanel(messageResource.getEscapedMessage("user_selector.group_by.by_user_groups") + ":");
        }
        else if (this.timeZoneGroupsEnabled)
        {
            this.addTextPanel(messageResource.getEscapedMessage("user_selector.group_by.by_time_zone_groups") + ":");
        }
    },

    addSortByUserGroupLink: function()
    {
        this.addSortByLink(messageResource.getEscapedMessage("user_selector.group_by.by_user_groups"), "userGroupLabel", this.groupByUserGroupCallback);
    },

    addSortByTimeZoneGroupLink: function()
    {
        this.addSortByLink(this.getSortByTimeZoneGroupLinkText(), "timeZoneGroupLabel", this.groupByTimeZoneGroupCallback);
    },

    getSortByTimeZoneGroupLinkText: function()
    {
        return messageResource.getEscapedMessage("user_selector.group_by.by_time_zones");
    },

    addSortByLink: function(text, className, groupByCallback)
    {
        var sortByTimeZoneGroupLink = new AT.FlatButton({
                                                            text: text,
                                                            cls: "sortingLabel " + className
                                                        });
        sortByTimeZoneGroupLink.on("click", groupByCallback, this);
        this.add(sortByTimeZoneGroupLink);
    },

    addTextPanel: function(text)
    {
        this.add(new Ext.Panel(
                {
                    html: text,
                    cls: "sortingLabel textLabel"
                }
        ));
    },

    updateUI: function(groupType)
    {
        this.removeClass('groupedByUserGroup');
        this.removeClass('groupedByTimeZoneGroup');
        if (groupType)
        {
            if (groupType == GroupType.TIME_ZONE_GROUP)
            {
                this.addClass('groupedByTimeZoneGroup');
            }
            else
            {
                this.addClass('groupedByUserGroup');
            }
        }
    }

});

Ext.reg("at_group_selector_switcher", AT.GroupSelectorSwitcher);


AT.ExtendedGroupSelectorSwitcher = Ext.extend(AT.GroupSelectorSwitcher, {

    init: function(userGroupsEnabled, timeZoneGroupsEnabled, groupByUserCallback, groupByUserGroupCallback, groupByTimeZoneGroupCallback)
    {
        this.groupByUserCallback = groupByUserCallback;
        AT.ExtendedGroupSelectorSwitcher.superclass.init.call(this, userGroupsEnabled, timeZoneGroupsEnabled, groupByUserGroupCallback,
                                                              groupByTimeZoneGroupCallback);
    },

    addComponents: function()
    {
        if (this.userGroupsEnabled && this.timeZoneGroupsEnabled)
        {
            this.addSortByUserLink();
            this.addSeparator("nearByUsers");
            this.addSortByUserGroupLink();
            this.addSeparator("betweenGroups");
            this.addSortByTimeZoneGroupLink();
        }
        else if (this.userGroupsEnabled)
        {
            this.addSortByUserLink();
            this.addSortByUserGroupLink();
        }
        else if (this.timeZoneGroupsEnabled)
        {
            this.addSortByUserLink();
            this.addSortByTimeZoneGroupLink();
        }
        else
        {
            this.addTextPanel(messageResource.getMessage("user_selector.group_by.by_users") + ":");
        }
    },

    addSortByUserLink: function()
    {
        this.addSortByLink(messageResource.getMessage("user_selector.group_by.by_users"), "userLabel", this.groupByUserCallback);
    },

    addSeparator: function(additionalClass)
    {
        this.add(
                new Ext.BoxComponent({
                    autoEl: {
                        tag: 'div',
                        cls: 'vertical-separator ' + additionalClass
                    }
                })
        );
    },

    updateUI: function(groupType)
    {
        this.removeClass('groupedByUsers');
        if(!groupType)
        {
            this.addClass('groupedByUsers');
        }
        AT.ExtendedGroupSelectorSwitcher.superclass.updateUI.apply(this, arguments);
    },

    getSortByTimeZoneGroupLinkText: function()
    {
        if (!this.userGroupsEnabled)
            return messageResource.getEscapedMessage("user_selector.group_by.by_time_zone_groups");

        return AT.ExtendedGroupSelectorSwitcher.superclass.getSortByTimeZoneGroupLinkText.call(this);
    }

});

Ext.reg("at_extended_group_selector_switcher", AT.ExtendedGroupSelectorSwitcher);