Ext.namespace("AT");

AT.SharedUserButtonMenu = Ext.extend(Ext.menu.Menu,
{
    cls: "at-selector-menu",

    constructor: function(config)
    {
        this.panel = config.panel;
        if (config.additionalCls)
        {
            this.cls += " " + config.additionalCls;
        }
        this.shadow = false;
        AT.SharedUserButtonMenu.superclass.constructor.call(this, {items: [config.panel]});
        this.panel.on("close", this.hide, this);
    },

    show: function(el, pos, parentMenu)
    {
        // we need to render the panel before calling of this.panel.showCallback(), since it may
        // need to update its content.
        if(!this.el)
            this.render();

        if (this.panel.showCallback)
            this.panel.showCallback();

        AT.SharedUserButtonMenu.superclass.show.call(this, el, pos, parentMenu);

        var diff = this.el.getBottom() - (getBodyScrollTop() + document.body.clientHeight);
        if (diff > 0) {
            Ext.getBody().scroll("down", diff);
        }
    },

    getSelectorPanel: function()
    {
        return this.panel;
    },

    hide: function()
    {
        if (this.panel.processHide())
        {
            AT.SharedUserButtonMenu.superclass.hide.call(this);
        }
    },

    /*
    following functionality is for reusing one instance of the panel
    for multiple controls
     */
    currentParentId: null,
    states: new AtMap(),

    setStateFor: function(parent, state)
    {
        this.setCurrentParent(parent);
        this.panel.loadState(state);
    },

    switchState: function (parentId, noSwitchState)
    {
        if(!noSwitchState){
            if(this.currentParentId != null)
                this.states.put(this.currentParentId, this.panel.getState());
            if (!this.states.get(parentId))
                this.states.put(parentId, this.panel.createDefaultState(parentId));
        }
        this.currentParentId = parentId;
        if(!noSwitchState)
            this.panel.loadState(this.states.get(parentId));
    },

    setCurrentParent: function (parent, noSwitchState)
    {
        var parentId = parent.id;
        if (this.currentParentId != parentId)
            this.switchState(parentId, noSwitchState);
    },

    isCurrentParent: function(toCheck)
    {
        return this.currentParentId == toCheck.id;
    },

    makeHandlerForSpecificParent: function(parent, callback)
    {
        var self = this;
        return function()
        {
            if (self.isCurrentParent(parent))
                return callback.apply(this, arguments)
        }
    }
});