Ext.namespace("AT");

/**
 * Tree label for grouping leave types
 * Ext.js component is label
 *
 */
AT.LeaveGroupingItem = Ext.extend(AT.SelectorTreeNode,{

    initComponent : function(){
        AT.LeaveGroupingItem.superclass.initComponent.call(this);

    },
    getNodeBody : function(){
        return new Ext.form.Label(
            {
                hidden: false,
                cls: "leaveGroupLabel",
                html:
                "<table>" +
                    "<tbody>" +
                        "<tr>" +
                            "<td>" + this.label +"</td>" +
                            "<td class='lineTd'><div class='lineDiv'></div></td>" +
                        "</tr>" +
                    "</tbody>"
            }
        )
    }
});

/**
 * Tree checkbox with icon label
 * Ext.js component is checkbox
 */
AT.LeaveSelectorTreeCheckbox = Ext.extend(AT.SelectorTreeCheckbox, {

    getNodeBody: function() {
        var labelCls = this.labelCls;
        var title = this.title;
        return new Ext.form.Checkbox({
            cls: this.itemCls,
            boxLabel: this.getLeaveTypeImg(this.item) + Ext.util.Format.htmlEncode(this.label),
            checked: this.item[this.selectedProperty],
            afterRender: function() {
                Ext.form.Checkbox.prototype.afterRender.call(this);
                var label = Ext.get(this.el.dom.parentNode).child("label:first");
                if(title && title != "") label.dom.title = title;
                label.unselectable();
                label.addClass(labelCls);
            }
        });
    },

    getLeaveTypeImg: function(item) {
        return createImg(getLeaveTypeIconPath(item.iconId));
    }

});


/**
 * Selector tree for a list of leave types with icons without grouping
 */
AT.LeavesSelectorTree = Ext.extend(AT.SimpleListSelectorTree, {

    cls: "at-selector-tree",

    selectorTreeCheckboxCtor: AT.LeaveSelectorTreeCheckbox,

    createNodes: function() {
        var nodes = [];
        Ext.each(this.data, function(item) {
            var node = this._createLeaveNode(item);
            nodes.push(node);
            node.on("change", this.onChange, this);
        }, this);

        return nodes;
    },

    _createLeaveNode: function(item){
        return new this.selectorTreeCheckboxCtor({
            item: item,
            label: item.active ? this.getNameWithStatus(item) : this.getNameWithStatus(item) + " " + messageResource.getMessage('common.archived'),
            isActive: item.active
        });
    },

    selectAll : function(){
        this.items.each(function(item) {
            if (!item.hidden) {
                item.select();
            }
        });
        this.onChange();
    },

    setInitialSelectionFromCurrent: function(newData)
    {
        Ext.each(this.data, function(item) {
            this.propertiesController.setSelectedInit(item, this.propertiesController.isSelected(item));
            this.propertiesController.setHiddenInit(item, this.propertiesController.isHidden(item));
        }, this);
    },

    isSelectionChanged: function() {
        var selectionChanged = false;
        //If function inside cascade returns false, it immediately stop.
        this.cascade(function(node) {
            if (node.item) {
                var isSelectionChanged = this.propertiesController.isSelectedInit(node.item) != this.propertiesController.isSelected(node.item);
                var isVisibilityChanged =  this.propertiesController.isHiddenInit(node.item) != this.propertiesController.isHidden(node.item);
                if (isSelectionChanged || isVisibilityChanged) {
                    selectionChanged = true;
                    return false;
                }
            }
        }, this);
        return selectionChanged;
    }
});


/**
 * Selector tree for leave types with grouping by balances
 */
AT.LeavesGroupingSelectorTree = Ext.extend(AT.LeavesSelectorTree, {

    cls: "at-selector-tree groupingSelectorTree",

    groupingItemCtor: AT.LeaveGroupingItem,

    createNodes: function(){
        var groupNodes = [];
        Ext.each(this.data, function(item) {
            var groupNode;
            if(item.children.length != 0){
                var children = [];

                Ext.each(item.children, function(child){
                    var childNode = this._createLeaveNode(child);
                    childNode.on("change", this.onChange, this);
                    children.push(childNode);
                }, this);


                groupNode = new this.groupingItemCtor( {item : item, label: item.label, children: children});

                groupNodes.push(groupNode);
            }
        }, this);
        return groupNodes;

    },

    filter: function(condition, scope) {
        if(typeof this.items != 'undefined'){
            this.items.each(function(group) {
                Ext.each(group.children, function(item){
                    item.filter(condition, scope);
                })
            });
        }
        this.onChange();
    },

    getSelection: function(selectHidden) {
        var selection = [];
        Ext.each(this.data, function(group){
            Ext.each(group.children, function(entry) {
                if (this.propertiesController.isSelected(entry) && (selectHidden || !this.propertiesController.isHidden(entry))) {
                    selection.push(entry);
                }
            }, this);
        }, this);

        return selection;
    },

    selectAll : function(){
        this.items.each(function(group) {
            Ext.each(group.children, function(child){
                if (!child.hidden) {
                    child.select(true, false, false);
                }
            })

        });
        this.onChange();
    },

    setInitialSelectionFromCurrent: function(newData)
    {
        Ext.each(this.data, function(group) {
            Ext.each(group.children, function(item){
                this.propertiesController.setSelectedInit(item, this.propertiesController.isSelected(item));
                this.propertiesController.setHiddenInit(item, this.propertiesController.isHidden(item));
            }, this);
        }, this);
    }
});

AT.BaseLeavesSelectorPanel = Ext.extend(AT.SimpleSelectorPanel, {

    cls: "selector-panel leave-types-selector-panel",

    selectorTreeCtor: AT.LeavesSelectorTree, // by default leaves without grouping

    createComponents: function()
    {
        AT.BaseLeavesSelectorPanel.superclass.createComponents.call(this);

        if (this.showShowInactiveCheckbox)
        {
            var anyInactive = _.any(LeaveTypeStore.getLeaveTypesOrdered(), function (lt) {return !lt.active});
            if (!anyInactive)
                this.showShowInactiveCheckbox = false;
        }
    }
});

Ext.reg("base_leave_types_selector_panel", AT.BaseLeavesSelectorPanel);