DeadlineEditor = function($containerEl, editable)
{
    TemplateView.call(this, $containerEl, editable);
    this.editable = editable;
};

DeadlineEditor.prototype = _.create(TemplateView.prototype, {
    html: "<div class='deadlineEditor'>" +
          // No EOL because we need remove whitespace in HTML code (inline-block)
          "<span class='value'></span><span class='na'>N/A</span><span class='icon'>&nbsp;</span>" +
          "</div>",

    updateOnInit: false,

    afterRender: function()
    {
        this.deadline = null;
        this.disabled = false;
        this.$value = this.find(".value");
        this.$el.toggleClass('editable', this.editable);
        this.deadlinePickerMenu = this.getDataPickerMenu();
        this.closeTime = this._currentTime();
        if (this.deadlinePickerMenu)
        {
            this._subscribeClickEvent();
        }
    },

    update: function()
    {
        this.$value.text(this.deadline ? this.formatDate() : "");
        this.$el.toggleClass("noDeadline", !this.deadline);
        this.$el.toggleClass("deadlinePassed", !!this.deadline && currentUserTime().clearTime().getTime() > this.deadline.getTime());
        this.$el.toggleClass("disabled", this.disabled);
        this.$el.toggleClass('editable', this.editable);
    },

    formatDate: function() {
        return this.deadline.format(this.dateFormat)
    },

    getDeadline: function()
    {
        return this.deadline;
    },

    setDeadline: function(deadline)
    {
        this.deadline = deadline;
        this.update();
    },

    setDateFormat: function(dateFormat)
    {
        this.dateFormat = dateFormat;
    },

    setEditable: function(editable)
    {
        this.editable = editable;

    },

    isDisabled: function()
    {
        return this.disabled;
    },

    setDisabled: function(disabled)
    {
        this.disabled = disabled;
        this.update();
    },
    
    getDataPickerMenu: function()
    {
        return this.deadlinePickerMenu;
    },

    setDataPickerMenu: function(menu)
    {
        this.deadlinePickerMenu = menu;
        this._subscribeClickEvent();
    },

    _subscribeClickEvent: function()
    {
        var self = this;
        this.$el.click(function()
                       {
                           if (!self.editable || self.disabled)
                               return;

                           if (self.closeTime + 300 > self._currentTime())
                               return;

                           self.$el.addClass("pressed");
                           self.deadlinePickerMenu.setValue(self.deadline || new Date());
                           self.deadlinePickerMenu.showAt(self.$el,
                                                          function(date)
                                                          {
                                                              self.$el.removeClass("pressed");
                                                              self.closeTime = self._currentTime();
                                                              if (self.deadline && date && self.deadline.getTime() !== date.getTime() ||
                                                                  !!self.deadline !== !!date)
                                                              {
                                                                  self.deadline = date;
                                                                  self.update();
                                                                  $(self).trigger("change", date);
                                                              }
                                                          },
                                                          function()
                                                          {
                                                              self.$el.removeClass("pressed");
                                                              self.closeTime = self._currentTime();
                                                          });
                       });

    },

    _currentTime: function()
    {
        var date = new Date();
        return date.getTime();
    }
});

_.extend(DeadlineEditor.prototype, LoadingControlMixin);