TaskManagement.EditActionMenu = function($containerEl, $button)
{
    DropdownTemplateView.call(this, $containerEl);
    this.needCopy = false;
    this.needMove = false;
    this.needDelete = false;
    this.deleteDisable = false;
    this.$button = $button;
};

TaskManagement.EditActionMenu.prototype = _.create(DropdownTemplateView.prototype, {
    html: "<div class='dropdownContainer actionsMenu'>" +
          "  <div class='buttonsWrapper'>" +
          "    <div class='copyButton'>" +
          "      <div class='title'><%=copyTitle%></div>" +
          "    </div>" +
          "    <div class='moveButton'>" +
          "      <div class='title'><%=moveTitle%></div>" +
          "    </div>" +
          "    <div class='deleteButton'>" +
          "      <div class='title'><%=deleteTitle%></div>" +
          "    </div>" +
          "  </div>" +
          "</div>",

    afterRender: function()
    {
        var self = this;
        this.$wrapper = this.find(".buttonsWrapper");
        this.$copy = this.find(".copyButton");
        this.$move = this.find(".moveButton");
        this.$delete = this.find(".deleteButton");
        this.$copy.hide();
        this.$move.hide();
        this.$delete.hide();

        this.findAndBindClick(".moveButton", function() { self.triggerAction("move"); });
        this.findAndBindClick(".copyButton", function() { self.triggerAction("copy"); });
        this.findAndBindClick(".deleteButton", function()
                                                {
                                                    if (!self.deleteDisable)
                                                        self.triggerAction("delete");
                                                });

        this.disableTooltip = TaskManagement.EditActionButton.DisableTooltip.getInstance();
        this.disableTooltip.subscribeToMouseEvents(this.$delete);
        this.disableTooltip.disable();
    },

    update: function()
    {
        var widthWrapper = 0;
        if (this.needCopy)
        {
            widthWrapper += 112;
            this.$copy.show();
        }
        if (this.needMove)
        {
            widthWrapper += 112;
            this.$move.show();
        }
        if (this.needDelete)
        {
            widthWrapper += 112;
            this.$delete.show();
        }
        this.$wrapper.width(widthWrapper);
    },

    triggerAction: function(eventName)
    {
        $(this).trigger(eventName);
        this.hide();
    },

    selectMove: function(isSelected)
    {
        this.$copy.toggleClass("selected", false);
        this.$delete.toggleClass("selected", false);

        this.$move.toggleClass("selected", isSelected);
    },

    selectDelete: function(isSelected)
    {

        this.$move.toggleClass("selected", false);
        this.$copy.toggleClass("selected", false);

        this.$delete.toggleClass("selected", isSelected);
    },

    selectCopy: function(isSelected)
    {
        this.$move.toggleClass("selected", false);
        this.$delete.toggleClass("selected", false);

        this.$copy.toggleClass("selected", isSelected);
    },

    showAt: function($button, $header, positionParams, callback)
    {
        DropdownTemplateView.prototype.showAt.call(this, $button, positionParams, $header);
        this.update();
        this.callback = callback;
    },

    messageKeys: function()
    {
        return {
            moveTitle: "edit_panel.actions.move",
            copyTitle: "edit_panel.actions.copy",
            deleteTitle: "edit_panel.actions.delete"
        }
    },

    setMove: function(needMove)
    {
        this.needMove = needMove;
    },

    setCopy: function(needCopy)
    {
        this.needCopy = needCopy;
    },

    setDelete: function(needDelete)
    {
        this.needDelete = needDelete;
    },

    onHide: function()
    {
        DropdownTemplateView.prototype.onHide.call(this);
        this.callback(this.active);
    },

    setDisableDeleteButton: function(text)
    {
        this.deleteDisable = true;

        this.disableTooltip.$el.text(text);
        this.disableTooltip.enable();

        this.$delete.addClass("disable");
    },

    setEnableDeleteButton: function()
    {
        this.deleteDisable = false;

        this.disableTooltip.disable();

        this.$delete.removeClass("disable");
    }
});

TaskManagement.EditActionButton.DisableTooltip = function($containerEl) {
    Tooltip.call(this, $containerEl);
};

TaskManagement.EditActionButton.DisableTooltip.prototype = _.create(Tooltip.prototype, {
    html: "<div class='disableDeleteTooltip yellowTooltip tooltipTex noprint'/>"
});

TaskManagement.EditActionButton.DisableTooltip.getInstance = function()
{
    if (!this._instance)
        this._instance = new this($("body")).init();
    return this._instance;
};
