TypeOfWorkMenu = function($containerEl)
{
    TemplateView.call(this, $containerEl);
};

TypeOfWorkMenu.BILLING_TYPE_NOT_SELECTED_VALUE = -1;

TypeOfWorkMenu.getInstance = function()
{
    if (!TypeOfWorkMenu.instance)
        TypeOfWorkMenu.instance = new TypeOfWorkMenu($("body")).init();
    return TypeOfWorkMenu.instance;
};

TypeOfWorkMenu.updateInstance = function()
{
    if (TypeOfWorkMenu.instance)
    {
        TypeOfWorkMenu.instance.$el.remove();
    }
    TypeOfWorkMenu.instance = new TypeOfWorkMenu($("body")).init();
};

TypeOfWorkMenu.prototype = _.create(DropdownTemplateView.prototype, {
    html: "<div class='dropdownContainer typeOfWorkMenu'>" +
          "</div>",

    updateOnInit: false,

    afterRender: function()
    {
        var self = this;
        this.typeOfWorkId = TypeOfWorkMenu.BILLING_TYPE_NOT_SELECTED_VALUE;
        this.scrollView = new ScrollView(this.$el, 22).init();
        this.updateTypesOfWork();
    },

    updateTypesOfWork: function()
    {
        var self = this;
        this.typeRows = [];
        this.rowCount = 0;

        function clickHandler(e, id)
        {
            stopEventPropagation(e);
            self._itemClick(id);
        }

        _.each(BillingTypeStore.getAll(), function(type)
        {
            var index = self.rowCount++;
            self.typeRows[index] = new TypeOfWorkMenu.TypeRow(type, BillingTypeStore.getDefaultBillingTypeId());
            self.typeRows[index].on("click", clickHandler);
            self.typeRows[index].toggleSelection(type.id === self.typeOfWorkId);
        });

        this.scrollView.setItems(this.typeRows);
    },

    _itemClick: function(typeId)
    {
        this.typeOfWorkId = typeId;
        this.hide();
    },

    setTypeOfWorkId: function(id)
    {
        this.typeOfWorkId = id;
    },

    update: function()
    {
        var typeOfWorkId = this.typeOfWorkId;
        _.forEach(this.typeRows, function(row)
        {
            row.toggleSelection(row.id === typeOfWorkId);
        });
    },

    showAt: function($element, positionParams, callback)
    {
        DropdownTemplateView.prototype.showAt.call(this, $element, positionParams);
        this.update();
        this.scrollView.refreshScroll();
        this.callback = callback;
    },

    onHide: function()
    {
        DropdownTemplateView.prototype.onHide.call(this);
        this.callback(this.typeOfWorkId);
    }
});


TypeOfWorkMenu.TypeRow = function(typeOfWork, defaultBillingTypeId)
{
    TemplateView.call(this);
    this.billable = typeOfWork.billable;
    this._default = typeOfWork.id === defaultBillingTypeId;
    this.name = typeOfWork.name;
    this.selected = false;
    this.id = typeOfWork.id;
};

TypeOfWorkMenu.TypeRow.prototype = _.create(TemplateView.prototype, {
    templateKey: "TypeOfWorkMenu.TypeRow",

    html: "<div class='item <% if (this._default) { %> defaultType <%}%>'>" +
          "    <div class='typeLabel ellipsis'/>" +
          "</div>",

    afterRender: function()
    {
        if (this.id === BillingTypeCommon.NON_BILLABLE_DEFAULT_TYPE_ID)
            this.find('.typeLabel').text(this.name);
        else
            this.find('.typeLabel').text(messageResource.getMessage((this.billable) ?
                                                                    'billing_type.billable' : 'billing_type.non_billable') + ': ' + this.name);

        var self = this;
        this.$el.click(function() { $(self).trigger("click", self.id) })
    },

    update: function()
    {
        if (this.rendered)
            this.$el.toggleClass('selected', this.selected);
    },

    toggleSelection: function(selected)
    {
        this.selected = selected;
        this.update();
    }
});

