function TreeSelectorDialog() {
    this.constructor = TreeSelectorDialog;
}

TreeSelectorDialog.prototype.init = function ( config ) {
    var self = this;

    this._spinnerController = new SpinnerController();

    this.selector = null;

    this._callBacks = {};
    this._callBacks.abortDataLoading = config.abortDataLoading;
    this._callBacks.loadData = config.loadData;
    this._callBacks.saveData = config.saveData;

    this._panelConstructor = config.panelConstructor;
    this._configCtr = config.configCtr;
    this._dialogId = config.dialogId;

    this._dialogDiv = $( "#" + this._dialogId );
    this._treeContainer = $( "#" + this._dialogId + "_mainSection" );

    this._submitButton = $( "#" + this._dialogId + "_submitButton" );
    this._submitButton.click( this._self( this._save ) );

    this._cancelButton = $( "#" + this._dialogId + "_cancelButton" );
    this._cancelButton.click( this._self( this.close ) );

    this._resetSelectionLink = $( "#" + this._dialogId + "_resetSelectionLink" );

    this._dialog = this._dialogDiv.dialog( {
                                               autoOpen: false,
                                               draggable: false,
                                               resizable: false,
                                               modal: true, // this is needed for overlay to appear, so we can handle 'outside' clicks to close the dialog
                                               close: function ()
                                               {
                                                   if( self._onClose != null )
                                                   {
                                                       self._onClose();
                                                       self._onClose = null;
                                                   }
                                                   self._callBacks.abortDataLoading();
                                               },
                                               width: config.dialogWidth ? config.dialogWidth : 420,
                                               minHeight: config.dialogMinHeight ? config.dialogMinHeight : 348,
                                               dialogClass: this._dialogId + ( config.dialogClass ? " " + config.dialogClass : "" )
                                           } );

    this.showInactiveState = false;

    return this;
};

TreeSelectorDialog.prototype._initPanel = function ( data )
{
    // abstract
};

TreeSelectorDialog.prototype._self = function ( func )
{
    var self = this;
    return function ()
    {
        return func.apply( self, arguments );
    }
};

TreeSelectorDialog.prototype._dataLoadedCallback = function ( data ) {
    this._initPanel( data );
    this._switchDialogInReadyState();
};

TreeSelectorDialog.prototype._loadingFailedCallback = function ()
{
    this._close();
};

TreeSelectorDialog.prototype._loadingState = { save: function(){}, close: function( dialog ){ dialog._close(); } };
TreeSelectorDialog.prototype._savingState = { save: function(){}, close: function(){} };
TreeSelectorDialog.prototype._readyState = {
    save: function( dialog ){
        dialog._readyStateSave();
    },
    close: function( dialog ){ dialog._close(); }
};

TreeSelectorDialog.prototype._readyStateSave = function () {
    // abstract
};

TreeSelectorDialog.prototype._switchDialogInSavingState = function ()
{
    this._state = TreeSelectorDialog.prototype._savingState;
    this._dialogDiv.removeClass( 'loadingState minimizedMode' ).addClass( 'savingState' );
    this._dialog.off("dialogbeforeclose");
    this._dialog.on("dialogbeforeclose", false);
    this._submitButton.addClass( 'disabled' );
    this._cancelButton.addClass( 'disabled' );
};

TreeSelectorDialog.prototype._switchDialogInLoadingState = function (minimizedMode)
{
    this._state = TreeSelectorDialog.prototype._loadingState;
    this._dialogDiv.removeClass( 'savingState' ).addClass( 'loadingState' );
    if (minimizedMode) this._dialogDiv.addClass('minimizedMode');
    this._dialog.off("dialogbeforeclose");
    this._dialog.on("dialogbeforeclose", this._self(this._isCanClose));
    this._submitButton.addClass( 'disabled' );
    this._cancelButton.removeClass( 'disabled' );
    this._spinnerController.spin( this._dialogId + "_spinnerPlaceholder" );

    //it is needed because while data was saving and close was inactive, window may be resized
    this._initCloseEvents();
};

TreeSelectorDialog.prototype._switchDialogInReadyState = function ()
{
    this._state = TreeSelectorDialog.prototype._readyState;
    this._dialogDiv.removeClass( 'savingState loadingState minimizedMode' );
    this._dialog.off("dialogbeforeclose");
    this._dialog.on("dialogbeforeclose", this._self(this._isCanClose));
    this._submitButton.removeClass( 'disabled' );
    this._cancelButton.removeClass( 'disabled' );
    this._spinnerController.stop();

    //it is needed because while data was saving and close was inactive, window may be resized
    this._initCloseEvents();
};

TreeSelectorDialog.prototype.setOnCloseCallback = function ( callback )
{
    this._onClose = callback;
};

TreeSelectorDialog.prototype.showAt = function ( dialogInfo )
{
    this._switchDialogInLoadingState(dialogInfo.minimizedMode);
    this._open(dialogInfo);
    this._callBacks.loadData( dialogInfo, this._self( this._dataLoadedCallback ), this._self( this._loadingFailedCallback ) );
};

TreeSelectorDialog.prototype.updatePosition = function( dialogInfo )
{
    this._dialogInfo = dialogInfo;
    // when the dialog is created, it is wrapped into a container, that's why we use parent()
    this._dialog.parent().position( {
                                        of: $( this._dialogInfo.element ),
                                        my: "left top",
                                        at: "left bottom",
                                        offset: this._dialogInfo.offset,
                                        collision: "dropdown"
                                    } );
};

TreeSelectorDialog.prototype._open = function (dialogInfo)
{
    this._dialog.dialog( "open" );
    this.updatePosition(dialogInfo);
    this._initCloseEvents();
};

TreeSelectorDialog.prototype._save = function () {
    this._state.save(this);
};

TreeSelectorDialog.prototype.hideAfterSaving = function () {
    this._switchDialogInReadyState();
    this.close();
};

TreeSelectorDialog.prototype.getShowInactive = function () {
    //abstract
};

TreeSelectorDialog.prototype.getShowInactiveState = function () {
    return this.showInactiveState;
};

TreeSelectorDialog.prototype._isCanClose = function()
{
    if( this.selector )
    {
        this.showInactiveState = this.getShowInactive();
        if( this.selector.processHide ) return this.selector.processHide();
    }
    return true;
};

TreeSelectorDialog.prototype._close = function () {
    this._dialog.dialog( 'close' );
};

TreeSelectorDialog.prototype.close = function () {
    this._state.close(this);
};

TreeSelectorDialog.prototype._initCloseEvents = function(){
    // close the dialog when user clicks outside
    doOnceOn( $( '.ui-widget-overlay' ), 'click', this._self( this.close ) );
    doOnceOnWindowResize( this._self( this.close ) );
};
