function UsersForApproverDialog() {
    this.constructor = UsersForApproverDialog;
}

UsersForApproverDialog.prototype = new TreeSelectorDialog();

UsersForApproverDialog.prototype.init = function ( config ) {

    this._userGroupsEnabled = config.userGroupsEnabled;
    this._timeZoneGroupsEnabled = config.timeZoneGroupsEnabled;

    config.dialogClass = this._userGroupsEnabled || this._timeZoneGroupsEnabled ? "groupsEnabled" : "groupsDisabled";
    config.dialogClass += " hideCustomOptionField";

    TreeSelectorDialog.prototype.init.call( this, config );

    return this;
};

UsersForApproverDialog.prototype.getShowInactive = function () {
    return this.selector.getShowInactive();
};

UsersForApproverDialog.prototype._initPanel = function ( assignedUsersData )
{
    var users = this._convertToUsersArray(assignedUsersData.users);
    var userGroups = this._userGroupsEnabled ? this._convertToGroupsArray(assignedUsersData.userGroups, true, SystemSettings.usersWithoutGroupId) : null;
    var timeZoneGroups = this._timeZoneGroupsEnabled ? this._convertToGroupsArray(assignedUsersData.timeZoneGroups, false, DEFAULT_TIME_ZONE_GROUP_ID) : null;

    this._treeContainer.find(".selector-panel").remove();

    var param = {};
    param = this._addSpecificInitialPanelParams( param, assignedUsersData );
    this.selector = new this._panelConstructor( param );
    this.selector.initialize(this._userGroupsEnabled, this._timeZoneGroupsEnabled, users, userGroups, timeZoneGroups,
                             {
                                 assignedUserIds: assignedUsersData.assignedUserIds,
                                 canManageAllUsers: assignedUsersData.canManageAllUsers,
                                 showInactive: this.getShowInactiveState()
                             });
    this.loadStateToPanel( {    assignedUserIds: assignedUsersData.assignedUserIds,
                                canManageAllUsers: assignedUsersData.canManageAllUsers,
                                showInactive: this.getShowInactiveState() }
    );

    var self = this;
    this._resetSelectionLink.click( function () {
        self.selector.beforeResetAction();
        self.selector.resetState(true);
        self.selector.afterResetAction();
    } );


    this.selector.renderOnPage( this._treeContainer[0] );
};

UsersForApproverDialog.prototype.loadStateToPanel = function ( conf )
{
    this.selector.loadState( {
                                 selection: new TwoLevelSelection([], conf.assignedUserIds),
                                 optionValue: conf.canManageAllUsers,
                                 showInactive: conf.showInactive
                             } );
};

UsersForApproverDialog.prototype._readyStateSave = function () {
    var selectedUserIds = this.selector.getSelectedUserIds();
    this._switchDialogInSavingState();
    this._callBacks.saveData(this._dialogInfo, this.selector.getCanManageAllUsers(), selectedUserIds);
};

UsersForApproverDialog.prototype._getIsActiveInProduct = function ( user ) {
    // abstract
};

UsersForApproverDialog.prototype._getIsAutoApprovedInProduct = function ( user ) {
    // abstract
};

UsersForApproverDialog.prototype._convertToUsersArray = function ( allUsers ) {
    var self = this;
    var users = [];
    $.each( allUsers, function() {
        var user = new User(this.id,
                            this.firstName,
                            this.lastName,
                            this.middleName,
                            this.username,
                            self._getIsActiveInProduct(this),
                            undefined,
                            this.userGroupId,
                            this.timeZoneGroupId);
        user.setRequiresTTApproval(!self._getIsAutoApprovedInProduct(this));
        users.push( user );
    });
    return users;
};

UsersForApproverDialog.prototype._usersWithoutGroupUserGroup = new Group( SystemSettings.usersWithoutGroupId, messageResource.getMessage('common.users_without_group'));

UsersForApproverDialog.prototype._convertToGroupsArray = function ( allGroups, includeNullGroup, defaultGroupId ) {
    if ( !allGroups ) return [];

    var groups = [];
    if (includeNullGroup)
        groups.push(this._usersWithoutGroupUserGroup);
    $.each( allGroups, function() {
        groups.push( new Group( this.id, this.name) );
    });
    return groups;
};

UsersForApproverDialog.prototype._dataLoadedNoSelectionCallback = function(data)
{
    data.assignedUserIds = [];
    this._dataLoadedCallback(data);
};

UsersForApproverDialog.prototype.refreshData = function(dialogInfo, withoutSelection) {
    this.updatePosition(dialogInfo);
    this._callBacks.loadData(dialogInfo,
                             this._self( withoutSelection ?
                                         this._dataLoadedNoSelectionCallback :
                                         this._dataLoadedCallback
                             ),
                             this._self( this._loadingFailedCallback ));
};

UsersForApproverDialog.prototype._addSpecificInitialPanelParams = function( params)
{
     var self = this;
    params.onAllOptionSelected = function(){ self._dialogDiv.addClass("hideCustomOptionField")};
    params.onCustomOptionSelected = function(){self._dialogDiv.removeClass("hideCustomOptionField")};
    return params;
};