function CPSelectorTreeBuilder(rootNode, projects, customersMap, customersOrder, createEmptyCustomers, observer)
{
    TreeBuilder.call(this, rootNode, projects, customersMap, customersOrder);
    this.createEmptyCustomers = createEmptyCustomers;
    this.observer = observer;
}

CPSelectorTreeBuilder.CUSTOMER_NODE_LEVEL = 1;
CPSelectorTreeBuilder.PROJECT_NODE_LEVEL = 2;

CPSelectorTreeBuilder.prototype = _.create(TreeBuilder.prototype, {
    createEmptyFirstLevelEntityNodes: function()
    {
        return this.createEmptyCustomers;
    },

    createTreeNodeForFirstLevelEntity: function(customer)
    {
        return new CustomerTreeNode(customer);
    },

    createTreeNodeForSecondLevelEntity: function(project)
    {
        return new ProjectTreeNode(project);
    },

    createDisabledNodesParentForFirstLevelEntity: function()
    {
        return new ArchivedCustomersParentTreeNode();
    },

    createDisabledNodesParentForSecondLevelEntity: function(parentId)
    {
        return new ArchivedProjectsParentTreeNode(parentId);
    },

    getSecondLevelEntityParentId: function(project)
    {
        return project.customerId;
    },

    isFirstLevelEntityEnabled: function(customer)
    {
        return !customer.archived;
    },

    isSecondLevelEntityEnabled: function(project)
    {
        return !project.archived;
    },

    onTreeNodeForFirstLevelEntityAdded: function(treeNode)
    {
        this.observer.customerNodeAdded(treeNode);
    },

    onTreeNodeForSecondLevelEntityAdded: function(treeNode)
    {
        this.observer.projectNodeAdded(treeNode);
    },

    onDisabledNodesParentForFirstLevelEntitiesAdded: function(treeNode)
    {
        this.observer.archivedCustomersParentNodeAdded(treeNode);
    },

    onDisabledNodesParentForSecondLevelEntitiesAdded: function(treeNode)
    {
        this.observer.archivedProjectsParentNodeAdded(treeNode);
    }
});