function CPReportSelector(panelConfig)
{
    this.panelConfig = _.defaults(panelConfig || {}, {
        isCustomersProjectsFiltered: true,
        allActiveItemsOptionLabelKeyFiltered: "projects_customers_selector.report.all_active_projects_customers_filtered",
        allItemsOptionLabelKeyFiltered: "projects_customers_selector.report.all_projects_customers_filtered"
    });
}

CPReportSelector.prototype.constructor = CPReportSelector;
/**
 * Init selector with report config where placed cpSelectorData (selected customer, projects, qualifier)
 * Create button and disable it while CPReportSelector.initializeData not performed
 * This code is modification of CPSelectorServerUtil and CPSelectorClientUtil classes
 * @returns {*}
 */
CPReportSelector.prototype.init = function( customersProjectsSelection, cpSelectorId, changedCallback, withApplyButton ) {

    this.cpSelection = customersProjectsSelection;
    this.predefinedState = this._createPredefinedStateFromSelection(customersProjectsSelection);

    // init panel
    var panelConfig = Ext.apply(this.panelConfig, {});
    if ( withApplyButton )
        Ext.apply( panelConfig, SelectorPanelWithApplyButtonHelper.getApplyButtonExtension( AT.CustomerProjectSelectorTwoGenericOptionsPanel ) );
    this.panel = new AT.CustomerProjectSelectorTwoGenericOptionsPanel( panelConfig );

    this.menu = new AT.SharedUserButtonMenu(
        {
            panel : this.panel
        });
    // set handler for get predefined state
    var self = this;
    this.panel.setGetPredefinedSelectionCallback(function() {
        return self.predefinedState;
    });

    // override process without use ajax call ( some code copied from AT.HierarchicalSelectorPanel.processFirstPanelOpen)
    this.panel.processFirstPanelOpen = function() {

        this.treePanel = document.createElement("div");
        var treePanelClasses = this.getTreePanelClasses();
        treePanelClasses.push("spinnerPanel"); // spin while panel is not loaded
        this.treePanel.className = treePanelClasses.join(" ");
        var targetEl = this.detailsContainer.el.dom;
        var firstChild = targetEl.childNodes[0];
        if (firstChild) {
            targetEl.insertBefore(this.treePanel, firstChild);
        } else {
            targetEl.appendChild(this.treePanel);
        }
        this.initializeSelector(self.preloadedData);
    };
    this.wasChanged = false;
    var buttonConfig = {
        renderTo: cpSelectorId,
        labelCls: "hierarchySelectorButton",
        menu: this.menu,
        tooltipObject: new DhtmlTitle("form-field-tooltip")
    };
    // create drop down button
    this.button = this._createDropDownButton(buttonConfig);
    this._initDropDownButton(changedCallback);

    return this;
};

CPReportSelector.prototype._createDropDownButton = function(buttonConfig)
{
    return new AT.HierarchySelectorDropdownButton(buttonConfig);
};

CPReportSelector.prototype._initDropDownButton = function(changedCallback)
{
    // show message while data download
    this.button.setText(messageResource.getMessage('saved_reports.cp_selector.data_loading'));
    this.button.setTooltipEnabled(false);
    // of button while data not loaded
    this.button.setEnabled(false);
    var self = this;
    this.button.setMenuHideCallback(function(){
        if (changedCallback)
        {
            self.wasChanged = false;
            changedCallback();
        }
    });
};

CPReportSelector.prototype._createPredefinedStateFromSelection = function( cpSelection ){
     return {
        selection: cpSelection.qualifier == CustomersProjectsSelectionQualifier.SELECTED ?
            new TwoLevelSelection(cpSelection.customers, cpSelection.projects) : TwoLevelSelection.emptySelection(),
        optionValue: cpSelection.qualifier == CustomersProjectsSelectionQualifier.SELECTED ? CoarseSelection.SPECIFIC:
            cpSelection.qualifier == CustomersProjectsSelectionQualifier.ALL ? CoarseSelection.ALL : CoarseSelection.ALL_ACTIVE,
        showInactive: cpSelection.showArchivedProjects
    };
};

CPReportSelector.prototype.setWasChanged = function(){
    this.wasChanged = true;
};
/**
 * Use this method while data load from server is performed
 * Activate button and set correct label
 * @param customersProjects CPList from server
 */
CPReportSelector.prototype.initializeData = function( customersProjects ) {
    this.preloadedData = customersProjects;
    // remove selection c&p that were deleted
    this._removeDeletedCPFromSelection(customersProjects, this.cpSelection);
    // set text when data is loaded
    this.button.setText(this._makePredefinedSelection(customersProjects));
    this.button.setEnabled(true);
};

CPReportSelector.prototype._removeDeletedCPFromSelection = function(cpData, cpSelection) {
    var selectedCustomers = cpSelection.customers;
    var availableCustomersMap = {};
    for (var i = 0; i < cpData.customers.length; i++ ){
        availableCustomersMap[cpData.customers[i].id] = true;
    }

    var selectedProjects = cpSelection.projects;
    var availableProjectsMap = {};
    for (i = 0; i < cpData.projects.length; i++ ){
        availableProjectsMap[cpData.projects[i].id] = true;
    }

    // remove not exists selection
    for(i = selectedProjects.length - 1; i >= 0; i--) {
        if (!availableProjectsMap[selectedProjects[i]]){
            selectedProjects.splice(i ,1);
        }
    }
    for(i = selectedCustomers.length; i >= 0; i--) {
        if (!availableCustomersMap[selectedCustomers[i]]){
            selectedCustomers.splice(i,1);
        }
    }
};

/**
 * Create string for text in button
 * @returns {*} text of label
 * @private
 */
CPReportSelector.prototype._makePredefinedSelection = function(customersProjects) {
    var state = this.predefinedState;
    if (state.optionValue == CoarseSelection.SPECIFIC) {
        //slice(0) is a workaround in purpose of array cloning
        var selectedCustomers = state.selection.getFirstLevel().slice(0);
        var selectedProjects = state.selection.getSecondLevel().slice(0);
        if (selectedCustomers.length + selectedProjects.length == 1) {
            if (selectedCustomers.length == 1) {
                // get data of selected customer
                selectedCustomers[0] = ($.map(customersProjects.customers, function( n ) {
                    if (n.id == selectedCustomers[0]) {
                        return n;
                    }
                }))[0];
            } else {
                // get data of selected prj
                var prj = selectedProjects[0] = ($.map(customersProjects.projects, function( n ) {
                    if (n.id == selectedProjects[0]) {
                        return n;
                    }
                }))[0];
            }
        }
        return this.panel.getSelectionHelper().getDescription(new TwoLevelSelection(selectedCustomers, selectedProjects), undefined, state.showInactive);
    } else if (state.optionValue == CoarseSelection.ALL) {
        return this.panel.getAllSelectedMessage();
    } else {
        return this.panel.getAllActiveItemsMessage();
    }
};

/**
 * Return false if there no one selection or true otherwise
 * @returns {boolean}
 */
CPReportSelector.prototype.checkSelected = function () {
    var isAllPredefined = this.predefinedState.optionValue == CoarseSelection.ALL_ACTIVE || this.predefinedState.optionValue == CoarseSelection.ALL;
    return !((this.panel.isAlreadyShown() && this.panel.isCustomOptionSelected() && this.panel.isEmptySelection())
        || (!this.panel.isAlreadyShown() && this.predefinedState.selection.isEmpty() && !isAllPredefined));
};

/**
 * Save selected c&p in report config
 *
 */
CPReportSelector.prototype.getCpSelection = function () {
    // if panel not shown do not read values from panel
    if (!this.panel.isAlreadyShown()) {
        return this.cpSelection;
    } else {
        var selection = this.panel.getSelection(false);
        // update predefined state
        return  {
            customers : selection.getFirstLevel(),
            projects : selection.getSecondLevel(),
            qualifier : (this.panel.isCustomOptionSelected()) ? CustomersProjectsSelectionQualifier.SELECTED :
                ((this.panel.isAllItemsOptionSelected()) ? CustomersProjectsSelectionQualifier.ALL : CustomersProjectsSelectionQualifier.ALL_ACTIVE),
            showArchivedProjects : this.panel.getShowInactive()
        };
    }

};

CPReportSelector.prototype.setInitialValueFromCurrent = function() {
    // only if panel already shown
    if (this.panel.isAlreadyShown()) {
        var selection = this.panel.getSelection(false);
        var predefinedState = {
            selection: this.panel.isCustomOptionSelected() ?
                new TwoLevelSelection(selection.getFirstLevel(), selection.getSecondLevel()) : TwoLevelSelection.emptySelection(),
            optionValue: (this.panel.isCustomOptionSelected()) ? CoarseSelection.SPECIFIC :
                ((this.panel.isAllItemsOptionSelected()) ? CoarseSelection.ALL : CoarseSelection.ALL_ACTIVE),
            showInactive: this.panel.getShowInactive()
        };
        this.panel.setPredefinedParameters(predefinedState);
        this.panel.loadState(predefinedState);
    }
};

/**
 * Set selection from customersProjectsSelection
 * Method sets new initial values of selector
 * @param customersProjectsSelection
 */
CPReportSelector.prototype.setSelection = function( customersProjectsSelection ) {
    this.predefinedState = this._createPredefinedStateFromSelection(customersProjectsSelection);
    // only if panel already shown
    if (this.panel.isAlreadyShown()) {
        this.panel.loadState(this.predefinedState);
        this.panel.setPredefinedParameters(this.predefinedState);
    } else {
        // need to update text label on button if c/p data is loaded from server and panel not show
        if (this.preloadedData) {
            this.button.setText(this._makePredefinedSelection(this.preloadedData));
        }
    }
};


CPReportSelector.prototype.resetSelection = function() {
    // if panel is shown
    if (this.panel.isAlreadyShown()) {
        this.panel.resetState(true);
    }
};

CPReportSelector.prototype.isStateChanged = function()
{
    return this.panel.isStateChanged();
};

CPReportSelector.prototype.saveState = function()
{
    var cpSelection = this.getCpSelection();
    var predefinedState = this._createPredefinedStateFromSelection(cpSelection);
    this.panel.setPredefinedParameters(predefinedState);
    this.panel.loadState(predefinedState);
};