/**
 * This rpc manager is intended for load customers and projects data for using in customer-project selectors
 */
function BaseRetrieveCustomersProjectsRPCManager() {
    RpcControlManager.call(this);
    this.selectorMenu = null;
}

BaseRetrieveCustomersProjectsRPCManager.prototype = _.create(RpcControlManager.prototype);

BaseRetrieveCustomersProjectsRPCManager.prototype.getService = function() {
    return jsonrpc.RetrieveCustomersProjectsService;
};

BaseRetrieveCustomersProjectsRPCManager.prototype.getMethods = function() {
    return ['RetrieveCustomersProjectsService.getActiveCustomersProjects',
            'RetrieveCustomersProjectsService.getAllCustomersProjects',
            'RetrieveCustomersProjectsService.getCustomersProjectsForInsertExistingTasks'];
};

BaseRetrieveCustomersProjectsRPCManager.prototype.setSelectorMenu = function(selectorMenu) {
    this.selectorMenu = selectorMenu;
};

BaseRetrieveCustomersProjectsRPCManager.prototype.setSpinner = function(spinner) {
    this.spinner = spinner;
};

BaseRetrieveCustomersProjectsRPCManager.prototype.getSpinner = function() {
    return this.spinner;
};

BaseRetrieveCustomersProjectsRPCManager.prototype.onDataSavingComplete = function(result) {
    this.spinner.stop();
    this.selectorMenu.panel.initializeSelector(result);
};

BaseRetrieveCustomersProjectsRPCManager.prototype.endSavingData = function(success) {
    if (!success) {
        this.selectorMenu.hide();
        this.selectorMenu.panel.enableControls();
    }
};

BaseRetrieveCustomersProjectsRPCManager.prototype.getTimeoutMsg = function() {
    return messageResource.getMessage("common.ajax.timeout.read_data");
};

BaseRetrieveCustomersProjectsRPCManager.prototype.getRpcErrorMsg = function()
{
    return messageResource.getMessage("common.ajax.unexpected_server_error");
};
