CPTManagement.DeleteConfirmationDialog = function(config)
{
    TemplateView.call(this, config.$containerElement);
    this.hasManagerApprover = config.hasManagerApprover;
    this.idPrefix = _.isUndefined(config.idPrefix) ? "operationConfirmDialog_" : config.idPrefix + '_deleteConfirm_';
    this.cancelHandler = _.isUndefined(config.cancelHandler) ? function() {} : config.cancelHandler;
    this.deleteHandler = config.deleteHandler;

    this.entityName = config.entityName;
    this.entitiesToDeleteCount = config.entitiesToDeleteCount;

    this.entitiesBlocked = config.entitiesBlocked;
    this.entitiesBlockedCount = config.entitiesBlockedCount;
    this.entitiesBlockedDueToLockedTimeCount = config.entitiesBlockedDueToLockedTimeCount;
    this.entitiesBlockedDueToUnavailableToModifyTimeTrackCount = config.entitiesBlockedDueToUnavailableToModifyTimeTrackCount;
    this.entitiesWithManagerTimeTrackCount = config.entitiesWithManagerTimeTrackCount;
    this.entitiesWithOtherUsersTimeTrackCount = config.entitiesWithOtherUsersTimeTrackCount;
    this.entitiesWithManagerApprovedTimeTrackCount = config.entitiesWithManagerApprovedTimeTrackCount;
    this.entitiesWithOtherUsersApprovedTimeTrackCount = config.entitiesWithOtherUsersApprovedTimeTrackCount;
};

CPTManagement.DeleteConfirmationDialog.prototype = _.create(TemplateView.prototype, {
    html: '<div class="operationConfirmDialog">' +
          '  <div class="operationConfirmDialogInnerContainer">' +
          '    <div class="warningIcon"></div>' +
          '    <div class="title"><%=titleText%></div>' +
          '    <div class="orangeBlock">' +
          '      <span><%-firstOrangeSentence%></span>' +
          '      <br>' +
          '      <span><%-secondOrangeSentence%></span>' +
          '    </div>' +
          '    <ul class="pointsBlock">' +
          '      <li class="point"><%-removeEntitiesPointText%></li>' +
          '      <li class="point"><%-removeTimeTrackPointText%></li>' +
          '      <li class="point managerApproveWeeksPoint"><%-removeManagerApprovedTimeTrackText%></li>' +
          '      <li class="point approveWeeksPoint"><%-approveRejectWeeksText%></li>' +
          '      <table class="choiceApprovalWeeks">' +
          '        <tr>' +
          '          <td><input type="radio" name="<%-idPrefix%>revokeWeeksStatuses" value="true" checked="checked" id="<%-idPrefix%>revokeWeeksStatuses"></td>' +
          '          <td><label for="<%-idPrefix%>revokeWeeksStatuses"><%=revokeWeeksStatusesLabel%></label></td>' +
          '        </tr>' +
          '        <tr>' +
          '         <td><input type="radio" name="<%-idPrefix%>revokeWeeksStatuses" value="false" id="<%-idPrefix%>keepWeeksStatuses"></td>' +
          '         <td><label for="<%-idPrefix%>keepWeeksStatuses"><%=keepWeeksStatusesLabel%></label></td>' +
          '        </tr>' +
          '      </table>' +
          '    </ul>' +
          '    <div class="blockedWarningContainer">' +
          '      <div class="link"><%-blockedWarningLinkText%></div>' +
          '      <div><%-blockedWarningOtherText%></div>' +
          '    </div>' +
          '    <div class="buttonsContainer">' +
          '      <div id="<%-idPrefix%>submitBtn" class="withIcon commitBtn greyButton" tabindex="-1">' +
          '        <div class="buttonIcon">' +
          '          <span id="<%-idPrefix%>submitTitle" class="buttonTitle"><%-deleteButtonText%></span>' +
          '        </div>' +
          '      </div>' +
          '      <div id="<%-idPrefix%>cancelBtn" class="cancelBtn greyButton"><%=cancelButtonText%></div>' +
          '    </div>' +
          '  </div>' +
          '</div>',

    messageKeys: function()
    {
        return {
            approveRejectWeeksText: "delete.confirm.dialog.about_approve_weeks",
            revokeWeeksStatusesLabel: "delete.confirm.dialog.revoke_weeks_statuses_label",
            keepWeeksStatusesLabel: "delete.confirm.dialog.keep_weeks_statuses_label",
            cancelButtonText: "delete.confirm.dialog.cancel"
        };
    },

    afterRender: function()
    {
        this.$el.toggleClass("allBlocked", this.allBlocked());
        this.$el.toggleClass("someBlocked", this.someBlocked());
        this.$el.toggleClass("hasTimeTrack", this.hasTimeTrack());
        this.$el.toggleClass("hasApprovedWeeks", this.hasApprovedWeeks());
        this.$el.toggleClass("hasManagerApprovedWeeksAndApprover", this.hasManagerApprovedWeeksAndApprover());

        this.setHandlers();
    },

    allBlocked: function()
    {
        return this.entitiesBlockedCount == this.entitiesToDeleteCount
    },

    someBlocked: function()
    {
        return this.entitiesBlockedCount > 0 && this.entitiesBlockedCount < this.entitiesToDeleteCount;
    },

    hasTimeTrack: function()
    {
        return this.entitiesWithOtherUsersTimeTrackCount > 0 || this.entitiesWithManagerTimeTrackCount > 0
    },

    hasApprovedWeeks: function()
    {
        return this.entitiesWithOtherUsersApprovedTimeTrackCount > 0
    },

    hasManagerApprovedWeeksAndApprover: function()
    {
        return this.entitiesWithManagerApprovedTimeTrackCount > 0 && this.hasManagerApprover;
    },

    show: function()
    {
        TemplateView.prototype.show.call(this);
        if (this.showHandler) 
            this.showHandler();
        var self = this;
        var $submitBtn = this.getById("submitBtn");
        $submitBtn.off("keydown").on("keydown", function(event)
                                                {
                                                    switch (event.keyCode)
                                                    {
                                                        case KEY_CODE_SPACE:
                                                        case KEY_CODE_ENTER:
                                                            self.onSubmit();
                                                            cancelBubble(event.originalEvent);
                                                            return false;
                                                    }
                                                });
        $submitBtn.focus();
    },
    
    setHandlers: function()
    {
        this.getById("cancelBtn").unbind().click(_.bind(this.onCancel, this));
        this.getById("submitBtn").unbind().click(_.bind(this.onSubmit, this));

        var $blockedWarningLink = this.find(".blockedWarningContainer .link");
        var blockedEntitiesList = this.createBlockedEntitiesList();
        $blockedWarningLink.click(function()
                                  {
                                      blockedEntitiesList.showAt($blockedWarningLink, {
                                          offset: "10px 20px",
                                          at: "right bottom",
                                          my: "left bottom",
                                          collision: "none"
                                      });
                                  });
    },

    onCancel: function()
    {
        if (this.savingState)
            return;

        this.cancelHandler();
    },

    onSubmit: function()
    {
        if (this.savingState)
            return;

        this.$el.addClass("savingState");
        this.savingState = true;
        var revokeStatuses = $('input[name=' + this.idPrefix + 'revokeWeeksStatuses]:checked').val();
        this.deleteHandler(revokeStatuses);
    },

    getById: function(id)
    {
        return $('#' + this.idPrefix + id);
    },

    templateModel: function()
    {
        return {
            titleText: this.titleText(),
            firstOrangeSentence: this.firstOrangeSentence(),
            secondOrangeSentence: this.secondOrangeSentence(),
            removeEntitiesPointText: this.removeEntitiesPointText(),
            removeTimeTrackPointText: this.removeTimeTrackPointText(),
            blockedWarningLinkText: this.blockedWarningLinkText(),
            blockedWarningOtherText: this.blockedWarningOtherText(),
            deleteButtonText: this.deleteButtonText(),
            idPrefix: this.idPrefix,
            removeManagerApprovedTimeTrackText: this.getRemoveManagerApprovedTimeTrackText()
        }
    },

    getRemoveManagerApprovedTimeTrackText: function()
    {
        return messageResource.getMessage(this.entitiesWithManagerApprovedTimeTrackCount > 1 ?
                                          "delete.confirm.dialog.about_approved_manager_time_track" :
                                          "delete.confirm.dialog.about_approved_manager_time_track_one_task");
    },

    createBlockedEntitiesList: function()
    {
        // Abstract
    },

    titleText: function()
    {
        // Abstract
    },

    firstOrangeSentence: function()
    {
        // Abstract
    },

    secondOrangeSentence: function()
    {
        // Abstract
    },

    removeEntitiesPointText: function()
    {
        // Abstract
    },

    removeTimeTrackPointText: function()
    {
        // Abstract
    },

    blockedWarningLinkText: function()
    {
        // Abstract
    },

    blockedWarningOtherText: function()
    {
        // Abstract
    },

    deleteButtonText: function()
    {
        // Abstract
    }
});
