CPTManagement.DeleteCustomersProjectsConfirmationDialog = function(config)
{
    CPTManagement.DeleteConfirmationDialog.call(this, config);
    this.customers = config.customers;
    this.projects = config.projects;
};

CPTManagement.DeleteCustomersProjectsConfirmationDialog.prototype = _.create(CPTManagement.DeleteConfirmationDialog.prototype, {
    createBlockedEntitiesList: function()
    {
        return new CPTManagement.DeleteConfirmationDialog.BlockedProjectsList($("body"), _.filter(this.entitiesBlocked, function(entity) { return entity instanceof ProjectManagementEntity })).init();
    },

    _calculateChildProjects: function()
    {
        var count = 0;
        for (var i = 0; i < this.customers.length; i++)
            count += this.customers[i].getChildCount();
        return count;
    },

    _calculateHiddenProjects: function()
    {
        var count = 0;
        for (var i = 0; i < this.customers.length; i++)
            count += this.customers[i].hiddenProjectCount;
        return count;
    },

    _calculateActiveProjects: function()
    {
        return this._calculateChildProjects();
    },

    _calculateArchivedProjects: function()
    {
        return this._calculateHiddenProjects();
    },

    _getProjectsDescription: function()
    {
        var activeProjectsCount = this._calculateActiveProjects();
        var archivedProjectsCount = this._calculateArchivedProjects();

        if (activeProjectsCount == 0 && archivedProjectsCount == 0)
            return null;

        var bracketText;
        if (activeProjectsCount == 0)
            bracketText = messageResource.getMessage('common.archived_multiple', [archivedProjectsCount]);
        else if (archivedProjectsCount == 0)
            bracketText = messageResource.getMessage('common.active_multiple', [activeProjectsCount]);
        else
            bracketText = messageResource.getMessage('common.active_and_archived_multiple', [activeProjectsCount, archivedProjectsCount]);

        bracketText = '<nobr>' + bracketText + '</nobr>';
        return bracketText;
    },

    _titleTextInner: function()
    {
        if (this.customers.length > 0 && this.projects.length > 0)
        {
            var customersProjectsDescription = this._getProjectsDescription();
            var customersText;

            if (customersProjectsDescription != null)
            {
                if (this.customers.length == 1)
                    customersText = messageResource.getEscapedMessageNoEscapeArgs('delete.confirm.dialog.along_with_its.customer',
                                                                                  [customersProjectsDescription]);
                else
                    customersText = messageResource.getEscapedMessageNoEscapeArgs('delete.confirm.dialog.along_with_their.customers',
                                                                                  [this.customers.length, customersProjectsDescription]);
            }
            else
            {
                customersText = this.customers.length + ' ' + escapeHtmlSymbols(this._getCustomerText());
            }

            var projectsWithoutSelectedCustomer = _.filter(this.projects, _.bind(function(project)
            {
                return !_.findWhere(this.customers, {id: project.customerId});
            }, this));
            if (projectsWithoutSelectedCustomer.length > 0)
            {
                var projectText = ' ' + messageResource.getEscapedMessage('delete.confirm.dialog.selected_project' + (projectsWithoutSelectedCustomer.length == 1 ? '' : 's'));
                return messageResource.getMessage('common.smth_and_smth', [customersText, projectsWithoutSelectedCustomer.length + projectText]);
            }
            return customersText;
        }
        else if (this.customers.length > 0)
        {
            var projectsDescription = this._getProjectsDescription();
            if (this.customers.length == 1)
            {
                if (projectsDescription != null)
                    return messageResource.getEscapedMessageNoEscapeArgs('delete.confirm.dialog.along_with_its.customer',
                                                                         [projectsDescription]);
                else
                    return messageResource.getEscapedMessage('delete.confirm.dialog.customer');
            }
            else
            {
                if (projectsDescription != null)
                    return messageResource.getEscapedMessageNoEscapeArgs('delete.confirm.dialog.along_with_their.customers',
                                                                         [this.customers.length, projectsDescription]);
                else
                    return this.customers.length + ' ' + messageResource.getEscapedMessage('common.customers');
            }
        }
        else if (this.projects.length > 0)
        {
            if (this.projects.length == 1)
                if (this.projects[0].taskCount > 0)
                    return messageResource.getEscapedMessage('delete.confirm.dialog.along_with_its.project');
                else
                    return messageResource.getEscapedMessage('delete.confirm.dialog.project');
            else
                return this.projects.length + ' ' + messageResource.getEscapedMessage('common.projects');
        }
    },

    titleText: function()
    {
        if (this.allBlocked())
        {
            if (this.customers.length > 0 && this.projects.length > 0)
                return messageResource.getEscapedMessage("delete.confirm.dialog.cp.all_blocked.title.customers_and_projects");
            else if (this.customers.length > 0)
                return messageResource.getEscapedMessage("delete.confirm.dialog.cp.all_blocked.title.customers");
            else if (this.projects.length > 0)
                return messageResource.getEscapedMessage("delete.confirm.dialog.cp.all_blocked.title.projects");
        }
        else
        {
            return messageResource.getMessage("delete.confirm.dialog.cp.title_prefix") + " " + this._titleTextInner();
        }
    },

    firstOrangeSentence: function()
    {
        if (this.allBlocked())
        {
            if (this.customers.length > 0 && this.projects.length > 0)
                return messageResource.getMessage("delete.confirm.dialog.cp.all_blocked.first_orange_sentence.customers_and_projects");
            else if (this.customers.length > 0)
                return messageResource.getMessage("delete.confirm.dialog.cp.all_blocked.first_orange_sentence.customers");
            else if (this.projects.length > 0)
                return messageResource.getMessage("delete.confirm.dialog.cp.all_blocked.first_orange_sentence.projects");
        }
        else
        {
            if (this.customers.length > 0 && this.projects.length > 0)
                return messageResource.getMessage("delete.confirm.dialog.cp.first_orange_sentence.customers_and_projects");
            else if (this.customers.length > 0)
                return messageResource.getMessage("delete.confirm.dialog.cp.first_orange_sentence.customers");
            else if (this.projects.length > 0)
                return messageResource.getMessage("delete.confirm.dialog.cp.first_orange_sentence.projects");
        }
    },

    secondOrangeSentence: function()
    {
        if (this.allBlocked())
            return this.blockedWarningOtherText();
        else
            return messageResource.getMessage(this.hasTimeTrack() ?
                                              "delete.confirm.dialog.cp.second_orange_sentence.please_confirm_actions" :
                                              "delete.confirm.dialog.cp.second_orange_sentence.please_confirm");
    },

    removeEntitiesPointText: function()
    {
        if (this.customers.length > 0 && this.projects.length > 0)
            return messageResource.getMessage("delete.confirm.dialog.cp.remove_entities_point_text.customers_and_projects", [this._getCustomerText(), this._getPronoun(this.customers)]);
        else if (this.customers.length > 0)
            return messageResource.getMessage("delete.confirm.dialog.cp.popup.remove_entities_point_text", [this._getCustomerText(), this._getPronoun(this.customers)]);
        else if (this.projects.length > 0)
            return messageResource.getMessage("delete.confirm.dialog.cp.popup.remove_entities_point_text", [this._getProjectText(), this._getPronoun(this.projects)]);
    },

    _getPronoun: function(array)
    {
        return messageResource.getMessage(array.length > 1 ? "delete.confirm.dialog.their" : "delete.confirm.dialog.its");
    },

    _getProjectText: function()
    {
        return messageResource.getMessage(this.projects.length == 1 ? "common.project" : "common.projects");
    },

    _getCustomerText: function()
    {
        return messageResource.getMessage(this.customers.length == 1 ? "common.customer" : "common.customers");
    },

    removeTimeTrackPointText: function()
    {
        return messageResource.getMessage("delete.confirm.dialog.cp.remove_time_track_point_text")
    },

    blockedWarningLinkText: function()
    {
        return messageResource.getMessage("delete.confirm.dialog.cp.blocked_warning_link_text")
    },

    blockedWarningOtherText: function()
    {
        if (this.entitiesBlockedDueToLockedTimeCount > 0 && this.entitiesBlockedDueToUnavailableToModifyTimeTrackCount == 0)
            return messageResource.getMessage("delete.confirm.dialog.cp.blocked_warning_other_text.locked");
        else if (this.entitiesBlockedDueToLockedTimeCount == 0 && this.entitiesBlockedDueToUnavailableToModifyTimeTrackCount > 0)
            return messageResource.getMessage("delete.confirm.dialog.cp.blocked_warning_other_text.cannot_modify");
        else if (this.entitiesBlockedDueToLockedTimeCount > 0 && this.entitiesBlockedDueToUnavailableToModifyTimeTrackCount > 0)
            return messageResource.getMessage("delete.confirm.dialog.cp.blocked_warning_other_text.locked_and_cannot_modify");
    },

    deleteButtonText: function()
    {
        if (this.hasTimeTrack())
            return messageResource.getMessage("delete.confirm.dialog.delete_task_with_entries");
        else
            return messageResource.getMessage("delete.confirm.dialog.cp.delete_button_text.no_tt")
    }
});
