function CreateCustomerLightBox () {
    this.constructor = CreateCustomerLightBox;
}

CreateCustomerLightBox.WARNING_ERROR_DURING_IMPORT = 300;

CreateCustomerLightBox.prototype = new CustomerLightBox();

CreateCustomerLightBox.prototype.close_tab_confirmation_message_key = "customer.create.close_tab.confirmation";

CreateCustomerLightBox.prototype.init = function (popupId, closeBtnId, customerInfos, featuresInfo, successHandler) {
    CustomerLightBox.prototype.init.call(this, popupId, closeBtnId);

    this.warningMessages = {};
    this.warningMessages[CreateCustomerLightBox.WARNING_ERROR_DURING_IMPORT] = messageResource.getMessage('customer.create.error_during_importing');

    this.featuresInfo = featuresInfo;
    $(this.lightBoxEl ).toggleClass("projectAssignmentsDisabled", !featuresInfo.projectAssignmentsEnabled);

    this.customerSelectorPlaceHolderEl = this.getElementBySubId("customerSelectorPlaceholder");
    this.initCustomerSelector(customerInfos);
    this.initImportSettingsCheckboxes();

    this.setTitle(messageResource.getMessage("customer.create.popup.title"));
    this.cancel_confirmation_message_key = "customer.create.exit_confirmation";
    this.setCommitButtonTitle(messageResource.getMessage("customer.create.commit_button.text"),
                              messageResource.getMessage("customer.create.commit_button.default_text"),
                               17);
    $(this.lightBoxEl ).removeClass("editMode");
    this.initPlaceholders();

    this.successCallback = function ( customerName, customerDescription, customerIdToImportDataFrom, importProjectsAssignments, importProjectsDescriptions, importTasksDescriptions, featuresInfo ) {
        var self = this;

        this.disableUserInputWhileSaving();
        makeRpcCall("CustomerInfoService.createCustomer", [customerName, customerDescription, customerIdToImportDataFrom, importProjectsAssignments, importProjectsDescriptions, importTasksDescriptions, featuresInfo],
                    function (result) {
                        if (result.resultCode == CustomerLightBox.RESULT_SUCCESSFULL)
                        {
                            if (result.warning)
                                if (self.warningMessages[result.warning])
                                    alert(self.warningMessages[result.warning]);

                            if (successHandler)
                                successHandler(result);

                            self.hide();
                        }
                        else if (result.resultCode == CustomerLightBox.ERROR_NAME_DUPLICATE)
                        {
                            self.displayNameDuplicateError();
                            self.enableUserInputWhileSaving();
                        }
                    },
                    function (jqXHR, textStatus, errorThrown) {
                        self.enableUserInputWhileSaving();
                    }
        );
    };

    return this;
};

CreateCustomerLightBox.prototype.show = function()
{
    CustomerLightBox.prototype.show.call(this);
    setFocusIntoFieldWithPlaceholder( this.nameField );
};

CreateCustomerLightBox.prototype.getDataToCommit = function()
{
    return [ this.getCustomerName(), this.getDescription(), this.getCustomerSelectorValue(),
             this.getIsImportProjectsAssignments(), this.getIsImportProjectsDescriptions(), this.getIsImportTasksDescriptions(),
             this.featuresInfo ];
};

CreateCustomerLightBox.prototype.initCustomerSelector = function ( customerInfos )
{
    var self = this;
    var initialValue = -1;

    var items = [];
    items.push({id: -1, name: messageResource.getEscapedMessage("customer.create.selector.placeholder.text")});
    $.each(customerInfos.list, function() {
        var item = {id: this.id, name: escapeHtmlSymbols(this.name)};
        items.push(item);
    });

    this.customerSelectorPlaceHolderEl.innerHTML = "";
    this.customerSelector = new ListSelector({
                                                 menuCls: "customerSelectorMenu customerLightbox",
                                                 renderTo: this.customerSelectorPlaceHolderEl,
                                                 value: initialValue,
                                                 listItems: items,
                                                 textPropName: "name",
                                                 valuePropName: "id"
                                             });
    var handler = function (value) {
        value == -1 ? $(self.customerSelectorPlaceHolderEl ).addClass("notSelected") : $(self.customerSelectorPlaceHolderEl).removeClass("notSelected");
        $(self.customerSelectorPlaceHolderEl).find(".x-btn-wrap.x-btn.at-dropdown-list-btn").width("496px");
    };
    handler(initialValue);
    this.customerSelector.on( "change", function () { self.changesCallback(); handler(self.getCustomerSelectorValue()); } );
};

CreateCustomerLightBox.prototype.getCustomerSelectorValue = function ()
{
    return this.customerSelector.getSelectedItem().id;
};

CreateCustomerLightBox.prototype.initImportSettingsCheckboxes = function ()
{
    this.importProjectsAssignmentsCheckbox = this.getElementBySubId("importProjectsAssignments");
    this.importProjectsDescriptionsCheckbox = this.getElementBySubId("importProjectsDescriptions");
    this.importTasksDescriptionsCheckbox = this.getElementBySubId("importTasksDescriptions");

    this.importProjectsAssignmentsCheckbox.checked = true;
    this.importProjectsDescriptionsCheckbox.checked = true;
    this.importTasksDescriptionsCheckbox.checked = true;
};
CreateCustomerLightBox.prototype.getIsImportProjectsAssignments = function ()
{
    return this.importProjectsAssignmentsCheckbox.checked;
};
CreateCustomerLightBox.prototype.getIsImportProjectsDescriptions = function ()
{
    return this.importProjectsDescriptionsCheckbox.checked;
};
CreateCustomerLightBox.prototype.getIsImportTasksDescriptions = function ()
{
    return this.importTasksDescriptionsCheckbox.checked;
};