function CustomerLightBox () {
    this.constructor = CustomerLightBox;
}
CustomerLightBox.prototype = new BasicLightbox();
CustomerLightBox.NAME_EMPTY = "nameEmptyError";
CustomerLightBox.NAME_DUPLICATE = "nameDuplicateError";
CustomerLightBox.DESCRIPTION_TOO_LONG = "descriptionTooLongError";

CustomerLightBox.RESULT_SUCCESSFULL = 200;
CustomerLightBox.ERROR_NAME_DUPLICATE = 400;

CustomerLightBox.MAX_DESCRIPTION_LENGTH = 2000;



CustomerLightBox.prototype.exist_invalid_fields_message_key = "customer.edit.correct_errors";
CustomerLightBox.prototype.init = function ( popupId, closeBtnId ) {
    BasicLightbox.prototype.init.call(this, popupId, closeBtnId);

    var self = this;

    this.titlePlaceholder = this.getElementBySubId("titlePlaceholder");
    this.commitButtonTitlePlaceholder = $(document.getElementById( this.lightBoxEl.id + "_commitBtn" )).find(".buttonTitle");
    this.customerInfoDiv = this.getElementBySubId("customerInfoDiv");

    this.dataChanged = false;
    this.cancelCallback = self.hide;
    this.errors = {};

    this.initNameField();
    this.initDescriptionField();
    this.initTextAreaLengthNote();

    return this;
};

CustomerLightBox.prototype.setTitle = function( title )
{
    this.titlePlaceholder.innerHTML = escapeHtmlSymbols( title );
};

CustomerLightBox.prototype.initNameField = function () {
    this.nameField = $(this.getElementBySubId("nameField"));

    var self = this;
    this.nameField.off();
    this.nameField.blur( this.basicOnBlurFieldHandler( function () { self.checkNameOnChange(); } ) );
    this.nameField.keyup( this.basicKeyUpChangesHandler() );
    this.nameField.change( function () { self.hideNameDuplicateError() } );
    this.nameField.keydown( this.basicOnKeyPressedHandler );
};

CustomerLightBox.prototype.initPlaceholders = function () {
    this.nameField.placeholder();
    this.descriptionField.placeholder();
};

CustomerLightBox.prototype.checkNameOnChange = function ()
{
    this.checkEmpty( this.nameField, CustomerLightBox.NAME_EMPTY, true, true );
    this.updateErrors(false);
};
CustomerLightBox.prototype.isNameValid = function( onlyRemoveEmptyError )
{
    return !this.checkEmpty( this.nameField, CustomerLightBox.NAME_EMPTY, onlyRemoveEmptyError, true );
};

CustomerLightBox.prototype.getCustomerName = function()
{
    return this.nameField.pVal();
};

CustomerLightBox.prototype.initDescriptionField = function () {
    this.descriptionField = $(this.getElementBySubId("descriptionField"));

    var self = this;
    this.descriptionField.off();
    this.descriptionField.blur( this.basicOnBlurFieldHandler( function () { self.checkDescriptionOnChange() } ) );
    this.descriptionField.keyup( this.basicKeyUpChangesHandler( function () { self.checkDescriptionOnChange() } ) );
    this.descriptionField.keydown( this.basicOnKeyPressedHandler );
};

CustomerLightBox.prototype.getDescription = function ()
{
    return this.descriptionField.pVal();
};

CustomerLightBox.prototype.getDescriptionForUpdatingLengthNote = function ()
{
    return trim(getStringWithUnifiedNewLineSymbol(this.getDescription()));
};

CustomerLightBox.prototype.isDescriptionValid = function ()
{
    this.descriptionLengthNote.update(this.getDescriptionForUpdatingLengthNote());
    var textLength = this.descriptionLengthNote.getTextLength();
    var result = textLength > this.descriptionLengthNote.maxValue;
    this.errors[ CustomerLightBox.DESCRIPTION_TOO_LONG ] = result;
    this.descriptionField.attr("title", result ? messageResource.getMessage("customer.edit.description_too_big", [textLength]) : "");
    return !result;
};

CustomerLightBox.prototype.checkDescriptionOnChange = function ()
{
    this.isDescriptionValid();
    this.updateErrors(false);
};

CustomerLightBox.prototype.initTextAreaLengthNote = function ()
{
    this.descriptionLengthNote = new TextAreaLengthNote( this.getElementIdBySubId('customerDescription'), 'Description', CustomerLightBox.MAX_DESCRIPTION_LENGTH);
    this.descriptionLengthNote.setTextLengthCounter( this.descriptionLengthNote.fullCounter );
    this.descriptionLengthNote.update( this.getDescriptionForUpdatingLengthNote() );
    this.descriptionLengthNote.show();
};

CustomerLightBox.prototype.changesDetected = function()
{
    if (!this.dataChanged)
        this.dataChanged = true;
};

CustomerLightBox.prototype.validate = function ()
{
    return this.isNameValid(false) && this.isDescriptionValid();
};

CustomerLightBox.prototype.setErrorClasses = function( classes )
{
    this.customerInfoDiv.className = classes.join( " " );
};

CustomerLightBox.prototype.updateUI = function ()
{
    this.updateErrors(true);
};

CustomerLightBox.prototype.getErrorsMap = function()
{
    return this.errors;
};

CustomerLightBox.prototype.getElementBySubId = function( subId )
{
    return document.getElementById( this.getElementIdBySubId(subId) );
};
CustomerLightBox.prototype.getElementIdBySubId = function ( subId )
{
    return this.lightBoxEl.id + "_" + subId;
};
CustomerLightBox.prototype.displayErrorInFooter = function ()
{
    this._errorMessageBox.removeClass( "disabled" );
};
CustomerLightBox.prototype.updateErrors = function ( showErrorInFooter )
{
    if (this.hasErrors())
    {
        if( showErrorInFooter )
            this.displayErrorInFooter();
    }
    else
        this._removeErrorInFooter();
};
CustomerLightBox.prototype.displayNameDuplicateError = function ()
{
    this.errors[CustomerLightBox.NAME_DUPLICATE] = true;
    this.updateErrors(true);
};

CustomerLightBox.prototype.hideNameDuplicateError = function ()
{
    this.errors[CustomerLightBox.NAME_DUPLICATE] = false;
    this.updateErrors();
};
