function EditCustomerLightBox () {
    this.constructor = EditCustomerLightBox;
}

EditCustomerLightBox.ERROR_DELETED_CONCURRENTLY = 401;


EditCustomerLightBox.prototype = new CustomerLightBox();

EditCustomerLightBox.prototype.close_tab_confirmation_message_key = "customer.edit.close_tab.confirmation";

EditCustomerLightBox.prototype.init = function(popupId, closeBtnId, customerInfo, showConfirmDeleteDlgFnc, hasManagerApprover)
{
    CustomerLightBox.prototype.init.call(this, popupId, closeBtnId);

    var self = this;
    this.setTitle(messageResource.getMessage("customer.edit.popup.title"));
    this.cancel_confirmation_message_key = "customer.edit.exit_confirmation";
    this.setCommitButtonTitle(messageResource.getMessage("customer.edit.commit_button.text"));

    this.customerInfo = customerInfo;
    this.setValue(this.nameField, customerInfo.name);
    //IE8 writes 'null' as string, in any case we send empty string to backend
    //bug 55185, 55184
    this.setValue(this.descriptionField, customerInfo.description || '');
    this.descriptionLengthNote.update( this.getDescriptionForUpdatingLengthNote() );
    this._popupEl = $(this.lightBoxEl);
    this._popupEl.addClass("editMode");

    this.footer = $(this.getElementBySubId("footer"));

    this.statusSelectorPlaceHolderEl = this.getElementBySubId("statusSelectorPlaceholder");
    this.initStatusSelector(this.customerInfo.active);
    this.initPlaceholders();

    this._deleteEnabled = true;
    this._deleteBlockedDueToLockedTime = customerInfo.blockedDueToLockedTime;
    this._deleteBlockedDueToUnavailableToModifyTimeTrack = customerInfo.blockedDueToUnavailableToModifyTimeTrack;

    this.onDelete = function( revokeStatuses ) {
        self.disableUserInputWhileSaving();
        makeRpcCall("CustomerInfoService.deleteCustomer",
                    [self.customerInfo.id, revokeStatuses],
                    function(result)
                    {
                        self.hide();
                        window.location.reload();
                    },
                    function(jqXHR, textStatus, errorThrown)
                    {
                        self.enableUserInputWhileSaving();
                    }
        );
    };

    this.deleteButton = this.getElementBySubId("deleteBtn");

    if (!this._deleteBlockedDueToLockedTime && !this._deleteBlockedDueToUnavailableToModifyTimeTrack)
    {
        if (this._deleteEnabled)
        {
            this.mainContent = this.findElementBySubId("content");
            this.moireIn = new EditCPTPopupMoire({
                $containerElement: self._popupEl,
                idPrefix: self.popupId + "_in"
            }).init();

            this.moireOut = new EditCPTPopupMoire({
                $containerElement: self._popupEl,
                idPrefix: self.popupId + "_out"
            }).init();

            this.moireInEl = this.findElementBySubId("in_switchStateMoire");
            this.moireOutEl = this.findElementBySubId("out_switchStateMoire");

            var customerEntity = new EditDialogCustomerManagementEntity(
                    this.customerInfo.id,
                    this.customerInfo.name,
                    this.customerInfo.blockedDueToLockedTime,
                    this.customerInfo.blockedDueToUnavailableToModifyTimeTrack,
                    this.customerInfo.customerHasManagerTimeTrack,
                    this.customerInfo.customerHasOtherUsersTimeTrack,
                    this.customerInfo.customerHasManagerApprovedTimeTrack,
                    this.customerInfo.customerHasOtherUsersApprovedTimeTrack,
                    this.customerInfo.active,
                    this.customerInfo.totalActiveProjectsTasksNum + self.customerInfo.archivedProjectsTasksNum,
                    this.getHiddenProjectCount(),
                    this.getVisibleProjectCount());

            this.deletePopup = new CPTManagement.DeleteCustomerPopupConfirmationDialog({
                $containerElement: self._popupEl,
                idPrefix: self.popupId,
                entitiesToDelete: [customerEntity],
                hasManagerApprover: this.customerInfo.hasManagerApprover,
                cancelHandler: _.bind(self.switchToMainContent, self),
                deleteHandler: _.bind(self.onDelete, self)
            }).init();

            this.savedEscape = this.onEscape;
            this.deleteButton.onclick = _.bind(this.switchToConfirmContent, this);
        }
    }
    else
    {
        var titleText;
        if (this._deleteBlockedDueToLockedTime)
        {
            titleText = messageResource.getMessage("task.edit_popup.cannot_delete_due_to_lock_time");
        }
        else
        {
            var customerText = messageResource.getMessage("common.customer");
            titleText = messageResource.getMessage(this.customerInfo.canManageAllUsersWithoutAutoApproved ?
                                                   "hierarchy_entity.edit_popup.cannot_delete_due_to_unavailable_modify_time_of_autoapproved_users" :
                                                   "hierarchy_entity.edit_popup.cannot_delete_due_to_unavailable_modify_time",
                                                   [customerText]);
        }
        $(this.deleteButton).addClass("disabled").get(0).title = titleText;
    }

    this.successCallback = function ( customerId, customerName, customerDescription, status ) {
        this.disableUserInputWhileSaving();
        makeRpcCall("CustomerInfoService.updateCustomer", [customerId, customerName, customerDescription, status],
                    function (result) {
                        if (result.resultCode == CustomerLightBox.RESULT_SUCCESSFULL)
                        {
                            self.hide();
                            window.location.reload();
                        }
                        else if (result.resultCode == CustomerLightBox.ERROR_NAME_DUPLICATE)
                        {
                            self.displayNameDuplicateError();
                            self.enableUserInputWhileSaving();
                        }
                        else if (result.resultCode == EditCustomerLightBox.ERROR_DELETED_CONCURRENTLY)
                        {
                            alert(messageResource.getMessage("customer.edit.error.concurrently_deleted"));
                            self.hide();
                            window.location.reload();
                        }
                    },
                    function (jqXHR, textStatus, errorThrown) {
                        self.enableUserInputWhileSaving();
                    }
        );
    };

    return this;
};

EditCustomerLightBox.prototype.switchToMainContent = function()
{
    this.onEscape = this.savedEscape;
    EditCPTPopupMoire.animateMoire(this.deletePopup, this.mainContent, this.moireInEl, this.moireOutEl);
};

EditCustomerLightBox.prototype.switchToConfirmContent = function()
{
    this.deletePopup.$el.height(this._popupEl.height());
    this.onEscape = _.bind(this.switchToMainContent, this);
    EditCPTPopupMoire.animateMoire(this.mainContent, this.deletePopup, this.moireInEl, this.moireOutEl);
};

EditCustomerLightBox.prototype.findElementBySubId = function(id)
{
    return this._popupEl.find('#' + this.getElementIdBySubId(id));
};

/* Status Selector */
EditCustomerLightBox.prototype.initStatusSelector = function ( value )
{
    var self = this;
    var statusSelectorItems = [
        {
            text: messageResource.getMessage("customer.edit.status.active"),
            value: true
        },
        {
            text: messageResource.getMessage("customer.edit.status.archived"),
            value: false
        }
    ];

    this.statusSelectorPlaceHolderEl.innerHTML = "";
    this.statusSelector = new ListSelector({
                                               menuCls: "customerStatusSelectorMenu",
                                               renderTo: this.statusSelectorPlaceHolderEl,
                                               value: value,
                                               listItems: statusSelectorItems
                                           });
    var handler = function (value) {
        $(self.statusSelectorPlaceHolderEl).removeClass("active archived");
        $(self.statusSelectorPlaceHolderEl ).addClass(value ? "active" : "archived");
        self.showWarningOnArchive && !value ? $(self.lightBoxEl).addClass("showArchiveWarning") : $(self.lightBoxEl).removeClass("showArchiveWarning");
    };
    handler(value);
    this.statusSelector.on( "change", function () { self.changesCallback(); handler(self.getStatus()); } );

    this.initArchiveWarning();
};
EditCustomerLightBox.prototype.initArchiveWarning = function () {
    var warningText = "";
    this.showWarningOnArchive = false;
    if (this.customerInfo.active && this.customerInfo.activeProjectsNum > 0)
    {
        this.showWarningOnArchive = true;
        var countActiveProject = this.customerInfo.activeProjectsNum;
        if (countActiveProject > 1){
            warningText += messageResource.getMessage("customer.edit.archive.warning.text.part1.ml", [countActiveProject]);
        }else
            warningText += messageResource.getMessage("customer.edit.archive.warning.text.part1", [countActiveProject]);

        var countActiveTasks = this.customerInfo.activeProjectsOpenTasksNum;
        if (countActiveTasks)
            if (countActiveTasks > 1 ){
                warningText += " " + messageResource.getMessage("customer.edit.archive.warning.text.part2.ml", [countActiveTasks]);
            }else
                warningText += " " + messageResource.getMessage("customer.edit.archive.warning.text.part2", [countActiveTasks]);

        $(this.getElementBySubId("archiveWarning")).find(".warningText").text(warningText);
    }
};
EditCustomerLightBox.prototype.getStatus = function ()
{
    return this.statusSelector.getSelectedItem().value;
};
/* End Status Selector */

EditCustomerLightBox.prototype.changesDetected = function()
{
    CustomerLightBox.prototype.changesDetected.call(this);
    this.footer.addClass( "changesDetected" );
};

EditCustomerLightBox.prototype.getDataToCommit = function()
{
    return [ this.customerInfo.id, this.getCustomerName(), this.getDescription(), this.getStatus() ];
};

EditCustomerLightBox.prototype.disableButtons = function ()
{
    CustomerLightBox.prototype.disableButtons.call(this);
    this._deleteEnabled = false;
};

EditCustomerLightBox.prototype.enableButtons = function ()
{
    CustomerLightBox.prototype.enableButtons.call(this);
    this._deleteEnabled = true;
};

/* methods to get data to construct EditDialogCustomerEntity for initializing delete confirm dialog (OperationConfirmDlg.js):
*   hiddenProjectsCount - amount of archived projects for active customers and projects page, amount of active projects for archived customers and projects page;
*   visibleProjectsCount - amount of active projects for active customers and projects page, amount of archived projects for archived customers and projects page.
* */
EditCustomerLightBox.prototype.getHiddenProjectCount = function () { };
EditCustomerLightBox.prototype.getVisibleProjectCount = function () { };