function setAllCheckboxesState(form, state) {
    for (i=0; i<form.elements.length; i++) {
        if (form.elements[i].name.indexOf('customerById[') != -1 || form.elements[i].name.indexOf('projectById[') != -1)
            form.elements[i].checked = state;
    }
}

function BaseConfirmController()
{
    this.initialize = function(dialog, onConfirmAction)
    {
        this.dialog = dialog;
        this.dialog.initWithDataAsConfirmDeletionDialog(this.isAttention(),
                                                this.getHeaderText(),
                                                this.getDeleteButtonsCaptions(),
                                                this.getDeletingItemsDescription(),
                                                this.getDeleteAttentionText(),
                                                onConfirmAction);
    };

    this.showDialog = function( doNotShowMoire )
    {
        this.dialog.open(doNotShowMoire);
    };

    this.getHeaderText = function()
    {
        //should override this method
    };

    this.getDeleteButtonCaptionKeyPrefix = function()
    {
        //should override this method
    };

    this.getDeleteButtonsCaptions = function()
    {
        var messageKeyPrefix = this.getDeleteButtonCaptionKeyPrefix();
        if( hasApprovedTimeTrack( this.customers ) || hasApprovedTimeTrack( this.projects ) )
        {
            return [ messageResource.getMessage( messageKeyPrefix ) + " " +
                     messageResource.getMessage( "delete.confirm.dialog.delete_btn.caption.and_revoke_weeks" ),
                     messageResource.getMessage( messageKeyPrefix ) + " " +
                     messageResource.getMessage( "delete.confirm.dialog.delete_btn.caption.and_keep_weeks" ) ]
        }
        else
            return [ messageResource.getMessage( messageKeyPrefix ) ];
    };

    this.getDeletingItemsDescription = function()
    {
        //should override this method
    };

    this.getDeleteAttentionMainText = function()
    {
        //should override this method
    };

    this.getDeleteAttentionText = function()
    {
        var result = this.getDeleteAttentionMainText();
        if( hasApprovedTimeTrack( this.customers ) || hasApprovedTimeTrack( this.projects ) )
            result += "<p>" + messageResource.getMessage( 'delete.confirm.dialog.has_approved_tt' ) + "</p>" +
                      "<p>" + messageResource.getMessage( 'delete.confirm.dialog.change_approved_weeks' ) + "</p>";
        return result;
    };

    this.isAttention = function()
    {
        //should override this method
    };

    this.hasManagerApprover = function()
    {
        //should override this method
    }
}

/////////////////////////////////////////// CustomersDeleteConfirmController ///////////////////////////////////////
function CustomersDeleteConfirmController(customers, hasManagerApprover)
{
    this.customers = customers;
    this._hasManagerApprover = hasManagerApprover;
}
CustomersDeleteConfirmController.prototype = new BaseConfirmController();

CustomersDeleteConfirmController.prototype.getHeaderText = function() {
    return messageResource.getMessageWithEscapedMacros('delete.confirm.dialog.confirm_deletion.' + (this.customers.length > 1 ? 'customers' : 'customer'));
};

CustomersDeleteConfirmController.prototype.getDeleteButtonCaptionKeyPrefix = function() {
    return "customers_projects.js.delete_btn.customer" + ( this.customers.length == 1 ? "" : "s" );
};

CustomersDeleteConfirmController.prototype.getDeletingItemsDescription = function() {
    var projectsDescription = getProjectsDescription(this.customers);
    if (this.customers.length == 1)
    {
        if (projectsDescription != null)
            return messageResource.getEscapedMessageNoEscapeArgs('delete.confirm.dialog.along_with_its.customer',
                                                            [ projectsDescription ]);
        else
            return messageResource.getEscapedMessage('delete.confirm.dialog.customer');
    }
    else
    {
        if (projectsDescription != null)
            return messageResource.getEscapedMessageNoEscapeArgs('delete.confirm.dialog.along_with_their.customers',
                                                            [ this.customers.length, projectsDescription ]);
        else
            return this.customers.length + ' ' + messageResource.getEscapedMessage('common.customers');
    }
};

CustomersDeleteConfirmController.prototype.getDeleteAttentionMainText = function() {
    if (hasTasks(this.customers))
    {
        var taskCount = calculateTasks(this.customers, []);

        var result = "<p>";
        result += messageResource.getMessageWithEscapedMacros('delete.confirm.dialog.projects_of_'+(this.customers.length == 1 ? "customer" : "customers")+'_contain_' + (taskCount == 1 ? "task" : "tasks"));
        if (hasTimeTrack(this.customers))
        {
            result += " ";
            result += taskCount == 1 ?
                           messageResource.getMessageWithEscapedMacros('delete.confirm.dialog.this_task_has_tt') :
                           messageResource.getMessageWithEscapedMacros('delete.confirm.dialog.some_tasks_have_tt');
        }
        result += "</p><p>";
        result += messageResource.getEscapedMessage('delete.confirm.dialog.loose_data_permanently.selected_' + (this.customers.length == 1 ? "customer" : "customers"));
        result += "</p>";
        return result;
    }
    else
    {
        return messageResource.getMessageWithEscapedMacros('delete.confirm.dialog.cp_will_be_deleted.'+ (this.customers.length == 1 ? 'customer' : 'customers'));
    }
};

CustomersDeleteConfirmController.prototype.isAttention = function() {
    return hasTasks(this.customers);
};

CustomersDeleteConfirmController.prototype.isAttention = function() {
    return this._hasManagerApprover;
};


/////////////////////////////////////////// ProjectsDeleteConfirmController ///////////////////////////////////////
function ProjectsDeleteConfirmController(projects, hasManagerApprover)
{
    this.projects = projects;
    this._hasManagerApprover = hasManagerApprover;
}
ProjectsDeleteConfirmController.prototype = new BaseConfirmController();

ProjectsDeleteConfirmController.prototype.getHeaderText = function() {
    return messageResource.getMessageWithEscapedMacros('delete.confirm.dialog.confirm_deletion.' +
                                      (this.projects.length > 1 ? 'projects' : 'project'));
};

ProjectsDeleteConfirmController.prototype.getDeleteButtonCaptionKeyPrefix = function() {
    return "customers_projects.js.delete_btn.project" + ( this.projects.length == 1 ? "" : "s" );
};

ProjectsDeleteConfirmController.prototype.getDeletingItemsDescription = function() {
    if (this.projects.length == 1)
        if (this.projects[0].taskCount > 0)
            return messageResource.getEscapedMessage('delete.confirm.dialog.along_with_its.project');
        else
            return messageResource.getEscapedMessage('delete.confirm.dialog.project');
    else
        return this.projects.length + ' ' + messageResource.getEscapedMessage('common.projects');
};

ProjectsDeleteConfirmController.prototype.getDeleteAttentionMainText = function() {
    if (hasTasks(this.projects))
    {
        var taskCount = calculateTasks([], this.projects);
        var taskText = taskCount == 1 ? 'task' : 'tasks';

        var result = "<p>";
        if (this.projects.length == 1)
            result += messageResource.getMessageWithEscapedMacros('delete.confirm.dialog.selected_cp_contains.' + taskText, [taskCount]);
        else
            result += messageResource.getMessageWithEscapedMacros('delete.confirm.dialog.selected_cp_contain.' + taskText, [taskCount]);

        if (hasTimeTrack(this.projects))
        {
            result += " ";
            result += taskCount == 1 ?
                           messageResource.getMessageWithEscapedMacros('delete.confirm.dialog.this_task_has_tt') :
                           messageResource.getMessageWithEscapedMacros('delete.confirm.dialog.some_tasks_have_tt');
        }
        result += "</p><p>";
        result += messageResource.getEscapedMessage('delete.confirm.dialog.loose_data_permanently.selected_' + ( this.projects.length == 1 ? "project" : "projects" ));
        result += "</p>";
        return result;
    }
    else
    {
        return messageResource.getMessageWithEscapedMacros('delete.confirm.dialog.cp_will_be_deleted.' + (this.projects.length == 1 ? 'project' : 'projects'));
    }
};

ProjectsDeleteConfirmController.prototype.isAttention = function() {
    return hasTasks(this.projects);
};

ProjectsDeleteConfirmController.prototype.hasManagerApprover = function() {
    return this._hasManagerApprover;
};


/////////////////////////////////////////// CustomersProjectsDeleteConfirmController ///////////////////////////////////////
function CustomersProjectsDeleteConfirmController(customers, projects, hasManagerApprover)
{
    this.customers = customers;
    this.projects = projects;
    this._hasManagerApprover = hasManagerApprover;
}
CustomersProjectsDeleteConfirmController.prototype = new BaseConfirmController();

CustomersProjectsDeleteConfirmController.prototype.getHeaderText = function() {
    return messageResource.getEscapedMessage('customers_projects.js.delete_header');
};

CustomersProjectsDeleteConfirmController.prototype.getDeleteButtonCaptionKeyPrefix = function() {
    return "customers_projects.js.delete_btn." + getCpLowerCaseText( this.customers, this.projects );
};

CustomersProjectsDeleteConfirmController.prototype.getDeletingItemsDescription = function() {
    var customersProjectsDescription = getProjectsDescription(this.customers);
    var customersText;
    
    if (customersProjectsDescription != null)
    {
        if (this.customers.length == 1)
            customersText = messageResource.getEscapedMessageNoEscapeArgs('delete.confirm.dialog.along_with_its.customer',
                                                       [ customersProjectsDescription ]);
        else
            customersText = messageResource.getEscapedMessageNoEscapeArgs('delete.confirm.dialog.along_with_their.customers',
                                                       [ this.customers.length, customersProjectsDescription ]);
    }
    else
    {
        customersText = this.customers.length + ' ' + messageResource.getEscapedMessage('common.' + (this.customers.length == 1 ? "customer" : "customers"));
    }

    var projectText = ' ' + messageResource.getEscapedMessage('delete.confirm.dialog.selected_project' + (this.projects.length == 1 ? '' : 's'));
    return messageResource.getMessage('common.smth_and_smth',
                                      [ customersText,
                                        this.projects.length + projectText ]);
};

CustomersProjectsDeleteConfirmController.prototype.getDeleteAttentionMainText = function() {
    var taskCount = calculateTasks(this.customers, this.projects);
    if (taskCount == 0)
    {
        return messageResource.getMessageWithEscapedMacros('delete.confirm.dialog.cp_will_be_deleted.' + getCpLowerCaseText(this.customers, this.projects));
    }
    else
    {
        var result = "<p>";
        var taskText = taskCount == 1 ? 'task' : 'tasks';
        result += messageResource.getMessageWithEscapedMacros('delete.confirm.dialog.selected_cp_contain.' + taskText, [taskCount]);

        if (hasTimeTrack(this.projects) || hasTimeTrack(this.customers))
        {
            result += " ";
            if (taskCount == 1)
                result += messageResource.getMessageWithEscapedMacros('delete.confirm.dialog.this_task_has_tt');
            else
                result += messageResource.getMessageWithEscapedMacros('delete.confirm.dialog.some_tasks_have_tt');
        }

        result += "</p><p>";
        result += messageResource.getEscapedMessage('delete.confirm.dialog.loose_data_permanently.selected_' + getCpLowerCaseText(this.customers, this.projects));
        result += "</p>";
        return result;
    }
};

CustomersProjectsDeleteConfirmController.prototype.isAttention = function() {
    return hasTasks(this.customers) || hasTasks(this.projects);
};

CustomersProjectsDeleteConfirmController.prototype.hasManagerApprover = function() {
    return this._hasManagerApprover;
};


function createConfirmationMessage(form, customerMessage, projectMessage, customerProjectMessage) {
    var msg;
    var customers = false;
    var projects = false;

    for (var i=0; i<form.elements.length; i++) {
        if (form.elements[i].name.indexOf('customerById[') != -1 && form.elements[i].checked)
            customers = true;
        else if (form.elements[i].name.indexOf('projectById[') != -1 && form.elements[i].checked)
            projects = true;

        if (customers && projects) break;
    }

    if (!customers && !projects)
        return null;

    if (customers) {
        msg = customerMessage;
        if (projects)
            msg = customerProjectMessage;
    } else {
        if (projects)
            msg = projectMessage;
    }

    return msg;
}

function getSelectedCustomers(form)
{
    return getSelectedEntities(form, /customerById\[(\d+)\]/, CPTManagement.Store.getCustomers());
}

function getSelectedProjects(form)
{
    return getSelectedEntities(form, /projectById\[(\d+)\]/, CPTManagement.Store.getProjects());
}

function getSelectedEntities(form, regexp, entities)
{
    var els = form.elements;
    var selectedEntities = [];
    for (var i = 0; i < els.length; i++)
    {
        if (els[i].name != null && els[i].name != "" && regexp.test(els[i].name))
        {
            if (els[i].checked)
            {
                var entityId = regexp.exec(els[i].name)[1];
                selectedEntities.push(entities[entityId]);
            }
        }
    }
    return selectedEntities;
}

// function Entity(id, taskCount, hasManagerTimeTrack, hasOtherUsersTimeTrack, hasManagerApprovedTimeTrack, hasOtherUsersApprovedTimeTrack, isActive)
// {
//     this.id = id;
//     this.taskCount = taskCount;
//     this.hasManagerTimeTrack = hasManagerTimeTrack;
//     this.hasOtherUsersTimeTrack = hasOtherUsersTimeTrack;
//     this.hasManagerApprovedTimeTrack = hasManagerApprovedTimeTrack;
//     this.hasOtherUsersApprovedTimeTrack = hasOtherUsersApprovedTimeTrack;
//     this.isActive = isActive;
//
//     var children = [];
//     this.addChild = function(id) {
//         children.push(id);
//     };
//
//     this.getChildren = function() {
//         return children;
//     };
//
//     this.isContainChild = function(id) {
//         return isArrayContains(children, id);
//     };
//
//     this.hasChild = function() {
//         return children.length > 0;
//     };
//
//     this.getChildCount = function() {
//         return children.length;
//     };
//
//     this.toString = function()
//     {
//         return "id=" + this.id +
//                " taskCount=" + this.taskCount +
//                " hasManagerTimeTrack=" + this.hasManagerTimeTrack +
//                " hasOtherUsersTimeTrack=" + this.hasOtherUsersTimeTrack +
//                " hasManagerApprovedTimeTrack=" + this.hasManagerApprovedTimeTrack +
//                " hasOtherUsersApprovedTimeTrack=" + this.hasOtherUsersApprovedTimeTrack;
//     }
// }
//
// function CustomerEntity(id, taskCount, hasManagerTimeTrack, hasOtherUsersTimeTrack, hasManagerApprovedTimeTrack,
//                         hasOtherUsersApprovedTimeTrack, isActive, hiddenProjectCount)
// {
//     this.base = Entity;
//     this.base(id, taskCount, hasManagerTimeTrack, hasOtherUsersTimeTrack, hasManagerApprovedTimeTrack,
//               hasOtherUsersApprovedTimeTrack, isActive);
//     this.hiddenProjectCount = hiddenProjectCount;
// }
//
// CustomerEntity.prototype = new Entity();
//
// function EditDialogCustomerEntity(id, taskCount, hasManagerTimeTrack, hasOtherUsersTimeTrack, hasManagerApprovedTimeTrack,
//                                   hasOtherUsersApprovedTimeTrack, isActive, hiddenProjectCount, visibleProjectCount)
// {
//     this.baseCustomerEntity = CustomerEntity;
//     this.baseCustomerEntity(id, taskCount, hasManagerTimeTrack, hasOtherUsersTimeTrack, hasManagerApprovedTimeTrack,
//                             hasOtherUsersApprovedTimeTrack, isActive, hiddenProjectCount);
//
//     this.visibleProjectCount = visibleProjectCount;
//     this.getChildCount = function () {
//         return this.visibleProjectCount;
//     }
// }
//
// EditDialogCustomerEntity.prototype = new CustomerEntity();

function hasTasks(entities)
{
    for (var i = 0; i < entities.length; i++)
    {
        if (entities[i].taskCount > 0)
            return true;
    }
    return false;
}

function checkSelectionForDelete(selectedCustomers, selectedProjects, isShowProjects)
{
    if (selectedCustomers.length == 0 && selectedProjects.length == 0)
    {
        var deletingItems;
        if (isShowProjects) {
            deletingItems = 'customers_and_projects';
        }
        else {
            deletingItems = 'customers';
        }
        alert(messageResource.getMessage('customers_projects.js.please_select_to_delete.' + deletingItems ));
        return false;
    }
    return true;
}

function getCpLowerCaseText(customers, projects)
{
    var str = "";
    if( customers.length == 1 )
        str += "customer";
    else if( customers.length > 1 )
        str += "customers";

    if( projects.length > 0 && customers.length > 0 )
        str += "_and_";

    if( projects.length == 1 )
        str += "project";
    else if( projects.length > 1 )
        str += "projects";

    return str;
}

function createDialogController(customers, projects, hasManagerApprover) {
    if (customers.length == 0)
        return new ProjectsDeleteConfirmController(projects, hasManagerApprover);

    if (projects.length == 0)
        return new CustomersDeleteConfirmController(customers, hasManagerApprover);

    var filteredProjects = filterCustomersProjects(customers, projects);
    if (filteredProjects.length == 0)
        return new CustomersDeleteConfirmController(customers, hasManagerApprover);

    return new CustomersProjectsDeleteConfirmController(customers, filteredProjects, hasManagerApprover);
}

function filterCustomersProjects(customers, projects) {
    var filteredProjects = [];
    for (var i = 0; i < projects.length; i++)
    {
        if (!containChild(customers, projects[i].id))
            filteredProjects.push(projects[i]);
    }
    return filteredProjects;
}

// parameter [doNotShowMoire] is optional.
// if it is not defined, confirm delete dialog will be opened with moire and will hide moire while closing.
function showConfirmDeleteDlg(selectedCustomers, selectedProjects, hasManagerApprover, deleteAction, doNotShowMoire)
{
    var dialogController = createDialogController(selectedCustomers, selectedProjects, hasManagerApprover);
    dialogController.initialize(operationConfirmDlg, deleteAction);
    dialogController.showDialog(doNotShowMoire);
}

function hasTimeTrack(entities)
{
    for (var i = 0; i < entities.length; i++)
    {
        if (entities[i].hasManagerTimeTrack || entities[i].hasOtherUsersTimeTrack)
            return true;
    }
    return false;
}

function hasApprovedTimeTrack(entities)
{
    if( !entities ) return false;
    for( var i = 0; i < entities.length; i++ )
    {
        if (entities[i].hasOtherUsersApprovedTimeTrack)
            return true;
    }
    return false;
}

function containChild(customers, projectId)
{
    for (var i = 0; i < customers.length; i++)
    {
        if (customers[i].isContainChild(projectId))
            return true;
    }
    return false;
}

function calculateTasks(customers, projects)
{
    var taskCount = 0;
    for (var i = 0; i < projects.length; i++)
    {
        taskCount += projects[i].taskCount;
    }

    for (i = 0; i < customers.length; i++)
    {
        taskCount += customers[i].taskCount;
    }

    return taskCount;
}

function getProjectsDescription(customers)
{
    var activeProjectsCount = calculateActiveProjects(customers);
    var archivedProjectsCount = calculateArchivedProjects(customers);

    if (activeProjectsCount == 0 && archivedProjectsCount == 0)
        return null;

    var bracketText;
    if (activeProjectsCount == 0)
        bracketText = messageResource.getMessage('common.archived_multiple',[archivedProjectsCount]);
    else if (archivedProjectsCount == 0)
        bracketText = messageResource.getMessage('common.active_multiple', [activeProjectsCount]);
    else
        bracketText = messageResource.getMessage('common.active_and_archived_multiple', [activeProjectsCount, archivedProjectsCount]);

    bracketText = '<nobr>' + bracketText + '</nobr>';
    return  bracketText;
}

//should associate these vars with appropriate functions (different for C&P and Archives interfacers)
var calculateActiveProjects;
var calculateArchivedProjects;

function calculateHiddenProjects(customers)
{
    var count = 0;
    for (var i = 0; i < customers.length; i++)
    {
        count += customers[i].hiddenProjectCount;
    }
    return count;
}

function calculateChildProjects(customers)
{
    var count = 0;
    for (var i = 0; i < customers.length; i++)
    {
        count += customers[i].getChildCount();
    }
    return count;
}

function editCustomer( customerId ) {
    loadPopupContent(CustomerProjectConstants.editCustomerUrl + '?customerId=' + customerId,
                     function() { getShowEditCustomerPopupFunction()(showConfirmDeleteDlg, hasManagerApprover); },
                     true,
                     function(status, responseText) {
                         if (status == CustomerManagementConstants.CUSTOMER_CONCURRENTLY_DELETED) {
                             alert(messageResource.getMessage("customer.edit.error.concurrently_deleted"));
                             window.location.reload();
                             return true;
                         }
                     },
                     true
    );
}

function editProject(projectId){
    loadPopupContent( CustomerProjectConstants.editProjectUrl + '?projectId=' + projectId, function() { showEditProjectPopup(); }, true,
                      function(status, responseText) {
                          if (status == ProjectManagementConstants.PROJECT_CONCURRENTLY_DELETED) {
                              alert(messageResource.getMessage("project.edit.no_such_project"));
                              window.location.reload();
                              return true;
                          }
                      }, true);
}