prepareNameAsArray.defaultNameMaxLength = 43;


/**
 * Prepares name for displaying - constructs array of name parts using input params.
 *
 * @param active - is true when item is active
 * @param maxlength - max length of name, if name greater than max length then it is cut and followed by '...'
 * @param showArchivedLabels - when it is true name part with archived suffix is added
 * @param archivedText - text is added to name parts when user is archived
 * @param considerArchivedLabel - if true, then the length of the archived label will be
 * considered when truncating
 * @returns array with parts of name. 
 */
function prepareNameAsArray(name, active, maxlength, showArchivedLabels, archivedText, considerArchivedLabel) {
    var text, suffix;

    suffix = "";
    
    if (!maxlength)
        maxlength = prepareNameAsArray.defaultNameMaxLength;

    if (!archivedText)
        archivedText = '(archived)';

    if (considerArchivedLabel && showArchivedLabels && !active)
    {
        maxlength -= archivedText.length;
    }

    if (name.length > maxlength)
        text = name.substring(0, maxlength) + '...';
    else
        text = name;

    if (showArchivedLabels && !active)
        suffix +=  archivedText;

    result = [];

    result[0] = text;
    
    if (suffix.length > 0)
        result[1] = suffix;

    return result;
}

/**
 * same as prepareNameAsArray but result is a string where parts are separated with space
 */
function prepareName(name, active, maxlength, showArchivedLabels, archivedText, considerArchivedLabel) {
    var arr = prepareNameAsArray(name, active, maxlength, showArchivedLabels, archivedText, considerArchivedLabel);
    var result = arr[0];
    for( var i = 1; i < arr.length; i++ )
        result += " " + arr[i];
    return result;
}

/**
 * Renders string array with name parts in plain text string;
 * @param nameParts
 */
function getSimpleTextNameFromArray(nameParts)
{
    if (nameParts == null) return "";

    return nameParts.join(' ');
}

function Customer(id, name, active, truncateStrings, nameMaxLength) {
    this.id = id;
    //TODO code below is a support of legacy code
    this.name = (truncateStrings) ? prepareName(name, active, nameMaxLength, true) : name;
    this.fullName = name;
    this.active = active;
    this.archived = !active;
}

function Project(id, name, active, customerId, truncateStrings, nameMaxLength) {
    this.id = id;
    //TODO code below is a support of legacy code
    this.name = (truncateStrings) ? prepareName(name, active, nameMaxLength, true) : name;
    this.active = active;
    this.archived = !active;
    this.customerId = customerId;
}

function loadCustomers(form, withoutFirstOption) {
    var activeOnly = form.customerProjectStatus[0].checked;

    var customersSelect = form.customerId;
    if (!customersSelect) return;

    var selectedValue;
    if (customersSelect.selectedIndex >= 0)
        selectedValue = customersSelect.options[customersSelect.selectedIndex].value;

    var customerOptions = [];
    if (!withoutFirstOption) {
        if (activeOnly)
            customerOptions[0] = {name: '-- ' + messageResource.getMessage("customers_projects.All_active_customers_filter_value") + ' --', value: 0};
        else
            customerOptions[0] = {name: '-- ' + messageResource.getMessage("customers_projects.All_customers_filter_value") + ' --', value: 0};
    }

    for (i=0; i<customers.length; i++) {
        if (activeOnly && customers[i].active || !activeOnly) {
            customerOptions[customerOptions.length] = { name: customers[i].name, value: customers[i].id};
        }
    }

    var startFrom = 0;
    if (withoutFirstOption)
        startFrom = 1;
    fillInSelect(customersSelect, customerOptions, startFrom);
    selectOptionWithValue(customersSelect, selectedValue);
}

function loadProjects(form, withoutFirstOption) {
    var activeOnly = form.customerProjectStatus[0].checked;
    var customerId = form.customerId.options[form.customerId.selectedIndex].value;
    var options = getCustomerProjectsOptions(customerId, activeOnly, withoutFirstOption);

    var selectedValue;
    if (form.projectId.selectedIndex >= 0)
        selectedValue = form.projectId.options[form.projectId.selectedIndex].value;

    fillInSelect(form.projectId, options, 0);

    selectOptionWithValue(form.projectId, selectedValue);
}

function fillInSelect(select, options, startFrom, selectedValue) {
    select.options.length = startFrom;

    for (i=0; i<options.length; i++) {
        var optionName = options[i].name;
        if (options[i].name.length >= 71)
            optionName = options[i].name.substring(0, 70) + '...';
        if(typeof selectedValue != typeof undefined && selectedValue == options[i].value){
            select.options[startFrom+i] = new Option(optionName, options[i].value, true, true);
        } else {
            select.options[startFrom+i] = new Option(optionName, options[i].value);
        }
    }

}

function clearSelect(select) {
    select.options.length = 0;
}

function getCustomerProjectsOptions(customerId, activeOnly, withoutFirstOption) {
    var result = [];
    if (!withoutFirstOption) {
        if (activeOnly)
            result[0] = {name: '-- ' + messageResource.getMessage("customers_projects.All_active_projects_filter_value") + ' --', value: 0};
        else
            result[0] = {name: '-- ' + messageResource.getMessage("customers_projects.All_projects_filter_value") + ' --', value: 0};
    }

    for (i=0; i<projects.length; i++) {
        if (activeOnly) {
            if ((customerId <= 0 || projects[i].customerId == customerId) && projects[i].active)
                result[result.length] = {name: projects[i].name, value: projects[i].id};
        } else {
            if (customerId <= 0 || projects[i].customerId == customerId) {
                result[result.length] = {name: projects[i].name, value: projects[i].id};
            }
        }
    }

    return result;
}

function selectOptionWithValue(select, optionValue) {
    if (!optionValue) return;

    for (i=0; i<select.options.length; i++) {
        if (optionValue == select.options[i].value) {
            select.options[i].selected = true;
            break;
        }
    }
}

function selectCustomer(form, customerId) {
    if (customerId <= 0)
        form.customerId.selectedIndex = 0;
    else
        selectOptionWithValue(form.customerId, customerId);

    loadProjects(form);
}

function findProjectCustomerId(projectId) {
    for (i=0; i<projects.length; i++) {
        if (projects[i].id == projectId)
            return projects[i].customerId;
    }

    return -1;
}

function customerHasProjects(customerId) {
    for (i=0; i<projects.length; i++) {
        if (projects[i].customerId == customerId)
            return true;
    }

    return false;
}

