/**
 * This classed is used to manage pair of fields - predefined select and custom edit field.
 */
function CustomizableField(baseName)
{
    this.baseName = baseName;
}

/**
 * @returns String custom field name based on base name.
 */
CustomizableField.prototype.getCustomFieldName = function()
{
  return this.baseName + "Custom";
};

/**
 * @returns String predefined field name based on base name.
 */
CustomizableField.prototype.getPredefinedFieldName = function()
{
  return this.baseName + "Predefined";
};

/**
 * Gets custom field value - if it is not custom, so get predefined.
 */
CustomizableField.prototype.getValue = function()
{
    var custom = document.getElementById(this.getCustomFieldName());
    var predefined = document.getElementById(this.getPredefinedFieldName());

    if (predefined.value == SystemSettings.CUSTOM_VALUE_STRING)
        return custom.value;

    return predefined.value;
};

/**
 * Validates state of customizable field (select+input). It must be called after each predefined selec input control
 * value change. Also it updates format preview.
 */
CustomizableField.prototype.validateField = function()
{
    var predefined = document.getElementById(this.getPredefinedFieldName());
    var custom = document.getElementById(this.getCustomFieldName());

    if ( (predefined.selectedIndex < 0) || (predefined.selectedIndex == predefined.options.length - 1) )
    {
        predefined.selectedIndex = predefined.options.length - 1;
        custom.style.display = "block";
    }
    else
    {
        custom.style.display = "none";
    }
};

/**
 * Initializes the customizable field set with value.
 * @param value - value to set into field set.
 */
CustomizableField.prototype.initField = function(value)
{
    var predefined = document.getElementById(this.getPredefinedFieldName());
    var custom = document.getElementById(this.getCustomFieldName());

    for (var i=0; i<predefined.options.length; i++)
    {
        if (predefined.options[i].value == value)
        {
            predefined.selectedIndex = i;
            predefined.options[i].defaultSelected = true;
            custom.style.display = "none";
            custom.value="";
            custom.defaultValue="";
            return;
        }
    }

    custom.defaultValue = custom.value;
    custom.style.display = "block";

    predefined.selectedIndex = predefined.options.length - 1;
    predefined.options[predefined.selectedIndex].defaultSelected = true;
};
