<%--

This file defines many useful date/time members for Date object.
It should be loaded after the ExtJS library as it's based on its Date object.


Format  Description                                                               Example returned values
------  -----------------------------------------------------------------------   -----------------------
  d     Day of the month, 2 digits with leading zeros                             01 to 31
  D     A short textual representation of the day of the week                     Mon to Sun
  j     Day of the month without leading zeros                                    1 to 31
  l     A full textual representation of the day of the week                      Sunday to Saturday
  N     ISO-8601 numeric representation of the day of the week                    1 (for Monday) through 7 (for Sunday)
  S     English ordinal suffix for the day of the month, 2 characters             st, nd, rd or th. Works well with j
  w     Numeric representation of the day of the week                             0 (for Sunday) to 6 (for Saturday)
  z     The day of the year (starting from 0)                                     0 to 364 (365 in leap years)
  W     ISO-8601 week number of year, weeks starting on Monday                    01 to 53
  F     A full textual representation of a month, such as January or March        January to December
  m     Numeric representation of a month, with leading zeros                     01 to 12
  M     A short textual representation of a month                                 Jan to Dec
  n     Numeric representation of a month, without leading zeros                  1 to 12
  t     Number of days in the given month                                         28 to 31
  L     Whether it's a leap year                                                  1 if it is a leap year, 0 otherwise.
  o     ISO-8601 year number (identical to (Y), but if the ISO week number (W)    Examples: 1998 or 2004
        belongs to the previous or next year, that year is used instead)
  Y     A full numeric representation of a year, 4 digits                         Examples: 1999 or 2003
  y     A two digit representation of a year                                      Examples: 99 or 03
  a     Lowercase Ante meridiem and Post meridiem                                 am or pm
  A     Uppercase Ante meridiem and Post meridiem                                 AM or PM
  g     12-hour format of an hour without leading zeros                           1 to 12
  G     24-hour format of an hour without leading zeros                           0 to 23
  h     12-hour format of an hour with leading zeros                              01 to 12
  H     24-hour format of an hour with leading zeros                              00 to 23
  i     Minutes, with leading zeros                                               00 to 59
  s     Seconds, with leading zeros                                               00 to 59
  u     Decimal fraction of a second                                              Examples:
        (minimum 1 digit, arbitrary number of digits allowed)                     001 (i.e. 0.001s) or
                                                                                  100 (i.e. 0.100s) or
                                                                                  999 (i.e. 0.999s) or
                                                                                  999876543210 (i.e. 0.999876543210s)
  O     Difference to Greenwich time (GMT) in hours and minutes                   Example: +1030
  P     Difference to Greenwich time (GMT) with colon between hours and minutes   Example: -08:00
  T     Timezone abbreviation of the machine running the code                     Examples: EST, MDT, PDT ...
  Z     Timezone offset in seconds (negative if west of UTC, positive if east)    -43200 to 50400
  c     ISO 8601 date
        Notes:                                                                    Examples:
        1) If unspecified, the month / day defaults to the current month / day,   1991 or
           the time defaults to midnight, while the timezone defaults to the      1992-10 or
           browser's timezone. If a time is specified, it must include both hours 1993-09-20 or
           and minutes. The "T" delimiter, seconds, milliseconds and timezone     1994-08-19T16:20+01:00 or
           are optional.                                                          1995-07-18T17:21:28-02:00 or
        2) The decimal fraction of a second, if specified, must contain at        1996-06-17T18:22:29.98765+03:00 or
           least 1 digit (there is no limit to the maximum number                 1997-05-16T19:23:30,12345-0400 or
           of digits allowed), and may be delimited by either a '.' or a ','      1998-04-15T20:24:31.2468Z or
        Refer to the examples on the right for the various levels of              1999-03-14T20:24:32Z or
        date-time granularity which are supported, or see                         2000-02-13T21:25:33
        http://www.w3.org/TR/NOTE-datetime for more info.                         2001-01-12 22:26:34
  U     Seconds since the Unix Epoch (January 1 1970 00:00:00 GMT)                1193432466 or -2138434463
  M$    Microsoft AJAX serialized dates                                           \/Date(1238606590509)\/ (i.e. UTC milliseconds since epoch) or
                                                                                  \/Date(1238606590509+0800)\/


--%><%@ page import="com.actimind.actitime.util.messages.ATMessageResources"%>
<%@page import="java.util.Locale"%><%@
page import="static com.actimind.actitime.util.messages.ATMessageResources.message"%><%@ page import="com.actimind.actitime.Keys"%><%@ page import="com.actimind.common.Product"%><%@
page contentType="text/javascript;"%><%
    com.actimind.actitime.util.ResponseUtil.enableJSPCaching( response  );
%><%!
    private String getJSString( String key, HttpServletRequest req )
    {
        return "\"" + ATMessageResources.message( (Product) req.getAttribute( Keys.PRODUCT ), Locale.US, key ) + "\"";
    }
    private String ws( String weekDayName, HttpServletRequest req ) { return getJSString( "week.short." + weekDayName, req ); }
    private String wsc( String weekDayName, HttpServletRequest req ) { return getJSString( "week.short.capital." + weekDayName, req ); }
    private String wf( String weekDayName, HttpServletRequest req ) { return getJSString( "week.full." + weekDayName, req ); }
    private String wa( String weekDayName, HttpServletRequest req ) { return getJSString( "week.abbr." + weekDayName, req ); }
    private String mf( String monthName, HttpServletRequest req ) { return getJSString( "month.full." + monthName, req ); }
    private String ms( String monthName, HttpServletRequest req ) { return getJSString( "month.short." + monthName, req ); }
%><% HttpServletRequest r = request; %>
Date.dayNames = [<%=wf("sun",r)%>,<%=wf("mon",r)%>,<%=wf("tue",r)%>,<%=wf("wed",r)%>,<%=wf("thu",r)%>,<%=wf("fri",r)%>,<%=wf("sat",r)%>];
Date.dayNamesShort = [<%=ws("sun",r)%>,<%=ws("mon",r)%>,<%=ws("tue",r)%>,<%=ws("wed",r)%>,<%=ws("thu",r)%>,<%=ws("fri",r)%>,<%=ws("sat",r)%>];
Date.dayNamesShortCapital = [<%=wsc("sun",r)%>,<%=wsc("mon",r)%>,<%=wsc("tue",r)%>,<%=wsc("wed",r)%>,<%=wsc("thu",r)%>,<%=wsc("fri",r)%>,<%=wsc("sat",r)%>];
Date.dayNamesAbbr = [<%=wa("sun",r)%>,<%=wa("mon",r)%>,<%=wa("tue",r)%>,<%=wa("wed",r)%>,<%=wa("thu",r)%>,<%=wa("fri",r)%>,<%=wa("sat",r)%>];
Date.getShortDayName = function (i) { return Date.dayNamesShort[i]; };
Date.getDayNameAbbr = function (i) { return Date.dayNamesAbbr[i]; };
Date.monthNames = [<%=mf("jan",r)%>,<%=mf("feb",r)%>,<%=mf("mar",r)%>,<%=mf("apr",r)%>,<%=mf("may",r)%>,<%=mf("jun",r)%>,<%=mf("jul",r)%>,<%=mf("aug",r)%>,<%=mf("sep",r)%>,<%=mf("oct",r)%>,<%=mf("nov",r)%>,<%=mf("dec",r)%>];
Date.monthNamesShort = [<%=ms("jan",r)%>,<%=ms("feb",r)%>,<%=ms("mar",r)%>,<%=ms("apr",r)%>,<%=ms("may",r)%>,<%=ms("jun",r)%>,<%=ms("jul",r)%>,<%=ms("aug",r)%>,<%=ms("sep",r)%>,<%=ms("oct",r)%>,<%=ms("nov",r)%>,<%=ms("dec",r)%>];
Date.getShortMonthName = function (i) { return Date.monthNamesShort[i]; };
Date.monthNumbers = {
<%=message(request, "month.short.jan")%>: 0,
<%=message(request, "month.short.feb")%>: 1,
<%=message(request, "month.short.mar")%>: 2,
<%=message(request, "month.short.apr")%>: 3,
<%=message(request, "month.short.may")%>: 4,
<%=message(request, "month.short.jun")%>: 5,
<%=message(request, "month.short.jul")%>: 6,
<%=message(request, "month.short.aug")%>: 7,
<%=message(request, "month.short.sep")%>: 8,
<%=message(request, "month.short.oct")%>: 9,
<%=message(request, "month.short.nov")%>: 10,
<%=message(request, "month.short.dec")%>: 11
};

Date.JSON_DATE_FORMAT = "Y-m-d";
Date.JSON_TIME_FORMAT = "Y-m-d H:i:s.u O";
Date.JSON_TIME_FORMAT_WITHOUT_TIME_ZONE = "Y-m-d H:i:s.u";

Date.prototype.toJSONString = function()
{
    return this.toJSON();
};

Date.prototype.toJSON = function()
{
    return this.format( Date.JSON_DATE_FORMAT );
};

Date.parseJSONDate = function( dateStr )
{
    return Date.parseDate(dateStr.substring(0, 10), Date.JSON_DATE_FORMAT, true ).truncateTime();
};

Date.parseJSONTime = function( timeStr )
{
    return Date.parseDate(timeStr, Date.JSON_TIME_FORMAT, true );
};

Date.parseJSONTimeWithoutTimeZone = function( timeStr )
{
    return Date.parseDate(timeStr.substring(0, timeStr.length - 6), Date.JSON_TIME_FORMAT_WITHOUT_TIME_ZONE, true);
};

String.prototype.parseJSONDate = function()
{
    return Date.parseJSONDate(this);
};

String.prototype.parseJSONTime = function()
{
    return Date.parseJSONTime(this);
};

Date.prototype.truncateTime = function(clone)
{
    if (clone)
        return this.clone().truncateTime();

    // clear time
    this.setHours(12);
    this.setMinutes(0);
    this.setSeconds(0);
    this.setMilliseconds(0);

    return this;
};

Date.create = function( y, m, d, h, mi, s, ms )
{
    if( !h ) h = 12;
    if( !mi ) mi = 0;
    if( !s ) s = 0;
    if( !ms ) ms = 0;
    return new Date(y, m, d, h, mi, s, ms);
};

Date.prototype.isSameDate = function( d )
{
    return this.getDate() == d.getDate() &&
           this.getMonth() == d.getMonth() &&
           this.getFullYear() == d.getFullYear();
};

Date.prototype.offsetInDaysFrom = function(d)
{
    var date1 = Date.UTC( d.getFullYear(), d.getMonth(), d.getDate() );
    var date2 = Date.UTC( this.getFullYear(), this.getMonth(), this.getDate() );
    var ms = date2 - date1;
    return Math.floor( ms / 1000/60/60/24 ); //floor should be unnecessary, but just in case
};

Date.max = function( d1, d2 )
{
    return d1.getTime() > d2.getTime() ? d1 : d2;
};

Date.min = function( d1, d2 )
{
    return d1.getTime() > d2.getTime() ? d2 : d1;
};

Date.prototype.nextDay = function ()
{
    return Date.create( this.getFullYear(), this.getMonth(), this.getDate() + 1);
};

Date.prototype.prevDay = function ()
{
    return Date.create( this.getFullYear(), this.getMonth(), this.getDate() - 1);
};

Date.prototype.getNDaysBefore = function (n)
{
    return Date.create( this.getFullYear(), this.getMonth(), this.getDate() - n);
};

Date.prototype.getNDaysAfter = function (n)
{
    return Date.create( this.getFullYear(), this.getMonth(), this.getDate() + n);
};

Date.prototype.getWeekStartDate = function ( weekStartDay )
{
    return this.getNDaysBefore( (this.getDay() - weekStartDay + 7) % 7 );
};

/**
 * Converts Smart date format (see DateFormat class) to ExtJS date
 * format (see Date.parseDate).
 * <br/>
 * The implementation is extremely trivial and can cause rather weird
 * results.
 *
 * @param smartDateFormat
 */
Date.convertSmartToExtDateFormat = function( smartDateFormat )
{
    var extDateFormat = smartDateFormat;
    extDateFormat = extDateFormat.replace(/MMMM/, "F");
    extDateFormat = extDateFormat.replace(/MMM/, "M");
    extDateFormat = extDateFormat.replace(/MM/, "m");
    extDateFormat = extDateFormat.replace(/dd/, "d");
    extDateFormat = extDateFormat.replace(/yyyy/, "Y");
    extDateFormat = extDateFormat.replace(/yy/, "y");
    return extDateFormat;
};


function DatesIterator( dateFrom, dateTo )
{
    this.yFrom = dateFrom.getFullYear();
    this.mFrom = dateFrom.getMonth();
    this.dFrom = dateFrom.getDate();
    this.nextDate = Date.create(this.yFrom, this.mFrom, this.dFrom);
    this.dateTo = dateTo;
}

DatesIterator.prototype.next = function()
{
    var result = this.nextDate;
    this.dFrom++;
    this.nextDate = Date.create(this.yFrom, this.mFrom, this.dFrom);
    return result;
};

DatesIterator.prototype.hasNext = function()
{
    return this.dateTo && this.nextDate <= this.dateTo;
};

function MonthIterator(dateFrom, dateTo)
{
    this.yFrom = dateFrom.getFullYear();
    this.mFrom = dateFrom.getMonth();
    this.nextDate = new Date(this.yFrom, this.mFrom);
    this.dateTo = dateTo;
}

MonthIterator.prototype.next = function()
{
    var result = this.nextDate;
    this.mFrom++;
    this.nextDate = new Date(this.yFrom, this.mFrom);
    return result;
};
MonthIterator.prototype.hasNext = function()
{
    return this.dateTo && this.nextDate <= this.dateTo;
};