var months = new Array("jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec");

function parseDate( str, formatHint )
{
	if( typeof str != "string" )
		return null

	// convert the string to lower case
	str = str.toLowerCase();

	// split the string into array of tokens
	var tokens = str.match( /[0-9]+|[a-z]+/g );

    // if we've got less than 2 tokens or no tokens at all
	// then give up
	if( tokens == null || tokens.length < 2 )
		return null;

	var ambiguousTokens = new Array();

	var month = -1;
	var monthTokenIdx = -1;
	var year = -1;

	// if we have exactly two tokens they are considered as day and month -
	// the year should be current
	if( tokens.length == 2 )
		year = (new Date()).getFullYear();

	// consider not more than 3 tokens
	var tokensCount = Math.min( tokens.length, 3 );

	// we read not more than 3 valid tokens
	for( var i = 0; i < tokensCount; i++ )
	{
        var intValue = parseInt( tokens[i], 10 );

		// is it not a number?
		if( isNaN( intValue ) )
		{
			if( month != -1 ) // was month already found?
				return null;

			// match the month by 3 staring letters
			for (var j = 0; j < months.length; j++)
			{
				if( tokens[i].indexOf( months[j] ) == 0 )
				{
					month = j+1;
					monthTokenIdx = i+1;
					break; // month was found
				}
			}

			// if month was not recognized then we have an invalid token, so give up
			if( month == -1 )
				return null;
		}
		else
		{
			// do not accep zeros
			if( intValue < 1 )
				return null;

			if( intValue > 31 ) // is it a year?
			{
				if( i == 1 ) // year cannot be at the middle
					return null;

				if( year != -1 ) // was year already found?
					return null;

				year = intValue; // we've found year
			}
			else // we have an ambiguous token, let's process it later
			{
				ambiguousTokens[ambiguousTokens.length] = intValue;
			}
		}
	}

    var hint = null;
    if( typeof formatHint == "string" )
	{
		formatHint = formatHint.toUpperCase();
		hint = new Array( formatHint.indexOf('DD'), formatHint.indexOf('MM'), formatHint.indexOf('YY') );
		//if( hint[0] == -1 || hint[1] == -1 || hint [2] == -1 )
		//	hint = null;
	}

    switch( ambiguousTokens.length )
	{
		case 0: // there can't be zero ambiguities because at least day is ambiguous at this stage
			codeBug();
		case 1: // the day is ambiguous here
			return createDate( ambiguousTokens[0], month, year);
		case 2: // there can be an ambiguity between day and month OR between day and year
			if( year != -1 ) // An ambiguity between day and month
				return resolveDayMonthAmbiguity(
						ambiguousTokens[ 0 ],
						ambiguousTokens[ 1 ],
						year,
						hint
				);
			else if( month != -1 )  // An ambiguity between day and year
				return resolveDayYearAmbiguity(
						ambiguousTokens[ 0 ],
						ambiguousTokens[ 1 ],
						month,
						monthTokenIdx,
						hint
				);
			else codeBug(); // there can't be other ambiguities here
		default: // fully ambiguous situation
			return resolveTotalAmbiguity(
					ambiguousTokens[ 0 ],
					ambiguousTokens[ 1 ],
					ambiguousTokens[ 2 ],
					hint
			);
	}
}


function resolveDayMonthAmbiguity( first, second, year, hint )
{
    if( first > 12 )
	{
		return second > 12 ?
			   null : // there are no month at all (both tokens are greater than 12)
			   createDate( first, second, year ); // ok, first is day
	}
	else if( second > 12 )
	{
		return createDate( second, first, year ); // ok, second is day
	}

	if( hint[0] == -1 || hint[1] == -1)
		return null;

	// still ambigious situation. Let's use the hint
	return hint[1] > hint[0] ?
	       createDate( first, second, year ) :
		   createDate( second, first, year );
}

function resolveDayYearAmbiguity( first, second, month, monthTokenIdx, hint )
{
	// year cannot be in the middle
	if( monthTokenIdx == 1 )
		return createDate( first,  month, second )

	if( monthTokenIdx == 3 )
		return createDate( second,  month, first )

	if( hint[0] == -1 || hint[2] == -1)
		return null;

	// only hint can help here
	return hint[2] > hint[0] ?
		   createDate( first,  month, second ) :
		   createDate( second, month, first  );
}

function resolveTotalAmbiguity( first, second, third, hint )
{
	firstCanBeMonth  = first <= 12;
	secondCanBeMonth = second <= 12;
	thirdCanBeMonth  = third <= 12;

	// it's possible that there no month at all
	if( !firstCanBeMonth && !secondCanBeMonth && !thirdCanBeMonth )
		return null; // Oops. Where the month???

	// try to guess the month
	if( firstCanBeMonth && !secondCanBeMonth && !thirdCanBeMonth )
		return resolveDayYearAmbiguity( second, third, first, 1, hint );
	if( !firstCanBeMonth && secondCanBeMonth && !thirdCanBeMonth )
		return resolveDayYearAmbiguity( first, third, second, 2, hint );
	if( !firstCanBeMonth && !secondCanBeMonth && thirdCanBeMonth )
		return resolveDayYearAmbiguity( first, second, third, 3, hint );

	if(hint[0] == -1 || hint[1] == -1 || hint [2] == -1)
		return null;


	// full ambiguity. Let's see the hint ...
	if( hint[0] < hint[1] ) // D < M
	{
		if( hint[0] < hint[2] ) // D < Y
		{
			// D < M < Y, not D < Y < M since year cannot be in the middle
			if( secondCanBeMonth )
			{
				return createDate( first, second, third );
			}
			else // second can't be month and can't be year since year cannot be at the middle
			{
				// M < D < Y
				return  createDate( second, first, third );
			}

			//return createDate( first, second, third );
		}
		else // Y < D < M
		{
			if( thirdCanBeMonth )
			{
				return createDate( second, third, first );
			}
			else // third can't be month
			{
				// Y < M < D
				return createDate( third, second, first );
			}
		}
	}
	else // D > M
	{
		if( hint[0] > hint[2] ) // D > Y
		{
			// Y < M < D, not M < Y < D since year cannot be in the middle
			if( secondCanBeMonth )
			{
				return createDate( third, second, first );
			}
			else
			{
				// Y < D < M
				return createDate( second, third, first );
			}
		}
		else // M < D < Y
		{
			if( firstCanBeMonth )
			{
				return createDate( second, first, third );
			}
			else
			{
				// D < M < Y
				return createDate( first, second, third );
			}
		}
	}
}

function createDate( day, month, year )
{
	// process the case when the year is entered with two digits
	if( year < 100 )
		year = Math.floor( (new Date()).getFullYear() / 100 ) * 100 + year;

    if(year < 1970 || year > 3000)
        return null;

    // hours are set to 12 consciously. This is done to avoid problems related to daylight saving time.
    // In some countries time translation happens at 00:00, so setting hours to this value is not safe.
    // In result of JavaScript automatic time correction, today can become yesterday.
    // e.g., suppose time is translated to 1 hour forward at April 1st.
    // Then an attempt to create new Date(2011, 3, 1, 0), which should return "Apr 01 00:00:00"
    // will not be successful. Instead you get "Mar 31 23:00:00".
    var result = new Date(year, month - 1, day, 12);

	// adjust date to the maximum if necessary
	if( result.getDate() != day )
		result = new Date( year, month - 1, day - result.getDate(), 12 );

	return result;
}

function codeBug()
{
	alert("CodeBug");
}
