function EditCommentState(objectId, timestamp, cell, commentSpan, link, descriptionUtil) {
    this.objectId = objectId;
    this.timestamp = timestamp;
    this.cell = cell;
    this.commentSpan = commentSpan;
    this.link = link;
    this.state = 'EDITING'; // SAVING, LOADING
    this.timeCommentsRecord = null;
    this.originalDescription = '';
    this.descriptionUtil = descriptionUtil;
    this.textLengthNote = new TextAreaLengthNote('descLength', 'Comment', 2000);
    this.noRightsErrorMessage = null;
    this.dayWasLockedErrorMessage = null;
    this.taskWasCompletedErrorMessage = null;
    this.updatePopupState();
}

EditCommentState.prototype = new EditDescriptionState(null, null, null, null, false);

EditCommentState.prototype.updateCell =
function EditCommentState_updateCell() {
    if (this.cell==null)
        return;

    if (editDescriptionPopup.isEmpty()) {
        this.commentSpan.innerHTML = '';
        this.commentSpan.style.display = 'inline';
        if (this.link)
            this.link.innerHTML = messageResource.getMessage("task_details.js.enter_comment");
    } else {
        var html = convertLineBreaksToBR(toUnixLineFeeds(trim(editDescriptionPopup.getPopupTextValue().replace(/&/g, '&amp;').replace(/</g, '&lt;'))));
        this.commentSpan.innerHTML = html;
        this.commentSpan.style.display = 'block';
        if (this.link)
            this.link.innerHTML = messageResource.getMessage("task_details.js.edit_your_comment");
    }
}

function convertLineBreaksToBR( str )
{
    return str.replace(/\n/g, "<br>");
}

/**
 *
 * @param objectId
 * @param dateText
 * @param timestamp
 * @param cell
 * @param commentSpan
 * @param link
 * @param descriptionUtil
 * @param noRightsErrorMessage is message for 'no rights' concurrent issue.
 */
function showComment(objectId, dateText, timestamp, cell, commentSpan, link, descriptionUtil, noRightsErrorMessage,
                     dayWasLockedErrorMessage, taskWasCompletedErrorMessage, cannotModifyFutureErrorMessage)
{
    if (isBusySavingState()) return;

    setPopupHandlers(cell, descriptionUtil);

    editDescriptionState = new EditCommentState(objectId, timestamp, cell, commentSpan, link, descriptionUtil);

    editDescriptionState.noRightsErrorMessage = noRightsErrorMessage;
    editDescriptionState.dayWasLockedErrorMessage = dayWasLockedErrorMessage;
    editDescriptionState.taskWasCompletedErrorMessage = taskWasCompletedErrorMessage;
    editDescriptionState.cannotModifyFutureErrorMessage = cannotModifyFutureErrorMessage;

    document.getElementById('editPopupCommentDate').innerHTML = dateText;

    editDescriptionState.setState('LOADING');
    editDescriptionPopup.show(descriptionUtil.EDIT_DESCRIPTION_POPUP_CAPTION, '', -100, +15);
    loadComment(objectId, timestamp, descriptionUtil);
}

function commentSavingTimeout() {
    if( editDescriptionState && editDescriptionState.retryController )
        editDescriptionState.retryController.stop();

    descriptionSavingLoadingTimeout(editDescriptionState.descriptionUtil.Messages.TIMEOUT_DESCRIPTION_SAVING);
}

function commentLoadingTimeout() {
    if( editDescriptionState && editDescriptionState.retryController )
        editDescriptionState.retryController.stop();
    
    descriptionSavingLoadingTimeout(editDescriptionState.descriptionUtil.Messages.TIMEOUT_DESCRIPTION_LOADING);
}

function loadComment(objectId, timestamp, descriptionUtil) {
    editDescriptionState.retryController = new RpcRetryController( timeout,
        function()
        {
            if( !editDescriptionState ) return;
                                    
            descriptionUtil.doLoadComment(
                objectId, timestamp, commentLoaded,
                function(e) {
                    editDescriptionState.retryController.handleErrorOrRetry(e)
                }
            );
        },
        commentLoadingTimeout, processCommentError);
    editDescriptionState.retryController.start();
}

function commentLoaded(result, error) {
    if (!isBusySavingState()) return;

    if (error != null) {
        editDescriptionState.retryController.handleErrorOrRetry(error);
        return;
    }

    if (editDescriptionState && editDescriptionState.retryController)
        editDescriptionState.retryController.stop();

    editDescriptionState.timeCommentsRecord = result;
    var comment = result && result.comments ? result.comments : "";
    editDescriptionState.originalDescription = comment;
    editDescriptionState.setState('EDITING');
    editDescriptionPopup.update(editDescriptionState.descriptionUtil.EDIT_DESCRIPTION_POPUP_CAPTION, comment);
    editDescriptionState.textLengthNote.update(comment);
}

function saveComment(afterSaveHandler) {
    if (isBusySavingState()) return;

    var descriptionUtil = editDescriptionState.descriptionUtil;

    if(!isCorrectDescriptionLength(descriptionUtil))
        return;

    editDescriptionState.setState('SAVING');
    editDescriptionState.retryController = new RpcRetryController( timeout,
        function()
        {
            if( !editDescriptionState ) return;

            descriptionUtil.doSaveComment( editDescriptionState.objectId, editDescriptionState.timestamp, trim( editDescriptionPopup.getPopupTextValue( ) ),
                function( result, error )
                {
                    if( commentSaved( result, error ) && afterSaveHandler )
                        afterSaveHandler( );
                },
                function( error )
                {
                    editDescriptionState.retryController.handleErrorOrRetry( error );
                })
        },
        commentSavingTimeout, processCommentError );

    editDescriptionState.retryController.start();
}

var needReloadAfterSavingComment = false;

function commentSaved(result, error) {
    if (!isBusySavingState()) return false;

    if (error != null) {
        editDescriptionState.retryController.handleErrorOrRetry(error);
        return false;
    }

    needReloadAfterSavingComment = !editDescriptionState.timeCommentsRecord && result || editDescriptionState.timeCommentsRecord && !result;
    if(!needReloadAfterSavingComment)
        editDescriptionState.updateCell();

    descriptionSavingCleanup(needReloadAfterSavingComment);

    return true;
}

function processCommentError(exception) {
    if (editDescriptionState && editDescriptionState.retryController)
        editDescriptionState.retryController.stop();

    if (exception.name.indexOf('com.actimind.actitime.services.NoRightsForRemoteCallException') != -1) {
        if (editDescriptionState.noRightsErrorMessage)
            alert(editDescriptionState.noRightsErrorMessage);

        document.location.reload();
        return;
    }

    if (exception.name.indexOf('com.actimind.actitime.task.NoSuchTaskException') != -1) {
        document.location.reload();
        return;
    }

    if (exception.name.indexOf('com.actimind.actitime.task.TaskCompletedException') != -1) {
        if (editDescriptionState.taskWasCompletedErrorMessage)
            alert(editDescriptionState.taskWasCompletedErrorMessage);

        document.location.reload();
        return;
    }

    if (exception.name.indexOf('com.actimind.actitime.task.DayIsLockedException') != -1) {
        if (editDescriptionState.dayWasLockedErrorMessage)
            alert(editDescriptionState.dayWasLockedErrorMessage);

        document.location.reload();
        return;
    }

    if (exception.name.indexOf('java.lang.IllegalArgumentException') != -1) {
         if (editDescriptionState.cannotModifyFutureErrorMessage)
            alert(editDescriptionState.cannotModifyFutureErrorMessage);

        document.location.reload();
        return;    
    }

    if (editDescriptionState.state == 'LOADING')
        alert(editDescriptionState.descriptionUtil.Messages.ERROR_DESCRIPTION_LOADING);
    else
        alert(editDescriptionState.descriptionUtil.Messages.ERROR_DESCRIPTION_SAVING);

    descriptionSavingCleanup(true);
}
