var deadlineState;
var timeout = 10000;
var deadlinePopup = new EditableTextDhtmlPopup('deadlinePopup');

var DEADLINE_TIMEOUT_SAVING_MESSAGE;
var ERR_TASK_WAS_DELETED_MESSAGE;
var ERR_CANNOT_SAVE_DEADLINE;

function DeadlineState(taskId, cell, dateFormat) {
    this.taskId = taskId;
    this.cell = cell;
    this.deadline = trim(cell.firstChild.innerHTML);
    if (this.deadline == messageResource.getMessage('common.not_available')) this.deadline = '';
    this.state = 'EDITING'; // EDITING, SAVING
    this.dateFormat = dateFormat;

    var deadlineFieldFunc = function() { return document.getElementById('deadlinePopupText'); };
    window.deadline = new SmartDate(deadlineFieldFunc, getParsedDeadlineHolder, dateFormat);

    this.turnPopupToEditingState();
}

DeadlineState.prototype.setState =
function DeadlineState_setState(state) {
    this.state = state;
    if (state == 'EDITING')
        this.turnPopupToEditingState();
    else if (state == 'SAVING')
        this.turnPopupToSavingState();
}

DeadlineState.prototype.isDeadlineModified =
function DeadlineState_isDeadlineModified() {
    return trim(window.deadline.getDateFieldValue()) != this.deadline;
}

DeadlineState.prototype.turnPopupToEditingState =
function DeadlineState_turnPopupToEditingState() {
    document.getElementById('deadlinePopupEditingState').style.display = 'block';
    document.getElementById('deadlinePopupButtons').style.display = 'block';
    document.getElementById('deadlinePopupSavingState').style.display = 'none';
    document.getElementById('closeDeadlinePopupDisabledSpan').style.display = 'none';
    document.getElementById('closeDeadlinePopupActiveSpan').style.display = 'block';
}

DeadlineState.prototype.turnPopupToSavingState =
function DeadlineState_turnPopupToSavingState() {
    document.getElementById('deadlinePopupEditingState').style.display = 'none';
    document.getElementById('deadlinePopupButtons').style.display = 'none';
    document.getElementById('deadlinePopupSavingState').style.display = 'block';
    document.getElementById('closeDeadlinePopupActiveSpan').style.display = 'none';
    document.getElementById('closeDeadlinePopupDisabledSpan').style.display = 'block';
}

function highlightDeadlineOnMouseOver(cell) {
    if (isDeadlineSaving()) return;

    try { cell.style.cursor = 'pointer'; } catch (e) {}
    if (!deadlineState || deadlineState.cell != cell)
        cell.style.backgroundColor = DHTMLColors.mouseOverHighlightColor;
}

function removeDeadlineHighligthing(cell) {
    if (isDeadlineSaving()) return;

    try { cell.style.cursor = ''; } catch (e) {}
    if (!deadlineState || deadlineState.cell != cell)
        cell.style.backgroundColor = '';
}

function showDeadlinePopup(taskId, cell, dateFormat) {
    if (isDeadlineSaving()) return;
    if (!closeAllPopups()) return;

    deadlinePopup.setAfterShowHandler(
        function() {
            if (cell) cell.style.backgroundColor = DHTMLColors.onClickHighlightColor;
        }
    );
    deadlinePopup.setAfterHideHandler(
        function() {
            ExitConfirmationHandlersManager.restoreExitConfirmation();
            if (cell) cell.style.backgroundColor = '';
            deadlineState = null;
        }
    );
    deadlinePopup.setBeforeHideHandler(
        function() {
            if (isDeadlineSaving()) return false;

            if (deadlineState && deadlineState.isDeadlineModified())
                return confirm(messageResource.getMessage('edit_deadline.modifications_not_saved'));

            return true;
        }
    );

    deadlineState = new DeadlineState(taskId, cell, dateFormat);
    deadlineState.isConfirmationOnUnloadRequired = true;
    
    ExitConfirmationHandlersManager.setUpExitConfirmation( messageResource.getMessage('edit_deadline.modifications_not_saved'),
                                                           function () { 
                                                               return deadlineState && 
                                                                      deadlineState.isConfirmationOnUnloadRequired &&
                                                                      deadlineState.isDeadlineModified(); 
                                                           },
                                                           false );
    
    deadlinePopup.show(messageResource.getMessage('edit_deadline.title'), deadlineState.deadline, -50, +25);
    Ext.getCmp("deadlineFieldContainer").render();
}

function isDeadlineSaving() {
    return deadlineState && deadlineState.state == 'SAVING';
}

function getParsedDeadlineHolder() {
    return document.getElementById('parsedDeadline');
}

function deadlineSaved(result, error) {
    if (!isDeadlineSaving()) return false;
    if (error != null) {
        deadlineState.retryController.handleErrorOrRetry(error);
        return false;
    }

    var cell = deadlineState.cell;
    var deadlineDate = deadline.getHolderValue();
    if (deadlineDate.length == 0) {
        cell.firstChild.innerHTML = messageResource.getMessage('common.not_available');
        cell.firstChild.className = "deadline";
    }
    else {
        var d = deadline.parseDate();
        var df = new DateFormat(deadlineState.dateFormat);
        cell.firstChild.innerHTML = df.formatDate(d);
        cell.firstChild.className = isDeadlineInThePast(deadlineDate) ? "deadline_in_the_past" : "deadline";
    }

    deadlineSavingCleanup(false);
    deadlinePopup.hide();
    return true;
}

function updateDeadline(taskId, deadline, deadlineSavedHandler) {
    try {
        if (!isRpcReady( [ 'TaskInfoService.updateDeadline' ] )) {
            var func = function() { updateDeadline(taskId, deadline, deadlineSavedHandler); }
            window.setTimeout(func, 200);
            return;
        }

        if (!jsonrpc.TaskInfoService) {
            /* service is unavailable, most likely we've been logged out */
            document.location.reload();
            return;
        }

        var reqId = jsonrpc.TaskInfoService.updateDeadline(deadlineSavedHandler, taskId, deadline);
        if (deadlineState) deadlineState.requestId = reqId;
    } catch(e) {
        if( deadlineState ) deadlineState.retryController.handleErrorOrRetry( e );
        return null;
    }
}

function saveDeadline(afterSaveHandler) {
    if (isDeadlineSaving()) return;

    if (!deadline.checkDate(true)) {
        deadline.getDateField().focus();
        setRedirectUrl("");
        doCancel();
        return;
    }

    deadlineState.setState('SAVING');

    deadlineState.cell.style.backgroundColor = DHTMLColors.beingSavedColor;

    var deadlineStr = deadline.getHolderValue();

    deadlineState.retryController = new RpcRetryController( timeout,
        function() {
            updateDeadline(
                deadlineState.taskId, deadlineStr,
                function(result, error) {
                    if (deadlineSaved(result, error) && afterSaveHandler) afterSaveHandler();
                });
        },
        deadlineTimeoutSaving, processDeadlineSavingError );

    deadlineState.retryController.start();
    
    return;
}

function deadlineSavingCleanup(immediate) {
    if (deadlineState && deadlineState.retryController) {
        deadlineState.retryController.stop();

        deadlinePopup.setAfterHideHandler(null);
        ExitConfirmationHandlersManager.restoreExitConfirmation();

        var cell = deadlineState.cell;
        var removeCellHighlightingFunc = function() {
            removeDeadlineHighligthing(cell);
        };

        deadlineState = null;
        deadlinePopup.hide();

        if (immediate)
            removeCellHighlightingFunc();
        else
            window.setTimeout(removeCellHighlightingFunc, 1000);
    }
}

function deadlineTimeoutSaving() {
    if (isDeadlineSaving()) {
        if (!confirm(DEADLINE_TIMEOUT_SAVING_MESSAGE)) {
            if (isDeadlineSaving()) {
                if (deadlineState && deadlineState.requestId) {
                    JSONRpcClient.cancelRequest(deadlineState.requestId);
                }
                jsonrpc.abortAllRequests();
                deadlineSavingCleanup();
            }
        } else {
            if( !deadlineState || !deadlineState.retryController )  return;

            deadlineState.retryController.stop();
            if (isDeadlineSaving()) {
                deadlineState.retryController.start();
            }

        }
        jsonrpc = null;
    }
}

function processDeadlineSavingError(exception) {
    if (exception.name.indexOf('com.actimind.actitime.services.NoRightsForRemoteCallException') != -1) {
        if (noRightsURL)
            document.location.href = noRightsURL;
        else
            document.location.reload();
        return;
    }

    if (exception.name.indexOf('com.actimind.actitime.task.NoSuchTaskException') != -1) {
        alert(ERR_TASK_WAS_DELETED_MESSAGE);
        document.location.reload();
        return;
    }

    alert(ERR_CANNOT_SAVE_DEADLINE);
    deadlineSavingCleanup(true);
}
