function EditProjectLightbox()
{
}

EditProjectLightbox.prototype = new ProjectLightbox();
EditProjectLightbox.prototype.cancel_confirmation_message_key = "project.edit.js.cancel_confirmation";
EditProjectLightbox.prototype.close_tab_confirmation_message_key = "project.edit.js.close_tab_confirmation";
EditProjectLightbox.CREATE_POPUP_DEFAULT_PROJECT_NAME = "";

EditProjectLightbox.prototype.init = function ( customersInfo, projectData, isEnabledPAS )
{
    ProjectLightbox.prototype.init.call( this, customersInfo, projectData.customerId, true,  isEnabledPAS );
    this._popupEl.addClass( "editMode" );
    this.projectData = projectData;
    var self = this;
    self.setTitle( messageResource.getMessage( "project.edit.page.title" ) );
    self.setValue( self.projectNameField, projectData.name );
    //IE8 writes 'null' as string, in any case we send empty string to backend
    //bug 55185, 55184
    self.setValue( self.descriptionField, projectData.description || '');

    self.setCommitButtonTitle( messageResource.getMessage( "common.buttons.save_changes" ));

    this.changesCallback = function ()
    {
        if( !self.dataChanged )
            self.changesDetected();
        self.dataChanged = true;
    };

    this.initOpenTaskCompletedMessage( projectData.active, projectData.numberOpenTasks );
    this.initStatusSelector( projectData.active );
    this.descriptionLengthNote.update( this.getDescriptionForUpdatingLengthNote() );

    this.deleteButton = this.getElementBySubId("deleteBtn");
    this.initPlaceholders();

    this._deleteEnabled = true;
    this._deleteBlockedDueToLockedTime = projectData.blockedDueToLockedTime;
    this._deleteBlockedDueToUnavailableToModifyTimeTrack = projectData.blockedDueToUnavailableToModifyTimeTrack;

    this.onDelete = function( revokeStatuses ) {
        self.disableUserInputWhileSaving();
        makeRpcCall("ProjectInfoService.deleteProject",
                    [projectData.id, revokeStatuses],
                    function(result)
                    {
                        self.hide();
                        window.location.reload();
                    }, function(jqXHR, textStatus, errorThrown)
                    {
                        self.enableUserInputWhileSaving();
                    }
        );
    };

    if (!this._deleteBlockedDueToLockedTime && !this._deleteBlockedDueToUnavailableToModifyTimeTrack)
    {
        if (this._deleteEnabled)
        {
            this.mainContent = this.findElementBySubId("content");
            this.moireIn = new EditCPTPopupMoire({
                $containerElement: self._popupEl,
                idPrefix: self.popupId + "_in"
            }).init();

            this.moireOut = new EditCPTPopupMoire({
                $containerElement: self._popupEl,
                idPrefix: self.popupId + "_out"
            }).init();

            this.moireInEl = this.findElementBySubId("in_switchStateMoire");
            this.moireOutEl = this.findElementBySubId("out_switchStateMoire");

            var projectEntity = new ProjectManagementEntity(
                    projectData.id,
                    projectData.name,
                    projectData.blockedDueToLockedTime,
                    projectData.blockedDueToUnavailableToModifyTimeTrack,
                    projectData.projectHasManagerTimeTrack,
                    projectData.projectHasOtherUsersTimeTrack,
                    projectData.projectHasManagerApprovedTimeTrack,
                    projectData.projectHasOtherUsersApprovedTimeTrack,
                    projectData.active,
                    projectData.numberTotalTasks,
                    projectData.customerId);

            this.deletePopup = new CPTManagement.DeleteProjectPopupConfirmationDialog({
                $containerElement: self._popupEl,
                idPrefix: self.popupId,
                entitiesToDelete: [projectEntity],
                hasManagerApprover: projectData.hasManagerApprover,
                cancelHandler: _.bind(self.switchToMainContent, self),
                deleteHandler: _.bind(self.onDelete, self)
            }).init();

            this.savedEscape = this.onEscape;
            this.deleteButton.onclick = _.bind(this.switchToConfirmContent, this);
        }
    }
    else
    {
        var titleText;
        if (this._deleteBlockedDueToLockedTime)
        {
            titleText = messageResource.getMessage("project.edit.cannot_delete_due_to_lock_time");
        }
        else
        {
            var projectText = messageResource.getMessage("common.project");
            titleText = messageResource.getMessage(projectData.canManageAllUsersWithoutAutoApproved ?
                                                   "hierarchy_entity.edit_popup.cannot_delete_due_to_unavailable_modify_time_of_autoapproved_users" :
                                                   "hierarchy_entity.edit_popup.cannot_delete_due_to_unavailable_modify_time",
                                                   [projectText]);
        }
        $(this.deleteButton).addClass("disabled").get(0).title = titleText;
    }

    this.setSuccessCallback(
            function ( projectData, teamProject)
            {
                this._makeUpdateProjectRequest( projectData, teamProject);
            }
    );
    sectionsStateController.showOrHideBlock( "" );

    if (!_.isUndefined(window.userSelectorChangeListener)) 
        userSelectorChangeListener.callback = function(){ self.changesCallback(); };

    return this;
};

EditProjectLightbox.prototype.switchToMainContent = function()
{
    this.onEscape = this.savedEscape;
    EditCPTPopupMoire.animateMoire(this.deletePopup, this.mainContent, this.moireInEl, this.moireOutEl);
};

EditProjectLightbox.prototype.switchToConfirmContent = function()
{
    this.deletePopup.$el.height(this._popupEl.height());
    this.onEscape = _.bind(this.switchToMainContent, this);
    EditCPTPopupMoire.animateMoire(this.mainContent, this.deletePopup, this.moireInEl, this.moireOutEl);
};

EditProjectLightbox.prototype.findElementBySubId = function(id)
{
    return this._popupEl.find('#' + this.getElementIdBySubId(id));
};

EditProjectLightbox.prototype.getDataToCommit = function ()
{
    var self = this;
    return [
        {   id: self.projectData.id,
            customerId: self.getCustomerId(),
            customerName: self.getCustomerName(),
            name: self.getProjectName(),
            description: self.getDescription(),
            active: self.getStatus()},
        self.getTeamProject()
    ]
};

EditProjectLightbox.prototype.initOpenTaskCompletedMessage = function ( initialStatusIsActive, countOpenTasks )
{
    var self = this;
    this.countOpenTask = countOpenTasks;
    this.initialStatusIsActive = initialStatusIsActive;
    this.openTaskCompleteMessageEl = this.$subId( "openTaskComleteMsg" );
    if (countOpenTasks == 1){
        this.openTaskCompleteMessageEl.text( messageResource.getMessage( "project.edit.complete_open_task", [countOpenTasks] ) );
    }else
        this.openTaskCompleteMessageEl.text( messageResource.getMessage( "project.edit.complete_open_tasks", [countOpenTasks] ) );
    this.openTaskCompleteMessageEl.hide();
    this.openTaskCompleteMessageEl.showMessage = function ( newStatusIsActive )
    {
        var flag = self.countOpenTask > 0 && self.initialStatusIsActive && !newStatusIsActive;
        self.openTaskCompleteMessageEl.toggle( flag );
    }
};

EditProjectLightbox.prototype.initStatusSelector = function ( isActive )
{
    var self = this;
    var statusSelectorItems = [
        {
            text: messageResource.getMessage( "project.edit.active" ),
            value: true,
            itemCls: "active"
        },
        {
            text: messageResource.getMessage( "project.edit.archived" ),
            value: false,
            itemCls: "archived"
        }
    ];

    this.statusSelectorPlaceHolderEl = this.getElementBySubId( "statusSelectorPlaceholder" );
    this.statusSelectorPlaceHolderEl.innerHTML = "";
    this.statusSelector = new ListSelector( {
                                                menuCls: "statusSelectorMenu",
                                                renderTo: this.statusSelectorPlaceHolderEl,
                                                value: isActive,
                                                listItems: statusSelectorItems,
                                                scrollableContainer: this._innerContentEl
                                            } );
    var handler = function ( value )
    {
        $( self.statusSelectorPlaceHolderEl ).removeClass( "active archived" );
        $( self.statusSelectorPlaceHolderEl ).addClass( value ? "active" : "archived" );
        self.openTaskCompleteMessageEl.showMessage( value );
    };
    handler( isActive );
    this.statusSelector.on( "change", function ()
    {
        self.changesCallback();
        handler( self.getStatus() );
    } );
};

EditProjectLightbox.prototype.getStatus = function ()
{
    return this.statusSelector.getSelectedItem().value;
};

EditProjectLightbox.prototype._makeUpdateProjectRequest = function (projectData, projectTeam)
{
    this.disableUserInputWhileSaving();
    var self = this;
    self.customFailureCommitCallback = function ( jqXHR, textStatus, errorThrown )
    {
        if( errorThrown.exception.indexOf( 'com.actimind.actitime.project.NoSuchProjectException' ) != -1 )
        {
            alert( messageResource.getMessage('project.edit.no_such_project'));
            self.hide();
            document.location.reload();
            return true;
        }
        return false;
    };

    makeRpcCall( "ProjectInfoService.updateProject", [projectData, projectTeam],
                 function ( result ) {return self.successCommitCallback( result );},
                 function ( jqXHR, textStatus, errorThrown ) {return self.failureCommitCallback( jqXHR, textStatus, errorThrown );} );
};

EditProjectLightbox.prototype.changesDetected = function ()
{
    this.getFooterEl().className = this.footerClassName + ' changesDetected';
};

EditProjectLightbox.prototype.disableButtons = function ()
{
    ProjectLightbox.prototype.disableButtons.call(this);
    this._deleteEnabled = false;
};

EditProjectLightbox.prototype.enableButtons = function ()
{
    ProjectLightbox.prototype.enableButtons.call(this);
    this._deleteEnabled = true;
};
