function EditTaskController(taskParametersRow, timeReportedModeSelector,
                            warningMessage, initialTimeReportedOnTasks,
                            isSubtask, estimateInput, formattedDeadlineInput,
                            parsedDeadlineInput, billingTypeInput)
{
    this.taskParametersRow = taskParametersRow;
    this.timeReportedModeSelector = timeReportedModeSelector;
    this.warningMessage = warningMessage;
    this.initialTimeReportedOnTasks = initialTimeReportedOnTasks;
    this.selectedTasks = {};
    this.initialState = true;
    this.changeAlreadyFired = false;
    this.isSubtask = isSubtask;
    this.estimateInput = estimateInput;
    this.formattedDeadlineInput = formattedDeadlineInput;
    this.parsedDeadlineInput = parsedDeadlineInput;
    this.billingTypeInput = billingTypeInput;
    this.defineTimeReporting();
    this.isTimeReportingChanged = false;
}

EditTaskController.prototype.defineTimeReporting = function()
{
    this.isTimeReportedOnTask = (this.isSubtask) ? false : (!this.timeReportedModeSelector || "true" == this.timeReportedModeSelector.options[this.timeReportedModeSelector.selectedIndex].value);
}

EditTaskController.prototype.timeReportedModeChanged = function()
{
    this.defineTimeReporting();
    if (!this.isTimeReportedOnTask)
    {
        this.taskParametersRow.style.display = 'none';
        if (this.initialTimeReportedOnTasks)
        {
            this.warningMessage.style.display = '';
        }
    }
    else
    {
        this.taskParametersRow.style.display = '';
        if (!this.isTimeReportingChanged)
        {
            initCalendar();
        }
        this.warningMessage.style.display = 'none';
    }
    this.isTimeReportingChanged = true;
}

EditTaskController.prototype.disableAdditionalFields = function()
{
    this.estimateInput.disabled = 'disabled';
    this.formattedDeadlineInput.disabled = 'disabled';
    this.parsedDeadlineInput.disabled = 'disabled';
    this.billingTypeInput.disabled = 'disabled';
}

EditTaskController.prototype.getTaskOptions = function(customerMap, projectMap, selectedCustomer, selectedProject)
{
    var result = new Array();
    this.selectedTasks = {};
    if (selectedProject == -1 && customerMap[selectedCustomer]) {
        for (var i = 0; i < customerMap[selectedCustomer].length; i++) {
            this.selectedTasks[customerMap[selectedCustomer][i].id] = customerMap[selectedCustomer][i].projectId;
            result.push({fullName: customerMap[selectedCustomer][i].name,
                                name: customerMap[selectedCustomer][i].name,
                                value: customerMap[selectedCustomer][i].id});
        }
    }

    if (selectedProject != -1 && projectMap[selectedProject]) {
        for (var j = 0; j < projectMap[selectedProject].length; j++) {
            this.selectedTasks[projectMap[selectedProject][j].id] = projectMap[selectedProject][j].projectId;
            result.push({fullName: projectMap[selectedProject][j].name,
                            name: projectMap[selectedProject][j].name,
                            value: projectMap[selectedProject][j].id});
        }
    }

    return result;
}

EditTaskController.prototype.taskChanged = function(form)
{
    var selectedProject = form.projectId.options[form.projectId.selectedIndex].value;
    var selectedTask = form.parentTaskId.options[form.parentTaskId.selectedIndex].value;
    if (selectedProject == -1 && selectedTask != -1)
    {
        selectOptionWithValue(form.projectId, this.selectedTasks[selectedTask]);
        loadCustomerProjects(form);
    }
    if (!this.changeAlreadyFired)
    {
        handleChange(true);
        this.changeAlreadyFired = true;
    }
}

EditTaskController.prototype.isAdditionalFieldsShown = function()
{
    return this.isSubtask || this.isTimeReportedOnTask;
}

function Task(id, name, projectId)
{
    this.id = id;
    this.name = name;
    this.projectId = projectId;
}