function EditTaskPopup()
{
}

EditTaskPopup.prototype = _.create(BaseTaskPopup.prototype);

EditTaskPopup.prototype.titleMessageKey = "task.edit_popup.edit_title";
EditTaskPopup.prototype.exist_invalid_fields_message_key = "task.edit_popup.footer_error.exist_invalid_fields";
EditTaskPopup.prototype.cancel_confirmation_message_key = "task.edit_popup.cancel_confirmation";
EditTaskPopup.prototype.close_tab_confirmation_message_key = "task.edit_popup.close_tab_confirmation";
EditTaskPopup.prototype.description_tooltip_message_key =  "task.edit_popup.edit_task_description.title";

EditTaskPopup.prototype.NAME_EMPTY = "nameEmptyError";
EditTaskPopup.prototype.NAME_DUPLICATE = "nameDuplicateError";
EditTaskPopup.prototype.DESCRIPTION_TOO_LONG = "descriptionTooLongError";
EditTaskPopup.prototype.PROJECT_NOT_SELECTED = "projectNotSelected";
EditTaskPopup.prototype.COMPLETION_DATE_NOT_SET = "completionDateNotSet";
EditTaskPopup.prototype.ESTIMATE_INVALID = "estimateInvalidError";

EditTaskPopup.prototype.MAX_DESCRIPTION_LENGTH = 2000;

EditTaskPopup.prototype.init = function( editTaskPopupData, estEnabled, sortConstants, resultCodes,
                                         customers, projects, billingTypes, defaultBillingTypeId, weekStartDay, scheduleCalculator,
                                         successTaskEditCallback, failureTaskEditCallback, taskDeleteCallback  )
{
    // must be before parent constructor
    var self = this;
    this.dataChanged = false;
    this.errors = {};

    this.customers = customers;
    this.projects = projects;
    this.billingTypes = billingTypes;
    this.defaultBillingTypeId = defaultBillingTypeId;
    this.weekStartDay = weekStartDay;
    this.scheduleCalculator = scheduleCalculator;

    BaseTaskPopup.prototype.init.call( this, "editTaskPopup", editTaskPopupData,
                                       estEnabled, !!this.billingTypes, sortConstants,
                                       failureTaskEditCallback, taskDeleteCallback );

    this.resultCodes = resultCodes;
    this.successTaskEditCallback = successTaskEditCallback;
    this.failureTaskEditCallback = failureTaskEditCallback;
    this.taskDeleteCallback = taskDeleteCallback;

    this._deleteEnabled = true;
    this._deleteBlockedDueToLockedTime = editTaskPopupData.blockedDueToLockedTime;
    this._deleteBlockedDueToUnavailableToModifyTimeTrack = editTaskPopupData.blockedDueToUnavailableToModifyTimeTrack;
    this.deleteButton = this.getElementBySubId("deleteBtn");

    this.initPlaceholders();

    this.onDelete = function( revokeStatuses ) {
        self.disableUserInputWhileSaving();
        makeRpcCall("EditTaskService.deleteTask", [self.task.id, revokeStatuses],
                    function (result) {
                        self.hide();
                        self.taskDeleteCallback();
                    },
                    function (jqXHR, textStatus, errorThrown) {
                        return self.handleException(errorThrown);
                    }
        );
    };

    this.successCallback = function () { self.editTaskCall(false); };

    var hasTimeTrack = editTaskPopupData.taskHasManagerTimeTrack || editTaskPopupData.taskHasOtherUsersTimeTrack;
    this.findElementBySubId("content").toggleClass('hasTimeTrack', hasTimeTrack).toggleClass('noTimeTrack', !hasTimeTrack);

    if (!this._deleteBlockedDueToLockedTime && !this._deleteBlockedDueToUnavailableToModifyTimeTrack)
    {
        if (this._deleteEnabled)
        {
            this.mainContent = this.findElementBySubId("content");
            this.moireIn = new EditCPTPopupMoire({
                $containerElement: self.popupEl,
                idPrefix: self.popupId + "_in"
            }).init();

            this.moireOut = new EditCPTPopupMoire({
                $containerElement: self.popupEl,
                idPrefix: self.popupId + "_out"
            }).init();

            this.moireInEl = this.findElementBySubId("in_switchStateMoire");
            this.moireOutEl = this.findElementBySubId("out_switchStateMoire");

            this.deletePopup = new CPTManagement.DeleteTasksConfirmationDialog({
                $containerElement: self.popupEl,
                idPrefix: self.popupId,
                entitiesToDeleteCount: 1,
                entitiesBlocked: 0,
                entitiesBlockedCount: 0,
                entitiesBlockedDueToLockedTimeCount: 0,
                entitiesBlockedDueToUnavailableToModifyTimeTrackCount: 0,
                entitiesWithManagerTimeTrackCount: editTaskPopupData.taskHasManagerTimeTrack ? 1 : 0,
                entitiesWithOtherUsersTimeTrackCount: editTaskPopupData.taskHasOtherUsersTimeTrack ? 1 : 0,
                entitiesWithManagerApprovedTimeTrackCount: editTaskPopupData.taskHasManagerApprovedTimeTrack? 1 : 0,
                entitiesWithOtherUsersApprovedTimeTrackCount: editTaskPopupData.taskHasOtherUsersApprovedTimeTrack? 1 : 0,
                hasManagerApprover: editTaskPopupData.hasManagerApprover,
                cancelHandler: _.bind(self.switchToMainContent, self),
                deleteHandler: _.bind(self.onDelete, self)
            }).init();

            this.savedEscape = this.onEscape;
            this.deleteButton.onclick = _.bind(this.switchToConfirmContent, this);
        }
    }
    else
    {
        var titleText;
        if (this._deleteBlockedDueToLockedTime)
        {
            titleText = messageResource.getMessage("task.edit_popup.cannot_delete_due_to_lock_time");
        }
        else
        {
            var taskText = messageResource.getMessage("common.task");
            titleText = messageResource.getMessage(editTaskPopupData.canManageAllUsersWithoutAutoApproved ?
                                                   "hierarchy_entity.edit_popup.cannot_delete_due_to_unavailable_modify_time_of_autoapproved_users" :
                                                   "hierarchy_entity.edit_popup.cannot_delete_due_to_unavailable_modify_time",
                                                   [taskText]);
        }
        this.findElementBySubId("deleteBtn").addClass("disabled").get(0).title = titleText;
    }

    this.initTextAreaLengthNote();

    return this;
};

EditTaskPopup.prototype.switchToMainContent = function()
{
    this.onEscape = this.savedEscape;
    EditCPTPopupMoire.animateMoire(this.deletePopup, this.mainContent, this.moireInEl, this.moireOutEl);
};

EditTaskPopup.prototype.switchToConfirmContent = function()
{
    this.onEscape = _.bind(this.switchToMainContent, this);
    EditCPTPopupMoire.animateMoire(this.mainContent, this.deletePopup, this.moireInEl, this.moireOutEl);
};

EditTaskPopup.prototype.handleException = function( errorThrown )
{
    this.dataChanged = false;
    if( errorThrown.exception == 'com.actimind.actitime.task.TaskIsNotAssignedToUserException' )
    {
        alert( messageResource.getMessage("common.errors.task_not_assined") );
        this.failureTaskEditCallback();
        return true;
    }

    if( errorThrown.exception == 'com.actimind.actitime.services.EditTaskService$NoRightsToManageTaskException' )
    {
        alert( messageResource.getMessage("task.edit_popup.no_rights_to_manage_tasks") );
        this.failureTaskEditCallback();
        return true;
    }

    if (errorThrown.exception == "com.actimind.actitime.task.NoPermissionToModifyTTOfSpecifiedUsers")
    {
        alert(messageResource.getMessage("hierarchy_entity.edit_popup.cannot_delete_due_to_unavailable_modify_time", [messageResource.getMessage("common.task")]));
        this.failureTaskEditCallback();
        return true;
    }

    if( errorThrown.exception == "com.actimind.actitime.task.TaskHasLockedTTException" )
    {
        alert( messageResource.getMessage("task.edit_popup.cannot_delete_due_to_lock_time") );
        this.failureTaskEditCallback();
        return true;
    }

    if( errorThrown.exception == "java.lang.IllegalArgumentException" )
    {
        alert( messageResource.getMessage("task.edit_popup.footer_error.invalid_fields_sent_to_server") );
        this.validate();
        this.enableUserInputWhileSaving();
        return true;
    }

    this.enableUserInputWhileSaving();
    return true;
};

EditTaskPopup.prototype.disableButtons = function(){
    BaseTaskPopup.prototype.disableButtons.call(this);
    this._deleteEnabled = false;
};

EditTaskPopup.prototype.enableButtons = function(){
    BaseTaskPopup.prototype.enableButtons.call(this);
    this._deleteEnabled = true;
};

EditTaskPopup.prototype.getErrorsMap = function()
{
    return this.errors;
};

EditTaskPopup.prototype.setTaskName = function( taskName ) {
    this.initNameField();
    this.setValue(this.nameField, taskName);
};

EditTaskPopup.prototype.initCustomerProjectSelector = function() {
    var self = this;
    this.findElementBySubId("customerSelector").addClass("selector");
    this.findElementBySubId("projectSelector").addClass("selector");
    this.findElementBySubId("customerSelectorContainer").addClass("withSelector");
    this.customerProjectSelector = new EditTaskCustomerProjectSelector().init(
            this.getElementIdBySubId("taskInfoDiv"), this.getElementIdBySubId("customerSelector"),
            this.getElementIdBySubId("projectSelector"),
            this.customers, this.projects, this.task.project.customerId, this.task.project.id,
            function() {
                self.checkProjectOnChange();
                self.changesCallback();
                self.updateReopenWarning(self.isOpenTask());
            },
            this.scrollableContainer );
};

EditTaskPopup.prototype.initBillingTypeSelector = function() {
    this.billingTypeSelectorPlaceHolderEl = this.findElementBySubId("taskBillingType").addClass("selector").get(0);
    this.billingTypeSelectorPlaceHolderEl.innerHTML = "";

    var listItems = [];
    for (var i = 0; i < this.billingTypes.length; i++)
    {
        var billingType = this.billingTypes[i];
        listItems.push( { text: escapeHtmlSymbols( billingType.name ), value: billingType.id } );
    }

    this.billingTypeSelector = new ListSelector({
                                                   menuCls: "billingTypeSelectorMenu editTask",
                                                   renderTo: this.billingTypeSelectorPlaceHolderEl,
                                                   value: this.task.billingType.id,
                                                   listItems: listItems,
                                                   scrollableContainer: this.scrollableContainer
                                               });

    var self = this;
    this.billingTypeSelector.on( "change", function () { self.changesCallback(); } );
};

EditTaskPopup.prototype.initDeadlineDate = function() {
    var self = this;
    this.findElementBySubId("deadlineContainer").addClass("withSelector");

    this.deadlineCellRenderer =  new TaskDeadlineCellRenderer().init( this.weekStartDay, this.scheduleCalculator,
                                                                      function () {
                                                                          self.updateDeadlineDate();
                                                                          self.changesCallback();
                                                                      },
                                                                      this.scrollableContainer,
                                                                      messageResource.getMessage("task.edit_popup.set_deadline"),
                                                                      messageResource.getMessage("task.edit_popup.deadline_is_in_past"),
                                                                      this.isOpenTask());

    var deadlineEl = this.findElementBySubId("taskDeadline").addClass("selector").get(0);
    var deadlineDate = this.task.deadlineDate ? this.task.deadlineDate.parseJSONDate() : null;
    this.taskDeadlineCell = {
        value: deadlineDate,
        getCellEl: function() { return deadlineEl; },
        getValue: function() { return this.value; } ,
        setValue: function(newValue) { this.value = newValue; },
        updateUI: function() {}
    };
    this.deadlineCellRenderer.generateContent(this.taskDeadlineCell, true);
    this.updateDeadlineDate();
};

EditTaskPopup.prototype.updateDeadlineDate = function() {
    this.deadlineCellRenderer.checkPastDate = this.isOpenTask();
    this.deadlineCellRenderer.updateDeadlineCell(this.taskDeadlineCell);
};

EditTaskPopup.prototype.initCompletionDateSelector = function()
{
    var self = this;

    this.findElementBySubId("completionDateText").addClass("withSelector");
    this.taskComletionDateValueEl.addClass("selector");

    this.completionDateCellRenderer =  new TaskDeadlineCellRenderer().init( this.weekStartDay, this.scheduleCalculator,
                                                                      function () {
                                                                          self.updateCompletionDate();
                                                                          self.changesCallback();
                                                                      },
                                                                      this.scrollableContainer,
                                                                      messageResource.getMessage( "task.edit_popup.set_completion_date" ),
                                                                      "", false);

    var completionEl = this.getElementBySubId("taskCompletionDate");
    var today = currentUserTime().clearTime();
    var completionDate = this.task.completionDate ? this.task.completionDate.parseJSONDate() : today;
    this.taskCompletionDateCell = {
        value: completionDate,
        getCellEl: function() { return completionEl; },
        getValue: function() { return this.value; } ,
        setValue: function(newValue) { this.value = newValue; },
        updateUI: function() {}
    };
    this.completionDateCellRenderer.generateContent(this.taskCompletionDateCell, false);
    this.updateCompletionDate();
};

EditTaskPopup.prototype.updateCompletionDate = function() {
    this.completionDateCellRenderer.updateDeadlineCell(this.taskCompletionDateCell);
    this.isCompletionDateValid();
    this.updateErrors(false);
};


/* Status Selector */
EditTaskPopup.prototype.setTaskStatus = function ( isOpenTask )
{
    var self = this;

    var statusSelectorItems = [
        {
            text: this.statusOpenMessage,
            value: true
        },
        {
            text: this.statusCompletedMessage,
            value: false
        }
    ];

    this.statusSelectorPlaceHolderEl = this.getElementBySubId("statusSelectorPlaceholder");
    this.statusSelectorPlaceHolderEl.innerHTML = "";
    this.statusSelector = new ListSelector({
                                               menuCls: "taskStatusSelectorMenu",
                                               renderTo: this.statusSelectorPlaceHolderEl,
                                               value: isOpenTask,
                                               listItems: statusSelectorItems,
                                               menuAlign : "tr-br",
                                               scrollableContainer: this.scrollableContainer
                                           });
    var handler = function ( isOpen ) {
        $(self.statusSelectorPlaceHolderEl).removeClass("active completed");
        $(self.statusSelectorPlaceHolderEl).addClass(isOpen ? "active" : "completed");
        self.setCompletionDate( isOpen, !this.taskCompletionDateCell);
        if (self.taskDeadlineCell) self.updateDeadlineDate();

        self.updateReopenWarning(isOpen);
    };
    handler( isOpenTask );
    this.statusSelector.on( "change", function () { self.changesCallback(); handler(self.isOpenTask()); } );
};

EditTaskPopup.prototype.updateReopenWarning = function ( isOpen ) {
    $(this.lightBoxEl).toggleClass("showReopenWarning", this.isForcedToReopenProject());
};

EditTaskPopup.prototype.initDescriptionField = function () {
    this.description.attr("placeholder", messageResource.getMessage("task.edit_popup.description_placeholder"));

    var self = this;
    this.description.off();
    this.description.blur( this.basicOnBlurFieldHandler( function () { self.checkDescriptionOnChange() } ) );
    this.description.keyup( this.basicKeyUpChangesHandler( function () { self.checkDescriptionOnChange() } ) );
    this.description.keydown( this.basicOnKeyPressedHandler );
};

EditTaskPopup.prototype.createEstimateCell = function ( estimate, estimateCellRenderer, estimateCellValidator )
{
    var estimateStr = ( estimate || estimate === 0 ) ? UserTime.formatTime( estimate ) : "";
    var taskEstimateCell = new TabularDataTableCell().init( estimateStr, estimateCellValidator, this );
    taskEstimateCell.setRenderer( estimateCellRenderer );
    return taskEstimateCell;
};

EditTaskPopup.prototype.initEstimatedHours = function()
{
    BaseTaskPopup.prototype.initEstimatedHours.call(this);
    var self = this;

    this.findElementBySubId("summaryValueEditor").addClass("editable");

    this.estimatedHoursEl.addClass("editable");
    this.estimateEditButton = this.findElementBySubId("estimateEditButton");

    this.estimateEditor = this.findElementBySubId("estimatedHoursEditor").blur( function () {
        var estPValue = self.estimateEditor.pVal();
        var isNotNeeded = estPValue.length == 0;
        var newEstimate = isNotNeeded ? null : UserTime.parseTimeWithSystemSettings( estPValue );
        if ( isNotNeeded || (newEstimate != null && newEstimate >= 0 && newEstimate < 60000000)) {
            self.taskEstimateValue = newEstimate;
            self.setEstimatedHours();

            self.estimateEditor.removeClass("error");
            self.errors[self.ESTIMATE_INVALID] = false;
            self.updateErrors(false);
            self.estimateEditor.removeClass("visible");
            self.estimatedHoursEl.removeClass("hidden");
            self.estimateEditor.get(0).title = "";

            self.updateEstimateContent();
        } else {
            self.taskEstimateValue = -1;
            self.estimateEditor.addClass("error");
            self.errors[self.ESTIMATE_INVALID] = true;
            self.estimateEditor.get(0).title = messageResource.getMessage('task.edit_popup.err_invalid_budget', [SystemSettings.decimalSeparator]);
        }
    });

    this.estimateEditor.keyup( this.basicKeyUpChangesHandler() );
    this.estimateEditor.keydown( this.basicOnKeyPressedHandler );

    var editEstimateFunction = function() {
        if (self.taskEstimateValue != -1)
            self.estimateEditor.val(self.taskEstimateValue == null ? null : UserTime.formatTime(self.taskEstimateValue));

        self.estimatedHoursEl.addClass("hidden");
        self.estimateEditor.addClass("visible").focus();
    };

    this.estimateEditButton.click( editEstimateFunction );
    this.estimatedHoursEl.click( editEstimateFunction );
};

EditTaskPopup.prototype.checkNameOnChange = function ()
{
    this.checkEmpty( this.nameField, this.NAME_EMPTY, true, true );
    this.updateErrors(false);
};

EditTaskPopup.prototype.checkProjectOnChange = function ()
{
    this.isProjectValid();
    this.updateErrors(false);
};

EditTaskPopup.prototype.isNameValid = function( onlyRemoveEmptyError )
{
    return !this.checkEmpty( this.nameField, this.NAME_EMPTY, onlyRemoveEmptyError, true );
};

EditTaskPopup.prototype.getTaskName = function()
{
    return this.nameField.pVal();
};

EditTaskPopup.prototype.getTaskDescription = function()
{
    return this.description.pVal();
};

EditTaskPopup.prototype.getTaskDeadline = function()
{
    return this.taskDeadlineCell.getValue();
};

EditTaskPopup.prototype.getTaskEstimate = function()
{
    return this.taskEstimateValue;
};

EditTaskPopup.prototype.getTaskBillingType = function()
{
    return this.btyEnabled ? this.billingTypeSelector.getSelectedItem().value : null;
};

EditTaskPopup.prototype.getTaskProjectId = function()
{
    return this.customerProjectSelector.getSelectedProjectId();
};

EditTaskPopup.prototype.isTaskInActiveProject = function()
{
    var projectId = this.getTaskProjectId();
    for ( var i = 0; i < this.projects.length; i++ ) {
        if ( this.projects[i].id == projectId )
            return this.projects[i].activeProject;
    }
    return true;
};

EditTaskPopup.prototype.restoreTaskProjectStatusAfterConcurrentModification = function()
{
    var projectId = this.task.project.id;
    for ( var i = 0; i < this.projects.length; i++ ) {
        if ( this.projects[i].id == projectId )
            return this.projects[i].activeProject = true;
    }
    return true;
};

EditTaskPopup.prototype.isTaskCompleted = function()
{
    return !this.isOpenTask();
};

EditTaskPopup.prototype.isOpenTask = function()
{
    return this.statusSelector.getSelectedItem().value;
};

EditTaskPopup.prototype.getTaskCompletionDate = function()
{
    return !this.taskCompletionDateCell ? null : this.taskCompletionDateCell.getValue();
};

EditTaskPopup.prototype.isForcedToReopenProject = function()
{
    return !this.isTaskInActiveProject() && this.isOpenTask();
};

EditTaskPopup.prototype.updateUI = function ()
{
    this.updateErrors(true);
};

EditTaskPopup.prototype.hideNameDuplicateError = function ()
{
    this.errors[this.NAME_DUPLICATE] = false;
    this.updateErrors();
};

EditTaskPopup.prototype.displayNameDuplicateError = function ()
{
    this.errors[this.NAME_DUPLICATE] = true;
    this._printErrorInFooter( messageResource.getMessage( this.exist_invalid_fields_message_key ) );
    this.updateErrors(true);
};

EditTaskPopup.prototype.getDescriptionForUpdatingLengthNote = function ()
{
    return getStringWithUnifiedNewLineSymbol(this.getTaskDescription());
};

/* offset of the innerContent in the popup content
 *  used when scrolling to section with error*/
EditTaskPopup.prototype.getContentOffset = function() {
    return -5;
};

EditTaskPopup.prototype.scrollToMainSection = function() {
    this.scrollToSection(this.findElementBySubId("taskNameContainer"));
};

EditTaskPopup.prototype.validate = function ()
{
    var descriptionValid = this.isDescriptionValid();
    var result = this.isNameValid(false) && descriptionValid && this.isProjectValid() && this.isCompletionDateValid() && this.isEstimateValid();

    if ( !descriptionValid && !this.descriptionVisible )
        this.toggleDescription();

    if ( !result )
        this.scrollToMainSection();

    return result;
};

EditTaskPopup.prototype.isProjectValid = function ()
{
    var projectSelected = this.customerProjectSelector.isProjectSelected();
    this.errors[this.PROJECT_NOT_SELECTED] = !projectSelected;
    return projectSelected;
};

EditTaskPopup.prototype.isDescriptionValid = function ()
{
    this.descriptionLengthNote.update(this.getDescriptionForUpdatingLengthNote());
    var textLength = this.descriptionLengthNote.getTextLength();
    var result = textLength > this.descriptionLengthNote.maxValue;
    this.errors[ this.DESCRIPTION_TOO_LONG ] = result;
    this.description.attr("title", result ? messageResource.getMessage("task.edit_popup.description_too_big", [textLength]) : "");
    return !result;
};

EditTaskPopup.prototype.isCompletionDateValid = function ()
{
    var result = this.isOpenTask() || this.getTaskCompletionDate() != null;
    this.errors[this.COMPLETION_DATE_NOT_SET] = !result;
    return result;
};

EditTaskPopup.prototype.isEstimateValid = function ()
{
    return this.taskEstimateValue == null || this.taskEstimateValue >= 0;
};

EditTaskPopup.prototype.checkDescriptionOnChange = function ()
{
    this.isDescriptionValid();
    this.updateErrors(false);
    this.updateDescriptionImage();
};

EditTaskPopup.prototype.setErrorClasses = function( classes )
{
    this.taskInfoDiv.className = classes.join( " " );
};

EditTaskPopup.prototype.displayErrorInFooter = function ()
{
    this._errorMessageBox.removeClass( "disabled" );
};

EditTaskPopup.prototype.updateErrors = function ( showErrorInFooter )
{
    if (this.hasErrors())
    {
        if( showErrorInFooter )
            this.displayErrorInFooter();
    }
    else
        this._removeErrorInFooter();
};

EditTaskPopup.prototype.initNameField = function () {
    this.nameField = $(this.getElementBySubId("nameField"));

    var self = this;
    this.nameField.off();
    this.nameField.blur( this.basicOnBlurFieldHandler( function () { self.checkNameOnChange(); } ) );
    this.nameField.keyup( this.basicKeyUpChangesHandler() );
    this.nameField.change( function () { self.hideNameDuplicateError() } );
    this.nameField.keydown( this.basicOnKeyPressedHandler );
};

EditTaskPopup.prototype.initPlaceholders = function () {
    this.nameField.placeholder();
    this.description.placeholder();
};

EditTaskPopup.prototype.initTextAreaLengthNote = function ()
{
    this.descriptionLengthNote = new TextAreaLengthNote( this.getElementIdBySubId('taskDescriptionCounter'), 'Description', this.MAX_DESCRIPTION_LENGTH);
    this.descriptionLengthNote.setTextLengthCounter( this.descriptionLengthNote.fullCounter );
    this.descriptionLengthNote.update( this.getDescriptionForUpdatingLengthNote() );
    this.descriptionLengthNote.show();
};

EditTaskPopup.prototype.changesDetected = function()
{
    if (!this.dataChanged)
        this.dataChanged = true;
    this.footer.addClass( "changesDetected" );
};

EditTaskPopup.prototype.editTaskCall = function( forcedToReopen )
{
    var self = this;
    var task = this.task;
    this.disableUserInputWhileSaving();
    makeRpcCall( "EditTaskService.editTask", [task.id, this.getTaskName(), this.getTaskDescription(), this.getTaskDeadline(),
                                              this.getTaskEstimate(), this.estEnabled, this.getTaskBillingType(),
                                              this.getTaskProjectId(), this.isTaskCompleted(), this.getTaskCompletionDate(), forcedToReopen || this.isForcedToReopenProject()],
                 function (result) {
                     if ( result.resultCode == self.resultCodes.success )
                     {
                         self.hide();
                         self.successTaskEditCallback();
                         return true;
                     } else
                         self.scrollToMainSection();

                     if ( result.resultCode == self.resultCodes.taskExists )
                     {
                         self.displayNameDuplicateError();
                         self.enableUserInputWhileSaving();
                         return true;
                     }

                     if ( result.resultCode == self.resultCodes.taskNotExists )
                     {
                         alert( messageResource.getMessage("task.edit_popup.error.concurrently_deleted") );
                         self.hide();
                         self.taskDeleteCallback();
                         return true;
                     }

                     if ( result.resultCode == self.resultCodes.noRightsToReopenProject )
                     {
                         alert( messageResource.getMessage("task.edit_popup.no_rights_to_reopen_project") );
                         self.handleCustoemersProjectsError(result);
                         return true;
                     }

                     if ( result.resultCode == self.resultCodes.projectIsArchived )
                     {
                         if( confirm( messageResource.getMessage( "task.error.reopen.project_archived" ) ) )
                             self.editTaskCall(true);
                         else {
                             self.handleCustoemersProjectsError(result);
                             self.restoreTaskProjectStatusAfterConcurrentModification();
                         }
                         return true;
                     }

                     if ( result.resultCode == self.resultCodes.projectNotAvailable )
                     {
                         alert( messageResource.getMessage("task.edit_popup.project_not_available") );
                         self.handleCustoemersProjectsError(result);
                         return true;
                     }

                     if ( result.resultCode == self.resultCodes.projectNotExists )
                     {
                         alert( messageResource.getMessage("task.edit_popup.project_not_exists") );
                         self.handleCustoemersProjectsError(result);
                         return true;
                     }

                     if ( result.resultCode == self.resultCodes.billingTypeNotExists )
                     {
                         alert( messageResource.getMessage("task.edit_popup.billing_type_not_exists") );
                         self.billingTypes = result.billingTypes.list;
                         self.initBillingTypeSelector();
                         self.enableUserInputWhileSaving();
                         return true;
                     }

                     if ( result.resultCode == self.resultCodes.billingTypeIsArchived )
                     {
                         alert( messageResource.getMessage("task.edit_popup.billing_type_was_archived") );
                         self.billingTypes = result.billingTypes.list;
                         self.initBillingTypeSelector();
                         self.enableUserInputWhileSaving();
                         return true;
                     }

                     return false;
                 },
                 function (jqXHR, textStatus, errorThrown) {
                     return self.handleException(errorThrown);
                 }
    );
};

EditTaskPopup.prototype.handleCustoemersProjectsError = function( result ) {
    this.customers = result.customers.list;
    this.projects = result.projects.list;
    this.initCustomerProjectSelector();
    this.enableUserInputWhileSaving();
};

