/**
 * Handle all operations with add user group row in lightbox
 */
function AddGroupController() {}

/**
 * @param config.newGroupInputId input for new group name
 * @param config.newGroupInputCellId cell, where newGroupInputId situates
 * @param config.addGroupButtonId button for accept new group creation
 * @param config.addGroupRowId common element of all controls
 * @param config.rowHighlightedClassName class name for highlighted row
 * @param config.ajaxProcessingClassName class name for paint row as ajax processd
 * @param config.waitingAnimationCellId cell with animation needed for ajax
 * @param config.spinnerController spinner controller
 */

AddGroupController.prototype.init = function(config) {
    this.newGroupInputId = config.newGroupInputId;
    this.newGroupInputCellId = config.newGroupInputCellId;
    this.addGroupButtonId = config.addGroupButtonId;
    this.addGroupRowId = config.addGroupRowId;
    this.rowHighlightedClassName = config.rowHighlightedClassName;
    this.ajaxProcessingClassName = config.ajaxProcessingClassName;
    this.waitingAnimationCellId = config.waitingAnimationCellId;
    this.groupNamingKey = config.groupNamingKey;
    this.GroupNamingKey = config.GroupNamingKey;
    this.spinnerController = config.spinnerController;
    return this;
};

/**
 * Init controller
 * @param controller user group management controller
 */
AddGroupController.prototype.initAfterShowing = function(controller) {

    this.newGroupInput = document.getElementById(this.newGroupInputId);
    this.newGroupInputCell = document.getElementById(this.newGroupInputCellId);
    this.addGroupButton = document.getElementById(this.addGroupButtonId);
    this.addGroupRow = document.getElementById(this.addGroupRowId);
    this.waitingAnimationCell = document.getElementById(this.waitingAnimationCellId);

    this.initFieldController(controller);
    var context = this;

    this.changeButtonState(controller, true);

    this.newGroupInput.onkeyup = function(event) {
        GroupManagementUtil.createNoEnterAndEscPressedHandler(event, function(){
            var isEmpty = context.emptyFieldController.isEmpty();
            controller.passiveEventHandler(function() {context.changeButtonState(controller, isEmpty)}, false);
            //Fix for FF bug. If field is empty FF places cursor at the end of the input.
            if (isEmpty) {
                context.newGroupInput.blur();
                context.newGroupInput.focus();
            }
        });
    };

    this.newGroupInput.onkeydown = function(event) {
        cancelBubble(event);
        GroupManagementUtil.createEscPressedHandler(event, function() {
            context.resetRowIntoDefaultState();
        });
        GroupManagementUtil.createEnterPressedHandler(event, function() {controller.groupEventHandler(function() {context.createUserGroup(controller)}, false)});
    };

    this.newGroupInput.onblur = function(event) {
        var isEmpty = context.emptyFieldController.isEmpty();
        controller.passiveEventHandler(function() {context.defineHighlightingState(isEmpty); context.changeButtonState(controller, isEmpty); }, false);
    };

    this.newGroupInput.onfocus = function(event) {
        controller.passiveEventHandler(function() {context.highlightRow()}, false);
    };
};

/**
 * Init field controller needed for paint empty caption
 */
AddGroupController.prototype.initFieldController = function() {
    this.emptyFieldController = new EnhancedFieldController();
    this.emptyFieldController.emptyCaption = this.getNewGroupFieldEmptyCaption();
    this.emptyFieldController.init(this.newGroupInput.id);
};

AddGroupController.prototype.getNewGroupFieldEmptyCaption = function()
{
    return  messageResource.getMessage("group_management.new_user_group", [messageResource.getMessage( this.GroupNamingKey)]);
};

/**
 * Change state of all row controls in dependence of input field emptiness
 * @param controller user group management controller
 */
AddGroupController.prototype.changeStateIfNeeded = function(controller) {
    var isEmpty = this.emptyFieldController.isEmpty();
    this.changeButtonState(controller, isEmpty);
    this.defineHighlightingState(isEmpty);
};

/**
 * Enable or disable add new user group button
 * @param controller user group management controller
 * @param isEmpty is input field empty
 */
AddGroupController.prototype.changeButtonState = function(controller, isEmpty)
{
    var isDisabledButton = isEmpty || controller.isAJAXRequestProcessedNow;

    $(this.addGroupButton).toggleClass("disabled", isDisabledButton);

    var context = this;
    this.addGroupButton.onclick = isDisabledButton ?
                                  function() {} :
                                  function()
                                  {
                                      controller.groupEventHandler(function() {context.createUserGroup(controller)}, false);
                                  };
};

/**
 * Paint border of a input as solid
 */
AddGroupController.prototype.highlightRow = function() {
    addClass(this.newGroupInputCell, this.rowHighlightedClassName);
};

AddGroupController.prototype.unhighlightRow = function() {
    removeClass(this.newGroupInputCell, this.rowHighlightedClassName);
};

/**
 * Define state of row highlighting in dependence of input field emptiness
 * @param isEmpty is input filed empty
 */
AddGroupController.prototype.defineHighlightingState = function(isEmpty) {
    var context = this;
    if (isEmpty) {
        this.unhighlightRow();
        $(this.newGroupInputCell).hover(function(e)
                                  {
                                      context.highlightRow();
                                  },
                                  function(e)
                                  {
                                      context.unhighlightRow();
                                  });
    } else {
        this.highlightRow();
        $(this.newGroupInputCell).off("hover");
    }
};

/**
 * Create new user group
 * @param controller user group management controller
 */
AddGroupController.prototype.createUserGroup = function(controller) {
    var context = this;
    if (this.emptyFieldController.isEmpty()) {
        return;
    }
    if (controller.getGroupController().isDuplicateName(this.newGroupInput.value)) {
        window.setTimeout(function() {
                var errorMessage = messageResource.getMessage("group_management.error.duplicate_name.user_group", [messageResource.getMessage(context.GroupNamingKey)]);
                GroupManagementUtil.markCellAsInvalid(context.newGroupInputCell, errorMessage);
                controller.showErrorMessage(escapeHtmlSymbols(errorMessage));
        }, 100);
    } else {
        var manager = new GroupCreateRPCManager(controller.getServiceName(), controller, context.getNewGroupData(), this.GroupNamingKey);
        RpcControlManager.saveRpcControlBoundData(manager, false);
        this.spinnerController.spin(this.waitingAnimationCellId);
    }
};

AddGroupController.prototype.getNewGroupData = function()
{
    return {
        id: -1,
        name: this.newGroupInput.value
    }
};

/**
 * Paint row as ajax processed when new user group saving begins
 */
AddGroupController.prototype.markRowAsAJAXProcessedNow = function() {
    addClass(this.addGroupRow, this.ajaxProcessingClassName);
    this.newGroupInput.disabled = "disabled";
    this.waitingAnimationCell.style.display = "table-cell";

};

/**
 * Reset add user group row into default state
 */
AddGroupController.prototype.resetRowIntoDefaultState = function() {
    removeClass(this.addGroupRow, this.ajaxProcessingClassName);
    this.newGroupInput.disabled = "";
    this.newGroupInput.value = "";
    this.waitingAnimationCell.style.display = "none";
    this.changeStateIfNeeded();
    this.newGroupInput.blur();
    this.emptyFieldController.showGreyTextIfEmpty();
    this.spinnerController.stop();
};



