/**
 * Controls highlighting rows with user groups
 * @param selectedGroupDivClassName class name of selected user group row
 * @param hoveredGroupDivClassName  class name of hovered user group row
 * @param hoveredControlClassName class name of hovered edit and delete buttons on user group row
 * @param hoverUndoAcceptEditButtonClassName class name of hovered accept edit and undo edit buttons on user group row
 */
function HighlightController(selectedGroupDivClassName, hoveredGroupDivClassName, hoveredControlClassName, hoverUndoAcceptEditButtonClassName) {
    this.hoveredGroupDivClassName = hoveredGroupDivClassName;
    this.hoveredControlClassName = hoveredControlClassName;
    this.selectedGroupDivClassName = selectedGroupDivClassName;
    this.hoverUndoAcceptEditButtonClassName = hoverUndoAcceptEditButtonClassName;
}

/**
 * Highlight selected user group row
 * @param group GroupRow instance
 */
HighlightController.prototype.highlightSelectedGroup = function(group) {
    addClass(group.div, this.selectedGroupDivClassName);
}

/**
 * Unhighlight selected user group row
 * @param group GroupRow instance
 */
HighlightController.prototype.unhighlightSelectedGroup = function(group) {
    removeClass(group.div, this.selectedGroupDivClassName);
}

/**
 * Highlight hovered user group row
 * @param group GroupRow instance
 */
HighlightController.prototype.highlightHoveredGroup = function(group) {
    addClass(group.div, this.hoveredGroupDivClassName);
}

/**
 * Unhighlight hovered user group row
 * @param group GroupRow instance
 */
HighlightController.prototype.unhighlightHoveredGroup = function(group) {
    removeClass(group.div, this.hoveredGroupDivClassName);
}

/**
 * Highlight hovered edit and delete buttons on user group row
 * @param element dom element to highlight
 */
HighlightController.prototype.highlightHoveredControl = function(element) {
    addClass(element, this.hoveredControlClassName);
}

/**
 * Unhighlight hovered edit and delete buttons on user group row
 * @param element dom element to unhighlight
 */
HighlightController.prototype.unhighlightHoveredControl = function(element) {
    removeClass(element, this.hoveredControlClassName);
}

/**
 * Highlight hovered accept edit and undo edit buttons on user group row
 * @param element dom element to highlight
 */
HighlightController.prototype.highlightHoveredUndoAcceptEditButton = function(element) {
    addClass(element, this.hoverUndoAcceptEditButtonClassName);
}

/**
 * unhighlight hovered accept edit and undo edit buttons on user group row
 * @param element dom element to unhighlight
 */
HighlightController.prototype.unhighlightHoveredUndoAcceptEditButton = function(element) {
    removeClass(element, this.hoverUndoAcceptEditButtonClassName);
}