/**
 * Base rpc manager for operations with groups(except data reading)
 * @param serviceName service name
 * @param controller group management controller
 *
 *
 * @name PRODUCT_NAME (defined in constants_js.jsp)
 */
function BaseGroupRPCManager(serviceName, controller)
{
    RpcControlManager.call(this);
    this.serviceName = serviceName;
    this.controller = controller;
}

BaseGroupRPCManager.prototype = new RpcControlManager();

BaseGroupRPCManager.prototype.getService = function() {
    return jsonrpc[this.serviceName];
};

BaseGroupRPCManager.prototype.getMethods = function() {
    return [ this.serviceName + '.deleteGroup',
             this.serviceName + '.updateGroup',
             this.serviceName + '.createGroup',
             this.serviceName + '.moveUsersToAnotherGroup'];
};

BaseGroupRPCManager.prototype.beginSavingData = function() {
    this.controller.lockNeededElementsDueToAJAX();
    this.controller.isChangesMade = true;
    this.doBeginSavingData();
};

BaseGroupRPCManager.prototype.doBeginSavingData = function() {
    /* note: inheritor should override this method */
};

BaseGroupRPCManager.prototype.endSavingData = function(success) {
    this.controller.unlockNeededElementsDueToAJAX();
    this.doEndSavingData(success);
};

BaseGroupRPCManager.prototype.doEndSavingData = function(success) {
    if (!success) {
        this.controller.rebuildViewAfterTimeOutError();
    }
};

BaseGroupRPCManager.prototype.handleNoRightsError = function(exception) {
    if (exception.name.indexOf('com.actimind.actitime.services.NoRightsForRemoteCallException') != -1)
    {
        alert( JQUERY_RPC_ERROR_MESSAGES.NO_RIGHTS );
        document.location.reload();
        return true;
    }
    return false;
};

BaseGroupRPCManager.prototype.handleSpecificError = function(exception) {
    this.controller.unlockNeededElementsDueToAJAX();
    if( exception.name.indexOf( 'com.actimind.common.services.UserGroupsDisabledException' ) != -1 )
    {
        if ( PRODUCT_NAME == Product.actiPLANS )
            alert( messageResource.getMessage( 'actiplans.user_list.js.error.user_groups_disabled' ) );
        else
            alert( messageResource.getMessage( 'userlist.js.error_ugr_disabled' ) );
        document.location.reload();
        return true;
    }
    if( exception.name.indexOf( 'com.actimind.actitime.services.NoRightsToManageAccountSettings' ) != -1 )
    {
        alert( messageResource.getMessage( 'group_management.permissions_changed' ) );
        document.location.reload();
        return true;
    }
    if (this.doHandleSpecificError(exception)) {
        return true;
    }
    var context = this;
    //Looks bad, but in the current architecture is the better solution.
    window.setTimeout(function() {context.controller.handleUnexpectedServerError()}, 500);
    return true;
};

BaseGroupRPCManager.prototype.doHandleSpecificError = function(exception) {
    /* note: inheritor should override this method */
};