/**
 * RPC manager for group creation
 * @param controller group management controller
 * @param group new group
 */
function GroupCreateRPCManager(serviceName, controller, group, groupNamingKey)
{
    BaseGroupRPCManager.call(this, serviceName, controller);
    this.group = group;
    this.groupNamingKey = groupNamingKey
}

GroupCreateRPCManager.prototype = new BaseGroupRPCManager();

GroupCreateRPCManager.prototype.doBeginSavingData = function() {
    this.controller.getAddGroupController().markRowAsAJAXProcessedNow();
};

GroupCreateRPCManager.prototype.doEndSavingData = function(success) {
    if (!success) {
        this.controller.getAddGroupController().resetRowIntoDefaultState();
    }
};

GroupCreateRPCManager.prototype.onDataSavingComplete = function(result) {
    this.group.id = result;
    this.controller.rebuildViewAfterGroupCreation(this.group);
};

GroupCreateRPCManager.prototype.doHandleSpecificError = function(exception) {
    if (exception.name.indexOf('com.actimind.actitime.administration.GroupExistsException') != -1) {
        this.controller.getAddGroupController().resetRowIntoDefaultState();
        this.controller.showDuplicateUserGroupMessage();
        var context = this;
        //Looks bad, but in the current architecture is the better solution.
        window.setTimeout(function() {context.controller.readData()}, 500);
        return true;
    }
    return false;
};

GroupCreateRPCManager.prototype.makeRemoteCall = function(callback) {
    this.getService().createGroup(callback, this.group);
};

GroupCreateRPCManager.prototype.getTimeoutMsg = function() {
    return messageResource.getEscapedMessage("group_management.timeout.create_group.user_group", [messageResource.getEscapedMessage(this.groupNamingKey)]);
};
