/**
 * RPC manager for deletion of group
 * @param controller group management controller
 * @param groupId id of group intended for deletion
 */
function GroupDeleteRPCManager(serviceName, controller, groupId, groupNamingKey)
{
    BaseGroupRPCManager.call(this, serviceName, controller);
    this.groupId = groupId;
    this.groupNamingKey = groupNamingKey;
}

GroupDeleteRPCManager.prototype = new BaseGroupRPCManager();

GroupDeleteRPCManager.prototype.doBeginSavingData = function() {
    this.controller.getGroupController().markGroupAsDeletedNow(this.groupId);
};

GroupDeleteRPCManager.prototype.onDataSavingComplete = function(result) {
    this.controller.rebuildViewAfterGroupDeletion(this.groupId);
};

GroupDeleteRPCManager.prototype.doHandleSpecificError = function(exception) {
    if (exception.name.indexOf('com.actimind.actitime.administration.NoSuchUserGroupException') != -1) {
        //If group was concurrently deleted, we ignore this fact
        this.onDataSavingComplete();
        return true;
    }
    return false;
};

GroupDeleteRPCManager.prototype.makeRemoteCall = function(callback) {
    this.getService().deleteGroup(callback, this.groupId);
};

GroupDeleteRPCManager.prototype.getTimeoutMsg = function() {
    return messageResource.getEscapedMessage("group_management.timeout.delete_group.user_group", [messageResource.getEscapedMessage(this.groupNamingKey)]);
};