/**
 * RPC manager for group editing
 * @param controller group management controller
 * @param group edited group
 */

function GroupEditRPCManager(serviceName, controller, group, groupNamingKey)
{
    BaseGroupRPCManager.call(this, serviceName, controller);
    this.group = group;
    this.groupNamingKey = groupNamingKey;
}

GroupEditRPCManager.prototype = new BaseGroupRPCManager();

GroupEditRPCManager.prototype.doBeginSavingData = function() {
    var context = this;
    this.controller.groupEventHandler(function() {context.controller.getGroupController().markGroupAsEditedNow(context.group.id)}, false);
};

GroupEditRPCManager.prototype.onDataSavingComplete = function(result) {
    this.controller.rebuildViewAfterGroupEditing(this.group);
};

GroupEditRPCManager.prototype.doHandleSpecificError = function(exception) {
    var context = this;
    if (exception.name.indexOf('com.actimind.actitime.administration.NoSuchUserGroupException') != -1) {
        this.controller.showErrorMessage(  messageResource.getMessage("group_management.group_deleted.user_group", [messageResource.getEscapedMessage(this.groupNamingKey)]));
        //Looks bad, but in the current architecture is the better solution.
        window.setTimeout(function() {context.controller.readData()}, 500);
        return true;
    } else if (exception.name.indexOf('com.actimind.actitime.administration.GroupExistsException') != -1) {
        this.controller.showDuplicateUserGroupMessage();
        //Looks bad, but in the current architecture is the better solution.
        window.setTimeout(function() {context.controller.readData(function() {context.controller.getGroupController().paintDuplicateGroupsAsRed(context.group.id, context.group.name)})}, 500);
        return true;
    }
    return false;
};

GroupEditRPCManager.prototype.makeRemoteCall = function(callback) {
    this.getService().updateGroup(callback, this.group);
};

GroupEditRPCManager.prototype.getTimeoutMsg = function() {
    return messageResource.getMessage("group_management.timeout.edit_group.user_group", [messageResource.getEscapedMessage(this.groupNamingKey)]);
};