/**
 * RPC manager for moving users into another group
 * @param serviceName service name
 * @param controller group management controller
 * @param targetGroupId id of target group. If users intended to unaasign from groups, passes Constants.USERS_WITHOUT_GROUP_ID
 * @param targetGroupName name of target group(needed to display this name in message if group deleted)
 * @param userIds ids of users intended for moving
 */

function GroupMoveRPCManager(serviceName, controller, targetGroupId, targetGroupName, userIds, groupNamingKey)
{
    BaseGroupRPCManager.call(this, serviceName, controller);
    this.targetGroupId = targetGroupId;
    this.targetGroupName = targetGroupName;
    this.userIds = userIds;
    this.groupNamingKey = groupNamingKey;
}

GroupMoveRPCManager.prototype = new BaseGroupRPCManager();

GroupMoveRPCManager.prototype.doBeginSavingData = function() {
    this.controller.getGroupController().markGroupAsMovedInNow(this.targetGroupId);
};

GroupMoveRPCManager.prototype.onDataSavingComplete = function(result) {
    if (result.length == this.userIds.length) {
        this.controller.showErrorMessage(messageResource.getMessage("group_management.drag_and_drop.all_users_deleted"));
    } else if (result.length > 0) {
        this.controller.showErrorMessage(messageResource.getMessage("group_management.drag_and_drop.some_users_deleted"));
    }
    this.controller.rebuildViewAfterUsersMoving(this.targetGroupId, this.userIds, result);
};

GroupMoveRPCManager.prototype.doHandleSpecificError = function(exception) {
    if (exception.name.indexOf('com.actimind.actitime.administration.NoSuchUserGroupException') != -1) {
        this.controller.showErrorMessage(  messageResource.getMessage("group_management.drag_and_drop.group_deleted.user_group", [messageResource.getEscapedMessage(this.groupNamingKey),escapeHtmlSymbols(this.targetGroupName)]));
        this.controller.rebuildViewAfterGroupDeletion(this.targetGroupId);
        return true;
    }
    return false;
};

GroupMoveRPCManager.prototype.makeRemoteCall = function(callback) {
    this.getService().moveUsersToAnotherGroup(callback, this.targetGroupId, this.userIds);
};

GroupMoveRPCManager.prototype.getTimeoutMsg = function() {
    return messageResource.getMessage("group_management.timeout.move_into_group.user_group", [messageResource.getEscapedMessage(this.groupNamingKey)]);
};