/**
 * Renders rows for users
 * @param userDivClassName class name of user row
 * @param userTableClassName class name of first table in user group row div
 * @param dragImgClassName class name of cell with drag arrow image
 * @param usernameCellClassName class name of cell with user name
 * @param grayedClassName class name of cell with user name grayed text
 * @param checkboxCellClassName class name of cell with checkbox
 * @param userDivIdPrefix dom id prefix of user row element
 * @param userCheckboxIdPrefix dom id prefix of user checkbox element
 */
function UserRenderer(userDivClassName, userTableClassName, dragImgClassName, usernameCellClassName, grayedClassName, checkboxCellClassName,
                      userDivIdPrefix, userCheckboxIdPrefix) {
    this.userDivClassName = userDivClassName;
    this.userTableClassName = userTableClassName;
    this.dragImgClassName = dragImgClassName;
    this.usernameCellClassName = usernameCellClassName;
    this.grayedClassName = grayedClassName;
    this.checkboxCellClassName = checkboxCellClassName;
    this.userDivIdPrefix = userDivIdPrefix;
    this.userCheckboxIdPrefix = userCheckboxIdPrefix;
}

/**
 * Generate one user row
 * @param controller user group management controller
 * @param user UserInGroup instance
 * @param highlighter for highlight user names
 */
UserRenderer.prototype.generateContent = function(controller, user, highlighter) {
    var div = document.createElement("div");
    div.className = this.userDivClassName;
    div.id = formatId(this.userDivIdPrefix, user.id);
    var table = document.createElement("table");
    table.className = this.userTableClassName;
    var tbody = document.createElement("tbody");
    var tr = document.createElement("tr");

    var dragImgTd = document.createElement("td");
    dragImgTd.className = this.dragImgClassName;
    var dragImg = document.createElement("img");
    dragImg.src = atDefImgPath("user_group_lightbox/move-group-arrow.png");
    dragImgTd.appendChild(dragImg);

    var checkBoxTd = document.createElement("td");
    checkBoxTd.className = this.checkboxCellClassName;
    var checkBox = document.createElement("input");
    checkBox.type = "checkbox";
    var checkboxId = formatId(this.userCheckboxIdPrefix, user.id);
    checkBox.id = checkboxId;

    checkBox.onfocus = function() {
        this.blur();
    };

    var nameTd = document.createElement("td");
    nameTd.className = this.usernameCellClassName;
    var overflowDiv = document.createElement("div");
    overflowDiv.className = GroupManagementUtil.ellipsisOverflowClassName;
    overflowDiv.title = user.name;
    var label = document.createElement("label");
    label.htmlFor = checkboxId;
    label.innerHTML = highlighter.highlightTokens(user.name);
    disableTextSelection(label);
    if (!user.active) {
        addClass(nameTd, this.grayedClassName);
    }

    div.appendChild(table);
    table.appendChild(tbody);
    tbody.appendChild(tr);
    tr.appendChild(dragImgTd);
    tr.appendChild(checkBoxTd);
    checkBoxTd.appendChild(checkBox);
    nameTd.appendChild(overflowDiv);
    overflowDiv.appendChild(label);
    tr.appendChild(nameTd);

    return div;
}
