function CustomerWithoutCheckboxNodeRenderer(tree, selectorFilter, expandCollapseCallback, selectionHelper)
{
    CustomerNodeRenderer.call(this, tree, selectorFilter, expandCollapseCallback);
    this.selectionHelper = selectionHelper;
}

CustomerWithoutCheckboxNodeRenderer.prototype = _.create(CustomerNodeRenderer.prototype);

CustomerWithoutCheckboxNodeRenderer.prototype.TOGGLE_DELAY_AFTER_EXPAND = 250;

CustomerWithoutCheckboxNodeRenderer.prototype.generateContent = function (treeNode, targetEl)
{
    var tableEl = document.createElement("table");
    var tBodyEl = document.createElement("tbody");
    var trEl = document.createElement("tr");
    var titleTdEl = document.createElement("td");
    var collapseTdEl = document.createElement("td");

    //visibility of this buttons are maintained by css
    var normalExpandCollapseButton = this.generateExpandCollapseButton(treeNode, true, "normalExpandCollapseButton", null);
    var disabledExpandCollapseButtonWithTitle = this.generateExpandCollapseButton(treeNode, false, "disabledExpandCollapseButtonWithTitle", this.getExpandCollapseButtonTitle(treeNode));
    var disabledExpandCollapseButtonWithoutTitle = this.generateExpandCollapseButton(treeNode, false, "disabledExpandCollapseButtonWithoutTitle", null);

    tableEl.appendChild(tBodyEl);
    tBodyEl.appendChild(trEl);
    trEl.appendChild(titleTdEl);
    trEl.appendChild(collapseTdEl);
    collapseTdEl.appendChild(normalExpandCollapseButton);
    collapseTdEl.appendChild(disabledExpandCollapseButtonWithTitle);
    collapseTdEl.appendChild(disabledExpandCollapseButtonWithoutTitle);

    var spacerDiv = document.createElement("div");
    spacerDiv.className = "spacer";
    spacerDiv.innerHTML = "<b></b>"; // for IE to display correctly

    targetEl.appendChild(tableEl);
    targetEl.appendChild(spacerDiv);

    var titleNode = document.createElement("a");
    titleNode.className = "titleNode";
    titleNode.hidefocus = "true";

    var context = this;
    titleTdEl.ondblclick = function ()
    {
        context.expandCollapseCallback(function ()
        {
            var expandController = context.tree.getExpandController();
            var selectionController = context.tree.getSelectionController();
            if (expandController.isCollapsed(treeNode))
            {
                expandController.expandAndScrollIfNeeded(treeNode);
                setTimeout(function() {selectionController.toggle(treeNode);}, context.TOGGLE_DELAY_AFTER_EXPAND);
            } else
            {
                expandController.scrollToIfNeeded(treeNode);
                selectionController.toggle(treeNode);
            }
        });
    };
    disableTextSelection(titleNode);

    var labelEl = document.createElement("span");
    labelEl.className = "ellipsisLabel";
    this.generateLabel(treeNode, labelEl);

    titleTdEl.appendChild(titleNode);
    titleNode.appendChild(labelEl);

    this.tree.getUIObjectForNode(treeNode).cbEl = titleNode;
    this.tree.getUIObjectForNode(treeNode).labelEl = labelEl;
};

CustomerWithoutCheckboxNodeRenderer.prototype.updateUI = function (treeNode)
{
    var selectionController = this.tree.getSelectionController();
    var className = "ellipsisLabel";
    if (selectionController.containsSelectedChildren(treeNode))
    {
        className += " checked";
    }
    this.tree.getUIObjectForNode(treeNode).labelEl.className = className;
    SelectionTreeNodeRenderer.prototype.updateUI.call(this, treeNode);
};

