function ExpandCollapseNodeRenderer(tree, selectorFilter, expandCollapseCallback)
{
    HierarchySelectorNodeRenderer.call(this, tree, selectorFilter);
    this.expandCollapseCallback = expandCollapseCallback;
}

ExpandCollapseNodeRenderer.prototype = _.create(HierarchySelectorNodeRenderer.prototype);

ExpandCollapseNodeRenderer.prototype.generateContent = function ( treeNode, targetEl )
{
    var tableEl = document.createElement( "table" );
    var tBodyEl = document.createElement( "tbody" );
    var trEl = document.createElement( "tr" );
    var checkboxTdEl = document.createElement( "td" );
    var collapseTdEl = document.createElement( "td" );

    //visibility of this buttons are maintained by css
    var normalExpandCollapseButton = this.generateExpandCollapseButton(treeNode, true, "normalExpandCollapseButton", null);
    var disabledExpandCollapseButtonWithTitle = this.generateExpandCollapseButton(treeNode, false, "disabledExpandCollapseButtonWithTitle", this.getExpandCollapseButtonTitle(treeNode));
    var disabledExpandCollapseButtonWithoutTitle = this.generateExpandCollapseButton(treeNode, false, "disabledExpandCollapseButtonWithoutTitle", null);

    tableEl.appendChild( tBodyEl );
    tBodyEl.appendChild( trEl );
    trEl.appendChild( checkboxTdEl );
    trEl.appendChild( collapseTdEl );
    collapseTdEl.appendChild( normalExpandCollapseButton );
    collapseTdEl.appendChild( disabledExpandCollapseButtonWithTitle );
    collapseTdEl.appendChild( disabledExpandCollapseButtonWithoutTitle );

    var spacerDiv = document.createElement("div");
    spacerDiv.className = "spacer";
    spacerDiv.innerHTML = "<b></b>"; // for IE to display correctly

    targetEl.appendChild( tableEl );
    targetEl.appendChild( spacerDiv );

    HierarchySelectorNodeRenderer.prototype.generateContent.call(this, treeNode, checkboxTdEl);
};

ExpandCollapseNodeRenderer.prototype.generateExpandCollapseButton = function(treeNode, withOnClick, additionalClass, title)
{
    var expandCollapseBtn = document.createElement( "div" );
    expandCollapseBtn.className = "collapseButton " + additionalClass;
    var context = this;
    expandCollapseBtn.onclick = (withOnClick) ? function ()
    {
        context.expandCollapseCallback( function ()
                                        {
                                            var expandController = context.tree.getExpandController();
                                            if( expandController.isCollapsed( treeNode ) )
                                                expandController.expandAndScrollIfNeeded( treeNode );
                                            else
                                                expandController.collapse( treeNode );
                                            //Fix bug for IE ( :active style doesn't disappear after click )
                                            expandCollapseBtn.blur();
                                        } );
    } : function () {};
    if (title)
    {
        expandCollapseBtn.title = title;
    }
    return expandCollapseBtn;
};

ExpandCollapseNodeRenderer.prototype.getExpandCollapseButtonTitle = function(treeNode)
{
    if( this.tree.hasActiveFilter() && !this.tree.getActiveFilter().hasKeywordFilter() && !this.tree.doesContainChildrenMatchingFilterCriteria( treeNode ) )
    {
        return this.getEmptyEntityTooltip();
    }
    return null;
};

ExpandCollapseNodeRenderer.prototype.generateLabel = function ( treeNode, labelEl )
{
    var name = treeNode.getValue().name;
    labelEl.innerHTML = this.selectorFilter.highlightTokens(name);
    labelEl.title = this.getTitle(treeNode.getValue());
};

ExpandCollapseNodeRenderer.prototype.getSpecificNodeClassName = function()
{
    return "expandCollapseNode";
};

ExpandCollapseNodeRenderer.prototype.getEmptyEntityTooltip = function()
{
    //abstract
};

ExpandCollapseNodeRenderer.prototype.getTitle = function(entity) {
    return entity.name;
};
