// TODO: This class shall not do any scrolling
// TODO: Table shall subscribe to expand / collapse events and update UI itself
function ExpandController(tree, collapsedNodeIds)
{
    this.tree = tree;
    this.nodeSet = new TreeNodeSet();
    if( collapsedNodeIds )
    {
        var rootNode = this.tree.getRootNode();
        for( var i = 0; i < collapsedNodeIds.length; i++ )
        {
            var treeNode = rootNode.getNodeById( collapsedNodeIds[i] );
            if( treeNode )
                this.nodeSet.add( treeNode );
        }
    }
}

ExpandController.prototype.expand = function( treeNode )
{
    this.nodeSet.remove( treeNode );
    this.tree.updateUIForTreeNode(treeNode);
};

ExpandController.prototype.getElementHeight = function( elm )
{
    var elmHeight = elm.offsetHeight;
    var elmMargin;
    if(document.all) {  // IE
        var marginTop = parseInt(elm.currentStyle.marginTop, 10);
        var marginBottom = parseInt(elm.currentStyle.marginBottom, 10);
        elmMargin = ( isNaN( marginTop ) ? 0 : marginTop ) + ( isNaN( marginBottom ) ? 0 : marginBottom );
    } else {  // Mozilla
        elmMargin = parseInt(document.defaultView.getComputedStyle(elm, '').getPropertyValue('margin-top')) + parseInt(document.defaultView.getComputedStyle(elm, '').getPropertyValue('margin-bottom'));
    }
    return elmHeight+elmMargin;
};


ExpandController.prototype.expandAndScrollIfNeeded = function( treeNode )
{
    this.expand( treeNode );
    this.scrollToIfNeeded( treeNode );
};

ExpandController.prototype.scrollToIfNeeded = function( treeNode )
{
    var nodeEl = this.tree.getUIObjectForNode(treeNode).nodeEl;
    var treePanel = this.tree.treeContainer.parentNode;

    var nodeElHeight = this.getElementHeight( nodeEl ) + 2;

    var customScroll = this.tree.customScroll;
    if(customScroll)
    {
        // if there is custom scroll - use scroll own functions
        var expandedNodeBottomPosition = nodeEl.offsetTop + nodeElHeight - treePanel.offsetHeight;
        if(Math.min(nodeEl.offsetTop,expandedNodeBottomPosition)  > Math.abs(customScroll.y)) // scroll only if top of nodEl will be in visible area
        {
            customScroll.scrollToElement(nodeEl, 200, null, -(customScroll.wrapperHeight - nodeElHeight));
        }
    }
    else
    {
        var to = Math.min( nodeEl.offsetTop, nodeEl.offsetTop + nodeElHeight - treePanel.offsetHeight );

        if( to > treePanel.scrollTop )
        {
            var scrollerCtor = function(panel, to)
            {
                this.panel = panel;
                this.to = to;
                this.scroll = function()
                {
                    if( this.panel.scrollTop < this.to )
                    {
                        var lastScrollTop = this.panel.scrollTop;
                        this.panel.scrollTop += ((this.to - this.panel.scrollTop) / 10) + 1;
                        if( this.panel.scrollTop <= lastScrollTop ) // prevent indefinite updates
                            return;
                        var me = this;
                        setTimeout( function(){ me.scroll() }, 7 );
                    }
                    else
                    {
                        this.panel.scrollTop = this.to;
                    }
                }
            };

            new scrollerCtor( treePanel, to ).scroll();
        }
    }
};

ExpandController.prototype.collapse = function( treeNode )
{
    if (!treeNode.isCollapsible())
        return;
    this.nodeSet.add( treeNode );
    this.tree.updateUIForTreeNode(treeNode)
};

ExpandController.prototype.toggle = function( treeNode )
{
    if( this.isCollapsed( treeNode ) )
        this.expand( treeNode );
    else
        this.collapse( treeNode );
};

ExpandController.prototype.isExpanded = function( treeNode )
{
    return !this.isCollapsed( treeNode );
};

ExpandController.prototype.isCollapsed = function( treeNode )
{
    return treeNode.isCollapsible() ? this.nodeSet.contains( treeNode ) : false;
};

ExpandController.prototype.getCollapsedNodeSet = function()
{
    return this.nodeSet;
};

ExpandController.prototype.expandSubtree = function( treeNode )
{
    if( !treeNode.hasChildren() )
        return;
    this.expandChildren( treeNode );
    this.expand( treeNode );
};

ExpandController.prototype.expandChildren = function( treeNode )
{
    var children = treeNode.getChildren();
    for( var i = 0; i < children.length; i++ )
        this.expandSubtree( children[i] );
};

ExpandController.prototype.collapseSubtree = function( treeNode )
{
    if (!treeNode.hasChildren())
        return;
    this.collapseChildren( treeNode );
    this.collapse( treeNode );
};

ExpandController.prototype.collapseChildren = function( treeNode )
{
    var children = treeNode.getChildren();
    for( var i = 0; i < children.length; i++ )
        this.collapseSubtree( children[i] );
};
