function TreeNode(value, id)
{
    this.value = value;
    this.id = !_.isUndefined(id) ? id : this.createId();
    this.level = 0;
    this.parent = null;
    this.children = [];
}

TreeNode.TYPE_DEFAULT = 0;
TreeNode.TYPE_CUSTOMER = 1;
TreeNode.TYPE_PROJECT = 2;
TreeNode.TYPE_GROUP = 3;
TreeNode.TYPE_USER = 4;
TreeNode.TYPE_DISABLED_USERS_PARENT = 5;
TreeNode.TYPE_ARCHIVED_PROJECTS_PARENT = 6;
TreeNode.TYPE_ARCHIVED_CUSTOMERS_PARENT = 7;

TreeNode.prototype = {
    getParent: function()
    {
        return this.parent;
    },

    setParent: function(parent)
    {
        this.parent = parent;
    },

    _notifyChildAddedInDepth: function(child, root)
    {
        root._childAdded(child);
        var children = child.getChildren();
        for (var i = 0; i < children.length; i++)
            this._notifyChildAddedInDepth(children[i], root);
    },

    appendChild: function(child)
    {
        this.children.push(child);
        child.setParent(this);
        child.setLevel(this.level + 1);
        var root = this.getRoot();
        if (root)
            this._notifyChildAddedInDepth(child, root);
    },

    getChildren: function()
    {
        return this.children;
    },

    getValue: function()
    {
        return this.value;
    },

    getLevel: function()
    {
        return this.level;
    },

    setLevel: function(level)
    {
        return this.level = level;
    },

    hasChildren: function()
    {
        return this.children.length > 0;
    },

    getId: function()
    {
        return this.id;
    },

    getRoot: function()
    {
        return this.parent ? this.parent.getRoot() : null;
    },

    createId: function()
    {
        return HierarchySelectorUtil.makeTreeNodeId(this.getIdPrefix(), this.getIdIndex());
    },

    getIdPrefix: function()
    {
        // Abstract
    },

    getIdIndex: function()
    {
        return this.value ? this.value.id : 0;
    },

    getType: function()
    {
        // Abstract
    },

    isCollapsible: function()
    {
        return true;
    }
};
