function TreeNodeRenderer(tree)
{
    this.tree = tree;
}

// abstract
TreeNodeRenderer.prototype.generateContent = function ( treeNode, targetEl )
{
};

TreeNodeRenderer.prototype.getNodeClasses = function ( treeNode )
{
    var classes = [ "node", this.getSpecificNodeClassName() ];

    if (this.tree.getExpandController().isCollapsed( treeNode ))
        classes.push("collapsed");

    if( this.tree.hasActiveFilter() )
    {
        if( this.tree.doesMatchFilterCriteria( treeNode ) )
            classes.push("matchesFilterCriteria");
        if( this.tree.doesContainChildrenMatchingFilterCriteria( treeNode ) )
            classes.push("containsChildrenMatchingFilterCriteria");
    }

    var parentTreeNode = treeNode.getParent();
    if (parentTreeNode && parentTreeNode.getChildren().length > 1 )
        classes.push("hasSiblings");

    return classes;
};

// abstract
TreeNodeRenderer.prototype.getSpecificNodeClassName = function()
{

};

TreeNodeRenderer.prototype.updateUI = function ( treeNode )
{
    this.tree.getUIObjectForNode(treeNode).nodeEl.className = this.getNodeClasses( treeNode ).join(" ");
};

