function TreeNodeSet()
{
    this.map = {};
}

TreeNodeSet.prototype.clone = function()
{
    var target = {};
    for( var level in this.map )
    {
        //noinspection JSUnfilteredForInLoop
        var sourceObj = this.map[level];
        var targetObj = {};

        for( var id in sourceObj )
        {
            //noinspection JSUnfilteredForInLoop
            targetObj[id] = sourceObj[id];
        }

        //noinspection JSUnfilteredForInLoop
        target[level] = targetObj;
    }

    var result = new TreeNodeSet();
    result.map = target;

    return result;
};

TreeNodeSet.prototype.add = function( treeNode )
{
    var levelMap = this.getLevelMap( treeNode.getLevel() );
    levelMap[treeNode.getId()] = treeNode;
};

TreeNodeSet.prototype.remove = function( treeNode )
{
    var levelMap = this.getLevelMap( treeNode.getLevel() );
    delete levelMap[treeNode.getId()];
};

TreeNodeSet.prototype.contains = function( treeNode )
{
    var levelObj = this.map[ treeNode.getLevel() ];
    return levelObj && levelObj[treeNode.getId()] ? true: false;
};

TreeNodeSet.prototype.isEmpty = function()
{
    for( var level in this.map )
        if( !this.isLevelEmpty( level ) )
            return false;
    return true;
};

TreeNodeSet.prototype.isLevelEmpty = function( level )
{
    if(!this.map[level])
        return true;
    //noinspection LoopStatementThatDoesntLoopJS, JSUnusedLocalSymbols
    for( var p in this.map[level] )
        return false;
    return true;
};

TreeNodeSet.prototype.getLevelMap = function( level )
{
    var levelMap = this.map[level];
    if( !levelMap )
    {
        levelMap = {};
        this.map[level] = levelMap;
    }
    return levelMap;
};

TreeNodeSet.prototype.getAllLevels = function(  )
{
    var levels = [];

    for ( var level in this.map )
        levels.push( level );

    return levels;
};

TreeNodeSet.prototype.getAllNodes = function(  )
{
    var nodes = [];
    _.each(_.values(this.map), function(levelMap)
    {
        _.each(_.values(levelMap), function(node)
        {
            nodes.push(node);
        });
    });

    return nodes;
};

TreeNodeSet.prototype.addAll = function( otherTreeNodeSet )
{
    for( var level in otherTreeNodeSet.map )
    {
        //noinspection JSUnfilteredForInLoop
        var sourceObj = otherTreeNodeSet.map[ level ];
        //noinspection JSUnfilteredForInLoop
        var targetObj = this.getLevelMap( level );
        for( var id in sourceObj )
            //noinspection JSUnfilteredForInLoop
            targetObj[id] = sourceObj[id];
    }
};


TreeNodeSet.prototype.makePathsNodeSet = function()
{
    var target = {};
    _.each(this.getAllNodes(), function(node)
    {
        var parent = node.getParent();
        while (parent)
        {
            var targetObj = target[parent.getLevel()];
            if (!targetObj)
            {
                targetObj = {};
                target[parent.getLevel()] = targetObj;
            }

            targetObj[parent.getId()] = parent;

            parent = parent.getParent();
        }
    });

    var result = new TreeNodeSet();
    result.map = target;
    return result;
};

TreeNodeSet.prototype.getNodeIds = function( level )
{
    var result = [];
    if( this.map[level] )
        for( var p in this.map[level] )
            result.push( p );
    return result
};


TreeNodeSet.prototype.intersection = function( treeNodeSet )
{
    var result = new TreeNodeSet();

    for( var level in this.map )
    {
        //noinspection JSUnfilteredForInLoop
        var levelObj = this.map[level];
        for( var id in levelObj )
        {
            //noinspection JSUnfilteredForInLoop
            var node = levelObj[id];
            if( treeNodeSet.contains( node ) )
                result.add(node);
        }
    }

    return result;
};

TreeNodeSet.prototype.union = function( treeNodeSet )
{
    var result = this.clone();
    result.addAll( treeNodeSet );
    return result;
};

TreeNodeSet.prototype.equals = function(other)
{
    var self = this;
    var selfNodes = self.getAllNodes();
    var otherNodes = other.getAllNodes();
    return selfNodes.length == otherNodes.length &&
           _.all(selfNodes, function(node) { return other.contains(node) }) &&
           _.all(otherNodes, function(node) { return self.contains(node) });
};
