function TriStateCheckboxTreeNodeRenderer(tree)
{
    SelectionTreeNodeRenderer.call(this, tree);
}

TriStateCheckboxTreeNodeRenderer.prototype = _.create(SelectionTreeNodeRenderer.prototype);

TriStateCheckboxTreeNodeRenderer.prototype.generateContent = function ( treeNode, targetEl )
{
    var triStateEl = document.createElement( "a" );
    triStateEl.hidefocus="true";
    triStateEl.onfocus = function() { this.blur() };
    var context = this;
    triStateEl.onclick = function() {
        context.tree.getSelectionController().toggle( treeNode );

        // if some node is selected/deselected inside a collapsed node,
        // then it would be smart to expand the outer node automatically,
        // so I can see my changes even after I cancel keyword search.
        var parentNode = treeNode.getParent();
        if( parentNode && (context.tree.isRootVisible() || parentNode.getLevel() > 0)  )
            context.tree.getExpandController().expand( parentNode );
    };
    disableTextSelection( triStateEl, true );

    var checkbox = document.createElement("span");
    checkbox.className = "checkbox";
    checkbox.innerHTML = "<b></b>"; // for IE to display it correctly

    var labelEl = document.createElement("span");
    labelEl.className = "ellipsisLabel";
    this.generateLabel( treeNode, labelEl );

    targetEl.appendChild( triStateEl );
    triStateEl.appendChild( checkbox );
    triStateEl.appendChild( labelEl );

    this.appendAdditionalElements(targetEl, treeNode);

    this.tree.getUIObjectForNode(treeNode).cbEl = triStateEl;
    this.tree.getUIObjectForNode(treeNode).labelEl =  labelEl;
};

TriStateCheckboxTreeNodeRenderer.prototype.appendAdditionalElements = function ( targetEl, treeNode )
{
    // do nothing by default
};

// abstract
TriStateCheckboxTreeNodeRenderer.prototype.generateLabel = function ( treeNode, labelEl )
{
};

TriStateCheckboxTreeNodeRenderer.prototype.updateLabel = function ( treeNode )
{
    this.generateLabel( treeNode, this.tree.getUIObjectForNode(treeNode).labelEl );
};


TriStateCheckboxTreeNodeRenderer.prototype.updateUI = function ( treeNode )
{
    var selectionController = this.tree.getSelectionController();
    var className = "triState";
    if( selectionController.isSelected( treeNode ) )
        className += " checked";
    else if( selectionController.containsSelectedChildren( treeNode ) )
        className += " intermediate";
    this.tree.getUIObjectForNode(treeNode).cbEl.className = className;
    SelectionTreeNodeRenderer.prototype.updateUI.call( this, treeNode );
};

