/**
 * Intended for selectors with access right filtration
 */
function AccessRightFilter(tree, searchIndex, allActiveSet, accessRight, allAccessRightGrantedSet)
{
    SelectorFilter.call(this, tree, searchIndex, allActiveSet);
    this.allAccessRightGrantedSet = allAccessRightGrantedSet;
    this.accessRight = accessRight;
    this.accessRightSet = this.allAccessRightGrantedSet;
}

AccessRightFilter.prototype = _.create(SelectorFilter.prototype);

AccessRightFilter.prototype.hasNoFilter = function()
{
    return SelectorFilter.prototype.hasNoFilter.call(this) && !this.accessRightSet;
};

AccessRightFilter.prototype.setShowUsersWithoutGrantedAccessRight = function( showWithout )
{
    if( showWithout && this.accessRightSet )
    {
        this.accessRightSet = null;
        this.reFilter();
    }
    else if( !showWithout && !this.accessRightSet )
    {
        this.accessRightSet = this.allAccessRightGrantedSet;
        this.reFilter();
    }
};

AccessRightFilter.prototype.doFilter = function()
{
    var result;
    if( this.keywordSet )
    {
        if( this.activeOnlySet || this.accessRightSet)
        {
            result = this.keywordSet.clone();
            if (this.activeOnlySet)
            {
                result = result.intersection( this.activeOnlySet );
            }
            if (this.accessRightSet)
            {
                result = result.intersection( this.accessRightSet );
            }
            if( result.isEmpty() && !this.keywordSet.isEmpty() )
            {
                result = this.keywordSet.clone();
            }
        }
        else
        {
            result = this.keywordSet.clone();
        }
    }
    else
    {
        if (this.selectedOnlySet)
        {
            result = this.selectedOnlySet.clone();
        }
        if (this.activeOnlySet)
        {
            result = this.safeIntersection(this.activeOnlySet, result);
        }
        if (this.accessRightSet)
        {
            result = this.safeIntersection(this.accessRightSet, result);
        }
    }

    return result;
};

AccessRightFilter.prototype.getVisibleNodeSet = function(selectedNodeSet)
{
    return this.safeIntersection(SelectorFilter.prototype.getVisibleNodeSet.call(this, selectedNodeSet), this.accessRightSet);
};

AccessRightFilter.prototype.getClassesForEmptyMessages = function()
{
    var classesMap = SelectorFilter.prototype.getClassesForEmptyMessages.call(this);

    if(!this.keywordSet && !this.selectedOnlySet && this.accessRightSet)
    {
        if (this.activeOnlySet)
        {
            classesMap[this.NO_ACTIVE_NODES_EMPTY_MESSAGE_KEY] = "showNoEnabledAccessRightGrantedUsers";
        }
        else
        {
            classesMap[this.NO_NODES_EMPTY_MESSAGE_KEY] = "showNoAccessRightGrantedUsers";
        }
    }
    return classesMap;
};

AccessRightFilter.prototype.getClassNameForEmptyVisibleSelection = function()
{
    return "showNoSelectedNodesHasDisabledOrWithoutAccessRight";
};
