
// requires at.js


/*
    This file is for providing visual feedback (by changing mouse cursor) when user submits a form.
 */

var hourglassedElements = [];
var stopEventsCounter = 0;

/*
    Changes the mouse cursor to hourglass-and-arrow on the specified elements.
    Sets the document.onstop event handler to change cursor back to normal when user presses the browser stop button.

    NOTE: if you call showHourglassCursorForSubmit() on form submit,
    the cursor will be changed back to normal only when the user presses the stop button twice.
    This is because the first onstop event will be fired to the document which the browser was waiting for, not the
    current document.

    @param elements - an array of document elements on which to change the cursor
                      (usually, you'll want to pass document.body and the sumbit button)
 */
function showHourglassCursorForSubmit(elements)
{
    hourglassedElements = elements;
    
    for (var i=0; i < elements.length; i++)
    {
        if (typeof(elements[i].style) != typeof(undefined))
            elements[i].style.cursor = 'progress';
    }

    stopEventsCounter = 0;
    
    document.onstop = onBrowserStop;
}


/*
    Handles the onstop event, which is fired in IE when user presses browser's 'Stop' button.

    Unfortunately, this event is also fired twice when user submits the form,
    so we ignore first two invocations and clear the cursor only on the third.

    And unfortunately, the event won't be fired the first time the user presses 'Stop' button
    (see the comment to showHourglassCursorForSubmit() function).
 */
function onBrowserStop()
{
    stopEventsCounter++;
    if (stopEventsCounter >= 3)
    {
        for (var i=0; i < hourglassedElements.length; i++)
        {
            if (typeof(hourglassedElements[i].style) != typeof(undefined))
                hourglassedElements[i].style.cursor = 'default';
        }
    }
}