/**
 * @author filatov
 */
function BaseIntegrationSection() {

}

BaseIntegrationSection.baseSpinnerOptions = {
    lines: 100, // The number of lines to draw
    length: 3, // The length of each line
    width: 2, // The line thickness
    corners: 1, // Corner roundness (0..1)
    rotate: 0, // The rotation offset
    direction: -1, // 1: clockwise, -1: counterclockwise
    color: '#9ac050', // #rgb or #rrggbb
    speed: 1, // Rounds per second
    trail: 50, // Afterglow percentage
    hwaccel: false, // Whether to use hardware acceleration
    className: 'spinner', // The CSS class to assign to the spinner
    zIndex: 2e9, // The z-index (defaults to 2000000000)
    shadow: false // Whether to render a shadow
};

BaseIntegrationSection.createAccountSpinnerOptions = $.extend({},BaseIntegrationSection.baseSpinnerOptions,
{
    radius: 11, // The radius of the inner circle
    top: '50px', // Top position relative to parent in px
    left: 'auto' // Left position relative to parent in px
});

BaseIntegrationSection.additionalRequestSpinnerOptions =$.extend({},BaseIntegrationSection.baseSpinnerOptions,{
    radius: 6, // The radius of the inner circle
    top: '17px', // Top position relative to parent in px
    left: '10px' // Left position relative to parent in px
});

BaseIntegrationSection.prototype.init = function ( sectionId, data)
{

    this._sectionId = sectionId;
    this._sectionEl = $( "#"+this._sectionId );
    this._state = IntegrationStates.ACCOUNT_DOES_NOT_EXIST;
    if ( data )
    {
        this.setState( data.integrationState );
        this.setProductUrl( data.productUrl );
        this.setContextName( data.contextName );
    }
};

BaseIntegrationSection.prototype.setContextName = function ( contextName )
{
    this._sectionEl.find(".contextName" ).html( contextName );
    this._sectionEl.find(".trialUrl" ).attr('href', this._productUrl + contextName );
};

BaseIntegrationSection.prototype.setProductUrl = function ( productUrl )
{
    this._productUrl = productUrl;
    this._sectionEl.find(".productUrl" ).html( productUrl );
};

BaseIntegrationSection.prototype.setState = function ( state )
{
    this._sectionEl.find(".integrationState" ).hide();
    this._sectionEl.removeClass(this._state);
    if ( this._runnerSpinner )
        this._runnerSpinner.stop();
    this._runnerSpinner = undefined;
    this._state = state;
    this._sectionEl.addClass(state);
    switch ( state )
    {
        case IntegrationStates.ACCOUNT_DOES_NOT_EXIST:
            this._sectionEl.find(".accountDoesNotExist").show();
            this.includeAbilityToIntegrate();
            break;
        case IntegrationStates.ACCOUNT_ACTIVE:
            this._sectionEl.find(".accountActive").show();
            break;
        case IntegrationStates.ACCOUNT_EXPIRED:
            this._sectionEl.find(".accountExpired").show();
            break;
        case IntegrationStates.ACCOUNT_CREATING:
            this.accountCreatingBlock = this._getIntegrationStateBlock("accountCreating");
            this.accountCreatingBlock.show();
            this.runSpinner(BaseIntegrationSection.createAccountSpinnerOptions,  this.accountCreatingBlock.find(".spinnerPlaceholder")[0]);
            break;
        case IntegrationStates.ACCOUNT_ERROR:
            this._sectionEl.find(".accountError").show();
            break;
        default:
            this.setCustomChangeState(state);
    }
    if (this.changeStateCallback){
        this.changeStateCallback();
    }
};

BaseIntegrationSection.prototype.runSpinner = function(spinnerOptions, spinnerPlaceholder){
    this._runnerSpinner = new Spinner(spinnerOptions);
    this._runnerSpinner.spin(spinnerPlaceholder);
};

BaseIntegrationSection.prototype.setCustomChangeState = function(state){
    //abstract
};

BaseIntegrationSection.prototype.includeAbilityToIntegrate = function ()
{
    this._integrationAbilityEnabled = true;
};

BaseIntegrationSection.prototype.turnOffAbilityToIntegrate = function ()
{
    this._integrationAbilityEnabled = false;
};

BaseIntegrationSection.prototype.isCanIntegrate = function ()
{
    return this._canIntegrate && this._integrationAbilityEnabled;
};

BaseIntegrationSection.prototype.abortAllRequests = function ()
{
    if( this._integrationRequestObject != null )
        this._integrationRequestObject.abort();
};

BaseIntegrationSection.prototype._integrationRpcErrorHandler  = function ( jqXHR, textStatus, errorThrown )
{
    this.setState( IntegrationStates.ACCOUNT_DOES_NOT_EXIST );
    if( errorThrown.exception == 'com.actimind.common.integration.HMSPWSCommunicationException' )
    {
        alert( IntegrationPopupErrorMessages.COMMUNICATION_ERROR );
        return true;
    }
};

BaseIntegrationSection.prototype._setErrorMessage = function ( message )
{
    this._sectionEl.find(".integrationError" ).html( message );
};

BaseIntegrationSection.prototype._setCanIntegrate = function ( canIntegrate )
{
    this._canIntegrate = canIntegrate;
    if (!this._canIntegrate)
    {
        this._sectionEl.addClass("noRightsForIntegration");
        this.turnOffAbilityToIntegrate();
    }
    else
    {
        this._sectionEl.addClass("hasRightsForIntegration");
        this.includeAbilityToIntegrate();
    }
};

BaseIntegrationSection.prototype._getIntegrationStateBlock = function(classBlock){
   return this._sectionEl.find("." + classBlock);
};

BaseIntegrationSection.prototype.addClass = function(className){
    this._sectionEl.addClass(className);
};


BaseIntegrationSection.prototype.removeClass = function(className){
    this._sectionEl.removeClass(className);
};

BaseIntegrationSection.prototype.getState = function()
{
    return this._state;
};

BaseIntegrationSection.prototype.setChangeStateCallback = function(changeStateCallback){
    this.changeStateCallback = changeStateCallback;
};


