/**
 * @author filatov
 */
function IndependentTrialSection() {

}

IndependentTrialSection.prototype = _.create(BaseIntegrationSection.prototype);

IndependentTrialSection.prototype.init = function ( sectionId, data, hasEmail )
{
    BaseIntegrationSection.prototype.init.call( this, sectionId, data );
    var self = this;

    this._getIntegrationStateBlock("accountDoesNotExist").on("click", function()
    {
        if (self.isCanIntegrate())
        {
            self.setState(IntegrationStates.ACCOUNT_INSERT_NAME);
        }
    });


    if ( !data )
    {
        this.setState( IntegrationStates.ACCOUNT_ERROR );
        return;
    }

    this._contextNameErrorEl = $(".contextNameError" );
    this._contextNameInput = this._getIntegrationStateBlock("nameAccountContext").find("#contextNameInput");
    this._contextNameInput.val( data.contextName );
    this._contextNameInput.off().keyup( function ()
                                        {
                                            if( !this.prevValue || this.prevValue !== this.value )
                                            {
                                                self._hideError();
                                                self.setContextNameValid(true);
                                                this.prevValue = this.value;

                                                self._abortCheckContextNameRequest();

                                                if ( !self._validateContextName() )
                                                    return;

                                                self._checkContextNameDelayTimer = setTimeout( function ()
                                                                                               {
                                                                                                   self._checkContextName();
                                                                                                   self._checkContextNameDelayTimer = null;
                                                                                               }, 200 );
                                            }
                                        } );

    this._setCanIntegrate( true );
    this.setContextNameValid(true);

    return this;
};

IndependentTrialSection.prototype._hideError = function ()
{
    this.removeClass( 'error' ) ;
};

IndependentTrialSection.prototype.setContextNameValid = function(isValid){
    this._isContextNameValid = isValid;
};

IndependentTrialSection.prototype.isContextNameValid = function(){
   return this._isContextNameValid;
};

IndependentTrialSection.prototype._showContextNameError = function ( errorMessage )
{
    this.addClass( 'error' ) ;
    this._contextNameErrorEl.html( errorMessage );
    this.setContextNameValid(false);
};

IndependentTrialSection.prototype._validateContextName = function ()
{
    var contextName = this._contextNameInput.val();
    if ( contextName == '' )
        this._showContextNameError( IntegrationPopupErrorMessages.EMPTY_CONTEXT_NAME );

    return this.isContextNameValid();
};

IndependentTrialSection.prototype.setCustomChangeState = function(state){
    switch (state){
        case IntegrationStates.ACCOUNT_INSERT_NAME:
            this.initAndShowAccountInputNameBlock();
            break;
    }
};

IndependentTrialSection.prototype.initAndShowAccountInputNameBlock = function(){
    this._stateBlockElement =  this._getIntegrationStateBlock("nameAccountContext");
    this._stateBlockElement.find(".cancelButton").off()
            .on("click", _.bind(this.setState, this, IntegrationStates.ACCOUNT_DOES_NOT_EXIST ));
    this._stateBlockElement.find(".applyButton").off()
            .on("click", _.bind(this._startIndependentTrial, this));
    this._stateBlockElement.show();
};

IndependentTrialSection.prototype._startIndependentTrial = function ()
{
    var self = this;
    if ( !this.isCanIntegrate() || !this._validateContextName() )
    {
        if ( this._integrationAbilityEnabled )
            this._blinkError();
        return;
    }

    this._abortCheckContextNameRequest();

    var contextName = this._contextNameInput.val();
    this.setState( IntegrationStates.ACCOUNT_CREATING );
    this._integrationRequestObject = makeRpcCall( "IntegrationService.startIndependentTrial", [contextName],
                                                  function () {
                                                      self.setState( IntegrationStates.ACCOUNT_ACTIVE );
                                                      self.setContextName( contextName );
                                                  },
                                                  function ( jqXHR, textStatus, errorThrown )
                                                  {
                                                      return self._integrationRpcErrorHandler(jqXHR, textStatus, errorThrown)
                                                  },
                                                  null,
                                                  IntegrationPopup.JQUERY_RPC_TIMEOUT
    );
};

IndependentTrialSection.prototype._integrationRpcErrorHandler  = function ( jqXHR, textStatus, errorThrown )
{
    if( errorThrown.exception == 'com.actimind.common.integration.ContextNameExistsException' )
    {
        this.setState( IntegrationStates.ACCOUNT_INSERT_NAME );
        this._showContextNameError( IntegrationPopupErrorMessages.CONTEXT_NAME_EXISTS );
        return true;
    }
    if( errorThrown.exception == 'com.actimind.common.integration.MalformedContextNameException' )
    {
        this.setState( IntegrationStates.ACCOUNT_INSERT_NAME );
        this._showContextNameError( IntegrationPopupErrorMessages.WRONG_CONTEXT_NAME_FORMAT );
        return true;
    }
    if( errorThrown.exception == 'com.actimind.common.integration.UserAlreadyHasIntegratedTrialException' )
    {
        this.setContextName( errorThrown.contextName );
        this.setState( IntegrationStates.ACCOUNT_ACTIVE );
        alert( IntegrationPopupErrorMessages.USER_ALREADY_HAS_TRIAL );
        return true;
    }
    if( errorThrown.exception == 'com.actimind.common.integration.UserAlreadyHasIntegratedTrialException' )
    {
        this.setContextName( errorThrown.contextName );
        this.setState( IntegrationStates.ACCOUNT_ACTIVE );
        alert( IntegrationPopupErrorMessages.USER_ALREADY_HAS_TRIAL );
        return true;
    }
    return BaseIntegrationSection.prototype._integrationRpcErrorHandler.call(this,jqXHR, textStatus, errorThrown );
};

IndependentTrialSection.prototype._abortCheckContextNameRequest = function ( )
{
    if( this._checkContextNameDelayTimer != null )
    {
        clearTimeout( this._checkContextNameDelayTimer );
        this._checkContextNameDelayTimer = null;
    }

    if( this._checkContextNameRequestObject != null )
        this._checkContextNameRequestObject.abort();
};

IndependentTrialSection.prototype._checkContextName = function ( )
{
    if ( !this.isCanIntegrate() ) return;

    var self = this;
    var contextName = this._contextNameInput.val();

    this._abortCheckContextNameRequest();

    this._checkContextNameRequestObject = makeRpcCall( "IntegrationService.checkContextNameExists",
                                                       [contextName],
                                                       function ( contextNameExists) {
                                                           if ( contextNameExists )
                                                               self._showContextNameError( IntegrationPopupErrorMessages.CONTEXT_NAME_EXISTS );
                                                       },
                                                       function ( jqXHR, textStatus, errorThrown )
                                                       {
                                                           if( errorThrown.exception == 'com.actimind.common.integration.MalformedContextNameException' )
                                                               self._showContextNameError( IntegrationPopupErrorMessages.WRONG_CONTEXT_NAME_FORMAT );
                                                           return true;
                                                       },
                                                       null,
                                                       IntegrationPopup.JQUERY_RPC_TIMEOUT
    );
};

IndependentTrialSection.prototype.setState = function ( state )
{
    if ( state == IntegrationStates.ACCOUNT_EXPIRED )
        state = IntegrationStates.ACCOUNT_ACTIVE;
    BaseIntegrationSection.prototype.setState.call( this, state );
};

IndependentTrialSection.prototype.abortAllRequests = function ()
{
    BaseIntegrationSection.prototype.abortAllRequests.call( this );
    this._abortCheckContextNameRequest();
};

IndependentTrialSection.prototype._blinkError = function ()
{
    this._sectionEl.removeClass("error");
    var self = this;
    window.setTimeout( function ()
                       {
                           self._sectionEl.addClass("error");
                       }, 250 );
};
