/**
 * @author filatov
 */
function IntegrationSection() {

}

IntegrationSection.prototype = _.create(BaseIntegrationSection.prototype);

IntegrationSection.prototype.init = function ( sectionId, data, canIntegrate, canRenew, licenseRenewalData)
{
    BaseIntegrationSection.prototype.init.call( this, sectionId, data);

    this.accountDoesNotExitsBlock = this._getIntegrationStateBlock("accountDoesNotExist");
    this.accountDoesNotExitsBlock.click(_.bind(this._integrateProduct, this, []));

    this._renewButton = this._getIntegrationStateBlock("accountExpired").find("#renewButton");

    this._setCanIntegrate( canIntegrate );
    this._setCanRenew( canRenew );
    this._licenseRenewalData = licenseRenewalData;
    if ( this._licenseRenewalData )
    {
        if (this._licenseRenewalData.trial)
        {
            this._renewButton.click(_.bind(this._purchaseLicense, this));
            this.addClass("purchase");
        }
        else
        {
            this._renewButton.click(_.bind(this._renewLicense, this));
            this.addClass("renew");
        }
    }
    return this;
};

IntegrationSection.prototype._setCanRenew = function ( canRenew )
{
    this._canRenew = canRenew;
    this.addClass(this._canRenew? "withRightsForRenewal" : "noRightsForRenewal" );
};

IntegrationSection.prototype._integrateProduct = function (params)
{
    if ( !this.isCanIntegrate() )
        return;
    var self = this;
    this.setState( IntegrationStates.ACCOUNT_CREATING );
    this._integrationRequestObject = makeRpcCall( "IntegrationService.integrateProduct", params,
                                                  function () {
                                                      self.setState( IntegrationStates.ACCOUNT_ACTIVE );
                                                  },
                                                  function ( jqXHR, textStatus, errorThrown )
                                                  {
                                                     return self._integrationRpcErrorHandler(jqXHR, textStatus, errorThrown)
                                                  },
                                                  null,
                                                  IntegrationPopup.JQUERY_RPC_TIMEOUT
    );
};

IntegrationSection.prototype._integrationRpcErrorHandler  = function ( jqXHR, textStatus, errorThrown )
{
    if( errorThrown.exception == 'com.actimind.common.integration.NoRightsForIntegrationException' )
    {
        this.setState( IntegrationStates.ACCOUNT_DOES_NOT_EXIST );
        this._setCanIntegrate( false );
        alert( IntegrationPopupErrorMessages.NO_RIGHT_FOR_INTEGRATION );
        return true;
    }
    if( errorThrown.exception == 'com.actimind.common.integration.ContextAlreadyIntegratedException' )
    {
        this.setState( IntegrationStates.ACCOUNT_ACTIVE );
        return true;
    }
    if( errorThrown.exception == 'com.actimind.common.integration.ContextNotExistsException' )
    {
        this.setState( IntegrationStates.ACCOUNT_ERROR );
        this._setErrorMessage( IntegrationPopupErrorMessages.CONTEXT_NOT_EXISTS );
        return true;
    }
    if( errorThrown.exception == 'com.actimind.common.integration.NewLicenseNotFoundException' )
    {
        this.setState( IntegrationStates.ACCOUNT_ERROR );
        return true;
    }
    return BaseIntegrationSection.prototype._integrationRpcErrorHandler.call(this,jqXHR, textStatus, errorThrown );
};

IntegrationSection.prototype._renewLicense = function ()
{
    if ( !this._canRenew || !this._licenseRenewalData )
        return;

    $('<form>', {
        "html": '<input type="text" name="installation_code" value="' + this._licenseRenewalData.installationCode + '" />' +
                '<input type="text" name="ua" value="0"/>',
        "action": this._licenseRenewalData.renewUrl,
        "method": 'post',
        "target": '_blank'
    }).hide().appendTo(document.body).submit();
};

IntegrationSection.prototype._purchaseLicense = function ()
{
    if ( !this._canRenew || !this._licenseRenewalData )
        return;

    window.open(this._licenseRenewalData.purchaseUrl,"_blank")
};
