function BaseLockingManager(userIds, fromDateStr, toDateStr, selectionUpdater)
{
    RpcControlManager.call(this);
    this.userIds = userIds;
    this.fromDateStr = fromDateStr;
    this.toDateStr = toDateStr;
    this.selectionUpdater = selectionUpdater;
}

BaseLockingManager.prototype = _.create(RpcControlManager.prototype, {
    getService: function()
    {
        return jsonrpc.LockTTService;
    },

    getMethods: function()
    {
        return ['LockTTService.lock', 'LockTTService.unlock'];
    },

    beginSavingData: function()
    {
        this.selectionUpdater.beforeRpcCall();
    },

    endSavingData: function(success)
    {
        this.selectionUpdater.afterRpcCall(success);
    },

    onDataSavingComplete: function(result)
    {
        this.selectionUpdater.handleRpcCallResult(result);
    },

    handleSpecificError: function(exception)
    {
        if (exception.name.indexOf('TimeTrackChangedException') != -1)
        {
            alert(messageResource.getMessage("lock_tt.table.selected_tt_modified"));
            document.location.reload();
            return true;
        }

        if (exception.name.indexOf('NoSuchUserException') != -1)
        {
            if (this.userIds.length == 1)
                alert(messageResource.getMessage("lock_tt.table.selected_user_deleted"));
            else
                alert(messageResource.getMessage("lock_tt.table.one_of_selected_user_deleted"));

            document.location.reload();
            return true;
        }
        if (exception.name.indexOf('CannotLockFutureException') != -1)
        {
            alert(messageResource.getMessage("lock_tt.table.future_days_not_locked"));
            document.location.reload();
            return true;
        }

        return false;
    },

    handleNoRightsError: function(exception)
    {
        if (exception.name.indexOf('com.actimind.actitime.services.NoRightsForRemoteCallException') != -1)
        {
            alert(messageResource.getMessage("error.js.json_rpc.no_rights"));
            document.location.reload();
            return true;
        }
        return false;
    }
});
