// Depends on <code>/js/message_resources.js</code> with prefix "lock_tt.table" included.

function LockEventsHandler(dateFormat, dateParser, timestamp, selectionVeil,
                           menuCancelHandler, menuCancelScope, menuDateFormat)
{
    this.menuCancelHandler = menuCancelHandler;
    this.menuCancelScope = menuCancelScope;
    this.menuDateFormat = menuDateFormat;
    this.dateFormat = dateFormat;
    this.dateParser = dateParser;
    this.timestamp = timestamp;
    this.selectionVeil = selectionVeil;
}

LockEventsHandler.prototype = _.create(Object.prototype, {
    selectionRemoved: function()
    {
        if (this.lockMenu)
        {
            this.lockMenu.hide();
        }
    },

    selectionFinished: function(selection, mouseEvent)
    {
        this.lockMenu = this.getLockMenu(selection);
        this.lockMenu.setCancelHandler(this.menuCancelHandler, this.menuCancelScope);
        if (selection.canBeLocked())
        {
            this.lockMenu.setLockEnabled(true);
            this.lockMenu.setLockHandler(this.getLockHandler(selection));

        }
        else
        {
            this.lockMenu.setLockEnabled(false);
        }

        if (selection.canBeUnlocked())
        {
            this.lockMenu.setUnlockEnabled(true);
            this.lockMenu.setUnlockHandler(this.getUnlockHandler(selection));

        }
        else
        {
            this.lockMenu.setUnlockEnabled(false);
        }

        var panelPosition = this.getLockPanelPosition(
                selection,
                mouseEvent.clientX + getBodyScrollLeft(),
                mouseEvent.clientY + getBodyScrollTop());
        this.lockMenu.show(panelPosition.x, panelPosition.y);
    },

    getLockPanelPosition: function(selection, clickX, clickY)
    {
        if (selection == NullTableRegion.instance)
            return {x: clickX + 10, y: clickY + 10};

        var topLeftCorner = calculateObjectCoords(selection.getTopLeftVisibleCell().getCellEl());
        var bottomRightTd = selection.getBottomRightVisibleCell().getCellEl();
        var bottomRightCorner = calculateObjectCoords(bottomRightTd);
        bottomRightCorner.x += bottomRightTd.clientWidth;
        bottomRightCorner.y += bottomRightTd.clientHeight;

        if (clickX < topLeftCorner.x)
            clickX = topLeftCorner.x + 10;
        else if (clickX > bottomRightCorner.x)
            clickX = bottomRightCorner.x - 10;
        else
            clickX += 10;

        if (clickY < topLeftCorner.y)
            clickY = topLeftCorner.y + 10;
        else if (clickY > bottomRightCorner.y)
            clickY = bottomRightCorner.y - 10;
        else
            clickY += 10;

        return {x: clickX, y: clickY};
    },

    /**
     * Gets lock handler for specified selection
     */
    getLockHandler: function(selection)
    {
        var selectedData = selection.getSelectedData();
        var dateFormat = this.dateFormat;
        var dateParser = this.dateParser;
        var timestamp = this.timestamp;
        var selectionVeil = this.selectionVeil;

        return selection.type.accept({
                                         visitCellSelection: function()
                                         {
                                             return function()
                                             {
                                                 var lockManager = new LockUsersManager(selectedData.userIds,
                                                         dateFormat.formatDate(selectedData.dateRange.dateFrom),
                                                         dateFormat.formatDate(selectedData.dateRange.dateTo),
                                                         new LockingSelectionUpdater(selection.lockTable, selection, dateParser, selectionVeil),
                                                         timestamp);
                                                 RpcControlManager.saveRpcControlBoundData(lockManager);
                                             };
                                         },

                                         visitDateSelection: function()
                                         {
                                             return function() { lockSelectedDates(selectedData.dateRange.dateFrom, selectedData.dateRange.dateTo); };
                                         },

                                         visitRunnerSelection: function()
                                         {
                                             return function() { lockByRunner(selectedData.dateRange.dateFrom, selectedData.dateRange.dateTo); };
                                         }
                                     });
    },

    getLockMenu: function(selection)
    {
        var leh = this;
        var getCommonMenu = function()
        {
            return new LockMenu({
                text: messageResource.getMessage("lock_tt.table.menu.lock_selected"),
                icon: imgPath("/lock_tt/lock-icon.png")
            }, {
                text: messageResource.getMessage("lock_tt.table.menu.unlock_selected"),
                icon: imgPath("/lock_tt/unlock-icon.png")
            });
        };

        var getSliderMenu = function()
        {
            var dateRange = selection.getSelectedData().dateRange;

            var dateToStr = leh.menuDateFormat.formatDate(dateRange.dateTo);

            var lockText, unlockText;
            if (selection.canBeLocked())
            {
                lockText = messageResource.getMessage("lock_tt.table.slider_menu.lock_enabled",
                                                      [dateToStr]);
                unlockText = messageResource.getMessage("lock_tt.table.slider_menu.unlock_disabled");
            }
            else
            {
                var dateBeforeStr = leh.menuDateFormat.formatDate(dateRange.dateFrom.add(Date.DAY, -1));
                var dateFromStr = leh.menuDateFormat.formatDate(dateRange.dateFrom);
                lockText = messageResource.getMessage("lock_tt.table.slider_menu.lock_disabled");
                unlockText = messageResource.getMessage("lock_tt.table.slider_menu.unlock_enabled",
                                                        [dateBeforeStr, dateFromStr, dateToStr]);
            }
            return new LockMenu({
                text: lockText,
                icon: imgPath("/lock_tt/lock-icon.png")
            }, {
                text: unlockText,
                icon: imgPath("/lock_tt/unlock-icon.png")
            });
        };

        return selection.type.accept({
                                         visitCellSelection: getCommonMenu,
                                         visitDateSelection: getCommonMenu,
                                         visitRunnerSelection: getSliderMenu
                                     })
    },

    /**
     * Gets unlock handler for specified selection
     */
    getUnlockHandler: function(selection)
    {
        var selectedData = selection.getSelectedData();
        var dateFormat = this.dateFormat;
        var selectionVeil = this.selectionVeil;

        return selection.type.accept({
                                         visitCellSelection: function()
                                         {
                                             return function()
                                             {
                                                 var unlockManager = new UnlockUsersManager(selectedData.userIds,
                                                         dateFormat.formatDate(selectedData.dateRange.dateFrom),
                                                         dateFormat.formatDate(selectedData.dateRange.dateTo),
                                                         new UnlockingSelectionUpdater(selection.lockTable, selection, selectionVeil));
                                                 RpcControlManager.saveRpcControlBoundData(unlockManager);
                                             };
                                         },

                                         visitDateSelection: function()
                                         {
                                             return function() { unlockSelectedDates(selectedData.dateRange.dateFrom, selectedData.dateRange.dateTo); };
                                         },

                                         visitRunnerSelection: function()
                                         {
                                             return function() { unlockByRunner(selectedData.dateRange.dateFrom, selectedData.dateRange.dateTo); };
                                         }
                                     });
    },

    userDayClicked: function(userId, date, dayCell)
    {
        var manager = null;
        var dateStr = this.dateFormat.formatDate(date);
        var lockTable = dayCell.getRow().getTable();
        var selection = lockTable.getSelectionController().getSelection();

        if (dayCell.getValue().locked)
        {
            manager = new UnlockUsersManager([userId], dateStr, dateStr,
                    new UnlockingSelectionUpdater(lockTable, selection, this.selectionVeil));
        }
        else
        {
            manager = new LockUsersManager([userId], dateStr, dateStr,
                    new LockingSelectionUpdater(lockTable, selection, this.dateParser, this.selectionVeil),
                    this.timestamp);
        }

        RpcControlManager.saveRpcControlBoundData(manager, false);
    }
});
