function LockTable(todayDate, weekStartDay)
{
    this.listeners = [];
    this.viewerTodayDate = todayDate;
    this.viewerTodayColIdx = -1;
    this.weekStartDay = weekStartDay;

    Table.call(this);

    this.datesRow = null;

    this.tooltipController = null;
    this.commentsController = null;
    this.runnerController = null;
}

LockTable.prototype = _.create(Table.prototype, {

    createSelectionController: function()
    {
        return new LockTableSelectionController();
    },

    createHoverController: function()
    {
        return new LockTableHoverController();
    },

    createMouseEventsHandler: function()
    {
        var lockTableMouseEventsHandler = new LockTableMouseEventsHandler(this, this.getSelectionController(), this.getHoverController(), this.getRunnerController());
        var table = this;
        lockTableMouseEventsHandler.addSelectionFinishedListener(
                function(mouseUpEvent)
                {
                    table.fireSelectionFinished(getEventObj(mouseUpEvent));
                }
        );
        return lockTableMouseEventsHandler;
    },

    init: function(runnerController, tooltipController, commentsController)
    {
        this.runnerController = runnerController;
        this.tooltipController = tooltipController;
        this.commentsController = commentsController;

        // TODO: remove mouseEventsHandler initialization and call Table.prototype.init method after Table class refactoring, now is calling in constructor
        this.mouseEventsHandler = this.createMouseEventsHandler();

        var self = this;
        commentsController.addViewCommentsListener({
                                                       onViewComments: function()
                                                       {
                                                           self.setIgnoreUserActivity(true);
                                                           self.hoverController.unhover();
                                                           tooltipController.enableTooltips(false);
                                                       },

                                                       onCloseComments: function()
                                                       {
                                                           self.setIgnoreUserActivity(false);
                                                           tooltipController.enableTooltips(true);
                                                       }
                                                   });

        this.selectionController.addSelectionChangeListener(function(selectionChangeEvent)
                                                       {
                                                           if (selectionChangeEvent.newValue == NullTableRegion.instance)
                                                           {
                                                               commentsController.enableComments(true);
                                                               tooltipController.enableTooltips(true);
                                                           }
                                                           else
                                                           {
                                                               commentsController.enableComments(false);
                                                               tooltipController.enableTooltips(false);
                                                           }
                                                       });

    },

    getHoverController: function()
    {
        return this.hoverController;
    },

    getRunnerController: function()
    {
        return this.runnerController;
    },

    getCommentsController: function()
    {
        return this.commentsController;
    },

    getTooltipController: function()
    {
        return this.tooltipController;
    },

    containsViewerTodayColumn: function()
    {
        return this.viewerTodayColIdx > 0;
    },

    getViewerTodayColIdx: function()
    {
        return this.viewerTodayColIdx;
    },

    addDatesRow: function(datesRow)
    {
        this.addRow(datesRow);
        this.datesRow = datesRow;

        for (var colIdx = 1; colIdx < datesRow.getCells().length; colIdx++)
        {
            if (datesRow.getCell(colIdx).getValue().date.getTime() == this.viewerTodayDate.getTime())
            {
                this.viewerTodayColIdx = colIdx;
                break;
            }
        }
    },

    getDatesRow: function()
    {
        return this.datesRow;
    },

    selectionChanged: function(selectionChangeEvent)
    {
        var updateRegion = this.computeChangedRegion(selectionChangeEvent);

        if (updateRegion != NullTableRegion.instance)
        {
            //should update selection bottom border
            if (updateRegion.rowMax < this.rows.length - 1)
                updateRegion = new TableRegion(updateRegion.rowMin, updateRegion.colMin, updateRegion.rowMax + 1, updateRegion.colMax);

            //should update selection right border
            if (updateRegion.colMax < this.getLastRow().getCells().length - 1)
                updateRegion = new TableRegion(updateRegion.rowMin, updateRegion.colMin, updateRegion.rowMax, updateRegion.colMax + 1);

            this.updateCellRegionUI(updateRegion);

            //update marked selected dates
            if (updateRegion.rowMin > 0)
                for (var colIdx = updateRegion.colMin; colIdx <= updateRegion.colMax; colIdx++)
                    this.getDatesRow().updateCellUI(colIdx);

            for (var rowIdx = updateRegion.rowMin; rowIdx <= updateRegion.rowMax; rowIdx++)
                this.rows[rowIdx].updateCellUI(0);
        } else if (selectionChangeEvent.oldValue == NullTableRegion.instance ||
                 selectionChangeEvent.newValue == NullTableRegion.instance)
        {
            var userRows = this.getUserRows();
            for (var userRowIdx = 0; userRowIdx < userRows.length; userRowIdx++)
                userRows[userRowIdx].updateCellUI(0);
        }

        if (selectionChangeEvent.newValue != NullTableRegion.instance)
            this.fireSelectionRemoved();
    },

    hoverChanged: function(regionChangeEvent)
    {
        if (regionChangeEvent.oldValue != NullTableRegion.instance)
            this.updateCellRegionUI(regionChangeEvent.oldValue);
        if (regionChangeEvent.newValue != NullTableRegion.instance)
            this.updateCellRegionUI(regionChangeEvent.newValue);
    },

    addLockEventsListener: function(listener)
    {
        this.listeners.push(listener);
    },

    fireSelectionFinished: function(mouseUpEvent)
    {
        if (this.selectionController.hasSelection())
        {
            var selection = this.getSelectionController().getSelection();
            for (var i = 0; i < this.listeners.length; i++)
                this.listeners[i].selectionFinished(selection, mouseUpEvent);
        }
    },

    fireSelectionRemoved: function()
    {
        for (var i = 0; i < this.listeners.length; i++)
            this.listeners[i].selectionRemoved();
    },

    fireUserCellClicked: function(userDayCell)
    {
        for (var i = 0; i < this.listeners.length; i++)
        {
            this.listeners[i].userDayClicked(
                    userDayCell.getRow().getHeaderCell().getValue().id,
                    this.getDatesRow().getCell(userDayCell.getColIdx()).getValue().date,
                    userDayCell
            );
        }
    },

    /**
     * Gets user rows from specified row range.
     * Returns all user rows if row range not specified.
     *
     * @param startRowIdx - range start
     * @param endRowIdx - range end
     */
    getUserRows: function(startRowIdx, endRowIdx)
    {
        var startIdx = startRowIdx ? startRowIdx : 0;
        var endIdx = endRowIdx ? endRowIdx : (this.rows.length - 1);

        var userRows = [];
        for (var i = startIdx; i <= endIdx; i++)
        {
            var currentRow = this.rows[i];
            var isUserRow = currentRow.accept({
                                                  visitUserRow: function(cell) { return true; },
                                                  visitDetailsRow: function(cell) { return false; },
                                                  visitDatesRow: function(cell) { return false; },
                                                  visitRunnerRow: function(cell) { return false; },
                                                  visitInactiveRow: function(cell) { return false; }
                                              }, null);

            if (isUserRow)
                userRows.push(currentRow);
        }
        return userRows;
    },

    getLastSelectableRow: function()
    {
        for (var i = this.rows.length - 1; i > 0; i--)
        {
            var row = this.rows[i];
            var isSelectableRow = row.accept({
                                                 visitUserRow: function(cell) { return true; },
                                                 visitDetailsRow: function(cell) { return true; },
                                                 visitDatesRow: function(cell) { return true; },
                                                 visitRunnerRow: function(cell) { return true; },
                                                 visitInactiveRow: function(cell) { return false; }
                                             }, null);

            if (isSelectableRow)
                return row;
        }
    },

    setIgnoreUserActivity: function(ignore)
    {
        this.getMouseEventsHandler().setIgnoreMouseEvents(ignore);

        var userRows = this.getUserRows();
        for (var i = 0; i < userRows.length; i++)
        {
            userRows[i].disableExpandCollapse(ignore);
        }
    },

    getWeekStartDay: function()
    {
        return this.weekStartDay;
    },

    isHolidayColumn: function(colIdx)
    {
        return !this.getDatesRow().getCell(colIdx).getValue().working;
    },

    updateCellRegionUI: function(region)
    {
        // update main region and update also cells that appears right from that region
        for (var rowIdx = region.rowMin; rowIdx <= region.rowMax; rowIdx++)
            for (var colIdx = region.colMin; colIdx <= Math.min(region.colMax + 1, this.getLastRow().getCells().length - 1); colIdx++)
                this.rows[rowIdx].updateCellUI(colIdx);
    }
});
