function LockTableCellSelectionState(mouseEventsHandler)
{
    LockTableMouseEventsHandlerState.apply(this, arguments);
}

LockTableCellSelectionState.prototype = _.create(LockTableMouseEventsHandlerState.prototype, {
    mouseOver: function(evt, cell)
    {
        // refine columns if we have a row header cell
        if (cell.getColIdx() == 0)
            cell = cell.getRow().getCell(1);

        var mouseEventsHandler = this.mouseEventsHandler;

        return cell.getRow().accept(
                {
                    computeAndSetSelection: function(cell2)
                    {
                        var cell1 = mouseEventsHandler.getSelectionController().getSelectionAnchor();
                        var lowerCell = (cell1.getRow().getRowIdx() > cell2.getRow().getRowIdx()) ? cell1 : cell2;

                        if (lowerCell.getRow().getChildRows().length > 0)
                        {
                            lowerCell = lowerCell.getRow().getLastChildRow().getCell(lowerCell.getColIdx());
                        }
                        mouseEventsHandler.getSelectionController().changeSelectionTo(cell2);
                    },

                    visitUserRow: function(cell)
                    {
                        this.computeAndSetSelection(cell);
                    },

                    visitDetailsRow: function(cell)
                    {
                        this.computeAndSetSelection(cell.getRow().getParentRow().getCell(cell.getColIdx()));
                    },

                    visitDatesRow: function(cell)
                    {
                        cell = cell.getRow().getTable().getCell(cell.getRow().getRowIdx() + 1, cell.getColIdx());
                        cell.getRow().accept(this, cell);
                    },

                    visitRunnerRow: function(cell)
                    {
                        cell = cell.getRow().getTable().getCell(cell.getRow().getRowIdx() + 2, cell.getColIdx());
                        cell.getRow().accept(this, cell);
                    },

                    visitInactiveRow: function(cell)
                    {
                        var anchor = mouseEventsHandler.getSelectionController().getSelectionAnchor();
                        var rowIdx = cell.getRow().getRowIdx() +
                                     ( anchor.getRow().getRowIdx() > cell.getRow().getRowIdx() ? 1 : -1 );
                        mouseEventsHandler.mouseOver(evt, cell.getRow().getTable().getCell(rowIdx, cell.getColIdx()));
                    }
                },
                cell
        );
    },

    mouseUp: function(evt, cell)
    {
        var selection = this.mouseEventsHandler.getSelectionController().getSelection();
        if (selection == null || selection == NullTableRegion.instance)
        {
            var userCell;

            if (cell)
            {
                userCell = cell.getRow().accept(
                        {
                            visitUserRow: function(cell) { return cell; },
                            visitDetailsRow: function(cell) { return cell.getRow().getParentRow().getCell(cell.getColIdx()); },
                            visitDatesRow: function(cell) { return null; },
                            visitRunnerRow: function(cell) { return null; },
                            visitInactiveRow: function(cell) { return null; }
                        },
                        cell
                );
            }
            else
            {
                userCell = this.mouseEventsHandler.getSelectionController().getSelectionAnchor();
            }

            if (userCell != null)
            {
                var lastDetailsRow = userCell.getRow().getLastChildRow();
                if (lastDetailsRow != null)
                    cell = lastDetailsRow.getCell(userCell.getColIdx());
                else
                    cell = userCell;

                this.mouseEventsHandler.getSelectionController().changeSelectionTo(cell);
                this.mouseEventsHandler.setRangeSelectedState();
                cell.getRow().getTable().fireUserCellClicked(userCell);
            }
        }
        else
        {
            this.mouseEventsHandler.setRangeSelectedState();
            this.mouseEventsHandler.fireSelectionFinished(evt);
        }
    },

    mouseMove: function(evt, cell)
    {
        if (Ext.isIE && window.event && window.event.button == 0)
        {
            this.mouseEventsHandler.getSelectionController().clearSelection();
            this.mouseEventsHandler.setHoverState();
        }
    }
});