function LockTableHoverState(mouseEventsHandler)
{
    LockTableMouseEventsHandlerState.apply(this, arguments);
}

LockTableHoverState.prototype = _.create(LockTableMouseEventsHandlerState.prototype, {
    mouseOver: function(evt, cell)
    {
        //ignore hover for row header
        if (cell.getColIdx() == 0)
            return;

        //ignore hover if cell already hovered
        if (this.mouseEventsHandler.hoverController.isHovered(cell))
            return;

        var mouseEventsHandler = this.mouseEventsHandler;
        cell.getRow().accept(
                {
                    visitUserRow: function(cell)
                    {
                        var endCell = cell;
                        if (cell.getRow().getChildRows().length > 0)
                        {
                            endCell = cell.getRow().getLastChildRow().getCell(cell.getColIdx());
                        }
                        mouseEventsHandler.hoverController.hover(cell, endCell);
                    },

                    visitDetailsRow: function(cell)
                    {
                        this.visitUserRow(cell.getRow().getParentRow().getCell(cell.getColIdx()));
                    },

                    visitDatesRow: function(cell)
                    {
                        mouseEventsHandler.hoverController.hover(cell, cell);
                    },

                    visitRunnerRow: function(cell)
                    {
                        mouseEventsHandler.getRunnerController().hover(cell);
                    },

                    visitInactiveRow: function(cell)
                    {
                        //do nothing for inactive row
                    }
                },
                cell
        );
    },

    mouseOut: function(evt, cell)
    {
        evt = getEventObj(evt);
        if (this.shouldRemoveHover(cell, getTargetElement(evt)))
        {
            var mouseEventsHandler = this.mouseEventsHandler;
            cell.getRow().accept(
                    {
                        visitUserRow: function(cell) { mouseEventsHandler.hoverController.unhover(); },
                        visitDetailsRow: function(cell) { mouseEventsHandler.hoverController.unhover(); },
                        visitDatesRow: function(cell) { mouseEventsHandler.hoverController.unhover(); },
                        visitRunnerRow: function(cell)
                        {
                            mouseEventsHandler.getRunnerController().unhover(cell);
                        },
                        visitInactiveRow: function(cell) { }
                    },
                    cell
            );
        }
    },

    shouldRemoveHover: function(cell, targetElement)
    {
        var targetCell = null;
        while (targetElement)
        {
            if (targetElement.cellObj)
            {
                targetCell = targetElement.cellObj;
                break;
            }
            targetElement = targetElement.parentNode;
        }

        if (targetCell == null)
            return true;

        return !this.hasEqualHover(cell, targetCell);
    },

    hasEqualHover: function(cell1, cell2)
    {
        if (cell1 == cell2)
            return true;

        //only details and user cells in the same column can have equal hovers
        if (cell1.getColIdx() != cell2.getColIdx())
            return false;

        var userCellGetter = {
            visitUserRow: function(cell) { return cell },
            visitDetailsRow: function(cell) { return cell.getRow().getParentRow().getCell(cell.getColIdx()) },
            visitDatesRow: function(cell) { return null },
            visitRunnerRow: function(cell) { return null },
            visitInactiveRow: function(cell) { return null }
        };

        var userDayCell1 = cell1.getRow().accept(userCellGetter, cell1);
        var userDayCell2 = cell2.getRow().accept(userCellGetter, cell2);

        return userDayCell1 != null && userDayCell1 == userDayCell2;
    },

    mouseDown: function(evt, cell)
    {
        //ignore selection start for row headers
        if (cell.getColIdx() == 0)
            return;

        var mouseEventsHandler = this.mouseEventsHandler;
        cell.getRow().accept(
                {
                    visitUserRow: function(cell)
                    {
                        mouseEventsHandler.hoverController.unhover();
                        mouseEventsHandler.setCellSelectionState(cell);
                    },

                    visitDetailsRow: function(cell)
                    {
                        mouseEventsHandler.hoverController.unhover();
                        mouseEventsHandler.setCellSelectionState(
                                cell.getRow().getParentRow().getCell(cell.getColIdx())
                        );
                    },

                    visitDatesRow: function(cell)
                    {
                        mouseEventsHandler.hoverController.unhover();
                        mouseEventsHandler.setColumnSelectionState(cell);
                    },

                    visitRunnerRow: function(cell) { },

                    visitInactiveRow: function(cell) { }
                },
                cell
        );
    },

    mouseUp: function(evt, cell)
    {
        if (cell == null)
            return;

        if (cell.getColIdx() == 0)
            return;

        var mouseEventsHandler = this.mouseEventsHandler;

        cell.getRow().accept(
                {
                    visitUserRow: function(cell) { },
                    visitDetailsRow: function(cell) { },
                    visitDatesRow: function(cell) { },
                    visitRunnerRow: function(cell)
                    {
                        mouseEventsHandler.getSelectionController().clearSelection();
                        mouseEventsHandler.getRunnerController().setNewLastLockedCell(cell);
                        if (mouseEventsHandler.getSelectionController().hasSelection())
                        {
                            mouseEventsHandler.setRangeSelectedState();
                            mouseEventsHandler.fireSelectionFinished(evt);
                        }
                    },
                    visitInactiveRow: function(cell) { }
                },
                cell
        );
    }
});