function LockTableMouseEventsHandler(table, selectionController, hoverController, runnerController)
{
    TableMouseEventsHandler.call(this, table, selectionController, hoverController);
    this.runnerController = runnerController;
    this.cellSelectionState = new LockTableCellSelectionState(this);
    this.columnSelectionState = new ColumnSelectionState(this);
    this.delayedSelectionState = new DelayedSelectionState(this);
    this.rangeSelectedState = new LockTableRangeSelectedState(this);
    this.hoverState = new LockTableHoverState(this);
    this.state = this.hoverState;
    this.selectionFinishedListeners = [];
    this.ignoreMouseEvents = false;

    //should handle mouseUp which occurs beyond table borders
    var self = this;
    var prevMouseUp = document.onmouseup;
    $(document).mouseup(function(evt)
                        {
                            if (prevMouseUp)
                                prevMouseUp.call(this, evt);
                            self.mouseUp(evt, null);
                        });
}

LockTableMouseEventsHandler.prototype = _.create(TableMouseEventsHandler.prototype, {
    mouseOver: function(evt, cell)
    {
        this.state.mouseOver(evt, cell);
    },

    mouseOut: function(evt, cell)
    {
        this.state.mouseOut(evt, cell);
    },

    mouseDown: function(evt, cell)
    {
        if (this.isLeftButton(evt))
            this.state.mouseDown(evt, cell);
    },

    mouseUp: function(evt, cell)
    {
        if (this.isLeftButton(evt))
            this.state.mouseUp(evt, cell);
    },

    mouseMove: function(evt, cell)
    {
        this.state.mouseMove(evt, cell);
    },

    isLeftButton: function(evt)
    {
        return whichButtonPressed(getEventObj(evt)) == "left";
    },

    getRunnerController: function()
    {
        return this.runnerController;
    },

    setIgnoreMouseEvents: function(ignore)
    {
        if (ignore && !this.ignoredState)
        {
            this.ignoredState = this.state;
            this.state = new IgnoreMouseEventsState(this);
        }
        else if (!ignore && this.ignoredState)
        {
            if (this.selectionController.hasSelection())
            {
                this.state = this.ignoredState;
                this.ignoredState = undefined;
            }
            else
            {
                this.ignoredState = undefined;
                var lastHoveredCell = this.state.lastHoveredCell;
                this.setHoverState();
                if (lastHoveredCell)
                    this.mouseOver(null, lastHoveredCell);
            }
        }
    },

    setCellSelectionState: function(anchorCell)
    {
        this.getSelectionController().setSelectionAnchor(anchorCell);
        this.getSelectionController().setSelectionType(new CellSelectionType());
        this.state = this.cellSelectionState;
    },

    setColumnSelectionState: function(anchorCell)
    {
        this.getSelectionController().setSelectionAnchor(anchorCell);
        this.getSelectionController().setSelectionType(new DateSelectionType());
        this.getSelectionController().changeSelectionTo(anchorCell.getRow().getTable().getLastSelectableRow().getCell(anchorCell.getColIdx()));
        this.state = this.columnSelectionState;
    },

    setHoverState: function()
    {
        this.state = this.hoverState;
    },

    setDelayedSelectionState: function(stateSwitcher)
    {
        this.delayedSelectionState.setDelayedStateSwitcher(stateSwitcher);
        this.state = this.delayedSelectionState;
    },

    setRangeSelectedState: function()
    {
        this.state = this.rangeSelectedState;
    },


    addSelectionFinishedListener: function(listener)
    {
        this.selectionFinishedListeners.push(listener);
    },

    fireSelectionFinished: function(mouseUpEvent)
    {
        for (var i = 0; i < this.selectionFinishedListeners.length; i++)
            this.selectionFinishedListeners[i].call(this.selectionFinishedListeners[i], mouseUpEvent);
    }
});
