function LockTableRegion(type, lockTable, startRow, startCol, endRow, endCol)
{
    TableRegion.call(this, startRow, startCol, endRow, endCol);
    this.type = type;
    this.lockTable = lockTable;
}

LockTableRegion.prototype = _.create(TableRegion.prototype, {

    createFromCells: function(type, lockTable, startCell, endCell)
    {
        return new LockTableRegion(
                type, lockTable,
                startCell.getRow().getRowIdx(), startCell.getColIdx(),
                endCell.getRow().getRowIdx(), endCell.getColIdx()
        );
    },

    getSelectedData: function()
    {
        return this.type.getSelectedData(this.lockTable, this);
    },

    canBeLocked: function()
    {
        return this.type.canBeLocked(this.lockTable, this);
    },

    canBeUnlocked: function()
    {
        return this.type.canBeUnlocked(this.lockTable, this);
    },

    getTopLeftVisibleCell: function()
    {
        return this.lockTable.getCell(this.rowMin, this.colMin);
    },

    getBottomRightVisibleCell: function()
    {
        var region = this;
        var row = this.lockTable.rows[region.rowMax];
        return row.accept({
                              visitUserRow: function(cell) { return row.getCell(region.colMax); },
                              visitDetailsRow: function(cell)
                              {
                                  var userRow = row.getParentRow();
                                  if (userRow.isExpanded())
                                      return row.getCell(region.colMax);
                                  else
                                      return userRow.getCell(region.colMax);
                              },
                              visitDatesRow: _.constant(null),
                              visitRunnerRow: _.constant(null),
                              visitInactiveRow: _.constant(null)
                          }, null
        );
    },

    visitUserCells: function(visitor)
    {
        var userRows = this.lockTable.getUserRows(this.rowMin, this.rowMax);
        for (var i = 0; i < userRows.length; i++)
        {
            var currentRow = userRows[i];

            for (var colIdx = this.colMin; colIdx <= this.colMax; colIdx++)
            {
                visitor(currentRow.getCell(colIdx));
            }
        }
    }

});
