function LockTableSelectionController()
{
    SelectionController.call(this);
    this.selectionType = null;
    this.lockTable = null;
}

LockTableSelectionController.prototype = _.create(SelectionController.prototype, {
    setLockTable: function(lockTable)
    {
        this.lockTable = lockTable;
    },

    isLeftmostSelectedCell: function(cell)
    {
        if (this.isSelected(cell.getRow().getRowIdx(), cell.getColIdx()))
            return this.selectedRegion.colMin == cell.getColIdx();
    },

    isUpwardmostSelectedCell: function(cell)
    {
        if (this.isSelected(cell.getRow().getRowIdx(), cell.getColIdx()))
            return this.selectedRegion.rowMin == cell.getRow().getRowIdx();
    },

    isRightmostSelectedCell: function(cell)
    {
        if (this.isSelected(cell.getRow().getRowIdx(), cell.getColIdx()))
            return this.selectedRegion.colMax == cell.getColIdx();
    },

    isDownwardmostSelectedCell: function(cell)
    {
        if (this.isSelected(cell.getRow().getRowIdx(), cell.getColIdx()))
            return this.selectedRegion.rowMax == cell.getRow().getRowIdx();
    },

    visitSelection: function(visitor)
    {
        if (this.hasSelection())
            return this.selectionType.accept(visitor, this.getSelection());
    },

    setSelectionType: function(selectionType)
    {
        var oldSelection = this.getSelection();
        this.selectionType = selectionType;
        this.fireSelectionChangeEvent(oldSelection);
    },

    createSelection: function(cell)
    {
        return LockTableRegion.prototype.createFromCells(this.selectionType, this.lockTable, this.selectionAnchor, cell)
    },


    clearSelection: function()
    {
        this.selectionAnchor = null;
        if (this.hasSelection())
        {
            var oldSelection = this.getSelection();
            this.selectionType = null;
            this.selectedRegion = NullTableRegion.instance;
            this.fireSelectionChangeEvent(oldSelection);
        }
    },

    hasSelection: function()
    {
        return this.selectedRegion != NullTableRegion.instance || this.selectionType != null;
    },

    isRowIncludedIntoSelection: function(rowIdx)
    {
        return this.visitSelection({
                                       visitCellSelection: function(selection)
                                       {
                                           if (selection != NullTableRegion.instance)
                                               return selection.rowMin <= rowIdx && rowIdx <= selection.rowMax;
                                           return false;
                                       },
                                       visitDateSelection: _.constant(true),
                                       visitRunnerSelection: _.constant(true)
                                   });
    }
});
