function DayDetailsCellRenderer(cssBaseClass, cssTopBorderClass)
{
    DayCellRenderer.call(this);
    this.cssBaseClass = cssBaseClass;
    this.cssTopBorderClass = cssTopBorderClass;
}

DayDetailsCellRenderer.prototype = _.create(DayCellRenderer.prototype, {
    getTopBorderClassName: function(cell)
    {
        return this.cssTopBorderClass;
    },

    getComplicatedBorderClassName: function(cell)
    {
        var firstRow = cell.getRow().isFirstChildRow();
        var lastRow = cell.getRow().isLastChildRow();
        if (firstRow || lastRow)
            return (firstRow && lastRow) ? "singleDetailsRow" :
                   (firstRow ? "firstDetailsRow" : "lastDetailsRow");
        else
            return "";
    },

    getCellClassName: function(cell)
    {
        var className = [this.cssBaseClass];

        if (cell.getRow().getParentRow().isTodayCell(cell))
            className.push("Today");
        else if (cell.getRow().getParentRow().isHolidayCell(cell))
            className.push("Holiday");

        if (this.isLocked(cell))
            className.push("Locked");
        if (this.isApproved(cell))
            className.push("Approved");
        if (this.isSelected(cell))
            className.push("Selected");
        else if (this.isHovered(cell))
            className.push("Hover");
        return className.join(" ");
    },

    isLocked: function(cell)
    {
        return cell.getRow().getParentRow().getCell(cell.getColIdx()).getValue().locked;
    },

    isApproved: function(cell)
    {
        var approvedValue = cell.getRow().getParentRow().getCell(cell.getColIdx()).getValue().approveInfo;
        return approvedValue && ( approvedValue.approved || approvedValue.autoApproved);
    },

    addTooltip: function(cell)
    {
        var leaveInfo = cell.getValue();
        cell.getRow().getTable().getTooltipController().addDayTooltip(cell.getCellEl(),
                                                                      cell.getRow().getParentRow().getHeaderCell().getValue().id,
                                                                      cell.getRow().getTable().getDatesRow().getCell(cell.getColIdx()).getValue().date,
                                                                      {showWithoutTime: true, leaveInfo: leaveInfo.leaveInfo, time: leaveInfo.time},
                                                                      {showOvertimeWhenZero: false, time: 0, showTime: false, overtimeText: cell.getRow().getHeaderCell().getValue()},
                                                                      cell.getRow().getParentRow().getCell(cell.getColIdx()).getValue());
    }
});
