function LockTableCellRenderer()
{
    TableCellRenderer.call(this);
}

LockTableCellRenderer.prototype = _.create(TableCellRenderer.prototype, {
    updateUI: function(cell)
    {
        var className = [this.getCellClassName(cell),
                         this.getTopBorderClassName(cell),
                         this.getRightBorderClassName(cell),
                         this.getLeftBorderClassName(cell),
                         this.getComplicatedBorderClassName(cell)].join(" ");
        this.setCssClass(cell, className);
    },

    getComplicatedBorderClassName: function(cell)
    {
        return "";
    },

    getTopBorderClassName: function(cell)
    {
        var selectionController = this.getSelectionController(cell);
        if (this.isSelected(cell))
        {
            if (selectionController.isUpwardmostSelectedCell(cell) &&
                selectionController.visitSelection(this.getSelectionTopBorderChecker()))
                return "selectionTopBorder";
        }
        else
        {
            var upperRowIdx = cell.getRow().getRowIdx() - 1;
            if (upperRowIdx > 0)
            {
                var upperCell = cell.getRow().getTable().getCell(upperRowIdx, cell.getColIdx());
                if (selectionController.isDownwardmostSelectedCell(upperCell) &&
                    this.shouldDrawSelectionTopBorder(selectionController))
                    return "selectionTopBorder";
            }
        }
        return "";
    },

    getRightBorderClassName: function(cell)
    {
        if (this.isLastCell(cell))
        {
            if (this.isSelected(cell) &&
                this.getSelectionController(cell).visitSelection(this.getSelectionRightBorderChecker(cell)))
            {
                return "selectionRightBorder";
            }
            else
            {
                return this.getLastCellRightBorderClassName();
            }
        }
        return "";
    },

    getLastCellRightBorderClassName: function()
    {
        return "lockTableRightBorder";
    },

    getSelectionLeftBorderChecker: function(cell)
    {
        return {
            visitCellSelection: function() { return true; },
            visitDateSelection: function() { return true; },
            visitRunnerSelection: function()
            {
                var runnerController = cell.getRow().getTable().getRunnerController();
                return runnerController.getLastLockedDate() != null &&
                       runnerController.getNewLastLockDate() < runnerController.getLastLockedDate();
            }
        };
    },

    shouldDrawSelectionTopBorder: function(selectionController)
    {
        return selectionController.visitSelection({
                                                      visitCellSelection: function() { return true; },
                                                      visitDateSelection: function() { return false; },
                                                      visitRunnerSelection: function() { return false; }
                                                  });
    },

    getSelectionTopBorderChecker: function()
    {
        return {
            visitCellSelection: function() { return true; },
            visitDateSelection: function() { return false; },
            visitRunnerSelection: function() { return false; }
        };
    },

    getSelectionRightBorderChecker: function(cell)
    {
        return {
            visitCellSelection: function() { return true; },
            visitDateSelection: function() { return true; },
            visitRunnerSelection: function()
            {
                var runnerController = cell.getRow().getTable().getRunnerController();
                return runnerController.getLastLockedDate() == null ||
                       runnerController.getNewLastLockDate() > runnerController.getLastLockedDate();
            }
        };
    },

    getLeftBorderClassName: function(cell)
    {
        var selectionController = this.getSelectionController(cell);
        if (this.isSelected(cell))
        {
            if (selectionController.isLeftmostSelectedCell(cell))
            {
                if (selectionController.visitSelection(this.getSelectionLeftBorderChecker(cell)))
                    return "selectionLeftBorder";
            }
        }
        else
        {
            var leftColIdx = cell.getColIdx() - 1;
            if (leftColIdx > 0)
            {
                if (selectionController.isRightmostSelectedCell(cell.getRow().getCell(leftColIdx)) &&
                    selectionController.visitSelection(this.getSelectionRightBorderChecker(cell)))
                    return "selectionLeftBorder";
            }
        }

        if (cell.getColIdx() > 1)
        {
            if (this.isFirstCellOfWeek(cell))
                return this.getWeekSeparatorClass();
        }
        return this.getLeftBorderSpecificClassName(cell);
    },

    getLeftBorderSpecificClassName: function(cell)
    {
        return "";
    },

    getWeekSeparatorClass: function(cell)
    {
        return "weekSeparator";
    },

    isFirstCellOfWeek: function(cell)
    {
        var columnHeader = cell.getRow().getTable().getFirstRow().getCell(cell.getColIdx());
        return columnHeader.getValue().date.getDay() == cell.getRow().getTable().getWeekStartDay();
    },

    getCellClassName: function(cell)
    {
        //Note: should be implemented in inheritors
    },

    isHovered: function(cell)
    {
        return cell.getRow().getTable().getHoverController().isHovered(cell);
    },

    isLastCell: function(cell)
    {
        return cell.getRow().getCells().length - 1 == cell.getColIdx();
    },

    isWorkingCell: function(cell)
    {
        return cell.getRow().getTable().getDatesRow().getCell(cell.getColIdx()).getValue().working;
    }
});
