function RunnerCellRenderer()
{
    LockTableCellRenderer.apply(this, arguments);
}

RunnerCellRenderer.prototype = _.create(LockTableCellRenderer.prototype, {
    generateContent: function(cell)
    {
        cell.getCellEl().innerHTML = '<div class="runnerCellStateImg"><div class="runnerCellArrow"></div></div>';
    },

    getCellClassName: function(cell)
    {
        return [this.getBackgroundClass(cell),
                this.getStateImgClass(cell),
                this.getArrowClass(cell)].join(" ");
    },

    getBackgroundClass: function(cell)
    {
        if (this.isStateChanging(cell))
            return "runnerCellStateChanging";

        if (cell.getValue().runnerOver)
            return "runnerCellLocked";

        if (cell.getRow().isViewerTodayCell(cell))
            return "runnerCellToday";

        if (cell.getValue().working)
            return "runnerCellWorking";
        else
            return "runnerCellNonworking";
    },

    getStateImgClass: function(cell)
    {
        if (this.isStateChanging(cell))
        {
            if (cell.getValue().runnerOver)
                return "unlockingImg";
            else if (this.isNewLastLocked(cell))
                return "lastLockingImg";
            else
                return "lockingImg";
        }
        else if (this.isHighlighted(cell) || this.isOldLastLocked(cell))
        {
            return "lastLockedImg"
        }

        return "";
    },

    getArrowClass: function(cell)
    {
        if (this.isHighlighted(cell))
        {
            if (this.isNewLastLocked(cell) || this.isOldLastLocked(cell))
            {
                if (!this.isStateChanging(cell))
                    return "lockArrowHoverBg";
                else if (cell.getValue().working)
                    return "hoverArrowWorkingBg";
                else
                    return "hoverArrowDayoffBg";
            }
        }
        else if (this.isOldLastLocked(cell))
        {
            if (cell.getValue().working)
                return "lockArrowWorkingBg";
            else
                return "lockArrowDayoffBg";
        }

        return "";
    },

    isStateChanging: function(cell)
    {
        if (this.isHighlighted(cell))
        {
            var newLastLockDate = this.getRunnerController(cell).getNewLastLockDate();
            if (newLastLockDate != null)
            {
                var isCurrentlyUnlocked = newLastLockDate < cell.getValue().date;
                return isCurrentlyUnlocked != !cell.getValue().runnerOver;
            }
        }
    },

    getLastCellRightBorderClassName: function()
    {
        return "lockTableLightRightBorder";
    },

    getRunnerController: function(cell)
    {
        return cell.getRow().getTable().getRunnerController();
    },

    isHighlighted: function(cell)
    {
        return this.isHovered(cell) || this.getRunnerController(cell).isSelected(cell);
    },

    isNewLastLocked: function(cell)
    {
        return this.getRunnerController(cell).isNewLastLocked(cell);
    },

    isOldLastLocked: function(cell)
    {
        var lastLockedDate = this.getRunnerController(cell).getLastLockedDate();
        if (lastLockedDate != null)
            return lastLockedDate.getTime() == cell.getValue().date.getTime();
        else
            return false;
    }
});


