function UserRow(userId, expandedUsersStorage, today, workdayDurations)
{
    LockTableRow.call(this);
    this.childRows = [];
    this.userId = userId;
    this.expandedUsersStorage = expandedUsersStorage;
    this.expanded = expandedUsersStorage.contains(userId);
    this.expandCollapseDisabled = false;
    this.expandCollapseListeners = [];
    this.today = today;
    this.workdayDurations = workdayDurations;
}

UserRow.prototype = _.create(LockTableRow.prototype, {
    createRowHeaderRenderer: function()
    {
        return new UserNameCellRenderer();
    },

    createLockCellRenderer: function()
    {
        return new UserDayCellRenderer();
    },

    disableExpandCollapse: function(disable)
    {
        this.expandCollapseDisabled = disable;
    },

    addExpandCollapseListener: function(listener)
    {
        this.expandCollapseListeners.push(listener);
    },

    isExpanded: function()
    {
        return this.expanded;
    },

    addChildRow: function(row)
    {
        this.childRows.push(row);
        row.setParentRow(this);
        row.setTable(this.getTable());
        row.showHide(this.expanded);
        if (row instanceof OvertimeRow && !this.overtimeRow)
            this.overtimeRow = row;
    },

    getChildRows: function()
    {
        return this.childRows;
    },

    getLastChildRow: function()
    {
        if (this.childRows.length > 0)
            return this.childRows[this.childRows.length - 1];
        else
            return null;
    },

    expand: function()
    {
        if (!this.expanded)
            this.changeExpandCollapse();
    },

    collapse: function()
    {
        if (this.expanded)
            this.changeExpandCollapse();
    },

    changeExpandCollapse: function()
    {
        if (this.childRows.length == 0 || this.expandCollapseDisabled)
            return;

        this.expanded = !this.expanded;

        if (this.expanded)
            this.expandedUsersStorage.add(this.userId);
        else
            this.expandedUsersStorage.remove(this.userId);

        this.updateUI();
        for (var i = 0; i < this.expandCollapseListeners.length; i++)
        {
            this.expandCollapseListeners[i](this);
        }
    },

    accept: function(visitor, cell)
    {
        return visitor.visitUserRow(cell);
    },

    updateUI: function()
    {
        for (var i = 0; i < this.childRows.length; i++)
        {
            this.childRows[i].showHide(this.expanded);
        }

        for (i = 0; i < this.cells.length; i++)
        {
            this.updateCellUI(i);
        }
    },

    createUserLeavesRow: function(headerText)
    {
        this.userLeavesRow = new UserLeavesRow();
        this.userLeavesRow.addCell(headerText);
        this.addChildRow(this.userLeavesRow);
        this.getTable().addRow(this.userLeavesRow);
        return this.userLeavesRow;
    },

    getUserLeavesRow: function()
    {
        return this.userLeavesRow;
    },

    getUserLeaveInfo: function(cellIdx)
    {
        if (this.userLeavesRow)
            return this.userLeavesRow.getCell(cellIdx).getValue();

        return {time: 0, leaveInfo: undefined};
    },

    getUserOvertimeInfo: function(cellIdx)
    {
        if (this.overtimeRow)
            return this.overtimeRow.getCell(cellIdx).getValue();
        else
            return 0;
    },

    isTodayCell: function(cell)
    {
        return this.getTable().getDatesRow().getCell(cell.getColIdx()).getValue().date.getTime() == this.today.getTime();
    },

    isHolidayCell: function(cell)
    {
        return this.workdayDurations[cell.getColIdx() - 1] == 0; // -1 because of one header column
    }
});
