MessageResource = function() {};

MessageResource.prototype.applyMacrosSubstitutions = function(message, escapeMacros)
{
    // quick check
    if( message.indexOf("#") < 0 )
        return message;

    for( var i = 0; i < MESSAGE_SUBSTITUTIONS.length; i++ )
        message = message.replace(
                    MESSAGE_SUBSTITUTIONS[i].regexp,
                    escapeMacros ? escapeHtmlSymbols( MESSAGE_SUBSTITUTIONS[i].value ) : MESSAGE_SUBSTITUTIONS[i].value
        );
    return message;
};

MessageResource.prototype.applyParamsSubstitutions = function(message, params)
{
    if( params && params.length > 0 )
        for( var j = 0; j < params.length; j ++ )
            message = message.replace( new RegExp( "{" + j + "[,a-zA-Z]*}", "g" ), params[ j ] );
    return message;
};

MessageResource.prototype.applyAllSubstitutions = function(message, params, escapeMacros)
{
    message = this.applyMacrosSubstitutions( message, escapeMacros );
    message = this.applyParamsSubstitutions( message, params );
    return message;
};

MessageResource.prototype.getMessage = function(key, params, escapeMacros)
{
    //noinspection JSUnresolvedVariable
    var message = MESSAGE_BUNDLE[key];
    if( typeof message == typeof undefined )
    {
        if( typeof console != typeof undefined && console.trace )
            console.trace( "could not find message with key: " + key );
        return undefined;
    }
    return this.applyAllSubstitutions( message, params, escapeMacros );
};

MessageResource.prototype.getEscapedMessage = function ( key, params )
{
    return escapeHtmlSymbols( this.getMessage( key, params ) );
};

MessageResource.prototype.getMessageWithEscapedMacros = function(key, params)
{
    return this.getMessage( key, params, true );
};

MessageResource.prototype.getEscapedMessageNoEscapeArgs = function (key, params)
{
    //noinspection JSUnresolvedVariable
    var message = MESSAGE_BUNDLE[key];
    if( typeof message == typeof undefined )
    {
        if( typeof console != typeof undefined && console.trace )
            console.trace( "could not find message with key: " + key );
        return undefined;
    }
    message = this.applyMacrosSubstitutions( message );
    message = escapeHtmlSymbols( message );
    message = this.applyParamsSubstitutions( message, params );
    return message;
};

MessageResource.prototype.printMessage = function (key, params, escapeMacros)
{
    document.write( this.getMessage( key, params, escapeMacros ) );
};


messageResource = new MessageResource();

MessageResource.formatMessage = function( message, params )
{
    return messageResource.applyParamsSubstitutions( message, params );
};
