
ScheduleDhtmlPanel.prototype = new DhtmlPopup;

/**
 * ScheduleDhtmlPanel constructor
 * @param divId - the ID of DIV element which represents the panel
 * @param scheduleInputName - name of html form input which specifies selected days
 * @param messageBoxId - element's id where display messages to
 * @param saveButtonId - element's id where Save button is placed
 * @param cancelButtonId - element's id where Save button is placed
 * @param formName - name of form which panle operates with. [Optional]
 *                   Default value - divId + "-form";
 * @param scheduleInputIdPostfix - to reflect changes over schedule in panel, realization expects html input
 *                                 elements' IDs and corresponding markup element's IDs to conform the following
 *                                 scheduleInputId = scheduleMarkupId + scheduleInputIdPostfix. [Optional]
 *                                 Default - "-box"
 *
 */
function ScheduleDhtmlPanel(divId,
                            scheduleInputName,
                            messageBoxId ,
                            saveButtonId,
                            cancelButtonId,
                            savingMessageId,
                            formName,
                            scheduleInputIdPostfix)
{
    DhtmlPopup.call(this, divId);
    // Specify events to be dereferenced
    this.addEventDereferencer("mousedown");

    this.scheduleInputName = scheduleInputName;
    this.messageBoxId = messageBoxId;
    this.saveButtonId = saveButtonId;
    this.cancelButtonId = cancelButtonId;
    this.savingMessageId = savingMessageId;
    this.formName = formName ? formName : this.divId + "-form";
    this.scheduleInputIdPostfix = scheduleInputIdPostfix ? scheduleInputIdPostfix : "-box";

    this.panelController = null;
    this.dataSavingCallBack = null;
    this.beforeShowHandler = ScheduleDhtmlPanel.prototype.beforeShowHandler;
    this.beforeHideHandler = ScheduleDhtmlPanel.prototype.beforeHideHandler;
    this.doSave = false;

    this.hiddenElementCls = "hidden";
}

/**
 * Sets panel's controller
 *
 * @param controller
 */
ScheduleDhtmlPanel.prototype.setPanelController = function( controller )
{
    this.panelController = controller;
}

/**
 * Returns panel's controller
 *
 */
ScheduleDhtmlPanel.prototype.getPanelController = function()
{
    return this.panelController;
}


/**
 * Sets a function to be called after data saving procedure being completed.
 *
 * @param callback a function to be called
 */
ScheduleDhtmlPanel.prototype.setDataSavingCallBack = function( callback )
{
    this.dataSavingCallBack = callback;
}

/**
 * Sets panel's form name
 *
 * @param formName
 */
ScheduleDhtmlPanel.prototype.setFormName = function( formName )
{
    this.formName = formName;
}

/**
 * Return array of checkbox elements of panel's form
 *
 */
ScheduleDhtmlPanel.prototype.getScheduleInputs = function()
{
    return document.forms[this.formName].elements[this.scheduleInputName]
}

ScheduleDhtmlPanel.prototype.saveAndHide = function()
{
    this.doSave = true;
    this.hide();
}

/**
 * Saves changes changes have been made
 */
ScheduleDhtmlPanel.prototype.save = function()
{
    var checkboxes = this.getScheduleInputs();
    var scheduledDays = new Array();
    for( var i = 0; i < checkboxes.length; i++ )
    {
        if( checkboxes[i].checked )
        {
            scheduledDays.push( {"relativeDay": i, "absoluteDay": parseInt(checkboxes[i].value) } );
        }
    }
    this.panelController.saveSchedule(scheduledDays);
}

/**
 * Cancels changes have been made and closes panel
 */
ScheduleDhtmlPanel.prototype.cancelChanges = function()
{
    this.doSave = false;
    this.hide();
}

/**
 * Disables/Enables panels's form. As a result it disables/enables form's inputs and hides/shows buttons
 * @param flag specifies value of <code>disabled</code> attribute of form's elements. If true form will be disabled, if false - enabled
 * @param savingMessage message to be shown in replace for form's buttons
 */

ScheduleDhtmlPanel.prototype.setFormDisable = function( flag, savingMessage )
{
    var checkboxes = this.getScheduleInputs();
    for( var i = 0; i < checkboxes.length; i++ )
    {
        checkboxes[i].disabled = flag;
    }
    this.displayButtonsSection( savingMessage )
}

/**
 * Hides/Shows forms' buttons
 * @param message - message to be shown in replace for form's buttons. If message is null or not specified buttons will
 *                  be shown
 */
ScheduleDhtmlPanel.prototype.displayButtonsSection = function( message )
{
    var saveButton = getElementById(this.saveButtonId);
    var cancelButton = getElementById(this.cancelButtonId);
    var messageElement = getElementById(this.savingMessageId);
    if(message)
    {
        addClass (saveButton, this.hiddenElementCls);
        addClass (cancelButton, this.hiddenElementCls);
        messageElement.innerHTML = message;
        removeClass (messageElement, this.hiddenElementCls);
    }
    else
    {
        removeClass (saveButton, this.hiddenElementCls);
        removeClass (cancelButton, this.hiddenElementCls);
        addClass (messageElement, this.hiddenElementCls);
    }
}

/*
    Displays message withing message box
 */
ScheduleDhtmlPanel.prototype.displayMessage = function( message )
{
    var messageBox = getElementById( this.messageBoxId );
    messageBox.style.display = "block";
    messageBox.innerHTML = message;
}

ScheduleDhtmlPanel.prototype.onBeginSaving = function( savingMessage )
{
    this.setFormDisable(true, savingMessage);
}

ScheduleDhtmlPanel.prototype.onEndSaving = function()
{
    this.setFormDisable(false);
}

ScheduleDhtmlPanel.prototype.onDataSavingComplete = function()
{
    this.doSave = false;
    this.hide();
    if(this.dataSavingCallBack)
    {
        this.dataSavingCallBack();
        this.dataSavingCallBack = null;
    }
};

ScheduleDhtmlPanel.prototype.handelError = function (message)
{
    this.setFormDisable(false);
    this.displayMessage( message );
}

ScheduleDhtmlPanel.prototype.setUpForm = function()
{
    var scheduledDays = this.panelController.getScheduledDays();
    var checkboxes = this.getScheduleInputs();
    for( var day=0; day < scheduledDays.length; day++ )
    {
        if( scheduledDays[day][0] )
        {
            checkboxes[day].checked = true;
            var dayElement = this.getMarkupElementByInput(checkboxes[day]);
            addClass(dayElement, "scheduled");
        }
    }
}

ScheduleDhtmlPanel.prototype.cleanForm = function()
{
    var checkboxes = this.getScheduleInputs();
    for( var i = 0; i < checkboxes.length; i++ )
    {
        checkboxes[i].checked = false;
        var dayElement = this.getMarkupElementByInput(checkboxes[i]);
        removeClass(dayElement, "scheduled");
    }
}

/**
 * Process schedule changes action
 * @param element form input element on which click action has occurred
 */
ScheduleDhtmlPanel.prototype.scheduledDayChanges = function ( element )
{
    this.doSave = true;
    var dayElement = this.getMarkupElementByInput(element);
    var isChecked = element.checked;
    if ( isChecked )
        addClass(dayElement, "scheduled");
    else
        removeClass(dayElement, "scheduled");
}

/**
 * Registers event handlers
 */
ScheduleDhtmlPanel.prototype.registerEventHandlers = function()
{
    var thisPopup = this;
    KeyDownHandler.setOnKeyDownHandler(function(event)
    {
        // Popup closes without saving entered data when 'Esc' key is pressed
        handleEscape(function(){thisPopup.cancelChanges()}, event);
        // Data has been entered is saved and popup closes when 'Enter' key is pressed
        handleEnter(function(){thisPopup.hide()}, event);
        // Switching between elements by pressing 'tab' key is blocked
        handleTab(function(){window.event.cancelBubble = true;}, event);
    });

    // Register "onmousedown" handler
    this.oldDocumentMousedown = document.onmousedown;
    document.onmousedown = function ()
    {
        var target = event.target || event.srcElement;
        // if popup's owner is not defined or the user has clicked outside the owner then
        // save changes have been made and close the popup
        if (thisPopup.getOwner() == null || thisPopup.getOwner() != target)
        {
            thisPopup.hide();
        }
    };

    // Set "onmousedown" handler to popup window. The purpose of the handler is to stop propagation of
    // the event thus preventing popup being closed by dosument's "onmousedown" handler
    this.getPopupDiv().onmousedown = function(event)
    {
        window.event.cancelBubble = true;
    };

    // Register window's "onresize" handler. If window resize occurred then
    // save changes have been made and close the popup
    this.oldWindowResize = window.onresize;
    window.onresize = function()
    {
        thisPopup.hide();
    }
}

ScheduleDhtmlPanel.prototype.restoreEventHandlers = function()
{
    KeyDownHandler.restoreOnKeyDownHandler();               // restoring keydown
    document.onmousedown = this.oldDocumentMousedown;   // restoring mousedown
    window.onresize = this.oldWindowResize;             // restoring onresize
}

ScheduleDhtmlPanel.prototype.getMarkupElementByInput = function ( checkbox )
{
    var dayId = checkbox.id.substr(0, checkbox.id.indexOf(this.scheduleInputIdPostfix));
    return getElementById( dayId );
}

ScheduleDhtmlPanel.prototype.beforeShowHandler = function()
{
    this.cleanForm();
    this.setUpForm();
    getElementById( this.messageBoxId ).style.display = "none";
}

/**
 * returns false if data saving is required
 */
ScheduleDhtmlPanel.prototype.beforeHideHandler = function()
{
    if ( !this.doSave )
    {
        return true;
    }
    this.save();
    return false;
}

ScheduleDhtmlPanel.prototype.toggleDay = function(checkBoxId)
{
    var inputElem =  getElementById( checkBoxId );
    inputElem.checked = !inputElem.checked;
    this.scheduledDayChanges( inputElem );
};