
/**
 * SchedulePanelController constructor
 * @param type - notification type
 * @param scheduledDays - days which for this notification was scheduled. An array where first element in a row
 *                        contains day's offset relatively week start day, second one contains corresponding day's
 *                        alphabetical representation.
 * @param schedulePanel - schedulePanel object is used to manipulate with the schedule
 *
 */
SchedulePanelController = function( type, scheduledDays, schedulePanel )
{
    this.type = type;
    this.scheduledDays = scheduledDays;
    this.schedulePanel = schedulePanel;
    this.newScheduledDays = null;
    this.isSaving = false;
    this.rpcDelay = 300;
}

/**
 * Saves new schedule
 * @param newScheduledDays - array contains new schedule for current notification. Every element in the array
 *                           is an associative array with two elements "absoluteDay" and "relativeDay",
 *                           day's order numbers withing regular (Sunday started) and current week correspondingly.
 */
SchedulePanelController.prototype.saveSchedule = function( newScheduledDays )
{
    this.newScheduledDays = newScheduledDays;
    RpcControlManager.saveRpcControlBoundData( new ScheduleSavingManager(this) );
}

/**
 * Returns scheduledDays
 */
SchedulePanelController.prototype.getScheduledDays = function()
{
    return this.scheduledDays;
}

/**
 * Returns string contains alphabetical representation of scheduled days. Only scheduled days are included.
 */
SchedulePanelController.prototype.getScheduledDaysString = function()
{
    var stringScheduledDays = this.scheduledDays.toString();
    //If every day has been selected
    if ( stringScheduledDays.indexOf("false") == -1 )
        return messageResource.getMessage("mail_notifications.schedule_button.title.daily");
    //If no day has been selected
    if ( stringScheduledDays.indexOf("true") == -1 )
        return messageResource.getMessage("mail_notifications.schedule_button.title.no-days");

    var buttonText = new String();
    for( var day=0; day < this.scheduledDays.length; day++ )
    {
        if( this.scheduledDays[day][0] )
            buttonText += (this.scheduledDays[day][1] + ", ");
    }

    //Substring is returned in order to remove ", " at the end of the string
    return buttonText.substr( 0, buttonText.length - 2 );
}

/**
 * Returns type
 */
SchedulePanelController.prototype.getNotificationType = function()
{
    return this.type;
}

SchedulePanelController.prototype.getScheduleList = function()
{
    var daysToSchedule = new Array();
    for( var day = 0; day < this.newScheduledDays.length; day++ )
    {
        daysToSchedule.push(this.newScheduledDays[day]["absoluteDay"]);
    }
    return {"javaClass": "java.util.ArrayList",
            "list": daysToSchedule };

}

SchedulePanelController.prototype.onBeginSaving = function( savingMessage )
{
    this.schedulePanel.onBeginSaving( savingMessage );
}

SchedulePanelController.prototype.onEndSaving = function()
{
    this.schedulePanel.onEndSaving();
}

SchedulePanelController.prototype.onDataSavingComplete = function()
{
    var day;
    for( day = 0; day < this.scheduledDays.length; day++ )
        this.scheduledDays[day][0] = false;

    for( day = 0; day < this.newScheduledDays.length; day++ )
    {
        this.scheduledDays[ this.newScheduledDays[day]["relativeDay"] ][0] = true;
    }
    this.newScheduledDays = null;
    this.schedulePanel.onDataSavingComplete();
};

SchedulePanelController.prototype.handelError = function (message)
{
    this.schedulePanel.handelError( message );
}


