function PagerController()
{
    this.constructor = PagerController;
}

/**
 * @param conf must has fields:
 *      .pagerData.pageNum
 *      .pagerData.recordsPerPage
 *      .pagerData.totalRecords
 *      .handlersMap.pageChanged (function(pageNum){})
 *      .handlersMap.recordsPerPageChanged (function(recordsPerPage){})
 *      .pagerControlsElementId
 *      .pagerElementId
 *      .recordsPerPageSelectorId
 *      .minRecordsPerPage
 *      .ignoreHashChange
 */
PagerController.prototype.init = function ( conf )
{
    this.pageNum = conf.pagerData.pageNum;
    this.recordsPerPage = conf.pagerData.recordsPerPage;
    // validate records per page
    if(this.recordsPerPage < 1)
    {
        console.warn("Incorrect records per page value: " + this.recordsPerPage);
    }
    this.totalRecords = conf.pagerData.totalRecords;
    this.pagerData = conf.pagerData;
    this.pagerControlsElement = document.getElementById(conf.pagerControlsElementId);
    this.minRecordsPerPage = conf.minRecordsPerPage;

    var pagerContentEl = document.getElementById( conf.pagerElementId );
    this.pager = new Pager( pagerContentEl, "&nbsp;", "&nbsp;", "...", conf.handlersMap.pageChanged, conf.locationHashString, conf.ignoreHashChange );

    var self = this;
    this.recordsPerPageSelector = document.getElementById( conf.recordsPerPageSelectorId );
    this.recordsPerPageSelector.value = this.recordsPerPage;
    this.recordsPerPageSelector.onchange = function ()
    {
        conf.handlersMap.recordsPerPageChanged( self.recordsPerPageSelector.value );
    };

    return this;
};

PagerController.prototype.renderThisController = function ()
{
    this.render();
};


PagerController.prototype.getPageNum = function ()
{
    return this.pageNum;
};

PagerController.prototype.getTotalPages = function ()
{
    return Pager.getTotalPages( this.totalRecords, this.recordsPerPage );
};

PagerController.prototype.getTotalRecords = function ()
{
    return this.totalRecords;
};

PagerController.prototype.getRecordsPerPage = function ()
{
    return this.recordsPerPage;
};

PagerController.prototype.render = function ()
{
    if( this.totalRecords > this.minRecordsPerPage )
    {
        this.setRecordsPerPage( this.recordsPerPage );
        this.pagerControlsElement.style.display="table";

        if( this.getTotalPages() > 1 )
            this.pager.draw( this.totalRecords, this.recordsPerPage, this.pageNum );
        else
            this.pager.remove();
    }else{
        this.pagerControlsElement.style.display="none";
    }
};

PagerController.prototype.setRecordsPerPage = function( value )
{
    this.recordsPerPageSelector.value = value;
};

PagerController.prototype.setPagerStateAndUpdateUI = function ( totalRecords, pageNum, recordsPerPage )
{
    this.totalRecords = totalRecords;
    this.pageNum = pageNum;
    this.recordsPerPage = recordsPerPage;
    this.render();
};

PagerController.prototype.loadPage = function ( pageNum )
{
    this.pager.loadPage( pageNum );
};

PagerController.prototype.reloadEmptyPage = function ()
{
    if (this.getTotalPages() == this.getPageNum() && this.getTotalPages() > 1) {
        this.loadPage(this.getPageNum() - 1);
    } else {
        this.loadPage(this.getPageNum());
    }
};
