SlidingPanel.DescriptionInput = function($containerEl, editable, placeholder)
{
    TemplateView.call(this, $containerEl);
    this.editable = editable;
    this.placeholder = placeholder;
};

SlidingPanel.DescriptionInput.prototype = _.create(TemplateView.prototype, {
    html: "<div class='descriptionInfo'>" +
          "  <div class='descriptionInput'><textarea class='textarea' maxlength='2000'/></div>" +
          "  <div class='characterCounter'></div>" +
          "</div>",

    updateOnInit: false,

    afterRender: function()
    {
        var self = this;
        this.$el.toggleClass('editable', this.editable);

        var $input = this.$input = this.find('.textarea');
        this.$input.attr('placeholder', this.placeholder).placeholder();
        this.$input.blur(_.bind(this._onBlur, this));

        fixIScrollInnerWheelEvent(this.$input);

        this.$input.keyup(function()
                          {
                              self.counterUpdate();
                          });
        this.$input.keydown(function(e) {
            handleEscape(function(){stopEventPropagation(e); $input.blur();}, e);
        });

        this.$descriptionInput = this.find('.descriptionInput');

        this.$descriptionInput.click(_.bind(this._onFocus, this));

        this.$counter = this.find('.characterCounter');
        this.$counter.hide();
    },

    update: function()
    {
        this.$input.pVal(this.description);
        if (!this.editable)
        {
            this.$input.attr('disabled', 'disabled');
            this.$input.removeAttr('placeholder')
        }
        else
        {
            this.$input.removeAttr('disabled');
            this.$input.attr('placeholder', this.placeholder);
        }

        this.counterUpdate();
    },

    setDescription: function(description)
    {
        this.description = description;
        this.update();
    },

    counterUpdate: function()
    {
        var characterCount = this.$input.pVal().length;
        var counterText = characterCount + "/2000";
        this.$counter.text(counterText);

        if (characterCount <= 2000)
            this.clearError();
        else
            this.setError();
    },

    setError: function()
    {
        this.$el.addClass("invalid");
        this.invalid = true;
    },

    clearError: function()
    {
        this.$el.removeClass("invalid");
        this.invalid = false;
    },

    _onFocus: function()
    {
        if (!this.editable)
            return;
        this.$counter.show();
    },

    _onBlur: function()
    {
        var newDescription = this.$input.pVal().trim();
        if (newDescription !== this.description && !this.invalid)
        {
            this.trigger('change', newDescription);
        }
        this.setDescription(newDescription);

        if (!this.invalid)
        {
            this.$counter.hide();
        }
    },

    setEditable: function(editable)
    {
        this.editable = editable;
    }
});

_.extend(SlidingPanel.DescriptionInput.prototype, LoadingControlMixin);