function MoveToDialog($containerEl, mode, titleMessageKey)
{
    TemplateView.call(this, $containerEl);
    this.mode = mode;
    this.titleMessageKey = titleMessageKey;
}

MoveToDialog.itemHtmlCallback = function(item)
{
    var text = item.name;
    if (item.archived)
        text += " " + messageResource.getMessage("common.archived");
    return text;
};

MoveToDialog.prototype = _.create(TemplateView.prototype, {
    html: "<div class='moveToDialog'>" +
          "  <div class='title'><%-titleMessageKey%></div>" +
          "  <div class='customer'>" +
          "    <div class='label ellipsis'><%-customerLabel%>:</div>" +
          "    <div class='selectorContainer'></div>" +
          "  </div>" +
          "  <div class='points'/>" +
          "  <div class='project'>" +
          "    <div class='label ellipsis'><%-projectLabel%>:</div>" +
          "    <div class='selectorContainer'></div>" +
          "  </div>" +
          "  <div class='buttonsContainer'>" +
          "    <div class='move button'><%=moveLabel%></div>" +
          "    <div class='cancel button'><%=cancelLabel%></div>" +
          "  </div> " +
          "</div>",

    afterRender: function()
    {
        var self = this;

        this.customerMenuButton = new SimpleListMenuButton(
                this.find(".customer .selectorContainer"),
                {
                    emptySelectionMessageKey: "task.list.move_to.select_customer",
                    emptyListMessageKey: "task.list.move_to.no_customers",
                    itemHtmlCallback: MoveToDialog.itemHtmlCallback
                }).init();
        this.customerMenuButton.on("select", function(e, customer)
        {
            self.selectedCustomer = customer;

            if (customer && self.projectMenuButton)
            {
                var projects = ProjectStore.getByCustomer(customer.id);
                if (!self.showArchived)
                    projects = _.filter(projects, function(p) { return !p.archived });

                self.selectedProject = projects.length === 1 ? projects[0] : null;
                self.projectMenuButton.setItems(projects, self.selectedProject);
            }

            self._updateButtons();
        });

        if (this.mode === MoveToDialog.Mode.CUSTOMER_PROJECT)
        {
            this.projectMenuButton = new SimpleListMenuButton(
                    this.find(".project .selectorContainer"),
                    {
                        emptySelectionMessageKey: "task.list.move_to.select_project",
                        emptyListMessageKey: "task.list.move_to.no_projects",
                        itemHtmlCallback: MoveToDialog.itemHtmlCallback
                    }).init();
            this.projectMenuButton.on("select", function(e, project)
            {
                self.selectedProject = project;
                self._updateButtons();
            });
            this.$el.addClass("hasProject");
        }
        else
        {
            this.find(".points").hide();
        }

        this.find(".move.button").click(function()
                                        {
                                            if (self.moveEnabled)
                                            {
                                                $(self).trigger("move", {
                                                    customer: self.selectedCustomer,
                                                    project: self.selectedProject
                                                })
                                            }
                                        });

        this.find(".cancel.button").click(function()
                                          {
                                              $(self).trigger("cancel")
                                          });

        this._updateButtons();
    },

    show: function()
    {
        TemplateView.prototype.show.call(this);
        this.updateSelector();
    },

    hide: function()
    {
        this.customerMenuButton.getMenu().hide();
        if (this.mode === MoveToDialog.Mode.CUSTOMER_PROJECT)
        {
            this.projectMenuButton.getMenu().hide();
        }
        this.$el.hide();
    },

    updateSelector: function()
    {
        this.selectedCustomer = null;
        this.selectedProject = null;

        var customers = CustomerStore.getAll();
        if (!this.showArchived)
            customers = _.filter(customers, function(c) { return !c.archived });
        this.customerMenuButton.setItems(customers);

        if (this.mode === MoveToDialog.Mode.CUSTOMER_PROJECT)
            this.projectMenuButton.setItems([]);

        this._updateButtons();
    },

    messageKeys: function()
    {
        return {
            titleMessageKey: this.titleMessageKey,
            customerLabel: "common.Customer",
            projectLabel: "common.Project",
            moveLabel: "edit_panel.action.move",
            cancelLabel: "common.cancel"
        }
    },

    setShowArchived: function(showArchived)
    {
        this.showArchived = showArchived;
        if (this.projectMenuButton)
            this.projectMenuButton.emptyListMessageKey = showArchived ? "task.list.move_to.no_projects" : "task.list.move_to.no_active_projects";
    },

    _updateButtons: function()
    {
        this.moveEnabled = !!this.selectedCustomer && (this.mode === MoveToDialog.Mode.CUSTOMER || !!this.selectedProject);
        this.$el.toggleClass("moveEnabled", this.moveEnabled);
    }
});

_.extend(MoveToDialog.prototype, LoadingControlMixin);

MoveToDialog.Mode = {
    CUSTOMER_PROJECT: "CUSTOMER_PROJECT",
    CUSTOMER: "CUSTOMER"
};