SlidingPanel.NameInput = function($containerEl, editable, placeholder, maxLength)
{
    TemplateView.call(this, $containerEl);
    this.editable = editable;
    this.placeholder = placeholder;
    this.maxLength = maxLength;

    this.invalid = false;
    this.typing = false;
};

SlidingPanel.NameInput.prototype = _.create(TemplateView.prototype, {

    html: "<div class='nameInfo'>" +
          " <div class='nameLabel'/>" +
          " <div class='nameInput'><input type='text' <% if (maxLength) {%>maxlength='<%-maxLength%>'<%}%>/></div>" +
          "</div>",

    updateOnInit: false,

    templateModel: function()
    {
        return {
            "maxLength": this.maxLength
        }
    },

    afterRender: function()
    {
        this.$el.toggleClass('editable', this.editable);
        this.$nameLabel = this.find('.nameLabel');
        var $input = this.$input = this.find('input');
        this.$input.attr('placeholder', this.placeholder);
        this.$nameInput = this.find('.nameInput');
        this.$nameLabel.click(_.bind(this._onFocus, this));
        this.$nameInput.click(_.bind(this._onFocus, this));
        this.$input.blur(_.bind(this._onBlur, this));
        this.$input.keydown(function(e) {
            handleEnter(function(){$input.blur();}, e);
            handleEscape(function(){stopEventPropagation(e);$input.blur();}, e);
        });
        this.errorTooltip = SlidingPanel.NameInput.ErrorTooltip.getInstance();
        this.errorTooltip.subscribeToMouseEvents(this.$nameInput);
    },

    update: function()
    {
        if (this.editable && !this.invalid)
            this._removeError();

        this.$nameInput.toggle(this.typing);
        this.$nameLabel.toggle(!this.typing);

        if (!this.invalid)
        {
            this.$nameLabel.text(this.name);
            this.$input.pVal(this.name);
        }

        this.$el.toggleClass('editable', this.editable);
    },

    getName: function()
    {
        return this.name;
    },

    /** Set name to input and remove all errors, remove focus from input **/
    resetName: function(name)
    {
        this.name = name;
        this.invalid = false;
        this.typing = false;
        this.update();
    },

    error: function(errMsg)
    {
        if (!this.editable)
            return;
        this.invalid = true;
        this.typing = true;

        this.$el.addClass('invalid');
        this.errorTooltip.$el.text(errMsg);
        this.errorTooltip.enable();

        this.update();
    },

    _onFocus: function()
    {
        if (!this.editable)
            return;

        if (!this.typing)
        {
            this.typing = true;
            this.update();
            this.$input.focus();
        }
    },

    _onBlur: function()
    {
        var newName = this.$input.pVal().trim();
        if (!newName)
        {
            this.error(messageResource.getMessage('panel.common.error_tooltip.empty_name'));
            return;
        }
        // invalid is always trigger change
        if (newName !== this.name)
            this.trigger('change', newName);
        else
        {
            this.invalid = false;
            this.typing = false;
            this.update();
        }
    },

    _removeError: function()
    {
        this.$el.removeClass('invalid');
        this.errorTooltip.disable();
    },

    setEditable: function(editable)
    {
        this.editable = editable;
    }
});

SlidingPanel.NameInput.ErrorTooltip = function($containerEl) {
    Tooltip.call(this, $containerEl);
};

SlidingPanel.NameInput.ErrorTooltip.prototype = _.create(Tooltip.prototype, {
    html: "<div class='nameInputTooltip yellowTooltip tooltipTex noprint'/>"
});

SlidingPanel.NameInput.ErrorTooltip.getInstance = function()
{
    if (!this._instance)
        this._instance = new this($("body")).init();
    return this._instance;
};

_.extend(SlidingPanel.NameInput.prototype, LoadingControlMixin);