function SlidingPanel($containerEl, $pageEl)
{
    WindowSizeDependedView.call(this, $containerEl);
    this.$pageEl = $pageEl;
}

SlidingPanel.prototype = _.create(WindowSizeDependedView.prototype, {

    html: "<div class='<%=typeClass%> sliding_panel'>" +
          " <div class='headerBlock'>" +
          "     <div class='closeButton'/>" +
          " </div>" +
          " <div class='contentScrollableWrapper'><div class='content'/></div>" +
          "</div>",

    updateOnInit: false,

    type: null,

    templateKey: "SlidingPanel",

    hide: function()
    {
        if (!this.isShown)
            return;
        this.isShown = false;

        this.onHide();

        var hideAnimation = this.hideAnimation();
        if (hideAnimation)
            this.$el.atAnimation().hideAnimated(hideAnimation);
        else
            this.$el.hide();

        this.$pageEl.off("click." + this.type);
        KeyDownHandler.restoreOnKeyDownHandler();

        $(this).trigger("hidePanel");
    },

    show: function()
    {
        if (this.isShown)
            return;
        this.isShown = true;

        var self = this;
        onOutsideClick(this.$el,
                       this.type,
                       function() { self.hide() },
                       function(e) { return e.originalEvent._at_panel_preventHidding || e.originalEvent['_at_'+ self.type +'_preventHidding'] },
                       this.$pageEl);

        KeyDownHandler.setOnKeyDownHandler(function(e) {
            handleEscape(function(e){
                self.hide();
            }, e);
        });

        var showAnimation = this.showAnimation();
        if (showAnimation)
            this.$el.atAnimation().showAnimated(showAnimation);
        else
            this.$el.show();

        this.adjustSize();

        $(this).trigger("showPanel");
    },

    afterRender: function()
    {
        this.isShown = false;
        WindowSizeDependedView.prototype.afterRender.call(this);
        var $contentWrapper = this.find('.contentScrollableWrapper');
        this.verticalScroll = ScrollUtils.addVerticalScroll($contentWrapper, {probeType: 2});
        if (!isIE8())
        {
            this.verticalScroll.on("scroll", _.bind(this._checkIfScrolledToEnd, this));
            this.suppressScrollCheck = true;
        }
        this.find(".closeButton").click(_.bind(this.hide, this));

        window['panel_' + this.type] = this;

        this._getLoadingControlMoire().showAnimated = false;
    },

    templateModel: function()
    {
        return {typeClass: this.type};
    },

    adjustSize: function()
    {
        this.$el.outerHeight(this.$containerEl.height());
        var $contentWrapper = this.find('.contentScrollableWrapper');
        var $header = this.find('.headerBlock');
        $contentWrapper.outerHeight(this.$el.innerHeight() - $header.outerHeight());
        this.refreshScroll();
    },

    refreshScroll: function()
    {
        ScrollUtils.refreshCustomScroll(this.verticalScroll);
        if (this.verticalScroll)
        {
            this._checkIfScrolledToEnd();
        }
    },

    scrollToTop: function()
    {
        if (this.verticalScroll)
        {
            this.verticalScroll.scrollTo(0, 0);
        }
    },

    _checkIfScrolledToEnd: function()
    {
        if (!isIE8() && !this.suppressScrollCheck && this.isShown && !this.isLoading()  && this.verticalScroll.y - this.verticalScroll.maxScrollY < 100)
        {
            this.trigger("scrollToEnd");
            this.suppressScrollCheck = true;
        }
    },

    hideAnimation: function()
    {
        return null
    },

    showAnimation: function()
    {
        return null
    },

    onHide: function()
    {


    }
});

_.extend(SlidingPanel.prototype, LoadingControlMixin);