function EditCustomerPanel($containerEl, $pageEl)
{
    SlidingPanel.call(this, $containerEl, $pageEl);
}

EditCustomerPanel.prototype = _.create(SlidingPanel.prototype, {

    type: "edit_customer_sliding_panel",

    afterRender: function()
    {
        SlidingPanel.prototype.afterRender.call(this);

        this.header = new EditCustomerPanel.Header(this.find(".headerBlock"), this.$el).init();
        this.content = new EditCustomerPanel.Content(this.find(".content")).init();
        this.$deleteDialogContainer = $("<div class='deleteDialogContainer'></div>").appendTo(this.find(".content"));
    },

    update: function(config)
    {
        if (!config)
            return;

        var customerInfo = config.customerInfo;

        this.header.update(customerInfo);

        this.content.update(customerInfo.active, config.totalSpentTime, config.activeSpentTime, config.archivedSpentTime, customerInfo.description);

        this._initDeleteDialog(customerInfo);
        this.hideDeleteDialog();
    },

    hideAnimation: function()
    {
        return "anim-hide-slide-right"
    },

    showAnimation: function()
    {
        return "anim-show-slide-right"
    },

    _initDeleteDialog: function(config)
    {
        var self = this;
        delete this.deleteDialog;
        this.$deleteDialogContainer.empty();

        var isBlocked = config.blockedDueToUnavailableToModifyTimeTrack || config.blockedDueToLockedTime;

        this.deleteDialog = new CPTManagement.DeleteCustomerPopupConfirmationDialog({
            idPrefix: "customerPanel",
            $containerElement: this.$deleteDialogContainer,
            visibleProjectCount: (config.activeProjectsNum + config.archivedProjectsNum),
            isCustomerActive: config.active,
            entityName: config.name,
            entitiesToDeleteCount: isBlocked ? 0 : 1,
            entitiesBlocked: [],
            entitiesBlockedCount: 0,
            entitiesBlockedDueToLockedTimeCount: 0,
            entitiesBlockedDueToUnavailableToModifyTimeTrackCount: 0,
            entitiesWithManagerTimeTrackCount: config.customerHasManagerTimeTrack ? 1 : 0,
            entitiesWithOtherUsersTimeTrackCount: config.customerHasOtherUsersTimeTrack ? 1 : 0,
            entitiesWithManagerApprovedTimeTrackCount: config.customerHasManagerApprovedTimeTrack ? 1 : 0,
            entitiesWithOtherUsersApprovedTimeTrackCount: config.customerHasOtherUsersApprovedTimeTrack ? 1 : 0,
            hasManagerApprover: config.hasManagerApprover,

            cancelHandler: function()
            {
                self.hideDeleteDialog();
                $(self).trigger("cancelDelete")
            },
            deleteHandler: function(revokeStatuses)
            {
                $(self).trigger("delete", revokeStatuses)
            }
        }).init();
        this.deleteDialog.hide();

    },

    hideDeleteDialog: function()
    {
        if (this.deleteDialog)
        {
            this.header.actions.menu.selectDelete(false);
            this.deleteDialog.hide();
            this.content.show();
            this.header.actions.hideMenu();
        }
    },

    showDeleteDialog: function()
    {
        if (this.deleteDialog)
        {
            this.header.actions.menu.selectDelete(true);
            this.content.hide();
            this.deleteDialog.show();
        }
    },

    onHide: function()
    {
        this.header.actions.hideMenu();
    }
});

