TaskManagement.EditProjectPanel.Header = function($containerEl, $panel)
{
    TemplateView.call(this, $containerEl);
    this.$panel = $panel;
};

TaskManagement.EditProjectPanel.Header.prototype = _.create(TemplateView.prototype, {

    updateOnInit: false,

    html: "<div class='editProjectPanelHeader'>" +
          "  <div class='cpInfo'>" +
          "    <span class='customerName ellipsis'/><img src='<%=separatorArrow%>' class='separator'/>" +
          "  </div>" +
          "  <div class='projectNamePlaceholder'/>" +
          "  <div class='actions'></div>" +
          "  <div class='tabs'>" +
          "    <div class='details'><span class='tabText'><%=detailsTab%></span></div>" +
          "    <div class='assignedUsers'><span class='tabText'><%=assignedUsersTab%> (<span class='assignedUsersCount'/>)</span></div>" +
          "  </div>" +
          "</div>",

    messageKeys: function()
    {
        return {
            detailsTab: "project.edit_panel.tabs.details",
            assignedUsersTab: "project.edit_panel.tabs.assigned_users"
        }
    },

    templateModel: function()
    {
        return {
            separatorArrow: atDefImgPath('taskManagement/arrow.png')
        }
    },

    afterRender: function()
    {
        this.projectId = null;
        this.$projectName = this.find('.projectName');
        this.$customerName = this.find('.customerName');
        this.$tabs = [];
        this.$tabs[TaskManagement.EditProjectPanel.Tabs.DETAILS_TAB] = this.find('.details');

        if (EnabledFeatures.PAS)
        {
            this.$tabs[TaskManagement.EditProjectPanel.Tabs.ASSIGNED_USERS_TAB] = this.find('.assignedUsers');
        }
        else
        {
            this.$containerEl.addClass('hideTabs');
        }
        
        this.nameInput = new SlidingPanel.NameInput(this.$containerEl.find('.projectNamePlaceholder'), true,
                messageResource.getMessage('project.common.projectNamePlaceholder'), 191).init();
        this.actions = new TaskManagement.EditActionButton(this.find(".actions"), this.$panel, this.$containerEl, true, false, true).init();
        this.$assignedUsersCount = this.find('.assignedUsersCount');
    },

    update: function(config)
    {
        var project = ProjectStore.get(config.id);
        this.updateCustomerName(project.customerId);
        this.nameInput.resetName(project.name);
        this.actions.disableDeleteButton(this.disableDeleteButtonText(config));
        
        if (!config.assignmentUsers) 
        {
            this.$containerEl.addClass('hideTabs');
        } else if (EnabledFeatures.PAS)
        {
            this.$containerEl.removeClass('hideTabs');
        }
    },

    updateCustomerName: function(customerId)
    {
        this.$customerName.text(CustomerStore.get(customerId).name);
    },

    disableDeleteButtonText: function(config)
    {
        var disableText = null;
        if (config.blockedDueToLockedTime)
        {
            disableText = messageResource.getMessage("project.edit.cannot_delete_due_to_lock_time");
        }
        else if (config.blockedDueToUnavailableToModifyTimeTrack)
        {
            var projectText = messageResource.getMessage("common.project");
            disableText = messageResource.getMessage(config.canManageAllUsersWithoutAutoApproved ?
                                                     "hierarchy_entity.edit_popup.cannot_delete_due_to_unavailable_modify_time_of_autoapproved_users" :
                                                     "hierarchy_entity.edit_popup.cannot_delete_due_to_unavailable_modify_time",
                                                     [projectText]);
        }
        return disableText;
    },

    onDependantDataUpdate: function(event)
    {
        if (event.type == 'assignmentsUpdated') 
        {
            this.$assignedUsersCount.text(event['assignedUsersNumber']);
        }
    }
});

