TaskPanel.Details = function($containerEl, editable)
{
    TemplateView.call(this, $containerEl);
    this.editable = editable;
};

TaskPanel.Details.prototype = _.create(TemplateView.prototype, {

    updateOnInit: false,

    html: "<div class='detailsTab panelTab'>" +
          " <div class='detailsSection section'>" +
          "     <div class='detailsHeader sectionHeader'><%=details%></div>" +
          "     <div class='detailsRow'>" +
          "         <div class='statusCell'><div class='cellHeader'><%=status%></div><div class='cellButton'/></div>" +
          "         <div class='deadlineEditorCell'><div class='cellHeader'><%=deadline%></div><div class='cellButton'/></div>" +
          "         <div class='typeOfWorkCell'><div class='cellHeader'><%=workType%></div><div class='cellButton'/></div>" +
          "     </div>" +
          " </div>" +
          " <div class='estimateSection section'>" +
          "     <div class='estimateHeader sectionHeader'><%=estimate%></div>" +
          "     <div class='collapseButton'/>" +
          "     <div class='estimateRow'>" +
          "         <div class='estimatedHours'><%=estimatedHours%>:</div>" +
          "         <div class='estimatedTimeWrapper'/>" +
          "         <div class='progress'/>" +
          "     </div>" +
          " </div>" +
          " <div class='descriptionSection section'>" +
          "  <div class='title sectionHeader'><%=description%></div>" +
          "  <div class='description'></div>" +
          " </div>" +
          "</div>",

    afterRender: function()
    {
        this.$detailsRow = this.find(".detailsRow");
        this.taskStatusButton = new TaskStatusButton(this.find('.statusCell .cellButton'), this.editable).init();
        var deadlineEditor = new DeadlineEditor(this.find('.deadlineEditorCell .cellButton'), this.editable).init();
        // clear deadline
        deadlineEditor.find('.icon').click(function(e)
                                                {
                                                    stopEventPropagation(e);
                                                    $(deadlineEditor).trigger('change', null);
                                                });
        this.deadlineEditor = deadlineEditor;
        this.typeOfWorkEditor = new TypeOfWorkButton(this.find('.typeOfWorkCell .cellButton'), this.editable).init();

        this.$descriptionSection = this.find('.descriptionSection');
        this.$description = this.find(".description");
        var emptyFieldText = messageResource.getMessage('task.edit_panel.name.empty');
        this.description = new SlidingPanel.DescriptionInput(this.$description, this.editable, emptyFieldText).init();

        this.$estimateSection = this.find(".estimateSection");
        var estimatedTimeEditor = new TimeEditor(this.find('.estimatedTimeWrapper'), this.editable);
        estimatedTimeEditor.placeholder = messageResource.getMessage('task.edit_panel.estimated_placeholder');
        this.estimatedTimeEditor = estimatedTimeEditor.init();

        var $clearButton = $("<div class='clearButton'/>");
        estimatedTimeEditor.$el.append($clearButton);
        $clearButton.click(function()
                           {
                               estimatedTimeEditor._triggerChange(null);
                           });

        var progressBar = new TaskPanel.Details.ProgressBar(this.find('.progress')).init();
        this.estimatedTimeEditor.on('change', function(e, time)
        {
            progressBar.setEstimatedTime(time);
            progressBar.update();
        });
        this.progressBar = progressBar;

    },

    messageKeys: function()
    {
        return {
            description: "task.edit_popup.task_description",
            details: 'task.edit_panel.sections.details',
            status: 'common.status',
            deadline: 'task.edit_popup.task_deadline',
            workType: 'task.edit_popup.task_billing_type',
            estimate: 'task.edit_panel.sections.estimate',
            estimatedHours: 'task.edit_popup.task_estimated_hours'
        }
    },

    update: function()
    {
        var task = TaskStore.get(this.taskId);
        this.$detailsRow.toggleClass('withTypeOfWork', !this.typeOfWorkEditor.isHidden());
        this.taskStatusButton.setCompleted(task['completed']);
        this.deadlineEditor.setDeadline(task['deadline']);
        this.$estimateSection.toggle(EnabledFeatures.EST && this.completeDetailsAccess);
        this.estimatedTimeEditor.setTime(EnabledFeatures.EST && this.completeDetailsAccess ? task['estimatedTime'] : null);
        this.progressBar.setEstimatedTime(task['estimatedTime']);
        this.progressBar.update();
    },

    setTaskId: function(taskId)
    {
        this.taskId = taskId;
    },

    setDescription: function(description)
    {
        this.description.setDescription(description);
        this.$descriptionSection.toggle(this.editable || StringUtils.isNotEmptyString(description));
    },

    setEditable: function(editable)
    {
        this.editable = editable;
        this.taskStatusButton.setEditable(editable);
        this.deadlineEditor.setEditable(editable);
        this.typeOfWorkEditor.setEditable(editable);
        this.description.setEditable(editable);
        this.estimatedTimeEditor.setEditable(editable);
    },

    setCompleteDetailsAccess: function(accessEnabled)
    {
        this.completeDetailsAccess = accessEnabled;
    }

});