TaskPanel.Header = function($containerEl, $panel, editable)
{
    TemplateView.call(this, $containerEl);
    this.$panel = $panel;
    this.editable = editable;
};

TaskPanel.Header.prototype = _.create(TemplateView.prototype, {

    updateOnInit: false,

    html: "<div class='header'>" +
          "  <div class='cpInfo'>" +
          "   <span class='customerName ellipsis'/><img src='<%=separatorArrow%>' class='separator'/><span class='projectName ellipsis'/>" +
          "  </div>" +
          "  <div class='taskNamePlaceholder'/>" +
          "   <div class='actions'></div>" +
          "  <div class='tabs'>" +
          "    <div class='details'><span class='tabText'><%=detailsTab%></span></div>" +
          "    <div class='reportedTime'><span class='tabText'><%=reportedTimeTab%> <span class='timeValue'/></span></div>" +
          "  </div>" +
          "</div>",

    messageKeys: function()
    {
        return {
            detailsTab: "task.edit_panel.tabs.details",
            reportedTimeTab: "task.edit_panel.tabs.reportedTime"
        }
    },

    templateModel: function()
    {
        return {
            separatorArrow: atDefImgPath('taskManagement/arrow.png')
        }
    },

    afterRender: function()
    {
        this.taskId = null;
        this.reportedTime = null;
        this.$projectName = this.find('.projectName');
        this.$cutomerName = this.find('.customerName');
        this.$reportedTime = this.find('.reportedTime .timeValue');
        this.$tabs = [];
        this.$tabs[TaskPanel.Tabs.DETAILS_TAB] = this.find('.details');
        this.$tabs[TaskPanel.Tabs.REPORTED_TAB] = this.find('.reportedTime');
        this.taskNameInput = new SlidingPanel.NameInput(this.$containerEl.find('.taskNamePlaceholder'), this.editable, messageResource.getMessage('task.edit_popup.name_placeholder'), 191).init();
        this.actions = new TaskManagement.EditActionButton(this.$containerEl.find(".actions"), this.$panel, this.$containerEl, true, false, true).init();
    },

    update: function()
    {
        var task = TaskStore.get(this.taskId);
        this.projectName = ProjectStore.get(task.projectId).name;
        this.customerName = CustomerStore.get(task.customerId).name;
        this.taskName = task.name;

        this.$cutomerName.text(this.customerName);
        this.$projectName.text(this.projectName);
        this.$reportedTime.text(_.isNull(this.reportedTime) ?  '' : '(' + UserTime.formatTimeWithDigitSeparator(this.reportedTime) + ')');
        this.taskNameInput.update();
        this.actions.hide();
        if (this.editable)
        {
            this.actions.show();
        }
    },

    updateActions: function(config) 
    {
        this.actions.disableDeleteButton(this.disableDeleteButtonText(config));
    },
    
    disableDeleteButtonText: function(config)
    {
        var disableText = null;
        if (config.blockedDueToLockedTime)
        {
            disableText = messageResource.getMessage("task.edit_popup.cannot_delete_due_to_lock_time");
        }
        else if (config.blockedDueToUnavailableToModifyTimeTrack)
        {
            var text = messageResource.getMessage("common.task");
            disableText = messageResource.getMessage(config.canManageAllUsersWithoutAutoApproved ?
                                                     "hierarchy_entity.edit_popup.cannot_delete_due_to_unavailable_modify_time_of_autoapproved_users" :
                                                     "hierarchy_entity.edit_popup.cannot_delete_due_to_unavailable_modify_time",
                                                     [text]);
        }
        return disableText;
    },

    setEditable: function(editable)
    {
        this.editable = editable;
        this.taskNameInput.setEditable(editable);
    },

    setTaskId: function(taskId)
    {
        this.taskId = taskId;
    },

    setReportedTime: function(reportedTime)
    {
        this.reportedTime = reportedTime;
    }
});
