function TaskPanel($containerEl, $pageEl)
{
    SlidingPanel.call(this, $containerEl, $pageEl);
}

TaskPanel.Tabs = {
    DETAILS_TAB: 'details',
    REPORTED_TAB: 'reportedTime'
};

TaskPanel.prototype = _.create(SlidingPanel.prototype, {

    type: "edit_task_sliding_panel",

    _switchTab: function(tabKey)
    {
        if (this.selectedTab !== tabKey)
        {
            this.selectedTab = tabKey;
            var self = this;
            _.forEach(TaskPanel.Tabs, function(key)
            {
                self.header.$tabs[key].toggleClass('selected', key === tabKey);
                self.tabs[key].$el.toggleClass('displayed', key === tabKey);
            });
            this.refreshScroll();
            this.scrollToTop();
        }
        this.hideDeleteDialog();
        this.hideMoveToDialog();
        this.trigger('switchTab', tabKey);
    },

    _subscribeTabs: function()
    {
        var self = this;
        _.forEach(TaskPanel.Tabs, function(key)
        {
            self.header.$tabs[key].click(_.bind(self._switchTab, self, key));
        });
    },

    update: function()
    {
        var self = this;
        if (!this.selectedTab)
            this._switchTab(TaskPanel.Tabs.DETAILS_TAB);
        this.header.update();
        _.forEach(TaskPanel.Tabs, function(key)
        {
            self.tabs[key].update();
        });
        this.hideDeleteDialog();
        this.hideMoveToDialog();
        this.refreshScroll();
        this.scrollToTop();
    },

    updateDeleteDialog: function(name, state, keepShown)
    {
        this._initDeleteDialog(name, state, keepShown);
    },

    setEditable: function(editable)
    {
        this.editable = editable;
        var self = this;
        this.header.setEditable(editable);
        _.forEach(TaskPanel.Tabs, function(key)
        {
            if (self.tabs[key].setEditable)
                self.tabs[key].setEditable(editable);
        });
    },

    setTaskId: function(taskId)
    {
        this.taskId = taskId;
        this.header.setTaskId(taskId);
        this.tabs[TaskPanel.Tabs.DETAILS_TAB].setTaskId(taskId);
    },

    afterRender: function()
    {
        SlidingPanel.prototype.afterRender.call(this);
        this.editable = false;
        this.header = new TaskPanel.Header(this.find(".headerBlock"), this.$el, this.editable).init();
        this.$content = this.find(".content");
        this.$deleteDialogContainer = $("<div class='deleteDialodContainer'></div>").appendTo(this.$content);
        
        this.tabs = [];
        this.tabs[TaskPanel.Tabs.DETAILS_TAB] = new TaskPanel.Details(this.find(".content"), this.editable).init();
        this.tabs[TaskPanel.Tabs.REPORTED_TAB] = new TaskPanel.ReportedTime(this.find(".content")).init();

        this.moveToDialog = new MoveToDialog(this.$content, MoveToDialog.Mode.CUSTOMER_PROJECT, "task.list.move_to.task.title").init();
        this._subscribeTabs();
    },

    // TODO: refactor move to and delete dialogs and handle escape keydown using KeydownHandler
    showMoveToDialog: function()
    {
        if (!this.moveToDialogShown)
        {
            this.moveToDialogShown = true;
            this.deleteDialogShown = false;

            this.tabs[this.selectedTab].$el.removeClass('displayed');
            if (this.deleteDialog)
                this.deleteDialog.hide();
            this.header.actions.menu.selectMove(true);
            this.moveToDialog.setShowArchived(TaskStore.get(this.taskId).completed);
            this.moveToDialog.show();

            this.suppressScrollCheck = true;
            this.refreshScroll();
            this.scrollToTop();
        }
    },

    hideMoveToDialog: function()
    {
        this.moveToDialogShown = false;
        this.moveToDialog.hide();
        this.header.actions.menu.selectMove(false);
        this.tabs[this.selectedTab].$el.addClass('displayed');
        this.header.actions.hideMenu();

        this.refreshScroll();
    },

    showDeleteDialog: function()
    {
        if (!this.deleteDialogShown)
        {
            this.deleteDialogShown = true;
            this.moveToDialogShown = false;

            this.tabs[this.selectedTab].$el.removeClass('displayed');
            this.moveToDialog.hide();
            this.header.actions.menu.selectDelete(true);
            this.deleteDialog.show();

            this.suppressScrollCheck = true;
            this.refreshScroll();
            this.scrollToTop();
        }
    },

    hideDeleteDialog: function()
    {
        this.deleteDialogShown = false;
        if (this.deleteDialog)
            this.deleteDialog.hide();
        this.header.actions.menu.selectDelete(false);
        this.tabs[this.selectedTab].$el.addClass('displayed');

        this.refreshScroll();
    },
    
    _initDeleteDialog: function(name, config, keepShown)
    {
        var self = this;
        delete this.deleteDialog;
        this.$deleteDialogContainer.empty();
        if (!config.blockedDueToLockedTime && !config.blockedDueToUnavailableToModifyTimeTrack)
        {
            this.deleteDialog = new CPTManagement.DeleteTaskPopupConfirmationDialog({
                $containerElement: this.$deleteDialogContainer,
                idPrefix: 'taskPanel',
                entityName: name,
                entitiesToDeleteCount: 1,
                entitiesBlocked: 0,
                entitiesBlockedCount: 0,
                entitiesBlockedDueToLockedTimeCount: 0,
                entitiesBlockedDueToUnavailableToModifyTimeTrackCount: 0,
                entitiesWithManagerTimeTrackCount: config.taskHasManagerTimeTrack ? 1 : 0,
                entitiesWithOtherUsersTimeTrackCount: config.taskHasOtherUsersTimeTrack ? 1 : 0,
                entitiesWithManagerApprovedTimeTrackCount: config.taskHasManagerApprovedTimeTrack? 1 : 0,
                entitiesWithOtherUsersApprovedTimeTrackCount: config.taskHasOtherUsersApprovedTimeTrack? 1 : 0,
                hasManagerApprover: config.hasManagerApprover,
                cancelHandler: function()
                {
                    $(self).trigger("cancelDelete")
                },
                deleteHandler: function(revokeStatuses)
                {
                    $(self).trigger("delete", revokeStatuses)
                }
            }).init();
            if (keepShown && this.deleteDialogShown)
                this.deleteDialog.show();
            else
                this.deleteDialog.hide();
        }
    },

    hideAnimation: function()
    {
        return "anim-hide-slide-left"
    },

    showAnimation: function()
    {
        return "anim-show-slide-left"
    },

    onHide: function()
    {
        this.header.actions.hideMenu();
    }
});