TaskPanel.ReportedTime = function($containerEl)
{
    TemplateView.call(this, $containerEl)
};

TaskPanel.ReportedTime.SORT_CONSTANTS = {
    byDate: "date",
    byUser: "user",
    asc: "Asc",
    desc: "Desc"
};

TaskPanel.ReportedTime.prototype = _.create(TemplateView.prototype, {
    updateOnInit: false,


    html: "<div class='reportedTimeTab panelTab'>" +
          " <div class='summarySection section'>" +
          "     <div class='totalSpentHeader sectionHeader'/><span class='ofEstimated'><%=of%>  <span class='estimatedValue'/></span>" +
          "     <div class='summaryRecords'/>" +
          " </div>" +
          " <div class='reportedTimeSection section'>" +
          "     <div class='reportedTimeHeader sectionHeader'><%=reportedTimeComments%>:</div>" +
          "     <table class='reportedTimeTable' cellpadding='0' cellspacing='0'><thead>" +
          "         <tr><th class='dateColumn'><span class='dateCell dateHeader'/></th>" +
          "         <th class='userColumn'><span class='shortSeparator'>&nbsp;</span><span class='userCell userHeader'/></th>" +
          "         <th class='timeColumn'><span class='shortSeparator'>&nbsp;</span><span class='timeCell timeHeader'><%=time%></span></th>" +
          "         <th class='commentColumn'><span class='shortSeparator'>&nbsp;</span><span class='commentCell commentHeader'><%=comment%></span></th></th>" +
          "     </thead><tbody class='timeRows'/></table>" +
          "     <div class='tasksNotFoundMessage'><%-noTimeComments%></div>" +
          "     <div class='loadingIndicatorContainer'></div>" +
          " </div>" +
          "</div>",

    messageKeys: function()
    {
        return {
            time: 'common.time',
            comment: 'common.comment',
            of: 'common.of',
            reportedTimeComments: 'task.edit_panel.reported_time',
            noTimeComments: 'task.edit_popup.no_time_reported_by_you_for_task'
        }
    },

    afterRender: function()
    {
        var self = this;

        this.totalsSummary = new TaskPanel.ReportedTime.SummaryRecord(this.find('.totalSpentHeader'),
                                                                      messageResource.getMessage('task.edit_panel.total_spent') + ':').init();
        this.$ofEsimated = this.find('.ofEstimated');
        this.$estimated = this.find('.estimatedValue');
        this.$summaryRecords = this.find(".summaryRecords");
        this.$summarySection = this.find('.summarySection');

        this.timeRows = {};
        this.timeRowsCount = 0;
        this.$timeRows = this.find('.timeRows');

        // todo: {SortableTableHeader} move component to controls
        this.dateHeader = new TaskManagement.SortableTableHeader(this.find('.dateHeader'),
                                                                 this.sortBy === TaskPanel.ReportedTime.SORT_CONSTANTS['byDate'],
                                                                 this.sortOrder === TaskPanel.ReportedTime.SORT_CONSTANTS['asc'], 'common.date').init();
        this.dateHeader.sortBy = TaskPanel.ReportedTime.SORT_CONSTANTS['byDate'];
        this.userHeader = new TaskManagement.SortableTableHeader(this.find('.userHeader'),
                                                                 this.sortBy === TaskPanel.ReportedTime.SORT_CONSTANTS['byUser'],
                                                                 this.sortOrder === TaskPanel.ReportedTime.SORT_CONSTANTS['asc'],'common.User').init();
        this.userHeader.sortBy = TaskPanel.ReportedTime.SORT_CONSTANTS['byUser'];
        this.$loadingIndicatorContainer = this.find(".loadingIndicatorContainer");

        this.appendLoadingMoire = new LoadingMoire(this.find(this.$loadingIndicatorContainer), {}, false).init();
    },


    setTableSettings: function(settings)
    {
        this.sortBy = settings['sortBy'];
        this.sortOrder = settings['sortOrder'];
        this.dateHeader.setActive(this.sortBy === TaskPanel.ReportedTime.SORT_CONSTANTS['byDate']);
        this.userHeader.setActive(this.sortBy === TaskPanel.ReportedTime.SORT_CONSTANTS['byUser']);
        if (this.dateHeader.isActive())
            this.dateHeader.setAsc(this.sortOrder === TaskPanel.ReportedTime.SORT_CONSTANTS['asc']);
        if (this.userHeader.isActive())
            this.userHeader.setAsc(this.sortOrder === TaskPanel.ReportedTime.SORT_CONSTANTS['asc']);

    },

    updateTimeRows: function(timeRows)
    {
        if (timeRows.length === 0)
        {
            this.find('.tasksNotFoundMessage').show();
        }
        this.addTimeRows(timeRows);
    },

    updateEstimated: function(estimatedTime)
    {
        if (_.isNumber(estimatedTime) && EnabledFeatures.EST)
        {
            this.$estimated.text(UserTime.formatTimeWithDigitSeparator(estimatedTime));
        }
        this.$ofEsimated.toggleClass('displayed', _.isNumber(estimatedTime) && EnabledFeatures.EST);

    },

    updateSummaryRecords: function(totalTime, estimatedTime, summaryRecords)
    {
        var self = this;
        this.$summarySection.toggle(!_.isNull(totalTime));
        this.totalsSummary.setValue(totalTime);
        this.updateEstimated(estimatedTime);
        this.$summaryRecords.empty();
        _.each(summaryRecords, function(record)
        {
            new TaskPanel.ReportedTime.SummaryRecord(self.$summaryRecords, record['formattedName']).init().setValue(record['spentTime']);
        });
    },

    addTimeRows: function(timeRows)
    {
        var self = this;
        _.forEach(timeRows, function(rowData)
        {
            var newRow = new TaskPanel.ReportedTime.TimeRow(self.$timeRows).init();
            newRow.setRowData(rowData.eventDate.parseJSONDate(), rowData.formattedName, rowData.actuals, (rowData.commentsPresent) ? rowData.comments : null);
            self.timeRows[self.timeRowsCount++] = newRow;
        });
    },

    clearTimeRows: function()
    {
        this.$timeRows.empty();
        this.find('.tasksNotFoundMessage').hide();
        this.timeRowsCount = 0;
        this.timeRows = {};
    },

    setTimeAndCommentsLoading: function(isLoading)
    {
        this.$loadingIndicatorContainer.toggle(isLoading);
        this.appendLoadingMoire.toggle(isLoading)
    }
});

TaskPanel.ReportedTime.SummaryRecord = function($containerEl, label)
{
    TemplateView.call(this, $containerEl);
    this.label = label;
};

TaskPanel.ReportedTime.SummaryRecord.prototype = _.create(TemplateView.prototype, {
    updateOnInit: false,

    html: "<div class='summaryRecord'><span class='label ellipsis'><%-label%> </span><span class='value'/></div>",

    templateModel: function()
    {
        return {label: this.label}
    },

    afterRender: function()
    {
        this.value = null;
        this.$value = this.find('.value');
    },
    
    setValue: function(value)
    {
        this.value = value;
        this.update();
    },

    update: function()
    {
        if (!_.isNull(this.value))
            this.$value.text('  ' + UserTime.formatTimeWithDigitSeparator(this.value));
    }
});