PDFDefinitionColumnBuilder = function() {};

PDFDefinitionColumnBuilder.prototype = _.create(Object.prototype, {
    tableHeaderStyleNoMargin: "reportHeader",
    tableHeaderMargin: "reportHeaderMargin",
    tableHeaderStyle: ["reportHeader", "reportHeaderMargin"],
    tableSubHeaderStyle: ["reportSubHeader", "reportHeaderMargin"],
    tableSubHeaderWithMarginStyle: ["reportSubHeader", "reportTitleNormalMargin"],
    tableThirdLvlHeaderStyle: ["reportSubHeader", "reportHeaderMargin", "smallText"],

    reportTotalValue: "reportTotalValue",
    reportTotalTitle: ["reportHeader", "reportTotalTitle"],
    reportTitleBigMargin: ["reportHeader", "reportTitleBigMargin"],
    reportTitleNormalMargin: ["reportHeader", "reportTitleNormalMargin"],
    reportTitleSmallMargin: ["reportHeader", "reportTitleSmallMargin"],
    reportGroupingValues: ["reportFirstGroupingValue", "reportSecondGroupingValue", "reportThirdGroupingValue", "reportFourthGroupingValue"],

    asterisk: "asterisk",

    smallText: "smallText",
    tinyText: "tinyText",
    negativeValue: "negativeValue",

    init: function(config, rowSpanNumber)
    {
        this.timeFormat = config.timeFormat;
        this.formatUtil = new BaseReport.FormatUtils();

        this.rowSpanNumber = rowSpanNumber;

        var emptyFn = _.constant(false);
        this.valueMask = [emptyFn, emptyFn, emptyFn, emptyFn];

        return this;
    },

    pushWidth: function(destination, sizes) {},
    setParams: function(params) 
    {
        this.params = params
    },
    makeHeader: function() {},

    makeTotals: function(data)
    {
        return this._makeRow(this.reportTotalValue, data, _.constant(false));
    },

    makeSubHeader: function() {return ''},
    makeThirdLvlHeader: function() {return ''},

    makeRow: function(lvl, data)
    {
        return this._makeRow(this.reportGroupingValues[lvl], data, this.valueMask[lvl]);
    },

    makeFirstRow: function(data) {return this.makeRow(0, data)},
    makeSecondRow: function(data) {return this.makeRow(1, data)},
    makeThirdRow: function(data) {return this.makeRow(2, data)},
    makeFourthRow: function(data) {return this.makeRow(3, data)},

    _makeRowDefinition: function(style, text)
    {
        return {
            noWrap: true,
            style: style,
            text: text
        };
    },

    _makeHeaderDefinition: function(text, style)
    {
        if (_.isUndefined(style))
            style = this.tableHeaderStyle;

        return {
            noWrap: true,
            style: style,
            text: text,
            rowSpan: this.rowSpanNumber
        };
    },

    _formatTimeValue: function(value)
    {
        return this.formatUtil.formatPureTime(value, this.timeFormat);
    },

    _formatPercentValue: function(value)
    {
        return this.formatUtil.formatPercent(value);
    },

    _formatStyle: function(value, style)
    {
        return (value < 0 ) ? [style, this.negativeValue] : style;
    },

    makeTimeCell: function(value, style, valueIsMasked)
    {
        var newStyle = valueIsMasked ? (valueIsMasked.style ? [valueIsMasked.style, style] : style) : this._formatStyle(value, style);
        var newValue = valueIsMasked ? valueIsMasked.msg : this._formatTimeValue(value);
        return {
            noWrap: true,
            style: newStyle,
            text: newValue
        }
    },

    makeSignedTimeCell: function(value, style, valueIsMasked)
    {
        var newStyle = valueIsMasked ? (valueIsMasked.style ? [valueIsMasked.style, style] : style) : this._formatStyle(value, style);
        var newValue = valueIsMasked ? valueIsMasked.msg : this._formatTimeValue(value);
        var signedValue = (value > 0) ? '+' + newValue : newValue;
        return {
            noWrap: true,
            style: newStyle,
            text: signedValue
        }
    },

    makePercentCell: function(value, style, valueIsMasked)
    {
        var newStyle = valueIsMasked ? (valueIsMasked.style ? valueIsMasked.style : style) : this._formatStyle(value, style);
        var newValue = valueIsMasked ? valueIsMasked.msg : this._formatPercentValue(value);
        return {
            noWrap: true,
            style: newStyle,
            text: newValue 
        }
    },

    addAsterisk: function(text, asteriskChar)
    {
        return [
            {
                width: '*',
                text: ''
            },
            {
                width: 'auto',
                text: text,
                marginRight: 1
            },
            {
                width: 'auto',
                style: "asterisk",
                text: asteriskChar,
                marginLeft: 1,
                alignment: 'left'
            },
            {
                width: '*',
                text: ''
            }
        ];
    }
});

PDFDefinitionColumnBuilder.Levels = function() {};
PDFDefinitionColumnBuilder.Levels.prototype = _.create(PDFDefinitionColumnBuilder.prototype, {
    _getTitle: function(groupingLevel, identity, maxLen)
    {
        switch (groupingLevel)
        {
            case ReportGroupingLevel.CUSTOMER:
                return this.formatUtil.formatByName(this.store.getCustomer(identity));
            case ReportGroupingLevel.PROJECT:
                return this.formatUtil.formatProject(this.store.getProject(identity), this.store.getGroupingPresentationFlags());
            case ReportGroupingLevel.TASK:
                return this.formatUtil.formatTask(this.store.getTask(identity), this.store.getGroupingPresentationFlags());
            case ReportGroupingLevel.USER:
                return this.formatUtil.formatUserAndTrim(this.store.getUser(identity), maxLen);
            case ReportGroupingLevel.USER_GROUP:
                return this.formatUtil.formatByName(this.store.getUserGroup(identity), maxLen);
            case ReportGroupingLevel.TIME_ZONE_GROUP:
                return this.formatUtil.formatByName(this.store.getTimeZoneGroup(identity), maxLen);
            case ReportGroupingLevel.PROJECT_NAME:
                return this.formatUtil.formatByName(this.store.getProjectName(identity));
            case ReportGroupingLevel.TASK_NAME:
                return this.formatUtil.formatByName(this.store.getTaskName(identity));
            case ReportGroupingLevel.BILLING_TYPE:
                return this.formatUtil.formatByName(this.store.getTypeOfWork(identity));
        }
    }
});

PDFDefinitionColumnBuilder.Totals = function() {};
PDFDefinitionColumnBuilder.Totals.prototype = _.create(PDFDefinitionColumnBuilder.prototype, {

    pushWidth: function(destination, sizes)
    {
        destination.push(sizes["totals"] + '%');
    },

    _getValue: function(data)
    {
        //abstract
    },

    _makeRow: function(style, data, valueMask)
    {
        return this._makeRowDefinition(style, valueMask(this, data) ? "" : this._formatTimeValue(this._getValue(data)));
    }

});
