EVAReport.PDFDefinitionColumnBuilder = function() {};

EVAReport.PDFDefinitionColumnBuilder.prototype = _.create(PDFDefinitionColumnBuilder.prototype, {

    init: function(config, rowSpanNumber)
    {
        PDFDefinitionColumnBuilder.prototype.init.call(this, config, rowSpanNumber);

        if (config.fourthGroupingLevel == ReportGroupingLevel.USER)
            this.valueMask[3] = function(column) { return column.isUserMasked ? {msg: ""} : false; };

        return this;
    }

});


EVAReport.PDFDefinitionColumnBuilder.Totals = function() {};
EVAReport.PDFDefinitionColumnBuilder.Totals.prototype = _.create(PDFDefinitionColumnBuilder.Totals.prototype, {

    init: function(config, rowSpanNumber)
    {
        PDFDefinitionColumnBuilder.Totals.prototype.init.call(this, config, rowSpanNumber);

        if (config.fourthGroupingLevel == ReportGroupingLevel.USER)
            this.valueMask[3] = function(column) { return !_.isUndefined(column.isUserMasked) && column.isUserMasked ? {msg: ""} : false; };

        return this;
    }

});


EVAReport.PDFDefinitionColumnBuilder.Levels = function() {};
EVAReport.PDFDefinitionColumnBuilder.Levels.prototype = _.create(PDFDefinitionColumnBuilder.Levels.prototype, {
    init: function(config, store, rowSpanNumber)
    {
        EVAReport.PDFDefinitionColumnBuilder.prototype.init.call(this, config, rowSpanNumber);

        this.store = store;
        this.groupingLevels = [config.firstGroupingLevel, config.secondGroupingLevel, config.thirdGroupingLevel, config.fourthGroupingLevel];
        this.reportGroupingStyles = ["reportFirstGroupingRow", "reportSecondGroupingRow", "reportThirdGroupingRow", "reportFourthGroupingRow"];
        this.reportGroupingMargins = ["reportFirstGroupingRowMargin", "reportSecondGroupingRowMargin", "reportThirdGroupingRowMargin", "reportFourthGroupingRowMargin"];

        this.isWide = !config.includeNotEstimated;
        this.maxUserNameLen = 14;

        return this;
    },

    _makeCustomRow: function(style, groupingLevel, identity)
    {
        var text = this._getTitle(groupingLevel, identity, this.maxUserNameLen);
        return this._makeRowDefinition(style, text);
    },

    pushWidth: function(destination, sizes)
    {
        var baseWidth = this.isWide ? sizes["levelsWide"] : sizes["levels"];
        destination.push(baseWidth + '%');
    },

    makeHeader: function()
    {
        var names = [];
        for (var i = 0; i < 4; i++)
        {
            if (this.groupingLevels[i] != ReportGroupingLevel.NONE)
            {
                names.push(this.formatUtil.formatGroupingLevel(this.groupingLevels[i]));
            }
        }
        return this._makeHeaderDefinition(names.join(' / '), [this.tableHeaderStyleNoMargin, "reportLevelsTitle"]);
    },

    makeTotals: function()
    {
        return this._makeRowDefinition("reportTotal", messageResource.getMessage("report.total"));
    },

    makeRow: function(lvl, data)
    {
        return this._makeCustomRow([this.reportGroupingStyles[lvl], this.reportGroupingMargins[data.level]], this.groupingLevels[lvl], data.id);
    }

});


EVAReport.PDFDefinitionColumnBuilder.TotalSpentTime = function() {};
EVAReport.PDFDefinitionColumnBuilder.TotalSpentTime.prototype = _.create(EVAReport.PDFDefinitionColumnBuilder.Totals.prototype, {

    _getValue: function(data)
    {
        return data.totalSpentTime;
    },

    makeHeader: function()
    {
        return this._makeHeaderDefinition(messageResource.getMessage("report.budgeted_vs_actual.spent_time.total"), this.reportTitleBigMargin);
    }
});


EVAReport.PDFDefinitionColumnBuilder.SpentTimeForNotEstimated = function() {};
EVAReport.PDFDefinitionColumnBuilder.SpentTimeForNotEstimated.prototype = _.create(EVAReport.PDFDefinitionColumnBuilder.Totals.prototype, {

    _getValue: function(data)
    {
        return data.spentTimeForNotEstimated;
    },

    makeHeader: function()
    {
        return this._makeHeaderDefinition(messageResource.getMessage("report.budgeted_vs_actual.spent_time.nonbudgeted"), this.reportTitleSmallMargin);
    }
});


EVAReport.PDFDefinitionColumnBuilder.EstimatedTime = function() {};
EVAReport.PDFDefinitionColumnBuilder.EstimatedTime.prototype = _.create(EVAReport.PDFDefinitionColumnBuilder.Totals.prototype, {
    isUserMasked: true,

    _getValue: function(data)
    {
        return data.estimatedTime;
    },

    makeHeader: function()
    {
        return this._makeHeaderDefinition(messageResource.getMessage("report.budgeted_vs_actual.budgeted"), this.reportTitleNormalMargin);
    }
});


EVAReport.PDFDefinitionColumnBuilder.SpentTimeForEstimated = function() {};
EVAReport.PDFDefinitionColumnBuilder.SpentTimeForEstimated.prototype = _.create(EVAReport.PDFDefinitionColumnBuilder.Totals.prototype, {

    _getValue: function(data)
    {
        return data.spentTimeForEstimated;
    },

    makeHeader: function()
    {
        return this._makeHeaderDefinition(messageResource.getMessage("report.budgeted_vs_actual.spent_time"), this.reportTitleNormalMargin);
    }
});


EVAReport.PDFDefinitionColumnBuilder.VarianceTime = function() {};
EVAReport.PDFDefinitionColumnBuilder.VarianceTime.prototype = _.create(EVAReport.PDFDefinitionColumnBuilder.prototype, {
    isUserMasked: true,

    pushWidth: function(destination, sizes)
    {
        destination.push(sizes["variance"] + '%');
        destination.push(sizes["variance"] + '%');
    },

    makeHeader: function()
    {
        var varianceTxt = [
            {text: messageResource.getMessage("report.budgeted_vs_actual.variance")},
            {text: "\n(" + messageResource.getMessage("report.budgeted_vs_actual.budgeted_minus_spent") + ")", style: this.tinyText}
        ];

        return [{
            noWrap: true,
            style: this.tableHeaderStyle,
            text: varianceTxt,
            colSpan: 2
        }, {}];
    },

    makeSubHeader: function()
    {
        return [
            {
                noWrap: true,
                style: this.tableHeaderStyle,
                text: messageResource.getMessage("report.budgeted_vs_actual.variance.mh")
            },
            {
                noWrap: true,
                style: this.tableHeaderStyle,
                text: messageResource.getMessage("report.budgeted_vs_actual.variance.percent")
            }
        ];
    },

    _makeRow: function(style, data, valueMask)
    {
        var invalidPercentage = (!_.isUndefined(data.estimatedTime) && data.estimatedTime < 0.0001);
        return [
            this.makeTimeCell(data.varianceTime, style, valueMask(this, data)),
            this.makePercentCell(invalidPercentage? null : data.varianceTimeInPercentage, style, valueMask(this, data))
        ];
    }
});


EVAReport.PDFDefinitionColumnBuilder.EstimatedTasksBlock = function() {};
EVAReport.PDFDefinitionColumnBuilder.EstimatedTasksBlock.prototype = _.create(EVAReport.PDFDefinitionColumnBuilder.prototype, {
    
    init: function(config, store, rowSpanNumber)
    {
        EVAReport.PDFDefinitionColumnBuilder.prototype.init.call(this, config, rowSpanNumber);
        
        var lvls = [config.firstGroupingLevel, config.secondGroupingLevel, config.thirdGroupingLevel, config.fourthGroupingLevel];
        for (var i = 0; i < 4; i++)
        {
            if (lvls[i] == ReportGroupingLevel.TASK)
                this.valueMask[i] = function(column, data)
                {
                    return !_.isUndefined(column.maskedTaskValue) && !store.getTask(data.id).estimated ? column.maskedTaskValue : false;
                };
        }
        
        return this;
    },

    pushWidth: function(destination, sizes)
    {
        destination.push(sizes["estimatedTime"] + '%');
        destination.push(sizes["estimatedTime"] + '%');
        destination.push(sizes["variance"] + '%');
        destination.push(sizes["variance"] + '%');
    },

    makeHeader: function()
    {
        return [{
            noWrap: true,
            style: this.tableHeaderStyle,
            text: messageResource.getMessage("report.budgeted_vs_actual.spent.budgeted_tasks"),
            colSpan: 4
        }, {}, {}, {}];
    },

    makeSubHeader: function()
    {
        var varianceTxt = [
            {text: messageResource.getMessage("report.budgeted_vs_actual.variance")},
            {text: "(" + messageResource.getMessage("report.budgeted_vs_actual.budgeted_minus_spent") + ")", style: this.tinyText}
        ];

        return [
            {
                noWrap: true,
                style: this.tableSubHeaderWithMarginStyle,
                text: messageResource.getMessage("report.budgeted_vs_actual.spent_time"),
                rowSpan: 2
            },
            {
                noWrap: true,
                style: this.tableSubHeaderWithMarginStyle,
                text: messageResource.getMessage("report.budgeted_vs_actual.budgeted"),
                rowSpan: 2
            },
            {
                noWrap: true,
                style: this.tableSubHeaderStyle,
                stack: varianceTxt,
                colSpan: 2
            }, {}
        ];
    },

    makeThirdLvlHeader: function()
    {
        return [
            {}, {},
            {
                noWrap: true,
                style: this.tableThirdLvlHeaderStyle,
                text: messageResource.getMessage("report.budgeted_vs_actual.variance.mh")
            },
            {
                noWrap: true,
                style: this.tableThirdLvlHeaderStyle,
                text: messageResource.getMessage("report.budgeted_vs_actual.variance.percent")
            }
        ];
    },

    cellMasks: [
            {isUserMasked: false},
            {isUserMasked: true, maskedTaskValue: {msg: messageResource.getMessage("report.budgeted_vs_actual.not_budgeted"), style: "notEstimated"}},
            {isUserMasked: true},
            {isUserMasked: true, maskedTaskValue: {msg: "-"}}
    ],
    
    _makeRow: function(style, data, valueMask)
    {
        var invalidPercentage = (!_.isUndefined(data.estimatedTime) && data.estimatedTime < 0.0001);
        return [
            this.makeTimeCell(data.spentTimeForEstimated, style, valueMask(this.cellMasks[0], data)),
            this.makeTimeCell(data.estimatedTime, style, valueMask(this.cellMasks[1], data)),
            this.makeTimeCell(data.varianceTime, style, valueMask(this.cellMasks[2], data)),
            this.makePercentCell(invalidPercentage? null : data.varianceTimeInPercentage, style, valueMask(this.cellMasks[3], data))
        ];
    }
});

