EVAReport.PDFDefinitionCreator = function(parameterFormatter) {};

EVAReport.PDFDefinitionCreator.prototype = _.create(PDFDefinitionCreator.prototype, {

    createDocDefinition: function(pdfConfig, parameterFormatter, store)
    {
        this.parameterFormatter = parameterFormatter;
        this.pdfPrintParams = pdfConfig.pdfPrintParameters;
        this.pdfData = pdfConfig.pdfData;
        this.pdfConfig = pdfConfig;
        this.reportParams = pdfConfig.reportParams;
        this.dateFormat = this.pdfConfig.dateFormat;

        var rowSpanNumber = this.reportParams.includeNotEstimated ? 3 : 2;
        
        var columns = [];
        columns.push(new EVAReport.PDFDefinitionColumnBuilder.Levels().init(this.reportParams, store, rowSpanNumber));
        
        if (this.reportParams.includeNotEstimated)
        {
            columns.push(new EVAReport.PDFDefinitionColumnBuilder.TotalSpentTime().init(this.reportParams, rowSpanNumber));
            columns.push(new EVAReport.PDFDefinitionColumnBuilder.SpentTimeForNotEstimated().init(this.reportParams, rowSpanNumber));
            columns.push(new EVAReport.PDFDefinitionColumnBuilder.EstimatedTasksBlock().init(this.reportParams, store, rowSpanNumber));
        }
        else
        {
            columns.push(new EVAReport.PDFDefinitionColumnBuilder.SpentTimeForEstimated().init(this.reportParams, rowSpanNumber));
            columns.push(new EVAReport.PDFDefinitionColumnBuilder.EstimatedTime().init(this.reportParams, rowSpanNumber));
            columns.push(new EVAReport.PDFDefinitionColumnBuilder.VarianceTime().init(this.reportParams, rowSpanNumber));
        }

        var ddb = new EVAReport.PDFDefinitionBuilder();
        ddb.init(this.pdfPrintParams.paperOrientation, this.pdfPrintParams.paperSize, columns, rowSpanNumber);

        // start create doc definition
        return PDFImageCache
                .preloadImages(this._imagesMap(pdfConfig)) // preload needed images
                .then(
                        _.bind(this._processCreateDocDefinition, this, ddb) // construct doc definition
                )
                .then(function()
                      {
                          // return object for generate pdf
                          return {
                              "docDefinition": ddb.getDocDefinitionObject(),
                              "tableLayouts": ddb.getTableLayouts()
                          }
                      });
    },

    _imagesMap: function()
    {
        return {
            "logo": {
                "url": REPORT_LOGO_PATH
            },
            "filterIcon": {
                "url": atDefImgPath('filter-for-pdf.png'),
                "saveAlpha": true
            }
        };
    },

    _processCreateDocDefinition: function(ddb)
    {
        ddb.initDocDefinitionObject(this.pdfPrintParams.showSignatureOnEachPage && this.pdfPrintParams.showSignature, this.pdfPrintParams.showHeaderOnEachPage);

        var headerData = this._createHeaderData();
        if (this.pdfPrintParams.paperOrientation == PaperOrientation.PORTRAIT)
        {
            ddb.addPortraitHeader(this.pdfPrintParams.showHeaderOnEachPage, this.pdfPrintParams.showLogoInHeader, headerData);
        }
        else
        {
            ddb.addLandscapeHeader(this.pdfPrintParams.showHeaderOnEachPage, this.pdfPrintParams.showLogoInHeader, headerData)
        }


        this._addTableData(ddb);
    },

    _createHeaderData: function()
    {
        var selectedParameters = this.pdfConfig.reportParams;
        var assignedInfo = this.pdfConfig.assignedInfo;

        return {
            informationTexts: {
                title: this.parameterFormatter.formatChartPDFReportTitle(selectedParameters),
                dateRange: this.parameterFormatter.formatChartPDFHeaderDateInformation(selectedParameters)
            },
            filterTexts: this.parameterFormatter.formatChartPDFFilterInformation(selectedParameters, assignedInfo),
            logo: PDFImageCache.getImage('logo')
        }
    },

    _addTableData: function(ddb)
    {
        new LeavesReportPDFGenerator.PDFDefinitionCreator.DataTableCreateDefinitionHelper(ddb, this.pdfData.tableData, {
            dateFormat: this.dateFormat
        }).createTable();
    }

});
