EVAReport.PDFParametersFormattingStrategy = function() {};

EVAReport.PDFParametersFormattingStrategy.prototype = _.create(Object.prototype, {
    init: function(config)
    {
        this.productSettingsValidator = config.productSettingsValidator;
        this.reportRightsValidator = config.reportRightsValidator;
        this.dateParametersFormattingStrategy = this.createDateParametersFormattingStrategy(config.dateFormat, config.dateFormatForFile);
        this.reportParamsStringBuilder = new ReportParamsStringBuilder().init(
                {
                    users: config.users,
                    cpList: config.cpList
                },
                config.dateFormat
        );
        
        this.shortDateFormat = new DateFormat(SystemSettings.dateFormat);
        this.parametersFormatter = new EVAReport.ParametersFormatter().init(config, config.dateFormat);
        return this;
    },

    createDateParametersFormattingStrategy: function(dateFormat, dateFormatForFile)
    {
        return new ChartPDFSingleDateFormattingStrategy().init(dateFormat, dateFormatForFile);
    },

    formatChartPDFFilterInformation: function(selectedParameters, assignedInfo)
    {
        return {
            firstRow: this.formatCPSelection(selectedParameters),
            secondRow: this.formatAdditionalInfo(selectedParameters)
        }
    },

    formatCPSelection: function(selectedParameters)
    {
        return this.parametersFormatter.formatCustomersAndProjectsSelection(selectedParameters.customersProjectsSelection);
    },

    formatAdditionalInfo: function(selectedParameters)
    {
        return this.parametersFormatter.formatTaskSelectionParamsString(selectedParameters.tasksConfig,
                                                                        selectedParameters.includeNotEstimated,
                                                                        selectedParameters.includeNotStarted,
                                                                        {dateSelection: selectedParameters.dateSelection, isParsed: true});
    },

    formatChartPDFHeaderDateInformation: function(selectedParameters)
    {
        return this.dateParametersFormattingStrategy.formatDateInformation(selectedParameters);
    },

    formatChartPDFFileName: function(selectedParameters)
    {
        var dateRange = selectedParameters.dateSelection;
        var formattedDateRange = "(" + this.shortDateFormat.formatDate(dateRange.date) + ")";

        var fileNameParts = [];
        fileNameParts.push(messageResource.getMessage('estimate_vs_actual_report.pdf.title'));
        fileNameParts.push(formattedDateRange);

        return messageResource.getMessage('common.leave.reports.pdf.file_name', fileNameParts);
    },

    _makePDFTitle: function(andSubstitutionKey)
    {
        return messageResource.getMessage('estimate_vs_actual_report.pdf.title');
    },

    formatChartPDFHeaderTitle: function(selectedParameters)
    {
        return messageResource.getMessage('common.leave.reports.pdf.header.report', [this._makePDFTitle()]);
    },

    formatChartPDFReportTitle: function(selectedParameters)
    {
        var showBy = BaseReport.FormatUtils.prototype.formatGroupingLevel(selectedParameters.firstGroupingLevel);
        return messageResource.getMessage('common.leave.reports.pdf.header.report_by', [this._makePDFTitle(), showBy]);
    }
});
