LeavesReportPDFGenerator.PDFDefinitionColumnBuilder = function() {};

LeavesReportPDFGenerator.PDFDefinitionColumnBuilder.prototype = _.create(PDFDefinitionColumnBuilder.prototype, {
    reportGroupingValues: ["reportFirstGroupingValue", "reportThirdGroupingValue"],

    init: function(config, rowSpanNumber)
    {
        PDFDefinitionColumnBuilder.prototype.init.call(this, config, rowSpanNumber ? rowSpanNumber : 2);
        return this;
    },

    pushWidth: function(destination, sizes) {},
    makeHeader: function() {},

    makeRow: function(lvl, data)
    {
        return this._makeRow(this.reportGroupingValues[lvl], data);
    },

    makeSubHeader: function() {return ''},
    makeThirdLvlHeader: function() {return ''},

    makeFirstRow: function(data) {return this.makeRow(0, data)},
    makeSecondRow: function(data) {return this.makeRow(1, data)}

});

LeavesReportPDFGenerator.PDFDefinitionColumnBuilder.Levels = function() {};

LeavesReportPDFGenerator.PDFDefinitionColumnBuilder.Levels.prototype = _.create(LeavesReportPDFGenerator.PDFDefinitionColumnBuilder.prototype, {
    init: function(config, leavesNumber, hasLeaves, hasPto, hasSick, store)
    {
        LeavesReportPDFGenerator.PDFDefinitionColumnBuilder.prototype.init.call(this, config);

        this.store = store;
        this.groupingLevels = [config.firstGroupingLevel, config.secondGroupingLevel];
        this.reportGroupingStyles = [
            ["reportFirstGroupingRow", "reportFirstGroupingRowMargin"],
            ["reportThirdGroupingRow", "reportSecondGroupingRowMargin"]];

        this.leavesNumber = leavesNumber;
        this.hasLeaves = hasLeaves;
        this.hasPto = hasPto;
        this.hasSick = hasSick;
        this.maxLen = 14;

        return this;
    },

    _getTitle: function(groupingLevel, identity, maxLen)
    {
        switch (groupingLevel)
        {
            case ReportGroupingLevel.DATE:
                return LeavesReport.FormatUtils.formatDateForDataRow(identity.date);
            case ReportGroupingLevel.WEEK:
                return LeavesReport.FormatUtils.formatWeekForDataRow(identity);
            case ReportGroupingLevel.MONTH:
                return LeavesReport.FormatUtils.formatMonthForDataRow(identity.date);
            case ReportGroupingLevel.USER:
                return LeavesReport.FormatUtils.formatUserAndTrim(this.store.getUser(identity), maxLen);
            case ReportGroupingLevel.USER_GROUP:
                return LeavesReport.FormatUtils.formatUserGroup(this.store.getUserGroup(identity));
            case ReportGroupingLevel.TIME_ZONE_GROUP:
                return LeavesReport.FormatUtils.formatTimeZoneGroup(this.store.getTimeZoneGroup(identity));
        }
    },

    _makeRow: function(style, groupingLevel, identity)
    {
        var text = this._getTitle(groupingLevel, identity, this.maxLen);
        return this._makeRowDefinition(style, text);
    },

    pushWidth: function(destination, sizes)
    {
        var baseWidth = sizes["levels"];
        var width = 100 - (this.hasLeaves ? sizes["totals"] : 0);
        width -= this.leavesNumber * sizes["leave"];
        width -= this.hasPto ? 2 * sizes["balance"] : 0;
        width -= this.hasSick ? 2 * sizes["balance"] : 0;

        if (width > baseWidth)
            baseWidth = width;

        destination.push(baseWidth + '%');
    },

    makeHeader: function()
    {
        var firstLevelName = LeavesReport.FormatUtils.formatGroupingLevel(this.groupingLevels[0]);
        var secondLevelName = ( this.groupingLevels[1] == ReportGroupingLevel.NONE) ? "" : ' / ' + LeavesReport.FormatUtils.formatGroupingLevel(
                this.groupingLevels[1]);
        return this._makeHeaderDefinition(firstLevelName + secondLevelName, [this.tableHeaderStyleNoMargin, "reportLevelsTitle"]);
    },

    makeTotals: function()
    {
        return this._makeRowDefinition("reportTotal", messageResource.getMessage("report.total"));
    },

    makeRow: function(lvl, data)
    {
        return this._makeRow(this.reportGroupingStyles[lvl], this.groupingLevels[lvl], data.identity);
    }

});

LeavesReportPDFGenerator.PDFDefinitionColumnBuilder.Totals = function() {};

LeavesReportPDFGenerator.PDFDefinitionColumnBuilder.Totals.prototype = _.create(PDFDefinitionColumnBuilder.Totals.prototype, {
    reportGroupingValues: ["reportFirstGroupingValue", "reportThirdGroupingValue"],

    init: function(config)
    {
        PDFDefinitionColumnBuilder.prototype.init.call(this, config, 2);

        this.onlyLeaves = (config.columnsConfig == LeaveTypeColumnConfig.LEAVE_TIME_ONLY);
        return this;
    },

    makeHeader: function()
    {
        var text = messageResource.getMessage("common.leave.reports.pdf.total_leave_time") +
                   "\n(" + LeavesReport.FormatUtils.formatTimeFormat(this.timeFormat) + ")";

        return this.onlyLeaves ? this._makeHeaderDefinition(text) : this._makeHeaderDefinition(text, [this.tableHeaderStyleNoMargin, "reportTotalTitle"]);
    },

    _getValue: function(data)
    {
        return data.total;
    }
});

LeavesReportPDFGenerator.PDFDefinitionColumnBuilder.Leave = function() {};

LeavesReportPDFGenerator.PDFDefinitionColumnBuilder.Leave.prototype = _.create(LeavesReportPDFGenerator.PDFDefinitionColumnBuilder.prototype, {
    init: function(config, leaveTypeId, leaveTypeIndex)
    {
        LeavesReportPDFGenerator.PDFDefinitionColumnBuilder.prototype.init.call(this, config);

        this.leaveTypeId = leaveTypeId;
        this.leaveTypeIndex = leaveTypeIndex;
        this.leaveMaxLen = 18;

        this.onlyLeaves = (config.columnsConfig == LeaveTypeColumnConfig.LEAVE_TIME_ONLY);

        return this;
    },

    pushWidth: function(destination, sizes)
    {
        destination.push(sizes["leave"] + '%');
    },

    makeHeader: function()
    {
        var leaveType = LeaveTypeStore.getLeaveType(this.leaveTypeId);
        var style = this.onlyLeaves ? this.tableHeaderStyle : [this.tableHeaderStyleNoMargin, "reportLeaveTitle"];

        return {
            noWrap: true,
            style: style,
            stack: [
                {width: 9, image: PDFImageCache.getImage("leaveTypeIcon_" + leaveType["iconId"])["image"]},
                {text: "\n" + StringUtils.cutLongText(leaveType.name, this.leaveMaxLen)}],
            rowSpan: 2
        };
    },

    _makeRow: function(style, data)
    {
        return this._makeRowDefinition(style, LeavesReport.FormatUtils.formatTime(data.leaveTypeTotals[this.leaveTypeIndex], this.timeFormat));
    }
});

LeavesReportPDFGenerator.PDFDefinitionColumnBuilder.Balance = function() {};

LeavesReportPDFGenerator.PDFDefinitionColumnBuilder.Balance.prototype = _.create(LeavesReportPDFGenerator.PDFDefinitionColumnBuilder.prototype, {
    init: function(config, type, defaultWorkDayDuration)
    {
        LeavesReportPDFGenerator.PDFDefinitionColumnBuilder.prototype.init.call(this, config);

        this.type = type;
        this.defaultWorkDayDuration = defaultWorkDayDuration;
        var dateRange = config.dateRangeSelection;

        this.templateModel = LeavesReport.FormatUtils.makeBalanceTemplateModel(
                this.type, config.firstGroupingLevel, config.secondGroupingLevel, dateRange.fromDate, dateRange.toDate);

        return this;
    },

    pushWidth: function(destination, sizes)
    {
        var size = sizes["balance"] + '%';
        destination.push(size);
        destination.push(size);
    },

    makeHeader: function()
    {
        return [{
            noWrap: true,
            style: this.tableHeaderStyle,
            text: this.templateModel.title,
            colSpan: 2
        }, {}];
    },

    makeSubHeader: function()
    {
        var firstColumnText = [
            {text: this.templateModel.startDate},
            {text: this.templateModel.startDateDetails, style: this.smallText},
            {text: " (" + this.templateModel.unit + ")", style: this.smallText}
        ];

        var secondColumnText = [
            {text: this.templateModel.endDate},
            {text: this.templateModel.endDateDetails, style: this.smallText},
            {text: " (" + this.templateModel.unit + ")", style: this.smallText}
        ];

        return [
            {
                noWrap: true,
                style: this.tableHeaderStyle,
                stack: firstColumnText
            },
            {
                noWrap: true,
                style: this.tableHeaderStyle,
                text: secondColumnText
            }
        ];
    },

    getBalance: function(data)
    {
        return ( this.type == BalanceType.PTO ) ? data.ptoBalances : data.sickBalances;
    },

    _makeRow: function(style, data)
    {
        var balance = this.getBalance(data);
        var startStyle = (balance.start < 0 ) ? [style, this.negativeValue] : style;
        var endStyle = (balance.end < 0 ) ? [style, this.negativeValue] : style;
        return [
            {
                noWrap: true,
                style: startStyle,
                text: LeavesReport.FormatUtils.formatBalance(balance.start, this.type, this.defaultWorkDayDuration)
            },
            {
                noWrap: true,
                style: endStyle,
                text: LeavesReport.FormatUtils.formatBalance(balance.end, this.type, this.defaultWorkDayDuration)
            }
        ];
    }
});