LeavesReportPDFGenerator.PDFGenerator = function(){};

LeavesReportPDFGenerator.PDFGenerator.prototype = _.create(Object.prototype, {
    init: function(store, generalStore, productSettingsValidator, reportRightsValidator)
    {
        this.docDefinitionCreator = new LeavesReportPDFGenerator.PDFDefinitionCreator();
        this.onGenerateBreaker = {needBreak: false};
        this.store = store;
        this.generalStore = generalStore;
        this.productSettingsValidator = productSettingsValidator;
        this.reportRightsValidator = reportRightsValidator;

        return this;
    },

    interruptGeneration: function()
    {
        this.onGenerateBreaker.needBreak = true;
    },
    
    _getCreatePDFDefinitionStrategy: function()
    {
        return null;
    },

    _getParametersFormattingStrategy: function(config)
    {
        var dateFormatForFileName = new DateFormat('yyyy-MM-dd');
        return (new LeavesReportPDFGenerator.PDFParametersFormattingStrategy()).init(
                {
                    users: this.generalStore.getVisibleUsers(),
                    userGroups: this.generalStore.getUserGroups(),
                    timeZoneGroups: this.generalStore.getTimeZoneGroups(),
                    leaveTypes: LeaveTypeStore.getLeaveTypes(),
                    dateFormat: config.dateFormat, 
                    dateFormatForFile: dateFormatForFileName,
                    productSettingsValidator : this.productSettingsValidator,
                    reportRightsValidator : this.reportRightsValidator
                }
        );
    },

    calculateTotalNumberOfPages: function(params)
    {
        return 0;
    },

    generatePDF: function(additionalDataProvider)
    {
        var newBreaker = {
            needBreak: false
        };
        this.onGenerateBreaker = newBreaker;

        var pdfPrintParameters = this.store.getPrintParametersForSave();
        var chartSectionConfig = this.store.getChartSectionConfig();
        var reportParams = this.store.getReportParams();
        
        var showAssignedOnly = this.store.isShowAssignedOnlyEnabled() && this.store.isShowAssignedOnly();
        var viewerInfo = this.store.getViewerInfo();
        var assignedInfo = {isShowOnlyAssigned: showAssignedOnly, viewerInfo : viewerInfo};

        reportParams.showAssignedOnly = showAssignedOnly;
        
        var reportData = this.store.getReportData();
        var defaultWorkDayDuration = this.store.getDefaultWorkDayDuration();
        
        var totals = {
            totalValue: reportData.totalLeaveSpent,
            totalsMap: _.object(_.map(reportData.leaveSpent, function(item) {
                return [item.leaveTypeId, item.leaveSpent];
            })),
            ptoBalance: reportData.ptoBalance,
            sickBalance: reportData.sickBalance
        };
        var rows = new LeavesReport.TableDataConverter(reportData, reportParams).convert();
        var leaveTypeIds = _.pluck(reportData["leaveSpent"], "leaveTypeId");

        var pdfConfig = {
            pdfPrintParameters: pdfPrintParameters,
            chartSectionConfig: chartSectionConfig,
            pdfData: {
                tableData: rows,
                leaveTypeIds: leaveTypeIds,
                chartImage: null,
                totals: totals,
                defaultWorkDayDuration: defaultWorkDayDuration
            },
            reportParams: reportParams,
            assignedInfo: assignedInfo,
            dateFormat: new DateFormat(SystemSettings.dateFormat)
        };

        var s = this;
        return new Promise(
                function(onComplete)
                {
                    if (s.isNeedRetrieveAdditionalData(pdfConfig))
                    {
                        additionalDataProvider(
                                s.getAdditionalDataConfig(pdfConfig),
                                function(additionalData)
                                {
                                    s.updateConfigWithAdditionalData(pdfConfig, additionalData);
                                    onComplete(pdfConfig);
                                })
                    }
                    else
                    {
                        onComplete(pdfConfig);
                    }
                }
        ).then(
                function (pdfConfig) {
                    return new Promise(function (onComplete, onError){
                        defer(_.bind(onComplete, this, pdfConfig));
                    });
                }
        ).then(
                function(pdfConfig)
                {
                    if (newBreaker["needBreak"])
                    {
                        return Promise.reject(new Error("abort"));
                    }
                    return s._generatePDF(pdfConfig, newBreaker);
                }
        );
    },

    isNeedRetrieveAdditionalData: function(pdfConfig)
    {
        return this.store.getChartSectionConfig()["showChart"];
    },

    getAdditionalDataConfig: function(pdfConfig)
    {
        return {
            chartImageSizes: this.docDefinitionCreator.getChartImageSizes(pdfConfig.pdfPrintParameters)
        }
    },

    updateConfigWithAdditionalData: function(pdfConfig, additionalData)
    {
        pdfConfig.pdfData.chartImage = additionalData["chartImage"];
    },

    _generatePDF: function(config, breaker)
    {
        var paramFormattingStrategy = this._getParametersFormattingStrategy(config);

        return this.docDefinitionCreator
                .createDocDefinition(config, paramFormattingStrategy, this.store)
                .then(
                        function(ddObject)
                        {
                            pdfMake.fonts = PDFFonts;
                            var doc = pdfMake.createPdf(ddObject["docDefinition"]);
                            var currentFileName = paramFormattingStrategy.formatChartPDFFileName(config.reportParams);

                            return new Promise(function(onComplete, onError)
                            {
                                try
                                {
                                    doc._createDoc(
                                            {
                                                tableLayouts: ddObject["tableLayouts"]
                                            },
                                            function(result, pages)
                                            {
                                                if (breaker.needBreak)
                                                {
                                                    return onError(new Error("abort"));
                                                }

                                                onComplete({
                                                               encodedPDF: result,
                                                               fileName: currentFileName,
                                                               pageCount: pages.length
                                                           })
                                            }
                                    );

                                }
                                catch (e)
                                {
                                    onError(e);
                                }
                            });
                        }
                );
    }
});