LeavesReportPDFGenerator.PDFParametersFormattingStrategy = function() {};

LeavesReportPDFGenerator.PDFParametersFormattingStrategy.prototype = _.create(Object.prototype, {
    init: function(config)
    {
        this.productSettingsValidator = config.productSettingsValidator;
        this.reportRightsValidator = config.reportRightsValidator;
        this.dateParametersFormattingStrategy = this.createDateParametersFormattingStrategy(config.dateFormat, config.dateFormatForFile);
        this.reportParamsStringBuilder = new ReportParamsStringBuilder().init(
                {
                    users: config.users,
                    userGroupsFeatureEnabled: !_.isNull(config.userGroups) && this.productSettingsValidator.isUGREnabled(),
                    tzGroupsFeatureEnabled: !_.isNull(config.timeZoneGroups)&& this.productSettingsValidator.isTZGEnabled(),
                    userGroups: config.userGroups,
                    timeZoneGroups: config.timeZoneGroups,
                    leaveTypesMap: config.leaveTypes
                },
                config.dateFormat
        );
        
        this.shortDateFormat = new DateFormat(SystemSettings.dateFormat);
        return this;
    },

    createDateParametersFormattingStrategy: function(dateFormat, dateFormatForFile)
    {
        return new ChartPDFDateRangeFormattingStrategy().init(dateFormat, dateFormatForFile);
    },

    formatChartPDFFilterInformation: function(selectedParameters, assignedInfo)
    {
        return {
            firstRow: this.formatUsersSelection(selectedParameters.userSelection, assignedInfo),
            secondRow: this.formatLeaveTypesSelection(selectedParameters.leaveTypeSelection)
        }
    },

    formatUsersSelection: function(userSelection, assignedInfo)
    {
        return this.reportParamsStringBuilder.getUserSelectionParamString(userSelection, assignedInfo);
    },

    formatLeaveTypesSelection: function(leaveTypeSelection)
    {
        return this.reportParamsStringBuilder.getLeaveTypeSelectionParamString(leaveTypeSelection);
    },

    formatChartPDFHeaderDateInformation: function(selectedParameters)
    {
        return this.dateParametersFormattingStrategy.formatDateInformation(selectedParameters);
    },

    formatChartPDFFileName: function(selectedParameters)
    {
        var dateRange = selectedParameters.dateRangeSelection;
        var formattedDateRange = "(" + this.shortDateFormat.formatDate(dateRange.fromDate) + " - " + this.shortDateFormat.formatDate(dateRange.toDate) + ")";

        var fileNameParts = [];
        fileNameParts.push(this._makePDFTitle('common.leave.reports.pdf.header.and'));
        fileNameParts.push(formattedDateRange);

        return messageResource.getMessage('common.leave.reports.pdf.file_name', fileNameParts);
    },

    _makePDFTitle: function(andSubstitutionKey)
    {
        andSubstitutionKey = _.isUndefined(andSubstitutionKey) ? 'common.leave.reports.pdf.header.amp' : andSubstitutionKey;
        var andMessage = messageResource.getMessage(andSubstitutionKey);
        
        var enabledPTO =  this.productSettingsValidator.isPTOEnabled();
        var enabledSick =  this.productSettingsValidator.isSickEnabled();
        
        if (!this.reportRightsValidator.isCanViewBalances() || (!enabledPTO && !enabledSick))
            return messageResource.getMessage('common.leave.reports.pdf.header.leave_time');

        var balanceMessage = andMessage + " " + this.reportParamsStringBuilder.getBalancesParamString(enabledPTO, enabledSick);
        return messageResource.getMessage('common.leave.reports.pdf.header.leave_time_and_balances', [balanceMessage]);
    },

    formatChartPDFHeaderTitle: function(selectedParameters)
    {
        return messageResource.getMessage('common.leave.reports.pdf.header.report', [this._makePDFTitle()]);
    },

    formatChartPDFReportTitle: function(selectedParameters)
    {
        var showBy = LeavesReport.FormatUtils.formatGroupingLevel(selectedParameters.firstGroupingLevel);
        return messageResource.getMessage('common.leave.reports.pdf.header.report_by', [this._makePDFTitle(), showBy]);
    }
});
