OvertimeReport.PDFDefinitionColumnBuilder = function() {};

OvertimeReport.PDFDefinitionColumnBuilder.prototype = _.create(PDFDefinitionColumnBuilder.prototype, {
    init: function(config, rowSpanNumber)
    {
        PDFDefinitionColumnBuilder.prototype.init.call(this, config, rowSpanNumber);
        this.reportGroupingValues = ["reportFirstGroupingValue", "reportThirdGroupingValue"];
        
        return this;
    }
});


OvertimeReport.PDFDefinitionColumnBuilder.Levels = function() {};
OvertimeReport.PDFDefinitionColumnBuilder.Levels.prototype = _.create(PDFDefinitionColumnBuilder.Levels.prototype, {
    init: function(config, store, rowSpanNumber)
    {
        OvertimeReport.PDFDefinitionColumnBuilder.prototype.init.call(this, config, rowSpanNumber);

        this.store = store;
        this.groupingLevels = [config.firstGroupingLevel, config.secondGroupingLevel];
        this.reportGroupingStyles = ["reportFirstGroupingRow", "reportThirdGroupingRow"];
        this.reportGroupingMargins = ["reportFirstGroupingRowMargin", "reportSecondGroupingRowMargin"];

        this.isWide = config.columnsConfig != OvertimeReport.ColumnsConfig.TIME_BALANCE_AND_OVERTIME;
        this.maxUserNameLen = this.isWide ? "maxUserNameLenWide" : "maxUserNameLen";

        return this;
    },

    _makeCustomRow: function(style, groupingLevel, identity)
    {
        var text = this._getTitle(groupingLevel, identity, this.params[this.maxUserNameLen]);
        return this._makeRowDefinition(style, text);
    },

    pushWidth: function(destination, sizes)
    {
        var baseWidth = this.isWide ? sizes["levelsWide"] : sizes["levels"];
        destination.push(baseWidth + '%');
    },

    makeHeader: function()
    {
        var names = [];
        for (var i = 0; i < 2; i++)
        {
            if (this.groupingLevels[i] != ReportGroupingLevel.NONE)
            {
                names.push(this.formatUtil.formatGroupingLevel(this.groupingLevels[i]));
            }
        }
        return this._makeHeaderDefinition(names.join(' / '), [this.tableHeaderStyleNoMargin, "reportLevelsTitle"]);
    },

    makeTotals: function()
    {
        return this._makeRowDefinition("reportTotal", messageResource.getMessage("report.total"));
    },

    makeRow: function(lvl, data)
    {
        return this._makeCustomRow([this.reportGroupingStyles[lvl], this.reportGroupingMargins[lvl]], this.groupingLevels[lvl], data.id);
    }

});


OvertimeReport.PDFDefinitionColumnBuilder.TimeBalance = function() {};
OvertimeReport.PDFDefinitionColumnBuilder.TimeBalance.prototype = _.create(OvertimeReport.PDFDefinitionColumnBuilder.prototype, {

    init: function(config, store, rowSpanNumber)
    {
        OvertimeReport.PDFDefinitionColumnBuilder.prototype.init.call(this, config, rowSpanNumber);
        this.sizeName = (config.columnsConfig == OvertimeReport.ColumnsConfig.TIME_BALANCE_ONLY)  ? "timeBalanceWide" : "timeBalance";

        return this;
    },
    
    pushWidth: function(destination, sizes)
    {
        destination.push(sizes[this.sizeName] + '%');
        destination.push(sizes[this.sizeName] + '%');
        destination.push(sizes[this.sizeName] + '%');
    },

    makeHeader: function()
    {
        var varianceTxt = messageResource.getMessage("report.overtime.time_balance");

        return [{
            noWrap: true,
            style: this.tableHeaderStyle,
            text: varianceTxt,
            colSpan: 3
        }, {}, {}];
    },
    
    makeSubHeader: function()
    {
        return [
            {
                noWrap: true,
                style: this.tableHeaderStyle,
                text: messageResource.getMessage("report.overtime.time_balance.scheduled")
            },
            {
                noWrap: true,
                style: this.tableHeaderStyle,
                text: messageResource.getMessage("report.overtime.time_balance.reported")
            },
            {
                noWrap: true,
                style: this.tableHeaderStyle,
                text: messageResource.getMessage("report.overtime.time_balance.difference")
            }
        ];
    },

    _makeRow: function(style, data, valueMask)
    {
        return [
            this.makeTimeCell(data.scheduledTime, style),
            this.makeTimeCell(data.reportedTime, style),
            this.makeSignedTimeCell(data.differenceTime, style)
        ];
    }
});

OvertimeReport.PDFDefinitionColumnBuilder.Overtime = function() {};
OvertimeReport.PDFDefinitionColumnBuilder.Overtime.prototype = _.create(OvertimeReport.PDFDefinitionColumnBuilder.prototype, {

    init: function(config, store, rowSpanNumber)
    {
        OvertimeReport.PDFDefinitionColumnBuilder.prototype.init.call(this, config, rowSpanNumber);
        this.sizeName = (config.columnsConfig == OvertimeReport.ColumnsConfig.OVERTIME_ONLY) ? "overtimeWide" : "overtime";

        return this;
    },
    
    pushWidth: function(destination, sizes)
    {
        destination.push(sizes[this.sizeName] + '%');
        destination.push(sizes[this.sizeName] + '%');
    },

    makeHeader: function()
    {
        var varianceTxt = messageResource.getMessage("report.overtime.overtime");

        return [{
            noWrap: true,
            style: this.tableHeaderStyle,
            text: varianceTxt,
            colSpan: 2
        }, {}];
    },

    makeSubHeader: function()
    {
        return [
            {
                noWrap: true,
                style: this.tableHeaderStyle,
                columns: this.addAsterisk(messageResource.getMessage("report.overtime.overtime.reported_by_user"), '*')
            },
            {
                noWrap: true,
                style: this.tableHeaderStyle,
                columns: this.addAsterisk(messageResource.getMessage("report.overtime.overtime.auto_calculated"), '**')
            }
        ];
    },

    _makeRow: function(style, data, valueMask)
    {
        return [
            this.makeTimeCell(data.reportedOvertime, style),
            this.makeTimeCell(data.autoOvertime, style)
        ];
    }
});