OvertimeReport.PDFParametersFormattingStrategy = function() {};

OvertimeReport.PDFParametersFormattingStrategy.prototype = _.create(Object.prototype, {
    init: function(config)
    {
        this.dateParametersFormattingStrategy = this.createDateParametersFormattingStrategy(config.dateFormat, config.dateFormatForFile);
        this.shortDateFormat = new DateFormat(SystemSettings.dateFormat);
        this.parametersFormatter = new OvertimeReport.ParametersFormatter().init(config, config.dateFormat);
        
        return this;
    },

    createDateParametersFormattingStrategy: function(dateFormat, dateFormatForFile)
    {
        return new ChartPDFDateRangeFormattingStrategy().init(dateFormat, dateFormatForFile);
    },

    formatChartPDFFilterInformation: function(selectedParameters)
    {
        return {
            firstRow: this.formatUsersSelection(selectedParameters)
        }
    },

    formatUsersSelection: function(selectedParameters)
    {
        return this.parametersFormatter.formatUsersSelection(selectedParameters.userSelection);
    },

    formatChartPDFHeaderDateInformation: function(selectedParameters)
    {
        return this.dateParametersFormattingStrategy.formatDateInformation(selectedParameters);
    },

    formatChartPDFFileName: function(selectedParameters)
    {
        var dateRange = selectedParameters.dateRangeSelection;
        var formattedDateRange = "(" + this.shortDateFormat.formatDate(dateRange.fromDate) + " - " + this.shortDateFormat.formatDate(dateRange.toDate) + ")";

        var fileNameParts = [];
        fileNameParts.push(this._makePDFTitle(selectedParameters, 'common_report.pdf.header.and'));
        fileNameParts.push(formattedDateRange);

        return messageResource.getMessage('common_report.pdf.file_name', fileNameParts);
    },

    _makePDFTitle: function(selectedParameters, andSubstitutionKey)
    {
        return OvertimeReport.FormatUtils.makePDFTitle(selectedParameters, andSubstitutionKey)
    },

    formatChartPDFHeaderTitle: function(selectedParameters)
    {
        return messageResource.getMessage('common_report.pdf.header.report', [this._makePDFTitle(selectedParameters)]);
    },

    formatChartPDFReportTitle: function(selectedParameters)
    {
        var showBy = BaseReport.FormatUtils.prototype.formatGroupingLevel(selectedParameters.firstGroupingLevel);
        return messageResource.getMessage('common_report.pdf.header.report_by', [this._makePDFTitle(selectedParameters), showBy]);
    }
});
