// TODO: make this component markup independent from lightbox at some point
function PDFSettingsPreview($containerEl, lightboxId)
{
    TemplateView.call(this, $containerEl);
    this.lightboxId = lightboxId;
}

PDFSettingsPreview.prototype = _.create(TemplateView.prototype, {
    html: "<div class='pdfPreviewModeElement pdfPreviewInnerContent'>" +
          "  <table cellpadding='0' cellspacing='0'>" +
          "    <tr>" +
          "      <td class='pdfParametersSectionTd'>" +
          "        <div id='<%=id%>_pdfParametersSection' class='pdfParametersSection'>" +
          "          <div class='pageSetupSection'>" +
          "            <div class='parametersSectionHeader'><%=pageSetupLabel%></div>" +
          "            <div>" +
          "              <div class='selectorLabel'><%=paperSizeLabel%></div>" +
          "              <div class='selectorContainer paperSizeSelectorCt' id='<%=id%>_paperSizeSelector'></div>" +
          "            </div>" +
          "            <div class='pageOrientationSelector'>" +
          "              <div class='selectorLabel'><%=orientationLabel%></div>" +
          "              <div class='selectorContainer paperOrientationSelectorCt'>" +
          "                <table>" +
          "                  <tr>" +
          "                    <td><input type='radio' name='<%=id%>_pageOrientation' id='<%=id%>_portrait'/></td>" +
          "                    <td><label for='<%=id%>_portrait'><%=orientationPortraitLabel%></label></td>" +
          "                  </tr>" +
          "                  <tr>" +
          "                    <td><input type='radio' name='<%=id%>_pageOrientation' id='<%=id%>_landscape'/></td>" +
          "                    <td><label for='<%=id%>_landscape'><%=orientationLandscapeLabel%></label></td>" +
          "                  </tr>" +
          "                </table>" +
          "              </div>" +
          "            </div>" +
          "            <div class='showHeaderOnEachPage'>" +
          "              <table>" +
          "                <tr>" +
          "                  <td><input type='checkbox' id='<%=id%>_headerOnEachPage'/></td>" +
          "                  <td><label for='<%=id%>_headerOnEachPage'><%=showHeaderLabel%></label></td>" +
          "                </tr>" +
          "              </table>" +
          "            </div>" +
          "            <div class='showLogoInHeader'>" +
          "              <table>" +
          "                <tr>" +
          "                  <td><input type='checkbox' id='<%=id%>_logoInHeader'/></td>" +
          "                  <td><label for='<%=id%>_logoInHeader'><%=showLogoLabel%></label></td>" +
          "                </tr>" +
          "              </table>" +
          "            </div>" +
          "          </div>" +
          "          <div class='delimiter'></div>" +
          "          <div class='customSettingsSectionContainer'></div>" +
          "          <div class='bottomSection'>" +
          "            <div class='delimiter'></div>" +
          "            <span id='<%=id%>_numberOfPages' class='numberOfPages'>" +
          "              <%=pagesNumberLabel%>" +
          "              <span id='<%=id%>_numberOfPagesContainer'></span>" +
          "            </span>" +
          "            <div id='<%=id%>_updatePreviewBtn' class='updatePreviewButton disabled'><%=updatePreviewButtonLabel%></div>" +
          "          </div>" +
          "        </div>" +
          "      </td>" +
          "      <td class='pdfPreviewTd'>" +
          "        <div id='pdfPreviewSectionId' class='pdfPreviewSection'></div>" +
          "        <div id='pdfPreviewSectionMoireId' class='pdfPreviewSectionMoire'>" +
          "          <div class='centeredContent'>" +
          "            <div class='previewSpinnerPlaceholder' id='previewChartPDFSpinner'></div>" +
          "            <div class='previewLoadingMessage'><%=loadingPreviewLabel%></div>" +
          "          </div>" +
          "        </div>" +
          "      </td>" +
          "    </tr>" +
          "  </table>" +
          "</div>",

    templateKey: "PDFPreview",

    afterRender: function()
    {
        var self = this;

        this.parametersChanged = false;
        this.changesCallback = function()
        {
            self.handleParameterChange();
        };

        this.section = $(document.getElementById(this.lightboxId + "_pdfParametersSection"));

        this.initPaperSizeSelector();
        this.initPaperOrientationRadio();
        this.initHeaderCheckboxes();

        this.numberOfPagesContainer = $(document.getElementById(this.lightboxId + "_numberOfPagesContainer"));

        this.updatePreviewButton = $(document.getElementById(this.lightboxId + "_updatePreviewBtn"));
        this.updatePreviewButton.off();
        this.updatePreviewButton.hover(
                function() { $(this).addClass("hovered"); },
                function() { $(this).removeClass("hovered"); }
        );
        this.updatePreviewButton.click(function() { self.onUpdatePreviewButtonClicked() });

        this.subscribeToEvents();
    },

    update: function()
    {
        var printParameters = this.getStore().getPrintParametersForSave();

        this.paperSizeSelectorContainer.innerHTML = "";
        this.paperSizeSelector = new ListSelector({
            menuCls: "paperSizeSelectorMenu",
            renderTo: this.paperSizeSelectorContainer,
            value: printParameters.paperSize,
            listItems: [
                {
                    text: messageResource.getMessage("report.chart_pdf_preview.paper_size.letter"),
                    value: PaperSize.LETTER
                },
                {
                    text: messageResource.getMessage("report.chart_pdf_preview.paper_size.a4"),
                    value: PaperSize.A4
                }
            ]
        });
        this.paperSizeSelector.on("change", this.changesCallback);

        if (printParameters.paperOrientation == PaperOrientation.PORTRAIT)
            this.portraitOrientationRadio.checked = true;
        else
            this.landscapeOrientationRadio.checked = true;

        this.showHeaderOnEachPageCheckbox.checked = printParameters.showHeaderOnEachPage;
        this.showLogoInHeaderCheckbox.checked = printParameters.showLogoInHeader;
    },

    subscribeToEvents: function()
    {
        var $store = $(this.getStore());
        var self = this;
        $store.on("printParametersForSaveChange", function()
        {
            self.update();
        });
        $store.on("printNumberOfPagesChange", function()
        {
            self.updateNumberOfPagesInformation();
        });
    },

    unsubscribeFromEvents: function()
    {
        $(this.getStore()).off("printParametersForSaveChange").off("printNumberOfPagesChange");
    },

    templateModel: function()
    {
        return {id: this.lightboxId}
    },

    messageKeys: function()
    {
        return {
            "pageSetupLabel": "report.chart_pdf_preview.page_setup",
            "paperSizeLabel": "report.chart_pdf_preview.paper_size",
            "orientationLabel": "report.chart_pdf_preview.orientation",
            "orientationPortraitLabel": "report.chart_pdf_preview.orientation.portrait",
            "orientationLandscapeLabel": "report.chart_pdf_preview.orientation.landscape",
            "showHeaderLabel": "report.chart_pdf_preview.show_header_each_page",
            "showLogoLabel": "report.chart_pdf_preview.show_logo",
            "pagesNumberLabel": "report.chart_pdf_preview.pages_number",
            "updatePreviewButtonLabel": "report.chart_pdf_preview.updatePreviewButton.caption",
            "loadingPreviewLabel": "report.chart_pdf_preview.loading_preview"
        };
    },

    getStore: function()
    {
        // Abstract
    },

    getCustomSettingsSectionContainer: function()
    {
        return this.find(".customSettingsSectionContainer");
    },

    initPaperSizeSelector: function()
    {
        this.paperSizeSelectorContainer = document.getElementById(this.lightboxId + "_paperSizeSelector");
    },

    initPaperOrientationRadio: function()
    {
        this.portraitOrientationRadio = document.getElementById(this.lightboxId + "_portrait");
        this.landscapeOrientationRadio = document.getElementById(this.lightboxId + "_landscape");
        var radioGroupName = this.lightboxId + "_pageOrientation";
        $("input[type=radio][name=" + radioGroupName + "]").off().change(this.changesCallback);
    },

    initHeaderCheckboxes: function()
    {
        this.showHeaderOnEachPageCheckbox = document.getElementById(this.lightboxId + "_headerOnEachPage");
        this.showHeaderOnEachPageCheckbox.onchange = this.changesCallback;
        this.showLogoInHeaderCheckbox = document.getElementById(this.lightboxId + "_logoInHeader");
        this.showLogoInHeaderCheckbox.onchange = this.changesCallback;
    },

    handleParameterChange: function()
    {
        if (!this.parametersChanged)
        {
            this.parametersChanged = true;
            this.enableUpdatePreviewButton();
        }

        this.triggerIntermediatePrintParametersChange(this.collectPrintParametersForGenerate());
    },

    _getPaperOrientationParameter: function()
    {
        return this.portraitOrientationRadio.checked ? PaperOrientation.PORTRAIT : PaperOrientation.LANDSCAPE;
    },

    _getPaperSizeParameter: function()
    {
        return this.paperSizeSelector.getSelectedItem().value;
    },

    collectPrintParametersForGenerate: function()
    {
        return {
            paperSize: this._getPaperSizeParameter(),
            paperOrientation: this._getPaperOrientationParameter(),
            showHeaderOnEachPage: this.showHeaderOnEachPageCheckbox.checked,
            showLogoInHeader: this.showLogoInHeaderCheckbox.checked
        };
    },

    collectPrintParametersForSave: function()
    {
        return {
            paperSize: this._getPaperSizeParameter(),
            showHeaderOnEachPage: this.showHeaderOnEachPageCheckbox.checked,
            showLogoInHeader: this.showLogoInHeaderCheckbox.checked,
            paperOrientation: this._getPaperOrientationParameter()
        };
    },

    disableUpdatePreviewButton: function()
    {
        this.updatePreviewBtnEnabled = false;
        this.updatePreviewButton.addClass("disabled");
    },

    enableUpdatePreviewButton: function()
    {
        this.updatePreviewBtnEnabled = true;
        this.updatePreviewButton.removeClass("disabled");
    },

    onUpdatePreviewButtonClicked: function()
    {
        if (this.updatePreviewBtnEnabled)
        {
            this.unsubscribeFromEvents();
            this.triggerPrintParametersForSaveChange(this.collectPrintParametersForSave());
            this.subscribeToEvents();

            this.triggerPrintParametersForGenerateChange(this.collectPrintParametersForGenerate());
        }
    },

    triggerIntermediatePrintParametersChange: function(newPrintParameters)
    {
        // Abstract
    },

    triggerPrintParametersForSaveChange: function(printParametersForSave)
    {
        // Abstract
    },

    triggerPrintParametersForGenerateChange: function(printParametersForGenerate)
    {
        // Empty by default
    },

    beforePreviewUpdate: function()
    {
        this.disableUpdatePreviewButton();
        this.parametersChanged = false;
    },

    updateNumberOfPagesInformation: function()
    {
        this.numberOfPagesContainer.text(this.getStore().getPrintNumberOfPages());
    }
});