/**
 * Definitions used on the Prepare Invoice form.
 *
 * required files:
 * client_side_errors.js
 * FieldController.js
 * util.js
 * message_resources.js
 * at.js
 */

//var requiredFields = []; //variable was commented because assigned a value more than once TODO(petrov):check it

var emptyItemFieldTooltips;


var formElements;

var emptyInvoiceError;

// flag not to move "hovered" class from selected row when dropdown menu is displayed
var stopRowsSelection = false;

function addItemRow(focus)
{
    hideEmptyDetailsStub();
    addRow("itemsTableBody", "itemRowPrototype");
    applyToSingleOrLast(formElements.itemDescriptions, makeElastic);
    applyToSingleOrLast(formElements.itemNames, makeElastic);
    applyToSingleOrLast(formElements.itemTaxableFlags, makeDropdownListButton);
    applyToSingleOrLast(formElements.itemAmounts, function (field) {
        field.value = amountValidator.formatNumberWithDigitSeparator(0);
    });
    applyToSingleOrLast(formElements.itemAmounts, function (field) {
        addClass(field, "emptyAmount");
    });
    applyToSingleOrLast(formElements.itemDescriptions, function (field) {
        initGrayText(field, "report.prepare_invoice.js.item_description_hint")
    });
    applyToSingleOrLast(formElements.itemNames, function (field) {
        initGrayText(field, "report.prepare_invoice.js.item_name_hint")
    });
    if (focus) applyToSingleOrLast(formElements.itemDescriptions, setFocus);
}

function addDiscountRow(focus)
{
    addRow("discountsTableBody", "discountRowPrototype");
    applyToSingleOrLast(formElements.discountNames, makeElastic);
    applyToSingleOrLast(formElements.discountAmounts, function (field) {
        field.value = addBrackets(amountValidator.formatNumberWithDigitSeparator(0));
    });
    applyToSingleOrLast(formElements.discountAmounts, function (field) {
        addClass(field, "emptyAmount");
    });
    applyToSingleOrLast(formElements.discountNames, function (field) {
        initGrayText(field, "report.prepare_invoice.js.discount_name_hint")
    });
    if (focus) applyToSingleOrLast(formElements.discountNames, setFocus);
}

function addTaxRow(focus)
{
    addRow("taxesTableBody", "taxRowPrototype");
    applyToSingleOrLast(formElements.taxNames, makeElastic);
    applyToSingleOrLast(formElements.taxAmounts, function (field) {
        field.value = amountValidator.formatNumberWithDigitSeparator(0);
    });
    applyToSingleOrLast(formElements.taxAmounts, function (field) {
        addClass(field, "emptyAmount");
    });
    applyToSingleOrLast(formElements.taxNames, function (field) {
        initGrayText(field, "report.prepare_invoice.js.tax_name_hint")    
    });
    if (focus) applyToSingleOrLast(formElements.taxNames, setFocus);
}

function addRow(tableBodyId, rowPrototypeId)
{
    var clone = document.getElementById(rowPrototypeId).cloneNode(true);
    clone.id = null;
    document.getElementById(tableBodyId).appendChild(clone);
    forceDomUpdate();
}

function deleteItemRow(link)
{
    deleteRow(link);
    if (isNoItemRows()) {
        showEmptyDetailsStub();
    }
}

function deleteRow(link)
{
    var row = findElementRow(link);
    unregisterChildErrors(row);
    ClientSideErrors.hideClientSideErrorMessageIfNoErrors( true );
    row.parentNode.removeChild(row);
    calculateAll();
    registerChange();
    forceDomUpdate()
}

// fix bug 53453 (force dom update for slowpoke ie8)
function forceDomUpdate(){
    $('body').addClass("forcePaint").removeClass("forcePaint");
}

function addBrackets(str)
{
    return "(" + str + ")";
}

function removeBrackets(str)
{
    return str.replace("(", "").replace(")", "");
}

function showEmptyDetailsStub()
{
    document.getElementById("emptyItemRowStub").style.display = "";
}

function hideEmptyDetailsStub()
{
    document.getElementById("emptyItemRowStub").style.display = "none";
}

function isNoItemRows()
{
    return !formElements.itemDescriptions || (formElements.itemDescriptions.length == 0); 
}

function unregisterChildErrors(element)
{
    if (!element.childNodes) return;

    for (var child in element.childNodes) {
        if (element.childNodes[child].tagName == "INPUT" || element.childNodes[child].tagName == "TEXTAREA") {
            ClientSideErrors.markTextFieldAsValid(element.childNodes[child]);
        } else {
            unregisterChildErrors(element.childNodes[child]);
        }
    }
}

function onItemFieldChange(field)
{
    emptyInvoiceError.hide();
    registerChange();
}

function onItemRateChange(field)
{
    emptyInvoiceError.hide();
    validateRate(field);
    calculateAll();
    registerChange();
}

function onItemQuantityChange(field)
{
    emptyInvoiceError.hide();
    validateQuantity(field);
    calculateAll();
    registerChange();
}

function onItemTaxableFlagChange(field)
{
    calculateAll();
    registerChange();
}

function onDiscountValueChange(field)
{
    validateDiscountValue(field);
    calculateAll();
    registerChange();
}

function onTaxValueChange(field)
{
    validateTaxValue(field);
    calculateAll();
    registerChange();
}

function onTextareaChange()
{
    registerChange();
}

function onLongTextareaBlur(field, fieldName)
{
    validateFieldLength(field, 2000, fieldName, false);
    ClientSideErrors.hideClientSideErrorMessageIfNoErrors( true );
    return true;
}

function onShortTextareaBlur(field, fieldName)
{
    validateFieldLength(field, 255, fieldName, false);
}

function onBillToHistorySelect(select)
{
    if (select.value != "")
    {
        formElements.billTo.value = select.value;
        select.value = "";
        
        if (formElements.billTo.elastic)
            formElements.billTo.elastic.fitToHeight();

        registerChange();
    }
}

function submitForm( strictValidation, reportId )
{
    formElements.configId.value = reportId == null ? -1: reportId;
    var validationResult = validateAll( strictValidation );
    calculateAll();

    if (validationResult) {
        formElements.customerMessage.value
                = formElements.customerMessageField.controller.isEmpty() ? "" : formElements.customerMessageField.value;
        formElements.additionalInfo.value
                = formElements.additionalInfoField.controller.isEmpty() ? "" : formElements.additionalInfoField.value;

        //to avoid showing "modifications not saved" dialog after invoice is already created
        formModified = false;

        hideGrayTextInRows();
        hideGrayTextInFields();
        document.PrepareInvoiceForm.submit();
        showGrayTextInFields();
        showGrayTextInRows();

        return true;
    }
    return false;
}

/**
 * Creates an object that corresponds to com.actimind.actitime.reports.InvoiceParams
 * and holds current form settings.
 */
function getInvoiceParams()
{
    hideGrayTextInRows();
    hideGrayTextInFields();

    var params = {};
    params.billTo = formElements.billTo.value;
    params.companyAddress = formElements.companyAddress.value;
    params.companyName = formElements.companyName.value;
    params.header = formElements.invoiceHeader.value;
    params.terms = formElements.terms.value;
    params.customerMessage = formElements.customerMessageField.controller.isEmpty() ? "" : formElements.customerMessageField.value;
    params.additionalInfo = formElements.additionalInfoField.controller.isEmpty() ? "" : formElements.additionalInfoField.value;

    params.discountNames = getArrayOfValues( formElements.discountNames );
    params.taxNames = getArrayOfValues( formElements.taxNames );

    params.taxRateValues = new Array();
    params.taxPercentFlags = new Array();

    applyToSingleOrEach( formElements.taxRates, function( field )
    {
        if( !field.value || !isNotEmpty(field.value) || !rateOrPercentValidator.isNumberStringValid( field.value ) )
        {
            params.taxPercentFlags.push( null );
            params.taxRateValues.push( null );
            return;
        }

        params.taxPercentFlags.push( rateOrPercentValidator.isPercentage( field.value ) );
        params.taxRateValues.push( rateOrPercentValidator.parseNumber( field.value ) );
    });

    params.extraFieldNames = new Array();
    params.extraFieldNames.push( document.getElementsByName("extraFieldNames[0]")[0].value );
    params.extraFieldNames.push( document.getElementsByName("extraFieldNames[1]")[0].value );

    showGrayTextInFields();
    showGrayTextInRows();

    return params;
}

function getArrayOfValues( elementsArray )
{
    var res = new Array();

    if( elementsArray )
    {
        applyToSingleOrEach( elementsArray, function( elem )
        {
            var value = elem.value;
            res.push( elem.value )            
        });
    }

    return res;
}

function hideGrayTextInRows()
{
    applyToSingleOrEach(formElements.itemNames, hideHint);
    applyToSingleOrEach(formElements.itemDescriptions, hideHint);
    applyToSingleOrEach(formElements.taxNames, hideHint);
    applyToSingleOrEach(formElements.discountNames, hideHint);
}

function showGrayTextInRows()
{
    applyToSingleOrEach(formElements.itemNames, showHint);
    applyToSingleOrEach(formElements.itemDescriptions, showHint);
    applyToSingleOrEach(formElements.taxNames, showHint);
    applyToSingleOrEach(formElements.discountNames, showHint);
}

function hideGrayTextInFields()
{
    applyToSingleOrEach( getFieldsWithHint(), hideHint );
}

function showGrayTextInFields()
{
    applyToSingleOrEach( getFieldsWithHint(), showHint );
}

function getFieldsWithHint()
{
    return [formElements.billTo, formElements.companyAddress, formElements.companyName,
        formElements.additionalInfoField, formElements.customerMessageField ];
}

function hideHint(field) {
    if (field.controller) field.controller.hideGrayTextIfShown();
}

function showHint(field) {
    if (field.controller) field.controller.showGreyTextIfEmpty();
}

function onRowMouseOver(row, event)
{
    if (stopRowsSelection) return;
    var evt = event || window.event;
    if (isMouseOverElement(row, evt)) {
        addClass(row, "hovered");
    }
}

function onRowMouseOut(row, event)
{
    if (stopRowsSelection) return;
    var evt = event || window.event;
    if (isMouseOutElement(row, evt)) {
        removeClass(row, "hovered");
    }
}

function onRowFieldFocus(field)
{
    addClass(findElementRow(field), "focused");
}

function onRowFieldBlur(field)
{
    removeClass(findElementRow(field), "focused");
}

function onItemRateFocus(field)
{
    field.value = rateValidator.removeDigitSeparator(field.value);
    onRowFieldFocus(field);
}

function onItemQuantityFocus(field)
{
    field.value = quantityValidator.removeDigitSeparator(field.value);
    onRowFieldFocus(field);
}

function onDiscountOrTaxValueFocus(field)
{
    field.value = rateOrPercentValidator.removeDigitSeparator(field.value);
    onRowFieldFocus(field);
}

function onItemRateBlur(field)
{
    onRowFieldBlur(field);
    onItemRateChange(field);
}

function onItemQuantityBlur(field)
{
    onRowFieldBlur(field);
    onItemQuantityChange(field);
}

function onDiscountValueBlur(field)
{
    ClientSideErrors.markTextFieldAsValid(field);
    onRowFieldBlur(field);
    onDiscountValueChange(field);
}

function onTaxValueBlur(field)
{
    onRowFieldBlur(field);
    onTaxValueChange(field);
}

function findElementRow(element)
{
    while (element.nodeName != "TR")
        element = element.parentNode;

    return element;
}

function validateAll( strict )
{
    emptyInvoiceError.hide();
    ClientSideErrors.hideClientSideErrorMessage();

    // first check that at least one not empty line is added to the invoice.
    // if invoice details are empty we should not start validation
    // as special error message will be displayed instead of a standard one.
    if (strict && !validateInvoiceIsNotEmpty()) {
        return false;
    }

    var valid = true;

    valid = validateNumbers() && valid;

    valid = validateMaximumLengths() && valid;

    if( strict )
    {
        valid = validateRequiredFields( ) && valid;

        valid = validateItemRows( ) && valid;
        valid = validateTaxRows( ) && valid;
        valid = validateDiscountRows( ) && valid;

        valid = validateDiscountAmounts( ) && valid;

        valid = validateExtraFields( ) && valid;
    }

    if (!valid) {
        ClientSideErrors.showClientSideErrorMessage( true );
    }

    return valid;
}

function validateRate(field)
{
    return rateValidator.checkNumber(field, "rates.errors.invalid_rate", "rates.errors.large.rate", true);
}

function validateQuantity(field)
{
    return quantityValidator.checkNumber(field, "report.prepare_invoice.js.invalid_quantity", "rates.errors.large.rate", true);
}

function validateDiscountValue(field)
{
    resetInvalidRateOrPercent(field);
    return true;
}

function validateTaxValue(field)
{
    resetInvalidRateOrPercent(field);
    return true;
}

function resetInvalidRateOrPercent( field )
{
    if( !isNotEmpty(field.value) ) return;

    var valid = rateOrPercentValidator.isNumberStringValid(field.value);
    if( valid )
    {
        rateOrPercentValidator.checkNumber(field, "report.prepare_invoice.js.invalid_quantity", "rates.errors.large.rate", true);
    }
    else
    {
        field.value = rateOrPercentValidator.formatNumber(0, false);
    }
}

function validateNumbers()
{
    var valid = true;

    applyToSingleOrEach(formElements.itemRates, function(field) {
        if (! validateRate(field))
            valid = false;
    });

    applyToSingleOrEach(formElements.itemQuantities, function(field) {
        if (! validateQuantity(field))
            valid = false;
    });

    applyToSingleOrEach(formElements.discountRates, function(field) {
        if (! validateDiscountValue(field))
            valid = false;
    });

    applyToSingleOrEach(formElements.taxRates, function(field) {
        if (! validateTaxValue(field))
            valid = false;
    });

    return valid;
}

function validateRequiredFields()
{
    if (!requiredFields) return true;

    var valid = true;

    for (var i = 0; i < requiredFields.length; i++){
        if (!validateFieldNotEmpty(requiredFields[i], messageResource.getMessage("report.prepare_invoice.error.missing_field"))) {
            valid = false;
            scheduleValidationAfterNextChange(requiredFields[i], messageResource.getMessage("report.prepare_invoice.error.missing_field"), function(field, tooltip) {
                var result = validateFieldNotEmpty(field, tooltip);
                ClientSideErrors.hideClientSideErrorMessageIfNoErrors( true );
                return result;
            }, null);
        }
    }

    return valid;
}

function validateFieldNotEmpty(field, tooltip)
{
    if (isEmptyField(field)) {
        ClientSideErrors.markTextFieldAsInvalid(field, tooltip);
        return false;
    }
    return true;
}

function validateRow(index, isValidateAll, columns, errorTooltips, getIndexFunc)
{
    var valid = isRowValid(columns, index);
    if (valid) {
        for (var col = 0; col < columns.length; col++) {
            applyToSingleOrIndexed(columns[col], index, function (field) {
                safeMarkAsValidRowField(field);
            });
        }
        return true;
    } else {
        for (var i = 0; i < columns.length; i++)  {
            applyToSingleOrIndexed(columns[i], index, function (field) {
                if (isEmptyField(field)) {
                    if (isValidateAll || field.markedAsInvalid) {
                        ClientSideErrors.markTextFieldAsInvalid(field, errorTooltips[i]);
                        field.markedAsInvalid = true;
                    }
                } else if (!isValidateAll) {
                    safeMarkAsValidRowField(field);
                }
                if (isValidateAll) {
                    scheduleValidationAfterNextChange(field, "", function (f) {
                        var rowIdx = getIndexFunc(f);
                        var valid = validateRow(rowIdx, false, columns, errorTooltips, getIndexFunc);
                        if (valid) {
                            for (var j = 0; j < columns.length; j++) {
                                applyToSingleOrIndexed(columns[j], rowIdx, function(field) {
                                    safeMarkAsValidRowField(field);
                                });
                            }
                            ClientSideErrors.hideClientSideErrorMessageIfNoErrors( true );
                        }
                    }, null);
                }
            });
        }
        return false;
    }
}

// removes highlight from the element only if it has been previously marked as invalid by validateRow() method.
// after this calls onchange handler to apply other validators that may be assigned to the element
function safeMarkAsValidRowField(field)
{
    if (field.markedAsInvalid) {
        ClientSideErrors.markTextFieldAsValid(field);
        field.markedAsInvalid = false;

        if (field.onchange) {
            field.onchange(field);
        }
        if (field.onblur) {
            field.onblur(field);
        }
    }
}

function validateItemRow(index)
{
   return validateRow(index, true, [
                formElements.itemDescriptions,
                formElements.itemNames,
                formElements.itemQuantities,
                formElements.itemRates
            ], emptyItemFieldTooltips, getItemIndexByField);
}

function validateItemRows()
{
    var valid = true;
    applyToSingleOrEach(formElements.itemDescriptions, function (field, i) {
        valid = validateItemRow(i, true) && valid;
    });
    return valid;
}

function validateTaxRow(index)
{
    var nameField;
    var rateField;
    applyToSingleOrIndexed(formElements.taxNames, index, function (field) {nameField = field;});
    applyToSingleOrIndexed(formElements.taxRates, index, function (field) {rateField = field;});
    return validateExtraField(nameField, rateField, messageResource.getMessage("report.prepare_invoice.error.missing_tax_name"), true);
}

function validateTaxRows()
{
    var valid = true;
    applyToSingleOrEach(formElements.taxNames, function (field, i) {
        valid = validateTaxRow(i) && valid;
    });
    return valid;
}

function validateDiscountRow(index)
{
    var nameField;
    var rateField;
    applyToSingleOrIndexed(formElements.discountNames, index, function (field) {nameField = field;});
    applyToSingleOrIndexed(formElements.discountRates, index, function (field) {rateField = field;});
    return validateExtraField(nameField, rateField, messageResource.getMessage("report.prepare_invoice.error.missing_discount_name"), true);
}

function validateDiscountRows()
{
    var valid = true;
    applyToSingleOrEach(formElements.discountNames, function (field, i) {
        valid = validateDiscountRow(i, true) && valid;
    });
    return valid;
}

function isRowValid(columns, index)
{
    var nColumns = columns.length;
    var emptyFlags = [];

    for (var col = 0; col < nColumns; col++) {
        var field = getRowField(columns[col], index);
        emptyFlags[col] = isEmptyField(field);
    }

    var allEmpty = true;
    var allNotEmpty = true;
    var j;
    for (j = 0; j < nColumns; j++) {
        allEmpty = allEmpty && emptyFlags[j];
        allNotEmpty = allNotEmpty && !emptyFlags[j];
    }

    return (allEmpty || allNotEmpty);
}

function isRowEmpty(columns, index)
{
    var field;
    for (var col = 0; col < columns.length; col++) {
        field = getRowField(columns[col], index);
        if (!isEmptyField(field)) return false;
    }

    return true;
}

function validateDiscountAmounts()
{
    // remove previously highlighted errors
    applyToSingleOrEach(formElements.discountRates, function (field) {
        markTextFieldValueAsValid(field);
    });

    var valid = true;
    var subtotal = rateValidator.parseNumber(formElements.subtotal.value);
    var amountSum = 0;

    // each discount shall not exceed sub-total amount
    applyToSingleOrEach(formElements.discountAmounts, function(field, i) {
        var value = removeBrackets(field.value);
        var amount = rateValidator.parseNumber(value);

        if (!amount) return; //this means the value cannot be parsed as a number
                             // this error is handled by numbers validator separately

        amountSum = amountSum +  amount;

        if (amount > subtotal){
            valid = false;
            applyToSingleOrIndexed(formElements.discountRates, i, function(rate) {
                ClientSideErrors.markTextFieldAsInvalid(rate, messageResource.getMessage("report.prepare_invoice.error.discount_exceeds_subtotal"));
            });
        } else if (amount <= rateValidator.maxNumber) {
            applyToSingleOrIndexed(formElements.discountRates, i, function(rate) {
                ClientSideErrors.markTextFieldAsValid(rate);
            });
        }
    });

    // the sum of all discounts shall not exceed sub-total amount
    if (valid && (amountSum > subtotal)) {
        valid = false;
        applyToSingleOrEach(formElements.discountRates, function (field) {
            markTextFieldValueAsInvalid(field, messageResource.getMessage("report.prepare_invoice.error.discounts_sum_exceeds_subtotal"));
    });
    }

    ClientSideErrors.hideClientSideErrorMessageIfNoErrors( true );

    return valid;
}

/**
 * Checks if given pair of fields is valid. A pair is invalid if value is specified, but name is not.
 */
function validateExtraField(nameField, valueField, errorMsg, scheduleValidationOnError)
{
    if (!isEmptyField(valueField) && isEmptyField(nameField)) {
        ClientSideErrors.markTextFieldAsInvalid(nameField, errorMsg);
        if (scheduleValidationOnError)
            scheduleValidationAfterNextChange(nameField, errorMsg, function (field, tooltip) {
                return validateExtraField(nameField, valueField, tooltip, false);
            }, null);
            scheduleValidationAfterNextChange(valueField, errorMsg, function (field, tooltip) {
                return validateExtraField(nameField, valueField, tooltip, false);
            }, null);
        return false;
    } else {
        return true;
    }
}

function validateExtraFields()
{
    var valid = true;

    for (var i = 0; i < 2; i++) {
        var nameField = document.getElementsByName("extraFieldNames[" + i +"]")[0];
        var valueField = document.getElementsByName("extraFieldValues[" + i +"]")[0];
        valid = validateExtraField(nameField, valueField,
                messageResource.getMessage("report.prepare_invoice.error.missing_extra_field_name"), true) && valid;
    }               

    return valid;
}

function validateFieldLength(field, maxLength, fieldName, showAlert)
{
    // since we do not trim strings when storing to the DB, we should count whitespaces too
    var valid = (field.value.length <= maxLength);

    if (valid) {
        ClientSideErrors.markTextFieldAsValid(field);
        return true;
    } else {
        var errMsg = messageResource.getMessage("report.prepare_invoice.error.too_long",
                        [fieldName, maxLength, field.value.length]);
        ClientSideErrors.markTextFieldAsInvalid(field, errMsg);

        if ( showAlert ) {
            alert(errMsg);
            focusElement(field);
        }
        return false;
    }
}

function validateMaximumLengths()
{
    var valid = true;
    valid = validateFieldLength(formElements.invoiceHeader, 255, messageResource.getMessage('report.prepare_invoice.error.too_long_title'), valid);
    valid = validateFieldLength(formElements.companyAddress, 2000, messageResource.getMessage("report.prepare_invoice.error.too_long_company_address"), valid) && valid;
    valid = validateFieldLength(formElements.companyName, 2000, messageResource.getMessage("report.prepare_invoice.error.too_long_company_name"), valid) && valid;
    valid = validateFieldLength(formElements.billTo, 2000, messageResource.getMessage("report.prepare_invoice.error.too_long_bill_to", valid)) && valid;
    valid = validateFieldLength(document.getElementsByName("extraFieldNames[0]")[0], 2000, messageResource.getMessage("report.prepare_invoice.error.too_long_extra_field_1_name"), valid) && valid;
    valid = validateFieldLength(document.getElementsByName("extraFieldNames[1]")[0], 2000, messageResource.getMessage("report.prepare_invoice.error.too_long_extra_field_2_name"), valid) && valid;
    valid = validateFieldLength(formElements.invoiceNumber, 255, messageResource.getMessage("report.prepare_invoice.error.too_long_invoice_number"), valid) && valid;
    valid = validateFieldLength(formElements.customerMessageField, 2000, messageResource.getMessage("report.prepare_invoice.error.too_long_customer_message"), valid) && valid;
    valid = validateFieldLength(formElements.additionalInfoField, 2000, messageResource.getMessage("report.prepare_invoice.error.too_long_footer"), valid) && valid;
    valid = validateFieldLength(formElements.terms, 255, messageResource.getMessage("report.prepare_invoice.error.too_long_terms"), valid) && valid;
    applyToSingleOrEach(formElements.taxNames, function(field) {
        valid = validateFieldLength(field, 2000, messageResource.getMessage("report.prepare_invoice.error.too_long_tax_name"), valid) && valid;
    });
    applyToSingleOrEach(formElements.discountNames, function(field) {
        valid = validateFieldLength(field, 2000, messageResource.getMessage("report.prepare_invoice.error.too_long_discount_name"), valid) && valid;
    });
    return valid;
}

function validateInvoiceIsNotEmpty()
{
    var empty = true;

    applyToSingleOrEach(formElements.itemDescriptions, function (field, index) {
        empty = isRowEmpty([
                formElements.itemDescriptions,
                formElements.itemNames,
                formElements.itemQuantities,
                formElements.itemRates
            ], index) && empty;
    });

    if (empty) {
        emptyInvoiceError.show();
    } else {
        emptyInvoiceError.hide();
    }

    return !empty;
}

function getRowField(columnFields, rowIndex)
{
    var res;
    applyToSingleOrIndexed(columnFields, rowIndex, function(field) {
        res = field;
    });
    return res;
}

function scheduleValidationAfterNextChange(field, tooltip, validationFunc, afterValidationFunc)
{
    var handler = function() {
        if (validationFunc(field, tooltip)) {
            ClientSideErrors.markTextFieldAsValid(field);
            removeListener(field, 'blur', handler);

            if (field.onchange) {
                field.onchange(field);
            }
            ClientSideErrors.hideClientSideErrorMessageIfNoErrors( true );
        }

        if (afterValidationFunc) {
            afterValidationFunc(field);
        }
    };

    addListener(field, 'blur', handler);
}

function onValidationChange()
{

}

function isItemRateOrItemQty(field)
{
    return field && (field.name == "itemQuantities" || field.name == "itemRates");
}

function isEmptyStr(str)
{
    return (!str || !(typeof str === "string") || (trim(str).length == 0));
}

function isEmptyField(field)
{
    return (field.controller && field.controller.isEmpty() || isEmptyStr(field.value));
}

function calculateAll()
{
    //remove brackets from discount amounts
    applyToSingleOrEach(formElements.discountAmounts, function (field) {
        if (!isEmptyStr(field.value)) {
            field.value = removeBrackets(field.value);
        }
    });

    // 1. Definitions

    var quantities = [];
    var rates = [];
    var taxable = [];

    var discounts = [];
    var taxes = [];

    var subtotal = 0;
    var subtotalTaxable = 0;

    var totalTaxable;
    var total;

    function processDiscountsOrTaxes(rateElements, amountElements, baseValue, outArray)
    {
        var i = 0;
        outArray.total = 0;

        applyToSingleOrEach(rateElements, function(field) {
            var absValue = 0;
            var roundedAbsValue = 0;
            var fieldValue = rateOrPercentValidator.parseNumber(field.value);

            if (fieldValue)
            {
                if (rateOrPercentValidator.isPercentage(field.value))
                    absValue = baseValue * fieldValue / 100.0;
                else
                    absValue = fieldValue;

                //round amount values before calculating total
                roundedAbsValue = new Number(amountValidator.round(absValue));
            }

            outArray.push(roundedAbsValue);
            outArray.total += roundedAbsValue;
            setFormElementValue(amountElements, i, amountValidator.formatNumberWithDigitSeparator(absValue));
            i++;
        })
    }


    // 2. Processing items

    applyToSingleOrEach(formElements.itemQuantities, function(field) {
        quantities.push(quantityValidator.parseNumber(field.value));
    });

    applyToSingleOrEach(formElements.itemRates, function(field) {
        rates.push(rateValidator.parseNumber(field.value));
    });

    applyToSingleOrEach(formElements.itemTaxableFlags, function(field) {
        taxable.push(field.value > 0);
    });

    for (var i = 0; i < quantities.length; i++)
    {
        var amount;
        var roundedAmount = 0;

        if (quantities[i] && rates[i])
            amount = quantities[i] * rates[i];
        else
            amount = 0;

        roundedAmount = new Number(amountValidator.round(amount));

        subtotal += roundedAmount;
        
        if (taxable[i])
            subtotalTaxable += roundedAmount;

        setFormElementValue(formElements.itemAmounts, i, amountValidator.formatNumberWithDigitSeparator(amount));
    }

    formElements.subtotal.value = rateValidator.formatNumberWithDigitSeparator(subtotal);


    // 3. Processing discounts & taxes 

    processDiscountsOrTaxes(formElements.discountRates, formElements.discountAmounts, subtotal, discounts);

    totalTaxable = subtotal > 0 ? (subtotalTaxable - discounts.total * subtotalTaxable / subtotal) : 0;

    if (totalTaxable < 0) totalTaxable = 0;

    processDiscountsOrTaxes(formElements.taxRates, formElements.taxAmounts, totalTaxable, taxes);

    total = subtotal - discounts.total + taxes.total;

    if (total < 0) total = 0;

    formElements.total.value = amountValidator.formatNumberWithDigitSeparator(total);
    document.getElementById("totalDuplicate").innerHTML = formElements.total.value;

    //add brackets to discount amounts
    applyToSingleOrEach(formElements.discountAmounts, function (field) {
        if (!isEmptyStr(field.value)) field.value = addBrackets(field.value);
    });

    // display gray amount for null values
    grayEmptyAmounts(formElements.itemAmounts);

    grayEmptyAmounts(formElements.discountAmounts);

    grayEmptyAmounts(formElements.taxAmounts);

    // since subtotal has changed, check that discounts do not exceed it
    validateDiscountAmounts();
}

function grayEmptyAmounts(amounts)
{
    applyToSingleOrEach(amounts, function (field) {
        var value = removeBrackets(field.value);

        if (amountValidator.parseNumber(value) == 0) {
            addClass(field, "emptyAmount");
        } else {
            removeClass(field, "emptyAmount");
        }
    });
}

function setFormElementValue(singleOrCollectionElem, index, value)
{
    applyToSingleOrIndexed(singleOrCollectionElem, index, function(elem) {
        if (elem.controller)
            elem.controller.hideGrayTextIfShown();

        elem.value = value;

        if (elem.elastic)
            elem.elastic.fitToHeight();

        if (elem.controller)
            elem.controller.showGreyTextIfEmpty();
    })
}

function copyInvoiceNumber(numberSpan)
{
    formElements.invoiceNumber.value = htmlToJsNewLines(unescapeHtmlSymbols(numberSpan.innerHTML));
    if (formElements.invoiceNumber.elastic)
        formElements.invoiceNumber.elastic.fitToHeight();
    registerChange();
    setFocus(formElements.invoiceNumber);
}

function registerChange(){
    forceDomUpdate();
    reportMediator.registerFieldsWasChanged();
}

function makeElastic(textarea, expandMin)
{
    var config = {};
    config.textarea = textarea;
    if (expandMin) config.expandMin = expandMin;

    new AT.ElasticTextArea(config);
}

function makeDropdownListButton(element)
{
    var selectEl;

    //formElements.itemTaxableFlags[i] sometimes appears to be OPTION element instead of SELECT
    if (element.tagName == "OPTION") {
        selectEl = element.parentNode;
    } else {
        selectEl = element;
    }
    Ext.id(selectEl);
    var button = new AT.DropdownListButton({
        applyTo: selectEl.id,
        listeners: { change: onItemTaxableFlagChange }
    });

   button.on({
        'menushow': {
            fn: function(button) {
                if (!this.el) return;

                //mark button's row as hovered and stop any further moving of selection unless menu is hidden
                addClass(findElementRow(this.el.dom), "hovered");
                stopRowsSelection = true;
            },
            scope: button
        },
        'menuhide': {
            fn: function(button) {
                if (!this.el) return;

                //remove row selection if mouse is not over the selected row
                var row = findElementRow(this.el.dom);
                removeClass(findElementRow(this.el.dom),  "hovered");
                stopRowsSelection = false;
            },
            scope: button
        }
    });
}

function setFocus(element)
{
    element.focus();
}

function initTerms(termsList, defaultTerms)
{
    var termsCb = new Ext.form.ComboBox(
    {
        id: "termsCombobox",
        renderTo :"termsComboBoxCt",
        mode: "local",
        store: termsList,
        autoCreate:{tag: "input", type: "text", autocomplete: "off", maxLength: "255", size:"32"},
        listWidth: 184,
        listeners: { 'change': function() {
            registerChange();
        }},
        name: "terms",
        validationEvent:false,
        emptyText: "",
        typeAhead: false,
        triggerAction:"all",
        enableKeyEvents: true,
        ctCls :"terms-combo-box",
        triggerClass: "down-arrow",
        value: (isEmptyStr(defaultTerms) ? termsList[0] : defaultTerms),
        width: 186
    });

    termsCb.on({
        'select': {
            scope: termsCb,
            fn: function() {
                var value = this.getValue();
                if (!value) return;
                var str = unescapeHtmlSymbols(value);
                if (str != value) {
                    this.setValue(str);
                }
            }
        },
        'focus': {
            scope: termsCb,
            fn: function(){
                this.prevValueOfField = this.getEl().dom.value; // need for fix bug 52979
            }
        },
        'blur': {
            scope: termsCb,
            fn: function(){
                if (this.prevValueOfField != this.getEl().dom.value){
                    registerChange();
                }
            }
        }
    })
}

function makeBillToMenu(billToHistory)
{
    var menuItems = [];
    for (var i = 0; i < billToHistory.length; i++) {
        menuItems[i] = new Ext.menu.Item({
            text: jsToHtmlNewLines(escapeHtmlSymbols(trim(billToHistory[i])))
        });
        menuItems[i].entryValue = billToHistory[i];
        menuItems[i].on({
            'click': {
                fn: function () {
                    if (formElements.billTo.controller) formElements.billTo.controller.hideGrayTextIfShown();
                    formElements.billTo.value = this.entryValue;
                    if (formElements.billTo.elastic)
                        formElements.billTo.elastic.fitToHeight();

                    registerChange();
                    ClientSideErrors.markTextFieldAsValid(formElements.billTo);
                    ClientSideErrors.hideClientSideErrorMessageIfNoErrors( true );
                },
                scope: menuItems[i]
            }
        });

    }

    var button = new Ext.Button({
        applyTo: "billToLoadRecentEntry",
        cls: "at-dropdown-list-btn",
        ctCls: "billToButton",
        menu: new Ext.menu.Menu({
                items: menuItems,
                cls: "at-scrollable-menu"
        }),
        menuAlign: "tr-br",
        text: messageResource.getMessage("common.load_recent_entry"),
        listeners: {
            menushow: function(button, menu) {
                scrollWindowIfNeeded(menu.el.dom);    
            }
        }
    });

    if (menuItems.length == 0) button.disable();

}

function makeRequired(field)
{
    requiredFields.push(field);
}

function initGrayText(field, messageKey)
{
    var controller = new ElasticTextareaController();
    controller.emptyCaption=messageResource.getMessage(messageKey);
    controller.init(field.id);
    controller.showGreyTextIfEmpty();
}

function registerChangeHandlers()
{
    confirmDlg = new ConfirmExitDlg(400,
            "",
            messageResource.getMessage("report.prepare_invoice.js.modifications_not_saved.question"),
            null,
            messageResource.getMessage("report.prepare_invoice.js.modifications_not_saved.discard"),
            messageResource.getMessage("common.buttons.remain_on_page"));

    //firtChild.nodeValue that is used in constructor does not support line breaks, so need to use innerHTML property instead
    confirmDlg.setWarningInnerHTML(messageResource.getMessage("report.prepare_invoice.js.modifications_not_saved.message"));
    ExitConfirmation.setUpBaseExitConfirmation( "report.prepare_invoice.js.exit_confirmation", false );
}

function init(items, discounts, taxes, predefinedTerms, defaultTerms, billToHistory)
{
    requiredFields = [];
    formElements = document.PrepareInvoiceForm.elements;  // Do not use form instead of form.elements,
                                   // or Firefox & Chrome will behave incorrectly
                                   // when all elements with specific name are deleted.

    emptyItemFieldTooltips = [
        messageResource.getMessage("report.prepare_invoice.error.missing_in_row", [messageResource.getMessage("report.prepare_invoice.error.missing_in_row_description")]),
        messageResource.getMessage("report.prepare_invoice.error.missing_in_row", [messageResource.getMessage("report.prepare_invoice.error.missing_in_row_name")]),
        messageResource.getMessage("report.prepare_invoice.error.missing_in_row", [messageResource.getMessage("report.prepare_invoice.error.missing_in_row_quantity")]),
        messageResource.getMessage("report.prepare_invoice.error.missing_in_row", [messageResource.getMessage("report.prepare_invoice.error.missing_in_row_rate")])
    ];

    emptyInvoiceError = new EmptyInvoiceError();
    initTerms(predefinedTerms, defaultTerms);
    makeBillToMenu(billToHistory);

    var i;

    for (i = 0; i < items.length; i++)
    {
        addItemRow(false);
        setFormElementValue(formElements.itemNames,        i, items[i].name);
        setFormElementValue(formElements.itemDescriptions, i, items[i].description);
        setFormElementValue(formElements.itemQuantities,   i, items[i].quantity);
        setFormElementValue(formElements.itemRates,        i, items[i].rate);
    }

    if (items.length == 0)
        addItemRow(false); // by default, at least one items raw shall be present

    for (i = 0; i < discounts.length; i++)
    {
        addDiscountRow(false);
        setFormElementValue(formElements.discountNames, i, discounts[i].name);
        setFormElementValue(formElements.discountRates, i, discounts[i].rate);
    }

    for (i = 0; i < taxes.length; i++)
    {
        addTaxRow(false);
        setFormElementValue(formElements.taxNames, i, taxes[i].name);
        setFormElementValue(formElements.taxRates, i, taxes[i].rate);
    }
    
    validateNumbers();
    calculateAll();

    registerFormChangedHandlers(true);

    //show page content only after the table is formed to avoid blinking
    document.getElementById("mainContent").style.display = "block";

    makeElastic(formElements.companyName);
    makeElastic(formElements.companyAddress, 77);
    makeElastic(formElements.invoiceHeader);
    makeElastic(formElements.billTo, 125);
    makeElastic(formElements.customerMessageField);
    makeElastic(formElements.additionalInfoField);
    makeElastic(formElements.invoiceNumber);
    makeElastic(formElements.date);
    //no easy way to access indexed properties thru formElements
    makeElastic(document.getElementsByName("extraFieldNames[0]")[0]);
    makeElastic(document.getElementsByName("extraFieldNames[1]")[0]);
    makeElastic(document.getElementsByName("extraFieldValues[0]")[0]);
    makeElastic(document.getElementsByName("extraFieldValues[1]")[0]);

    makeRequired(formElements.companyName);
    makeRequired(formElements.invoiceNumber);
    makeRequired(formElements.date);
    makeRequired(formElements.invoiceHeader);
    makeRequired(formElements.companyAddress);
    makeRequired(formElements.billTo);
    makeRequired(formElements.terms);

    initGrayText(formElements.companyName, "report.prepare_invoice.js.company_name_hint");
    initGrayText(formElements.companyAddress, "report.prepare_invoice.js.company_address_hint");
    initGrayText(formElements.billTo, "report.prepare_invoice.js.bill_to_hint");

    formElements.customerMessageField.value = formElements.customerMessage.value;
    if (formElements.customerMessageField.elastic)
        formElements.customerMessageField.elastic.fitToHeight();
    initGrayText(formElements.customerMessageField, "report.prepare_invoice.js.customer_message_hint");

    formElements.additionalInfoField.value = formElements.additionalInfo.value;
    if (formElements.additionalInfoField.elastic)
        formElements.additionalInfoField.elastic.fitToHeight();
    initGrayText(formElements.additionalInfoField, "report.prepare_invoice.js.additional_info_hint");

    applyToSingleOrEach(formElements.itemDescriptions, function (field) {
        if (field.elastic) field.elastic.fitToHeight();
    });
    applyToSingleOrEach(formElements.itemNames, function (field) {
        if (field.elastic) field.elastic.fitToHeight();
    });
    applyToSingleOrEach(formElements.discountNames, function (field) {
        if (field.elastic) field.elastic.fitToHeight();
    });
    applyToSingleOrEach(formElements.taxNames, function (field) {
        if (field.elastic) field.elastic.fitToHeight();
    });
}

function getItemIndexByField(field)
{
    var index = getIndexByElement(formElements.itemDescriptions, field);
    if (index < 0) index = getIndexByElement(formElements.itemNames, field);
    if (index < 0) index = getIndexByElement(formElements.itemQuantities, field);
    if (index < 0) index = getIndexByElement(formElements.itemRates, field);

    return index;
}

function getIndexByElement(singleOrCollection, element)
{
    var index = -1;
    applyToSingleOrEach(singleOrCollection, function (field, i) {
        if (field == element) index = i;
    });
    return index;
}

// -- -- -- --

function markTextFieldValueAsInvalid(field, tooltip)
{
    addClass(field, "invalidValue");
    field.title = tooltip;
}

function markTextFieldValueAsValid(field, errTooltip)
{
    removeClass(field, "invalidValue");
    if (field.title == errTooltip) field.title = "";  
}

/**
 * Replaces new line symbols with <br> tags
 */
function jsToHtmlNewLines(text)
{
    if (text) {
        return text.replace(/\r/g, "").replace(/\n/g, "<br>");
    } else return text;
}

/**
 * Replaces <br> tags with new line symbols
 */
function htmlToJsNewLines(text)
{
    if (text) {
        return text.replace(/<br>/gi, "\n");
    } else return text;
}

// -- -- -- --

function EmptyInvoiceError()
{
    this.shown = false;
}

EmptyInvoiceError.prototype.display = function ()
{
    document.getElementById("EmptyInvoiceError").style.display = "block";
};

EmptyInvoiceError.prototype.show = function ()
{
    this.hide();
    window.setTimeout(this.display, 200);
    this.shown = true;
    window.scrollTo(0, 0);
};

EmptyInvoiceError.prototype.hide = function ()
{
    if (this.shown) {
        document.getElementById("EmptyInvoiceError").style.display = "none";
        this.shown = false;
    }
};

// -- -- -- --

// Redefine ClientSideErrors functions, so that they perform errors registering
// based on element ID rather than name, because it works incorrectly when
// there are multiple fields with the same name.

ClientSideErrors.errors =
{
    _errors: [],
    _getIndex: function( id )
    {
        for (var i = 0; i < this._errors.length; i++)
        {
            if (this._errors[i].id == id)
                return i;
        }
        return -1;
    },
    put: function( value )
    {
        if ( this._getIndex( value.id ) == -1 )
            this._errors.push( value );
    },
    get: function( value ) {
        var index = this._getIndex( value.id );
        if ( index != -1 )
            return this._errors[index];
        return undefined;
    },
    remove: function( value )
    {
        var index = this._getIndex( value.id );
        if ( index != -1 )
            this._errors.splice( index, 1 );
    },
    count: function()
    {
        return this._errors.length;
    }
};

ClientSideErrors.markTextFieldAsValid = function(field)
{
    ClientSideErrors.paintElementAsValid(field);

    ClientSideErrors.unregisterError(field);
};

ClientSideErrors.markTextFieldAsInvalid = function(field, errorTitle)
{
    ClientSideErrors.paintElementAsInvalid(field, errorTitle);
    ClientSideErrors.registerError(field);    
};

ClientSideErrors.unregisterError = function(field)
{
    Ext.id(field);
    ClientSideErrors.errors.remove(field);
};

ClientSideErrors.registerError = function(field)
{
    Ext.id(field);
    ClientSideErrors.errors.put(field);
};

ClientSideErrors.hasErrors = function()
{
    return ClientSideErrors.errors.count() > 0;
};

ClientSideErrors.isMarkedAsInvalid = function (field)
{
    if (field.id)
        return ClientSideErrors.errors.get(field);
    else return undefined;
};

// -- -- -- --

// Extend FieldController to adapt it to elastic textarea

function ElasticTextareaController()
{
    this.setDefaultValueIfNeeded = function()
    {
        if (trim(this.fieldElement.value).length == 0)
            this.fieldElement.defaultValue = this.emptyCaption;
    };
}

ElasticTextareaController.prototype = new FieldController();

ElasticTextareaController.prototype.showGreyText = function()
{
    addClass(this.fieldElement, this.emptyCaptionStyleClass);
    if (this.fieldElement.value != this.emptyCaption) {
        this.fieldElement.value = this.emptyCaption;
        if (this.fieldElement.elastic) {
            this.fieldElement.elastic.fitToHeight();
        }
        this.emptyCaptionShown = true;
    }
};

ElasticTextareaController.prototype.hideGrayText = function()
{
    this.fieldElement.value = "";
    removeClass(this.fieldElement, this.emptyCaptionStyleClass);
    this.emptyCaptionShown = false;
};

ElasticTextareaController.prototype.hideGrayTextIfShown = function()
{
    if (this.emptyCaptionShown)
        this.hideGrayText();
};

ElasticTextareaController.prototype.isEmpty = function()
{
    //get rid of carriage return symbols before comparing strings
    var normalizedValue = this.fieldElement.value.replace(/\r/gi, "");
    return ((normalizedValue == this.emptyCaption) || (trim(this.fieldElement.value).length == 0));
};


// -- -- -- --
// 2 digits after decimal point for numbers, 0-4 digits - for percents
function MoneyOrPercentValidator(decimalSeparator, digitalGrouping, maxNumber)
{
    this.decimalSeparator = decimalSeparator;
    this.digitalGrouping = digitalGrouping;
    this.maxNumber = maxNumber;
    this.allowPercent = true;
    this.maxFractionalDigits = 4;
    this.trimTrailingNulls = true;

    //additional validator to handle non-percent values which should be formatted differently
    this.moneyValidator = new NumbersValidator(decimalSeparator, digitalGrouping, maxNumber, false, 2, false);

    this.formatNumber = function (value, isPercent)
    {
        var str = new String(value);
        if (isPercent || this.isPercentage(str)) {
            return MoneyOrPercentValidator.prototype.formatNumber.call(this, str);
        } else {
            return this.moneyValidator.formatNumber(str);
        }
    };

    this.checkNumber = function (field, invalidNumberError, maxNumberError, emptyFieldIsValid, oldValue, doNotMarkAsValid)
    {
        var str = new String(field.value);
        if (this.isPercentage(field.value)) {
            return MoneyOrPercentValidator.prototype.checkNumber.call(
                    this, field, invalidNumberError, maxNumberError, emptyFieldIsValid, oldValue, doNotMarkAsValid);
        } else {
            return this.moneyValidator.checkNumber(field, invalidNumberError, maxNumberError, emptyFieldIsValid, oldValue, doNotMarkAsValid);
        }
    };
}

MoneyOrPercentValidator.prototype = new NumbersValidator;

