function ProjectLightbox()
{
}

ProjectLightbox.prototype = new BasicLightbox();

ProjectLightbox.NAME_PROJECT_EMPTY = "nameProjectEmpty";
ProjectLightbox.DESCRIPTION_VERY_LARGE = "descriptionError";
ProjectLightbox.CUSTOMER_NOT_SELECTED = "customerNotSelected";
ProjectLightbox.PROJECT_EXISTS = "projectExists";
ProjectLightbox.CUSTOMER_EXISTS = "customerExists";
ProjectLightbox.NAME_CUSTOMER_EMPTY = "customerNameEmpty";
ProjectLightbox.USERS_WAS_DELETED = "usersWasDeletedError";
ProjectLightbox.prototype.exist_invalid_fields_message_key = "project.common.errormsg";

ProjectLightbox.prototype.init = function ( customersInfo, customerId, isInstantValidation, isEnabledPas, successHandler )
{
    var self = this;
    this._popupId = "projectPopup";
    this._popupEl = $( "#projectPopup" );
    this._innerContentEl = $( ".innerContent" );
    this.isEnabledPAS = isEnabledPas;
    this.dataChanged = false;
    this.errors = {};
    this.changesCallback = function ()
    {
        if( !self.dataChanged )
            self.dataChanged = true;
    };
    
    this.successHandler = successHandler;

    onChangeAssignTeamUserSelectorCallback = function(){
        self.changesCallback();
    };

    BasicLightbox.prototype.init.call( this, this._popupId, "closeProjectLightBoxBtn" );

    this._isInstantValidation = isInstantValidation;
    this.sectionEl = this.$subId( "sectionDetails" );
    this.commitButtonTitlePlaceholder = self.$subId("commitBtn").find(".buttonTitle");

    this.projectNameField = this.$subId( "projectNameField" );
    this.descriptionField = this.$subId( "projectDescriptionField" );
    this.newCustomerField = this.$subId( "newCustomerNameField" );

    this.titlePlaceholder = this.getElementBySubId( "titlePlaceholder" );
    this.projectNameError = this.getElementBySubId( "projectNameError" );

    this.customerSelectorConteiner = this.getElementBySubId( "customerContainer" );
    this.customerSelectorError = this.getElementBySubId( "customerSelectorError" );
    this.customerSelectorPlaceHolderEl = this.getElementBySubId( "customerSelectorPlaceholder" );

    this.initCustomerSelector( customersInfo, customerId );
    this.initTextAreaLengthNote();

    this.projectNameField.off();
    this.descriptionField.off();
    this.newCustomerField.off();

    this.setCancelCallback( function () { self.hide(); } );


    this.keyUpChangesHandler = function(additionalHandler) {
        return function () {
            this.keyPressed = false;
            if( (!this.prevValue && !!this.value) || this.prevValue != this.value )
                self.changesCallback();

            if( additionalHandler )
                additionalHandler();
        }
    };
    this.onBlurFieldHandler = function(additionalHandler){
        return function()
        {
            $(this).pVal( trim(this.value) );
            //see Bug 48539
            if ( this.keyPressed ) self.keyUpChangesHandler.call(this);

            if (additionalHandler)
                additionalHandler();
        };
    };

    this.onKeyPressedHandler = function ()
    {
        this.keyPressed = true;
    };


    this.projectNameField.keydown( this.onKeyPressedHandler() );
    this.projectNameField.blur( this.onBlurFieldHandler( function () { self.checkProjectNameOnChange(); } ) );
    this.projectNameField.keyup( this.keyUpChangesHandler() );
    this.projectNameField.change(function(){self.hideError(ProjectLightbox.PROJECT_EXISTS)});

    this.descriptionField.keydown( this.onKeyPressedHandler() );
    this.descriptionField.blur( this.onBlurFieldHandler( function () { self.checkDescriptionOnChange( ); } ) );
    this.descriptionField.keyup( this.keyUpChangesHandler( function () {  self.checkDescriptionOnChange(); } ) );

    this.newCustomerField.keydown( this.onKeyPressedHandler() );
    this.newCustomerField.blur( this.onBlurFieldHandler( function () { self.checkCustomerNameOnChange(); } ) );
    this.newCustomerField.keyup( this.keyUpChangesHandler() );
    this.newCustomerField.change(function(){self.hideError(ProjectLightbox.CUSTOMER_EXISTS)});

    if( this.isEnabledPAS )
    {
        sectionsStateController.showOrHideBlock( "userAssignmentBlock" );
        this.initAssignersTooltip();
        $( ".projectTeamHeaderWrapper" ).click( function ()
                                                  {
                                                    if (!sectionsStateController.isSectionHidden("userAssignmentBlock"))
                                                      self._innerContentEl.animate( {scrollTop: $( "#projectSection" ).outerHeight()}, 'slow' )
                                                  } );
    }
    return this;
};

ProjectLightbox.prototype.initPlaceholders = function ()
{
    this.projectNameField.placeholder();
    this.descriptionField.placeholder();
    this.newCustomerField.placeholder();
};

ProjectLightbox.prototype.initAssignersTooltip = function(){
     var self = this;
     this.assignHeaderTooltip = new DropdownDialog({
                                                         wrapperClass: "projectPopup_AssignTooltipWrapper",
                                                         dialogDiv: $("#projectPopup_AssignerTooltip"),
                                                         hideButtons: true,
                                                         hideOnResize: true,
                                                         showEmptyTitleBar: false,
                                                         isWithBeak: true
                                                     });
    $("#infoAssignIcon" ).click( function (event) {event.stopPropagation();  self.showAssignersTooltip();});
};

ProjectLightbox.prototype.showAssignersTooltip = function ( )
{
    this.assignHeaderTooltip.showAt( document.getElementById("infoAssignIcon"), {my: "left bottom", at: "left top", offset: "-40px -8px", collision: "none"} );
    $(".ui-widget-overlay" ).zIndex(this._popupEl.zIndex() + 1);
};



ProjectLightbox.prototype.updateUI = function ()
{
    this.updateErrors(true);
};

ProjectLightbox.prototype.$subId = function ( subId )
{
    return $( this.getElementBySubId( subId ) );
};

ProjectLightbox.prototype.getElementBySubId = function ( name )
{
    return document.getElementById( this.lightBoxEl.id + "_" + name );
};

ProjectLightbox.prototype._checkProjectNameValid = function ( onlyRemoveEmptyError )
{
   return !this.checkEmpty( this.projectNameField, ProjectLightbox.NAME_PROJECT_EMPTY, onlyRemoveEmptyError, true );
};

ProjectLightbox.prototype._checkCustomerNameValid = function ( onlyRemoveEmptyError )
{
   if (this.getCustomerId() == PROJECT_CONSTANTS.NEW_CUSTOMER_OPTION_INDEX)
       return !this.checkEmpty( this.newCustomerField, ProjectLightbox.NAME_CUSTOMER_EMPTY, onlyRemoveEmptyError, true );

    return true;
};

ProjectLightbox.prototype._checkCustomerValid = function ( )
{
    var idValid = this._checkCustomerId();
    var nameValid = this._checkCustomerNameValid();
    return idValid && nameValid;
};

ProjectLightbox.prototype.checkDescriptionOnChange = function ()
{
    this.isDescriptionValid();
    this.updateErrors(false);
};

ProjectLightbox.prototype.isDescriptionValid = function ()
{
    this.descriptionLengthNote.update(this.getDescriptionForUpdatingLengthNote());
    var textLength = this.descriptionLengthNote.getTextLength();
    var result = textLength > this.descriptionLengthNote.maxValue;
    this.errors[ ProjectLightbox.DESCRIPTION_VERY_LARGE ] = result;
    this.descriptionField.attr("title", result ? messageResource.getMessage("customer.edit.description_too_big", [textLength]) : "");

    return !result;
};

ProjectLightbox.prototype.checkProjectNameOnChange = function ()
{
    this.checkEmpty( this.projectNameField, ProjectLightbox.NAME_PROJECT_EMPTY, true, true );
    this.updateErrors(false);
};


ProjectLightbox.prototype.checkCustomerNameOnChange = function ()
{
    this.checkEmpty( this.newCustomerField, ProjectLightbox.NAME_CUSTOMER_EMPTY, true, true );
    this.updateErrors(false);
};

ProjectLightbox.prototype.getErrorsMap = function()
{
    return this.errors;
};

ProjectLightbox.prototype._checkCustomerId = function ()
{
    var self = this;
    var customerId = self.getCustomerId();
    if (customerId <= 0 && customerId != PROJECT_CONSTANTS.NEW_CUSTOMER_OPTION_INDEX){
        self.displayError(ProjectLightbox.CUSTOMER_NOT_SELECTED);
        return false;
    }
    return true;
};

/* offset of the innerContent in the popup content
 *  used when scrolling to section with error*/
ProjectLightbox.prototype.getContentOffset = function() {
    return -60;
};

ProjectLightbox.prototype.scrollToProjectName = function() {
    this.scrollToSection( $(this.getElementBySubId("projectNameField")) );
};

ProjectLightbox.prototype.scrollToDescription = function() {
    this.scrollToSection( $(this.getElementBySubId("projectDescriptionField")) );
};

ProjectLightbox.prototype.scrollToCustomerSelector = function() {
    this.scrollToSection( $(this.getElementBySubId("customerContainer")) );
};

ProjectLightbox.prototype.scrollToUserAssignments = function() {
    this.scrollToSection( $("#userAssignmentBlock") );
};

ProjectLightbox.prototype.validate = function ()
{
    this.errors = {};

    var scrolled = false;
    if ( !this._checkProjectNameValid() ) {
        this.scrollToProjectName();
        scrolled = true;
    }

    if ( !this._checkCustomerValid() && !scrolled ) {
        this.scrollToCustomerSelector();
        scrolled = true;
    }

    if ( !this.isDescriptionValid() && !scrolled ) {
        this.scrollToDescription();
        scrolled = true;
    }

    if ( !this._customValidate() && !scrolled ) {
        this._customValidateScroll();
    }

    this.updateUI();

    return !this.hasErrors();
};


ProjectLightbox.prototype.getProjectName = function ()
{
    return this.projectNameField.pVal();
};

ProjectLightbox.prototype.getDescription = function ()
{
    return this.descriptionField.pVal();
};


ProjectLightbox.prototype.getCustomerName = function ()
{
    return this.newCustomerField.pVal();
};


ProjectLightbox.prototype.getCustomerId = function ()
{
    return this.customerSelector.getSelectedItem().id;
};

ProjectLightbox.prototype._changesDetected = function ( notRequireConfirmationOnCopySettings )
{
    if( this.changesCallback )
        this.changesCallback( notRequireConfirmationOnCopySettings );
};


ProjectLightbox.prototype.setTitle = function ( title )
{
    this.titlePlaceholder.innerHTML = escapeHtmlSymbols( title );
};

ProjectLightbox.prototype.getTeamProject = function ()
{
    if( this.isEnabledPAS )
    {
        var rightListUsers = rightListController.objects.getValues();
        var teamProject = [];
        for(var i = 0; i < rightListUsers.length; ++i )
            teamProject.push( rightListUsers[i].id );
        return teamProject;
    }
    return null;
};


ProjectLightbox.prototype.setError = function ( errorKey )
{
    this.errors[errorKey] = true;
};

ProjectLightbox.prototype.initCustomerSelector = function ( customerInfos, customerId )
{
    var self = this;
    var items = [];
    self.getElementBySubId("customerSelectorPlaceholder" ).innerHTML = "";
    if (customerId <=0){
        items.push({id: PROJECT_CONSTANTS.SELECT_CUSTOMER_OPTION_INDEX, name: messageResource.getEscapedMessage( "project.common.please_select_one" )});
        customerId = PROJECT_CONSTANTS.SELECT_CUSTOMER_OPTION_INDEX;
    }

    var newCustomerIndex = PROJECT_CONSTANTS.NEW_CUSTOMER_OPTION_INDEX;
    items.push({id: newCustomerIndex, name: messageResource.getEscapedMessage( "project.common.new_customer" )});
    if( customerInfos.list.length == 0 )
        customerId = newCustomerIndex;

    $.each( customerInfos.list, function ()
    {
        items.push( {id: this.id, name: escapeHtmlSymbols( this.name )} );
    });

    this.customerSelector = new ListSelector( {
                                                  menuCls: "customerSelectorMenu projectPopup",
                                                  renderTo: this.customerSelectorPlaceHolderEl,
                                                  value: customerId,
                                                  listItems: items,
                                                  textPropName: "name",
                                                  valuePropName: "id",
                                                  scrollableContainer: this._innerContentEl
                                              } );
    var handler = function (customerValue)
    {
        $( self.customerSelectorConteiner ).removeClass( "newCustomer" );
        $( self.customerSelectorPlaceHolderEl ).removeClass( "notSelected" );
        self.hideError(ProjectLightbox.CUSTOMER_EXISTS);
        self.hideError(ProjectLightbox.CUSTOMER_NOT_SELECTED);
        if( customerValue == PROJECT_CONSTANTS.NEW_CUSTOMER_OPTION_INDEX )
        {
            $( self.customerSelectorConteiner ).addClass( "newCustomer" );
            setFocusIntoFieldWithPlaceholder( self.newCustomerField );
        }
        else self.hideError(ProjectLightbox.NAME_CUSTOMER_EMPTY);

        if( customerValue == PROJECT_CONSTANTS.SELECT_CUSTOMER_OPTION_INDEX )
            $( self.customerSelectorPlaceHolderEl ).addClass( "notSelected" );
    };
    handler(customerId);
    this.customerSelector.on( "change", function () { handler(self.getCustomerId()); self.changesCallback();} );
};

ProjectLightbox.prototype.hasErrors = function ()
{
    var classes = [];
    for( var p in this.errors )
            if( this.errors[p] )
            classes.push( p );
    this.sectionEl.context.className = classes.join( " " );
    return classes.length > 0;
};

ProjectLightbox.prototype.displayError = function ( errorCode )
{
    this.errors[errorCode] = true;
    this.updateErrors( true );
};

ProjectLightbox.prototype.hideError = function ( errorCode )
{

    this.errors[errorCode] = false;
    this.updateErrors( true );
};


ProjectLightbox.prototype.updateErrors = function ( showErrorInFooter )
{
    if( this.hasErrors() )
    {
        if( showErrorInFooter )
        {
            this._printErrorInFooter();
        }
    }
    else
        this._removeErrorInFooter();
};

ProjectLightbox.prototype.resetErrors = function ()
{
    this.errors = {};
    this.updateErrors( true );
};

ProjectLightbox.prototype.successCommitCallback = function ( result )
{

    var self = this;
    if( result.errorCode == PROJECT_CONSTANTS.RESULT_CODE_SUCESS )
    {
        if (!_.isUndefined(this.successHandler))
        {
            this.successHandler(result);
            self.hide();
        }
        else
            document.location.reload();
    }

    if( result.errorCode == PROJECT_CONSTANTS.CUSTOMER_WAS_ARCHIVED )
    {
        alert( messageResource.getMessage( 'project.common.customer.archived' ) );
        self.initCustomerSelector( result.customers, 0 );
        self.scrollToCustomerSelector();
    }

    if( result.errorCode == PROJECT_CONSTANTS.CUSTOMER_WAS_DELETED )
    {
        alert( messageResource.getMessage( 'project.common.customer.deleted' ) );
        self.initCustomerSelector( result.customers, 0 );
        self.scrollToCustomerSelector();
    }
    if( result.errorCode == PROJECT_CONSTANTS.USERS_WAS_DELETED )
    {
        reloadSelector( result.users.list );
        self.setError( ProjectLightbox.USERS_WAS_DELETED );
        sectionsStateController.showBlock( "userAssignmentBlock" );
        self.scrollToUserAssignments();
    }
    self.customSuccessCommitCallback(result);
    self.enableUserInputWhileSaving();
    self.updateUI();
    return true;
};

ProjectLightbox.prototype.failureCommitCallback = function ( jqXHR, textStatus, errorThrown )
{
    var self = this;
    if (!errorThrown.exception){
        self.enableUserInputWhileSaving();
        return false;
    }
    if( errorThrown.exception.indexOf( 'com.actimind.actitime.project.ProjectExistsException' ) != -1 )
    {
        self.enableUserInputWhileSaving();
        self.displayError( ProjectLightbox.PROJECT_EXISTS );
        self.scrollToProjectName();
        return true;
    }
    if( errorThrown.exception.indexOf( 'com.actimind.actitime.customer.CustomerExistsException' ) != -1 )
    {
        self.enableUserInputWhileSaving();
        self.displayError( ProjectLightbox.CUSTOMER_EXISTS );
        self.scrollToCustomerSelector();
        return true;
    }
    if (!self.customFailureCommitCallback( jqXHR, textStatus, errorThrown )) {
        self.enableUserInputWhileSaving();
        return false;
    }else
        return true;
};

ProjectLightbox.prototype.customSuccessCommitCallback = function(result){
  return true;  //abstract;
};

ProjectLightbox.prototype.customFailureCommitCallback = function ( jqXHR, textStatus, errorThrown )
{
  return false; //abstract;
};

ProjectLightbox.prototype._customValidate = function(){
    return true; //abstract
};

ProjectLightbox.prototype._customValidateScroll = function(){
    //abstract
};

ProjectLightbox.prototype.initTextAreaLengthNote = function ()
{
    var self = this;
    this.descriptionLengthNote = new TextAreaLengthNote( self.getElementIdBySubId('projectDescription'), 'Description', 2000);
    this.descriptionLengthNote.setTextLengthCounter( this.descriptionLengthNote.fullCounter );
    this.descriptionLengthNote.update( this.getDescriptionForUpdatingLengthNote() );
    this.descriptionLengthNote.show();
};

ProjectLightbox.prototype.getDescriptionForUpdatingLengthNote = function ()
{
    return trim(getStringWithUnifiedNewLineSymbol(this.getDescription()));
};

ProjectLightbox.prototype.getElementIdBySubId = function ( subId )
{
    return this.lightBoxEl.id + "_" + subId;
};




