var XmlHttpRequestUtil = {

    /*
     * Request ready statuses.
     */
    UNINITIALIZED: 0,
    LOADING:       1,
    LOADED:        2,
    INTERACTIVE:   3,
    COMPLETED:     4,

    /*
     * Request status codes.
     */
    OK_STATUS: 200,
    BAD_REQUEST_STATUS: 400,
    HTTP_VERSION_NOT_SUPPORTED_STATUS: 505,

    /*
     * Request method constants.
     */
    GET_REQUEST: "GET",
    POST_REQUEST: "POST",

    /*
     * Creates XMLHttpRequest object.
     * Returns created object or null, if XMLHttpRequest is not supported.
     */
    createRequestObject : function () {
        var request = null;

        try {
            request = new ActiveXObject('Msxml2.XMLHTTP');
        } catch (e){}
        if(!request) try {
            request = new ActiveXObject('Microsoft.XMLHTTP');
        } catch (e){}
        if(!request) try {
            request = new XMLHttpRequest();
        } catch (e){}

        return request;
    }
};

var RefreshSessionUtil = {

    /*
     * Timeout (in ms) between requests to refresh user session.
     */
    REFRESH_TIMEOUT: 60*60*1000, // one hour in ms.

    /*
     * Determines whether an error has been already handled before or not.
     * See RefreshSessionUtil.refreshSession().
     */
    ERROR_WAS_HANDLED: false,

    /**
     * Sets timer for delayed call RefreshSessionUtil.refreshSession() function. Timeout value equals RefreshSessionUtil.REFRESH_TIMEOUT.
     */
    delayedRefreshSession: function()
    {
        var func = function() { RefreshSessionUtil.refreshSession(); };
        window.setTimeout(func, RefreshSessionUtil.REFRESH_TIMEOUT);
    },

    /*
     * Do GET-request to refresh user session with RefreshSessionUtil.REFRESH_TIMEOUT interval.
     */
    refreshSession: function () {
        var request = XmlHttpRequestUtil.createRequestObject();
        if (request == null) {
            return;
        }

        try {
            request.open(XmlHttpRequestUtil.GET_REQUEST, REFRESH_SESSION_URL, true);
            request.onreadystatechange = function () {
                if (request.readyState == XmlHttpRequestUtil.COMPLETED) {
                    RefreshSessionUtil.delayedRefreshSession();
                    try {
                        if (RefreshSessionUtil.isHandledError(request.status)) {
                            RefreshSessionUtil.ERROR_WAS_HANDLED = true;
                            RefreshSessionUtil.showMessageForReceivedStatusCode();
                        } else if (request.status == XmlHttpRequestUtil.OK_STATUS) {
                            RefreshSessionUtil.ERROR_WAS_HANDLED = false;
                        }
                    } catch (e) {
                        // Can be throw when server is not available, in this case statement request.status throws exception.
                    }
                }
            };
            request.send(null);
        } catch (e) {
            // Can be thrown when user works offline and XMLHttpRequest object attempts to send request.
        }
    },

    /*
     * Determine whether specified status code should be handled.
     *
     * status - the code of status of the request retrieved from the server.
     * return true if retrieved status code is a Client Error 4xx code or Server Error 5xx code and false otherwise.
     */
    isHandledError: function(status) {
        return !RefreshSessionUtil.ERROR_WAS_HANDLED &&
               status >= XmlHttpRequestUtil.BAD_REQUEST_STATUS &&
               status < XmlHttpRequestUtil.HTTP_VERSION_NOT_SUPPORTED_STATUS;
    },

    /*
     * Displays pop-up message box explaining retrieved status code.
     */
    showMessageForReceivedStatusCode: function() {
        alert(messageResource.getMessage("common.ajax.unavailable_server_error"));
    }
};


RefreshSessionUtil.delayedRefreshSession();