BaseReport.PDFGenerator = function(){};

BaseReport.PDFGenerator.prototype = _.create(Object.prototype, {
    init: function(store, generalStore, productSettingsValidator, reportRightsValidator)
    {
        this.onGenerateBreaker = {needBreak: false};
        this.store = store;
        this.generalStore = generalStore;
        this.productSettingsValidator = productSettingsValidator;
        this.reportRightsValidator = reportRightsValidator;

        return this;
    },

    interruptGeneration: function()
    {
        this.onGenerateBreaker.needBreak = true;
    },

    _getCreatePDFDefinitionStrategy: function()
    {
        return null;
    },

    calculateTotalNumberOfPages: function(params)
    {
        return 0;
    },

    generatePDF: function()
    {
        var newBreaker = {
            needBreak: false
        };
        this.onGenerateBreaker = newBreaker;

        var pdfPrintParameters = this.store.getPrintParametersForSave();
        var reportParams = this.store.getReportParams();

        var rows = this.store.getConvertedData();

        var pdfConfig = {
            pdfPrintParameters: pdfPrintParameters,
            pdfData: {
                tableData: rows
            },
            reportParams: reportParams,
            dateFormat: new DateFormat(SystemSettings.dateFormat)
        };

        var s = this;
        return new Promise(
                function(onComplete)
                {
                    onComplete(pdfConfig);
                }
        ).then(
                function (pdfConfig) {
                    return new Promise(function (onComplete, onError){
                        defer(_.bind(onComplete, this, pdfConfig));
                    });
                }
        ).then(
                function(pdfConfig)
                {
                    if (newBreaker["needBreak"])
                    {
                        return Promise.reject(new Error("abort"));
                    }
                    return s._generatePDF(pdfConfig, newBreaker);
                }
        );
    },

    _generatePDF: function(config, breaker)
    {
        var paramFormattingStrategy = this._getParametersFormattingStrategy(config);

        return this.docDefinitionCreator
                .createDocDefinition(config, paramFormattingStrategy, this.store)
                .then(
                        function(ddObject)
                        {
                            pdfMake.fonts = PDFFonts;
                            var doc = pdfMake.createPdf(ddObject["docDefinition"]);
                            var currentFileName = paramFormattingStrategy.formatChartPDFFileName(config.reportParams);

                            return new Promise(function(onComplete, onError)
                            {
                                try
                                {
                                    doc._createDoc(
                                            {
                                                tableLayouts: ddObject["tableLayouts"]
                                            },
                                            function(result, pages)
                                            {
                                                if (breaker.needBreak)
                                                {
                                                    return onError(new Error("abort"));
                                                }

                                                onComplete({
                                                               encodedPDF: result,
                                                               fileName: currentFileName,
                                                               pageCount: pages.length
                                                           })
                                            }
                                    );

                                }
                                catch (e)
                                {
                                    onError(e);
                                }
                            });
                        }
                );
    }
});