function BudgetedVsActualReportMediator()
{
    this.constructor = BudgetedVsActualReportMediator;
}

BudgetedVsActualReportMediator.prototype = new ReportMediator();

BudgetedVsActualReportMediator.prototype.init = function(config)
{
    ReportMediator.prototype.init.call(this, config);
    this._initStore(config);
    this._initControllers();

    this.eventNamespace = "EVAReport";
    this.store = EVAReport.Store;

    var self = this;
    var oldIE = isOldIE();
    if (!oldIE)
    {
        this.productSettingsValidator = new ATProductSettingsValidator();
        this.reportRightsValidator = new ATReportRightsValidator();

        new BaseReport.PDFPreviewLightboxTemplateView($("body"), "leaveTimeAndBalancesPdfPreviewLightbox", "lightboxWithWhiteContent pdfPreviewLightbox noprint").init();

        this.pdfPreviewLightbox = new EVAReport.PDFPreviewLightbox().init(
                {
                    popupId: "leaveTimeAndBalancesPdfPreviewLightbox",
                    closeBtnId: "closeViewTTPdfPreviewLightboxButton",
                    eventNamespace: this.eventNamespace,
                    productSettingsValidator: this.productSettingsValidator,
                    reportRightsValidator: this.reportRightsValidator
                });

        this.initPDFEvents();
    }
    
    this.initCSVEvents();
    
    this.setConfigDateRangeSelection(this.dateRangeSelector.getSelectedRange());
    
    return this;
};

BudgetedVsActualReportMediator.prototype.cancelAllPDFExportingProcesses = function()
{
    if (this.pdfGenerator)
        this.pdfGenerator.interruptGeneration();
    this.pdfPreviewLightbox.cancelAllExportingProcesses();
};

BudgetedVsActualReportMediator.prototype.generatePDF = function()
{
    this.pdfGenerator = new EVAReport.PDFGenerator().init(this.store, ATReportsStore, this.productSettingsValidator, this.reportRightsValidator);

    $(document).trigger(this.getEventName("pdfGenerator.start"));
    var self = this;
    this.pdfGenerator.generatePDF(function(config, onDataProvided) {}).then(
            function(pdf)
            {
                self.store.setGeneratedPDF(pdf.encodedPDF);
                self.store.setPDFFileName(pdf.fileName);
            },
            function(error)
            {
                if (error.message != "abort")
                {
                    console.error(error);
                }
            });
};

BudgetedVsActualReportMediator.prototype._initStore = function(config)
{
    ATReportsStore.setCPList(config.cpList);
};

BudgetedVsActualReportMediator.prototype.getEventName = function(event)
{
    return this.eventNamespace + "." + event;
};

BudgetedVsActualReportMediator.prototype._initControllers = function()
{
    var self = this;
    var oldIE = isOldIE();
    this.ExportSelector = new ExportPanel($("body"), oldIE ? null : "EVAReport.showPDFPreview", "EVAReport.createCSV").init();

    this.$exportButton = $(".export");
    this.$exportButton.click(function() { if (!self.$exportButton.hasClass('disabled'))  self.ExportSelector.showAt($(this)); });

    if (oldIE) // IE8 and IE9 don't support pdf preview and pdf generate
    {
        ReportUtils.generateOldIEPDFTooltip(this.ExportSelector.findPDFButton());
    }
};

BudgetedVsActualReportMediator.prototype.generateCSV = function()
{
    var customersMap = _.extend(arrayToMapById(ATReportsStore.getCPList().customers), EVAReport.Store.getCustomers());
    var projectMap = _.extend(arrayToMapById(ATReportsStore.getCPList().projects), EVAReport.Store.getProjects());
    var tasksMap = EVAReport.Store.getTasks();
    var usersMap = EVAReport.Store.getUsers();
    var projectNamesMap = EVAReport.Store.getProjectNames();
    var taskNamesMap = EVAReport.Store.getTaskNames();
    var typesOfWorkMap = EVAReport.Store.getTypesOfWork();

    var generator = new EVACSVJsonConverter().init(customersMap, projectMap, tasksMap, usersMap, projectNamesMap, taskNamesMap, typesOfWorkMap, null,
                                                   EVAReport.Store.getGroupingPresentationFlags(), EVAReport.Store);

    var jsonTitle = generator.convertTitle(EVAReport.Store.getReportParams());
    var jsonTable = generator.convertLinearTable(EVAReport.Store.getReportParams(),
                                                 EVAReport.Store.getReportData());

    var jsonTotal = generator.convertTotal(EVAReport.Store.getReportParams(),
                                           EVAReport.Store.getReportData());
    
    this.downloadJsonAsCSV([jsonTitle, jsonTable, jsonTotal],
                           generator.generateCSVName(EVAReport.Store.getReportParams()));
};

BudgetedVsActualReportMediator.prototype.getReportType = function()
{
    return ReportType.ESTIMATED_VS_ACTUAL;
};

BudgetedVsActualReportMediator.prototype.initEditConfigController = function(config)
{
    var self = this;
    this.controller = new BudgetedVsActualResultConfigController().init(
            {
                reportParameters: config.reportParameters,
                cpList: config.cpList,
                changeCallback: function() { self.changeParametersCallback(); }
            });
    return this.controller;
};

BudgetedVsActualReportMediator.prototype.createDateRangeSelector = function(reportParameters)
{
    var dateRangeSelectorConfig = {
        dateRangeSelection: reportParameters.dateSelection,
        menuStates: new DateRangeMenuItemsUtil().createDefaultSingleDateMenuItems(),
        dateRangeParsed: true
    };

    return new SingleDateSelectorController().init(dateRangeSelectorConfig);
};

BudgetedVsActualReportMediator.prototype.setDateRangeParameter = function(reportParameters)
{
    this.dateRangeSelector.setSelectedRange(reportParameters.dateSelection);
};

BudgetedVsActualReportMediator.prototype.onDateRangeChanged = function()
{
    this.setConfigDateRangeSelection(this.dateRangeSelector.getSelectedRange());

    if (this.hasErrors())
        return;

    this.dateRangeSelector.setInitStateFromCurrent();
    this.registerChanges();
    this._createPrintableHeaders();
    this.generateReport(false); // do not verify if date range really changed
};

BudgetedVsActualReportMediator.prototype.setConfigDateRangeSelection = function(selection)
{
    this.reportParameters.dateSelection = selection;
};

BudgetedVsActualReportMediator.prototype._populateHeadersArray = function(headersArray)
{
    this.printableHeadersBuilder.addCPSelectionParamsStringToArray(this.reportParameters.customersProjectsSelection, headersArray); //customers and projects
    this.printableHeadersBuilder.addTaskSelectionParamsStringToArray(
            this.reportParameters.tasksConfig,
            this.reportParameters.includeNotEstimated,
            this.reportParameters.includeNotStarted,
            this.reportParameters.dateSelection,
            headersArray
    ); //tasks
};

BudgetedVsActualReportMediator.prototype.initPrintableHeadersBuilder = function(config)
{
    return new ReportPrintableHeadersBuilder().init(
            {
                reportParameters: config.reportParameters,
                cpList: config.cpList,
                isFilteredByAssignments: config.isFilteredByAssignments
            }
    );
};

BudgetedVsActualReportMediator.prototype._createPrintableDateRangeHeader = function()
{
    return this.printableHeadersBuilder.addEvaDateHeaderStringParsed(this.reportParameters.dateSelection);
};

BudgetedVsActualReportMediator.prototype.requestGenerateReport = function(successHandler)
{
    // service
    var self = this;

    function successMethod(data, textStatus, jqXHR)
    {
        if (successHandler)
            successHandler(data, textStatus, jqXHR);
        self.$exportButton.toggleClass('disabled', false);
    }

    function errorMethod(jqXHR, textStatus, errorThrown)
    {
        self.$exportButton.toggleClass('disabled', true);

        if (textStatus == "rpc_noRights")
        {
            alert(messageResource.getMessage("common.no_rights_for_operation_anymore"));
            document.location.href = ReportConfigConstants.SAVED_REPORTS_PAGE_URL;
            return true;
        }
        return false;
    }

    var params = this.makeServiceParams(this.reportParameters);
    return makeRpcCall("EvAReportService.getReport",
                       [
                           params.dateSelection,
                           EVAReport.ConvertUtils.getGroupingLevels(params),
                           params.customersProjectsSelection,
                           params.tasksConfig,
                           params.includeNotStarted,
                           params.includeNotEstimated
                       ],
                       successMethod,
                       errorMethod);
};

BudgetedVsActualReportMediator.prototype.makeServiceParams = function (reportParams) {
    var serviceParams = _.clone(reportParams);
    serviceParams.dateSelection = {
        qualifier: reportParams.dateSelection.qualifier,
        date: reportParams.dateSelection.date.toJSON()
    };

    return serviceParams;
};

BudgetedVsActualReportMediator.prototype.renderGeneratedReport = function(data)
{
    var reportParams = $.extend(true, {}, this.reportParameters);

    var info = {};
    var groupingLevels = EVAReport.ConvertUtils.getGroupingLevels(reportParams);
    var groupingLevelsCollapsed = EVAReport.ConvertUtils.getGroupingLevelsCollapsed(groupingLevels);
    for (var i = 0; i < data["groupingLevelsInfoLists"].length; ++i)
    {
        info[groupingLevelsCollapsed[i]] = data["groupingLevelsInfoLists"][i];
    }

    var customers = arrayToMapById(info[ReportGroupingLevel.CUSTOMER]);
    var projects = arrayToMapById(info[ReportGroupingLevel.PROJECT]);
    var projectNames = arrayToMapById(info[ReportGroupingLevel.PROJECT_NAME]);
    var tasks = arrayToMapById(info[ReportGroupingLevel.TASK]);
    var taskNames = arrayToMapById(info[ReportGroupingLevel.TASK_NAME]);
    var typesOfWork = arrayToMapById(info[ReportGroupingLevel.BILLING_TYPE]);
    var users = arrayToMapById(info[ReportGroupingLevel.USER]);

    this.reportPlaceholder.empty();
    EVAReport.Store.setReportData(data["rootItem"], customers, projects, tasks, users, projectNames, taskNames, typesOfWork);
    EVAReport.Store.setReportParams(reportParams);

    new EVAReport.Table(this.reportPlaceholder, EVAReport.Store).init().update();
};