function CommentsTableRender(){
    this.constructor = CommentsTableRender;
}

CommentsTableRender.prototype.init = function(tableEl, userComments, timeFormat) {

    this.timeformat = timeFormat;

    this.tableEl = tableEl; // table in light box
    this.tbodyEl = tableEl.find("tbody").first();
    this.rowTemplate = this.tableEl.find("tbody.templates").find("tr.commentsReportRow"); // row for template

    this.tooltip = new TableTooltip().init("reportCommentsPopupTooltip");
    this.tooltipTextPlaceholder = $("#reportCommentsPopupTooltip_textPlaceholder");

    this.userComments = userComments; // commentaries and user info

    this.numberFormatter = new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 2, false );

    return this;
};

// render table with user comments
CommentsTableRender.prototype.render = function() {
    this.tbodyEl.empty(); // clear tbody
    for (var i = 0; i < this.userComments.length; i++) {
        var newRow = this.newRow(this.userComments[i]);
        this.tbodyEl.append(newRow); // add row to table
    }
};

// add one row
CommentsTableRender.prototype.newRow = function( rowData ) {
    var newRow = this.rowTemplate.clone();
    this._initIdentityCell(newRow, rowData.customer, rowData.project, rowData.task);
    this._initDateCell(newRow, rowData.date);
    this._initTimeCell(newRow, rowData.time);
    this._initCommentCell(newRow, rowData.comments);
    return newRow;
};

CommentsTableRender.prototype._initIdentityCell = function( row, customer, project, task) {
    var cell = row.find(".identityCell");
    var text = customer + " / " + project + " / " + task;
    cell.append(escapeHtmlSymbols(text));
    this._initTooltip(cell);
};

CommentsTableRender.prototype._initDateCell = function( row, date ) {
    var cell = row.find(".dateCell");
    cell.append(new DateFormat(SystemSettings.dateFormat).formatDate(new Date.parseJSONDate(date)));
};

CommentsTableRender.prototype._initTimeCell = function( row, time ) {
    var cell = row.find(".timeCell");
    var timeStr;
    // time in minutes
    if (this.timeformat == TimeFormat.TIME) {
        timeStr = UserTime.formatTime(time);
    } else {
        timeStr = this.numberFormatter.format( time / 60 ); // get hours and format
    }
    cell.append( timeStr );
};

CommentsTableRender.prototype._initCommentCell = function( row, comments ) {
    var cell = row.find(".commentsCell");
    var text = escapeHtmlSymbols(comments);
    cell.append(text);
    this._initTooltip(cell);
};

CommentsTableRender.prototype._initTooltip = function(cell) {
    var self = this;
    cell.hover(function (event) {self._showTooltip(event);}, function () {self._hideTooltip();});
    cell.mousemove( function (event) {self._moveTooltip(event);} );
};

/**
 * Show tooltip only if text length more than cell
 * @param event
 * @private
 */
CommentsTableRender.prototype._showTooltip = function(event) {
    if($(event.target).outerWidth() < event.target.scrollWidth){
        this.tooltip.show();
        this.tooltipTextPlaceholder.text($(event.target).text());
        this.tooltip.move( calculateTooltipOffset( this.tooltip, event ) )
    }
};

CommentsTableRender.prototype._hideTooltip = function() {
    this.tooltip.hide();
};

CommentsTableRender.prototype._moveTooltip = function( event ) {
    this.tooltip.move( calculateTooltipOffset( this.tooltip, event ) );
};