BaseReport.FormatUtils = function() {};

BaseReport.FormatUtils.prototype = _.create(Object.prototype, {
    _dateForHeaderFormat: new DateFormat(SystemSettings.dateFormat),
    _dateForDataRowFormat: new DateFormat(SystemSettings.dateFormat + ", EEEE"),
    _dateWithYearFormat: new DateFormat("MMM dd, yyyy"),
    _weekForDataRowFormat: new DateFormat("MMM dd"),
    _monthForDataRowFormat: new DateFormat("MMMM yyyy"),
    _balanceNumberFormatter: new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 2, true),
    _timeNumberFormatter: new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 2, false),

    formatTime: function(time, timeFormat)
    {
        return time ? (timeFormat == TimeFormat.TIME ? UserTime.formatTimeWithDigitSeparator(time) : this._timeNumberFormatter.format(time / 60)) : "-"
    },

    formatPureTime: function(time, timeFormat)
    {
        if (time == null)
            return "-";
        return (timeFormat == TimeFormat.TIME ? UserTime.formatTimeWithDigitSeparator(time) : this._timeNumberFormatter.format(time / 60));
    },

    formatDifferenceTime: function(time, timeFormat)
    {
        var sign = time > 0 ? "+" : "";
        return sign + this.formatPureTime(time, timeFormat);
    },

    formatTimeFormat: function(timeFormat)
    {
        return timeFormat === TimeFormat.DECIMAL ?
               messageResource.getEscapedMessage("time_format_selector.time_format_decimal", SystemSettings.decimalSeparator) :
               messageResource.getMessage("common.hh_mm");
    },

    formatPercent: function(value)
    {
        if (value == null)
            return "-";
        return parseFloat(Math.round(value * 10) / 10).toFixed(1).toString() + "%";
    },

    formatGroupingLevel: function(groupingLevel)
    {
        switch (groupingLevel)
        {
            case ReportGroupingLevel.USER:
                return messageResource.getMessage("common.Users");
            case ReportGroupingLevel.USER_GROUP:
                return messageResource.getMessage("common.User_Groups");
            case ReportGroupingLevel.TIME_ZONE_GROUP:
                return messageResource.getMessage("common.Time_Zone_Groups");
            
            case ReportGroupingLevel.DATE:
                return messageResource.getMessage("common.Days");
            case ReportGroupingLevel.WEEK:
                return messageResource.getMessage("common.Weeks");
            case ReportGroupingLevel.MONTH:
                return messageResource.getMessage("common.Months");

            case ReportGroupingLevel.CUSTOMER:
                return messageResource.getMessage("common.Customers");
            case ReportGroupingLevel.PROJECT:
                return messageResource.getMessage("common.Projects");
            case ReportGroupingLevel.TASK:
                return messageResource.getMessage("common.Tasks");
            
            case ReportGroupingLevel.PROJECT_NAME:
                return messageResource.getMessage("report.common.projects_by_name");
            case ReportGroupingLevel.TASK_NAME:
                return messageResource.getMessage("report.common.tasks_by_name");
            case ReportGroupingLevel.BILLING_TYPE:
                return messageResource.getMessage("report.common.billing_types");
            
            case ReportGroupingLevel.NONE:
                return "";
        }
    },

    formatByName: function(namedBean)
    {
        return namedBean.name;
    },
    
    makeGroupingPresentationFlags: function (groupingLevels) {
        var groupingPresentationFlags = {};
        
        if (_.contains(groupingLevels, ReportGroupingLevel.CUSTOMER))
        {
            groupingPresentationFlags.customerIsShown = true;
        }
        
        if (_.contains(groupingLevels, ReportGroupingLevel.PROJECT))
        {
            groupingPresentationFlags.projectIsShown = true;
        }

        return groupingPresentationFlags;
    },
    
    formatTask: function(task, groupingPresentationFlags, isCSV)
    {
        var taskName = task.name;

        if (!groupingPresentationFlags.projectIsShown)
        {
            var project = task.project;
            var customer = project.customer;

            if (groupingPresentationFlags.customerIsShown)
                taskName += " (" + project.name + ")";
            else
                taskName += " (" + customer.name + ", " + project.name + ")";
        }

        if (!task.estimated && isCSV) {
            taskName += ' (' + messageResource.getMessage("report.budgeted_vs_actual.not_estimated_note") + ")";
        }

        return taskName;
    },
    
    formatProject: function(project, groupingPresentationFlags)
    {
        if (groupingPresentationFlags.customerIsShown)
            return project.name;
        
        return project.name + " (" + project.customer.name + ")";
    },
    
    formatUser: function(user)
    {
        return UserNameFormat.formatLFM(user["firstName"], user["lastName"], user["middleInitial"]);
    },

    formatUserAndTrim: function(user, maxLen)
    {
        var fn = StringUtils.cutLongText(user["firstName"], maxLen);
        var ln = StringUtils.cutLongText(user["lastName"], maxLen);
        var mn = StringUtils.cutLongText(user["middleInitial"], maxLen);
        return UserNameFormat.formatLFM(fn, ln, mn);
    },

    formatUserGroup: function(group)
    {
        return group.id !== USERS_WITHOUT_GROUP_ID ? group.name : messageResource.getMessage("report.user_groups.default_group_name");
    },

    formatTimeZoneGroup: function(group)
    {
        return group.id !== DEFAULT_TIME_ZONE_GROUP_ID ? group.name : group.name + " " + messageResource.getMessage("common.default_group");
    }
});