/**
 * @author evdokimov
 */

function LeaveTimeFormMediator()
{}

LeaveTimeFormMediator.prototype = new BaseFormMediator();
LeaveTimeFormMediator.superclass = BaseFormMediator.prototype;

LeaveTimeFormMediator.prototype.init = function(config)
{
    BaseFormMediator.prototype.init.call(this, config);
    
    this.users = config.users;
    this.userGroups = config.userGroups;
    this.timeZoneGroups = config.timeZoneGroups;
    this.validateAndUpdateUIIfInvalid();
    return this;
};

LeaveTimeFormMediator.prototype.initParamsStringBuilder = function(config)
{
    this.reportBuliderConfig = config;
    return new ReportParamsStringBuilder().init({
                                                    users: config.users,
                                                    userGroups: config.userGroups,
                                                    timeZoneGroups: config.timeZoneGroups,
                                                    leaveTypes: config.leavesData
                                                });
};

LeaveTimeFormMediator.prototype.initEditConfigController = function(config)
{
    var self = this;
    return new LeavesEditFormConfigController().init(
            {
                panelId: "mainContainer",
                reportParameters: config.reportParameters,
                users: config.users,
                userGroups: config.userGroups,
                timeZoneGroups: config.timeZoneGroups,
                leavesData: config.leavesData,
                changeCallback: function() {self.changeParametersCallback()},
                productSettingsValidator: new ATProductSettingsValidator(),
                reportRightsValidator: new ATReportRightsValidator()
            })
};

LeaveTimeFormMediator.prototype.initCsvGenerator = function()
{
    return new LeaveCSVPreviewGenerator().init();
};

LeaveTimeFormMediator.prototype.initHtmlGenerator = function()
{
    return new LeaveHTMLPreviewGenerator().init();
};

LeaveTimeFormMediator.prototype.createDateRangeSelector = function(reportParameters)
{
    var dateRangeMenuItemsUtil = new DateRangeMenuItemsUtil();
    var futureValidator = function()
    {
        return dateRangeMenuItemsUtil.isShowTTFItems(reportParameters.dateRangeSelection.qualifier);
    };

    var dateRangeSelectorConfig = {
        dateRangeSelection: reportParameters.dateRangeSelection,
        menuStates: dateRangeMenuItemsUtil.createDateRangeMenuItemsWithFuture(futureValidator), //for future items
        shortDateFormat: true,
        callOnChangeCallbackIfInvalid: true
    };

    return new DateRangeSelectorController().init(dateRangeSelectorConfig);
};

LeaveTimeFormMediator.prototype.generateCSVPreview = function(reportParameters)
{
    LeaveTimeFormMediator.superclass.generateCSVPreview.apply(this, arguments);
    this._initCSVPreviewCustomScroll(this.reportPreviewPlaceholder);
};

/**
 * NOTE: this method can be used for all csv previews (need to move it in base formMediator)
 * Init custom scroll used iScroll library
 * for ie8 used standard scroll with custom mousewhell event that prevent default scrolling and allow hor scrolling
 * @param placeholderEl placeholder element (scroll wrapper) important! first child will be scrolled
 * @private
 */
LeaveTimeFormMediator.prototype._initCSVPreviewCustomScroll = function(placeholderEl)
{
    ScrollUtils.addHorizontalScroll(placeholderEl, {isFaded: false});
};

LeaveTimeFormMediator.prototype.exportToCSV = function()
{
    var self = this;
    var params = this.collectReportParameters();
    makeRpcCall("LeavesReportService.getLeavesReport", [this.makeServiceParams(params)], function(result)
    {
        var users = mergeArraysById(result.users, this.users);
        var groups = mergeArraysById(result.userGroups, this.userGroups);
        var tzGroups = mergeArraysById(result.timeZoneGroups, this.timeZoneGroups);
        
        var data = result.rootItem;
        var generator = new LeaveCSVJsonConverter().init(users, groups, tzGroups, result.defaultWorkDayDuration);

        var jsonTitle = generator.convertTitle(params);
        var jsonTable = generator.convertTable(params, data);
        var dateRange = {
            fromDate: params.dateRangeSelection.fromDate,
            toDate: params.dateRangeSelection.toDate
        };
        self.downloadJsonAsCSV([jsonTitle, jsonTable],
                               generator.generateCSVName(params, new ATProductSettingsValidator(), UserRights.MANAGE_PTO_AND_SICK));
    },
    function(jqXHR, textStatus, errorThrown)
    {
        if (textStatus == "rpc_customException")
        {
            var messageKey = null;

            switch (errorThrown.exception)
            {
                case "com.actimind.actitime.reports.AllSelectedUsersWereDeletedException":
                    messageKey = "userlist.js.error_all_selected_useds_deleted";
                    break;
                case "com.actimind.actitime.reports.AllSelectedLeaveTypesWereDeletedException":
                    messageKey = "report.common.all_selected_leave_types_deleted";
                    break;
            }

            if (messageKey)
            {
                alert(messageResource.getMessage(messageKey));
                window.location.reload();
                return true;
            }
        }

        return false;
    });
};