function LeavesReportMediator()
{
    this.constructor = LeavesReportMediator;
}

LeavesReportMediator.prototype = new ReportMediator();

LeavesReportMediator.prototype.init = function(config)
{
    ReportMediator.prototype.init.call(this, config);

    var leaveTypes = {};
    _.each(config.leavesData, function(leaveType) { leaveTypes[leaveType.id] = leaveType; });
    LeaveTypeStore.setLeaveTypes(leaveTypes);

    var self = this;
    var oldIE = isOldIE();
    this.ExportSelector = new ExportPanel($("body"), oldIE ? null : "aTReports.leaveTime.showPDFPreview", "aTReports.leaveTime.createCSV").init();
    this.$exportButton = $(".export");
    this.$exportButton.click(function()
                       {
                           if (!self.$exportButton.hasClass('disabled'))
                               self.ExportSelector.showAt($(this));
                       });

    $(document).on("aTReports.leaveTime.createCSV", function()
    {
        self.generateCSV();
    });

    if (oldIE) // IE8 and IE9 don't support pdf preview and pdf generate
    {
        ReportUtils.generateOldIEPDFTooltip(this.ExportSelector.findPDFButton());
    }
    else
    {
        this.productSettingsValidator = new ATProductSettingsValidator();
        this.reportRightsValidator = new ATReportRightsValidator();

        new BaseReport.PDFPreviewLightboxTemplateView($("body"),
                "leaveTimeAndBalancesPdfPreviewLightbox", "lightboxWithWhiteContent pdfPreviewLightbox noprint").init();

        this.pdfPreviewLightbox = new LeavesReport.PDFPreviewLightbox().init(
                {
                    popupId: "leaveTimeAndBalancesPdfPreviewLightbox",
                    closeBtnId: "closeViewTTPdfPreviewLightboxButton",
                    eventNamespace: "leavesReport",
                    productSettingsValidator: this.productSettingsValidator,
                    reportRightsValidator : this.reportRightsValidator
                });

        $(document).on("aTReports.leaveTime.showPDFPreview", function()
        {
            self.pdfPreviewLightbox.show();
        });
        $(document).on("leavesReport.pdfPreview.lightbox.hide", function()
        {
            //todo
        });
        $(document).on("leavesReport.pdfPreview.printParametersForSaveChange", function(event, printParametersForSave)
        {
            LeavesReport.Store.setPrintParametersForSave(printParametersForSave);
        });
        $(document).on("leavesReport.pdfPreview.printParametersForGenerateChange", function(event, printParametersForGenerate)
        {
            LeavesReport.Store.setPrintParametersForGenerate(printParametersForGenerate);
            self.cancelAllPDFExportingProcesses();
            self.generatePDF();
        });
        $(document).on("leavesReport.downloadPDF", function()
        {
            new PDFGeneratorUtil().downloadPDF(LeavesReport.Store.getGeneratedPDF(), LeavesReport.Store.getPDFFileName());
        });
    }

    ATReportsStore.setVisibleUsers(config.users);
    ATReportsStore.setUserGroups(config.userGroups);
    ATReportsStore.setTimeZoneGroups(config.timeZoneGroups);
    
    if (!this.editConfigController.validate())
        this.editConfigController.updateUI(true);

    return this;
};

LeavesReportMediator.prototype.generateCSV = function()
{
    var store = LeavesReport.Store;
    var users = mergeArraysById(store.getUsers(), ATReportsStore.getVisibleUsers());
    var groups = mergeArraysById(store.getUserGroups(), ATReportsStore.getUserGroups());
    var tzGroups = mergeArraysById(store.getTimeZoneGroups(), ATReportsStore.getTimeZoneGroups());

    var generator = new LeaveCSVJsonConverter().init(users, groups, tzGroups, store.getDefaultWorkDayDuration());
    
    var jsonTitle = generator.convertTitle(LeavesReport.Store.getReportParams());
    var jsonTable = generator.convertTable(LeavesReport.Store.getReportParams(),
                                           LeavesReport.Store.getReportData());
    
    this.downloadJsonAsCSV([jsonTitle, jsonTable],
                           generator.generateCSVName(LeavesReport.Store.getReportParams(), new ATProductSettingsValidator(), UserRights.MANAGE_PTO_AND_SICK));
};

LeavesReportMediator.prototype.generatePDF = function()
{
    this.pdfGenerator = new LeavesReportPDFGenerator.PDFGenerator().init(LeavesReport.Store, ATReportsStore,
                                                                         this.productSettingsValidator, this.reportRightsValidator);

    $(document).trigger("leavesReport.pdfGenerator.start");
    var self = this;
    this.pdfGenerator.generatePDF(
            function(config, onDataProvided)
            {
                self.chartController.exportChart(config["chartImageSizes"], function(img)
                {
                    onDataProvided({
                                       chartImage: img
                                   });
                });
            }
    ).then(
            function(pdf)
            {
                LeavesReport.Store.setGeneratedPDF(pdf.encodedPDF);
                LeavesReport.Store.setPDFFileName(pdf.fileName);
            },
            function(error)
            {
                if (error.message != "abort")
                {
                    console.error(error);
                }
            });
};

LeavesReportMediator.prototype.cancelAllPDFExportingProcesses = function()
{
    if (this.pdfGenerator)
        this.pdfGenerator.interruptGeneration();
    this.pdfPreviewLightbox.cancelAllExportingProcesses();
};

LeavesReportMediator.prototype.getReportType = function()
{
    return ReportType.LEAVE_TIME;
};

LeavesReportMediator.prototype.initEditConfigController = function(config)
{
    var self = this;
    return new LeavesEditResultConfigController().init(
            {
                leavesData: config.leavesData,
                reportParameters: config.reportParameters,
                users: config.users,
                userGroups: config.userGroups,
                timeZoneGroups: config.timeZoneGroups,
                changeCallback: function() { self.changeParametersCallback(); },
                productSettingsValidator: config.productSettingsValidator,
                reportRightsValidator: config.reportRightsValidator
            });
};

// override for date range selector
LeavesReportMediator.prototype.createDateRangeSelector = function(reportParameters)
{
    var dateRangeMenuItemsUtil = new DateRangeMenuItemsUtil();
    var futureValidator = function()
    {
        return dateRangeMenuItemsUtil.isShowTTFItems(reportParameters.dateRangeSelection.qualifier);
    };

    var dateRangeSelectorConfig = {
        dateRangeSelection: reportParameters.dateRangeSelection,
        menuStates: dateRangeMenuItemsUtil.createDateRangeMenuItemsWithFuture(futureValidator), //for future items
        dateRangeParsed: true
    };

    return new DateRangeSelectorController().init(dateRangeSelectorConfig);
};

LeavesReportMediator.prototype.initPrintableHeadersBuilder = function(config)
{
    this.reportConfig = config;
    return new ReportPrintableHeadersBuilder().init(
            {
                users: config.users,
                reportParameters: config.reportParameters,
                userGroups: config.userGroups,
                timeZoneGroups: config.timeZoneGroups,
                leaveTypes: config.leavesData
            }
    );
};

LeavesReportMediator.prototype._populateHeadersArray = function(headersArray)
{
    this.printableHeadersBuilder.addStaffSelectionParamsToArray(this.reportParameters.userSelection, headersArray); //staff
    this.printableHeadersBuilder.addLeaveTypesParamsToArray(this.reportParameters.leaveTypeSelection, this.reportParameters.columnsConfig, headersArray); //leave types
};

LeavesReportMediator.prototype.requestGenerateReport = function(successHandler)
{
    var self = this;

    function successMethod(data, textStatus, jqXHR)
    {
        if (successHandler)
            successHandler(data, textStatus, jqXHR);
        self.$exportButton.toggleClass('disabled', false);
    }

    function errorMethod(jqXHR, textStatus, errorThrown)
    {
        self.$exportButton.toggleClass('disabled', true);
        self.hideOverlay();

        if (textStatus == "rpc_sessionExpired" || textStatus == "rpc_noRights")
            return false;

        if (textStatus == "rpc_customException")
        {
            var messageKey;
            switch (errorThrown.exception)
            {
                case "com.actimind.actitime.reports.NoRightsToManageBalancesException":
                    if (EnabledFeatures.PTO && EnabledFeatures.SDB)
                        messageKey = "report.js.report_loading_error.no_rights_manage_pto_and_sick";
                    else if (EnabledFeatures.PTO)
                        messageKey = "report.js.report_loading_error.no_rights_manage_pto";
                    else if (EnabledFeatures.SDB)
                        messageKey = "report.js.report_loading_error.no_rights_manage_sick";
                    break;
                case "com.actimind.actitime.reports.AllSelectedUsersWereDeletedException":
                    messageKey = "report.js.report_loading_error.no_selected_users";
                    break;
                case "com.actimind.actitime.reports.AllSelectedLeaveTypesWereDeletedException":
                    messageKey = "report.js.report_loading_error.no_selected_leave_types";
                    break;
            }

            if (messageKey)
            {
                self.showReportLoadingError(messageResource.getMessage(messageKey));
                return true;
            }
        }
        else
        {
            self.showReportLoadingError(messageResource.getMessage("report.js.report_loading_error.network_error", [PRODUCT_NAME]));
            return true;
        }

        return false;
    }
    
    return makeRpcCall("LeavesReportService.getLeavesReport", [this.makeServiceParams(this.reportParameters)], successMethod, errorMethod);
};

LeavesReportMediator.prototype.validateGeneratedReport = function(data)
{
    return true;
};

LeavesReportMediator.prototype.renderGeneratedReport = function(data)
{
    var reportParams = $.extend(true, {}, this.reportParameters);
    var info = {};
    var groupingLevels = LeavesReport.ConvertUtils.getGroupingLevels(reportParams);
    for (var i = 0; i < data["groupingLevelsInfoLists"].length; ++i) {
        var groupingType = LeavesReport.ConvertUtils.getGroupingType(i, groupingLevels);
        info[groupingType] = data["groupingLevelsInfoLists"][i];
    }

    var usersMap = arrayToMapById(info[ReportGroupingLevel.USER]);
    var userGroupsMap = arrayToMapById(info[ReportGroupingLevel.USER_GROUP]);
    var timeZoneGroupsMap = arrayToMapById(info[ReportGroupingLevel.TIME_ZONE_GROUP]);

    LeavesReport.Store.setReportData(data["rootItem"], usersMap, userGroupsMap, timeZoneGroupsMap);
    LeavesReport.Store.setReportParams(reportParams);
    LeavesReport.Store.setDefaultWorkDayDuration(data["defaultWorkDayDuration"]);

    if (this.leavesReportTable)
        this.leavesReportTable.cancelRendering();

    this.reportPlaceholder.empty();
    this.leavesReportTable = new LeavesReport.Table(this.reportPlaceholder, LeavesReport.Store).init();
    this.leavesReportTable.update();
};