function OvertimeFormMediator(){}
OvertimeFormMediator.prototype = new BaseFormMediator();

OvertimeFormMediator.prototype.init = function ( config ) {
    BaseFormMediator.prototype.init.call(this, config);

    ATReportsStore.setVisibleUsers(config.users);
    ATReportsStore.setUserGroups(config.userGroups);
    ATReportsStore.setTimeZoneGroups(config.timeZoneGroups);
    
    return this;
};

OvertimeFormMediator.prototype.initParamsStringBuilder = function ( config )
{
    return new ReportParamsStringBuilder().init( {
                                                     users: config.users,
                                                     userGroups: config.userGroups,
                                                     timeZoneGroups: config.timeZoneGroups
                                                 } );
};

OvertimeFormMediator.prototype.initEditConfigController = function ( config ) {
    var self = this;
    return new OvertimeEditFormConfigController().init(
        {
            panelId: "mainContainer",
            reportParameters: config.reportParameters,
            users: config.users,
            userGroups: config.userGroups,
            timeZoneGroups: config.timeZoneGroups,
            changeCallback: function(){ self.changeParametersCallback()}
        } )
};

OvertimeFormMediator.prototype.initCsvGenerator = function(){
    return new OvertimeCSVPreviewGenerator().init();
};

OvertimeFormMediator.prototype.initHtmlGenerator = function(){
    return new OvertimeHTMLPreviewGenerator().init();
};

OvertimeFormMediator.prototype.exportToCSV = function()
{
    var reportParams = this.collectReportParameters();

    var successMethod = function(data)
    {
        var info = {};
        var groupingLevels = LeavesReport.ConvertUtils.getGroupingLevels(reportParams);
        for (var i = 0; i < data["groupingLevelsInfoLists"].length; ++i)
        {
            var groupingType = LeavesReport.ConvertUtils.getGroupingType(i, groupingLevels);
            info[groupingType] = data["groupingLevelsInfoLists"][i];
        }

        var usersMap = arrayToMapById(info[ReportGroupingLevel.USER]);
        var userGroupsMap = arrayToMapById(info[ReportGroupingLevel.USER_GROUP]);
        var timeZoneGroupsMap = arrayToMapById(info[ReportGroupingLevel.TIME_ZONE_GROUP]);

        OvertimeReport.Store.setReportData(data["rootItem"], usersMap, userGroupsMap, timeZoneGroupsMap);
        OvertimeReport.Store.setReportParams(reportParams);
        
        return new OvertimeCSVJsonConverter().init(OvertimeReport.Store, ATReportsStore);
    };

    this._exportToCSV(reportParams, successMethod, "TimeBalanceAndOvertimeReportService.getReport", [this.makeServiceParams(reportParams)]);
};

OvertimeFormMediator.prototype.fillDateSelection = function ( reportParametersObject )
{
    reportParametersObject.dateRangeSelection = this.dateRangeSelector.getSelectedRange();
};
