function OvertimeReportMediator ()
{
    this.constructor = OvertimeReportMediator;
}

OvertimeReportMediator.prototype = new ReportMediator();

OvertimeReportMediator.prototype.init = function ( config )
{
    ReportMediator.prototype.init.call(this, config);

    this._initStore(config);
    this._initControllers();

    this.eventNamespace = "OvertimeReport";
    this.store = OvertimeReport.Store;

    var self = this;
    var oldIE = isOldIE();
    if (!oldIE)
    {
        this.productSettingsValidator = new ATProductSettingsValidator();
        this.reportRightsValidator = new ATReportRightsValidator();

        new BaseReport.PDFPreviewLightboxTemplateView($("body"), "OvertimePdfPreviewLightbox", "lightboxWithWhiteContent pdfPreviewLightbox noprint").init();

        this.pdfPreviewLightbox = new OvertimeReport.PDFPreviewLightbox().init(
                {
                    popupId: "OvertimePdfPreviewLightbox",
                    closeBtnId: "closeViewTTPdfPreviewLightboxButton",
                    eventNamespace: this.eventNamespace,
                    productSettingsValidator: this.productSettingsValidator,
                    reportRightsValidator: this.reportRightsValidator
                });

        this.initPDFEvents();
    }

    this.initCSVEvents();
    
    ATReportsStore.setVisibleUsers(config.users);
    ATReportsStore.setUserGroups(config.userGroups);
    ATReportsStore.setTimeZoneGroups(config.timeZoneGroups);
    
    return this;
};


OvertimeReportMediator.prototype._initStore = function(config)
{
    //
};

OvertimeReportMediator.prototype._initControllers = function()
{
    var self = this;
    var oldIE = isOldIE();
    this.ExportSelector = new ExportPanel($("body"), oldIE ? null : "OvertimeReport.showPDFPreview", "OvertimeReport.createCSV").init();

    this.$exportButton = $(".export");
    this.$exportButton.click(function() { if (!self.$exportButton.hasClass('disabled'))  self.ExportSelector.showAt($(this)); });

    if (oldIE) // IE8 and IE9 don't support pdf preview and pdf generate
    {
        ReportUtils.generateOldIEPDFTooltip(this.ExportSelector.findPDFButton());
    }
};

OvertimeReportMediator.prototype.generateCSV = function()
{
    var generator = new OvertimeCSVJsonConverter().init(this.store, ATReportsStore);
    var params = this.store.getReportParams();
    
    var jsonTitle = generator.convertTitle(params);
    var jsonTable = generator.convertLinearTable(params, this.store.getReportData());
    var jsonTotal = generator.convertTotal(params, this.store.getReportData());
    
    this.downloadJsonAsCSV([jsonTitle, jsonTable, jsonTotal], generator.generateCSVName(params));
};

OvertimeReportMediator.prototype.getEventName = function(event)
{
    return this.eventNamespace + "." + event;
};

OvertimeReportMediator.prototype.cancelAllPDFExportingProcesses = function()
{
    if (this.pdfGenerator)
        this.pdfGenerator.interruptGeneration();
    this.pdfPreviewLightbox.cancelAllExportingProcesses();
};

OvertimeReportMediator.prototype.generatePDF = function()
{
    this.pdfGenerator = new OvertimeReport.PDFGenerator().init(this.store, ATReportsStore, this.productSettingsValidator, this.reportRightsValidator);

    $(document).trigger(this.getEventName("pdfGenerator.start"));
    var self = this;
    this.pdfGenerator.generatePDF(function(config, onDataProvided) {}).then(
            function(pdf)
            {
                self.store.setGeneratedPDF(pdf.encodedPDF);
                self.store.setPDFFileName(pdf.fileName);
            },
            function(error)
            {
                if (error.message != "abort")
                {
                    console.error(error);
                }
            });
};

OvertimeReportMediator.prototype.getReportType = function()
{
    return ReportType.OVERTIME;
};

OvertimeReportMediator.prototype.initEditConfigController = function( config )
{
    var self = this;
    return new OvertimeEditResultConfigController().init(
            {
                reportParameters:config.reportParameters,
                users: config.users,
                userGroups: config.userGroups,
                timeZoneGroups: config.timeZoneGroups,
                changeCallback: function(){ self.changeParametersCallback(); }
            } );
};

OvertimeReportMediator.prototype.initPrintableHeadersBuilder = function( config ) {
    return new ReportPrintableHeadersBuilder().init(
            {
                users: config.users,
                reportParameters: config.reportParameters,
                userGroups: config.userGroups,
                timeZoneGroups: config.timeZoneGroups
            }
    );
};

OvertimeReportMediator.prototype._populateHeadersArray = function( headersArray ) {
    this.printableHeadersBuilder.addStaffSelectionParamsToArray( this.reportParameters.userSelection, headersArray ); //staff
};

OvertimeReportMediator.prototype.requestGenerateReport = function(successHandler)
{
    var self = this;

    function successMethod(data, textStatus, jqXHR)
    {
        if (successHandler)
            successHandler(data, textStatus, jqXHR);
        self.$exportButton.toggleClass('disabled', false);
    }

    function errorMethod(jqXHR, textStatus, errorThrown)
    {
        self.$exportButton.toggleClass('disabled', true);
        self.hideOverlay();

        if (textStatus === "rpc_noRights")
        {
            alert(messageResource.getMessage("common.no_rights_for_operation_anymore"));
            document.location.href = ReportConfigConstants.SAVED_REPORTS_PAGE_URL;
            return true;
        }

        if (textStatus == "rpc_sessionExpired")
            return false;

        if (textStatus == "rpc_customException")
        {
            if (errorThrown.exception == "com.actimind.actitime.reports.AllSelectedUsersWereDeletedException")
            {
                self.showReportLoadingError(messageResource.getMessage("report.js.report_loading_error.no_selected_users"));
                return true;
            }
        }
        else
        {
            self.showReportLoadingError(messageResource.getMessage("report.js.report_loading_error.network_error", [PRODUCT_NAME]));
            return true;
        }

        return false;
    }

    return makeRpcCall("TimeBalanceAndOvertimeReportService.getReport",
                       [this.makeServiceParams(this.reportParameters)],
                       successMethod,
                       errorMethod);
};

OvertimeReportMediator.prototype.renderGeneratedReport = function(data)
{
    var reportParams = _.clone(this.reportParameters);

    var info = {};
    var groupingLevels = LeavesReport.ConvertUtils.getGroupingLevels(reportParams);
    for (var i = 0; i < data["groupingLevelsInfoLists"].length; ++i)
    {
        var groupingType = LeavesReport.ConvertUtils.getGroupingType(i, groupingLevels);
        info[groupingType] = data["groupingLevelsInfoLists"][i];
    }

    var usersMap = arrayToMapById(info[ReportGroupingLevel.USER]);
    var userGroupsMap = arrayToMapById(info[ReportGroupingLevel.USER_GROUP]);
    var timeZoneGroupsMap = arrayToMapById(info[ReportGroupingLevel.TIME_ZONE_GROUP]);

    OvertimeReport.Store.setReportData(data["rootItem"], usersMap, userGroupsMap, timeZoneGroupsMap);
    OvertimeReport.Store.setReportParams(reportParams);

    if (this.overtimeReportTable)
        this.overtimeReportTable.cancelRendering();

    this.reportPlaceholder.empty();
    this.overtimeReportTable = new OvertimeReport.Table(this.reportPlaceholder, OvertimeReport.Store).init();
    this.overtimeReportTable.update();

    $("#overtimeExtraNotes").toggle(reportParams.columnsConfig !== OvertimeReport.ColumnsConfig.TIME_BALANCE_ONLY);
};