function ReportParamsStringBuilder ()
{
    this.constructor = ReportParamsStringBuilder;
}

ReportParamsStringBuilder.prototype.DATE_FORMAT = SystemSettings.dateFormat;

ReportParamsStringBuilder.prototype.init = function ( data, dateFormat )
{
    this.userGroupsFeatureEnabled = _.isUndefined(data.userGroupsFeatureEnabled) ? EnabledFeatures.UGR : data.userGroupsFeatureEnabled;
    this.tzGroupsFeatureEnabled = _.isUndefined(data.tzGroupsFeatureEnabled) ? EnabledFeatures.TZG : data.tzGroupsFeatureEnabled;

    this.users = data.users;
    this.userGroups = data.userGroups;
    this.timeZoneGroups = data.timeZoneGroups;
    this.cpList = data.cpList;
    this.leaveTypes = data.leaveTypes;
    this.leaveTypesMap = data.leaveTypesMap;
    this.statuses = data.statuses;

    this.groupFormatter = {
        format: function(group) {
            return group.name;
        }
    };

    this.userFormatter = {
        format: function(user) {
            return UserNameFormat.formatLFM( user.firstName, user.lastName, user.middleName );
        }
    };

    this.dateFormat = !!dateFormat ? dateFormat : this.DATE_FORMAT;
    
    return this;
};

ReportParamsStringBuilder.prototype.getUser = function( id ){
    var self = this;
    return $.grep(self.users, function(e){ return e.id == id; } ).length ? $.grep(self.users, function(e){ return e.id == id; } )[0] : null;
};

ReportParamsStringBuilder.prototype.getFilteredUsers = function( ids ){
    var self = this;
    var result = [];
    for(var i = 0; i < ids.length; i++)
    {
        if ( self.getUser(ids[i]) )
            result.push(self.getUser(ids[i]));
    }
    return result;
};

ReportParamsStringBuilder.prototype.getGroup = function( id ){
    var self = this;
    return $.grep(self.userGroups, function(e){ return e.id == id; }).length ? $.grep(self.userGroups, function(e){ return e.id == id; } )[0] : null;
};

ReportParamsStringBuilder.prototype.getTimeZoneGroup = function( id ){
    var self = this;
    return $.grep(self.timeZoneGroups, function(e){ return e.id == id; }).length ? $.grep(self.timeZoneGroups, function(e){ return e.id == id; } )[0] : null;
};

ReportParamsStringBuilder.prototype.getFilteredGroups = function( ids, USERS_WITHOUT_GROUP_ID ){
    var self = this;
    var result = [];
    for(var i = 0; i < ids.length; i++)
    {
        if(ids[i] != USERS_WITHOUT_GROUP_ID && self.getGroup(ids[i]) )
            result.push(self.getGroup(ids[i]));
    }
    return result;
};

ReportParamsStringBuilder.prototype.getFilteredTimeZoneGroups = function( ids){
    var self = this;
    var result = [];
    for(var i = 0; i < ids.length; i++)
    {
        if(self.getTimeZoneGroup(ids[i]) )
            result.push(self.getTimeZoneGroup(ids[i]));
    }
    return result;
};

ReportParamsStringBuilder.prototype.getUsersByGroup = function( groupId, groupType ){
    var self = this;
    return  $.grep(self.users, function(e){ return e[groupType == GroupType.USER_GROUP ? "userGroupId" : "timeZoneGroupId"] == groupId; } );
};

ReportParamsStringBuilder.prototype.getCustomer = function( id ){
    var self = this;
    return $.grep(self.cpList.customers, function(e){ return e.id == id; }).length ? $.grep(self.cpList.customers, function(e){ return e.id == id; } )[0] : null;
};

ReportParamsStringBuilder.prototype.getFilteredCustomers = function( ids ){
    var self = this;
    var result = [];
    for(var i = 0; i < ids.length; i++)
    {
        if ( self.getCustomer(ids[i]) )
            result.push(self.getCustomer(ids[i]));
    }
    return result;
};

ReportParamsStringBuilder.prototype.getProject = function( id ){
    var self = this;
    return $.grep(self.cpList.projects, function(e){ return e.id == id; }).length ? $.grep(self.cpList.projects, function(e){ return e.id == id; } )[0] : null;
};

ReportParamsStringBuilder.prototype.getFilteredProjects = function( ids ){
    var self = this;
    var result = [];
    for(var i = 0; i < ids.length; i++)
    {
        if ( self.getProject(ids[i]) )
            result.push(self.getProject(ids[i]));
    }
    return result;
};

ReportParamsStringBuilder.prototype.getLeaveType = function( id ){
    if (!_.isUndefined(this.leaveTypesMap))
        return this.leaveTypesMap[id];
    
    var self = this;
    return $.grep(self.leaveTypes, function(e){ return e.id == id; }).length ? $.grep(self.leaveTypes, function(e){ return e.id == id; } )[0] : null;
};

ReportParamsStringBuilder.prototype.getFilteredLeaveTypes = function( ids ){
    var self = this;
    var result = [];
    for(var i = 0; i < ids.length; i++)
    {
        if ( self.getLeaveType(ids[i]) )
            result.push(self.getLeaveType(ids[i]));
    }
    return result;
};

ReportParamsStringBuilder.prototype.getTitle = function( header, entities, formatter ) {
    var list = [];
    var MAX_ENTITIES_IN_THE_TITLE = 12;
    var BULLET_STRING = " - ";
    var ELLIPSIS = "...";

    for(var i = 0; i < entities.length; i++) {
        list.push(formatter.format(entities[i]))
    }
    var result = header;

    for(i = 0; i < list.length; i++){
        if (i >= MAX_ENTITIES_IN_THE_TITLE)
            break;

        result += "\n" + BULLET_STRING + list[i];
    }

    if( entities.length > MAX_ENTITIES_IN_THE_TITLE )
        result += "\n" + ELLIPSIS ;

    return result;
};

ReportParamsStringBuilder.prototype.joinStrings = function( col, separator, lastSeparator ) {
    if( col == null || col.length == 0 )
        return "";

    var res = col[0];

    for(var i = 1; i < col.length; i++) {
        if (i == col.length - 1)
            res += lastSeparator;
        else
            res += separator;
        res += col[i];
    }

    return res;
};

ReportParamsStringBuilder.prototype.getUserSelectionDetailsString = function( userSelection ) {
    var self = this;

    var USERS_WITHOUT_GROUP_ID = -1;
    var selectedUserWithoutUserGroup = false;
    var users = self.getFilteredUsers(userSelection.selectedUsers);

    var groups = [];
    if(userSelection.groupType == GroupType.USER_GROUP)
    {
        selectedUserWithoutUserGroup = $.inArray(USERS_WITHOUT_GROUP_ID, userSelection.selectedGroups) != -1;
        groups = self.getFilteredGroups(userSelection.selectedGroups, USERS_WITHOUT_GROUP_ID);
    }

    if(userSelection.groupType == GroupType.TIME_ZONE_GROUP)
    {
        groups = self.getFilteredTimeZoneGroups(userSelection.selectedGroups);
    }
    users = self._convertSelectedGroupsToSelectedUsersIfNecessary( users, userSelection );
    var allUserWithoutUserGroupMessage = messageResource.getMessage( "saved_reports.user_selector.all_users_without_user_group", [messageResource.getMessage("common.users_without_group")] );

    if(!userSelection.allUsers && (users.length || groups.length) && !( users.length == 1 && groups.length == 0 && ( !selectedUserWithoutUserGroup ) ))
    {
        var res = [];
        if( users.length > 0 )
        {
            res.push(self.getTitle( messageResource.getMessage("saved_reports.user_selector.title_header" ), users, self.userFormatter ));
        }
        if( groups.length > 0 )
        {
            res.push( self.getTitle(messageResource.getMessage(userSelection.groupType == GroupType.USER_GROUP ? "saved_reports.user_selector.title_header.user_groups" :
                                                               "saved_reports.user_selector.title_header.time_zone_groups"  ), groups, self.groupFormatter ) );
        }

        if( selectedUserWithoutUserGroup && self.userGroupsFeatureEnabled )
        {
            res.push(allUserWithoutUserGroupMessage);
        }

        return self.joinStrings( res, "\n", "\n" );
    }
};

ReportParamsStringBuilder.prototype.getUserSelectionClass = function( userSelection )
{
    var self = this;

    var CLASS_NAME = "users-filter";
    var ERROR_CLASS_NAME = "error-users-filter";
    var USERS_WITHOUT_GROUP_ID = -1;

    var users = this.getFilteredUsers(userSelection.selectedUsers);
    var selectedUserWithoutUserGroup = false;
    var groups = [];
    if(userSelection.groupType == GroupType.USER_GROUP)
    {
        groups = self.getFilteredGroups(userSelection.selectedGroups, USERS_WITHOUT_GROUP_ID);
        selectedUserWithoutUserGroup = $.inArray(USERS_WITHOUT_GROUP_ID, userSelection.selectedGroups) != -1;
    }
    if(userSelection.groupType == GroupType.TIME_ZONE_GROUP)
        groups = self.getFilteredTimeZoneGroups(userSelection.selectedGroups);

    users = self._convertSelectedGroupsToSelectedUsersIfNecessary( users, userSelection );


    if( !userSelection.allUsers && users.length == 0 && groups.length == 0 && !selectedUserWithoutUserGroup )
        return CLASS_NAME + " " + ERROR_CLASS_NAME;
    else
        return CLASS_NAME;
};

ReportParamsStringBuilder.prototype.getUserSelectionParamString = function( userSelection, assignedInfo ) {
    var self = this;
    var USERS_WITHOUT_GROUP_ID = SystemSettings.usersWithoutGroupId;
    if(userSelection.allUsers) {
        if (!_.isUndefined(assignedInfo) && assignedInfo.isShowOnlyAssigned )
        {
            return messageResource.getMessage("common.report.all_staff.assigned_to_me", [assignedInfo.viewerInfo.formatNameFML()]);
        } else
        {
            return messageResource.getMessage("common.pdf.formatting.all_staff");
        }
    } else {
        var selectedUserWithoutUserGroup = false;
        var users = self.getFilteredUsers(userSelection.selectedUsers);
        var groups = [];
        var groupType = userSelection.groupType;

        if(userSelection.groupType == GroupType.USER_GROUP)
        {
            groups = self.getFilteredGroups(userSelection.selectedGroups, USERS_WITHOUT_GROUP_ID);
            selectedUserWithoutUserGroup = $.inArray(USERS_WITHOUT_GROUP_ID, userSelection.selectedGroups) != -1;
        }
        if(userSelection.groupType == GroupType.TIME_ZONE_GROUP)
            groups = self.getFilteredTimeZoneGroups(userSelection.selectedGroups);


        users = this._convertSelectedGroupsToSelectedUsersIfNecessary( users, userSelection );

        var allUserWithoutUserGroupMessage = messageResource.getMessage( "saved_reports.user_selector.all_users_without_user_group", [messageResource.getMessage("common.users_without_group")] );
        if (users.length == 0)
        {
            if (groups.length == 0 && !selectedUserWithoutUserGroup )
            {
                return messageResource.getMessage( "usersselector.no_users_selected" );
            }
            if( groups.length == 1 && ( !selectedUserWithoutUserGroup ))
            {
                return messageResource.getMessage(groupType == GroupType.USER_GROUP ? "common.User_Group":"common.Time_Zone_Group") + " \"" + self.groupFormatter.format( groups[0] ) + "\"";
            }
            if (groups.length == 0 && selectedUserWithoutUserGroup)
            {
                return  allUserWithoutUserGroupMessage;
            }
        }
        if( users.length == 1 && groups.length == 0 && ( !selectedUserWithoutUserGroup ) )
            return self.userFormatter.format( users[0] );

        var res = [];

        if( users.length > 0 )
        {
            res.push( users.length == 1 ? messageResource.getMessage("saved_reports.user_selector.one_user") : messageResource.getMessage("saved_reports.user_selector.multiple_users", [users.length.toString()]));
        }
        if( groups.length > 0 )
        {
            res.push( groups.length + " " + messageResource.getMessage( (groupType == GroupType.USER_GROUP ? "common.user_group" : "common.time_zone.group")+( groups.length == 1 ? "" : "s" )));
        }

        if( selectedUserWithoutUserGroup && self.userGroupsFeatureEnabled )
        {
            res.push(allUserWithoutUserGroupMessage.toLowerCase());
        }

        return self.joinStrings( res, ", ", " " + messageResource.getMessage( "report.common.and_staff_delimiter" ) + " " );
    }
};

ReportParamsStringBuilder.prototype._convertSelectedGroupsToSelectedUsersIfNecessary = function( users, userSelection )
{
    var convertedUsers = UserSelectorUtils.convertSelectedGroupsIntoSelectedUsersIfNecessary(this.users, userSelection.groupType, userSelection.allUsers, 
                                                                                           userSelection.showDisabled, userSelection.selectedGroups, this.userGroupsFeatureEnabled, this.tzGroupsFeatureEnabled );
    
    return $.merge(users, convertedUsers);
};

ReportParamsStringBuilder.prototype.getCPSelectionParamString = function( cpSelection ) {
    var self = this;

    var AND_DELIMITER = " & ";

    var customerFormatter = {
        format: function(customer) {
            return customer.name;
        }
    };

    var projectFormatter = {
        format: function(project) {
            return project.name;
        }
    };

    if (cpSelection.qualifier == CustomersProjectsSelectionQualifier.ALL)
        return messageResource.getMessage("saved_reports.cp_selector.all_projects");

    if( cpSelection.qualifier == CustomersProjectsSelectionQualifier.ALL_ACTIVE )
        return messageResource.getMessage("saved_reports.cp_selector.all_active_projects");
    else {
        var customers = self.getFilteredCustomers(cpSelection.customers);
        var projects = self.getFilteredProjects(cpSelection.projects);

        if( customers.length == 0 && projects.length == 0 )
            return messageResource.getMessage( "saved_reports.cp_selector.emty_selection" ); //error_class

        var message;

        if( customers.length == 1 && projects.length == 0 )
        {
            message = messageResource.getMessage( "saved_reports.cp_selector.single_element.customer",
                                                  [customerFormatter.format( customers[0] )] );
            return message;
        }

        if( projects.length == 1 && customers.length == 0 )
        {
            message = messageResource.getMessage( "saved_reports.cp_selector.single_element.project",
                                                  [projectFormatter.format( projects[0] )] );
            return message;
        }


        var getCustomersPart = function (customers, availableSuffix, activeProjectsSuffix)
        {
            return messageResource.getMessage( "saved_reports.cp_selector.multiple_customer" + ( customers.length == 1 ? "" : "s" ), [customers.length.toString() ,
                                                                                                                                                       availableSuffix, activeProjectsSuffix] );
        };

        var getProjectsPart = function (projects)
        {
            return messageResource.getMessage( "saved_reports.cp_selector.multiple_project" + ( projects.length == 1 ? "" : "s" ), [projects.length.toString()] );
        };

        var customersPart = "", projectsPart = "";
        if (customers.length > 0)
        {
            customersPart = getCustomersPart(customers, "", "");
        }

        if (projects.length > 0)
        {
            projectsPart = getProjectsPart( projects );
        }
        return customersPart + (customersPart.length > 0 && projectsPart.length > 0 ? AND_DELIMITER : "") + projectsPart;
    }
};

ReportParamsStringBuilder.prototype.getStatusesSelectionParamString = function (weekApprovalStatuses)
{
    if (weekApprovalStatuses.length == 0) 
    {
        return messageResource.getMessage("common.selector.statuses.no_tt_statuses");
    }
    
    var showApproved = _.contains(weekApprovalStatuses, WeekApprovalStatus.APPROVED);
    var showReady = _.contains(weekApprovalStatuses, WeekApprovalStatus.READY);
    var showNotReady = _.contains(weekApprovalStatuses, WeekApprovalStatus.NOT_READY);
    var showRejected = _.contains(weekApprovalStatuses, WeekApprovalStatus.REJECTED);
    
    if (showApproved && showReady && showNotReady && showRejected)
        return messageResource.getMessage("common.selector.statuses.all");

    var messages = [];
    
    if (showApproved)
        messages.push(messageResource.getMessage("common.selector.statuses.approved"));
    
    if (showReady)
        messages.push(messageResource.getMessage("common.selector.statuses.ready"));
    
    if (showNotReady && showRejected)
        messages.push(messageResource.getMessage("common.selector.statuses.not_ready_and_rejected"));
    else if (showNotReady)
        messages.push(messageResource.getMessage("common.selector.statuses.not_ready"));
    else if (showRejected)
        messages.push(messageResource.getMessage("common.selector.statuses.rejected"));
    
    return messages.join(", ") + " " + messageResource.getMessage("common.selector.statuses.tt_statuses");
};

ReportParamsStringBuilder.prototype.getLeaveTypeSelectionParamString = function ( leaveTypeSelection )
{
    var self = this;

    var leaveTypesFormatter = {
        format: function ( leaveType )
        {
            return leaveType.name;
        }
    };

    if( leaveTypeSelection.qualifier == LeaveTypeSelectionQualifier.ALL )
        return messageResource.getMessage( "leaves_selector.options.all_items" );
    else
    {
        var leaveTypes = self.getFilteredLeaveTypes( leaveTypeSelection.leaveTypes );

        if( leaveTypes.length == 0 )
            return messageResource.getMessage( "report.leaves.no_leave_types" ); //error_class

        if( leaveTypes.length == 1 )
            return messageResource.getMessage( "saved_reports.leaves_selector.single_element",
                                               [leaveTypesFormatter.format( leaveTypes[0] )] );

        return messageResource.getMessage( "saved_reports.leaves_selector.multiple_types", [leaveTypes.length.toString()] );

    }
};

ReportParamsStringBuilder.prototype.getLeaveTypeSelectionClass = function ( leaveTypeSelection )
{
    var self = this;

    var CLASS_NAME = "leaves-filter";
    var ERROR_CLASS_NAME = "error-leaves-filter";

    if( leaveTypeSelection.qualifier == LeaveTypeSelectionQualifier.SELECTED )
    {
        var leaveTypes = self.getFilteredLeaveTypes( leaveTypeSelection.leaveTypes );

        if( leaveTypes.length == 0 )
            return CLASS_NAME + " " + ERROR_CLASS_NAME;
    }
    return CLASS_NAME;
};

ReportParamsStringBuilder.prototype.getLeaveTypeSelectionDetailsString = function ( leaveTypeSelection )
{
    var self = this;

    var leaveTypesFormatter = {
        format: function ( leaveType )
        {
            return leaveType.name;
        }
    };

    if( leaveTypeSelection.qualifier != LeaveTypeSelectionQualifier.ALL)
    {
        var leaveTypes = self.getFilteredLeaveTypes( leaveTypeSelection.leaveTypes );

        if( leaveTypes.length == 0 || leaveTypes.length == 1 )
            return ""; //error_class


        return this.getTitle(messageResource.getMessage( "saved_reports.leaves_selector.title_header"), leaveTypes,
                                                              leaveTypesFormatter);


    }
};

ReportParamsStringBuilder.prototype.getEvaDateParamString = function ( dateSelection )
{
    if ( dateSelection.qualifier != DateRangeSelectionQualifier.CUSTOM ) {
        return messageResource.getMessage(EvaDateStateUtil[dateSelection.qualifier]);
    }
    var date = dateSelection.date.parseJSONDate();
    return new DateFormat(this.dateFormat).formatDate(date) + ", " + Date.dayNames[date.getDay()];
};

ReportParamsStringBuilder.prototype.getEvaDateDetailsString = function ( dateSelection )
{
    if ( dateSelection.qualifier != DateRangeSelectionQualifier.CUSTOM ) {
        var date = dateSelection.date.parseJSONDate();
        return new DateFormat(this.dateFormat).formatDate(date) + ", " + Date.dayNames[date.getDay()];
    }
    return "";
};

ReportParamsStringBuilder.prototype.getEvaDateCSVString = function ( dateSelection )
{
    var date = dateSelection.date;
    return new DateFormat(this.dateFormat).formatDate( date ) + ", " + Date.dayNames[date.getDay()];
};

ReportParamsStringBuilder.prototype.getDateRangeParamString = function ( dateRangeSelection ) {
    if ( dateRangeSelection.qualifier != DateRangeSelectionQualifier.CUSTOM ) {
      return messageResource.getMessage(DateRangeStateUtil[dateRangeSelection.qualifier]);
    }
    
    var fromDate = dateRangeSelection.fromDate.parseJSONDate();
    var toDate = dateRangeSelection.toDate.parseJSONDate();

    var dateRange = new DateRange( fromDate, toDate );
    if (dateRange.isOneDay())
        return new DateFormat(this.dateFormat).formatDate(fromDate) + ", " + Date.dayNames[fromDate.getDay()];
    
    return messageResource.getMessage("range.custom.pattern", [
        new DateFormat(this.dateFormat).formatDate(fromDate),
        new DateFormat(this.dateFormat).formatDate(toDate) ]);
};

ReportParamsStringBuilder.prototype.getDateRangeDetailsString = function ( dateRangeSelection )
{
    if ( dateRangeSelection.qualifier == DateRangeSelectionQualifier.CUSTOM )
        return "";
    
    return messageResource.getMessage( "range.custom.pattern", [
        new DateFormat(this.dateFormat).formatDate( dateRangeSelection.fromDate.parseJSONDate() ),
        new DateFormat(this.dateFormat).formatDate( dateRangeSelection.toDate.parseJSONDate() ) ] );
};

ReportParamsStringBuilder.prototype.getDateRangeCSVString = function ( dateRangeSelection )
{
    var fromDate = this.convertToJSON(dateRangeSelection.fromDate);
    var toDate = this.convertToJSON(dateRangeSelection.toDate);

    var dateRange = new DateRange( fromDate, toDate );

    if ( dateRange.isOneDay() )
        return new DateFormat(this.dateFormat).formatDate( fromDate ) + ", " + Date.dayNames[fromDate.getDay()];

    return messageResource.getMessage( "range.custom.detailed_string", [
        new DateFormat(this.dateFormat).formatDate( fromDate ) + ", " + Date.dayNames[fromDate.getDay()],
        new DateFormat(this.dateFormat).formatDate( toDate ) + ", " + Date.dayNames[toDate.getDay()] ] );


};

ReportParamsStringBuilder.prototype.getEvATaskDetailsString = function ( taskParams, dateSelection )
{
    var mainInfo = "", additionalInfo="";
    switch( taskParams.tasksConfig )
    {
        case "ALL":
            mainInfo = messageResource.getMessage("report.budgeted_vs_actual.filter_view.open_and_completed_tasks");
            break;
        case "OPEN_ONLY":
            mainInfo = messageResource.getMessage("report.budgeted_vs_actual.filter_view.open_tasks");
            break;
        case "COMPLETED_ONLY":
            mainInfo = messageResource.getMessage("report.budgeted_vs_actual.filter_view.completed_tasks");
    }

    if (!taskParams.includeNotEstimated && !taskParams.includeNotStarted)
        additionalInfo = messageResource.getMessage("report.budgeted_vs_actual.except_started_after_and_not_estimated", [this.getEvaDateCSVString(dateSelection)]);
    else if (!taskParams.includeNotStarted)
        additionalInfo = messageResource.getMessage("report.budgeted_vs_actual.except_started_after", [this.getEvaDateCSVString(dateSelection)]);
    else if (!taskParams.includeNotEstimated)
        additionalInfo = messageResource.getMessage("report.budgeted_vs_actual.except_not_estimated");

    return mainInfo + " " + additionalInfo;
};

ReportParamsStringBuilder.prototype.getBalancesParamString = function (enabledPTO, enabledSick)
{
    if (enabledPTO && enabledSick)
        return messageResource.getMessage("common.leave.reports.pdf.header.balances");

    if (enabledPTO)
        return messageResource.getMessage("common.leave.reports.pdf.header.pto");

    if (enabledSick)
        return messageResource.getMessage("common.leave.reports.pdf.header.sick");

    return null;
};

ReportParamsStringBuilder.prototype.convertToJSON = function(date)
{
    if (_.isString(date)) {
        return date.parseJSONDate();
    }
    return date;
};
