
function SaveReportManager(){}

SaveReportManager.prototype.init = function( saveConfigControl, successHandler, errorHandler )
{
    this.saveConfigControl = saveConfigControl;
    this.successHandler = successHandler;
    this.errorHandler = errorHandler;
    return this;
};

SaveReportManager.prototype.isChart = function(){
    return false;
};

SaveReportManager.prototype.commonSuccessSaveHandler =function(newConfigName){
    // enable controls/ hide spinner
    this.saveConfigControl.cancelSaving();
    this.saveConfigControl.setReportConfigName( newConfigName );

    this.saveConfigControl.setStateAfterSave();
};

SaveReportManager.prototype.saveReport = function(reportConfig){
    var self = this;
    this.saveConfigControl.startSaving();
    this.saveNewConfig(reportConfig,    function(result){
                                            self.commonSuccessSaveHandler(reportConfig.reportName);
                                            self.successHandler( result, reportConfig.reportName );
                                        },
                                          function(jqXHR, textStatus, errorThrown) {
                                              self.saveConfigControl.cancelSaving();
                                              if( errorThrown.exception && errorThrown.exception.indexOf( 'com.actimind.actitime.reports.ReportConfigExistsException' ) != -1 )
                                              {
                                                  self.saveConfigControl.showError( SaveConfigurationControl.DUPLICATE_NAME );
                                                  return true;
                                              }

                                              if( self.errorHandler(errorThrown) )
                                                  return true;
                                         });
};

SaveReportManager.prototype.updateReport = function(reportConfig){
    var self = this;
    this.saveConfigControl.startSaving();
    this.updateConfig(reportConfig,   function(){
                                          self.commonSuccessSaveHandler(reportConfig.reportName);
                                          self.successHandler(undefined, reportConfig.reportName);
                                      },
                                      function(jqXHR, textStatus, errorThrown) {
                                          self.saveConfigControl.cancelSaving();
                                          if( errorThrown.exception && errorThrown.exception.indexOf( 'com.actimind.actitime.reports.ReportConfigExistsException' ) != -1 )
                                          {
                                              self.saveConfigControl.showError( SaveConfigurationControl.DUPLICATE_NAME );
                                              return true;
                                          }

                                          if( errorThrown.exception && errorThrown.exception.indexOf( 'com.actimind.actitime.reports.NoSuchReportConfigException' ) != -1 )
                                          {
                                              self.reportConfigId = null;
                                              if(confirm(messageResource.getMessage("report.common.no_such_configuration",
                                                                                    [self.isChart()? messageResource.getMessage("report.dashboard.chart") :
                                                                                                     messageResource.getMessage("report.dashboard.report")]))){
                                                  self.saveConfigControl.setStateNewEdit();
                                                  self.saveConfigControl.setReportConfigName(reportConfig.reportName);
                                                  self.saveReport(reportConfig);
                                              }
                                              else
                                              {
                                                  self.saveConfigControl.setStateNew();
                                              }
                                              return true;
                                          }

                                          if( self.errorHandler(errorThrown) )
                                              return true;
                                      });
};

SaveReportManager.prototype.saveNewConfig = function( reportConfig, successHandler, errorHandler )
{
    var self = this;
    makeRpcCall(
            this.getServiceSaveName(),
            this.getServiceParamsForSave(reportConfig),
            function( result )
            {
                return successHandler(result);
            },
            function ( jqXHR, textStatus, errorThrown ){
                if(errorHandler){
                    return errorHandler( jqXHR, textStatus, errorThrown )
                }
            }
    );
};

SaveReportManager.prototype.updateConfig = function( configName, successHandler, errorHandler)
{
    makeRpcCall(
            this.getServiceUpdateName(), this.getServiceParamsForUpdate(configName),
            function()
            {
                return successHandler();
            },
            function ( jqXHR, textStatus, errorThrown ){
                if(errorHandler){
                    return  errorHandler( jqXHR, textStatus, errorThrown );
                }
            }
    );
};

SaveReportManager.prototype.getServiceSaveName = function(){
    return "SavedReportConfigsService.saveConfig"
};

SaveReportManager.prototype.getServiceUpdateName = function(){
    return "SavedReportConfigsService.updateConfig"
};

SaveReportManager.prototype.getServiceParamsForSave = function(reportConfig){
    return  [reportConfig.reportType, reportConfig.reportParameters, reportConfig.reportName]
};

SaveReportManager.prototype.getServiceParamsForUpdate = function(reportConfig){
    return  [reportConfig.reportId, reportConfig.reportType,  reportConfig.reportParameters, reportConfig.reportName];
};