function ChartController() {
}

ChartController.prototype.init = function( config )
{
    this.containerId = config.containerId;
    this.partsContainer = $("#" + config.partsContainerId);
    this.chartFormatter = new ChartFormatterCreator();
    this.currentData = config.data;
    this.currentState = config.settings;
    this._extendCurrentStateWithFormattedFunctions( config.settings.groupingLevel, config.settings.reportChartType );

    if( this.currentState.sortOption == ChartSortOptions.BY_VALUES)
        this.sortDataByValues( this.currentState.sortOrder, this.currentState.sortCategoryComparator);
    else
        this.sortDataByCategory( this.currentState.sortOrder,  this.currentState.sortCategoryComparator);

    this.numberFormatter = new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 2, false);

    if (!this.currentData.chartPoints || !this.currentData.chartPoints.length)
        return this;

    this.chart =  this.showChart( this.currentData, this.currentState);

    return this;
};

ChartController.prototype.gerBarChartConfiguration = function( dataProvider, title, settings ) {
    var self = this;
    var total = this.currentData.total;
    var conf = {
        "type": "serial",
        "rotate" : settings.horizontal,
        "categoryField": "category",
        "startDuration": settings.withoutAnimation ? 0 : 0.3,
        "startEffect": "easeOutSine",
        "sequencedAnimation": false,
        "theme": AmCharts.themes["light"],
        "fontSize": settings.fontSize ? settings.fontSize : 11,
        "chartCursor": {
            "categoryBalloonFunction": settings.horizontal ? self.currentState.formatHBarChartCursorLabelFunction : self.currentState.formatVBarChartCursorLabelFunction
        },
        "categoryAxis": {
            "color": "#414141",
            "labelRotation": 48,
            "labelFrequency": 1,
            "minHorizontalGap": 65,
            "minVerticalGap" : 50,
            "labelOffset": settings.horizontal ? 11 : 0,
            "categoryFunction" : self.currentState.formatCategoryFunction
        },
        "chartScrollbar": settings.withoutScroll ?
                          false :
                          {
                              "scrollbarHeight": 16,
                              "dragIconHeight": 30,
                              "dragIconWidth": 30,
                              "backgroundColor" : "#f9f9f9",
                              "backgroundAlpha" : 1,
                              "selectedBackgroundAlpha" : 1,
                              "selectedBackgroundColor" : "#e8e8e8"
                          } ,
        "graphs": [
            {
                "fillAlphas": 1,
                "fillColors": self.currentState.color,
                "labelPosition": "middle",
                "labelRotation": settings.horizontal ? 0 : 270,
                "labelText": "[[value]]",
                "labelFunction": function( item )
                {
                    return self.currentState.formatValueFunction(item.dataContext.value) +
                        (self.currentState.showPercents && item.dataContext.value.toString().length ?
                        " " + "(" + self.numberFormatter.format(total == 0 ? 0.00 : (item.dataContext.value * 100  / total)) + "%)" : "");
                },
                "balloonFunction": function( item )
                {
                    return self.currentState.formatValueFunction(item.dataContext.value);
                },
                "lineThickness": 0,
                "type": "column",
                "valueField": "value"
            }
        ],
        "valueAxes": [
            {
                "gridCount": 5,
                "labelFrequency": 1,
                "autoGridCount": false,
                "includeAllValues": true,
                "minHorizontalGap": 70,
                "labelFunction": self.currentState.formatValueFunction,
                "fontSize": 11,
                "title": title
            }
        ],
        "titles": [
            {
                "text": ""
            }
        ],
        "dataProvider": dataProvider,
        "export": {
            "enabled": true,
            "menu": [],
            "libs" : {
                "autoLoad": false
            }
        }
    };

    if( settings.minimum )
        conf.valueAxes[0].minimum = settings.minimum;
    else if (this.currentData.min > 0)
        conf.valueAxes[0].minimum = 0;
    else
        conf.valueAxes[0].minimum = this.currentData.min * 1.1;

    if( settings.maximum )
        conf.valueAxes[0].maximum = settings.maximum;
    else if (this.currentData.max < 0)
        conf.valueAxes[0].maximum = 0;
    else
        conf.valueAxes[0].maximum = this.currentData.max * 1.1;

    return conf;
};

ChartController.prototype.showChart = function( data, currentState ) {
    switch (currentState.chartType) {
        case ChartType.HORIZONTAL_BAR:
            return this.buildBarChart( this.containerId,
                                       this.gerBarChartConfiguration(
                                               data.chartPoints,
                                               data.title,
                                               {
                                                   horizontal: true
                                               } ) );
        case ChartType.VERTICAL_BAR:
            return this.buildBarChart( this.containerId,
                                       this.gerBarChartConfiguration(
                                               data.chartPoints,
                                               data.title,
                                               { } ) );
        case ChartType.PIE_CHART:
        {
            if (data.max == 0 && data.min == 0)
                this.currentState.emptyPieCallback();
            return this.buildPieChart( this.containerId,
                                       this.buildPlainDataProvider( data.min, data.max ),
                                       11,
                                       290,
                                       true);
        }
    }
};

ChartController.prototype.buildPlainDataProvider = function( minValue, maxValue ) {
    var self = this;
    var plainDataProvider = [];

    var abs = !isNaN(maxValue) && !isNaN(minValue); //for chart we need positive values but for data-tables we need values "as-is"
    var filter = abs && (minValue < 0 && maxValue > 0);
    $.each( self.currentData.chartPoints, function ( i, item )
    {
        if ( !filter || item.value > 0)
            plainDataProvider.push( {
                                    category: self.currentState.toStringFunction( item.category ),
                                    value: abs ? Math.abs(item.value) : item.value,
                                    formattedValue: self.currentState.formatValueFunction( item.value )
                                    } );
    } );
    return plainDataProvider;
};

ChartController.prototype.buildPieChart = function( container, dataProvider, fontSize, maxLabelWidth, animate, onRenderedHandler ) {
    var self = this;
    var chartTheme = self.currentState.theme? AmCharts.themes[self.currentState.theme] : AmCharts.themes["light"];
    var chart = new AmCharts.AmPieChart( chartTheme );
    if( onRenderedHandler )
        chart.addListener( "rendered", function ( e )
        {
            onRenderedHandler( e );
        } );
    chart.addListener( "drawn", function ( e )
    {
        if (document.createElementNS && e.chart.realHeight > 0 && e.chart.realWidth > 0 ) {
            var rectangle= document.createElementNS('http://www.w3.org/2000/svg', "rect");
            rectangle.setAttribute("width", e.chart.realWidth);
            rectangle.setAttribute("height", e.chart.realHeight * 0.08);
            rectangle.setAttribute("fill", "white");
            $(e.chart.chartDiv).find(".amcharts-title").before(rectangle);
        }
    } );

    AmCharts.extend( chart, {
        "type": "pie",
        "startDuration": animate ? 0.6 : 0,
        "startEffect": "easeOutSine",
        "startRadius": 120,
        "sequencedAnimation": false,
        "balloon": {
          "maxWidth": 280
        },
        "balloonFunction": function ( item )
        {
            return escapeHtmlSymbols( item.dataContext.category ) +
                   "<br><span style='font-size:14px'><b>" +
                   item.dataContext.formattedValue +
                   " </b>(" +
                   self.numberFormatter.format(item.percents) +
                   "%)</span>";
        },
        "dataProvider": dataProvider,
        "titleField": "category",
        "valueField": "value",
        "outlineThickness": 1,
        "outlineAlpha": 1,
        "fontSize" : fontSize,
        "labelRadius": 20,
        "maxLabelWidth": maxLabelWidth,
        "hideLabelsPercent": 1.5,
        "labelFunction" : self.currentState.formatLabelFunction,
        "radius": "30%",
        "innerRadius": "38%",
        "titles": [
            {
                "text": self.currentData.title
            }
        ],
        "export": {
            "enabled": true,
            "menu" : [],
            "libs" : {
                "autoLoad": false
            }
        }
    } );

    if (self.currentState.theme == "none")
        chart.colors = ["#FF0F00","#B0DE09","#FF9E01","#0D8ECF","#FCD202","#9e7beb","#F8FF01","#4bd291","#0D52D1","#FF6600","#04D215","#8A0CCF","#CD0D74",
                  "#754DEB","#DDDDDD","#333333","#999999","#57032A","#CA9726","#990000","#4B0C25"];


    AmCharts.isReady ?
    chart.write( container )
            : AmCharts.ready( function ()
                              {
                                  chart.write( container );
                              } );

    return chart;
};

ChartController.prototype.buildBarChart = function( container, settings, onRenderedHandler ) {

    var theme = AmCharts.themes["light"];
    var chart = new AmCharts.AmSerialChart( theme );
    var self = this;
    if( onRenderedHandler ){
        chart.addListener( "rendered", function ( e )
        {
            onRenderedHandler( e );
        } );
    } else
    {
        chart.addListener( "zoomed", function ( e )
        {
            self.currentBarCount = e.endIndex - e.startIndex + 1;
        } );
    }

    AmCharts.extend( chart, settings );

    AmCharts.isReady ?
    chart.write( container )
            : AmCharts.ready( function ()
                              {
                                  chart.write( container );
                              } );
    return chart;
};

ChartController.prototype.sortByValues = function ( asc, secondaryComparator ) {
    this.currentState.sortOrder = asc;
    this.sortDataByValues(asc, secondaryComparator);
    this.chart = this.showChart( this.currentData, this.currentState );
};

ChartController.prototype.sortDataByValues = function ( asc, secondaryComparator ) {
    this.currentData.chartPoints.sort(
        function(x, y) {
            if(x.value == y.value){
                return secondaryComparator(x.category, y.category);
            }
            return x.value < y.value ? (asc ?  -1: 1) : (asc ?  1: -1)
        }
    );
};

ChartController.prototype.sortByCategory = function ( asc, comparator) {
    this.currentState.sortCategoryComparator = comparator;
    this.currentState.sortOrder = asc;
    this.sortDataByCategory(asc, comparator);
    this.chart = this.showChart( this.currentData, this.currentState );
};

ChartController.prototype.sortDataByCategory = function(asc, comparator){
    this.currentData.chartPoints.sort( function(x, y) { return asc ? comparator( x.category, y.category) : -comparator( x.category, y.category) } );
};

ChartController.prototype.changeChartType = function ( type ) {
    this.currentState.chartType = type;
    return this.showChart( this.currentData, this.currentState );
};

ChartController.prototype.changeChartColor = function ( color ) {
    this.currentState.color = color;
    this.chart = this.showChart( this.currentData, this.currentState );
};

ChartController.prototype.changeChartTheme = function ( theme ) {
    this.currentState.theme = theme;
    this.chart = this.showChart( this.currentData, this.currentState );
};

ChartController.prototype.changeShowPercents = function ( isShown ) {
    this.currentState.showPercents = isShown;
    this.chart = this.showChart( this.currentData, this.currentState );
};

ChartController.prototype.getCurrentDataProvider = function () {
    return this.chart.dataProvider;
};

ChartController.prototype.updateDisplayData = function(config){
    this.currentState.color = config.color;
    this.currentState.theme = config.theme;
    this.currentState.chartType = config.chartType;
    this.currentState.sortOption = config.sortOption;
    this.currentState.sortOrder = config.sortOrder;
    this.currentState.showPercents = config.isShowPercents;
    if( this.currentState.sortOption == ChartSortOptions.BY_VALUES)
        this.sortDataByValues( this.currentState.sortOrder, this.currentState.sortCategoryComparator );
    else
        this.sortDataByCategory( this.currentState.sortOrder,  this.currentState.sortCategoryComparator);

    this.chart = this.showChart( this.currentData, this.currentState );
};

ChartController.prototype.updateData = function ( data, state ) {
    this.currentData = data;
    this.currentState.sortCategoryComparator =state.sortCategoryComparator;
    this.currentState.sortOption = state.sortOption;
    this.currentState.sortOrder = state.sortOrder;
    this.currentState.showPercents = state.isShowPercents;
    this._extendCurrentStateWithFormattedFunctions( state.groupingLevel, state.reportChartType );

    if( this.currentState.sortOption == ChartSortOptions.BY_VALUES)
        this.sortDataByValues( this.currentState.sortOrder, this.currentState.sortCategoryComparator);
    else
        this.sortDataByCategory( this.currentState.sortOrder,  this.currentState.sortCategoryComparator);

    this.chart = this.showChart( this.currentData, this.currentState );
};

ChartController.prototype.exportChart = function( size, callback, barCount ) {
    this.partsContainer.width(size.width);
    this.partsContainer.height(size.height);

    var breaker = { 'break': false };
    this.currentExportBreaker = breaker;

    switch (this.currentState.chartType) {
        case ChartType.HORIZONTAL_BAR:
            this._exportBarChart( barCount, true, breaker, callback );
            return;
        case ChartType.VERTICAL_BAR:
            this._exportBarChart( barCount, false, breaker, callback );
            return;
        case ChartType.PIE_CHART:
            this._exportPieChart( callback );
            return;
    }
};

ChartController.prototype._exportPieChart = function( callback ) {
        var result = [{
        data: this.buildPlainDataProvider()
    }];

    var getIndexOf = function(array, item) {
        var r = $.map(array, function(obj, index) {
            if(Math.abs(obj.value) == Math.abs(item.value) && obj.category== item.category) {
                return index;
            }
        });
        return r.length ? r[0] : -1;
    };

    this.buildPieChart( "chartPart",
                        this.buildPlainDataProvider(this.currentData.min, this.currentData.max),
                        14,
                        325,
                        false,
                        function ( e )
                        {
                            $.each( e.chart.chartData, function ( i, item ) {
                                var index = getIndexOf(result[0].data, item.dataContext);
                                if (index >= 0)
                                    result[0].data[index].color = item.color;
                            } );
                            var interval = setInterval( function ()
                                                        {
                                                            if( window.fabric )
                                                            {
                                                                clearTimeout( interval );
                                                                // CAPTURE CHART
                                                                e.chart['export'].capture( {}, function ()
                                                                {
                                                                    // SAVE TO JPG
                                                                    this.toJPG( {}, function ( base64 )
                                                                    {
                                                                        result[0].image = base64;
                                                                        if( callback )
                                                                            callback( result );
                                                                    } );
                                                                } )
                                                            }
                                                        }, 50 );
                        } );
};

ChartController.prototype._exportBarChart = function( barCount, horizontal, breaker, callback ) {
    var self = this;
    var result = [];

    var minimumValue = this.chart.valueAxes[0].min;
    var maximumValue = this.chart.valueAxes[0].max;

    var dataParts = [];
    for (var i = 0; i < Math.ceil(this.currentData.chartPoints.length / barCount); i++){
        dataParts.push(this.currentData.chartPoints.slice(i * barCount, i * barCount + barCount));
    }
    if (dataParts[dataParts.length - 1].length < barCount){
        var emptyItem = {
            category: {}, // ant need return string value
            value: ""
        };
        for(i = dataParts[dataParts.length - 1].length; i < barCount; i++)
            dataParts[dataParts.length - 1].push(emptyItem);
    }

    var chart;

    var setPart = function(i) {
        if (i == dataParts.length){
            callback( result );
            return;
        }

        result[i] = {};
        result[i].data = dataParts[i];

        if (i == 0)
            chart = self.buildBarChart( "chartPart",
                                        self.gerBarChartConfiguration(
                                                dataParts[i],
                                                self.currentData.title,
                                                {
                                                    horizontal: self.currentState.chartType == ChartType.HORIZONTAL_BAR,
                                                    withoutAnimation: true,
                                                    withoutScroll: true,
                                                    minimum: minimumValue,
                                                    maximum: maximumValue,
                                                    fontSize: 14
                                                } ),
                                        function ( e )
                                        {
                                            var interval = setInterval( function ()
                                                                        {
                                                                            if( window.fabric )
                                                                            {
                                                                                clearTimeout( interval );
                                                                                // CAPTURE CHART
                                                                                e.chart['export'].capture( {}, function ()
                                                                                {
                                                                                    // SAVE TO JPG
                                                                                    this.toJPG( {}, function ( base64 )
                                                                                    {
                                                                                        if( !breaker['break'] )
                                                                                        {
                                                                                            result[i].image = base64;
                                                                                            setPart( ++i );
                                                                                        }

                                                                                    } );
                                                                                } )
                                                                            }
                                                                        }, 50 );
                                        } );
        else
        {
            chart.dataProvider = dataParts[i];
            chart.validateData();
            chart.validateNow();
        }

    };

    setPart( 0 );

};

ChartController.prototype.getCurrentBarCount = function() {
    return this.currentBarCount ? this.currentBarCount : 0;
};

ChartController.prototype.breakExport = function( ) {
    if (this.currentExportBreaker)
        this.currentExportBreaker['break'] = true;
};

ChartController.prototype._extendCurrentStateWithFormattedFunctions = function( groupingLevel, reportChartType )
{
    this.currentState.formatCategoryFunction = this.chartFormatter.getFormatCategoryFunctionForChart( groupingLevel, false, 17, false );
    this.currentState.formatHBarChartCursorLabelFunction = this.chartFormatter.getFormatCategoryFunctionForChart( groupingLevel, true, 18, true );
    this.currentState.formatVBarChartCursorLabelFunction = this.chartFormatter.getFormatCategoryFunctionForChart( groupingLevel, true, 30, true );
    this.currentState.formatValueFunction    = this.chartFormatter.getValueFormattedFunction( reportChartType );
    this.currentState.formatLabelFunction    = this.chartFormatter.getFormatPieLabelFunction( groupingLevel );
    this.currentState.toStringFunction       = this.chartFormatter.getCategoryToStringFunction( groupingLevel );
};