function ChartDataParametersSectionController() {}

ChartDataParametersSectionController.ERROR_EMPTY_USER_SELECTION = "errorEmptyUserSelection";
ChartDataParametersSectionController.ERROR_EMPTY_OVERTIME_USER_SELECTION = "errorEmptyOvertimeUserSelection";
ChartDataParametersSectionController.ERROR_EMPTY_CP_SELECTION = "errorEmptyCPSelection";
ChartDataParametersSectionController.ERROR_EMPTY_LEAVE_TYPES_SELECTION = "errorEmptyLeaveTypesSelection";
ChartDataParametersSectionController.ERROR_INCORRECT_DATE_RANGE = "errorIncorrectDateRange";

ChartDataParametersSectionController.prototype.possibleErrors = [
    ChartDataParametersSectionController.ERROR_EMPTY_USER_SELECTION,
    ChartDataParametersSectionController.ERROR_EMPTY_OVERTIME_USER_SELECTION,
    ChartDataParametersSectionController.ERROR_EMPTY_CP_SELECTION,
    ChartDataParametersSectionController.ERROR_EMPTY_LEAVE_TYPES_SELECTION,
    ChartDataParametersSectionController.ERROR_INCORRECT_DATE_RANGE
];

ChartDataParametersSectionController.prototype.init = function( containerId, chartPopupData, chartType, chartDataParameters, changesCallback )
{
    this.containerId = containerId;

    this.enabledFeatures = chartPopupData.enabledFeatures;
    this.canGenerateTimeReports = chartPopupData.hasGenerateTimeReportsRight;
    this.canGenerateMoneyReports = chartPopupData.hasGenerateMoneyReportsRight;
    this.canManageBalances = chartPopupData.hasManagePTOAndSickRight;

    this.customersProjects = chartPopupData.customersProjects;
    this.users = chartPopupData.users;
    this.userGroups = chartPopupData.userGroups;
    this.timeZoneGroups = chartPopupData.timeZoneGroups;
    this.leaveTypes = chartPopupData.leaveTypes;

    this.initiallySelectedGroupingLevel = chartDataParameters.groupingLevel;

    this.changesCallback = changesCallback;

    this.dateSelectorContainer = this.getElementBySubId("dateSelector");
    this.singleDateSelectorContainer = this.getElementBySubId("singleDateSelector");
    this.chartTypeListSelectorContainer = this.getElementBySubId("chartTypeListSelectorPlaceholder");
    this.groupingLevelListSelectorContainer = this.getElementBySubId("groupingLevelListSelectorPlaceholder");
    this.hierarchicalSelectorsSection = this.getElementBySubId("hierarchicalSelectorsSection");
    this.dateRangeSection = this.getElementBySubId("dateRangeSection");
    this.cpSelectorContainer = this.getElementBySubId("cpSelectorId");
    this.usersSelectorContainer = this.getElementBySubId("userSelectorButtonWrapperId");
    this.overtimeUsersSelectorContainer = this.getElementBySubId("overtimeUserSelectorButtonWrapperId");
    this.leaveTypesSelectorContainer = this.getElementBySubId("leaveTypesSelectorId");
    this.noteMsgContainer = this.getElementBySubId("noteMsgId");

    this.errorsToSectionElsMap = {};
    this.errorsToSectionElsMap[ ChartDataParametersSectionController.ERROR_EMPTY_USER_SELECTION ] = $( this.usersSelectorContainer );
    this.errorsToSectionElsMap[ ChartDataParametersSectionController.ERROR_EMPTY_OVERTIME_USER_SELECTION ] = $( this.overtimeUsersSelectorContainer );
    this.errorsToSectionElsMap[ ChartDataParametersSectionController.ERROR_EMPTY_CP_SELECTION ] = $( this.cpSelectorContainer );
    this.errorsToSectionElsMap[ ChartDataParametersSectionController.ERROR_EMPTY_LEAVE_TYPES_SELECTION ] = $( this.leaveTypesSelectorContainer );

    var self = this;
    this.errorTooltipsForSectionsController = new ErrorTooltipsForSectionsController().init( "createChartErrorTooltip",
                                                                                             this.errorsToSectionElsMap,
                                                                                             function( error ) { return self.getErrorMessageForSingleError( error ); } );

    var $infoIconWrappers = $("#" + this.containerId ).find(".infoIconWrapper");
    _.each($infoIconWrappers, function(wrapper)
    {
        var $wrapper = $(wrapper);
        $wrapper.empty();
        new IconWithTooltip($wrapper, messageResource.getMessage("report.create_chart.users_wo_data_note")).init();
    });

    this.setParameters( chartType, chartDataParameters );

    return this;
};

ChartDataParametersSectionController.prototype.setParameters = function( chartType, chartDataParameters )
{
    this.deleteOldSelectors();

    this.dateRangeSelector = this.createDateRangeSelector( chartDataParameters );
    this.singleDateSelector = this.createSingleDateSelector( chartDataParameters );
    this.chartTypeSelector = this.createChartTypeSelector( chartType );
    this.groupingLevelSelector = this.createGroupingSelector( chartType, chartDataParameters.groupingLevel );
    this.cpSelector = this.makeCPSelector( chartDataParameters, this.customersProjects );
    this.userSelector = this.makeUserSelector( chartDataParameters, this.users, this.userGroups, this.timeZoneGroups );
    this.overtimeUserSelector = this.makeOvertimeUserSelector( chartDataParameters, this.users, this.userGroups, this.timeZoneGroups );
    this.leaveTypesSelector = this.makeLeaveTypesSelector( chartDataParameters, this.leaveTypes );

    this.dateRangeSectionStrategy = this.getDateRangeSectionStrategy( chartType );
    this.hierarchicalSelectorsSectionStrategy = this.getHierarchicalSelectorsSectionStrategy( chartType );

    this.errors = [];

    this.updateUI();
};

ChartDataParametersSectionController.prototype.DEFAULT_SELECTOR_PARAMETERS = {
    dateRangeSelection: {
        qualifier: DateRangeSelectionQualifier.CURRENT_MONTH,
        fromDate: null,
        toDate: null
    },
    singleDateSelection: {
        qualifier: EvaDateQualifier.TODAY,
        date: null
    },
    cpSelection: {
        qualifier: CustomersProjectsSelectionQualifier.ALL,
        customers: [],
        projects: [],
        showArchivedProjects: false
    },
    userSelection: {
        allUsers: true,
        selectedUsers: [],
        selectedGroups: [],
        showDisabled: false
    },
    leavesSelection: {
        qualifier: LeaveTypeSelectionQualifier.ALL,
        leaveTypes: [],
        showInactive: false
    }
};

ChartDataParametersSectionController.prototype.onChartTypeChanged = function()
{
    var newChartType = this.chartTypeSelector.getSelectedItem().value;
    this.hierarchicalSelectorsSectionStrategy = this.getHierarchicalSelectorsSectionStrategy( newChartType );
    this.dateRangeSectionStrategy = this.getDateRangeSectionStrategy( newChartType );
    var oldSelectedGroupingLevel = this.groupingLevelSelector.getSelectedItem().value;
    $( this.groupingLevelListSelectorContainer ).empty();
    this.groupingLevelSelector = this.createGroupingSelector( newChartType, oldSelectedGroupingLevel );
    this.changesDetected();
};

ChartDataParametersSectionController.prototype.changesDetected = function()
{
    this.validate();
    this.updateUI();

    if ( this.changesCallback )
        this.changesCallback();
};

ChartDataParametersSectionController.prototype.updateUI = function()
{
    this.hierarchicalSelectorsSectionStrategy.updateSectionUI();
    this.dateRangeSectionStrategy.updateSectionUI();

    var containerEl = $( "#" + this.containerId );

    for ( var i = 0; i < this.possibleErrors.length; i++ )
        containerEl.removeClass( this.possibleErrors[i] );

    for ( i = 0; i < this.errors.length; i++ )
        containerEl.addClass( this.errors[i] );

    this.errorTooltipsForSectionsController.updateTooltips( this.errors );
};

ChartDataParametersSectionController.prototype.getElementBySubId = function( id )
{
    return document.getElementById( this.containerId + "_" + id );
};

ChartDataParametersSectionController.prototype.deleteOldSelectors = function()
{
    $( this.dateSelectorContainer ).empty();
    $( this.singleDateSelectorContainer ).empty();
    $( this.chartTypeListSelectorContainer ).empty();
    $( this.groupingLevelListSelectorContainer ).empty();
    $( this.cpSelectorContainer ).empty();
    $( this.usersSelectorContainer ).empty();
    $( this.overtimeUsersSelectorContainer ).empty();
    $( this.leaveTypesSelectorContainer ).empty();
};

ChartDataParametersSectionController.prototype.createDateRangeSelector = function( chartDataParameters )
{
    var dateRangeSelection = chartDataParameters.dateRangeSelection ? chartDataParameters.dateRangeSelection :
                             this.DEFAULT_SELECTOR_PARAMETERS.dateRangeSelection;

    var self = this;
    var dateRangeSelectorConfig = {
        dateRangeSelection: dateRangeSelection,
        menuStates: new DateRangeMenuItemsUtil().createDefaultDateRangeMenuItems( dateRangeSelection.qualifier ),
        layout: "auto",
        renderTo: self.dateSelectorContainer.id,
        callOnChangeCallbackIfInvalid: true,
        shortDateFormat: true
    };

    var dateRangeSelector = new DateRangeSelectorController().init( dateRangeSelectorConfig );
    dateRangeSelector._flushErrorState();
    dateRangeSelector.setOnChangeCallback( function() { self.changesDetected(); } );
    return dateRangeSelector;
};

ChartDataParametersSectionController.prototype.createSingleDateSelector = function( chartDataParameters )
{
    var dateSelection = chartDataParameters.dateSelection ? chartDataParameters.dateSelection :
                        this.DEFAULT_SELECTOR_PARAMETERS.singleDateSelection;

    var self = this;
    var dateSelectorConfig = {
        dateRangeSelection: dateSelection,
        menuStates: new DateRangeMenuItemsUtil().createDefaultSingleDateMenuItems(),
        layout: "auto",
        renderTo: self.singleDateSelectorContainer.id,
        shortDateFormat: true
    };

    var dateSelector = new SingleDateSelectorController().init( dateSelectorConfig );
    dateSelector.setOnChangeCallback( function() { self.changesDetected(); } );

    var labelEl = $("<span class='singleDateRangeLabel'>" + messageResource.getMessage("report.create_chart.as_of") + "</span>");
    labelEl.insertBefore( $( this.singleDateSelectorContainer ).find( ".dateRangeDropdownSelector " ) );

    return dateSelector;
};

ChartDataParametersSectionController.prototype.createChartTypeSelector = function( chartType )
{
    var self = this;
    return this.createListSelector( this.chartTypeListSelectorContainer,
                                    function( listSelector ) { self.fillChartTypeListSelector( listSelector, chartType ); },
                                    "chartTypeListSelector",
                                    chartType,
                                    function() { self.onChartTypeChanged(); } );
};

ChartDataParametersSectionController.prototype.createGroupingSelector = function( chartType, selectedGroupingLevel )
{
    var groupingLevelsSelectorFillingStrategy = this.getGroupingLevelSelectorFillingStrategy( chartType );

    var self = this;
    return this.createListSelector( this.groupingLevelListSelectorContainer,
                                    function( listSelector ) { groupingLevelsSelectorFillingStrategy.fillGroupingLevelListSelector( listSelector ); },
                                    "groupingLevelListSelector",
                                    selectedGroupingLevel,
                                    function() { self.changesDetected(); });
};

ChartDataParametersSectionController.prototype.createListSelector = function( containerEl, fillSelectorCallback, selectorCls,
                                                                              selectedValue, changesCallback )
{
    var listSelector = new ListSelector({
                                            renderTo: containerEl,
                                            menuCls: selectorCls
                                        });
    fillSelectorCallback( listSelector );
    if ( selectedValue )
        listSelector.setActiveItemByValue( selectedValue, true );

    if ( listSelector.getSelectedItem() === null )
        listSelector.setItem( listSelector.menu.items.items[0] );

    listSelector.on( "change", function () {
        if ( changesCallback )
            changesCallback();
    });

    return listSelector;
};

ChartDataParametersSectionController.prototype.fillChartTypeListSelector = function( listSelector, chartType )
{
    var items = [];
    if( this.canGenerateTimeReports )
    {
        items.push( {text: messageResource.getMessage( "report.chart_type.total_working_hours" ), value: ReportChartType.TOTAL_WORKING_HOURS} );
        if ( this.isEnabledFeature( ReportChartConstants.TTAFeature ) || chartType == ReportChartType.APPROVED_WORKING_HOURS )
            items.push( {text: messageResource.getMessage( "report.chart_type.approved_working_hours" ), value: ReportChartType.APPROVED_WORKING_HOURS} );
        if ( this.isEnabledFeature( ReportChartConstants.LVTFeature ) || chartType == ReportChartType.LEAVE_HOURS )
            items.push( {text: messageResource.getMessage( "report.chart_type.leave_hours" ), value: ReportChartType.LEAVE_HOURS} );
        if( this.canManageBalances && this.isEnabledFeature( ReportChartConstants.PTOFeature ) || chartType == ReportChartType.PTO_BALANCE )
            items.push( {text: messageResource.getMessage( "report.chart_type.pto_balance" ), value: ReportChartType.PTO_BALANCE} );
        if( this.canManageBalances && this.isEnabledFeature( ReportChartConstants.SDBFeature ) || chartType == ReportChartType.SICK_DAYS_BALANCE )
            items.push( {text: messageResource.getMessage( "report.chart_type.sick_days_balance" ), value: ReportChartType.SICK_DAYS_BALANCE} );
        if ( this.isEnabledFeature( ReportChartConstants.OVRFeature ) || chartType == ReportChartType.OVERTIME_HOURS )
            items.push( {text: messageResource.getMessage( "report.chart_type.overtime_hours" ), value: ReportChartType.OVERTIME_HOURS} );
    }
    if( this.canGenerateMoneyReports )
    {
        if ( this.isEnabledFeature( ReportChartConstants.URAFeature ) || chartType == ReportChartType.COST_OF_WORK )
            items.push( {text: messageResource.getMessage( "report.chart_type.cost_of_work" ), value: ReportChartType.COST_OF_WORK} );
        if ( ( this.isEnabledFeature( ReportChartConstants.BTYFeature ) && this.isEnabledFeature( ReportChartConstants.BRAFeature ) ) || chartType == ReportChartType.BILLABLE_AMOUNTS )
            items.push( {text: messageResource.getMessage( "report.chart_type.billable_amounts" ), value: ReportChartType.BILLABLE_AMOUNTS} );
    }
    listSelector.addItems(items);
};

ChartDataParametersSectionController.prototype.isEnabledFeature = function( featureName )
{
    return this.enabledFeatures.hasOwnProperty( featureName );
};

ChartDataParametersSectionController.prototype.getDateRangeSectionStrategy = function( selectedValueType )
{
    if ( selectedValueType == ReportChartType.PTO_BALANCE || selectedValueType == ReportChartType.SICK_DAYS_BALANCE )
        return new SingleDateChartDateRangeSectionStrategy().init( this.dateRangeSection, this.dateRangeSelector, this.singleDateSelector );

    return new DateRangeChartDateRangeSectionStrategy().init( this.dateRangeSection, this.dateRangeSelector, this.singleDateSelector );
};

ChartDataParametersSectionController.prototype.getHierarchicalSelectorsSectionStrategy = function( selectedValueType )
{
    var strategy;
    switch ( selectedValueType )
    {
        case ReportChartType.TOTAL_WORKING_HOURS:
        case ReportChartType.APPROVED_WORKING_HOURS:
        case ReportChartType.COST_OF_WORK:
        case ReportChartType.BILLABLE_AMOUNTS:
            strategy = new UsersAndCPHierarchicalSelectorsSectionStrategy();
            break;

        case ReportChartType.LEAVE_HOURS:
            strategy = new LeaveHierarchicalSelectorsSectionStrategy();
            break;

        case ReportChartType.PTO_BALANCE:
        case ReportChartType.SICK_DAYS_BALANCE:
            strategy = new BalanceHierarchicalSelectorsSectionStrategy();
            break;

        case ReportChartType.OVERTIME_HOURS:
            strategy = new OvertimeHoursHierarchicalSelectorsSectionStrategy();
            break;

        default:
            throw "Incorrect report chart type";
    }
    strategy.init( this.hierarchicalSelectorsSection, this.cpSelector, this.userSelector, this.overtimeUserSelector, this.leaveTypesSelector, this.noteMsgContainer );
    return strategy;
};

ChartDataParametersSectionController.prototype.getGroupingLevelSelectorFillingStrategy = function( selectedChartType )
{
    var ugrEnabled = this.isEnabledFeature( ReportChartConstants.UGRFeature );
    var btyEnabled = this.isEnabledFeature( ReportChartConstants.BTYFeature );
    var tzgEnabled = this.isEnabledFeature( ReportChartConstants.TZGFeature );

    if ( selectedChartType == ReportChartType.PTO_BALANCE || selectedChartType == ReportChartType.SICK_DAYS_BALANCE )
        return new UsersGroupsGroupingLevelSelectorFillingStrategy().init( ugrEnabled, btyEnabled, tzgEnabled, this.initiallySelectedGroupingLevel );
    else if ( selectedChartType == ReportChartType.LEAVE_HOURS || selectedChartType == ReportChartType.OVERTIME_HOURS )
        return new UsersGroupsMonthsWeeksDaysGroupingLevelSelectorFillingStrategy().init( ugrEnabled, btyEnabled, tzgEnabled, this.initiallySelectedGroupingLevel );
    else if ( selectedChartType == ReportChartType.TOTAL_WORKING_HOURS || selectedChartType == ReportChartType.APPROVED_WORKING_HOURS )
        return new AllItemsGroupingLevelSelectorFillingStrategy().init( ugrEnabled, btyEnabled, tzgEnabled, this.initiallySelectedGroupingLevel );
    else
        return new UsersGroupsCPMonthsWeeksDaysGroupingLevelSelectorFillingStrategy().init( ugrEnabled, btyEnabled, tzgEnabled, this.initiallySelectedGroupingLevel );
};

ChartDataParametersSectionController.prototype.makeCPSelector = function ( chartDataParameters, cpList )
{
    var cpSelection = chartDataParameters.customersProjectsSelection ? chartDataParameters.customersProjectsSelection :
                      this.DEFAULT_SELECTOR_PARAMETERS.cpSelection;

    var self = this;
    var cpSelector = new CPReportSelector().init( cpSelection,
                                                  this.cpSelectorContainer.id,
                                                  function() {
                                                      if ( self.cpSelector.isStateChanged() )
                                                          self.changesDetected();
                                                      self.cpSelector.setInitialValueFromCurrent();
                                                  },
                                                  true );
    cpSelector.initializeData( cpList );

    return cpSelector;
};

ChartDataParametersSectionController.prototype.makeUserSelector = function ( chartDataParameters, users, userGroups, timeZoneGroups )
{
    return this.createUserSelector( this.usersSelectorContainer, chartDataParameters, users, userGroups, timeZoneGroups, AT.ReportsSelectorUsersPanel );
};

ChartDataParametersSectionController.prototype.makeOvertimeUserSelector = function ( chartDataParameters, users, userGroups, timeZoneGroups )
{
    var overtimeUsers = [];
    for ( var i=0; i < users.length; i++ )
    {
        if ( users[i].overtimeEnabled )
            overtimeUsers.push( users[i] );
    }

    return this.createUserSelector( this.overtimeUsersSelectorContainer, chartDataParameters,
                                    overtimeUsers, userGroups, timeZoneGroups, AT.OvertimeReportSelectorPanel );
};

ChartDataParametersSectionController.prototype.createUserSelector = function ( selectorContainer, chartDataParameters,
                                                                               users, userGroups, timeZoneGroups, panelConstructor )
{
    var defaultUserSelection = {
        allUsers: true,
        selectedUsers: [],
        selectedGroups: [],
        showDisabled: false,
        groupType: null
    };
    var userSelection = chartDataParameters.userSelection ? chartDataParameters.userSelection : defaultUserSelection;

    var self = this;

    var userSelector = new ReportUserSelectorButton();

    var selectorConfig = {
        selectorId: selectorContainer.id,
        userSelection: userSelection,
        specificConstructorPanel: panelConstructor,
        changeCallback: function() {
            if ( userSelector.isStateChanged() )
                self.changesDetected();
            userSelector.setInitialValueFromCurrent();
        },
        withApplyButton: true,
        needAdditionalCheckbox: false,
        includeUsersWithoutDataDefaultValue: true
    };

    userSelector.init(selectorConfig);
    userSelector.updateSelector(users, userGroups, timeZoneGroups);
    return userSelector;
};

ChartDataParametersSectionController.prototype.makeLeaveTypesSelector = function ( chartDataParameters, leaveTypes )
{
    var leavesSelection = chartDataParameters.leaveTypeSelection ? chartDataParameters.leaveTypeSelection :
                          this.DEFAULT_SELECTOR_PARAMETERS.leavesSelection;

    var self = this;
    return new ReportLeaveTypesSelectorButton().init( this.leaveTypesSelectorContainer.id, leavesSelection, leaveTypes,
                                                      this.isEnabledFeature( ReportChartConstants.PTOFeature ),
                                                      this.isEnabledFeature( ReportChartConstants.SDBFeature ),
                                                      function() {
                                                          if ( self.leaveTypesSelector.isStateChanged() )
                                                              self.changesDetected();
                                                          self.leaveTypesSelector.setInitialValuesFromCurrent();
                                                      }, true );
};

ChartDataParametersSectionController.prototype.validate = function()
{
    this.errors = [];

    var dateRangeErrors = this.dateRangeSectionStrategy.validate();
    var hierarchicalSelectorsErrors = this.hierarchicalSelectorsSectionStrategy.validate();

    var i;
    for ( i = 0; i < dateRangeErrors.length; i++ )
        this.errors.push( dateRangeErrors[i] );
    for ( i = 0; i < hierarchicalSelectorsErrors.length; i++ )
        this.errors.push( hierarchicalSelectorsErrors[i] );

    return this.errors.length == 0;
};

ChartDataParametersSectionController.prototype.hasErrors = function()
{
    return this.errors.length != 0;
};

ChartDataParametersSectionController.prototype.getChartType = function()
{
    return this.chartTypeSelector.getSelectedItem().value;
};

ChartDataParametersSectionController.prototype.getSelectedValues = function ()
{
    return {
        chartTypeValue: this.getChartType(),
        groupingLevelValue: this.getGroupingLevel(),
        dateParameters: this.dateRangeSectionStrategy.getSelectedParams(),
        hierarchicalSelectorsParameters: this.hierarchicalSelectorsSectionStrategy.getSelectedParams()
    };
};

ChartDataParametersSectionController.prototype.getSelectedParams = function()
{
    var params = {
        groupingLevel: this.getGroupingLevel()
    };
    var hierarchicalParams = this.hierarchicalSelectorsSectionStrategy.getSelectedParams();
    var dateRangeParams = this.dateRangeSectionStrategy.getSelectedParamsJSON();
    return $.extend( $.extend( params, hierarchicalParams ), dateRangeParams);
};

ChartDataParametersSectionController.prototype.getGroupingLevel = function()
{
   return this.groupingLevelSelector.getSelectedItem().value
};


ChartDataParametersSectionController.prototype.getErrorMessage = function()
{
    if ( this.errors.length == 0)
        return "";
    if ( this.errors.length > 1 )
        return messageResource.getMessage("report.js.error.exist_invalid_fields");

    return this.getErrorMessageForSingleError( this.errors[0] );
};

ChartDataParametersSectionController.prototype.getErrorMessageForSingleError = function( error )
{
    return escapeHtmlSymbols(this.getUnescapedErrorMessageForSingleError(error))
};

ChartDataParametersSectionController.prototype.getUnescapedErrorMessage = function()
{
    if ( this.errors.length == 0)
        return "";
    if ( this.errors.length > 1 )
        return messageResource.getMessage("report.js.error.exist_invalid_fields");

    return this.getUnescapedErrorMessageForSingleError( this.errors[0] );
};

ChartDataParametersSectionController.prototype.getUnescapedErrorMessageForSingleError = function( error )
{
    switch (error)
    {
        case ChartDataParametersSectionController.ERROR_EMPTY_CP_SELECTION:
            return messageResource.getMessage("projects_customers_selector.please_select_customer_or_project");
            break;
        case ChartDataParametersSectionController.ERROR_EMPTY_USER_SELECTION:
        case ChartDataParametersSectionController.ERROR_EMPTY_OVERTIME_USER_SELECTION:
            return messageResource.getMessage("report.js.choose_person");
            break;
        case ChartDataParametersSectionController.ERROR_EMPTY_LEAVE_TYPES_SELECTION:
            return messageResource.getMessage("report.leaves.js.choose_leave_type");
            break;
        case ChartDataParametersSectionController.ERROR_INCORRECT_DATE_RANGE:
            return messageResource.getMessage( this.dateRangeSelector.getErrorMessageKey() );
            break;
    }
    return "";
};