/**
 * Created by filatov on 30.10.2015.
                              */
function ChartDisplayParametersSectionController(){}

ChartDisplayParametersSectionController.prototype.init = function(chartDisplayParameters, groupingLevel, minChartPointValue, maxChartPointValue, changeCallback){
    this.createChartModeElement =  $(".createChartModeElement " );

    this.changeCallback = changeCallback;

  
    this.sortCategoryClasses = {date:"date", name:"name"};

    this.setParameters( chartDisplayParameters, groupingLevel );
    this.changeCheckboxStateIfNeededAndGetShowPercentsValue( minChartPointValue, maxChartPointValue );

    return this;
};

ChartDisplayParametersSectionController.prototype.setParameters = function( chartDisplayParameters, groupingLevel )
{
    var self = this;

    this.sortOption = {};
    this.sortOption.by =  chartDisplayParameters.sortOption;
    this.sortOption.isAsc = chartDisplayParameters.sortOrderIsAsc;

    this.chartColorSelectorPanel = new ChartColorSelectorPanel().init(function(){self.chartColorSelector.updateOpenState(false)});
    this.chartColorSelector = new ChartColorSelector().init("chartColorSelector", this.chartColorSelectorPanel, chartDisplayParameters.chartColor,
                                                            function(){self.changeCallback()});
    this.chartThemeSelector = new ChartThemeSelector().init("themeSelector", chartDisplayParameters.chartTheme, function(){self.changeCallback()});

    this.chartPercentsCheckbox = new ChartPercentsCheckbox().init( "chartShowPercents",
                                                                   function() {self.changeCallback() },
                                                                   chartDisplayParameters.showPercents);

    this.chartSortContainer = $("#chartSortContainer" );
    this.chartSortContainer.removeClass();
    this.chartSortContainer.addClass("sortContainer");

    this.changeSortClassAccordingGroupingLevel(groupingLevel);
    this.changeSort(this.sortOption.by);
    this.initSortButtons();

    this.chartTypeSelector = new ChartTypeSelector().init("chartTypeSelector", chartDisplayParameters.chartType, self.changeCallback);

};

ChartDisplayParametersSectionController.prototype.getSortOption = function(){
    return this.sortOption;
};

ChartDisplayParametersSectionController.prototype.getChartType = function(){
    return this.chartTypeSelector.getChartType();
};

ChartDisplayParametersSectionController.prototype.initSortButtons = function(){
    var self = this;
    this.chartSortContainer.find(".sortLink" ).off("click").on("click", function(event){
        self.changeSort(event.target.id, true);
        if(self.changeCallback)
            self.changeCallback();
    })
};

ChartDisplayParametersSectionController.prototype.changeSort = function(sortValue, changeSortOrder){
    this.chartSortContainer.removeClass(this.getClassSortOption(this.sortOption.by));
    this.chartSortContainer.removeClass(this.sortOption.isAsc ? "asc" : "desc");

    if( changeSortOrder )
        this.sortOption.isAsc = !this.sortOption.isAsc;
    this.sortOption.isAsc = this.sortOption.by != sortValue &&
                            (sortValue == ChartSortOptions.BY_VALUES
                             || this.sortOption.by == ChartSortOptions.BY_VALUES) ? true : this.sortOption.isAsc;
    this.sortOption.by = sortValue;

    this.chartSortContainer.addClass(this.getClassSortOption(this.sortOption.by));
    this.chartSortContainer.addClass(this.sortOption.isAsc ? "asc" : "desc");
};

ChartDisplayParametersSectionController.prototype.getClassSortOption = function ( sortOptionBy )
{
    var res = "sort";
    var cls = "";
    switch( sortOptionBy )
    {
        case  ChartSortOptions.ALPHABETICALLY:
            cls = "Alphabetically";
            break;
        case  ChartSortOptions.BY_VALUES:
            cls = "Values";
            break;
        case  ChartSortOptions.CHRONOLOGICALLY:
            cls = "Chronologically";
            break;
    }
    return res.concat( cls);
};

ChartDisplayParametersSectionController.prototype.changeSortClassAccordingGroupingLevel = function(groupingLevel){
    this.createChartModeElement.removeClass(this.sortCategoryClasses.date);
    this.createChartModeElement.removeClass(this.sortCategoryClasses.name);
    switch (groupingLevel) {
        case ReportGroupingLevel.DATE:
        case ReportGroupingLevel.WEEK:
        case ReportGroupingLevel.MONTH:
        {
            this.createChartModeElement.addClass(this.sortCategoryClasses.date);
            if(this.sortOption.by !=  ChartSortOptions.BY_VALUES){
                this.changeSort(ChartSortOptions.CHRONOLOGICALLY);
            }
            break;
        }
        case ReportGroupingLevel.USER:
        case ReportGroupingLevel.USER_GROUP:
        case ReportGroupingLevel.TIME_ZONE_GROUP:
        case ReportGroupingLevel.CUSTOMER:
        case ReportGroupingLevel.PROJECT:
        case ReportGroupingLevel.BILLING_TYPE:
        {
            this.createChartModeElement.addClass(this.sortCategoryClasses.name);
            if(this.sortOption.by !=  ChartSortOptions.BY_VALUES){
                this.changeSort(ChartSortOptions.ALPHABETICALLY);
            }
            break;
        }
    }
};

ChartDisplayParametersSectionController.prototype.getChartColor = function(){
    return this.chartColorSelector.getColor();
};

ChartDisplayParametersSectionController.prototype.getChartTheme = function(){
    return this.chartThemeSelector.getThemeValue();
};

ChartDisplayParametersSectionController.prototype.isPercentsCheckboxVisibleAndChecked = function(){
    return this.chartPercentsCheckbox.isVisibleAndChecked();
};

ChartDisplayParametersSectionController.prototype.getDisplayParameters = function(){
    return {
        showPercents: this.isPercentsCheckboxVisibleAndChecked(),
        sortOption : this.getSortOption().by,
        sortOrderIsAsc: this.getSortOption().isAsc,
        chartType : this.getChartType(),
        chartColor : this.chartColorSelector.getColorId(),
        chartTheme : this.chartThemeSelector.getThemeName()
    }

};

ChartDisplayParametersSectionController.prototype.changeCheckboxStateIfNeededAndGetShowPercentsValue = function( minChartPointValue, maxChartPointValue ){
    if ( this.getChartType() == ChartType.PIE_CHART || ( minChartPointValue < 0 && maxChartPointValue > 0 ) )
    {
        this.chartPercentsCheckbox.hidePercentsCheckbox();
        return false;
    }
    else
    {
        this.chartPercentsCheckbox.showPercentsCheckbox();
        return this.chartPercentsCheckbox.getValue();
    }
};

ChartDisplayParametersSectionController.getDefaultDisplayParameters = function(){
    return {
        showPercents:false,
        sortOption : ChartSortOptions.ALPHABETICALLY,
        sortOrderIsAsc: true,
        chartType : ChartType.VERTICAL_BAR,
        chartColor: this.getDefaultColor(),
        chartTheme: ChartPieTheme.NONE
    }
};

ChartDisplayParametersSectionController.getDefaultColor = function(){
   return CustomizationSettings.colorScheme.toUpperCase();
};





