function CreateChartLightbox() {}

CreateChartLightbox.prototype = new BasicLightbox();

CreateChartLightbox.prototype.exist_invalid_fields_message_key = "";
CreateChartLightbox.prototype.close_tab_confirmation_message_key = "";

CreateChartLightbox.prototype.init = function( config )
{
    BasicLightbox.prototype.init.call(this, config.popupId, config.closeBtnId);

    var self = this;
    this.titleEl = $(this.lightBoxEl).find(".lightBoxTitle");
    this.setCancelCallback( function() { self.hide(); } );

    this.reloadPageAfterClose = false;
    this.formatter = new BaseFormatterCreator();

    this.createChartErrorEl = this.innerContent.find( ".createChartError" );

    // get display parameters from cookies only if this is new config
    if( !config.chartId )
    {
        config.chartDisplayParameters = ChartDisplayParametersSectionController.getDefaultDisplayParameters();
        $.extend( config.chartDisplayParameters, this.read_cookie( "actiTIME-CHARTDisplayParameters" ) );
    }

    this.chartDataParametersSectionController = new ChartDataParametersSectionController().init( config.popupId,
                                                                                                 config.chartPopupData,
                                                                                                 config.chartType,
                                                                                                 config.chartDataParameters,
                                                                                                 function() { self.onDataParametersChanged(); } );

    this.chartDataController = new ChartDataController().init( config.chartPoints );
    this.chartDisplayParametersSectionController = new ChartDisplayParametersSectionController().init( config.chartDisplayParameters, config.chartDataParameters.groupingLevel,
                                                                                                       this.chartDataController.getMinValue(), this.chartDataController.getMaxValue(),
                                                                                                       function(){self.displayParametersChanged();} );
    this.totalValueContainer = $(config.totalValueContainer);

    var chartData = {
        title: ReportChartTitle[config.chartType],
        chartPoints: config.chartPoints,
        total: this.chartDataController.getTotal(),
        max: this.chartDataController.getMaxValue(),
        min: this.chartDataController.getMinValue()
    };

    this.chartController = new ChartController().init( {
                                                           containerId: config.chartContainerId,
                                                           partsContainerId: config.partsContainerId,
                                                           data: chartData,
                                                           settings: {
                                                               showPercents: config.chartDisplayParameters.showPercents,
                                                               color: ChartColor[config.chartDisplayParameters.chartColor],
                                                               theme: self.chartDisplayParametersSectionController.getChartTheme(),
                                                               chartType: config.chartDisplayParameters.chartType,
                                                               reportChartType: config.chartType,
                                                               groupingLevel: config.chartDataParameters.groupingLevel,
                                                               sortCategoryComparator: self.getSortCategoryFunctionForChart(),
                                                               sortOption: config.chartDisplayParameters.sortOption,
                                                               sortOrder: config.chartDisplayParameters.sortOrderIsAsc,
                                                               emptyPieCallback: $.proxy(self.showChartErrorMessageWithControls, this)
                                                           }
                                                       });
    this.showOrHideNotFullDataMessage(this.chartDataController.getMinValue(), this.chartDataController.getMaxValue());
    this.showTotalValue(this.chartDataController.getTotal(), this.formatter.getValueFormattedFunction( config.chartType ), this.formatter.getMeasureUnit(config.chartType));

    this.initPdfPreviewViewController( config );

    this.successCallback = function () {
        self.pdfPreviewViewController.switchLightboxToPDFPreviewView(
            self.chartDataParametersSectionController.getSelectedValues(),
            self.chartDisplayParametersSectionController.getChartType(),
            self.chartDataController
        )
    };

    this.initConfig = this.getReportConfig();

    self.chartId = config.chartId;

    this.saveConfigControl = new SaveConfigurationControl().init(
            {
                reportName: config.chartName,
                hasChanges: false
            },
            SaveConfigurationControl.CHART_TYPE,
            function(reportName) {
                var reportConfig = self.getReportConfig();
                reportConfig.reportName = reportName;
                self.saveChartManager.saveReport( reportConfig )
            },
            function(reportName) {
                var reportConfig = self.getReportConfig();
                reportConfig.reportId = self.chartId;
                reportConfig.reportName = reportName;
                self.saveChartManager.updateReport( reportConfig )
            },
            function() {
                return self.resetConfig();
            }
    );
    this.saveChartManager = new SaveReportChartManager().init(
            this.saveConfigControl,
            function( reportId )
            {
                if( reportId )
                    self.chartId = reportId;
                self.initConfig = self.getReportConfig();
                self.reloadPageAfterClose = true;
            },
            function( errorThrown )
            {
                // default
                return false;
            }
    );

    if ( config.makeChartPointsException )
        this.handleMakeChartPointsException( config.makeChartPointsException );
    else if ( config.chartPoints.length == 0 )
        this.showEmptyChartDataMessage();
    else if ( config.chartDisplayParameters.chartType == ChartType.PIE_CHART && chartData.max == 0 && chartData.min == 0)
        this.showChartErrorMessageWithControls();
    else
        this.removeChartErrorMessage();

    var spinnerOptions = {
        lines: 11, // The number of lines to draw
        length: 8, // The length of each line
        width: 3, // The line thickness
        radius: 6, // The radius of the inner circle
        corners: 1, // Corner roundness (0..1)
        rotate: 0, // The rotation offset
        direction: 1, // 1: clockwise, -1: counterclockwise
        color: '#888888', // #rgb or #rrggbb
        speed: 1, // Rounds per second
        trail: 60, // Afterglow percentage
        shadow: false, // Whether to render a shadow
        hwaccel: false, // Whether to use hardware acceleration
        className: 'spinner', // The CSS class to assign to the spinner
        zIndex: 2e9, // The z-index (defaults to 2000000000)
        top: 'auto', // Top position relative to parent in px
        left: 'auto' // Left position relative to parent in px
    };
    this.spinnerController = new SpinnerController();
    this.spinnerController.setSpinner( new Spinner( spinnerOptions ) );

    this.switchLightboxToCreateChartView();
    this.updateLightboxChartTypeClass();

    return this;
};

CreateChartLightbox.prototype.initPdfPreviewViewController = function ( config )
{
    var self = this;
    this.pdfPreviewViewController = new CreateChartLightboxPDFPreviewViewController().init(
            this.lightBoxEl,
            this.titleEl,
            config.popupId,
            config.chartPopupData.printParameters,
            function () { self.switchLightboxToCreateChartView(); },
            this.chartController,
            {
                users: config.chartPopupData.users,
                enabledFeatures: config.chartPopupData.enabledFeatures,
                userGroups: config.chartPopupData.userGroups,
                timeZoneGroups: config.chartPopupData.timeZoneGroups,
                customersProjects: config.chartPopupData.customersProjects,
                leaveTypes: config.chartPopupData.leaveTypes
            }
    );
};

CreateChartLightbox.prototype.hide = function ()
{
    this.cancelAllExportingProcesses();
    this.abortMakeChartPointsRequest();
    BasicLightbox.prototype.hide.call(this);

    if ( this.reloadPageAfterClose )
        document.location.reload();

    $(document).trigger("reportCharts.createChartLightbox.hide");
};

CreateChartLightbox.prototype.showTotalValue = function( value, formatter, unit ){
    this.totalValueContainer.html( formatter( value ) + "&nbsp;" + unit);
};

CreateChartLightbox.prototype.getReportConfig = function()
{
    return {
        reportType: this.chartDataParametersSectionController.getChartType(),
        reportParameters: {
            dataParameters: this.chartDataParametersSectionController.getSelectedParams(),
            displayParameters: this.chartDisplayParametersSectionController.getDisplayParameters()
        }
    };
};

CreateChartLightbox.prototype.resetConfig = function()
{
    this.chartDataParametersSectionController.setParameters(
            this.initConfig.reportType,
            this.initConfig.reportParameters.dataParameters
    );
    this.chartDisplayParametersSectionController.setParameters(
            this.initConfig.reportParameters.displayParameters,
            this.initConfig.reportParameters.dataParameters.groupingLevel
    );
    this.displayParametersChanged();
    this.onDataParametersChanged();
    return true;
};


CreateChartLightbox.prototype.displayParametersChanged = function(){
    var self = this;
    this.updateLightboxChartTypeClass();
    this.write_cookie("actiTIME-CHARTDisplayParameters", self.chartDisplayParametersSectionController.getDisplayParameters() );
    this.saveConfigControl.registerConfigChanged();
    this.removeChartErrorMessage();
    this.chartController.updateDisplayData(
            {
               chartType: self.chartDisplayParametersSectionController.getChartType(),
               color: self.chartDisplayParametersSectionController.getChartColor(),
               theme: self.chartDisplayParametersSectionController.getChartTheme(),
               sortOption: self.chartDisplayParametersSectionController.getSortOption().by,
               sortOrder: self.chartDisplayParametersSectionController.getSortOption().isAsc,
               isShowPercents: self.chartDisplayParametersSectionController.changeCheckboxStateIfNeededAndGetShowPercentsValue(
                       self.chartDataController.getMinValue(), self.chartDataController.getMaxValue() )
            }
    )
};

CreateChartLightbox.prototype.updateLightboxChartTypeClass = function ()
{
    var chartType = this.chartDisplayParametersSectionController.getChartType();
    var cssClass = "";
    switch (chartType) {
        case ChartType.VERTICAL_BAR:
            cssClass = "verticalBar"; break;
        case ChartType.HORIZONTAL_BAR:
            cssClass = "horizontalBar"; break;
        case ChartType.PIE_CHART:
            cssClass = "pieChart";
    }
    $(this.lightBoxEl).removeClass("verticalBar horizontalBar pieChart" ).addClass(cssClass);
};

CreateChartLightbox.prototype.onDataParametersChanged = function()
{
    this.abortMakeChartPointsRequest();

    if ( !this.chartDataParametersSectionController.validate() )
    {
        this._printErrorInFooter( this.chartDataParametersSectionController.getUnescapedErrorMessage() );
        this._blinkErrorInFooter();
        this.showChartErrorMessage( messageResource.getMessage( "report.create_chart.error.params_not_valid" ) );
        return;
    }

    this._removeErrorInFooter();
    this.removeChartErrorMessage();
    this.showLoadingChartSpinner();

    var self = this;
    var newChartType = this.chartDataParametersSectionController.getChartType();
    var newParams = this.chartDataParametersSectionController.getSelectedParams();
    this.saveConfigControl.registerConfigChanged();
    this.makeChartPointsRequestObject = makeRpcCall("ReportChartService.makeChartPoints",
                [newChartType, newParams],
                function ( result )
                {
                    self.hideLoadingChartSpinner();
                    if ( result && result.length > 0 )
                    {
                        self.chartDataController.updateData(result);
                        self.chartController.updateData( {
                                                             title: ReportChartTitle[newChartType],
                                                             chartPoints: result,
                                                             total: self.chartDataController.getTotal(),
                                                             max: self.chartDataController.getMaxValue(),
                                                             min: self.chartDataController.getMinValue()
                                                         },
                                                         {
                                                             groupingLevel: newParams.groupingLevel,
                                                             reportChartType: newChartType,
                                                             sortOption: self.chartDisplayParametersSectionController.getSortOption().by,
                                                             sortOrder: self.chartDisplayParametersSectionController.getSortOption().isAsc,
                                                             sortCategoryComparator: self.getSortCategoryFunctionForChart(),
                                                             isShowPercents: self.chartDisplayParametersSectionController.changeCheckboxStateIfNeededAndGetShowPercentsValue(
                                                                     self.chartDataController.getMinValue(), self.chartDataController.getMaxValue() )
                                                         } );
                        self.showOrHideNotFullDataMessage(self.chartDataController.getMinValue(), self.chartDataController.getMaxValue());
                        self.showTotalValue(self.chartDataController.getTotal(), self.formatter.getValueFormattedFunction( newChartType ), self.formatter.getMeasureUnit( newChartType ));
                        self.chartDisplayParametersSectionController.changeSortClassAccordingGroupingLevel(newParams.groupingLevel);
                    }
                    else
                    {
                        self.showEmptyChartDataMessage();
                    }
                },
                function (jqXHR, textStatus, errorThrown)
                {
                    if ( textStatus == 'rpc_sessionExpired' ||  textStatus == 'rpc_noRights' )
                        return false;

                    self.hideLoadingChartSpinner();

                    if( textStatus == "abort" )
                        return true;

                    if( textStatus == "timeout" )
                    {
                        self.showChartErrorMessage( messageResource.getMessage( "report.create_chart.error.timeout" ) );
                        return true;
                    }

                    return self.handleMakeChartPointsException( errorThrown.exception );
                }
    );
};

CreateChartLightbox.prototype.abortMakeChartPointsRequest = function()
{
    if( this.makeChartPointsRequestObject )
        this.makeChartPointsRequestObject.abort();
};

CreateChartLightbox.prototype.handleMakeChartPointsException = function( exception )
{
    if( exception == 'com.actimind.actitime.reports.AllSelectedProjectsAreUnavailableException' )
    {
        this.showChartErrorMessage( messageResource.getEscapedMessage( "report.create_chart.error.no_selected_projects" ) );
        return true;
    }
    if( exception == 'com.actimind.actitime.reports.AllSelectedUsersWereDeletedException' )
    {
        this.showChartErrorMessage( messageResource.getMessage( "report.create_chart.error.no_selected_users" ) );
        return true;
    }
    if( exception == 'com.actimind.actitime.reports.NoSelectedUsersWithEnabledOvertimeException' )
    {
        this.showChartErrorMessage( messageResource.getMessage( "report.create_chart.error.no_selected_overtime_users" ) );
        return true;
    }
    if( exception == 'com.actimind.actitime.reports.charts.NoPtoUsersSelectedException' )
    {
        this.showChartErrorMessage( messageResource.getMessage( "report.create_chart.error.no_selected_pto_users" ) );
        return true;
    }
    if( exception == 'com.actimind.actitime.reports.charts.NoSickUsersSelectedException' )
    {
        this.showChartErrorMessage( messageResource.getMessage( "report.create_chart.error.no_selected_sick_users" ) );
        return true;
    }
    if( exception == 'com.actimind.actitime.reports.AllSelectedLeaveTypesWereDeletedException' )
    {
        this.showChartErrorMessage( messageResource.getMessage( "report.create_chart.error.no_selected_leave_types" ) );
        return true;
    }
    if( exception == 'com.actimind.actitime.reports.NoBillableTasksException' )
    {
        this.showChartErrorMessage( messageResource.getEscapedMessage( "report.create_chart.error.no_billable_tasks" ) );
        return true;
    }

    this.showChartErrorMessage( messageResource.getMessage( "report.create_chart.error.unexpected_error" ) );
    return false;
};

CreateChartLightbox.prototype.getSortCategoryFunctionForChart = function() {
    var groupingLevel = this.chartDataParametersSectionController.getGroupingLevel();
    return ReportChartUtils.getSortCategoryFunctionForChart(groupingLevel);
};

CreateChartLightbox.prototype.changeShowPercents = function( checked )
{
    this.chartController.changeShowPercents( checked );
};

CreateChartLightbox.prototype.validate = function ()
{
    return this.chartDataParametersSectionController.validate();
};

CreateChartLightbox.prototype.updateUI = function ()
{
    return this.chartDataParametersSectionController.updateUI();
};

CreateChartLightbox.prototype.switchLightboxToCreateChartView = function ()
{
    this.cancelAllExportingProcesses();
    $( this.lightBoxEl ).removeClass("pdfPreviewMode").addClass("createChartMode");
    this.titleEl.text( messageResource.getMessage("report.create_chart.title") );
};

CreateChartLightbox.prototype.cancelAllExportingProcesses = function ()
{
    this.pdfPreviewViewController.cancelAllExportingProcesses();
};

CreateChartLightbox.prototype.showEmptyChartDataMessage = function ()
{
    this.showChartErrorMessage( messageResource.getMessage( "report.create_chart.error.empty_result_data" ) );
};

CreateChartLightbox.prototype.showChartErrorMessage = function ( errorMessage )
{
    this.createChartErrorEl.find(".createChartErrorMessage" ).html( errorMessage );
    this.innerContent.addClass("inErrorState");
    this.disableCommitButton();
};

CreateChartLightbox.prototype.showChartErrorMessageWithControls = function ( )
{
    this.createChartErrorEl.find(".createChartErrorMessage" ).html( messageResource.getMessage( "report.create_chart.error.empty_result_data" ) );
    this.innerContent.addClass("emptyPieState");
    this.disableCommitButton();
};


CreateChartLightbox.prototype.removeChartErrorMessage = function ()
{
    this.innerContent.removeClass("inErrorState");
    this.innerContent.removeClass("emptyPieState");
    this.enableCommitButton();
};

CreateChartLightbox.prototype.showLoadingChartSpinner = function ()
{
    this.innerContent.addClass("inLoadingState");
    this.spinnerController.spinInElement( this.innerContent.find( ".createChartSpinnerPlaceholder" ) );
    this.disableCommitButton();
};

CreateChartLightbox.prototype.hideLoadingChartSpinner = function ()
{
    this.innerContent.removeClass("inLoadingState");
    this.spinnerController.stop();
    this.enableCommitButton();
};

CreateChartLightbox.prototype.disableCommitButton = function ()
{
    this._commitEnabled = false;
    $( this.commitButton ).addClass( 'disabled' );
};

CreateChartLightbox.prototype.enableCommitButton = function ()
{
    this._commitEnabled = true;
    $( this.commitButton ).removeClass( 'disabled' );
};

CreateChartLightbox.prototype.write_cookie = function(name, value) {
    $.cookie(name,  JSON.stringify(value));
};

CreateChartLightbox.prototype.read_cookie = function(name) {
    var result = $.cookie(name);
    result = JSON.parse(result);
    return result;
};

CreateChartLightbox.prototype.showOrHideNotFullDataMessage = function ( minValue, maxValue ) {
    if (minValue < 0 && maxValue > 0)
        $("#infoMessageLine" ).html(messageResource.getMessage( "report.create_chart.negative_values_are_missed_message" ) );
    else
        $("#infoMessageLine" ).html("");
};

CreateChartLightbox.prototype.showOrHideNotFullDataMessage = function ( minValue, maxValue ) {
    if (minValue < 0 && maxValue > 0)
        $("#infoMessageLine" ).html(messageResource.getMessage( "report.create_chart.negative_values_are_missed_message" ) );
    else
        $("#infoMessageLine" ).html("");
};