CreateChartLightboxPDFPreviewViewController = function() {};

CreateChartLightboxPDFPreviewViewController.prototype = _.create(PDFPreviewController.prototype, {

    init: function(lightBoxEl, titleEl, elementsIdPrefix, printParameters, switchBackCallback, chartController, selectorsData)
    {
        ReportChartsStore.setPrintParametersForSave(printParameters);

        PDFPreviewController.prototype.init.call(this, lightBoxEl, titleEl, elementsIdPrefix);

        this.backToChartParametersBtn = $(document.getElementById( elementsIdPrefix + "_backBtn" ));
        this.backToChartParametersBtn.off();
        this.backToChartParametersBtn.click( switchBackCallback );

        var self = this;
        this.pdfGenerator = new ChartPDFGenerator().init( function(pdfData) { self.onGeneratePDFCompleted(pdfData); } );
        this.chartController = chartController;
        this.selectorsData = selectorsData;

        return this;
    },

    _initPdfSettingsPreview: function(elementsIdPrefix)
    {
        if (!CreateChartLightboxPDFPreviewViewController.pdfSettingsPreview)
        {
            CreateChartLightboxPDFPreviewViewController.pdfSettingsPreview =
                    new ChartsPDFSettingsPreview($(this.lightBoxEl).find(".pdfPreviewContainer"), elementsIdPrefix).init();
        }
        this.pdfSettingsPreview = CreateChartLightboxPDFPreviewViewController.pdfSettingsPreview;
    },

    _subscribeForEvents: function()
    {
        var self = this;
        $(document).on("reportCharts.intermediatePrintParametersChange", function(event, printParameters)
        {
            ReportChartsStore.setPrintNumberOfPages(self._calculateTotalNumberOfPages(printParameters));
        });
        $(document).on("reportCharts.printParametersForSaveChange", function(event, printParametersForSave)
        {
            ReportChartsStore.setPrintParametersForSave(printParametersForSave);
            self.cancelAllExportingProcesses();
            self.updatePDF();
        });
        $(document).on("reportCharts.createChartLightbox.hide", function()
        {
            $(document)
                    .off("reportCharts.createChartLightbox.hide")
                    .off("reportCharts.intermediatePrintParametersChange")
                    .off("reportCharts.printParametersForSaveChange");
        });
    },

    cancelAllExportingProcesses: function ()
    {
        this.chartController.breakExport();
        this.pdfGenerator.interruptGeneration();
        this.pdfViewer.interruptPreview();
    },

    switchLightboxToPDFPreviewView: function (selectedParameters, chartType, chartDataController)
    {
        this.chartType = chartType;
        this.chartDataController = chartDataController;
        this.selectedParameters = selectedParameters;

        $( this.lightBoxEl ).removeClass("createChartMode").addClass("pdfPreviewMode");

        this._setTitle(
                this.createFormattingStrategy(selectedParameters.chartTypeValue),
                selectedParameters
        );

        this.pdfSettingsPreview.onShow(
            this.chartType,
            this.chartDataController.getCountPoints(),
            this.chartController.getCurrentBarCount(),
            this.chartDataController.getMinValue(),
            this.chartDataController.getMaxValue()
        );

        ReportChartsStore.setPrintNumberOfPages(this._calculateTotalNumberOfPages(this.pdfSettingsPreview.collectPrintParametersForGenerate()));

        this.updatePDF();
    },

    createFormattingStrategy: function ( chartTypeValue )
    {
        switch (chartTypeValue)
        {
            case ReportChartType.TOTAL_WORKING_HOURS:
            case ReportChartType.APPROVED_WORKING_HOURS:
            case ReportChartType.COST_OF_WORK:
            case ReportChartType.BILLABLE_AMOUNTS:
                return new WorkingHoursCostBillingParametersFormattingStrategy().init(this.selectorsData, this.dateFormat);
            case ReportChartType.LEAVE_HOURS:
                return new LeaveParametersFormattingStrategy().init(this.selectorsData, this.dateFormat);
            case ReportChartType.PTO_BALANCE:
            case ReportChartType.SICK_DAYS_BALANCE:
                return new BalanceParametersFormattingStrategy().init(this.selectorsData, this.dateFormat);
            case ReportChartType.OVERTIME_HOURS:
                return new OvertimeParametersFormattingStrategy().init(this.selectorsData, this.dateFormat);
        }
    },

    updatePDF: function(){
        var self = this;

        this.disableDownloadPDFButton();
        this.pdfSettingsPreview.beforePreviewUpdate();
        this.pdfViewer.showMoire();

        var printParams = this.pdfSettingsPreview.collectPrintParametersForGenerate();

        this.chartController.exportChart(
                self.pdfGenerator.getNeededChartDimensionsInPixels(self.chartType, printParams),
                function(pagesData)
                {
                    self.pdfGenerator.generatePDF({
                                                      pagesData: pagesData,
                                                      totalValue: self.chartDataController.getTotal(),
                                                      chartType: self.chartType,
                                                      pdfPrintParameters: printParams,
                                                      selectedParameters: self.selectedParameters,
                                                      selectorsData: self.selectorsData,
                                                      dateFormat: self.dateFormat
                                                  })
                },
                printParams.barsOnPage
        );
    },

    onDownloadPDFBtnClicked: function()
    {
        if (!this.downloadPDFBtnEnabled)
            return;

        this.pdfSettingsPreview.sendSaveChartPrintParametersRequest();
        this.pdfGenerator.downloadPDF();
    },

    onGeneratePDFCompleted: function ( pdfData )
    {
        this.pdfViewer.showPDF(pdfData);
        this.enableDownloadPDFButton();
    },

    _calculateTotalNumberOfPages: function(printParameters)
    {
        return this.pdfGenerator.calculateTotalNumberOfPages(this.chartDataController.getCountPoints(), this.chartType, printParameters);
    }
});