function BaseFormatterCreator(){}

BaseFormatterCreator.prototype.DATE_FORMAT = SystemSettings.dateFormat;
BaseFormatterCreator.prototype.TIME_FORMAT = SystemSettings.defaultTimeFormatForHTMLReports;

BaseFormatterCreator.prototype.getCategoryToStringFunction = function( selectedCategoryType ) {
    var self = this;
    switch ( selectedCategoryType ) {
        case ReportGroupingLevel.USER:
            return function( user ) {
                return UserNameFormat.formatLFM( user.firstName, user.lastName, user.middleName);
            };
        case ReportGroupingLevel.USER_GROUP:
            return function( group ) {
                if (group.id == SystemSettings.usersWithoutGroupId) {
                    return messageResource.getMessage("common.users_without_group");
                }
                return group.name;
            };
        case ReportGroupingLevel.TIME_ZONE_GROUP:
            return function( group ) {
                var name = group.name;
                if (group.id == DEFAULT_TIME_ZONE_GROUP_ID) {
                    name += " " + messageResource.getMessage("common.default_group")
                }
                return name;
            };
        case ReportGroupingLevel.CUSTOMER:
            return function( customer ) {
                return customer.name;
            };
        case ReportGroupingLevel.PROJECT:
            return function( project ) {
                return project.name + " (" + project.customerInfo.name + ")";
            };
        case ReportGroupingLevel.BILLING_TYPE:
            return function( type ) {
                return type.name;
            };
        case ReportGroupingLevel.DATE:

            return function( date ) {
                if ($.isEmptyObject(date)) return "";
                return new DateFormat(self.DATE_FORMAT).formatDate(date.parseJSONDate());
            };

        case ReportGroupingLevel.WEEK:
            return function( week ) {
                if ($.isEmptyObject(week)) return "";
                var from = week.from.parseJSONDate();
                var to = week.to.parseJSONDate();
                var firstPart = week.from == week.to ? "" : new DateFormat('MMM dd').formatDate(from) + " - ";
                return firstPart + new DateFormat('MMM dd, yyyy').formatDate(to);
            };

        case ReportGroupingLevel.MONTH:
            return function( month ) {
                if ($.isEmptyObject(month)) return "";
                return new DateFormat('MMMM , yyyy').formatDate(month.parseJSONDate());
            };
    }
};

BaseFormatterCreator.prototype.getValueFormattedFunction = function(selectedValueType){
    var self = this;
    var numberFormatter = new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 2, false);

    switch(selectedValueType){
        case ReportChartType.APPROVED_WORKING_HOURS:
        case ReportChartType.LEAVE_HOURS:
        case ReportChartType.OVERTIME_HOURS:
        case ReportChartType.TOTAL_WORKING_HOURS:
            return function(value){
                if (!value.toString().length) return "";
                return self.TIME_FORMAT == TimeFormat.TIME ?
                       UserTime.formatTime( value ) :
                       numberFormatter.format( value / 60 );
            };
        case ReportChartType.PTO_BALANCE:
            return function(value){
                return numberFormatter.format( SystemSettings.showPTOInHours ? value * (DEFAULT_WORKDAY_DURATION / 60) : value);
            };
        case ReportChartType.SICK_DAYS_BALANCE:
            return function(value){
                return numberFormatter.format( SystemSettings.showSickInHours ? value * (DEFAULT_WORKDAY_DURATION / 60) : value);
            };
        case ReportChartType.BILLABLE_AMOUNTS:
        case ReportChartType.COST_OF_WORK:
            return function(value) {
                return numberFormatter.format(value);
            }
    }
};

BaseFormatterCreator.prototype.getMeasureUnit = function(selectedValueType){
    switch(selectedValueType){
        case ReportChartType.APPROVED_WORKING_HOURS:
        case ReportChartType.LEAVE_HOURS:
        case ReportChartType.OVERTIME_HOURS:
        case ReportChartType.TOTAL_WORKING_HOURS:
            return messageResource.getMessage("report.create_chart.man_hours_sign");
        case ReportChartType.PTO_BALANCE:
            return SystemSettings.showPTOInHours ? messageResource.getMessage("report.create_chart.hours_sign") : messageResource.getMessage("report.create_chart.days_sign");
        case ReportChartType.SICK_DAYS_BALANCE:
            return SystemSettings.showSickInHours ? messageResource.getMessage("report.create_chart.hours_sign") : messageResource.getMessage("report.create_chart.days_sign");
        case ReportChartType.BILLABLE_AMOUNTS:
        case ReportChartType.COST_OF_WORK:
            return escapeHtmlSymbols(SystemSettings.currencySign);
    }
};