// TODO: this doesn't comply to the new naming convention, because all other components under /reports/.. are old
function ChartsPDFSettingsPreview($containerEl, lightboxId)
{
    PDFSettingsPreview.call(this, $containerEl, lightboxId);
}

ChartsPDFSettingsPreview.prototype = _.create(PDFSettingsPreview.prototype, {
    afterRender: function()
    {
        PDFSettingsPreview.prototype.afterRender.call(this);
        this.lastSavedPaperOrientation = this.getStore().getPrintParametersForSave().paperOrientation;

        this.customSettingsSection = new ChartsPDFSettingsPreview.CustomSettingsSection(
                this.getCustomSettingsSectionContainer(),
                this.printParameters,
                this.lightboxId,
                this.changesCallback
        ).init();
    },

    update: function()
    {
        PDFSettingsPreview.prototype.update.call(this);
        this.customSettingsSection.update();
    },

    getStore: function()
    {
        return ReportChartsStore;
    },

    triggerIntermediatePrintParametersChange: function(newPrintParameters)
    {
        $(document).trigger("reportCharts.intermediatePrintParametersChange", newPrintParameters);
    },

    triggerPrintParametersForSaveChange: function(printParametersForSave)
    {
        $(document).trigger("reportCharts.printParametersForSaveChange", printParametersForSave);
    },

    /**
     * Show checkbox if there is data list and number of data values between maxOnPage and maxOnFitPage
     * or
     * if there is no data list but number of data points more than 2 * maxOnPage and chart type is not PIE(for pie without data only one page is used)
     * @param totalNumberOfPoints number of values in current chart
     */
    _updateFitDataCheckboxVisibility: function(totalNumberOfPoints)
    {
        var paperOrientation = this.chartType == ChartType.PIE_CHART ? PaperOrientation.PORTRAIT : this._getPaperOrientationParameter();
        var paperSize = this._getPaperSizeParameter();
        var maxValuesOnFitPage = ChartPDF.getMaxValuesOnFitPage(
                paperOrientation,
                paperSize,
                this.chartType
        );
        var maxValuesOnPage = ChartPDF.getMaxValuesOnPage(
                paperOrientation,
                paperSize,
                this.chartType
        );
        var tableIsShowed =
                this.customSettingsSection.showListOfDataValuesCheckbox.checked ||
                (this.customSettingsSection.showListOfPercentValuesCheckboxIsVisible
                        ? this.customSettingsSection.showListOfPercentValuesCheckbox.checked : false);


        var needToShowCheckBox = tableIsShowed ?
                                 (totalNumberOfPoints > maxValuesOnPage && totalNumberOfPoints <= maxValuesOnFitPage) :
                                 (this.chartType != ChartType.PIE_CHART && totalNumberOfPoints > 2 * maxValuesOnPage);

        if (needToShowCheckBox)
        {
            this.isfitDataCheckboxVisible = true;
            this.section.removeClass("fitDataCheckboxHidden");
        }
        else
        {
            this.section.addClass("fitDataCheckboxHidden");
            this.isfitDataCheckboxVisible = false;
        }
    },

    // this method removes old chart size selector from dom and initializes new selector according to current chart and print parameters
    _reinitializeChartSizeSelector: function(numberOfBars)
    {
        this.customSettingsSection.chartSizeSelectorContainer.innerHTML = "";

        var dataFitToOnePage = this.isfitDataCheckboxVisible ? this.customSettingsSection.fitDataToOnePageCheckbox.checked : false;

        var maxNumberOfBarsForCurrentChart = dataFitToOnePage ?
                                             this.totalNumberOfPoints :
                                             Math.min(this.totalNumberOfPoints, this.getMaxCountBarsOnPage());

        // save previous value if selector is enable or set prev value if selector is disable
        if (this.customSettingsSection.chartSizeSelector && !this.customSettingsSection.chartSizeSelector.disabled)
        {
            if (numberOfBars)
            {
                this.prevChartSizeValue = numberOfBars;
            }
        }
        else
        {
            if (this.prevChartSizeValue)
            {
                numberOfBars = this.prevChartSizeValue;
            }
        }

        var initialValue = numberOfBars > 0 ? Math.min(numberOfBars, maxNumberOfBarsForCurrentChart) : maxNumberOfBarsForCurrentChart;

        var chartSizeItems = [];
        for (var i = 1; i <= maxNumberOfBarsForCurrentChart; i++)
            chartSizeItems.push({text: i, value: i});

        this.customSettingsSection.chartSizeSelector = new ListSelector({
            menuCls: "chartSizeSelectorMenu",
            renderTo: this.customSettingsSection.chartSizeSelectorContainer,
            listItems: chartSizeItems,
            value: dataFitToOnePage ? maxNumberOfBarsForCurrentChart : initialValue
        });
        this.customSettingsSection.chartSizeSelector.on("change", this.changesCallback);
        if (dataFitToOnePage)
        {
            this.customSettingsSection.chartSizeSelector.disable();
        }
    },

    handleParameterChange: function()
    {
        this._updateFitDataCheckboxVisibility(this.totalNumberOfPoints);

        if (this.chartType != ChartType.PIE_CHART)
            this._reinitializeChartSizeSelector(this.getCurrentChartSizeSelectorValue());

        PDFSettingsPreview.prototype.handleParameterChange.call(this);
    },

    getCurrentChartSizeSelectorValue: function()
    {
        return this.customSettingsSection.chartSizeSelector.getSelectedItem().value;
    },

    getMaxCountBarsOnPage: function()
    {
        return ChartPDF.getMaxValuesOnPage(
                this._getPaperOrientationParameter(),
                this._getPaperSizeParameter(), this.chartType);
    },

    collectPrintParametersForGenerate: function()
    {
        var printParameters = PDFSettingsPreview.prototype.collectPrintParametersForGenerate.call(this);
        if (this.chartType == ChartType.PIE_CHART)
            printParameters.paperOrientation = PaperOrientation.PORTRAIT;

        printParameters.showListOfDataValues = this.customSettingsSection.showListOfDataValuesCheckbox.checked;
        if (this.chartType != ChartType.PIE_CHART)
        {
            printParameters.barsOnPage = this.getCurrentChartSizeSelectorValue();
        }
        printParameters.showListOfPercentValues = this.customSettingsSection.showListOfPercentValuesCheckboxIsVisible
                ? this.customSettingsSection.showListOfPercentValuesCheckbox.checked : false;
        printParameters.fitDataToOnePage = this.isfitDataCheckboxVisible ? this.customSettingsSection.fitDataToOnePageCheckbox.checked : false;

        return printParameters;
    },

    collectPrintParametersForSave: function()
    {
        var printParametersForSaving = PDFSettingsPreview.prototype.collectPrintParametersForSave.call(this);
        if (this.chartType == ChartType.PIE_CHART)
        {
            printParametersForSaving.paperOrientation = this.lastSavedPaperOrientation;
        }

        printParametersForSaving.showListOfDataValues = this.customSettingsSection.showListOfDataValuesCheckbox.checked;
        printParametersForSaving.showListOfPercentValues = this.customSettingsSection.showListOfPercentValuesCheckboxIsVisible
                ? this.customSettingsSection.showListOfPercentValuesCheckbox.checked : this.customSettingsSection.lastSavedShowListOfPercentValues;
        printParametersForSaving.fitDataToOnePage = this.isfitDataCheckboxVisible
                ? this.customSettingsSection.fitDataToOnePageCheckbox.checked : this.customSettingsSection.lastSavedFitDataToOnePage;

        return printParametersForSaving;
    },

    updateNeededLastSavedParameters: function()
    {
        var lastSavedPrintParameters = ReportChartsStore.getPrintParametersForSave();
        this.lastSavedPaperOrientation = lastSavedPrintParameters.paperOrientation;
        this.customSettingsSection.lastSavedShowListOfPercentValues = lastSavedPrintParameters.showListOfPercentValues;
        this.customSettingsSection.lastSavedFitDataToOnePage = lastSavedPrintParameters.fitDataToOnePage;
    },

    onShow: function(chartType, totalNumberOfPoints, numberOfBars, minValue, maxValue)
    {
        this.chartType = chartType;

        this.totalNumberOfPoints = totalNumberOfPoints;

        this._updateShowListOfPercentValuesCheckboxVisibility(minValue, maxValue);
        this._updateFitDataCheckboxVisibility(totalNumberOfPoints);

        if (this.chartType != ChartType.PIE_CHART)
        {
            this._reinitializeChartSizeSelector(numberOfBars);
        }
    },

    _updateShowListOfPercentValuesCheckboxVisibility: function(minValue, maxValue)
    {
        if (minValue < 0 && maxValue > 0)
        {
            this.customSettingsSection.showListOfPercentValuesCheckboxIsVisible = false;
            this.section.addClass("percentCheckboxHidden");
        }
        else
        {
            this.customSettingsSection.showListOfPercentValuesCheckboxIsVisible = true;
            this.section.removeClass("percentCheckboxHidden");
        }
    },

    sendSaveChartPrintParametersRequest: function()
    {
        var self = this;
        makeRpcCall(
                "ReportChartService.saveChartPrintParameters",
                [ReportChartsStore.getPrintParametersForSave()],
                function() { self.updateNeededLastSavedParameters() },
                function() { return true; }
        );
    }
});