function ChartPDFCreateDefinitionObjectStrategy(){}

ChartPDFCreateDefinitionObjectStrategy.prototype.createDocDefinitionObject = function(ddBuilder, pagesData, pdfPrintParameters){
    var adjustedPagesData = this.adjustDataParameters(pagesData, pdfPrintParameters);

    var dd = ddBuilder.createNewDocDefinitionWithFooter();

    var needToShowTable = pdfPrintParameters.showListOfDataValues || pdfPrintParameters.showListOfPercentValues;
    for(var i = 0; i < adjustedPagesData.length; i++){
        var isFirstPage = (i == 0);
        var currentPage = adjustedPagesData[i];
        if(!isFirstPage) {
            ddBuilder.addPageBreaker(dd);
        }
        if(isFirstPage || pdfPrintParameters.showHeaderOnEachPage){
            ddBuilder.addHeader(dd, pdfPrintParameters.showLogoInHeader );
        }
        if (needToShowTable){
            this.addPageDataWithDataList(ddBuilder, dd, currentPage, pdfPrintParameters, isFirstPage);
        } else {
            this.addPageDataWithoutDataList(ddBuilder, dd, currentPage);
        }
    }
    return dd;
};

/**
 * if need to adjust parameters for parameters specific
 * @param pagesData - original data
 * @param pdfPrintParameters - pdf display parameters
 */
ChartPDFCreateDefinitionObjectStrategy.prototype.adjustDataParameters = function(pagesData, pdfPrintParameters) {
  // abstract
};

/**
 * Add page to doc definition if need to show data list
 * @param ddBuilder - doc definition builder
 * @param docDefinition - current doc definition
 * @param pageData - current page data (image + data list)
 * @param pdfPrintParameters - pdf display parameters
 * @param isFirstPage - is current page is first
 */
ChartPDFCreateDefinitionObjectStrategy.prototype.addPageDataWithDataList = function(ddBuilder, docDefinition, pageData, pdfPrintParameters, isFirstPage){
  // abstract
};

/**
 * Add page to doc definition if no need show data list
 * @param ddBuilder - doc definition builder
 * @param docDefinition - current doc definition
 * @param pageData - current page data (only image(es))
 */
ChartPDFCreateDefinitionObjectStrategy.prototype.addPageDataWithoutDataList = function(ddBuilder, docDefinition, pageData) {
  // abstract
};