var ChartPDF = {};
/*
Its file contains parts for construct pdf files
All sizes are specific for pdfmake .For convert it to mm need divide by coeff = 2.835
 */
ChartPDF.TemplateConstructors = {

    imageWithSizes: function(imageSizes, imageBase64) {
        return {
            height: imageSizes.h,
            width: imageSizes.w,
            image: imageBase64
        };
    },

    fakeImage: function() {
        return '';
    },

    logoPicture: function( logoPictureBase64, dimensions){
        return {
            width: dimensions.w,
            height: dimensions.h,
            marginTop: dimensions.mT,
            marginLeft: dimensions.mL,
            rowSpan: 3,
            image: logoPictureBase64
        };
    },

    filterView: function(filterViewParams, firstRowText, secondRowText){
        var margins = filterViewParams.margins;
        var marginsImage = filterViewParams.imageMargins;
        var firstRowMargins = filterViewParams.firstRowMargins;
        var secondRowMargins = filterViewParams.secondRowMargins;

        return {
            style: 'filterView',
            layout: 'thinBordersNoPaddingsForFilterView',
            marginTop: margins.top,
            fontSize: filterViewParams.fontSize,

            table: {
                widths: filterViewParams.widths,

                body: [
                    [
                        {
                            marginLeft: marginsImage.left,
                            marginTop: marginsImage.top,
                            rowSpan: 2,
                            width: filterViewParams.imageWidth,
                            // filter.png image in base64 format
                            image: 'filter-for-pdf.png' // this image is placed in vfs_fonts.js
                        },
                        { marginTop: firstRowMargins.top, marginBottom: firstRowMargins.bottom , noWrap: true, text: firstRowText}
                    ],
                    [
                        '',
                        { marginBottom: secondRowMargins.bottom, noWrap: true, text: secondRowText}
                    ]
                ]
            }
        };
    },

    chartInformation: function( chartInfoParams, logoElement, reportName, dateDescription, grandTotalDescription, grandTotalValue){
        var columnWidths = chartInfoParams.widths;
        var splitterLen = chartInfoParams.splitterLen;
        return {
            widths: columnWidths,
            body: [
                [
                    logoElement,
                    [
                        { style: 'chartNameLabel', text: reportName },
                        { style: 'dateRangeLabel', text: dateDescription }
                    ]
                ],
                [
                    '',
                    {
                        style: 'headerSeparator',
                        canvas: [
                            {
                                type: 'line',
                                x1: 0, x2: splitterLen,
                                y1: 0, y2:0,
                                lineWidth: 0.9,
                                dash: {length: 2},
                                lineColor: '#b7b7b7'
                            }
                        ]
                    }
                ],
                [
                    '',
                    {
                        style: 'grandTotalLabel',
                        text:
                                [
                                    { text: grandTotalDescription.label},
                                    ' ',
                                    { style: 'grandValue', text: grandTotalValue },
                                    ' ',
                                    { style: 'grandPostfix', text: grandTotalDescription.postfix}
                                ]
                    }
                ]
            ]
        };
    },

    colorValuePercentCell: function( color, categoryName, value, percents, styles ){
        var rectDimensions = styles.colorRectangleDimensions;
        return [
            {
                width: rectDimensions.width,
                canvas: [
                    {
                        type: 'rect',
                        x:rectDimensions.x,
                        y:rectDimensions.y,
                        w:rectDimensions.w,
                        h:rectDimensions.h,
                        color: color,
                        fillOpacity: 0.001 //need to hide rect without color
                    }
                ]
            },
            {width: '*', noWrap: true, style: styles.dataNameStyle, text: categoryName},
            {width: 'auto', noWrap: true, style: styles.dataValueWithPercentStyle, text: value},
            {width: 43, style: styles.dataValueStyle, text: percents}
        ]
    },

    colorValueCell: function( color, categoryName, value, styles ){
        var rectDimensions = styles.colorRectangleDimensions;
        return [
            {
                width: rectDimensions.width,
                canvas: [
                    {
                        type: 'rect',
                        x:rectDimensions.x,
                        y:rectDimensions.y,
                        w:rectDimensions.w,
                        h:rectDimensions.h,
                        color: color,
                        fillOpacity: 0.001 //need to hide rect without color
                    }
                ]
            },
            {width: '*', noWrap: true, style: styles.dataNameStyle, text: categoryName},
            {width: 'auto', noWrap: true, style: styles.dataValueStyle, text: value}
        ]
    },

    valueCell: function(categoryName, value, styles){
        return [
            {width: '*', noWrap: true, style: styles.dataNameStyle, text: categoryName},
            {width: 'auto', noWrap: true, style: styles.dataValueStyle, text: value}
        ]
    },

    cpRow: function(name, formattedValue, description, styles)
    {
        return [
            {style: styles.nameValueStyle, text: name},
            {style: styles.timeValueStyle, text: formattedValue},
            {style: styles.commentValueStyle, text: description}
        ];
    },

    headerCell: function(text, styles)
    {
        return [
            {width: '100%', noWrap: true, style: styles.tableHeaderStyle, text: text, colSpan: 3}, {}, {}
        ]
    },

    dateCell: function(value, formattedValue, styles)
    {                     
        return [
            {width: '*', noWrap: true, style: styles.dataNameStyle, text: value.name},
            {width: 'auto', noWrap: true, style: styles.dataNameStyle, text: formattedValue},
            {width: 100, noWrap: true, style: styles.dataNameStyle, text: !!value.description ? value.description : ""}
        ]
    },

    valuePercentCell: function(categoryName, value, percents, styles) {
        return [
            {width: '*', noWrap: true, style: styles.dataNameStyle, text: categoryName},
            {width: 'auto', noWrap: true, style: styles.dataValueWithPercentStyle, text: value},
            {width: 43, style: styles.dataValueStyle, text: percents}

        ]
    },

    singleColumnRow: function(cell, isSecondRow) {
        var rowStyle = isSecondRow ? 'secondDataRow' : '';
        return [
            {
                style: rowStyle,
                columns: cell
            },
            '',
            ''
        ];
    },

    doubleColumnsRow: function(leftCell, rightCell, isSecondRow) {
        var rowStyle = isSecondRow ? 'secondDataRow' : '';
        return [
            {
                style: rowStyle,
                columns: leftCell
            },
            '',
            {
                style: rowStyle,
                columns: rightCell
            }
        ];
    }
};

ChartPDF.PlaceholderConstructors = {

    headerLandscape: function( chartInformation, filterView ){
        return {
            columns: [
                {
                    width:'67%',
                    layout: 'noBordersNoPadding',

                    table: chartInformation
                },
                filterView
            ]
        };
    },

    headerPortrait: function( chartInformation ){
        return {
            layout: 'noBordersNoPadding',

            table: chartInformation
        };
    },

    singleImage: function( image, margins ){
        return {
            layout: 'noBordersNoPadding',
            marginTop: margins.top,
            table: {
                width: ['100%'],
                body: [
                    [
                        image
                    ]
                ]
            }
        };
    },

    pageBreaker : function() {
        return {
            pageBreak: 'after',
            text: ''
        }
    },

    horizontalBarsChartDoubleImage: function(imageLeft, imageRight, marginsImage){
        return {
            columns: [
                {
                    layout: 'noBordersNoPadding',
                    width: '49%',
                    marginTop: marginsImage.top,
                    table: {
                        width: ['100%'],
                        body: [
                            [
                                imageLeft
                            ]
                        ]
                    }
                },
                {
                    width: '2%',
                    text: ''
                },
                {
                    layout: 'noBordersNoPadding',
                    marginTop: marginsImage.top,
                    table: {
                        width: ['100%'],
                        body: [
                            [
                                imageRight
                            ]
                        ]
                    }
                }
            ]
        };
    },

    horizontalBarsChartImageAndData: function(image, dataRows, marginsImage, marginsTable, tableStyle){
        return {
            columns: [
                {
                    width: '67%',
                    layout: 'noBordersNoPadding',
                    marginTop: marginsImage.top,
                    table: {
                        width: ['100%'],
                        body: [
                            [
                               image
                            ]
                        ]
                    }
                },
                {
                    style: tableStyle,
                    layout: 'noBordersNoPadding',
                    marginTop: marginsTable.top,
                    table: {
                        dontBreakRows: true,
                        widths: ['100%',0,0],
                        body: dataRows
                    }
                }
            ]
        };
    },

    chartDoubleColumnsTable: function(dataRows, margins, tableStyle) {
        return {
            style: tableStyle,
            layout: 'noBordersNoPadding',
            marginTop: margins.top,

            table: {
                dontBreakRows: true,
                widths: ['49%', '2%', '49%'],
                body: dataRows
            }
        };
    }
};


ChartPDF.Options = {
    tableLayouts : {
        noBordersNoPadding : {
            hLineWidth: function(i, node) { return 0; },
            vLineWidth: function(i, node) { return 0; },
            paddingLeft: function(i, node) { return 0; },
            paddingRight: function(i, node) { return 0; },
            paddingTop: function(i, node) { return 0; },
            paddingBottom: function(i, node) { return 0; }
        },
        simpleBorders : {
            hLineWidth: function(i, node) { return 1; },
            vLineWidth: function(i, node) { return 1; },
            paddingLeft: function(i, node) { return 0; },
            paddingRight: function(i, node) { return 0; },
            paddingTop: function(i, node) { return 0; },
            paddingBottom: function(i, node) { return 0; }
        },
        thinBordersNoPaddingsForFilterView: {
            hLineWidth: function (i, node) { return 0.4; },
            vLineWidth: function (i, node) { return 0.4; },
            paddingLeft: function (i, node) { return 0; },
            paddingRight: function (i, node) { return 0; },
            paddingTop: function (i, node) { return 0; },
            paddingBottom: function (i, node) { return 0; },
            hLineColor: function (i, node) { return '#EFEFEF' },
            vLineColor: function (i, node) { return '#EFEFEF' }
        },
        headerBorderSeparatedRows: {
            hLineWidth: function(i, node) { return 0.4; },
            vLineWidth: function(i, node) { return 0; },
            paddingLeft: function(i, node) { return 5; },
            paddingRight: function(i, node) { return 5; },
            paddingTop: function(i, node) { return 5; },
            paddingBottom: function(i, node) { return 5; },
            hLineColor: function(i, node) {
                return (i === 0 || i === node.table.body.length) ? '#fbfbfb' : '#f0f0f0';
            },
            vLineColor: function(i, node) {
                return (i === 0 || i === node.table.widths.length) ? 'red' : 'blue';
            }
        }
    }
};

ChartPDF._getSmallStyleIfNeed = function(style, isSmall) {
    return isSmall ? style + "Small": style;
};

ChartPDF.Styles = {

    headerSeparator: {
        marginTop: 6,
        marginBottom: 8
    },
    chartNameLabel: {
        fontSize: 12,
        bold: true
    },
    dateRangeLabel: {
        marginTop: 5,
        fontSize: 12
    },
    grandTotalLabel: {
        fontSize: 9
    },
    grandValue: {
        bold: true
    },
    grandPostfix: {
        bold: true
    },

    filterView: {
        fillColor: '#EFEFEF',
        color: '#373737'
    },
    dataTable: {
        fontSize: 7,
        color: '#111111'
    },

    dataTableSmall : {
        fontSize: 5.4,
        color: '#111111'
    },
    secondDataRow: {
        fillColor: '#EFEFEF'
    },
    dataName: {
        alignment: 'left',
        margin: [10, 2, 0, 2]
    },
    dataNameSmall: {
        alignment: 'left',
        margin: [10, 0, 0, 0]
    },
    dataValue: {
        alignment: 'right',
        margin: [5, 2, 5, 2]
    },
    dataValueSmall :{
        alignment: 'right',
        margin: [5, 0, 5, 0]
    },
    dataValueWithPercents :{
        alignment: 'right',
        margin: [2, 2, 0, 2]
    },
    dataValueWithPercentsSmall :{
        alignment: 'right',
        margin: [2, 0, 0, 0]
    },
    footerTable: {
        margin: [23,0, 23, 0],
        fontSize: 7
    },

    rightFooterPart: {
        alignment: 'right',
        color: '#3D3D3D'
    },
    leftFooterPart: {
        alignment: 'left',
        color: '#6A6A6A'
    }
};

ChartPDF.Footers = {
    simpleFooter : function() {

        return function(currentPage, pageCount) {

            return {
                style: 'footerTable',
                layout: 'noBordersNoPadding',

                table: {
                    widths: ['90%','10%'],
                    body: [
                        [
                            {
                                style: 'leftFooterPart',
                                text: messageResource.getMessage('report.chart_pdf.leftFooterLabel', [new DateFormat( SystemSettings.dateFormat ).formatDate( new Date() )])
                            },
                            {
                                style: 'rightFooterPart',
                                text: messageResource.getMessage('report.chart_pdf.rightFooterLabel', [currentPage, pageCount])
                            }
                        ]
                    ]
                }
            }
        };
    },
    signatureFooter : function() {
        return function(currentPage, pageCount) {

            return {
                style: 'footerTable',
                layout: 'noBordersNoPadding',

                table: {
                    widths: ['90%','10%'],
                    body: [
                        [
                            {
                                style: 'leftFooterPart',
                                text: messageResource.getMessage('view_tt.pdf.signature')
                            },
                            {
                                style: 'rightFooterPart',
                                text: messageResource.getMessage('report.chart_pdf.rightFooterLabel', [currentPage, pageCount])
                            }
                        ]
                    ]
                }
            }
        };
    }
};

/**
 * utils method needed to return sizes of images depends on parameters
 * @param paperOrientation - page orientation
 * @param paperSize - papers size
 * @param chartType - current chartType
 * @param isDoubleImagesOnPage - singleOrDoubleImages on page (double if no need to show data list)
 * @returns {*}
 */
ChartPDF.getNeededImageDimensions = function(paperOrientation, paperSize, chartType, isDoubleImagesOnPage){
    var dimensions = {};
    $.extend(true, dimensions, ChartPDF._getChartDependentsParameters(paperOrientation, paperSize, chartType).imageParams);
    if (isDoubleImagesOnPage){
        return dimensions.doubleImage.sizes;
    } else {
        return dimensions.singleImage.sizes;
    }
};

ChartPDF.getMaxValuesOnPage = function(paperOrientation, paperSize, chartType){
    return ChartPDF._getChartDependentsParameters(paperOrientation, paperSize, chartType).maxValuesOnPage;
};

ChartPDF.getMaxValuesOnFitPage = function(paperOrientation, paperSize, chartType) {
    return ChartPDF._getChartDependentsParameters(paperOrientation, paperSize, chartType).maxValuesOnFitPage;
};

ChartPDF.getMaxValuesOnPageWithoutImage = function(paperOrientation, paperSize, withHeader, chartType){
    if (withHeader){
        return ChartPDF._getChartDependentsParameters(paperOrientation, paperSize, chartType).maxValuesOnPageWithoutImage.withHeader;
    }
    return ChartPDF._getChartDependentsParameters(paperOrientation, paperSize, chartType).maxValuesOnPageWithoutImage.withoutHeader;
};

ChartPDF._getChartDependentsParameters = function(paperOrientation, paperSize, chartType){
    var paramsForReturn = {};
    var orientParams = paperOrientation == PaperOrientation.LANDSCAPE ? ChartPDF.Parameters.Landscape : ChartPDF.Parameters.Portrait;
    var sizeParams = paperSize == PaperSize.A4 ? orientParams.A4 : orientParams.Letter;
    var chartTypeParams = chartType == ChartType.HORIZONTAL_BAR ? sizeParams.horizontalBar :
        chartType == ChartType.VERTICAL_BAR ? sizeParams.verticalBar : sizeParams.pie;
    $.extend(paramsForReturn, chartTypeParams);
    return paramsForReturn;
};

ChartPDF.Parameters = {

    dataTableCommonStyles : {
        small : {
            tableStyle : 'dataTableSmall',
            tableValueStyles : {
                dataValueStyle : 'dataValueSmall',
                dataValueWithPercentStyle : 'dataValueWithPercentsSmall',
                dataNameStyle : 'dataNameSmall',
                colorRectangleDimensions: {
                    width : 8,
                    x: 4,
                    y: 1.25,
                    h: 3.8,
                    w: 7
                }
            }
        },
        big : {
            tableStyle : 'dataTable',
            tableValueStyles : {
                dataValueStyle : 'dataValue',
                dataValueWithPercentStyle : 'dataValueWithPercents',
                dataNameStyle : 'dataName',
                colorRectangleDimensions: {
                    width: 10,
                    x: 4,
                    y: 4,
                    h: 5,
                    w: 9
                }
            }
        },
        
        viewTT: {
            tableHeaderStyle: "viewTTHeader",
            tableStyle : 'viewTTRows',
            tableValueStyles : {
                nameValueStyle : 'viewTTRows',
                timeValueStyle : 'coloredTime',
                commentValueStyle : 'viewTTRows'
            }
        }
    },

    logoPlaceholderSizes : {
        width: 100,
        height: 56
    },

    Landscape : {
        pageOrientation : 'landscape',
        pageMargins: [56, 28, 56, 28],

        filterViewParameters : {
            margins : {
                top: 2
            },
            imageMargins : {
                top: 12,
                left: 10
            },
            firstRowMargins: {
                top: 12,
                bottom: 5
            },
            secondRowMargins: {
                bottom: 12
            },
            widths: ['12%' ,'88%'],
            imageWidth: 9,
            fontSize: 8
        },

        A4 : {
            pageSize: 'A4',

            filterViewTextLen: 51,//57

            chartInfoParameters : {
                withLogo : {
                    splitterLen: 350,
                    widths : ['25%', '75%']
                },
                withoutLogo : {
                    splitterLen: 460,
                    widths : ['0%','100%']
                }
            },

            verticalBar: {
                maxValuesOnPage: 42,
                maxValuesOnFitPage : 84,

                rowDataMaxLen: {
                    normal : {
                        valueAndPercents : 68, // mid 58 safe 49,
                        valueOrPercents : 74 //mid 64 safe 54
                    },
                    fit : {
                        valueAndPercents : 78,
                        valueOrPercents: 84
                    }
                },
                dataTableMargins : {
                    top: 10
                },
                imageParams: {
                    singleImage : {
                        margins : {
                            top: 20
                        },
                        sizes : {
                            h : 180,
                            w : 725
                        }
                    },
                    doubleImage : {
                        margins : {
                            top: 20
                        },
                        sizes :{
                            h : 180,
                            w : 725
                        }
                    }
                }
            },
            horizontalBar: {
                maxValuesOnPage: 38,
                maxValuesOnFitPage: 76,

                rowDataMaxLen : {
                    normal : {
                        valueAndPercents : 41,//mid 36 safe 29,
                        valueOrPercents : 48 //mid 43 safe 35
                    },
                    fit : {
                        valueAndPercents : 51,
                        valueOrPercents: 58
                    }
                },
                dataTableMargins : {
                    top: 0
                },
                imageParams: {
                    singleImage : {
                        margins : {
                            top: 15
                        },
                        sizes : {
                            h : 460,
                            w : 460
                        }
                    },
                    doubleImage : {
                        margins : {
                            top: 15
                        },
                        sizes : {
                            h : 460,
                            w : 354
                        }
                    }
                }
            },
            pie : {} // no need fo this chart type
        },

        Letter : {
            pageSize: 'Letter',

            filterViewTextLen: 48,//54

            chartInfoParameters : {
                withLogo : {
                    splitterLen: 315,
                    widths : ['25%', '75%']
                },
                withoutLogo : {
                    splitterLen: 430,
                    widths : ['0%','100%']
                }
            },

            verticalBar: {
                maxValuesOnPage: 44,
                maxValuesOnFitPage: 88,

                rowDataMaxLen: {
                    normal : {
                        valueAndPercents : 63, //mid 53 safe 45,
                        valueOrPercents : 69 //mid 59 safe 50
                    },
                    fit : {
                        valueAndPercents : 73,
                        valueOrPercents: 79
                    }
                },
                dataTableMargins : {
                    top: 10
                },
                imageParams: {
                    singleImage : {
                        margins : {
                            top: 20
                        },
                        sizes : {
                            h : 180,
                            w : 675
                        }
                    },
                    doubleImage : {
                        margins : {
                            top: 20
                        },
                        sizes : {
                            h : 180,
                            w : 675
                        }
                    }
                }
            },
            horizontalBar: {
                maxValuesOnPage: 38,
                maxValuesOnFitPage: 76,

                rowDataMaxLen : {
                    normal : {
                        valueAndPercents : 38, //mid 33 safe 27,
                        valueOrPercents : 44 //mid 39 safe 32
                    },
                    fit : {
                        valueAndPercents : 48,
                        valueOrPercents: 54
                    }
                },
                dataTableMargins : {
                    top: 15
                },
                imageParams: {

                    singleImage : {
                        margins : {
                            top: 15
                        },
                        sizes : {
                            h : 465,
                            w : 425
                        }
                    },
                    doubleImage : {
                        margins : {
                            top: 15
                        },
                        sizes : {
                            h : 465,
                            w : 330
                        }
                    }
                }
            },
            pie : {} // no need fo this chart type
        }
    },
    Portrait : {
        pageOrientation : 'portrait',
        pageMargins: [ 40, 28, 40, 28 ],

        filterViewParameters : {
            margins : {
                top: 12
            },
            imageMargins : {
                top: 11,
                left: 12
            },
            firstRowMargins: {
                top: 10,
                bottom: 5
            },
            secondRowMargins: {
                bottom: 9
            },
            widths: ['6%' , '94%'],
            imageWidth: 10,
            fontSize: 9
        },

        A4 : {
            pageSize: 'A4',

            filterViewTextLen: 102, //108

            chartInfoParameters : {
                withLogo : {
                    splitterLen: 403,
                    widths : ['22%', '78%']
                },
                withoutLogo : {
                    splitterLen: 517,
                    widths : ['0%','100%']
                }
            },

            verticalBar: {
                maxValuesOnPage: 56,
                maxValuesOnFitPage: 112,

                rowDataMaxLen: {
                    normal : {
                        valueAndPercents : 43,// mid 38 safe 31
                        valueOrPercents : 49// mid 44 safe 36
                    },
                    fit : {
                        valueAndPercents : 53,
                        valueOrPercents: 59
                    }
                },
                dataTableMargins : {
                    top: 18
                },
                imageParams: {
                    singleImage : {
                        margins : {
                            top: 15
                        },
                        sizes : {
                            h : 260,
                            w : 513
                        }
                    },
                    doubleImage : {
                        margins : {
                            top: 15
                        },
                        sizes : {
                            h : 260,
                            w : 513
                        }
                    }
                }
            },
            horizontalBar: {
                maxValuesOnPage:48,
                maxValuesOnFitPage: 96,

                rowDataMaxLen: {
                    normal : {
                        valueAndPercents : 43,//mid 38 safe 31
                        valueOrPercents : 49//mid 44 safe 36
                    },
                    fit : {
                        valueAndPercents : 53,
                        valueOrPercents: 59
                    }
                },
                dataTableMargins : {
                    top: 18
                },
                imageParams: {
                    singleImage : {
                        margins : {
                            top: 15
                        },
                        sizes : {
                            h : 330,
                            w : 513
                        }
                    },
                    doubleImage : {
                        margins : {
                            top: 20
                        },
                        sizes : {
                            h : 290,
                            w : 513
                        }
                    }
                }
            },
            pie : {
                maxValuesOnPage: 52,
                maxValuesOnFitPage: 104,

                maxValuesOnPageWithoutImage: {
                    withHeader: 104,
                    withoutHeader: 124
                },

                rowDataMaxLen: {
                    normal : {
                        valueAndPercents : 41, //mid 36 safe 29
                        valueOrPercents : 49 //mid 44 safe 35
                    },
                    fit : {
                        valueAndPercents : 51,
                        valueOrPercents: 59
                    }
                },
                dataTableMargins : {
                    top: 8
                },
                imageParams: {
                    singleImage : {
                        margins : {
                            top: 25
                        },
                        sizes : {
                            h : 305,
                            w : 513
                        }
                    },
                    doubleImage : {} //no need fot this chart type
                }
            }
        },
        Letter : {
            pageSize: 'Letter',

            filterViewTextLen: 105,// 111

            chartInfoParameters : {
                withLogo : {
                    splitterLen: 415,
                    widths : ['22%', '78%']
                },
                withoutLogo : {
                    splitterLen: 533,
                    widths : ['0%','100%']
                }
            },

            verticalBar: {
                maxValuesOnPage: 48,
                maxValuesOnFitPage: 96,

                rowDataMaxLen: {
                    normal : {
                        valueAndPercents : 44, //mid 39 safe 32,
                        valueOrPercents : 51 //mid 46 safe 38
                    },
                    fit : {
                        valueAndPercents : 54,
                        valueOrPercents: 61
                    }
                },
                dataTableMargins : {
                    top: 18
                },
                imageParams: {
                    singleImage : {
                        margins : {
                            top: 15
                        },
                        sizes : {
                            h : 260,
                            w : 530
                        }
                    },
                    doubleImage : {
                        margins : {
                            top: 15
                        },
                        sizes : {
                            h : 260,
                            w : 530
                        }
                    }
                }
            },
            horizontalBar: {
                maxValuesOnPage: 40,
                maxValuesOnFitPage: 80,

                rowDataMaxLen: {
                    normal : {
                        valueAndPercents : 44, //mid 39 safe 32,
                        valueOrPercents : 51 //mid 46 safe 38
                    },
                    fit : {
                        valueAndPercents : 54,
                        valueOrPercents: 61
                    }
                },
                dataTableMargins : {
                    top: 18
                },
                imageParams: {
                    singleImage : {
                        margins : {
                            top: 15
                        },
                        sizes : {
                            h : 330,
                            w : 530
                        }
                    },
                    doubleImage : {
                        margins : {
                            top: 15
                        },
                        sizes : {
                            h : 280,
                            w : 530
                        }
                    }
                }
            },
            pie : {
                maxValuesOnPage: 44,
                maxValuesOnFitPage: 88,

                maxValuesOnPageWithoutImage: {
                    withHeader: 98,
                    withoutHeader: 116

                },

                rowDataMaxLen: {
                    normal : {
                        valueAndPercents : 43, //mid 38 safe 31,
                        valueOrPercents : 49 //mid 44 safe 36
                    },
                    fit : {
                        valueAndPercents : 53,
                        valueOrPercents: 59
                    }
                },
                dataTableMargins : {
                    top: 8
                },
                imageParams: {
                    singleImage : {
                        margins : {
                            top: 25
                        },
                        sizes : {
                            h : 305,
                            w : 530
                        }
                    },
                    doubleImage : {} //no need fot this chart type
                }
            }
        }
    }
};
