function PieChartPDFCreateDefinitionObjectStrategy(){}

PieChartPDFCreateDefinitionObjectStrategy.prototype = new ChartPDFCreateDefinitionObjectStrategy();


/**
 * for pie chart data are provided in one page, however we need to fit data to some pages (only if it needs)
 * this function split one page data depends on max number of values on one page
 * @param pagesData
 * @param pdfPrintParameters parameters for display pdf
 * @returns {Array}
 * @private
 */
PieChartPDFCreateDefinitionObjectStrategy.prototype.adjustDataParameters = function(pagesData, pdfPrintParameters) {

    // if data need to fit to one page return original data
    if(pdfPrintParameters.fitDataToOnePage){
        return pagesData;
    }
    var newPages = [];
    var currentPage = {
        image : pagesData[0].image,
        data: []
    };

    // if no need data list - return page only with image
    if (!pdfPrintParameters.showListOfDataValues && !pdfPrintParameters.showListOfPercentValues){
        newPages.push(currentPage);
        return newPages;
    }

    var nValuesOnPageWithImage = ChartPDF.getMaxValuesOnPage(PaperOrientation.PORTRAIT, pdfPrintParameters.paperSize, ChartType.PIE_CHART);
    var nValuesOnPageWithoutImage = ChartPDF.getMaxValuesOnPageWithoutImage(PaperOrientation.PORTRAIT, pdfPrintParameters.paperSize, pdfPrintParameters.showHeaderOnEachPage, ChartType.PIE_CHART);

    var currentPageCount = 0;
    var enoughValuesToPage = false;
    for (var i = 0; i < pagesData[0].data.length; i++){
        currentPage.data.push(pagesData[0].data[i]);
        currentPageCount++;

        if(newPages.length == 0){
            if(currentPageCount == nValuesOnPageWithImage){
                enoughValuesToPage = true;
            }
        } else {
            if(currentPageCount == nValuesOnPageWithoutImage){
                enoughValuesToPage = true;
            }
        }

        if (enoughValuesToPage){
            enoughValuesToPage = false;
            newPages.push(currentPage);
            currentPage = {data:[]};
            currentPageCount = 0;
        }
    }
    if (currentPage.data.length != 0){
        newPages.push(currentPage); // add last page if there are no added values
    }
    return newPages;
};

PieChartPDFCreateDefinitionObjectStrategy.prototype.addPageDataWithDataList = function(ddBuilder, docDefinition, pageData, pdfPrintParameters, isFirstPage) {
    if ( isFirstPage ){
        ddBuilder.addImageAndData(docDefinition, pageData.image, pageData.data, pdfPrintParameters.showListOfDataValues, pdfPrintParameters.showListOfPercentValues, pdfPrintParameters.fitDataToOnePage );
    } else {
        ddBuilder.addOnlyTableToPage(docDefinition, pageData.data, pdfPrintParameters.showListOfDataValues, pdfPrintParameters.showListOfPercentValues, pdfPrintParameters.showHeaderOnEachPage );
    }
};

PieChartPDFCreateDefinitionObjectStrategy.prototype.addPageDataWithoutDataList = function(ddBuilder, docDefinition, pageData) {
    ddBuilder.addOnlyImageToPage(docDefinition, pageData.image);
};
