function ChartPDFParametersFormattingStrategy(){
}

ChartPDFParametersFormattingStrategy.prototype.constructor = ChartPDFParametersFormattingStrategy;

ChartPDFParametersFormattingStrategy.prototype.init = function(selectorsData, dateFormat, dateFormatForFile){
    var ugrEnabled = selectorsData.enabledFeatures ?
                     selectorsData.enabledFeatures.hasOwnProperty(ReportChartConstants.UGRFeature) :
                     null;
    var tzgEnabled = selectorsData.enabledFeatures ?
                     selectorsData.enabledFeatures.hasOwnProperty(ReportChartConstants.TZGFeature) :
                     null;
    this.parametersFormatter = new ChartPDFParametersFormatter().init(selectorsData, dateFormat, ugrEnabled, tzgEnabled);
    this.dateParametersFormattingStrategy = this.createDateParametersFormattingStrategy(dateFormat, dateFormatForFile);

    return this;
};

ChartPDFParametersFormattingStrategy.prototype.createDateParametersFormattingStrategy = function ( dateFormat, dateFormatForFile ) {
    // abstract
};

ChartPDFParametersFormattingStrategy.prototype.formatChartPDFFilterInformation = function ( selectedParameters ) {
    // abstract
};

ChartPDFParametersFormattingStrategy.prototype.formatChartPDFHeaderTitle = function( selectedParameters ){
    return messageResource.getMessage(
        'report.chart_pdf.header_label',
        [
            this.parametersFormatter.formatChartTypeValue(selectedParameters.chartTypeValue),
            this.parametersFormatter.formatGroupingLevelValue(selectedParameters.groupingLevelValue)
        ]
    );
};

ChartPDFParametersFormattingStrategy.prototype.formatChartPDFHeaderDateInformation = function( selectedParameters ) {
    return this.dateParametersFormattingStrategy.formatDateInformation(selectedParameters.dateParameters);
};

ChartPDFParametersFormattingStrategy.prototype.formatGrandTotalDescription = function( selectedParameters ){
    return {
        label : messageResource.getMessage('report.chart_pdf.grand_total_label', [this.parametersFormatter.formatGrandTotalValueType(selectedParameters.chartTypeValue)]),
        postfix: this.parametersFormatter.formatGrandTotalPostfix(selectedParameters.chartTypeValue)
    }
};

ChartPDFParametersFormattingStrategy.prototype.formatChartPDFFileName = function( selectedParameters ) {
    var fileNameParts = [];
    fileNameParts.push(this._capitalizeFirstLetterInWords(this.parametersFormatter.formatChartTypeValue(selectedParameters.chartTypeValue)));//show..
    fileNameParts.push(this._capitalizeFirstLetterInWords(this.parametersFormatter.formatGroupingLevelValue(selectedParameters.groupingLevelValue)));//by..
    fileNameParts.push(this.dateParametersFormattingStrategy.formatDateForFileName(selectedParameters.dateParameters));//date range
    return messageResource.getMessage('report.chart_pdf.file_name', fileNameParts);
};

ChartPDFParametersFormattingStrategy.prototype._capitalizeFirstLetterInWords = function(string){
    var replacer = function(str, p1, p2, offset, s){
        return p1 ? p1.toUpperCase() : p2.toUpperCase();
    };
    return string.replace(/(\s\w)|(^\w)/g,replacer);
};