function BaseCSVJsonConverter() {}

BaseCSVJsonConverter.prototype = {
    init: function()
    {
        return this;
    },

    convertData: function(layer, depth, data, line)
    {
        var grouping = this.groupingLevels[layer];
        this.addGroupingInfo(grouping, this.getIdentity(data, grouping), line);

        if (layer + 1 == depth)
        {
            this.addDataToTable(data, line);
            this.lines.push(_.clone(line));
            return;
        }

        for (var i = 0; i < data.items.length; ++i)
            this.convertData(layer + 1, depth, data.items[i], line);
    },

    convertLinearData: function(layer, depth, data)
    {
        var line = {};
        var i;
        for (i = 0; i < depth; i++) {
            var grouping = this.groupingLevels[i];
            if (i == layer) {
                this.addGroupingInfo(grouping, this.getIdentity(data, grouping), line);
            }
        }
        this.addDataToTable(data, line);
        this.lines.push(line);

        if (layer + 1 == depth) {
            return;
        }

        for (i = 0; i < data.items.length; ++i)
            this.convertLinearData(layer + 1, depth, data.items[i]);
    },

    generateTableHeader: function(data, config, columnNames, columns)
    {
        // abstract
    },

    convertTable: function(config, data)
    {
        this.groupingLevels = GroupingUtils.getGroupingLevels(config);
        this.preprocess(config, data);

        var columnNames = {};
        var columns = [];
        this.generateTableHeader(data, config, columnNames, columns);

        // generate entity of table
        this.lines = [];
        for (var i = 0; i < data.items.length; ++i)
            this.convertData(0, this.groupingLevels.length, data.items[i], {});

        return {
            columnNames: columnNames,
            columns: columns,
            table: this.lines
        };
    },

    convertLinearTable: function(config, data)
    {
        this.groupingLevels = GroupingUtils.getGroupingLevels(config);
        this.preprocess(config, data);

        var columnNames = {};
        var columns = [];
        this.generateTableHeader(data, config, columnNames, columns);

        // generate entity of table
        this.lines = [];
        for (var i = 0; i < data.items.length; ++i)
            this.convertLinearData(0, this.groupingLevels.length, data.items[i]);

        return {
            columnNames: columnNames,
            columns: columns,
            table: this.lines
        };
    },
    //
    addGroupingInfo: function(groupingLevel, identity, line)
    {
        // abstract
    },

    //
    getIdentity: function(data, grouping)
    {
        // abstract
    },
    //
    addDataToTable: function(data, line)
    {
        // abstract
    },
    //
    addOtherColumnNames: function(columnNames, columns, config, data)
    {
        // abstract
    },
    //
    preprocess: function(config, data)
    {
        // abstract
    },

    convertTitle: function(params, config)
    {
        var paramsBuilder = new CSVParametersBuilder().init();

        this.processParamsBuilder(paramsBuilder, params, config);

        var parameters = paramsBuilder.getParameters();
        var title = [];
        for (var i = 0; i < parameters.length; ++i)
        {
            var line = {
                title: parameters[i][0],
                description: parameters[i][1]
            };
            title.push(line);
        }
        title.push({title: "", description: ""});

        return {
            columns: ["title", "description"],
            table: title
        };
    },
    //
    processParamsBuilder: function(paramsBuilder, params, config)
    {
        // abstract
    }
};
