var GroupingUtils = {
    getGroupingLevels: function(config)
    {
        var first = config.firstGroupingLevel;
        var second = config.secondGroupingLevel;
        var third = config.thirdGroupingLevel;
        var fourth = config.fourthGroupingLevel;
        return _.filter([first, second, third, fourth], function(group) { return group != null && group != ReportGroupingLevel.NONE;});
    },

    isTimeGrouping: function(group)
    {
        return group == ReportGroupingLevel.DATE ||
               group == ReportGroupingLevel.WEEK ||
               group == ReportGroupingLevel.MONTH;
    },

    searchLastGroupingTime: function(groups)
    {
        var result = ReportGroupingLevel.NONE;
        for (var i = 0; i < groups.length; i++)
        {
            var group = groups[i];
            if (GroupingUtils.isTimeGrouping(group))
            {
                result = group;
            }
        }
        return result;
    },

    addGroupingInfo: function(grouping, value, line)
    {
        switch (grouping)
        {
            case ReportGroupingLevel.USER:
                line["Last Name"] = value["lastName"];
                line["First Name"] = value["firstName"];
                line["MI"] = value["middleInitial"];
                line["username"] = value["username"];
                break;
            case ReportGroupingLevel.USER_GROUP:
                line["User group"] = value["name"] != null ? value["name"] : messageResource.getMessage("reports.without_department");
                break;
            case ReportGroupingLevel.TIME_ZONE_GROUP:
                line["Time Zone"] = value["name"];
                break;
            case ReportGroupingLevel.DATE:
                var day = value.date;
                line["Day"] = ConverterFormatUtils.formatDay.formatDate(day.parseJSONDate());
                break;
            case ReportGroupingLevel.WEEK:
                var from = value.from;
                var to = value.to;
                line["Week"] = ConverterFormatUtils.formatWeekBegin.formatDate(from.parseJSONDate()) +
                               " - " +
                               ConverterFormatUtils.formatWeekEnd.formatDate(to.parseJSONDate());
                break;
            case ReportGroupingLevel.MONTH:
                var month = value.date;
                line["Month"] = ConverterFormatUtils.formatMonth.formatDate(month.parseJSONDate());
                break;
            case ReportGroupingLevel.PROJECT_NAME:
                line["Project name"] = value["name"];
                break;
            case ReportGroupingLevel.PROJECT:
                line["Project"] = value["name"];
                break;
            case ReportGroupingLevel.CUSTOMER:
                line["Customer"] = value["name"];
                break;
            case ReportGroupingLevel.TASK:
                line["Task"] = value["name"];
                break;
            case ReportGroupingLevel.TASK_NAME:
                line["Task name"] = value["name"];
                break;
            case ReportGroupingLevel.BILLING_TYPE:
                line["Type of work"] = value["name"];
                break;
        }
    },

    addGroupingColumnName: function(grouping, columnsMap, columns)
    {
        // TODO: change to custom from message
        switch (grouping)
        {
            case ReportGroupingLevel.USER:
                columnsMap["Last Name"] = "Last Name";
                columnsMap["First Name"] = "First Name";
                columnsMap["MI"] = "MI";
                columnsMap["username"] = "username";
                columns.push("Last Name");
                columns.push("First Name");
                columns.push("MI");
                columns.push("username");
                break;
            case ReportGroupingLevel.USER_GROUP:
                columnsMap["User group"] = messageResource.getMessage("user.group");
                columns.push("User group");
                break;
            case ReportGroupingLevel.TIME_ZONE_GROUP:
                columnsMap["Time Zone"] = "Time Zone";
                columns.push("Time Zone");
                break;
            case ReportGroupingLevel.DATE:
                columnsMap["Day"] = "Day";
                columns.push("Day");
                break;
            case ReportGroupingLevel.WEEK:
                columnsMap["Week"] = "Week";
                columns.push("Week");
                break;
            case ReportGroupingLevel.MONTH:
                columnsMap["Month"] = "Month";
                columns.push("Month");
                break;
            case ReportGroupingLevel.PROJECT_NAME:
                columnsMap["Project name"] = messageResource.getMessage("common.Project");
                columns.push("Project name");
                break;
            case ReportGroupingLevel.PROJECT:
                columnsMap["Project"] = messageResource.getMessage("common.Project");
                columns.push("Project");
                break;
            case ReportGroupingLevel.CUSTOMER:
                columnsMap["Customer"] = messageResource.getMessage("common.Customer");
                columns.push("Customer");
                break;
            case ReportGroupingLevel.TASK:
                columnsMap["Task"] = messageResource.getMessage("common.Task");
                columns.push("Task");
                break;
            case ReportGroupingLevel.TASK_NAME:
                columnsMap["Task name"] = messageResource.getMessage("common.Task");
                columns.push("Task name");
                break;
            case ReportGroupingLevel.BILLING_TYPE:
                columnsMap["Type of work"] = "Type of work";
                columns.push("Type of work");
                break;
        }
    }
};