function LeaveCSVJsonConverter() {}

LeaveCSVJsonConverter.prototype = _.create(BaseCSVJsonConverter.prototype, {
    init: function(allUsers, allUserGroups, allTimeZoneGroups, defaultWorkDayDuration)
    {
        BaseCSVJsonConverter.prototype.init.call(this);

        this.users = allUsers;
        this.usersMap = arrayToMapById(allUsers);
        this.usersGroups = allUserGroups;
        this.usersGroupsMap = arrayToMapById(allUserGroups);
        this.timeZoneGroups = allTimeZoneGroups;
        this.timeZoneGroupingMap = arrayToMapById(allTimeZoneGroups);
        // get in minutes, need in hours
        this.defaultWorkDayDuration = defaultWorkDayDuration / 60;
        return this;
    },

    generateTableHeader: function(data, config, columnNames, columns)
    {
        for (var i = 0; i < this.groupingLevels.length; ++i)
            GroupingUtils.addGroupingColumnName(this.groupingLevels[i], columnNames, columns);

        this.addOtherColumnNames(columnNames, columns, config, data);
    },

    // override
    addOtherColumnNames: function(columnsMap, columns, config, data)
    {
        if (this.showLeaveTime)
            this.addLeaveTimeNames(columnsMap, columns, data.leaveSpent);
        if (this.showBalances)
            this.addBalancesNames(columnsMap, columns);
    },

    addLeaveTimeNames: function(columnsMap, columns, leaveTypes)
    {
        columns.push("Total");
        columnsMap["Total"] = messageResource.getMessage("report.leaves.total.html_csv");
        for (var j = 0; j < leaveTypes.length; ++j)
        {
            var columnName = this.getLeaveTypeColumnName(leaveTypes[j]);
            columnsMap[columnName] = LeaveTypeStore.getLeaveType(leaveTypes[j].leaveTypeId).name;
            columns.push(columnName);
        }
    },

    // override
    getIdentity: function(data, grouping)
    {
        switch (grouping)
        {
            case ReportGroupingLevel.USER:
                return this.usersMap[data.id];
            case ReportGroupingLevel.USER_GROUP:
                return this.usersGroupsMap[data.id];
            case ReportGroupingLevel.TIME_ZONE_GROUP:
                return this.timeZoneGroupingMap[data.id];
            case ReportGroupingLevel.DATE:
                return data.id;
            case ReportGroupingLevel.WEEK:
                return data.id;
            case ReportGroupingLevel.MONTH:
                return data.id;
        }
    },

    addBalancesNames: function(columnsMap, columns)
    {
        if (this.pto)
        {
            var ptoMessage = messageResource.getMessage("report.leaves.balance_column_name.pto");
            columns.push("PTO begin");
            columnsMap["PTO begin"] = ptoMessage + this.balanceColumnNameBegin(SystemSettings.showPTOInHours);
            columns.push("PTO end");
            columnsMap["PTO end"] = ptoMessage + this.balanceColumnNameEnd(SystemSettings.showPTOInHours);
        }
        if (this.sick)
        {
            var sickMessage = messageResource.getMessage("report.leaves.balance_column_name.sick_days");
            columns.push("Sick begin");
            columnsMap["Sick begin"] = sickMessage + this.balanceColumnNameBegin(SystemSettings.showSickInHours);
            columns.push("Sick end");
            columnsMap["Sick end"] = sickMessage + this.balanceColumnNameEnd(SystemSettings.showSickInHours);
        }
    },

    getLeaveTypeColumnName: function(leaveType)
    {
        return "leaveType" + leaveType.leaveTypeId.toString();
    },

    balanceColumnNameBegin: function(showInHours)
    {
        var timeGrouping = GroupingUtils.searchLastGroupingTime(this.groupingLevels);
        var postfix = "";
        var dateRange = this.dateRange;
        switch (timeGrouping)
        {
            case ReportGroupingLevel.NONE:
                postfix = " " + ConverterFormatUtils.formatDay.formatDate(dateRange.fromDate);
                break;
            case ReportGroupingLevel.DATE:
                postfix = " " + messageResource.getMessage("report.leaves.first_balance_subcolumn.dates");
                break;
            case ReportGroupingLevel.WEEK:
                postfix = " " + messageResource.getMessage("report.leaves.first_balance_subcolumn.weeks");
                break;
            case ReportGroupingLevel.MONTH:
                postfix = " " + messageResource.getMessage("report.leaves.first_balance_subcolumn.months");
                break;
        }
        postfix += " (" + messageResource.getMessage(showInHours ? "common.pto_units.hours" : "common.pto_units.days") + ")";
        return postfix;
    },

    balanceColumnNameEnd: function(showInHours)
    {
        var timeGrouping = GroupingUtils.searchLastGroupingTime(this.groupingLevels);
        var postfix = "";
        var dateRange = this.dateRange;
        switch (timeGrouping)
        {
            case ReportGroupingLevel.NONE:
                postfix = " " + ConverterFormatUtils.formatDay.formatDate(dateRange.toDate);
                break;
            case ReportGroupingLevel.DATE:
                postfix = " " + messageResource.getMessage("report.leaves.second_balance_subcolumn.dates");
                break;
            case ReportGroupingLevel.WEEK:
                postfix = " " + messageResource.getMessage("report.leaves.second_balance_subcolumn.weeks");
                break;
            case ReportGroupingLevel.MONTH:
                postfix = " " + messageResource.getMessage("report.leaves.second_balance_subcolumn.months");
                break;
        }
        postfix += " (" + messageResource.getMessage(showInHours ? "common.pto_units.hours" : "common.pto_units.days") + ")";
        return postfix;
    },

    formatLeaveTime: function(time)
    {
        return ConverterFormatUtils.numberFormatter.format(time / 60);
    },

    addLeaveTimeToTable: function(data, line)
    {
        line["Total"] = this.formatLeaveTime(data.totalLeaveSpent);
        var leaveSpends = data.leaveSpent;
        for (var l = 0; l < leaveSpends.length; ++l)
        {
            var columnName = this.getLeaveTypeColumnName(leaveSpends[l]);
            line[columnName] = this.formatLeaveTime(leaveSpends[l].leaveSpent);
        }
    },

    addBalancesToTable: function(data, line)
    {

        var durationSick = SystemSettings.showSickInHours ? this.defaultWorkDayDuration : 1;
        var durationPto = SystemSettings.showPTOInHours ? this.defaultWorkDayDuration : 1;

        if (this.pto)
        {
            var pto = data.ptoBalance;
            line["PTO begin"] = (pto != null && pto.balanceForFirstColumn != null) ?
                                ConverterFormatUtils.numberFormatter.format(pto.balanceForFirstColumn * durationPto) :
                                "";
            line["PTO end"] = (pto != null && pto.balanceForSecondColumn != null) ?
                              ConverterFormatUtils.numberFormatter.format(pto.balanceForSecondColumn * durationPto) :
                              "";
        }
        if (this.sick)
        {
            var sick = data.sickBalance;
            line["Sick begin"] = (sick != null && sick.balanceForFirstColumn != null) ?
                                 ConverterFormatUtils.numberFormatter.format(sick.balanceForFirstColumn * durationSick) :
                                 "";
            line["Sick end"] = (sick != null && sick.balanceForSecondColumn != null) ?
                               ConverterFormatUtils.numberFormatter.format(sick.balanceForSecondColumn * durationSick) :
                               "";
        }

    },

    // override
    addDataToTable: function(data, line)
    {
        if (this.showLeaveTime)
            this.addLeaveTimeToTable(data, line);

        if (this.showBalances)
            this.addBalancesToTable(data, line);
    },

    // override
    preprocess: function(config, data)
    {
        this.dateRange = config.dateRangeSelection;
        this.showBalances = (config.columnsConfig == LeaveTypeColumnConfig.LEAVE_TIME_AND_BALANCES ||
                             config.columnsConfig == LeaveTypeColumnConfig.BALANCES_ONLY);
        this.showLeaveTime = (config.columnsConfig == LeaveTypeColumnConfig.LEAVE_TIME_AND_BALANCES ||
                              config.columnsConfig == LeaveTypeColumnConfig.LEAVE_TIME_ONLY);

        this.pto = (config.balanceQualifier == LeaveTypeBalanceQualifier.PTO_AND_SICK_DAYS ||
                    config.balanceQualifier == LeaveTypeBalanceQualifier.PTO_ONLY);
        this.sick = (config.balanceQualifier == LeaveTypeBalanceQualifier.PTO_AND_SICK_DAYS ||
                     config.balanceQualifier == LeaveTypeBalanceQualifier.SICK_DAYS_ONLY);
    },

    addGroupingInfo: function(groupingLevel, identity, line)
    {
        GroupingUtils.addGroupingInfo(groupingLevel, identity, line);
    },
    // override
    processParamsBuilder: function(paramsBuilder, params, config)
    {
        var userGroupFeature = config.userGroupFeature;
        var tzGroupFeature = config.tzGroupFeature;
        var reportParamsStringBuilder = new ReportParamsStringBuilder().init(
                {
                    users: this.users,
                    userGroupsFeatureEnabled: userGroupFeature,
                    tzGroupsFeatureEnabled: tzGroupFeature,
                    userGroups: this.usersGroups,
                    timeZoneGroups: this.timeZoneGroups,
                    leaveTypes: _.values(LeaveTypeStore.getLeaveTypes())
                }
        );

        paramsBuilder.addDateRangeParam(params.dateRangeSelection, reportParamsStringBuilder);
        paramsBuilder.addUsersParam(params.userSelection, reportParamsStringBuilder,
                                    {isShowOnlyAssigned: params.isShowOnlyAssigned, viewerInfo: params.viewerInfo});
        paramsBuilder.addLeavesParam(params.leaveTypeSelection, reportParamsStringBuilder);
    },
    // in AP canManageBalances always true
    generateCSVName: function(reportParams, settingValidator, canManageBalances)
    {
        var postfix = "";
        if (canManageBalances)
        {
            if (settingValidator.isPTOEnabled() && settingValidator.isSickEnabled())
                postfix = messageResource.getMessage("common.leave.reports.csv.file_name.balances");
            else if (settingValidator.isPTOEnabled())
                postfix = messageResource.getMessage("common.leave.reports.csv.file_name.pto");
            else if (settingValidator.isSickEnabled())
                postfix = messageResource.getMessage("common.leave.reports.csv.file_name.sick");
            else
                postfix = messageResource.getMessage("common.leave.reports.csv.file_name.no_balances")
        }

        var fileNameParts = [];
        var dateFormatter = new DateFormat(SystemSettings.dateFormat);
        fileNameParts.push(postfix);
        fileNameParts.push(dateFormatter.formatDate(reportParams.dateRangeSelection.fromDate));
        fileNameParts.push(dateFormatter.formatDate(reportParams.dateRangeSelection.toDate));

        var filename = messageResource.getMessage('common.leave.reports.csv.file_name', fileNameParts);
        if (isOldIE())
        {
            filename = filename.replace(/\//g, "_")
        }
        return filename;
    },

    convertTitle: function(params, userGroupFeature, tzGroupFeature)
    {
        return BaseCSVJsonConverter.prototype.convertTitle.call(this, params, {userGroupFeature: userGroupFeature, tzGroupFeature: tzGroupFeature});
    }
});
