function CostReportEditConfigController ()
{
    this.constructor = CostReportEditConfigController;
}

CostReportEditConfigController.prototype = new ReportEditConfigController();

CostReportEditConfigController.prototype.init = function ( config )
{
    ReportEditConfigController.prototype.init.call(this, config);

    var self = this;
    var reportParameters = this.makeReportParamsAccordingToFeatures(config.reportParameters);
    this.configChangedCallback = config.changeCallback;
    var configChanged = function () { self.configChanged(); };

    this.timeFormatSelector = this.initTimeFormatSelector(reportParameters.timeFormat, configChanged);
    var groupingLevelsSelection = [reportParameters.firstGroupingLevel, reportParameters.secondGroupingLevel, reportParameters.thirdGroupingLevel];
    this.groupingSelector = new ThreeLevelReportGroupingSelector().init( {
                                                                             selectorId: "reportGroupingSelector",
                                                                             enableNoneFirstGrouping: false,
                                                                             enableGroupByBillingType: EnabledFeatures.BTY || this.checkGroupingLevel(ReportGroupingLevel.BILLING_TYPE, groupingLevelsSelection),
                                                                             enableGroupByUsers: false,
                                                                             enableGroupByUserGroups: false,
                                                                             includeTTCommentsInDetailsLevel: false,
                                                                             groupingLevelsSelection: groupingLevelsSelection,
                                                                             changesCallback: function() { self.removeGroupingLevelsSelectionError(); configChanged(); }
                                                                         });

    var isShowUGR =  EnabledFeatures.UGR || reportParameters.columnsConfig != ReportColumnsFormatCostOfWork.USERS_ONLY;
    if (isShowUGR)
        this.columnsFormatSelector = this.initListSelector("columnsDisplayFormat", reportParameters.columnsConfig, configChanged);

    this.columnsFormat = new ReportColumnsSelectorButtonsCostOfWork().init({containerId :"columnsFormatCheckboxesTable",
                                                                               selector: self.columnsFormatSelector,
                                                                               columnsFormat:  reportParameters.columnsConfig,
                                                                               showSpentTime: reportParameters.showSpentTime,
                                                                               showAverageRates: reportParameters.showAverageRates,
                                                                               showUserRates: reportParameters.showUserRates,
                                                                               showSubtotalsByDepartments: reportParameters.showSubtotalsByDepartments,
                                                                               changeCallback: configChanged});
    
    this.cpSelector = this.makeCPSelector(reportParameters.customersProjectsSelection);
    this.userSelector = this.makeUserSelector(config);
    
    if( EnabledFeatures.LVT || reportParameters.showLeaveTimeCosts )
    {
        $("#leaveTimeCosts").show();
        $("#generalCostsConfigSection").show();
        this.showLeaveTimeCostsChexbox = $( "#showLeaves" );
        this.showLeaveTimeCostsChexbox.prop("checked", reportParameters.showLeaveTimeCosts);
        this.showLeaveTimeCostsChexbox.on("change", configChanged);
    }

    if( EnabledFeatures.OVR || reportParameters.showOvertimeAddition )
    {
        $("#overtimeAddition").show();
        $("#generalCostsConfigSection").show();
        this.showOvertimeChexbox = $("#showOvertime");
        this.showOvertimeChexbox.prop("checked", reportParameters.showOvertimeAddition);
        this.showOvertimeChexbox.on("change", configChanged);
    }

    return this;
};

CostReportEditConfigController.prototype.getConfig = function () {
    var reportColumnsResult = this.columnsFormat.getSelection();
    var groupingLevelValues = this.groupingSelector.getValues();
    return {
        timeFormat: this.timeFormatSelector.getValue(),
        userSelection: this.userSelector.getSelection(),
        showLeaveTimeCosts: !!this.showLeaveTimeCostsChexbox && this.showLeaveTimeCostsChexbox.is(":checked"),
        showOvertimeAddition: !!this.showOvertimeChexbox && this.showOvertimeChexbox.is(":checked"),
        columnsConfig:reportColumnsResult.columnsConfig,
        showSpentTime : reportColumnsResult.showSpentTime,
        showAverageRates : reportColumnsResult.showAverageRates,
        showUserRates : reportColumnsResult.showUserRates,
        showSubtotalsByDepartments: reportColumnsResult.showSubtotalsByDepartments,
        customersProjectsSelection: this.cpSelector.getCpSelection(),
        firstGroupingLevel: groupingLevelValues[0],
        secondGroupingLevel: groupingLevelValues[1],
        thirdGroupingLevel: groupingLevelValues[2]
    };
};

CostReportEditConfigController.prototype.resetConfig = function () {
    this.timeFormatSelector.setActiveItemByValue(this.initReportParameters.timeFormat, true);
    this.userSelector.setSelection(this.initReportParameters.userSelection);
    if(this.showLeaveTimeCostsChexbox)
        this.showLeaveTimeCostsChexbox.prop("checked", this.initReportParameters.showLeaveTimeCosts);
    if(this.showOvertimeChexbox)
        this.showOvertimeChexbox.prop("checked", this.initReportParameters.showOvertimeAddition);
    this.cpSelector.setSelection(this.initReportParameters.customersProjectsSelection);
    this.columnsFormat.setValue(this.initReportParameters.columnsConfig,
                                this.initReportParameters.showSpentTime,
                                this.initReportParameters.showAverageRates,
                                this.initReportParameters.showUserRates,
                                this.initReportParameters.showSubtotalsByDepartments);
    this.groupingSelector.setValues([this.initReportParameters.firstGroupingLevel, this.initReportParameters.secondGroupingLevel, this.initReportParameters.thirdGroupingLevel]);
};

CostReportEditConfigController.prototype.validate = function()
{
    this.errors = [];

    if (!this.userSelector.validateIsUserSelected())
        this.errors.push( this.possibleErrors.ERROR_EMPTY_USER_SELECTION );

    if (!this.cpSelector.checkSelected())
        this.errors.push( this.possibleErrors.ERROR_EMPTY_CP_SELECTION );

    if (!this.groupingSelector.validate())
        this.errors.push(this.possibleErrors.ERROR_IN_GROUPING_LEVELS_SELECTOR);

    return this.errors.length == 0;
};

CostReportEditConfigController.prototype.initErrorsToSectionElsMap = function()
{
    var errorsToSectionElsMap = {};
    errorsToSectionElsMap[this.possibleErrors.ERROR_EMPTY_USER_SELECTION] = this.panelEl.find( "#userSelectorId" );
    errorsToSectionElsMap[this.possibleErrors.ERROR_EMPTY_CP_SELECTION] = this.panelEl.find( "#cpSelectorId" );
    errorsToSectionElsMap[this.possibleErrors.ERROR_IN_GROUPING_LEVELS_SELECTOR] = this.panelEl.find( ".reportGroupingSelectorContainer" );
    return errorsToSectionElsMap;
};

CostReportEditConfigController.prototype.setSelectorsInitStateFromCurrent = function () {
    ReportEditConfigController.prototype.setSelectorsInitStateFromCurrent.call(this);

    if(this.userSelector){
        this.userSelector.setInitialValueFromCurrent();
    }
    if(this.cpSelector) {
        this.cpSelector.setInitialValueFromCurrent();
    }
};

// -------------------

function CostReportEditFormConfigController() {}

CostReportEditFormConfigController.prototype = new CostReportEditConfigController();

CostReportEditFormConfigController.prototype.makeReportParamsAccordingToFeatures = function ( reportParameters ) {
    var result = $.extend({}, reportParameters);
    if ( !EnabledFeatures.UGR && result.columnsConfig != ReportColumnsFormatCostOfWork.USERS_ONLY )
        result.columnsConfig = ReportColumnsFormatCostOfWork.USERS_ONLY;

    if (!EnabledFeatures.BTY)
    {
        var groupings = this.makeGroupingSelectorValuesAccordingToNotAvailableValueLevel(ReportGroupingLevel.BILLING_TYPE,
                                                                                  [result.firstGroupingLevel, result.secondGroupingLevel],
                                                                                  ReportGroupingLevel.CUSTOMER
        );
        result.firstGroupingLevel = groupings[0];
        result.secondGroupingLevel = groupings[1];
    }

    if( !EnabledFeatures.LVT )
        result.showLeaveTimeCosts = false;

    if( !EnabledFeatures.OVR )
        result.showOvertimeAddition = false;

    return result;
};


// -------------------

function CostReportEditResultConfigController() {}

CostReportEditResultConfigController.prototype = new CostReportEditConfigController();

CostReportEditResultConfigController.prototype.makeReportParamsAccordingToFeatures = function ( reportParameters ) {
    return reportParameters;
};