function InvoiceReportEditConfigController ()
{
    this.constructor = InvoiceReportEditConfigController;
}

InvoiceReportEditConfigController.prototype = new ReportEditConfigController();

InvoiceReportEditConfigController.prototype.init = function ( config )
{
    ReportEditConfigController.prototype.init.call(this, config);

    var self = this;
    this.configChangedCallback = config.changeCallback;
    var configChanged = function () { self.configChanged(); };
    var reportParameters = this.makeReportParamsAccordingToFeatures(config.reportParameters);
    this.groupingSelector = new ThreeLevelReportGroupingSelector().init( {
                                                                             selectorId: "reportGroupingSelector",
                                                                             enableNoneFirstGrouping: true,
                                                                             enableGroupByBillingType: false,
                                                                             enableGroupByUsers: true,
                                                                             enableGroupByUserGroups: EnabledFeatures.UGR || reportParameters.firstGroupingLevel == ReportGroupingLevel.USER_GROUP
                                                                                                                    ||  reportParameters.secondGroupingLevel == ReportGroupingLevel.USER_GROUP  ,
                                                                             includeTTCommentsInDetailsLevel: true,
                                                                             groupingLevelsSelection: [ reportParameters.firstGroupingLevel, reportParameters.secondGroupingLevel, reportParameters.thirdGroupingLevel ],
                                                                             changesCallback: function() { self.removeGroupingLevelsSelectionError(); configChanged(); }
                                                                         });
    this.cpSelector = this.makeCPSelector(reportParameters.customersProjectsSelection);
    
    this.chooseTypeSelector = new ChooseTypeSelector().init(reportParameters.reportType, configChanged);

    return this;
};

InvoiceReportEditConfigController.prototype.setSelectorsInitStateFromCurrent = function () {
    ReportEditConfigController.prototype.setSelectorsInitStateFromCurrent.call(this);

    if(this.cpSelector) {
        this.cpSelector.setInitialValueFromCurrent();
    }
};

InvoiceReportEditConfigController.prototype.getConfig = function () {
    var groupingLevelValues = this.groupingSelector.getValues();
    return {
        reportType: this.chooseTypeSelector.getValue(),
        customersProjectsSelection: this.cpSelector.getCpSelection(),
        firstGroupingLevel: groupingLevelValues[0],
        secondGroupingLevel: groupingLevelValues[1],
        thirdGroupingLevel: groupingLevelValues[2]
    };
};

InvoiceReportEditConfigController.prototype.resetConfig = function () {
    this.chooseTypeSelector.setValue(this.initReportParameters.reportType);
    this.cpSelector.setSelection(this.initReportParameters.customersProjectsSelection);
    this.groupingSelector.setValues([this.initReportParameters.firstGroupingLevel, this.initReportParameters.secondGroupingLevel, this.initReportParameters.thirdGroupingLevel]);
};

InvoiceReportEditConfigController.prototype.validate = function()
{
    this.errors = [];

    if (!this.cpSelector.checkSelected())
        this.errors.push( this.possibleErrors.ERROR_EMPTY_CP_SELECTION );

    if (!this.groupingSelector.validate())
        this.errors.push(this.possibleErrors.ERROR_IN_GROUPING_LEVELS_SELECTOR);

    return this.errors.length == 0;
};

InvoiceReportEditConfigController.prototype.initErrorsToSectionElsMap = function()
{
    var errorsToSectionElsMap = {};
    errorsToSectionElsMap[this.possibleErrors.ERROR_EMPTY_CP_SELECTION] = this.panelEl.find( "#cpSelectorId" );
    errorsToSectionElsMap[this.possibleErrors.ERROR_IN_GROUPING_LEVELS_SELECTOR] = this.panelEl.find( ".reportGroupingSelectorContainer" );
    return errorsToSectionElsMap;
};

// -------------------

function InvoiceEditFormConfigController() {}

InvoiceEditFormConfigController.prototype = new InvoiceReportEditConfigController();

InvoiceEditFormConfigController.prototype.makeReportParamsAccordingToFeatures = function ( reportParameters ) {
    var result = $.extend({}, reportParameters);

    if (!EnabledFeatures.UGR)
    {
        var groupings = this.makeGroupingSelectorValuesAccordingToNotAvailableValueLevel(ReportGroupingLevel.USER_GROUP,
                                                                                  [result.firstGroupingLevel, result.secondGroupingLevel],
                                                                                  ReportGroupingLevel.CUSTOMER);
        result.firstGroupingLevel = groupings[0];
        result.secondGroupingLevel = groupings[1];
    }
    return result;
};


// -------------------

function InvoiceEditResultConfigController() {}

InvoiceEditResultConfigController.prototype = new InvoiceReportEditConfigController();

InvoiceEditResultConfigController.prototype.makeReportParamsAccordingToFeatures = function ( reportParameters ) {
    return reportParameters;
};