function LeavesReportEditConfigController ()
{
    this.constructor = LeavesReportEditConfigController;
}

LeavesReportEditConfigController.prototype = new ReportEditConfigController();

LeavesReportEditConfigController.prototype.init = function ( config )
{
    ReportEditConfigController.prototype.init.call(this, config);

    var self = this;
    var reportParameters = this.makeReportParamsAccordingToFeatures(config.reportParameters);

    this.configChangedCallback = config.changeCallback;
    var configChanged = function () { self.configChanged(); };
    this.timeFormatSelector = this.initTimeFormatSelector( reportParameters.timeFormat, configChanged);

    var groupingLevelsSelection = [reportParameters.firstGroupingLevel, reportParameters.secondGroupingLevel];
    this.groupingSelector = new LeavesGroupingSelector().init(
        "reportGroupingSelector",
        groupingLevelsSelection,
        this.productSettingsValidator.isUGREnabled() || this.checkGroupingLevel(ReportGroupingLevel.USER_GROUP, groupingLevelsSelection),
        this.productSettingsValidator.isTZGEnabled() || this.checkGroupingLevel(ReportGroupingLevel.TIME_ZONE_GROUP, groupingLevelsSelection),
        function() {
                self.removeGroupingLevelsSelectionError();
                configChanged();
        });

    this.initLeavesSelector(reportParameters.leaveTypeSelection, config.leavesData, configChanged);

    // if feature enabled and user has rights otherwise only if data is not "LeaveTimeOnly" - show selector
    this.columnsDisplayFormat = this.addColumnFormatFilter(reportParameters.columnsConfig, reportParameters.balanceQualifier, configChanged);

    config.additionalCheckboxLabel = messageResource.getMessage("users_selector.reports.include_users_wo_data");

    this.userSelector = this.makeUserSelector(config);
    
    return this;
};

/**
 * add column format filter if need
 * @param columnsConfig - leave tyime only or balance + leave time or both
 * @param balanceQualifier - for type of balance sdb or pto or both
 * @param onChangeCallback
 * @returns {{getValue, setActiveItemByValue}}
 */
LeavesReportEditConfigController.prototype.addColumnFormatFilter = function( columnsConfig, balanceQualifier, onChangeCallback ) {

    var enabledPTO =  this.productSettingsValidator.isPTOEnabled();
    var enabledSick =  this.productSettingsValidator.isSickEnabled();

    var ifNeedColumnFormatSelector = (enabledPTO || enabledSick) && this.reportRightsValidator.isCanViewBalances() ||
        columnsConfig != LeaveTypeColumnConfig.LEAVE_TIME_ONLY;
    var selectorRow = $("#columnsFormatFilterRow");

    var initValue = {
        columnsConfig : columnsConfig,
        balanceQualifier : balanceQualifier
    };

    if (ifNeedColumnFormatSelector){
        selectorRow.show();
        return new LeaveReportColumnsFormatSelector().init("columnsDisplayFormat", initValue, onChangeCallback, enabledPTO, enabledSick);
    } else {
        selectorRow.hide();
        return this.initEmptyListSelector(initValue);
    }
};

LeavesReportEditConfigController.prototype.initLeavesSelector = function ( leavesSelection, leavesData, changeCallback )
{
    var self = this;
    var leavesSelectorChangesCallback = function()
    {
        if ( !self.leavesSelector.isEmptySelection() )
            self.errors.remove(self.possibleErrors.ERROR_EMPTY_LEAVE_TYPES_SELECTION);
        self.updateUI();
        if ( changeCallback )
            changeCallback();
    };
    this.leavesSelector = new ReportLeaveTypesSelectorButton().init( "leavesButton", leavesSelection, leavesData,
                                                                     this.productSettingsValidator.isPTOEnabled(),
                                                                     this.productSettingsValidator.isSickEnabled(), leavesSelectorChangesCallback );
};

LeavesReportEditConfigController.prototype.getConfig = function () {
    var groupingLevelValues = this.groupingSelector.getValues();
    var columnsDisplayFormatValue = this.columnsDisplayFormat.getValue();
    return {
        timeFormat: this.timeFormatSelector.getValue(),
        leaveTypeSelection : this.leavesSelector.getLeaveTypeSelection(),
        columnsConfig: columnsDisplayFormatValue.columnsConfig,
        balanceQualifier: columnsDisplayFormatValue.balanceQualifier,
        userSelection: this.userSelector.getSelection(),
        firstGroupingLevel: groupingLevelValues[0],
        secondGroupingLevel: groupingLevelValues[1]
    };
};

LeavesReportEditConfigController.prototype.resetConfig = function () {
    this.timeFormatSelector.setActiveItemByValue(this.initReportParameters.timeFormat, true);
    this.columnsDisplayFormat.setActiveItemByValue(
        {
            columnsConfig : this.initReportParameters.columnsConfig,
            balanceQualifier : this.initReportParameters.balanceQualifier
        },
        true
    );
    this.groupingSelector.setValues([this.initReportParameters.firstGroupingLevel, this.initReportParameters.secondGroupingLevel]);
    this.leavesSelector.resetLeavesSelectionFunction();
    this.userSelector.setSelection(this.initReportParameters.userSelection);
};

LeavesReportEditConfigController.prototype.setSelectorsInitStateFromCurrent = function () {
    ReportEditConfigController.prototype.setSelectorsInitStateFromCurrent.call(this);

    if(this.userSelector){
        this.userSelector.setInitialValueFromCurrent();
    }
    if(this.leavesSelector){
        this.leavesSelector.setInitialValuesFromCurrent ();
    }
};

LeavesReportEditConfigController.prototype.validate = function ()
{
    this.errors = [];

    if (this.leavesSelector.isEmptySelection())
        this.errors.push(this.possibleErrors.ERROR_EMPTY_LEAVE_TYPES_SELECTION);

    if (!this.userSelector.validateIsUserSelected())
        this.errors.push( this.possibleErrors.ERROR_EMPTY_USER_SELECTION );

    if (!this.groupingSelector.validate())
        this.errors.push(this.possibleErrors.ERROR_IN_GROUPING_LEVELS_SELECTOR);

    return this.errors.length == 0;
};

LeavesReportEditConfigController.prototype.initErrorsToSectionElsMap = function()
{
    var errorsToSectionElsMap = {};
    errorsToSectionElsMap[this.possibleErrors.ERROR_EMPTY_USER_SELECTION] = this.panelEl.find( "#userSelectorId" );
    errorsToSectionElsMap[this.possibleErrors.ERROR_EMPTY_LEAVE_TYPES_SELECTION] = this.panelEl.find( "#leavesButton" );
    errorsToSectionElsMap[this.possibleErrors.ERROR_IN_GROUPING_LEVELS_SELECTOR] = this.panelEl.find( ".reportGroupingSelectorContainer" );
    return errorsToSectionElsMap;
};

// -------------------

function LeavesEditFormConfigController() {}

LeavesEditFormConfigController.prototype = new LeavesReportEditConfigController();
LeavesEditFormConfigController.prototype.makeReportParamsAccordingToFeatures = function ( reportParameters ) {
    var result = $.extend({}, reportParameters);

    var isEnabledPTO = this.productSettingsValidator.isPTOEnabled();
    var isEnabledSick = this.productSettingsValidator.isSickEnabled();

    if (!isEnabledPTO && reportParameters.balanceQualifier != LeaveTypeBalanceQualifier.SICK_DAYS_ONLY) {
        result.balanceQualifier = LeaveTypeBalanceQualifier.SICK_DAYS_ONLY;
    }
    if (!isEnabledSick && reportParameters.balanceQualifier != LeaveTypeBalanceQualifier.PTO_ONLY) {
        result.balanceQualifier = LeaveTypeBalanceQualifier.PTO_ONLY;
    }

    if ((!isEnabledPTO && !isEnabledSick) || !this.reportRightsValidator.isCanViewBalances()) {
        result.columnsConfig = LeaveTypeColumnConfig.LEAVE_TIME_ONLY;
    }
    if (!this.productSettingsValidator.isUGREnabled())
    {
        var groupings = this.makeGroupingSelectorValuesAccordingToNotAvailableValueLevel(ReportGroupingLevel.USER_GROUP,
                                                                                  [result.firstGroupingLevel, result.secondGroupingLevel],
                                                                                  ReportGroupingLevel.USER);
        result.firstGroupingLevel = groupings[0];
        result.secondGroupingLevel = groupings[1];
    }
    return result;
};


// -------------------

function LeavesEditResultConfigController() {}

LeavesEditResultConfigController.prototype = new LeavesReportEditConfigController();

LeavesEditResultConfigController.prototype.makeReportParamsAccordingToFeatures = function ( reportParameters ) {
    return reportParameters;
};