function OvertimeReportEditConfigController ()
{
    this.constructor = OvertimeReportEditConfigController;
}

OvertimeReportEditConfigController.prototype = new ReportEditConfigController();

OvertimeReportEditConfigController.prototype.init = function ( config )
{
    ReportEditConfigController.prototype.init.call(this, config);

    var self = this;
    var reportParameters = this.makeReportParamsAccordingToFeatures(config.reportParameters);
    this.configChangedCallback = config.changeCallback;
    var configChanged = function () { self.configChanged(); };

    this.timeFormatSelector = this.initTimeFormatSelector(reportParameters.timeFormat, configChanged );

    var groupingLevelsSelection = [reportParameters.firstGroupingLevel, reportParameters.secondGroupingLevel];
    var ugrEnabled = EnabledFeatures.UGR || this.checkGroupingLevel(ReportGroupingLevel.USER_GROUP, groupingLevelsSelection);
    var tzgEnabled = EnabledFeatures.TZG || this.checkGroupingLevel(ReportGroupingLevel.TIME_ZONE_GROUP, groupingLevelsSelection);
    if (ugrEnabled || tzgEnabled)
    {
        this.groupingSelector = new OvertimeGroupingSelector().init(
                "reportGroupingSelector",
                groupingLevelsSelection,
                ugrEnabled,
                tzgEnabled,
                function() {
                    self.removeGroupingLevelsSelectionError();
                    configChanged();
                }
        );
        $("#reportConfigDataGroupingRow").show();
    }

    config.additionalCheckboxLabel = messageResource.getMessage("users_selector.reports.include_users_wo_data");

    this.userSelector = this.makeUserSelector(config);

    if (this.shouldMakeColumnsConfigSelector(config))
    {
        this.columnsConfigSelector = this.makeColumnsConfigSelector(config, configChanged);
        $("#columnsFormatFilterRow").show();
    }

    return this;
};

OvertimeReportEditConfigController.prototype.getConfig = function () {
    var groupingLevelValues = this.groupingSelector ? this.groupingSelector.getValues() : [ReportGroupingLevel.USER, ReportGroupingLevel.NONE];
    return {
        timeFormat: this.timeFormatSelector.getValue(),
        userSelection: this.userSelector.getSelection(),
        firstGroupingLevel: groupingLevelValues[0],
        secondGroupingLevel: groupingLevelValues[1],
        columnsConfig: this.columnsConfigSelector ? this.columnsConfigSelector.getSelectedItem().value : OvertimeReport.ColumnsConfig.TIME_BALANCE_ONLY
    };
};

OvertimeReportEditConfigController.prototype.resetConfig = function () {
    this.timeFormatSelector.setActiveItemByValue(this.initReportParameters.timeFormat, true);
    this.userSelector.setSelection(this.initReportParameters.userSelection);
    if (this.groupingSelector)
        this.groupingSelector.setValues([this.initReportParameters.firstGroupingLevel, this.initReportParameters.secondGroupingLevel]);
};

OvertimeReportEditConfigController.prototype.validate = function ()
{
    this.errors = [];

    if (!this.userSelector.validateIsUserSelected())
        this.errors.push( this.possibleErrors.ERROR_EMPTY_USER_SELECTION );

    if (this.groupingSelector && !this.groupingSelector.validate())
        this.errors.push(this.possibleErrors.ERROR_IN_GROUPING_LEVELS_SELECTOR);

    return this.errors.length == 0;
};

OvertimeReportEditConfigController.prototype.initErrorsToSectionElsMap = function()
{
    var errorsToSectionElsMap = {};
    errorsToSectionElsMap[this.possibleErrors.ERROR_EMPTY_USER_SELECTION] = this.panelEl.find( "#userSelectorId" );
    errorsToSectionElsMap[this.possibleErrors.ERROR_IN_GROUPING_LEVELS_SELECTOR] = this.panelEl.find( ".reportGroupingSelectorContainer" );
    return errorsToSectionElsMap;
};

OvertimeReportEditConfigController.prototype.setSelectorsInitStateFromCurrent = function () {
    ReportEditConfigController.prototype.setSelectorsInitStateFromCurrent.call(this);

    if(this.userSelector){
        this.userSelector.setInitialValueFromCurrent();
    }
};

OvertimeReportEditConfigController.prototype.shouldMakeColumnsConfigSelector = function(config)
{
    return false;
};

OvertimeReportEditConfigController.prototype.makeColumnsConfigSelector = function(config, onConfigChanged)
{
    var columnsConfigSelector = new ListSelector({
                                                     renderTo: "columnsTypeSelector",
                                                     menuCls: "columnsFormatMenu"
                                                 });
    columnsConfigSelector.addItems([
                                       {
                                           text: messageResource.getEscapedMessage("report.overtime.columns_config.time_balance_and_overtime"),
                                           value: OvertimeReport.ColumnsConfig.TIME_BALANCE_AND_OVERTIME
                                       },
                                       {
                                           text: messageResource.getEscapedMessage("report.overtime.columns_config.time_balance_only"),
                                           value: OvertimeReport.ColumnsConfig.TIME_BALANCE_ONLY
                                       },
                                       {
                                           text: messageResource.getEscapedMessage("report.overtime.columns_config.overtime_only"),
                                           value: OvertimeReport.ColumnsConfig.OVERTIME_ONLY
                                       }
                                   ]);

    columnsConfigSelector.on("change", onConfigChanged);

    columnsConfigSelector.setActiveItemByValue(config.reportParameters.columnsConfig, true);

    return columnsConfigSelector;
};


// -------------------

function OvertimeEditFormConfigController() {}

OvertimeEditFormConfigController.prototype = new OvertimeReportEditConfigController();
OvertimeEditFormConfigController.prototype.makeReportParamsAccordingToFeatures = function ( reportParameters ) {
    var result = $.extend({}, reportParameters);
    if (!EnabledFeatures.UGR)
    {
        var groupings = this.makeGroupingSelectorValuesAccordingToNotAvailableValueLevel(ReportGroupingLevel.USER_GROUP,
                                                                                  [result.firstGroupingLevel, result.secondGroupingLevel],
                                                                                  ReportGroupingLevel.USER);
        result.firstGroupingLevel = groupings[0];
        result.secondGroupingLevel = groupings[1];
    }
    return result;
};

OvertimeEditFormConfigController.prototype.shouldMakeColumnsConfigSelector = function(config)
{
    return EnabledFeatures.OVR;
};

// -------------------

function OvertimeEditResultConfigController() {}

OvertimeEditResultConfigController.prototype = new OvertimeReportEditConfigController();

OvertimeEditResultConfigController.prototype.makeReportParamsAccordingToFeatures = function ( reportParameters ) {
    return reportParameters;
};

OvertimeEditResultConfigController.prototype.shouldMakeColumnsConfigSelector = function(config)
{
    return EnabledFeatures.OVR || config.reportParameters.columnsConfig !== OvertimeReport.ColumnsConfig.TIME_BALANCE_ONLY;
};