function ReportEditConfigController ()
{
    this.constructor = ReportEditConfigController;
}

ReportEditConfigController.prototype.possibleErrors = {
    ERROR_EMPTY_USER_SELECTION: "errorEmptyUserSelection",
    ERROR_EMPTY_CP_SELECTION: "errorEmptyCPSelection",
    ERROR_EMPTY_LEAVE_TYPES_SELECTION: "errorEmptyLeaveTypesSelection",
    ERROR_IN_GROUPING_LEVELS_SELECTOR: "errorInGroupingLevelsSelector",
    ADDITIONAL_ERROR: "additionalError"
};

ReportEditConfigController.prototype.init = function ( config )
{
    this.productSettingsValidator = _.isUndefined(config.productSettingsValidator) ? new ATProductSettingsValidator() : config.productSettingsValidator;
    this.reportRightsValidator = _.isUndefined(config.reportRightsValidator) ? new ATReportRightsValidator() : config.reportRightsValidator;
    this.initReportParameters = config.reportParameters;

    this.panelId = config.panelId ? config.panelId : "editReportConfigurationPanel";
    this.panelEl = $("#" + this.panelId);

    this.errorsToSectionElsMap = this.initErrorsToSectionElsMap();

    var self = this;
    this.errors = [];
    this.errorMessageEl = this.panelEl.find(".errorMessage" );
    this.errorTooltipsForSectionsController = new ErrorTooltipsForSectionsController().init( "reportEditConfigErrorTooltip",
                                                                                             this.errorsToSectionElsMap,
                                                                                             function( error ) { return self.getErrorMessage( error ); } );

    this._initTimeFormatTooltip();

    this.preloadedState = {
                              initCPSelector: function ( cpSelector ) {
                                  cpSelector.initializeData(config.cpList);
                              },
                              initUserSelector: function ( userSelector ) {
                                  userSelector.initializeData(config.userGroups != null,
                                                              config.timeZoneGroups != null,
                                                              config.users,
                                                              config.userGroups,
                                                              config.timeZoneGroups);
                              }
                          };
};

/**
 * Set default values for selectors are dependent on features
 */
ReportEditConfigController.prototype.makeReportParamsAccordingToFeatures = function() {
    // abstract
};

ReportEditConfigController.prototype.makeCPSelector = function ( customersProjectsSelection ) {
    var self = this;
    var configChanged = function () { self.removeCPSelectionError(); self.configChanged(); };

    var cpSelector = new CPReportSelector().init( customersProjectsSelection,
                                                  "cpSelectorId", configChanged );
    this.preloadedState.initCPSelector(cpSelector);

    cpSelector.panel.on("change", function() {
        cpSelector.setWasChanged();
    });

    return cpSelector;
};

ReportEditConfigController.prototype.adjust = function(groups, groupType, users, showDisabled)
{
    var adjustUsers;
    if (!showDisabled)
        users = _.filter(users, function(user) { return user.active; });

    if (groupType == "USER_GROUP")
    {
        adjustUsers = _.filter(users, function(user) { return _.contains(groups, user.userGroupId)});
    }
    if (groupType == "TIME_ZONE_GROUP")
    {
        adjustUsers = _.filter(users, function(user) { return _.contains(groups, user.timeZoneGroupId)});
    }
    return _.map(adjustUsers, function(user) { return user.id; });
};


ReportEditConfigController.prototype.makeUserSelector = function ( config, specificConstructorPanel ) {
    var self = this;
    var configChanged = function () { self.removeUserSelectionError(); self.configChanged(); };

    var userSelection = config.reportParameters.userSelection;
    if (((!this.productSettingsValidator.isUGREnabled() && config.reportParameters.userSelection.groupType == GroupType.USER_GROUP) ||
            (!this.productSettingsValidator.isTZGEnabled() && config.reportParameters.userSelection.groupType == GroupType.TIME_ZONE_GROUP)) &&
            config.reportParameters.userSelection.selectedGroups.length > 0 )
    {
        userSelection.selectedUsers = userSelection.selectedUsers.concat(self.adjust(userSelection.selectedGroups,
                                                                                     userSelection.groupType,
                                                                                     config.users,
                                                                                     userSelection.showDisabled));
        userSelection.selectedGroups = [];
    }

    var selectorConfig = {
        selectorId: "userSelectorId",
        userSelection: userSelection,
        changeCallback: configChanged,
        needAdditionalCheckbox : config.needAdditionalCheckbox !== undefined ? config.needAdditionalCheckbox : true,
        additionalCheckboxLabel: config.additionalCheckboxLabel || messageResource.getMessage("users_selector.reports.include_users_wo_time")
    };

    if ( specificConstructorPanel )
        selectorConfig.specificConstructorPanel = specificConstructorPanel;

    var userSelector = new ReportUserSelectorButton().init(selectorConfig);

    userSelector.initializeData(config.userGroups != null,
                                config.timeZoneGroups != null,
                                config.users,
                                config.userGroups,
                                config.timeZoneGroups);

    userSelector.panel.on("change", function() {
        userSelector.setWasChanged();
    });

    return userSelector;
};

ReportEditConfigController.prototype.getConfig = function () {
    // abstract
};

ReportEditConfigController.prototype.resetConfig = function ( config ) {
    // abstract
};

ReportEditConfigController.prototype.hasErrors = function () {
    return this.errors.length > 0;
};

ReportEditConfigController.prototype.setConfigButtonHandlers = function ( handlers ) {
    $("#applyReportConfiguration").click(function () {handlers.onApply();});
    $("#cancelReportConfiguration").click(function () {handlers.onCancel();});
};

ReportEditConfigController.prototype.initListSelector = function ( selectId, initValue, changeCallback, additionalClass )
{
    var selector = this.initCustomListSelector( selectId, changeCallback, additionalClass );
    selector.setActiveItemByValue(initValue);

    return selector;
};

ReportEditConfigController.prototype.initTimeFormatSelector = function(timeFormat, changedCallback){
    return this.initListSelector("timeFormatSelect", timeFormat, changedCallback, "timeFormatMenu");
};

ReportEditConfigController.prototype._initTimeFormatTooltip = function() {
    // init tooltip for time format selector
    var timeFormatTooltip = new TableTooltip().init("timeFormatTooltip");
    var icon = $("#timeFormatInfo");
    icon.hover(function(event) {timeFormatTooltip.show(); timeFormatTooltip.move(calculateTooltipOffset( timeFormatTooltip, event ))}, function() {timeFormatTooltip.hide()});
    icon.mousemove(function(event) { timeFormatTooltip.move(calculateTooltipOffset( timeFormatTooltip, event ))});
};

ReportEditConfigController.prototype.setSelectorsInitStateFromCurrent = function () {
    this.initReportParameters = this.getConfig();
};

ReportEditConfigController.prototype.setSelectorsFromReportParameters = function (reportParameters) {
    this.initReportParameters = reportParameters;
};

ReportEditConfigController.prototype.addApprovedTTFilter = function ( initValue, changeCallback )
{
    var isNeedApprovedTTFilter = EnabledFeatures.TTA || (initValue && initValue != TimeApprovalStatus.BOTH);
    var selectorRow = $("#ttStatusFilterRow");
    if (isNeedApprovedTTFilter) {
        selectorRow.show();
        return new ReportTimeTrackStatusFilter().init( "filterByTTStatus", initValue, changeCallback )
    } else {
        selectorRow.hide();
        return this.initEmptyListSelector(initValue);
    }
};

ReportEditConfigController.prototype.initEmptyListSelector = function( initValue) {
    return { getValue: function(){ return initValue; }, setActiveItemByValue: function() {} };
};

ReportEditConfigController.prototype.initRadioGroupSelector = function ( radioGroupSelectorId, radioGroupName, initValue, onChangeCallback )
{
    var selector = $("#" + radioGroupSelectorId);
    selector.groupName = radioGroupName;
    selector.setItemByValue = function(value){
        this.find("[name=" + selector.groupName + "][value=" + value +"]").prop( 'checked', true );
    };

    selector.getValue= function ()
    {
        return this.find("[name=" + selector.groupName + "]:checked").val();
    };
    selector.setItemByValue(initValue);

    if ( onChangeCallback )
        selector.on("change", onChangeCallback);

    return selector;
};


ReportEditConfigController.prototype.initCustomListSelector = function ( selectId, onChangeCallback, additionalCls, initValue )
{
    var menuCls = "columns-format-menu";
    menuCls += additionalCls ? " " + additionalCls : "";
    var selector = new AT.DropdownListButton( {
                                                  applyTo: selectId,
                                                  menuCls: menuCls,
                                                  cls: "report-config-control-button"
                                              } );

    if( initValue) {
        selector.setActiveItemByValue(initValue);
    }

    if( onChangeCallback != null )
        selector.on("change", onChangeCallback );

    return selector;
};
// method for grouping level selectors (check if selector contains concrete value)
ReportEditConfigController.prototype.checkGroupingLevel = function(levelValue, groupingParams) {
    for(var i = 0; i < groupingParams.length; i++) {
        if(groupingParams[i] == levelValue) {
            return true;
        }
    }
    return false;
};

ReportEditConfigController.prototype.validate = function ()
{
    return true;
};

ReportEditConfigController.prototype.updateUI = function(blinkError)
{
    for (var error in this.possibleErrors)
    {
        if (this.possibleErrors.hasOwnProperty(error))
            this.panelEl.removeClass( this.possibleErrors[error] );
    }

    for (var i=0; i<this.errors.length; i++)
        this.panelEl.addClass(this.errors[i]);

    var errorMsg = "";

     if (this.errors.length == 1)
        errorMsg = this.getErrorMessage(this.errors[0]);
    else if (this.errors.length > 1)
        errorMsg = messageResource.getMessage("report.js.error.exist_invalid_fields");

    this.errorMessageEl.html(errorMsg);
    if (errorMsg && blinkError)
        this.blinkErrorInFooter();

    this.errorTooltipsForSectionsController.updateTooltips( this.errors );
};

ReportEditConfigController.prototype.initErrorsToSectionElsMap = function()
{
    // abstract
    return {};
};

ReportEditConfigController.prototype.removeGroupingLevelsSelectionError = function()
{
    this.errors.remove(this.possibleErrors.ERROR_IN_GROUPING_LEVELS_SELECTOR);
    this.updateUI();
};

ReportEditConfigController.prototype.removeCPSelectionError = function()
{
    this.errors.remove(this.possibleErrors.ERROR_EMPTY_CP_SELECTION);
    this.updateUI();
};

ReportEditConfigController.prototype.removeUserSelectionError = function()
{
    this.errors.remove(this.possibleErrors.ERROR_EMPTY_USER_SELECTION);
    this.updateUI();
};

ReportEditConfigController.prototype.getErrorMessage = function( error)
{
    switch (error)
    {
        case this.possibleErrors.ERROR_EMPTY_CP_SELECTION:
            return messageResource.getEscapedMessage("projects_customers_selector.please_select_customer_or_project");
            break;
        case this.possibleErrors.ERROR_EMPTY_USER_SELECTION:
            return messageResource.getMessage("report.js.choose_person");
            break;
        case this.possibleErrors.ERROR_IN_GROUPING_LEVELS_SELECTOR:
            return this.groupingSelector.getErrorMessage();
            break;
        case this.possibleErrors.ERROR_EMPTY_LEAVE_TYPES_SELECTION:
            return messageResource.getMessage("report.leaves.js.choose_leave_type");
            break;
        case this.possibleErrors.ADDITIONAL_ERROR:
            return messageResource.getMessage(this.additionalErrorMessageKey);
    }
    return "";
};

ReportEditConfigController.prototype.hideErrorTooltip = function()
{
    this.errorTooltipsForSectionsController.hideErrorTooltip();
};

ReportEditConfigController.prototype.blinkErrorInFooter = function ()
{
    this.errorMessageEl.addClass( "blinkDisabled" );
    var self = this;
    window.setTimeout( function ()
                       {
                           self.errorMessageEl.removeClass( "blinkDisabled" );
                       }, 250 );
};

ReportEditConfigController.prototype.configChanged = function() {
    if ( this.configChangedCallback )
        this.configChangedCallback();
};

/**
 * Disable time format selector (and set decimal format) when configuration for CSV
 * Enable time format selector with previous value on HTML configuration
 * @param isCSV true - on CSV , false - on HTML
 */
ReportEditConfigController.prototype.switchTimeFormatBetweenCSVAndHTML = function( isCSV ) {
    if (this.timeFormatSelector){
        var timeFormatSelector = this.timeFormatSelector;
        if (isCSV){
            // if previous value has already been set, it means we are on csv preview tab now,
            // so we must not update previous value again, because we will spoil it
            if (!timeFormatSelector.previousValue)
                timeFormatSelector.previousValue = timeFormatSelector.getValue();
            timeFormatSelector.setActiveItemByValue(TimeFormat.DECIMAL, true);
            timeFormatSelector.disable();
        } else {
            if (timeFormatSelector.previousValue) {
                timeFormatSelector.setActiveItemByValue( timeFormatSelector.previousValue, true );
                // delete previous value after it has been used in order not to get back to it anymore
                timeFormatSelector.previousValue = undefined;
            }
            timeFormatSelector.enable();
        }
    }
};

/**
 * Change grouping selection depending on actual features and right
 * If top level value is not available that all under levels will be shifted to top
 * @param notAvailableLevel - level that now not available
 * @param groupings - levels where can be a not available level
 * @param defaultValueForFirstGrouping - default value for first level (set only all levels are NONE)
 * @returns {*}
 */
ReportEditConfigController.prototype.makeGroupingSelectorValuesAccordingToNotAvailableValueLevel = function(notAvailableLevel, groupings, defaultValueForFirstGrouping){
    for(var i = 0; i < groupings.length; i++){
        if(groupings[i] == notAvailableLevel){
            for(var j = i; j < groupings.length - 1; ++j){
                groupings[j] = groupings[j+1];
            }
            groupings[groupings.length - 1] = ReportGroupingLevel.NONE;
        }
    }
    if(defaultValueForFirstGrouping && groupings[0] == ReportGroupingLevel.NONE){
        groupings[0] = defaultValueForFirstGrouping;
    }
    return groupings;
};

ReportEditConfigController.prototype.setAdditionalError = function(messageKey){
    this.errors.push(this.possibleErrors.ADDITIONAL_ERROR);
    this.additionalErrorMessageKey = messageKey;
};
ReportEditConfigController.prototype.removeAdditionalError = function(){
    this.errors.remove(this.possibleErrors.ADDITIONAL_ERROR);
    this.additionalErrorMessageKey = "";
    this.updateUI();
};