function StaffOutputReportEditConfigController ()
{
    this.constructor = StaffOutputReportEditConfigController;
}

StaffOutputReportEditConfigController.prototype = new ReportEditConfigController();

StaffOutputReportEditConfigController.prototype.init = function ( config )
{
    ReportEditConfigController.prototype.init.call(this, config);

    var self = this;
    var reportParameters = this.makeReportParamsAccordingToFeatures(config.reportParameters);

    this.configChangedCallback = config.changeCallback;
    var configChanged = function () { self.configChanged(); };
    
    // if feature enabled otherwise only if value is not "users only" - show selector
    this.columnFormatSelector = this.addColumnFormatFilter(reportParameters.columnsConfig, configChanged);

    // if feature enabled otherwise only if value is not "both" - show selector
    this.approvedTTFilter = this.addApprovedTTFilter(reportParameters.timeApprovalStatus, configChanged);


    this.timeFormatSelector = this.initTimeFormatSelector(reportParameters.timeFormat, configChanged);
    
    this.cpSelector = this.makeCPSelector(reportParameters.customersProjectsSelection);

    var groupingLevelsSelection = [reportParameters.firstGroupingLevel, reportParameters.secondGroupingLevel, reportParameters.thirdGroupingLevel];
    this.groupingSelector = new ThreeLevelReportGroupingSelector().init( {
        selectorId: "reportGroupingSelector",
        enableNoneFirstGrouping: false,
        enableGroupByBillingType: EnabledFeatures.BTY || this.checkGroupingLevel(ReportGroupingLevel.BILLING_TYPE, groupingLevelsSelection),
        enableGroupByUsers: false,
        enableGroupByUserGroups: false,
        includeTTCommentsInDetailsLevel: false,
        groupingLevelsSelection: groupingLevelsSelection,
        changesCallback: function() {
            self.removeGroupingLevelsSelectionError();
            configChanged();
        }
    });

    // if feature enabled or checkbox is selected
    if ( this.verifyIsShowLeavesCheckbox(reportParameters) ) {
        $("#leavesCheckbox").show();
        this.showLeaveTimeChexbox = $( "#showLeavesCheckbox" );
        this.showLeaveTimeChexbox.prop( "checked", reportParameters.showLeaveTime );
        this.showLeaveTimeChexbox.on("change", configChanged);
    }

    this.userSelector = this.makeUserSelector(config);

    return this;
};

/**
 * add column format filter if need
 * @param initValue
 * @param onChangeCallback
 * @returns {{getValue, setActiveItemByValue}}
 */
StaffOutputReportEditConfigController.prototype.addColumnFormatFilter = function( initValue, onChangeCallback) {
    var ifNeedColumnFormatSelector = EnabledFeatures.UGR || EnabledFeatures.TZG|| (initValue != StaffPerformanceColumnsConfig.USERS_ONLY);
    var selectorRow = $("#columnsFormatFilterRow");
    if (ifNeedColumnFormatSelector){
        selectorRow.show();
        return new StaffoutputColumnsFormatSelector().init("columnsDisplayFormat", initValue, onChangeCallback);
    } else {
        selectorRow.hide();
        return this.initEmptyListSelector(initValue);
    }
};

StaffOutputReportEditConfigController.prototype.verifyIsShowLeavesCheckbox = function ( reportParameters ) {
    return EnabledFeatures.LVT || reportParameters.showLeaveTime;
};

StaffOutputReportEditConfigController.prototype.getConfig = function () {
    var groupingLevelValues = this.groupingSelector.getValues();
    return {
        columnsConfig: this.columnFormatSelector.getValue(),
        timeFormat: this.timeFormatSelector.getValue(),
        timeApprovalStatus: this.approvedTTFilter.getValue(),
        userSelection: this.userSelector.getSelection(),
        firstGroupingLevel: groupingLevelValues[0],
        secondGroupingLevel: groupingLevelValues[1],
        thirdGroupingLevel: groupingLevelValues[2],
        customersProjectsSelection: this.cpSelector.getCpSelection(),
        showLeaveTime: !!this.showLeaveTimeChexbox && this.showLeaveTimeChexbox.is(":checked")
    };
};

StaffOutputReportEditConfigController.prototype.resetConfig = function () {
    this.columnFormatSelector.setActiveItemByValue(this.initReportParameters.columnsConfig, true);    
    this.timeFormatSelector.setActiveItemByValue(this.initReportParameters.timeFormat, true);    
    this.approvedTTFilter.setActiveItemByValue(this.initReportParameters.timeApprovalStatus, true);
    this.userSelector.setSelection(this.initReportParameters.userSelection);
    this.groupingSelector.setValues([this.initReportParameters.firstGroupingLevel, this.initReportParameters.secondGroupingLevel, this.initReportParameters.thirdGroupingLevel]);
    this.cpSelector.setSelection(this.initReportParameters.customersProjectsSelection);
    if(this.showLeaveTimeChexbox){
        this.showLeaveTimeChexbox.prop("checked", this.initReportParameters.showLeaveTime);
    }
};

StaffOutputReportEditConfigController.prototype.setSelectorsInitStateFromCurrent = function () {
    ReportEditConfigController.prototype.setSelectorsInitStateFromCurrent.call(this);
    
    if(this.userSelector){
        this.userSelector.setInitialValueFromCurrent();
    }
    if(this.cpSelector) {
        this.cpSelector.setInitialValueFromCurrent();
    }
};

StaffOutputReportEditConfigController.prototype.validate = function()
{
    this.errors = [];

    if (!this.userSelector.validateIsUserSelected())
        this.errors.push( this.possibleErrors.ERROR_EMPTY_USER_SELECTION );

    if (!this.cpSelector.checkSelected())
        this.errors.push( this.possibleErrors.ERROR_EMPTY_CP_SELECTION );

    if (!this.groupingSelector.validate())
        this.errors.push(this.possibleErrors.ERROR_IN_GROUPING_LEVELS_SELECTOR);

    return this.errors.length == 0;
};

StaffOutputReportEditConfigController.prototype.initErrorsToSectionElsMap = function()
{
    var errorsToSectionElsMap = {};
    errorsToSectionElsMap[this.possibleErrors.ERROR_EMPTY_USER_SELECTION] = this.panelEl.find( "#userSelectorId" );
    errorsToSectionElsMap[this.possibleErrors.ERROR_EMPTY_CP_SELECTION] = this.panelEl.find( "#cpSelectorId" );
    errorsToSectionElsMap[this.possibleErrors.ERROR_IN_GROUPING_LEVELS_SELECTOR] = this.panelEl.find( ".reportGroupingSelectorContainer" );
    return errorsToSectionElsMap;
};

// ----------------------

function StaffOutputReportEditFormConfigController() {}
StaffOutputReportEditFormConfigController.prototype = new StaffOutputReportEditConfigController();

StaffOutputReportEditFormConfigController.prototype.makeReportParamsAccordingToFeatures = function( reportParameters ){
    var result = $.extend({}, reportParameters);
    if (!EnabledFeatures.UGR){
        result.columnsConfig = StaffPerformanceColumnsConfig.USERS_ONLY;
    }
    if (!EnabledFeatures.TTA) {
        result.timeApprovalStatus = TimeApprovalStatus.BOTH;
    }
    if (!EnabledFeatures.LVT) {
        result.showLeaveTime = false;
    }
    if (!EnabledFeatures.BTY)
    {
        var groupings = this.makeGroupingSelectorValuesAccordingToNotAvailableValueLevel(ReportGroupingLevel.BILLING_TYPE, [result.firstGroupingLevel, result.secondGroupingLevel], ReportGroupingLevel.CUSTOMER);
        result.firstGroupingLevel = groupings[0];
        result.secondGroupingLevel = groupings[1];
    }

    return result;
};

// ---------------------
function StaffOutputReportEditResultConfigController() {}
StaffOutputReportEditResultConfigController.prototype = new StaffOutputReportEditConfigController();

StaffOutputReportEditResultConfigController.prototype.makeReportParamsAccordingToFeatures = function( reportParameters ){
    return reportParameters;
};