function TTDetailReportEditConfigController ()
{
    this.constructor = TTDetailReportEditConfigController;
}

TTDetailReportEditConfigController.prototype = new ReportEditConfigController();

TTDetailReportEditConfigController.groupingLevels = {
    NO_GROUPING : 0,
    ONE_LEVEL : 1,
    TWO_LEVELS : 2,
    THREE_LEVELS : 3
};

TTDetailReportEditConfigController.prototype.init = function ( config )
{
    ReportEditConfigController.prototype.init.call(this, config);

    var reportParameters = this.makeReportParamsAccordingToFeatures(config.reportParameters);

    var self = this;
    this.configChangedCallback = config.changeCallback;
    var configChanged = function () { self.configChanged(); };

    this.timeFormatSelector = this.initListSelector("timeFormatSelect", reportParameters.timeFormat, configChanged);

    var groupingSelection = [reportParameters.firstGroupingLevel, reportParameters.secondGroupingLevel, reportParameters.thirdGroupingLevel];

    this.groupingSelector = new TTDetailsReportGroupingSelector().init("reportGroupingSelector",
        groupingSelection,
        function() { 
            self.removeGroupingLevelsSelectionError();
            self.joinCommentsCheckbox.activate(self._checkIfNeedDisableJoinCommentsCheckbox());
            configChanged();});

    // init list selector
    this.numberOfGroupingLevelsSelector = this._initNumberOfGroupingLevelsListSelector(
        "numberOfGroupingLevelsSelect",
        this.groupingSelector.calculateNumberOfVisibleLevels(groupingSelection),
        function() {
            var numberOfGroupingLevels = self.numberOfGroupingLevelsSelector.getSelectedItem().value;
            self.groupingSelector.setNumberOfVisibleLevels(numberOfGroupingLevels);

            self.removeGroupingLevelsSelectionError();
            // change state of checkbox
            self.joinCommentsCheckbox.activate(self._checkIfNeedDisableJoinCommentsCheckbox() );
            configChanged();
        }
    );

    this.joinCommentsCheckbox = $("#joinComments");
    // disable checkbox or activate with previous value
    this.joinCommentsCheckbox.activate = function(isDisable){
        if (isDisable) {
            if (!this.prop("disabled")) {
                this.prevValue = this.prop("checked");
                this.prop("checked", false);
                this.prop("disabled", true );
            }
        } else {
            if (this.prop("disabled")) {
                this.prop("checked", this.prevValue);
            }
            this.prop("disabled", false );
        }
    };
    this.joinCommentsCheckbox.on("change", function(){
       configChanged();
    });
    // set value
    this.joinCommentsCheckbox.prop("checked", reportParameters.joinCommentsByTask);
    // than activate checkbox
    this.joinCommentsCheckbox.activate(this._checkIfNeedDisableJoinCommentsCheckbox());

    config.needAdditionalCheckbox = false;
    this.userSelector = this.makeUserSelector(config);

    this.approvedTTFilter = this.addApprovedTTFilter(reportParameters.timeApprovalStatus, configChanged);

    this.cpSelector = this.makeCPSelector(reportParameters.customersProjectsSelection);

    return this;
};

/**
 * Return true if need disable join comments checkbox (if DAY && USER (disable  joinComments checkbox) or if n grouping levels = 0)
 * return  true if need disable or false otherwise
 * @returns {boolean|*}
 * @private
 */
TTDetailReportEditConfigController.prototype._checkIfNeedDisableJoinCommentsCheckbox = function() {
    var numberOfGroupingLevels = this.numberOfGroupingLevelsSelector.getValue();
    var values = this.groupingSelector.getValues(); // get grouping selection

    return numberOfGroupingLevels == TTDetailReportEditConfigController.groupingLevels.NO_GROUPING ||
        (this.checkGroupingLevel(ReportGroupingLevel.USER, values) && this.checkGroupingLevel(ReportGroupingLevel.DATE, values));
};

TTDetailReportEditConfigController.prototype._initNumberOfGroupingLevelsListSelector = function(id, selectedValue, onChangeCallback) {
    var numberOfGroupingLevelsSelector = new ListSelector({
        renderTo: $.find("#" + id)[0]
    });
    numberOfGroupingLevelsSelector.addItems(([
        {
            text :messageResource.getEscapedMessage("reports.comments_report.form.format.no_grouping"),
            value: TTDetailReportEditConfigController.groupingLevels.NO_GROUPING
        },
        {
            text :messageResource.getEscapedMessage("reports.comments_report.form.format.1_level"),
            value: TTDetailReportEditConfigController.groupingLevels.ONE_LEVEL
        },
        {
            text :messageResource.getEscapedMessage("reports.comments_report.form.format.2_levels"),
            value: TTDetailReportEditConfigController.groupingLevels.TWO_LEVELS
        },
        {
            text :messageResource.getEscapedMessage("reports.comments_report.form.format.3_levels"),
            value: TTDetailReportEditConfigController.groupingLevels.THREE_LEVELS
        }
    ]));

    numberOfGroupingLevelsSelector.on("change", onChangeCallback);

    numberOfGroupingLevelsSelector.setActiveItemByValue(selectedValue, true);

    return numberOfGroupingLevelsSelector;
};

TTDetailReportEditConfigController.prototype.getConfig = function () {
    var groupingLevelValues = this.groupingSelector.getValues();
    return {
        timeFormat: this.timeFormatSelector.getValue(),
        joinCommentsByTask: this.joinCommentsCheckbox.prop("checked"),
        userSelection: this.userSelector.getSelection(),
        timeApprovalStatus: this.approvedTTFilter.getValue(),
        customersProjectsSelection: this.cpSelector.getCpSelection(),
        firstGroupingLevel: groupingLevelValues[0],
        secondGroupingLevel: groupingLevelValues[1],
        thirdGroupingLevel: groupingLevelValues[2]
    };
};

TTDetailReportEditConfigController.prototype.resetConfig = function () {
    var groupingSelection = [this.initReportParameters.firstGroupingLevel, this.initReportParameters.secondGroupingLevel, this.initReportParameters.thirdGroupingLevel];

    this.timeFormatSelector.setActiveItemByValue(this.initReportParameters.timeFormat, true);
    this.numberOfGroupingLevelsSelector.setActiveItemByValue(this.groupingSelector.calculateNumberOfVisibleLevels(groupingSelection), false);
    this.joinCommentsCheckbox.prop("checked", this.initReportParameters.joinCommentsByTask);
    this.userSelector.setSelection(this.initReportParameters.userSelection);
    this.approvedTTFilter.setActiveItemByValue(this.initReportParameters.timeApprovalStatus, true);
    this.cpSelector.setSelection(this.initReportParameters.customersProjectsSelection);
    this.groupingSelector.setValues(groupingSelection);
};

TTDetailReportEditConfigController.prototype.validate = function()
{
    this.errors = [];

    if (!this.userSelector.validateIsUserSelected())
        this.errors.push( this.possibleErrors.ERROR_EMPTY_USER_SELECTION );

    if (!this.cpSelector.checkSelected())
        this.errors.push( this.possibleErrors.ERROR_EMPTY_CP_SELECTION );

    if (!this.groupingSelector.validate())
        this.errors.push(this.possibleErrors.ERROR_IN_GROUPING_LEVELS_SELECTOR);

    return this.errors.length == 0;
};

TTDetailReportEditConfigController.prototype.initErrorsToSectionElsMap = function()
{
    var errorsToSectionElsMap = {};
    errorsToSectionElsMap[this.possibleErrors.ERROR_EMPTY_USER_SELECTION] = this.panelEl.find( "#userSelectorId" );
    errorsToSectionElsMap[this.possibleErrors.ERROR_EMPTY_CP_SELECTION] = this.panelEl.find( "#cpSelectorId" );
    errorsToSectionElsMap[this.possibleErrors.ERROR_IN_GROUPING_LEVELS_SELECTOR] = this.panelEl.find( ".dataGroupingControlContainer" );
    return errorsToSectionElsMap;
};

TTDetailReportEditConfigController.prototype.setSelectorsInitStateFromCurrent = function () {
    ReportEditConfigController.prototype.setSelectorsInitStateFromCurrent.call(this);

    if(this.userSelector){
        this.userSelector.setInitialValueFromCurrent();
    }
    if(this.cpSelector) {
        this.cpSelector.setInitialValueFromCurrent();
    }
};

// -------------------------------
function TTDetailReportEditFormConfigController() {}

TTDetailReportEditFormConfigController.prototype = new TTDetailReportEditConfigController();
TTDetailReportEditFormConfigController.prototype.makeReportParamsAccordingToFeatures = function( reportParameters ){
    var results = $.extend({}, reportParameters);
    var groupingSelection = [results.firstGroupingLevel, results.secondGroupingLevel, results.thirdGroupingLevel];

    if ( !EnabledFeatures.BTY ){
        groupingSelection = this.makeGroupingSelectorValuesAccordingToNotAvailableValueLevel(ReportGroupingLevel.BILLING_TYPE, groupingSelection);
    }
    if ( !EnabledFeatures.UGR ){
        groupingSelection = this.makeGroupingSelectorValuesAccordingToNotAvailableValueLevel(ReportGroupingLevel.USER_GROUP, groupingSelection);
    }
    if ( !EnabledFeatures.TTA ) {
        results.timeApprovalStatus = TimeApprovalStatus.BOTH;
    }
    results.firstGroupingLevel = groupingSelection[0];
    results.secondGroupingLevel = groupingSelection[1];
    results.thirdGroupingLevel = groupingSelection[2];
    return results;
};

TTDetailReportEditFormConfigController.prototype.getJoinCommentsByTask = function ()
{
    return this.joinCommentsCheckbox.prop("checked");
};

// ------------------------------
function TTDetailReportEditResultConfigController() {}

TTDetailReportEditResultConfigController.prototype = new TTDetailReportEditConfigController();

TTDetailReportEditResultConfigController.prototype.showErrorTooltip = function(event, error)
{
    TTDetailReportEditConfigController.prototype.showErrorTooltip.call(this, event, error);
    if (error == this.possibleErrors.ERROR_IN_GROUPING_LEVELS_SELECTOR)
        this.groupingLevelsSelectorErrorTooltipShown = true;
};

TTDetailReportEditResultConfigController.prototype.hideErrorTooltip = function()
{
    TTDetailReportEditConfigController.prototype.hideErrorTooltip.call(this);
    this.groupingLevelsSelectorErrorTooltipShown = false;
};

TTDetailReportEditResultConfigController.prototype.init = function ( config ) {
    TTDetailReportEditConfigController.prototype.init.call(this, config);

    var self = this;
    $("#joinCommentsInfoContainer").show();
    var icon = $("#joinCommentsInfo");
    var joinCommentsTooltip = new TableTooltip().init("joinCommentsTooltip");
    icon.hover(
        function(event) {
            if (self.groupingLevelsSelectorErrorTooltipShown)
            {
                self.errorTooltip.hide();
                self.errorTooltipTemporarilyHidden = true;
            }

            joinCommentsTooltip.show();
            joinCommentsTooltip.move(calculateTooltipOffset( joinCommentsTooltip, event ));
        },
        function(event) {
            joinCommentsTooltip.hide();

            if (self.groupingLevelsSelectorErrorTooltipShown && self.errorTooltipTemporarilyHidden)
            {
                self.errorTooltip.show();
                self.moveErrorTooltip(event);
                self.errorTooltipTemporarilyHidden = false;
            }
        }
    );
    
    icon.mousemove(
        function(event) {
            joinCommentsTooltip.move(calculateTooltipOffset( joinCommentsTooltip, event ));
            event.stopPropagation();
        }
    );
    
    return this;
};

TTDetailReportEditResultConfigController.prototype.makeReportParamsAccordingToFeatures = function( reportParameters ){
    return reportParameters;
};