function EVACSVJsonConverter() {}

EVACSVJsonConverter.prototype = _.create(BaseCSVJsonConverter.prototype, {
    init: function(allCustomers, allProjects, tasksMap, usersMap, projectNamesMap, taskNamesMap, typesOfWorkMap, printableHeadersBuilder,
                   groupingPresentationFlags, store)
    {
        BaseCSVJsonConverter.prototype.init.call(this);
        this.customersMap = allCustomers;
        this.customers = _.values(allCustomers);
        this.projectsMap = allProjects;
        this.projects = _.values(allProjects);
        this.tasksMap = tasksMap;
        this.usersMap = usersMap;
        this.projectNamesMap = projectNamesMap;
        this.taskNamesMap = taskNamesMap;
        this.typesOfWorkMap = typesOfWorkMap;
        this.printableHeadersBuilder = printableHeadersBuilder;
        this.groupingPresentationFlags = groupingPresentationFlags;
        this.store = store;
        
        return this;
    },
    // override
    addGroupingInfo: function(groupingLevel, identity, line)
    {
        if (groupingLevel == ReportGroupingLevel.USER)
            line["user info"] = UserNameFormat.formatLFM(identity.firstName, identity.lastName, identity.middleInitial);
                
        else if (groupingLevel == ReportGroupingLevel.PROJECT)
            line["Project"] = EVAReport.FormatUtils.formatProject(identity, this.groupingPresentationFlags);
                
        else if (groupingLevel == ReportGroupingLevel.TASK)
            line["Task"] = EVAReport.FormatUtils.formatTask(identity, this.groupingPresentationFlags, true);
                
        else
            GroupingUtils.addGroupingInfo(groupingLevel, identity, line);
    },
    // override
    addDataToTable: function(data, line)
    {
        line["totalSpentTime"] = this.formatTime(data.totalSpentTime);
        if (this.includeNotEstimated)
        {
            line["spentTimeForNotEstimated"] = this.formatTime(data.spentTimeForNotEstimated);
            line["spentTimeForEstimated"] = this.formatTime(data.spentTimeForEstimated);
        }
        var invalidPercentage = (data.estimatedTime != null && data.estimatedTime < 0.0001);
        line["estimatedTime"] = this.formatTime(data.estimatedTime);
        line["varianceTime"] = this.formatVarianceTime(data.varianceTime);
        line["varianceTimeInPercentage"] = this.formatVariancePercent(invalidPercentage ? null : data.varianceTimeInPercentage);
    },
    // override
    addOtherColumnNames: function(columnNames, columns, config, data)
    {
        columns.push("totalSpentTime");
        columnNames["totalSpentTime"] = messageResource.getMessage("report.budgeted_vs_actual.spent_time.total");

        if (this.includeNotEstimated)
        {
            columns.push("spentTimeForNotEstimated");
            columnNames["spentTimeForNotEstimated"] = messageResource.getMessage("report.budgeted_vs_actual.spent_time.nonbudgeted");

            columns.push("spentTimeForEstimated");
            columnNames["spentTimeForEstimated"] = messageResource.getMessage("report.budgeted_vs_actual.spent_time.budgeted");
        }

        columns.push("estimatedTime");
        columnNames["estimatedTime"] = messageResource.getMessage("report.budgeted_vs_actual.budgeted");

        columns.push("varianceTime");
        columnNames["varianceTime"] = "Variance (mh)";

        columns.push("varianceTimeInPercentage");
        columnNames["varianceTimeInPercentage"] = "Variance (%)";
    },
    // override
    getIdentity: function(data, grouping)
    {
        switch (grouping)
        {
            case ReportGroupingLevel.USER:
                return this.usersMap[data.id];
            case ReportGroupingLevel.CUSTOMER:
                return this.customersMap[data.id];
            case ReportGroupingLevel.PROJECT:
                return this.store ? this.store.getProject(data.id) : this.projectsMap[data.id];
            case ReportGroupingLevel.TASK:
                return this.tasksMap[data.id];
            case ReportGroupingLevel.BILLING_TYPE:
                return this.typesOfWorkMap[data.id];
            case ReportGroupingLevel.TASK_NAME:
                return this.taskNamesMap[data.id];
            case ReportGroupingLevel.PROJECT_NAME:
                return this.projectNamesMap[data.id];
        }
        return data.id;
    },
    // override
    preprocess: function(config, data)
    {
        this.includeNotEstimated = config.includeNotEstimated;
        this.date = config.dateSelection;
    },
    // override
    processParamsBuilder: function(paramsBuilder, params, config)
    {
        var reportParamsStringBuilder = new ReportParamsStringBuilder().init({cpList: ATReportsStore.getCPList()});
        var reportParameters = params;
        var taskParams = {
            tasksConfig: reportParameters.tasksConfig,
            includeNotEstimated: !!reportParameters.includeNotEstimated,
            includeNotStarted: !!reportParameters.includeNotStarted
        };

        paramsBuilder.addCPParam(reportParameters.customersProjectsSelection, reportParamsStringBuilder);
        paramsBuilder.addEvATasksParam(taskParams, reportParameters.dateSelection, reportParamsStringBuilder);
        paramsBuilder.parameters.push([messageResource.getMessage("report.budgeted_vs_actual.date_label"),
                                       reportParamsStringBuilder.getEvaDateCSVString(reportParameters.dateSelection)]);
    },

    formatTime: function(time)
    {
        if (time == null)
            return "";
        return ConverterFormatUtils.numberFormatter.format(time / 60);
    },

    formatVarianceTime: function(time)
    {
        if (time == null)
            return "";
        return ConverterFormatUtils.varianceNumberFormatter.format(time / 60);
    },

    formatVariancePercent: function(percent)
    {
        if (percent == null)
            return "";
        return ConverterFormatUtils.varianceNumberFormatter.format(percent) + "%";
    },

    generateCSVName: function(reportParams)
    {
        var fileNameParts = [];
        var dateFormatter = new DateFormat(SystemSettings.dateFormat);
        fileNameParts.push(dateFormatter.formatDate(new Date(reportParams.dateSelection.date)));

        var filename = messageResource.getMessage('common.eva.reports.csv.file_name', fileNameParts);
        if (isOldIE())
        {
            filename = filename.replace(/\//g, "_")
        }
        return filename;
    },

    generateTableHeader: function(data, config, columnNames, columns)
    {
        for (var i = 0; i < this.groupingLevels.length; ++i)
        {
            if (this.groupingLevels[i] != ReportGroupingLevel.USER)
                GroupingUtils.addGroupingColumnName(this.groupingLevels[i], columnNames, columns);
            else
            {
                columnNames["user info"] = "User";
                columns.push("user info");
            }
        }
        this.addOtherColumnNames(columnNames, columns, config, data);
    },

    convertTotal: function(config, data)
    {
        // add total info from root
        var columnNames = {};
        var columns = [];
        this.generateTableHeader(data, config, columnNames, columns);
        var line = {};
        this.addDataToTable(data, line);

        // changing name for total
        columns[0] = "Total";
        line["Total"] = "Total:";
        // no columnNames because we don't need headers
        // {} in field table for blank line
        return {
            columns: columns,
            table: [{}, line]
        };
    }
});
