EVAReport.Table.DataRow = function($containerEl, config, store, includeNotEstimated, levelClass)
{
    TemplateView.call(this, $containerEl);
    this.config = config;
    this.store = store;
    this.includeNotEstimated = includeNotEstimated;
    this.levelClass = levelClass;
};

EVAReport.Table.DataRow.prototype = _.create(TemplateView.prototype, {
    html: "<tr class='dataRow <%=rowClass%>' >" +
          "  <td class='titleCell <%=levelShift%>'>" +
          "    <div style='width:99%'>" +
          "      <div style='float: left'><%=title%></div>" +
          "      <div class='<%=noteStyles%>' style='float:right; white-space: nowrap; padding-left: 5px' align='right'><%=titleNote%></div>" +
          "    </div>" +
          "  </td>" +
          "  <td class='totalSpentCell' style='padding-left: 4px'><%=totalSpent%></td>" +
          "  <td class='timeNotEstimateCell' style='padding-left: 4px'><%=timeNotEstimate%></td>" +
          "  <td class='estimate totalSpent' style='padding-left: 4px'><%=estimateTotalSpentTime%></td>" +
          "  <td class='estimate estimatedTime' style='padding-left: 4px'><%=estimatedTime%></td>" +
          "  <td class='estimate varianceHour' style='padding-left: 4px'><%=varianceHour%></td>" +
          "  <td class='estimate variancePercentage' style='padding-left: 4px'><%=variancePercent%></td>" +
          "</tr>",

    templateKey: "EVAReport.Table.DataRow",

    afterRender: function()
    {
        if (!this.includeNotEstimated)
        {
            $(".totalSpentCell").hide();
            $(".timeNotEstimateCell").hide();
        }
    },

    templateModel: function()
    {
        var config = this.config;
        var timeFormat = this.store.getTimeFormat();
        var model = {
            title: this.title(config.id, config.type, config.groupingType),
            noteStyles: this.noteStyles(config.id, config.groupingType),
            titleNote: this.titleNoteCell(config.id, config.type, config.groupingType),
            totalSpent: this.createTypeCell(config.totalSpentTime, EVAReport.Table.DataRow.CellType.TIME, timeFormat),
            timeNotEstimate: this.createTypeCell(config.spentTimeForNotEstimated, EVAReport.Table.DataRow.CellType.TIME, timeFormat),
            estimateTotalSpentTime: this.createTypeCell(config.spentTimeForEstimated, EVAReport.Table.DataRow.CellType.TIME, timeFormat),
            estimatedTime: this.createTypeCell(config.estimatedTime, EVAReport.Table.DataRow.CellType.TIME, timeFormat),
            varianceHour: this.createTypeCell(config.varianceTime, EVAReport.Table.DataRow.CellType.TIME, timeFormat),
            variancePercent: this.createTypeCell(config.varianceTimeInPercentage, EVAReport.Table.DataRow.CellType.PERCENT, timeFormat),
            rowClass: "new_report_" + this.levelClass + "_row",
            levelShift: "new_report_real_level" + config.level
        };


        /** Logic below describe situations when field in table can be zero or whitespace or "-".
         *  Full describe logic here https://jira.actimind.com/browse/AT-675
         * **/
        var missCell = this.createCell("-", "");
        var negativeMissCell = this.createCell("-", this.stylesForCell(-1));
        var emptyCell = this.createCell("", "");
        var eps = 0.0001;

        switch (config.groupingType)
        {
            case ReportGroupingLevel.USER:
                model.varianceHour = emptyCell;
                model.variancePercent = emptyCell;
                model.estimatedTime = emptyCell;

                if (_.isUndefined(config.spentTimeForEstimated))
                    model.estimateTotalSpentTime = missCell;

                if (_.isUndefined(config.spentTimeForNotEstimated))
                    model.timeNotEstimate = missCell;
                break;
            case ReportGroupingLevel.TASK:
                if (!EVAReport.Store.getTask(config.id).estimated)
                {
                    model.estimatedTime = this.createCell(messageResource.getMessage("report.budgeted_vs_actual.not_budgeted"), "time not_budgeted");
                    model.varianceHour = missCell;
                    model.variancePercent = missCell;
                    model.estimateTotalSpentTime = missCell;
                }

                if (_.isUndefined(config.spentTimeForEstimated))
                    model.estimateTotalSpentTime = missCell;
                if (_.isUndefined(config.spentTimeForNotEstimated))
                    model.timeNotEstimate = missCell;

                break;
            case ReportGroupingLevel.PROJECT:
            case ReportGroupingLevel.CUSTOMER:
            case ReportGroupingLevel.TASK_NAME:
            case ReportGroupingLevel.PROJECT_NAME:
            case ReportGroupingLevel.BILLING_TYPE:
                if (_.isUndefined(config.estimatedTime))
                {
                    model.estimatedTime = missCell;
                    model.variancePercent = missCell;
                    model.varianceHour = missCell;
                }
                break;
        }

        if (config.groupingType != ReportGroupingLevel.USER)
        {
            if (config.estimatedTime < eps)
            {
                model.variancePercent = missCell;
                model.variancePercent = negativeMissCell;
            }
        }


        return model;
    },

    createTypeCell: function(value, type, timeFormat)
    {
        return this.createCell(this.formatValue(value, type, timeFormat), this.stylesForCell(value, type));
    },


    formatValue: function(value, type, timeFormat)
    {
        var result;
        switch (type)
        {
            case EVAReport.Table.DataRow.CellType.TIME:
                result = EVAReport.FormatUtils.formatPureTime(value, timeFormat);
                break;
            case EVAReport.Table.DataRow.CellType.PERCENT:
                result = EVAReport.FormatUtils.formatPercent(value);
                break;
        }
        return result;
    },

    stylesForCell: function(value, type)
    {
        return type + ((value < 0) ? " negative_value" : "");
    },

    createCell: function(value, style)
    {
        return "<div align='right' class='" + style + "'><nobr>" + value + "</nobr></div>";
    },

    noteStyles: function(id, groupingType)
    {
        if (groupingType == ReportGroupingLevel.TASK && !EVAReport.Store.getTask(id).estimated)
            return "smalltext";
        return "";
    },

    titleNoteCell: function(id, type, groupingType)
    {
        if (type == BaseTableDataConverter.RowType.TOTALS)
            return messageResource.getMessage("report.total");

        if (groupingType == ReportGroupingLevel.TASK && !EVAReport.Store.getTask(id).estimated)
            return messageResource.getEscapedMessage("report.budgeted_vs_actual.not_estimated_note");

        return "";
    },

    title: function(id, type, groupingType)
    {
        if (type == BaseTableDataConverter.RowType.TOTALS)
            return "";

        var store = this.store;
        switch (groupingType)
        {
            case ReportGroupingLevel.USER:
                return escapeHtmlSymbols(EVAReport.FormatUtils.formatUser(store.getUser(id)));
            case ReportGroupingLevel.PROJECT:
                return escapeHtmlSymbols(EVAReport.FormatUtils.formatProject(store.getProject(id), store.getGroupingPresentationFlags()));
            case ReportGroupingLevel.CUSTOMER:
                return escapeHtmlSymbols(store.getCustomer(id).name);
            case ReportGroupingLevel.PROJECT_NAME:
                return escapeHtmlSymbols(store.getProjectName(id).name);
            case ReportGroupingLevel.TASK:
                return escapeHtmlSymbols(EVAReport.FormatUtils.formatTask(store.getTask(id), store.getGroupingPresentationFlags()));
            case ReportGroupingLevel.TASK_NAME:
                return escapeHtmlSymbols(store.getTaskName(id).name);
            case ReportGroupingLevel.BILLING_TYPE:
                return escapeHtmlSymbols(store.getTypeOfWork(id).name);
        }
    }
});


EVAReport.Table.DataRow.CellType = {
    TIME: "time",
    PERCENT: "percent"
};