LeavesReport.FormatUtils = _.create(BaseReport.FormatUtils.prototype, {
    formatDateForHeader: function(date)
    {
        return this._dateForHeaderFormat.formatDate(date)
    },

    formatDateForDataRow: function(date)
    {
        return this._dateForDataRowFormat.formatDate(date)
    },

    formatWeekForDataRow: function(dateRange)
    {
        // look at TimeUtils.java#formatDateRange
        if (dateRange.from.getFullYear() == dateRange.to.getFullYear())
        {
            // check if interval contains only one day
            if (dateRange.from.getMonth() == dateRange.to.getMonth() && dateRange.from.getDate() == dateRange.to.getDate())
                return this._dateWithYearFormat.formatDate(dateRange.from);
            return this._weekForDataRowFormat.formatDate(dateRange.from) + " - " + this._dateWithYearFormat.formatDate(dateRange.to);
        }

        return this._dateWithYearFormat.formatDate(dateRange.from) + " - " + this._dateWithYearFormat.formatDate(dateRange.to);
    },

    formatMonthForDataRow: function(date)
    {
        return this._monthForDataRowFormat.formatDate(date)
    },

    formatBalance: function(balance, balanceType, defaultWorkDayDuration)
    {
        if (_.isNumber(balance))
        {
            if (balanceType == BalanceType.PTO && SystemSettings.showPTOInHours || balanceType == BalanceType.SICK && SystemSettings.showSickInHours)
                balance *= defaultWorkDayDuration / 60;
            return this._balanceNumberFormatter.format(balance);
        }
        else
            return "-";
    },

    formatChartType: function(chartType)
    {
        var chartTypeMessageKey;
        switch (chartType)
        {
            case APReportChartType.LEAVE_TIME:
                chartTypeMessageKey = "common.reports.chart_type_selector.leave_time";
                break;
            case APReportChartType.PTO_BALANCE:
                chartTypeMessageKey = "common.reports.chart_type_selector.pto_balance";
                break;
            case APReportChartType.SICK_BALANCE:
                chartTypeMessageKey = "common.reports.chart_type_selector.sick_balance";
                break;
        }

        return messageResource.getMessage(chartTypeMessageKey);
    },

    makeBalanceTemplateModel: function(balanceType, firstGroupingLevel, secondGroupingLevel, startDate, endDate)
    {
        var titleKey = balanceType == BalanceType.PTO ? "report.leaves.balance_column_name.pto" : "report.leaves.balance_column_name.sick_days";
        var unitKey = balanceType == BalanceType.PTO && SystemSettings.showPTOInHours || balanceType == BalanceType.SICK && SystemSettings.showSickInHours ? "common.pto_units.hours" : "common.pto_units.days";
        var firstLevelStartDateKey, firstLevelEndDateKey, secondLevelStartDateKey, secondLevelEndDateKey;
        switch (firstGroupingLevel)
        {
            case ReportGroupingLevel.DATE:
                firstLevelStartDateKey = "report.leaves.first_balance_subcolumn.dates";
                firstLevelEndDateKey = "report.leaves.second_balance_subcolumn.dates";
                break;
            case ReportGroupingLevel.WEEK:
                firstLevelStartDateKey = "report.leaves.first_balance_subcolumn.weeks";
                firstLevelEndDateKey = "report.leaves.second_balance_subcolumn.weeks";
                break;
            case ReportGroupingLevel.MONTH:
                firstLevelStartDateKey = "report.leaves.first_balance_subcolumn.months";
                firstLevelEndDateKey = "report.leaves.second_balance_subcolumn.months";
                break;
        }
        switch (secondGroupingLevel)
        {
            case ReportGroupingLevel.DATE:
                secondLevelStartDateKey = "report.leaves.start_of_the_day";
                secondLevelEndDateKey = "report.leaves.end_of_the_day";
                break;
            case ReportGroupingLevel.WEEK:
                secondLevelStartDateKey = "report.leaves.start_of_the_week";
                secondLevelEndDateKey = "report.leaves.end_of_the_week";
                break;
            case ReportGroupingLevel.MONTH:
                secondLevelStartDateKey = "report.leaves.start_of_the_month";
                secondLevelEndDateKey = "report.leaves.end_of_the_month";
                break;
        }

        return {
            title: messageResource.getMessage(titleKey),
            startDate: firstLevelStartDateKey ? messageResource.getMessage(firstLevelStartDateKey) : LeavesReport.FormatUtils.formatDateForHeader(startDate),
            endDate: firstLevelEndDateKey ? messageResource.getMessage(firstLevelEndDateKey) : LeavesReport.FormatUtils.formatDateForHeader(endDate),
            startDateDetails: secondLevelStartDateKey ? messageResource.getMessage(secondLevelStartDateKey) : "",
            endDateDetails: secondLevelEndDateKey ? messageResource.getMessage(secondLevelEndDateKey) : "",
            unit: messageResource.getMessage(unitKey)
        }
    }
});