LeavesReport.TableDataConverter = function(reportData, reportParams)
{
    BaseTableDataConverter.call(this, reportData, reportParams);
    this.hasPto = LeavesReportUtils.reportParamsWithPTO(this.reportParams);
    this.hasSick = LeavesReportUtils.reportParamsWithSick(this.reportParams);
};

LeavesReport.TableDataConverter.prototype = _.create(BaseTableDataConverter.prototype, {
    _createHeader: function()
    {
        return {
            type: BaseTableDataConverter.RowType.HEADER,
            leaveTypeIds: _.pluck(this.reportData["leaveSpent"], "leaveTypeId"),
            // TODO: aT and aP have different format of dates from some reason
            balancesRange: {startDate: this.reportParams.dateRangeSelection.fromDate, endDate: this.reportParams.dateRangeSelection.toDate},
            hasPto: this.hasPto,
            hasSick: this.hasSick
        }
    },

    _createDataRow: function(level, type, groupingType, leavesReportItem)
    {
        var dataRow = {
            type: type,
            total: leavesReportItem["totalLeaveSpent"],
            leaveTypeTotals: _.pluck(leavesReportItem["leaveSpent"], "leaveSpent")
        };

        var id = leavesReportItem["id"];
        var date = leavesReportItem["date"];
        var dateRange = leavesReportItem["dateRange"];
        var ptoBalance = leavesReportItem["ptoBalance"];
        var sickBalance = leavesReportItem["sickBalance"];

        var identity;
        switch (groupingType) {
            case ReportGroupingLevel.DATE:
            case ReportGroupingLevel.MONTH:
                identity = {date: Date.parseJSONDate(id["date"]), dayOff: id["dayOff"], corporateDayOff: id["corporateDayOff"]};
                break;
            case ReportGroupingLevel.WEEK:
                identity = {from: Date.parseJSONDate(id["from"]), to: Date.parseJSONDate(id["to"])};
                break;
            default:
                identity = id;
        }

        dataRow.identity = identity;

        if (this.hasPto)
            dataRow.ptoBalances = {start: ptoBalance ? ptoBalance["balanceForFirstColumn"] : null, end: ptoBalance ? ptoBalance["balanceForSecondColumn"] : null};
        if (this.hasSick)
            dataRow.sickBalances = {start: sickBalance ? sickBalance["balanceForFirstColumn"] : null, end: sickBalance ? sickBalance["balanceForSecondColumn"] : null};

        return dataRow;
    }
});