LeavesReport.PDFPreviewLightboxController = function(){};

LeavesReport.PDFPreviewLightboxController.prototype = _.create(PDFPreviewController.prototype, {
    
    init: function(lightBoxEl, titleEl, popupId, eventNamespace, productSettingsValidator, reportRightsValidator)
    {
        this.eventNamespace = eventNamespace;
        PDFPreviewController.prototype.init.call(this, lightBoxEl, titleEl, popupId);

        this.productSettingsValidator = productSettingsValidator;
        this.reportRightsValidator = reportRightsValidator;
        return this;
    },

    _initPdfSettingsPreview: function(elementsIdPrefix)
    {
        this.pdfSettingsPreview = new LeavesReport.PDFSettingsPreview($(this.lightBoxEl).find(".pdfPreviewContainer"), elementsIdPrefix, this.eventNamespace).init();
    },

    _subscribeForEvents: function()
    {
        var self = this;
        $(document).on(this.eventNamespace + ".pdfGenerator.start", function() { self.onGeneratorStart(); });
        $(this.getStore()).on("generatedPDFChange", function() { self.onGeneratePDFCompleted(); });
    },

    cancelAllExportingProcesses: function ()
    {
        this.pdfViewer.interruptPreview();
    },

    onShow: function()
    {
        this._setTitle(this.createFormattingStrategy(), this._getSelectedParameters());
    },

    createFormattingStrategy: function ()
    {
        return new LeavesReport.ParametersFormattingStrategy().init(this._getSelectorsData(), this.dateFormat, this.productSettingsValidator, this.reportRightsValidator);
    },

    onGeneratorStart: function()
    {
        this.disableDownloadPDFButton();
        this.pdfSettingsPreview.beforePreviewUpdate();
        this.pdfViewer.showMoire();
    },

    onDownloadPDFBtnClicked: function()
    {
        if (!this.downloadPDFBtnEnabled)
            return;

        $(document).trigger(this.eventNamespace + ".downloadPDF");
    },

    onGeneratePDFCompleted: function()
    {
        this.pdfViewer.showPDF(this.getStore().getGeneratedPDF());
        this.enableDownloadPDFButton();
    },
    
    getStore: function()
    {
        return LeavesReport.Store;
    },

    getGeneralStore: function()
    {
        return ATReportsStore;
    },

    _getSelectedParameters: function()
    {
        var reportParams = this.getStore().getReportParams();
        return {
            balanceQualifier: reportParams.balanceQualifier,
            columnsConfig: reportParams.columnsConfig,
            firstGroupingLevel: reportParams.firstGroupingLevel,
            dateParameters: {
                dateRangeSelection : reportParams.dateRangeSelection
            },
            userSelection: reportParams.userSelection,
            leaveTypeSelection: reportParams.leaveTypeSelection
        }
    },

    _getSelectorsData: function()
    {
        var store = this.getGeneralStore();
        return {
            users: store.getVisibleUsers(),
            userGroupsEnabled: store.getUserGroupsEnabled(),
            timeZoneGroupsEnabled: store.getTimeZoneGroupsEnabled(),
            userGroups: store.getUserGroups(),
            timeZoneGroups: store.getTimeZoneGroups(),
            leaveTypesMap: LeaveTypeStore.getLeaveTypes()
        }
    }
});