LeavesReport.Table.DataRow = function($containerEl, store, level, groupingLevel, identity, total, leaveTypeTotals, ptoBalances, sickBalances, timeFormat)
{
    TemplateView.call(this, $containerEl);
    this.store = store;
    this.level = level;
    this.groupingLevel = groupingLevel;
    this.identity = identity;
    this.total = total;
    this.leaveTypeTotals = leaveTypeTotals;
    this.ptoBalances = ptoBalances;
    this.sickBalances = sickBalances;
    this.timeFormat = timeFormat;
};

LeavesReport.Table.DataRow.prototype = _.create(TemplateView.prototype, {
    html: "<tr class='dataRow <%=rowClass%>'>" +
          "  <td class='<%=titleLevelClass%> titleCell'><%-title%><span class='corporateDayOffLabel'> <%=corporateDayOffLabel%></span></td>" +
          "  <td class='totalCell <%=levelClass%>'><%-total%></td>" +
          "</tr>",

    templateKey: "LeavesReport.Table.DataRow",

    afterRender: function()
    {
        _.each(this.leaveTypeTotals, function(leaveTypeTotal)
        {
            new LeavesReport.Table.TimeCell(this.$el, this.level, leaveTypeTotal, this.timeFormat).init();
        }, this);

        if (this.ptoBalances)
            this._addBalances(this.ptoBalances, BalanceType.PTO);
        if (this.sickBalances)
            this._addBalances(this.sickBalances, BalanceType.SICK);

        if (this.groupingLevel == ReportGroupingLevel.DATE)
        {
            this.$el.toggleClass("dayOff", this.identity.dayOff);
            this.$el.toggleClass("corporateDayOff", this.identity.corporateDayOff);
        }
    },

    templateModel: function()
    {
        return {
            rowClass: this.rowClass(),
            title: this._title(),
            total: LeavesReport.FormatUtils.formatTime(this.total, this.timeFormat),
            titleLevelClass: "reportrow" + this.level,
            levelClass: "reportrow" + this.level + "num"
        }
    },

    messageKeys: function()
    {
        return {
            corporateDayOffLabel: "common.corporate_day_off_text"
        }
    },

    rowClass: function()
    {
        // Abstract
    },

    _title: function()
    {
        switch (this.groupingLevel)
        {
            case ReportGroupingLevel.DATE:
                return LeavesReport.FormatUtils.formatDateForDataRow(this.identity.date);
            case ReportGroupingLevel.WEEK:
                return LeavesReport.FormatUtils.formatWeekForDataRow(this.identity);
            case ReportGroupingLevel.MONTH:
                return LeavesReport.FormatUtils.formatMonthForDataRow(this.identity.date);
            case ReportGroupingLevel.USER:
                return LeavesReport.FormatUtils.formatUser(this.store.getUser(this.identity));
            case ReportGroupingLevel.USER_GROUP:
                return LeavesReport.FormatUtils.formatUserGroup(this.store.getUserGroup(this.identity));
            case ReportGroupingLevel.TIME_ZONE_GROUP:
                return LeavesReport.FormatUtils.formatTimeZoneGroup(this.store.getTimeZoneGroup(this.identity));
        }
    },

    _addBalances: function(balances, balanceType)
    {
        if (this.leaveTypeTotals && this.leaveTypeTotals.length > 0)
            this.$el.append("<td class='separator'></td>");

        new LeavesReport.Table.BalanceCell(this.$el, this.store, this.level, balances.start, balanceType).init();
        new LeavesReport.Table.BalanceCell(this.$el, this.store, this.level, balances.end, balanceType).init();
    }
});