LeavesReport.Table = function($containerEl, store)
{
    TemplateView.call(this, $containerEl);
    this.store = store;
};

LeavesReport.Table.prototype = _.create(TemplateView.prototype, {
    html: "<table class='leavesReportTable display-for-print' cellspacing='0' cellpadding='0'></table>",

    updateOnInit: false,

    update: function()
    {
        var reportData = this.store.getReportData();
        var reportParams = this.store.getReportParams();
        var rows = new LeavesReport.TableDataConverter(reportData, reportParams).convert();

        this.cancelRendering();

        this.$el.empty();

        this.rowInsertProcessor = new ChunkIterator(rows, 20, function(row)
        {
            switch (row.type)
            {
                case BaseTableDataConverter.RowType.HEADER:
                    new LeavesReport.Table.HeaderRow(this.$el, reportParams.firstGroupingLevel, reportParams.secondGroupingLevel, reportParams.timeFormat, row.leaveTypeIds, row.balancesRange, row.hasPto, row.hasSick).init();
                    break;
                case BaseTableDataConverter.RowType.TOTALS:
                    new LeavesReport.Table.TotalsRow(this.$el, this.store, row.total, row.leaveTypeTotals, row.ptoBalances, row.sickBalances, reportParams.timeFormat).init();
                    break;
                case BaseTableDataConverter.RowType.FIRST_LEVEL:
                    new LeavesReport.Table.FirstLevelRow(this.$el, this.store, reportParams.firstGroupingLevel, row.identity, row.total, row.leaveTypeTotals, row.ptoBalances, row.sickBalances, reportParams.timeFormat).init();
                    break;
                case BaseTableDataConverter.RowType.SECOND_LEVEL:
                    new LeavesReport.Table.SecondLevelRow(this.$el, this.store, reportParams.secondGroupingLevel, row.identity, row.total, row.leaveTypeTotals, row.ptoBalances, row.sickBalances, reportParams.timeFormat).init();
                    break;
            }
        }, this).start();

        this.$el.toggleClass("noSecondGroupingLevel", !reportParams.secondGroupingLevel || reportParams.secondGroupingLevel == ReportGroupingLevel.NONE);
        this.$el.toggleClass("noLeaves", !LeavesReportUtils.reportParamsWithLeaves(reportParams));
        this.$el.toggleClass("noPto", !LeavesReportUtils.reportParamsWithPTO(reportParams));
        this.$el.toggleClass("noSick", !LeavesReportUtils.reportParamsWithSick(reportParams));
    },

    cancelRendering: function()
    {
        if (this.rowInsertProcessor)
            this.rowInsertProcessor.pause();
    }
});