function OvertimeCSVJsonConverter() {}

OvertimeCSVJsonConverter.prototype = _.create(BaseCSVJsonConverter.prototype, {
    init: function(store, generalStore)
    {
        BaseCSVJsonConverter.prototype.init.call(this);
        this.store = store;
        this.generalStore = generalStore;

        this.formatUtils = OvertimeReport.FormatUtils;

        return this;
    },

    addGroupingInfo: function(groupingLevel, identity, line)
    {
        if (groupingLevel == ReportGroupingLevel.USER)
            line["user info"] = UserNameFormat.formatLFM(identity.firstName, identity.lastName, identity.middleInitial);
        else
            GroupingUtils.addGroupingInfo(groupingLevel, identity, line);
    },

    addDataToTable: function(data, line)
    {
        var config = this.store.getReportParams();
        if (config.columnsConfig != OvertimeReport.ColumnsConfig.OVERTIME_ONLY)
        {
            line["scheduledTimeBalance"] = this.formatTime(data.scheduled);
            line["reportedTimeBalance"] = this.formatTime(data.reported);
            line["timeBalanceDifference"] = this.formatTime(data.difference);
        }

        if (config.columnsConfig != OvertimeReport.ColumnsConfig.TIME_BALANCE_ONLY)
        {
            line["reportedOvertime"] = this.formatTime(data.reportedOvertime);
            line["autoOvertime"] = this.formatTime(data.autoCalculatedOvertime);
        }
    },

    addOtherColumnNames: function(columnNames, columns, config, data)
    {
        if (config.columnsConfig != OvertimeReport.ColumnsConfig.OVERTIME_ONLY)
        {
            columns.push("scheduledTimeBalance");
            columnNames["scheduledTimeBalance"] = messageResource.getMessage("report.overtime.csv.time_balance.scheduled");
            
            columns.push("reportedTimeBalance");
            columnNames["reportedTimeBalance"] = messageResource.getMessage("report.overtime.csv.time_balance.reported");
            
            columns.push("timeBalanceDifference");
            columnNames["timeBalanceDifference"] = messageResource.getMessage("report.overtime.csv.time_balance.difference");
        }

        if (config.columnsConfig != OvertimeReport.ColumnsConfig.TIME_BALANCE_ONLY)
        {
            columns.push("reportedOvertime");
            columnNames["reportedOvertime"] = messageResource.getMessage("report.overtime.csv.reported_by_user");

            columns.push("autoOvertime");
            columnNames["autoOvertime"] = messageResource.getMessage("report.overtime.csv.auto_calculated");
        }
    },

    getIdentity: function(data, grouping)
    {
        switch (grouping)
        {
            case ReportGroupingLevel.USER:
                return this.store.getUser([data.id]);
            case ReportGroupingLevel.USER_GROUP:
                return this.store.getUserGroup([data.id]);
            case ReportGroupingLevel.TIME_ZONE_GROUP:
                return this.store.getTimeZoneGroup([data.id]);
            
        }
        return data.id;
    },

    processParamsBuilder: function(paramsBuilder, params, config)
    {
        var reportParamsStringBuilder = new ReportParamsStringBuilder().init({
                                                                                 users: this.generalStore.getVisibleUsers(),
                                                                                 userGroups: this.generalStore.getUserGroups(),
                                                                                 timeZoneGroups: this.generalStore.getTimeZoneGroups()
                                                                             });
        var reportParameters = params;
        
        paramsBuilder.addDateRangeParam(reportParameters.dateRangeSelection, reportParamsStringBuilder);
        paramsBuilder.addUsersParam(reportParameters.userSelection, reportParamsStringBuilder);
    },

    formatTime: function(time)
    {
        if (time == null)
            return "";
        return ConverterFormatUtils.numberFormatter.format(time / 60);
    },

    formatVarianceTime: function(time)
    {
        if (time == null)
            return "";
        return ConverterFormatUtils.varianceNumberFormatter.format(time / 60);
    },

    generateCSVName: function(reportParams)
    {
        var fileNameParts = [];
        var dateFormatter = new DateFormat(SystemSettings.dateFormat);
        fileNameParts.push(dateFormatter.formatDate(reportParams.dateRangeSelection.fromDate));
        fileNameParts.push(dateFormatter.formatDate(reportParams.dateRangeSelection.toDate));

        var key = EnabledFeatures.OVR ? "overtime_report.csv.file_name.time_and_overtime" : "overtime_report.csv.file_name.time_only";
        var filename = messageResource.getMessage(key, fileNameParts);
        if (isOldIE())
        {
            filename = filename.replace(/\//g, "_")
        }
        return filename;
    },

    generateTableHeader: function(data, config, columnNames, columns)
    {
        for (var i = 0; i < this.groupingLevels.length; ++i)
        {
            if (this.groupingLevels[i] != ReportGroupingLevel.USER)
                GroupingUtils.addGroupingColumnName(this.groupingLevels[i], columnNames, columns);
            else
            {
                columnNames["user info"] = "User";
                columns.push("user info");
            }
        }
        this.addOtherColumnNames(columnNames, columns, config, data);
    },

    convertTotal: function(config, data)
    {
        // add total info from root
        var columnNames = {};
        var columns = [];
        this.generateTableHeader(data, config, columnNames, columns);
        var line = {};
        this.addDataToTable(data, line, config);

        // changing name for total
        columns[0] = "Total";
        line["Total"] = "Total:";
        // no columnNames because we don't need headers
        // {} in field table for blank line
        return {
            columns: columns,
            table: [{}, line]
        };
    }
});
