OvertimeReport.Table = function($containerEl, store)
{
    TemplateView.call(this, $containerEl);
    this.store = store;
};

OvertimeReport.Table.prototype = _.create(TemplateView.prototype, {
    html: "<table class='overtimeReportTable display-for-print' cellspacing='0' cellpadding='0'></table>",

    updateOnInit: false,

    update: function()
    {
        var reportData = this.store.getReportData();
        var reportParams = this.store.getReportParams();
        var rows = new OvertimeReport.TableDataConverter(reportData, reportParams).convert();
        OvertimeReport.Store.setConvertedData(rows);
        
        this.cancelRendering();

        this.$el.empty();

        this.rowInsertProcessor = new ChunkIterator(rows, 20, function(row)
        {
            switch (row.type)
            {
                case BaseTableDataConverter.RowType.HEADER:
                    new OvertimeReport.Table.HeaderRow(this.$el, reportParams.firstGroupingLevel, reportParams.secondGroupingLevel, row.hasTimeBalance, row.hasOvertime).init();
                    break;
                case BaseTableDataConverter.RowType.TOTALS:
                    new OvertimeReport.Table.TotalsRow(this.$el, this.store, row, reportParams.timeFormat).init();
                    break;
                case BaseTableDataConverter.RowType.FIRST_LEVEL:
                    new OvertimeReport.Table.FirstLevelRow(this.$el, this.store, reportParams.firstGroupingLevel, row, reportParams.timeFormat).init();
                    break;
                case BaseTableDataConverter.RowType.SECOND_LEVEL:
                    new OvertimeReport.Table.SecondLevelRow(this.$el, this.store, reportParams.secondGroupingLevel, row, reportParams.timeFormat).init();
                    break;
            }
        }, this).start();

        this.$el.toggleClass("noSecondGroupingLevel", !reportParams.secondGroupingLevel || reportParams.secondGroupingLevel === ReportGroupingLevel.NONE);
        this.$el.toggleClass("onlyOneColumnGroup", reportParams.columnsConfig !== OvertimeReport.ColumnsConfig.TIME_BALANCE_AND_OVERTIME)
    },

    cancelRendering: function()
    {
        if (this.rowInsertProcessor)
            this.rowInsertProcessor.pause();
    }
});