ReportPreviewUtils = {};
ReportPreviewUtils.getCSVTimeValue = function()
{
    return "1" + SystemSettings.decimalSeparator + "2345";
};

ReportPreviewUtils.getHTMLTimeValue = function( timeFormat )
{
    return messageResource.getMessage( timeFormat == TimeFormat.TIME ? "common.hh_mm" : "common.##_##" );
};

ReportPreviewUtils.countNotEmptyLevels = function( groupingLevels )
{
    var count = 0;
    for( var i = 0; i < groupingLevels.length; i++ )
        if( groupingLevels[i] != ReportGroupingLevel.NONE )
            count++;
    return count;
};

ReportPreviewUtils.filterNotEmptyLevels = function( groupingLevels )
{
    var result = [];
    for( var i = 0; i < groupingLevels.length; i++ )
        if( groupingLevels[i] != ReportGroupingLevel.NONE )
            result.push(groupingLevels[i]);
    return result;
};

/**
 * Check if specific level is selected
 * @param levelValue checked value
 * @param groupingLevels selected levels
 * @returns {boolean} true if levels is selected, false otherwise
 */
ReportPreviewUtils.checkGroupingLevel = function(levelValue, groupingLevels) {
    for(var i = 0; i < groupingLevels.length; i++){
        if (levelValue == groupingLevels[i]) {
            return true;
        }
    }
    return false;
};

ReportPreviewUtils.createCell = function( innerHTML, className, colSpan, rowSpan )
{
    var td = document.createElement( "td" );
    td.innerHTML = innerHTML;
    if( className )
        td.className = className;
    if( colSpan )
        td.colSpan = colSpan;
    if( rowSpan )
        td.rowSpan = rowSpan;
    return td;
};

ReportPreviewUtils.addCreatedCell = function ( row, cell )
{
    row.appendChild( cell );
};

ReportPreviewUtils.addCell = function( row, notEscapedContent, className, colSpan, rowSpan )
{
    row.appendChild( ReportPreviewUtils.createCell( escapeHtmlSymbols( notEscapedContent ), className, colSpan, rowSpan ) );
};

ReportPreviewUtils.createCellWithFloatingContent = function( notEscapedContent, className, colSpan, rowSpan )
{
    var cl = "floatingContainer";
    if( className )
        cl += " " + className;
    var innerHTML = "<div class='floatingContent'>" + escapeHtmlSymbols( notEscapedContent ) + "</div>";
    return ReportPreviewUtils.createCell( innerHTML, cl, colSpan, rowSpan );
};

ReportPreviewUtils.addCellWithFloatingContent = function( tr, notEscapedContent )
{
    tr.appendChild( ReportPreviewUtils.createCellWithFloatingContent( notEscapedContent ) );
};

ReportPreviewUtils.addCSVTimeValueCell = function( tr )
{
    ReportPreviewUtils.addCell( tr, ReportPreviewUtils.getCSVTimeValue() );
};

ReportPreviewUtils.addHTMLTimeValueCell = function ( tr, timeFormat, className )
{
    ReportPreviewUtils.addCell( tr, ReportPreviewUtils.getHTMLTimeValue( timeFormat), "timeValueCell " + ( className ? className : "" ) );
};

ReportPreviewUtils.addEmptyCell = function( tr )
{
    ReportPreviewUtils.addCell( tr, "" );
};

ReportPreviewUtils.addEmptyCells = function( tr, num )
{
    for( var i = 0; i < num; i++ )
        ReportPreviewUtils.addEmptyCell( tr );
};

ReportPreviewUtils.addSheetHeaderRow = function( tBody, totalColumnsNum )
{
    var tr = document.createElement("tr");
    tr.className = "csvPreviewHeaderRow";
    tr.appendChild( ReportPreviewUtils.createCell( "", "csvTolLeftCell" ));
    for( var i = 0; i < totalColumnsNum; i++ )
    {
        var colHeader = String.fromCharCode( 65 + i );
        tr.appendChild( ReportPreviewUtils.createCell( colHeader, colHeader ));
    }
    tBody.appendChild(tr);
};

ReportPreviewUtils.getLevelPreviewName = function( groupingLevel )
{
    switch (groupingLevel)
    {
        case ReportGroupingLevel.BILLING_TYPE: return messageResource.getMessage("report.billing_type");
        case ReportGroupingLevel.CUSTOMER: return SystemSettings.firstLevelNamingSingular;
        case ReportGroupingLevel.PROJECT: return SystemSettings.secondLevelNamingSingular;
        case ReportGroupingLevel.PROJECT_NAME: return SystemSettings.secondLevelNamingSingular;
        case ReportGroupingLevel.TASK: return SystemSettings.thirdLevelNamingSingular;
        case ReportGroupingLevel.TASK_NAME: return SystemSettings.thirdLevelNamingSingular;

        case ReportGroupingLevel.USER: return  messageResource.getMessage("report.user");
        case ReportGroupingLevel.USER_GROUP: return SystemSettings.userGroupNamingSingular;
        case ReportGroupingLevel.TIME_ZONE_GROUP: return messageResource.getMessage("common.Time_Zone_Group");
        case ReportGroupingLevel.USER_RATE: return messageResource.getMessage("report.cost.rate"); // TODO: cost only?

        case ReportGroupingLevel.TT_COMMENTS: return messageResource.getMessage("reports.comments_report.report.comments");

        /* Combined levels should be used separately
        case ReportGroupingLevel.TASK_AND_TT_COMMENTS: ;
        case ReportGroupingLevel.PROJECT_AND_TT_COMMENTS: ;
        case ReportGroupingLevel.CUSTOMER_AND_TT_COMMENTS: ;
        */

        case ReportGroupingLevel.DATE: return messageResource.getMessage("report.date");
        case ReportGroupingLevel.WEEK: return messageResource.getMessage("report.week");
        case ReportGroupingLevel.MONTH: return messageResource.getMessage("report.month");

        default: return "None";
    }
};
ReportPreviewUtils.addLevelCellWithUserExtendedInCSV = function(row, groupingLevel){
    groupingLevel == ReportGroupingLevel.USER ?
    ReportPreviewUtils.addHeaderUserNamesCells( row, true ):
    ReportPreviewUtils.addCell( row, ReportPreviewUtils.getLevelPreviewName( groupingLevel ));
};
ReportPreviewUtils.getGroupingOptionName = function( groupingLevel )
{
    switch (groupingLevel)
    {
        case ReportGroupingLevel.BILLING_TYPE: return messageResource.getMessage("report.common.billing_types");
        case ReportGroupingLevel.CUSTOMER: return SystemSettings.firstLevelNamingPlural;
        case ReportGroupingLevel.PROJECT: return SystemSettings.secondLevelNamingPlural;
        case ReportGroupingLevel.PROJECT_NAME: return messageResource.getMessage( "report.projects_joined_by_name" );
        case ReportGroupingLevel.TASK: return SystemSettings.thirdLevelNamingPlural;
        case ReportGroupingLevel.TASK_NAME: return messageResource.getMessage( "report.tasks_joined_by_name" );

        case ReportGroupingLevel.USER: return  messageResource.getMessage("report.users");
        case ReportGroupingLevel.USER_GROUP: return SystemSettings.userGroupNamingPlural;
        case ReportGroupingLevel.TIME_ZONE_GROUP: return messageResource.getMessage("common.Time_Zone_Groups");

        case ReportGroupingLevel.TT_COMMENTS: return messageResource.getMessage("reports.comments_report.report.comments");
        case ReportGroupingLevel.TASK_AND_TT_COMMENTS: return messageResource.getMessage("report.tasks.and_tt_comments");
        case ReportGroupingLevel.PROJECT_AND_TT_COMMENTS: return messageResource.getMessage("report.projects.and_tt_comments");
        case ReportGroupingLevel.CUSTOMER_AND_TT_COMMENTS: return messageResource.getMessage("report.customers.and_tt_comments");

        case ReportGroupingLevel.DATE: return messageResource.getMessage("report.dates");
        case ReportGroupingLevel.WEEK: return messageResource.getMessage("report.weeks");
        case ReportGroupingLevel.MONTH: return messageResource.getMessage("report.months");

        default: return "None";
    }
};

ReportPreviewUtils.makeGroupingHeader = function( groupingLevelsArray )
{
    var groupingHeader = "";
    var oneLevelAlreadyExists = false;
    for ( var i = 0; i < groupingLevelsArray.length; i++ )
    {
        if ( groupingLevelsArray[i] != ReportGroupingLevel.NONE )
        {
            if (oneLevelAlreadyExists)
                groupingHeader += " / ";
            groupingHeader += ReportPreviewUtils.getGroupingOptionName( groupingLevelsArray[i] );
            oneLevelAlreadyExists = true;
        }
    }
    return groupingHeader;
};

ReportPreviewUtils.getCountUserCSVValues = function(){
    return 4;
};

ReportPreviewUtils.addHeaderUserNamesCells = function(tr, isNeedMI){
    this.addCell(tr, messageResource.getMessage("user.last_name"));
    this.addCell(tr, messageResource.getMessage("user.first_name"));
    if(isNeedMI)
        this.addCell(tr, messageResource.getMessage("user.mi"));
    else this.addEmptyCell(tr);
    this.addCell(tr, messageResource.getMessage("user.username"));
};

ReportPreviewUtils.isLevelCanBeAddedMoreThanOnce = function ( level, higherGroupingLevelsArray ) {
    switch (level) {
        case ReportGroupingLevel.CUSTOMER:
            return ReportPreviewUtils.levelsArrayContainsNoneOfLevels(
                    higherGroupingLevelsArray,
                    [ReportGroupingLevel.PROJECT, ReportGroupingLevel.TASK] );
        case ReportGroupingLevel.CUSTOMER_AND_TT_COMMENTS:
            return ReportPreviewUtils.levelsArrayContainsNoneOfLevels(
                    higherGroupingLevelsArray,
                    [ReportGroupingLevel.CUSTOMER, ReportGroupingLevel.PROJECT, ReportGroupingLevel.TASK] );
        case ReportGroupingLevel.PROJECT:
            return ReportPreviewUtils.levelsArrayContainsNoneOfLevels(
                    higherGroupingLevelsArray,
                    [ReportGroupingLevel.TASK] );
        case ReportGroupingLevel.PROJECT_NAME:
        case ReportGroupingLevel.PROJECT_AND_TT_COMMENTS:
            return ReportPreviewUtils.levelsArrayContainsNoneOfLevels(
                    higherGroupingLevelsArray,
                    [ReportGroupingLevel.TASK, ReportGroupingLevel.PROJECT] );
        case ReportGroupingLevel.TASK_NAME:
        case ReportGroupingLevel.TASK_AND_TT_COMMENTS:
            return ReportPreviewUtils.levelsArrayContainsNoneOfLevels(
                    higherGroupingLevelsArray,
                    [ReportGroupingLevel.TASK] );
        case ReportGroupingLevel.USER_GROUP:
            return ReportPreviewUtils.levelsArrayContainsNoneOfLevels(
                    higherGroupingLevelsArray,
                    [ReportGroupingLevel.USER] );
        case ReportGroupingLevel.BILLING_TYPE:
            return ReportPreviewUtils.levelsArrayContainsNoneOfLevels(
                    higherGroupingLevelsArray,
                    [ReportGroupingLevel.TASK] );
        case ReportGroupingLevel.TASK:
            return $.inArray(ReportGroupingLevel.TASK_NAME, higherGroupingLevelsArray) == -1 ||
                   $.inArray(ReportGroupingLevel.PROJECT, higherGroupingLevelsArray) == -1;
        default:
            return true;
    }
};

ReportPreviewUtils.levelsArrayContainsNoneOfLevels = function ( levelsArray, levelsForSearching )
{
    for (var i = 0; i < levelsArray.length; i++)
    {
        for( var j = 0; j < levelsForSearching.length; j++ )
            if( levelsArray[i] == levelsForSearching[j] )
                return false;
    }

    return true;
};