function CostCSVPreviewGenerator() {}

CostCSVPreviewGenerator.prototype.init = function()
{
    return this;
};

CostCSVPreviewGenerator.prototype.generatePreviewTable = function( reportParameters, reportParamsStringBuilder )
{
    var numOfGroupingColumns = ReportPreviewUtils.countNotEmptyLevels( [
                                                                           reportParameters.firstGroupingLevel,
                                                                           reportParameters.secondGroupingLevel,
                                                                           reportParameters.thirdGroupingLevel
                                                                       ] );

    var columnsStrategy = this.getColumnsStrategy( reportParameters );
    var totalColumnsNum = numOfGroupingColumns + columnsStrategy.getNumOfDataColumns();
    var totalRowNum = 17;


    var table = document.createElement( "table" );
    table.className = "reportCsvPreviewTable cost";
    var tBody = document.createElement( "tbody" );
    table.appendChild( tBody );

    ReportPreviewUtils.addSheetHeaderRow( tBody, totalColumnsNum );

    var paramsBuilder = new CSVParametersBuilder().init();
    paramsBuilder.addDateRangeParam( reportParameters.dateRangeSelection, reportParamsStringBuilder );
    paramsBuilder.addCPParam( reportParameters.customersProjectsSelection, reportParamsStringBuilder );
    paramsBuilder.addUsersParam(reportParameters.userSelection, reportParamsStringBuilder);

    var parametersSection = new CSVParametersSection().init( paramsBuilder, totalColumnsNum );

    parametersSection.addRows( tBody );

    var headerSection = new CostCSVPreviewHeaderSection().init( columnsStrategy,
                                                                reportParameters.firstGroupingLevel,
                                                                reportParameters.secondGroupingLevel,
                                                                reportParameters.thirdGroupingLevel,
                                                                numOfGroupingColumns );

    var groupingSection = new CostCSVPreviewGroupingSection().init(
                                                                reportParameters.firstGroupingLevel,
                                                                reportParameters.secondGroupingLevel,
                                                                reportParameters.thirdGroupingLevel,
                                                                numOfGroupingColumns,
                                                                columnsStrategy.getNumOfDataColumns(),
                                                                5 );



    var emptySection = new CSVPreviewEmptySection().init(1, totalColumnsNum);

    headerSection.addRows( tBody );
    groupingSection.addRows( tBody);
    emptySection.addRows(tBody);
    if (reportParameters.showLeaveTimeCosts || reportParameters.showOvertimeAddition )
    {
        new CSVPreviewCostProfitLossAdditionalSection().init(
                columnsStrategy.getNumOfDataColumns(),
                numOfGroupingColumns,
                reportParameters.showOvertimeAddition,
                reportParameters.showLeaveTimeCosts
        ).addRows( tBody );
    }
    this.addEmptySectionIfNeeded(totalRowNum, tBody, totalColumnsNum);
    return table;
};

CostCSVPreviewGenerator.prototype.getColumnsStrategy = function( reportParameters )
{
    if( reportParameters.columnsConfig == ReportColumnsFormatCostOfWork.USERS_ONLY )
    {
        if( reportParameters.showSpentTime || reportParameters.showUserRates )
            return new CostCSVPreviewTwoLevelsUsersOnlyColumnsStrategy().init(
                    reportParameters.showSpentTime,
                    reportParameters.showAverageRates,
                    reportParameters.showUserRates
            );
        else
            return new CostCSVPreviewOneLevelsColumnsStrategy().init(reportParameters.showAverageRates,"report.user");
    }
    else if( reportParameters.columnsConfig == ReportColumnsFormatCostOfWork.GROUPS_ONLY )
    {
        if( reportParameters.showSpentTime || reportParameters.showAverageRates )
            return new CostCSVPreviewTwoLevelsGroupsOnlyColumnsStrategy().init(
                    reportParameters.showSpentTime,
                    reportParameters.showAverageRates,
                    reportParameters.showUserRates
            );
        else
            return new CostCSVPreviewOneLevelsColumnsStrategy().init(reportParameters.showAverageRates, "user.group");

    }
    else
    {
        if( reportParameters.showSpentTime || reportParameters.showUserRates ) {

            return new CostCSVPreviewUserByGroupsThreeLevels().init(
                reportParameters.showSpentTime,
                reportParameters.showAverageRates,
                reportParameters.showUserRates,
                reportParameters.showSubtotalsByDepartments
            );
        }
        else
            return new CostCSVPreviewUserByGroupsOnlyColumnsStrategy().init(
                reportParameters.showAverageRates,
                reportParameters.showSubtotalsByDepartments
            );
    }
};

CostCSVPreviewGenerator.prototype.addEmptySectionIfNeeded = function ( totalRowsNum, tBody, totalColumnsNum )
{
    var numOfEmptyRows = totalRowsNum + 1 - tBody.childNodes.length;
    if ( numOfEmptyRows > 0 )
        new CSVPreviewEmptySection().init( numOfEmptyRows, totalColumnsNum ).addRows( tBody );
};
