function EstimateVsActualCSVPreviewGenerator() {}

EstimateVsActualCSVPreviewGenerator.prototype.init = function(){
    return this;
};

EstimateVsActualCSVPreviewGenerator.prototype.generatePreviewTable = function( reportParameters, reportParamsStringBuilder) 
{
    var taskParams = {
        tasksConfig: reportParameters.tasksConfig,
        includeNotEstimated: !!reportParameters.includeNotEstimated,
        includeNotStarted: !!reportParameters.includeNotStarted
    };
    
    var groupingLevelsArray = [
        reportParameters.firstGroupingLevel,
        reportParameters.secondGroupingLevel,
        reportParameters.thirdGroupingLevel,
        reportParameters.fourthGroupingLevel
    ];
    
    var filteredLevelsArray = ReportPreviewUtils.filterNotEmptyLevels(groupingLevelsArray);

    var table = document.createElement( "table" );
    table.className = "reportCsvPreviewTable eva";
    var tBody = document.createElement( "tbody" );
    table.appendChild( tBody );
    
    var columnsStrategy = this.createPreviewStrategy(taskParams);
    var totalColumnsNum = filteredLevelsArray.length + 4 + columnsStrategy.getNumOfDataColumns(); // total spent, est time, variance mh, variance %

    ReportPreviewUtils.addSheetHeaderRow( tBody, totalColumnsNum );

    var header = new EstimateVsActualCSVPreviewHeader().init(filteredLevelsArray, columnsStrategy);
    var dataGroupingSection = new EstimateVsActualCSVPreviewGroupingSection().init(filteredLevelsArray, columnsStrategy);
    var paramsBuilder = new CSVParametersBuilder().init();
    
    paramsBuilder.addCPParam( reportParameters.customersProjectsSelection, reportParamsStringBuilder );
    paramsBuilder.addEvATasksParam( taskParams, reportParameters.dateSelection, reportParamsStringBuilder );
    this.addDateParam( paramsBuilder, reportParameters.dateSelection, reportParamsStringBuilder );

    var parametersSection = new CSVParametersSection().init( paramsBuilder, totalColumnsNum );

    parametersSection.addRows( tBody );
    header.addRows( tBody );
    dataGroupingSection.addRows( tBody );

    return table;
};

EstimateVsActualCSVPreviewGenerator.prototype.addDateParam = function( paramsBuilder, dateSelection, reportParamsStringBuilder ) {
    paramsBuilder.parameters.push( [messageResource.getMessage("report.budgeted_vs_actual.date_label"),
                                    reportParamsStringBuilder.getEvaDateCSVString( dateSelection )] );
};

EstimateVsActualCSVPreviewGenerator.prototype.createPreviewStrategy = function ( taskParams )
{
    if ( taskParams.includeNotEstimated )
        return new AllTasksEvACSVPreviewColumnStrategy();

    return new TasksWithEstimatesEvACSVPreviewColumnStrategy();
};


// ---------------------------

function AllTasksEvACSVPreviewColumnStrategy () {}

AllTasksEvACSVPreviewColumnStrategy.prototype = new CSVPreviewSection();

AllTasksEvACSVPreviewColumnStrategy.prototype.getNumOfDataColumns = function () {
    return 2; // total for not est, total for est
};

AllTasksEvACSVPreviewColumnStrategy.prototype.addDataColumns = function ( row )
{
    this.addCell( row, this.message("report.budgeted_vs_actual.time_for_not_estimated_tasks") );
    this.addCell( row, this.message("report.budgeted_vs_actual.time_for_estimated_tasks") );
};

AllTasksEvACSVPreviewColumnStrategy.prototype.addCellValues = function ( row, groupingLevel )
{
    if ( groupingLevel == ReportGroupingLevel.USER  || groupingLevel == ReportGroupingLevel.TASK )
        this.addCell(row, "");
    else 
        this.addTimeValueCell(row);

    this.addTimeValueCell(row);
};


// ---------------------------

function TasksWithEstimatesEvACSVPreviewColumnStrategy () {}

TasksWithEstimatesEvACSVPreviewColumnStrategy.prototype = new CSVPreviewSection();

TasksWithEstimatesEvACSVPreviewColumnStrategy.prototype.getNumOfDataColumns = function () {
    return 0;
};

TasksWithEstimatesEvACSVPreviewColumnStrategy.prototype.addDataColumns = function () { 
    // do nothing
};
TasksWithEstimatesEvACSVPreviewColumnStrategy.prototype.addCellValues = function () {
    // do nothing
};


