function LeaveCSVPreviewGenerator() {}
LeaveCSVPreviewGenerator.prototype.init = function(){
    return this;
};

LeaveCSVPreviewGenerator.NUM_OF_DATA_COLUMNS = 10; // (o/u reported by user, o/u auto calculated, working time, leave time)

LeaveCSVPreviewGenerator.prototype.generatePreviewTable = function( reportParameters, reportParamsStringBuilder) {

    var numOfGroupingColumns = ReportPreviewUtils.countNotEmptyLevels( [
                                                                           reportParameters.firstGroupingLevel,
                                                                           reportParameters.secondGroupingLevel
                                                                       ] );
    if (reportParameters.firstGroupingLevel == ReportGroupingLevel.USER || reportParameters.secondGroupingLevel == ReportGroupingLevel.USER)
        numOfGroupingColumns += ReportPreviewUtils.getCountUserCSVValues() - 1;
    var columnsStrategy = this.createPreviewStrategy( reportParameters );
    var totalColumnsNum = numOfGroupingColumns + columnsStrategy.getNumOfDataColumns();
    var header = new LeaveCSVPreviewHeader().init(
            reportParameters.firstGroupingLevel,
            reportParameters.secondGroupingLevel,
            columnsStrategy,
            numOfGroupingColumns,
            reportParameters.columnsConfig
    );

    var dataGroupingSection = new LeaveCSVPreviewGroupingSection().init(
            reportParameters.firstGroupingLevel,
            reportParameters.secondGroupingLevel,
            columnsStrategy,
            numOfGroupingColumns,
            columnsStrategy.getNumOfDataColumns()
    );

    var table = document.createElement( "table" );
    table.className = "reportCsvPreviewTable leave";
    var tBody = document.createElement( "tbody" );
    table.appendChild( tBody );

    ReportPreviewUtils.addSheetHeaderRow( tBody, totalColumnsNum );

    var paramsBuilder = new CSVParametersBuilder().init();
    paramsBuilder.addDateRangeParam( reportParameters.dateRangeSelection, reportParamsStringBuilder );
    paramsBuilder.addUsersParam( reportParameters.userSelection, reportParamsStringBuilder );
    paramsBuilder.addLeavesParam( reportParameters.leaveTypeSelection, reportParamsStringBuilder );

    var parametersSection = new CSVParametersSection().init( paramsBuilder, totalColumnsNum );

    parametersSection.addRows( tBody );
    header.addRows( tBody );
    dataGroupingSection.addRows(tBody);
    return table;
};


LeaveCSVPreviewGenerator.prototype.createPreviewStrategy = function ( reportParameters )
{
    switch ( reportParameters.columnsConfig )
    {
        case LeaveTypeColumnConfig.LEAVE_TIME_ONLY:
            return new LeaveCSVPreviewLeaveTimeOnlyStrategy().init();
            break;
        case LeaveTypeColumnConfig.BALANCES_ONLY:
            return new LeaveCSVPreviewBalancesOnlyStrategy().init(reportParameters.balanceQualifier);
            break;
        case LeaveTypeColumnConfig.LEAVE_TIME_AND_BALANCES:
            return new LeaveCSVPreviewTimeAndBalancesStrategy().init(reportParameters.balanceQualifier);
            break;
        }
};
