function StaffPerformanceCSVPreviewGenerator() {}
StaffPerformanceCSVPreviewGenerator.prototype.init = function ()
{
    return this;
};

StaffPerformanceCSVPreviewGenerator.prototype.generatePreviewTable = function ( reportParameters, reportParamsStringBuilder )
{
    var numOfGroupingColumns = ReportPreviewUtils.countNotEmptyLevels( [
                                                                     reportParameters.firstGroupingLevel,
                                                                     reportParameters.secondGroupingLevel,
                                                                     reportParameters.thirdGroupingLevel
                                                                 ] );

    var numOfDataColumns = numOfGroupingColumns == 1 ? 5 : 4;

    var groupMessageProvider = this.getGroupMessageProvider( reportParameters.columnsConfig );

    var columnsStrategy = new StaffPerformanceCSVPreviewColumnsStrategy().init( reportParameters.columnsConfig,
                                                                                numOfDataColumns,
                                                                                groupMessageProvider);

    var header = new StaffPerformanceCSVPreviewHeader().init(
            reportParameters.firstGroupingLevel,
            reportParameters.secondGroupingLevel,
            reportParameters.thirdGroupingLevel,
            columnsStrategy,
            reportParameters.columnsConfig,
            numOfGroupingColumns,
            groupMessageProvider
    );

    var dataGroupingSection = new StaffPerformanceCSVPreviewGroupingSection().init(
            reportParameters.firstGroupingLevel,
            reportParameters.secondGroupingLevel,
            reportParameters.thirdGroupingLevel,
            columnsStrategy,
            header.hasGroupingRow() ? 2 : 3
    );


    var table = document.createElement( "table" );
    table.className = "reportCsvPreviewTable staffPerfomance";
    var tBody = document.createElement( "tbody" );
    table.appendChild( tBody );

    var totalColumnsNum = numOfGroupingColumns + numOfDataColumns;
    ReportPreviewUtils.addSheetHeaderRow( tBody, totalColumnsNum );

    var paramsBuilder = new CSVParametersBuilder().init();
    paramsBuilder.addDateRangeParam( reportParameters.dateRangeSelection, reportParamsStringBuilder );
    paramsBuilder.addCPParam( reportParameters.customersProjectsSelection, reportParamsStringBuilder );
    paramsBuilder.addUsersParam( reportParameters.userSelection, reportParamsStringBuilder );
    if ( EnabledFeatures.TTA )
        paramsBuilder.addTimeTrackStatusParam( reportParameters.timeApprovalStatus );

    var parametersSection = new CSVParametersSection().init( paramsBuilder, totalColumnsNum );

    parametersSection.addRows( tBody );
    header.addRows( tBody );
    dataGroupingSection.addRows( tBody );

    if( reportParameters.showLeaveTime )
        new StaffPerformanceCSVPreviewLeavesSection().init( columnsStrategy, numOfGroupingColumns ).addRows( tBody );
    else
        new CSVPreviewEmptySection().init( 5, totalColumnsNum ).addRows( tBody );

    return table;
};

StaffPerformanceCSVPreviewGenerator.prototype.getGroupMessageProvider = function(columnConfig){
    switch (columnConfig){
        case StaffPerformanceColumnsConfig.GROUPS_ONLY:
        case StaffPerformanceColumnsConfig.USERS_BY_GROUPS_WITH_SUBTOTALS:
        case StaffPerformanceColumnsConfig.USERS_BY_GROUPS_WITHOUT_SUBTOTALS:
            return new UserGroupMessageProvider();
        case StaffPerformanceColumnsConfig.USERS_BY_TIME_ZONE_GROUPS_WITHOUT_SUBTOTALS:
        case StaffPerformanceColumnsConfig.TZ_GROUPS_ONLY:
        case StaffPerformanceColumnsConfig.USERS_BY_TIME_ZONE_GROUPS_WITH_SUBTOTALS:
            return new TimeZoneGroupMessageProvider();
    }
};