function CostHTMLPreviewGenerator() {}

CostHTMLPreviewGenerator.prototype.init = function()
{
    return this;
};

CostHTMLPreviewGenerator.prototype.generatePreviewTable = function( reportParameters )
{
    var table = document.createElement( "table" );
    table.className = "reportHtmlPreviewTable cost";
    var tBody = document.createElement( "tbody" );
    table.appendChild( tBody );

    var columnsStrategy = this.getColumnsStrategy( reportParameters );

    var groupingRowsCount = 3;
    var generalCostsSection = null;
    if( reportParameters.showLeaveTimeCosts || reportParameters.showOvertimeAddition )
    {
        generalCostsSection = new CostHTMLPreviewGeneralCostsSection().init( reportParameters.showLeaveTimeCosts, reportParameters.showOvertimeAddition, columnsStrategy );
        if( !reportParameters.showLeaveTimeCosts )
            groupingRowsCount += 3;
        if( !reportParameters.showOvertimeAddition )
            groupingRowsCount ++;
    }
    else
    {
        groupingRowsCount += 5;
    }

    var groupingSection = new CostHTMLPreviewGroupingSection().init(
            reportParameters.firstGroupingLevel,
            reportParameters.secondGroupingLevel,
            reportParameters.thirdGroupingLevel,
            columnsStrategy, groupingRowsCount
    );

    var headerSection = new CostHTMLPreviewHeaderSection().init( columnsStrategy );

    var grandTotalSection = new CostHTMLPreviewGrandTotalSection().init( columnsStrategy );

    headerSection.addRows( tBody );
    groupingSection.addRows( tBody );
    if( generalCostsSection )
        generalCostsSection.addRows( tBody );
    grandTotalSection.addRows( tBody );

    return table;
};

CostHTMLPreviewGenerator.prototype.getColumnsStrategy = function( reportParameters )
{
    if( reportParameters.columnsConfig == ReportColumnsFormatCostOfWork.USERS_ONLY )
    {
        if( reportParameters.showSpentTime || reportParameters.showUserRates )
            return new CostHTMLPreviewTwoLevelsHeaderUsersOnlyColumnsStrategy().init(
                    reportParameters.timeFormat,
                    reportParameters.showSpentTime,
                    reportParameters.showAverageRates,
                    reportParameters.showUserRates
            );
        else
            return new CostHTMLPreviewOneLevelHeaderUsersOnlyColumnsStrategy().init(
                    reportParameters.timeFormat,
                    reportParameters.showAverageRates
            );
    }
    else if( reportParameters.columnsConfig == ReportColumnsFormatCostOfWork.GROUPS_ONLY )
    {
        if( reportParameters.showSpentTime || reportParameters.showAverageRates )
            return new CostHTMLPreviewTwoLevelsHeaderGroupsOnlyColumnsStrategy().init(
                    reportParameters.timeFormat,
                    reportParameters.showSpentTime,
                    reportParameters.showAverageRates
            );
        else
            return new CostHTMLPreviewOneLevelHeaderGroupsOnlyColumnsStrategy().init(
                    reportParameters.timeFormat
            );

    }
    else
    {
        if( reportParameters.showSpentTime || reportParameters.showUserRates )
            return new CostHTMLPreviewUsersByGroupsThreeLevelsColumnsStrategy().init(
                    reportParameters.timeFormat,
                    reportParameters.showSpentTime,
                    reportParameters.showAverageRates,
                    reportParameters.showUserRates,
                    reportParameters.showSubtotalsByDepartments
            );
        else
            return new CostHTMLPreviewUsersByGroupsCostOnlyColumnsStrategy().init( reportParameters.timeFormat, reportParameters.showAverageRates, reportParameters.showSubtotalsByDepartments );
    }
};
