function EstimateVsActualHTMLPreviewGroupingSection() {}

EstimateVsActualHTMLPreviewGroupingSection.prototype = new PreviewSection();

EstimateVsActualHTMLPreviewGroupingSection.prototype.init = function( groupingLevelsArray, hasForthLevel, columnsStrategy, timeFormat )
{
    PreviewSection.prototype.init.call( this );
    this.groupingLevelsArray = groupingLevelsArray;
    this.hasForthLevel = hasForthLevel;
    this.columnsStrategy = columnsStrategy;
    this.timeFormat = timeFormat;
    
    return this;
};

EstimateVsActualHTMLPreviewGroupingSection.prototype.levelClasses = ["firstLevel", "secondLevel","thirdLevel", "forthLevel"];
EstimateVsActualHTMLPreviewGroupingSection.prototype.groupingClasses= ["firstGrouping", "secondGrouping","thirdGrouping", "forthGrouping"];

EstimateVsActualHTMLPreviewGroupingSection.prototype.addRows = function( tBody )
{
    this.addRowWithMainCell( tBody, this.message("report.budgeted_vs_actual.total_sc"), "totalRow", "totalValue");

    var length = this.groupingLevelsArray.length;
    var i;

    var levels;

    if( length == 1 )
        levels = [0, 0, 0, 0, 0, 0, 0, 0, 0];

    if( length == 2 )
    {
        if (ReportPreviewUtils.isLevelCanBeAddedMoreThanOnce(this.groupingLevelsArray[1], [this.groupingLevelsArray[0]]))
            levels = [0, 1, 1, 1, 0, 1, 1, 1, 1];
        else
            levels = [0, 1, 0, 1, 0, 1, 0, 1, 0];
    }

    if( length == 3 )
    {
        if(ReportPreviewUtils.isLevelCanBeAddedMoreThanOnce(this.groupingLevelsArray[2], this.groupingLevelsArray.slice(0, 2)))
            levels = [0, 1, 2, 2, 0, 1, 2, 2, 0];
        else
            levels = [0, 1, 2, 0, 1, 2, 0, 1, 2];
    }

    if( length > 3 )
    {
        if (ReportPreviewUtils.isLevelCanBeAddedMoreThanOnce(this.groupingLevelsArray[3], this.groupingLevelsArray.slice(0, 3)) &&
            ReportPreviewUtils.isLevelCanBeAddedMoreThanOnce(this.groupingLevelsArray[2], this.groupingLevelsArray.slice(0, 2)))
            levels = [0, 1, 2, 3, 3, 2, 3, 3, 0];
        else
            levels = [0, 1, 2, 3, 0, 1, 2, 3, 0];
    }

    for ( i = 0; i < levels.length; ++i )
        this.addGroupingRow(tBody, levels[i]);
};

EstimateVsActualHTMLPreviewGroupingSection.prototype.addGroupingRow = function( tBody, index )
{
    // if this grouping level is the lowest and forth level is present, it means
    //  this grouping level is really forth (even if his index can be 1 or 2, because second and third level can be skipped).
    // in this case we should add css-class for forth grouping for this row, so grIndex must be 3.
    var grIndex = ( index == this.groupingLevelsArray.length - 1 ) && this.hasForthLevel ? 3 : index;
    var groupingType = this.groupingLevelsArray[index];
    this.addRowWithMainCell( tBody, ReportPreviewUtils.getLevelPreviewName(groupingType), this.groupingClasses[grIndex], this.levelClasses[index], groupingType);
};

EstimateVsActualHTMLPreviewGroupingSection.prototype.addRowWithMainCell = function( tBody, text, rowClassName, className, groupingType )
{
    var row = this.createRow(rowClassName);
    this.addCell(row, text, className);
    this.addDataCells(row, groupingType);
    this.addRow(tBody, row);
};

EstimateVsActualHTMLPreviewGroupingSection.prototype.addDataCells = function( row, groupingType )
{
    this.addTimeValueCell(row);
    if ( groupingType && groupingType == ReportGroupingLevel.USER ) 
        this.columnsStrategy.addEmptyColumns(row);
    else
        this.columnsStrategy.addColumnValues(row);
};

EstimateVsActualHTMLPreviewGroupingSection.prototype.addTimeValueCell = function( row )
{
    ReportPreviewUtils.addHTMLTimeValueCell( row, this.timeFormat );
};