function LeaveHTMLPreviewBalanceColumnsRender(){}

LeaveHTMLPreviewBalanceColumnsRender.prototype.init = function( withSpaces ){
    this.withSpaces = withSpaces;
    this.showSickInHours = SystemSettings.showSickInHours;
    this.showPTOInHours = SystemSettings.showPTOInHours;
    return this;
};

LeaveHTMLPreviewBalanceColumnsRender.prototype.addHeaderFirstRowCells = function( fRow ){
    //abstract
};

LeaveHTMLPreviewBalanceColumnsRender.prototype.addHeaderSecondRowCells = function( sRow ){
    //abstract
};

LeaveHTMLPreviewBalanceColumnsRender.prototype.addValueCells = function(row){
    //abstract
};

LeaveHTMLPreviewBalanceColumnsRender.prototype.getRenderSpecificCssClass = function(){
    //abstract
};

LeaveHTMLPreviewBalanceColumnsRender.prototype._addPTOFRowCells = function(row){
    ReportPreviewUtils.addCreatedCell(row, this._getBalanceTypeCell("report.leaves.pto_column.preview.html.pto_balance"));
};

LeaveHTMLPreviewBalanceColumnsRender.prototype._addSickDaysFRowCells = function(row){
    ReportPreviewUtils.addCreatedCell(row, this._getBalanceTypeCell("report.leaves.pto_column.preview.html.sick_days_balance"));
};

LeaveHTMLPreviewBalanceColumnsRender.prototype._addRangeSRowCells = function(inHours, row){
    ReportPreviewUtils.addCreatedCell(row, this._getRangeCell(inHours, "report.leaves.pto_column.preview.html.beginning"));
    ReportPreviewUtils.addCreatedCell(row, this._getRangeCell(inHours, "report.leaves.pto_column.preview.html.ending"));
};

LeaveHTMLPreviewBalanceColumnsRender.prototype._getBalanceTypeCell = function(textKey){
    return ReportPreviewUtils.createCell(messageResource.getMessage(textKey) , "balanceCell", 2, 1);
};

LeaveHTMLPreviewBalanceColumnsRender.prototype._getRangeCell = function(inHours, textKey) {
    return ReportPreviewUtils.createCell(messageResource.getMessage(textKey, [this._getHoursOrDays(inHours)]), "balanceRangeCell");
};

LeaveHTMLPreviewBalanceColumnsRender.prototype._getHoursOrDays = function (inHours) {
    if(inHours){
        return messageResource.getMessage("common.pto_units.hours");
    }
    return messageResource.getMessage("common.pto_units.days");
};

LeaveHTMLPreviewBalanceColumnsRender.prototype._addBalanceValueCell = function(row)
{
    ReportPreviewUtils.addCreatedCell( row, ReportPreviewUtils.createCell( "12"+ SystemSettings.decimalSeparator + "345", "balanceValueCell"));
};

LeaveHTMLPreviewBalanceColumnsRender.prototype._addEmptyCellIfNeed = function (row ) {
    if(this.withSpaces){
        ReportPreviewUtils.addCell( row, "", "emptyCell" );
    }
};

//-----------------------------------------------------------------------------------

function LeaveHTMLPreviewBothBalanceColumnsRender(){}

LeaveHTMLPreviewBothBalanceColumnsRender.prototype = new LeaveHTMLPreviewBalanceColumnsRender();

LeaveHTMLPreviewBothBalanceColumnsRender.prototype.addHeaderFirstRowCells = function( fRow ){
    this._addEmptyCellIfNeed(fRow);
    this._addPTOFRowCells(fRow);
    this._addEmptyCellIfNeed(fRow);
    this._addSickDaysFRowCells(fRow);
};

LeaveHTMLPreviewBothBalanceColumnsRender.prototype.addHeaderSecondRowCells = function( sRow ){
    this._addEmptyCellIfNeed(sRow);
    this._addRangeSRowCells(this.showPTOInHours, sRow);
    this._addEmptyCellIfNeed(sRow);
    this._addRangeSRowCells(this.showSickInHours, sRow);
};

LeaveHTMLPreviewBothBalanceColumnsRender.prototype.addValueCells = function(row){
    this._addEmptyCellIfNeed(row);
    this._addBalanceValueCell(row);
    this._addBalanceValueCell(row);
    this._addEmptyCellIfNeed(row);
    this._addBalanceValueCell(row);
    this._addBalanceValueCell(row);
};

LeaveHTMLPreviewBothBalanceColumnsRender.prototype.getRenderSpecificCssClass = function(){
    return "bothBalances"
};

//-----------------------------------------------------------------------------------

function LeaveHTMLPreviewPTOOnlyBalanceColumnsRender(){}

LeaveHTMLPreviewPTOOnlyBalanceColumnsRender.prototype = new LeaveHTMLPreviewBalanceColumnsRender();

LeaveHTMLPreviewPTOOnlyBalanceColumnsRender.prototype.addHeaderFirstRowCells = function( fRow ){
    this._addEmptyCellIfNeed(fRow);
    this._addPTOFRowCells(fRow);
};

LeaveHTMLPreviewPTOOnlyBalanceColumnsRender.prototype.addHeaderSecondRowCells = function( sRow ){
    this._addEmptyCellIfNeed(sRow);
    this._addRangeSRowCells(this.showPTOInHours, sRow);
};

LeaveHTMLPreviewPTOOnlyBalanceColumnsRender.prototype.addValueCells = function(row){
    this._addEmptyCellIfNeed(row);
    this._addBalanceValueCell(row);
    this._addBalanceValueCell(row);
};

LeaveHTMLPreviewPTOOnlyBalanceColumnsRender.prototype.getRenderSpecificCssClass = function(){
    return "oneBalance"
};

//-----------------------------------------------------------------------------------

function LeaveHTMLPreviewSickDaysOnlyBalanceColumnsRender(){}

LeaveHTMLPreviewSickDaysOnlyBalanceColumnsRender.prototype = new LeaveHTMLPreviewBalanceColumnsRender();

LeaveHTMLPreviewSickDaysOnlyBalanceColumnsRender.prototype.addHeaderFirstRowCells = function( fRow ){
    this._addEmptyCellIfNeed(fRow);
    this._addSickDaysFRowCells(fRow);
};

LeaveHTMLPreviewSickDaysOnlyBalanceColumnsRender.prototype.addHeaderSecondRowCells = function( sRow ){
    this._addEmptyCellIfNeed(sRow);
    this._addRangeSRowCells(this.showSickInHours, sRow);
};

LeaveHTMLPreviewSickDaysOnlyBalanceColumnsRender.prototype.addValueCells = function(row){
    this._addEmptyCellIfNeed(row);
    this._addBalanceValueCell(row);
    this._addBalanceValueCell(row);
};

LeaveHTMLPreviewSickDaysOnlyBalanceColumnsRender.prototype.getRenderSpecificCssClass = function(){
    return "oneBalance"
};
