function StaffPerformanceHTMLPreviewGenerator() {}

StaffPerformanceHTMLPreviewGenerator.prototype.init = function()
{
    return this;
};
StaffPerformanceHTMLPreviewGenerator.prototype.generatePreviewTable = function( reportParameters )
{
    var columnsStrategy;
    switch (reportParameters.columnsConfig) {
        case StaffPerformanceColumnsConfig.USERS_ONLY:
            columnsStrategy = new StaffPerformanceHTMLPreviewUsersOnlyColumnsStrategy().init(reportParameters.timeFormat);
            break;
        case StaffPerformanceColumnsConfig.GROUPS_ONLY:
        case StaffPerformanceColumnsConfig.TZ_GROUPS_ONLY:
            columnsStrategy = new StaffPerformanceHTMLPreviewGroupsOnlyColumnsStrategy().init(reportParameters.timeFormat, this.getGroupMessageProvider(reportParameters.columnsConfig));
            break;
        case StaffPerformanceColumnsConfig.USERS_BY_GROUPS_WITHOUT_SUBTOTALS:
        case StaffPerformanceColumnsConfig.USERS_BY_TIME_ZONE_GROUPS_WITHOUT_SUBTOTALS:
            columnsStrategy = new StaffPerformanceHTMLPreviewUsersByGroupsWithoutSubtotalsColumnsStrategy().init(reportParameters.timeFormat,  this.getGroupMessageProvider(reportParameters.columnsConfig));
            break;
        case StaffPerformanceColumnsConfig.USERS_BY_GROUPS_WITH_SUBTOTALS:
        case StaffPerformanceColumnsConfig.USERS_BY_TIME_ZONE_GROUPS_WITH_SUBTOTALS:
            columnsStrategy = new StaffPerformanceHTMLPreviewUsersByGroupsWithSubtotalsColumnsStrategy().init(reportParameters.timeFormat,  this.getGroupMessageProvider(reportParameters.columnsConfig));
    }

    var header = new StaffPerformanceHTMLPreviewHeaderSection().init(
            reportParameters.firstGroupingLevel, reportParameters.secondGroupingLevel, reportParameters.thirdGroupingLevel,
            columnsStrategy
    );


    var dataGroupingSection = new StaffPerformanceHTMLPreviewGroupingSection().init(
            reportParameters.firstGroupingLevel, reportParameters.secondGroupingLevel, reportParameters.thirdGroupingLevel,
            !reportParameters.showLeaveTime, columnsStrategy
    );

    var table = document.createElement( "table" );
    table.className = "reportHtmlPreviewTable staffPerformance";
    var tBody = document.createElement( "tbody" );
    table.appendChild( tBody );
    header.addRows( tBody );

    if( reportParameters.showLeaveTime )
        new StaffPerformanceHTMLPreviewLeaveSection().init( columnsStrategy ).addRows( tBody );

    dataGroupingSection.addRows( tBody );

    return table;
};

StaffPerformanceHTMLPreviewGenerator.prototype.getGroupMessageProvider = function(columnConfig){
         switch (columnConfig){
             case StaffPerformanceColumnsConfig.GROUPS_ONLY:
             case StaffPerformanceColumnsConfig.USERS_BY_GROUPS_WITH_SUBTOTALS:
             case StaffPerformanceColumnsConfig.USERS_BY_GROUPS_WITHOUT_SUBTOTALS:
                 return new UserGroupMessageProvider();
             case StaffPerformanceColumnsConfig.USERS_BY_TIME_ZONE_GROUPS_WITHOUT_SUBTOTALS:
             case StaffPerformanceColumnsConfig.TZ_GROUPS_ONLY:
             case StaffPerformanceColumnsConfig.USERS_BY_TIME_ZONE_GROUPS_WITH_SUBTOTALS:
                 return new TimeZoneGroupMessageProvider();
         }
};