var ReportConfigsAction = {
    DELETE_CONFIG_ACTION: "delete_config",
    EDIT_CONFIG_ACTION: "edit_config",
    SHOW_REPORT_ACTION: "show_report"
};

function submitActionForm(actionName, configId, isChart, isScrollClicked )
{
    var form = document.getElementById("actionForm");
    form["pageAction"].value = actionName;
    form["configId"].value = configId;
    form["chart"].value = isChart;
    if ( isScrollClicked )
        form.target = '_blank';
    else
        form.target = '_self';
    form.submit();
}
function deleteConfig(id, isChart)
{   $('#reportConfig_' + id ).addClass("reportToDelete");
    if (confirm(messageResource.getMessage("saved_reports.delete.confirmation",[messageResource.getMessage(isChart ? "report.dashboard.chart":"report.dashboard.report")] )))
        submitActionForm( ReportConfigsAction.DELETE_CONFIG_ACTION, id, isChart );
    $('#reportConfig_' + id ).removeClass("reportToDelete");
}

function showReport(id, isScrollClicked)
{
    submitActionForm(ReportConfigsAction.SHOW_REPORT_ACTION, id, "", isScrollClicked);
}

function showChart(id)
{
    if ( typeof chartMediator != typeof undefined && chartMediator != null )
        chartMediator.showChart(id);
}

function submitSortingOrderForm()
{
    var form = document.getElementById("sortingOrderForm");
    form.submit();
}

var ReportConfigFactory = new function()
{
    this.configs = [];
    this.create = function (node, id, category, users, userGroups, timeZoneGroups, configFormat, configType)
    {
        this.configs.push(new ReportConfig(node, id, category, users, userGroups, timeZoneGroups, configFormat, configType))
    };
    this.getActiveConfig = function()
    {
        for (var i = 0; i < this.configs.length; i++)
            if (this.configs[i].isActive())
                return this.configs[i];
        return null;
    };
    this.isActiveThisOrNone = function(config)
    {
        var active = this.getActiveConfig();
        return active == null || active == config;
    };
};


///////////////////////////////////////
ReportConfig = function(node, id, configCategory, users, userGroups, timeZoneGroups, configFormat, configType)
{
    this.node = node;
    this.id = id;
    this.active = false;
    this.configFormat = configFormat;
    this.configType = configType;
    if (hasParentWithClassName(node, "enabled") &&  !$(node).hasClass("CSV"))
        this.createShareButton(configCategory, users, userGroups, timeZoneGroups, configFormat);
    this.tooltip = new DhtmlTitle("title_div", true, false);
    this.convertTitles();
    addListener(this.node, "mouseover", makeHandler(this.mouseOver, this));
    addListener(this.node, "mouseout", makeHandler(this.mouseOut, this));
    addListener(this.node, "mousedown", makeHandler(this.mouseDown, this));
    addListener(this.node, "mouseup", makeHandler(this.mouseUp, this));
    addListener(this.tooltip.getDiv(), "mouseover", makeHandler(this.tooltip.immediateHide, this.tooltip));
};

ReportConfig.prototype =
{
    activeClass: "active",
    generateReportHoverClass: "generate-report-hover",

    getButtonMenu: function (typeName, users, userGroups, timeZoneGroups, configFormat)
    {
        if(!this.isChart(configFormat))
        {
            switch( typeName )
            {
                case "money":
                    return this.getShareMoneyReportsButtonMenu( users, userGroups, timeZoneGroups );
                case "invoice":
                    return this.getShareInvoiceReportsButtonMenu( users, userGroups, timeZoneGroups );
                default:
                    return this.getShareTimeReportsButtonMenu( users, userGroups, timeZoneGroups );
            }
        }
        else{
            switch( typeName )
            {
                case "money":
                    return this.getShareMoneyChartsButtonMenu( users, userGroups, timeZoneGroups );
                default:
                    return this.getShareTimeChartsButtonMenu( users, userGroups, timeZoneGroups );
            }
        }
    },
    getShareTimeReportsButtonMenu: function( users, userGroups, timeZoneGroups )
    {
        if( !ReportConfig.TIME_REPORTS_BUTTON_MENU )
            ReportConfig.TIME_REPORTS_BUTTON_MENU = buildShareReportSelectorMenu(
                users,
                userGroups,
                timeZoneGroups,
                getShareReportsButtonMenuMessages(),
                new GenerateTimeReportRight() );
        return ReportConfig.TIME_REPORTS_BUTTON_MENU;
    },

    getShareMoneyReportsButtonMenu: function( users, userGroups, timeZoneGroups )
    {
        if( !ReportConfig.MONEY_REPORTS_BUTTON_MENU )
            ReportConfig.MONEY_REPORTS_BUTTON_MENU = buildShareReportSelectorMenu(
                users,
                userGroups,
                timeZoneGroups,
                getShareReportsButtonMenuMessages(),
                new GenerateMoneyReportRight() );
        return ReportConfig.MONEY_REPORTS_BUTTON_MENU;
    },

    getShareMoneyChartsButtonMenu: function( users, userGroups, timeZoneGroups )
    {
        if( !ReportConfig.MONEY_CHARTS_BUTTON_MENU )
            ReportConfig.MONEY_CHARTS_BUTTON_MENU = buildShareReportSelectorMenu(
                    users,
                    userGroups,
                    timeZoneGroups,
                    getShareChartsButtonMenuMessages(),
                    new GenerateMoneyReportRight() );
        return ReportConfig.MONEY_CHARTS_BUTTON_MENU;
    },

    getShareTimeChartsButtonMenu: function( users, userGroups, timeZoneGroups )
    {
        if( !ReportConfig.TIME_CHARTS_BUTTON_MENU )
            ReportConfig.TIME_CHARTS_BUTTON_MENU = buildShareReportSelectorMenu(
                    users,
                    userGroups,
                    timeZoneGroups,
                    getShareChartsButtonMenuMessages(),
                    new GenerateTimeReportRight() );
        return ReportConfig.TIME_CHARTS_BUTTON_MENU;
    },

    getShareInvoiceReportsButtonMenu: function( users, userGroups, timeZoneGroups )
    {
        if( !ReportConfig.INVOICE_REPORTS_BUTTON_MENU )
            ReportConfig.INVOICE_REPORTS_BUTTON_MENU = buildShareReportSelectorMenu(
                users,
                userGroups,
                timeZoneGroups,
                getShareInvoiceButtonMenuMessages(),
                new GenerateMoneyReportRight() );
        return ReportConfig.INVOICE_REPORTS_BUTTON_MENU;
    },

    isChart: function(configFormat)
    {
       return configFormat == "CHART";
    },

    createShareButton: function (configCategory, users, userGroups, timeZoneGroups, configFormat)
    {
        var self = this;
        this.shareButtonMenu = this.getButtonMenu(configCategory, users, userGroups, timeZoneGroups, configFormat);
        this.shareButton = new AT.SimpleDropdownButtonForSharedSelector({
            template: new Ext.Template(
                '<table border="0" cellpadding="0" cellspacing="0" class="x-btn-wrap"><tbody><tr>',
                '<td class="x-btn-left"><i>&#160;</i></td><td class="x-btn-center"><em unselectable="on"><a href="#" class="x-btn-text" type="{1}" style="font-size: 11px!important;">{0}</a></em></td><td class="x-btn-right"><i>&#160;</i></td>',
                "</tr></tbody></table>"),
            renderTo: "shareConfig"+this.id,
            configId: this.id,
            menu: this.shareButtonMenu,
            onMenuShowCallback: makeHandler(this.onShareMenuShow, this),
            onMenuHideCallback: makeHandler(this.onShareMenuHide, this),
            text: messageResource.getMessage("saved_reports.share.title"),
            showMenu: function(){
                this.menu.panel.beforeShow();
                this.menu.setCurrentParent(this, true);
                this.menu.panel.setCurrentSaveOptions({
                    configId: this.configId,
                    isChart: self.isChart(configFormat)
                });
                this.menu.panel.setSaveCallback(shareSavedConfig);
                AT.SimpleDropdownButtonForSharedSelector.superclass.showMenu.call(this);
                this.menu.panel.afterShow();
            }
        });
    },

    isActive: function()
    {
        return this.active;
    },

    onShareMenuShow: function()
    {
        this.updateClasses();
        this.active = true;
        this.sharedMenuShown = true;
    },
    onShareMenuHide: function()
    {
        this.sharedMenuShown = false;
        this.active = false;
        this.updateClasses();
    },

    updateClasses: function()
    {
        if (ReportConfigFactory.isActiveThisOrNone(this))
        {
            if (!this.sharedMenuShown)
            {
                if (this.isMouseOver)
                    addClass(this.node, this.activeClass);
                else
                    removeClass(this.node, this.activeClass);
            }
            if (this.isMouseOverGenerateReportArea)
                addClass(this.node, this.generateReportHoverClass);
            else
                removeClass(this.node, this.generateReportHoverClass);
        }
    },

    mouseOver: function(event)
    {
        var target = getEventTarget(event);
        if (this.isConfigBody(target))
        {
            this.isMouseOver = true;
            if (this.isGenerateReportArea(target))
                this.isMouseOverGenerateReportArea = true
        }
        this.updateClasses();
    },
    mouseOut: function(event)
    {
        var relatedTarget = event.relatedTarget || event.toElement;
        if (!isSameOrParentOf(relatedTarget, this.node) ||
            !this.isConfigBody(relatedTarget))
        {
            this.isMouseOver = false;
            this.isMouseOverGenerateReportArea = false;
        }
        else if (!this.isGenerateReportArea(relatedTarget))
            this.isMouseOverGenerateReportArea = false;
        this.updateClasses();
    },

    mouseDown: function(event)
    {
        if (event.preventDefault)
            event.preventDefault();
        else // IE8
        {
            event.returnValue = false;
            if ( event.button && event.button & 4 ) // scroll is clicked
            {
                event.which = 2;
                this.mouseUp( event );
            }
        }
        return false;
    },

    mouseUp: function(event)
    {
        if ( !event.which && event.button ) // IE8
        {
            if ( event.button & 1 ) // left mouse button is clicked
                event.which = 1;
            else
                return;
        }

        if ( event.which == 3 )
            return;

        var isScrollClicked = event.which == 2;
        var target = getEventTarget(event);
        if( ReportConfigFactory.isActiveThisOrNone( this ) && this.isGenerateReportArea( target ) && this.isEnabled( target ) )
        {
            var self = this;
            var submitCSVForm = function(generateCSVUrl, isScrollClicked){
                var form = $( "#csvConfigForm" );
                form.attr( "action", generateCSVUrl );
                form.find( "input[name=configId]" ).val( self.id );
                if ( isScrollClicked )
                    form.attr( "target", "_blank" );
                else
                    form.attr( "target", "_self" );
                form.submit();
            };
            if ( this.configFormat == "CHART" )
                showChart( this.id );
            else if ( this.configFormat == "CSV" )
                submitCSVForm( reportCSVPathFromDB[self.configType], isScrollClicked );
            else
                showReport( this.id, isScrollClicked );
        }
    },

    isGenerateReportArea: function(node)
    {
        return this.isConfigBody(node) && !hasParentWithClassName(node, "configHeaderShareCell") && !hasParentWithClassName(node, "configHeaderRemoveCell");
    },

    isEnabled: function(node)
    {
        return hasParentWithClassName(node, "enabled");
    },

    isConfigBody: function(node)
    {
        return node != null && hasParentWithClassName(node, "config") &&
            !hasParentWithClassName(node, "space") &&
            !hasParentWithClassName(node, "transparent-blank-cell");
    },

    convertTitles: function(parentNode)
    {
        parentNode = parentNode || this.node;
        if (parentNode.title)
            new TitleAdapter(this.tooltip, parentNode);
        else
            for (var i = 0; i < parentNode.childNodes.length; i++)
                this.convertTitles(parentNode.childNodes[i]);
    }
};

    // The purpose of this subclass is to analyze selection of users and
    // reload configuration if a user shares a configuration to himself.
    function ShareSavedReportConfigReloadingManager( userSelectorMenu ) {
        ShareSavedReportConfigManager.call(this, userSelectorMenu);
        var needReload = false;
        var selectedUsers = userSelectorMenu.getOnlyUsersSelection();
        for (var i = 0; i < selectedUsers.length; i++) {
            if (selectedUsers[i].id == LOGGED_IN_USER_ID) {
                needReload = true;
                break;
            }
        }
        this.reloadAfterSave = needReload;
    }

    ShareSavedReportConfigReloadingManager.prototype = new ShareSavedReportConfigManager();

    ShareSavedReportConfigReloadingManager.prototype.endSavingData = function(success)
    {
        ShareSavedReportConfigManager.prototype.endSavingData.call(this, success);
        if(this.reloadAfterSave)
            document.location.reload();
    };

    function shareSavedConfig()
    {
        var shareMgr = new ShareSavedReportConfigReloadingManager( this );
        RpcControlManager.saveRpcControlBoundData( shareMgr, false );
    }

    TitleAdapter = function (tooltip, node)
    {
        this.node = node;
        this.titleText = this.node.title;
        this.node.title = "";
        this.tooltip = tooltip;
        this.mouse = new MouseController();
        this.getDiv = function () {return node; };
        addListener(this.node, "mouseover", makeHandler(this.showTitle, this));
        addListener(this.node, "mouseout", makeHandler(this.hideTitle, this));
        addListener(this.node, "mousemove", makeHandler(this.updateMousePosition, this))
    };

    TitleAdapter.prototype = {

        showTitle: function(event)
        {
            this.tooltip.cancelHide();
            this.mouse.update(event || window.event);
            this.tooltip.showDelayedWithTimeoutNearMouse(this.titleText, null, this.mouse, 200);
        },

        hideTitle: function()
        {
            this.tooltip.cancelDelayedShow();
            this.tooltip.immediateHide();
        },

        updateMousePosition: function(event)
        {
            this.tooltip.immediateHide();
            this.showTitle(event);
        }
    };

    function MouseController() {}

MouseController.prototype =
{
    getX: function()
    {
        return this.x;
    },

    getY: function()
    {
        return this.y;
    },

    getCursorWidth: function()
    {
        return 20;
    },

    getCursorHeight: function()
    {
        return 20;
    },

    update: function(event)
    {
        this.x= event.clientX + getBodyScrollLeft();
        this.y= event.clientY + getBodyScrollTop();
    }
};