/**
 * Created by filatov on 28.08.2015.
 */

function SaveDialogControl(){}

SaveDialogControl.prototype.init = function(controlDiv, type, submitCallback, onCloseHandler){
    var self = this;
    this.type = type;
    this.onCloseHandler = onCloseHandler;
    this.saveDialogDiv = controlDiv;
    this.saveDialog = new DropdownDialog(
            {
                wrapperClass: "reportHeaderSaveChangesDialog",
                dialogDiv: this.saveDialogDiv,
                hideButtons: false,
                okButtonText: messageResource.getMessage( "common.save" ),
                buttonsPlaceholder: this.saveDialogDiv.find( ".footer" ),
                isWithBeak: true,
                hideOnResize: true,
                onOkClick: submitCallback,
                width: "380px",
                height: "240px",
                zIndex: 3003,
                onClose : function(event){
                    self.hideError(SaveConfigurationControl.EMPTY_NAME); // hide error on close (empty name)
                    self.hideErrorTooltip();
                    self.onCloseHandler(); // update error on top field

                    /* here we restore old key-down-handler
                     * and stop propagation of the event in order to avoid restored handler immediate invoking,
                     * because current key-down event is related only to panel and should be handled only by panel
                     * */
                    event.stopPropagation();
                    KeyDownHandler.restoreOnKeyDownHandler();
                }
            }
    );

    this.updateNameEdit = this.saveDialogDiv.find( "#updateNameEdit" );
    this.saveNewNameEdit = this.saveDialogDiv.find( ".saveNewNameEdit" );
    this.radio = this.saveDialogDiv.find( "input[name='option']" );
    this.radioUpdate = this.saveDialogDiv.find( "input#saveConfigUpdate" );
    this.radioNew = this.saveDialogDiv.find( "input#saveConfigNew" );
    this.saveNewBlock = this.saveDialogDiv.find(".saveNew");
    this.updateBlock = this.saveDialogDiv.find(".update");

    this.errorTooltip = new TableTooltip().init("saveChangesDialogErrorTooltip");

    this.errorTooltip.updateUI = function ( config )
    {
        this._tooltipTextPlaceholder.html(config.errorMessage) ;
    };


    $("#saveConfigNew").click( function() { self.changeBlockFocus(self.blocks.NEW) } );
    this.saveNewNameEdit.click(function(){self.changeBlockFocus(self.blocks.NEW)});

    $("#saveConfigUpdate").click( function() { self.changeBlockFocus(self.blocks.UPDATE) } );
    this.updateNameEdit.click(function(){self.changeBlockFocus(self.blocks.UPDATE)});

    // if values is changed hide errors
    this.updateNameEdit.change(function(){self.hideErrors();});
    // if values is changed hide errors
    this.saveNewNameEdit.change(function(){self.hideErrors();});

    // block of dialog
    this.blocks = {
        UPDATE: {
            error: {},
            blockEl: self.updateBlock,
            input: self.updateNameEdit,
            radioButton: self.radioUpdate
        },
        NEW: {
            error: {},
            blockEl: self.saveNewBlock,
            input: self.saveNewNameEdit,
            radioButton: self.radioNew
        }
    };

    this.changeBlockFocus(this.blocks.UPDATE); // default

    return this;
};

SaveDialogControl.prototype.hideErrors = function(){
    this.currentBlock.error = {};
    this.currentBlock.input.removeClass("error");
    this.updateTooltipHandlersForSection(this.currentBlock.input);
};

SaveDialogControl.prototype.hideError = function(error) {
    if (this.currentBlock.error == error) {
        this.currentBlock.error = {};
        this.currentBlock.input.removeClass("error");
        this.updateTooltipHandlersForSection(this.currentBlock.input);
    }
};

SaveDialogControl.prototype.hasErrors = function() {
    return this.currentBlock.error == SaveConfigurationControl.DUPLICATE_NAME || this.currentBlock.error == SaveConfigurationControl.EMPTY_NAME;
};

SaveDialogControl.prototype.getError = function() {
    return this.currentBlock.error;
};

SaveDialogControl.prototype.changeBlockFocus = function( block ){

    if (block != this.currentBlock) {
        if (this.currentBlock) {
            this.currentBlock.blockEl.removeClass("inFocus");
            this.hideErrors();
        }
        block.blockEl.addClass("inFocus");
        block.radioButton.prop("checked", true);
        block.input.focus();
        this.currentBlock = block;
    }
};

SaveDialogControl.prototype.hide = function(){
    this.saveDialog.hide();
};

SaveDialogControl.prototype.showAt = function(element, offset){
    if (!this.hasErrors()){
        this.changeBlockFocus(this.blocks.UPDATE);
    }
    this.saveDialog.showAt( element, { offset: "0 10px", collision: "none" } );

    /* we set empty key-down-handler, because:
     * - after the panel is shown, all key-down-events are related only to it and it handles them itself;
     * - we need to save old handler and restore it after the panel is hidden.
     * */
    KeyDownHandler.setOnKeyDownHandler( function () {} );
};

SaveDialogControl.prototype.isShow = function(){
    return this.dialogShow;
};

SaveDialogControl.prototype.updateControlForUpdate = function(val){
    this.updateNameEdit.val( val );
};

SaveDialogControl.prototype.updateControlForNew = function(val){
    this.saveNewNameEdit.val( val );
};

SaveDialogControl.prototype.isUpdate = function(){
    return this.blocks.UPDATE == this.currentBlock;
};

SaveDialogControl.prototype.getReportName = function() {
    this.currentBlock.input.val(trim(this.currentBlock.input.val()));
    return this.currentBlock.input.val();
};

SaveDialogControl.prototype.focusOnError = function() {
    this.currentBlock.input.focus();
};
SaveDialogControl.prototype.showDialogError = function( error ) {
    this.currentBlock.input.addClass("error");
    this.updateTooltipHandlersForSection(this.currentBlock.input, error);
    this.currentBlock.error = error;
    this.focusOnError();
};

SaveDialogControl.prototype.getErrorMessage = function( error)
{
    var messageKey;
    if ( SaveConfigurationControl.REPORT_TYPE === this.type )
        messageKey = "common";
    else if ( SaveConfigurationControl.CHART_TYPE === this.type )
        messageKey = "chart";

    if ( messageKey )
        switch (error)
        {
            case SaveConfigurationControl.EMPTY_NAME:
                return messageResource.getMessage("report." + messageKey + ".empty_name");
                break;
            case SaveConfigurationControl.DUPLICATE_NAME:
                return messageResource.getMessage("report." + messageKey + ".duplicate_name");
                break;
        }
    return "";
};


SaveDialogControl.prototype.showErrorTooltip = function(event, error)
{
    this.errorTooltip.updateUIAndShow( {errorMessage: this.getErrorMessage(error)},
                                       calculateTooltipOffset( this.errorTooltip, event ));
};

SaveDialogControl.prototype.moveErrorTooltip = function(event)
{
    this.errorTooltip.move( calculateTooltipOffset( this.errorTooltip, event ) );
};

SaveDialogControl.prototype.hideErrorTooltip = function()
{
    this.errorTooltip.hide();
};

SaveDialogControl.prototype.updateTooltipHandlersForSection = function(sectionEl, error)
{
    if (!sectionEl)
        return;

    var self = this;
    if (error)
    {
        sectionEl.hover( function ( event ) { self.showErrorTooltip( event, error ) },
                         function () { self.hideErrorTooltip(); } );
        sectionEl.mousemove( function ( event ) {self.moveErrorTooltip( event ) } );
    }
    else
    {
        sectionEl.unbind('hover mousemove');
    }
};
