function SaveConfigurationControl() {}

SaveConfigurationControl.EMPTY_NAME = "emptyName";
SaveConfigurationControl.DUPLICATE_NAME = "duplicateName";

SaveConfigurationControl.REPORT_TYPE = "report";
SaveConfigurationControl.CHART_TYPE = "chart";

SaveConfigurationControl.prototype.init = function(reportConfig, type, saveCallback, updateCallback, resetConfigCallback)
{
    var self = this;
    this.reportConfigName = reportConfig.reportName;
    this.reportType = reportConfig.reportType;

    this.controlBlock = $( ".saveConfigurationControl" );
    this.controlBlock.removeClass().addClass( "saveConfigurationControl" );
    this.addToDashboardIcon = this.controlBlock.find(".addToDashboardIcon");
    this.addToDashboard = this.controlBlock.find( ".addToDashboard" );
    this.reportNameDiv = this.controlBlock.find( ".reportName" );
    this.reportNameEdit = this.controlBlock.find( ".reportNameEdit" );

    this.reportNameEdit.placeholder();

    this.saveNewConfigButton = this.controlBlock.find( ".saveNewConfigButton" );
    this.cancelNewConfigButton = this.controlBlock.find( ".cancelNewConfigButton" );
    this.saveCurrentConfigButton = this.controlBlock.find( ".saveCurrentConfigButton" );
    this.cancelCurrentConfigButton = this.controlBlock.find( ".cancelCurrentConfigButton" );
    this.savingSpinner = new SpinnerController();

    this.resetConfigCallback = resetConfigCallback;
    this.saveCallback = saveCallback;
    this.updateCallback = updateCallback;

    this.saveDialogDiv = $( ".saveChangesDialogDiv" );
    this.saveDialogDiv.css( "width", "" );

    this.saveDialog = new SaveDialogControl().init(this.saveDialogDiv,
        type,
        function(){self._saveChanges()}, // on submit
        function(error) {self.onCloseDialogHandler()} // on close
    );

    this.currentState = null;
    this.errors = {};
    if( !this.reportConfigName )
    {
        this.reportNameDiv.text( "" );
        this._changeState( this.states.NEW_CONFIG );
    }
    else
    {
        this.reportNameDiv.text( this.reportConfigName ); // with escape
        if( reportConfig.hasChanges )
            this._changeState( this.states.CHANGED_CONFIG );
        else
            this._changeState( this.states.CURRENT_CONFIG );
    }

    this.keyUpChangesHandler = function(additionalHandler) {
        return function () {

            this.keyPressed = false;

            if( additionalHandler )
                additionalHandler();
        }
    };
    this.onBlurFieldHandler = function(additionalHandler){
        return function()
        {
            $(this).pVal( trim(this.value) );
            //see Bug 48539
            if ( this.keyPressed ) self.keyUpChangesHandler.call(this);

            if (additionalHandler)
                additionalHandler();
        };
    };

    this.onKeyPressedHandler = function ()
    {
        this.keyPressed = true;
    };

    this.reportNameEdit.on("keydown", this.onKeyPressedHandler );
    this.reportNameEdit.on("blur", this.onBlurFieldHandler( function(){
        if (self.errors[SaveConfigurationControl.EMPTY_NAME] && trim(self.reportNameEdit.val()) != "")
             self.hideError(SaveConfigurationControl.EMPTY_NAME);} ) );

    this.reportNameEdit.on("keyup", this.keyUpChangesHandler );
    this.reportNameEdit.on("change", function(){self.hideError(SaveConfigurationControl.DUPLICATE_NAME); self.saveDialog.hideErrors()}); // if change hide dialog errors

    this.addToDashboard.hover( function(event) { self.addToDashboardIcon.addClass("lighting")}, function(){self.addToDashboardIcon.removeClass("lighting") } );
    this.addToDashboardIcon.hover( function(event) { self.reportNameDiv.addClass("startToEdit")}, function() {self.reportNameDiv.removeClass("startToEdit") } );

    return this;
};

SaveConfigurationControl.prototype.states =
{
    NEW_CONFIG: {
        className: "newConfig",
        edit: false,
        updateUI: function( control )
        {
            control.addToDashboard.off( "click" ).click( function() { control._editNewConfig(); } );
            control.addToDashboardIcon.off("click").click( function() { control._editNewConfig(); } );
            control.reportNameDiv.off( "click" );
            control.saveNewConfigButton.off( "click" );
            control.cancelNewConfigButton.off( "click" );
            control.saveCurrentConfigButton.off( "click" );
            control.cancelCurrentConfigButton.off( "click" );
        }
    },
    NEW_CONFIG_EDIT: {
        className: "newConfigEdit",
        edit: true,
        updateUI: function( control )
        {
            control.addToDashboard.off( "click" );
            control.addToDashboardIcon.off( "click" );
            control.reportNameDiv.off( "click" );
            control.saveNewConfigButton.off( "click" ).click( function() { control._saveNewConfig(); } );
            control.cancelNewConfigButton.off( "click" ).click( function() { control._cancelNewConfig(); } );
            control.saveCurrentConfigButton.off( "click" );
            control.cancelCurrentConfigButton.off( "click" );
            control.reportNameEdit.focus();
         }
    },
    CURRENT_CONFIG: {
        className: "currentConfig",
        edit: false,
        updateUI: function( control )
        {
            control.addToDashboard.off( "click" );
            control.addToDashboardIcon.off( "click" ).click( function() { control._editChangedConfig(); } );
            control.reportNameDiv.off( "click" ).click( function() { control._editChangedConfig(); } );
            control.saveNewConfigButton.off( "click" );
            control.cancelNewConfigButton.off( "click" );
            control.saveCurrentConfigButton.off( "click" );
            control.cancelCurrentConfigButton.off( "click" );
        }
    },
    CHANGED_CONFIG: {
        className: "changedConfig",
        edit: false,
        updateUI: function( control )
        {
            control.addToDashboard.off( "click" );
            control.addToDashboardIcon.off( "click" ).click( function() { control._editChangedConfig(); } );
            control.reportNameDiv.off( "click" ).click( function() { control._editChangedConfig(); } );
            control.saveNewConfigButton.off( "click" );
            control.cancelNewConfigButton.off( "click" );
            control.saveCurrentConfigButton.off( "click" ).click( function() { control._saveCurrentConfig(); } );
            control.cancelCurrentConfigButton.off( "click" ).click( function() { control._cancelCurrentConfig(); } );
        }
    },
    CHANGED_CONFIG_EDIT: {
        className: "changedConfigEdit",
        edit: true,
        updateUI: function( control )
        {
            control.addToDashboard.off( "click" );
            control.addToDashboardIcon.off( "click" );
            control.reportNameDiv.off( "click" );
            control.saveNewConfigButton.off( "click" );
            control.cancelNewConfigButton.off( "click" );
            control.saveCurrentConfigButton.off( "click" ).click( function() { control._saveCurrentConfig(); } );
            control.cancelCurrentConfigButton.off( "click" ).click( function() { control._cancelCurrentConfig(); } );
            control.reportNameEdit.focus();
        }
    }
};

SaveConfigurationControl.prototype.onCloseDialogHandler = function() {
    if(this.saveDialog.hasErrors()) {
        this.displayError(this.saveDialog.getError());
    }
};


SaveConfigurationControl.prototype.cancelSaving = function() {
    this.isSaving = false;

    this.controlBlock.removeClass( "saving" );
    this.reportNameEdit.prop("disabled", false);
    this._changeState( this.currentState);

    this.savingSpinner.stop();
};

SaveConfigurationControl.prototype.startSaving = function() {
    this.isSaving = true;

    this.savingSpinner.spin("savingSpinner");

    this.controlBlock.addClass( "saving" );
    this.reportNameEdit.prop("disabled", true);
    this.addToDashboard.off( "click" );
    this.addToDashboardIcon.off( "click" );
    this.reportNameDiv.off( "click" );
    this.saveNewConfigButton.off( "click" );
    this.cancelNewConfigButton.off( "click" );
    this.saveCurrentConfigButton.off( "click" );
    this.cancelCurrentConfigButton.off( "click" );
};

SaveConfigurationControl.prototype.setReportConfigName = function( configName )
{
    this.reportConfigName = configName;
    this.reportNameDiv.text( configName );
};

SaveConfigurationControl.prototype._changeState = function( newState )
{
    if( this.currentState != null && this.currentState != newState )
    {
        this.controlBlock.removeClass( this.currentState.className );
        this.hideErrors();
        this.saveDialog.hideErrors();
    }
    this.controlBlock.addClass( newState.className );
    this.currentState = newState;
    this.currentState.updateUI( this );
};

SaveConfigurationControl.prototype._editNewConfig = function()
{
    this.reportNameEdit.val( trim(this.reportNameDiv.text()) );
    this._changeState( this.states.NEW_CONFIG_EDIT );
};

SaveConfigurationControl.prototype.showDisabledStateAlert = function() {
    var messageKey = this.reportType == ReportType.INVOICING ?
                     "report.prepare_invoice.cannot_save_incorrect_invoice" :
                     "report.prepare_invoice.cannot_save_incorrect_report";
    alert(messageResource.getMessage(messageKey));
};

SaveConfigurationControl.prototype.disableEditConfig = function()
{
    var self = this;
    var disabledState = {
        updateUI: function( control )
        {
            control.addToDashboard.off( "click" ).click( function() { self.showDisabledStateAlert(); } );
            control.addToDashboardIcon.off( "click" ).click( function() { self.showDisabledStateAlert(); } );
            control.reportNameDiv.off( "click" ).click( function() { self.showDisabledStateAlert(); } );
            control.saveNewConfigButton.off( "click" ).click( function() { self.showDisabledStateAlert(); } );
            control.cancelNewConfigButton.off( "click" );
            control.saveCurrentConfigButton.off( "click" ).click( function() { self.showDisabledStateAlert(); } );
            control.cancelCurrentConfigButton.off( "click" ).click( function() { control._cancelCurrentConfig(); } );
        }
    };
    disabledState.disabled = true;
    disabledState.wrappedState = this.currentState;
    disabledState.className = disabledState.wrappedState.className;
    disabledState.edit = disabledState.wrappedState.edit;
    this._changeState(disabledState);
};

SaveConfigurationControl.prototype.enableEditConfig = function()
{
    if ( this.currentState.disabled )
        this._changeState(this.currentState.wrappedState);
};

SaveConfigurationControl.prototype._editChangedConfig = function()
{
    this._changeState( this.states.CHANGED_CONFIG_EDIT );
    this.reportNameEdit.val( trim(this.reportNameDiv.text()) );
};

SaveConfigurationControl.prototype._saveNewConfig = function()
{
    var self = this;

    var newConfigName = trim(this.reportNameEdit.pVal());
    this.reportNameEdit.pVal(newConfigName);
    if(newConfigName == ""){
        this.errors[ SaveConfigurationControl.EMPTY_NAME ] = true;
        this.updateErrors();
        return;
    }
    this.saveCallback( newConfigName);
};

SaveConfigurationControl.prototype.updateErrors = function ()
{
    for( var p in this.errors ){
        this.controlBlock.toggleClass(p, this.errors[p]);
    }
};

SaveConfigurationControl.prototype.hideErrors = function ()
{
    for( var p in this.errors ){
        this.errors[p] = false;
    }
    this.updateErrors();
};

SaveConfigurationControl.prototype.displayError = function ( errorCode )
{
    this.errors[errorCode] = true;
    this.updateErrors();
};

SaveConfigurationControl.prototype.hideError = function ( errorCode )
{
    this.errors[errorCode] = false;
    this.updateErrors();
};

SaveConfigurationControl.prototype._cancelNewConfig = function()
{
    this._changeState( this.states.NEW_CONFIG );
};

SaveConfigurationControl.prototype._saveCurrentConfig = function()
{
    this.hideErrors();
    if(!this.saveDialog.hasErrors()){ // if there are no errors - update dialog
        if( this.currentState.edit ) {
            var reportEditName = trim(this.reportNameEdit.val());
            this.saveDialog.updateControlForUpdate( reportEditName );
            reportEditName !=  this.reportConfigName ?
             this.saveDialog.updateControlForNew( reportEditName ) : this.saveDialog.updateControlForNew("");
        } else {
            this.saveDialog.updateControlForUpdate( this.reportConfigName );
            this.saveDialog.updateControlForNew( "");
        }

    }
    this.saveDialog.showAt( this.saveCurrentConfigButton );
};

SaveConfigurationControl.prototype._cancelCurrentConfig = function()
{
    if(this.resetConfigCallback())
    {
        this.reportNameDiv.text( this.reportConfigName ); // with escape
        this._changeState( this.states.CURRENT_CONFIG );
    }
};

SaveConfigurationControl.prototype.getReportConfigName = function(){
    return this.reportConfigName;
};

SaveConfigurationControl.prototype.setStateCurrentConfig = function(){
    this._changeState( this.states.CURRENT_CONFIG );
};

SaveConfigurationControl.prototype.setStateNewEdit = function(){
    this._changeState( this.states.NEW_CONFIG_EDIT );
};

SaveConfigurationControl.prototype.setStateNew = function(){
    this._changeState( this.states.NEW_CONFIG );
};

SaveConfigurationControl.prototype.showError = function(error){
    this.saveDialog.showDialogError(error);
    this.displayError(error);
};

SaveConfigurationControl.prototype.hidePanel = function(){
   this.saveDialog.hide();
};

SaveConfigurationControl.prototype._saveChanges = function()
{
    if ( this.currentState.disabled ) {
        this.showDisabledStateAlert();
        return;
    }
    
    var newConfigName = this.saveDialog.getReportName();
    if(newConfigName == "")
    {
        this.saveDialog.showDialogError( SaveConfigurationControl.EMPTY_NAME );
    }
    if(this.saveDialog.hasErrors()){
        this.saveDialog.focusOnError();
        return;
    }
    this.hidePanel();
    // update names;
    this.reportNameEdit.val(newConfigName);
    this.reportNameDiv.text(newConfigName);

    if( this.saveDialog.isUpdate() )
       this.updateCallback( newConfigName );
    else
        this.saveCallback( newConfigName);
    return true;
};

/**
 *  if changes was performing during save - set changed config
 */
SaveConfigurationControl.prototype.setStateAfterSave = function() {
    if (this.changeWhileSaving) {
        this._changeState(this.states.CHANGED_CONFIG);
        this.changeWhileSaving = false;
    } else {
        this._changeState(this.states.CURRENT_CONFIG);
    }
};

SaveConfigurationControl.prototype.registerConfigChanged = function()
{
    if (this.isSaving)
        this.changeWhileSaving = true;

    var state = this.currentState;
    if (state.disabled)
        state = state.wrappedState;
    if (state == this.states.CURRENT_CONFIG)
        this._changeState(this.states.CHANGED_CONFIG);
    else if (state == this.states.CHANGED_CONFIG || state == this.states.NEW_CONFIG)
        this._changeState(state);
};
