BaseTableDataConverter = function(reportData, reportParams)
{
    this.reportData = reportData;
    this.reportParams = reportParams;
};

BaseTableDataConverter.prototype = {
    convert: function()
    {
        var groupingLevels = this.getGroupingLevels(this.reportParams);
        this.types = this.getTypes(groupingLevels);
        this.groupingTypes = this.getGroupingLevelsCollapsed(groupingLevels);

        var rows = [];
        rows.push(this._createHeader());
        rows.push(this._createDataRow(-1, BaseTableDataConverter.RowType.TOTALS, ReportGroupingLevel.NONE, this.reportData));

        this.addDataRecursive(this.reportData, rows, 0);
        return rows;
    },

    addDataRecursive: function(item, rows, level)
    {
        if (item.items.length == 0)
        {
            return;
        }

        _.each(item["items"], function(item)
        {
            var groupingType = this.groupingTypes[level];
            var type = this.types[level];
            rows.push(this._createDataRow(level, type, groupingType, item));
            this.addDataRecursive(item, rows, level + 1);
        }, this);
    },

    getTypes: function(groupingLevels)
    {
        var types = {};
        var level = 0;
        for (var i = 0; i < groupingLevels.length; ++i) {
            if (groupingLevels[i] != ReportGroupingLevel.NONE) {
                types[level] = i + BaseTableDataConverter.RowType.FIRST_LEVEL;
                level++;
            }
        }
        return types;
    },

    getGroupingLevelsCollapsed: function(groupingLevels)
    {
        return _.filter(groupingLevels, function(level) {return level != ReportGroupingLevel.NONE;})
    },

    _createHeader: function(reportData, reportParams)
    {
        // abstract
    },

    _createDataRow: function(level, type, groupingLevel, leavesReportItem)
    {
        // abstract
    },

    getGroupingLevels: function(params)
    {
        var array = [];
        array.push(params.firstGroupingLevel);
        array.push(params.secondGroupingLevel);
        return array;
    }
};


BaseTableDataConverter.RowType = {
    HEADER: 1,
    TOTALS: 2,
    FIRST_LEVEL: 3,
    SECOND_LEVEL: 4,
    THIRD_LEVEL: 5,
    FORTH_LEVEL: 6
};