function SectionsStateController() {}

SectionsStateController.prototype.init = function( cookieName )
{
    this.cookieName = cookieName;
    this.sectionBlockIds = [];
    this.sections = {};
    var self = this;
    $(".collapsibleBlock" ).each(function(){
        var blockId = this.id;
        self.sectionBlockIds.push(blockId);
        self.sections[this] = false;
        var blockButton = $('#'+blockId+'Button' );
        var parent = blockButton.parent();
        parent.click( function() {
            if ( !parent.hasClass("disabled") )
                self.showOrHideBlock(blockId, true);
        });
    });
    return this;
};

SectionsStateController.prototype.showOrHideBlock = function(blockId, updateExpandCollapseButtons)
{
    var block = $('#'+blockId);
    var isHiding = !block.hasClass('hiddenBlock');
    this.sections[blockId] = isHiding;
    $('#'+blockId+'Button').toggleClass("hiddenBlock", isHiding);
    block.toggleClass('hiddenBlock', isHiding);
    
    if ( updateExpandCollapseButtons ) 
        this.updateExpandCollapseButtonsVisibility();
    
    this.saveSectionsStateInCookies();
};

SectionsStateController.prototype.showBlock = function( blockId, updateExpandCollapseButtons )
{
    if (this.sections[blockId])
        this.showOrHideBlock(blockId, updateExpandCollapseButtons);
};

SectionsStateController.prototype.loadSectionsStateFromCookies = function()
{
    var sectionsFromCookiesStr = $.cookie( this.cookieName );
    var self = this;
    if (sectionsFromCookiesStr)
    {
        var sectionsFromCookies = sectionsFromCookiesStr.split(",");
        $.each(self.sectionBlockIds, function() {
            if ($.inArray( this.toString(), sectionsFromCookies ) != -1)
            {
                $("#"+this).addClass('withoutTransition');
                var sectionId = this;
                self.showOrHideBlock(this);
                window.setTimeout( function ()
                                   {
                                       $("#"+sectionId).removeClass('withoutTransition');
                                   }, 250 );
            }
        });
    }

    this.initExpandCollapseButtons();
};

SectionsStateController.prototype.saveSectionsStateInCookies = function() {
    var hiddenSectionsIds = [];
    var self = this;
    $.each(self.sectionBlockIds, function() {
        if (self.sections[this])
            hiddenSectionsIds.push(this);
    });
    $.cookie(this.cookieName, hiddenSectionsIds.join(","), { expires: 365 });
};

SectionsStateController.prototype.isSectionHidden = function( sectionId )
{
    return this.sections[sectionId];
};

SectionsStateController.prototype.baseOpenSectionsWithErrors = function( fieldErrors )
{
    for( var errorFieldName in fieldErrors )
    {
        var field = $( "[name='" + errorFieldName + "']" );
        this.showBlock( $( field ).closest( ".collapsibleBlock" )[0].id, false );
    }
    
    this.updateExpandCollapseButtonsVisibility();
};

SectionsStateController.prototype.expandAll = function () {
    this.toggleAll(true);
};

SectionsStateController.prototype.collapseAll = function () {
    this.toggleAll(false);
};

SectionsStateController.prototype.toggleAll = function ( expand ) {
    var self = this;
    $.each(self.sectionBlockIds, function() {
        if ( !!self.sections[this] == expand )
            self.showOrHideBlock(this, false);
    });

    this.updateExpandCollapseButtonsVisibility();
};

SectionsStateController.prototype.initExpandCollapseButtons = function () {
    var self = this;

    this.collapseButton = $("#collapseAllButton").click( function() {self.collapseAll();});
    this.expandButton = $("#expandAllButton").click( function() {self.expandAll();});
    
    $("#collapseMenuBlock").css("visibility", "visible");
    
    this.updateExpandCollapseButtonsVisibility();
};

SectionsStateController.prototype.updateExpandCollapseButtonsVisibility = function () {
    var self = this;
    var hiddenSectionsIds = [];

    $.each(self.sectionBlockIds, function() {
        if (self.sections[this])
            hiddenSectionsIds.push(this);
    });
    
    var showCollapse = hiddenSectionsIds.length < this.sectionBlockIds.length;
    var showExpand = hiddenSectionsIds.length > 0;

    if(this.collapseButton) {
        this.collapseButton.toggle(showCollapse);
    }
    if(this.expandButton){
        this.expandButton.toggle(showExpand);
    }
};
