function SendTestMailManager(hostName, portName,
                             isEncryptionEnabled, encryptionType,
                             isAuthenticationEnabled, username, password,
                             from, prefix, url, footer, states, statesManager, isOnline)
{
    RpcControlManager.call(this);
    this.hostName = hostName;
    this.portName = portName;
    this.isEncryptionEnabled = isEncryptionEnabled;
    this.encryptionType = encryptionType;
    this.isAuthenticationEnabled = isAuthenticationEnabled;
    this.username = username;
    this.password = password;
    this.from = from;
    this.prefix = prefix;
    this.url = url;
    this.footer = footer;
    this.states = states;
    this.statesManager = statesManager;
    this.isOnline = isOnline;

}

SendTestMailManager.prototype = new RpcControlManager();

SendTestMailManager.prototype.getService = function()
{
    return jsonrpc.SendTestMailService;
};

SendTestMailManager.prototype.getMethods = function()
{
    return [ 'SendTestMailService.sendTestMessage',
             'SendTestMailService.sendTestMessageInOnline' ];
};

SendTestMailManager.prototype.handleSpecificError = function(exception)
{
    if(exception){
        var additionalMessage = (exception.exceptionClass =='javax.mail.SendFailedException') ?
                                ". " + messageResource.getMessage( this.isOnline ? "general_settings.edit.mail.send_test_message.email_error_message_online" :
                                        "general_settings.edit.mail.send_test_message.email_error_message") : "";
        this.setState(this.states.error, exception.email, exception.message, additionalMessage);
        return true;
    }
    return false;
};

SendTestMailManager.prototype.getTimeoutMsg = function()
{
    return messageResource.getMessage("general_settings.edit.mail.send_test_message.timeout");
};

SendTestMailManager.prototype.makeRemoteCall = function(callback)
{
    if(!this.isOnline)
        return this.getService().sendTestMessage(callback, this.hostName, this.portName,
                                                 this.isEncryptionEnabled, this.encryptionType,
                                                 this.isAuthenticationEnabled, this.username, this.password,
                                                 this.from, this.prefix, this.url, this.footer);
    else
        return this.getService().sendTestMessageInOnline(callback, this.prefix, this.footer)
};

SendTestMailManager.prototype.beginSavingData = function()
{
    this.setState(this.states.sending);
};

SendTestMailManager.prototype.endSavingData = function(result)
{
    if( !result )
        this.setState(this.states.cancel);
};

SendTestMailManager.prototype.onDataSavingComplete = function(result)
{
    if(result.success)
        this.setState(this.states.success, result.sentTo);
    else
        this.setState(this.states.error, result.sentTo);
};

SendTestMailManager.prototype.setState = function(state, userEmail, errorMessage, additionalMessage)
{
    this.statesManager.setState( state, userEmail, errorMessage, additionalMessage );
};