/*
 * Updates hierarchy levels preview panel when value of select/input is changed and on load of the page (should be
 * registered on the page with setHandlersForControls(controls, func)). If changed control is select then function
 * show/hide appropriate inputs if it is needed. After preview updating checkDuplication() is called.
 */
function hierarchyLevelsPreviewUpdate() {
    if (isFormElement(this) && this.name != "" && typeof (this.name) != typeof undefined) {
        if (isSelectElement(this)) {
            showHideCustomATNamingInputs(this.name);
            markAsValidBundledWithSelectInputs(this.name);
        }

        printPreviewRow(this.name);
    } else {
        showHideCustomATNamingInputs(FIRST_HIERARCHY_LEVEL_SELECT_NAME);
        printPreviewRow(FIRST_HIERARCHY_LEVEL_SELECT_NAME);

        showHideCustomATNamingInputs(SECOND_HIERARCHY_LEVEL_SELECT_NAME);
        printPreviewRow(SECOND_HIERARCHY_LEVEL_SELECT_NAME);

        showHideCustomATNamingInputs(THIRD_HIERARCHY_LEVEL_SELECT_NAME);
        printPreviewRow(THIRD_HIERARCHY_LEVEL_SELECT_NAME);

        if( SUB ) {
            showHideCustomATNamingInputs(FOURTH_HIERARCHY_LEVEL_SELECT_NAME);
            printPreviewRow(FOURTH_HIERARCHY_LEVEL_SELECT_NAME);
        }
    }

    checkDuplication();
}


/*
 * Validates form on submit (should be registered as listener).
 */
function validateForm() {
    deleteMessages();
    removeClientSideErrors();

    validateAdminAddress();
    if(!ONLINE) {
        validateAddressFrom();
        validateUrlOfActitimeInstallation();
        validateSMTPPort();
    }
    if( EMA ) {
        validateDayTimeToSendNotificationAt();
        validateNotificationMessageFooter();
    }
    var result = checkMissedData();
    result = checkDuplication() && result;

    if( URA && OVR )
        validateRateFields();

    validateFormatFields();
    validateWorkdayDuration();
    validateMaximumMinutesPerDay();

    result = result && !ClientSideErrors.hasErrors();

    if (!result) {
        ClientSideErrors.showClientSideErrorMessage();
    }

    return result;
}

function validateNotificationMessageFooter()
{
    var footerStr = document.getElementById("mailMessageFooter").value;
    checkEmailMessageFooter ( footerStr );
}

function validateRateFields()
{
        checkRateCoefficient(document.getElementById("overtimeRateCoefficient"));
}

function validateMaximumMinutesPerDay()
{
    var field = document.getElementById("maximumMinutesPerDayId");
    if (document.getElementById("hoursPerDayLimited_LimitTo").checked)
        checkDuration(field);
    else
    {
        ClientSideErrors.markTextFieldAsValid(field);
        ClientSideErrors.markFieldLabelsAsValid(document.getElementById(field.name+"FieldLabel"));
        hideClientSideErrorMessageIfNoErrors();
    }
}

function validateDayTimeToSendNotificationAt()
{
    checkDayTimeToSendNotificationAt();
}

function validateSMTPPort()
{
    checkPort()
}

function validateAddressFrom()
{
    checkAddressFrom()
}

function validateUrlOfActitimeInstallation()
{
    checkUrlOfActitimeInstallation();
}

function validateAdminAddress()
{
    checkAdminAddress();
}


/*
 * Determines whether form fields has missed values, and if it is marks them as invalid.
 */
function checkMissedData() {
    var isValid = true;

    var emptyLevelFields = getEmptyCustomLevelFields();
    if (emptyLevelFields.length > 0) {
        markEmptyFieldsAsInvalid(emptyLevelFields, messageResource.getMessage("general_settings.edit.one_form_was_missed"));
        isValid = false;
    }

    isValid = isValid && checkMissedMailData(true);

    if (!isDecimalFieldIsNotEmpty())
    {
        var message = messageResource.getMessage("general_settings.edit.empty_decimal_field");

        ClientSideErrors.markTextFieldAsInvalid(document.getElementById(decimalSeparatorField.getCustomFieldName()), message);
        ClientSideErrors.markFieldLabelsAsInvalid(document.getElementById("decimalSeparatorFieldLabel"), message);
        isDecimalSeparatorRowHighlightedOnSubmit = true;
        isValid = false;
    }
    else
        isDecimalSeparatorRowHighlightedOnSubmit = false;

    if( URA || BRA ) {
        if (trim(currencySignField.getValue()).length == 0)
        {
            var currencyIsEmptyMessage = messageResource.getMessage("general_settings.edit.empty_currency_sign");

            ClientSideErrors.markTextFieldAsInvalid(document.getElementById(currencySignField.getCustomFieldName()), currencyIsEmptyMessage);
            ClientSideErrors.markFieldLabelsAsInvalid(document.getElementById("currencySignFieldLabel"), currencyIsEmptyMessage);
        }
    }

    isValid = checkDefaultTZGroupNameField(true) && isValid;

    return isValid;
}

function checkMissedMailData(checkUrl)
{
    var isValid = true;
    var isAdminAddressEmpty = adminMailAddressController.isEmpty();
    var settingsForm = document.getElementById('settingsForm');
    if(!ONLINE)
    {
        var isHostEmpty = smtpHostNameController.isEmpty();
        var isPortEmpty = isEqualsIgnoreCase(trim(settingsForm.smtpPort.value), "");
        var isFromEmpty = mailAddressFromController.isEmpty();
        var isUrlEmpty = isEqualsIgnoreCase(trim(settingsForm.urlOfActitimeInstallation.value), "");
        var isUsernameEmpty = smtpUsernameController.isEmpty();
        var isPasswordEmpty = smtpPasswordController.isEmpty() && !SAVED_SMTP_PASSWORD;

        //If all required fields are empty then Mail data is valid.
        if ( isHostEmpty && isFromEmpty && isAdminAddressEmpty &&
             (!isSMTPAuthenticationEnabled()) )
            return true;

        if(isHostEmpty)
        {
            markMailEmptyFieldAsInvalid(settingsForm.smtpHostName, messageResource.getMessage("general_settings.edit.empty_smtp_host"));
            isValid = false;
        }
        if(isPortEmpty)
        {
            markMailEmptyFieldAsInvalid(settingsForm.smtpPort, messageResource.getMessage("general_settings.edit.empty_smtp_port"));
            isValid = false;
        }
        if( isSMTPAuthenticationEnabled() )
        {
            if(isUsernameEmpty)
            {
                markMailEmptyFieldAsInvalid(settingsForm.smtpUsername, messageResource.getMessage("general_settings.edit.empty_smtp_username"));
                ClientSideErrors.markFieldLabelsAsInvalid(document.getElementById("loginInfoLabel"),
                                                          messageResource.getMessage("general_settings.edit.empty_smtp_login_info"),
                                                          FieldController.LABEL_ERROR_CLASS);
                isValid = false;
            }
            if(isPasswordEmpty)
            {
                markMailEmptyFieldAsInvalid(settingsForm.smtpPassword, messageResource.getMessage("general_settings.edit.empty_smtp_password"));
                ClientSideErrors.markFieldLabelsAsInvalid(document.getElementById("loginInfoLabel"),
                                                          messageResource.getMessage("general_settings.edit.empty_smtp_login_info"),
                                                          FieldController.LABEL_ERROR_CLASS);
                smtpPasswordLabelController.setDisplay(false);
                isValid = false;
            }
        }
        if(isFromEmpty)
        {
            markMailEmptyFieldAsInvalid(settingsForm.mailAddressFrom, messageResource.getMessage("general_settings.edit.empty_address_from"));
            mailAddressFromController.showGreyTextIfEmpty();
            isValid = false;
        }
        if(checkUrl && isUrlEmpty)
        {
            markMailEmptyFieldAsInvalid(settingsForm.urlOfActitimeInstallation, messageResource.getMessage("general_settings.edit.empty_url_of_actitime_installation"));
            isValid = false;
        }
        if(isAdminAddressEmpty)
        {
            markMailEmptyFieldAsInvalid(settingsForm.adminMailAddress, messageResource.getMessage("general_settings.edit.empty_admin_mail_address"));
            isValid = false;
        }
    }
    if( EMA )
    {
        var isDayTimeEmpty = isEqualsIgnoreCase(trim(settingsForm.dayTimeToSendNotificationAt.value), "");
        if( isDayTimeEmpty ) {
            markMailEmptyFieldAsInvalid(settingsForm.dayTimeToSendNotificationAt, messageResource.getMessage("general_settings.edit.empty_day_time"));
            isValid = false;
        }
    }
    return isValid;
}

function markMailEmptyFieldAsInvalid(field, message)
{
    field.value = "";
    ClientSideErrors.markFieldLabelsAsInvalid(document.getElementById(field.name + "FieldLabel"), message);
    ClientSideErrors.markTextFieldAsInvalid(field, message);
    setModificationsHandler(field);
}

function updateMaximumReportedHoursField(){
   document.getElementById("maximumMinutesPerDayId" ).disabled = !document.getElementById("hoursPerDayLimited_LimitTo").checked;
}

/**
 * Updates time format preview from current field values.
 */
function updateTimeFormatPreview()
{
    var previewField = document.getElementById('timeFormatPreviewField');

    var timeFormats = document.getElementById('settingsForm').timeFormat;
    var timeFormat;
    for(var i=0; i< timeFormats.length; i++)
    {
        timeFormat = timeFormats[i].checked ? timeFormats[i].value : timeFormat;
    }
    var text = "25:45";
    if(timeFormat == "decimal") text= "25" + decimalSeparatorField.getValue() + "75";
    previewField.innerHTML = text;
}

 var currencySignField;
 var decimalSeparatorField;
 var digitGroupingSymbolField;

/**
 * Initializes format symbols fields.
 *
 * @param decimalSeparatorFieldName - base name for custom and predefined decimal separator fields
 * @param decimalSeparatorInitialValue - initial value for decimal separator field
 * @param digitGroupingSymbolFieldName - base name for custom and predefined digit grouping symbol fields
 * @param digitGroupingSymbolInitialValue - initial value for digit grouping symbol field
 * @param currencySignFieldName - base name for custom and predefined currency sign fields;
 * @param currencySignInitialValue - initial value for currency sign field
 */
function initCustomizedFields(currencySignFieldName, currencySignInitialValue,
                                 decimalSeparatorFieldName, decimalSeparatorInitialValue,
                                 digitGroupingSymbolFieldName, digitGroupingSymbolInitialValue)
{
     currencySignField = null;
     if( URA || BRA )
         currencySignField = new CustomizableField(currencySignFieldName);

     decimalSeparatorField = new CustomizableField(decimalSeparatorFieldName);
     digitGroupingSymbolField = new CustomizableField(digitGroupingSymbolFieldName);

     decimalSeparatorField.initField(decimalSeparatorInitialValue);
     digitGroupingSymbolField.initField(digitGroupingSymbolInitialValue);

     if( URA || BRA )
        currencySignField.initField(currencySignInitialValue);

     validateFormatFields();
}

/**
 *  Hides error messages for currency sign if currency sign is valid.
 */
function hideCurrencyErrorsIfFieldValid()
{
    if (trim(currencySignField.getValue()).length > 0)
    {
        ClientSideErrors.markTextFieldAsValid(document.getElementById(currencySignField.getCustomFieldName()));
        ClientSideErrors.markFieldLabelsAsValid(document.getElementById("currencySignFieldLabel"));
    }
}

/**
 * Trims currency field custom value.
 */
function trimCustomFieldsValues()
{
    var field;

    field = document.getElementById(currencySignField.getCustomFieldName());
    field.value = trim(field.value);
}
