function SmartDate(getDateFieldFunc, getDateHolderFunc, dateFormat) {
    this.dateFormat = dateFormat;
    this.dateFieldFunc = getDateFieldFunc;
    this.dateHolderFunc = getDateHolderFunc;
    this.holderDateFormat = 'yyyy-MM-dd';
    this.updateHolder(this.parseDate());
}

SmartDate.prototype.getDateField = function()
{
    return this.dateFieldFunc();
};

SmartDate.prototype.getDateFieldValue = function()
{
    return this.dateFieldFunc().value;
};

SmartDate.prototype.getHolderValue = function()
{
    return this.dateHolderFunc().value;
};

SmartDate.prototype.isParseable = function()
{
    return this.parseDate() != null;
};

SmartDate.prototype.isEmpty = function()
{
    return trim(this.getDateFieldValue()).length == 0;
};

SmartDate.prototype.updateHolder = function(parsedDate)
{
    if (parsedDate == null)
    {
        this.setHolderValue('');
        return;
    }

    var df = new DateFormat(this.holderDateFormat);
    this.setHolderValue(df.formatDate(parsedDate));
};

SmartDate.prototype.setDate = function(date)
{
    if (date == null)
    {
        this.setDateFieldValue('');
        return;
    }
    var df = new DateFormat(this.dateFormat);
    this.setDateFieldValue(df.formatDate(date));
    this.updateHolder(date);
};

SmartDate.prototype.parseDate = function()
{
    if (this.isEmpty())
        return null;

    var date = parseDate(this.getDateFieldValue(), this.dateFormat);
    if (date)
    {
        var df = new DateFormat(this.dateFormat);
        this.setDateFieldValue(df.formatDate(date));
        return date;
    }

    return null;
};

SmartDate.prototype.parseDateAndUpdateHolder = function()
{
    var date = this.parseDate();
    this.updateHolder(date);
    return date;
};

SmartDate.prototype.setDateFieldValue = function(value)
{
    this.dateFieldFunc().value = value;
};

SmartDate.prototype.setHolderValue = function(value)
{
    this.dateHolderFunc().value = value;
};

SmartDate.prototype.checkDate = function(withAlerts)
{
    this.parseDateAndUpdateHolder();

    if (this.isEmpty()) return true;

    var date = this.parseDate();
    if (date == null)
    {
        if (withAlerts)
        {
            alert(messageResource.getMessage('common.js.invalid_date'));
            this.getDateField().focus();
        }
        return false;
    }

    return true;
};