function EntityStore() {}

EntityStore.prototype = {
    isIE8: isIE8(),

    init: function()
    {
        this._map = {};
        this._array = [];
        this._orderIndex = {};
        return this;
    },

    mergeAll: function(entities)
    {
        _.each(entities, function(entity)
        {
            this.add(entity)
        }, this)
    },

    replaceAll: function(entities)
    {
        this._map = {};
        this._array = [];
        this.mergeAll(entities);
    },

    add: function(entity)
    {
        if (!this.isIE8 && _.isObject(entity))
        {
            entity = Object.freeze(entity);
        }

        var id = this.getEntityId(entity);
        var oldEntity = this._map[id];

        if (!oldEntity)
        {
            this._array.push(entity);
            this._orderIndex[id] = this._array.length - 1;
            this._onAdd(entity, null, false);
        }
        else
        {
            this._array[this._orderIndex[id]] = entity;
            this._onAdd(entity, oldEntity, false);
        }

        this._map[id] = entity;
        return oldEntity;
    },

    addFirst: function(item)
    {
        this.addManyFirst([item]);
    },

    addManyFirst: function(entities)
    {
        var newArray = [];
        var newEntitiesSet = {};

        _.each(entities, function(entity)
        {
            if (_.isObject(entity))
                entity = Object.freeze(entity);

            var id = this.getEntityId(entity);
            if (!newEntitiesSet[id])
            {
                var oldEntity = this._map[id];
                if (oldEntity)
                {
                    this._array[this._orderIndex[id]] = entity;
                    this._onAdd(entity, oldEntity, true);
                }
                else
                {
                    newArray.push(entity);
                    this._orderIndex[id] = newArray.length - 1;
                    this._onAdd(entity, null, true);
                }

                this._map[id] = entity;
                newEntitiesSet[id] = true;
            }
        }, this);

        _.each(this._array, function(entity)
        {
            var id = this.getEntityId(entity);
            newArray.push(entity);
            this._orderIndex[id] = newArray.length - 1;
        }, this);

        this._array = newArray;
    },

    remove: function(id)
    {
        this.removeMany([id]);
    },

    removeMany: function(ids)
    {
        this._orderIndex = {};

        var idsToRemoveSet = arrayToSet(ids);
        var newArray = [];
        _.each(this._array, function(entity)
        {
            var id = this.getEntityId(entity);
            if (!idsToRemoveSet[id])
            {
                newArray.push(entity);
                this._orderIndex[id] = newArray.length - 1;
            }
            else
            {
                delete this._map[id];
                this._onRemove(entity);
            }
        }, this);

        this._array = newArray;
    },

    get: function(id)
    {
        return this._map[id];
    },

    getAll: function()
    {
        return this._array;
    },

    getEntityId: function(entity)
    {
        return entity
    },

    _onAdd: function(newEntity, oldEntity, first)
    {
    },

    _onRemove: function(entity)
    {
    }
};

function EntityWithIdStore()
{
    EntityStore.call(this);
}

EntityWithIdStore.prototype = _.create(EntityStore.prototype, {
    getEntityId: function(entity)
    {
        return entity.id
    }
});

function EntityWithNameStore()
{
    EntityWithIdStore.call(this);
}

EntityWithNameStore.prototype = _.create(EntityWithIdStore.prototype, {
    init: function()
    {
        EntityStore.prototype.init.call(this);
        this._stringIndex = new StringIndex();
        return this;
    },

    searchByName: function(name, sorted)
    {
        var resultObject = this._stringIndex.search(this._stringIndex.getTokenizer().getTokens(name.toLowerCase()));
        var resultArray = _.keys(resultObject);
        return sorted ? _.sortBy(resultArray, function(id) { return this._orderIndex[id] }, this) : resultArray;
    },

    getEntityName: function(entity)
    {
        return entity.name
    },

    _onAdd: function(newEntity, oldEntity)
    {
        if (oldEntity)
            this._removeFromStringIndex(oldEntity);
        this._stringIndex.add(this.getEntityName(newEntity), this.getEntityId(newEntity));
    },

    _onRemove: function(entity)
    {
        this._removeFromStringIndex(entity);
    },

    _removeFromStringIndex: function(entity)
    {
        this._stringIndex.remove(this.getEntityName(entity), this.getEntityId(entity));
    }
});