ProjectStore = _.create(EntityWithNameStore.prototype, {
    init: function()
    {
        EntityWithNameStore.prototype.init.call(this);
        this._customerProjects = {};
        return this;
    },

    getByCustomer: function(customerId)
    {
        var projectsByCustomer = this._customerProjects[customerId];
        return projectsByCustomer ? projectsByCustomer.getAll() : [];
    },

    _onAdd: function(newProject, oldProject, first)
    {
        EntityWithNameStore.prototype._onAdd.call(this, newProject, oldProject, first);

        if (oldProject && oldProject.customerId !== newProject.customerId)
            this._customerProjects[oldProject.customerId].remove(oldProject.id);

        var projectsByCustomer = this._customerProjects[newProject.customerId];
        if (!projectsByCustomer)
        {
            projectsByCustomer = new EntityWithIdStore().init();
            this._customerProjects[newProject.customerId] = projectsByCustomer;
        }

        if (first)
            projectsByCustomer.addFirst(newProject);
        else
            projectsByCustomer.add(newProject);
    },

    _onRemove: function(project)
    {
        EntityWithNameStore.prototype._onRemove.call(this, project);
        this._customerProjects[project.customerId].remove(project.id);
    }
}).init();

function Project(id, name, customerId, archived)
{
    this.id = id;
    this.name = name;
    this.customerId = customerId;
    this.archived = archived;
}

Project.fromRPC = function(data)
{
    return new Project(
            data["id"],
            data["name"],
            data["customerId"],
            _.isUndefined(data["archived"]) ? !data["activeProject"] : data["archived"]
    );
};