// requires:
//  leave.js

var oldKeydownEvent;
// current field value
var currentValue = null;
var dataModified = {};

var SubmitTTConstants = {};

var cursorMovingForbidden = false;

var lockedBySubmitting = false;

SubmitTTConstants.thirdHierarchyLevelNameSingular = "";
SubmitTTConstants.secondHierarchyLevelNameSingular = "";
SubmitTTConstants.secondHierarchyLevelNamePlural = "";
SubmitTTConstants.thirdHierarchyLevelNamePlural = "";
SubmitTTConstants.thirdHierarchyLevelNameSingularCapitalized = "";

var INVALID_VALUE_CLASS_NAME = 'invalidValue';

var TaskRowStatus = {
    SELECTED: -201,
    SLIGHTLY_SELECTED: -202,
    NORMAL: -203
};

/*
 Used to determine which row is currently selected.
 We need to control this not to move selection to another row if button's menu is currently shown
 */
var currentSelectedRow = null;
var currentSelectedRowStatus = TaskRowStatus.NORMAL;

var mouseOverRowTimeout = null;
var mouseOutRowTimeout = null;
var mouseWheelEngaged = 0;

var isConfirmationOnUnloadRequired = true; // true if we need to show warning on page unload

function getTaskIdFromTTField(field)
{
    if(field && field.name)
    {
        return parseInt(field.name.substring(10, field.name.indexOf("].spentStr[")));
    }
    return NaN;
}

function registerModifiedTTField(day, field)
{
    var taskId = getTaskIdFromTTField(field);
    if(field && !isNaN(taskId))
    {
        if(isNaN(taskId)) return;
        if(!dataModified[day])
            dataModified[day] = {};
        dataModified[day][taskId] = field;
    }
}

function unregisterModifiedTTField(day, field)
{
    var taskId = getTaskIdFromTTField(field);
    if(field && !isNaN(taskId))
    {
        if(dataModified[day])
            delete dataModified[day][taskId];
    }
}

function dayTTValuesWereChanged(day)
{
    if(!dataModified[day]) return false;
    var count = 0;
    for(var i in dataModified[day])
    {
        if(dataModified[day][i])
            count++;
    }
    return count > 0;
}

// TIME_TRACK FUNCTIONS
// returns day total for a given day. takes it from "day total" text element
function getDayTotal(day)
{
    var el = document.getElementById("dayTotal[" + day + "]");
    if(el)
        return UserTime.parseTimeWithSystemSettings(el.getAttribute("value"), true);
    else
        return 0;
}

function setDayTotal(day, dayTotal)
{
    el = document.getElementById("dayTotal[" + day + "]");
    if(el) {
        var total = UserTime.formatTimeWithDigitSeparator(dayTotal);
        el.firstChild.nodeValue = total;
        el.setAttribute("value", total);
    }
}

function updateWeekTotal(plusWeekTotal)
{
    el = document.getElementById("weekTotal");
    if(el)
    {
        var weekTotal = UserTime.parseTimeWithSystemSettings(el.getAttribute("value"), true);
        weekTotal += plusWeekTotal;
        weekTotal = UserTime.formatTimeWithDigitSeparator(weekTotal);
        el.firstChild.nodeValue = weekTotal;
        el.setAttribute("value", weekTotal);
    }
}

/**
 * @return int - leave time for the day. If leave time is full day, then returns workday duration.
 */
function getDayLeaveTime(day)
{
    var dayLeaveTime = LeaveButtons.getLeaveTimeForDay(day);
    if( dayLeaveTime == -1 || dayLeaveTime == null )
        return Overtime.workdayDuration[day];
    return Math.min( dayLeaveTime, Overtime.workdayDuration[day] );
}


function fieldModified(nowValue)
{
    return nowValue != currentValue;
}

// handles onchange event on time-track inputs.
// validates the input, updates day total, updates week total, sets/removes data modified flag,
// sets/removes overtime exclamation
function checkSpent(day, field, notifyAboutTimeEntriesChanged)
{
    if(field.value != field.defaultValue)
        registerModifiedTTField(day, field);
    else
        unregisterModifiedTTField(day, field);

    // if field was not modified, exit immediately
    if(!fieldModified(field.value)) return true;

    var ttValue = UserTime.parseTimeWithSystemSettings(field.value);
    if(spentTimeIsInvalid(ttValue))
    {
        // entered time-track value is invalid. mark field as erroneous.
        ClientSideErrors.markTextFieldAsInvalid(field, SubmitTTErrors.INCORRECT_TT_VALUE, INVALID_VALUE_CLASS_NAME);
        ttValue = 0;
    }
    else
    {
        // mark field as valid
        ClientSideErrors.markTextFieldAsValid(field, INVALID_VALUE_CLASS_NAME);
        // if field is not empty, format time-track
        if(!(trim(field.value) == ""))
            field.value = UserTime.formatTime(ttValue);
        else
            field.value = "";
    }
    SubmitTTChanges.registerChange(field.name);
    // set "modifications not saved" flag
    submitTTManager.handleChange(true, notifyAboutTimeEntriesChanged);
    updateTotal(day, ttValue, UserTime.parseTimeWithSystemSettings(currentValue));

    // update currentValue now, not waiting for the next onfocus, because onblur might be fired twice in Chrome
    currentValue = field.value;

    if (Overtime.overtimeShown) {
        Overtime.updateDayAndTotal(day);
    }

    SubmitTTErrors.hideClientSideErrorMessageIfNoErrors();

    return true;
}

function updateTotal(day, nowValue, previousValue)
{
    // get daytotal
    var dayTotal = getDayTotal(day);
    var diff = nowValue - ((previousValue == null || previousValue < 0) ? 0 : previousValue);
    dayTotal += diff;
    setDayTotal(day, dayTotal);
    var leaveTime = getDayLeaveTime(day);

    // clean previous error, if exists
    SubmitTTErrors.markDayAsValid(day);

    // if day total for a day is greater than limit, highlight an error, but only if TT for the day was changed 
    if (isHoursPerDayLimited && dayTotal > maxMinutesPerDay && dayTTValuesWereChanged(day))
        SubmitTTErrors.markDayTTAsInvalid(day);

    // if day leave time is greater than limit, highlight an error, but only if the leave info for the day was changed
    var maxLeaveDuration = Overtime.holidays[day] ? Overtime.maxLeaveDuration[day] : Overtime.workdayDuration[day];
    if (leaveTime > maxLeaveDuration && LeaveButtons.isLeaveInfoChanged(day))
        SubmitTTErrors.markDayLeaveInfoAsInvalid(day);

    updateWeekTotal(diff);
}

function onkeyupHandler(evt)
{
    var defaultValue = currentValue; // variable is needed for firefox, because it lost "currenValue" variable before "field.value != defaultValue" statement
    var event = evt ? evt : window.event ? window.event : null;
    if(event)
    {
        var field = event.target ? event.target : event.srcElement;
        if(field && field.type && field.type == 'text' && field.name &&
           field.form == document.SubmitTTForm &&
           field.value != defaultValue) {
            submitTTManager.handleChange(true, true);
            SubmitTTChanges.registerChange(field.name);
        }
    }
    return true;
}

// handler for onfocus event on time-track input fields. saves the value to the global variable
function focusSpent(field)
{
    currentValue = field.value;
}

// calculates total for a given task
function calculateTimeTrack(taskId)
{
   var total = 0;
   for(i = 0; i < 7; i++)
   {
        if(document.SubmitTTForm.elements["timeTrack[" + taskId + "].spentStr[" + i + "]"])
            total += UserTime.parseTimeWithSystemSettings(document.SubmitTTForm.elements["timeTrack[" + taskId + "].spentStr[" + i + "]"].value);
   }
   return total;
}

/**
 * Returns true if task marked for deletion
 * @param taskId is id of the task to get status for. Not null.
 */
function isTaskMarkedForDeletion(taskId)
{
    var prop = document.getElementById("isRowDeleted"+taskId);
    if (!prop)
        return false;

    return prop.value == "1";
}

/**
 * Toggles "marked for deletion" flag for the specified time track.
 * Actual deletion will happen when form is submitted.
 * If this method is called for a task that is already marked for deletion, the flag will be reseted.
 */

function deleteTimeTrack(taskId)
{
    if (!document.getElementById("isRowDeleted"+taskId))
        return false;  //there is no task with specified id at the page.

    if (!closeAllPopups()) {
        return false;
    }

    var isMarkedForDeletion = isTaskMarkedForDeletion(taskId);

    var deletionConfirmed = true;
    var taskHasTimeTrack = calculateTimeTrack(taskId) != 0;
    if (!isMarkedForDeletion)
    {
        if(taskHasTimeTrack || commentsPresent(taskId))
        {
            forbidMovingCursor();
            deletionConfirmed = confirm( messageResource.getMessage( 'submit_tt.delete_tt_confirm.task' ) );
            allowMovingCursor();
        }
        else
            deletionConfirmed = true;
    }

    if (deletionConfirmed)
    {
        submitTTManager.handleChange(true, taskHasTimeTrack);
        SubmitTTChanges.registerChange("rowDeleted_"+taskId);

        isMarkedForDeletion = !isMarkedForDeletion;

        // here we check if timeTrack to be deleted is the only modified data
        for(i = 0; i < 7; i++)
        {
            var ttInputName = "timeTrack[" + taskId + "].spentStr[" + i + "]";
            ClientSideErrors.unregisterFieldError(ttInputName);
        }

        SubmitTTErrors.hideClientSideErrorMessageIfNoErrors();

        setEnabledStatusForRowItems(taskId, !isMarkedForDeletion);

        for (var day=0; day < 7; day++)
        {
            var spentField = document.getElementById("spent_" + taskId + "_" + day);

            if (!spentField)
                continue;

            var ttvalue = spentField.value;

            registerModifiedTTField(day, spentField);

            if (isMarkedForDeletion)
            {
                updateTotal(day, 0, UserTime.parseTimeWithSystemSettings(ttvalue));
            }
            else
            {
                currentValue = 0;
                // do not notify about time entries changing, we have done it earlier
                // when calling submitTTManager.handleChange(true, taskHasTimeTrack);
                checkSpent(day, spentField, false);
            }

            if (Overtime.overtimeShown) {
                Overtime.updateDayAndTotal(day);
            }
        }

        SubmitTTErrors.hideClientSideErrorMessageIfNoErrors();
        return true;
    } else {
        return false;
    }
}

var taskRowsInitialTitle = {};

/**
 * Sets enabled status for elements of row with specified taskId
 * @param taskId is id of task to set disabled element status for.
 * @param isEnabled true if function must set status to 'enable', false otherwise.
 */
function setEnabledStatusForRowItems(taskId, isEnabled)
{
    document.getElementById("isRowDeleted"+taskId).value = isEnabled ? "0" : "1";

    var taskRow = document.getElementById("taskRow"+taskId);

    if (isEnabled)
    {
        replaceClass(taskRow, "rowToDelete", "actualRow");
        taskRow.title = '';
    }
    else
    {
        replaceClass(taskRow, "actualRow", "rowToDelete");
        taskRow.title = messageResource.getMessage('submit_tt.row_marked_for_removal.task' );
    }

    var deadlineBlock = document.getElementById('deadline' + taskId);

    if (isEnabled)
    {
        rowToRemoveCellsTitle = taskRowsInitialTitle[taskId].names;
        deadlineCellsTitle = taskRowsInitialTitle[taskId].deadline;
    }
    else
    {
        taskRowsInitialTitle[taskId] = {};
        taskRowsInitialTitle[taskId].names = document.getElementById('taskNameCell' + taskId).title;

        if (deadlineBlock)
            taskRowsInitialTitle[taskId].deadline = deadlineBlock.title;
        
        rowToRemoveCellsTitle = messageResource.getMessage('submit_tt.row_marked_for_removal.task' );
        deadlineCellsTitle = rowToRemoveCellsTitle;
    }

    document.getElementById('taskNameCell' + taskId).title = rowToRemoveCellsTitle;
    if (deadlineBlock)
        deadlineBlock.title = deadlineCellsTitle;

    for (var day = 0; day < 7; day++)
    {
        var spentField = document.getElementById("spent_" + taskId + "_" + day);

        if (spentField)
        {
            ClientSideErrors.paintElementAsValid(spentField);

            if (isEnabled)
            {
                spentField.readOnly = "";
                spentField.title = "";
                spentField.className = spentField.className.replace(" modifiedSpentField", "");
            }
            else
            {
                spentField.readOnly = "true";

                if ( spentField.value.length > 0 )
                {
                    spentField.className = spentField.className + " modifiedSpentField";
                }

                spentField.title = rowToRemoveCellsTitle;
            }
        }

        var commentImg = document.getElementById("timeTrack["+taskId+"].commentImg["+day+"]");

        if (commentImg)
        {
            if (isEnabled)
            {
                if (commentImg.src.indexOf("_disabled.png") > 0)
                    commentImg.src = commentImg.src.replace("_disabled.png", ".png");

                commentImg.title = messageResource.getMessage("submit_tt.description_tooltip");
            }
            else
            {
                commentImg.src = commentImg.src.replace(".png", "_disabled.png");
                commentImg.title = messageResource.getMessage('submit_tt.row_marked_for_removal.task' );
            }
        }
    }
}

/**
 * Toggles "marked for completion" flag for given task.
 * Completion will be actually performed after the form is submitted.
 * If this method is called for a task that is already marked for completion, the flag will be reseted.
 * @param taskId Task ID
 */
function completeTask(taskId) {
    if (!document.getElementById("isTaskCompleted"+taskId))
        return false;  //there is no task with specified id at the page.

    if (!closeAllPopups()) {
        return false;
    }

    var wasMarkedForCompletion = isTaskMarkedForCompletion(taskId);
    submitTTManager.handleChange(true, false);
    SubmitTTChanges.registerChange("taskCompleted_"+taskId);
    setCompletedStatusForRowItems(taskId, !wasMarkedForCompletion);
    SubmitTTErrors.hideClientSideErrorMessageIfNoErrors();
    return true;
}

/**
 * Returns true if specified task is marked for completion
 * @param taskId Task ID
 */
function isTaskMarkedForCompletion(taskId) {
    var prop = document.getElementById("isTaskCompleted"+taskId);
    if (!prop) return false;

    return (prop.value == 1);
}

/**
 * Returns true if specified task belongs to an archived project.
 * @param taskId Task ID
 */
function isTaskInArchivedProject(taskId) {
    var prop = document.getElementById("isInArchivedProject"+taskId);
    if (!prop) return false;

    return (prop.value == 1);
}

/**
 * Set style and status for row elements corresponding to given task.
 * @param taskId Task ID
 * @param isMarkedForCompletion Whether the task is marked for completion
 */
function setCompletedStatusForRowItems(taskId, isMarkedForCompletion) {
    document.getElementById("isTaskCompleted"+taskId).value = isMarkedForCompletion ? "1" : "0";

    var taskRow = document.getElementById("taskRow"+taskId);

    if (isMarkedForCompletion)
    {
        replaceClass(taskRow, "actualRow", "rowToComplete");
    }
    else
    {
        replaceClass(taskRow, "rowToComplete", "actualRow");
    }
}

function initDeletedTaskRow(taskId)
{
    for (day=0; day < 7; day++)
    {
        var spentField = document.getElementById("spent_" + taskId + "_" + day);

        if (!spentField)
            continue;

        var value = spentField.value;
        spentField.defaultValue = "";
        spentField.value = value;
    }

    setEnabledStatusForRowItems(taskId, false);
}


function SubmitTimeTrack(evt, hoursPerDayLimited, maximumMinutesPerDay, maximumMinutesPerDayString)
{
    if(!isLocked())
    {
        setLock();
        var event = evt ? evt : (window.event ? window.event : null);

        if ( editDescriptionState && editDescriptionState.isDescriptionModified() && !editDescriptionPopup.hide() ) {
                resetLock();
                return false;
        }

        if(SubmitTTErrors.hasErrors())
        {
            setRedirectUrl("");
            deleteMessages();
            ClientSideErrors.showClientSideErrorMessage();
            if(document.all && event != null)
                event.returnValue = false;
            resetLock();
            return false;
        }

        if (!LeavePopup.validateAndSave())
        {
            resetLock();
            return false;
        }

        document.SubmitTTForm.pageAction.value = 'save_tt';
        document.SubmitTTForm.submit();
        return true;
    }
    return false;
}

function isLocked()
{
    return lockedBySubmitting;
}

function setLock()
{
    lockedBySubmitting = true;

    $(".submitTTPanel").addClass("saving");
    
    setTimeout("resetLock()", 60000);
}

function resetLock()
{
    $(".submitTTPanel").removeClass("saving");
    lockedBySubmitting = false;
}

function registerHandlers()
{
    submitTTManager.registerFormChangedHandlers(true);
    document.body.onkeyup = onkeyupHandler;
    if(document.SubmitTTForm.formDataModified.value == "true")
    {
        submitTTManager.handleChange(false, document.SubmitTTForm.timeEntriesChanged ? document.SubmitTTForm.timeEntriesChanged.value == "true" : false);
    }

    ExitConfirmationHandlersManager.setUpExitConfirmation(messageResource.getMessage("submit_tt.modification_not_finished_page_unload_warning"), isConfirmationOnUnloadSubmitTTRequired, false);
}

function isConfirmationOnUnloadSubmitTTRequired() {
    return isConfirmationOnUnloadRequired && !isLocked() && SubmitTTChanges.hasChanges();
}

function commentsPresent(taskId) {
    var f = document.SubmitTTForm;
    for(var i = 0; i < 7; i++) {
        s = "timeTrack[" + taskId + "].hasComments[" + i + "]";
        if( f.elements[s] && f.elements[s].value == "true" )
               return true;
    }
    return false;
}

function updateLeaveTimeTotal()
{
    var container = document.getElementById( "leaveTimeTotal" );
    if( container )
    {
        var weekLeaveTotal = 0;
        for( var day = 0; day < 7; day++ )
        {
            weekLeaveTotal += getDayLeaveTime( day );
        }
        container.innerHTML = UserTime.formatTimeWithDigitSeparator( weekLeaveTotal );
    }
}

function onDayLeaveInfoChange(day)
{
    submitTTManager.handleChange(true, true);
    SubmitTTChanges.registerChange("leaveInfo_"+day);

    updateLeaveTimeTotal();

    // We call updateTotal in order to:
    //  - clean leave info error status, if exists
    //  - keep TT entries error status, if exists
    updateTotal(day, 0, 0);

    SubmitTTErrors.hideClientSideErrorMessageIfNoErrors();

    if (Overtime.overtimeShown) {
        Overtime.updateDayAndTotal(day);
    }
}


/**
 * Firefox remembers entered form data on exit. When the user starts the browser again, it reloads the page and
 * pre-fills text fields with remembered data. As a result, the user thinks that this data was loaded from
 * the server, but actually it exists only in browser.
 *
 * See bug #29419.
 *
 * This object is a workaround for this bug.
 */
var FirefoxFormCacheFix = {

    /**
     * Initializes the fix.
     * Must be called from the script placed just after the form.
     * Remembers the data loaded from the server, before the browser replaces it.
     * Also hides all the fields to prevent flickering.
     * @param form
     */
    init: function (form)
    {
        if (navigator.userAgent.toLowerCase().indexOf('firefox') == -1)
            return;

        this.form = form;
        this.fieldValues = {};
        this.fieldVisibilities = {};

        var inputFields = this.form.getElementsByTagName('input');

        for (var i = 0; i < inputFields.length; i++)
        {
            var element = inputFields[i];

            if (element.name)
            {
                this.fieldValues[element.name] = element.value;
                this.fieldVisibilities[element.name] = element.style.visibility;
                element.style.visibility = "hidden";
            }
        }
    },

    /**
     * Applies the fix - restores the form data and fields visibility remembered in the init method.
     * Must be called from the body onload handler (after the browser replaces the form data).
     */
    apply: function ()
    {
        if (! this.form || ! this.fieldValues || ! this.fieldVisibilities)
            return;

        for (var name in this.fieldValues)
        {
            if (this.form[name])
            {
                this.form[name].value = this.fieldValues[name];
                this.form[name].style.visibility = this.fieldVisibilities[name];
            }
        }
    }
}

function allowMovingCursor()
{
    cursorMovingForbidden = false;
}

function forbidMovingCursor()
{
    cursorMovingForbidden = true;
}
    
function isCursorMovingForbidden()
{
    return cursorMovingForbidden;
}

/**
 * Removes selection cursor if mouse leaves the table when dropdown menu is displayed.
 */
function hideSelectionCursorIfNeeded( event )
{
    if( isCursorMovingForbidden( ) ) return;

    event = event || window.event;

    if( currentSelectedRow != null )
    {
        var actualRowsBody = document.getElementById( "actualTTRows" );
        if( !isMouseOverElement( actualRowsBody, event ) )
        {
            return onMouseOutRow( currentSelectedRow, event );
        }
    }
}


function disableWarningOnUnloadPage() {
    isConfirmationOnUnloadRequired = false;
}

function SubmitTTManager() {}

SubmitTTManager.prototype.cancelButtonFunction = function () {
    if (lockedBySubmitting)
        return;

    if ( !confirm(messageResource.getMessage("submit_tt.cancel_alert")) )
        return;

    disableWarningOnUnloadPage();
    window.location.reload();
};

SubmitTTManager.prototype.setOnTimeEntriesChangedCallback = function ( callback )
{
    this.onTimeEntriesChangedCallback = callback;
};

SubmitTTManager.prototype._handleChange = function() {
    $("#enterTTMainContent").addClass("formModified");
    $("#SubmitTTButton" ).attr("disabled", false);
};

SubmitTTManager.prototype.handleChange = function ( shouldDeleteMessages, timeEntriesChanged ) {
    var self = this;
    handleChange(shouldDeleteMessages, function(){self._handleChange()});
    if (timeEntriesChanged && this.onTimeEntriesChangedCallback)
        this.onTimeEntriesChangedCallback();
};

SubmitTTManager.prototype.registerFormChangedHandlers = function ( shouldDeleteMessages ) {
    var self = this;
    registerFormChangedHandlers(shouldDeleteMessages, function(){self._handleChange()});
};

var submitTTManager = new SubmitTTManager();