function GettingStartedShortcuts(checkedShortcuts)
{
    var context = this;
    this.isMenuShowed = false;
    this.checkedShortcuts = checkedShortcuts;

    this.$gettingStartedMenuContainer = $("#gettingStartedShortcutsMenuWrapper");
    this.$gettingStartedPanel = $("#gettingStartedShortcutsPanelId");

    this.$gettingStartedPanel
            .on("click", function(event)
            {
                var $ignoredArea = context.$gettingStartedMenuContainer;
                if ($ignoredArea[0] != event.target && $(event.target).parents().index($ignoredArea) == -1)
                {
                    if (context.isMenuShowed)
                    {
                        context.hideMenuAnimated();
                        context._menuVisible(false);
                    }
                    else
                    {
                        context.showMenuAnimated();
                        context._menuVisible(true);
                    }
                }
            });

    $("#gettingStartedShortcutsMenuCloseId").on("click", function(event)
    {
        context.hideMenuAnimated();
        context._menuVisible(false);
        cancelBubble(event);
    });

    this.updatePanelCloseAbility();

    _.each(_.keys(GettingStartedShortcut), function(key){
        context._registerShortcut(GettingStartedShortcut[key]);
    });
}

GettingStartedShortcuts.prototype.showPanelAfterInit = function()
{
    this.$gettingStartedPanel.atAnimation().showAnimated("showPanelSlideUp", _.bind(this.showMenuAnimated, this));
    this._menuVisible(true);
};

GettingStartedShortcuts.prototype.showMenuIfNeed = function()
{
    if (this._menuVisible())
    {
        this.showMenu();
        this.isMenuShowed = true;
    }
};

GettingStartedShortcuts.prototype.showMenu = function()
{
    this.$gettingStartedMenuContainer.show();
    this.isMenuShowed = true;
};

GettingStartedShortcuts.prototype.showMenuAnimated = function(onShowCb)
{
    this.$gettingStartedMenuContainer.atAnimation().showAnimated("showSlideUpMenu", onShowCb);
    this.isMenuShowed = true;
};

GettingStartedShortcuts.prototype.hideMenuAnimated = function(onHideCb)
{
    this.$gettingStartedMenuContainer.atAnimation().hideAnimated("hideSlideDownMenu", onHideCb);
    this.isMenuShowed = false;
};

GettingStartedShortcuts.prototype._menuVisible = function(isVisible)
{
    try {
        var locStorage = window.localStorage;
        if (!_.isUndefined(isVisible))
        {
            isVisible ? (delete locStorage.gettingStartedShortcutsMenuHidden) : (locStorage.gettingStartedShortcutsMenuHidden = true);
        }
        return _.isUndefined(locStorage.gettingStartedShortcutsMenuHidden);
    } catch (e) { return true;}
};

GettingStartedShortcuts.prototype.showMenuWithMarkCheckedShortcut = function(shortcutType)
{
    var $checkedShortcut = this._getShortcutElement(shortcutType);
    // set initial states
    $checkedShortcut.toggleClass("checked", false);
    $checkedShortcut.toggleClass("hovered", true);

    var canUnHoverCheckedShortcut = false;
    var funForShowMarkedShortcut = function()
    {
        $checkedShortcut.toggleClass("checked", true);
        $checkedShortcut.toggleClass("hovered", false);
        $checkedShortcut.toggleClass("transformFromHoveredToChecked", true);
        canUnHoverCheckedShortcut = true;
    };
    // unhover checked shortcut if user move cursor under menu
    this.$gettingStartedPanel.on("mouseover", function()
    {
        if (!canUnHoverCheckedShortcut)
        {
            return;
        }
        $checkedShortcut.toggleClass("transformFromHoveredToChecked", false);
        $(this).off("mouseover");
    });

    if (this._menuVisible())
    {
        this.showMenu();
        setTimeout(funForShowMarkedShortcut, 400);
    }
    else
    {
        setTimeout(
                _.bind(
                        function()
                        {
                            this.showMenuAnimated(funForShowMarkedShortcut);
                        },
                        this
                ),
                500
        );
    }
};

GettingStartedShortcuts.prototype.isShortcutChecked = function(shortcutType)
{
    return !_.isUndefined(this.checkedShortcuts[shortcutType]);
};

GettingStartedShortcuts.prototype.addCheckedShortcuts = function(shortcutType)
{
    this.checkedShortcuts[shortcutType] = true;
};

GettingStartedShortcuts.prototype.updatePanelCloseAbility = function()
{
    var context = this;

    var canBeClosed = true;
    _.each(_.keys(GettingStartedShortcut), function(type){
        canBeClosed &= context.isShortcutChecked(type);
    });

    if (canBeClosed)
    {
        this.$gettingStartedPanel.removeClass("noCloseable");

        $("#gettingStartedShortcutsClose").off().on("click", function(event)
        {
            if (context.isMenuShowed)
            {
                context._menuVisible(false);
                context.hideMenuAnimated(function()
                                         {
                                             context.$gettingStartedPanel.remove();
                                         });
            }
            else
            {
                context.$gettingStartedPanel.remove();
            }
            makeRpcCall("GettingStartedService.hideShortcuts", [], _.noop(), function() {return true;});
            stopEventPropagation(event);
        });
    }
};

GettingStartedShortcuts.prototype._registerShortcut = function(shortcut)
{
    var shortcutType = shortcut.type;
    this._getShortcutElement(shortcutType)
            .toggleClass("checked", this.isShortcutChecked(shortcutType))
            .on("click", _.bind(
                    function(shortcutUrl)
                    {
                        window.location.href = shortcutUrl;
                    },
                    this,
                    shortcut.url
            ));
};

GettingStartedShortcuts.prototype._getShortcutElement = function(shortcutType)
{
    return $("#gettingStartedShortcutLink_" + shortcutType);
};

/**
 * Global function for mark shortcut as checked - stores checked shortcut in local storage
 * Method mark shortcut only if its not already checked
 * @param shortcut type of shortcut (in js - global object GettingStartedShortcut)
 */
GettingStartedShortcuts.markShortcutAsChecked = function(shortcut)
{
    if(window.gettingStartedShortcutsPanel)
    {
        if(!gettingStartedShortcutsPanel.isShortcutChecked(shortcut.type))
        {
            gettingStartedShortcutsPanel.addCheckedShortcuts(shortcut.type);
            try {
                window.localStorage.justCheckedShortcutType = shortcut.type;
            } catch (e) {}
        }
    }
};

/**
 * Show checked shortcut if it exist in local storage and remove from local storage ( only one read )
 * It need for show checked shortcut if page will be reloaded after mark shortcut as checked
 * This method call method from global shortcut panel "gettingStartedShortcutsPanel"
 */
GettingStartedShortcuts.showJustCheckedShortcutIfExists = function()
{
    if(window.gettingStartedShortcutsPanel)
    {
        try {
            if(!_.isUndefined(window.localStorage.justCheckedShortcutType))
            {
                gettingStartedShortcutsPanel.showMenuWithMarkCheckedShortcut(window.localStorage.justCheckedShortcutType);
                gettingStartedShortcutsPanel.updatePanelCloseAbility();
                delete window.localStorage.justCheckedShortcutType;
            }
        } catch (e) {}
    }
};