/**
 * @author filatov
 */
function InviteColleaguesBlock(){}

InviteColleaguesBlock.FIRST_NAME_EMPTY = "firstNameEmpty";
InviteColleaguesBlock.LAST_NAME_EMPTY = "lastNameEmpty";
InviteColleaguesBlock.EMAIL_EMPTY = "emailEmpty";
InviteColleaguesBlock.EMAIL_INVALID = "emailInvalid";
InviteColleaguesBlock.USER_DISABLED_BEACAUSE_LICENSE ="userIsDisabledBecauseLicense";

InviteColleaguesBlock.prototype.init = function(config){
    this.containerId = config.containerId;
    this.containerDiv =  $("#inviteUserFromHelpPopup");
    if(this.containerDiv.length == 0){
        return;
    }
    this.errors = {};
    this.states = {
         beginnerState : "beginnerState",
         fillUserFieldsState : "fillUserFieldsState",
         sendInvitationState : "sendInvitationState",
         invitationWasSendState:"invitationWasSendState"
    };
    this.curState = this.states.beginnerState;
    this.initInviteUsersContainer();
    this.changeState(this.curState);
    return this;
};

InviteColleaguesBlock.prototype.changeState = function(state) 
{
    // Container's parent is also granted with state class (with suffix),
    // because container siblings' styles can also change accordingly
    this.containerDiv.removeClass(this.curState);
    this.containerDiv.parent().removeClass("inviteUserFromHelpPopup_" + this.curState);
    this.curState = state;
    this.containerDiv.addClass(this.curState);
    this.containerDiv.parent().addClass("inviteUserFromHelpPopup_" + this.curState);
};

InviteColleaguesBlock.prototype.initInviteUsersContainer = function(){
    var self = this;
   
    this.containerDiv.on("click", function(event){event.stopPropagation();}) ;
    this.beforeInviteDiv = this.containerDiv.find(".scrollInviteContainer");
    this.createAccountForColleagueButton = this.containerDiv.find(self.getFullIdElement("createAccountColleague"));
    this.cancelInviteColleagueButton = this.containerDiv.find(self.getFullIdElement("cancelInvitationButton"));
    this.sendInvitationButton = this.containerDiv.find(self.getFullIdElement("sendInvitationButton"));

    this.firstNameField = this.containerDiv.find(self.getFullIdElement("firstNameInviteMenu"));

    this.lastNameField = this.containerDiv.find(self.getFullIdElement("lastNameInviteMenu"));
    this.emailField = this.containerDiv.find(self.getFullIdElement("emailInviteMenu"));

    this.firstNameField.off();
    this.lastNameField.off();
    this.emailField.off();

    this.firstNameField.placeholder();
    this.lastNameField.placeholder();
    this.emailField.placeholder();

    this.firstNameField.on("selectstart", function(e){stopEventPropagation(e);});
    this.lastNameField.on("selectstart",function(e){stopEventPropagation(e);});
    this.emailField.on("selectstart", function(e){stopEventPropagation(e);});

    this.onBlurFieldHandler = function(){
        return function()
        {
            $(this).pVal( trim(this.value) );
        };
    };

    this.firstNameField.blur(this.onBlurFieldHandler());
    this.lastNameField.blur(this.onBlurFieldHandler());
    this.emailField.blur(this.onBlurFieldHandler());

    this.inviteAnotherPersonBtn = this.containerDiv.find(self.getFullIdElement("inviteAnotherPersonButton"));
    this.inviteAnotherPersonBtn.on("click", function(){
        self.clearInvitationFields();
        self.changeState(self.states.fillUserFieldsState);
    });


    this.closeInviteBtn = this.containerDiv.find(self.getFullIdElement("closeInviteButton"));
    this.closeInviteBtn.on("click", function(){
        self.clearInvitationFields();
        self.changeState(self.states.beginnerState);
        self.beforeInviteDiv.css("transform", "translate(0px, 0px)");
    });

    this.errorTooltip = new TableTooltip().init(this.containerId + "_inviteColleaguesErrorTooltip");
    this.errorTooltipPlaceholderText =  this.getElementById("inviteColleaguesErrorTooltipTextPlaceholder");


    this.initTooltipError(this.firstNameField, InviteColleaguesBlock.FIRST_NAME_EMPTY, "menu.support.item.invite_colleague.firstName_empty");
    this.initTooltipError(this.lastNameField, InviteColleaguesBlock.LAST_NAME_EMPTY, "menu.support.item.invite_colleague.lastName_empty");
    this.initTooltipError(this.emailField, InviteColleaguesBlock.EMAIL_EMPTY, "menu.support.item.invite_colleague.email_empty");
    this.initTooltipError(this.emailField, InviteColleaguesBlock.EMAIL_INVALID, "menu.support.item.invite_colleague.email_invalid");

    this.firstNameField.change(function(){self.hideError(InviteColleaguesBlock.FIRST_NAME_EMPTY)});
    this.lastNameField.change(function(){self.hideError(InviteColleaguesBlock.LAST_NAME_EMPTY)});
    this.emailField.change(function(){self.hideError(InviteColleaguesBlock.EMAIL_EMPTY); self.hideError(InviteColleaguesBlock.EMAIL_INVALID)});

    this.createAccountForColleagueButton.on("click", function(){
        if(self.curState == self.states.beginnerState){
            self.changeState(self.states.fillUserFieldsState);
            self.beforeInviteDiv.css("transform", "translate(0px, -115px)");
        }
    });

    this.cancelInviteColleagueButton.on("click", function(){
        if (self.curState == self.states.fillUserFieldsState)
        {
            self.changeState(self.states.beginnerState);
            self.beforeInviteDiv.css("transform", "translate(0px, 0px)");
        }
    });

    this.sendInvitationButton.on("click", function(){

        var hasError = false;
        var firstName = self.firstNameField.pVal();
        if(firstName == ""){
            self.addError(InviteColleaguesBlock.FIRST_NAME_EMPTY);
            hasError = true;
        }
        var lastName = self.lastNameField.pVal();
        if(lastName == ""){
            self.addError(InviteColleaguesBlock.LAST_NAME_EMPTY);
            hasError = true;
        }
        var email = self.emailField.pVal();
        if(email == ""){
            self.addError(InviteColleaguesBlock.EMAIL_EMPTY);
            hasError = true;
            return;
        }
        if(!isEmailValid(email)){
            self.addError(InviteColleaguesBlock.EMAIL_INVALID);
            hasError = true;
        }
        if(!hasError){
            self.changeState(self.states.sendInvitationState);
            self.sendInvitation(firstName,lastName,email);
        }
    });
};

InviteColleaguesBlock.prototype.clearInvitationFields = function(){
    this.firstNameField.pVal('');
    this.lastNameField.pVal('');
    this.emailField.pVal('');
};

InviteColleaguesBlock.prototype.sendInvitation = function(firstName, lastName, email){
    var self = this;
    this.hideError(InviteColleaguesBlock.USER_DISABLED_BEACAUSE_LICENSE);
    makeRpcCall("UserManagementService.sendInvitation",[{firstName:firstName, lastName:lastName, email:email}],
                function(result){
                    self.containerDiv.find(self.getFullIdElement("invitedText")).html(
                            "<span class='firstName'>" + firstName + "</span> <span class='lastName'>" + lastName + "</span><span> " + messageResource.getMessage(
                                    "menu.support.item.invite_colleague.invitedText.text")+"</span>");
                    if (result.codeResult == InviteColleaguesResult.WARNING_ACCOUNT_DISABLED_DUE_TO_LICENSE_TERMS)
                    {
                        self.addError(InviteColleaguesBlock.USER_DISABLED_BEACAUSE_LICENSE);
                        self.containerDiv.find(self.getFullIdElement("errorInviteText")).html(
                                messageResource.getMessage("menu.support.user_disabled.license_maximum"));
                    }
                    self.changeState(self.states.invitationWasSendState);
                },
                function ( jqXHR, textStatus, errorThrown ) {
                    if( errorThrown.exception )
                    {
                        if (errorThrown.exception && errorThrown.exception.indexOf('com.actimind.common.services.UserEmailInvalidException') != -1)
                        {
                            self.changeState(self.states.fillUserFieldsState);
                            self.addError(InviteColleaguesBlock.EMAIL_INVALID);
                            return true;
                        }
                        if (errorThrown.exception && errorThrown.exception == 'com.actimind.actitime.services.NoRightsToManageAccountSettings')
                        {
                            alert(messageResource.getMessage('menu.support.not_rights_invite_account'));
                            document.location.reload();
                            return true;
                        }
                    }
                    self.changeState(self.states.fillUserFieldsState);
                });

};

InviteColleaguesBlock.prototype.initTooltipError = function(inputField, error, messageTextKey){
    var self = this;
    inputField.hover(function(event)
                     {
                         if (self.errors[error])
                         {
                             self.errorTooltipPlaceholderText.html(messageResource.getMessage(messageTextKey));
                             self.errorTooltip.updateUIAndShow({}, calculateTooltipOffset(self.errorTooltip, event));
                             self.errorTooltip.show();
                         }
                     },
                     function(){
                         self.errorTooltip.hide();
                     }
    );
    inputField.mousemove(function(event){
        self.errorTooltip.move( calculateTooltipOffset( self.errorTooltip, event) );
    });

};

InviteColleaguesBlock.prototype.addError = function(error){
    this.errors[error] = true;
    this.containerDiv.addClass(error);
};

InviteColleaguesBlock.prototype.hideError = function(error){
    if(this.errors[error])
    {
        this.errors[error] = false;
        this.containerDiv.removeClass(error);
    }

};

InviteColleaguesBlock.prototype.getElementById = function(idElement){
    return $(this.getFullIdElement(idElement));
};

InviteColleaguesBlock.prototype.getFullIdElement = function(idElement){
    return "#" + this.containerId + "_" + idElement;
};



