/**
 * @author filatov
 */

function SubmitContactBlock(){}

SubmitContactBlock.prototype.init = function(idSuffix, showDropdownToTop, fixedPositionOfDropdown){

    var self = this;
    this.requestPhoneCallContainer = $("#requestCallPhoneContainer" + idSuffix);
    if(this.requestPhoneCallContainer.length == 0){
        return;
    }
    this.requestPhoneCallContainer.on("click", function(e){e.stopPropagation()});

    this.closePhoneButton =  this.requestPhoneCallContainer.find("#closePhoneWasSendButton" + idSuffix);
    this.closePhoneButton.on("click", function(){
        self.requestPhoneCallContainer.find(".phoneWasSendBlock").hide();
        self.requestPhoneCallContainer.find(".phoneInputBlock").show();
        self.phoneInput.pVal('');
        self.unHighlightBlockIfNeeded(false);
    });

    this.tryAgainButton = this.requestPhoneCallContainer.find(".pleaseTryAgainButton");
    this.tryAgainButton.on("click", function(){
        self.requestPhoneCallContainer.find(".inputPhoneContainer").show();
        self.requestPhoneCallContainer.find(".errorAnswer").hide();
        self.unHighlightBlockIfNeeded(true);
    });
    var phoneCountries = [];
    $.each(countries, function(index, country){
        if(country.phoneCodes.length > 0){
            for(var i = 0; i < country.phoneCodes.length; ++i){
                phoneCountries.push({
                                        id: country.id,
                                        name: country.shortName ? country.shortName : country.name,
                                        phoneCode: country.phoneCodes[i]
                                    }
                )
            }
        }
    });

    var charFilter = function(charCode){
        var sym = String.fromCharCode( charCode );
        return sym >= '0' && sym <= '9'
    };
    this.phoneInput = this.requestPhoneCallContainer.find("#phoneInput" + idSuffix);
    this.phoneInput.off();
    this.phoneInput.placeholder();
    this.phoneInput.focus(function(){self.highlightBlock()   });
    this.phoneInput.blur(
            function(){
                $(this).pVal( trim(this.value) );
                self.unHighlightBlockIfNeeded(false);
                self.phoneInput.removeClass("empty");
            }
    );
    this.phoneInput.on("change", function()
    {
        self.phoneInput.removeClass("empty");
    });
    this.phoneInput.on("selectstart", function(e){stopEventPropagation(e);});
    new SmartField(this.phoneInput,charFilter, function(){});
    this.phoneInput.on( 'input', function()
    {
        if(!isNumber(this.value))
        {
            this.value = this.value.replace(/[^/0-9]/g, '');
        }
        if(this.value != "")
        {
            self.phoneInput.removeClass("empty");
        }
    } );

    this.countryCodeSelector = new CodePhoneSelector().init(
            {
                containerId: "countryCodePlaceholder" + idSuffix, options: phoneCountries,
                createNameFunction: function(country) {return country.name + ' (+' + country.phoneCode + ')'},
                setInputByValueFunction: function(country) {return "+" + country.phoneCode},
                getTokenFunction: function(country) { return "" + country.phoneCode},
                predefinedValue: "+1",
                showToTop: showDropdownToTop,
                isFixedDropDown: fixedPositionOfDropdown || false
            }
    );

    this.sendButton = this.requestPhoneCallContainer.find("#sendPhoneButton" + idSuffix);
    this.sendButton.on("click", function(){
        var contact = self.phoneInput.pVal();
        var hasError = false;
        if(contact.length == 0){
            self.phoneInput.addClass("empty");
            hasError = true;
        }
        var countryCode = self.countryCodeSelector.getValue();
        if(trim(countryCode) ==""){
            self.countryCodeSelector.addEmptyError("empty");
            hasError = true;
        }
        if (!hasError)
            self.sendPhoneCall(countryCode+"-" + contact);
    });
};

SubmitContactBlock.prototype.getPhoneNumber = function(){
   return trim(this.phoneInput.pVal());
};

SubmitContactBlock.prototype.onBlockHighlightChanged = function(isHighlight){};

SubmitContactBlock.prototype.highlightBlock = function(){
    this.requestPhoneCallContainer.toggleClass("submitContact", true);
    this.onBlockHighlightChanged(true);
};

SubmitContactBlock.prototype.unHighlightBlockIfNeeded = function(ignoreValCheck){
    if(ignoreValCheck || this.getPhoneNumber() == '')
    {
        this.requestPhoneCallContainer.removeClass("submitContact");
        this.onBlockHighlightChanged(false);
    }
};

SubmitContactBlock.prototype.sendPhoneCall = function(phoneNumber){
    var self = this;
    this.requestPhoneCallContainer.find(".phoneInputBlock").hide();
    this.requestPhoneCallContainer.find(".phoneSendBlock").show();

    makeRpcCall("ContactSupportService.sendPhoneNumber", [phoneNumber],
                function(){
                    self.requestPhoneCallContainer.find(".phoneSendBlock").hide();
                    self.requestPhoneCallContainer.find(".phoneWasSendBlock").show();
                },
                function ( jqXHR, textStatus, errorThrown ) {
                    if( (textStatus == "timeout") || errorThrown.exception && errorThrown.exception.indexOf( 'com.actimind.common.integration.HMSPWSConnector$CommunicationException' ) != -1 )
                    {
                        self.requestPhoneCallContainer.find(".inputPhoneContainer").hide();
                        self.requestPhoneCallContainer.find(".phoneInputBlock").show();
                        self.requestPhoneCallContainer.find(".phoneSendBlock").hide();
                        self.requestPhoneCallContainer.find(".errorAnswer").show();
                        return true;
                    }
                    self.requestPhoneCallContainer.find(".phoneInputBlock").show();
                    self.requestPhoneCallContainer.find(".phoneSendBlock").hide();
                });
};

/**
 * returns true if drop down code selector is shown
 */
SubmitContactBlock.prototype.hide = function()
{
    if(this.countryCodeSelector.isDropDownShown())
    {
        this.countryCodeSelector.hide();
        return true;
    }
    return false;
};
