/**
 * Message manager (Requires "/js/message_resources.js" be included; uses messageResource object)
 * @param entityName name of form (bug/feature/question), used as message parameter
 * @param msgContainerId id of the element where messages from validator are displayed
 * @param clientSideErrorsContainerId id of the error container element
 */
function SupportFormMessageManager(entityName, msgContainerId, clientSideErrorsContainerId)
{

    this.entityName = entityName;   // Name of the form
    this.msgContainerId = msgContainerId;   // Container for the validator messages
    this.clientSideErrorsContainerId = clientSideErrorsContainerId;   // Container for the validator messages
}

SupportFormMessageManager.prototype = {
    /**
     * Hides error message
     */
    hideErrorMessage: function()
    {
        document.getElementById(this.clientSideErrorsContainerId).style.visibility = "hidden";
    },

    /**
     * Shows error message
     * @param errorMessage message to display
     */
    showErrorMsg: function(errorMessage)
    {
        var errorMsgContainer = document.getElementById(this.msgContainerId);
        errorMsgContainer.innerHTML = errorMessage;

        this.hideErrorMessage();
        var self = this;
        window.setTimeout(function()
                          {
                              var errorsContainer = document.getElementById(self.clientSideErrorsContainerId);
                              errorsContainer.style.display = "block";
                              errorsContainer.style.visibility = "visible";
                          }, 200);
    },

    /**
     * Shows empty field error message
     */
    showEmptyFieldMsg: function()
    {
        this.showErrorMsg(messageResource.getMessage("support.form.empty_field_error"));
    },

    /**
     * Shows invalid email error message
     */
    showInvalidEmailMsg: function()
    {
        this.showErrorMsg(messageResource.getMessage("support.form.ivalid_email_msg"));
    },

    /**
     * Shows message about too long bug/question/feature description
     */
    showTooLongDescMsg: function(maxLength)
    {
        this.showErrorMsg(messageResource.getMessage("support.form.long_desc_error", [this.entityName, maxLength]));
    },

    /**
     * Shows form submit error
     */
    showFormSubmitErrorMsg: function()
    {
        this.showErrorMsg(messageResource.getMessage("support.form.submit_error_msg", [this.entityName, PRODUCT_NAME]));
    },

    /**
     * Shows form submit error
     */
    showRedirectErrorMsg: function()
    {
        this.showErrorMsg(messageResource.getMessage("support.form.redirect_error_msg", [this.entityName, PRODUCT_NAME]));
    },

    /**
     * Shows error message if form submit has not been verified
     */
    showVerificationErrorMsg: function()
    {
        // Same as "submit error", subclasses override this method to provide specific error message
        this.showFormSubmitErrorMsg();
    }
};
