function WelcomeScreen(afterLoadDemoDataUrl, isNeedLoadDemoData)
{
    this.afterLoadDemoDataUrl = afterLoadDemoDataUrl;
    this.isNeedLoadDemoData = isNeedLoadDemoData;
}

WelcomeScreen.prototype = {
    start: function()
    {
        this.show();
        this.enableTips(true);
    },

    show: function()
    {
        // show moire on screen
        TransformedScreenMoire.show(1);

        this.welcomeScreenBoxId = "welcomeScreenBoxId";
        this._renderBoxContent();

        this.welcomeScreenBox = new DHTMLPopupWithMoire(this.welcomeScreenBoxId);
        this.welcomeScreenBox.hideOnMouseDown = function() { return false; };
        this.welcomeScreenBox.hideOnEscape = function() { return false; };
        this.welcomeScreenBox.hideOnResize = function() { return false; };

        var context = this;
        this.$startExploringLink.on("click", function(e)
        {
            context.performStartExploringAction();
            e.preventDefault();
        });
        $(document).on("keydown.welcomescreen", function(event)
        {
            if (event.keyCode == KEY_CODE_ENTER || event.keyCode == KEY_CODE_SPACE)
            {
                event.preventDefault();
                context.performStartExploringAction();
            }
        });

        $(document).on("ready", _.bind(this.startShowInitialScreenAction, this));
    },

    hide: function()
    {
        var self = this;
        this.welcomeScreenBox.hide(false);
        TransformedScreenMoire.hide("1s", "ease-in", null, function()
        {
            self.showFirstTimeTips();
        });
        this.rememberUserSeenWelcomeScreen();
    },

    enableTips: function(showLater)
    {
        PageOverlayTipsManager.enableCookie();
        PageOverlayTipsManager.enableFirstTimeCookie();
        if (showLater)
            PageOverlayTipsManager.enableShowLater();
    },

    showFirstTimeTips: function()
    {
        PageOverlayTipsManager.showFirstTimeTips();
    },

    rememberUserSeenWelcomeScreen: function()
    {
        makeRpcCall("WelcomeScreenService.checkThatCurrentUserSeenWelcomeScreen", [], _.noop(), function() {return true;});
    },

    _showInitialScreenState: function()
    {
        TransformedScreenMoire.transformTo(0.95, "1.5s", "ease-in-out");

        this.$welcomeScreenBox.addClass("initialState");
        this.$welcomeScreenBox.removeClass("loadDemoDataState");

        this.welcomeScreenBox.show(false);
        $(this.welcomeScreenBox.getMoireDiv()).hide();

        this.$welcomeScreenBox.atAnimation().showAnimated("startShowingWelcomeScreenBox");
    },

    startShowInitialScreenAction: function()
    {
        this._showInitialScreenState();
    },

    _showLoadDemoDataState: function()
    {
        this.$welcomeScreenBox.removeClass("initialState");
        this.$welcomeScreenBox.addClass("loadDemoDataState");
    },

    _renderBoxContent: function()
    {
        this.$welcomeScreenBox = $("<div id='" + this.welcomeScreenBoxId + "' class='welcomeScreenBox'></div>");

        var $header = $("<div class='welcomeScreenBoxHeaderContainer'></div>");
        this.fillWelcomeScreenBoxHeader($header);

        var $mainContent = $("<div class='welcomeScreenBoxMainContentContainer'></div>");
        this.fillWelcomeScreenBoxMainContentContainer($mainContent);

        this.$startExploringLink = $("<div class='startExploringLink'></div>")
                .append("<span class='startExploringText'>" + messageResource.getMessage("welcome_screen.intro.start_exploring_link") + "</span>")
                .append("<span class='startExploringArrow'>→</span>");
        var footerContainer = $("<div class='welcomeScreenBoxFooterContainer'></div>").append(this.$startExploringLink);

        this.$percentsPlaceholder = $("<div class='loadDemoDataProgressPercents'></div>");

        var $loadDemoDataProgressContainer = $("<div class='loadDemoDataProgressContainer'></div>")
                .append(
                        "<div class='loadProgressAnimateCircle1'>" +
                        "<div class='loadProgressAnimateCircle2'>" +
                        "<div class='loadProgressAnimateCircle3'>" +
                        "<div class='loadProgressAnimateCircle4'></div>" +
                        "</div>" +
                        "</div>" +
                        "</div>"
                )
                .append(this.$percentsPlaceholder)
                .append($("<div class='loadDemoDataLabel'>" + messageResource.getMessage("welcome_screen.load_demo_data.label") + "</div>"));
        if (!AtAnimation.isAnimationSupported()) // check that animation impossible
        {
            $loadDemoDataProgressContainer.addClass("noAnimation");
        }

        this.$welcomeScreenBox.append($header).append($mainContent).append(footerContainer).append($loadDemoDataProgressContainer).appendTo("body");
    },

    fillWelcomeScreenBoxHeader: function($headerContainer)
    {
        return $headerContainer.html(
                "<div class='firstPart'>" + messageResource.getMessage("welcome_screen.intro.header.part1") + "</div>" +
                "<div class='secondPart'>" + messageResource.getEscapedMessage("welcome_screen.intro.header.part2") + "</div>")
    },

    fillWelcomeScreenBoxMainContentContainer: function($mainContentContainer)
    {
        // abstract
    },

    performStartExploringAction: function()
    {
        $(document).off("keydown.welcomescreen");
        if (this.isNeedLoadDemoData)
        {
            this._showLoadDemoDataState();
            this._startLoadDemoDataProgress();
            this._loadDemoData();
        }
        else
        {
            this.hide();
        }
    },

    _startLoadDemoDataProgress: function()
    {
        var actualPercents = 0;
        var baseTimeOut = 200;
        var baseStep = 2;
        var context = this;
        var increasePercentsFunc = function()
        {
            context._setPercentsToLoadDemoDataProgress(actualPercents);
            // increase percent value from 0 to 95
            if (actualPercents < 95)
            {
                actualPercents += Math.round(baseStep + Math.random() * baseStep);
                context.percentsTimer = setTimeout(increasePercentsFunc, baseTimeOut + Math.random() * baseTimeOut);
            }
        };
        increasePercentsFunc();
    },

    _setPercentsToLoadDemoDataProgress: function(percents)
    {
        this.$percentsPlaceholder.text(percents + "%");
    },

    _loadDemoData: function()
    {
        var context = this;
        makeRpcCall(
                "DemoDataService.loadDemoData",
                [],
                function()
                {
                    context._setPercentsToLoadDemoDataProgress(100); // set finally percents
                    clearTimeout(context.percentsTimer);
                    window.location.href = context.afterLoadDemoDataUrl;
                },
                function()
                {
                    window.location.reload();
                    return false;
                }
        );
    }
};
