function SupportPopup()
{
    DHTMLPopupWithMoire.call(this, "contactSupportPopup");
}

SupportPopup.States = {
    MAIN: 0,
    PROGRESS: 1,
    SUCCESS: 2
};

SupportPopup.prototype = _.create(DHTMLPopupWithMoire.prototype, {
    init: function()
    {
        this.popupEl = $("#" + this.popupId);

        this.closeButton = document.getElementById("supportPopupCloseButton");
        $(this.closeButton).click(_.bind(this.cancel, this));

        this.submitButton = this.getElementBySubId("submitButton");
        $(this.submitButton).click(_.bind(this.onSubmitButtonClick, this));

        this.sendAgainButton = this.getElementBySubId("sendAgain");
        $(this.sendAgainButton).click(_.bind(this.onSendAgainButtonClick, this));

        this.textArea = this.getElementBySubId("description");

        this.textArea.onpaste = _.bind(this.updateTextLengthNote, this, true);
        this.textArea.onblur = _.bind(this.updateTextLengthNote, this, false);
        this.textArea.onkeyup = _.bind(this.updateTextLengthNote, this, false);

        this.textLengthNote = new TextAreaLengthNote(this.popupId + '_descLength',
                messageResource.getMessage("support.report_bug.text_area_name"),
                32000);
        this.textLengthNote.update(this.textArea.value);
        this.textLengthNote.show();

        this.messageManager = new SupportFormMessageManager(
                messageResource.getMessage("support.submitQuestion.entity"),
                this.popupId + "_errorMsgContainer",
                this.popupId + "_clientSideErrorMessage");
        this.formValidator = new SupportFormValidator(this.messageManager);
        this.formValidator.addTextArea(this.textArea);
        this.addChangesHandler(this.textArea);

        if (CONTACT_SUPPORT_SHOW_EXTRA_FIELDS)
        {
            this.popupEl.find(".contactInformationContainer").show();

            this.firstNameField = this.getElementBySubId("first_name");
            this.lastNameField = this.getElementBySubId("last_name");
            this.emailField = this.getElementBySubId("email");
            this.companyNameField = this.getElementBySubId("company_name");

            $(this.firstNameField).placeholder();
            $(this.lastNameField).placeholder();
            $(this.emailField).placeholder();
            $(this.companyNameField).placeholder();

            this.formValidator.addField(this.firstNameField);
            this.formValidator.addField(this.lastNameField);
            this.formValidator.addEmailField(this.emailField);
            this.formValidator.addField(this.companyNameField);

            this.addChangesHandler(this.firstNameField);
            this.addChangesHandler(this.lastNameField);
            this.addChangesHandler(this.emailField);
            this.addChangesHandler(this.companyNameField);
        }
        else
        {
            this.popupEl.find(".contactInformationContainer").hide();
        }

        this.stateSwitcher = this.popupEl.find(".innerContent").queueViewStateSwitcher({
                                                                                           index: SupportPopup.States.MAIN,
                                                                                           hideAnimatedClass: "anim-fade-out",
                                                                                           showAnimatedClass: "anim-fade-in"
                                                                                       });

        this.uploadId = 0;

        var self = this;
        this.dropzone = new Dropzone("#" + this.popupId + " .dropzone", {
            previewTemplate: document.querySelector('.dropzoneTemplate').innerHTML,
            url: UPLOAD_SUPPORT_FILE_URL,
            createImageThumbnails: false,
            maxFiles: 1,
            dictDefaultMessage: messageResource.getMessage("contact_support.main.dropzone_placeholder"),
            dictMaxFilesExceeded: messageResource.getMessage("common.multiple_upload_error"),
            fallback: function()
            {
                self.popupEl.find(".dropzoneContainer").hide();
            },
            accept: function(file, cb)
            {
                if (file.size == 0)
                    cb(messageResource.getMessage("contact_support.error.empty_file"));
                else if (file.size > 10 * 1024 * 1024)
                    cb(messageResource.getMessage("contact_support.error.large_file", [new NumberFormatter(".", "", 2).format(file.size / 1024.0 / 1024.0), 10]));
                else
                    cb();
            }
        });
        this.dropzone.on("success", function(file, response)
        {
            var error = false;
            try
            {
                response = JSON.parse(response);
                error = response.error;
            }
            catch (e)
            {
                error = true;
            }

            if (!error)
            {
                if (self.uploadId > 0) self.rpcCancel();
                self.uploadId = response.result;
                self.changesDetected();
            }
            else
            {
                self.onDropzoneError(file, messageResource.getMessage("contact_support.error.upload"));
            }
        });
        this.dropzone.on("error", function(file, error)
        {
            if (file.xhr)
            {
                var httpStatus = file.xhr.status;
                if (httpStatus == 0)
                    error = messageResource.getMessage("contact_support.error.http_0");
                else if (httpStatus >= 400 && httpStatus <= 499)
                    error = messageResource.getMessage("contact_support.error.http_4xx");
                else if (httpStatus >= 500)
                    error = messageResource.getMessage("contact_support.error.http_5xx");
            }

            self.onDropzoneError(file, error);
        });
        this.dropzone.on("removedfile", function()
        {
            self.rpcCancel();
            self.uploadId = 0;
            self.dropzone.setupEventListeners();
        });
        this.dropzone.on("maxfilesreached", function()
        {
            self.dropzone.removeEventListeners();
        });
        this.dropzone.on("sending", function()
        {
            self.changesDetected();
        });

        return this;
    },

    centrePopup: function()
    {
        if (!this._height)
        {
            var margin = 80;
            this._height = $(this.getElementBySubId("supportForm")).outerHeight() + margin;
        }

        this.popupEl.find(".content").css("height", this._height + "px");
        this.popupEl.find(".innerContent").css("height", this._height + "px");

        DHTMLPopupWithMoire.prototype.centrePopup.call(this);
    },

    show: function()
    {
        DHTMLPopupWithMoire.prototype.show.call(this, true);

        var self = this;
        ExitConfirmationHandlersManager.setUpExitConfirmation(messageResource.getMessage("contact_support.main.cancel_confirmation"),
                                                              function() { return self.dataChanged }, false);
    },

    hide: function()
    {
        ExitConfirmationHandlersManager.restoreExitConfirmation();

        DHTMLPopupWithMoire.prototype.hide.call(this, true);
        $(this.popupEl).empty();
    },

    cancel: function()
    {
        if (this.stateSwitcher.index == SupportPopup.States.MAIN && this.dataChanged && !confirm(messageResource.getMessage("contact_support.main.cancel_confirmation")))
            return;

        if (this.stateSwitcher.index == SupportPopup.States.PROGRESS && !confirm(messageResource.getMessage("contact_support.progress.cancel_confirmation")))
            return;

        var uploadingFile = this.dropzone.getUploadingFiles()[0];
        if (uploadingFile)
            this.dropzone.cancelUpload(uploadingFile);

        this.rpcCancel();
        this.hide();
    },

    rpcSend: function()
    {
        var params = CONTACT_SUPPORT_SHOW_EXTRA_FIELDS ?
                [this.textArea.value,
                 this.firstNameField.value,
                 this.lastNameField.value,
                 this.emailField.value,
                 this.companyNameField.value,
                 this.uploadId] :
                [this.textArea.value, null, null, null, null, this.uploadId];

        makeRpcCall("ContactSupportService.send", params, _.bind(this.onSubmitRpcSuccess, this), _.bind(this.onSubmitRpcError, this));
    },

    rpcCancel: function()
    {
        makeRpcCall("ContactSupportService.cancel", [this.uploadId], null, _.constant(true));
    },

    hideOnMouseDown: _.constant(false),

    hideOnResize: _.constant(false),

    hideOnEscape: _.constant(false),

    onEscape: function()
    {
        this.cancel();
    },

    changesDetected: function()
    {
        this.dataChanged = true;
    },

    addChangesHandler: function(input)
    {
        $(input).valueChanged(_.bind(this.changesDetected, this));
    },

    wereDataChanged: function()
    {
        return this.dataChanged;
    },

    updateTextLengthNote: function(withDelay)
    {
        var self = this;
        if (withDelay)
        {
            setTimeout(function() { self.updateTextLengthNote(false) }, 40);
            return;
        }
        this.textLengthNote.update(this.textArea.value);
    },

    getElementBySubId: function(id)
    {
        return document.getElementById(this.popupId + "_" + id);
    },

    onSubmitButtonClick: function()
    {
        if (this.formValidator.validateForm())
        {
            this.stateSwitcher.switchState(SupportPopup.States.PROGRESS);
            this.rpcSend();
        }
    },

    onSubmitRpcSuccess: function()
    {
        this.stateSwitcher.switchState(SupportPopup.States.SUCCESS);
    },

    onSubmitRpcError: function(jqXHR, textStatus, errorThrown)
    {
        this.stateSwitcher.switchState(SupportPopup.States.MAIN);

        if (errorThrown.exception == "com.actimind.common.integration.HMSPWSConnector$CommunicationException")
        {
            alert(messageResource.getMessage("contact_support.error.communication_exception"));
        }
        else if (errorThrown.exception == "com.actimind.actitime.services.ContactSupportValidationException")
        {
            alert(messageResource.getMessage("contact_support.error.validation_exception"));
        }
        else if (errorThrown.exception == "com.actimind.actitime.services.UploadExpiredException")
        {
            alert(messageResource.getMessage("contact_support.error.upload_expired_exception"));
            this.dropzone.removeAllFiles();
        }
        else
        {
            alert(messageResource.getMessage("contact_support.error.generic"));
        }

        return true;
    },

    onSendAgainButtonClick: function()
    {
        this.textArea.value = "";
        this.updateTextLengthNote(false);
        this.textArea.prevValue = this.textArea.value;

        this.dataChanged = false;
        this.uploadId = 0;
        this.dropzone.removeAllFiles();
        this.stateSwitcher.switchState(SupportPopup.States.MAIN);
    },

    onDropzoneError: function(file, errorText)
    {
        if (file.status != "canceled")
        {
            alert(errorText);
            this.dropzone.removeFile(file);
        }
    }
});