function TableCell(value, rowObj)
{
    this.value = value;
    this.colIdx = -1;
    this.rowObj = rowObj;
    this.renderer = null;
    this.cellElem = document.createElement("td");
    this.cellElem.cellObj = this;
}

TableCell.prototype = {
    getRow: function()
    {
        return this.rowObj;
    },

    setColspan: function(colspan)
    {
        return this.colspan = colspan;
    },

    setRow: function(rowObj)
    {
        this.rowObj = rowObj;
    },

    setColIdx: function(colIdx)
    {
        this.colIdx = colIdx;
    },

    getColIdx: function()
    {
        return this.colIdx;
    },

    getCellEl: function()
    {
        return this.cellElem;
    },

    setRenderer: function(renderer)
    {
        this.renderer = renderer;
    },

    getRenderer: function()
    {
        return this.renderer;
    },

    updateUI: function()
    {
        this.renderer.updateUI(this);
    },

    writeContent: function()
    {
        this.renderer.writeContent(this);
    },

    // use, when you need to update cell content and cell UI with new data (see UsersForTTApproverCellRenderer, AssignTTApproversCellRenderer, for example)
    updateCell: function()
    {
        this.renderer.updateContent(this);
        this.updateUI();
    },

    getValue: function()
    {
        return this.value;
    },

    setValue: function(value)
    {
        this.value = value;
    },

    isFirst: function()
    {
        return this.colIdx == 0;
    },

    isLast: function()
    {
        return this.colIdx == this.getRow().getCells().length - 1;
    },

    getTable: function()
    {
        return this.getRow().getTable();
    },

    leftCell: function()
    {
        var leftIndex = this.getColIdx() - 1;
        if (leftIndex >= 0)
            return this.getRow().getCell(leftIndex);
    },

    upperCell: function()
    {
        var upperRowIndex = this.getRow().getRowIdx() - 1;
        if (upperRowIndex >= 0)
            return this.getRow().getTable().getCell(upperRowIndex, this.getColIdx());
    },

    getMouseHandler: function()
    {
        return this.getRow().getTable().getMouseEventsHandler();
    },

    mouseOver: function(evt) { return this.getMouseHandler().mouseOver(evt, this); },
    mouseOut: function(evt) { return this.getMouseHandler().mouseOut(evt, this); },
    mouseDown: function(evt) { return this.getMouseHandler().mouseDown(evt, this); },
    mouseUp: function(evt) { return this.getMouseHandler().mouseUp(evt, this); },
    mouseMove: function(evt) { return this.getMouseHandler().mouseMove(evt, this); },
    contextMenu: function(evt) { return this.getMouseHandler().contextMenu(evt, this); },

    setupMouseHandlers: function()
    {
        this.cellElem.onmouseover = function(evt) { this.cellObj.mouseOver(getEventObj(evt)); };
        this.cellElem.onmouseout = function(evt) { this.cellObj.mouseOut(getEventObj(evt)); };
        this.cellElem.onmousedown = function(evt) { this.cellObj.mouseDown(getEventObj(evt)); };
        this.cellElem.onmouseup = function(evt) { this.cellObj.mouseUp(getEventObj(evt)); };
        this.cellElem.onmousemove = function(evt) { this.cellObj.mouseMove(getEventObj(evt)); };
        this.cellElem.oncontextmenu = function(evt) { this.cellObj.contextMenu(getEventObj(evt));};
    }
};
