function TableCellRenderer()
{
    this.extraCls = [];
}

TableCellRenderer.prototype = {
    addExtraClass: function(cls)
    {
        this.extraCls.push(cls);
    },

    removeExtraClass: function(cls)
    {
        var idx = $.inArray(cls, this.extraCls);
        if (idx != -1)
            this.extraCls.splice(idx, 1);
    },

    writeContent: function(cell)
    {
        this.generateContent(cell);
        this.updateUI(cell);
    },

    generateContent: function(cell)
    {
        cell.getCellEl().innerHTML = cell.getValue();
    },

    updateContent: function(cell)
    {
        // should be implemented if necessary
        // for example when you need to update only one cell with new data without recreating the whole row
        // see AssignTTApproversCell, AssignTTApproversCellRenderer
    },

    updateUI: function(cell)
    {
        var classesArray = this.getCellClassName(cell);
        this.appendToCellClassesArray(cell, classesArray);
        classesArray = this.concatExtraClasses(cell, classesArray);

        this.setCssClass(cell, classesArray.join(" "));
    },

    appendToCellClassesArray: function(cell, classesArray)
    {
        this.appendPositionClassesIfNeeded(cell, classesArray);
        this.appendSelectionClassesIfNeeded(cell, classesArray);
        this.appendHoverClassesIfNeeded(cell, classesArray);
    },

    appendPositionClassesIfNeeded: function(cell, classesArray)
    {
        if (cell.isFirst())
            classesArray.push("first");

        if (cell.isLast())
            classesArray.push("last");
    },

    appendSelectionClassesIfNeeded: function(cell, classesArray)
    {
        var selectionController = this.getSelectionController(cell);

        if (selectionController.getSelection().containsCell(cell))
            classesArray.push("selected");

        if (selectionController.hasSelection())
        {
            if (this.needsSelectionTopBorder(cell))
                classesArray.push("selectionTopBorder");
            if (this.needsSelectionRightBorder(cell))
                classesArray.push("selectionRightBorder");
            if (this.needsSelectionBottomBorder(cell))
                classesArray.push("selectionBottomBorder");
            if (this.needsSelectionLeftBorder(cell))
                classesArray.push("selectionLeftBorder");
        }
    },

    appendHoverClassesIfNeeded: function(cell, classesArray)
    {
        if (this.isHovered(cell))
            classesArray.push("hover");
    },

    concatExtraClasses: function(cell, classesArray)
    {
        return classesArray.concat(this.extraCls);
    },

    //noinspection JSUnusedLocalSymbols
    getCellClassName: function(cell)
    {
        return [];
    },

    needsSelectionTopBorder: function(cell)
    {
        var selection = this.getSelectionController(cell).getSelection();
        if (selection.containsCell(cell))
        {
            return selection.isUpwardmostCell(cell);
        }
        else
        {
            var upperCell = cell.upperCell();
            if (upperCell)
                return selection.containsCell(upperCell) && selection.isDownwardmostCell(upperCell);
        }

        return false;
    },

    needsSelectionRightBorder: function(cell)
    {
        if (cell.isLast())
        {
            var selection = this.getSelectionController(cell).getSelection();
            return selection.containsCell(cell);
        }

        return false;
    },

    needsSelectionLeftBorder: function(cell)
    {
        var selection = this.getSelectionController(cell).getSelection();
        if (selection.containsCell(cell))
        {
            return selection.isLeftmostCell(cell)
        }
        else
        {
            var leftCell = cell.leftCell();
            if (leftCell)
                return selection.containsCell(leftCell) && selection.isRightmostCell(leftCell);
        }

        return false;
    },

    needsSelectionBottomBorder: function(cell)
    {
        if (cell.getRow().isLast())
        {
            var selection = this.getSelectionController(cell).getSelection();
            return selection.containsCell(cell);
        }

        return false;
    },


    setCssClass: function(cell, cssClassName)
    {
        cell.getCellEl().className = cssClassName;
    },

    getSelectionController: function(cell)
    {
        return cell.getRow().getTable().getSelectionController();
    },

    getHoverController: function(cell)
    {
        return cell.getRow().getTable().getHoverController();
    },

    isSelected: function(cell)
    {
        return this.getSelectionController(cell).isSelected(cell.getRow().getRowIdx(), cell.getColIdx());
    },

    isHovered: function(cell)
    {
        return this.getHoverController(cell).isHovered(cell);
    }
};


