function TableMouseEventsHandler(table, selectionController, hoverController)
{
    this.table = table;
    this.selectionController = selectionController;
    this.hoverController = hoverController;
    this.selectionAnchor = null;
    this.mouseIsDown = false;
}

TableMouseEventsHandler.prototype = _.create(Object.prototype, {
    mouseOver: function(evt, cell)
    {
        if (this.mouseIsDown)
            this.selectionController.changeSelectionTo(cell);
        else
            this.hoverController.hover(cell, cell);
    },

    mouseOut: function(evt, cell)
    {
        if (!this.mouseIsDown)
            this.hoverController.unhover();
    },

    mouseDown: function(evt, cell)
    {
        this.mouseIsDown = true;
        this.selectionController.clearSelection();
        this.selectionController.setSelectionAnchor(cell);
        this.selectionController.changeSelectionTo(cell);
    },

    mouseUp: function(evt, cell)
    {
        this.mouseIsDown = false;
    },

    mouseMove: function(evt, cell)
    {
        // nothing to do
    },

    contextMenu: function(evt, cell)
    {
        // nothing
    },

    getSelectionController: function()
    {
        return this.selectionController;
    },

    getHoverController: function()
    {
        return this.hoverController;
    }
});
