function TableRegion(startRow, startCol, endRow, endCol)
{
    this.rowMin = Math.min(startRow, endRow);
    this.rowMax = Math.max(startRow, endRow);
    this.colMin = Math.min(startCol, endCol);
    this.colMax = Math.max(startCol, endCol);
}

TableRegion.prototype = _.create(Object.prototype, {
    createFromCells: function(startCell, endCell)
    {
        return new TableRegion(
                startCell.getRow().getRowIdx(), startCell.getColIdx(),
                endCell.getRow().getRowIdx(), endCell.getColIdx()
        );
    },

    containsCol: function(col)
    {
        return this.colMin <= col && this.colMax >= col;
    },

    containsRow: function(row)
    {
        return this.rowMin <= row && this.rowMax >= row;
    },


    contains: function(row, col)
    {
        return this.containsCol(col) && this.containsRow(row);
    },

    containsCell: function(cell)
    {
        return this.containsCol(cell.getColIdx()) && this.containsRow(cell.getRow().getRowIdx());
    },

    union: function(otherRegion)
    {
        return otherRegion == this.NULL ? this : new TableRegion(
                Math.min(this.rowMin, otherRegion.rowMin),
                Math.min(this.colMin, otherRegion.colMin),
                Math.max(this.rowMax, otherRegion.rowMax),
                Math.max(this.colMax, otherRegion.colMax)
        );
    },

    equals: function(region)
    {
        return region != this.NULL &&
               this.rowMin == region.rowMin &&
               this.colMin == region.colMin &&
               this.rowMax == region.rowMax &&
               this.colMax == region.colMax;
    },

    toString: function()
    {
        return this.rowMin + "," +
               this.colMin + "," +
               this.rowMax + "," +
               this.colMax;
    },
    
    isLeftmostCell: function(cell)
    {
        return this.colMin == cell.getColIdx();
    },

    isUpwardmostCell: function(cell)
    {
        return this.rowMin == cell.getRow().getRowIdx();
    },

    isRightmostCell: function(cell)
    {
        return this.colMax == cell.getColIdx();
    },

    isDownwardmostCell: function(cell)
    {
        return this.rowMax == cell.getRow().getRowIdx();
    }
});
