TaskManagement.DatePickerMenu = function(scheduleCalculator)
{
    this.scheduleCalculator = scheduleCalculator;
};

TaskManagement.DatePickerMenu.getInstance = function()
{
    if (!TaskManagement.DatePickerMenu.instance)
        TaskManagement.DatePickerMenu.instance = new TaskManagement.DatePickerMenu(CorporateScheduleCalculator.instance).init();
    return TaskManagement.DatePickerMenu.instance;
};

TaskManagement.DatePickerMenu.prototype = {
    init: function()
    {
        var self = this;

        this.menu = new AT.menu.DatePickerMenu({
            mode: "dynamic",
            minDate: new Date(0),
            format: Date.convertSmartToExtDateFormat(SystemSettings.dateFormat).replace(/F/, "M"),
            showToday: true,
            showClearButton: true,
            showOkCancelButtons: true,
            scheduleCalculator: this.scheduleCalculator,
            startDay: SystemSettings.weekStartDay - 1,
            value: new Date(),
            cancelOnHide: true
        });

        this.menu.on("select", function()
        {
            self.selectCallback(self.getValue());
        });
        this.menu.on("datePickerClear", function()
        {
            self.selectCallback(null);
        });
        this.menu.on("datePickerCancel", function()
        {
            self.cancelCallback();
        });

        this.menu.on('beforeshow', function()
        {
            $(window).on("resize.datePickerMenu", _.bind(self._onResize, self));
            KeyDownHandler.setOnKeyDownHandler(function(e) {
                // ignore old event handler (ex: hide panel on escape keypressed)
            })
        });

        this.menu.on('beforehide', function()
        {
            $(window).off("resize.datePickerMenu");
            KeyDownHandler.restoreOnKeyDownHandler();
        });

        return this;
    },

    _onResize: function()
    {
        this.hideMenu();
    },

    _onScroll: function()
    {
        this.hideMenu();
    },

    hideMenu: function() {
        this.menu.hide();
    },
    
    getValue: function()
    {
        return this.menu.picker.getValue();
    },

    setValue: function(value)
    {
        this.menu.picker.setValue(value);
    },

    showAt: function($el, selectCallback, cancelCallback)
    {
        this.selectCallback = selectCallback;
        this.cancelCallback = cancelCallback;
        this.menu.show($el[0]);
    }
};