TaskManagement.Mediator = function(data)
{
    PageMediator.call(this);
    this.data = data;
};

TaskManagement.Mediator.prototype = _.create(PageMediator.prototype, {
    getPage: function()
    {
        return this.page;
    },

    beforeOpenPageFirstTime: function()
    {
        this.initStore();
        this.initComponents();
        this.reloadTasks();
        this.subscribeToEvents();
    },

    initStore: function()
    {
        StoreUtils.mergeCustomersProjectsFromJSP(this.data.cpData);
        TaskManagement.Store.setFilterCustomerProjectSelection(CustomerProjectSelection.fromPlainObject(this.data.pageSettings.customersProjectsSelection));
        TaskManagement.Store.setVisibleCustomers(this.buildVisibleCustomers());
        TaskManagement.Store.setSingleCustomerProjectSelection({customerId: null, projectId: null});
        TaskManagement.Store.setIntersectedCustomerProjectSelection(this.intersectCustomerProjectSelections());
        TaskManagement.Store.setTaskSortBy(this.data.pageSettings['sortBy']);
        TaskManagement.Store.setTaskSortAsc(this.data.pageSettings['ascSort']);
        TaskManagement.Store.setTaskStatusFilter(this.data.pageSettings['taskStatusFilter']);
        TaskManagement.Store.setManagementSettings(this.data.managementSettings);
    },

    initComponents: function()
    {
        var self = this;
        this.page = new TaskManagement.Page($("body")).init();
        var $taskListBlock = $("#taskListBlock");
        this.tasksTable = new TaskManagement.TasksTable($taskListBlock).init();
        this.customersProjectsPanel = new TaskManagement.CustomersProjectsPanel($("#cpTreeBlock")).init();
        this.deleteTaskView = new LightboxTemplateView($(".deleteTaskPopupContainer"), "deleteTaskPopup", "deleteTaskPopup").init();
        this.deleteTaskLightBox = new TaskDeleteLightBox().init({popupId: "deleteTaskPopup"});

        this.customerPanelController = new TaskManagement.CustomerPanelController().init();
        $(this.customerPanelController).on("customerChanged", function()
        {
            self._updateVisibleCustomersProjects();
        });
        $(this.customerPanelController).on("customerDeleted", function()
        {
            self._updateAll();
        });
        $(this.customerPanelController).on("customerArchived", function(e, customerId)
        {
            self._archiveCustomer(customerId);
        });
        $(this.customerPanelController).on("deleteBegin", function()
        {
            self.pageLoadingMoire.show();
        });
        $(this.customerPanelController).on("deleteEnd", function()
        {
            self.pageLoadingMoire.hide();
        });
        $(this.customerPanelController).on("hidePanel", function(e, data)
        {
            $(document).trigger("taskManagement_customerEditEnd", data.customerId);
            if (data.changed && TaskManagement.Store.getIntersectedCustomerProjectSelection().includesCustomer(CustomerStore.get(data.customerId), true))
                self.reloadTasks();
        });

        this.taskPanelController = new TaskManagement.TaskPanelController().init();
        $(this.taskPanelController).on("taskDeleted", function(e, taskId)
        {
            self._onTasksDeleted([taskId]);
        });
        $(this.taskPanelController).on("taskEdited", function(e, taskId)
        {
            self._triggerTaskEdited(taskId);
        });
        $(this.taskPanelController).on("taskChanged", function(e, taskId)
        {
            self._triggerTaskUpdated(taskId);
        });
        $(this.taskPanelController).on("taskMoved", function(e, data)
        {
            var project = ProjectStore.get(data.projectId);
            if (!project || !TaskManagement.Store.getIntersectedCustomerProjectSelection().includesProject(project))
            {
                TaskManagement.Store.removeTasks([data.taskId]);
                $(document).trigger("taskManagement_removeTasks", [[data.taskId]]);
            }
            else
            {
                self._triggerTaskUpdated(data.taskId);
            }
        });
        $(this.taskPanelController).on("cpChanged", function()
        {
            self._updateVisibleCustomersProjects();
        });
        $(this.taskPanelController).on("projectDeleted", function()
        {
            self._updateAll();
        });

        this.projectPanelController = new TaskManagement.ProjectPanelController().init();
        $(this.projectPanelController).on("projectChanged customerChanged", function()
        {
            self._updateVisibleCustomersProjects();
        });
        $(this.projectPanelController).on("projectDeleted customerDeleted", function()
        {
            self._updateAll();
        });
        $(this.projectPanelController).on("projectArchived", function(e, projectId)
        {
            self._archiveProject(projectId);
        });
        $(this.projectPanelController).on("projectMoved", function(e, projectId)
        {
            if (TaskManagement.Store.getSingleCustomerProjectSelection().projectId === projectId)
            {
                TaskManagement.Store.setSingleCustomerProjectSelection({
                                                                           customerId: ProjectStore.get(projectId).customerId,
                                                                           projectId: projectId
                                                                       });
            }
            self.customersProjectsPanel.updateCpSelector();
        });
        $(this.projectPanelController).on("deleteBegin", function()
        {
            self.pageLoadingMoire.show();
        });
        $(this.projectPanelController).on("deleteEnd", function()
        {
            self.pageLoadingMoire.hide();
        });
        $(this.projectPanelController).on("hidePanel", function(e, data)
        {
            $(document).trigger("taskManagement_projectEditEnd", data.projectId);
            if (data.changed)
                self.reloadTasks();
        });

        this.loadTasksFromCSVManager = new LoadTasksFromCSVManager().init(this.data.importCsv.actionUrl,
                                                                          SystemSettings.csvSeparator,
                                                                          this.data.importCsv.errorCodes,
                                                                          this.data.importCsv.errorMessages,
                                                                          function(projectIds) { self._onImportFromCsvCompleted(projectIds); }
        );
        PageOverlayTipsManager.getInstance().refreshDynamicTips();

        this.pageLoadingMoire = new LoadingMoire($("body"), {
            radius: 17,
            length: 13,
            width: 4
        }, true, 1000).init();
    },

    subscribeToEvents: function()
    {
        var self = this;
        $(document).on("taskManagement_tasksTable_allCustomersClick taskManagement_tasksTable_allTasksClick " +
                       "taskManagement_customersProjectsPanel_allCustomersNode_select",
                       function()
                       {
                           self._setSingleCustomerProjectSelection({customerId: null, projectId: null});
                       });
        $(document).on("taskManagement_tasksTable_customerClick", function()
        {
            var singleCPSelection = TaskManagement.Store.getSingleCustomerProjectSelection();
            self._setSingleCustomerProjectSelection({customerId: singleCPSelection.customerId, projectId: null});
        });
        $(document).on("taskManagement_tasksTable_scrollToEnd", function()
        {
            if (!TaskManagement.Store.isAllTasksLoaded())
            {
                self.loadTasks();
            }
        });
        $(document).on("taskManagement_tasksTable_sortingHeaderClick", function(e, sortConfig)
        {
            TaskManagement.Store.setTaskSortBy(sortConfig.taskSortBy);
            TaskManagement.Store.setTaskSortAsc(sortConfig.asc);
            self.reloadTasks();
            self.savePageSettings();
        });
        $(document).on("taskManagement_tasksTable_statusFilterChange", function(e, taskStatusFilter)
        {
            TaskManagement.Store.setTaskStatusFilter(taskStatusFilter);
            self.reloadTasks();
            self.savePageSettings();
        });
        $(document).on("taskManagement_tasksTable_tasksSearchField_change", function(e, text)
        {
            TaskManagement.Store.setTaskNameFilter(text);
            self.reloadTasks();
        });
        $(document).on("taskManagement_tasksTable_taskRow_statusChange", function(e, changeData)
        {
            self.updateTask("TaskManagementService.updateTaskStatus", [changeData.taskId, changeData.completed, false], changeData.taskId);
        });
        $(document).on("taskManagement_tasksTable_taskRow_estimateChange", function(e, changeData)
        {
            self.updateTask("TaskManagementService.updateTaskEstimatedTime", [changeData.taskId, changeData.estimatedTime], changeData.taskId);
        });
        $(document).on("taskManagement_tasksTable_taskRow_deadlineChange", function(e, changeData)
        {
            self.updateTask("TaskManagementService.updateTaskDeadline", [changeData.taskId, changeData.deadline], changeData.taskId);
        });
        $(document).on("taskManagement_customersProjectsPanel_cpSelectorFilter_change", function(e, filterCustomerProjectSelection)
        {
            if (filterCustomerProjectSelection.isEmpty())
                return;

            TaskManagement.Store.setFilterCustomerProjectSelection(filterCustomerProjectSelection);
            var intersectedCPSelection = self.intersectCustomerProjectSelections();
            if (intersectedCPSelection.isEmpty())
            {
                TaskManagement.Store.setSingleCustomerProjectSelection({customerId: null, projectId: null});
                intersectedCPSelection = self.intersectCustomerProjectSelections();
            }
            TaskManagement.Store.setIntersectedCustomerProjectSelection(intersectedCPSelection);

            self._updateVisibleCustomersProjects();
            self.reloadTasks();
            self.savePageSettings();
        });
        $(document).on("taskManagement_customersProjectsPanel_projectNode_select", function(e, projectId)
        {
            var project = ProjectStore.get(projectId);
            TaskManagement.Store.setSingleCustomerProjectSelection({customerId: project.customerId, projectId: projectId});
            TaskManagement.Store.setIntersectedCustomerProjectSelection(self.intersectCustomerProjectSelections());
            TaskManagement.Store.setAllTasksLoaded(false);
            self.loadTasksForProject(true, projectId);
        });
        $(document).on("taskManagement_customersProjectsPanel_customerNode_select", function(e, customerId)
        {
            TaskManagement.Store.setSingleCustomerProjectSelection({customerId: customerId, projectId: null});
            TaskManagement.Store.setIntersectedCustomerProjectSelection(self.intersectCustomerProjectSelections());
            TaskManagement.Store.setAllTasksLoaded(false);
            self.loadTasksForCustomer(true, customerId);
        });
        $(document).on("taskManagement_customersProjectsPanel_searchField_change", function(e, text)
        {
            text = text.trim();
            TaskManagement.Store.setCustomerProjectNameFilter(text);
            TaskManagement.Store.setVisibleCustomersFiltered(text ? self.buildVisibleCustomersFiltered() : null);
        });

        $(document).on("taskManagement_taskTable_taskRow_selectionChange", function(e, data)
        {
            if (TaskManagement.Store.isAllTasksSelected())
                return;

            if (data.isSelected)
                TaskManagement.Store.addSelectedTask(data.taskId);
            else
                TaskManagement.Store.removeSelectedTask(data.taskId);
        });

        $(document).on("taskManagement_taskTable_selectAllChange", function(e, isAllSelected)
        {
            TaskManagement.Store.setTaskSelection(isAllSelected, {});
        });

        $(document).on("taskManagement_closeGroupOperationMenu", function()
        {
            TaskManagement.Store.setTaskSelection(false, {});
        });

        $(document).on("taskManagement_tasksTable_addNewTaskMenu_createNewTask", function()
        {
            self._showCreateTaskPopup();
        });

        $(document).on("taskManagement_tasksTable_addNewTaskMenu_importTasks", function()
        {
            self._showImportTasksPopup();
        });

        $(document).on("taskManagement_tasksTable_taskRow_click", function(e, clickEvent, taskId)
        {
            clickEvent._at_edit_task_sliding_panel_preventHidding = true;
            self._showEditTaskPanel(taskId);
        });

        $(document).on("taskManagement_editCustomer", function(e, customerId)
        {
            var prevCustomerId = self.customerPanelController.customerId;

            if (prevCustomerId)
                $(document).trigger("taskManagement_customerEditEnd", prevCustomerId);

            if (prevCustomerId !== customerId)
                self.customerPanelController.openPanel(customerId);
            else
                self.customerPanelController.closePanel();
        });

        $(document).on("taskManagement_customersProjectsPanel_addNewMenu_createNewCustomer", function()
        {
            self._showCreateCustomerPopup();
        });

        $(document).on("taskManagement_customersProjectsPanel_addNewMenu_createNewProject", function()
        {
            self._showCreateProjectPopup();
        });

        $(document).on("taskManagement_customersProjectsPanel_editProject", function(e, projectId)
        {
            var prevProjectId = self.projectPanelController.projectId;

            if (prevProjectId)
                $(document).trigger("taskManagement_projectEditEnd", prevProjectId);

            if (prevProjectId !== projectId)
                self.projectPanelController.openPanel(projectId);
            else
                self.projectPanelController.closePanel();
        });

        $(document).on("taskManagement_groupingStatusChange", function(e, completed)
        {
            if (_.isUndefined(completed))
                return;

            var successHandler = function(data)
            {
                self.pageLoadingMoire.hide();

                if (data.length == 0)
                    Toasts.addToastWithText(messageResource.getMessage("status.tasks.toast.none"));
                else if (data.length == 1)
                {
                    var name = TaskStore.get(data[0]).name;
                    Toasts.addToastWithText(messageResource.getMessage("status.tasks.toast.singular", [name]));
                }
                else
                    Toasts.addToastWithText(messageResource.getMessage("status.tasks.toast", [data.length]));

                var result = {
                    taskIds: data,
                    isCompleted: completed
                };

                _.each(data, function(taskId)
                {
                    var task = TaskStore.get(taskId);
                    if (task)
                    {
                        task = _.clone(task);
                        task.completed = completed;
                        TaskStore.add(task);
                    }
                });

                $(document).trigger("taskManagement_tasksStatusChange", result);
                TaskManagement.Store.setTaskSelection(false, {});
            };

            var errorHandler = function(jqXHR, textStatus, errorThrown)
            {
                self.pageLoadingMoire.hide();

                if (textStatus === "rpc_customException")
                {
                    switch (errorThrown.exception)
                    {
                        case "com.actimind.actitime.services.BaseNewTasksService$NoRightsToManageTasksException":
                            alert(messageResource.getMessage("common.no_rights_for_operation_anymore"));
                            window.location.reload();
                            return true;
                        case "com.actimind.actitime.task.ProjectIsArchivedException":
                            // confirm reopening of archived project containing this task
                            if (confirm(messageResource.getMessage('task.complete.js.reopening_confirmation_tasks_archived')))
                            {
                                groupStatusChangeCall(true);
                            }
                            return true;
                    }
                }

                return false;
            };

            var groupStatusChangeCall = function(forceReopenArchivedProject)
            {
                self.pageLoadingMoire.show();

                makeRpcCall("TaskManagementService.changeStatusTasks",
                            [
                                completed,
                                TaskManagement.Store.isAllTasksSelected(),
                                _.keys(TaskManagement.Store.getSelectedTasks()),
                                TaskManagement.Store.getIntersectedCustomerProjectSelection(),
                                TaskManagement.Store.getTaskStatusFilter(),
                                TaskManagement.Store.getTaskNameFilter(),
                                TaskManagement.Store.getTaskIds(),
                                forceReopenArchivedProject
                            ],
                            successHandler,
                            errorHandler);
            };
            groupStatusChangeCall(false);

        });

        $(document).on("taskManagement_deleteTasks", function()
        {
            var onDeleteAction = function(revokeStatus)
            {
                var successHandler = function(data)
                {
                    self.pageLoadingMoire.hide();
                    self.deleteTaskLightBox.hide();

                    // TODO: put making toast into separate function and reuse it in other places
                    var toast;
                    if (data.length === 0)
                    {
                        toast = messageResource.getMessage("delete.tasks.toast.none");
                    }
                    else if (data.length === 1)
                    {
                        var task = TaskStore.get(data[0]);
                        if (task)
                            toast = messageResource.getMessage("delete.tasks.toast.singular", [task.name]);
                        else
                            toast = messageResource.getMessage("delete.unknown_task.toast");
                    }
                    else
                    {
                        toast = messageResource.getMessage("delete.tasks.toast", [data.length]);
                    }
                    Toasts.addToastWithText(toast);

                    self._onTasksDeleted(data);
                    TaskManagement.Store.setTaskSelection(false, {});
                };

                makeRpcCall(
                        "TaskManagementService.deleteTasks",
                        [
                            revokeStatus,
                            TaskManagement.Store.isAllTasksSelected(),
                            _.keys(TaskManagement.Store.getSelectedTasks()),
                            TaskManagement.Store.getIntersectedCustomerProjectSelection(),
                            TaskManagement.Store.getTaskStatusFilter(),
                            TaskManagement.Store.getTaskNameFilter(),
                            TaskManagement.Store.getTaskIds()
                        ],
                        successHandler,
                        errorHandler);

                self.pageLoadingMoire.show();
            };

            var successHandler = function(data)
            {
                self.pageLoadingMoire.hide();

                if (data.entitiesToDeleteCount == 0)
                {
                    alert(messageResource.getMessage('task.delete.js.select_tasks_to_delete'));
                    return;
                }

                $("#" + self.deleteTaskView.lightboxId).empty();

                var deletePopupConfig = _.extend(data, {
                    $containerElement: $("#deleteTaskPopup"),
                    idPrefix: "deleteTaskPopup",
                    otherBlockedTaskCount: data.entitiesBlockedCount - data.entitiesBlocked.length,
                    hasManagerApprover: TaskManagement.Store.getManagerNeedApproval(),
                    cancelHandler: function() { self.deleteTaskLightBox.hide(); },
                    deleteHandler: onDeleteAction
                });

                new CPTManagement.DeleteTasksConfirmationDialog(deletePopupConfig).init().show();
                self.deleteTaskLightBox.show();
            };

            var errorHandler = function(jqXHR, textStatus, errorThrown)
            {
                self.pageLoadingMoire.hide();
                self.deleteTaskLightBox.hide();

                if (textStatus === "rpc_customException")
                {
                    switch (errorThrown.exception)
                    {
                        case "com.actimind.actitime.services.BaseNewTasksService$NoRightsToManageTasksException":
                            alert(messageResource.getMessage("common.no_rights_for_operation_anymore"));
                            window.location.reload();
                            return true;
                    }
                }

                return false;
            };

            self.pageLoadingMoire.show();

            makeRpcCall(
                    "TaskManagementService.getDeleteTaskPopupInfo",
                    [
                        TaskManagement.Store.isAllTasksSelected(),
                        _.keys(TaskManagement.Store.getSelectedTasks()),
                        TaskManagement.Store.getIntersectedCustomerProjectSelection(),
                        TaskManagement.Store.getTaskStatusFilter(),
                        TaskManagement.Store.getTaskNameFilter(),
                        TaskManagement.Store.getTaskIds()
                    ],
                    successHandler,
                    errorHandler);
        });
    },

    reloadTasks: function()
    {
        TaskManagement.Store.setAllTasksLoaded(false);
        this.loadTasks(true);
    },

    loadTasks: function(reload)
    {
        var limit = 20;
        this._baseLoadTasks(reload,
                            "TaskManagementService.getTasks",
                            this._getBaseLoadTasksParams(reload, limit, TaskManagement.Store.getIntersectedCustomerProjectSelection()),
                            limit);
    },

    loadTasksForCustomer: function(reload, customerId)
    {
        var self = this;
        var limit = 20;
        // TODO: getTasksOfSelectedCustomer may be not needed in current form
        this._baseLoadTasks(reload,
                            "TaskManagementService.getTasksOfSelectedCustomer",
                            [
                                TaskManagement.Store.getTaskSortBy(),
                                TaskManagement.Store.getTaskSortAsc(),
                                customerId,
                                TaskManagement.Store.getIntersectedCustomerProjectSelection(),
                                TaskManagement.Store.getTaskStatusFilter(),
                                TaskManagement.Store.getTaskNameFilter(),
                                reload ? 0 : TaskManagement.Store.getTaskIds().length,
                                limit
                            ],
                            limit,
                            function(jqXHR, textStatus, errorThrown)
                            {
                                if (errorThrown.exception == "com.actimind.actitime.customer.NoSuchCustomerException")
                                {
                                    CustomerStore.remove(customerId);
                                    alert(messageResource.getMessage("project.edit.customer_deleted_concurrently"));
                                    self._updateVisibleCustomersProjects();
                                    return true;
                                }
                                return false;
                            });
    },

    loadTasksForProject: function(reload, projectId)
    {
        var limit = 20;
        this._baseLoadTasks(reload,
                            "TaskManagementService.getTasksOfSelectedProject",
                            this._getBaseLoadTasksParams(reload, limit, projectId),
                            limit);
    },

    _getBaseLoadTasksParams: function(reload, limit, selection)
    {
        return [
            TaskManagement.Store.getTaskSortBy(),
            TaskManagement.Store.getTaskSortAsc(),
            selection,
            TaskManagement.Store.getTaskStatusFilter(),
            TaskManagement.Store.getTaskNameFilter(),
            reload ? 0 : TaskManagement.Store.getTaskIds().length,
            limit
        ]
    },

    _baseLoadTasks: function(reload, serviceName, params, limit, customErrorHandler)
    {
        $(document).trigger(reload ? "taskManagement_loadTasks_withReload_begin" : "taskManagement_loadTasks_withAppend_begin");

        if (this.loadTasksRequest)
            this.loadTasksRequest.abort();

        var self = this;
        var customerProjectSelection = TaskManagement.Store.getIntersectedCustomerProjectSelection();

        function deleteProjectIfNeeded()
        {
            if (customerProjectSelection.projects.length === 1)
            {
                ProjectStore.remove(customerProjectSelection.projects[0]);
                self._updateVisibleCustomersProjects();
            }
        }

        this.loadTasksRequest = makeRpcCall(
                serviceName,
                params,
                function(data)
                {
                    StoreUtils.mergeCustomersProjectsFromRPC(data);

                    var loadedTasks = _.map(data["tasks"], Task.fromRPC);
                    if (loadedTasks.length < limit)
                        TaskManagement.Store.setAllTasksLoaded(true);

                    TaskStore.mergeAll(loadedTasks);
                    var taskIds = _.pluck(loadedTasks, "id");
                    if (reload)
                    {
                        self.intersectWithSelectedTasks(taskIds);
                    }

                    if (reload)
                    {
                        TaskManagement.Store.replaceTaskIds(taskIds);
                        self.openTaskFromLink();
                    }
                    else
                        TaskManagement.Store.appendTaskIds(taskIds);
                },
                function(jqXHR, textStatus, errorThrown)
                {
                    if (textStatus === "rpc_customException")
                    {
                        switch (errorThrown.exception)
                        {
                            case "com.actimind.actitime.project.NoSuchProjectException":
                                deleteProjectIfNeeded();
                                alert(messageResource.getMessage("task.list.error.load_tasks.no_such_project"));
                                return true;
                            case "com.actimind.actitime.project.ProjectIsNotAssignedException":
                                deleteProjectIfNeeded();
                                alert(messageResource.getMessage("task.list.error.load_tasks.project_not_assigned"));
                                return true;
                        }

                        if (customErrorHandler)
                            return customErrorHandler(jqXHR, textStatus, errorThrown);
                    }

                    return false;
                });
    },

    savePageSettings: function()
    {
        makeRpcCall("TaskManagementService.savePageSettings", [TaskManagement.Store.getFilterCustomerProjectSelection(), TaskManagement.Store.getTaskSortBy(),
                                                               TaskManagement.Store.getTaskSortAsc(), TaskManagement.Store.getTaskStatusFilter()],
                    function() {}, function() {return true;});
    },

    loadDeleteTasksPopup: function()
    {
        makeRpcCall(
                "TaskManagementService.getDeleteTaskInfo",
                [
                    TaskManagement.Store.getIntersectedCustomerProjectSelection(),
                    TaskManagement.Store.getTaskStatusFilter(),
                    TaskManagement.Store.getTaskNameFilter(),
                    TaskManagement.Store.getSelectedTasks()
                ],
                function(data)
                {
                    TaskManagement.Store.setPopupInfo(data);
                },
                function(jqXHR, textStatus, errorThrown)
                {
                    if (textStatus === "rpc_customException")
                    {
                        switch (errorThrown.exception)
                        {
                            case "com.actimind.actitime.services.BaseNewTasksService$NoRightsToManageTasksException":
                                alert(messageResource.getMessage("common.no_rights_for_operation_anymore"));
                                window.location.reload();
                                return true;
                        }
                    }

                    return false;
                });
    },


    intersectWithSelectedTasks: function(taskIds)
    {
        var selectedTaskIds = TaskManagement.Store.getSelectedTasks();
        var filteredTaskIds = _.filter(taskIds, function(key)
        {
            return !!selectedTaskIds[key];
        });
        var filteredTaskIdsMap = _.pick(selectedTaskIds, filteredTaskIds);
        TaskManagement.Store.setTaskSelection(false, filteredTaskIdsMap);
    },

    updateTask: function(rpcMethod, rpcParams, taskId)
    {
        var self = this;
        var task = TaskStore.get(taskId);
        makeRpcCall(rpcMethod,
                    rpcParams,
                    function(data)
                    {
                        var task = Task.fromRPC(data["task"]);
                        var project = Project.fromRPC(data["project"]);
                        var customer = Customer.fromRPC(data["customer"]);

                        TaskStore.add(task);

                        // TODO: remove this hot fix(trying to understand what method was called)
                        if (rpcMethod.indexOf("Status") !== -1)
                            Toasts.addToastWithText(messageResource.getMessage("status.tasks.toast.singular", [task.name]));

                        if (!_.isEqual(ProjectStore.get(task.projectId), project) ||
                            !_.isEqual(CustomerStore.get(project.customerId), customer))
                        {
                            ProjectStore.add(project);
                            CustomerStore.add(customer);
                            self._updateVisibleCustomersProjects();
                        }

                        $(document).trigger("taskManagement_taskUpdated", task.id);
                    },
                    function(jqXHR, textStatus, errorThrown)
                    {
                        if (textStatus === "rpc_customException")
                        {
                            switch (errorThrown.exception)
                            {
                                case "com.actimind.actitime.task.NoSuchTaskException":
                                    self._onTasksDeleted([taskId]);
                                    alert(messageResource.getMessage("task.error.concurrently_deleted.cannot_edit"));
                                    return true;
                                case "com.actimind.actitime.task.TaskUnavailableException":
                                    if (task)
                                    {
                                        ProjectStore.remove(task.projectId);
                                        self._updateVisibleCustomersProjects();
                                        self.reloadTasks();
                                    }
                                    alert(messageResource.getMessage("task.list.error.update_task.task_unavailable"));
                                    return true;
                                case "com.actimind.actitime.task.TaskCannotBeModifiedException":
                                    alert(messageResource.getMessage("common.no_rights_for_operation_anymore"));
                                    window.location.reload();
                                    return true;
                                case "com.actimind.actitime.services.TaskManagementService$TaskEstimateDisabledException":
                                    alert( messageResource.getMessage("task.list.error.estimate_off") );
                                    window.location.reload();
                                    return true;
                                case "com.actimind.actitime.task.ProjectIsArchivedException":
                                    // confirm reopening of archived project containing this task
                                    if (confirm(messageResource.getMessage('task.error.reopen.project_archived')))
                                    {
                                        var newRpcParams = rpcParams.slice();
                                        newRpcParams.pop();
                                        newRpcParams.push(true);
                                        self.updateTask(rpcMethod, newRpcParams, taskId);
                                    }
                                    else
                                    {
                                        $(document).trigger("taskManagement_taskUpdated", task.id);
                                    }
                                    return true;
                            }
                        }

                        return false;
                    });
    },

    buildVisibleCustomers: function()
    {
        var filterCPSelection = TaskManagement.Store.getFilterCustomerProjectSelection();

        var result = [];
        _.each(CustomerStore.getAll(), function(customer)
        {
            var customerIncluded = filterCPSelection.includesCustomer(customer);
            var projects = _.filter(ProjectStore.getByCustomer(customer.id), function(project)
            {
                return filterCPSelection.includesProject(project);
            });

            if (customerIncluded || projects.length > 0)
            {
                result.push({
                                customerId: customer.id,
                                projectIds: _.pluck(projects, "id")
                            })
            }
        });

        return result;
    },

    buildVisibleCustomersFiltered: function()
    {
        var result = [];
        var resultMap = {};
        var text = TaskManagement.Store.getCustomerProjectNameFilter();

        var getOrCreateVisibleCustomerData = function(customerId)
        {
            var visibleCustomerData = resultMap[customerId];
            if (!visibleCustomerData)
            {
                visibleCustomerData = {
                    customerId: customerId,
                    projectIds: []
                };
                result.push(visibleCustomerData);
                resultMap[customerId] = visibleCustomerData;
            }
            return visibleCustomerData;
        };

        var filterCPSelection = TaskManagement.Store.getFilterCustomerProjectSelection();
        _.each(CustomerStore.searchByName(text, true), function(customerId)
        {
            var customer = CustomerStore.get(customerId);
            if (filterCPSelection.includesCustomer(customer, true))
                getOrCreateVisibleCustomerData(customerId);
        });

        _.each(ProjectStore.searchByName(text, true), function(projectId)
        {
            var project = ProjectStore.get(projectId);
            if (filterCPSelection.includesProject(project))
            {
                var visibleCustomerData = getOrCreateVisibleCustomerData(ProjectStore.get(projectId).customerId);
                visibleCustomerData.projectIds.push(projectId);
            }
        });

        return result;
    },

    /**
     * @return {CustomerProjectSelection}
     */
    intersectCustomerProjectSelections: function()
    {
        var singleCPSelection = this.convertSingleCPSelection();
        var filterCPSelection = TaskManagement.Store.getFilterCustomerProjectSelection();
        return singleCPSelection.intersect(filterCPSelection);
    },

    /**
     * @return {CustomerProjectSelection}
     */
    convertSingleCPSelection: function()
    {
        var singleCPSelection = TaskManagement.Store.getSingleCustomerProjectSelection();

        var qualifier = "ALL";
        var customers = [];
        var projects = [];

        if (singleCPSelection.customerId)
        {
            qualifier = "SELECTED";
            if (singleCPSelection.projectId)
                projects.push(singleCPSelection.projectId);
            else
                customers.push(singleCPSelection.customerId);
        }

        return new CustomerProjectSelection(qualifier, customers, projects);
    },

    _showImportTasksPopup: function()
    {
        this.loadTasksFromCSVManager.showPopup();
    },

    _onImportFromCsvCompleted: function(projectIds)
    {
        if (projectIds.length > 0)
        {
            this._updateVisibleCustomersProjectsAfterNewDataLoaded(new CustomerProjectSelection(CustomersProjectsSelectionQualifier.SELECTED, [], projectIds),
                                                                   {customerId: null, projectId: null});
            this._markTaskShortcutIfPresent();
        }
    },

    _showCreateTaskPopup: function()
    {
        var params = '?openedFromOpenTasks=true';
        var singleSelection = TaskManagement.Store.getSingleCustomerProjectSelection();
        if (!_.isNull(singleSelection.customerId))
        {
            params += '&customerId=' + singleSelection.customerId;
            if (!_.isNull(singleSelection.projectId))
            {
                params += '&projectId=' + singleSelection.projectId;
            }
        }
        var self = this;
        loadPopupContent(BASE_PATH + 'tasks/add_new_tasks_popup.do' + params,
                         function()
                         {
                             var appendCreatedTasks = function(result)
                             {
                                 var createdTasks = result["createdTasks"].list;

                                 // mark shortcuts for tasks
                                 if (createdTasks.length > 0)
                                 {
                                     self._markTaskShortcutIfPresent();
                                 }

                                 var createdTaskIds = [];
                                 var currentSingleCPSelection = TaskManagement.Store.getSingleCustomerProjectSelection();
                                 var singleSelectionByCreatedTasks = {
                                     customerId: _.first(createdTasks) ? _.first(createdTasks)['customer']['id'] : null,
                                     projectId: _.first(createdTasks) ? _.first(createdTasks)['project']['id'] : null
                                 };
                                 var affectedProjectsSet = {};
                                 var affectedCustomersSet = {};
                                 var tasksIdsInCurrentSelection = [];
                                 _.forEach(createdTasks, function(taskInfo)
                                 {
                                     var task = Task.fromRPC(taskInfo["task"]);
                                     var project = Project.fromRPC(taskInfo["project"]);
                                     var customer = Customer.fromRPC(taskInfo["customer"]);

                                     TaskStore.add(task);
                                     ProjectStore.addFirst(project);
                                     CustomerStore.addFirst(customer);
                                     if (project.id !== singleSelectionByCreatedTasks.projectId)
                                         singleSelectionByCreatedTasks.projectId = null;
                                     if (customer.id !== singleSelectionByCreatedTasks.customerId)
                                         singleSelectionByCreatedTasks.customerId = null;
                                     if ((_.isNull(currentSingleCPSelection.customerId) || currentSingleCPSelection.customerId === customer.id) &&
                                         (_.isNull(currentSingleCPSelection.projectId) || currentSingleCPSelection.projectId === project.id))
                                         tasksIdsInCurrentSelection.push(task.id);
                                     createdTaskIds.push(task.id);

                                     affectedProjectsSet[project.id] = true;
                                     affectedCustomersSet[customer.id] = true;
                                 });

                                 if (_.isEmpty(tasksIdsInCurrentSelection))
                                 {
                                     self._setSingleCustomerProjectSelection(singleSelectionByCreatedTasks);
                                     $(TaskManagement.Store).one('createdTaskIdsReset', function()
                                     {
                                         TaskManagement.Store.replaceCreatedTaskIds(createdTaskIds);
                                     });
                                 }
                                 else
                                 {
                                     TaskManagement.Store.appendCreatedTaskIds(tasksIdsInCurrentSelection);
                                 }
                                 self._updateVisibleCustomersProjects();
                                 self.customersProjectsPanel.updateCpSelector();

                                 var affectedProjects = _.keys(affectedProjectsSet);
                                 var affectedCustomers = _.keys(affectedCustomersSet);
                                 var message = "";
                                 if (createdTaskIds.length > 0)
                                 {
                                     if (createdTaskIds.length > 1)
                                         message += messageResource.getMessage("create.tasks.toast", [createdTaskIds.length]);
                                     else
                                         message += messageResource.getMessage("create.task.toast", [TaskStore.get(createdTaskIds[0]).name]);

                                     if (affectedProjects.length > 1)
                                         message += " " + messageResource.getMessage("create.task.toast.project_postfix_plural", [affectedProjects.length]);
                                     else
                                         message += " " + messageResource.getMessage("create.task.toast.project_postfix_singular", [ProjectStore.get(affectedProjects[0]).name]);

                                     if (affectedCustomers.length > 1)
                                         message += " " + messageResource.getMessage("create.task.toast.customer_postfix_plural", [affectedCustomers.length]);
                                     else
                                         message += " " + messageResource.getMessage("create.task.toast.customer_postfix_singular", [CustomerStore.get(affectedCustomers[0]).name]);
                                 }
                                 else
                                 {
                                     message = messageResource.getMessage("create.task.toast.no_tasks_created");
                                 }
                                 Toasts.addToastWithText(message);
                             };
                             showCreateTasksPopup("TaskManagementService", appendCreatedTasks);
                         },
                         true,
                         function(status)
                         {
                             if (status === TaskManagementConstants.CREATE_TASKS_NO_ASSIGNED_PROJECTS_ERROR_CODE)
                             {
                                 alert(messageResource.getMessage("add_new_tasks.no_projects_assigned.error"));
                                 window.location.reload();
                                 return true;
                             }
                             return false;
                         },
                         true);
    },

    selectRowAndOpenTaskPanel: function(taskId)
    {
        this._showEditTaskPanel(taskId);
        var row = this.tasksTable.taskRows[taskId];
        if (_.isObject(this.tasksTable.taskRows[taskId]))
        {
            row._startEditing();
            ScrollUtils.scrollIntoViewVertical(this.tasksTable.verticalScroll, row.$el[0]);
        }
    },

    _showEditTaskPanel: function(taskId)
    {
        if (TaskManagement.Store.getTaskIdInEdit() === taskId)
        {
            this.taskPanelController.panel.hide();
            return;
        }

        if (this.taskPanelController.panel.isShown)
            this._triggerTaskEdited(TaskManagement.Store.getTaskIdInEdit());

        TaskManagement.Store.setTaskIdInEdit(taskId);
        this.taskPanelController.panel.setLoading(true);
        this.taskPanelController.panel.header.actions.hideMenu();
        this.taskPanelController.panel.show();
        var self = this;
        loadPopupContent(BASE_PATH + 'tasks/edit_task_popup.do?taskId=' + taskId + '&fromTM=true',
                         _.bind(this._onTaskDataLoaded, this),
                         false,
                         function(status)
                         {
                             if (status === TaskManagementConstants.TASK_CONCURRENTLY_DELETED)
                             {
                                 self.taskPanelController.panel.hide();
                                 alert(messageResource.getMessage("task.error.concurrently_deleted"));
                                 self.reloadTasks();
                                 return true;
                             }
                             else if (status === TaskManagementConstants.TASK_IS_NOT_ASSIGNED)
                             {
                                 self.taskPanelController.panel.hide();
                                 alert(messageResource.getMessage("task.error.task_not_assigned"));
                                 ProjectStore.remove(TaskStore.get(taskId)['projectId']);
                                 self._updateVisibleCustomersProjects();
                                 self.reloadTasks();
                                 return true;
                             }
                             return false;
                         },
                         true);
    },

    _onTaskDataLoaded: function()
    {
        var loadedData = getEditTaskData();
        var taskData = loadedData['taskDetails'];
        var taskProject = Project.fromRPC(taskData.task['project']);
        var taskCustomer = Customer.fromRPC(taskData.task['project']['customer']);
        var taskFromStore = _.clone(TaskStore.get(taskData.task.id));
        var taskFromRequest = Task.fromRPC(taskData.task);

        // TODO: task data should conform the format used elsewhere so that next two lines are not needed
        taskFromRequest.deadline = _.isString(taskData.task['deadlineDate']) ? Date.parseJSONDate(taskData.task['deadlineDate']) : null;
        taskFromRequest.lastTrackingDate = taskFromStore ? taskFromStore.lastTrackingDate : null;

        TaskStore.add((taskFromStore) ? _.extend(taskFromStore, taskFromRequest) : taskFromRequest);
        ProjectStore.add(taskProject);
        CustomerStore.add(taskCustomer);

        BillingTypeStore.setDefaultBillingTypeId(loadedData['defaultBillingTypeId']);
        StoreUtils.replaceBillingTypesFromRPC(loadedData);
        this.taskPanelController.loadStateFromRequest(taskData, loadedData['canEdit'], loadedData['btyEnabled']);
        this.taskPanelController.loadPanel();
    },

    _createProjectSuccess: function(result)
    {
        var customer = Customer.fromRPC(result["customer"]);
        CustomerStore.addFirst(customer);

        var project = Project.fromRPC(result["project"]);
        project.archived = false;
        ProjectStore.addFirst(project);

        var createdTaskList = result["tasks"].list;
        if (createdTaskList.length > 0)
        {
            this._markTaskShortcutIfPresent();
        }
        var createdTaskIds = [];
        _.forEach(createdTaskList, function(taskInfo)
        {
            var task = Task.fromRPC(taskInfo);

            TaskStore.add(task);
            createdTaskIds.push(task.id);
        });

        var newFilterCpSelection = TaskManagement.Store.getFilterCustomerProjectSelection();
        if (newFilterCpSelection.qualifier === CustomersProjectsSelectionQualifier.SELECTED)
            newFilterCpSelection = new CustomerProjectSelection(CustomersProjectsSelectionQualifier.SELECTED, newFilterCpSelection.customers, newFilterCpSelection.projects.concat(project.id));
        
        this._updateVisibleCustomersProjectsAfterNewDataLoaded(newFilterCpSelection, {customerId: customer.id, projectId: project.id});

        Toasts.addToastWithText(messageResource.getMessage("create.project.toast", [project.name]));
        if (TaskManagement.Store.getFilterCustomerProjectSelection().includesProject(project))
            this._setSingleCustomerProjectSelection({customerId: customer.id, projectId: project.id});
    },

    _createProjectFailure: function(jqXHR, textStatus, errorThrown)
    {
        console.log(errorThrown);
    },

    _showCreateCustomerPopup: function()
    {
        var self = this;
        loadPopupContent(BASE_PATH + 'project/customeradd.do',
                         function()
                         {
                             showCreateCustomerPopup(function(result)
                                                     {
                                                         // TODO: this piece of code is very similar to handlers for creating projects and tasks from popups

                                                         var customer = Customer.fromRPC(result["customer"]);
                                                         CustomerStore.addFirst(customer);

                                                         // if tasks copied (aka created) need to check shortcut
                                                         if (result["taskCount"] > 0)
                                                         {
                                                             self._markTaskShortcutIfPresent();
                                                         }

                                                         if (result["projects"])
                                                             ProjectStore.mergeAll(result["projects"].list.map(function(data)
                                                                                                               {
                                                                                                                   var p = Project.fromRPC(data);
                                                                                                                   p.archived = false;
                                                                                                                   return p;
                                                                                                               }));

                                                         var newFilterCpSelection = TaskManagement.Store.getFilterCustomerProjectSelection();
                                                         if (newFilterCpSelection.qualifier === CustomersProjectsSelectionQualifier.SELECTED)
                                                             newFilterCpSelection = new CustomerProjectSelection(CustomersProjectsSelectionQualifier.SELECTED,
                                                                                                                 newFilterCpSelection.customers.concat(customer.id),
                                                                                                                 newFilterCpSelection.projects);
                                                         self._updateVisibleCustomersProjectsAfterNewDataLoaded(newFilterCpSelection,
                                                                                                                {customerId: customer.id, projectId: null});

                                                         Toasts.addToastWithText(messageResource.getMessage("create.customer.toast", [customer.name]));
                                                         if (TaskManagement.Store.getFilterCustomerProjectSelection().includesCustomer(customer))
                                                             self._setSingleCustomerProjectSelection({customerId: customer.id, projectId: null});
                                                     });
                         },
                         true,
                         null,
                         true);

    },

    _showCreateProjectPopup: function()
    {
        var self = this;
        var store = TaskManagement.Store;
        var filter = store.getFilterCustomerProjectSelection();
        
        var singleSelection = store.getSingleCustomerProjectSelection();
        var customerId = (!_.isNull(singleSelection.customerId)) ? singleSelection.customerId : null;
        
        var params = ( "?customerId=" + customerId);
        loadPopupContent(BASE_PATH + 'createproject.do' + params,
                         function()
                         {
                             showCreateProjectPopup(customerId, function(result) { self._createProjectSuccess(result) }, function(result) { self._createProjectFailure(result) });
                         },
                         true,
                         null,
                         true);

    },

    _updateVisibleCustomersProjects: function()
    {
        $(document).trigger("taskManagement_visibleCustomersWillChange");

        var selection = TaskManagement.Store.getIntersectedCustomerProjectSelection();
        if (selection.projects.length === 1 && !ProjectStore.get(selection.projects[0]) ||
            selection.customers.length === 1 && !CustomerStore.get(selection.customers[0]))
        {
            this._setSingleCustomerProjectSelection({customerId: null, projectId: null});
            this.customersProjectsPanel.updateCpSelector();
        }

        TaskManagement.Store.setVisibleCustomers(this.buildVisibleCustomers());
        if (TaskManagement.Store.getVisibleCustomersFiltered())
            TaskManagement.Store.setVisibleCustomersFiltered(this.buildVisibleCustomersFiltered());
    },

    _updateVisibleCustomersProjectsAfterNewDataLoaded: function(newFilterCpSelection, newSingleCpSelection)
    {
        TaskManagement.Store.setFilterCustomerProjectSelection(newFilterCpSelection);
        this.customersProjectsPanel.updateCpSelector();
        this._setSingleCustomerProjectSelection(newSingleCpSelection);
        this._updateVisibleCustomersProjects();
        this.savePageSettings();
    },

    _updateAll: function()
    {
        this._updateVisibleCustomersProjects();
        this.customersProjectsPanel.updateCpSelector();
        this.reloadTasks();
    },

    _setSingleCustomerProjectSelection: function(selection)
    {
        TaskManagement.Store.setSingleCustomerProjectSelection(selection);
        TaskManagement.Store.setIntersectedCustomerProjectSelection(this.intersectCustomerProjectSelections());
        this.reloadTasks();
    },

    _onTasksDeleted: function(taskIds)
    {
        TaskStore.removeMany(taskIds);
        TaskManagement.Store.removeTasks(taskIds);
        $(document).trigger("taskManagement_removeTasks", [taskIds]);
    },

    _triggerTaskEdited: function(taskId)
    {
        TaskManagement.Store.setTaskIdInEdit(null);
        $(document).trigger('taskManagement_task_edited', taskId);
    },

    _triggerTaskUpdated: function(taskId)
    {
        $(document).trigger('taskManagement_task_updated_in_panel', taskId);
    },

    _archiveCustomer: function(customerId)
    {
        var self = this;
        _.map(ProjectStore.getAll(), function(project)
        {
            if (project.customerId === customerId && !project.archived)
            {
                self._archiveProject(project.id);
            }
        });
    },

    _archiveProject: function(projectId)
    {
        var projectToArchive = _.clone(ProjectStore.get(projectId));
        if (!projectToArchive.archived)
        {
            projectToArchive.archived = true;
            ProjectStore.add(projectToArchive);
        }
        var taskCompletedData = {
            taskIds: _.reduce(TaskStore.getAll(),
                              function(idsToComplete, task)
                              {
                                  if(task.projectId === projectId && !task.completed)
                                      idsToComplete.push(task.id);
                                  return idsToComplete;
                              }, []),
            isCompleted: true};
        $(document).trigger("taskManagement_tasksStatusChange", taskCompletedData);
    },

    _markTaskShortcutIfPresent: function()
    {
        // mark on client that task(s) will be created and show just checked shortcut
        if (!_.isUndefined(window.GettingStartedShortcuts))
        {
            GettingStartedShortcuts.markShortcutAsChecked(GettingStartedShortcut.TASKS);
            GettingStartedShortcuts.showJustCheckedShortcutIfExists();
        }
    }
});