TaskManagement.Store = {
    visibleCustomers: [],
    visibleCustomersFiltered: null,
    customerProjectNameFilter: null,
    filterCustomerProjectSelection: null,
    singleCustomerProjectSelection: null,
    intersectedCustomerProjectSelection: null,
    taskSortBy: null,
    taskSortAsc: false,
    taskStatusFilter: null,
    taskNameFilter: null,
    taskIds: new EntityStore().init(),
    createdTaskIds: [],
    allTasksLoaded: false,
    taskSelection: {
        isAllSelected: false,
        selectedTasks: {}
    },
    isNeedApproval: false,
    taskIdInEdit: null,
    managementSettings: {
        canManageTasks: false,
        canManageCP: false
    },

    getVisibleCustomers: function()
    {
        return this.visibleCustomers;
    },

    setVisibleCustomers: function(visibleCustomers)
    {
        this.visibleCustomers = visibleCustomers;
        $(this).trigger("visibleCustomersChange");
    },

    getVisibleCustomersFiltered: function()
    {
        return this.visibleCustomersFiltered;
    },

    setVisibleCustomersFiltered: function(visibleCustomersFiltered)
    {
        this.visibleCustomersFiltered = visibleCustomersFiltered;
        $(this).trigger("visibleCustomersFilteredChange");
    },

    getCustomerProjectNameFilter: function()
    {
        return this.customerProjectNameFilter;
    },

    setCustomerProjectNameFilter: function(customerProjectNameFilter)
    {
        this.customerProjectNameFilter = customerProjectNameFilter;
    },

    getFilterCustomerProjectSelection: function()
    {
        return this.filterCustomerProjectSelection;
    },

    setFilterCustomerProjectSelection: function(filterCustomerProjectSelection)
    {
        this.filterCustomerProjectSelection = filterCustomerProjectSelection;
    },

    getSingleCustomerProjectSelection: function()
    {
        return this.singleCustomerProjectSelection;
    },

    setSingleCustomerProjectSelection: function(singleCustomerProjectSelection)
    {
        this.singleCustomerProjectSelection = singleCustomerProjectSelection;
        $(this).trigger("singleCustomerProjectSelectionChange");
    },

    /**
     * @returns {CustomerProjectSelection}
     */
    getIntersectedCustomerProjectSelection: function()
    {
        return this.intersectedCustomerProjectSelection;
    },

    setIntersectedCustomerProjectSelection: function(intersectedCustomerProjectSelection)
    {
        this.intersectedCustomerProjectSelection = intersectedCustomerProjectSelection;
    },

    getTaskSortBy: function()
    {
        return this.taskSortBy;
    },

    setTaskSortBy: function(taskSortBy)
    {
        this.taskSortBy = taskSortBy;
    },

    getTaskSortAsc: function()
    {
        return this.taskSortAsc;
    },

    setTaskSortAsc: function(taskSortAsc)
    {
        this.taskSortAsc = taskSortAsc;
    },

    getTaskStatusFilter: function()
    {
        return this.taskStatusFilter;
    },

    setTaskStatusFilter: function(taskStatusFilter)
    {
        this.taskStatusFilter = taskStatusFilter;
        $(this).trigger("taskStatusFilterChange");
    },

    getTaskNameFilter: function()
    {
        return this.taskNameFilter;
    },

    setTaskNameFilter: function(taskNameFilter)
    {
        this.taskNameFilter = taskNameFilter;
        $(this).trigger("taskNameFilterChange");
    },

    getTaskIds: function()
    {
        return this.taskIds.getAll()
    },

    isAllTasksLoaded: function()
    {
        return this.allTasksLoaded
    },

    setAllTasksLoaded: function(allTasksLoaded)
    {
        this.allTasksLoaded = allTasksLoaded
    },

    getTaskSelection: function()
    {
        return this.taskSelection;
    },

    setTaskSelection: function(isAllSelected, selectedTasks)
    {
        this.taskSelection = {
            isAllSelected: isAllSelected,
            selectedTasks: selectedTasks
        };
        $(this).trigger("taskSelection_changed");
    },

    removeSelectedTask: function(taskId)
    {
        delete this.taskSelection.selectedTasks[taskId];
        $(this).trigger("taskSelection_changed");
    },

    addSelectedTask: function(taskId)
    {
        this.taskSelection.selectedTasks[taskId] = true;
        $(this).trigger("taskSelection_changed");
    },

    isTaskSelectionEmpty: function()
    {
        return (!this.taskSelection.isAllSelected) && (_.size(this.taskSelection.selectedTasks) === 0);
    },

    getSelectedTasks: function()
    {
        return this.taskSelection.selectedTasks;
    },

    isAllTasksSelected: function()
    {
        return this.taskSelection.isAllSelected;
    },

    setPopupInfo: function(popupInfo)
    {
        this.popupInfo = popupInfo;
        $(this).trigger("popupInfoReceived")
    },

    appendTaskIds: function(taskIds)
    {
        this.taskIds.mergeAll(taskIds);
        $(this).trigger("taskIdsAppend");
    },

    appendCreatedTaskIds: function(taskIds)
    {
        this.createdTaskIds.concat(taskIds);
        this.taskIds.mergeAll(taskIds);
        $(this).trigger("createdTaskIdsAppend", [taskIds]);
    },

    replaceCreatedTaskIds: function(taskIds)
    {
        this.createdTaskIds = taskIds;
        this.taskIds.mergeAll(taskIds);
    },

    getCreatedTaskIds: function()
    {
        return this.createdTaskIds;
    },

    resetCreatedTaskIds: function()
    {
        this.createdTaskIds = [];
        $(this).trigger("createdTaskIdsReset");
    },

    replaceTaskIds: function(taskIds)
    {
        this.taskIds.replaceAll(taskIds);
        this.resetCreatedTaskIds();
        $(this).trigger("taskIdsReplace");
    },

    removeTasks: function(taskIds)
    {
        this.taskIds.removeMany(taskIds);
        this.taskSelection.selectedTasks = _.omit(this.taskSelection.selectedTasks, taskIds);
        $(this).trigger("taskSelection_changed");
    },

    setManagerNeedApproval: function(isNeedApproval)
    {
        this.isNeedApproval = isNeedApproval;
    },

    getManagerNeedApproval: function()
    {
        return this.isNeedApproval;
    },

    setTaskIdInEdit: function(taskId)
    {
        this.taskIdInEdit = taskId;
    },

    getTaskIdInEdit: function()
    {
        return this.taskIdInEdit;
    },

    setManagementSettings: function(settings)
    {
        this.managementSettings['canManageTasks'] = settings['canManageTasks'];
        this.managementSettings['canManageCP'] = settings['canManageCP'];
        this.managementSettings['canAccessCompleteTaskInformation'] = settings['canAccessCompleteTaskInformation'];
    },

    setCanManageTasks: function(canManage)
    {
        this.managementSettings['canManageTasks'] = canManage;
    },

    canManageTasks: function()
    {
        return this.managementSettings['canManageTasks'];
    },

    canManageCP: function()
    {
        return this.managementSettings['canManageCP'];
    },

    canAccessCompleteTaskInformation: function()
    {
        return this.managementSettings['canAccessCompleteTaskInformation'];
    }
};