TaskManagement.CustomersProjectsPanel.Tree = function($containerEl)
{
    WindowSizeDependedView.call(this, $containerEl);
    this.editingNodesSet = {};
};

TaskManagement.CustomersProjectsPanel.Tree.CollapsedCustomersCookies = "tm_collapsed_customers";

TaskManagement.CustomersProjectsPanel.Tree.prototype = _.create(WindowSizeDependedView.prototype, {
    html: "<div class='customersProjectsTree'>" +
          "  <div class='header'>" +
          "    <div class='collapseAllButton'></div>" +
          "    <div class='separator'>&nbsp;</div>" +
          "    <div class='customerNameHeaderContainer'></div>" +
          "    <div class='separator'>&nbsp;</div>" +
          "    <div class='filterCPSelectorContainer'>" +
          "      <div class='icon'>&nbsp;</div>" +
          "      <div id='customersProjectsPanelFilterCPSelectorContainer' class='buttonContainer'></div>" +
          "    </div>" +
          "  </div>" +
          "  <div class='unfilteredContainer'></div>" +
          "  <div class='filteredContainer'></div>" +
          "  <div class='notFoundMessage'><%-notFoundMessage%></div>" +
          "</div>",

    afterRender: function()
    {
        var self = this;

        var collapsedSetFromCookies = JSON.parse($.cookie(TaskManagement.CustomersProjectsPanel.Tree.CollapsedCustomersCookies));
        this.unfilteredCollapsedSet = _.isNull(collapsedSetFromCookies) ? {} : collapsedSetFromCookies;
        this.filteredCollapsedSet = {};
        this.collapsedSet = this.unfilteredCollapsedSet;

        this.find(".collapseAllButton").on("click", function()
        {
            if (_.isEmpty(self.collapsedSet))
                self._collapseAll();
            else
                self._expandAll();
        });

        this.asc = true;
        this.customerNameHeader = new TaskManagement.SortableTableHeader(this.find(".customerNameHeaderContainer"), true, this.asc, "common.Customer").init();
        this.customerNameHeader.on("click", function()
        {
            self.asc = self.customerNameHeader.isAsc();
            self._updateUnfilteredNodes();
            if (self.filtered)
                self._showFiltered();
            else
                self._showUnfiltered();
        });

        this.stringTokenizer = new StringTokenizer(StringTokenizer.PUNCTUATORS_FOR_SEARCH);
        this.$unfilteredContainer = this.find(".unfilteredContainer");
        this.$filteredContainer = this.find(".filteredContainer");

        this.unfilteredScrollView = new ScrollView(this.$unfilteredContainer, 40).init();
        this.filteredScrollView = new ScrollView(this.$filteredContainer, 40).init();

        this._updateUnfilteredNodes();

        WindowSizeDependedView.prototype.afterRender.call(this);

        $(TaskManagement.Store).on("singleCustomerProjectSelectionChange", function()
        {
            if (self.suppressSelectionEvent)
                return;
            self._updateNodesWithSpecialStatus();
            self._currentScrollView().scrollToItem(self.selectedNode);
        });
        $(TaskManagement.Store).on("visibleCustomersChange", function()
        {
            self._updateUnfilteredNodes();
        });
        $(TaskManagement.Store).on("visibleCustomersFilteredChange", function()
        {
            if (TaskManagement.Store.getVisibleCustomersFiltered())
                self._showFiltered();
            else
                self._showUnfiltered();
        });

        $(document).on("taskManagement_visibleCustomersWillChange", function()
        {
            _.each(self.nodes.array, function(node)
            {
                node.disableRendering()
            });
        });
        $(document).on("taskManagement_taskUpdated", function(taskId)
        {
        //   todo: update visible nodes of archived project and customers for this task, if task becomes open
        });

        $(document).on("taskManagement_editCustomer", function(e, customerId)
        {
            self._setNodeEditing("c" + customerId, true)
        });
        $(document).on("taskManagement_customerEditEnd", function(e, customerId)
        {
            self._setNodeEditing("c" + customerId, false)
        });
        $(document).on("taskManagement_customersProjectsPanel_editProject", function(e, projectId)
        {
            self._setNodeEditing("p" + projectId, true)
        });
        $(document).on("taskManagement_projectEditEnd", function(e, projectId)
        {
            self._setNodeEditing("p" + projectId, false)
        });
    },

    adjustSize: function()
    {
        WindowSizeDependedView.prototype.adjustSize.call(this);
        if (!this.filtered)
        {
            this.unfilteredScrollView.adjustHeightToWindowSize();
            this.unfilteredScrollViewHeightValid = true;
            this.filteredScrollViewHeightValid = false;
        }
        else
        {
            this.filteredScrollView.adjustHeightToWindowSize();
            this.filteredScrollViewHeightValid = true;
            this.unfilteredScrollViewHeightValid = false;
        }
    },

    messageKeys: function()
    {
        return {
            notFoundMessage: "task.list.cp_not_found"
        }
    },

    _updateUnfilteredNodes: function()
    {
        this.unfilteredScrollViewHeightValid = false;
        this.unfilteredNodes = this._buildNodes(this.unfilteredScrollView, TaskManagement.Store.getVisibleCustomers(), this.unfilteredCollapsedSet, null);
        this.unfilteredScrollView.setItems(this._filterCollapsedNodes(this.unfilteredNodes.array));
        if (!this.filtered)
        {
            this.nodes = this.unfilteredNodes;
            this._updateNodesWithSpecialStatus();
            this._updateAllExpandedState();
            this._updateNotFoundState();
            this.unfilteredScrollView.adjustHeightToWindowSize();
            this.unfilteredScrollViewHeightValid = true;
        }
    },

    _showFiltered: function()
    {
        this.filtered = true;
        this.$el.addClass("filtered");

        if (!this.filteredScrollViewHeightValid)
        {
            this.filteredScrollView.adjustHeightToWindowSize();
            this.filteredScrollViewHeightValid = true;
        }

        var text = TaskManagement.Store.getCustomerProjectNameFilter();
        var data = TaskManagement.Store.getVisibleCustomersFiltered();
        this.filteredCollapsedSet = {};
        this.collapsedSet = this.filteredCollapsedSet;
        var tokens = this.stringTokenizer.getTokens(text);
        this.nodes = this._buildNodes(this.filteredScrollView,
                                      data,
                                      this.filteredCollapsedSet,
                                      text && tokens.length > 0 ? new TokensHighlighter(tokens, this.stringTokenizer.getPunctuators()) : null);
        this._updateNodesWithSpecialStatus();
        this._updateAllExpandedState();
        this._updateNotFoundState();
        this.filteredScrollView.setItems(this.nodes.array);
    },

    _showUnfiltered: function()
    {
        this.filtered = false;
        this.$el.removeClass("filtered");

        this.collapsedSet = this.unfilteredCollapsedSet;
        this.nodes = this.unfilteredNodes;

        this._updateNodesWithSpecialStatus();
        this._updateAllExpandedState();
        this._updateNotFoundState();

        if (!this.unfilteredScrollViewHeightValid)
        {
            this.unfilteredScrollView.adjustHeightToWindowSize();
            this.unfilteredScrollViewHeightValid = true;
        }
    },

    _updateNodesWithSpecialStatus: function()
    {
        var self = this;

        var findAndSelectNode = function(nodeId)
        {
            var node = self.nodes.map[nodeId];
            if (node)
                node.setSelected(true);
            self._setSelectedNode(node);
        };

        var singleCPSelection = TaskManagement.Store.getSingleCustomerProjectSelection();
        if (singleCPSelection.projectId)
            findAndSelectNode("p" + singleCPSelection.projectId);
        else if (singleCPSelection.customerId)
            findAndSelectNode("c" + singleCPSelection.customerId);
        else
            findAndSelectNode("all");

        _.each(this.editingNodesSet, function(v, nodeKey)
        {
            self._setNodeEditing(nodeKey, true);
        });
    },

    _buildNodes: function(scrollView, data, collapsedSet, tokensHighlighter)
    {
        var self = this;
        var nodesMap = {};
        var nodesArray = [];

        var addNode = function(node, id)
        {
            nodesArray.push(node);
            nodesMap[id] = node;
            return node;
        };
        var onNodeSelect = function(e)
        {
            self._setSelectedNode(e.target)
        };
        var onAllCustomersNodeSelect = function(e)
        {
            onNodeSelect(e);
            defer(function()
                  {
                      self.suppressSelectionEvent = true;
                      $(document).trigger("taskManagement_customersProjectsPanel_allCustomersNode_select");
                      self.suppressSelectionEvent = false;
                  });
        };
        var onProjectNodeSelect = function(e)
        {
            onNodeSelect(e);
            defer(function()
                  {
                      self.suppressSelectionEvent = true;
                      $(document).trigger("taskManagement_customersProjectsPanel_projectNode_select", e.target.projectId);
                      self.suppressSelectionEvent = false;
                  });
        };
        var onCustomerNodeSelect = function(e)
        {
            onNodeSelect(e);
            defer(function()
                  {
                      self.suppressSelectionEvent = true;
                      $(document).trigger("taskManagement_customersProjectsPanel_customerNode_select", e.target.customerId);
                      self.suppressSelectionEvent = false;
                  });
        };
        var onCustomerNodeCollapse = function(e)
        {
            scrollView.setItems(self._filterCollapsedNodes(nodesArray));
            if (e.target.isCollapsed())
                collapsedSet[e.target.customerId] = true;
            else
                delete collapsedSet[e.target.customerId];
            self._saveCollapsedInCookies();

            self._updateAllExpandedState();
        };

        var allCustomersNode = addNode(new TaskManagement.CustomersProjectsPanel.Tree.AllCustomersNode(), "all");
        allCustomersNode.on("select", onAllCustomersNodeSelect);

        var visibleCustomers = _.clone(data);
        if (!this.asc)
            visibleCustomers.reverse();
        _.each(visibleCustomers, function(visibleCustomer)
        {
            var collapsed = !!collapsedSet[visibleCustomer.customerId];
            var customerNode = addNode(new TaskManagement.CustomersProjectsPanel.Tree.CustomerNode(tokensHighlighter, visibleCustomer.customerId, collapsed), "c" + visibleCustomer.customerId);
            customerNode.on("select", onCustomerNodeSelect);
            customerNode.on("collapse", onCustomerNodeCollapse);

            _.each(visibleCustomer.projectIds, function(projectId)
            {
                var projectNode = addNode(new TaskManagement.CustomersProjectsPanel.Tree.ProjectNode(tokensHighlighter, projectId), "p" + projectId);
                projectNode.on("select", onProjectNodeSelect);
            }, this);
        }, this);

        _.each(_.keys(collapsedSet), function(collapsedCustomerId)
        {
            if (!nodesMap["c" + collapsedCustomerId])
            {
                delete collapsedSet[collapsedCustomerId];
            }
        }, this);

        this._saveCollapsedInCookies();
        return {
            map: nodesMap,
            array: nodesArray
        };
    },

    _saveCollapsedInCookies: function()
    {
        $.cookie(TaskManagement.CustomersProjectsPanel.Tree.CollapsedCustomersCookies, JSON.stringify(this.unfilteredCollapsedSet), {expires: 30});
    },

    _setSelectedNode: function(node)
    {
        if (this.selectedNode === node)
            return;

        if (this.selectedNode && this.selectedNode.isAvailableToUpdateSelection())
            this.selectedNode.setSelected(false);
        this.selectedNode = node;
    },

    _setNodeEditing: function(nodeKey, editing)
    {
        var node = this.nodes.map[nodeKey];
        if (node)
            node.setEditing(editing);

        if (editing)
            this.editingNodesSet[nodeKey] = true;
        else
            delete this.editingNodesSet[nodeKey];
    },

    _filterCollapsedNodes: function(nodes)
    {
        var result = [];
        var hasCollapsedNodeAbove = false;
        _.each(nodes, function(node)
        {
            var isCustomerNode = node instanceof TaskManagement.CustomersProjectsPanel.Tree.CustomerNode;

            if (hasCollapsedNodeAbove)
            {
                if (isCustomerNode)
                    hasCollapsedNodeAbove = false;
                else
                    return;
            }

            if (isCustomerNode && node.isCollapsed())
                hasCollapsedNodeAbove = true;

            result.push(node);
        });

        return result;
    },

    _collapseAll: function()
    {
        var self = this;
        _.each(this.nodes.array, function(node)
        {
            if (node instanceof TaskManagement.CustomersProjectsPanel.Tree.CustomerNode && !node.isCollapsed())
            {
                self.collapsedSet[node.customerId] = true;
                node.setCollapsed(true);
            }
        });
        this._saveCollapsedInCookies();

        this._currentScrollView().setItems(this._filterCollapsedNodes(this.nodes.array));

        self._updateAllExpandedState();
    },

    _expandAll: function()
    {
        var self = this;
        _.each(this.nodes.array, function(node)
        {
            if (node instanceof TaskManagement.CustomersProjectsPanel.Tree.CustomerNode && node.isCollapsed())
            {
                delete self.collapsedSet[node.customerId];
                node.setCollapsed(false);
            }
        });
        this._saveCollapsedInCookies();

        this._currentScrollView().setItems(this._filterCollapsedNodes(this.nodes.array));

        self._updateAllExpandedState();
    },

    _updateAllExpandedState: function()
    {
        this.$el.toggleClass("allExpanded", _.isEmpty(this.collapsedSet));
    },

    _updateNotFoundState: function()
    {
        this.$el.toggleClass("notFound", this.nodes.array.length === 1);
    },

    /**
     * @returns {ScrollView}
     * @private
     */
    _currentScrollView: function()
    {
        return this.filtered ? this.filteredScrollView : this.unfilteredScrollView;
    }
});