TaskManagement.CustomerPanelController = function() {};

TaskManagement.CustomerPanelController.prototype = {
    init: function()
    {
        var $taskBlock = $("#taskListBlock");
        var $page = $('#taskManagementPage');

        this.panel = new EditCustomerPanel($taskBlock, $page).init();
        if (!isIE8())
        {
            this.panel.verticalScroll.on('scroll', function(){
                CustomerStatusMenu.getInstance()._scrollHandler();
            });
        }

        this.subscribeForEvents();
        return this;
    },

    subscribeForEvents: function()
    {
        var self = this;
        var panel = self.panel;

        var errorHandler = function(customerId, jqXHR, textStatus, errorThrown)
        {
            if (errorThrown.exception == "com.actimind.actitime.customer.NoSuchCustomerException")
            {
                CustomerStore.remove(customerId);
                if (customerId === self.customerId)
                {
                    alert(messageResource.getMessage("customer.edit.cannot_edit_deleted_concurrently"));
                }
                $(self).trigger("customerDeleted", customerId);
                panel.hide();
                return true;
            }
            if (errorThrown.exception == "com.actimind.actitime.customer.CustomerExistsException")
            {
                if (customerId === self.customerId)
                {
                    self.panel.header.nameInput.error(messageResource.getMessage("customer.edit.error.name_duplicate"));
                }
                return true;
            }
            if (errorThrown.exception == "com.actimind.actitime.task.NoPermissionToModifyTTOfSpecifiedUsers") {
                if (customerId === self.customerId)
                {
                    var alertText = messageResource.getMessage(self.state.customerInfo.canManageAllUsersWithoutAutoApproved ?
                                                             "hierarchy_entity.edit_popup.cannot_delete_due_to_unavailable_modify_time_of_autoapproved_users" :
                                                             "hierarchy_entity.edit_popup.cannot_delete_due_to_unavailable_modify_time",
                                                             [messageResource.getMessage("common.customer")]);
                    alert(alertText);
                    panel.header.actions.disableDeleteButton(alertText);
                    panel.hideDeleteDialog();
                }
                return true;
            }
            if (errorThrown.exception == "com.actimind.actitime.task.TaskHasLockedTTException") {
                if (customerId === self.customerId)
                {
                    var alertText = messageResource.getMessage("customer.edit.cannot_delete_due_to_lock_time");
                    alert(alertText);
                    panel.header.actions.disableDeleteButton(alertText);
                    panel.hideDeleteDialog();
                }
                return true;
            }
            return false;
        };

        function makeUpdateRequest(service, params, targetControl, forceCompleteTasks)
        {
            var customerId = self.customerId;
            targetControl.setLoading(true);

            var paramsCopy = params.slice();
            if (!_.isUndefined(forceCompleteTasks)) {
                paramsCopy.push(forceCompleteTasks);
            }

            makeRpcCall(service,
                        paramsCopy,
                        function(data)
                        {
                            var config = data.customerInfo;
                            // customer set archived => update its projects status
                            if (config.active !== self.state.customerInfo.active)
                            {
                                var customerName = CustomerStore.get(customerId).name;
                                Toasts.addToastWithText(messageResource.getMessage("status.customer.toast", [customerName]));
                            }

                            if (!config.active && self.state.customerInfo.active)
                            {
                                $(self).trigger("customerArchived", customerId);
                            }
                            self.state = _.clone(data);
                            CustomerStore.add(new Customer(config.id, config.name, !config.active));
                            $(self).trigger("customerChanged", customerId);
                            self.changed = true;

                            self.panel.update(data);
                            targetControl.setLoading(false)
                        },
                        function(jqXHR, textStatus, errorThrown)
                        {

                            if (errorThrown.exception == 'com.actimind.actitime.customer.ProjectTasksNotArchivedException')
                            {
                                var message = errorThrown.taskCount > 1 ?
                                              messageResource.getMessage("task.error.complete_tasks.customer_archive", [errorThrown.taskCount]) :
                                              messageResource.getMessage("task.error.complete_one_task.customer_archive");

                                if (confirm(message))
                                {
                                    makeUpdateRequest(service, params, targetControl, true);
                                }
                                else
                                {
                                    targetControl.setLoading(false);
                                    self.panel.update(self.state);
                                }
                                return true;
                            }

                            targetControl.setLoading(false);
                            return errorHandler(customerId, jqXHR, textStatus, errorThrown);
                        });
        }



        panel.header.nameInput.on('change', function(e, text)
        {
            makeUpdateRequest("CustomerInfoService.setCustomerName", [self.customerId, text], panel.header.nameInput);
        });

        panel.header.actions.menu.on('delete', function()
        {
            panel.showDeleteDialog();
        });

        panel.content.details.statusButton.on('change', function(e, active)
        {
            makeUpdateRequest("CustomerInfoService.setCustomerActive", [self.customerId, active], panel.content.details.statusButton, false);
        });

        panel.content.description.on('change', function(e, text)
        {
            makeUpdateRequest("CustomerInfoService.setCustomerDescription", [self.customerId, text], panel.content.description);
        });

        panel.on('delete', function(e, revokeStatuses)
        {
            $(self).trigger("deleteBegin");

            var customerId = self.customerId;
            makeRpcCall("CustomerInfoService.deleteCustomer",
                        [customerId, revokeStatuses],
                        function()
                        {
                            $(self).trigger("deleteEnd");
                            var customerName = CustomerStore.get(customerId).name;
                            Toasts.addToastWithText(messageResource.getMessage("delete.customer.toast", [customerName]));
                            CustomerStore.remove(customerId);
                            $(self).trigger("customerDeleted", customerId);
                            panel.hide();
                        },
                        function(jqXHR, textStatus, errorThrown)
                        {
                            $(self).trigger("deleteEnd");

                            return !!errorHandler(customerId, jqXHR, textStatus, errorThrown);
                        })
        });

        panel.on('hidePanel', function()
        {
            $(self).trigger("hidePanel", {customerId: self.customerId, changed: self.changed});
            self.customerId = null;
            self.changed = false;
        });
    },

    updatePanel: function(config)
    {
        this.panel.update(config);
    },

    openPanel: function(customerId)
    {
        var self = this;
        self.customerId = customerId;

        var panel = this.panel;

        panel.header.actions.hideMenu();
        panel.show();
        panel.setLoading(true);
        var errorHandler =  function(jqXHR, textStatus, errorThrown)
        {
            if (errorThrown.exception == "com.actimind.actitime.customer.NoSuchCustomerException")
            {
                CustomerStore.remove(customerId);
                if (customerId === self.customerId)
                {
                    alert(messageResource.getMessage("project.edit.customer_deleted_concurrently"));
                }
                $(self).trigger("customerDeleted", customerId);
                panel.hide();
                return true;
            }
            return false;
        };
        makeRpcCall("CustomerInfoService.getEditCustomerInfo",
                    [customerId],
                    function(data)
                    {
                        self.state = _.clone(data);
                        panel.setLoading(false);
                        self.panel.update(data);
                        self.panel.show();
                    },
                    errorHandler);
    },

    closePanel: function()
    {
        this.panel.hide();
    }
};

