TaskManagement.EditProjectPanel = function($containerEl, $pageEl)
{
    SlidingPanel.call(this, $containerEl, $pageEl);

    this.TabNames = {
        DETAILS_TAB: TaskManagement.EditProjectPanel.Tabs.DETAILS_TAB
    };

    if (EnabledFeatures.PAS)
        this.TabNames.ASSIGNED_USERS_TAB = TaskManagement.EditProjectPanel.Tabs.ASSIGNED_USERS_TAB;
};

TaskManagement.EditProjectPanel.TaskPanelEvents = {
    hidePanelEventName: "taskManagement_project_edited",
    showPanelEventName: "taskManagement_project_in_edit"
};

TaskManagement.EditProjectPanel.Tabs = {
    DETAILS_TAB: 'details',
    ASSIGNED_USERS_TAB: 'assignedUsers'
};

TaskManagement.EditProjectPanel.prototype = _.create(SlidingPanel.prototype, {

    hidePanelEventName: null,
    showPanelEventName: null,
    type: "edit_project_sliding_panel",

    afterRender: function()
    {
        SlidingPanel.prototype.afterRender.call(this);

        this.$content = this.find(".content");
        this.header = new TaskManagement.EditProjectPanel.Header(this.find(".headerBlock"), this.$el).init();
        this.detailsTab = new TaskManagement.EditProjectPanel.Content(this.$content).init();
        this.moveToDialog = new MoveToDialog(this.$content, MoveToDialog.Mode.CUSTOMER, "task.list.move_to.project.title").init();
        this.$deleteDialogContainer = $("<div class='deleteDialogContainer'></div>").appendTo(this.$content);

        this.tabs = [];
        this.tabs[this.TabNames.DETAILS_TAB] = this.detailsTab;
       
        if (EnabledFeatures.PAS)
        {
            this.assignedUserTab = new TaskManagement.EditProjectPanel.AssignedUsers(this.$content, _.bind(this.onDependantDataUpdate, this), "projectPanel").init();
            this.tabs[this.TabNames.ASSIGNED_USERS_TAB] = this.assignedUserTab;
        }
        
        
        this._subscribeTabs();
        
        this.listeners = [this.header];
    },

    onDependantDataUpdate: function(event) 
    {
        _.each(this.listeners, function(listener) {listener.onDependantDataUpdate(event)});
    },
    
    update: function(config)
    {
        if (!config)
            return;

        this.projectId = config.id;
        this.header.update(config);
        this.detailsTab.update(config);
        
        if (this.assignedUserTab && config.assignmentUsers) 
        {
            this.assignedUserTab.update(config);
        }
        
        this._initDeleteDialog(config);

        if (!this.selectedTab)
            this._switchTab(this.TabNames.DETAILS_TAB);
        this.hideDeleteDialog();
        this.hideMoveToDialog();
    },

    hide: function()
    {
        SlidingPanel.prototype.hide.call(this);
        if (this.hidePanelEventName)
        {
            $(document).trigger(this.hidePanelEventName, this.projectId);
        }
    },

    show: function()
    {
        if (this.isShown)
            return;
        SlidingPanel.prototype.show.call(this);
        if (this.showPanelEventName)
        {
            $(document).trigger(this.showPanelEventName, this.projectId);
        }
    },

    showDeleteDialog: function()
    {
        if (!this.deleteDialogShown)
        {
            this.deleteDialogShown = true;
            this.moveToDialogShown = false;

            this.tabs[this.selectedTab].hide();
            this.header.actions.menu.selectDelete(true);
            if (this.deleteDialog)
                this.moveToDialog.hide();
            this.deleteDialog.show();
        }
    },

    hideDeleteDialog: function()
    {
        this.deleteDialogShown = false;
        if (this.deleteDialog)
            this.deleteDialog.hide();
        this.header.actions.menu.selectDelete(false);
        this.tabs[this.selectedTab].show();
    },

    showMoveToDialog: function()
    {
        if (!this.moveToDialogShown)
        {
            this.moveToDialogShown = true;
            this.deleteDialogShown = false;

            this.tabs[this.selectedTab].hide();
            this.header.actions.menu.selectMove(true);
            if (this.deleteDialog)
                this.deleteDialog.hide();
            this.moveToDialog.setShowArchived(ProjectStore.get(this.projectId).archived);
            this.moveToDialog.show();
        }
    },

    hideMoveToDialog: function()
    {
        this.moveToDialogShown = false;
        this.moveToDialog.hide();
        this.header.actions.menu.selectMove(false);
        this.tabs[this.selectedTab].show();
        this.header.actions.hideMenu();
    },

    _initDeleteDialog: function(config)
    {
        var self = this;
        delete this.deleteDialog;
        this.$deleteDialogContainer.empty();
        if (!config.blockedDueToLockedTime && !config.blockedDueToUnavailableToModifyTimeTrack)
        {
            this.deleteDialog = new CPTManagement.DeleteProjectPopupConfirmationDialog({
                                                                                           idPrefix: "projectPanel",
                                                                                           $containerElement: this.$deleteDialogContainer,
                                                                                           entityName: config.name,
                                                                                           entitiesToDeleteCount: 1,
                                                                                           entitiesBlocked: 0,
                                                                                           entitiesBlockedCount: 0,
                                                                                           entitiesBlockedDueToLockedTimeCount: 0,
                                                                                           entitiesBlockedDueToUnavailableToModifyTimeTrackCount: 0,
                                                                                           entitiesWithManagerTimeTrackCount: config.projectHasManagerTimeTrack ? 1 : 0,
                                                                                           entitiesWithOtherUsersTimeTrackCount: config.projectHasOtherUsersTimeTrack ? 1 : 0,
                                                                                           entitiesWithManagerApprovedTimeTrackCount: config.projectHasManagerApprovedTimeTrack ? 1 : 0,
                                                                                           entitiesWithOtherUsersApprovedTimeTrackCount: config.projectHasOtherUsersApprovedTimeTrack ? 1 : 0,
                                                                                           hasManagerApprover: config.hasManagerApprover,
                                                                                           cancelHandler: function()
                                                                                           {
                                                                                               $(self).trigger("cancelDelete")
                                                                                           },
                                                                                           deleteHandler: function(revokeStatuses)
                                                                                           {
                                                                                               $(self).trigger("delete", revokeStatuses)
                                                                                           }
                                                                                       }).init();
            this.deleteDialog.hide();
        }
    },

    _switchTab: function(tab)
    {
        if (this.selectedTab !== tab)
        {
            this.selectedTab = tab;
            var self = this;
            _.forEach(this.TabNames, function(key)
            {
                self.header.$tabs[key].toggleClass('selected', key === tab);
                self.tabs[key].$el.toggle(key === tab);
            });
            this.hideDeleteDialog();
            this.hideMoveToDialog();
        }
    },

    _subscribeTabs: function()
    {
        this._switchTab(this.TabNames.DETAILS_TAB);
        var self = this;
        _.forEach(this.TabNames, function(key)
        {
            self.header.$tabs[key].click(_.bind(self._switchTab, self, key));
        });
    },

    hideAnimation: function()
    {
        return "anim-hide-slide-right"
    },

    showAnimation: function()
    {
        return "anim-show-slide-right"
    },

    onHide: function()
    {
        this.header.actions.hideMenu();
    }
});

