TaskManagement.ProjectPanelController = function() {};

TaskManagement.ProjectPanelController.prototype = {

    init: function()
    {
        var $taskBlock = $("#taskListBlock");
        var $page = $('#taskManagementPage');
        this.panel = new TaskManagement.EditProjectPanel($taskBlock, $page).init();

        if (!isIE8())
        {
            this.panel.verticalScroll.on('scroll', function(){
                CustomerStatusMenu.getInstance()._scrollHandler();
            });
        }

        this.subscribeForEvents();
        return this;
    },

    subscribeForEvents: function()
    {
        var self = this;
        var panel = this.panel;

        function makeUpdateRequest(service, stateFieldName, args, targetControl, revertValueFn, customSuccessHandler, customErrorHandler)
        {
            var projectId = self.projectId;
            var newValue = _.isArray(args) ? args[0] : args;
            targetControl.setLoading(true);
            var setLoadingOfTargetControlToFalseAndCorrectState = function(newValue)
            {
                if (_.isUndefined(newValue))
                    revertValueFn();
                else if (stateFieldName && !_.isUndefined(self.state[stateFieldName]))
                    self.state[stateFieldName] = newValue;

                targetControl.setLoading(false);
            };
            makeRpcCall(service,
                        _.flatten([projectId, [args]]),
                        function()
                        {
                            if (projectId !== self.projectId)
                                return true;
                            self._updateProjectInStore(projectId);
                            setLoadingOfTargetControlToFalseAndCorrectState(newValue);
                            if (customSuccessHandler)
                                customSuccessHandler(projectId);
                        },
                        function(jqXHR, textStatus, errorThrown)
                        {
                            if (projectId !== self.projectId)
                                return true;
                            setLoadingOfTargetControlToFalseAndCorrectState();
                            return self._getErrorHandlerWithSelectedProjectDeletedConcurrentlyText(projectId, customErrorHandler)(jqXHR, textStatus, errorThrown);
                        });
        }

        panel.header.nameInput.on('change', function(e, name)
        {
            var projectId = self.projectId;
            makeUpdateRequest("ProjectInfoService.updateProjectName", "name", name, panel.header.nameInput,
                              function() {},
                              function()
                              {
                                  panel.header.nameInput.resetName(name);
                                  self._updateProjectInStore(projectId);
                              },
                              function(jqXHR, textStatus, errorThrown)
                              {
                                  if (errorThrown.exception == 'com.actimind.actitime.project.ProjectExistsException')
                                  {
                                      self.panel.header.nameInput.error(messageResource.getMessage("project.edit.project_name_exists"));
                                      return true;
                                  }
                                  return false;
                              });
        });

        panel.header.actions.menu.on('delete', function()
        {
            panel.showDeleteDialog();
        });

        panel.on('delete', function(e, revokeStatuses)
        {
            var projectId = self.projectId;
            var customErrorHandler = function(jqXHR, textStatus, errorThrown)
            {
                if (projectId !== self.projectId)
                    return false;

                if (errorThrown.exception == "com.actimind.actitime.task.TaskHasLockedTTException")
                {
                    alert(messageResource.getMessage("project.edit.cannot_delete_due_to_lock_time"));
                    panel.header.actions.disableDeleteButton(messageResource.getMessage("project.edit.cannot_delete_due_to_lock_time"));
                    panel.hideDeleteDialog();
                    return true;
                }
                if (errorThrown.exception == "com.actimind.actitime.task.NoPermissionToModifyTTOfSpecifiedUsers")
                {
                    alert(messageResource.getMessage("project.edit.cannot_delete_due_to_unavailable_to_modify"));
                    panel.header.actions.disableDeleteButton(
                            messageResource.getMessage("hierarchy_entity.edit_popup.cannot_delete_due_to_unavailable_modify_time",
                                                       [messageResource.getMessage("common.project")]));
                    panel.hideDeleteDialog();
                    return true;
                }
                return false;
            };

            $(self).trigger("deleteBegin");
            makeRpcCall("ProjectInfoService.deleteProject",
                        [projectId, revokeStatuses],
                        function()
                        {
                            $(self).trigger("deleteEnd");
                            var projectName = ProjectStore.get(projectId).name;
                            Toasts.addToastWithText(messageResource.getMessage("delete.project.toast", [projectName]));
                            self._deleteProjectFromStore(projectId);
                            panel.hide();
                        },
                        function(jqXHR, textStatus, errorThrown)
                        {
                            $(self).trigger("deleteEnd");
                            return self._getErrorHandlerWithSelectedProjectDeletedConcurrentlyText(projectId, customErrorHandler)(jqXHR, textStatus, errorThrown)
                        });
        });

        panel.on('cancelDelete', function()
        {
            panel.hideDeleteDialog();
        });

        panel.header.actions.menu.on('move', function()
        {
            panel.showMoveToDialog();
        });

        panel.moveToDialog.on('move', function(e, data)
        {
            var projectId = self.projectId;
            var customerId = data.customer.id;
            makeUpdateRequest("ProjectInfoService.changeCustomer",
                              null,
                              customerId,
                              panel.moveToDialog,
                              function(){},
                              function(projectId)
                              {
                                  var project = ProjectStore.get(projectId);
                                  if (project)
                                  {
                                      project = _.clone(project);
                                      project.customerId = data.customer.id;
                                      ProjectStore.addFirst(project);

                                      var projectName = project.name;
                                      var customerName = CustomerStore.get(project.customerId).name;
                                      panel.header.updateCustomerName(project.customerId);
                                      Toasts.addToastWithText(messageResource.getMessage("move.project.toast", [projectName, customerName]));

                                      $(self).trigger("projectMoved", projectId);
                                      self._triggerProjectChanged(project.id);
                                  }
                                  panel.hideMoveToDialog();
                              },
                              function(jqXHR, textStatus, errorThrown)
                              {
                                  if (errorThrown.exception == 'com.actimind.actitime.customer.NoSuchCustomerException')
                                  {
                                      CustomerStore.remove(customerId);
                                      alert(messageResource.getMessage("project.edit.customer_deleted_concurrently"));
                                      panel.moveToDialog.updateSelector();
                                      $(self).trigger("customerDeleted", customerId);
                                      return true;
                                  }
                                  if (errorThrown.exception == 'com.actimind.actitime.project.ProjectExistsException')
                                  {
                                      alert(messageResource.getMessage("project.error.project_with_same_name_exists"));
                                      return true;
                                  }
                                  if (errorThrown.exception == 'com.actimind.actitime.customer.CustomerArchivedException')
                                  {
                                      alert(messageResource.getMessage("project.edit.customer_archived_concurrently"));
                                      return true;
                                  }
                                  return false;
                              });

        });

        panel.moveToDialog.on('cancel', function()
        {
            panel.hideMoveToDialog();
        });

        var updateStatusRequest = function(active, forceReopenCustomer, forceCompleteTasks)
        {
            makeUpdateRequest("ProjectInfoService.updateProjectStatus", "status", [active, forceReopenCustomer, forceCompleteTasks], panel.detailsTab.details.statusButton,
                              function()
                              {
                                  panel.detailsTab.details.statusButton.setActive(self.state.status);
                              },
                              function(projectId)
                              {
                                  var project = ProjectStore.get(projectId);
                                  if (!active)
                                  {
                                      $(self).trigger("projectArchived", projectId);
                                      self.changed = true;
                                  }
                                  else if (forceReopenCustomer)
                                  {
                                      var customer = _.clone(CustomerStore.get(project.customerId));
                                      customer.archived = false;
                                      CustomerStore.add(customer);
                                      self._triggerCustomerChanged(customer.id);
                                  }

                                  var projectName = project.name;
                                  Toasts.addToastWithText(messageResource.getMessage("status.project.toast", [projectName]));
                              },
                              function(jqXHR, textStatus, errorThrown)
                              {
                                  if (errorThrown.exception == 'com.actimind.actitime.customer.CustomerArchivedException')
                                  {
                                      if (confirm(messageResource.getMessage('project.error.reopen.customer_archived')))
                                      {
                                          panel.detailsTab.details.statusButton.setActive(true);
                                          updateStatusRequest(active, true, false);
                                      }
                                      return true;
                                  }

                                  if (errorThrown.exception == 'com.actimind.actitime.customer.ProjectTasksNotArchivedException')
                                  {
                                      var message = errorThrown.taskCount > 1 ? 
                                                    messageResource.getMessage("task.error.complete_tasks.project_archive", [errorThrown.taskCount]) :
                                                    messageResource.getMessage("task.error.complete_one_task.project_archive");
                                      
                                      if (confirm(message))
                                      {
                                          panel.detailsTab.details.statusButton.setActive(false);
                                          updateStatusRequest(active, false, true);
                                      }
                                      return true;
                                  }
                              });
        };

        panel.detailsTab.details.statusButton.on('change', function(e, active)
        {
            updateStatusRequest(active, false, false);
        });

        panel.detailsTab.description.on('change', function(e, text)
        {
            makeUpdateRequest("ProjectInfoService.updateProjectDescription", "description", text, panel.detailsTab.description,
                              function()
                              {
                                  panel.detailsTab.description.setDescription(self.state.description);
                              },
                              null, null);
        });

        if (panel.assignedUserTab) 
        {
            panel.assignedUserTab.on('projectTeamChanged', function(e, projectTeam)
            {
                if (self.updateProjectTeamRequest)
                    self.updateProjectTeamRequest.abort();

                var projectId = self.projectId;

                var customErrorHandler = function(jqXHR, textStatus, errorThrown)
                {
                    if (projectId !== self.projectId)
                        return false;

                    if (errorThrown.exception == 'com.actimind.actitime.user.NoSuchUsersException')
                    {
                        alert(messageResource.getMessage('project.edit.assigned_deleted_users'));
                        panel.hide();
                        return true;
                    }
                    return false;
                };

                self.updateProjectTeamRequest = makeRpcCall("ProjectInfoService.updateProjectTeam",
                                                            [projectId, projectTeam],
                                                            function() { },
                                                            self._getErrorHandlerWithSelectedProjectDeletedConcurrentlyText(projectId, customErrorHandler));
            });
        }

        panel.on("hidePanel", function() {
            $(self).trigger("hidePanel", {projectId: self.projectId, changed: self.changed});
            self.projectId = 0;
            self.changed = false;
            panel.header.actions.hideMenu();
            self._abortLoadPanelRequest();
        });
    },

    _updateProjectInStore: function(projectId)
    {
        var existingProject = ProjectStore.get(projectId);
        if (existingProject)
        {
            ProjectStore.add(new Project(
                    projectId,
                    this.panel.header.nameInput.getName(),
                    existingProject.customerId,
                    !this.panel.detailsTab.details.statusButton.isActive()
            ));
        }
        this._triggerProjectChanged(projectId);
    },

    _deleteProjectFromStore: function(projectId)
    {
        ProjectStore.remove(projectId);
        $(this).trigger("projectDeleted", projectId);
    },

    _abortLoadPanelRequest: function()
    {
        if (this.loadPanelRequest)
            this.loadPanelRequest.abort();
    },

    openPanel: function(projectId)
    {
        this._abortLoadPanelRequest();

        var self = this;

        this.projectId = projectId;

        var panel = this.panel;
        panel.show();
        panel.setLoading(true);
        panel.header.actions.hideMenu();

        panel.header.nameInput.setLoading(false);
        panel.moveToDialog.setLoading(false);
        panel.detailsTab.details.statusButton.setLoading(false);
        panel.detailsTab.description.setLoading(false);

        this.loadPanelRequest = makeRpcCall("ProjectInfoService.getEditProjectData",
                                            [projectId],
                                            function(data)
                                            {
                                                var projectFromStore = ProjectStore.get(projectId) ?
                                                                       _.clone(ProjectStore.get(projectId)) : new Project(projectId);
			                        			var customerFromStore = CustomerStore.get(data.customerId) ?
                                                                        _.clone(CustomerStore.get(data['customerId'])) : new Customer(data['customerId']);
			                        			ProjectStore.add(_.extend(projectFromStore,
                                                                          {name: data['name'], customerId: data['customerId'], archived: !data['active']}));
		                        				CustomerStore.add(_.extend(customerFromStore, {name: data['customerName']}));
                                                if (projectId === self.projectId)
                                                {
                                                    panel.setLoading(false);
                                                    panel.update(data);
                                                    self._keepState(data);
                                                }
                                            },
                                            self._getErrorHandlerWithSelectedProjectDeletedConcurrentlyText(projectId)
                                            );
    },

    closePanel: function()
    {
        this.panel.hide()
    },

    _keepState: function(data)
    {
        this.state = {
            name: data.name,
            description: data.description,
            status: data.active
        }
    },

    _getErrorHandlerWithSelectedProjectDeletedConcurrentlyText: function(projectId, customHandler)
    {
        return this._getErrorHandler(projectId, messageResource.getMessage("project.edit.deleted_concurrently"), customHandler);
    },

    _getErrorHandler: function(projectId, deletedConcurrentlyText, customHandler)
    {
        return _.bind(function(jqXHR, textStatus, errorThrown)
                      {
                          if (errorThrown.exception == 'com.actimind.actitime.project.NoSuchProjectException')
                          {
                              this._deleteProjectFromStore(projectId);
                              if (projectId === this.projectId)
                              {
                                  this.panel.hide();
                                  alert(deletedConcurrentlyText);
                              }
                              return true;
                          }

                          if (customHandler)
                              return customHandler(jqXHR, textStatus, errorThrown);

                          return false;
                      }, this);
    },

    _triggerProjectChanged: function(projectId)
    {
        $(this).trigger("projectChanged", projectId);
        this.changed = true;
    },

    _triggerCustomerChanged: function(customerId)
    {
        $(this).trigger("customerChanged", customerId);
        this.changed = true;
    }
};