TaskManagement.TaskPanelController = function() {};

TaskManagement.TaskPanelController.prototype = {

    init: function()
    {
        this.state = new TaskManagement.TaskPanelController.State();
        var $taskBlock = $("#taskListBlock");
        var $page = $('#taskManagementPage');
        this.panel = new TaskPanel($taskBlock, $page).init();
        var deadlineEditor = this.panel.tabs[TaskPanel.Tabs.DETAILS_TAB].deadlineEditor;
        deadlineEditor.setDataPickerMenu(TaskManagement.DatePickerMenu.getInstance());
        deadlineEditor.setDateFormat(Date.convertSmartToExtDateFormat( SystemSettings.dateFormat ).replace(/F/, "M"));
        this._subscribeForEditPanelEvents(this.panel);


        this.editTaskServiceHelper = new EditTaskServiceHelper(this.panel);
        var self = this;

        this.editTaskServiceHelper.setNoRightsToManageTaskCallback(function()
                                                                   {
                                                                       alert( messageResource.getMessage("task.edit_popup.no_rights_to_manage_tasks") );
                                                                       self.panel.hide()
                                                                   });
        this.editTaskServiceHelper.setNoSuchTaskCallback(function(taskId)
                                                         {
                                                             alert( messageResource.getMessage("task.error.concurrently_deleted.cannot_edit") );
                                                             self.panel.hide();
                                                             self._triggerTaskRemoved(taskId);
                                                         });
        this.editTaskServiceHelper.setProjectNotAssignedCallback(function(taskId)
                                                                 {
                                                                     alert( messageResource.getMessage("task.error.task_not_assigned") );
                                                                     self.panel.hide();
                                                                     $(self).trigger('projectDeleted', TaskStore.get(taskId)['projectId']);
                                                                 });

        if (!isIE8())
        {
            this.panel.verticalScroll.on('scroll', function(){
                TaskManagement.DatePickerMenu.getInstance()._onScroll();
                TaskStatusMenu.getInstance()._scrollHandler();
                TypeOfWorkMenu.getInstance()._scrollHandler();
            });
        }
        return this;
    },

    loadStateFromRequest: function(data, canEdit, btyEnabled)
    {
        this.state['canEdit'] = canEdit;
        this.state.setCompleteDetailsAccess(data['completeDetailsAccess']);
        this.state.setExecutorsSummary(data['executorsSummary'] ? data['executorsSummary'].total : null,
                                       data['executorsSummary'] ? data['executorsSummary'].records.list : null);
        this.state.setTask(data['task']);
        this.state.setBtyEnabled(btyEnabled);

        _.extend(this.state, _.pick(data, _.keys(_.omit(this.state, 'executorsSummary', 'reportedTimeComments', 'task'))));
    },

    loadPanel: function()
    {
        var canEdit = this.state['canEdit'];
        if (this.panel.editable !== canEdit)
            this.panel.setEditable(canEdit);

        var detailsTab = this.panel.tabs[TaskPanel.Tabs.DETAILS_TAB];
        var taskId = this.state.getTaskId();
        var task = TaskStore.get(taskId);
        this.panel.setTaskId(taskId);
        this.panel.header.setReportedTime(this.state['executorsSummary']['total']);
        this.panel.header.updateActions(this.state);
        this.panel.header.actions.hideMenu();
        this.panel.updateDeleteDialog(task.name, this.state);

        this._setTypeOfWork();

        detailsTab.setCompleteDetailsAccess(this.state['completeDetailsAccess']);
        detailsTab.progressBar.setSpentTime(this.state['executorsSummary']['total']);
        detailsTab.setDescription(this.state['task']['description']);
        var reportedTime = this.panel.tabs[TaskPanel.Tabs.REPORTED_TAB];
        reportedTime.updateSummaryRecords(this.state['executorsSummary']['total'],
                                          task['estimatedTime'],
                                          this.state['executorsSummary']['records']);
        reportedTime.clearTimeRows();
        reportedTime.setTableSettings(this.state['settings']);
        this.panel.update();
        this.panel.header.taskNameInput.resetName(task.name);
        this._stopLoading();

        this._refreshSelectedTab();

        return this.panel;
    },

    _setTypeOfWork: function()
    {
        var detailsTab = this.panel.tabs[TaskPanel.Tabs.DETAILS_TAB];
        detailsTab.typeOfWorkEditor.getMenu().updateTypesOfWork();
        if (!this.state['btyEnabled'])
        {
            detailsTab.typeOfWorkEditor.setTypeOfWorkInfo(null);
            return;
        }

        if (this.state['canEdit'])
            detailsTab.typeOfWorkEditor.setTypeOfWorkId(this.state['task']['billingTypeId']);
        else
            detailsTab.typeOfWorkEditor.setTypeOfWorkInfo(this.state['task']['billingTypeInfo']);
    },

    _stopLoading: function()
    {
        this.panel.setLoading(false);
        this.panel.header.taskNameInput.setLoading(false);
        var detailsTab = this.panel.tabs[TaskPanel.Tabs.DETAILS_TAB];
        detailsTab.taskStatusButton.setLoading(false);
        detailsTab.deadlineEditor.setLoading(false);
        detailsTab.typeOfWorkEditor.setLoading(false);
        detailsTab.estimatedTimeEditor.setLoading(false);
        detailsTab.description.setLoading(false);
    },

    _subscribeForPanelEvents: function(panel)
    {
        // -- reported time and comments load todo:(refactor me)
        var self = this;
        var reportedTime = panel.tabs[TaskPanel.Tabs.REPORTED_TAB];
        var details = panel.tabs[TaskPanel.Tabs.DETAILS_TAB];
        var limit = 5;

        panel.on('hidePanel', function()
        {
            $(self).trigger('taskEdited', panel.taskId);
            panel.setTaskId(null);
        });

        var sortChangedHandler = function(header)
        {
            reportedTime.dateHeader.setActive(header === reportedTime.dateHeader);
            reportedTime.userHeader.setActive(header === reportedTime.userHeader);
            header.setAsc(header.isAsc());
            var newSortedOrder = (header.isAsc()) ? TaskPanel.ReportedTime.SORT_CONSTANTS['asc'] : TaskPanel.ReportedTime.SORT_CONSTANTS['desc'];
            self.state.setSettings({sortBy: header.sortBy, sortOrder: newSortedOrder, recordsPerPage: limit});
            reportedTime.clearTimeRows();
            reportedTime.setTimeAndCommentsLoading(true);
            self.editTaskServiceHelper.makeRequest(EditTaskServiceHelper.Methods.TIME_COMMENTS, [panel.taskId, header.sortBy, newSortedOrder, 0, 1],
                                                  function(result) { self._updateLoadedTimeComments(result) });
        };

        reportedTime.dateHeader.on('click', _.bind(sortChangedHandler, this, reportedTime.dateHeader));
        reportedTime.userHeader.on('click', _.bind(sortChangedHandler, this, reportedTime.userHeader));

        panel.on('scrollToEnd',
                 function()
                 {
                     if (panel.selectedTab === TaskPanel.Tabs.REPORTED_TAB)
                     {
                         if (!self.state['allTimeCommentsLoaded'])
                         {
                             reportedTime.setTimeAndCommentsLoading(true);
                             ScrollUtils.refreshCustomScroll(panel.verticalScroll);
                             self.editTaskServiceHelper.makeRequest(EditTaskServiceHelper.Methods.TIME_COMMENTS, [panel.taskId, self.state.settings['sortBy'],
                                                                                                                  self.state.settings['sortOrder'], 0,
                                                                                                                  Math.round(self.state['reportedTimeComments'].length / limit) + 1],
                                                                    function(result) { self._appendLoadedTimeComments(result) });
                         }
                     }
                 });

        panel.on('switchTab', function()
        {
            if (!panel.isShown)
                return;
            self._refreshSelectedTab();
        });

        panel.header.actions.menu.on('delete', function()
        {
            panel.showDeleteDialog();
        });

        panel.on('delete', function(e, revokeStatuses)
        {
            var taskId = self.state.task.id;

            var customErrorHandler = function(jqXHR, textStatus, errorThrown)
            {
                return self.handleDeleteException(taskId, panel, errorThrown);
            };

            self.editTaskServiceHelper.makeRequest(EditTaskServiceHelper.Methods.DELETE_TASK, [taskId, revokeStatuses],
                                                   function()
                                                   {
                                                       panel.hide();

                                                       var taskName = TaskStore.get(taskId).name;
                                                       Toasts.addToastWithText(messageResource.getMessage("delete.tasks.toast.singular", [taskName]));

                                                       self._triggerTaskRemoved(taskId);
                                                   },
                                                   customErrorHandler);
        });

        panel.on('cancelDelete', function()
        {
            panel.hideDeleteDialog();
            self._refreshSelectedTab();
        });
    },

    handleDeleteException: function(taskId, panel, errorThrown)
    {
        var message = null;
        if (errorThrown.exception === "com.actimind.actitime.task.NoPermissionToModifyTTOfSpecifiedUsers")
        {
            message = messageResource.getMessage("hierarchy_entity.edit_popup.cannot_delete_due_to_unavailable_modify_time",
                                                [messageResource.getMessage("common.task")])
        }
        else if (errorThrown.exception === "com.actimind.actitime.task.TaskHasLockedTTException")
        {
            message = messageResource.getMessage("task.edit_popup.cannot_delete_due_to_lock_time")
        }
        else if (errorThrown.exception === "java.lang.IllegalArgumentException")
        {
            message = messageResource.getMessage("task.edit_popup.footer_error.invalid_fields_sent_to_server")
        }
        
        if (message !== null)
        {           
            alert(message);
            panel.header.actions.disableDeleteButton(message);
            panel.hideDeleteDialog();
            return true;
        }

        return false;
    },

    _subscribeForEditPanelEvents: function(panel)
    {
        var self = this;
        this._subscribeForPanelEvents(panel);
        var reportedTime = panel.tabs[TaskPanel.Tabs.REPORTED_TAB];


        // --name control
        panel.header.taskNameInput.on('change', function(e, name)
        {
            panel.header.taskNameInput.setLoading(true);
            self.editTaskServiceHelper.updateTask(EditTaskServiceHelper.Methods.NAME, [panel.taskId, name],
                                                  function()
                                                  {
                                                      panel.header.taskNameInput.setLoading(false);
                                                      TaskStore.add(_.extend(_.clone(TaskStore.get(self.state.getTaskId())), {name: name}));
                                                      panel.header.taskNameInput.resetName(name);
                                                      self.panel.updateDeleteDialog(name, self.state, true);
                                                      self._triggerTaskChanged(panel.taskId);
                                                  },
                                                  function(jqXHR, textStatus, errorThrown)
                                                  {
                                                      panel.header.taskNameInput.setLoading(false);
                                                      if (errorThrown.exception === "com.actimind.actitime.task.TaskExistsException")
                                                      {
                                                          self.panel.header.taskNameInput.error(messageResource.getMessage("task.edit_popup.duplicate"));
                                                          return true;
                                                      }
                                                      return false;
                                                  });
        });

        var details = panel.tabs[TaskPanel.Tabs.DETAILS_TAB];


        //--status control
        var makeStatusChangeCall = function(completed, forceReopenArchivedProject)
        {
            details.taskStatusButton.setLoading(true);
            self.editTaskServiceHelper.updateTask(EditTaskServiceHelper.Methods.STATUS, [panel.taskId, completed, forceReopenArchivedProject],
                                                  function()
                                                  {
                                                      var task = _.clone(TaskStore.get(self.state.getTaskId()));
                                                      TaskStore.add(_.extend(task, {completed: completed}));
                                                      details.taskStatusButton.setLoading(false);
                                                      details.taskStatusButton.setCompleted(completed);
                                                      var project = _.clone(ProjectStore.get(task.projectId));
                                                      var customer = _.clone(CustomerStore.get(project.customerId));

                                                      Toasts.addToastWithText(messageResource.getMessage("status.task.toast", [task.name]));

                                                      self._triggerTaskChanged(panel.taskId);
                                                      if (forceReopenArchivedProject)
                                                      {
                                                          project.archived = false;
                                                          customer.archived = false;
                                                          ProjectStore.add(project);
                                                          CustomerStore.add(customer);
                                                          self._triggerCPChange();
                                                      }
                                                  },
                                                  function(jqXHR, textStatus, errorThrown)
                                                  {
                                                      if (errorThrown.exception === "com.actimind.actitime.task.ProjectIsArchivedException")
                                                      {
                                                          if (confirm(messageResource.getMessage('task.error.reopen.project_archived')))
                                                          {
                                                              makeStatusChangeCall(completed, true);
                                                          } else {
                                                              details.taskStatusButton.setLoading(false);
                                                              details.taskStatusButton.setCompleted(TaskStore.get(self.state.getTaskId())['completed']);
                                                          }
                                                          return true;
                                                      }
                                                      details.taskStatusButton.setLoading(false);
                                                      details.taskStatusButton.setCompleted(TaskStore.get(self.state.getTaskId())['completed']);
                                                      if (errorThrown.exception === "com.actimind.actitime.task.TaskCannotBeModifiedException")
                                                      {
                                                          alert(messageResource.getMessage("task.edit_popup.no_rights_to_reopen_project"));
                                                          return true;
                                                      }
                                                      return false;
                                                  });
        };

        details.taskStatusButton.on('change',
                                    function(e, completed)
                                    {
                                        makeStatusChangeCall(completed, false);
                                    });


        //-- deadline control
        details.deadlineEditor.on('change', function(e, date)
        {
            details.deadlineEditor.setLoading(true);
            self.editTaskServiceHelper.updateTask(EditTaskServiceHelper.Methods.DEADLINE, [panel.taskId, date],
                                                  function()
                                                  {
                                                      TaskStore.add(_.extend(_.clone(TaskStore.get(self.state.getTaskId())), {deadline: date}));
                                                      details.deadlineEditor.setLoading(false);
                                                      details.deadlineEditor.setDeadline(date);
                                                      self._triggerTaskChanged(panel.taskId);
                                                  },
                                                  function()
                                                  {
                                                      details.deadlineEditor.setLoading(false);
                                                      details.deadlineEditor.setDeadline(TaskStore.get(self.state.getTaskId())['deadline']);
                                                      return false;
                                                  });
        });

        // -- type of Work control
        details.typeOfWorkEditor.on('change', function(e, typeId)
        {
            details.typeOfWorkEditor.setLoading(true);
            var handleBillingTypesError = function(result)
            {
                StoreUtils.replaceBillingTypesFromRPC({billingTypes: result.billingTypes.list});
                details.typeOfWorkEditor.getMenu().updateTypesOfWork();
            };
            self.editTaskServiceHelper.updateTask(EditTaskServiceHelper.Methods.BILLING_TYPE, [panel.taskId, typeId],
                                                  function()
                                                  {
                                                      self.state.task['billingTypeId'] = typeId;
                                                      details.typeOfWorkEditor.setLoading(false);
                                                      details.typeOfWorkEditor.setTypeOfWorkId(typeId);
                                                  },
                                                  function(result)
                                                  {
                                                      details.typeOfWorkEditor.setLoading(false);
                                                      details.typeOfWorkEditor.setTypeOfWorkId(self.state.task['billingTypeId']);
                                                      if (result && result.resultCode ===  EditTaskServiceHelper.ResultCodes.billingTypeNotExists)
                                                      {
                                                          alert( messageResource.getMessage("task.edit_popup.billing_type_not_exists") );
                                                          handleBillingTypesError(result);
                                                          return true;
                                                      }
                                                      if (result && result.resultCode ===  EditTaskServiceHelper.ResultCodes.billingTypeIsArchived)
                                                      {
                                                          alert( messageResource.getMessage("task.edit_popup.billing_type_was_archived") );
                                                          handleBillingTypesError(result);
                                                          return true;
                                                      }
                                                      if (result && result.resultCode === EditTaskServiceHelper.ResultCodes.billingTypeFeatureOff)
                                                      {
                                                          alert( messageResource.getMessage("task.error.billing_types_off") );
                                                          self.state.setBtyEnabled(false);
                                                          self.panel.setLoading(true);
                                                          self.panel.header.actions.hideMenu();
                                                          self.loadPanel();
                                                          return true;
                                                      }
                                                      return false;
                                                  });
        });

        // -- estimated time control

        details.estimatedTimeEditor.on('change', function(e, time)
        {
            details.estimatedTimeEditor.setLoading(true);
            self.editTaskServiceHelper.updateTask(EditTaskServiceHelper.Methods.ESTIMATED_TIME, [panel.taskId, time],
                                                  function()
                                                  {
                                                      TaskStore.add(_.extend(_.clone(TaskStore.get(self.state.getTaskId())), {estimatedTime: time}));
                                                      details.estimatedTimeEditor.setLoading(false);
                                                      details.update();
                                                      reportedTime.updateEstimated(time);
                                                      self._triggerTaskChanged(panel.taskId);
                                                  },
                                                  function(result)
                                                  {
                                                      details.estimatedTimeEditor.setLoading(false);
                                                      details.estimatedTimeEditor.setTime(TaskStore.get(self.state.getTaskId())['estimatedTime']);
                                                      if (result && result.resultCode === EditTaskServiceHelper.ResultCodes.estimateFeatureOff)
                                                      {
                                                          alert( messageResource.getMessage("task.list.error.estimate_off") );
                                                          window.location.reload();
                                                          return true;
                                                      }
                                                      return false;
                                                  });
        });

        // -- description control
        details.description.on('change', function(e, description)
        {
            details.description.setLoading(true);
            self.editTaskServiceHelper.updateTask(EditTaskServiceHelper.Methods.DESCRIPTION, [panel.taskId, description],
                                                  function()
                                                  {
                                                      details.description.setLoading(false);
                                                      self.state.task['description'] = description;
                                                      details.setDescription(description);
                                                  },
                                                  function()
                                                  {
                                                      details.description.setLoading(false);
                                                      details.setDescription(self.state.task['description']);
                                                      return false;
                                                  });
        });

        // -- move actions
        panel.header.actions.menu.on('move', function()
        {
            panel.showMoveToDialog();
        });

        panel.moveToDialog.on('move', function(e, data)
        {
            panel.moveToDialog.setLoading(true);
            panel.header.actions.hideMenu();

            self.editTaskServiceHelper.updateTask(EditTaskServiceHelper.Methods.MOVE_TASK,
                              [panel.taskId, data.project.id],
                              function()
                              {
                                  var task = TaskStore.get(self.state.getTaskId());
                                  TaskStore.add(_.extend(_.clone(task), {projectId: data.project.id, customerId: data.customer.id}));

                                  var projectName = ProjectStore.get(data.project.id).name;
                                  Toasts.addToastWithText(messageResource.getMessage("move.task.toast", [task.name, projectName]));

                                  panel.header.update();
                                  panel.moveToDialog.setLoading(false);
                                  self.panel.setLoading(true);
                                  self.panel.header.actions.hideMenu();
                                  self.loadPanel();

                                  $(self).trigger("taskMoved", {taskId: task.id, projectId: data.project.id});
                              },
                              function(jqXHR, textStatus, errorThrown)
                              {
                                  panel.moveToDialog.setLoading(false);
                                  if (errorThrown.exception === "com.actimind.actitime.project.NoSuchProjectException")
                                  {
                                      alert( messageResource.getMessage("task.edit_popup.project_not_exists") );
                                      $(self).trigger("projectDeleted", data.project.id);
                                      panel.moveToDialog.updateSelector();
                                      return true;
                                  }
                                  if (errorThrown.exception === "com.actimind.actitime.task.TaskExistsException")
                                  {
                                      alert(messageResource.getMessage("task.error.task_with_same_name_exists"));
                                      return true;
                                  }
                                  if (compareExceptions(errorThrown, "com.actimind.actitime.task.ActiveTasksProjectIsArchivedException"))
                                  {
                                      alert(messageResource.getMessage("task.error.project_archived"));
                                      return true;
                                  }
                                  if (errorThrown.exception === "com.actimind.actitime.task.ProjectIsNotAvailableException")
                                  {
                                      alert(messageResource.getMessage("task.edit_popup.project_not_available"));
                                      $(self).trigger("projectDeleted", data.project.id);
                                      panel.moveToDialog.updateSelector();
                                      return true;
                                  }
                                  return false;
                              });
        });

        panel.moveToDialog.on('cancel', function()
        {
            panel.hideMoveToDialog();
            self._refreshSelectedTab();
        });
    },

    _updateLoadedTimeComments: function(result)
    {
        var limit = 5;
        var reportedTime = this.panel.tabs[TaskPanel.Tabs.REPORTED_TAB];

        this.state.setAllTimeCommentsLoaded(result.reportedTimeComments.list.length < limit);
        this.state.setCompleteDetailsAccess(result.completeDataAccess);
        this.state.setReportedTimeComments(result.reportedTimeComments.list);
        reportedTime.setTimeAndCommentsLoading(false);
        reportedTime.updateTimeRows(result.reportedTimeComments.list);
        this.panel.suppressScrollCheck = false;
        this.panel.refreshScroll();
    },

    _appendLoadedTimeComments: function(result)
    {
        var limit = 5;
        var reportedTime = this.panel.tabs[TaskPanel.Tabs.REPORTED_TAB];

        this.state.setAllTimeCommentsLoaded(result.reportedTimeComments.list.length < limit);
        this.state.setCompleteDetailsAccess(result.completeDataAccess);
        this.state.addReportedTimeComments(result.reportedTimeComments.list);
        reportedTime.setTimeAndCommentsLoading(false);
        reportedTime.addTimeRows(result.reportedTimeComments.list);
        this.panel.suppressScrollCheck = false;
        this.panel.refreshScroll();
    },

    _refreshSelectedTab: function()
    {
        var self = this;
        var panel = this.panel;
        var reportedTime = panel.tabs[TaskPanel.Tabs.REPORTED_TAB];

        if (panel.selectedTab === TaskPanel.Tabs.REPORTED_TAB)
        {
            reportedTime.clearTimeRows();
            reportedTime.setTimeAndCommentsLoading(true);
            this.editTaskServiceHelper.makeRequest(EditTaskServiceHelper.Methods.TIME_COMMENTS, [panel.taskId, this.state.settings['sortBy'],
                                                                                                 this.state.settings['sortOrder'], 0, 1],
                                                   function(result)
                                                   {
                                                       self._updateLoadedTimeComments(result);
                                                       panel.refreshScroll();
                                                   });
        }
    },

    _triggerCPChange: function()
    {
        $(this).trigger("cpChanged");
    },

    _triggerTaskRemoved: function(taskId)
    {
        $(this).trigger("taskDeleted", taskId);
    },

    _triggerTaskChanged: function(taskId)
    {
        $(this).trigger("taskChanged", taskId);
    }
};

TaskManagement.TaskPanelController.State = function() {
    this.task = {id: -1, billingTypeId: -1, description: null};
    this.executorsSummary = {total: 0, records: []};
    this.reportedTimeComments = [];
    this.recordCount = 0;
    this.completeDetailsAccess = false;
    this.settings = null;
    this.allTimeCommentsLoaded = false;
    this.btyEnabled = false;

    this.blockedDueToLockedTime = false;
    this.blockedDueToUnavailableToModifyTimeTrack = false;
    this.taskHasOtherUsersTimeTrack = false;
    this.taskHasManagerTimeTrack = false;
    this.taskHasOtherUsersApprovedTimeTrack = false;
    this.taskHasManagerApprovedTimeTrack = false;
    this.hasManagerApprover = false;
    this.canManageAllUsersWithoutAutoApproved = false;
};

TaskManagement.TaskPanelController.State.prototype = {
    setExecutorsSummary: function(total, records)
    {
        this.executorsSummary['total'] = total;
        this.executorsSummary['records'] = records;
    },

    setBtyEnabled: function(btyEnabled)
    {
        this.btyEnabled = btyEnabled;
    },

    setReportedTimeComments: function(comments)
    {
        this.reportedTimeComments = comments;
    },

    addReportedTimeComments: function(comments)
    {
        this.reportedTimeComments = this.reportedTimeComments.concat(comments);
    },

    setRecordCount: function(count)
    {
        this.recordCount = count;
    },

    setSettings: function(settings)
    {
        this.settings = settings;
    },

    setAllTimeCommentsLoaded: function(loaded)
    {
        this.allTimeCommentsLoaded = loaded;
    },

    setCompleteDetailsAccess: function(completeDetailsAccess)
    {
        this.completeDetailsAccess = completeDetailsAccess;
    },

    setTask: function(task)
    {
        this.task['id'] = task['id'];
        this.task['billingTypeId'] = task['billingTypeId'];
        this.task['description'] = task['description'];
        this.task['billingTypeInfo'] = task['billingType'];
    },

    getTaskId: function()
    {
        return this.task['id'];
    }

};