TaskManagement.TasksTable.GroupOperationsMenu = function($containerEl)
{
    TemplateView.call(this, $containerEl);
    this.isShown = false;
};

TaskManagement.TasksTable.GroupOperationsMenu.prototype = _.create(TemplateView.prototype, {
    html: "<div class='groupOperationMenu'>" +
          "  <div class='innerWrapper'>" +
          "    <div class='description'></div>" +
          "    <div class='changeStatusButton'>" +
          "      <span>Change Status</span>" +
          "    </div>" +
          "    <div class='separator'/>" +
          "    <div class='deleteButton'>" +
          "      <span>Delete</span>" +
          "    </div>" +
          "    <div class='closeButton'>" +
          "      <span>clear selection</span>" +
          "    </div>" +
          "  <div>" +
          "</div>",

    afterRender: function()
    {
        this.$description = this.$el.find(".description");
        this.$statusButton = this.$el.find(".changeStatusButton");
        this.$el.find(".deleteButton").click(function() { $(document).trigger("taskManagement_deleteTasks")});
        this.$el.find(".closeButton").click(function() { $(document).trigger("taskManagement_closeGroupOperationMenu") });

        var self = this;
        this.$statusButton.click(function()
                                 {
                                     var taskStatusMenu = TaskStatusMenu.getInstance();

                                     taskStatusMenu.completed = null;

                                     taskStatusMenu.showAt(self.$statusButton, {offset: "0px 1px"}, function(isCompleted)
                                     {
                                         // close without choosing anything
                                         if (_.isUndefined(isCompleted) || isCompleted == null)
                                             return;

                                         $(document).trigger("taskManagement_groupingStatusChange", isCompleted);
                                     });

                                     taskStatusMenu.resetSelection();
                                 });
    },


    fadeIn: function()
    {
        if (this.checkPermissions()) {
            if (!this.isShown)
            {
                this.isShown = true;
                this.update();
                this.$el.atAnimation().showAnimated("anim-show-slide-up");
            }
        }
    },

    checkPermissions: function()
    {
        var canEdit = TaskManagement.Store.canManageTasks();
        if (!canEdit && this.isShown)
            this.fadeOut();

        return canEdit;
    },

    update: function()
    {
        if (!this.checkPermissions())
            return;

        var taskSelection = TaskManagement.Store.getTaskSelection();
        var countSelectedTasks = _.size(taskSelection.selectedTasks);
        var text;
        if (taskSelection.isAllSelected)
        {
            var filterStatus = TaskManagement.Store.getTaskStatusFilter();
            switch (filterStatus) {
                case TaskStatusFilter.ALL:
                    text = messageResource.getMessage("task.grouping_menu.all_selected");
                    break;
                case TaskStatusFilter.OPEN:
                    text = messageResource.getMessage("task.grouping_menu.all_selected.open");
                    break;
                case TaskStatusFilter.COMPLETED:
                    text = messageResource.getMessage("task.grouping_menu.all_selected.closed");
                    break;
                default:
                    text = "";
                    break;
            }
        }
        else if (countSelectedTasks == 1)
            text = messageResource.getMessage("task.grouping_menu.selected.singular");
        else
            text = messageResource.getMessage("task.grouping_menu.selected.plural", [countSelectedTasks]);

        this.$description.text(text);
    },

    fadeOut: function()
    {
        if (this.isShown)
        {
            this.isShown = false;
            this.$el.atAnimation().hideAnimated("anim-hide-slide-up");
        }
    }
});