TaskManagement.TasksTable.TaskRow = function($containerEl, taskId, tokensHighlighter)
{
    TemplateView.call(this, $containerEl);
    this.taskId = taskId;
    this.tokensHighlighter = tokensHighlighter;
};

TaskManagement.TasksTable.TaskRow.prototype = _.create(TemplateView.prototype, {
    html: "<tr class='taskRow'>" +
          "  <td class='selection'></td>" +
          "  <td class='names'>" +
          "    <div class='taskRowCellWrapper'>" +
          "      <div class='subtitle ellipsis'></div>" +
          "      <div class='title ellipsis'></div>" +
          "    </div>" +
          "  </td>" +
          "  <td class='status'>" +
          "    <div class='taskRowCellWrapper'></div>" +
          "  </td>" +
          "  <td class='estimate'>" +
          "    <div class='taskRowCellWrapper'></div>" +
          "  </td>" +
          "  <td class='deadline'>" +
          "    <div class='taskRowCellWrapper'></div>" +
          "  </td>" +
          "  <td class='lastTrackingDate'>" +
          "    <div class='taskRowCellWrapper'>" +
          "      <span class='value'></span>" +
          "      <span class='na'>N/A</span>" +
          "    </div>" +
          "  </td>" +
          "</tr>",

    templateKey: "TaskManagement.TasksTable.TaskRow",

    afterRender: function()
    {
        var self = this;

        this.$selection = this.find(".selection");
        this.$namesCell = this.find("td.names");
        this.$subtitle = this.find("td.names .subtitle");
        this.$title = this.find("td.names .title");
        this.$lastTrackingDate = this.find("td.lastTrackingDate .value");
        this.$deadlineCell = this.find("td.deadline .taskRowCellWrapper");
        this.$estimateColumn = this.find("td.estimate .taskRowCellWrapper");
        this.$statusButton = this.find("td.status .taskRowCellWrapper");

        this.$namesCell.click(function(e)
                              {
                                  self._startEditing();
                                  $(document).trigger("taskManagement_tasksTable_taskRow_click", [e.originalEvent, self.taskId]);
                              });

        var canEdit = TaskManagement.Store.canManageCP() || TaskManagement.Store.canManageTasks() && !ProjectStore.get(TaskStore.get(this.taskId).projectId).archived;
        this.editable = canEdit;
        if (TaskManagement.Store.canManageTasks())
        {
            this.taskSelectionLabel = new TaskManagement.TaskSelectionLabel(this.$selection, canEdit).init();
            this.taskSelectionLabel.on("change", function(e, isSelected) { self.setClickSelection(isSelected); });
        }
        else
            this.$selection.hide();

        this.taskStatusButton = new TaskStatusButton(this.$statusButton, canEdit).init();
        this.taskStatusButton.on("change", function(e, completed)
        {
            self._startUpdating(true);
            self.completed = completed;
            $(document).trigger("taskManagement_tasksTable_taskRow_statusChange", {
                taskId: self.taskId,
                completed: completed
            })
        });

        if (EnabledFeatures.EST)
        {
            this.estimateEditor = new TimeEditor(this.$estimateColumn, canEdit).init();
            this.estimateEditor.on("change", function(e, time)
            {
                self._startUpdating(true);
                self.estimatedTime = time;
                $(document).trigger("taskManagement_tasksTable_taskRow_estimateChange", {
                    taskId: self.taskId,
                    estimatedTime: time
                });
            });
        }
        else
            this.find('td.estimate').hide();

        this.deadlineEditor = new TaskManagement.DeadlineEditor(this.$deadlineCell, canEdit).init();
        this.deadlineEditor.on("change", function(e, date)
        {
            self._startUpdating(true);
            self.deadline = date;
            $(document).trigger("taskManagement_tasksTable_taskRow_deadlineChange", {
                taskId: self.taskId,
                deadline: date
            });
        });
    },

    setClickSelection: function(isSelected)
    {
        $(document).trigger("taskManagement_taskTable_taskRow_selectionChange", {
            taskId: this.taskId,
            isSelected: isSelected
        });
    },

    setSelection: function(isSelected)
    {
        this.taskSelectionLabel.setSelection(isSelected);
    },

    update: function()
    {
        var task = TaskStore.get(this.taskId);

        this.completed = task.completed;
        this.estimatedTime = task.estimatedTime;
        this.deadline = task.deadline;

        this.$subtitle.html(this.getSubtitle(task));
        this.$subtitle.attr("title", this.getSubtitleTooltip(task));
        this.$title.html(this.tokensHighlighter ? this.tokensHighlighter.highlightTokens(task.name) : escapeHtmlSymbols(task.name));
        this.$title.attr("title", task.name);
        this.$lastTrackingDate.text(task.lastTrackingDate ? TaskManagement.TasksTable.TaskRow.dateFormat.formatDate(task.lastTrackingDate) : "");
        this.$el.toggleClass("noLastTrackingDate", !task.lastTrackingDate);

        if (this.taskSelectionLabel)
            this.taskSelectionLabel.update();
        this.taskStatusButton.setCompleted(this.completed);
        if (this.estimateEditor)
            this.estimateEditor.setTime(this.estimatedTime);
        this.deadlineEditor.setDeadline(this.deadline);
    },

    getSubtitle: function(task)
    {
        var singleCPSelection = TaskManagement.Store.getSingleCustomerProjectSelection();
        var subtitle = "";
        var customerEscapedName = escapeHtmlSymbols(CustomerStore.get(task.customerId).name);
        var projectEscapedName = escapeHtmlSymbols(ProjectStore.get(task.projectId).name);
        if (!singleCPSelection.customerId && !singleCPSelection.projectId)
            subtitle = customerEscapedName + this.getArrowImg() + projectEscapedName;
        else if (!singleCPSelection.projectId)
            subtitle = projectEscapedName;
        return subtitle;
    },

    getSubtitleTooltip: function(task)
    {
        var tooltip = "";
        var singleCPSelection = TaskManagement.Store.getSingleCustomerProjectSelection();
        if (!singleCPSelection.customerId && !singleCPSelection.projectId)
            tooltip = CustomerStore.get(task.customerId).name + " > " + ProjectStore.get(task.projectId).name;
        else if (!singleCPSelection.projectId)
            tooltip = ProjectStore.get(task.projectId).name;

        return tooltip;
    },

    getArrowImg: function()
    {
        return " <img src='" + atDefImgPath('taskManagement/arrow.png') + "'/> ";
    },

    setEditable: function(editable)
    {
        this.editable = editable;
        this.taskStatusButton.setEditable(editable);
        if (this.estimateEditor)
            this.estimateEditor.setEditable(editable);
        this.deadlineEditor.setEditable(editable);
        if (this.taskSelectionLabel)
            this.taskSelectionLabel.setEditable(editable);
    },

    // TODO: rename, too abstract name, may be startDataUpdating or startEntityUpdating
    _startUpdating: function(disable)
    {
        if (this.updating)
            return;

        this.updating = true;
        if (disable)
        {
            this.$el.addClass("disabled");
            this.taskStatusButton.setDisabled(true);
            if (this.estimateEditor)
                this.estimateEditor.setDisabled(true);
            this.deadlineEditor.setDisabled(true);
            this.taskSelectionLabel.setDisabled(true);
        }

        var self = this;
        var handler = function(e, taskId)
        {
            if (self.taskId === taskId)
            {
                self.updating = false;
                if (disable)
                {
                    self.$el.removeClass("disabled");
                    self.taskStatusButton.setDisabled(false);
                    if (self.estimateEditor)
                        self.estimateEditor.setDisabled(false);
                    self.deadlineEditor.setDisabled(false);
                    self.taskSelectionLabel.setDisabled(false);
                }
                self.update();
                $(document).off(e, handler);
            }
        };
        $(document).on("taskManagement_taskUpdated", handler);
    },

    _startEditing: function()
    {
        if (this.inEdit)
            return;

        this.$el.addClass("inEdit");
        this.inEdit = true;
        var self = this;
        var editedHandler = function(e, taskId)
        {
            if (self.taskId === taskId)
            {
                self.inEdit = false;
                self.$el.removeClass("inEdit");
                self.update();
                $(document).off(e, editedHandler);
                $(document).off("taskManagement_task_updated_in_panel", updateHandler);
            }
        };
        var updateHandler = function(e, taskId)
        {
            if (self.taskId === taskId)
            {
                self.update();
            }
        };
        $(document).on("taskManagement_task_updated_in_panel", updateHandler);
        $(document).on("taskManagement_task_edited", editedHandler);
    },

    fadeOut: function()
    {
        this.$el.atAnimation().fadeOut();
        this._hidden = true;
    },

    isHidden: function()
    {
        return this._hidden === true;
    }
});

TaskManagement.TasksTable.TaskRow.dateFormat = new DateFormat( SystemSettings.dateFormat );


/**
 * Class for created Tasks For TM interface
 * This class not used in sprint_40
 */

TaskManagement.TasksTable.CreatedTaskRow = function($containerEl, taskId, tokensHighlighter)
{
    TaskManagement.TasksTable.TaskRow.call(this, $containerEl, taskId, tokensHighlighter);
};

TaskManagement.TasksTable.CreatedTaskRow.prototype = _.create(TaskManagement.TasksTable.TaskRow.prototype, {

    appendToContainer: function()
    {
        this.$el.prependTo(this.$containerEl);
    },

    getSubtitle: function(task)
    {
        this.$namesCell.removeClass('withoutSubtitle');
        var singleCPSelection = TaskManagement.Store.getSingleCustomerProjectSelection();
        var customerEscapedName = escapeHtmlSymbols(CustomerStore.get(task.customerId).name);
        var projectEscapedName = escapeHtmlSymbols(ProjectStore.get(task.projectId).name);
        if (singleCPSelection.customerId === task.customerId && singleCPSelection.projectId === task.projectId)
        {
            this.$namesCell.addClass('withoutSubtitle');
            return "";
        }
        else if (!singleCPSelection.projectId && singleCPSelection.customerId === task.customerId)
            return projectEscapedName;
        return customerEscapedName + this.getArrowImg() + projectEscapedName;
    }
});