TaskManagement.TasksTable = function($containerEl)
{
    WindowSizeDependedView.call(this, $containerEl);
};

TaskManagement.TasksTable.prototype = _.create(WindowSizeDependedView.prototype, {
    html: "<div class='tasksTable'>" +
          "  <div class='header'>" +
          "    <div class='navigationLinks'>" +
          "      <span class='nonClickable'><%-tasksOf%></span>" +
          "      <a class='allCustomers'><%-allCustomers%></a>" +
          "      <span class='separator'>&nbsp;</span>" +
          "      <a class='customer ellipsis'></a>" +
          "    </div>" +
          "    <div class='title ellipsis'></div>" +
          "    <div class='addNewTaskButtonContainer'></div>" +
          "    <div class='searchFieldContainer'></div>" +
          "  </div>" +
          "  <table class='headerRowTable' cellpadding='0' cellspacing='0'>" +
          "    <tr>" +
          "      <td class='selection'></td>" +
          "      <td class='names'>" +
          "        <div class='headerCellWrapper'>" +
          "          <div class='tipPlaceholderContainer'><span class='tipPlaceholder'></span></div>" +
          "          <span class='customerNameHeader'></span>" +
          "          <span class='shortSeparator customerNameSeparator'>&nbsp;</span>" +
          "          <span class='projectNameHeader'></span>" +
          "          <span class='shortSeparator projectNameSeparator'>&nbsp;</span>" +
          "          <span class='taskNameHeader'></span>" +
          "        </div>" +
          "      </td>" +
          "      <td class='status'>" +
          "        <div class='headerCellWrapper'>" +
          "          <span class='statusHeader'></span>" +
          "          <span class='statusFilterButtonContainer'></span>" +
          "        </div>" +
          "      </td>" +
          "      <td class='estimate'>" +
          "        <div class='headerCellWrapper'><span class='estimateHeader'></span></div>" +
          "      </td>" +
          "      <td class='deadline'>" +
          "        <div class='headerCellWrapper'><span class='deadlineHeader'></span></div>" +
          "      </td>" +
          "      <td class='lastTrackingDate'>" +
          "        <div class='headerCellWrapper'><span class='lastTrackingDateHeader'></span></div>" +
          "      </td>" +
          "    </tr>" +
          "  </table>" +
          "  <div class='scrollableContainer'>" +
          "    <div class='taskRowsTableContainer'>" +
          "      <table class='createdTasksRowsTable' cellpadding='0' cellspacing='0'></table>" +
          "      <table class='taskRowsTable' cellpadding='0' cellspacing='0'></table>" +
          "      <div class='loadingIndicatorContainer'></div>" +
          "      <div class='tasksNotFoundMessage'>" +
          "        <p class='firstSentence'><%-notFoundFirstSentence%></p>" +
          "        <p class='secondSentence'><%-notFoundSecondSentence%> <a class='allTasksLink' href='#'><%-notFoundSecondSentenceAllTasks%></a></p>" +
          "      </div>" +
          "    </div>" +
          "  </div>" +
          "  <div class='groupOperationsContainer'></div>" +
          "</div>",

    updateOnInit: true,

    afterRender: function()
    {
        var self = this;
        self.taskRows = {};

        if (TaskManagement.Store.canManageTasks())
            new TaskManagement.TasksTable.AddNewTaskButton(this.find(".addNewTaskButtonContainer")).init();
        new TaskManagement.TasksTable.TasksSearchField(this.find(".searchFieldContainer")).init();

        this.customerNameHeader = new TaskManagement.SortableTableHeader(this.find(".headerRowTable .customerNameHeader"), false, true, "common.Customer").init();
        this.projectNameHeader = new TaskManagement.SortableTableHeader(this.find(".headerRowTable .projectNameHeader"), false, true, "common.Project").init();
        this.taskNameHeader = new TaskManagement.SortableTableHeader(this.find(".headerRowTable .taskNameHeader"), false, true, "common.Task").init();
        this.statusHeader = new TaskManagement.SortableTableHeader(this.find(".headerRowTable .statusHeader"), false, true, "task.list.status").init();
        if (EnabledFeatures.EST)
            this.estimateHeader = new TaskManagement.SortableTableHeader(this.find(".headerRowTable .estimateHeader"), false, true, "task.list.estimate").init();
        else
            this.find('td.estimate').hide();
        this.deadlineHeader = new TaskManagement.SortableTableHeader(this.find(".headerRowTable .deadlineHeader"), false, true, "task.list.deadline").init();
        this.lastTrackingDateHeader = new TaskManagement.SortableTableHeader(this.find(".headerRowTable .lastTrackingDateHeader"), false, true, "task.list.last_tracking_date").init();
        this.groupOperationMenu = new TaskManagement.TasksTable.GroupOperationsMenu(this.find(".groupOperationsContainer")).init();

        if (TaskManagement.Store.canManageTasks())
            this.allTaskSelectionLabel = new TaskManagement.TaskSelectionLabel(this.find(".selection"), true).init();
        else
        {
            this.find(".names .headerCellWrapper").css("border-left", "none");
            this.find(".selection").hide();
        }
        this.sortingHeaders = {};
        this.sortingHeaders[TaskSortBy.CUSTOMER] = this.customerNameHeader;
        this.sortingHeaders[TaskSortBy.PROJECT] = this.projectNameHeader;
        this.sortingHeaders[TaskSortBy.TASK] = this.taskNameHeader;
        this.sortingHeaders[TaskSortBy.STATUS] = this.statusHeader;
        if (this.estimateHeader)
            this.sortingHeaders[TaskSortBy.ESTIMATE] = this.estimateHeader;
        this.sortingHeaders[TaskSortBy.DEADLINE] = this.deadlineHeader;
        this.sortingHeaders[TaskSortBy.LAST_TRACKING_DATE] = this.lastTrackingDateHeader;

        _.each(_.keys(this.sortingHeaders), function(key)
        {
            var sortingHeader = this.sortingHeaders[key];
            sortingHeader.on("click", function()
            {
                self._setActiveSortingHeader(key, sortingHeader.isAsc());
                $(document).trigger("taskManagement_tasksTable_sortingHeaderClick", {
                    taskSortBy: key,
                    asc: sortingHeader.isAsc()
                });
            });
        }, this);

        this._refreshSortingHeaders();

        new TaskManagement.TasksTable.StatusFilterButton(this.find(".statusFilterButtonContainer"), TaskManagement.Store.getTaskStatusFilter()).init();

        this.$scrollableContainer = this.find(".scrollableContainer");
        this.$taskRowsTableContainer = this.$scrollableContainer.find(".taskRowsTableContainer");
        this.$taskRowsTable = this.$taskRowsTableContainer.find(".taskRowsTable");
        this.$createdTaskRowsTable = this.$taskRowsTableContainer.find(".createdTasksRowsTable");

        this.appendLoadingMoire = new LoadingMoire(this.find(".loadingIndicatorContainer"), {}, false).init();
        this.reloadLoadingMoire = new LoadingMoire(this.$taskRowsTableContainer).init();

        this.$title = this.find(".header > .title");
        this.$navigationAllCustomersLink = this.find(".navigationLinks .allCustomers");
        this.$navigationCustomerLink = this.find(".navigationLinks .customer");
        this.$navigationAllCustomersLink.click(function()
                                               {
                                                   $(document).trigger("taskManagement_tasksTable_allCustomersClick")
                                               });
        this.$navigationCustomerLink.click(function()
                                           {
                                               $(document).trigger("taskManagement_tasksTable_customerClick")
                                           });

        if (TaskManagement.Store.canManageTasks())
            this.allTaskSelectionLabel.on("change", function(e, isAllSelected)
            {
                $(document).trigger("taskManagement_taskTable_selectAllChange", isAllSelected);
            });

        this._updateNavigation();

        // TODO: ScrollView
        if (!isIE8())
        {
            this.verticalScroll = ScrollUtils.addVerticalScroll(this.$scrollableContainer, {
                probeType: 2
            });
            this.verticalScroll.on("scroll", function()
            {
                TaskManagement.DatePickerMenu.getInstance()._onScroll();
                TaskStatusMenu.getInstance()._scrollHandler();
                TypeOfWorkMenu.getInstance()._scrollHandler();
                CustomerStatusMenu.getInstance()._scrollHandler();
                self._checkIfScrolledToEnd();
            });
        }

        this.stringTokenizer = new StringTokenizer(StringTokenizer.PUNCTUATORS_FOR_SEARCH);

        this.$tasksNotFoundMessage = this.find(".tasksNotFoundMessage");
        this.$tasksNotFoundFirstSentence = this.$tasksNotFoundMessage.find(".firstSentence");
        this.$tasksNotFoundAllTasksLink = this.$tasksNotFoundMessage.find(".allTasksLink");
        this.$tasksNotFoundAllTasksLink.click(function()
                                              {
                                                  $(document).trigger("taskManagement_tasksTable_allTasksClick");
                                              });

        this.$el.toggleClass("incompleteTaskInformation", !TaskManagement.Store.canAccessCompleteTaskInformation());

        $(TaskManagement.Store).on("taskIdsReplace", function()
        {
            self._clearTaskRows();
            _.each(TaskManagement.Store.getCreatedTaskIds(), function(taskId)
            {
                self.taskRows[taskId] = new TaskManagement.TasksTable.TaskRow(self.$createdTaskRowsTable, taskId, this.tokensHighligter).init();
            });
            self._appendTaskRows();
            self._updateNavigation();
        });
        $(TaskManagement.Store).on("taskIdsAppend", function()
        {
            self._appendTaskRows();
        });
        $(TaskManagement.Store).on("createdTaskIdsAppend", function(e, taskIds)
        {
            self._appendCreatedTaskRows(taskIds);
        });
        $(TaskManagement.Store).on("singleCustomerProjectSelectionChange", function()
        {
            self._updateNavigation();
        });
        $(document).on("taskManagement_loadTasks_withReload_begin", function()
        {
            self._showLoadingMoire(self.reloadLoadingMoire);
        });
        $(document).on("taskManagement_loadTasks_withAppend_begin", function()
        {
            self._showLoadingMoire(self.appendLoadingMoire);
        });
        $(document).on("taskManagement_tasksTable_tasksSearchField_change", function(e, text)
        {
            var tokens = self.stringTokenizer.getTokens(text);
            self.tokensHighligter = text && (tokens.length > 0) ? new TokensHighlighter(tokens, self.stringTokenizer.getPunctuators()) : null;
        });

        $(document).on("taskManagement_tasksStatusChange", function(e, data)
        {
            var taskIds = data.taskIds;
            var isCompleted = data.isCompleted;

            for (var i = 0; i < taskIds.length; ++i)
            {
                if (_.isUndefined(self.taskRows[taskIds[i]]))
                    continue;

                self.taskRows[taskIds[i]].taskStatusButton._change(isCompleted);
                self.taskRows[taskIds[i]].taskStatusButton.update();
            }
        });

        $(document).on("taskManagement_removeTasks", function(e, taskIds)
        {
            if (_.isUndefined(taskIds))
                return;

            for (var i = 0; i < taskIds.length; ++i)
            {
                if (_.isUndefined(self.taskRows[taskIds[i]]))
                    continue;

                self.taskRows[taskIds[i]].fadeOut();
                self.taskRows[taskIds[i]].$el.remove();
                delete self.taskRows[taskIds[i]];
            }
            self._refreshScroll();
            self.$el.toggleClass("tasksNotFound", _.isEmpty(self.taskRows));
        });

        this._showLoadingMoire(this.reloadLoadingMoire);

        $(TaskManagement.Store).on("taskSelection_changed", function()
        {
            if (_.size(self.taskRows) <= 0)
            {
                self.groupOperationMenu.fadeOut();
                self.updateTaskSelectionView();
                return;
            }

            var isEmpty = TaskManagement.Store.isTaskSelectionEmpty();

            if (isEmpty)
                self.groupOperationMenu.fadeOut();
            else
                self.groupOperationMenu.fadeIn();

            self.updateTaskSelectionView();
            self.groupOperationMenu.update();
        });

        WindowSizeDependedView.prototype.afterRender.call(this);
    },

    messageKeys: function()
    {
        return {
            tasksOf: "task.list.tasks_of",
            allCustomers: "task.list.all_customers",
            notFoundFirstSentence: "task.list.tasks_not_found_first_sentence",
            notFoundSecondSentence: "task.list.tasks_not_found_second_sentence",
            notFoundSecondSentenceAllTasks: "task.list.tasks_not_found_second_sentence_all_tasks"
        }
    },

    updateTaskSelectionView: function()
    {
        if (TaskManagement.Store.canManageTasks())
        {
            var self = this;
            var taskSelection = TaskManagement.Store.getTaskSelection();
            this.$el.toggleClass("selectAll", taskSelection.isAllSelected);

            self.allTaskSelectionLabel.setSelection(taskSelection.isAllSelected);

            _.each(self.taskRows, function(row)
            {
                row.setSelection(false);
            });

            _.each(taskSelection.selectedTasks, function(value, key)
            {
                if (self.taskRows[key])
                    self.taskRows[key].setSelection(true);
            });
        }
    },

    adjustSize: function()
    {
        WindowSizeDependedView.prototype.adjustSize.call(this);
        this.$scrollableContainer.adjustHeightToWindowSize();
        this._refreshScroll();
    },

    _updateNavigation: function()
    {
        var singleCPSelection = TaskManagement.Store.getSingleCustomerProjectSelection();
        this.$navigationCustomerLink.text(singleCPSelection.customerId ? CustomerStore.get(singleCPSelection.customerId).name : "");
        this.$title.text(singleCPSelection.customerId ? (singleCPSelection.projectId ? ProjectStore.get(singleCPSelection.projectId).name : CustomerStore.get(singleCPSelection.customerId).name) :
                         messageResource.getMessage("task.list.all_customers"));
        this.$el.toggleClass("hasSelectedCustomer", !!singleCPSelection.customerId);
        this.$el.toggleClass("hasSelectedProject", !!singleCPSelection.projectId);
    },

    _clearTaskRows: function()
    {
        this.$taskRowsTable.empty();
        this.$createdTaskRowsTable.empty();
        this.taskRows = {};
        // TODO: Change to ScrollView
        if (!isIE8())
            this.verticalScroll.scrollTo(0, 0);
    },

    _appendTaskRows: function()
    {
        var taskIds = TaskManagement.Store.getTaskIds();
        this._updateTaskRowsInContainer(taskIds, this.$taskRowsTable, true);
        this._hideLoadingMoire();
        this.suppressScrollCheck = false;
        this._refreshScroll();
        PageOverlayTipsManager.getInstance().refreshDynamicTips();
    },

    _appendCreatedTaskRows: function(createdTaskIds)
    {
        this._updateTaskRowsInContainer(createdTaskIds, this.$createdTaskRowsTable, true);
        this._refreshScroll();
        PageOverlayTipsManager.getInstance().refreshDynamicTips();

    },

    _updateTaskRowsInContainer: function(taskIds, $container, updateExisting)
    {
        _.each(taskIds, function(taskId)
        {
            if (!this.taskRows[taskId])
                this.taskRows[taskId] = new TaskManagement.TasksTable.TaskRow($container, taskId, this.tokensHighligter).init();
            else if (updateExisting)
                this.taskRows[taskId].update();
        }, this);

        this.updateTaskSelectionView();

        this.$el.toggleClass("allTasksLoaded", TaskManagement.Store.isAllTasksLoaded());
        this.$el.toggleClass("tasksNotFound", taskIds.length === 0);
    },

    _refreshSortingHeaders: function()
    {
        this._setActiveSortingHeader(TaskManagement.Store.getTaskSortBy(), TaskManagement.Store.getTaskSortAsc());
    },

    _setActiveSortingHeader: function(taskSortBy, asc)
    {
        _.each(_.keys(this.sortingHeaders), function(key)
        {
            var sortingHeader = this.sortingHeaders[key];
            if (key == taskSortBy)
            {
                sortingHeader.setActive(true);
                sortingHeader.setAsc(asc);
            }
            else
            {
                sortingHeader.setActive(false);
            }
        }, this);
    },

    _refreshScroll: function()
    {
        ScrollUtils.refreshCustomScroll(this.verticalScroll);
        this._checkIfScrolledToEnd();
    },

    _checkIfScrolledToEnd: function()
    {
        // TODO: Change to ScrollView
        if (!isIE8() && !this.suppressScrollCheck && this.verticalScroll.y - this.verticalScroll.maxScrollY < 200)
        {
            $(document).trigger("taskManagement_tasksTable_scrollToEnd");
            this.suppressScrollCheck = true;
        }
    },

    _showLoadingMoire: function(moire)
    {
        this.currentLoadingMoire = moire;
        moire.show();
    },

    _hideLoadingMoire: function()
    {
        this.currentLoadingMoire.hide();
    }
});