function UserListPageOverlayTipsManager(viewerSettings)
{
    PageOverlayTipsManager.call(this);

    this.manageAccounts = viewerSettings.allowedToManageAccountSettings;
    this.manageBalances = viewerSettings.allowedToManageLeaveBalanceSettings;
    this.ptoEnabled = viewerSettings.enabledFeatures.PTO;
    this.sdbEnabled = viewerSettings.enabledFeatures.SDB;
    this.ttaEnabled = viewerSettings.enabledFeatures.TTA;
}

UserListPageOverlayTipsManager.prototype = _.create(PageOverlayTipsManager.prototype, {
    addBaseTips: function()
    {
        PageOverlayTipsManager.prototype.addBaseTips.apply(this);

        if (this.manageAccounts) this._addManageAccountsTips();
        if (this.manageBalances) this._addManageBalancesTips();
    },

    addTitleTips: function()
    {
        this.deferUntilEnabled(function()
                               {
                                   var titleTipMessageKey = this.manageAccounts ? "tips.user_list.manage_title" : "tips.user_list.view_title";
                                   if ($("#userListTabs").find(".tab").length > 0)
                                   {
                                       var tabAccountSelector = function() { return $("#tabAccountSettings.selected") };
                                       var tabPTOSelector = function() { return $("#tabPTOSettings.selected") };
                                       var tabSickDaysSelector = function() { return $("#tabSickDaysSettings.selected") };

                                       this.add(tabAccountSelector, {
                                           messageKey: titleTipMessageKey,
                                           direction: OverlayTips.DIRECTION_TOP_LEFT,
                                           offsetX: 10,
                                           outside: {
                                               direction: OverlayTips.DIRECTION_TOP_RIGHT,
                                               offsetY: 10
                                           }
                                       });

                                       var ptoTitleTipConfig = {
                                           messageKey: "tips.user_list.pto_title",
                                           direction: OverlayTips.DIRECTION_TOP_RIGHT,
                                           offsetY: 15
                                       };
                                       var sickDaysTitleTipConfig = {
                                           messageKey: "tips.user_list.sick_days_title",
                                           direction: OverlayTips.DIRECTION_TOP_RIGHT,
                                           offsetX: 10,
                                           offsetY: 5
                                       };

                                       if (this.ptoEnabled)
                                           this.add(tabPTOSelector, ptoTitleTipConfig);
                                       if (this.sdbEnabled)
                                           this.add(tabSickDaysSelector, sickDaysTitleTipConfig);
                                   }
                                   else
                                   {
                                       this.add($(".pagetitle span"), {
                                           messageKey: titleTipMessageKey,
                                           direction: OverlayTips.DIRECTION_TOP_LEFT,
                                           offsetX: 10,
                                           offsetY: -10,
                                           outside: {
                                               direction: OverlayTips.DIRECTION_TOP_RIGHT
                                           }
                                       });
                                   }
                               });
    },

    _addManageAccountsTips: function()
    {
        this.add(function() { return $(".showAccSettings .userListRow:not(.locked) .userNameCell").eqOrLast(0) }, {
            messageKey: "tips.user_list.user_profile",
            direction: OverlayTips.DIRECTION_BOTTOM_LEFT,
            offsetX: -20,
            offsetY: -20,
            outside: {
                direction: OverlayTips.DIRECTION_TOP_RIGHT,
                offsetX: -320,
                offsetY: 0
            }
        });

        this.add(function() { return $(".showAccSettings .userListRow .taskHierarchyManageCell").eqOrLast(3) }, {
            messageKey: "tips.user_list.user_access_level",
            direction: OverlayTips.DIRECTION_BOTTOM_LEFT,
            offsetX: -30,
            offsetY: 15
        });

        this.add(function() { return $(".showAccSettings .userListRow .taskHierarchyAssignmentsCell.enabled").eqOrLast(0) }, {
            messageKey: "tips.user_list.user_projects",
            direction: OverlayTips.DIRECTION_BOTTOM_RIGHT,
            offsetX: -115,
            offsetY: 20
        });

        this.add(function() { return $(".showAccSettings .userListRow").not(".disabled").children(".manageApproveCell.enabled").eqOrLast(0) }, {
            messageKey: this.ttaEnabled ? "tips.user_list.assign_to_approver" : "tips.user_list.assign_to_supervisor",
            direction: OverlayTips.DIRECTION_TOP_RIGHT,
            offsetX: 80,
            offsetY: 14
        });

        this.add(function() { return $(".showAccSettings .userListRow:not(.locked) .manageBalanceCell").eqOrLast(0) }, {
            messageKey: this.ptoEnabled && this.sdbEnabled ? "tips.user_list.user_pto_and_sdb_manage" :
                        (this.ptoEnabled ? "tips.user_list.user_pto_manage" : "tips.user_list.user_sdb_manage"),
            direction: OverlayTips.DIRECTION_TOP_RIGHT,
            offsetX: -20,
            outside: {
                direction: OverlayTips.DIRECTION_TOP_RIGHT,
                offsetY: 10
            }
        });
    },

    _addManageBalancesTips: function()
    {
        this.add(function() { return $(".showPTOSettings .userListRow .currentBalanceCell.ptoCell.on").eqOrLast(0) }, {
            messageKey: "tips.user_list.pto_balance",
            direction: OverlayTips.DIRECTION_BOTTOM_LEFT,
            offsetX: -40,
            offsetY: 20
        });

        this.add(function() { return $(".showPTOSettings .userListRow .plannedLeaveCell.ptoCell.on").eqOrLast(1) }, {
            messageKey: "tips.user_list.planned_leaves",
            direction: OverlayTips.DIRECTION_BOTTOM_RIGHT,
            offsetX: -65,
            offsetY: 15
        });

        this.add(function()
        {
            var element = $(".showPTOSettings .userListRow").children(".ptoRulesCell.accrualRule, .ptoRulesCell.defaultRule")
                    .not(".noBalance").not(".defaultRules.accrualRule").eqOrLast(0);
            return {
                element: element,
                config: {
                    messageKey: "tips.user_list.accrual",
                    direction: OverlayTips.DIRECTION_TOP_RIGHT,
                    offsetX: element.hasClass("defaultRule") ? -115 : -30,
                    offsetY: 15
                }
            }
        });


        this.add(function() { return $(".showPTOSettings .userListRow .ptoControlCell").eqOrLast(4) }, {
            messageKey: "tips.user_list.pto_toggle",
            direction: OverlayTips.DIRECTION_BOTTOM_RIGHT,
            offsetX: -30,
            offsetY: -5
        });

        this.add(function() { return $(".showSickDaysSettings .userListRow .currentBalanceCell.sickDaysCell.on").eqOrLast(0) }, {
            messageKey: "tips.user_list.sick_days_balance",
            direction: OverlayTips.DIRECTION_BOTTOM_LEFT,
            offsetX: -40,
            offsetY: 20
        });

        this.add(function() { return $(".showSickDaysSettings .userListRow .plannedLeaveCell.sickDaysCell.on").eqOrLast(1) }, {
            messageKey: "tips.user_list.planned_leaves",
            direction: OverlayTips.DIRECTION_BOTTOM_RIGHT,
            offsetX: -65,
            offsetY: 15
        });

        this.add(function()
        {
            var element = $(".showSickDaysSettings .userListRow").children(".sickDaysRulesCell.accrualRule, .sickDaysRulesCell.defaultRule")
                    .not(".noBalance").not(".defaultRules.accrualRule").eqOrLast(0);
            return {
                element: element,
                config: {
                    messageKey: "tips.user_list.accrual",
                    direction: OverlayTips.DIRECTION_TOP_RIGHT,
                    offsetX: element.hasClass("defaultRule") ? -115 : -30,
                    offsetY: 15
                }
            }
        });

        this.add(function() { return $(".showSickDaysSettings .userListRow .sickDaysControlCell").eqOrLast(4) }, {
            messageKey: "tips.user_list.sick_toggle",
            direction: OverlayTips.DIRECTION_BOTTOM_RIGHT,
            offsetX: -30,
            offsetY: -5
        });
    }
});