function renderTTAHistoryTable(renderTo, managerId, historyList) {
    function createEmptyCell() {
        return $("<td></td>");
    }

    function createDateCell(dateStr) {
        return createEmptyCell().append( dateStr ).addClass("dateCell");
    }

    function createStatusCell(statusValue, bySystem) {
        var statusCell = createEmptyCell().addClass("statusCell");
        if (statusValue == WeekApprovalStatus.READY)
            statusCell.append(messageResource.getMessage("submit_tt.tta_history.table.status_text.ready"));
        else if (statusValue == WeekApprovalStatus.APPROVED)
            statusCell.append(
                    bySystem ?
                    messageResource.getMessage("submit_tt.tta_history.table.status_text.auto_approved") :
                    messageResource.getMessage("submit_tt.tta_history.table.status_text.approved")
            ).addClass("approved");
        else if (statusValue == WeekApprovalStatus.REJECTED)
            statusCell.append(messageResource.getMessage("submit_tt.tta_history.table.status_text.rejected")).addClass("rejected");
        else
            throw new Error("Wrong history response format");

        return statusCell;
    }

    function createUserCell(user, auto) {
        var userCell = createEmptyCell().addClass("userCell");
        if (!auto)
            userCell.append(
                    managerId == user.id ?
                    messageResource.getMessage("submit_tt.tta_history.table.user_text.me") :
                    escapeHtmlSymbols(UserNameFormat.formatFML(user.firstName, user.lastName, user.middleName))
            );
        return userCell;
    }

    function createCommentCell(comment) {
        return createEmptyCell().append(escapeHtmlSymbols(comment)).addClass("commentCell");
    }

    var tbody = renderTo.find(".tableContentContainer table tbody");
    tbody.empty();
    if (!historyList || historyList.length == 0) {
        tbody.append('<tr><td colspan="4" class="emptyTable">' + messageResource.getMessage("submit_tt.tta_history.table.empty") + '</td></tr>');
    }

    var row = "";
    for (var i = 0, historyListLength = historyList.length; i < historyListLength; i++) {
        var entry = historyList[i],
            auto = entry.approverNameInfo == null;
        row = $("<tr/>")
                .append(createDateCell(entry.statusChangeDateInViewerTimezone))
                .append(createStatusCell(entry.status, auto))
                .append(createUserCell(entry.approverNameInfo, auto))
                .append(createCommentCell(entry.comment));
        if (i == historyListLength - 1)
            row.addClass("lastRow");
        tbody.append(row);
    }
};