/**
 * @author kostenko
 *
 * Requires:
 *  task_list_controller.js
 *
 * Markup: recent_tasks_control.jsp
 * Styles: add_tasks.css
 *
 * @name setMouseDownOutsideHandler (defined in at.js)
 *
 * @param id {string} id of the HTML element that contains the control
 * @param selectedUserId {number} selected user id
 * @param selectedUserName {string} display name of the selected user
 * @param isOwnTimeTrack {boolean} whether it is logged in user's own time-track page
 * @param alreadyAddedTaskIds {array} ids of task which are already added to TT
 * @param onTaskChosen {function(taskId)} that will be called when the user selects a task
 * @param spinnerPlaceholderId {string} id of the element in which to render spinner
 */
function initRecentTasksControl( id, selectedUserId, selectedUserName, isOwnTimeTrack,
                                 alreadyAddedTaskIds, onTaskChosen, spinnerPlaceholderId )
{
    var controlContainer = $("#" + id);
    var button = controlContainer.find(".addTasksButton");
    var desiredPortionSize = 30;
    var maxResultsPerPortion = 30;
    var taskListController = new TaskListController( controlContainer.find(".taskList"),
                                                     null, $('#' + spinnerPlaceholderId),
                                                     selectedUserId, selectedUserName, isOwnTimeTrack, true, false,
                                                     alreadyAddedTaskIds, desiredPortionSize, maxResultsPerPortion,
                                                     false, false, false,
                                                     null, null, onTaskSelect, null );

    function onButtonClick()
    {
        if( taskListController.isShown() )
        {
            hideRecentTasksControl();
            button.removeClass('active');
        }
        else
        {
            taskListController.makeInitialRequest( 0, 0 );
            KeyDownHandler.setOnKeyDownHandler( onKeyDown );
            button.addClass('active');
        }
    }

    function onClickOutside()
    {
        hideRecentTasksControl();
    }

    function onTaskSelect( taskId )
    {
        onTaskChosen( taskId );
        button.removeClass('active');
    }

    function onKeyDown( event )
    {
        event = event || window.event;
        if( event.keyCode == KEY_CODE_ESC )
        {
            hideRecentTasksControl();
        }
    }

    function hideRecentTasksControl()
    {
        if(taskListController.isShown())
        {
            button.removeClass('active');
            taskListController.hide();
            KeyDownHandler.restoreOnKeyDownHandler();
        }
    }

    button.click( onButtonClick );
    setMouseDownOutsideHandler( controlContainer, onClickOutside );
}