/**
 * @author kostenko
 *
 * Requires:
 *  task_list_controller.js
 *
 * Markup: task_search_control.jsp
 * Styles: add_tasks.css
 *
 * @name setMouseDownOutsideHandler (defined in at.js)
 *
 * @param id {string} id of the HTML element that contains the control
 * @param selectedUserId {number} selected user id
 * @param selectedUserName {string} display name of the selected user
 * @param isOwnTimeTrack {boolean} whether it is logged in user's own time-track page
 * @param alreadyAddedTaskIds {array} ids of task which are already added to TT
 * @param onTaskChosen {function(taskId)} that will be called when the user selects a task
 * @param spinnerPlaceholderId {string} id of the element in which to render spinner when doing an initial task list
 *                                      request (as opposed to the 'Show more' request)
 * @param isEnabled {boolean} whether the selector is enabled
 */
function initTaskSearchControl( id, selectedUserId, selectedUserName, isOwnTimeTrack,
                                alreadyAddedTaskIds, onTaskChosen, spinnerPlaceholderId, isEnabled )
{
    var controlContainer = $("#" + id);
    var searchField = controlContainer.find(".taskSearchField");

    // Select appropriate label depending on the task level name length
    var searchLabel = messageResource.getMessage("submit_tt.task_search.label_1");
    if( searchLabel.length > 22 )
    {
        searchLabel = messageResource.getMessage("submit_tt.task_search.label_2");
        if( searchLabel.length > 20 )
            searchLabel = messageResource.getMessage("submit_tt.task_search.label_3");
    }

    searchField.attr('placeholder', searchLabel).placeholder();

    if( isEnabled )
    {
        function onClickOutside()
        {
            taskListController.hide();
        }

        function onRequestProcessingComplete()
        {
            searchField.focus(); // if clicked 'show more', must return focus to the field
        }

        function onTaskSelect( taskId )
        {
            onTaskChosen( taskId );

            searchField.blur();
            taskListController.clearQuery();
        }

        var desiredPortionSize = 20;
        var maxResultsPerPortion = 30;
        var taskListController = new TaskListController( controlContainer.find( ".taskList" ),
                                                         searchField, $( '#' + spinnerPlaceholderId ),
                                                         selectedUserId, selectedUserName, isOwnTimeTrack, false, false,
                                                         alreadyAddedTaskIds, desiredPortionSize, maxResultsPerPortion,
                                                         false, true, true,
                                                         onRequestProcessingComplete, null, onTaskSelect, null, true );

        setMouseDownOutsideHandler( controlContainer, onClickOutside );
    }
    else
    {
        searchField.prop( 'disabled', true );
    }
}