function LockTTPager($containerEl, selectOptions)
{
    TemplateView.call(this, $containerEl);
    this.selectOptions = selectOptions;
}

LockTTPager.prototype = _.create(TemplateView.prototype, {
    html: "<table class='additionalLeftPadding tablePager'>" +
          "  <tr>" +
          "    <td>" +
          "      <%=resultsContain%>&nbsp;<span class='result-number'></span>&nbsp;<span class='endOfText'></span>" +
          "      <span class='recordsPerPageSelectorContainer'>" +
          "        <select>" +
          "          <% _.each(selectOptions, function(pageAmount) { %>" +
          "             <option value='<%=pageAmount%>'><%=pageAmount%></option>" +
          "          <% }) %>" +
          "        </select>&nbsp;<%=aPageText%>" +
          "      </span>" +
          "    </td>" +
          "    <td class='columnPagerSeparator'>" +
          "      <span class='pagerSeparator'></span>" +
          "    </td>" +
          "    <td class='pageLinksContainer'>" +
          "      <div class='prevNextLinks'>" +
          "        <nobr class='previousLink text'></nobr>" +
          "        <span>&nbsp;&nbsp;&nbsp;</span>" +
          "        <span class='pageLinks'></span>" +
          "        <span>&nbsp;&nbsp;</span>" +
          "        <nobr class='nextLink text'></nobr>" +
          "      </div>" +
          "    </td>" +
          "  </tr>" +
          "</table>",

    updateOnInit: false,

    afterRender: function()
    {
        this.$endOfText = this.find(".endOfText");
        this.$resultNumber = this.find(".result-number");
        this.$selectorContainer = this.find(".recordsPerPageSelectorContainer");
        this.$select = this.$selectorContainer.find("select");
        this.$previousLink = this.find(".previousLink");
        this.$nextLink = this.find(".nextLink");
        this.$pageLinks = this.find(".pageLinks");

        this.$select.on("change", function(event)
        {
            $(document).trigger("pager.records_per_page.change", event.target.value);
        });
    },

    update: function()
    {
        var pagerConfig = LockTTStore.getPagerConfig();
        if (_.isNull(pagerConfig) || pagerConfig.recordsNumber <= 0)
        {
            this.$el.hide();
            return;
        }
        var endOfTextKey = "";
        this.$resultNumber.text(pagerConfig.recordsNumber);
        if (pagerConfig.need)
        {
            var options = this.$select.find("option");
            var selected = options.find(":selected");
            if (selected)
            {
                selected.removeAttr("selected");
            }
            options.each(function()
                         {
                             if ($(this).prop("value") == pagerConfig.recordsPerPage)
                             {
                                 $(this).attr("selected", "selected");
                                 return false;
                             }
                         });
            this.$selectorContainer.show();
            endOfTextKey = "common.end_of_results_contain_x_users_shown_by";
        }
        else
        {
            endOfTextKey = pagerConfig.recordsNumber == 1 ? "common.user" : "common.users";
            this.$selectorContainer.hide();
        }
        this.$endOfText.text(messageResource.getMessage(endOfTextKey));

        if (pagerConfig.currentPagesCount > 1)
        {
            this._createPageLinks(pagerConfig);
        }
        this.$el.toggleClass("onlyOnePage", pagerConfig.currentPagesCount == 1);
        this.$el.show();
    },

    templateModel: function()
    {
        return {
            "selectOptions": this.selectOptions
        }
    },

    messageKeys: function()
    {
        return {
            "aPageText": "common.a_page",
            "resultsContain": "common.results_contain",
            "previous": "common.previous",
            "next": "common.next"
        }
    },
    
    _createPageLinks: function(pagerConfig)
    {
        this.$previousLink.empty();
        if (_.isNull(pagerConfig.previousPageFirstRecordNumber))
        {
            this.$previousLink.text(messageResource.getMessage("common.previous") + " --");
        }
        else
        {
            this.$previousLink.append(this._createPageLink(pagerConfig.previousPageFirstRecordNumber, messageResource.getMessage("common.previous")), " --");
        }

        var self = this;
        this.$pageLinks.empty();
        _.each(pagerConfig.pages, function(page)
        {
            var $nobr = $("<nobr class='text'></nobr>");
            var text = (page.firstRecordNumber + 1) + "-" + (page.lastRecordsNumber + 1);
            if (page.current)
            {
                $nobr.append(text + "&nbsp;&nbsp;&nbsp;");
            }
            else
            {
                $nobr.append(self._createPageLink(page.firstRecordNumber, text), "&nbsp;&nbsp;&nbsp;");
            }
            self.$pageLinks.append($nobr);
        });

        this.$nextLink.empty();
        if (_.isNull(pagerConfig.nextPageFirstRecordNumber))
        {
            this.$nextLink.text("-- " + messageResource.getMessage("common.next"));
        }
        else
        {
            this.$nextLink.append("-- ");
            this.$nextLink.append(this._createPageLink(pagerConfig.nextPageFirstRecordNumber, messageResource.getMessage("common.next")));
        }
    },

    _createPageLink: function(record, text)
    {
        var $link = $("<a data-record='" + record + "' href='javascript:void(0);'>" + text + "</a>");
        $link.click(function(event)
                    {
                        $(document).trigger("pager.shown_record.change", event.target.getAttribute("data-record"));
                    });
        return $link;
    }
});