function LockTTTableController() {}

LockTTTableController.prototype = _.create(Object.prototype, {
    init: function()
    {
        return this;
    },

    createLockTable: function(containerElement, onLockedTillDateChange)
    {
        var dateFormat = LockTTStore.getDateFormat();
        var dateParser = {
            parse: function(dateStr) { return parseDate(dateStr, TimeUtils.UNIFORM_DATE_FORMAT); }
        };

        var userGroupsList = [];
        var table = this._createTableObj(dateFormat, userGroupsList, onLockedTillDateChange);
        table.buildDOM(containerElement);
        for (var i = 0; i < userGroupsList.length; i++)
        {
            userGroupsList[i].fitHeightOnBuildDomComplete();
        }

        var scrollController = new SelectionScrollController();
        scrollController.linkTo(table);

        var selectionVeil = new SelectionVeil(LockTTStore.getSelectionVeilImages(), $(".lockTableWrapper")[0]);
        table.addLockEventsListener(new LockEventsHandler(dateFormat, dateParser, LockTTStore.getTimestamp(),
                selectionVeil, table.clearSelection, table, new DateFormat("MMM dd")));
        if (!LockTTStore.isLockTableImagesLoaded())
        {
            LockTTStore.setLockTableImagesLoaded(true);
            setTimeout(preloadLockTableImages, 500);
        }

        PageOverlayTipsManager.getInstance().refreshDynamicTips();
    },

    _createTableObj: function(dateFormat, userGroupsList, onLockedTillDateChange)
    {
        var userMap = this._convertBeanArrayToMap(LockTTStore.getUsers().concat(LockTTStore.getDeletedUsers()));
        var table = new LockTable(LockTTStore.getLoggedInUserToday(), LockTTStore.getWeekStartDay() - 1);
        var selectionController = table.getSelectionController();
        var hoverController = table.getHoverController();
        var tooltipController = new LockTableTooltipController(userMap, "lockDayTitle", Date.dayNamesShort);
        var runnerController = new RunnerController(LockTTStore.getLockedTillDate(),
                selectionController,
                hoverController,
                LockTTStore.getAllLockedTillDateBeyondDateRange());
        runnerController.addPositionChangeListener(onLockedTillDateChange);
        table.init(runnerController, tooltipController, new CommentsController(dateFormat));
        selectionController.setLockTable(table);
        var expandedUsersStorage = new ExpandedUsersStorage();


        //header rows
        var note = null;
        if (LockTTStore.getDisabledUsersSelected())
        {
            note = {
                imgPath: atImgPath("note_mark.gif"),
                text: messageResource.getMessage("lock_tt.table.header.disabled_users_note")
            };
        }
        var order = LockTTStore.getSortingOrder();
        var topMessageKey = "lock_tt.table.header.users_tasks";
        if (order == UserListSortConstant.ORDER_BY_USER_GROUP)
        {
            topMessageKey = "lock_tt.table.header.groups_users";
        }
        else if (order == UserListSortConstant.ORDER_BY_TIME_ZONE_GROUP)
        {
            topMessageKey = "lock_tt.table.header.time_zone_groups_users";
        }

        var topMessage = messageResource.getEscapedMessage(topMessageKey);
        var sortByLink = "";
        var isUserGroupEnabled = LockTTStore.getEnabledFeatures().UGR;
        var isTimeZoneGroupEnabled = LockTTStore.getEnabledFeatures().TZG;
        var onlyOneFeatureEnabled = isUserGroupEnabled ^ isTimeZoneGroupEnabled;
        if (isUserGroupEnabled || isTimeZoneGroupEnabled)
        {
            sortByLink = ' | ';
            var createSortLink = function(sortBy, text)
            {
                return '<a data-sortBy=' + sortBy +
                       ' href="javascript:void(0);" onclick="changeSortingOrder(this)" class="changeSortingOrder noprint">' +
                       text + '</a>\u00a0\u00a0';
            };
            if (order != UserListSortConstant.ORDER_BY_USERNAME)
            {
                sortByLink += createSortLink(UserListSortConstant.ORDER_BY_USERNAME,
                                             messageResource.getMessage(onlyOneFeatureEnabled ? "lock_tt.table.header.sort_by_user_name" : "lock_tt.table.header.by_user"));
            }
            if (order != UserListSortConstant.ORDER_BY_USER_GROUP && isUserGroupEnabled)
            {
                sortByLink += createSortLink(UserListSortConstant.ORDER_BY_USER_GROUP,
                                             messageResource.getMessage(onlyOneFeatureEnabled ? "lock_tt.table.header.sort_by_user_group" : "lock_tt.table.header.by_group"));
            }
            if (order != UserListSortConstant.ORDER_BY_TIME_ZONE_GROUP  && isTimeZoneGroupEnabled)
            {
                sortByLink += createSortLink(UserListSortConstant.ORDER_BY_TIME_ZONE_GROUP,
                                             messageResource.getMessage(onlyOneFeatureEnabled ? "lock_tt.table.header.sort_by_time_zone" : "lock_tt.table.header.by_time_zone"));
            }
        }

        buildMonthRow(table, LockTTStore.getMonths(), true, note, topMessage, sortByLink);

        var runner = new RunnerRow();
        var datesRow = new DatesRow(Date.dayNamesShort);
        datesRow.addCellObjWithoutAppend(TableRow.prototype.createCellObj(null));
        runner.addCellObjWithoutAppend(TableRow.prototype.createCellObj(null));

        _.each(LockTTStore.getHeaderData(), function(headerDatum)
        {
            var isRunnerOver = !_.isNull(LockTTStore.getLockedTillDate()) && headerDatum.date <= LockTTStore.getLockedTillDate();
            addHeaderCells(datesRow,
                           runner,
                           headerDatum.date,
                           headerDatum.isHoliday,
                           isRunnerOver,
                           headerDatum.hasLockedInvisibleUser,
                           headerDatum.hasUnlockedInvisibleUser);
        });

        table.addRow(runner);
        table.addDatesRow(datesRow);

        //User rows
        var isFirstGroup = true;
        var previousGroup;
        var from = LockTTStore.getDateRangeConfig().fromDate;
        var to = LockTTStore.getDateRangeConfig().toDate;
        var columnCount = to.offsetInDaysFrom(from) + 1;
        _.each(LockTTStore.getUserDataGroups(), function(userGroupObj)
        {
            var label = escapeHtmlSymbols(userGroupObj.label);
            if (userGroupObj.continued)
            {
                label += " " + messageResource.getMessage("lock_tt.table.user_group.continued_suffix");
            }
            if (_.isNull(userGroupObj.usersData) || userGroupObj.usersData.length == 0)
            {
                addSeparateRow(table, columnCount, isFirstGroup, label);
                previousGroup = null;
            }
            else
            {
                if (previousGroup)
                {
                    previousGroup.removeBottomBorder();
                }
                var userGroup = new UserGroup(label);
                previousGroup = userGroup;
                userGroupsList.push(userGroup);
                _.each(userGroupObj.usersData, function(userDatum)
                {
                    _.each(userDatum.userRowValues, function(userRowValue)
                    {
                        userRowValue.lockDate = userRowValue.lockDate ? Date.parseJSONDate(userRowValue.lockDate) : null;
                    });
                    _.each(userDatum.userRowApproveValues, function(userRowApproveValue)
                    {
                        userRowApproveValue.approvedDate = userRowApproveValue.approvedDate ? Date.parseJSONDate(userRowApproveValue.approvedDate) : null;
                    });

                    var userRow = addUserRow(userGroup, table, userMap[userDatum.userId], userDatum.userRowValues, userDatum.userRowApproveValues,
                                             userDatum.workingDays, expandedUsersStorage, Date.parseJSONDate(userDatum.userToday));
                    var leaveTimeText = messageResource.getMessage("lock_tt.table.leave_time");

                    _.each(userDatum.leaveTypes, function(leaveTypeInfo)
                    {
                        var leaveType = LeaveTypeStore.getLeaveType(leaveTypeInfo.leaveTypeId);
                        addLeaveTypeToUserRow(userRow,
                                              {
                                                  name: escapeHtmlSymbols(leaveType.name),
                                                  iconPath: getLeaveTypeIconPath(leaveType.iconId),
                                                  printIconPath: getLeaveTypePrintIconPath(leaveType.iconId)
                                              },
                                              leaveTypeInfo.leaveRowValues,
                                              leaveTimeText);
                    });

                    _.each(userDatum.tasks, function(userTask)
                    {
                        userTask.task.name = escapeHtmlSymbols(userTask.task.name);
                        userTask.task.projectName = escapeHtmlSymbols(userTask.task.projectName);
                        userTask.task.customerName = escapeHtmlSymbols(userTask.task.customerName);
                        addTaskRow(userRow, userTask.task, userTask.taskRowValues);
                    });

                    if (!_.isNull(userDatum.otherTasksTime))
                    {
                        var message = messageResource.getEscapedMessage(userDatum.tasks.length == 0 ?
                                                                        "lock_tt.table.tasks_not_assigned" :
                                                                        "lock_tt.table.other_tasks_not_assigned");
                        addOtherTasksRow(userRow, message, userDatum.otherTasksTime);
                    }

                    if (userDatum.showOvertime)
                    {
                        if (!_.isNull(userDatum.reportedOvertime))
                        {
                            addOvertimeRow(userRow,
                                           messageResource.getMessage("lock_tt.table.overtime_row.reported", [messageResource.getMessage("overtime.overtime")]),
                                           userDatum.reportedOvertime);
                        }
                        addOvertimeRow(userRow,
                                       messageResource.getMessage("lock_tt.table.overtime_row.auto_calculated", [messageResource.getMessage("overtime.overtime_lower_case")]),
                                       userDatum.autocalculatedOvertime);
                    }
                });
                isFirstGroup = false;
            }
        });
        addBottomBorderRow(table, columnCount);
        return table;
    },

    _convertBeanArrayToMap: function(beans)
    {
        var map = [];
        for (var i = 0; i < beans.length; i++)
        {
            var bean = beans[i];
            map[bean.id] = bean;
        }
        return map;
    }
});