ViewTTPageUtils = {
    tablePresentDataByDays: function()
    {
        var tableDataByDays = [];

        var viewTTStoreData = ViewTTStore.getData();
        var asc = ViewTTStore.getDetailsTableConfig().asc;
        var isTTAEnabled = ViewTTPageUtils.isFeatureEnabled(Features["TTA"]);
        var filter = ViewTTStore.getFilter();
        var filterMap = _.create(null);
        _.each(filter.weekApprovalStatuses, function(status) { filterMap[status] = status; });

        for (var i = 0; i < viewTTStoreData["daysInfoData"].length; i++)
        {
            var dayModel = viewTTStoreData["daysInfoData"][asc ? i : viewTTStoreData["daysInfoData"].length - i - 1];
            var approvalStatus = dayModel["approvalStatus"];
            if (!isTTAEnabled || approvalStatus == EffectiveWeekApprovalStatus.AUTO_APPROVED || filterMap.hasOwnProperty(approvalStatus))
            {
                tableDataByDays.push({type: ViewTTPageUtils.RowType.DAY, dayModel: dayModel});

                var subRows = [];

                if (dayModel["leave"] && dayModel["leave"].leaveTime > 0)
                    subRows.push({type: ViewTTPageUtils.RowType.LEAVE, dayModel: dayModel});

                _.each(dayModel["ttRecords"], function(taskModel)
                {
                    subRows.push({type: ViewTTPageUtils.RowType.TASK_BY_DAY, dayModel: dayModel, taskModel: taskModel});
                }, this);

                if (_.isObject(dayModel.otherTTInfo))
                    subRows.push({type: ViewTTPageUtils.RowType.OTHER_SPENT_BY_DAY, dayModel: dayModel});

                if (dayModel.overtime)
                {
                    var effectiveOvertime = _.isNumber(dayModel.overtime["reported"]) ? dayModel.overtime["reported"] : dayModel.overtime.auto;
                    if (_.isNumber(effectiveOvertime) && effectiveOvertime != 0 || effectiveOvertime == 0 && effectiveOvertime != dayModel.overtime.auto)
                        subRows.push({type: ViewTTPageUtils.RowType.OVERTIME, dayModel: dayModel});
                }

                if (subRows.length > 0)
                {
                    _.last(subRows).last = true;
                    _.each(subRows, function(row)
                    {
                        tableDataByDays.push(row);
                    });
                }
            }
        }

        return tableDataByDays;
    },

    tablePresentDataByCP: function()
    {
        var tableDataByCP = [];

        var viewTTStoreData = ViewTTStore.getData();
        var asc = ViewTTStore.getDetailsTableConfig().asc;
        var customersTTData = viewTTStoreData["customersData"]["customersTTData"];
        for (var i = 0; i < customersTTData.length; i++)
        {
            var customerModel = customersTTData[asc ? i : customersTTData.length - i - 1];
            tableDataByCP.push({type: ViewTTPageUtils.RowType.CUSTOMER, customerModel: customerModel});

            _.each(customerModel["projectTTData"], function(projectModel)
            {
                tableDataByCP.push({type: ViewTTPageUtils.RowType.PROJECT, projectModel: projectModel});

                _.each(projectModel["taskTTData"], function(taskModel, i)
                {
                    tableDataByCP.push({type: ViewTTPageUtils.RowType.TASK_BY_CP, taskModel: taskModel, last: i == projectModel["taskTTData"].length - 1});
                }, this);
            }, this);
        }

        if (_.isObject(viewTTStoreData["customersData"].otherTTInfo))
        {
            tableDataByCP.push({type: ViewTTPageUtils.RowType.OTHER_SPENT_BY_CP, otherTTInfo: viewTTStoreData["customersData"].otherTTInfo});
        }
        return tableDataByCP;
    },

    /**
     * Iterate leave totals in order depends of leave types position
     * And provide leave type information into applier function
     * @param leaveTotals array of leave totals
     * @param applier function applier that has two arguments 'leaveType' 'leaveTime'
     */
    leaveTotalsOrderedIterate: function(leaveTotals, applier)
    {
        _.each(LEAVE_TYPES_ORDERED, function(leaveType)
        {
            var matcher = _.matcher({
                                        "leaveTypeId": leaveType["id"]
                                    });
            var totalLeave = _.find(leaveTotals, matcher);
            if (_.isObject(totalLeave))
            {
                applier(leaveType, totalLeave["leaveTime"]);
            }
        });
    },

    /*
     check enabled feature (get features from store)
     */
    isFeatureEnabled: function(feature)
    {
        return !_.isUndefined(ViewTTStore.getEnabledFeatures()[feature]);
    },

    /**
     * Create url for redirect to enterTT
     * @param date day to redirect
     * @returns {string} url to enterTT with date parameter and current selected user id parameter (taken from ViewTTStore)
     */
    createEnterTTURL: function(date)
    {
        var selectedUserId = ViewTTStore.getSelectedUserId();
        return BASE_PATH + "user/submit_tt.do?dateStr=" + date.format(Date.convertSmartToExtDateFormat("yyyyMMdd")) + "&usersSelector.selectedUser=" + selectedUserId;
    },

    /**
     * Create url for open edit task popup
     * @param taskId task for edit
     * @returns {string} url for load task popup
     */
    createEditTaskPopupURL: function(taskId)
    {
        return BASE_PATH + 'tasks/edit_task_popup.do?taskId=' + taskId;
    },

    updateName: function(items, newItemsMap)
    {
        _.each(items, function(item)
        {
            var newItem = newItemsMap[item.id];
            if (!_.isUndefined(newItem))
                item.name = newItem.name;
        });
    },

    updateCPList: function(cpList, cptMaps)
    {
        ViewTTPageUtils.updateName(cpList.customers, cptMaps.customersById);
        ViewTTPageUtils.updateName(cpList.projects, cptMaps.projectsById);
    },

    customerComparator: function(customer1, customer2)
    {
        if (customer1.id == customer2.id)
            return 0;

        var customerName1 = customer1.name.toLowerCase();
        var customerName2 = customer2.name.toLowerCase();

        // name unique and id unique, so no equals
        if (customerName2 < customerName1)
            return 1;

        return -1;

    },

    getProjectComparator: function(customerMap)
    {
        return function(project1, project2)
        {
            // no need to compare project id, it`s unique
            // but in customer comparator we need it
            var ret = ViewTTPageUtils.customerComparator(customerMap[project1.customerId], customerMap[project2.customerId]);

            if (ret !== 0)
                return ret;

            var projectName1 = project1.name.toLowerCase();
            var projectName2 = project2.name.toLowerCase();

            // name unique and id unique, so no equals
            if (projectName2 < projectName1)
                return 1;

            return -1;
        };
    },

    /**
     * Merge two unordered cpLists
     * @param mainCPMap
     * @param mainCPList
     * @param ttCPList
     * @returns {{customers: (*|Array), projects: (*|Array)}}
     */
    mergeCPLists: function(mainCPMap, mainCPList, ttCPList)
    {
        var customers = this.orderLists(mainCPMap.customersById, mainCPList.customers, ttCPList.customers, ViewTTPageUtils.customerComparator);
        return {
            customers: customers,
            projects: this.orderLists(mainCPMap.projectsById, mainCPList.projects, ttCPList.projects, ViewTTPageUtils.getProjectComparator(arrayToMapById(customers)))
        };
    },

    /**
     * Get two unordered lists and return ordered one
     * @param mapById - map to check if this element not present in mainList
     * @param mainList
     * @param addOnList
     * @param comparator
     * @returns {Array}
     */
    orderLists: function(mapById, mainList, addOnList, comparator)
    {
        var result = mainList.slice();

        for (var i = 0; i < addOnList.length; i++)
        {
            var item = addOnList[i];
            if (!mapById[item.id])
                result.push(item);
        }

        result.sort(comparator);

        return result;
    },

    getChartTypeMessage: function(chartType)
    {
        var byTextKey;
        switch (chartType)
        {
            case ViewTTChartType.BY_DAYS:
                byTextKey = "view_tt.chart.view_by_days";
                break;
            case ViewTTChartType.BY_CUSTOMERS:
                byTextKey = "view_tt.chart.view_by_customers";
                break;
            case ViewTTChartType.BY_PROJECTS:
                byTextKey = "view_tt.chart.view_by_projects";
                break;
        }
        return messageResource.getMessage("view_tt.chart.view_by", [messageResource.getMessage(byTextKey)]);
    },

    calculateTotalSpent: function(storedData)
    {
        var totalSpent = storedData.totalWorking;

        if (storedData.totalLeaveTime)
            totalSpent += storedData.totalLeaveTime;

        return totalSpent;
    },

    calculateTotalSpentForToday: function(storedData)
    {
        var totalSpent = storedData.totalWorkingForToday;

        if (storedData.totalLeaveTimeForToday)
            totalSpent += storedData.totalLeaveTimeForToday;

        return totalSpent;
    },

    calculateOvertime: function(data)
    {
        var reported = 0;
        var auto = 0;

        if(data.totalOvertimeInfo) {
            reported = data.totalOvertimeInfo.reported;
            auto = data.totalOvertimeInfo.auto || 0;

            if (!_.isNumber(reported))
                reported = auto;
        }
        return {
            reported: reported,
            auto: auto
        }
    },

    makeStatusFilterToDisplay: function (statuses)
    {
        // do not mistaken null and [] - first means there is no statuses and the latter means status is not selected
        if (_.isNull(statuses))
            return null;

        var isStatusesApplied = !ViewTTStore.getUserInfo().autoApproved;

        // forced auto-approval status
        return isStatusesApplied ? statuses : [WeekApprovalStatus.APPROVED];
    },

    RowType: {
        DAY: 1,
        TASK_BY_DAY: 2,
        TASK_BY_CP: 3,
        PROJECT: 4,
        CUSTOMER: 5,
        LEAVE: 6,
        OVERTIME: 7,
        OTHER_SPENT_BY_DAY: 8,
        OTHER_SPENT_BY_CP: 9
    },

    SPINNER_SETTINGS: {
        lines: 12,
        length: 9,
        width: 5,
        radius: 11
    }
};

