ViewTTPage.ChartController = function() {};

ViewTTPage.ChartController.prototype = {
    DEFAULT_CHART_SIZE: {
        "height": 300,
        "width": 750
    },

    init: function()
    {
        this._viewTTChartContainerId = "viewTTChartContainer";
        this._viewTTChartContainerIdForExport = "viewTTChartContainerHidden";

        //init chart section with controls and chart placeholder
        this._chartSection = new ViewTTPage.ChartSection($("#viewTTChartSectionContainer")).init();
        this._renderChartMoire = new LoadingMoire(this._chartSection.find(".chartContainerWrapper"), ViewTTPageUtils.SPINNER_SETTINGS).init();

        // init available chart generators
        this._columnChartGenerator = new ViewTTPage.ColumnChartGenerator().init(this.DEFAULT_CHART_SIZE);
        this._pieChartByCustomersGenerator = new ViewTTPage.PieChartByCustomersGenerator().init(this.DEFAULT_CHART_SIZE);
        this._pieChartByProjectsGenerator = new ViewTTPage.PieChartByProjectsGenerator().init(this.DEFAULT_CHART_SIZE);

        var self = this;
        $(ViewTTStore).on("dataChange", function()
        {
            self._isNeedRebuildChart = true;
            self.showChart();
        });

        $(ViewTTStore).on("chartSectionConfigChange", function()
        {
            var chartSectionConfig = ViewTTStore.getChartSectionConfig();
            self._isChartVisible = chartSectionConfig["showChart"];
            var newChartType = chartSectionConfig["chartType"];
            if (newChartType != self._currentChartType)
            {
                self._isNeedRebuildChart = true;
                self._currentChartType = newChartType;
            }
            self.showChart();
        });

        var chartSectionConfig = ViewTTStore.getChartSectionConfig();

        this._isNeedRebuildChart = true;
        this._isChartVisible = chartSectionConfig["showChart"];
        this._currentChartType = chartSectionConfig["chartType"];
        this.showChart();

        return this;
    },

    showChart: function()
    {
        if (!this._isChartVisible)
        {
            this._chartSection.hideChartContainer();
            return;
        }
        this._chartSection.showChartContainer();

        if (!this._isNeedRebuildChart)
        {
            return;
        }
        this._chartSection.updateChartContainerHeight(this.DEFAULT_CHART_SIZE["height"]);
        this._renderChartMoire.show();

        var self = this;
        var preventSecondPieChartHeightPrepare = true;
        var currentChartGeneratorBondedWithData = this._getCurrentChartGeneratorBondedWithData();
        currentChartGeneratorBondedWithData.showChart(
                this._viewTTChartContainerId,
                {
                    "onRendered": function()
                    {
                        preventSecondPieChartHeightPrepare = !preventSecondPieChartHeightPrepare;
                        self._onChartReadyPostProcess(self._viewTTChartContainerId,
                                                      self.DEFAULT_CHART_SIZE,
                                                      preventSecondPieChartHeightPrepare ?
                                                      null :
                                                      function(topMargin, bottomMargin)
                                                      {
                                                          self._chartSection.updateChartContainerHeight(self.DEFAULT_CHART_SIZE["height"] + topMargin + bottomMargin);
                                                          currentChartGeneratorBondedWithData.resizeChartAndRemoveChartReference(topMargin, bottomMargin);
                                                      },
                                                      function()
                                                      {
                                                          self._renderChartMoire.hide();
                                                      }
                        );
                    }
                });

        this._isNeedRebuildChart = false;
    },

    exportChart: function(chartSize, onExportFinished)
    {
        this._chartSection.updateSizesForHiddenChartContainer(chartSize["height"], chartSize["width"]);

        var self = this;
        var preventSecondPieChartHeightPrepare = true;
        var currentChartGeneratorBondedWithData = this._getCurrentChartGeneratorBondedWithData();
        currentChartGeneratorBondedWithData.exportChart(
                this._viewTTChartContainerIdForExport,
                {
                    "onExported": function(img)
                    {
                        preventSecondPieChartHeightPrepare = !preventSecondPieChartHeightPrepare;
                        self._onChartReadyPostProcess(self._viewTTChartContainerIdForExport,
                                                      chartSize,
                                                      preventSecondPieChartHeightPrepare ?
                                                      null :
                                                      function(topMargin, bottomMargin, chartHeight)
                                                      {
                                                          currentChartGeneratorBondedWithData.rescaleChartAndRemoveChartReference(topMargin, bottomMargin, chartHeight);
                                                      },
                                                      function()
                                                      {
                                                          self._chartSection.clearHiddenChartContainer();
                                                          onExportFinished(img);
                                                      }
                        );
                    },
                    "chartSize": chartSize
                });
    },

    _onChartReadyPostProcess: function(containerId, chartSize, preparePieChartFn, onSuccess)
    {
        if (preparePieChartFn && !isIE8() && this._currentChartType != ViewTTChartType.BY_DAYS)
        {
            //changes height of pie chart container when labels don't fit on it
            var maxY = 0;
            var minY = 0;

            $('#' + containerId).find('g').each(function()
                                                {
                                                    var bBox = this.getBBox();
                                                    maxY = Math.max(maxY, bBox.y + bBox.height);
                                                    minY = Math.min(minY, bBox.y);
                                                });
            if (maxY > chartSize["height"] || minY < 0)
            {
                var topMargin = Math.abs(minY);
                var bottomMargin = maxY > chartSize["height"] ? maxY - chartSize["height"] : 0;
                preparePieChartFn(topMargin, bottomMargin, chartSize);
            }
            else
            {
                onSuccess();
            }
        }
        else
        {
            onSuccess();
        }
    },

    _getCurrentChartGeneratorBondedWithData: function()
    {
        var data = ViewTTStore.getData();
        var chartGenerator;
        var chartData;
        var chartConfig;
        var isEmptyData;
        if (this._currentChartType == ViewTTChartType.BY_DAYS)
        {
            chartData = data["daysInfoData"];
            isEmptyData = chartData.length == 0;
            chartGenerator = this._columnChartGenerator;
            chartConfig = this._prepareConfigForColumnChart(data);
        }
        else if (this._currentChartType == ViewTTChartType.BY_CUSTOMERS)
        {
            chartData = data["customersData"];
            isEmptyData = chartData["customersTTData"].length == 0 && _.isNull(chartData["otherTTInfo"]);
            chartGenerator = this._pieChartByCustomersGenerator;
            chartConfig = this._prepareConfigForPieChartByCustomers();
        }
        else
        {
            chartData = data["customersData"];
            isEmptyData = chartData["customersTTData"].length == 0 && _.isNull(chartData["otherTTInfo"]);
            chartGenerator = this._pieChartByProjectsGenerator;
            chartConfig = this._prepareConfigForPieChartByProjects();
        }

        var resizeChartAndRemoveChartReferenceFn;
        var rescaleChartAndRemoveChartReferenceFn;
        if (this._currentChartType == ViewTTChartType.BY_DAYS)
        {
            resizeChartAndRemoveChartReferenceFn = function(){};
            rescaleChartAndRemoveChartReferenceFn = function(){}
        }
        else
        {
            resizeChartAndRemoveChartReferenceFn = function(topMargin, bottomMargin)
            {
                chartGenerator.resizePieChart(topMargin, bottomMargin);
                chartGenerator.removeReferenceOnChart();
            };
            rescaleChartAndRemoveChartReferenceFn = function(topMargin, bottomMargin, chartHeight)
            {
                chartGenerator.rescalePieChartForExport(topMargin, bottomMargin, chartHeight);
                chartGenerator.removeReferenceOnChart();
            };
        }

        var showChartFn;
        var exportChartFn;
        if (isEmptyData)
        {
            var self = this;
            showChartFn = function(_, additionalConfig)
            {
                self._chartSection.showEmptyDataMessage();
                defer(function(){additionalConfig["onRendered"]()});
            };
            exportChartFn = function(_, additionalConfig)
            {
                additionalConfig["onExported"](null);
            }
        }
        else
        {
            showChartFn = function(container, additionalConfig)
            {
                chartConfig = $.extend(true, chartConfig, additionalConfig);
                chartGenerator.renderChart(container, chartData, chartConfig);
            };
            exportChartFn = function(container, additionalConfig)
            {
                chartConfig = $.extend(true, chartConfig, additionalConfig);
                chartGenerator.exportChartToJPG(container, chartData, chartConfig);
            }
        }

        return {
            "showChart": showChartFn,
            "exportChart": exportChartFn,
            "resizeChartAndRemoveChartReference": resizeChartAndRemoveChartReferenceFn,
            "rescaleChartAndRemoveChartReference": rescaleChartAndRemoveChartReferenceFn
        }
    },

    _prepareConfigForColumnChart: function(data)
    {
        var userToday = ViewTTStore.getUserInfo()["today"];
        var isOtherUser = ViewTTStore.getSelectedUserId() != ViewTTStore.getViewerId();
        return {
            "userToday": userToday,
            "userHireDate": ViewTTStore.getUserInfo()["hireDate"],
            "colorScheme": this._getColumnChartColorScheme(isOtherUser),
            "dateFormat": ViewTTStore.getDateFormat(),
            "weekStartDay": ViewTTStore.getWeekStartDay()
        }
    },

    _prepareConfigForPieChartByCustomers: function()
    {
        return {
            "customersMap": ViewTTStore.getCptMaps()["customersById"]
        }
    },

    _prepareConfigForPieChartByProjects: function()
    {
        return {
            "customersMap": ViewTTStore.getCptMaps()["customersById"],
            "projectsMap": ViewTTStore.getCptMaps()["projectsById"]
        }
    },

    _getColumnChartColorScheme: function(isOtherUser)
    {
        var allSchemes = ViewTTPage.ColumnChartGenerator.colorScheme;
        var colorScheme;
        switch (CustomizationSettings.colorScheme.toLowerCase())
        {
            case "red":
                colorScheme = isOtherUser ? allSchemes.RED_GREY : allSchemes.RED;
                break;
            case "orange":
                colorScheme = isOtherUser ? allSchemes.ORANGE_GREY : allSchemes.ORANGE;
                break;
            default:
                colorScheme = isOtherUser ? allSchemes.DEFAULT_GREY : allSchemes.DEFAULT;
        }
        return colorScheme;
    }
};
