ViewTTPage.ChartSection = function($containerElement)
{
    TemplateView.call(this, $containerElement);
};

ViewTTPage.ChartSection.prototype = _.create(TemplateView.prototype, {
    html: "<div class='viewTTChartSection'>" +
          "    <div class='containerHeader'>" +
          "        <span class='selectorLabel'><%=viewChartByLabel%></span><span class='selectorLink'><span class='selectorLinkText'></span></span>" +
          "        <div class='collapseButton noprint'></div>" +
          "    </div>" +
          "    <div class='chartContainerWrapper'>" +
          "        <div id='viewTTChartContainer' class='chartContainer'></div>" +
          "        <div id='viewTTChartContainerHidden' class='hiddenChartContainer'></div>" +
          "    </div>" +
          "</div>",

    afterRender: function()
    {
        this.config = ViewTTStore.getChartSectionConfig();
        this.$selectorLink = this.find(".selectorLink");
        this.$collapseBtn = this.find(".collapseButton");
        this.$chartContainer = this.find(".chartContainer");
        this.$hiddenChartContainer = this.find(".hiddenChartContainer");

        this._initChartSelector(this.config["chartType"]);

        this.$collapseBtn.on("click", _.bind(this._onCollapseExpandButtonClick, this));
    },

    update: function()
    {
        this._updateChartSelectorLinkText(this.config["chartType"]);
        this._updateCollapseExpandButton(this.config["showChart"]);
        this.$el.toggleClass("noprint", !this.config["showChart"]);
    },

    messageKeys: function()
    {
        return {
            "viewChartByLabel": "view_tt.chart.view_by_label"
        };
    },

    hideChartContainer: function()
    {
        this.$chartContainer.hide();
        //this.$chartContainer.atAnimation().hideAnimated("hideAnimated");
    },

    showChartContainer: function()
    {
        this.$chartContainer.show();
        //this.$chartContainer.atAnimation().showAnimated("showAnimated");
    },

    updateSizesForHiddenChartContainer: function(height, width)
    {
        this.$hiddenChartContainer.css("height", height).css("width", width);
    },

    clearHiddenChartContainer: function()
    {
        this.$hiddenChartContainer.empty();
    },

    showEmptyDataMessage: function()
    {
        this.$chartContainer.empty();
        this.$chartContainer.append("<div class='noData'>" + messageResource.getMessage("view_tt.chart.no_data") + "</div>");
    },

    _initChartSelector: function(chartType)
    {
        var chartTypeSelector = new ViewTTPage.ChartSection.ChartTypeSelector($("body"), chartType).init();
        $(chartTypeSelector).on("chartTypeChanged", _.bind(this._onChartTypeChanged, this));
        this.$selectorLink.on("click", function()
        {
            chartTypeSelector.showAt($(this));
        });
    },

    _updateChartSelectorLinkText: function(chartType)
    {
        this.find(".selectorLinkText").text(ViewTTPageUtils.getChartTypeMessage(chartType));
    },

    _onCollapseExpandButtonClick: function()
    {
        this.config["showChart"] = !this.config["showChart"];
        this._onChartSectionConfigChanged();
    },

    _onChartTypeChanged: function(e, chartType)
    {
        if (this.config["chartType"] != chartType) // if chart type changed - need to show chart if hidden
        {
            this.config["showChart"] = true;
        }
        this.config["chartType"] = chartType;
        this._onChartSectionConfigChanged();
    },

    _updateCollapseExpandButton: function(showChart)
    {
        this.$collapseBtn.toggleClass("collapsed", !showChart);
    },

    _onChartSectionConfigChanged: function()
    {
        this.update();
        $(document).trigger("viewTT.chartSection.change", this.config);
    },

    updateChartContainerHeight: function(height)
    {
        this.$chartContainer.css("height", height);
    }
});
