ViewTTPage.PieChartGenerator = function() {};

ViewTTPage.PieChartGenerator.prototype = _.create(ChartGenerator.prototype, {

    _prepareChartSettings: function(chartData, config)
    {
        var labelFunction = this._createLabelFunction();
        var balloonFunction = this._createBalloonFunction();

        var dataProvider = this._getDataProvider(chartData, config);

        return {
            "type": "pie",
            "autoResize": false,
            "radius": 104,
            "innerRadius": 40,
            "outlineThickness": 1,
            "outlineAlpha": 1,
            "theme": "light",
            "addClassNames": true,

            "fontSize": 11,
            "labelRadius": 20,
            "maxLabelWidth": 300,
            "hideLabelsPercent": 1.5,
            "labelFunction": labelFunction,

            "autoMargins": false,

            "startDuration": 0,
            "startEffect": "easeOutSine",
            "startRadius": 100,
            "sequencedAnimation": false,

            "balloon": {
                "maxWidth": 350 // text width limited by balloon inner content
            },
            "balloonFunction": balloonFunction,

            "titleField": "category",
            "valueField": "value",
            "colorField": "sectorColor",
            "dataProvider": dataProvider
        };
    },

    _updateSettingsForExport: function(settings, chartData, config)
    {
        var scales = this._getChartScales(config);
        settings["fontSize"] *= scales.common;
        settings["outlineThickness"] *= scales.common;
        settings["maxLabelWidth"] *= scales.horizontal;
        settings["radius"] *= scales.common;
        settings["innerRadius"] *= scales.common;
    },

    _getDataProvider: function(chartData, config)
    {
        // abstract
    },

    _createLabelFunction: function()
    {
        return function(slice)
        {
            return slice.dataContext.category.substring(0, 65);
        }
    },
    
    _createBalloonFunction: function()
    {
        var numberFormatter = new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 2, true);
        var balloonTemplate = _.template("<div class='cpChartsBalloon'>" +
                                         "   <span class='category'><%=category%></span>" +
                                         "   <div class='values'><span class='value'><%=value%></span> <span>(<%=percents%>%)</span></div>" +
                                         "</div>");
        return function(item)
        {
            return balloonTemplate({
                                       "category": escapeHtmlSymbols(item.dataContext.category),
                                       "value": UserTime.formatTime(item.dataContext.value),
                                       "percents": numberFormatter.format(item.percents)
                                   });
        }
    },

    _makeChart: function(container, settings)
    {
        this._chart = AmCharts.makeChart(container, settings, 10);
    },

    resizePieChart: function(marginTop, marginBottom)
    {
        this._chart.marginTop = marginTop;
        this._chart.marginBottom = marginBottom;
        this._redrawChart();
    },

    rescalePieChartForExport: function(marginTop, marginBottom, chartSize)
    {
        var scale = (chartSize["height"] - marginTop - marginBottom) / chartSize["height"];
        this._chart.radius *= scale;
        this._chart.innerRadius *= scale;
        this._chart.fontSize *= scale;
        this._chart.outlineThickness *= scale;
        this._redrawChart();
    },

    _redrawChart: function()
    {
        this._chart.validateNow(); // redraw chart and force 'onRendered' event
    },

    removeReferenceOnChart: function()
    {
        this._chart = null;
    }
});
