ViewTTPage.DetailsTable = function($containerEl)
{
    TemplateView.call(this, $containerEl);
};

ViewTTPage.DetailsTable.prototype = _.create(TemplateView.prototype, {
    html: "<div class='detailsTable'>" +
          "  <div class='title'><%=titleLabel%></div>" +
          "  <div class='configPanelContainer'></div>" +
          "  <div class='dayRowsContainer'>" +
          "    <table class='dayRows display-for-print' cellspacing='0' cellpadding='0'></table>" +
          "    <div class='noDayRowsLabel'><%=noDayRowsLabel%></div>" +
          "  </div>" +
          "  <div class='cpRowsContainer'>" +
          "    <table class='cpRows display-for-print' cellspacing='0' cellpadding='0'></table>" +
          "    <div class='noCPRowsLabel'><%-noCPRowsLabel%></div>" +
          "  </div>" +
          "</div>",

    afterRender: function()
    {
        this.$configPanelContainer = this.find(".configPanelContainer");
        this.$dayRowsTable = this.find(".dayRows");
        this.$cpRowsTable = this.find(".cpRows");
        this.configPanel = new ViewTTPage.DetailsTable.ConfigPanel(this.$configPanelContainer).init();
        this.loadingMoire = new LoadingMoire(this.$el, ViewTTPageUtils.SPINNER_SETTINGS).init();
        this._rememberDetailsTableConfig();

        var self = this;
        $(document).on("viewTT.beginLoadData", function()
        {
            self.disable();
        });
        $(document).on("viewTT.endLoadData", function()
        {
            self.enable();
        });
        $(ViewTTStore).on("dataChange", function()
        {
            self.update();
        });
        $(ViewTTStore).on("detailsTableConfigChange", function()
        {
            if (self.prevDetailsTableConfig.asc != ViewTTStore.getDetailsTableConfig().asc)
                self.update();
            else
            {
                self._updateVisibilityClasses();
                $(document).trigger("viewTT.detailsTable.update");
            }
            self._rememberDetailsTableConfig();
        });
    },

    update: function()
    {
        this._updateVisibilityClasses();

        this.$dayRowsTable.empty();
        this.$cpRowsTable.empty();

        var presentDataByDays = ViewTTPageUtils.tablePresentDataByDays();
        _.each(presentDataByDays, function(row)
        {
            switch (row.type)
            {
                case ViewTTPageUtils.RowType.DAY:
                    new ViewTTPage.DetailsTable.DayRow(this.$dayRowsTable, row.dayModel).init();
                    break;
                case ViewTTPageUtils.RowType.LEAVE:
                    new ViewTTPage.DetailsTable.LeaveRow(this.$dayRowsTable, row.dayModel, row.last).init();
                    break;
                case ViewTTPageUtils.RowType.TASK_BY_DAY:
                    new ViewTTPage.DetailsTable.ViewByDaysTaskRow(this.$dayRowsTable, row.dayModel, row.taskModel, row.last).init();
                    break;
                case ViewTTPageUtils.RowType.OTHER_SPENT_BY_DAY:
                    new ViewTTPage.DetailsTable.ViewByDaysOtherSpentRow(this.$dayRowsTable, row.dayModel, row.last).init();
                    break;
                case ViewTTPageUtils.RowType.OVERTIME:
                    new ViewTTPage.DetailsTable.OvertimeRow(this.$dayRowsTable, row.dayModel, row.last).init();
                    break;
            }
        }, this);

        var presentDataByCP = ViewTTPageUtils.tablePresentDataByCP();
        _.each(presentDataByCP, function(row)
        {
            switch (row.type)
            {
                case ViewTTPageUtils.RowType.CUSTOMER:
                    new ViewTTPage.DetailsTable.CustomerRow(this.$cpRowsTable, row.customerModel).init();
                    break;
                case ViewTTPageUtils.RowType.OTHER_SPENT_BY_CP:
                    new ViewTTPage.DetailsTable.ViewByCPOtherSpentRow(this.$cpRowsTable, row.otherTTInfo).init();
                    break;
                case ViewTTPageUtils.RowType.PROJECT:
                    new ViewTTPage.DetailsTable.ProjectRow(this.$cpRowsTable, row.projectModel).init();
                    break;
                case ViewTTPageUtils.RowType.TASK_BY_CP:
                    new ViewTTPage.DetailsTable.ViewByCPTaskRow(this.$cpRowsTable, row.taskModel, row.last).init();
                    break;
            }
        }, this);

        this.$el.toggleClass("hasDayRows", presentDataByDays.length > 0);
        this.$el.toggleClass("hasCPRows", presentDataByCP.length > 0);
        this.$el.toggleClass("otherUser", ViewTTStore.getSelectedUserId() != ViewTTStore.getViewerId());

        $(document).trigger("viewTT.detailsTable.update");
    },

    messageKeys: function()
    {
        return {
            titleLabel: "view_tt.details_table.title",
            noDayRowsLabel: "view_tt.details_table.no_day_rows",
            noCPRowsLabel: "view_tt.details_table.no_cp_rows"
        }
    },

    disable: function()
    {
        this.$el.attr("disabled", true);
        this.loadingMoire.show();
    },

    enable: function()
    {
        this.$el.attr("disabled", false);
        this.loadingMoire.hide();
    },

    _updateVisibilityClasses: function()
    {
        var detailsTableConfig = ViewTTStore.getDetailsTableConfig();
        this.$el.toggleClass("viewByDays", detailsTableConfig.viewByDays);
        this.$el.toggleClass("viewByCP", !detailsTableConfig.viewByDays);
        this.$el.toggleClass("showTasks", detailsTableConfig.showTasks);
        this.$el.toggleClass("hideTasks", !detailsTableConfig.showTasks);
    },

    _rememberDetailsTableConfig: function()
    {
        this.prevDetailsTableConfig = _.clone(ViewTTStore.getDetailsTableConfig());
    }
});