ViewTTPage.DetailsTable.DayRow = function($containerEl, dayModel)
{
    TemplateView.call(this, $containerEl);
    this.dayModel = dayModel;
};

ViewTTPage.DetailsTable.DayRow.prototype = _.create(TemplateView.prototype, {
    html: "<tr class='dayRow'>" +
          "  <td class='nameColumn'>" +
          "    <table cellspacing='0' cellpadding='0'>" +
          "      <tr>" +
          "        <td class='dateColumn'>" +
          "          <div class='subContainer'>" +
          "            <span><%-date%></span>" +
          "            <span class='nonWorkingLabel'><%=nonWorkingLabel%></span>" +
          "          </div>" +
          "        </td>" +
          "        <td class='approvedStatusColumn'>" +
          "          <div class='subContainer'>" +
          "            <span class='autoApprovedLabel'><%=autoApprovedLabel%></span>" +
          "            <span class='approvedLabel'><%=approvedLabel%></span>" +
          "            <span class='notReadyLabel'><%=notReadyLabel%></span>" +
          "            <span class='readyLabel'><%=readyLabel%></span>" +
          "            <span class='rejectedLabel'><%=rejectedLabel%></span>" +
          "            <span class='lockedLabel for-print-only'>Locked</span>" +
          "          </div>" +
          "        </td>" +
          "      </tr>" +
          "    </table>" +
          "  </td>" +
          "  <td class='totalColumn'><div class='subContainer'><%-total%></div></td>" +
          "  <td class='detailsColumn'>" +
          "    <table cellspacing='0' cellpadding='0'>" +
          "      <tr>" +
          "        <td class='workingTimeColumn'>" +
          "          <div class='subContainer'>" +
          "            <span><%=workingTimeLabel%>:</span>" +
          "            <span class='workingTime'><%-workingTime%></span>" +
          "          </div>" +
          "        </td>" +
          "        <td class='overtimeColumn'>" +
          "          <div class='subContainer'>" +
          "            <span class='overtimeLabel'><%=overtimeLabel%>:</span>" +
          "            <span class='overtime'><%-overtime%></span>" +
          "          </div>" +
          "        </td>" +
          "        <td class='leaveTimeColumn'>" +
          "          <div class='subContainer'>" +
          "            <span class='leaveTimeName'><%-leaveTimeName%>:</span>" +
          "            <span class='leaveTime'><%-leaveTime%></span>" +
          "          </div>" +
          "        </td>" +
          "      </tr>" +
          "    </table>" +
          "  </td>" +
          "</tr>",

    templateKey: "ViewTTPage.DetailsTable.DayRow",

    beforeRender: function()
    {
        this.leaveTime = this.dayModel.leave && this.dayModel.leave.leaveTime || 0;
        if (this.dayModel.overtime)
            this.overtime = _.isNumber(this.dayModel.overtime.reported) ? this.dayModel.overtime.reported : this.dayModel.overtime.auto;
        else
            this.overtime = 0;
        this.workingTime = this.dayModel.totalWorkingTime;
        this.totalTime = this.leaveTime + this.workingTime;
    },

    afterRender: function()
    {
        this.$el.toggleClass("autoApproved", this.dayModel.approvalStatus == EffectiveWeekApprovalStatus.AUTO_APPROVED);
        this.$el.toggleClass("approved", this.dayModel.approvalStatus == EffectiveWeekApprovalStatus.APPROVED);
        this.$el.toggleClass("notReady", this.dayModel.approvalStatus == EffectiveWeekApprovalStatus.NOT_READY);
        this.$el.toggleClass("ready", this.dayModel.approvalStatus == EffectiveWeekApprovalStatus.READY);
        this.$el.toggleClass("rejected", this.dayModel.approvalStatus == EffectiveWeekApprovalStatus.REJECTED);
        this.$el.toggleClass("locked", this.dayModel.locked);
        this.$el.toggleClass("nonWorking", this.dayModel.workdayDuration == 0);
        this.$el.toggleClass("noOvertime", !_.isObject(this.dayModel.overtime));
        this.$el.toggleClass("noLeaves", !_.isObject(this.dayModel.leave));
        this.$el.toggleClass("leaveTimeZero", this.leaveTime == 0);
        this.$el.toggleClass("workingTimeZero", this.workingTime == 0);
        this.$el.toggleClass("totalTimeZero", (this.workingTime + this.leaveTime) == 0);
        this.$el.toggleClass("overtimePositive", this.overtime > 0);
        this.$el.toggleClass("overtimeNegative", this.overtime < 0);
        this.$el.toggleClass("overtimeZero", this.overtime == 0);

        this.tooltip = ViewTTPage.DetailsTable.DayRow.Tooltip.getInstance();
        this.tooltip.subscribeToMouseEvents(this.$el, this.bind(this._beforeTooltipShow));

        this.$el.click(this.bind(this._onClick));
    },

    templateModel: function()
    {
        return {
            date: ViewTTPage.DetailsTable.DayRow.tableDateFormat.formatDate(this.dayModel.date),
            name: this.dayModel.name,
            total: UserTime.formatTimeWithDigitSeparator(this.totalTime),
            workingTime: this.workingTime ? UserTime.formatTimeWithDigitSeparator(this.workingTime) : "--",
            leaveTime: this.leaveTime ? UserTime.formatTimeWithDigitSeparator(this.leaveTime) : "--",
            overtime: this.overtime ? UserTime.formatTimeWithDigitSeparator(this.overtime) : "--",
            leaveTimeName: this.leaveTime ? LeaveTypeStore.getLeaveType(this.dayModel.leave.leaveTypeId).name :
                           messageResource.getMessage("view_tt.details_table.day_row.leave_time")
        }
    },

    messageKeys: function()
    {
        return {
            nonWorkingLabel: "view_tt.details_table.day_row.non_working",
            autoApprovedLabel: "view_tt.details_table.day_row.auto_approved",
            approvedLabel: "view_tt.details_table.day_row.approved",
            notReadyLabel: "view_tt.details_table.day_row.not_ready",
            readyLabel: "view_tt.details_table.day_row.ready",
            rejectedLabel: "view_tt.details_table.day_row.rejected",
            workingTimeLabel: "view_tt.details_table.day_row.working_time",
            overtimeLabel: "overtime.overtime"
        };
    },

    _onClick: function()
    {
        window.location.href = ViewTTPageUtils.createEnterTTURL(this.dayModel.date);
    },

    _beforeTooltipShow: function()
    {
        this.tooltip.updateDate(this.dayModel.date);
    }
});

ViewTTPage.DetailsTable.DayRow.tableDateFormat = new DateFormat("MMM dd, EEE");