ViewTTPage.TotalsForPeriod = function() {};

ViewTTPage.TotalsForPeriod.prototype = {

    monthName: function(monthNum)
    {
        var date = new Date();
        date.setMonth(monthNum, 1);
        return this.monthDateFormat.formatDate(date);
    },

    _currentMonth: function()
    {
        return currentUserTime().getMonth();
    },

    monthDateFormat: new DateFormat("MMMM"),
    
    init: function($containerEl)
    {
        this.$containerEl = $containerEl;
        this.$label = $('#totalsForPeriodLabel');
        this.$rangeTotal = $containerEl.find(".rangeTotal");
        this.$forTodayTotal = $containerEl.find(".forTodayTotal");
        this.$totalDiff = $containerEl.find(".totalDiff");

        return this;
    },

    formatDateQualifier: function(qualifier)
    {
        switch (qualifier)
        {
            case DateRangeSelectionQualifier.CURRENT_MONTH:
                return this._currentMonthLabel();
            case DateRangeSelectionQualifier.PREVIOUS_MONTH:
                return this._prevMonthLabel();
            case DateRangeSelectionQualifier.CURRENT_WEEK:
                return messageResource.getMessage('view_tt.totals.period.current_week');
            case DateRangeSelectionQualifier.PREVIOUS_WEEK:
                return messageResource.getMessage('view_tt.totals.period.previous_week');
            case DateRangeSelectionQualifier.CURRENT_QUARTER:
                return messageResource.getMessage('view_tt.totals.period.current_quarter');
            case DateRangeSelectionQualifier.PREVIOUS_QUARTER:
                return messageResource.getMessage('view_tt.totals.period.previous_quarter');
            case DateRangeSelectionQualifier.CUSTOM:
                return messageResource.getMessage('view_tt.totals.period.custom_range');
            default:
                return messageResource.getMessage('common.not_available');
        }
    },
    
    _changeDateRange: function()
    {
        var config = ViewTTStore.getDateRangeConfig();

        if (!config)
            return;

        var label = this.formatDateQualifier(config.qualifier);

        this.setPeriod(label);
    },

    /**
     * @param data - not empty data
     */
    changeTotals: function(data)
    {
        this._changeDateRange();

        var totalSpent = ViewTTPageUtils.calculateTotalSpent(data);

        this.$rangeTotal.find(".reported .value").text(UserTime.formatTimeWithDigitSeparator(totalSpent));
        if (!_.isNull(data.effectiveDuration))
        {
            this.$rangeTotal.find(".scheduled .value").text(UserTime.formatTimeWithDigitSeparator(data.effectiveDuration));
        }

        var totalDiff = 0;
        if (!_.isNull(data.totalWorkingForToday))
        {
            var forTodaySpent = ViewTTPageUtils.calculateTotalSpentForToday(data);
            this.$forTodayTotal.find(".reported .value").text(UserTime.formatTimeWithDigitSeparator(forTodaySpent));
            if (!_.isNull(data.effectiveDurationForToday))
            {
                this.$forTodayTotal.find(".scheduled .value").text(UserTime.formatTimeWithDigitSeparator(data.effectiveDurationForToday));
                totalDiff = forTodaySpent - data.effectiveDurationForToday;
            }
            this.$forTodayTotal.show();
        } else {
            if (!_.isNull(data.effectiveDuration))
            {
                totalDiff = totalSpent - data.effectiveDuration;
            }
            this.$forTodayTotal.hide();
        }

        var futureDateRange = ViewTTStore.getDateRangeConfig().fromDate > ViewTTStore.getUserInfo()["today"];
        if (totalDiff !== 0 && !futureDateRange)
        {
            this.$totalDiff.find(".value").text(UserTime.formatTimeWithDigitSeparator((totalDiff > 0) ? totalDiff : -totalDiff));
            this.$totalDiff.find(".diffMessage").text(messageResource.getMessage((totalDiff > 0) ? 'view_tt.totals.diff.more': 'view_tt.totals.diff.less'));
            this.$totalDiff.toggleClass("less", totalDiff < 0);
            this.$totalDiff.toggleClass("more", totalDiff > 0);
            this.$totalDiff.show();
        } else {
            this.$totalDiff.hide();
        }

    },

    _currentMonthLabel: function()
    {
        return this.monthName(this._currentMonth());
    },

    _prevMonthLabel: function()
    {
        // 0 - january, we can`t subtract month from it
        var month = this._currentMonth() || 12;
        return this.monthName(month - 1);
    },

    setPeriod: function(label)
    {
        this.$rangeTotal.find(".label").text(messageResource.getMessage('view_tt.totals.for', [label]));
    }

};