Ext.namespace("AT");

AT.UserSelectorWithAccessRightForOneTimeSelectorPanel = Ext.extend(AT.UsersSelectorPanel, {

    showResetButton: false,
    /**
     * @cfg {String} customCls style class for the custom option section container.
     */
    customCls: "custom-no-options",
    cls: "selector-panel no-options oneTimeSelectorPanel",

    /**
     * @cfg {String} detailsCls style class for the details section container.
     */
    detailsCls: "details-no-options",

    messages: {
        dialogHeader: "",
        buttonText: "",
        processText: ""
    },

    customSaveCallback: Ext.emptyFn,

    currentSaveOptions: null,

    accessRight: null,

    /**
     * @see AT.SelectorPanel
     */
    createComponents: function() {
        AT.UserSelectorWithAccessRightForOneTimeSelectorPanel.superclass.createComponents.call(this);
        this.mainLabel = new Ext.Panel({
            html: messageResource.getMessage( this.messages.dialogHeader ),
            cls: "text main-label"
        });
        this.saveButton = new AT.GradientButton({
            text: messageResource.getMessage( this.messages.buttonText ),
            minWidth: 265,
            disabled: true,
            cls: "greyButton copy-to-button"
        });
        this.savingLabel = new Ext.Panel({
            html: messageResource.getMessage( this.messages.processText ),
            cls: "text saving-label"
        });

        this.saveButton.on("click", this.saveCallback, this)
    },

    initComponent: function() {
        AT.UserSelectorWithAccessRightForOneTimeSelectorPanel.superclass.initComponent.call(this);
        this.on("change", this.updateSaveButtonState, this);
    },

    createCustomOption: function()
    {
        this.customOption = undefined;
    },

    createGenericOptions: function()
    {
        this.genericOptions = [];
    },

    disableTextSelectionForNeededElements: function() {
        disableTextSelection(this.customOptionsContainer.el.dom);
    },

    /**
     * Returns the value of current selected option.
     */
    getCurrentOptionValue: function() {
        return this.customOptionValue;
    },

    isCustomOptionSelected: function() {
        return true;
    },

    /**
     * @see AT.SelectorPanel
     */
    getCustomOptionSection: function()
    {
        var customOptionControls = [];
        // from left to right
        customOptionControls.push(this.viewByPanel);
        customOptionControls.push(this.getCustomOptionButtonsSection());

        this.customControlsContainer = new Ext.Container({
            cls: "userSelectorCustomControlsContainer",
            autoEl: "div",
            items: customOptionControls
        });
        return [this.mainLabel, this.customControlsContainer];
    },

    /**
     * @see AT.SelectorPanel
     */
    getDetailsSection: function() {
        return [ this.selectorTree ];
    },

    /**
     * @see AT.SelectorPanel
     */
    getFooterSection: function() {
        var res = AT.UserSelectorWithAccessRightForOneTimeSelectorPanel.superclass.getFooterSection.call(this);
        return res.concat(this.saveButton).concat(this.savingLabel);
    },

    setSaveCallback: function (callback) {
        this.customSaveCallback = callback;
    },

    saveCallback: function()
    {
        Ext.each(this.getFooterSection(), function(item){
            item.hide();
        });
        this.savingLabel.show();
        this.customSaveCallback.call(this);
    },

    setCurrentSaveOptions: function(options) {
        this.currentSaveOptions = options;
    },

    getCurrentSaveOptions: function(){
        return this.currentSaveOptions;
    },

    close: function() {
        this.fireEvent("close");
    },

    setSelection: function( selection ) {
        AT.UserSelectorWithAccessRightForOneTimeSelectorPanel.superclass.setSelection.call(this, selection);
        this.updateSaveButtonState();
    },

    updateSaveButtonState: function() {
        if( this.selectorTree.getSelectionController().getSelectedNodeSet().isEmpty())
            this.saveButton.disable();
        else
            this.saveButton.enable();
    },

    beforeShow: function() {
        Ext.each(this.getFooterSection(), function(item){
            item.show();
        });
        this.savingLabel.hide();
    },

    afterShow: function() {
        this.updateSaveButtonState();
    },

    getUsersFilter: function()
    {
        return new UsersFilterByAccessRight( this.accessRight );
    }

});

Ext.reg("at_users_selector_panel_one_time", AT.UserSelectorWithAccessRightForOneTimeSelectorPanel);