Ext.namespace("AT");

AT.UsersForApproveTTPageSelectorPanel = Ext.extend(AT.ThreePositionUserSelectorPanel, {
    groupBySettingCookieName: "approveTTPageUserSelectorGroupBySetting",
    collapsedCookiesKey: "approveTTPageUserSelector",
    allItemsOptionLabelKey: "approve_tt.users_selector.all_users",
    customOptionLabelKey: "approve_tt.users_selector.selected_users",
    showShowInactiveCheckbox: false,
    closeButtonLabelKey: "approve_tt.users_selector.apply_and_close.button.caption",
    cancelButtonLabelKey: "common.cancel",
    allItemsOptionValue: true,
    customOptionValue: false,

    initComponent: function ()
    {
        AT.UsersForApproveTTPageSelectorPanel.superclass.initComponent.call( this );

        if ( this.customOnChangeHandler )
            this.on( "change", this.customOnChangeHandler, this );
    },

    createFooterButtons: function() {
        this.resetButton = new AT.FlatButton({
                                                 icon: atImgPath("undo.gif"),
                                                 text: messageResource.getMessage(this.resetButtonLabelKey)
                                             });

        this.closeButton = new AT.GradientButton({
                                                     text: messageResource.getMessage(this.closeButtonLabelKey),
                                                     minWidth: 60,
                                                     cls: "greyButton close-button"
                                                 });
        
        this.cancelButton = new AT.GradientButton({
                                                      text: messageResource.getMessage(this.cancelButtonLabelKey),
                                                      minWidth: 60,
                                                      cls: "greyButton close-button cancel-button"
                                                  });
    },

    getFooterSection: function() {
        return [this.cancelButton, this.closeButton, this.resetButton];
    },

    saveCurrentState: function()
    {
        var selectedUsers = this.getSelectedUsers();
        this.setPredefinedParameters( {
                                        optionValue: selectedUsers.allUsers,
                                        selection: this.createUsersSelection(selectedUsers.selectedUserIds), 
                                        showInactive: false
                                      } );
        this.groupTypeInit = this.groupType;
    },

    canExpandCollapseNodes: function() {
        return this.userGroupsEnabled || this.timeZoneGroupsEnabled;
    },


    isSelectionChanged: function() {
        return this.selectorTree.isSelectionChanged();
    },

    resetState: function(isEnableActiveActions, doNotResetHideNonSelectedButton)
    {
        if (this.hideNonSelectedButtonChecked && !doNotResetHideNonSelectedButton)
        {
            this._toggleHideNonSelected();
        }
        AT.UsersForApproveTTPageSelectorPanel.superclass.resetState.call(this, isEnableActiveActions);
    },

    fullResetState: function(isEnableActiveActions, doNotResetHideNonSelectedButton)
    {
        this.beforeResetAction();
        this.resetState(isEnableActiveActions, doNotResetHideNonSelectedButton);
        this.afterResetAction();
    },

    loadState: function(state, doNotUpdateGroupType)
    {
        if (!doNotUpdateGroupType)
            this.setGroupType(state.groupType);
        this.setInitialOptionValue(this.getOptionValueByStateValue(state.optionValue));
        this.loadPermanentState(state);
        this.setSelection(state.selection);
        this.updateState();
    },

    initHideUserGroupsStrategy: function ()
    {
        return new HideUserGroupsStrategy();
    },

    initShowUserGroupsStrategy: function ()
    {
        return new ShowUserGroupsStrategy();
    },

    getSelectedUsers: function ()
    {
        return {
            allUsers: this.getStateValueByOptionValue(this.getCurrentOptionValue()),
            selectedUserIds: this.getSelectedUserIds()
        }
    },

    setAllUsersIfNoneSelected: function ()
    {
        if( this.getSelectedNodeSet().isEmpty() && this.isCustomOptionSelected() )
            this.selectOption(this.allItemsOptionValue);
    },

    getSelection: function(isFullFormed) {
        return this.userSelectorGroupsStrategy.makeSelectionFromNodeSet(this.getSelectedNodeSet(), isFullFormed, true, this.groupType);
    },

    getSelectedUserIds: function ()
    {
        return this.getSelection( false ).getSecondLevel();
    },

    collectCurrentSettings: function ()
    {
        return {
            allUsers: this.getStateValueByOptionValue(this.getCurrentOptionValue()),
            selectedUserIds: this.getSelectedUserIds(),
            searchString: this.filterWordsField ? this.filterWordsField.getSearchString() : null,
            hideNonSelectedButtonChecked: this.hideNonSelectedButtonChecked,
            optionValueInit: this.optionValueInit,
            initialSelection: this.userSelectorGroupsStrategy.makeSelectionFromNodeSet(
                this.selectorTree.getSelectionController().getInitiallySelectedNodeSet(), false, true)
        };
    },

    setSettingsBeforeTreeRebuilding: function ( settings )
    {
        this.loadState( {
                            optionValue: settings.allUsers,
                            selection: new TwoLevelSelection([], settings.selectedUserIds)
                        },
                        true);

        if (settings.hideNonSelectedButtonChecked)
            this.selectorFilter.setSelectedOnlyFromInitiallySelectedNodeSet();

        if (settings.searchString != null && settings.searchString.length > 0)
            this.selectorFilter.setSearchString( settings.searchString );
    },

    createSelectionFilters: function()
    {
        // do nothing
    },

    initialize: function (userGroupsEnabled, timeZoneGroupsEnabled, users, userGroups, timeZoneGroups, allUsers, selectedUserIds)
    {
        AT.UsersForApproveTTPageSelectorPanel.superclass.initialize.call(this, userGroupsEnabled, timeZoneGroupsEnabled, users, userGroups, timeZoneGroups);

        this.loadState({
                           optionValue: allUsers,
                           selection: this.createUsersSelection(selectedUserIds)
                       });
        if ( !allUsers && this.getSelectedNodeSet().isEmpty() )
            this.setInitialOptionValue(this.getOptionValueByStateValue(this.allItemsOptionValue));
    },

    getDescription: function(selectionHelper, dropdownButtonId) {
        return selectionHelper.getDescription(this.getSelection(true), dropdownButtonId);
    },

    createUsersSelection: function( selectedUserIds ) {
        return selectedUserIds == null || selectedUserIds.length == 0 ? TwoLevelSelection.emptySelection() : new TwoLevelSelection([], selectedUserIds);
    },

    makeSelectionHelper: function() {
        this.selectionHelper = new AT.SimpleUsersAndGroupsSelectionHelper({
                                                                              itemNounSingularKey: this.labelItemNounSingularKey,
                                                                              itemNounPluralKey: this.labelItemNounPluralKey,
                                                                              showInactiveCount: true,
                                                                              showSingleItemInactiveText: false,
                                                                              inactiveMultipleTextKey: this.labelDisabledMultipleKey,
                                                                              userSelectorGroupsStrategy: this.userSelectorGroupsStrategy,
                                                                              maxCharacters: this.labelMaxChars,
                                                                              pleaseSelectMessageKey: "approve_tt.users_selector.no_users_selected"
                                                                          });
    },

    createTree: function() {
        var tree = AT.UsersForApproveTTPageSelectorPanel.superclass.createTree.call(this);
        var selectionController = new TriStateSelectionController(tree);
        tree.setSelectionController(selectionController);
        return tree;
    },

    createEmptyMessageBuilder: function() {
        return new ApproveTTUsersSelectorEmptyMessageBuilder(this.userSelectorGroupsStrategy);
    },

    getFilterWordsPopupCoordinates: function()
    {
        if (this.userGroupsEnabled || this.timeZoneGroupsEnabled)
        {
            return AT.UsersForApproveTTPageSelectorPanel.superclass.getFilterWordsPopupCoordinates.apply(this, arguments);
        }
        else
        {
            return {
                x: -2,
                y: -4
            }
        }

    }

});