function TreeBuilder(rootNode, secondLevelEntities, firstLevelEntitiesMap, firstLevelEntitiesOrder)
{
    this.rootNode = rootNode;
    this.secondLevelEntities = secondLevelEntities;
    this.firstLevelEntitiesMap = firstLevelEntitiesMap;
    this.firstLevelEntitiesOrder = firstLevelEntitiesOrder;
    this.skipFirstLevel = !(this.firstLevelEntitiesMap && this.firstLevelEntitiesOrder);
    this.nodes = {};
    this.disabledNodesParents = {};
}

TreeBuilder.prototype = {
    buildTree: function()
    {
        // Add nodes for enabled second level entities
        _.each(this.secondLevelEntities, function(secondLevelEntity)
        {
            if (this.isSecondLevelEntityEnabled(secondLevelEntity))
            {
                var nodeForSecondLevelEntity = this.createTreeNodeForSecondLevelEntity(secondLevelEntity);
                var parentTreeNode = !this.skipFirstLevel ?
                                     this.getOrCreateTreeNodeForFirstLevelEntity(this.getSecondLevelEntityParentId(secondLevelEntity), true) :
                                     this.rootNode;
                parentTreeNode.appendChild(nodeForSecondLevelEntity);
                this.onTreeNodeForSecondLevelEntityAdded(nodeForSecondLevelEntity);
            }
        }, this);

        // Add nodes for disabled second level entities
        _.each(this.secondLevelEntities, function(secondLevelEntity)
        {
            if (!this.isSecondLevelEntityEnabled(secondLevelEntity))
            {
                var nodeForSecondLevelEntity = this.createTreeNodeForSecondLevelEntity(secondLevelEntity);
                // Node, which is normally a direct parent (for enabled second level entities) is a grandparent for disabled entities
                var grandParentTreeNode = !this.skipFirstLevel ?
                                          this.getOrCreateTreeNodeForFirstLevelEntity(this.getSecondLevelEntityParentId(secondLevelEntity), true) :
                                          this.rootNode;
                // Direct node parent is a special disabled nodes parent. Create it if not created before and append to grandparent
                var parentTreeNodeId = !this.skipFirstLevel ? grandParentTreeNode.getValue().id : 0;
                var parentTreeNode = this.disabledNodesParents[parentTreeNodeId];
                if (!parentTreeNode)
                {
                    parentTreeNode = this.createDisabledNodesParentForSecondLevelEntity(parentTreeNodeId);
                    this.disabledNodesParents[parentTreeNodeId] = parentTreeNode;
                    grandParentTreeNode.appendChild(parentTreeNode);
                    this.onDisabledNodesParentForSecondLevelEntitiesAdded(parentTreeNode);
                }

                parentTreeNode.appendChild(nodeForSecondLevelEntity);
                this.onTreeNodeForSecondLevelEntityAdded(nodeForSecondLevelEntity);
            }
        }, this);

        if (!this.skipFirstLevel)
        {
            // Add nodes for enabled first level entities
            _.each(this.firstLevelEntitiesOrder, function(firstLevelEntityId)
            {
                var firstLevelEntity = this.firstLevelEntitiesMap[firstLevelEntityId];
                if (this.isFirstLevelEntityEnabled(firstLevelEntity))
                {
                    var treeNodeForFirstLevelEntity = this.getOrCreateTreeNodeForFirstLevelEntity(firstLevelEntityId, this.createEmptyFirstLevelEntityNodes());
                    if (treeNodeForFirstLevelEntity)
                    {
                        this.rootNode.appendChild(treeNodeForFirstLevelEntity);
                        this.onTreeNodeForFirstLevelEntityAdded(treeNodeForFirstLevelEntity);
                    }
                }
            }, this);

            // Add nodes for disabled first level entities
            _.each(this.firstLevelEntitiesOrder, function(firstLevelEntityId)
            {
                var firstLevelEntity = this.firstLevelEntitiesMap[firstLevelEntityId];
                if (!this.isFirstLevelEntityEnabled(firstLevelEntity))
                {
                    var treeNodeForFirstLevelEntity = this.getOrCreateTreeNodeForFirstLevelEntity(firstLevelEntityId, this.createEmptyFirstLevelEntityNodes());
                    if (treeNodeForFirstLevelEntity)
                    {
                        if (!this.disabledNodesParentForFirstLevelEntities)
                        {
                            this.disabledNodesParentForFirstLevelEntities = this.createDisabledNodesParentForFirstLevelEntity(firstLevelEntity);
                            this.rootNode.appendChild(this.disabledNodesParentForFirstLevelEntities);
                            this.onDisabledNodesParentForFirstLevelEntitiesAdded(this.disabledNodesParentForFirstLevelEntities);
                        }

                        this.disabledNodesParentForFirstLevelEntities.appendChild(treeNodeForFirstLevelEntity);
                        this.onTreeNodeForFirstLevelEntityAdded(treeNodeForFirstLevelEntity);
                    }
                }
            }, this);
        }
    },

    getOrCreateTreeNodeForFirstLevelEntity: function(id, createNonExisting)
    {
        var treeNode = this.nodes[id];
        if (!treeNode && createNonExisting)
        {
            treeNode = this.createTreeNodeForFirstLevelEntity(this.firstLevelEntitiesMap[id]);
            treeNode.setLevel(1);
            this.nodes[id] = treeNode;
        }
        return treeNode;
    },

    createEmptyFirstLevelEntityNodes: function()
    {
        return false
    },

    createTreeNodeForFirstLevelEntity: function(firstLevelEntity)
    {
        // Abstract
    },

    createTreeNodeForSecondLevelEntity: function(secondLevelEntity)
    {
        // Abstract
    },

    createDisabledNodesParentForFirstLevelEntity: function()
    {
        // Abstract
    },

    createDisabledNodesParentForSecondLevelEntity: function(parentId)
    {
        // Abstract
    },

    getSecondLevelEntityParentId: function(secondLevelEntity)
    {
        // Abstract
    },

    isFirstLevelEntityEnabled: function(firstLevelEntity)
    {
        // Abstract
    },

    isSecondLevelEntityEnabled: function(secondLevelEntity)
    {
        // Abstract
    },

    onTreeNodeForFirstLevelEntityAdded: function(treeNode)
    {
        // Abstract
    },

    onTreeNodeForSecondLevelEntityAdded: function(treeNode)
    {
        // Abstract
    },

    onDisabledNodesParentForFirstLevelEntitiesAdded: function(treeNode)
    {
        // Abstract
    },

    onDisabledNodesParentForSecondLevelEntitiesAdded: function(treeNode)
    {
        // Abstract
    }
};