/**
 * Compare two User objects
 * @param a first user
 * @param b second user
 */
function userComparator( a, b )
{
    var result = stringComparator( a.lastName, b.lastName );
    if( result == 0 )
    {
        result = stringComparator( a.firstName, b.firstName );
        if( result == 0 )
        {
            var middleName1 = _.isNull(a.middleName) ? "" : a.middleName;
            var middleName2 = _.isNull(b.middleName) ? "" : b.middleName;
            
            result = stringComparator(middleName1, middleName2);
            if( result == 0 )
            {
                result = stringComparator( a.userName, b.userName );
            }
        }
    }
    return result;
}

/**
 * Save users selection from user selector to form
 * @param usersSelector user selector
 * @param form html form
 */
function saveReportUsers( usersSelector, form )
{
    var allUsers = !usersSelector.isCustomOptionSelected();
    form.allUsers.value = allUsers;
    if( !allUsers )
    {
        var selection = usersSelector.getSelection();
        if( selection.isEmpty() )
        {
            return false;
        }
        form.userGroupsString.value = selection.getFirstLevel().join();
        form.usersString.value = selection.getSecondLevel().join();
    }
    form.showDisabledUsers.value = usersSelector.getShowInactive();
    return true;
}

function buildUserSelectorTreeWithGroupsData( users, groupsData, rootNode, observer, filter, getGroupIdFromUserCallback )
{
    if (filter)
        users = _.filter(users, function(user) { return filter.matches(user); });

    var treeBuilder = new UserSelectorTreeBuilder(
            rootNode,
            users,
            groupsData ? groupsData.map : null,
            groupsData ? groupsData.order : null,
            observer,
            getGroupIdFromUserCallback);
    treeBuilder.buildTree();
}

function buildUserSelectorTreeWithUserGroupsData( users, userGroupsData, rootNode, observer, filter )
{
    buildUserSelectorTreeWithGroupsData(users, userGroupsData, rootNode, observer, filter,
                                        function( user ) { return user.userGroupId; });
}

function buildUserSelectorTreeWithTimeZoneGroupsData( users, timeZoneGroupsData, rootNode, observer, filter )
{
    buildUserSelectorTreeWithGroupsData(users, timeZoneGroupsData, rootNode, observer, filter,
                                        function( user ) { return user.timeZoneGroupId });
}

function getShareReportsButtonMenuMessages()
{
    return {
        dialogHeader: "report.common.user_selector.copy_report_config_to",
        buttonText: "report.common.user_selector.copy_report_config_to_selected_users",
        processText: "report.common.user_selector.copying_report_config_to_selected_users"
    };
}

function getShareChartsButtonMenuMessages()
{
    return {
        dialogHeader: "report.common.user_selector.copy_chart_config_to",
        buttonText: "report.common.user_selector.copy_chart_config_to_selected_users",
        processText: "report.common.user_selector.copying_chart_config_to_selected_users"
    };
}

function getShareInvoiceButtonMenuMessages()
{
    return {
        dialogHeader: "report.common.user_selector.copy_invoice_config_to",
        buttonText: "report.common.user_selector.copy_invoice_config_to_selected_users",
        processText: "report.common.user_selector.copying_invoice_config_to_selected_users"
    };
}


function buildShareReportSelectorMenu( users, userGroupsData, timeZoneGroupsData, messages, accessRight )
{
    var panel = new AT.UserSelectorWithAccessRightForOneTimeSelectorPanel({
        messages: messages,
        itemLabelMaxLength: 45,
        userSelectorGroupsStrategy: getUserSelectorFeatureStrategy(),
        accessRight: accessRight,
        showShowInactiveCheckbox: false
    });

    panel.initialize(EnabledFeatures.UGR, EnabledFeatures.TZG, users, userGroupsData, timeZoneGroupsData);

    return new AT.SharedUserButtonMenu({ panel: panel });
}

/*
 Intended to show tooltips with cause why user can not receive notification.
 */
function NoNotificationMessageParams( accessRight )
{
    this.accessRight = accessRight;
    this.disabledMessageKey = "mail_notifications.users_selector.no_notification.disabled";
    this.accessRightMessageKey = "mail_notifications.users_selector.no_notification.access_right";
    this.noEmailMessageKey = "mail_notifications.users_selector.no_notification.no_email";
}

NoNotificationMessageParams.prototype.getDisabledMessage = function ( user )
{
    return messageResource.getMessage( this.disabledMessageKey, [user.formatNameFML()] );
};

NoNotificationMessageParams.prototype.getAccessRightMessage = function ( user )
{
    return (this.accessRight) ? messageResource.getMessage( this.accessRightMessageKey, [user.formatNameFML(), this.accessRight.name] ) : null;
};

NoNotificationMessageParams.prototype.getNoEmailMessage = function ( user )
{
    return messageResource.getMessage( this.noEmailMessageKey, [user.formatNameFML()] );
};

function getUserSelectorFeatureStrategy()
{
    return EnabledFeatures.TZG || EnabledFeatures.UGR  ? new ShowUserGroupsStrategy() : new HideUserGroupsStrategy();
}