/**
 * Base Error Handler for AjaxUserAction
 * @param errorCode
 * @param [errorMessage]
 * @param [customErrorHandler] function (errorCode, errorMessage) error callback function.
 *              customErrorHandler is called before base handler.
 *              If customErrorHandler returns true, built-in handler will not be called.
 * @param [unknownErrorCodeHandler] function (errorCode, errorMessage) error callback function.
 *              unknownErrorCodeHandler is called if errorCode has no special handling in base handler.
 *              If unknownErrorCodeHandler is not specified, errorCode will be handled as unexpected error.
 */
function handleAjaxUserActionError( errorCode, errorMessage, customErrorHandler, unknownErrorCodeHandler )
{
    function showAlertAndReloadPage( message )
    {
        alert( message );
        document.location.reload();
    }

    if (!customErrorHandler)
        customErrorHandler = function(){};

    if ( !customErrorHandler( errorCode, errorMessage ) )
    {
        switch ( errorCode )
        {
            case AJAX_USER_ACTION_ERROR_CODES.NO_ACTIVE_LICENSE:
                showAlertAndReloadPage( AJAX_USER_ACTION_ERROR_MESSAGES.NO_ACTIVE_LICENSE );
                break;

            case AJAX_USER_ACTION_ERROR_CODES.LICENSE_VIOLATION:
                showAlertAndReloadPage( AJAX_USER_ACTION_ERROR_MESSAGES.LICENSE_VIOLATION );
                break;

            case AJAX_USER_ACTION_ERROR_CODES.SESSION_EXPIRED:
                showAlertAndReloadPage( JQUERY_RPC_ERROR_MESSAGES.SESSION_EXPIRED );
                break;

            case AJAX_USER_ACTION_ERROR_CODES.NO_SUCH_USER:
                showAlertAndReloadPage( AJAX_USER_ACTION_ERROR_MESSAGES.NO_SUCH_USER );
                break;

            case AJAX_USER_ACTION_ERROR_CODES.USER_IS_DISABLED:
                showAlertAndReloadPage( AJAX_USER_ACTION_ERROR_MESSAGES.USER_IS_DISABLED );
                break;

            case AJAX_USER_ACTION_ERROR_CODES.NO_RIGHTS:
                showAlertAndReloadPage( JQUERY_RPC_ERROR_MESSAGES.NO_RIGHTS );
                break;

            case AJAX_USER_ACTION_ERROR_CODES.PRODUCT_MISMATCH:
                showAlertAndReloadPage( JQUERY_RPC_ERROR_MESSAGES.SESSION_EXPIRED );
                break;

            case AJAX_USER_ACTION_ERROR_CODES.INCORRECT_CHALLENGE_RESPONSE:
                showAlertAndReloadPage( JQUERY_RPC_ERROR_MESSAGES.SESSION_EXPIRED );
                break;

            case AJAX_USER_ACTION_ERROR_CODES.ERROR_STATE:
            case AJAX_USER_ACTION_ERROR_CODES.UNEXPECTED_ERROR:
                alert( AJAX_USER_ACTION_ERROR_MESSAGES.UNEXPECTED_ERROR );
                if( errorMessage )
                    console.log( errorMessage );
                break;

            default:
                if ( unknownErrorCodeHandler )
                    unknownErrorCodeHandler( errorCode, errorMessage );
                else
                    alert( AJAX_USER_ACTION_ERROR_MESSAGES.UNEXPECTED_ERROR );
                break;
        }
    }
}