/**
 * @author evdokimov
 */

function _UnsupportedHistoryControllerState () {}
_UnsupportedHistoryControllerState.prototype.saveState = function ( state, url ) {};
_UnsupportedHistoryControllerState.prototype.updateState = function ( state, url ) {};
_UnsupportedHistoryControllerState.prototype.getState = function () {
    return null;
};


function _HistoryControllerState () {}

_HistoryControllerState.prototype.init = function ( onPopstate ) {
    window.addEventListener('popstate', onPopstate, true);
    return this;
};

_HistoryControllerState.prototype.saveState = function ( state, url ) {
    if ( !!url )
        window.history.pushState( state, "", url );
    else
        window.history.pushState( state, "" );   // ie 11 convert url from undefined to "undefined"
};

_HistoryControllerState.prototype.updateState = function ( state, url ) {
    if ( !!url )
        window.history.replaceState(state, "", url);
    else
        window.history.replaceState(state, "");   // ie 11 convert url from undefined to "undefined"
};

_HistoryControllerState.prototype.getState = function () {
    return window.history.state;
};


function HistoryController () {}

HistoryController.prototype.init = function ( config ) {
    this.state = this.isSupported() ? new _HistoryControllerState().init(config.onPopstate) : new _UnsupportedHistoryControllerState();

    if ( !this.getState() )
        this.updateState(config.initState);

    return this;
};

HistoryController.prototype.isSupported = function () {
    return !!window.history.pushState;
};

HistoryController.prototype.saveState = function ( state ) {
    this.state.saveState( state, this.url );
};

HistoryController.prototype.updateState = function ( state ) {
    this.state.updateState(state, this.url);
};

HistoryController.prototype.setUrlParam = function ( state, urlParam ) {
    this.url = window.location.pathname + urlParam;
    this.updateState(state);
};

HistoryController.prototype.getState = function () {
    if ( !this.url )
        this.url = window.location.href;

    var result = this.state.getState();
    if ( !!result )
        this.updateState(result);

    return result;
};

