
var contentLoadingPopup = null;

/**
 * Loads popup content and inserts it into div.
 * Can show a loading popup with spinner while a request is processed.
 *
 * @param pageUrl url of a page with popup content
 * @param [loadCompleteCallback] callback which will be called after successful loading
 * @param [showLoadingPopup] if true then a loading popup with spinner will be shown while a request is processed (false by default).
 * @param [customErrorHandler] if present, will be called before standard error handler. Should return true if the error is fully handled and no more action is needed.
 *                              If any other value is returned or if there's no error handler provided at all, the built-in handler is invoked.
 */
function loadPopupContent( pageUrl, loadCompleteCallback, showLoadingPopup, customErrorHandler, noDelayOnOpen )
{
    var placeholderDiv = $("#popupContentLoadingPlaceholder");
    deleteLoadedPopupContent();

    if ( showLoadingPopup )
    {
        if ( !contentLoadingPopup )
            contentLoadingPopup = new LoadingPopup().init( "contentLoadingPopup" );
        contentLoadingPopup.setOnEscapeCallback( function ()
                                                 {
                                                    if ( loadPopupRequestObject )
                                                        loadPopupRequestObject.abort();
                                                 });
        contentLoadingPopup.show();
    }

    function errorHandler( jqXHR, textStatus )
    {
        if ( showLoadingPopup )
            contentLoadingPopup.hide();

        var unknownErrorHandler = function()
        {
            if( jqXHR.status >= 100 && jqXHR.status < 1000 )
                alert( JQUERY_RPC_ERROR_MESSAGES.HTTP_ERROR + " " + jqXHR.status + " " + jqXHR.statusText );
            else
                alert( JQUERY_RPC_ERROR_MESSAGES.NETWORK_ERROR );
        };

        switch ( textStatus )
        {
            case 'abort':
                break;  // the request was aborted by the calling code; do nothing

            case 'timeout':
                alert( JQUERY_RPC_ERROR_MESSAGES.TIMEOUT );
                break;

            case 'error':
                handleAjaxUserActionError( jqXHR.status, jqXHR.responseText, customErrorHandler, unknownErrorHandler );
                break;

            default:
                alert( AJAX_USER_ACTION_ERROR_MESSAGES.UNEXPECTED_ERROR );
        }
    }

    function successHandler( data )
    {
        placeholderDiv.html( data );

        var completeFunction = function() {
            if ( showLoadingPopup )
                contentLoadingPopup.hide();

            if ( loadCompleteCallback )
                loadCompleteCallback();
        };

        if ( !noDelayOnOpen )
            // wait for loading css files
            window.setTimeout( completeFunction, 200);
        else
            completeFunction();
    }

    $.ajaxSetup({
                    cache: true
                });

    var loadPopupRequestObject = $.ajax({
               url: pageUrl,
               success: successHandler,
               error: errorHandler,
               timeout: 120000
           });

}

function deleteLoadedPopupContent()
{
    $( "#popupContentLoadingPlaceholder" ).empty();
}