function TabCycle(){}

/**
 * Initialize fields of TabCycle instance
 */

TabCycle.prototype.init = function()
{
    this.enabled = false;
    return this;
};

/**
 * Create sequence of focus jumps on pressing TAB and SHIFT + TAB keys.
 * On pressing TAB key sequence is $from -> $element -> $to.
 * On pressing TAB + SHIFT keys sequence is $from -> $element -> $to.
 * @param $from jQuery object
 * @param $element jQuery object
 * @param $to jQuery object
 */
TabCycle.prototype.createSequence = function($from, $element, $to)
{
    this._addListener($from, $element, true);
    this._addListener($element, $to, true);
    this._addListener($element, $from, false);
    this._addListener($to, $element, false);
};

/**
 * Create sequence of focus jumps on pressing TAB and SHIFT + TAB keys.
 * On pressing TAB key sequence is $firstEl -> ... -> $lastEl -> $firstEl.
 * On pressing TAB + SHIFT keys sequence is $lastEl -> ... -> $firstEl -> $lastEl.
 * @param $firstEl jQuery object
 * @param $lastEl jQuery object
 */
TabCycle.prototype.closeCycle = function($firstEl, $lastEl)
{
    this._addListener($firstEl, $lastEl, false);
    this._addListener($lastEl, $firstEl, true);
};

/**
 * Create a focus jump from $elem to $focusToEl
 * @param $elem jQuery object
 * @param $focusToEl jQuery object
 * @param isForwardDirection if isForwardDirection equals true, the jump will be on pressing TAB key, otherwise on pressing SHIFT + TAB
 * @private
 */
TabCycle.prototype._addListener = function($elem, $focusToEl, isForwardDirection)
{
    var self = this;
    $elem.keydown(function(event)
                  {
                      event = event || window.event;
                      if (self.enabled && event.which === KEY_CODE_TAB && event.shiftKey != isForwardDirection)
                      {
                          $focusToEl.focus();
                          event.preventDefault ? event.preventDefault() : (event.returnValue = false);
                      }
                  });
};

/**
 * Activate all created focus jumps
 */
TabCycle.prototype.enable = function()
{
    this.enabled = true;
};

/**
 * Deactivate all created focus jumps
 */
TabCycle.prototype.disable = function()
{
    this.enabled = false;
};

/**
 * Toggle enable status of all created focus jumps
 */
TabCycle.prototype.toggle = function()
{
    this.enabled = !this.enabled;
};