TransformedScreenMoire = {
    elemID: "transformedMoireId",
    elemClass: "transformedMoireClass",

    /**
     * Show moire with specific opacity on screen
     * @param initOpacity opacity (0-1)
     * @param additionalClass additional classes for moire element
     */
    show: function(initOpacity, additionalClass)
    {
        var $oldMoire = this.getMoire();
        if ($oldMoire.length != 0)
            $oldMoire.remove();

        $("<div>")
                .attr("id", this.elemID)
                .addClass(this.elemClass)
                .addClass(additionalClass)
                .css("opacity", initOpacity)
                .on("mousedown click dbclick ", function(e) {e.preventDefault()})
                .appendTo("body");
    },

    /**
     * Hide moire and delete from DOM
     * @param timeCss time for hiding
     * @param transitionTimingFunction timing function for hiding
     * @param moireId (optional) - if presents - perform action on moire found by id
     * @param cb (optional) - animation callback
     */
    hide: function(timeCss, transitionTimingFunction, moireId, cb)
    {
        var $moire = this.getMoire(moireId);
        if ($moire.length == 0)
            return;

        this.transformTo(0, timeCss, transitionTimingFunction, moireId, function()
        {
            $moire.remove();
            $("body").focus();
            if (cb) cb();
        });
    },

    /**
     * Transform moire opacity to new value with specific time and timeng function
     * @param newOpacity value of new opacity 0 - 1
     * @param timeCSS time if transformation (1s 200ms etc)
     * @param transitionTimingFunction linear, ease-in-out etc
     * @param moireId (optional) - if presents - perform action on moire found by id
     * @param cb (optional) - animation callback
     */
    transformTo: function(newOpacity, timeCSS, transitionTimingFunction, moireId, cb)
    {
        transitionTimingFunction = transitionTimingFunction || "linear";
        timeCSS = timeCSS || "0.1s";
        var moire = this.getMoire(moireId);
        if (cb)
            moire.atAnimation().oneTransitionEnd(cb);

        moire.css(AtAnimation.getVendorPrefixedCssName("transition"), "opacity " + timeCSS + " " + transitionTimingFunction);
        moire.css("opacity");
        moire.css("opacity", newOpacity);
    },

    /**
     * Show moire and immediately start the transition (see "show" and "transformTo" methods for params reference)
     */
    showAndTransformTo: function(additionalClass, newOpacity, timeCSS, transitionTimingFunction, moireId, cb)
    {
        // Initial opacity is made bigger than 0, because otherwise there is a chance of 
        // transition callback not firing (if no transition happened, because we ask to change opacity from 0 to 0).
        this.show(0.01, additionalClass);
        this.transformTo(newOpacity, timeCSS, transitionTimingFunction, moireId, cb);
    },

    /**
     * Get jquery element of current moire on page if exists
     * @returns {*|jQuery|HTMLElement}
     */
    getMoire: function(moireId)
    {
        moireId = moireId || this.elemID;
        return $("#" + moireId);
    }
};
