var YouTubeVideoPlayerHelper = {
    loadPlayerApiTimeout: 10000,

    showYTPlayer: function(videoPlaceholderId, videoId, onPlayerReady, onErrorShowPlayer, additionalPlayerVars)
    {
        this.onErrorShowPlayerCb = onErrorShowPlayer || function(){};
        this.onPlayerReadyCb = onPlayerReady || function(){};

        var videoWidth = 853;
        var videoHeight = 480;
        this.videoId = videoId;

        this.$loadPlayerProgressContainer = $("<div class='loadVideoPlayerProgressContainer'></div>")
                .css("width", videoWidth)
                .css("height", videoHeight)
                .appendTo($("#" + videoPlaceholderId).parent());

        this._showProgress(); // show progress while player loaded

        var context = this;
        // construct global function that fires after load api (call by player api)
        // don't change name of this function! see(https://developers.google.com/youtube/iframe_api_reference)
        window.onYouTubeIframeAPIReady = function()
        {
            context.playerApiIsLoaded = true;

            // check if api loaded after timeout
            if (context.apiLoadingTimeoutError)
            {
                return; //return if video loads after timeout
            }
            else
            {
                // reset api load timer for prevent unnecessary error message showing
                if(context.loadingAPITimer)
                {
                    clearTimeout(context.loadingAPITimer);
                }
            }

            // options of player see(https://developers.google.com/youtube/player_parameters)
            var playerVars =
            {
                rel: 0,
                controls: 2,
                showinfo: 0
                //origin : www.actitime.com its maybe needed for safety
            };
            $.extend(playerVars, additionalPlayerVars);

            var player = new YT.Player(
                    videoPlaceholderId,
                    {
                        videoId: context.videoId,
                        width: videoWidth,
                        height: videoHeight,
                        playerVars: playerVars,
                        events: {
                            'onReady': function()
                            {
                                context._onPlayerReady(player);
                            },
                            'onError': function(event)
                            {
                                console.error("Can not load video.Error code: " + event.data);
                                context._onErrorAction();
                            }
                        }
                    }
            );
            if (isIE8()) // hack for ie8 because 'onReady' event not fires
            {
                setTimeout(
                        function()
                        {
                            context._onPlayerReady(player);
                        },
                        1000
                );
            }

        };

        this._loadPlayerApi();
    },


    /**
     * Lazy load of youtube player api (perform only once on page !important)
     * @private
     */
    _loadPlayerApi: function()
    {
        if(this.playerApiIsLoaded)
        {
            onYouTubeIframeAPIReady();
        }
        else
        {
            var tag = document.createElement('script');
            tag.src = "https://www.youtube.com/iframe_api"; // get js api
            var firstScriptTag = document.getElementsByTagName('script')[0];
            firstScriptTag.parentNode.insertBefore(tag, firstScriptTag);

            this.apiLoadingTimeoutError = false;
            var context = this;
            this.loadingAPITimer = setTimeout(
                    function()
                    {
                        context.apiLoadingTimeoutError = true;
                        console.error("Cannot load player api");
                        context._onErrorAction();
                    },
                    this.loadPlayerApiTimeout
            );
        }
    },

    /**
     * Show progress while player loaded
     * @private
     */
    _showProgress: function()
    {
        this.spinner = new Spinner(
                {
                    lines: 12, // The number of lines to draw
                    length: 8, // The length of each line
                    width: 5, // The line thickness
                    radius: 14, // The radius of the inner circle
                    corners: 1, // Corner roundness (0..1)
                    rotate: 0, // The rotation offset
                    direction: 1, // 1: clockwise, -1: counterclockwise
                    color: '#ffffff', // #rgb or #rrggbb
                    speed: 1, // Rounds per second
                    trail: 60, // Afterglow percentage
                    shadow: false, // Whether to render a shadow
                    hwaccel: false, // Whether to use hardware acceleration
                    className: 'spinner', // The CSS class to assign to the spinner
                    zIndex: 2e9, // The z-index (defaults to 2000000000)
                    top: 'auto', // Top position relative to parent in px
                    left: 'auto' // Left position relative to parent in px
                }
        );
        this.spinner.spin();
        this.$loadPlayerProgressContainer
                .append(this.spinner.el)
                .append("<div class='videoLoading videoPlayerProgressText'>" + messageResource.getMessage("video_loading_progress.loading") + "</div>");
    },

    /**
     * Show player and call custom callback
     * @param player js youtube player
     * @private
     */
    _onPlayerReady: function(player)
    {
        this.onPlayerReadyCb(player);
        
        this.spinner.stop();
        this.$loadPlayerProgressContainer.hide();
    },

    /**
     * Show error message with link video and call custom error callback
     * @private
     */
    _onErrorAction: function()
    {
        this.onErrorShowPlayerCb();
        
        var videoExternalLink = "https://youtu.be/" + this.videoId;
        this.$loadPlayerProgressContainer
                .empty()
                .append("<div class='errorHeaderText videoPlayerProgressText'>" + messageResource.getMessage("video_loading_progress.error_loading.header") + "</div>")
                .append("<div class='errorHintText videoPlayerProgressText'>" + messageResource.getMessage("video_loading_progress.error_loading.hint") + "</div>")
                .append("<div class='errorLink videoPlayerProgressText'><a href='" + videoExternalLink + "' target='_blank'>" + videoExternalLink + "</a></div>")
    }
};