/**
 * Returns result of jQuery .eq() function or .last() if the index is out of bounds.
 * @param index Index for .eq()
 */
$.fn.eqOrLast = function(index)
{
    return this.length > index ? this.eq(index) : this.last();
};

/**
 * Returns animation utils wrapper of jQuery element.
 * @returns {AtAnimationJqueryPlugin}
 */
$.fn.atAnimation = function()
{
    return new AtAnimationJqueryPlugin(this);
};

/**
 * Subscribes to resize event just like .resize(), but event doesn't fire more often then specified.
 * @param f Event handler.
 * @param delay Throttle delay in ms.
 */
$.fn.resizeThrottle = function(f, delay)
{
    var timer;
    this.resize(function()
                {
                    clearTimeout(timer);
                    timer = setTimeout(f, delay);
                });
};

/**
 * Creates ViewStateSwitcher and binds it to selected DOM element.
 * If executed without params, returns existing state switcher.
 * @param config ViewStateSwitcher config.
 */
$.fn.queueViewStateSwitcher = function(config)
{
    var switcher;
    if (arguments.length == 0)
    {
        switcher = this[0]._queueViewStateSwitcher;
    }
    else
    {
        switcher = new QueueViewStateSwitcher(this, config);
        this[0]._queueViewStateSwitcher = switcher;
    }

    return switcher;
};

/**
 * Sets CSS left attribute in pixels.
 */
$.fn.cssLeft = function(size)
{
    return this.css("left", px(size));
};

/**
 * Sets CSS right attribute in pixels.
 */
$.fn.cssRight = function(size)
{
    return this.css("right", px(size));
};

/**
 * Sets CSS top attribute in pixels.
 */
$.fn.cssTop = function(size)
{
    return this.css("top", px(size));
};

/**
 * Sets CSS bottom attribute in pixels.
 */
$.fn.cssBottom = function(size)
{
    return this.css("bottom", px(size));
};

/**
 * Sets CSS width attribute in pixels.
 */
$.fn.cssWidth = function(size)
{
    return this.css("width", px(size));
};

/**
 * Sets CSS height attribute in pixels.
 */
$.fn.cssHeight = function(size)
{
    return this.css("height", px(size));
};

/**
 * Adds event handler, which fires when text input value is changed.
 * @param cb Event handler callback.
 */
$.fn.valueChanged = function(cb)
{
    // We use pVal() function to get the value, because the field might be under control
    // of jquery.placeholder library in old browsers
    this[0].prevValue = this.pVal();

    // We listen to many different events, which would trigger the check
    this.on("keyup paste cut blur", function(event)
    {
        // We only fire when the value is really changed
        var currentValue = $(this).pVal();
        if ((!this.prevValue && !!currentValue) || this.prevValue !== currentValue)
        {
            cb.call(this, event);
            // Remember new value. This also prevents from firing more than once on single change
            this.prevValue = currentValue;
        }
    });
};

/**
 * Sets element height making its bottom border match the window's bottom border.
 */
$.fn.adjustHeightToWindowSize = function()
{
    this.height($(window).height() - this.offset().top);
};