function ViewCommentsMessages() {
    this.TIMEOUT_DESCRIPTION_LOADING = messageResource.getMessage("view_comments.timeout_comments_loading");
    this.ERROR_DESCRIPTION_LOADING = messageResource.getMessage("view_comments.error_comments_loading");
}

function ViewCommentsUtil()
{
    //used as key prefix for request caching
    this.entityType = "Task";
    this.VIEW_DESCRIPTION_POPUP_CAPTION =  messageResource.getMessage("view_comments.popup_caption");
}

ViewCommentsUtil.prototype.Messages = _.create(ViewCommentsMessages.prototype);

ViewCommentsUtil.prototype.checkServiceAvailable = function () {
    if (!jsonrpc.TaskCommentService) {
        /* service is unavailable, most likely we've been logged out */
        document.location.reload();
        return false;
    }

    return true;
};

ViewCommentsUtil.prototype.doLoadDescription = function(parameters, descriptionLoadedHandler, errorHandler)
{
    try {
        var self = this;
        if( !isRpcReady( ['TaskCommentService.getTimeCommentsForUser'] ) )
        {
            this.tid = window.setTimeout(
                    function() { self.doLoadDescription(parameters, descriptionLoadedHandler, errorHandler); },
                    200);
            return;
        }

        if( !this.checkServiceAvailable() )
            return;

        jsonrpc.TaskCommentService.getTimeCommentsForUser(descriptionLoadedHandler,
                                                                parameters.taskId, parameters.userId, parameters.dateStr);
    } catch (e) {
        errorHandler(e);
    }
};

function ViewCommentsParameters( taskId, userId, dateStr )
{
    this.taskId = taskId;
    this.userId = userId;
    this.dateStr = dateStr;
}

ViewCommentsParameters.prototype.toString = function()
{
    return [ this.taskId, this.userId, this.dateStr ].join(",");
};

/**
 * Displays view task comments popup for specified user/task/date.
 *
 * @param parameters - ViewCommentsParameters object (contains userId, taskId, date)
*/
function showViewCommentsPopup(parameters)
{
    if (isBusyState()) return;

    var offsetX = -100;
    var offsetY = 15;
    var viewCommentsUtil = new ViewCommentsUtil();

    descriptionPopup.hide();
    descriptionState = new ViewDescriptionState(parameters, null, null, viewCommentsUtil);

    // Get description for the entity with specified id
    var description = getDescription(viewCommentsUtil.entityType, parameters);
    if (description == null) {
        descriptionState.setState('LOADING');
        descriptionPopup.show(viewCommentsUtil.VIEW_DESCRIPTION_POPUP_CAPTION, messageResource.getMessage("common.popup.loading"), offsetX, offsetY);
        loadDescription(parameters, viewCommentsUtil);
    } else {
        descriptionState.setState('VIEW');
        descriptionPopup.show(viewCommentsUtil.VIEW_DESCRIPTION_POPUP_CAPTION, description, offsetX, offsetY);
    }
}
