-- insert hire date to existing users

alter table at_user add column hire_date date;
alter table at_user add column release_date date;

create table tmpus
(
    id int(3) unsigned not null,
    hire_date date
) engine=InnoDB;

insert
    into
        tmpus
    select
        user_id, min(ddd)
    from
    (
        select user_id, record_date ddd from tt_record ttr
        union
        select user_id, comment_date ddd from user_task_comment utc
    ) ggg
    group by user_id;

update
    at_user atu
    set
        hire_date = ( select hire_date from tmpus tu where tu.id = atu.id )
    where
        atu.overtime_tracking = true;

update at_user set hire_date = now() where overtime_tracking = true and hire_date is null;

drop table tmpus;



create table task2
(
  id int(5) not null,
  customer_id int(3) not null,
  project_id int(3) not null,
  create_timestamp datetime not null,
  completion_date date,
  last_tt_date date,
  name varchar(255) not null,
  name_lower varchar(255) not null,
  description text,
  deadline_date date,
  billing_type_id int(3) unsigned,
  constraint task2_customer_fk foreign key (customer_id) references customer(id),
  constraint task2_project_fk foreign key (project_id) references project(id),
  constraint task2_billtype_fk foreign key (billing_type_id) references billing_type(id)
)
  engine=InnoDB
  comment='Temprary table (for table Task).';


insert
  into task2
  select t.id, p.customer_id,
         t.project_id,
         t.create_timestamp, t.completion_date, t.last_tt_date,
         t.name, lower(t.name), t.description,
         t.deadline_date, t.billing_type_id
    from task t
         left join project p on t.project_id = p.id;

commit;


create unique index task2_id_ui
  on task2 (id);

create unique index task2_project_name_ui
  on task2 (project_id, name_lower);

set foreign_key_checks=0;

drop table task;

set foreign_key_checks=1;


create table task
(
    id int(5) not null auto_increment primary key,
    customer_id int(3) not null,
    project_id int(3) not null,
    create_timestamp datetime not null,
    completion_date date,
    last_tt_date date,
    name varchar(255) not null,
    name_lower varchar(255) not null,
    description text,
    deadline_date date,
    billing_type_id int(3) unsigned,
    constraint task_customer_fk
        foreign key (customer_id)
            references customer(id) on update cascade,
    constraint task_project_fk
        foreign key (project_id)
            references project(id) on update cascade,
    constraint task_billtype_fk
        foreign key (billing_type_id)
            references billing_type(id) on update cascade
)
    engine=InnoDB
    default character set=utf8
    default collate=utf8_general_ci
    comment='Task information. One record relates one task.';


insert
  into task
  select *
    from task2;


commit;


create unique index task_project_name_ak
  on task (project_id, name_lower);

create index task_customer_name_i
  on task (customer_id, name_lower);


drop table task2;


create index project_customer_archts_i
  on project (customer_id, archiving_timestamp);

create index customer_archts_i
  on customer (archiving_timestamp);


-- as of now, we keep the pack's version in the database
alter table patch_history
  add column pack_version int(2) unsigned not null default 0;

alter table patch_history
  drop primary key;

alter table patch_history
  add primary key (patch, pack_version);

drop table feature_patch_history;


insert into
  patch_history (patch, pack_version, patch_timestamp)
  values (11, 1, now());

commit;
