-- create leave type icon table --

CREATE TABLE `leave_type_icon` (
  `id` int NOT NULL,
  `background_color` varchar(8),
  PRIMARY KEY (`id`)
);

insert into leave_type_icon(id, background_color) values(1, 'f2dede');
insert into leave_type_icon(id, background_color) values(2, null);


-- create leave type table --

CREATE TABLE `leave_type` (
 `id` counter NOT NULL,
 `name` varchar(80) NOT NULL,
 `name_lower` varchar(80) NOT NULL,
 `is_planned` bit NOT NULL,
 `is_active` bit NOT NULL,
 `icon_id` int NOT NULL,
 `whole_day_text` varchar(9),
 `position` int not null,
 PRIMARY KEY  (`id`),
 CONSTRAINT fk_leave_type_icon_id foreign key(icon_id) references leave_type_icon(id),
 CONSTRAINT leave_type_name unique (name_lower)
);

insert into leave_type (id, name, name_lower, is_planned, is_active, icon_id, whole_day_text, `position`)
    values (1, 'Sick Leave', 'sick leave', 1, 1, 1, 'sick day', 1);
insert into leave_type (id, name, name_lower, is_planned, is_active, icon_id, whole_day_text, `position`)
    values (2, 'Time Off', 'time off', 1, 1, 2, 'day off', 2);


CREATE TABLE `user_date_leave` (
  `user_id` int NOT NULL,
  `leave_date` date NOT NULL,
  `leave_type_id` int NOT NULL,
  `leave_duration` int NOT NULL,
  PRIMARY KEY (`user_id`, `leave_date`),
  CONSTRAINT `user_date_leave_ibfk_1` FOREIGN KEY (`user_id`) REFERENCES `at_user` (`id`),
  CONSTRAINT `user_date_leave_ibfk_2` FOREIGN KEY (`leave_type_id`) REFERENCES `leave_type` (`id`)
);


-- start updating users with empty workday --

create table tmp_at_user_empty_wday (
  id int not null,
  workday_duration int not null,
  primary key (id)
);

insert into tmp_at_user_empty_wday
  select id, property_value as workday_duration
    from at_user,
         system_settings
      where property_name = 'workday_duration'
        and (workday_duration is null or workday_duration = 0);

update at_user u
       inner join tmp_at_user_empty_wday t
         on t.id = u.id
  set u.workday_duration = t.workday_duration;

drop table tmp_at_user_empty_wday;

alter table at_user
  alter column workday_duration int not null;

-- end updating users with empty workday --

create table license
(
  id counter not null,
  issue_date datetime not null,
  body longtext,
  PRIMARY key(id)
);

create table report_configs
(
  id counter not null,
  name varchar(209) not null,
  name_lower varchar(209) not null,
  report_form int not null,
  constraint pk_report_configs primary key(id),
  constraint nr_report_configs unique(name_lower, report_form)
);

create table report_params
(
  config_id int not null,
  param_name varchar(255) not null,
  param_value varchar(255) not null,
  constraint fk_report_configs foreign key(config_id) references report_configs(id)
);

alter table project
    drop constraint fk_project_customer;
alter table project
    add constraint fk_project_customer foreign key(customer_id) references customer(id) on delete cascade;

alter table task
    drop constraint fk_task_project;
alter table task
    add constraint fk_task_project foreign key(project_id) references project(id) on delete cascade;

alter table tt_record
    drop constraint fk_tt_task;
alter table tt_record
    add constraint fk_tt_task foreign key(task_id) references task(id) on delete cascade;

alter table user_task_comment
    drop constraint fk_comment_taskid;
alter table user_task_comment
    add constraint fk_comment_taskid foreign key (task_id) references task(id) on delete cascade;

alter table user_task
    drop constraint fk_ut_task;
alter table user_task
    add constraint fk_ut_task foreign key(task_id) references task(id) on delete cascade;

update system_settings set property_value='yellowgreen' where property_name='color_scheme' and property_value='green';

insert into patch_history(patch, pack_version, patch_timestamp)
    values(12, 1, now());
