-- start updating users with empty workday --

create temporary table tmp_at_user_empty_wday as
  select id
    from at_user
      where workday_duration is null
         or workday_duration = 0;

update at_user
  set workday_duration =
    (select property_value
       from system_settings
       where property_name = 'workday_duration')
  where id in
    (select id
       from tmp_at_user_empty_wday);

drop temporary table tmp_at_user_empty_wday;

alter table at_user
  change workday_duration workday_duration int not null;
commit;
-- end updating users with empty workday --


CREATE TABLE `leave_type_icon` (
  `id` int(4) unsigned NOT NULL,
  `background_color` varchar(8),
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

insert into leave_type_icon(id, background_color) values(1, 'f2dede');
insert into leave_type_icon(id, background_color) values(2, null);
commit;


CREATE TABLE `leave_type` (
  `id` int(3) unsigned not null auto_increment,
  `name` varchar(80) not null,
  `name_lower` varchar(80) not null default '',
  `is_planned` bit not null default 0,
  `is_active` bit not null default 1,
  `icon_id` int(4) unsigned not null,
  `whole_day_text` varchar(9) default null,
  `position` int(3) unsigned not null,
  PRIMARY KEY  (`id`),
  unique key leave_type_name (name_lower),
  constraint fk_icon_id foreign key(icon_id) references leave_type_icon(id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

insert into leave_type (id, name, name_lower, is_planned, icon_id, whole_day_text, position)
    values (1, 'Sick Leave', 'sick leave', 1, 1, 'sick day', 1);
insert into leave_type (id, name, name_lower, is_planned, icon_id, whole_day_text, position)
    values (2, 'Time Off', 'time off', 1, 2, 'day off', 2);
commit;


CREATE TABLE `user_date_leave` (
  `user_id` int(3) unsigned NOT NULL,
  `leave_date` date NOT NULL,
  `leave_type_id` int(3) unsigned NOT NULL,
  `leave_duration` int(10) unsigned NOT NULL,
  PRIMARY KEY (`user_id`, `leave_date`),
  CONSTRAINT `user_date_leave_ibfk_1` FOREIGN KEY (`user_id`) REFERENCES `at_user` (`id`),
  CONSTRAINT `user_date_leave_ibfk_2` FOREIGN KEY (`leave_type_id`) REFERENCES `leave_type` (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

commit;

create table license
(
  id int(3) not null,
  issue_date datetime not null,
  body text,
  PRIMARY key(id)
);

create table report_configs
(
  id int(3) not null auto_increment,
  name varchar(209) not null,
  name_lower varchar(209) not null,
  report_form int not null,
  primary key(id),
  unique(name_lower, report_form)
) engine=InnoDB;

create table report_params
(
  config_id int(3) not null,
  param_name varchar(255) not null,
  param_value varchar(255) not null,
  index(config_id),
  foreign key(config_id) references report_configs(id)
) engine=InnoDB;

alter table project
    add constraint fk_project_customer foreign key(customer_id) references customer(id) on delete cascade;
alter table task
    add constraint fk_task_project foreign key(project_id) references project(id) on delete cascade;
alter table tt_record
    add constraint fk_tt_task foreign key(task_id) references task(id) on delete cascade;
alter table user_task_comment
    add constraint fk_comment_taskid foreign key (task_id) references task(id) on delete cascade;
alter table user_task
    add constraint fk_ut_task foreign key(task_id) references task(id) on delete cascade;

update system_settings set property_value='yellowgreen' where property_name='color_scheme' and property_value='green';

insert
  into patch_history (patch, pack_version, patch_timestamp)
  values (12, 2, now());

commit;