CREATE TABLE `lock_history` (
    `lock_date_from` date,
    `lock_date_to` date not null,
    `create_timestamp` date not null,
    `user_id` int not null,
    `description` longtext,
    CONSTRAINT `pk_lock_history` PRIMARY KEY(`lock_date_to`),
    CONSTRAINT `fk_lock_history_user` FOREIGN KEY(`user_id`) REFERENCES `at_user`(`id`)
    );

-- leave types

insert into leave_type_icon(id, background_color) values(3, null);
insert into leave_type_icon(id, background_color) values(4, null);
insert into leave_type_icon(id, background_color) values(5, null);
insert into leave_type_icon(id, background_color) values(6, null);
insert into leave_type_icon(id, background_color) values(7, null);
insert into leave_type_icon(id, background_color) values(8, null);
insert into leave_type_icon(id, background_color) values(9, null);
insert into leave_type_icon(id, background_color) values(10, 'f2dede');
insert into leave_type_icon(id, background_color) values(11, null);
insert into leave_type_icon(id, background_color) values(12, null);
insert into leave_type_icon(id, background_color) values(13, null);
insert into leave_type_icon(id, background_color) values(14, null);
insert into leave_type_icon(id, background_color) values(15, null);
insert into leave_type_icon(id, background_color) values(16, null);
insert into leave_type_icon(id, background_color) values(17, null);
insert into leave_type_icon(id, background_color) values(18, null);
insert into leave_type_icon(id, background_color) values(19, null);
insert into leave_type_icon(id, background_color) values(20, null);

insert into leave_type (id, name, name_lower, is_planned, is_active, icon_id, whole_day_text, `position`)
    values (3, 'Vacation', 'vacation', 1, 1, 3, 'day off', 3);


-- task estimates

alter table task add column budget int;

-- user-projects assignments

alter table at_user
  add all_projects_assigned bit not null;
      -- 0 means the system shows only projects explicitly assigned
      --         using the "user_projects" table;
      -- 1 means the system shows all projects not depends of the
      --         assignments in the "user_projects" table.


-- User-projects assignments. For a concrete users,
-- when the "all projects assigned" flag (in the "at_user" table)
-- is not set, these assignments determine which projects will
-- be shown to the user; when the "all projects assigned" is set
-- these assignments are holded in order to get possibility to restore
-- assignments when an admin will reset the flag mentioned above.
-- Note: the existent time track data don't depend on this table,
-- so it can be a case when a user has tracks in a project when
-- this project doesn't exist in his/her projects assignments.
-- So this relationship has "access rights" semantic.
create table user_project
  (
  user_id int not null,
  project_id int not null,
  constraint user_project_user_fk
    foreign key (user_id) references at_user (id),
  constraint user_project_pk
    primary key (user_id, project_id)
  );
alter table user_project
    add constraint user_project_project_fk
        foreign key (project_id) references project (id) on delete cascade;

create index user_project_project_i
  on user_project (project_id);


  -- Default user-project assignments: each user assigned to
  -- the projects meat one of the following conditions:
  --   a) projects which tasks are on the user's "enter TT" page,
  --   b) projects user has entered time track (even archived ones),
  --   c) projects with commented by this user tasks (even archived ones).
insert
  into user_project (user_id, project_id)
  select distinct ut.user_id, t.project_id
    from user_task ut,
         task t
    where t.id = ut.task_id;

  -- Initially the "all projects assigned" option have users
  -- that have at least one of the following rights:
  --  - Enter/Modify Time Track of Other Users (code 12),
  --  - Manage Clients, Projects & Tasks (code 2).
update at_user
  set all_projects_assigned = 1
  where id in ( select distinct user_id
                from user_access_right
                where access_right_id in (2,12) );

alter table leave_type
    add column rate_coefficient double not null;

update leave_type set rate_coefficient = 1;

create table user_rate
(
    id counter  not null  primary key,
    effective_date date not null,
    user_id int  not null,
    regular_rate double not null,
    overtime_rate double null,

    CONSTRAINT user_rate_id_fk_at_user_id FOREIGN KEY (user_id) REFERENCES at_user(id),
    CONSTRAINT user_id_effective_date_unique UNIQUE (effective_date, user_id)
);

create table leave_rate
(
    rate_id int not null,
    leave_type int not null,
    rate double not null,

    PRIMARY key(rate_id, leave_type),
    CONSTRAINT leave_rate_rate_id_fk_user_rate_id FOREIGN KEY (rate_id) REFERENCES user_rate(id)
);

ALTER TABLE leave_rate
   ADD CONSTRAINT leave_rate_leave_type_fk_leave_type_id FOREIGN KEY (leave_type) REFERENCES leave_type(id) ON DELETE CASCADE;

update access_right set name='reporting.generate_time_reports' where id = 1;
update access_right set name='reporting.generate_money_reports' where id = 8;

UPDATE billing_type
    SET is_default = 0 WHERE is_default IS NULL;

ALTER TABLE billing_type
    ALTER COLUMN is_default BIT NOT NULL;
ALTER TABLE billing_type
    ADD is_billable BIT NOT NULL;
ALTER TABLE billing_type
    ADD is_active BIT NOT NULL;
ALTER TABLE billing_type
    ADD `position` INT NULL;
ALTER TABLE billing_type
    ADD rate DOUBLE NULL;

UPDATE billing_type
    SET is_billable = 1, is_active = 1, rate = 0;

CREATE TABLE tempbt
(
	id INT NOT NULL,
	`position` COUNTER(2) NOT NULL
);

INSERT INTO tempbt
    SELECT id FROM billing_type ORDER BY name_lower ASC;

UPDATE (billing_type bt INNER JOIN tempbt tbt ON tbt.id = bt.id)
    SET bt.position = tbt.position;

DROP TABLE tempbt;

ALTER TABLE billing_type
    ALTER COLUMN `position` INT NOT NULL;

INSERT INTO billing_type
    (name, name_lower, `position`, is_billable, is_active) VALUES ('Non-Billable', 'non-billable', 1, 0, 1);

UPDATE task
    SET billing_type_id = DFirst("id", "billing_type", "is_billable = 0")
    WHERE billing_type_id IS NULL;

CREATE TABLE tempbt
(
    is_default BIT NOT NULL
);

INSERT INTO tempbt
    SELECT is_default FROM billing_type;

UPDATE billing_type
    SET is_default = 1
    WHERE is_billable = 0 AND NOT EXISTS (SELECT * FROM tempbt WHERE is_default = 1);

DROP TABLE tempbt;

ALTER TABLE task
    DROP CONSTRAINT fk_task_billing_type;

ALTER TABLE task
	ALTER COLUMN billing_type_id INT NOT NULL;

ALTER TABLE task
    ADD CONSTRAINT fk_task_billing_type FOREIGN KEY(billing_type_id) REFERENCES billing_type(id);

-- Invoices ------------------------------------------

create table invoice_bill_to_recent_entry
(
  entry             text  not null,
  normalized_entry  text  not null,
  `position`        int   not null,

  constraint uk_invoice_bill_to_recent_entry_position unique (`position`)
);

create table invoice_terms
(
  entry             varchar(255)   not null,
  normalized_entry  varchar(255)   not null,
  `position`        int            not null,

  constraint uk_invoice_terms_position unique (`position`)
);

insert into invoice_terms (entry, normalized_entry, `position`) values ('Net 60', 'net 60', 1);
insert into invoice_terms (entry, normalized_entry, `position`) values ('Net 30', 'net 30', 2);
insert into invoice_terms (entry, normalized_entry, `position`) values ('Net 15', 'net 15', 3);
insert into invoice_terms (entry, normalized_entry, `position`) values ('Net 10', 'net 10', 4);
insert into invoice_terms (entry, normalized_entry, `position`) values ('Net 7',  'net 7',  5);
insert into invoice_terms (entry, normalized_entry, `position`) values ('Due on receipt', 'due on receipt', 6);

create table invoice_tax
(
  `position` int           not null,
  `name`     text          not null,
  rate       double        null,
  is_percent bit           not null,

  constraint uk_invoice_tax_position unique (`position`)
);

create table invoice_discount
(
  `position` int   not null,
  `name`     text  not null,

  constraint uk_invoice_discount_position unique (`position`)
);

insert into invoice_tax (`position`, `name`, rate, is_percent) values (1, 'Tax', null, 0);
insert into invoice_discount (`position`, `name`) values (1, 'Discount');

create table invoice_params
(
  param_name  varchar(255)  not null,
  param_value text          not null,

  constraint uk_invoice_params_param_name unique (param_name)
);

insert into invoice_params (`param_name`, `param_value`) values ('title', 'INVOICE');
insert into invoice_params (`param_name`, `param_value`) values ('POLabel', 'PO#');
insert into invoice_params (`param_name`, `param_value`) values ('refCodeLabel', 'Reference Code');

-- ----------------------------------------------------



insert into patch_history(patch, pack_version, patch_timestamp)
    values(14, 0, now());
