alter table leave_type
    add column rate_coefficient double not null;

update leave_type set rate_coefficient = 1;

create table user_rate
(
    id counter  not null  primary key,
    effective_date date not null,
    user_id int  not null,
    regular_rate double not null,
    overtime_rate double null,

    CONSTRAINT user_rate_id_fk_at_user_id FOREIGN KEY (user_id) REFERENCES at_user(id),
    CONSTRAINT user_id_effective_date_unique UNIQUE (effective_date, user_id)
);

create table leave_rate
(
    rate_id int not null,
    leave_type int not null,
    rate double not null,

    PRIMARY key(rate_id, leave_type),
    CONSTRAINT leave_rate_rate_id_fk_user_rate_id FOREIGN KEY (rate_id) REFERENCES user_rate(id)
);

ALTER TABLE leave_rate
   ADD CONSTRAINT leave_rate_leave_type_fk_leave_type_id FOREIGN KEY (leave_type) REFERENCES leave_type(id) ON DELETE CASCADE;

update access_right set name='reporting.generate_time_reports' where id = 1;
update access_right set name='reporting.generate_money_reports' where id = 8;

UPDATE billing_type
    SET is_default = 0 WHERE is_default IS NULL;

ALTER TABLE billing_type
    ALTER COLUMN is_default BIT NOT NULL;
ALTER TABLE billing_type
    ADD is_billable BIT NOT NULL;
ALTER TABLE billing_type
    ADD is_active BIT NOT NULL;
ALTER TABLE billing_type
    ADD `position` INT NULL;
ALTER TABLE billing_type
    ADD rate DOUBLE NULL;

UPDATE billing_type
    SET is_billable = 1, is_active = 1, rate = 0;

CREATE TABLE tempbt
(
	id INT NOT NULL,
	`position` COUNTER(2) NOT NULL
);

INSERT INTO tempbt
    SELECT id FROM billing_type ORDER BY name_lower ASC;

UPDATE (billing_type bt INNER JOIN tempbt tbt ON tbt.id = bt.id)
    SET bt.position = tbt.position;

DROP TABLE tempbt;

ALTER TABLE billing_type
    ALTER COLUMN `position` INT NOT NULL;

INSERT INTO billing_type
    (name, name_lower, `position`, is_billable, is_active) VALUES ('Non-Billable', 'non-billable', 1, 0, 1);

UPDATE task
    SET billing_type_id = DFirst("id", "billing_type", "is_billable = 0")
    WHERE billing_type_id IS NULL;

CREATE TABLE tempbt
(
    is_default BIT NOT NULL
);

INSERT INTO tempbt
    SELECT is_default FROM billing_type;

UPDATE billing_type
    SET is_default = 1
    WHERE is_billable = 0 AND NOT EXISTS (SELECT * FROM tempbt WHERE is_default = 1);

DROP TABLE tempbt;

ALTER TABLE task
    DROP CONSTRAINT fk_task_billing_type;

ALTER TABLE task
	ALTER COLUMN billing_type_id INT NOT NULL;

ALTER TABLE task
    ADD CONSTRAINT fk_task_billing_type FOREIGN KEY(billing_type_id) REFERENCES billing_type(id);

-- Invoices ------------------------------------------

create table invoice_bill_to_recent_entry
(
  entry             text  not null,
  normalized_entry  text  not null,
  `position`        int   not null,

  constraint uk_invoice_bill_to_recent_entry_position unique (`position`)
);

create table invoice_terms
(
  entry             varchar(255)   not null,
  normalized_entry  varchar(255)   not null,
  `position`        int            not null,

  constraint uk_invoice_terms_position unique (`position`)
);

insert into invoice_terms (entry, normalized_entry, `position`) values ('Net 60', 'net 60', 1);
insert into invoice_terms (entry, normalized_entry, `position`) values ('Net 30', 'net 30', 2);
insert into invoice_terms (entry, normalized_entry, `position`) values ('Net 15', 'net 15', 3);
insert into invoice_terms (entry, normalized_entry, `position`) values ('Net 10', 'net 10', 4);
insert into invoice_terms (entry, normalized_entry, `position`) values ('Net 7',  'net 7',  5);
insert into invoice_terms (entry, normalized_entry, `position`) values ('Due on receipt', 'due on receipt', 6);

create table invoice_tax
(
  `position` int           not null,
  `name`     text          not null,
  rate       double        null,
  is_percent bit           not null,

  constraint uk_invoice_tax_position unique (`position`)
);

create table invoice_discount
(
  `position` int   not null,
  `name`     text  not null,

  constraint uk_invoice_discount_position unique (`position`)
);

insert into invoice_tax (`position`, `name`, rate, is_percent) values (1, 'Tax', null, 0);
insert into invoice_discount (`position`, `name`) values (1, 'Discount');

create table invoice_params
(
  param_name  varchar(255)  not null,
  param_value text          not null,

  constraint uk_invoice_params_param_name unique (param_name)
);

insert into invoice_params (`param_name`, `param_value`) values ('title', 'INVOICE');
insert into invoice_params (`param_name`, `param_value`) values ('POLabel', 'PO#');
insert into invoice_params (`param_name`, `param_value`) values ('refCodeLabel', 'Reference Code');

-- ----------------------------------------------------

insert into patch_history(patch, pack_version, patch_timestamp)
    values(14, 4, now());
