ALTER TABLE task CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci;

alter table leave_type
add column rate_coefficient double(10,2) unsigned not null default 1;

create table user_rate
(
    id int(5) unsigned not null auto_increment primary key,
    effective_date date not null,
    user_id int(3) unsigned not null,
    regular_rate double(10,2) unsigned not null,
    overtime_rate double(10,2) unsigned null,

    CONSTRAINT user_rate_id_fk_at_user_id FOREIGN KEY (user_id) REFERENCES at_user(id),
    CONSTRAINT user_id_effective_date_unique UNIQUE (effective_date, user_id)
)
  engine=InnoDB
  comment='Users rates information.';

create table leave_rate
(
    rate_id int(5) unsigned not null,
    leave_type int(3) unsigned not null,
    rate double(10,2) unsigned not null,

    PRIMARY key(rate_id, leave_type),
    CONSTRAINT leave_rate_rate_id_fk_user_rate_id FOREIGN KEY (rate_id) REFERENCES user_rate(id),
    CONSTRAINT leave_rate_leave_type_fk_leave_type_id FOREIGN KEY (leave_type) REFERENCES leave_type(id) ON DELETE CASCADE
)
  engine=InnoDB
  comment='Leave rates information.';

update access_right set name='reporting.generate_time_reports' where id = 1;
update access_right set name='reporting.generate_money_reports' where id = 8;

commit;

UPDATE billing_type
    SET is_default = 0 WHERE is_default IS NULL;

ALTER TABLE billing_type
	MODIFY COLUMN is_default TINYINT(1) NOT NULL DEFAULT 0,
	ADD COLUMN is_billable TINYINT(1) NOT NULL DEFAULT 1,
	ADD COLUMN is_active TINYINT(1) NOT NULL DEFAULT 1,
	ADD COLUMN position INT UNSIGNED NULL,
	ADD COLUMN rate DOUBLE(10,2) UNSIGNED NULL;

UPDATE billing_type
    SET rate = 0;

CREATE TABLE tempbt
(
	id INT NOT NULL,
	position INT AUTO_INCREMENT,
	KEY(position)
)
ENGINE=InnoDB;

INSERT INTO tempbt
    (id) VALUES (1);

DELETE FROM tempbt;

INSERT INTO tempbt
    SELECT id, position FROM billing_type ORDER BY name_lower ASC;

UPDATE billing_type bt
	SET position = (SELECT position FROM tempbt WHERE id = bt.id);

DROP TABLE tempbt;

ALTER TABLE billing_type
    MODIFY COLUMN position INT NOT NULL;

INSERT INTO billing_type
    (name, name_lower, position, is_billable) VALUES ('Non-Billable', 'non-billable', 1, 0);

CREATE TABLE tempbt
(
    is_default TINYINT(1) NOT NULL
);

INSERT INTO tempbt
    SELECT is_default FROM billing_type;

UPDATE billing_type
    SET is_default = 1
    WHERE is_billable = 0 AND NOT EXISTS (SELECT * FROM tempbt WHERE is_default = 1);

DROP TABLE tempbt;

UPDATE task
    SET billing_type_id = (SELECT id FROM billing_type WHERE is_billable = 0)
    WHERE billing_type_id IS NULL;

ALTER TABLE task
	MODIFY COLUMN billing_type_id INT(3) UNSIGNED NOT NULL;

-- Invoices ------------------------------------------

create table invoice_bill_to_recent_entry
(
  entry            text          not null,
  normalized_entry text          not null,
  position         int unsigned  not null comment 'the more recent, the larger number',

  constraint unique key uk_invoice_bill_to_recent_entry_position (position)
)
engine=InnoDB
default character set=utf8
default collate=utf8_general_ci;

create table invoice_terms
(
  entry            varchar(255)   not null,
  normalized_entry varchar(255)   not null,
  position         int            not null,

  constraint unique key uk_invoice_terms_position (position)
)
engine=InnoDB
default character set=utf8
default collate=utf8_general_ci;

insert into invoice_terms (entry, normalized_entry, position) values ('Net 60', 'net 60', 1);
insert into invoice_terms (entry, normalized_entry, position) values ('Net 30', 'net 30', 2);
insert into invoice_terms (entry, normalized_entry, position) values ('Net 15', 'net 15', 3);
insert into invoice_terms (entry, normalized_entry, position) values ('Net 10', 'net 10', 4);
insert into invoice_terms (entry, normalized_entry, position) values ('Net 7',  'net 7',  5);
insert into invoice_terms (entry, normalized_entry, position) values ('Due on receipt', 'due on receipt', 6);

create table invoice_tax
(
  position int          not null,
  name     text         not null,
  rate     double(12,4) null,
  is_percent bit        not null default 0,

  constraint unique key uk_invoice_tax_position (position)
)
engine=InnoDB
default character set=utf8
default collate=utf8_general_ci;

create table invoice_discount
(
  position  int    not null,
  name      text   not null,

  constraint unique key uk_invoice_discount_position (position)
)
engine=InnoDB
default character set=utf8
default collate=utf8_general_ci;

insert into invoice_tax (position, name, rate, is_percent) values (1, 'Tax', null, 0);
insert into invoice_discount (position, name) values (1, 'Discount');

create table invoice_params
(
  param_name  varchar(255) not null,
  param_value text         not null,

  constraint unique key uk_invoice_params_param_name (param_name)
)
engine=InnoDB
default character set=utf8
default collate=utf8_general_ci;

insert into invoice_params (param_name, param_value) values ('title', 'INVOICE');
insert into invoice_params (param_name, param_value) values ('POLabel', 'PO#');
insert into invoice_params (param_name, param_value) values ('refCodeLabel', 'Reference Code');
go

-- ----------------------------------------------------

insert
  into patch_history (patch, pack_version, patch_timestamp)
  values (14, 4, now());

commit;