-- add on delete cascade to rates

alter table user_rate
    drop constraint user_rate_id_fk_at_user_id;
alter table user_rate
    add CONSTRAINT user_rate_id_fk_at_user_id FOREIGN KEY (user_id) REFERENCES at_user(id) on delete cascade;

alter table leave_rate
    drop constraint leave_rate_rate_id_fk_user_rate_id;
alter table leave_rate
    add CONSTRAINT leave_rate_rate_id_fk_user_rate_id FOREIGN KEY (rate_id) REFERENCES user_rate(id) on delete cascade;

-- ----------------------------------------------------

-- Add Lock Time-Track by User functionality

-- Timestamps for time-track, needed to detect concurrent time-track changes.
-- Before applying this patch time-track is not timestamped. Here we don't add timestamps to the already existing
-- time-track. We will assume that the time-track without metadata was modified long ago in the past.

-- last_change_timestamp:
--   timestamp of the last TT changes for this user-date (either working time, leave time or overtime changes)

create table tt_metadata (

  user_id int not null,
  tt_date date not null,
  primary key (user_id, tt_date),
  constraint fk_tt_metadata_user_id foreign key (user_id) references at_user (id) on delete cascade,

  last_change_timestamp datetime not null

);


-- Time-track locking info storage.

-- date_from:
--   null means locked from the creation of the world

create table tt_lock (

  locked_user_id int not null,
  date_from date,
  date_to date not null,
  constraint tt_lock_locked_user_id foreign key (locked_user_id) references at_user (id) on delete cascade,

  when_locked datetime not null,
  who_locked_id int not null,
  constraint tt_lock_who_locked_id foreign key (who_locked_id) references at_user (id)

);


-- Convert old lock data.

insert into tt_lock (locked_user_id, date_from, date_to, when_locked, who_locked_id)
  select at_user.id, lock_date_from, lock_date_to, create_timestamp, lock_history.user_id
  from lock_history, at_user;


drop table lock_history;


-- Storage for users selection on LTT page

-- manager_user_id - the user who uses the LTT page
-- selected_user_id - the user which was selected by the manager

create table lock_tt_selected_users (

  manager_user_id  int not null,
  selected_user_id int not null,
  primary key (manager_user_id, selected_user_id),

  constraint lock_tt_selected_users_manager_user_id
    foreign key (manager_user_id)  references at_user (id) on delete cascade,
  constraint lock_tt_selected_users_selected_user_id
    foreign key (selected_user_id) references at_user (id) on delete cascade

);

-- Manage Tasks access right
update access_right set `name`='tasks.manage_customers_projects' where id=2;
insert into access_right (id, `name`) values (13, 'tasks.manage_tasks');

-- grant Manage Tasks right (code 13) to every user that already has Manage Customers & Projects right (code 2)
insert into user_access_right (user_id, access_right_id)
    select ar.user_id, 13
    from user_access_right ar where ar.access_right_id=2;

-- Notifications on online

create table user_upgrade_notification (
    user_id int not null,
    version varchar(32) not null,
    primary key(user_id),
    CONSTRAINT user_upgrade_notification_fk_at_user_id FOREIGN KEY (user_id) REFERENCES at_user(id) on delete cascade
);

-- ----------------------------------------------------

insert into patch_history(patch, pack_version, patch_timestamp)
    values(16, 0, now());
