-- Binary storage

create table bin_data (
    bin_data_id int(3) unsigned not null,
    data_length int(10) unsigned not null,
    file_name varchar(255),
    mime_type varchar(255),
    primary key (bin_data_id)
)
engine=InnoDB
default character set=utf8
default collate=utf8_unicode_ci;

create table bin_data_chunk (
    bin_data_id int(3) unsigned not null,
    chunk_num int(2) unsigned not null,
    chunk_data mediumblob,

    primary key (bin_data_id, chunk_num),

    constraint bin_data_chunk_bin_data_id
        foreign key (bin_data_id)  references bin_data (bin_data_id) on delete cascade
)
engine=InnoDB;

-- Mail messages

create table mail_message (
    message_id int(3) unsigned not null,
    subject text,
    body_bin_data_id int(3) unsigned not null,
    tries tinyint unsigned default 0,
    next_try_timestamp datetime default null,
    last_failure_reason text default null,
    primary key (message_id),
    constraint mail_message_body_bin_data_id
      foreign key (body_bin_data_id) references bin_data (bin_data_id) on delete no action
)
engine=InnoDB
default character set=utf8
default collate=utf8_unicode_ci;

create table mail_message_recipient (
    message_id int(3) unsigned not null,
    address_type tinyint unsigned not null default 0,
    personal varchar(255),
    address varchar(255) not null,
    constraint mail_message_recipient_message_id
      foreign key (message_id)  references mail_message (message_id) on delete cascade
)
engine=InnoDB
default character set=utf8
default collate=utf8_unicode_ci;

create table mail_message_attachment (
    message_id int(3) unsigned not null,
    bin_data_id int(3) unsigned not null,
    primary key(message_id, bin_data_id),
    constraint mail_message_attachment_message_id
      foreign key (message_id)  references mail_message (message_id) on delete cascade,
    constraint mail_message_attachment_bin_data_id
      foreign key (bin_data_id)  references bin_data (bin_data_id) on delete cascade
)
engine=InnoDB;

-- Mail notifications last date

create table notification_last_date (
    user_id int unsigned not null,
    last_date date not null,
    type int(2) unsigned not null,
    primary key(user_id, type),
    constraint notification_last_date_user_id
        foreign key (user_id) references at_user(id) on delete cascade
)
engine=InnoDB;

-- Mail notifications rules

create table notification_rule (
    id int(3) unsigned not null,
    type int(2) unsigned not null,
    is_active bit not null default 0,
    last_change_timestamp datetime not null,
    primary key (id)
)
engine=InnoDB;

-- Predefined rules

INSERT INTO `notification_rule` (`id`, `type`, `is_active`, `last_change_timestamp`) VALUES (1,1,false,now());
INSERT INTO `notification_rule` (`id`, `type`, `is_active`, `last_change_timestamp`) VALUES (2,2,false,now());
INSERT INTO `notification_rule` (`id`, `type`, `is_active`, `last_change_timestamp`) VALUES (3,3,false,now());
INSERT INTO `notification_rule` (`id`, `type`, `is_active`, `last_change_timestamp`) VALUES (4,3,false,now());
INSERT INTO `notification_rule` (`id`, `type`, `is_active`, `last_change_timestamp`) VALUES (5,3,false,now());
INSERT INTO `notification_rule` (`id`, `type`, `is_active`, `last_change_timestamp`) VALUES (6,4,false,now());
INSERT INTO `notification_rule` (`id`, `type`, `is_active`, `last_change_timestamp`) VALUES (7,4,false,now());
INSERT INTO `notification_rule` (`id`, `type`, `is_active`, `last_change_timestamp`) VALUES (8,4,false,now());
INSERT INTO `notification_rule` (`id`, `type`, `is_active`, `last_change_timestamp`) VALUES (9,5,false,now());
INSERT INTO `notification_rule` (`id`, `type`, `is_active`, `last_change_timestamp`) VALUES (10,6,false,now());
INSERT INTO `notification_rule` (`id`, `type`, `is_active`, `last_change_timestamp`) VALUES (11,6,false,now());
INSERT INTO `notification_rule` (`id`, `type`, `is_active`, `last_change_timestamp`) VALUES (12,7,false,now());

-- Parameters for mail notification rules

create table notification_rule_params (
    rule_id int(3) unsigned not null,
    param_name varchar(255) not null,
    param_value text not null,
    constraint notification_rule_params_rule_id
        foreign key (rule_id) references notification_rule(id) on delete cascade
)
engine=InnoDB
default character set=utf8
default collate=utf8_unicode_ci;

-- Predefined rule params

INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (1,'recipients.allUsers','true');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (1,'recipients.selectedUsers','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (1,'tt_limit.hours','7');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (1,'tt_limit.onlyWorkingDays','true');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (1,'tt_limit.rangeDays','1');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (2,'recipients.allUsers','true');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (2,'recipients.selectedUsers','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (2,'tt_limit.hours','35');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (2,'tt_users.allUsers','true');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (2,'tt_users.selectedUsers','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (2,'tt_limit.onlyWorkingDays','false');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (2,'tt_limit.rangeDays','7');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (3,'recipients.allUsers','true');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (3,'deadline','2');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (3,'customers_projects.showArchived','false');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (3,'customers_projects.projectList','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (3,'recipients.selectedUsers','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (3,'customers_projects.customerList','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (3,'customers_projects.coarseSelection','all_active');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (4,'recipients.allUsers','true');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (4,'deadline','0');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (4,'customers_projects.showArchived','false');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (4,'customers_projects.projectList','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (4,'customers_projects.customerList','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (4,'recipients.selectedUsers','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (4,'customers_projects.coarseSelection','all_active');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (5,'recipients.allUsers','true');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (5,'deadline','-2');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (5,'customers_projects.showArchived','false');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (5,'customers_projects.projectList','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (5,'recipients.selectedUsers','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (5,'customers_projects.customerList','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (5,'customers_projects.coarseSelection','all_active');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (6,'recipients.allUsers','true');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (6,'estimatePercents','80');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (6,'customers_projects.showArchived','false');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (6,'customers_projects.projectList','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (6,'customers_projects.customerList','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (6,'recipients.selectedUsers','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (6,'customers_projects.coarseSelection','all_active');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (7,'recipients.allUsers','true');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (7,'estimatePercents','100');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (7,'customers_projects.showArchived','false');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (7,'customers_projects.projectList','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (7,'customers_projects.customerList','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (7,'recipients.selectedUsers','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (7,'customers_projects.coarseSelection','all_active');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (8,'recipients.allUsers','true');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (8,'estimatePercents','150');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (8,'customers_projects.showArchived','false');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (8,'customers_projects.projectList','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (8,'customers_projects.customerList','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (8,'recipients.selectedUsers','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (8,'customers_projects.coarseSelection','all_active');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (9,'recipients.allUsers','true');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (9,'recipients.selectedUsers','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (10,'recipients.allUsers','true');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (10,'customers_projects.showArchived','false');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (10,'customers_projects.projectList','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (10,'recipients.selectedUsers','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (10,'customers_projects.customerList','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (10,'customers_projects.coarseSelection','all_active');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (10,'money.amount','1');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (10,'money.range','CURRENT_MONTH');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (11,'recipients.allUsers','true');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (11,'customers_projects.showArchived','false');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (11,'customers_projects.projectList','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (11,'recipients.selectedUsers','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (11,'customers_projects.customerList','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (11,'money.amount','1');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (11,'customers_projects.coarseSelection','all_active');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (11,'money.range','TOTAL');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (12,'recipients.allUsers','true');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (12,'recipients.selectedUsers','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (12,'money.amount','1');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (12,'tt_users.allUsers','true');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (12,'tt_users.selectedUsers','');
INSERT INTO notification_rule_params (rule_id, param_name, param_value) VALUES (12,'money.range','CURRENT_MONTH');

-- Days to send notifications on defined as cron expressions

create table notification_schedule (
    notification_type int(2) unsigned not null,
    cron varchar(255) not null,
    primary key (notification_type)
)
engine=InnoDB
default character set=utf8
default collate=utf8_unicode_ci;

insert into notification_schedule (notification_type, cron) values(1, "0 0 9 ? * 2,3,4,5,6 *");
insert into notification_schedule (notification_type, cron) values(2, "0 0 9 ? * 2 *");
insert into notification_schedule (notification_type, cron) values(4, "0 0 9 ? * * *");
insert into notification_schedule (notification_type, cron) values(6, "0 0 9 ? * * *");
insert into notification_schedule (notification_type, cron) values(7, "0 0 9 ? * * *");
-- ----------------------------------------------------

-- Time zones

alter table at_user add column time_zone varchar(64);

-- Notification on online

drop table if exists user_upgrade_notification;

create table gui_notification (
    id int(3) unsigned not null auto_increment,
    contents text not null,
    is_enabled bit not null default 1,
    expiration_date date,
    priority int not null default 0,
    primary key (id)
)engine=InnoDB
default character set=utf8
default collate=utf8_unicode_ci;

CREATE TABLE user_seen_gui_notif (
    user_id int(3) unsigned not null,
    seen_gui_notif_id int(3) unsigned not null,
    primary key (user_id, seen_gui_notif_id),
    CONSTRAINT gui_notification_user_id_fk_at_user_id FOREIGN KEY (user_id) REFERENCES at_user(id)
)engine=InnoDB;


CREATE TABLE mail_template (
    id int(3) unsigned not null auto_increment,
    name varchar(255) not null,
    template text not null,
    last_modified_timestamp datetime not null,
    primary key(id),
    unique(name)
)engine=InnoDB
default character set=utf8
default collate=utf8_general_ci;

INSERT INTO mail_template (name, template, last_modified_timestamp) VALUES ('assignments.html','#initStyles()
<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
#set($grantedColor = ''#688037'')
#set($revokedColor = ''#772F28'')
<table style=''width: 95%;$css_text font-size: 12px;''>
<tr>
<td style=''$css_text font-size: 14px; line-height:130%; background-color: #ffffff''>
Dear <b>$user.fullNameWithoutUserNameEscaped</b>,<br><br>
#if(!$addedAssignments && !$removedAssignments)
#if($assignments)
You no longer have access to all $hls.projects() of all $hls.customers().<br><br>
#else
You have been granted access to all $hls.projects() of all $hls.customers().
#end
#else
#if($addedAssignments && !$addedAssignments.isEmpty())
<div>Your were <span style=''color: $grantedColor;''>granted access</span> to the following #if($addedAssignments.size() > 1) $hls.projects()#else $hls.project()#end:</div>
<div><ul style=''padding-top:5px; padding-bottom: 5px;''>
#foreach($project in $addedAssignments)
<li style=''padding-top:2px; padding-bottom:3px;padding-left:10px;''>
<span style=''color: $grantedColor;''>$project.name</span> ($project.customer.name)
</li>
#end
</ul></div>
#end
#if($removedAssignments && !$removedAssignments.isEmpty())
<div>Your access to the following #if($removedAssignments.size() > 1) $hls.projects() #else $hls.project() #end has been <span style=''color: $revokedColor;''>restricted</span>:</div>
<div><ul style=''padding-top:5px; padding-bottom: 5px;''>
#foreach($project in $removedAssignments)
<li style=''padding-top:2px; padding-bottom:3px;padding-left:10px;''>
<span style=''color: $revokedColor;''>$project.name</span> ($project.customer.name)
</li>
#end
</ul></div>
#end
#end
#if($assignments)
#if (!$assignments.isEmpty())
<div style=''$css_table_title_text;margin-bottom: 0; padding-bottom: 0;''>Your current $hls.project() assignments:</div><br>
#drawCustomerProjectsTable($assignments)
#else
Currently you do not have access to any $hls.projects().
#end
#end
</td>
</tr>
<tr>
<td>#showFooter($footer)</td>
</tr>
</table>
</body>
</html>', now());
INSERT INTO mail_template (name, template, last_modified_timestamp) VALUES ('deadline.html','#initStyles()
<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
<table style=''width: 95%;$css_text font-size: 12px;''>
<tr>
<td style=''$css_text font-size: 14px; line-height:130%; background-color: #ffffff''>
Dear <b>$user.fullNameWithoutUserNameEscaped</b>,<br><br>
#set($tasksCount = $taskCount)
#if($deadlineStatus) There#if($tasksCount > 1) are $tasksCount $hls.tasks()#else is $tasksCount $hls.task()#end with $deadlineStatus deadline.<br><br>#end
#set($shouldDrawSeparator = false)
#foreach( $bean in $ruleBeans)
#foreach ( $dateRange in $bean.projectTasksMap.keySet() )
#if($shouldDrawSeparator)
<div style='' font-size:1px; line-height:1px;''><div style=''margin:25px 0px 25px 0px; width:100%; font-size:1px; height: 1px; border-top: 1px solid #BFBFBF; line-height: 1px;''>&nbsp;</div></div>
#end
#set($shouldDrawSeparator = true)
<div style=''$css_table_title_text''>$bean.getFormattedRuleTitle($dateRange, $textValueColor, $missedColor)</div><br>
<table style=''width:100%; border-collapse:collapse;''>
<thead>
<tr>
<th valign=''middle''
style=''
text-align: left;
padding: 10px 8px 10px 15px;
$css_bg_header
$css_header_text
$css_outer_border_top
$css_outer_border_left''>
<b>$hls.Customer() / $hls.Project() / $hls.Tasks()</b>
</th>
</tr>
</thead>
<tbody>
#set($drawTopCPBorder = false)
#set($projectNumber = 0)
#set( $oddRow = true )
#set( $projectTasks = $bean.projectTasksMap.get($dateRange) )
#foreach( $project in $projectTasks.keySet())
<tr>
<td style=''text-align: left;
background-color: #f2f2f2;
padding: 20px 8px 8px 15px;
$css_text
$css_outer_border_left
$css_outer_border_right
$css_cp_border_bottom
#if($drawTopCPBorder) $css_dark_border_top #else $css_outer_border_top #end''>
<b style=''font-size:14px; $css_details_header_color''>$bean.getNameEscaped($project.customer)</b> / <span style=''$css_parameters_name_text font-weight: bold;''> $bean.getNameEscaped($project) </span>
</td>
</tr>
#set($drawTopCPBorder = true)
#set( $rowNumber = 0)
#foreach( $task in $projectTasks.get($project))
<tr>
#set ( $drawOuterBorderBottom = ($projectNumber == ($projectTasks.keySet().size() - 1) && $rowNumber == ($projectTasks.get($project).size() - 1)) )
#if($oddRow)
#set ($bgColorFirst = ''#ffffff'')
#set ($bgColorSecond = ''#f7f7f7'')
#set ( $drawEvenBorderTop = false )
#set ( $drawEvenBorderBottom = false )
#else
#set ($bgColorFirst = ''#fcfaf3'')
#set ($bgColorSecond = ''#f5f3ed'')
#set ( $drawEvenBorderTop = ($rowNumber != 0) )
#set ( $drawEvenBorderBottom = ($rowNumber < ($projectTasks.get($project).size() - 1)) )
#end
<td valign=''middle''
style=''
text-align: left;
$css_details_font_size
background-color: $bgColorFirst;
padding: 8px 8px 8px 35px;
$css_text
$css_outer_border_left
$css_outer_border_right
#if($drawEvenBorderTop) $css_even_row_border_top #end
#if($drawEvenBorderBottom) $css_even_row_border_bottom #end
#if($drawOuterBorderBottom) $css_outer_border_bottom #end''>
<a href=''$bean.getLinkToTask($task)'' style=''$css_link_color''>$bean.getNameEscaped($task)</a>
</td>
#set($oddRow = !$oddRow)
#set($rowNumber = ($rowNumber + 1))
</tr>
#end
#set($projectNumber = ($projectNumber + 1))
#end
</tbody>
</table>
<br>
<a href=''$bean.link'' style=''$css_link_color; $css_report_link_font_size''>View Estimated vs. Actual Time Report for the listed#if($projectTasks.keySet().size() > 1) $hls.projects()#else $hls.project()#end</a>
#end
#end
</td>
</tr>
<tr>
<td>#showFooter($footer)</td>
</tr>
</table>
</body>
</html>', now());
INSERT INTO mail_template (name, template, last_modified_timestamp) VALUES ('estimate.html','#set($css_cp_border=''1px solid #d9d9d9 '')
#initStyles()
<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
<table style=''width: 95%;$css_text font-size: 12px;''>
<tr>
<td style=''$css_text font-size: 14px; line-height:130%; background-color: #ffffff''>
Dear <b>$user.fullNameWithoutUserNameEscaped</b>,<br>
<br>
#set($shouldDrawSeparator = false)
#foreach( $rule in $rules)
#set( $bean = $ruleBeans.get($rule) )
#set( $numberOfTasks = $bean.numberOfTasks)
#if($shouldDrawSeparator)
<br>
<div style=''font-size:1px; line-height:1px;''><div style=''margin:5px 0px 5px 0px; width:100%; font-size:1px; height: 1px; border-top: 1px solid #BFBFBF; line-height: 1px;''>&nbsp;</div></div>
<br>
#end
#set($shouldDrawSeparator = true)
<div style=''$css_table_title_text''><b>Time reported for the following
<span style=''$css_parameters_name_text font-weight: bold;''>$numberOfTasks</span>#if($numberOfTasks>1) $hls.tasks()#else $hls.task()#end reached
<span style=''$css_parameters_name_text font-weight: bold;''>$rule.estimatePercents%</span> of estimate:</b></div><br>
<table cellpadding=''0'' cellspacing=''0'' border=''0'' style=''width: 100%; border-collapse:collapse;''>
<thead>
<tr>
<th rowspan=''2'' valign=''middle''
style=''
text-align: left;
padding: 10px 8px 10px 15px;
$css_bg_header
$css_header_text
$css_outer_border_top
$css_outer_border_left''>
<b>$hls.Customer() / $hls.Project() / $hls.Tasks()</b>
</th>
<th colspan=''2'' valign=''middle''
style=''
text-align: center;
padding: 10px 8px 10px 8px;
$css_bg_header
$css_header_text
$css_outer_border_top
$css_light_border_left''>
<b>Spent on $bean.formattedRuleDateToShort</b></th>
<th rowspan=''2'' valign=''middle''
style=''
text-align: center;
padding: 0px 8px 0px 8px;
$css_bg_header
$css_header_text
$css_outer_border_top
$css_light_border_left''>
<b>Estimated<br>Hours</b></th>
<th colspan=''2'' valign=''middle''
style=''
text-align: center;
padding: 5px 8px 5px 8px;
$css_bg_header
$css_header_text
$css_outer_border_top
$css_light_border_right
$css_light_border_left''>
<b>Variance on $bean.formattedRuleDateToShort</b><br>
<span style=''font-size: 12px;''>(Estimated Hours - Spent)</span>
</th>
</tr>
<tr>
<th valign=''middle''
style=''width:90px;
text-align: center;
padding: 5px 8px 5px 8px;
$css_bg_header
$css_header_text
$css_outer_border_top
$css_light_border_left''>
<b>mh</b>
</th>
<th valign=''middle''
style=''width:90px;
text-align: center;
padding: 5px 8px 5px 8px;
$css_bg_header
$css_header_text
$css_outer_border_top
$css_light_border_left''>
<b>%</b>
</th>
<th valign=''middle''
style=''width:90px;
text-align: center;
padding: 5px 8px 5px 8px;
$css_bg_header
$css_header_text
$css_outer_border_top
$css_light_border_left''>
<b>mh</b>
</th>
<th valign=''middle''
style=''width:90px;
text-align: center;
padding: 5px 8px 5px 8px;
$css_bg_header
$css_header_text
$css_outer_border_top
$css_light_border_right
$css_light_border_left''>
<b>%</b>
</th>
</tr>
</thead>
<tbody>
#set($drawTopCPBorder = false)
#set($taskNumber = 0)
#set( $oddRow = true )
#foreach( $project in $bean.projectTasks.keySet())
<tr>
<td style=''text-align: left;
background-color: #f2f2f2;
padding: 20px 8px 8px 15px;
$css_text
$css_outer_border_left
$css_cp_border_bottom
#if($drawTopCPBorder) $css_dark_border_top #end''>
<b style=''font-size:14px; $css_details_header_color''>$project.customer.name</b> / <span style=''$css_parameters_name_text font-weight: bold;''> $project.name </span>
</td>
<td colspan=''2'' style=''text-align: left;
background-color: #f2f2f2;
padding: 20px 8px 8px 15px;
$css_text
$css_cp_border_bottom
#if($drawTopCPBorder) $css_dark_border_top #end''>
&nbsp;
</td>
<td style=''text-align: left;
background-color: #ededed;
padding: 20px 8px 8px 15px;
$css_text
$css_cp_border_bottom
$css_light_border_right
$css_light_border_left
#if($drawTopCPBorder) $css_dark_border_top #end'' >
&nbsp;
</td>
<td colspan=''2'' style=''text-align: left;
background-color: #f2f2f2;
padding: 20px 8px 8px 15px;
$css_text
$css_cp_border_bottom
$css_outer_border_right
#if($drawTopCPBorder) $css_dark_border_top #end''>
&nbsp;
</td>
</tr>
#set($drawTopCPBorder = true)
#set( $rowNumber = 0)
#foreach( $task in $bean.projectTasks.get($project))
<tr>
#set ( $drawOuterBorderBottom = ($taskNumber == ($numberOfTasks - 1)) )
#if($oddRow)
#set ($bgColorFirst = ''#ffffff'')
#set ($bgColorSecond = ''#f7f7f7'')
#set ( $drawEvenBorderTop = false )
#set ( $drawEvenBorderBottom = false )
#else
#set ($bgColorFirst = ''#fcfaf3'')
#set ($bgColorSecond = ''#f5f3ed'')
#set ( $drawEvenBorderTop = ($rowNumber != 0) )
#set ( $drawEvenBorderBottom = ($rowNumber < ($bean.projectTasks.get($project).size() - 1)) )
#end
<td valign=''middle''
style=''
text-align: left;
$css_details_font_size
background-color: $bgColorFirst;
padding: 8px 8px 8px 35px;
$css_text
$css_outer_border_left
#if($drawEvenBorderTop) $css_even_row_border_top #end
#if($drawEvenBorderBottom) $css_even_row_border_bottom #end
#if($drawOuterBorderBottom) $css_outer_border_bottom #end''>
<a href=''$task.link'' style=''$css_link_color''>$task.task.name</a>
</td>
<td nowrap valign=''middle''
style=''
text-align: right;
$css_details_font_size
background-color: $bgColorFirst;
padding: 8px 25px 8px 8px;
$css_text
#if($drawEvenBorderTop) $css_even_row_border_top #end
#if($drawEvenBorderBottom) $css_even_row_border_bottom #end
#if($drawOuterBorderBottom) $css_outer_border_bottom #end''
><div>$task.formattedSpent</div></td>
<td nowrap valign=''middle''
style=''
text-align: right;
$css_details_font_size
background-color: $bgColorFirst;
padding: 8px;
$css_text
#if($drawEvenBorderTop) $css_even_row_border_top #end
#if($drawEvenBorderBottom) $css_even_row_border_bottom #end
#if($drawOuterBorderBottom) $css_outer_border_bottom #end''
><div style=''margin-right:16px;''>$task.ofEstimate%</div></td>
<td nowrap valign=''middle''
style=''text-align: right;
$css_details_font_size
padding: 8px;
background-color: $bgColorSecond;
$css_text
$css_light_border_right
$css_light_border_left
#if($drawEvenBorderTop) $css_even_row_border_top #end
#if($drawEvenBorderBottom) $css_even_row_border_bottom #end
#if($drawOuterBorderBottom) $css_outer_border_bottom #end''
><div style=''margin-right:16px;''>$task.formattedBudgetTime</div></td>
<td nowrap valign=''middle''
style=''
text-align: right;
$css_details_font_size
background-color: $bgColorFirst;
padding: 8px;
$css_text
#if($drawEvenBorderTop) $css_even_row_border_top #end
#if($drawEvenBorderBottom) $css_even_row_border_bottom #end
#if($drawOuterBorderBottom) $css_outer_border_bottom #end''
><div style=''margin-right:16px;''>$task.formattedVariance</div></td>
<td nowrap valign=''middle''
style=''
text-align: right;
$css_details_font_size
background-color: $bgColorFirst;
padding: 8px;
$css_text
$css_outer_border_right
#if($drawEvenBorderTop) $css_even_row_border_top #end
#if($drawEvenBorderBottom) $css_even_row_border_bottom #end
#if($drawOuterBorderBottom) $css_outer_border_bottom #end''
><div style=''margin-right:16px;''>$task.percentVariance%</div></td>
#set($oddRow = !$oddRow)
#set($rowNumber = ($rowNumber + 1))
#set($taskNumber = ($taskNumber + 1))
</tr>
#end
#end
</tbody>
</table>
<br>
<a href=''$bean.link'' style=''$css_link_color; $css_report_link_font_size''>View Estimated vs. Actual Time Report for the listed#if($bean.projectTasks.keySet().size() > 1) $hls.projects()#else $hls.project()#end</a>
<br>
#end
</td>
</tr>
<tr>
<td>#showFooter($footer)</td>
</tr>
</table>
</body>
</html>', now());
INSERT INTO mail_template (name, template, last_modified_timestamp) VALUES ('personal_earnings.html','#initStyles()
<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
<table style=''width: 95%;$css_text font-size: 12px;''>
<tr>
<td style=''$css_text font-size: 14px; line-height:130%; background-color: #ffffff''>
Dear <b>$user.fullNameWithoutUserNameEscaped</b>,<br><br>
#set($shouldDrawSeparator = false)
#foreach( $rangeBean in $rangeBeans)
#if($shouldDrawSeparator)
<div style='' font-size:1px; line-height:1px;''><div style=''margin:25px 0px 25px 0px; width:100%; font-size:1px; height: 1px; border-top: 1px solid #BFBFBF; line-height: 1px;''>&nbsp;</div></div>
#end
#set($shouldDrawSeparator = true)
$rangeBean.getRangeStatus($textValueColor)<br>
<ul style=''padding-top:5px; padding-bottom: 5px;''>
#foreach( $user in $rangeBean.users )
<li style=''padding-top:2px; padding-bottom:3px;padding-left:9px;''>
$user.fullNameEscaped
</li>
#end
</ul>
<a href=''$rangeBean.link'' style=''$css_link_color; $css_report_link_font_size''>View Cost Report for the listed user#if($rangeBean.users.size() > 1)s#end</a>
<br>
#end
</td>
</tr>
<tr>
<td>#showFooter($footer)</td>
</tr>
</table>
</body>
</html>', now());
INSERT INTO mail_template (name, template, last_modified_timestamp) VALUES ('personal_tt.html','<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
#set($css_cp_border=''1px solid #d9d9d9 '')
#initStyles( )
<table style=''width: 95%;$css_text font-size: 12px;''>
<tr>
<td style=''$css_text font-size: 14px; line-height:130%; background-color: #ffffff''>
Dear <b>$user.fullNameWithoutUserNameEscaped</b>,<br>
<br> You have not recorded sufficient time for date range<br> $displaySettings.formattedDateRange.<br> <br>
<table cellpadding=''0'' cellspacing=''0'' border=''0'' style=''border-collapse: collapse;font-weight: bold; $css_parameters_text''>
<tr><td><b>Expected time-track:</b></td> <td style=''padding-left: 35px;''><b>$rule.ttLimit.hours:00</b></td></tr><tr><td><b>Recorded time-track:</b></td><td style=''color:red;padding-left: 35px;''><b>$total.formattedTotal</b></td></tr>
</table>
<br>
Please make sure that you entered all working and leave time for the specified date range.<br><br>
<table cellpadding=''0'' cellspacing=''0'' border=''0''
style=''font-size: 11px;
width: 100%;
border-collapse: collapse;
''>
<col/>
<col width=''60''/>
<col/>
#drawHeader( )
<tbody>
#foreach( $day in $days )
#set( $drawTotalTopBorder = ($day.rows.size() != 0) )
<tr>
<td colspan=''2'' valign=''bottom''
style=''
$css_text
$css_outer_border_left
$css_darkest_border_bottom
font-size: 18px;
padding: 30px 8px 10px 15px;
text-align: left; ''>
$day.formattedDay
</td>
<td valign=''bottom''
style=''
font-size: 13px;
text-align: right;
padding: 30px 15px 10px;
$css_text
$css_outer_border_right
$css_darkest_border_bottom
''>
<a href=''${displaySettings.getEditTTLink($day.day)}'' style=''$css_link_color''>Edit time-track</a>
</td>
</tr>
#drawDayTT( $day )
#drawDayTotal( $day $displaySettings $drawTotalTopBorder )
#end
<tr>
<td colspan=''3''><span style=''font-size: 12px; line-height:18px;''>&nbsp;<br></span></td>
</tr>
<tr>
<td valign=''middle''
style=''
text-align: right;
font-size: 15px;
padding: 12px 8px 6px 18px;
$css_text
$css_bg_range_total
$css_dark_border_left
$css_range_total_border_top
#if(!$displaySettings.showCalculatedOvertime) $css_range_total_border_bottom #end''>
<b>Total&nbsp;for&nbsp;the&nbsp;date&nbsp;range:</b></td>
<td valign=''middle''
style=''
text-align: right;
font-size: 15px;
$css_text
padding: 12px 15px 6px 8px;
$css_bg_range_total_middle
$css_light_border_left
$css_light_border_right
$css_range_total_border_top
#if(!$displaySettings.showCalculatedOvertime) $css_range_total_border_bottom #end''>
<b>$total.formattedTotal</b></td>
<td valign=''middle''
style=''
text-align: left;
font-size: 13px;
padding: 12px 8px 6px 18px;
$css_text
$css_bg_range_total
$css_dark_border_right
$css_range_total_border_top
#if(!$displaySettings.showCalculatedOvertime) $css_range_total_border_bottom #end''>
<table border=''0'' cellpadding=''0'' cellspacing=''0''>
<tr>
<td width=''80''
style=''
text-align: left;
font-size: 13px;
$css_text
$css_bg_range_total''>
Working&nbsp;Time:</td>
<td width=''40''
style=''
text-align: right;
font-size: 13px;
padding-left: 5px;
$css_text
$css_bg_range_total''>
$total.formattedWorking
</td>
#if($displaySettings.showLeaveTime)
<td style=''
padding-left: 25px;
text-align: left;
font-size: 13px;
$css_text
$css_bg_range_total''>
Leave&nbsp;Time:</td>
<td width=''40''
style=''
text-align: right;
font-size: 13px;
padding-left: 5px;
$css_text
$css_bg_range_total''>
$total.formattedLeave
</td>
#end
<td></td>
</tr>
</table>
</td>
</tr>
#if($displaySettings.showReportedOvertime)
<tr>
<td valign=''top''
style=''
text-align: right;
font-size: 14px;
padding: 0 8px 8px 15px;
$css_text
$css_bg_range_total
$css_dark_border_left''>
<span style=''white-space: nowrap;''>Reported&nbsp;Overtime&nbsp;#if($displaySettings.undertimeTracking)/&nbsp;Undertime#end:</span>
</td>
<td valign=''top''
style='' text-align: right;
font-size: 14px;
padding: 0 15px 8px 8px;
$css_text
$css_bg_range_total_middle
$css_light_border_left
$css_light_border_right''>
$total.formattedReportedOvertime
</td>
<td valign=''top''
style='' text-align: left;
font-size: 14px;
$css_text
$css_bg_range_total
$css_dark_border_right''>
&nbsp;</td>
</tr>
#end #if($displaySettings.showCalculatedOvertime)
<tr>
<td valign=''top''
style='' text-align: right;
font-size: 14px;
padding: 0 8px 16px 15px;
$css_text
$css_bg_range_total
$css_dark_border_left
$css_range_total_border_bottom''>
<span style=''white-space: nowrap;''>Auto-calculated&nbsp;Overtime#if($displaySettings.undertimeTracking)&nbsp;/&nbsp;Undertime#end:</span>
</td>
<td valign=''top''
style='' text-align: right;
font-size: 14px;
padding: 0 15px 8px 8px;
$css_text
$css_bg_range_total_middle
$css_light_border_left
$css_light_border_right
$css_range_total_border_bottom''>
$total.formattedCalculatedOvertime
</td>
<td valign=''top''
style=''text-align: left;
font-size: 14px;
$css_text
$css_bg_range_total
$css_dark_border_right
$css_range_total_border_bottom''>
&nbsp;</td>
</tr>
#end
</tbody>
</table>
</td>
</tr>
<tr>
<td>#showFooter($footer)</td>
</tr>
</table>
</body>
</html>', now());
INSERT INTO mail_template (name, template, last_modified_timestamp) VALUES ('project_budget.html','#initStyles()
<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
<table style=''width: 95%;$css_text font-size: 12px;''>
<tr>
<td style=''$css_text font-size: 14px; line-height:130%; background-color: #ffffff''>
Dear <b>$user.fullNameWithoutUserNameEscaped</b>,<br>
<br>
#set($shouldDrawSeparator = false)
#foreach( $rangeBean in $rangeBeans)
#if($shouldDrawSeparator)
<div style='' font-size:1px; line-height:1px;''><div style=''margin:25px 0px 25px 0px; width:100%; font-size:1px; height: 1px; border-top: 1px solid #BFBFBF; line-height: 1px;''>&nbsp;</div></div>
#end
#set($shouldDrawSeparator = true)
<div style=''$css_table_title_text;padding-bottom:0; margin-bottom:0;''>$rangeBean.getRangeStatus($textValueColor)</div><br>
#drawCustomerProjectsTable( $rangeBean.customerProjects )
<br>
<a href=''$rangeBean.link'' style=''$css_link_color; $css_report_link_font_size''>View Cost Report for the listed#if($rangeBean.getProjectsCount() > 1) $hls.projects()#else $hls.project()#end</a>
#end
</td>
</tr>
<tr>
<td>#showFooter($footer)</td>
</tr>
</table>
</body>
</html>', now());
INSERT INTO mail_template (name, template, last_modified_timestamp) VALUES ('shared_macro.vm','## some email clients do not support embedded or external css, so we define several commonly used
## rules here as velocity variables to re-use them later for inline css
#macro( initStyles )
#set($css_details_font_size=''font-size: 13px; '')
#set($css_report_link_font_size=''font-size: 13px; '')
#set($css_details_header_color=''color: #595959; '')
#set($css_light_border=''1px solid #e9e9e9 '')
#set($css_dark_border=''1px solid #d9d9d9 '')
#set($css_outer_border=''1px solid #cccccc '')
#set($css_darkest_border=''1px solid #999999 '')
#set($css_range_total_border=''1px solid #8799af '')
#set($css_link_color=''color: #003399 '')
#set($css_light_border_left=''border-left: '' + $css_light_border + ''; '')
#set($css_light_border_right=''border-right: '' + $css_light_border + ''; '')
#set($css_outer_border_left=''border-left: '' + $css_outer_border + ''; '')
#set($css_outer_border_right=''border-right: '' + $css_outer_border + ''; '')
#set($css_outer_border_bottom=''border-bottom: '' + $css_outer_border + ''; '')
#set($css_outer_border_top=''border-top: '' + $css_outer_border + ''; '')
#set($css_dark_border_left=''border-left: '' + $css_dark_border + ''; '')
#set($css_dark_border_right=''border-right: '' + $css_dark_border + ''; '')
#set($css_dark_border_top=''border-top: '' + $css_dark_border + ''; '')
#set($css_darkest_border_bottom=''border-bottom: '' + $css_darkest_border + ''; '')
#set($css_cp_border_bottom=''border-bottom: '' + $css_dark_border + ''; '')
#set($css_range_total_border_bottom=''border-bottom: '' + $css_range_total_border + ''; '')
#set($css_range_total_border_top=''border-top: '' + $css_range_total_border + ''; '')
#set($css_bg_header=''background-color: #366A98; '')
#set($css_bg_total=''background-color: #f8f4e9; '')
#set($css_bg_total_middle=''background-color: #f1eee5; '')
#set($css_bg_range_total=''background-color: #eef2f7; '')
#set($css_bg_range_total_middle=''background-color: #e9edf1; '')
#set($css_even_row_border_top=''border-top: 1px solid #f0eee7; '')
#set($css_even_row_border_bottom=''border-bottom: 1px solid #f0eee7; '')
#set($css_text=''font-family: Arial, sans-serif; font-weight: normal; '')
#set($css_header_text=$css_text + '' color: white; font-size: 14px; '')
#set($css_total_text=$css_text + '' color: #665946; '')
#set($textValueColor=''#366A98'')
#set($css_parameters_text=$css_text + '' font-size: 14px; '')
#set($css_parameters_name_text=$css_parameters_text + '' color: '' + $textValueColor + '';'')
#set($css_table_title_text=$css_text + '' color: #6A573B; font-size: 14px; font-weight: bold; '')
#set($missedColor=''#C00000'')
#end
#macro( drawHeader )
<thead>
<tr>
<th valign=''middle'' style=''
text-align: left;
padding: 10px 8px 10px 15px;
$css_bg_header
$css_header_text
$css_outer_border_top
$css_outer_border_left''>
<span style=''font-size: 14px; margin-bottom: 7px; margin-top: 10px''><b>$hls.Customer() / $hls.Project()</b></span><br>
<span style=''font-size: 12px;''>$hls.Tasks()</span>
</th>
<th valign=''middle'' style=''
text-align: center;
padding: 10px 8px;
$css_bg_header
$css_header_text
$css_outer_border_top
$css_light_border_right
$css_light_border_left''><b>Spent</b></th>
<th valign=''middle'' style=''
text-align: left;
padding: 17px 8px 17px 8px;
$css_bg_header
$css_header_text
$css_outer_border_top
$css_outer_border_right''><b><span>Comments</span></b></th>
</tr>
</thead>
#end
#macro( drawDayTT $day)
#set( $oddRow = true )
#set( $drawCPEntryTopBorder = false )
#if( $day.leaveTime > 0)
<tr>
<td valign=''bottom'' style=''
text-align: left;
background-color: #FCF8EF;
$css_details_header_color
padding: 20px 8px 8px 15px;
$css_text
$css_outer_border_left
border-bottom: 1px dashed #AFAFAF ;
''><b><span style=''font-size: 14px;''>$day.leaveName</span></b></td>
<td valign=''bottom'' style=''
text-align: right;
font-size: 14px;
padding: 20px 15px 8px 8px;
background-color: #F5F1EA;
$css_text
border-bottom: 1px dashed #BFBFBF ;
$css_light_border_right
$css_light_border_left
''><b>$day.formattedLeaveTime</b></td>
<td style=''
background-color: #FCF8EF;
padding: 20px 8px 8px 15px;
$css_outer_border_right
border-bottom: 1px dashed #AFAFAF ;
''>&nbsp;</td>
</tr>
#end
#foreach( $cpEntry in $day.rows.entrySet() )
<tr>
<td valign=''bottom'' style=''
text-align: left;
background-color: #f2f2f2;
padding: 20px 8px 8px 15px;
$css_text
$css_outer_border_left
$css_cp_border_bottom
#if($drawCPEntryTopBorder) $css_dark_border_top #end''>
<b><span style=''font-size: 14px; $css_details_header_color''>$cpEntry.key.customerName /</span>
<span style=''$css_parameters_name_text font-weight: bold; ''>$cpEntry.key.projectName</span></b>
</td>
<td valign=''bottom'' style=''
text-align: right;
font-size: 14px;
padding: 20px 15px 8px 8px;
background-color: #ededed;
$css_text
$css_cp_border_bottom
$css_light_border_right
$css_light_border_left
#if($drawCPEntryTopBorder) $css_dark_border_top #end''>
<b>$day.getFormattedSubTotal($cpEntry.key)</b></td>
<td style=''
background-color: #f2f2f2;
padding: 20px 8px 8px 15px;
$css_outer_border_right
$css_cp_border_bottom
#if($drawCPEntryTopBorder) $css_dark_border_top #end''>&nbsp;</td>
</tr>
#set( $drawCPEntryTopBorder = true )
#set ( $rowNumber = 0 )
#foreach( $ttRow in $cpEntry.value )
#set ( $drawEvenBorderTop = ($rowNumber != 0) )
#set ( $drawEvenBorderBottom = ($rowNumber < ($cpEntry.value.size() - 1)) )
#if( $oddRow )
<tr>
<td valign=''middle'' style=''
text-align: left;
padding: 8px 8px 8px 35px;
font-size: 13px;
background-color: #ffffff;
$css_text
$css_outer_border_left''>$ttRow.taskName</td>
<td valign=''middle'' style=''
text-align: right;
font-size: 13px;
background-color: #f7f7f7;
padding: 8px 15px 8px 8px;
$css_text
$css_light_border_right
$css_light_border_left''>$ttRow.formattedSpent</td>
<td valign=''middle'' style=''
text-align: left;
font-size: 13px;
background-color: #ffffff;
padding: 8px 8px 8px 18px;
$css_text
$css_outer_border_right''>$ttRow.comments</td>
</tr>
#else
<tr>
<td valign=''middle'' style=''
text-align: left;
padding: 8px 8px 8px 35px;
font-size: 13px;
background-color: #fcfaf3;
$css_text
$css_outer_border_left
#if($drawEvenBorderTop) $css_even_row_border_top #end
#if($drawEvenBorderBottom) $css_even_row_border_bottom #end
''>$ttRow.taskName</td>
<td valign=''middle'' style=''
text-align: right;
font-size: 13px;
background-color: #f5f3ed;
padding: 8px 15px 8px 8px;
$css_text
#if($drawEvenBorderTop) $css_even_row_border_top #end
#if($drawEvenBorderBottom) $css_even_row_border_bottom #end
$css_light_border_right
$css_light_border_left
''>$ttRow.formattedSpent</td>
<td valign=''middle'' style=''
text-align: left;
font-size: 13px;
background-color: #fcfaf3;
$css_text
padding: 8px 8px 8px 18px;
$css_outer_border_right
#if($drawEvenBorderTop) $css_even_row_border_top #end
#if($drawEvenBorderBottom) $css_even_row_border_bottom #end
''>$ttRow.comments</td>
</tr>
#end
#set( $oddRow = !$oddRow )
#set( $rowNumber = ( $rowNumber + 1 ) )
#end
#end
#end
#macro( drawDayTotal $day $displaySettings $drawTopBorder)
<tr>
<td valign=''middle'' style=''text-align: right;
font-size: 15px;
padding: 10px 8px 6px 18px;
$css_total_text
$css_bg_total
$css_outer_border_left
#if($drawTopBorder) $css_outer_border_top #end
#if(!$displaySettings.showCalculatedOvertime) $css_outer_border_bottom #end''>
<b>Total:</b></td>
<td valign=''middle'' style=''
text-align: right;
font-size: 15px;
$css_text
padding: 10px 15px 6px 8px;
$css_bg_total_middle
$css_light_border_left
$css_light_border_right
#if($drawTopBorder) $css_outer_border_top #end
#if(!$displaySettings.showCalculatedOvertime) $css_outer_border_bottom #end''>
<b>$day.total.formattedTotal</b></td>
<td valign=''middle'' style=''text-align: left;
font-size: 13px;
padding: 10px 18px 6px 18px;
$css_text
$css_bg_total
$css_outer_border_right
#if($drawTopBorder) $css_outer_border_top #end
#if(!$displaySettings.showCalculatedOvertime) $css_outer_border_bottom #end''>
<table border=''0'' cellpadding=''0'' cellspacing=''0''>
<tr>
<td width=''80'' style=''
text-align: left;
font-size: 13px;
$css_total_text
$css_bg_total''>
<span style=''white-space: nowrap;''>Working&nbsp;Time:</span></td>
<td width=''40'' style=''
text-align: right;
font-size: 13px;
padding-left: 5px;
$css_text
$css_bg_total''>$day.total.formattedWorking</td>
#if($displaySettings.showLeaveTime)
<td style=''
padding-left: 25px;
text-align: left;
font-size: 13px;
$css_total_text
$css_bg_total''>
#if( $day.total.leaveName )${day.total.leaveName}#{else}Leave&nbsp;Time#end:</td>
<td width=''40'' style=''
text-align: right;
font-size: 13px;
padding-left: 5px;
$css_text
$css_bg_total''>$day.total.formattedLeave</td>
#end
<td> </td>
</tr>
</table>
</td>
</tr>
#if($displaySettings.showReportedOvertime)
<tr>
<td valign=''top'' style=''
text-align: right;
font-size: 14px;
padding: 0 8px 6px 15px;
$css_total_text
$css_bg_total
$css_outer_border_left''>
<span style=''white-space: nowrap;''>Reported&nbsp;Overtime #if($displaySettings.undertimeTracking)&nbsp;/&nbsp;Undertime #end:</span></td>
<td valign=''top'' style=''
text-align: right;
font-size: 14px;
padding: 0 15px 8px 8px;
$css_text
$css_bg_total_middle
$css_light_border_left
$css_light_border_right''>$day.total.formattedReportedOvertime</td>
<td valign=''top'' style=''
text-align: left;
font-size: 14px;
$css_text
$css_bg_total
$css_outer_border_right''>&nbsp;</td>
</tr>
#end
#if($displaySettings.showCalculatedOvertime)
<tr>
<td valign=''top'' style=''
text-align: right;
font-size: 14px;
padding: 0 8px 14px 15px;
$css_total_text
$css_bg_total
$css_outer_border_left
$css_outer_border_bottom''><span style=''white-space: nowrap;''>Auto-calculated&nbsp;Overtime#if($displaySettings.undertimeTracking)&nbsp;/&nbsp;Undertime #end:</span></td>
<td valign=''top'' style=''
text-align: right;
font-size: 14px;
padding: 0 15px 8px 8px;
$css_text
$css_bg_total_middle
$css_light_border_left
$css_light_border_right
$css_outer_border_bottom''>$day.total.formattedCalculatedOvertime</td>
<td valign=''top'' style=''
text-align: left;
font-size: 14px;
$css_text
$css_bg_total
$css_outer_border_right
$css_outer_border_bottom''>&nbsp;</td>
</tr>
#end
#end
#macro ( drawCustomerProjectsTable $customerProjects )
<table cellpadding=''0'' cellspacing=''0'' border=''0'' style=''width: 100%; border-collapse:collapse;''>
<thead>
<tr>
<th valign=''middle''
style=''
text-align: left;
padding: 10px 8px 10px 15px;
$css_bg_header
$css_header_text
$css_outer_border_top
$css_outer_border_left''>
<b>$hls.Customer() / $hls.Projects()</b>
</th>
</tr>
</thead>
<tbody>
#set($drawTopCPBorder = false)
#set($customerNumber = 0)
#set( $oddRow = true )
#foreach( $customer in $customerProjects.keySet())
#set( $rowNumber = 0 )
#set( $projectsNumber = $customerProjects.get($customer).size() )
<tr>
<td style=''text-align: left;
background-color: #f2f2f2;
padding: 20px 8px 8px 15px;
$css_details_header_color
$css_details_font_size
$css_text
$css_outer_border_left
$css_outer_border_right
$css_cp_border_bottom
font-weight: normal;
#if($drawTopCPBorder) $css_dark_border_top #else $css_outer_border_top #end''>
<b>$customer.name</b> ($projectsNumber #if ($projectsNumber > 1) $hls.projects() #else $hls.project() #end )
</td>
</tr>
#set($drawTopCPBorder = true)
#foreach( $project in $customerProjects.get($customer) )
#set( $drawEvenBorderTop = ($rowNumber != 0) )
#set( $drawEvenBorderBottom = ($rowNumber < ($projectsNumber - 1)) )
#set( $drawOuterBorderBottom = (($customerNumber == ($customerProjects.keySet().size() - 1)) && !$drawEvenBorderBottom) )
<tr>
#if($oddRow)
<td valign=''bottom''
style=''
text-align: left;
$css_details_font_size
background-color: #ffffff;
padding: 8px 8px 8px 35px;
$css_text
$css_outer_border_left
$css_outer_border_right
#if($drawOuterBorderBottom) $css_outer_border_bottom #end''>
$project.name
</td>
#else
<td valign=''bottom''
style=''
text-align: left;
$css_details_font_size
background-color: #fcfaf3;
padding: 8px 8px 8px 35px;
$css_text
$css_outer_border_left
$css_outer_border_right
#if($drawEvenBorderTop) $css_even_row_border_top #end
#if($drawEvenBorderBottom) $css_even_row_border_bottom #end
#if($drawOuterBorderBottom) $css_outer_border_bottom #end''>
$project.name
</td>
#end
#set($oddRow = !$oddRow)
#set($rowNumber = ($rowNumber + 1))
</tr>
#end
#set($customerNumber = ($customerNumber + 1))
#end
</tbody>
</table>
#end
#macro ( showFooter $footer )
<br/>
<hr noshade size="1" style="background-color: #333333; color: #333333;" />
<pre style=''font-family: Arial, sans-serif; font-size: 12px; margin-top:10px;''>$footer</pre>
#end', now());
INSERT INTO mail_template (name, template, last_modified_timestamp) VALUES ('test_message.html','#initStyles()
<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
<table style=''width: 560px;$css_text font-size: 12px;'' cellspacing=''0''>
<tr>
<td
valign=''middle''>Dear $user.fullNameWithoutUserNameEscaped,<br><br>
This is an automatic email message sent by actiTIME to test email notification settings.<br>If you received it, then Outgoing mail server, port, and &#8220;From&#8221; address have been correctly set up.
</td>
</tr>
<tr>
<td>
<br>Please also verify that the following notification parameters are configured properly:
<br>
<ol>
<li style=''margin-left: 0; padding-left: 0; margin-bottom: 20px;'' > &#8220;From&#8221; address<br><br>
&#8220;From&#8221; address will be shown to all notification recipients in the same way as
in this message. Make sure that it is shown in the required format.<br>
</li>
<li style=''margin-left: 0; padding-left: 0; margin-bottom: 20px;'' >
Links to actiTIME interfaces<br><br>
Please follow the link below to make sure that notification recipients will be able
to open links in the notification messages.<br>
<a href=''$url'' style=''$css_link_color''>Sample link to general settings interface</a><br>
</li>
#if(!($!footer == ""))
<li style=''margin-left: 0; padding-left: 0; margin-bottom: 20px;'' >
Standard message footer<br><br>
Standard message footer will be added to all notification messages (including this one).
Make sure it looks as you expected.
</li>
#end
</ol>
</td>
</tr>
#if(!($!footer == ""))
<tr>
<td>#showFooter($footer)</td>
</tr>
#end
</table>
</body>
</html>', now());
INSERT INTO mail_template (name, template, last_modified_timestamp) VALUES ('unsubmitted_tt.html','<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
#set($css_cp_border=''1px solid #d9d9d9 '')
#initStyles()
<table style=''width: 95%;$css_text font-size: 12px;''>
<tr>
<td style=''$css_text font-size: 14px; line-height:130%; background-color: #ffffff''>
Dear <b>$user.fullNameWithoutUserNameEscaped</b>,<br><br>
There#if($usersCount > 1) are users#else is a user#end whose time-track is incomplete.<br><br>
#set($shouldDrawSeparator = false)
#foreach( $rule in $rules)
#if($shouldDrawSeparator)
<div style='' font-size:1px; line-height:1px;''><div style=''margin:25px 0px 25px 0px; width:100%; font-size:1px; height: 1px; border-top: 1px solid #BFBFBF; line-height: 1px;''>&nbsp;</div></div>
#end
#set($shouldDrawSeparator = true)
<table cellpadding=''0'' cellspacing=''0'' border=''0'' style=''border-collapse:collapse; text-align: left;''>
<tr>
<td style=''$css_parameters_name_text; padding-left: 0px; margin-left:0px;''>
<b style="white-space: nowrap;">Date Range:</b>
</td>
<td style=''$css_parameters_text padding-left: 25px;''>
<b style="white-space: nowrap; ">$ruleBeans.get($rule).formattedDateRange</b>
</td>
</tr>
<tr>
<td style=''$css_parameters_name_text; padding-left: 0px; margin-left:0px;''>
<b style="white-space: nowrap;">Expected Time-Track:</b>
</td>
<td style=''$css_parameters_text padding-left: 25px;''>
<b style="white-space: nowrap;">$rule.ttLimit.hours:00</b>
</td>
</tr>
</table>
<br>
<b><span style=''$css_table_title_text''>Time-Track Reported by User#if ($ruleBeans.get($rule).all.size() > 1)s#end:</span></b>
<br>
<br>
<table cellpadding=''0'' cellspacing=''0'' border=''0'' style=''width: 100%; border-collapse:collapse;''>
<col/> <col width=''80''/> <col width=''80''/>
<thead>
<tr>
<th valign=''middle''
style=''
text-align: left;
padding: 0px 8px 0px 15px;
$css_bg_header
$css_header_text
$css_outer_border_top
$css_outer_border_left''>
<b>User</b></th>
<th valign=''middle''
style='' width: 80px;
text-align: center;
padding: 0px 8px 0px 8px;
$css_bg_header
$css_header_text
$css_outer_border_top
$css_light_border_left''>
<b>Spent</b></th>
<th valign=''middle''
style='' width: 80px;
text-align: center;
padding: 10px 8px 10px 8px;
$css_bg_header
$css_header_text
$css_outer_border_top
$css_light_border_right
$css_light_border_left''>
<b>Missing<br>
Hours</b></th>
</tr>
</thead>
<tbody>
#set( $oddRow = true )
#set( $rowNumber = 0)
#foreach( $userInn in $ruleBeans.get($rule).all )
#set ( $drawEvenBorderTop = ($rowNumber != 0) )
#set ( $drawEvenBorderBottom = ($rowNumber < ($ruleBeans.get($rule).all.size() - 1)))
#set ( $drawBorderBottom = ($rowNumber == ($ruleBeans.get($rule).all.size() - 1)))
#if($oddRow)
<tr>
<td valign=''middle''
style=''
text-align: left;
font-size: 13px;
background-color: #ffffff;
padding: 8px 8px 8px 15px;
$css_text
$css_outer_border_left
#if($drawBorderBottom) $css_outer_border_bottom #end
''>
$userInn.user.fullNameEscaped
</td>
<td valign=''middle''
style=''text-align: right;
font-size: 13px;
padding: 8px 15px 8px 8px;
background-color: #ffffff;
$css_text
$css_dark_border_left
#if($drawBorderBottom) $css_outer_border_bottom #end''>
$userInn.formattedActuals</td>
<td valign=''middle''
style=''text-align: right;
font-size: 13px;
padding: 8px 15px 8px 8px;
background-color: #ffffff;
$css_text
$css_outer_border_right
$css_dark_border_left
#if($drawBorderBottom) $css_outer_border_bottom #end''>
<span style=''color:red;''>$userInn.formattedMissed</span></td>
</tr>
#else
<tr>
<td valign=''middle''
style=''
text-align: left;
font-size: 13px;
background-color: #fcfaf3;
padding: 8px 8px 8px 15px;
$css_text
$css_outer_border_left
#if($drawEvenBorderTop) $css_even_row_border_top #end
#if($drawEvenBorderBottom) $css_even_row_border_bottom #end
#if($drawBorderBottom) $css_outer_border_bottom #end
''>
$userInn.user.fullNameEscaped
</td>
<td valign=''middle''
style=''text-align: right;
font-size: 13px;
padding: 8px 15px 8px 8px;
background-color: #fcfaf3;
$css_text
$css_dark_border_left
#if($drawEvenBorderTop) $css_even_row_border_top #end
#if($drawEvenBorderBottom) $css_even_row_border_bottom #end
#if($drawBorderBottom) $css_outer_border_bottom #end
''>
$userInn.formattedActuals</td>
<td valign=''middle''
style=''text-align: right;
font-size: 13px;
padding: 8px 15px 8px 8px;
background-color: #fcfaf3;
$css_text
$css_outer_border_right
$css_dark_border_left
#if($drawEvenBorderTop) $css_even_row_border_top #end
#if($drawEvenBorderBottom) $css_even_row_border_bottom #end
#if($drawBorderBottom) $css_outer_border_bottom #end''>
<span style=''color:red;''>$userInn.formattedMissed</span></td>
</tr>
#end
#set($oddRow = !$oddRow)
#set($rowNumber = ($rowNumber + 1))
#end
</tbody>
</table>
<br>
<a href=''$ruleBeans.get($rule).link'' style=''$css_link_color; $css_report_link_font_size''>View detailed time-track data in the Staff Output Report</a>
#end
</td>
</tr>
<tr>
<td>#showFooter($footer)</td>
</tr>
</table>
</body>
</html>', now());

-- ----------------------------------------------------

-- fix table engine (bug #28262)
alter table license engine=InnoDB;

-- fix tables charset and collation
alter table leave_type collate utf8_unicode_ci;
alter table report_configs convert to character set utf8 collate utf8_unicode_ci;
alter table report_params convert to character set utf8 collate utf8_unicode_ci;
alter table crm_resource convert to character set utf8 collate utf8_unicode_ci;
alter table task collate=utf8_unicode_ci;
alter table invoice_params collate=utf8_unicode_ci;
alter table invoice_discount collate=utf8_unicode_ci;
alter table invoice_tax collate=utf8_unicode_ci;
alter table invoice_terms collate=utf8_unicode_ci;
alter table invoice_bill_to_recent_entry collate=utf8_unicode_ci;

-- remove duplicate foreign keys (bug #27909)

alter table task drop foreign key task_project_fk;
alter table task drop foreign key task_customer_fk;
alter table task drop foreign key fk_task_project;
alter table task add constraint fk_task_project foreign key(project_id) references project(id) on delete cascade;
alter table task add constraint task_customer_fk foreign key(customer_id) references customer(id) on delete cascade;

-- ----------------------------------------------------

-- some foreign keys have no defined name, so we can't remove them and need to re-create the whole table

set foreign_key_checks=0;

-- re-create project
create table project_temp (
    id int(3) not null,
    customer_id int not null,
    create_timestamp datetime not null,
    name varchar(255) not null,
    name_lower varchar(255) not null,
    description text,
    archiving_timestamp datetime null,
    primary key (id)
)
engine=InnoDB
default character set=utf8
default collate=utf8_unicode_ci;

insert into project_temp (id, customer_id, create_timestamp, name, name_lower, description, archiving_timestamp)
    select id, customer_id, create_timestamp, name, name_lower, description, archiving_timestamp from project;

drop table project;

create table project (
    id int(3) not null,
    customer_id int not null,
    create_timestamp datetime not null,
    name varchar(255) not null,
    name_lower varchar(255) not null,
    description text,
    archiving_timestamp datetime null,
    constraint pk_project primary key (id),
    constraint uk_project_customer_name unique key (customer_id, name_lower),
    index project_customer_id_i (customer_id),
    index project_customer_archts_i (customer_id, archiving_timestamp),
    constraint fk_project_customer_id foreign key(customer_id) references customer(id) on delete cascade
)
engine=InnoDB
default character set=utf8
default collate=utf8_unicode_ci;

insert into project (id, customer_id, create_timestamp, name, name_lower, description, archiving_timestamp)
    select id, customer_id, create_timestamp, name, name_lower, description, archiving_timestamp from project_temp;

drop table project_temp;

-- re-create tt_record
create table tt_record_temp (
  user_id int(3) unsigned not null,
  task_id int(5) not null,
  record_date date not null,
  actuals int not null
)
engine=InnoDB;

insert into tt_record_temp( user_id, task_id, record_date, actuals )
    select user_id, task_id, record_date, actuals from tt_record;

drop table tt_record;

create table tt_record (
  user_id int(3) unsigned not null,
  task_id int(5) not null,
  record_date date not null,
  actuals int not null,
  constraint pk_tt_record primary key( user_id, task_id, record_date ),
  index tt_record_user_i (user_id),
  constraint fk_tt_task foreign key (task_id) references task(id) on delete cascade,
  constraint fk_tt_user foreign key (user_id) references at_user(id) on delete cascade
)
engine=InnoDB
default character set=utf8
default collate=utf8_unicode_ci;

insert into tt_record( user_id, task_id, record_date, actuals )
    select user_id, task_id, record_date, actuals from tt_record_temp;

drop table tt_record_temp;

-- re-create user_task
create table user_task_temp
(
  user_id int(3) unsigned not null,
  task_id int(5) not null
)
engine=InnoDB;

insert into user_task_temp( user_id, task_id )
    select user_id, task_id from user_task;

drop table user_task;

create table user_task
(
  user_id int(3) unsigned not null,
  task_id int(5) not null,
  constraint pk_user_task primary key(user_id, task_id),
  index user_task_user_i (user_id),
  constraint fk_user_task_user foreign key(user_id) references at_user(id) on delete cascade,
  index user_task_task_i (task_id),
  constraint fk_user_task_task foreign key(task_id) references task(id) on delete cascade
)

engine=InnoDB
default character set=utf8
default collate=utf8_unicode_ci;

insert into user_task( user_id, task_id )
    select user_id, task_id from user_task_temp;

drop table user_task_temp;

-- re-create user_task_comment
create table user_task_comment_temp (
    user_id int(3) unsigned not null,
    task_id int(5) not null,
    comment_date date not null,
    comments text not null
)
engine=InnoDB
default character set=utf8
default collate=utf8_unicode_ci;

insert into user_task_comment_temp( user_id, task_id, comment_date, comments )
    select user_id, task_id, comment_date, comments from user_task_comment;

drop table user_task_comment;

create table user_task_comment (
    user_id int(3) unsigned not null,
    task_id int(5) not null,
    comment_date date not null,
    comments text not null,
    constraint pk_user_task_comment primary key (user_id, task_id, comment_date),
    index user_task_comment_user_i (user_id),
    index user_task_comment_task_i (task_id),
    index user_task_comment_user_date_i (user_id, comment_date),
    constraint fk_user_task_comment_user foreign key (user_id) references at_user(id) on delete cascade,
    constraint fk_user_task_comment_task foreign key (task_id) references task(id) on delete cascade
)
engine=InnoDB
default character set=utf8
default collate=utf8_unicode_ci;

insert into user_task_comment( user_id, task_id, comment_date, comments )
    select user_id, task_id, comment_date, comments from user_task_comment_temp;

drop table user_task_comment_temp;

set foreign_key_checks=1;

-- ------------------------------------------------

create table user_notif_rule_send (
    id int(5) unsigned not null,
    rule_id int(3) unsigned not null,
    user_id int(3) unsigned not null,
    notification_date date not null,
    range_start date,
    range_end date,
    constraint pk_user_notif_rule_send primary key (id),
    constraint fk_user_notif_rule_send_rule foreign key(rule_id) references notification_rule(id) on delete cascade,
    constraint fk_user_notif_rule_send_user foreign key(user_id) references at_user(id) on delete cascade
)engine=InnoDB
default character set=utf8
default collate=utf8_unicode_ci;

create table project_user_notif_rule_send (
    user_notif_rule_send_id int(5) unsigned not null,
    project_id int(3) not null,
    constraint pk_project_user_notif_rule_send primary key (user_notif_rule_send_id, project_id),
    constraint fk_project_user_notif_rule_send_id foreign key(user_notif_rule_send_id) references user_notif_rule_send(id) on delete cascade,
    constraint fk_project_user_notif_rule_send_project foreign key(project_id) references project(id) on delete cascade
)engine=InnoDB
default character set=utf8
default collate=utf8_unicode_ci;

create table user_user_notif_rule_send (
    user_notif_rule_send_id int(5) unsigned not null,
    user_id int(3) unsigned not null,
    constraint pk_user_user_notif_rule_send primary key (user_notif_rule_send_id, user_id),
    constraint fk_user_user_notif_rule_send_id foreign key(user_notif_rule_send_id) references user_notif_rule_send(id) on delete cascade,
    constraint fk_user_user_notif_rule_send_user foreign key(user_id) references at_user(id) on delete cascade
)engine=InnoDB
default character set=utf8
default collate=utf8_unicode_ci;

create table task_user_notif_rule_send (
    user_notif_rule_send_id int(5) unsigned not null,
    task_id int(5) not null,
    constraint pk_task_user_notif_rule_send primary key (user_notif_rule_send_id, task_id),
    constraint fk_task_user_notif_rule_send_id foreign key(user_notif_rule_send_id) references user_notif_rule_send(id) on delete cascade,
    constraint fk_task_user_notif_rule_send_task foreign key(task_id) references task(id) on delete cascade
)engine=InnoDB
default character set=utf8
default collate=utf8_unicode_ci;

ALTER TABLE system_settings MODIFY property_value text CHARACTER SET utf8 COLLATE utf8_unicode_ci;

-- ----------------------------------------------------
-- Reports restructuring

alter table report_configs add column description text default null;
alter table report_configs add column format_id int not null default 1; -- default is HTML
alter table report_configs drop key name_lower;


--  Copy report configs to all users
alter table report_configs add column user_id int unsigned;
alter table report_configs add column original_id int(3) not null;
-- duplicating time reports
insert into report_configs (name, name_lower, report_form, original_id, user_id)
    select rc.name, rc.name_lower, rc.report_form, rc.id, u.id
        from report_configs rc, at_user u inner join user_access_right ar on (u.id = ar.user_id)
        where ar.access_right_id = 1 and rc.report_form in (1,4,6,9,10);
-- duplicating money reports
insert into report_configs (name, name_lower, report_form, original_id, user_id)
    select rc.name, rc.name_lower, rc.report_form, rc.id, u.id
    from report_configs rc, at_user u inner join user_access_right ar on (u.id = ar.user_id)
    where access_right_id = 8 and rc.report_form in (3,7,8,12);
insert into report_params (config_id, param_name, param_value) select rc1.id, rp.param_name, rp.param_value from (report_configs rc1 inner join report_configs  rc2 on (rc1.original_id = rc2.id)) inner join report_params rp on (rp.config_id = rc2.id);
delete from report_params where config_id in (select id from report_configs rc where rc.user_id is null );
delete from report_configs where user_id is null;
alter table report_configs drop column original_id;
alter table report_configs modify column user_id int unsigned not null;
alter table report_configs add unique key name_form_user (name_lower, report_form, user_id);

alter table report_configs
    add constraint fk_report_configs_user_id foreign key (user_id) references at_user (id) on delete cascade;

create table temp_report_params
(
  config_id int(3) not null,
  param_name varchar(255) not null,
  param_value text not null
) engine=InnoDB default character set=utf8 default collate=utf8_unicode_ci;

insert into temp_report_params select * from report_params;

drop table report_params;

create table report_params
(
  config_id int(3) not null,
  param_name varchar(255) not null,
  param_value text not null,
  index(config_id),
  constraint fk_report_configs foreign key (config_id) references report_configs (id) on delete cascade
) engine=InnoDB default character set=utf8 default collate=utf8_unicode_ci;

insert into report_params select * from temp_report_params;

drop table temp_report_params;


-- tables for storing selected customers and projects for the report configurations
create table report_params_customer
(
    config_id int(3),
    customer_id int(3),
    constraint fk_rpc_config_id foreign key (config_id) references report_configs(id) on delete cascade,
    constraint fk_rpc_customer_id foreign key (customer_id) references customer(id) on delete cascade,
    primary key (config_id, customer_id)
);


create table report_params_project
(
    config_id int(3),
    project_id int(3),
    constraint fk_rpp_config_id foreign key (config_id) references report_configs(id) on delete cascade,
    constraint fk_rpp_project_id foreign key (project_id) references project(id) on delete cascade,
    primary key (config_id, project_id)
);

-- normalizing projects and customers
insert into report_params_customer (config_id, customer_id)
    select config_id, cr.id
    from report_params, customer cr
    where param_name='customersProjectsSelector.customerList'
        and position( concat(',', cast(cr.id as char CHARACTER SET utf8) collate utf8_unicode_ci, ',') in concat(',', param_value, ',') ) > 0;

insert into report_params_project (config_id, project_id)
    select config_id, p.id
    from report_params, project p
    where param_name='customersProjectsSelector.projectList'
        and position( concat(',', cast(p.id as char CHARACTER SET utf8) collate utf8_unicode_ci, ',') in concat(',', param_value, ',') ) > 0;

delete from  report_params where param_name='customersProjectsSelector.projectList'
                              or param_name='customersProjectsSelector.customerList';

update report_params rp set param_value = 'date.state.today'
    where param_value='range.state.today'
          and exists(select id from report_configs rc where rc.id=rp.config_id and rc.report_form=9);
update report_params rp set param_value = 'date.state.custom'
    where param_name='dateRangeState'
          and (param_value is null or param_value = '')
          and exists(select id from report_configs rc where rc.id=rp.config_id and rc.report_form=9);

-- invoice params
alter table invoice_params add column config_id int(3) null; -- null for default configuration without saved report_config (user_id must be specified)
alter table invoice_params add column user_id int(3) unsigned null; -- null for saved configuration, report_config must be specified
alter table invoice_params add constraint fk_invoice_params_config_id foreign key (config_id) references report_configs(id) on delete cascade;
alter table invoice_params add constraint fk_invoice_params_user_id foreign key (user_id) references at_user(id) on delete no action;
alter table invoice_params drop index uk_invoice_params_param_name;
alter table invoice_params add constraint uk_invoice_params_config_id_param_name unique (user_id, config_id, param_name);

insert into invoice_params(param_name, param_value, config_id, user_id) select 'billTo', entry, null, null from invoice_bill_to_recent_entry where position in (select min(position) from invoice_bill_to_recent_entry);

insert into invoice_params(param_name, param_value, user_id, config_id)
    select inv.param_name, inv.param_value, u.id, null from invoice_params inv, at_user u where inv.config_id is null and inv.user_id is null;
delete from invoice_params where user_id is null and config_id is null;

alter table invoice_tax add column config_id int(3) null;
alter table invoice_tax add column user_id int(3) unsigned null;
alter table invoice_tax add constraint fk_invocie_tax_config_id foreign key (config_id) references report_configs(id) on delete cascade;
alter table invoice_tax add constraint fk_invocie_tax_user_id foreign key (user_id) references at_user(id) on delete no action;
alter table invoice_tax drop index uk_invoice_tax_position;
alter table invoice_tax add constraint uk_invoice_tax_config_id_position unique (user_id, config_id, position);

insert into invoice_tax (position, name, rate, is_percent, user_id, config_id)
    select t.position, t.name, t.rate, t.is_percent, u.id, null from invoice_tax t, at_user u where t.config_id is null and t.user_id is null;
delete from invoice_tax where user_id is null and config_id is null;

alter table invoice_discount add column config_id int(3) null;
alter table invoice_discount add column user_id int(3) unsigned null;
alter table invoice_discount add constraint fk_invoice_discount_config_id foreign key (config_id) references report_configs(id) on delete cascade;
alter table invoice_discount add constraint fk_invoice_discount_user_id foreign key (user_id) references at_user(id) on delete no action;
alter table invoice_discount drop index uk_invoice_discount_position;
alter table invoice_discount add constraint uk_invoice_discount_config_id_position unique (user_id, config_id, position);

insert into invoice_discount (position, name, user_id, config_id)
    select d.position, d.name, u.id, null from invoice_discount d, at_user u where d.config_id is null and d.user_id is null;
delete from invoice_discount where user_id is null and config_id is null;

create table invoice_last_used_number (
    invoice_number varchar(255) not null
)
engine=InnoDB
default character set=utf8
default collate=utf8_general_ci;

insert into invoice_last_used_number(invoice_number) select param_value from invoice_params where param_name='lastNumber';
delete from invoice_params where param_name='lastNumber';

create table temp_invoice_bill_to_recent_entry like invoice_bill_to_recent_entry;
insert into temp_invoice_bill_to_recent_entry (entry, normalized_entry, position) select entry, normalized_entry, position from invoice_bill_to_recent_entry;
delete from invoice_bill_to_recent_entry;
alter table invoice_bill_to_recent_entry add column user_id int(3) unsigned not null;
alter table invoice_bill_to_recent_entry add constraint fk_invoice_bill_to_recent_entry_user_id foreign key (user_id) references at_user(id) on delete cascade;
alter table invoice_bill_to_recent_entry drop index uk_invoice_bill_to_recent_entry_position;
alter table invoice_bill_to_recent_entry add constraint uk_invoice_bill_to_recent_entry_position_user unique key (user_id, position);
insert into invoice_bill_to_recent_entry (entry, normalized_entry, position, user_id)
    select t.entry, t.normalized_entry, t.position, u.id from temp_invoice_bill_to_recent_entry t, at_user u;
drop table temp_invoice_bill_to_recent_entry;

create table temp_invoice_terms like invoice_terms;
insert into temp_invoice_terms (entry, normalized_entry, position) select entry, normalized_entry, position from invoice_terms;
delete from invoice_terms;
alter table invoice_terms add column user_id int(3) unsigned not null;
alter table invoice_terms add constraint fk_invoice_terms_user_id foreign key (user_id) references at_user(id) on delete cascade;
alter table invoice_terms drop index uk_invoice_terms_position;
alter table invoice_terms add constraint uk_invoice_terms_position_user unique key (user_id, position);
insert into invoice_terms (entry, normalized_entry, position, user_id)
    select t.entry, t.normalized_entry, t.position, u.id from temp_invoice_terms t, at_user u;
drop table temp_invoice_terms;
-- ----------------------------------------------------

-- Sync service: revision numbers support

-- Users

create table at_user_revision (
    id int unsigned not null,
    revision int unsigned not null,
    constraint pk_user_revision primary key (id),
    constraint uk_user_revision_value unique key (revision)
)
engine=InnoDB;

insert into at_user_revision( id, revision ) select id, id as revision from at_user;

alter table at_user add constraint fk_at_user_revision foreign key(id) references at_user_revision(id) on delete no action;


-- Customers

create table customer_revision (
    id int(3) not null,
    revision int unsigned not null,
    constraint pk_customer_revision primary key (id),
    constraint uk_customer_revision_value unique key (revision)
)
engine=InnoDB;

insert into customer_revision( id, revision ) select id, id as revision from customer;

alter table customer add constraint fk_customer_revision foreign key(id) references customer_revision(id) on delete no action;


-- Projects

create table project_revision (
    id int(3) not null,
    revision int unsigned not null,
    constraint pk_project_revision primary key (id),
    constraint uk_project_revision unique key (revision)
)
engine=InnoDB;

insert into project_revision( id, revision ) select id, id as revision from project;

alter table project add constraint fk_project_revision foreign key(id) references project_revision(id) on delete no action;


-- Tasks

create table task_revision (
    id int(3) not null,
    revision int unsigned not null,
    constraint pk_task_revision primary key (id),
    constraint uk_task_revision unique key (revision)    
)
engine=InnoDB;

insert into task_revision (id, revision) select id, id as revision from task;

alter table task add constraint fk_task_revision foreign key (id) references task_revision (id) on delete no action;


-- Billing types

create table billing_type_revision (
    id int(3) unsigned not null,
    revision int unsigned not null,
    constraint pk_billing_type_revision primary key (id),
    constraint uk_billing_type_revision unique key (revision)
)
engine=InnoDB;

insert into billing_type_revision select id, id from billing_type;

alter table billing_type add constraint fk_billing_type_revision foreign key (id) references billing_type_revision (id) on delete no action;


-- Leave types

create table leave_type_revision (
    id int(3) unsigned not null,
    revision int unsigned not null,
    constraint pk_leave_type_revision  primary key (id),
    constraint uk_leave_type_revision unique key (revision)
)
engine=InnoDB;

insert into leave_type_revision select id, id as revision from leave_type;

alter table leave_type add constraint fk_leave_type_revision foreign key (id) references leave_type_revision (id) on delete no action;


-- We need to generate unique revision numbers for entities that do not have numerical IDs.
-- The approach is to create revision column with 'auto_increment' option, let DBMS populate it while inserting values
-- and then remove 'auto_increment' from column definition.

-- Assignments

create table user_project_revision (
    user_id int(3) unsigned not null,
    project_id int(6) not null,
    revision int unsigned not null auto_increment,
    constraint uk_user_project_revision unique key (revision)
)
engine=InnoDB;

insert into user_project_revision (user_id, project_id) select user_id, project_id from user_project;

alter table user_project_revision modify revision int unsigned not null;
alter table user_project_revision add constraint pk_user_project_revision primary key (user_id, project_id);

alter table user_project add constraint fk_user_project_revision foreign key (user_id, project_id) references user_project_revision(user_id, project_id) on delete no action;


--  TT records and comments

create table tt_revision (
    user_id int(3) unsigned not null,
    task_id int(3) not null,
    record_date date not null,
    revision int unsigned not null auto_increment,
    constraint uk_tt_revision unique key (revision)
)
engine=InnoDB;

insert into tt_revision (user_id, task_id, record_date)
    select user_id, task_id, record_date from tt_record union select user_id, task_id, comment_date from user_task_comment;

alter table tt_revision modify revision int unsigned not null;
alter table tt_revision add constraint pk_tt_revision primary key (user_id, task_id, record_date);

alter table tt_record add constraint fk_tt_record_revision foreign key (user_id, task_id, record_date) references tt_revision (user_id, task_id, record_date) on delete no action;
alter table user_task_comment add constraint fk_user_task_comment_revision foreign key (user_id, task_id, comment_date) references tt_revision (user_id, task_id, record_date) on delete no action;


-- Leaves

create table user_date_leave_revision (
    user_id int(3) unsigned not null,
    leave_date date not null,
    revision int unsigned not null auto_increment,
    constraint uk_user_date_leave_revision unique key (revision)
)
engine=InnoDB;

insert into user_date_leave_revision (user_id, leave_date) select user_id, leave_date from user_date_leave;

alter table user_date_leave_revision modify revision int unsigned not null;
alter table user_date_leave_revision add constraint pk_user_date_leave_revision primary key (user_id, leave_date);

alter table user_date_leave add constraint fk_user_date_leave_revision foreign key (user_id, leave_date) references user_date_leave_revision (user_id, leave_date) on delete no action;


-- Inverted days

create table w_days_revision (
    w_date date not null,
    revision int unsigned not null auto_increment,
    constraint uk_w_days_revision unique key (revision)
)
engine=InnoDB;

insert into w_days_revision (w_date) select w_date from w_days;

alter table w_days_revision modify revision int unsigned not null;
alter table w_days_revision add constraint pk_w_days_revision primary key (w_date);

alter table w_days add constraint fk_w_days_revision foreign key (w_date) references w_days_revision (w_date) on delete no action;


-- System settings

create table system_settings_revision (
    revision int unsigned not null
)
engine=InnoDB;

insert into system_settings_revision (revision) values (1);

-- ----------------------------------------------------

create table user_without_mobile_button (
    user_id int(3) unsigned not null,
    primary key(user_id),
    constraint user_without_mobile_button_user_id
        foreign key (user_id) references at_user(id) on delete cascade
)
engine=InnoDB;


insert
  into patch_history (patch, pack_version, patch_timestamp)
  values (17, 0, now());

commit;
