insert into user_access_right (user_id, access_right_id)
	select 1, id from access_right where id <> 9;

create temporary table _temp_rights(
    user_id int(3) unsigned not null,
    access_right_id int(3) unsigned not null
) ENGINE=InnoDB;

insert into _temp_rights(user_id, access_right_id)
    select user_id, 2 from user_access_right where access_right_id in (2,3,4) group by user_id;

delete from user_access_right where access_right_id in (2,3,4);

insert into user_access_right(user_id, access_right_id)
    select user_id, access_right_id from _temp_rights;

delete from access_right where id in (3,4);

update access_right set name='tasks.manage_customers_projects_tasks', description='Manage Customers, Projects & Tasks' where id = 2;

update access_right set name='system.manage_work_schedule', description='Manage Work Schedule' where id = 10;

insert into access_right (id, name, description) values (11, 'system.manage_general_settings', 'Manage General Settings');

create temporary table _temp_users(
    user_id int(3) unsigned not null
) ENGINE=InnoDB;

insert into _temp_users (user_id)
    select user_id from user_access_right where access_right_id = 10;

insert into user_access_right (user_id, access_right_id)
    select user_id, 11 from _temp_users;

set foreign_key_checks=0;
set unique_checks=0;

-- save tasks
create temporary table _temp_task
(
  id int(5) not null auto_increment,
  project_id int not null,
  create_timestamp datetime not null,
  completion_date date,
  last_tt_date date,
  name varchar(255) not null,
  name_lower varchar(255) not null,
  description text,
  deadline_date date,
  billing_type_id int,
  primary key(id)
) ENGINE=InnoDB;

insert into _temp_task(id, project_id, create_timestamp, completion_date, last_tt_date, name, name_lower,
		     description, deadline_date, billing_type_id)
select t.id, t.project_id, t.create_timestamp, t.completion_date, t.last_tt_date, 
       t.name, t.name_lower, t.description, t.deadline_date, 
       if(t.billing_status and p.billing_status and c.billing_status, t.billing_type_id, null)
from task t, customer c, project p
where t.project_id=p.id and p.customer_id=c.id;

drop table task;

alter table customer drop column billing_status;

-- save projects
create temporary table _temp_project
(
  id int(3) not null auto_increment,
  customer_id int not null,
  create_timestamp datetime not null,
  name varchar(255) not null,
  name_lower varchar(255) not null,
  description text,
  primary key(id)
) ENGINE=InnoDB;

insert into _temp_project(id, customer_id, create_timestamp, name, name_lower, description)
select id, customer_id, create_timestamp, name, name_lower, description
from project;

drop table project;

-- save billing types
create temporary table _temp_billing_type
(
  id int(3) not null,
  name varchar(255),
  name_lower varchar(255)
) ENGINE=InnoDB;

insert into _temp_billing_type(id, name, name_lower)
    select id, name, name_lower from billing_type where id is not null;

drop table billing_type;

-- restore billing types
create table billing_type
(
    id int(3) unsigned not null auto_increment,
    name varchar(255) not null,
    name_lower varchar(255) not null,
    is_default bit null default null,
    primary key (id),
    unique(name_lower)
) ENGINE=InnoDB;

insert into billing_type(id, name, name_lower)
    select id, name, name_lower from _temp_billing_type;

-- restore tasks
create table task
(
  id int(5) not null auto_increment,
  project_id int not null,
  create_timestamp datetime not null,
  completion_date date,
  last_tt_date date,
  name varchar(255) not null,
  name_lower varchar(255) not null,
  description text,
  deadline_date date,
  billing_type_id int unsigned null,
  primary key(id),
  unique(project_id, name_lower),
  index(billing_type_id),
  foreign key(billing_type_id) references billing_type(id),
  index(project_id),
  foreign key(project_id) references project(id)
) ENGINE=InnoDB;

insert into task 
  select * from _temp_task;

-- restore projects
create table project
(
  id int(3) not null auto_increment,
  customer_id int not null,
  create_timestamp datetime not null,
  name varchar(255) not null,
  name_lower varchar(255) not null,
  description text,
  primary key(id),
  unique(customer_id, name_lower),
  index(customer_id),
  foreign key(customer_id) references customer(id)
) ENGINE=InnoDB;

insert into project
  select * from _temp_project;

set unique_checks=1;
set foreign_key_checks=1;

create table system_settings
(
    property_name varchar(32) not null,
    property_value varchar(255) null,
    unique(property_name)
) ENGINE=InnoDB;

insert into system_settings(property_name, property_value) values ('date_format', 'MMM dd, yyyy');
insert into system_settings(property_name, property_value) values ('maximum_hours_per_day', '1440');
insert into system_settings(property_name, property_value) values ('overtime_undertime_tracking', 'overtime');
insert into system_settings(property_name, property_value) values ('tt_approval_enabled', 'true');

insert into system_settings(property_name, property_value)
    select 'calendar_layout', cal_layout from system_prefs;
insert into system_settings(property_name, property_value)
    select 'workday_duration', default_worktime from system_prefs;

drop table system_prefs;

alter table customer add column archiving_timestamp datetime null;
alter table project add column archiving_timestamp datetime null;

alter table at_user drop column is_root;

alter table at_user add column overtime_level int null;

update at_user set overtime_level = 2 where not overtime_tracking = 0;
update at_user set overtime_level = 0 where overtime_tracking = 0;

delete from overtime where overtime is null;
alter table overtime modify column overtime int not null;

insert into patch_history(patch, patch_timestamp)
    values(3, now());

commit;