drop table user_new_crm_resource;
drop table crm_resource;

alter table user_work_schedule_calendar drop constraint fk_user_ws_calendar_user;
alter table user_seen_gui_notif drop constraint gui_notification_user_id_fk_at_user_id;
alter table tt_lock drop constraint tt_lock_who_locked_id;

delete from user_settings;
alter table user_settings drop column param_value;
alter table user_settings add column param_value longtext null;
alter table user_settings add param_category varchar(255) not null;

drop index k_user_settings_user_id_param_name on user_settings;
create index k_user_settings_user_id_param_category ON user_settings (user_id, param_category);

alter table at_user add is_enabled_ap bit NOT NULL;
alter table at_user add initial_schedule varchar(255) not null;
alter table at_user add initial_schedule_total int not null;
alter table at_user add is_default_initial_schedule bit not null;
alter table at_user add first_name_lower varchar(32) NOT NULL;
alter table at_user add last_name_lower varchar(32) NOT NULL;
alter table at_user add middle_name_lower varchar(32) NULL;

update at_user set first_name_lower = LCase( first_name ),
                   last_name_lower = LCase( last_name ),
                   middle_name_lower = LCase( middle_name );


update at_user set initial_schedule = '0,' & workday_duration &','& workday_duration &','& workday_duration &','& workday_duration &','& workday_duration & ',0',
                   initial_schedule_total = workday_duration * 5,
                   is_default_initial_schedule=(workday_duration=CInt( IIF( IsNull( DLookup("property_value" , "system_settings" , "property_name='workday_duration'") ), "480", DLookup("property_value" , "system_settings" , "property_name='workday_duration'")) ));



update mail_template set name = 'shared_macro_at.vm' where name = 'shared_macro.vm';

CREATE TABLE user_schedule (
  user_id int NOT NULL,
  effective_date date NOT NULL,
  schedule varchar(255) NOT NULL,
  week_total int NOT NULL,
  is_default bit NOT NULL,
  PRIMARY KEY (user_id, effective_date),
  CONSTRAINT user_schedule_at_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE
);


alter table at_user drop column workday_duration;

-- code below is intended to delete 'name_lower' column
-- it is so sophisticated because ms access can't delete column with unnamed unique constraint

select id, name into user_group_temp from user_group;

alter table at_user drop constraint fk_at_user_user_group;

drop table user_group;

create table user_group (
    id int not null,
    name varchar(40) not null,
    primary key (id),
    unique(name)
);

insert into user_group(id, name) select id, name from user_group_temp;

alter table at_user
    add constraint fk_at_user_user_group foreign key(user_group_id) references user_group(id) on delete set null;

drop table user_group_temp;

-- ---------------------------------------------------
-- Prepare non-billable default billing type with id=1
-- ---------------------------------------------------

-- 1. Turn off required constraint while converting data

ALTER TABLE task DROP constraint fk_task_billing_type;
alter table billing_type drop constraint fk_billing_type_revision;

alter table billing_type alter column id int not null;

-- 2. Free id=2 by setting the next id for the billing type with id=2

update billing_type_revision set id = DMAX("id", "billing_type_revision")+1 where id=2;
update billing_type set id = DMAX("id", "billing_type_revision") where id=2;

update task_revision set revision = revision+DMAX('revision','task_revision','') where id in (select id from task where billing_type_id=2);
update task set billing_type_id = DMAX("id", "billing_type_revision") where billing_type_id=2;

-- 3. Set id=2 to the existing candidate to our default billing type

update billing_type set rate=-123.456789
where name_lower = "non-billable" and is_billable = false and is_active=true and rate is null;

update task_revision set revision = revision+DMAX('revision','task_revision','') where id in (select id from task where billing_type_id in (select id from billing_type where rate = -123.456789 and is_billable = false));
update task set billing_type_id=2 where billing_type_id in (select id from billing_type where rate = -123.456789 and is_billable = false);

update billing_type_revision as t set t.id = 2 where  t.id in (select id from billing_type where rate = -123.456789 and is_billable = false);

update billing_type set id = 2, rate=null where rate = -123.456789 and is_billable = false;

-- 4. Create a new billing type if there are no candidates

insert into billing_type_revision (id, revision)
  select 2, iif(DMAX("revision", "billing_type_revision") is null, 1, DMAX("revision", "billing_type_revision")+1) from (select count(*) from billing_type_revision)
    where not exists (select 1 from billing_type_revision where id=2);

insert into billing_type (id, is_active, is_billable, is_default, `name`, name_lower, `position`)
select top 1
  2, true, false,
  DMAX("is_default", "billing_type", "is_default=true") is null,
  iif(dlookup("name_lower", "billing_type", "name='non-billable'") is null, "Non-Billable", "Non-Billable (created automatically)"),
  iif(dlookup("name_lower", "billing_type", "name='non-billable'") is null, "non-billable", "non-billable (created automatically) 3058tysbdo9283hfnsdidjksls4034DF345SDt34ge456sefsdfsdfsdfsdfs"),
  iif(DMAX("position", "billing_type") is null, 1, DMAX("position", "billing_type")+1)
from
  (select count(*) from billing_type_revision)
where
  not exists (select 1 from billing_type where id=2);

-- make sure we have the default billing type
update billing_type set
  is_default=iif(dlookup("id", "billing_type", "is_default=true") is null, true, dlookup("is_default", "billing_type", "id=2"))
where id=2;

-- 5. Update revisions of all existing billing types

update billing_type_revision set revision=revision+DMAX('revision','billing_type_revision','')
where id=2 or id in ( select id from billing_type );

-- 6. Turn on the required constraint back

alter table billing_type add constraint fk_billing_type_revision foreign key (id) references billing_type_revision (id) on delete no action;
ALTER TABLE task ADD CONSTRAINT fk_task_billing_type FOREIGN KEY(billing_type_id) REFERENCES billing_type(id) on update cascade;





-- Add 'parent_id' column to task in order to specify relationship task - subTask
alter table task add column parent_id int;

alter table task drop constraint pn_task;
alter table task add constraint task_project_name_uk unique (project_id, name_lower, parent_id);
alter table task add constraint fk_parent_task_id foreign key (parent_id) references task(id) on delete cascade;

-- Change TASK grouping levels to TASK_AND_SUBTASK for Estimate vs. Actuals Report
update report_params set param_value = 10
		where param_name in( 'firstGrouping', 'secondGrouping', 'thirdGrouping','detailsLevel') and
			  param_value = '4' and
		      config_id in ( select id from report_configs where report_form = 9 );

-- Import updated email notification templates
update mail_template
    set template = '## some email clients do not support embedded or external css, so we define several commonly used
## rules here as velocity variables to re-use them later for inline css
#macro( initStyles )
    #set($css_details_font_size=''font-size: 13px; '')
    #set($css_report_link_font_size=''font-size: 13px; '')
    #set($css_details_header_color=''color: #595959; '')
    #set($css_light_border=''1px solid #e9e9e9 '')
    #set($css_dark_border=''1px solid #d9d9d9 '')
    #set($css_outer_border=''1px solid #cccccc '')
    #set($css_darkest_border=''1px solid #999999 '')
    #set($css_range_total_border=''1px solid #8799af '')
    #set($css_link_color=''color: #003399 '')
    #set($css_light_border_left=''border-left: '' + $css_light_border + ''; '')
    #set($css_light_border_right=''border-right: '' + $css_light_border + ''; '')
    #set($css_outer_border_left=''border-left: '' + $css_outer_border + ''; '')
    #set($css_outer_border_right=''border-right: '' + $css_outer_border + ''; '')
    #set($css_outer_border_bottom=''border-bottom: '' + $css_outer_border + ''; '')
    #set($css_outer_border_top=''border-top: '' + $css_outer_border + ''; '')
    #set($css_dark_border_left=''border-left: '' + $css_dark_border + ''; '')
    #set($css_dark_border_right=''border-right: '' + $css_dark_border + ''; '')
    #set($css_dark_border_top=''border-top: '' + $css_dark_border + ''; '')
    #set($css_darkest_border_bottom=''border-bottom: '' + $css_darkest_border + ''; '')
    #set($css_cp_border_bottom=''border-bottom: '' + $css_dark_border + ''; '')
    #set($css_range_total_border_bottom=''border-bottom: '' + $css_range_total_border + ''; '')
    #set($css_range_total_border_top=''border-top: '' + $css_range_total_border + ''; '')
    #set($css_bg_header=''background-color: #366A98; '')
    #set($css_bg_total=''background-color: #f8f4e9; '')
    #set($css_bg_total_middle=''background-color: #f1eee5; '')
    #set($css_bg_range_total=''background-color: #eef2f7; '')
    #set($css_bg_range_total_middle=''background-color: #e9edf1; '')
    #set($css_even_row_border_top=''border-top: 1px solid #f0eee7; '')
    #set($css_even_row_border_bottom=''border-bottom: 1px solid #f0eee7; '')
    #set($css_text=''font-family: Arial Unicode MS, Arial, sans-serif; font-weight: normal; '')
    #set($css_header_text=$css_text + '' color: white; font-size: 14px; '')
    #set($css_total_text=$css_text + '' color: #665946; '')
    #set($textValueColor=''#366A98'')
    #set($css_parameters_text=$css_text + '' font-size: 14px; '')
    #set($css_parameters_name_text=$css_parameters_text + '' color: '' + $textValueColor + '';'')
    #set($css_table_title_text=$css_text + '' color: #6A573B; font-size: 14px; font-weight: bold; '')
    #set($missedColor=''#C00000'')
#end
#macro( drawHeader )
    <thead>
        <tr>
            <th valign=''middle'' style=''
                                       text-align: left;
                                       padding: 10px 8px 10px 15px;
                                       $css_bg_header
                                       $css_header_text
                                       $css_outer_border_top
                                       $css_outer_border_left''>
                <span style=''font-size: 14px; margin-bottom: 7px; margin-top: 10px''><b>$hls.Customer() / $hls.Project()</b></span><br>
                <span style=''font-size: 12px;''>$hls.TasksAMPSubtasks()</span>
            </th>
            <th valign=''middle'' style=''
                                       text-align: center;
                                       padding: 10px 8px;
                                       $css_bg_header
                                       $css_header_text
                                       $css_outer_border_top
                                       $css_light_border_right
                                       $css_light_border_left''><b>Spent</b></th>
            <th valign=''middle'' style=''
                                       text-align: left;
                                       padding: 17px 8px 17px 8px;
                                       $css_bg_header
                                       $css_header_text
                                       $css_outer_border_top
                                       $css_outer_border_right''><b><span>Comments</span></b></th>
        </tr>
    </thead>
#end
#macro( drawDayTT $day)
    #set( $oddRow = true )
    #set( $drawCPEntryTopBorder = false )
    #if( $day.leaveTime > 0)
        <tr>
            <td valign=''bottom'' style=''
                                       text-align: left;
                                       background-color: #FCF8EF;
                                       $css_details_header_color
                                       padding: 20px 8px 8px 15px;
                                       $css_text
                                       $css_outer_border_left
                                       border-bottom: 1px dashed #AFAFAF ;
                                       ''><b><span style=''font-size: 14px;''>$day.leaveName</span></b></td>
            <td valign=''bottom'' style=''
                                       text-align: right;
                                       font-size: 14px;
                                       padding: 20px 15px 8px 8px;
                                       background-color: #F5F1EA;
                                       $css_text
                                       border-bottom: 1px dashed #BFBFBF ;
                                       $css_light_border_right
                                       $css_light_border_left
                                       ''><b>$day.formattedLeaveTime</b></td>
            <td style=''
                       background-color: #FCF8EF;
                       padding: 20px 8px 8px 15px;
                       $css_outer_border_right
                       border-bottom: 1px dashed #AFAFAF ;
                       ''>&nbsp;</td>
        </tr>
    #end
    #foreach( $cpEntry in $day.rows.entrySet() )
        <tr>
            <td valign=''bottom'' style=''
                                       text-align: left;
                                       background-color: #f2f2f2;
                                       padding: 20px 8px 8px 15px;
                                       $css_text
                                       $css_outer_border_left
                                       $css_cp_border_bottom
                                       #if($drawCPEntryTopBorder) $css_dark_border_top #end''>
                <b><span style=''font-size: 14px; $css_details_header_color''>$cpEntry.key.customerName /</span>
                    <span style=''$css_parameters_name_text font-weight: bold; ''>$cpEntry.key.projectName</span></b>
            </td>
            <td valign=''bottom'' style=''
                                       text-align: right;
                                       font-size: 14px;
                                       padding: 20px 15px 8px 8px;
                                       background-color: #ededed;
                                       $css_text
                                       $css_cp_border_bottom
                                       $css_light_border_right
                                       $css_light_border_left
                                       #if($drawCPEntryTopBorder) $css_dark_border_top #end''>
                <b>$day.getFormattedSubTotal($cpEntry.key)</b></td>
            <td style=''
                       background-color: #f2f2f2;
                       padding: 20px 8px 8px 15px;
                       $css_outer_border_right
                       $css_cp_border_bottom
                       #if($drawCPEntryTopBorder) $css_dark_border_top #end''>&nbsp;</td>
        </tr>
        #set( $drawCPEntryTopBorder = true )
        #set ( $rowNumber = 0 )
        #foreach( $ttRow in $cpEntry.value )
            #set ( $drawEvenBorderTop = ($rowNumber != 0) )
            #set ( $drawEvenBorderBottom = ($rowNumber < ($cpEntry.value.size() - 1)) )
            #if( $oddRow  )
                <tr>
                    <td valign=''middle'' style=''
                                               text-align: left;
                                               padding: 8px 8px 8px 35px;
                                               font-size: 13px;
                                               background-color: #ffffff;
                                               $css_text
                                               $css_outer_border_left''>$ttRow.taskName</td>
                    <td valign=''middle'' style=''
                                               text-align: right;
                                               font-size: 13px;
                                               background-color: #f7f7f7;
                                               padding: 8px 15px 8px 8px;
                                               $css_text
                                               $css_light_border_right
                                               $css_light_border_left''>$ttRow.formattedSpent</td>
                    <td valign=''middle'' style=''
                                               text-align: left;
                                               font-size: 13px;
                                               background-color: #ffffff;
                                               padding: 8px 8px 8px 18px;
                                               $css_text
                                               $css_outer_border_right''>$ttRow.comments</td>
                </tr>
            #else
                <tr>
                    <td valign=''middle'' style=''
                                               text-align: left;
                                               padding: 8px 8px 8px 35px;
                                               font-size: 13px;
                                               background-color: #fcfaf3;
                                               $css_text
                                               $css_outer_border_left
                                               #if($drawEvenBorderTop) $css_even_row_border_top #end
                                               #if($drawEvenBorderBottom) $css_even_row_border_bottom #end
                                               ''>$ttRow.taskName</td>
                    <td valign=''middle'' style=''
                                               text-align: right;
                                               font-size: 13px;
                                               background-color: #f5f3ed;
                                               padding: 8px 15px 8px 8px;
                                               $css_text
                                               #if($drawEvenBorderTop) $css_even_row_border_top #end
                                               #if($drawEvenBorderBottom) $css_even_row_border_bottom #end
                                               $css_light_border_right
                                               $css_light_border_left
                                               ''>$ttRow.formattedSpent</td>
                    <td valign=''middle'' style=''
                                               text-align: left;
                                               font-size: 13px;
                                               background-color: #fcfaf3;
                                               $css_text
                                               padding: 8px 8px 8px 18px;
                                               $css_outer_border_right
                                               #if($drawEvenBorderTop) $css_even_row_border_top #end
                                               #if($drawEvenBorderBottom) $css_even_row_border_bottom #end
                                               ''>$ttRow.comments</td>
                </tr>
            #end
            #set( $oddRow = !$oddRow )
            #set( $rowNumber = ( $rowNumber + 1 ) )
        #end
    #end
#end
#macro( drawDayTotal $day $displaySettings $drawTopBorder)
    <tr>
        <td valign=''middle'' style=''text-align: right;
                                   font-size: 15px;
                                   padding: 10px 8px 6px 18px;
                                   $css_total_text
                                   $css_bg_total
                                   $css_outer_border_left
                                   #if($drawTopBorder) $css_outer_border_top #end
                                   #if(!$displaySettings.showCalculatedOvertime) $css_outer_border_bottom #end''>
            <b>Total:</b></td>
        <td valign=''middle'' style=''
                                   text-align: right;
                                   font-size: 15px;
                                   $css_text
                                   padding: 10px 15px 6px 8px;
                                   $css_bg_total_middle
                                   $css_light_border_left
                                   $css_light_border_right
                                   #if($drawTopBorder) $css_outer_border_top #end
                                   #if(!$displaySettings.showCalculatedOvertime) $css_outer_border_bottom #end''>
            <b>$day.total.formattedTotal</b></td>
        <td valign=''middle'' style=''text-align: left;
                                   font-size: 13px;
                                   padding: 10px 18px 6px 18px;
                                   $css_text
                                   $css_bg_total
                                   $css_outer_border_right
                                   #if($drawTopBorder) $css_outer_border_top #end
                                   #if(!$displaySettings.showCalculatedOvertime) $css_outer_border_bottom #end''>
            <table border=''0'' cellpadding=''0'' cellspacing=''0''>
                <tr>
                    <td width=''80'' style=''
                                          text-align: left;
                                          font-size: 13px;
                                          $css_total_text
                                          $css_bg_total''>
                        <span style=''white-space: nowrap;''>Working&nbsp;Time:</span></td>
                    <td width=''40'' style=''
                                          text-align: right;
                                          font-size: 13px;
                                          padding-left: 5px;
                                          $css_text
                                          $css_bg_total''>$day.total.formattedWorking</td>
         #if($displaySettings.showLeaveTime)
             <td style=''
                        padding-left: 25px;
                        text-align: left;
                        font-size: 13px;
                        $css_total_text
                        $css_bg_total''>
              #if( $day.total.leaveName )${day.total.leaveName}#{else}Leave&nbsp;Time#end:</td>
             <td width=''40'' style=''
                                   text-align: right;
                                   font-size: 13px;
                                   padding-left: 5px;
                                   $css_text
                                   $css_bg_total''>$day.total.formattedLeave</td>
            #end
            <td> </td>
        </tr>
        </table>
        </td>
    </tr>
    #if($displaySettings.showReportedOvertime)
        <tr>
            <td valign=''top'' style=''
                                    text-align: right;
                                    font-size: 14px;
                                    padding: 0 8px 6px 15px;
                                    $css_total_text
                                    $css_bg_total
                                    $css_outer_border_left''>
                <span style=''white-space: nowrap;''>Reported&nbsp;Overtime #if($displaySettings.undertimeTracking)&nbsp;/&nbsp;Undertime #end:</span></td>
            <td valign=''top'' style=''
                                    text-align: right;
                                    font-size: 14px;
                                    padding: 0 15px 8px 8px;
                                    $css_text
                                    $css_bg_total_middle
                                    $css_light_border_left
                                    $css_light_border_right''>$day.total.formattedReportedOvertime</td>
            <td valign=''top'' style=''
                                    text-align: left;
                                    font-size: 14px;
                                    $css_text
                                    $css_bg_total
                                    $css_outer_border_right''>&nbsp;</td>
        </tr>
    #end
    #if($displaySettings.showCalculatedOvertime)
    <tr>
        <td valign=''top'' style=''
                                text-align: right;
                                font-size: 14px;
                                padding: 0 8px 14px 15px;
                                $css_total_text
                                $css_bg_total
                                $css_outer_border_left
                                $css_outer_border_bottom''><span style=''white-space: nowrap;''>Auto-calculated&nbsp;Overtime#if($displaySettings.undertimeTracking)&nbsp;/&nbsp;Undertime #end:</span></td>
        <td valign=''top'' style=''
                                text-align: right;
                                font-size: 14px;
                                padding: 0 15px 8px 8px;
                                $css_text
                                $css_bg_total_middle
                                $css_light_border_left
                                $css_light_border_right
                                $css_outer_border_bottom''>$day.total.formattedCalculatedOvertime</td>
        <td valign=''top'' style=''
                                text-align: left;
                                font-size: 14px;
                                $css_text
                                $css_bg_total
                                $css_outer_border_right
                                $css_outer_border_bottom''>&nbsp;</td>
    </tr>
    #end
#end

#macro ( drawCustomerProjectsTable $customerProjects )
<table cellpadding=''0'' cellspacing=''0'' border=''0'' style=''width: 100%; border-collapse:collapse;''>
    <thead>
        <tr>
            <th valign=''middle''
                style=''
                       text-align: left;
                       padding: 10px 8px 10px 15px;
                       $css_bg_header
                       $css_header_text
                       $css_outer_border_top
                       $css_outer_border_left''>
                    <b>$hls.Customer() / $hls.Projects()</b>
                </th>
          </tr>
    </thead>
    <tbody>
    #set($drawTopCPBorder = false)
    #set($customerNumber = 0)
    #set( $oddRow = true )
    #foreach( $customer in $customerProjects.keySet())
        #set( $rowNumber = 0 )
        #set( $projectsNumber = $customerProjects.get($customer).size() )
        <tr>
            <td style=''text-align: left;
                                       background-color: #f2f2f2;
                                       padding: 20px 8px 8px 15px;
                                       $css_details_header_color
                                       $css_details_font_size
                                       $css_text
                                       $css_outer_border_left
                                       $css_outer_border_right
                                       $css_cp_border_bottom
                                       font-weight: normal;
                                       #if($drawTopCPBorder) $css_dark_border_top #else $css_outer_border_top #end''>
                <b>$escapeUtil.getNameEscaped($customer)</b> ($projectsNumber #if ($projectsNumber > 1) $hls.projects() #else $hls.project() #end )
            </td>
        </tr>
        #set($drawTopCPBorder = true)
        #foreach( $project in $customerProjects.get($customer) )
            #set( $drawEvenBorderTop = ($rowNumber != 0) )
            #set( $drawEvenBorderBottom = ($rowNumber < ($projectsNumber - 1)) )
            #set( $drawOuterBorderBottom = (($customerNumber == ($customerProjects.keySet().size() - 1)) && !$drawEvenBorderBottom) )
            <tr>
                #if($oddRow)
                    <td valign=''bottom''
                        style=''
                                   text-align: left;
                                   $css_details_font_size
                                   background-color: #ffffff;
                                   padding: 8px 8px 8px 35px;
                                   $css_text
                                   $css_outer_border_left
                                   $css_outer_border_right
                                   #if($drawOuterBorderBottom) $css_outer_border_bottom #end''>
                        $escapeUtil.getNameEscaped($project)
                    </td>
                #else
                    <td valign=''bottom''
                        style=''
                                   text-align: left;
                                   $css_details_font_size
                                   background-color: #fcfaf3;
                                   padding: 8px 8px 8px 35px;
                                   $css_text
                                   $css_outer_border_left
                                   $css_outer_border_right
                                   #if($drawEvenBorderTop) $css_even_row_border_top #end
                                   #if($drawEvenBorderBottom) $css_even_row_border_bottom #end
                                   #if($drawOuterBorderBottom) $css_outer_border_bottom #end''>
                        $escapeUtil.getNameEscaped($project)
                    </td>
                #end
                #set($oddRow = !$oddRow)
                #set($rowNumber = ($rowNumber + 1))
            </tr>
        #end
        #set($customerNumber = ($customerNumber + 1))
    #end
    </tbody>
</table>
#end

#macro ( showFooter $footer )
    <br/>
    <hr noshade size="1" style="background-color: #333333; color: #333333;" />
    <pre style=''font-family: Arial Unicode MS, Arial, sans-serif; font-size: 12px;''>$footer</pre>
#end

#macro ( showYourActiTIMEFooter )
&ndash;&ndash;&ndash;<br/>Your actiTIME
#end
#macro ( showYourActiTIMEFooterPlainText )
- - -
Your actiTIME
#end', last_modified_timestamp = now()
        where name = 'shared_macro_at.vm';

update mail_template
    set template = '#initStyles()
<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
<table style=''width: 95%;$css_text font-size: 12px;''>
    <tr>
        <td style=''$css_text font-size: 14px; line-height:130%; background-color: #ffffff''>
            Dear <b>$recipient.fullNameWithoutUserNameEscaped</b>,<br><br>
            #set($tasksCount = $taskCount)
            #if($deadlineStatusKey) There#if($tasksCount > 1) are $tasksCount $hls.tasksSLASHsubtasks()#else is $tasksCount $hls.taskSLASHsubtask()#end with $messageUtil.message($deadlineStatusKey) deadline.<br><br>#end
            #set($shouldDrawSeparator = false)
            #foreach( $bean in $ruleBeans)
              #foreach ( $dateRange in $bean.projectTasksMap.keySet() )
                  #if($shouldDrawSeparator)
                        <div style='' font-size:1px; line-height:1px;''><div style=''margin:25px 0px 25px 0px;  width:100%; font-size:1px; height: 1px; border-top: 1px solid #BFBFBF; line-height: 1px;''>&nbsp;</div></div>
                    #end
                    #set($shouldDrawSeparator = true)
                <div style=''$css_table_title_text''>$bean.getFormattedRuleTitle($messageUtil, $dateRange, $textValueColor, $missedColor)</div><br>
                <table style=''width:100%; border-collapse:collapse;''>
                    <thead>
                        <tr>
                            <th valign=''middle''
                                style=''
                                       text-align: left;
                                       padding: 10px 8px 10px 15px;
                                       $css_bg_header
                                       $css_header_text
                                       $css_outer_border_top
                                       $css_outer_border_left''>
                                    <b>$hls.Customer() / $hls.Project() /  $hls.TasksAMPSubtasks()</b>
                                </th>
                          </tr>
                    </thead>
                    <tbody>
                    #set($drawTopCPBorder = false)
                    #set($projectNumber = 0)
                    #set( $oddRow = true )
                    #set( $projectTasks = $bean.projectTasksMap.get($dateRange) )
                    #foreach( $project in $projectTasks.keySet())
                        <tr>
                            <td style=''text-align: left;
                                           background-color: #f2f2f2;
                                           padding: 20px 8px 8px 15px;
                                           $css_text
                                           $css_outer_border_left
                                           $css_outer_border_right
                                           $css_cp_border_bottom
                                           #if($drawTopCPBorder) $css_dark_border_top #else $css_outer_border_top #end''>
                                <b style=''font-size:14px; $css_details_header_color''>$escapeUtil.getNameEscaped($project.customer)</b> / <span style=''$css_parameters_name_text font-weight: bold;''> $escapeUtil.getNameEscaped($project) </span>
                            </td>
                        </tr>
                        #set($drawTopCPBorder = true)
                        #set( $rowNumber = 0)
                        #foreach( $task in $projectTasks.get($project))

                            <tr>
                                #set ( $drawOuterBorderBottom = ($projectNumber == ($projectTasks.keySet().size() - 1) && $rowNumber == ($projectTasks.get($project).size() - 1)) )
                                #if($oddRow)
                                    #set ($bgColorFirst = ''#ffffff'')
                                    #set ($bgColorSecond = ''#f7f7f7'')
                                    #set ( $drawEvenBorderTop = false )
                                    #set ( $drawEvenBorderBottom = false )
                                #else
                                    #set ($bgColorFirst = ''#fcfaf3'')
                                    #set ($bgColorSecond = ''#f5f3ed'')
                                    #set ( $drawEvenBorderTop = ($rowNumber != 0) )
                                    #set ( $drawEvenBorderBottom = ($rowNumber < ($projectTasks.get($project).size() - 1)) )
                                #end
                                <td valign=''middle''
                                    style=''
                                           text-align: left;
                                           $css_details_font_size
                                           background-color: $bgColorFirst;
                                           padding: 8px 8px 8px 35px;
                                           $css_text
                                           $css_outer_border_left
                                           $css_outer_border_right
                                           #if($drawEvenBorderTop) $css_even_row_border_top #end
                                           #if($drawEvenBorderBottom) $css_even_row_border_bottom #end
                                           #if($drawOuterBorderBottom) $css_outer_border_bottom #end''>
                                    <a href=''$bean.getLinkToTask($task)'' style=''$css_link_color''>$escapeUtil.getNameEscaped($task)</a>
                                </td>
                                #set($oddRow = !$oddRow)
                                #set($rowNumber = ($rowNumber + 1))
                            </tr>
                        #end
                        #set($projectNumber = ($projectNumber + 1))
                    #end
                    </tbody>
                </table>
                <br>
                <a href=''$bean.getLinkReport($dateRange)'' style=''$css_link_color; $css_report_link_font_size''>View Estimated vs. Actual Time Report for the listed#if($projectTasks.keySet().size() > 1) $hls.projects()#else $hls.project()#end</a>
                #end
            #end
        </td>
    </tr>
    <tr>
        <td>#showFooter($footer)</td>
    </tr>
</table>
</body>
</html>', last_modified_timestamp = now()
        where name = 'deadline.html';

update mail_template
    set template = '#set($css_cp_border=''1px solid #d9d9d9 '')
#initStyles()
<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
<table style=''width: 95%;$css_text font-size: 12px;''>
    <tr>
        <td style=''$css_text font-size: 14px; line-height:130%; background-color: #ffffff''>
            Dear <b>$recipient.fullNameWithoutUserNameEscaped</b>,<br>
            <br>
            #set($shouldDrawSeparator = false)
            #foreach( $rule in $rules)
            #set( $bean = $ruleBeans.get($rule) )
            #set( $numberOfTasks = $bean.numberOfTasks)
            #if($shouldDrawSeparator)
                    <br>
                    <div style=''font-size:1px; line-height:1px;''><div style=''margin:5px 0px 5px 0px;  width:100%; font-size:1px; height: 1px; border-top: 1px solid #BFBFBF; line-height: 1px;''>&nbsp;</div></div>
                    <br>
                #end
                #set($shouldDrawSeparator = true)
            <div style=''$css_table_title_text''><b>Time reported for the following
                        <span style=''$css_parameters_name_text font-weight: bold;''>$numberOfTasks</span>#if($numberOfTasks>1) $hls.tasksSLASHsubtasks()#else $hls.taskSLASHsubtask()#end reached
                        <span style=''$css_parameters_name_text font-weight: bold;''>$rule.estimatePercents%</span> of estimate:</b></div><br>
            <table cellpadding=''0'' cellspacing=''0'' border=''0'' style=''width: 100%; border-collapse:collapse;''>
                <thead>
                <tr>
                    <th rowspan=''2'' valign=''middle''
                        style=''
                               text-align: left;
                               padding: 10px 8px 10px 15px;
                               $css_bg_header
                               $css_header_text
                               $css_outer_border_top
                               $css_outer_border_left''>
                            <b>$hls.Customer() / $hls.Project() /  $hls.TasksAMPSubtasks()</b>
                        </th>
                    <th  colspan=''2'' valign=''middle''
                        style=''
                               text-align: center;
                               padding: 10px 8px 10px 8px;
                               $css_bg_header
                               $css_header_text
                               $css_outer_border_top
                               $css_light_border_left''>
                        <b>Spent on $bean.formattedRuleDateToShort</b></th>
                    <th rowspan=''2'' valign=''middle''
                        style=''
                               text-align: center;
                               padding: 0px 8px 0px 8px;
                               $css_bg_header
                               $css_header_text
                               $css_outer_border_top
                               $css_light_border_left''>
                        <b>Estimated<br>Hours</b></th>
                    <th  colspan=''2'' valign=''middle''
                        style=''
                               text-align: center;
                               padding: 5px 8px 5px 8px;
                               $css_bg_header
                               $css_header_text
                               $css_outer_border_top
                               $css_light_border_right
                               $css_light_border_left''>
                        <b>Variance on $bean.formattedRuleDateToShort</b><br>
                        <span style=''font-size: 12px;''>(Estimated Hours - Spent)</span>
                        </th>
                </tr>
                <tr>
                    <th valign=''middle''
                        style=''width:90px;
                               text-align: center;
                               padding: 5px 8px 5px 8px;
                               $css_bg_header
                               $css_header_text
                               $css_outer_border_top
                               $css_light_border_left''>
                        <b>mh</b>
                    </th>
                    <th valign=''middle''
                        style=''width:90px;
                               text-align: center;
                               padding: 5px 8px 5px 8px;
                               $css_bg_header
                               $css_header_text
                               $css_outer_border_top
                               $css_light_border_left''>
                        <b>%</b>
                    </th>
                    <th valign=''middle''
                        style=''width:90px;
                               text-align: center;
                               padding: 5px 8px 5px 8px;
                               $css_bg_header
                               $css_header_text
                               $css_outer_border_top
                               $css_light_border_left''>
                        <b>mh</b>
                    </th>
                    <th valign=''middle''
                        style=''width:90px;
                               text-align: center;
                               padding: 5px 8px 5px 8px;
                               $css_bg_header
                               $css_header_text
                               $css_outer_border_top
                               $css_light_border_right
                               $css_light_border_left''>
                        <b>%</b>
                    </th>
                </tr>
                </thead>
                <tbody>
                #set($drawTopCPBorder = false)
                #set($taskNumber = 0)
                #set( $oddRow = true )
                #foreach( $project in $bean.projectTasks.keySet())
                    <tr>
                        <td style=''text-align: left;
                                       background-color: #f2f2f2;
                                       padding: 20px 8px 8px 15px;
                                       $css_text
                                       $css_outer_border_left
                                       $css_cp_border_bottom
                                       #if($drawTopCPBorder) $css_dark_border_top #end''>
                            <b style=''font-size:14px; $css_details_header_color''>$escapeUtil.getNameEscaped($project.customer)</b> / <span style=''$css_parameters_name_text font-weight: bold;''> $escapeUtil.getNameEscaped($project) </span>

                        </td>
                        <td colspan=''2'' style=''text-align: left;
                                       background-color: #f2f2f2;
                                       padding: 20px 8px 8px 15px;
                                       $css_text
                                       $css_cp_border_bottom
                                       #if($drawTopCPBorder) $css_dark_border_top #end''>
                           &nbsp;
                        </td>
                        <td   style=''text-align: left;
                                       background-color: #ededed;
                                       padding: 20px 8px 8px 15px;
                                       $css_text
                                       $css_cp_border_bottom
                                       $css_light_border_right
                                       $css_light_border_left
                                       #if($drawTopCPBorder) $css_dark_border_top #end'' >
                           &nbsp;
                        </td>
                        <td colspan=''2'' style=''text-align: left;
                                       background-color: #f2f2f2;
                                       padding: 20px 8px 8px 15px;
                                       $css_text
                                       $css_cp_border_bottom
                                       $css_outer_border_right
                                       #if($drawTopCPBorder) $css_dark_border_top #end''>
                           &nbsp;
                        </td>
                    </tr>
                    #set($drawTopCPBorder = true)
                    #set( $rowNumber = 0)
                    #foreach( $task in $bean.projectTasks.get($project))

                        <tr>
                            #set ( $drawOuterBorderBottom = ($taskNumber == ($numberOfTasks - 1)) )
                            #if($oddRow)
                                #set ($bgColorFirst = ''#ffffff'')
                                #set ($bgColorSecond = ''#f7f7f7'')
                                #set ( $drawEvenBorderTop = false )
                                #set ( $drawEvenBorderBottom = false )
                            #else
                                #set ($bgColorFirst = ''#fcfaf3'')
                                #set ($bgColorSecond = ''#f5f3ed'')
                                #set ( $drawEvenBorderTop = ($rowNumber != 0) )
                                #set ( $drawEvenBorderBottom = ($rowNumber < ($bean.projectTasks.get($project).size() - 1)) )
                            #end
                            <td valign=''middle''
                                style=''
                                       text-align: left;
                                       $css_details_font_size
                                       background-color: $bgColorFirst;
                                       padding: 8px 8px 8px 35px;
                                       $css_text
                                       $css_outer_border_left
                                       #if($drawEvenBorderTop) $css_even_row_border_top #end
                                       #if($drawEvenBorderBottom) $css_even_row_border_bottom #end
                                       #if($drawOuterBorderBottom) $css_outer_border_bottom #end''>
                                <a href=''$task.link'' style=''$css_link_color''>$task.getFullNameEscaped()</a>
                            </td>
                            <td nowrap valign=''middle''
                                style=''
                                       text-align: right;
                                       $css_details_font_size
                                       background-color: $bgColorFirst;
                                       padding: 8px 25px 8px 8px;
                                       $css_text
                                       #if($drawEvenBorderTop) $css_even_row_border_top #end
                                       #if($drawEvenBorderBottom) $css_even_row_border_bottom #end
                                       #if($drawOuterBorderBottom) $css_outer_border_bottom #end''
                                ><div>$task.formattedSpent</div></td>
                            <td nowrap valign=''middle''
                                style=''
                                       text-align: right;
                                       $css_details_font_size
                                       background-color: $bgColorFirst;
                                       padding: 8px;
                                       $css_text
                                       #if($drawEvenBorderTop) $css_even_row_border_top #end
                                       #if($drawEvenBorderBottom) $css_even_row_border_bottom #end
                                       #if($drawOuterBorderBottom) $css_outer_border_bottom #end''
                                ><div style=''margin-right:16px;''>$task.ofEstimate%</div></td>
                            <td nowrap valign=''middle''
                                style=''text-align: right;
                                       $css_details_font_size
                                       padding: 8px;
                                       background-color: $bgColorSecond;
                                       $css_text
                                       $css_light_border_right
                                       $css_light_border_left
                                       #if($drawEvenBorderTop) $css_even_row_border_top #end
                                       #if($drawEvenBorderBottom) $css_even_row_border_bottom #end
                                       #if($drawOuterBorderBottom) $css_outer_border_bottom #end''
                                ><div style=''margin-right:16px;''>$task.formattedBudgetTime</div></td>
                            <td nowrap valign=''middle''
                                style=''
                                       text-align: right;
                                       $css_details_font_size
                                       background-color: $bgColorFirst;
                                       padding: 8px;
                                       $css_text
                                       #if($drawEvenBorderTop) $css_even_row_border_top #end
                                       #if($drawEvenBorderBottom) $css_even_row_border_bottom #end
                                       #if($drawOuterBorderBottom) $css_outer_border_bottom #end''
                                ><div style=''margin-right:16px;''>$task.formattedVariance</div></td>
                            <td nowrap valign=''middle''
                                style=''
                                       text-align: right;
                                       $css_details_font_size
                                       background-color: $bgColorFirst;
                                       padding: 8px;
                                       $css_text
                                       $css_outer_border_right
                                       #if($drawEvenBorderTop) $css_even_row_border_top #end
                                       #if($drawEvenBorderBottom) $css_even_row_border_bottom #end
                                       #if($drawOuterBorderBottom) $css_outer_border_bottom #end''
                                ><div style=''margin-right:16px;''>$task.percentVariance%</div></td>

                            #set($oddRow = !$oddRow)
                            #set($rowNumber = ($rowNumber + 1))
                            #set($taskNumber = ($taskNumber + 1))
                        </tr>
                    #end
                #end
                </tbody>
            </table>
            <br>
            <a href=''$bean.link'' style=''$css_link_color; $css_report_link_font_size''>View Estimated vs. Actual Time Report for the listed#if($bean.projectTasks.keySet().size() > 1) $hls.projects()#else $hls.project()#end</a>
            <br>
            #end
        </td>
    </tr>
    <tr>
        <td>#showFooter($footer)</td>
    </tr>
</table>
</body>
</html>', last_modified_timestamp = now()
        where name = 'estimate.html';

update mail_template
    set template = '#initStyles()
<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
#set($grantedColor = ''#688037'')
#set($revokedColor = ''#772F28'')
<table style=''width: 95%;$css_text font-size: 12px;''>
    <tr>
        <td style=''$css_text font-size: 14px; line-height:130%; background-color: #ffffff''>
            Dear <b>$recipient.fullNameWithoutUserNameEscaped</b>,<br><br>
            #if(!$addedAssignments && !$removedAssignments)
                #if($assignments)
                    You no longer have access to all $hls.projects() of all $hls.customers().<br><br>
                #else
                    You have been granted access to all $hls.projects() of all $hls.customers().
                #end
            #else
                #if($addedAssignments && !$addedAssignments.isEmpty())
                    <div>Your were <span style=''color: $grantedColor;''>granted access</span> to the following #if($addedAssignments.size() > 1) $hls.projects()#else $hls.project()#end:</div>
                    <div><ul style=''padding-top:5px; padding-bottom: 5px;''>
                        #foreach($project in $addedAssignments)
                            <li style=''padding-top:2px; padding-bottom:3px;padding-left:10px;''>
                                <span style=''color: $grantedColor;''>$escapeUtil.getNameEscaped($project)</span> ($escapeUtil.getNameEscaped($project.customer))
                            </li>
                        #end
                    </ul></div>
                #end
                #if($removedAssignments && !$removedAssignments.isEmpty())
                    <div>Your access to the following #if($removedAssignments.size() > 1) $hls.projects() #else $hls.project() #end has been <span style=''color: $revokedColor;''>restricted</span>:</div>
                    <div><ul style=''padding-top:5px; padding-bottom: 5px;''>
                        #foreach($project in $removedAssignments)
                            <li style=''padding-top:2px; padding-bottom:3px;padding-left:10px;''>
                                <span style=''color: $revokedColor;''>$escapeUtil.getNameEscaped($project)</span> ($escapeUtil.getNameEscaped($project.customer))
                            </li>
                        #end
                    </ul></div>
                #end
            #end
            #if($assignments)
                #if (!$assignments.isEmpty())
                    <div style=''$css_table_title_text;margin-bottom: 0; padding-bottom: 0;''>Your current $hls.project() assignments:</div><br>
                    #drawCustomerProjectsTable($assignments)
                #else
                    Currently you do not have access to any $hls.projects().
                #end
            #end
            </td>
        </tr>
    <tr>
        <td>#showFooter($footer)</td>
    </tr>
   </table>
</body>
</html>', last_modified_timestamp = now()
        where name = 'assignments.html';

update mail_template
    set template = '<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
#set($css_cp_border=''1px solid #d9d9d9 '')
#initStyles()
<table style=''width: 95%;$css_text font-size: 12px;''>
    <tr>
        <td style=''$css_text font-size: 14px; line-height:130%; background-color: #ffffff''>
            Dear <b>$recipient.fullNameWithoutUserNameEscaped</b>,<br><br>
            There#if($usersWithUnsubmittedTTCount > 1) are users#else is a user#end whose time-track is incomplete.<br><br>
            #set($shouldDrawSeparator = false)
            #foreach( $rule in $rules)
                #if($shouldDrawSeparator)
                    <div style='' font-size:1px; line-height:1px;''><div style=''margin:25px 0px 25px 0px;  width:100%; font-size:1px; height: 1px; border-top: 1px solid #BFBFBF; line-height: 1px;''>&nbsp;</div></div>
                #end
                #set($shouldDrawSeparator = true)
            <table cellpadding=''0'' cellspacing=''0'' border=''0'' style=''border-collapse:collapse; text-align: left;''>
                <tr>
                    <td style=''$css_parameters_name_text; padding-left: 0px; margin-left:0px;''>
                        <b style="white-space: nowrap;">Date#if(!($ruleBeans.get($rule).isOneDayDateRange())) Range#end:</b>
                    </td>
                    <td style=''$css_parameters_text padding-left: 25px;''>
                        <b style="white-space: nowrap; ">$ruleBeans.get($rule).formattedDateRange</b>
                    </td>
                </tr>
                <tr>
                    <td style=''$css_parameters_name_text; padding-left: 0px; margin-left:0px;''>
                        <b style="white-space: nowrap;">Expected Time-Track:</b>
                    </td>
                    <td style=''$css_parameters_text padding-left: 25px;''>
                        <b style="white-space: nowrap;">$rule.ttLimit.hours:00</b>
                    </td>
                </tr>
            </table>
            <br>
            <b><span style=''$css_table_title_text''>Time-Track Reported by User#if ($ruleBeans.get($rule).all.size() > 1)s#end:</span></b>
            <br>
            <br>
            <table cellpadding=''0'' cellspacing=''0'' border=''0'' style=''width: 100%;  border-collapse:collapse;''>
                <col/> 	<col width=''80''/> 	<col width=''80''/>
                <thead>
                    <tr>
                        <th valign=''middle''
                            style=''
                                   text-align: left;
                                   padding: 0px 8px 0px 15px;
                                   $css_bg_header
                                   $css_header_text
                                   $css_outer_border_top
                                   $css_outer_border_left''>
                            <b>User</b></th>
                        <th valign=''middle''
                            style='' width: 80px;
                                   text-align: center;
                                   padding: 0px 8px 0px 8px;
                                   $css_bg_header
                                   $css_header_text
                                   $css_outer_border_top
                                   $css_light_border_left''>
                            <b>Spent</b></th>
                        <th valign=''middle''
                            style='' width: 80px;
                                   text-align: center;
                                   padding: 10px 8px 10px 8px;
                                   $css_bg_header
                                   $css_header_text
                                   $css_outer_border_top
                                   $css_light_border_right
                                   $css_light_border_left''>
                            <b>Missing<br>
                                Hours</b></th>
                    </tr>
                </thead>
                <tbody>
                #set( $oddRow = true )
                #set( $rowNumber = 0)
                #foreach( $userInn in $ruleBeans.get($rule).all )
                    #set ( $drawEvenBorderTop = ($rowNumber != 0) )
                    #set ( $drawEvenBorderBottom = ($rowNumber < ($ruleBeans.get($rule).all.size() - 1)))
                    #set ( $drawBorderBottom = ($rowNumber == ($ruleBeans.get($rule).all.size() - 1)))
                    #if($oddRow)
                        <tr>
                            <td valign=''middle''
                                style=''
                                       text-align: left;
                                       font-size: 13px;
                                       background-color: #ffffff;
                                       padding: 8px 8px 8px 15px;
                                       $css_text
                                       $css_outer_border_left
                                       #if($drawBorderBottom) $css_outer_border_bottom #end
                                       ''>
                                $userInn.user.formattedNameEscaped
                            </td>
                            <td valign=''middle''
                                style=''text-align: right;
                                       font-size: 13px;
                                       padding: 8px 15px 8px 8px;
                                       background-color: #ffffff;
                                       $css_text
                                       $css_dark_border_left
                                       #if($drawBorderBottom) $css_outer_border_bottom #end''>
                                $userInn.formattedActuals</td>
                            <td valign=''middle''
                                style=''text-align: right;
                                       font-size: 13px;
                                       padding: 8px 15px 8px 8px;
                                       background-color: #ffffff;
                                       $css_text
                                       $css_outer_border_right
                                       $css_dark_border_left
                                       #if($drawBorderBottom) $css_outer_border_bottom #end''>
                                <span style=''color:red;''>$userInn.formattedMissed</span></td>
                        </tr>
                    #else
                        <tr>
                            <td valign=''middle''
                                style=''
                                       text-align: left;
                                       font-size: 13px;
                                       background-color: #fcfaf3;
                                       padding: 8px 8px 8px 15px;
                                       $css_text
                                       $css_outer_border_left
                                       #if($drawEvenBorderTop) $css_even_row_border_top #end
                                       #if($drawEvenBorderBottom) $css_even_row_border_bottom #end
                                       #if($drawBorderBottom) $css_outer_border_bottom #end
                                       ''>
                                $userInn.user.formattedNameEscaped
                            </td>
                            <td valign=''middle''
                                style=''text-align: right;
                                       font-size: 13px;
                                       padding: 8px 15px 8px 8px;
                                       background-color: #fcfaf3;
                                       $css_text
                                       $css_dark_border_left
                                       #if($drawEvenBorderTop) $css_even_row_border_top #end
                                       #if($drawEvenBorderBottom) $css_even_row_border_bottom #end
                                       #if($drawBorderBottom) $css_outer_border_bottom #end
                                       ''>
                                $userInn.formattedActuals</td>
                             <td valign=''middle''
                                style=''text-align: right;
                                       font-size: 13px;
                                       padding: 8px 15px 8px 8px;
                                       background-color: #fcfaf3;
                                       $css_text
                                       $css_outer_border_right
                                       $css_dark_border_left
                                       #if($drawEvenBorderTop) $css_even_row_border_top #end
                                       #if($drawEvenBorderBottom) $css_even_row_border_bottom #end
                                       #if($drawBorderBottom) $css_outer_border_bottom #end''>
                                <span style=''color:red;''>$userInn.formattedMissed</span></td>
                        </tr>
                    #end
                    #set($oddRow = !$oddRow)
                    #set($rowNumber = ($rowNumber + 1))
                #end
                </tbody>
            </table>
            <br>
            <a href=''$ruleBeans.get($rule).link'' style=''$css_link_color; $css_report_link_font_size''>View detailed time-track data in the Staff Performance Report</a>
            #end
        </td>
    </tr>
    <tr>
        <td>#showFooter($footer)</td>
    </tr>
  </table>
</body>
</html>', last_modified_timestamp = now()
        where name = 'unsubmitted_tt.html';

update mail_template
    set template = '#initStyles()
<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
<table style=''width: 95%;$css_text font-size: 12px;''>
    <tr>
        <td style=''$css_text font-size: 14px; line-height:130%; background-color: #ffffff''>
            Dear <b>$recipient.fullNameWithoutUserNameEscaped</b>,<br><br>
            #set($shouldDrawSeparator = false)
            #foreach( $rangeBean in $rangeBeans)
                #if($shouldDrawSeparator)
                    <div style='' font-size:1px; line-height:1px;''><div style=''margin:25px 0px 25px 0px;  width:100%; font-size:1px; height: 1px; border-top: 1px solid #BFBFBF; line-height: 1px;''>&nbsp;</div></div>
                #end
                #set($shouldDrawSeparator = true)
                $rangeBean.getRangeStatus($messageUtil, $textValueColor)<br>
                <ul style=''padding-top:5px; padding-bottom: 5px;''>
                    #foreach( $user in $rangeBean.usersExceededPersonalEarnings )
                    <li style=''padding-top:2px; padding-bottom:3px;padding-left:9px;''>
                        $user.formattedNameEscaped
                    </li>
                    #end
                </ul>
                <a href=''$rangeBean.link'' style=''$css_link_color; $css_report_link_font_size''>View Cost Report for the listed user#if($rangeBean.hasMultipleUsers())s#end</a>
                <br>
            #end
        </td>
    </tr>
    <tr>
        <td>#showFooter($footer)</td>
    </tr>
</table>
</body>
</html>', last_modified_timestamp = now()
        where name = 'personal_earnings.html';

update mail_template
    set template = '<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
#set($css_cp_border=''1px solid #d9d9d9 '')
#initStyles( )
<table style=''width: 95%;$css_text font-size: 12px;''>
    <tr>
        <td style=''$css_text font-size: 14px; line-height:130%; background-color: #ffffff''>
            Dear <b>$recipient.fullNameWithoutUserNameEscaped</b>,<br>
            <br> You have not recorded sufficient time for #if($displaySettings.showDateRangePrefix)date range<br>#end $displaySettings.formattedDateRange.<br> <br>
            <table cellpadding=''0'' cellspacing=''0'' border=''0'' style=''border-collapse: collapse;font-weight: bold; $css_parameters_text''>
            <tr><td><b>Expected time-track:</b></td> <td style=''padding-left: 35px;''><b>$rule.ttLimit.hours:00</b></td></tr><tr><td><b>Recorded time-track:</b></td><td style=''color:red;padding-left: 35px;''><b>$total.formattedTotal</b></td></tr>
            </table>
            <br>
            Please make sure that you entered all working and leave time for the specified date range.<br><br>
            <table cellpadding=''0'' cellspacing=''0'' border=''0''
                   style=''font-size: 11px;
                          width: 100%;
                          border-collapse: collapse;
                          ''>
                <col/>
                <col width=''60''/>
                <col/>
                #drawHeader( )
                <tbody>
                #foreach( $day in $days )
                #set( $drawTotalTopBorder = ($day.rows.size() != 0) )
                <tr>
                    <td colspan=''2'' valign=''bottom''
                        style=''
                               $css_text
                               $css_outer_border_left
                               $css_darkest_border_bottom
                               font-size: 18px;
                               padding: 30px 8px 10px 15px;
                               text-align: left; ''>
                        $day.formattedDay
                    </td>
                    <td valign=''bottom''
                        style=''
                               font-size: 13px;
                               text-align: right;
                               padding: 30px 15px 10px;
                               $css_text
                               $css_outer_border_right
                               $css_darkest_border_bottom
                               ''>
                        <a href=''${displaySettings.getEditTTLink($day.day)}'' style=''$css_link_color''>Edit time-track</a>
                    </td>
                </tr>
                #drawDayTT( $day )
                #drawDayTotal( $day $displaySettings $drawTotalTopBorder )
                #end
                <tr>
                    <td colspan=''3''><span style=''font-size: 12px; line-height:18px;''>&nbsp;<br></span></td>
                </tr>
                <tr>
                    <td valign=''middle''
                        style=''
                               text-align: right;
                               font-size: 15px;
                               padding: 12px 8px 6px 18px;
                               $css_text
                               $css_bg_range_total
                               $css_dark_border_left
                               $css_range_total_border_top
                               #if(!$displaySettings.showCalculatedOvertime) $css_range_total_border_bottom #end''>
                        <b>Total&nbsp;for&nbsp;the&nbsp;date&nbsp;range:</b></td>
                    <td valign=''middle''
                        style=''
                               text-align: right;
                               font-size: 15px;
                               $css_text
                               padding: 12px 15px 6px 8px;
                               $css_bg_range_total_middle
                               $css_light_border_left
                               $css_light_border_right
                               $css_range_total_border_top
                               #if(!$displaySettings.showCalculatedOvertime) $css_range_total_border_bottom #end''>
                        <b>$total.formattedTotal</b></td>
                    <td valign=''middle''
                        style=''
                               text-align: left;
                               font-size: 13px;
                               padding: 12px 8px 6px 18px;
                               $css_text
                               $css_bg_range_total
                               $css_dark_border_right
                               $css_range_total_border_top
                               #if(!$displaySettings.showCalculatedOvertime) $css_range_total_border_bottom #end''>
                        <table border=''0'' cellpadding=''0'' cellspacing=''0''>
                            <tr>
                                <td width=''80''
                                    style=''
                                           text-align: left;
                                           font-size: 13px;
                                           $css_text
                                           $css_bg_range_total''>
                                    Working&nbsp;Time:</td>
                                <td width=''40''
                                    style=''
                                           text-align: right;
                                           font-size: 13px;
                                           padding-left: 5px;
                                           $css_text
                                           $css_bg_range_total''>
                                    $total.formattedWorking
                                </td>
                                #if($displaySettings.showLeaveTime)
                                <td style=''
                                           padding-left: 25px;
                                           text-align: left;
                                           font-size: 13px;
                                           $css_text
                                           $css_bg_range_total''>
                                    Leave&nbsp;Time:</td>
                                <td width=''40''
                                    style=''
                                           text-align: right;
                                           font-size: 13px;
                                           padding-left: 5px;
                                           $css_text
                                           $css_bg_range_total''>
                                    $total.formattedLeave
                                </td>
                                #end
                                <td></td>
                            </tr>
                        </table>
                    </td>
                </tr>
                #if($displaySettings.showReportedOvertime)
                <tr>
                    <td valign=''top''
                        style=''
                               text-align: right;
                               font-size: 14px;
                               padding: 0 8px 8px 15px;
                               $css_text
                               $css_bg_range_total
                               $css_dark_border_left''>
                        <span style=''white-space: nowrap;''>Reported&nbsp;Overtime&nbsp;#if($displaySettings.undertimeTracking)/&nbsp;Undertime#end:</span>
                    </td>
                    <td valign=''top''
                        style='' text-align: right;
                                font-size: 14px;
                                padding: 0 15px 8px 8px;
                                $css_text
                                $css_bg_range_total_middle
                                $css_light_border_left
                                $css_light_border_right''>
                        $total.formattedReportedOvertime
                    </td>
                    <td valign=''top''
                        style='' text-align: left;
                                font-size: 14px;
                                $css_text
                                $css_bg_range_total
                                $css_dark_border_right''>
                        &nbsp;</td>
                </tr>
                #end #if($displaySettings.showCalculatedOvertime)
                <tr>
                    <td valign=''top''
                        style='' text-align: right;
                                font-size: 14px;
                                padding: 0 8px 16px 15px;
                                $css_text
                                $css_bg_range_total
                                $css_dark_border_left
                                $css_range_total_border_bottom''>
                        <span style=''white-space: nowrap;''>Auto-calculated&nbsp;Overtime#if($displaySettings.undertimeTracking)&nbsp;/&nbsp;Undertime#end:</span>
                    </td>
                    <td valign=''top''
                        style='' text-align: right;
                                font-size: 14px;
                                padding: 0 15px 8px 8px;
                                $css_text
                                $css_bg_range_total_middle
                                $css_light_border_left
                                $css_light_border_right
                                $css_range_total_border_bottom''>
                        $total.formattedCalculatedOvertime
                    </td>
                    <td valign=''top''
                        style=''text-align: left;
                               font-size: 14px;
                               $css_text
                               $css_bg_range_total
                               $css_dark_border_right
                               $css_range_total_border_bottom''>
                        &nbsp;</td>
                </tr>
                #end
                </tbody>
            </table>
            </td>
    </tr>
    <tr>
        <td>#showFooter($footer)</td>
    </tr>
</table>
</body>
</html>', last_modified_timestamp = now()
        where name = 'personal_tt.html';

update mail_template
    set template = '#initStyles()
<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
<table style=''width: 95%;$css_text font-size: 12px;''>
    <tr>
        <td style=''$css_text font-size: 14px; line-height:130%; background-color: #ffffff''>
            Dear <b>$recipient.fullNameWithoutUserNameEscaped</b>,<br>
            <br>
            #set($shouldDrawSeparator = false)
            #foreach( $rangeBean in $rangeBeans)
                #if($shouldDrawSeparator)
                    <div style='' font-size:1px; line-height:1px;''><div style=''margin:25px 0px 25px 0px;  width:100%; font-size:1px; height: 1px; border-top: 1px solid #BFBFBF; line-height: 1px;''>&nbsp;</div></div>
                #end
                #set($shouldDrawSeparator = true)
                <div style=''$css_table_title_text;padding-bottom:0; margin-bottom:0;''>$rangeBean.getRangeStatus($messageUtil, $textValueColor)</div><br>
                #drawCustomerProjectsTable( $rangeBean.customerProjects )
            <br>
            <a href=''$rangeBean.link'' style=''$css_link_color; $css_report_link_font_size''>View Cost Report for the listed#if($rangeBean.getProjectsCount() > 1) $hls.projects()#else $hls.project()#end</a>
            #end
        </td>
    </tr>
    <tr>
        <td>#showFooter($footer)</td>
    </tr>
</table>
</body>
</html>', last_modified_timestamp = now()
        where name = 'project_budget.html';

update mail_template
    set template = '#initStyles()
<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
<table style=''width: 560px;$css_text font-size: 12px;'' cellspacing=''0''>
    <tr>
        <td valign=''middle''>Dear $recipient.fullNameWithoutUserNameEscaped,<br><br>
            This is an automatic email message sent by actiTIME to test email settings.
            <br>If you received it, then Outgoing mail server, port, and &#8220;From&#8221; address have been correctly set up.
            <br><br>Please verify that the following notification parameters are configured as you expected:
            <ol>
                <li style=''margin-left: 0; padding-left: 0; margin-bottom: 20px;'' > &#8220;From&#8221; address<br><br>
                    &#8220;From&#8221; address will be shown to all notification recipients in the same way as
                    in this message. Make sure that it is shown in the required format.<br>
                </li>
                <li style=''margin-left: 0; padding-left: 0; margin-bottom: 20px;'' >
                    Prefix to add to the notification subject field<br><br>
                    Use of a prefix for actiTIME messages simplifies creation of
                    mail filtering rules in your email client.
                    $prefixNote
                </li>
                <li style=''margin-left: 0; padding-left: 0; margin-bottom: 20px;'' >
                    Links to actiTIME interfaces<br><br>
                    Please follow the link below to make sure that notification recipients will be able
                    to open links in the notification messages.<br>
                    <a href=''$url'' style=''$css_link_color''>Sample link to &quot;General Settings&quot; interface</a><br>
                </li>
                <li style=''margin-left: 0; padding-left: 0; margin-bottom: 20px;'' >
                    Standard message footer<br><br>
                    Standard message footer will be added to all notification messages (including this one).
                    $footerNote
                </li>
            </ol>
            #showYourActiTIMEFooter()
        </td>
    </tr>
    #if(!($!footer == ""))
    <tr>
        <td>#showFooter($footer)</td>
    </tr>
    #end
</table>
</body>
</html>', last_modified_timestamp = now()
        where name = 'test_message.html';

update mail_template
    set template = '#initStyles()
<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
<table style=''width: 560px;$css_text font-size: 12px;'' cellspacing=''0''>
    <tr>
        <td valign=''middle''>Dear $recipient.fullNameWithoutUserNameEscaped,<br><br>
            This is an automatic email message sent by actiTIME to test email settings.
            If you received it, then Outgoing mail server, port, and &#8220;From&#8221; address have been correctly set up.
            <br/>
            <br/>Please verify that address of your actiTIME installation is configured properly.
            <br/>Follow the link below to make sure that recipients will be able to open links in the system messages.
            <a href=''$url'' style=''$css_link_color''>Sample link to &quot;General Settings&quot; interface</a><br><br>
            #showYourActiTIMEFooter()
        </td>
    </tr>
</table>
</body>
</html>', last_modified_timestamp = now()
        where name = 'test_message_basic.html';

update mail_template
    set template = '#initStyles()
<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
<table style=''width: 560px;$css_text font-size: 12px;'' cellspacing=''0''>
    <tr>
        <td valign=''middle''>Dear $recipient.fullNameWithoutUserNameEscaped,<br><br>
            This is an automatic email message sent by actiTIME to test email settings.
            <br>Please verify that the following notification parameters are configured as you expected:
            <ol>
                <li style=''margin-left: 0; padding-left: 0; margin-bottom: 20px;'' >
                    Prefix to add to the notification subject field<br><br>
                    Use of a prefix for actiTIME messages simplifies creation of
                    mail filtering rules in your email client.
                    $prefixNote
                </li>
                <li style=''margin-left: 0; padding-left: 0; margin-bottom: 20px;'' >
                    Standard message footer<br><br>
                    Standard message footer will be added to all notification messages (including this one).
                    $footerNote
                </li>
            </ol>
            #showYourActiTIMEFooter()
        </td>
    </tr>
    #if(!($!footer == ""))
    <tr>
        <td>#showFooter($footer)</td>
    </tr>
    #end
</table>
</body>
</html>', last_modified_timestamp = now()
        where name = 'test_message_online.html';

-- next lines were added due to date range selector refactoring in reports
update report_params set param_name = 'dateRangeSelector.fromDate.internalText' where param_name = 'fromDate.internalText';
update report_params set param_name = 'dateRangeSelector.toDate.internalText' where param_name = 'toDate.internalText';
update report_params set param_name = 'dateRangeSelector.dateRangeState' where param_name = 'dateRangeState';

-- time-track for week
delete from user_task;

alter table user_task add column `date` date not null;
alter table user_task drop constraint pk_user_task;
alter table user_task add constraint pk_user_task_date primary key(`user_id`, `task_id`, `date`);

-- manage billing types access right was removed
delete from user_access_right where access_right_id=7;
delete from access_right where id=7;

-- hire date now is mandatory
create table udtmp ( uid int not null, dat date);
insert into udtmp (uid, dat) select id, DMIN("record_date", "tt_record", "user_id="&id) from at_user;
update udtmp set dat=DMIN("comment_date", "user_task_comment", "user_id="&uid) where dat is null or dat > DMIN("comment_date", "user_task_comment", "user_id="&uid);
update udtmp set dat=DMIN("leave_date", "user_date_leave", "user_id="&uid) where dat is null or dat > DMIN("leave_date", "user_date_leave", "user_id="&uid);
update udtmp set dat=DateAdd("d", -2, date()) where dat is null;
update at_user set hire_date=DLookup("dat", "udtmp", "uid="&id) where hire_date is null;
drop table udtmp;

alter table at_user alter column hire_date date not null;

-- inverted days now store their working/nonworking state
alter table w_days add column is_working_day bit not null;
-- old data conversion: inverted weekend days are working, inverted non-weekend days are nonworking
update w_days set is_working_day = (weekday(w_date) in (1, 7)); -- 1 = Sunday, 7 = Saturday

-- delete 'manage work schedule' access right
delete from user_access_right where access_right_id = 10;
delete from access_right where id = 10;


-- add tasks in user_task

-- see formula clarification in patch_21.mysql4.sql

insert into user_task (user_id, task_id, `date`)
    select DISTINCT user_id, task_id, DateAdd("d",
      iif(
        iif(exists (select 1 from system_settings where property_name='week_start_day'), (select CInt(property_value) from system_settings where property_name='week_start_day'),1) < 4,
      0, 7) + 3 - Weekday(t.dt, 1) -
      iif(Weekday(t.dt, 1) < iif(exists (select 1 from system_settings where property_name='week_start_day'), (select CInt(property_value) from system_settings where property_name='week_start_day'),1),
      7, 0), t.dt
    ) as dt
    from (
           select user_id, task_id, record_date as dt from tt_record
           union
           select user_id, task_id, comment_date as dt from user_task_comment
         ) as t
    group by user_id, task_id, dt;

-- ----------------------------------------------------

insert
  into patch_history (patch, pack_version, patch_timestamp)
  values (21, 0, now());