-- START : CHANGING COLUMN TYPE TO AUTOINCREMENT

-- README

-- It is not allowed in ACCESS to change column type to autoincrement if it contains any data.
-- The only way to rename column or table in ACCESS is to delete it at first and then recreate with new name.
-- So the simplest way to change column data type consists of following steps:
-- 1) Drop all foreign key constraints which references initial table.
-- 2) Create temp table and copy all data there.
--    In ACCESS we can do it using statement "select * into [temp_table_name] from [table_name];"
-- 3) Delete all data from initial table.
-- 4) Change column type to autoincrement.
-- 5) Copy the data back ordered by autoincrement column in ascending order.
-- 6) Delete temp table.
-- 7) Restore dropped constraints.

-- TEMPLATE

-- #{table_name} : #{column_name}
-- alter table #{table_referencing_this_table} drop constraint #{constraint_name};
-- select * into _temp_#{table_name} from #{table_name};
-- delete from #{table_name};
-- alter table #{table_name} alter column #{column_name} autoincrement;
-- insert into #{table_name}
--     select * from _temp_#{table_name};
-- drop table _temp_#{table_name};
-- alter table #{table_referencing_this_table} add constraint #{constraint_name} foreign key (#{table_referencing_this_table_column_name})
--     references #{table_name} (#{column_name}) on delete #{action_on_delete};


-- bin_data : bin_data_id
alter table bin_data_chunk drop constraint bin_data_chunk_bin_data_id;
alter table mail_message drop constraint mail_message_body_bin_data_id;
alter table mail_message_attachment drop constraint mail_message_attachment_bin_data_id;
select * into _temp_bin_data from bin_data;
delete from bin_data;
alter table bin_data alter column bin_data_id autoincrement;
insert into bin_data
    select * from _temp_bin_data;
drop table _temp_bin_data;
alter table bin_data_chunk add constraint bin_data_chunk_bin_data_id_fk_bin_data_bin_data_id foreign key (bin_data_id)
    references bin_data (bin_data_id) on delete cascade;
alter table mail_message add constraint mail_message_body_bin_data_id_fk_bin_data_bin_data_id foreign key (body_bin_data_id)
    references bin_data (bin_data_id) on delete no action;
alter table mail_message_attachment add constraint mail_message_attachment_bin_data_id_fk_bin_data_bin_data_id foreign key (bin_data_id)
    references bin_data (bin_data_id) on delete cascade;


-- mail_message : message_id
alter table mail_message_recipient drop constraint mail_message_recipient_message_id;
alter table mail_message_attachment drop constraint mail_message_attachment_message_id;

select * into _temp_mail_message from mail_message;
delete from mail_message;
alter table mail_message alter column message_id autoincrement;
insert into mail_message
    select * from _temp_mail_message;
drop table _temp_mail_message;

alter table mail_message_recipient add constraint mail_message_recipient_message_id
        foreign key (message_id) references mail_message (message_id) on delete cascade;
alter table mail_message_attachment add constraint mail_message_attachment_message_id
        foreign key (message_id) references mail_message (message_id) on delete cascade;

alter table mail_message add from_personal varchar(255);
alter table mail_message add from_address varchar(255) not null;

-- notification_rule : id
alter table notification_rule_params drop constraint notification_rule_params_rule_id;
alter table user_notif_rule_send drop constraint fk_user_notif_rule_send_rule;

select * into _temp_notification_rule from notification_rule;
delete from notification_rule;
alter table notification_rule alter column id autoincrement;
insert into notification_rule
    select * from _temp_notification_rule;
drop table _temp_notification_rule;

alter table notification_rule_params add constraint notification_rule_params_rule_id
        foreign key (rule_id) references notification_rule (id) on delete cascade;
alter table user_notif_rule_send add constraint fk_user_notif_rule_send_rule
        foreign key (rule_id) references notification_rule (id) on delete cascade;


-- user_notif_rule_send : id
alter table project_user_notif_rule_send drop constraint fk_project_user_notif_rule_send_id;
alter table user_user_notif_rule_send drop constraint fk_user_user_notif_rule_send_id;
alter table task_user_notif_rule_send drop constraint fk_task_user_notif_rule_send_id;

select * into _temp_user_notif_rule_send from user_notif_rule_send;
delete from user_notif_rule_send;
alter table user_notif_rule_send alter column id autoincrement;
insert into user_notif_rule_send
    select * from _temp_user_notif_rule_send;
drop table _temp_user_notif_rule_send;

alter table project_user_notif_rule_send add constraint fk_project_user_notif_rule_send_id
        foreign key (user_notif_rule_send_id) references user_notif_rule_send (id) on delete cascade;
alter table user_user_notif_rule_send add constraint fk_user_user_notif_rule_send_id
        foreign key (user_notif_rule_send_id) references user_notif_rule_send (id) on delete cascade;
alter table task_user_notif_rule_send add constraint fk_task_user_notif_rule_send_id
        foreign key (user_notif_rule_send_id) references user_notif_rule_send (id) on delete cascade;


-- at_user_revision : id
alter table at_user drop constraint fk_at_user_revision;

select * into _temp_at_user_revision from at_user_revision;
delete from at_user_revision;
alter table at_user_revision alter column id autoincrement;
insert into at_user_revision
    select * from _temp_at_user_revision;
drop table _temp_at_user_revision;

alter table at_user add constraint fk_at_user_revision
        foreign key (id) references at_user_revision (id) on delete no action;


-- task_revision : id
alter table task drop constraint fk_task_revision;

select * into _temp_task_revision from task_revision;
delete from task_revision;
alter table task_revision alter column id autoincrement;
insert into task_revision
    select * from _temp_task_revision;
drop table _temp_task_revision;

alter table task  add constraint fk_task_revision
        foreign key (id) references task_revision (id) on delete no action;

-- project_revision : id
alter table project drop constraint fk_project_revision;

select * into _temp_project_revision from project_revision;
delete from project_revision;
alter table project_revision alter column id autoincrement;
insert into project_revision
    select * from _temp_project_revision;
drop table _temp_project_revision;

alter table project add constraint fk_project_revision
        foreign key (id) references project_revision (id) on delete no action;

-- customer_revision : id
alter table customer drop constraint fk_customer_revision;

select * into _temp_customer_revision from customer_revision;
delete from customer_revision;
alter table customer_revision alter column id autoincrement;
insert into customer_revision
    select * from _temp_customer_revision;
drop table _temp_customer_revision;

alter table customer add constraint fk_customer_revision
        foreign key (id) references customer_revision (id) on delete no action;


-- billing_type_revision : id
alter table billing_type drop constraint fk_billing_type_revision;

select * into _temp_billing_type_revision from billing_type_revision;
delete from billing_type_revision;
alter table billing_type_revision alter column id autoincrement;
insert into billing_type_revision
    select * from _temp_billing_type_revision;
drop table _temp_billing_type_revision;

alter table billing_type add constraint fk_billing_type_revision
        foreign key (id) references billing_type_revision (id) on delete no action;

-- leave_type_revision : id
alter table leave_type drop constraint fk_leave_type_revision;

select * into _temp_leave_type_revision from leave_type_revision;
delete from leave_type_revision;
alter table leave_type_revision alter column id autoincrement;
insert into leave_type_revision
    select * from _temp_leave_type_revision;
drop table _temp_leave_type_revision;

alter table leave_type add constraint fk_leave_type_revision
        foreign key (id) references leave_type_revision (id) on delete no action;


-- license : id
select * into _temp_license from license;
delete from license;
alter table license alter column id autoincrement;
insert into license
    select * from _temp_license;
drop table _temp_license;


-- user_rate : id
alter table leave_rate drop constraint leave_rate_rate_id_fk_user_rate_id;

select * into _temp_user_rate from user_rate;
delete from user_rate;
alter table user_rate alter column id autoincrement;
insert into user_rate
    select * from _temp_user_rate;
drop table _temp_user_rate;

alter table leave_rate add constraint leave_rate_rate_id_fk_user_rate_id
        foreign key (rate_id) references user_rate (id) on delete cascade;


-- user_group : id
alter table at_user drop constraint fk_at_user_user_group;

select * into _temp_user_group from user_group;
delete from user_group;
alter table user_group alter column id autoincrement;
insert into user_group
    select * from _temp_user_group;
drop table _temp_user_group;

alter table at_user add constraint fk_at_user_user_group
        foreign key (user_group_id) references user_group(id) on delete set null;


-- report_configs : id
alter table report_params drop constraint fk_report_configs;
alter table report_params_customer drop constraint fk_rpc_config_id;
alter table report_params_project drop constraint fk_rpp_config_id;
alter table invoice_params drop constraint fk_invoice_params_config_id;
alter table invoice_tax drop constraint fk_invocie_tax_config_id;
alter table invoice_discount drop constraint fk_invoice_discount_config_id;

select * into _temp_report_configs from report_configs;
delete from report_configs;
alter table report_configs alter column id autoincrement;
insert into report_configs
    select * from _temp_report_configs;
drop table _temp_report_configs;

alter table report_params add constraint fk_report_configs
        foreign key (config_id) references report_configs (id) on delete cascade;
alter table report_params_customer add constraint fk_rpc_config_id
        foreign key (config_id) references report_configs(id) on delete cascade;
alter table report_params_project add constraint fk_rpp_config_id
        foreign key (config_id) references report_configs(id) on delete cascade;
alter table invoice_params add constraint fk_invoice_params_config_id
        foreign key (config_id) references report_configs(id) on delete cascade;
alter table invoice_tax add constraint fk_invoice_tax_config_id
        foreign key (config_id) references report_configs(id) on delete cascade;
alter table invoice_discount add constraint fk_invoice_discount_config_id
        foreign key (config_id) references report_configs(id) on delete cascade;



-- --------------------------------------------------------

INSERT INTO access_right VALUES (20,'role.account_settings_manager');
INSERT INTO access_right VALUES (21,'role.leave_time_manager');
INSERT INTO access_right VALUES (22,'role.pto_manager');
INSERT INTO access_right VALUES (14,'system.manage_pto');

-- grant Manage PTO to all users with Mange Users
insert into user_access_right (user_id, access_right_id)
  select user_id, 14 from user_access_right where access_right_id = 5;

alter table at_user add requires_approval bit NOT NULL;
alter table at_user add is_pto bit NOT NULL;
alter table at_user add def_pto_rules int NOT NULL;

update at_user set is_pto=false, def_pto_rules=true, requires_approval=false;

alter table leave_type add is_pto bit NOT NULL;
alter table leave_type add pto_coeff double NOT NULL;
alter table leave_type add requires_approval bit NOT NULL;

update leave_type set is_pto=false, pto_coeff=1.00, requires_approval=true;

CREATE TABLE user_approver (
  user_id int NOT NULL,
  approver_id int NOT NULL,
  PRIMARY KEY (user_id, approver_id)


  -- this constraints are processed by the application due to the constraints limitation (32) in ms access
  -- CONSTRAINT fk_user_approver_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE,
  -- CONSTRAINT fk_user_approver_approver_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE user_leave_time_request (
  id autoincrement,
  user_id int NOT NULL,
  leave_type_id int NOT NULL,
  status int NOT NULL,
  status_message text,
  status_timestamp datetime NOT NULL,
  approver_id int null,
  first_date date NOT NULL,
  last_date date NOT NULL,
  first_date_duration int NOT NULL,
  last_date_duration int NOT NULL,
  duration int NOT NULL,
  CONSTRAINT fk_user_leave_time_request_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT fk_user_leave_time_request_approver_id FOREIGN KEY (approver_id) REFERENCES at_user_revision (id),
  CONSTRAINT fk_user_leave_time_request_leave_type_id FOREIGN KEY (leave_type_id) REFERENCES leave_type (id) ON UPDATE CASCADE
);

CREATE TABLE user_pto_rule (
   id counter NOT NULL primary key,
   user_id int NOT NULL,
   type int NOT NULL,
   `value` double NOT NULL,
   frequency varchar(255) NOT NULL,
   frequency_params varchar(255) NOT NULL,
   effective_start_date date NOT NULL,
   effective_finish_date date,
   mgr_id int,
   mgr_comment text,
   CONSTRAINT user_pto_rule_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE,
   CONSTRAINT user_pto_rule_mgr_id FOREIGN KEY (mgr_id) REFERENCES at_user_revision (id)
 );

CREATE TABLE user_pto_cap (
  id counter NOT NULL PRIMARY KEY,
  user_id int NOT NULL,
  cap double NULL,
  effective_date date NOT NULL,
  CONSTRAINT uk_user_pto_cap_effective_date UNIQUE(user_id, effective_date),
  CONSTRAINT user_pto_cap_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE
);

INSERT INTO mail_template (name, template, last_modified_timestamp) VALUES
        ('shared_macro_ap.vm',       '## some email clients do not support embedded or external css, so we define several commonly used
## rules here as velocity variables to re-use them later for inline css
#macro( initStyles )
#set($css_h_width=''width: 150px'')
#set($css_link_color=''color: #003399 '')
#set($css_text=''font-family: Arial Unicode MS, Arial, sans-serif; font-weight: normal; '')
#end
#macro ( personLine $request)
<span style="$css_h_width">Person: </span><span>$request.owner.fullNameWithoutUserNameEscaped</span><br>
#end
#macro ( leaveTypeLine $request)
<span style="$css_h_width">Leave Type: </span><span>$request.leaveTypeName</span><br>
#end
#macro ( datesLine $request)
<span style="$css_h_width">Date#if(!$request.oneDayRequest)s#end: </span><span>$request.getFirstDate("EEE, MMM d")#if(!$request.oneDayRequest) - $request.getLastDate("EEE, MMM d")#end</span><br>
#end
#macro ( timeRequestedLine $request $showUnaffectedPTOMessage)
<span style="$css_h_width">Time Requested: </span><span>$request.leaveTimeTotal h#if($request.showPTO) ($request.requestedPTODays PTO days) #elseif($showUnaffectedPTOMessage) (this request does not affect PTO balance)#end</span><br>
#end
#macro ( statusLine $request)
<span style="$css_h_width">Request Status: </span><span>$request.status</span><br>
#end
#macro ( printRequestInfo $request $showUnaffectedPTOMessage )
#personLine( $request )
#leaveTypeLine($request)
#datesLine( $request )
#timeRequestedLine( $request $showUnaffectedPTOMessage )
#end
#macro ( header $recipient )
#initStyles()<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''><span style="font-family: Arial Unicode MS, Arial, sans-serif; font-weight: normal; font-size: 14px">
#greetByName( $recipient )
#end
#macro ( greetByName $recipient )
Dear $recipient.firstNameEscaped,<br><br>
#end
#macro ( showFooter $footer )
<br/>
<hr noshade size="1" style="background-color: #333333; color: #333333;" />
<pre style=''font-family: Arial Unicode MS, Arial, sans-serif; font-size: 12px;''>$footer</pre>
#end
#macro ( showYourActiPLANSFooterPlainText )
- - -
Your actiPLANS
#end
#macro ( andOtherProduct $bothPasswordsChangeNote )
#if ($bothPasswordsChangeNote)
and actiTIME #end
#end
#macro ( showOtherProductPasswordChangeNote $bothPasswordsChangeNote )
#if ($bothPasswordsChangeNote)
Please note, that the password will be changed for your actiTIME account too.
#end
#end
',        now());
INSERT INTO mail_template (name, template, last_modified_timestamp) VALUES
        ('request_mgmt_to_ltm.html', '#header( $recipient )
#if($action == "revoke")
$actor.fullNameWithoutUserNameEscaped revoked approval for the following leave time request:
#elseif($action == "approve")
$actor.fullNameWithoutUserNameEscaped approved the following leave time request:
#elseif($action == "reject")
$actor.fullNameWithoutUserNameEscaped rejected the following leave time request:
#elseif($action == "rollback")
$actor.fullNameWithoutUserNameEscaped rolled back rejection for the following leave time request:
#elseif($action == "create")
#if($request.approved)
$actor.fullNameWithoutUserNameEscaped created new leave time request that does not require approval.
#else
$actor.fullNameWithoutUserNameEscaped submitted new leave time request requiring your approval.
#end
#elseif($action == "edit")
#if($request.approved)
$actor.fullNameWithoutUserNameEscaped edited planned leave time.
#else
$actor.fullNameWithoutUserNameEscaped has edited leave time request waiting for your approval.
#end
<br>Request summary after editing:
#else
$actor.fullNameWithoutUserNameEscaped has deleted the following leave time request:
#end
<br><br>
#printRequestInfo( $request, false )
#if($action=="delete")
#statusLine( $request )
#end
<br>
#if($action=="approve")
#if($request.hasStatusMessage)
Approval Comments: $request.statusMessage
<br><br>
#end
#elseif($action=="reject")
#if($request.hasStatusMessage)
Reason: $request.statusMessage
<br><br>
#end
#end
#if( $action == "create" || $action == "edit" || $action == "rollback" )
#if($request.ownerPTOAffected)
$request.owner.firstName''s PTO Balance as of $request.getFirstDate("MMM d") (before request): $request.ownerBalanceBeforeRequest d<br>
$request.owner.firstName''s PTO Balance as of $request.getLastDate("MMM d") (after request): $request.ownerBalanceAfterRequest d
#else
This request does not affect $request.owner.firstName''s PTO balance.
#end
<br><br>
#end
<a href="$linkGenerator.leaveTimePlanningLink" style="$css_link_color;">View leave time chart</a><br>
<a href="$linkGenerator.leaveTimeRequestsLink" style="$css_link_color;">View leave time requests</a><br>
<br>
#if(!($!footer == ""))
#showFooter($footer)
#end',  now());
INSERT INTO mail_template (name, template, last_modified_timestamp) VALUES
        ('request_mgmt_to_user.html','#header( $recipient )
#if($action == "revoke")
$actor.fullNameWithoutUserNameEscaped revoked approval for your leave time request.
#elseif($action == "approve")
$actor.fullNameWithoutUserNameEscaped approved your leave time request.
#elseif($action == "reject")
$actor.fullNameWithoutUserNameEscaped rejected your leave time request.
#elseif($action == "rollback")
$actor.fullNameWithoutUserNameEscaped rolled back rejection for your leave time request.
#elseif($action == "create")
#if($request.approved)
$actor.fullNameWithoutUserNameEscaped planned the following leave time for you:
#else
$actor.fullNameWithoutUserNameEscaped submitted the following leave time request for you:
#end
#elseif($action == "edit")
#if($request.approved)
$actor.fullNameWithoutUserNameEscaped edited planned leave time for you.
#else
$actor.fullNameWithoutUserNameEscaped edited leave time request for you.
#end
<br>Request summary after editing:
#else
$actor.fullNameWithoutUserNameEscaped deleted your leave time request:
#end
<br><br>
#leaveTypeLine($request)
#datesLine( $request )
#timeRequestedLine( $request false )
#if($action=="delete")
#statusLine( $request )
#end
<br>
#if($action=="approve")
#if($request.hasStatusMessage)
Approval Comments: $request.statusMessage
<br><br>
#end
#elseif($action=="reject")
#if($request.hasStatusMessage)
Reason: $request.statusMessage
<br><br>
#end
#elseif( $action == "create" || $action == "edit" )
#if($request.ownerPTOAffected)
Your PTO Balance as of $request.getFirstDate("MMM d") (before request): $request.ownerBalanceBeforeRequest d<br>
Your PTO Balance as of $request.getLastDate("MMM d") (after request): $request.ownerBalanceAfterRequest d
#else
This request does not affect your PTO balance.
#end
<br><br>
#end
<a href="$linkGenerator.myScheduleLink" style="$css_link_color;">View your leave time requests in actiPLANS</a><br>
<br>
#if(!($!footer == ""))
#showFooter($footer)
#end', now());
INSERT INTO mail_template (name, template, last_modified_timestamp) VALUES
        ('n_days_to_ltm.html',       '#header( $recipient )
There are leave time requests waiting for your approval.<br>
<br>
#foreach($request in $requests)
#printRequestInfo( $request, true )
<br>
#end
Please remember that not approved requests will be automatically rejected in 24 hours after their start.<br>
<br>
<a href="$linkGenerator.leaveTimePlanningLink" style="$css_link_color;">View leave time chart</a><br>
<a href="$linkGenerator.leaveTimeRequestsLink" style="$css_link_color;">View leave time requests</a><br>
<br>
#if(!($!footer == ""))
#showFooter($footer)
#end',        now());
INSERT INTO mail_template (name, template, last_modified_timestamp) VALUES
('ap_test_message.html', '#initStyles()
<html>
<body>
<meta http-equiv=''Content-Type'' content=''text/html; charset=utf-8''>
<table style=''width: 560px;$css_text font-size: 12px;'' cellspacing=''0''>
<tr>
<td valign=''middle''>#greetByName($recipient)
This is an automatic email message sent by actiPLANS to test email settings.
<br>Please verify that the following notification parameters are configured as you expected:
<ol>
<li style=''margin-left: 0; padding-left: 0; margin-bottom: 20px;'' >
Prefix to add to the notification subject field<br><br>
Use of a prefix for actiPLANS messages simplifies creation of
mail filtering rules in your email client.
$prefixNote
</li>
<li style=''margin-left: 0; padding-left: 0; margin-bottom: 20px;'' >
Standard message footer<br><br>
Standard message footer will be added to all notification messages (including this one).
$footerNote
</li>
</ol>
</td>
</tr>
#if(!($!footer == ""))
<tr>
<td>#showFooter($footer)</td>
</tr>
#end
</table>
</body>
</html>', now());

INSERT INTO mail_template (name, template, last_modified_timestamp) VALUES
('ap_password_recovery_link.txt', CHR(10) & 'Hello $recipient.firstName,' &
               CHR(10) & CHR(10) & 'There was a request to reset password on your actiPLANS account.' &
               CHR(10) & CHR(10) & 'If you requested this password change,' & CHR(10) & 'please follow the link below to set a new password:' &
               CHR(10) & CHR(10) & '$url$token' &
               CHR(10) & '#showOtherProductPasswordChangeNote($bothPasswordsChangeNote)' & CHR(10) & CHR(10) & 'If you didn''t request to reset your password,' & CHR(10) & 'you can safely disregard this email.' &
               CHR(10) & CHR(10) & '#showYourActiPLANSFooterPlainText()', now());

INSERT INTO mail_template (name, template, last_modified_timestamp) VALUES
('ap_password_reset.txt', CHR(10) & 'Hello $recipient.firstName,' &
               CHR(10) & CHR(10) & 'Your actiPLANS #andOtherProduct($bothPasswordsChangeNote)password has been changed successfully.' & CHR(10) & 'Your username is: $recipient.name' &
               CHR(10) & CHR(10) & 'Keep on plannin''' &
               CHR(10) & CHR(10) & '#showYourActiPLANSFooterPlainText()', now());

CREATE TABLE user_cur_pto_balance (
  user_id int NOT NULL,
  balance double NOT NULL,
  expiry date NOT NULL,
  PRIMARY KEY (user_id),
  CONSTRAINT user_cur_pto_balance_at_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX user_cur_pto_balance_user_id_expiry ON user_cur_pto_balance (user_id, expiry);


create table default_workday_duration_history (
  duration int not null,
  effective_date date not null primary key
);

insert into default_workday_duration_history (duration, effective_date)
  select property_value, '1970-01-01' from system_settings where property_name = "workday_duration";

delete from system_settings where property_name = "workday_duration";

alter table user_date_leave alter column leave_duration int not null;
alter table leave_type drop column whole_day_text;

-- Update revisions of all leaves except deleted
update user_date_leave_revision as udlr
  inner join user_date_leave as udl on udlr.user_id = udl.user_id and udlr.leave_date = udl.leave_date
  set revision=revision+DMAX('revision','user_date_leave_revision','');

alter table leave_type drop column is_planned;


create table deleted_user (
  id          int not null primary key,
  username    varchar(255) not null,
  first_name  varchar(32)  not null,
  middle_name varchar(32)  default null,
  last_name   varchar(32)  not null,
  constraint fk_deleted_user_id foreign key (id) references at_user_revision(id)
);

-- reference to at_user from tt_lock was removed in the previous patch
alter table tt_lock add constraint tt_lock_who_locked_id foreign key (who_locked_id) references at_user_revision (id);


create table user_context_name (
  user_id int not null,
  at_context_name varchar(100) default null,
  ap_context_name varchar(100) default null,
  PRIMARY KEY (user_id)
);


-- update param name and param value of leave types selection in Leave Report
update report_params set param_name = 'leaveTypesOption' where param_name = 'allLeaveTypes';
update report_params set param_value = 'all' where param_name = 'leaveTypesOption' and param_value = 'true';
update report_params set param_value = 'specific' where param_name = 'leaveTypesOption' and param_value = 'false';

alter table overtime drop constraint fk_userid;
alter table overtime add constraint fk_userid foreign key (user_id) references at_user(id) ON DELETE CASCADE ON UPDATE CASCADE;

-- ----------------------------------------------------


insert
  into patch_history (patch, pack_version, patch_timestamp)
  values (22, 0, now());