CREATE TABLE user_tt_approver (
  user_id int not null,
  approver_id int not null,
  PRIMARY KEY (user_id, approver_id)

  -- these constraints are processed by the application due to the constraints limitation (32) in ms access
  -- CONSTRAINT fk_user_tt_approver_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE,
  -- CONSTRAINT fk_user_tt_approver_approver_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE TABLE user_ltr_approver (
  user_id int NOT NULL,
  approver_id int NOT NULL,
  PRIMARY KEY (user_id, approver_id)


  -- these constraints are processed by the application due to the constraints limitation (32) in ms access
  -- CONSTRAINT fk_user_approver_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE,
  -- CONSTRAINT fk_user_approver_approver_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE
);

INSERT INTO user_ltr_approver(user_id, approver_id) SELECT user_id, approver_id FROM user_approver;
DROP TABLE user_approver;

alter table at_user add requires_tt_approval bit NOT NULL;
update at_user set requires_tt_approval=true;
alter table at_user add requires_ltr_approval bit NOT NULL;
update at_user set requires_ltr_approval = requires_approval;
alter table at_user drop requires_approval;

alter table at_user add manage_all_users_tt bit NOT NULL;
update at_user set manage_all_users_tt=false;
alter table at_user add manage_all_users_ltr bit NOT NULL;
update at_user set manage_all_users_ltr=false;


create table user_pto_stats (
  user_id int not null,
  approved_pto double not null,
  not_approved_pto double not null,

  primary key (user_id)
);

create table week_approval_current_status (
  user_id int NOT NULL,
  week_date date not null,
  status int NOT NULL,
  comment text,
  status_timestamp datetime NOT NULL,
  approver_id int null,
  CONSTRAINT pk_user_week PRIMARY KEY (user_id, week_date),
  CONSTRAINT fk_week_approval_current_status_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT fk_week_approval_current_status_approver_id FOREIGN KEY (approver_id) REFERENCES at_user_revision (id)
);

create index week_approval_current_status_week_date_idx on week_approval_current_status (week_date);

create table week_approval_status_history (
  id autoincrement primary key,
  status_timestamp datetime NOT NULL,
  user_id int NOT NULL,
  approver_id int null,
  week_date date not null,
  status int NOT NULL,
  comment text,
  CONSTRAINT fk_week_approval_status_history_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT fk_week_approval_status_history_approver_id FOREIGN KEY (approver_id) REFERENCES at_user_revision (id)
);

create index week_approval_status_history_week_date_idx on week_approval_status_history (week_date);
create index week_approval_status_history_user_id_week_date_idx on week_approval_status_history (user_id, week_date);


update at_user set manage_all_users_ltr = true, requires_ltr_approval = false
  where (exists (select 1 from system_settings where property_name='ltm_can_manage_any_requests' and property_value = 'true'))
    and
  id in ( select user_id from user_access_right where access_right_id = 21 );

insert into system_settings (property_name, property_value)
  select top 1 'ltm_can_manage_auto_approved', 'true' from system_settings
where exists (select 1 from system_settings where property_name='ltm_can_manage_any_requests' and property_value = 'true');

delete from system_settings where property_name = 'ltm_can_manage_any_requests';

update at_user set manage_all_users_tt = true
  where  not exists ( select 1 from system_settings where property_name='tt_approval_enabled' )
    and
  id in ( select user_id from user_access_right where access_right_id = 12 );

insert into system_settings(property_name, property_value)
  select distinct 'tt_approval_enabled', 'false' from system_settings
  where not exists ( select 1 from system_settings where property_name='tt_approval_enabled' );

-- ----------------------------------------------------

insert
into patch_history (patch, pack_version, patch_timestamp)
values (28, 0, now());