CREATE TABLE user_tt_approver (
  user_id int(3) unsigned not null,
  approver_id int(3) unsigned not null,
  PRIMARY KEY (user_id, approver_id),
  CONSTRAINT fk_user_tt_approver_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT fk_user_tt_approver_approver_id FOREIGN KEY (approver_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

rename table user_approver to user_ltr_approver;

alter table at_user change requires_approval requires_ltr_approval tinyint(1) not null default 0;
alter table at_user add requires_tt_approval tinyint(1) not null default 0;
alter table at_user add manage_all_users_tt tinyint(1) not null default 0;
alter table at_user add manage_all_users_ltr tinyint(1) not null default 0;

update at_user set manage_all_users_tt=false, manage_all_users_ltr=false, requires_tt_approval=true;


create table user_pto_stats (
  user_id int(3) unsigned not null,
  approved_pto double not null,
  not_approved_pto double not null,

  primary key (user_id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

-- no FOREIGN KEY for user_id references at the at_user(id) because it is a table for temporary data.


create table week_approval_current_status (
  user_id int(3) unsigned NOT NULL,
  week_date date not null,
  status int(3) unsigned NOT NULL,
  comment text,
  status_timestamp datetime NOT NULL,
  approver_id int(3) unsigned null,
  PRIMARY KEY (user_id, week_date),
  CONSTRAINT fk_week_approval_current_status_user_id FOREIGN KEY (user_id) REFERENCES at_user (id)
    ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT fk_week_approval_current_status_approver_id FOREIGN KEY (approver_id) REFERENCES at_user_revision (id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

create index week_approval_current_status_week_date_idx on week_approval_current_status (week_date);

create table week_approval_status_history (
  id int(3) unsigned NOT NULL AUTO_INCREMENT,
  status_timestamp datetime NOT NULL,
  user_id int(3) unsigned NOT NULL,
  approver_id int(3) unsigned null,
  week_date date not null,
  status int(3) unsigned NOT NULL,
  comment text,
  PRIMARY KEY (id),
  CONSTRAINT fk_week_approval_status_history_user_id FOREIGN KEY (user_id) REFERENCES at_user (id)
    ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT fk_week_approval_status_history_approver_id FOREIGN KEY (approver_id) REFERENCES at_user_revision (id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

create index week_approval_status_history_week_date_idx on week_approval_status_history (week_date);
create index week_approval_status_history_user_id_week_date_idx on week_approval_status_history (user_id, week_date);


update at_user set manage_all_users_ltr = true, requires_ltr_approval = false
  where (exists (select 1 from system_settings where property_name='ltm_can_manage_any_requests' and property_value = 'true'))
    and
  id in ( select user_id from user_access_right where access_right_id = 21 );

insert into system_settings (property_name, property_value)
  select 'ltm_can_manage_auto_approved', 'true' from system_settings
  where exists (select 1 from system_settings where property_name='ltm_can_manage_any_requests' and property_value = 'true') limit 1;

delete from system_settings where property_name = 'ltm_can_manage_any_requests';

update at_user set manage_all_users_tt = true
  where  not exists ( select 1 from system_settings where property_name='tt_approval_enabled' )
    and
  id in ( select user_id from user_access_right where access_right_id = 12 );

insert into system_settings(property_name, property_value) 
  select distinct 'tt_approval_enabled', 'false' from system_settings 
   where not exists ( select 1 from system_settings where property_name='tt_approval_enabled' );

-- -----------------------------------------------------------------------------------------------------------

insert
into patch_history (patch, pack_version, patch_timestamp)
values (28, 0, now());

commit;