update mail_template set template = '##import_template(personal_earnings.html)', last_modified_timestamp = now() where name = 'personal_earnings.html';
update mail_template set template = '##import_template(request_mgmt_to_ltm.html)', last_modified_timestamp = now() where name = 'request_mgmt_to_ltm.html';
update mail_template set template = '##import_template(request_mgmt_to_user.html)', last_modified_timestamp = now() where name = 'request_mgmt_to_user.html';
update mail_template set template = '##import_template(shared_macro_ap.vm)', last_modified_timestamp = now() where name = 'shared_macro_ap.vm';
update mail_template set template = '##import_template(unsubmitted_tt.html)', last_modified_timestamp = now() where name = 'unsubmitted_tt.html';
update mail_template set template = '##import_template(test_message.html)', last_modified_timestamp = now() where name = 'test_message.html';
update mail_template set template = '##import_template(test_message_basic.html)', last_modified_timestamp = now() where name = 'test_message_basic.html';
update mail_template set template = '##import_template(test_message_online.html)', last_modified_timestamp = now() where name = 'test_message_online.html';

insert into system_settings ( property_name, property_value ) values ( 'sick_days_balance_enabled', 'true' );

-- update pto accrual rules in system settings
update system_settings set property_name = 'pto_accrual_rule_enabled' where property_name = 'accrual_rule_enabled';
update system_settings set property_name = 'pto_accrual_rule_frequency_type' where property_name = 'accrual_rule_frequency_type';
update system_settings set property_name = 'pto_accrual_rule_parameters' where property_name = 'accrual_rule_parameters';
update system_settings set property_name = 'pto_accrual_rule_value' where property_name = 'accrual_rule_value';

-- update pto reset rules in system settings
update system_settings set property_name = 'pto_reset_rule_enabled' where property_name = 'reset_rule_enabled';
update system_settings set property_name = 'pto_reset_rule_value' where property_name = 'reset_rule_value';
insert into system_settings ( property_name, property_value ) values ( 'pto_reset_rule_frequency_type', 'YEARLY' );
insert into system_settings ( property_name, property_value )
  select 'pto_reset_rule_parameters' as property_name,
         a.property_value & ',' & b.property_value as property_value
    from system_settings a, system_settings b
   where a.property_name = 'reset_rule_month' and b.property_name = 'reset_rule_day_of_month';
delete from system_settings where property_name like 'reset_rule_%month';

--  update pto cap rules in system settings
update system_settings set property_name = 'pto_cap_rule_enabled' where property_name = 'cap_rule_enabled';
update system_settings set property_name = 'pto_cap_rule_value' where property_name = 'cap_rule_value';

-- update report params (replace ONLY_WITH_BALANCE qualifier by leaves with PTO)

INSERT INTO report_params(config_id, param_name, param_value)
 SELECT
		rpt.config_id,
		switch(rpt.config_id in (SELECT id FROM report_configs WHERE format_id=4 ), 'dataParameters.leaveTypeSelection.leaveTypes', True, 'leaveTypeSelection.leaveTypes'),
		l.id
FROM leave_type l, report_params rpt WHERE l.is_pto=true and rpt.param_value='ONLY_WITH_BALANCE' ;

UPDATE report_params SET param_value='SELECTED' WHERE param_value='ONLY_WITH_BALANCE';

-- end update report params


-- add balance type

CREATE TABLE user_balance_rule (
   id counter NOT NULL primary key,
   user_id int NOT NULL,
   type int NOT NULL,
   `value` double NOT NULL,
   frequency varchar(255) NOT NULL,
   frequency_params varchar(255) NOT NULL,
   effective_start_date date NOT NULL,
   effective_finish_date date,
   mgr_id int,
   mgr_comment text,
   balance_type varchar(4) NOT NULL,
   CONSTRAINT user_balance_rule_mgr_id FOREIGN KEY (mgr_id) REFERENCES at_user_revision (id)
 );

INSERT INTO user_balance_rule (user_id, type, `value`, frequency, frequency_params, effective_start_date, effective_finish_date,
  mgr_id, mgr_comment, balance_type)
  SELECT user_id, type, `value`, frequency, frequency_params, effective_start_date, effective_finish_date,
  mgr_id, mgr_comment, 'PTO'
  FROM user_pto_rule;

DROP TABLE user_pto_rule;

ALTER TABLE user_balance_rule ADD CONSTRAINT user_balance_rule_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE;


CREATE TABLE user_balance_cap (
  id counter NOT NULL PRIMARY KEY,
  user_id int NOT NULL,
  cap double NULL,
  effective_date date NOT NULL,
  balance_type varchar(4) NOT NULL,
  CONSTRAINT uk_user_balance_cap_effective_date UNIQUE(user_id, effective_date, balance_type)
);

INSERT INTO user_balance_cap (user_id, cap, effective_date, balance_type)
  SELECT user_id, cap, effective_date, 'PTO'
  FROM user_pto_cap;

DROP TABLE user_pto_cap;

ALTER TABLE user_balance_cap ADD CONSTRAINT user_balance_cap_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE;


CREATE TABLE user_balance_stats (
  user_id int NOT NULL,
  approved_balance double NOT NULL,
  not_approved_balance double NOT NULL,
  balance_type varchar(4) NOT NULL,
  primary key (user_id, balance_type)
);

INSERT INTO user_balance_stats (user_id, approved_balance, not_approved_balance, balance_type)
  SELECT user_id, approved_pto, not_approved_pto, 'PTO'
  FROM user_pto_stats;

DROP TABLE user_pto_stats;


CREATE TABLE user_cur_balance (
  user_id int NOT NULL,
  balance double NOT NULL,
  expiry date NOT NULL,
  balance_type varchar(4) NOT NULL,
  PRIMARY KEY (user_id, balance_type)
);

CREATE INDEX user_cur_balance_user_id_expiry ON user_cur_balance (user_id, expiry);

INSERT INTO user_cur_balance (user_id, balance, expiry, balance_type)
  SELECT user_id, balance, expiry, 'PTO'
  FROM user_cur_pto_balance;

DROP TABLE user_cur_pto_balance;

ALTER TABLE user_cur_balance ADD CONSTRAINT user_cur_balance_at_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE;


alter table at_user add is_sick bit NOT NULL;
alter table at_user add def_sick_rules int NOT NULL;

update at_user set is_sick=false, def_sick_rules=true;

alter table leave_type add balance_type varchar(255);
update leave_type set balance_type = 'PTO' where is_pto = true;
alter table leave_type drop column is_pto;

update leave_type set balance_type = 'SICK' where name = 'Sick Leave' and balance_type is null;

ALTER TABLE leave_type ADD balance_coeff double NOT NULL ;
UPDATE leave_type SET balance_coeff = pto_coeff;
ALTER TABLE leave_type DROP COLUMN pto_coeff;

update access_right set name = 'role.leave_balance_manager' where id = 22;
update access_right set name = 'system.manage_pto_and_sick' where id = 14;

-- end add balance type

-- -----------------------------------------------------------------------------------------------------------

insert
into patch_history (patch, pack_version, patch_timestamp)
values (32, 0, now());