update mail_template set template = '##import_template(personal_earnings.html)', last_modified_timestamp = now() where name = 'personal_earnings.html';
update mail_template set template = '##import_template(request_mgmt_to_ltm.html)', last_modified_timestamp = now() where name = 'request_mgmt_to_ltm.html';
update mail_template set template = '##import_template(request_mgmt_to_user.html)', last_modified_timestamp = now() where name = 'request_mgmt_to_user.html';
update mail_template set template = '##import_template(shared_macro_ap.vm)', last_modified_timestamp = now() where name = 'shared_macro_ap.vm';
update mail_template set template = '##import_template(unsubmitted_tt.html)', last_modified_timestamp = now() where name = 'unsubmitted_tt.html';
update mail_template set template = '##import_template(test_message.html)', last_modified_timestamp = now() where name = 'test_message.html';
update mail_template set template = '##import_template(test_message_basic.html)', last_modified_timestamp = now() where name = 'test_message_basic.html';
update mail_template set template = '##import_template(test_message_online.html)', last_modified_timestamp = now() where name = 'test_message_online.html';

update system_settings
set property_value = 'ALL_USERS'
where property_name = 'colleagues_visibility' and property_value = 'HIDE_ALL';

insert into system_settings ( property_name, property_value ) values ( 'sick_days_balance_enabled', 'true' );

-- update pto accrual rules in system settings
update system_settings set property_name = 'pto_accrual_rule_enabled' where property_name = 'accrual_rule_enabled';
update system_settings set property_name = 'pto_accrual_rule_frequency_type' where property_name = 'accrual_rule_frequency_type';
update system_settings set property_name = 'pto_accrual_rule_parameters' where property_name = 'accrual_rule_parameters';
update system_settings set property_name = 'pto_accrual_rule_value' where property_name = 'accrual_rule_value';

-- update pto reset rules in system settings
update system_settings set property_name = 'pto_reset_rule_enabled' where property_name = 'reset_rule_enabled';
update system_settings set property_name = 'pto_reset_rule_value' where property_name = 'reset_rule_value';
insert into system_settings ( property_name, property_value ) values ( 'pto_reset_rule_frequency_type', 'YEARLY' );
select @reset_rule_parameters := concat( a.property_value, ',', b.property_value )
  from system_settings a join system_settings b on b.property_name = 'reset_rule_day_of_month'
 where a.property_name = 'reset_rule_month';
insert into system_settings ( property_name, property_value ) values ( 'pto_reset_rule_parameters', @reset_rule_parameters );
delete from system_settings where property_name like 'reset_rule_%month';

--  update pto cap rules in system settings
update system_settings set property_name = 'pto_cap_rule_enabled' where property_name = 'cap_rule_enabled';
update system_settings set property_name = 'pto_cap_rule_value' where property_name = 'cap_rule_value';

-- update report params (replace ONLY_WITH_BALANCE qualifier by leaves with PTO)
INSERT INTO report_params(config_id, param_name, param_value)
  SELECT
    rpt.config_id,
    IF(rpt.config_id in (SELECT id FROM report_configs WHERE format_id=4 ), 'dataParameters.leaveTypeSelection.leaveTypes', 'leaveTypeSelection.leaveTypes'),
    l.id
  FROM leave_type l INNER JOIN report_params rpt WHERE l.is_pto=true and rpt.param_value='ONLY_WITH_BALANCE' ;


UPDATE report_params SET param_value='SELECTED' WHERE param_value='ONLY_WITH_BALANCE';
-- end update report params


CREATE TABLE user_balance_rule (
  id int(5) unsigned NOT NULL AUTO_INCREMENT,
  user_id int(3) unsigned NOT NULL,
  type tinyint(1) unsigned NOT NULL,
  value double NOT NULL,
  frequency varchar(255) NOT NULL,
  frequency_params varchar(255) NOT NULL DEFAULT '',
  effective_start_date date NOT NULL,
  effective_finish_date date DEFAULT NULL,
  mgr_id int(3) unsigned DEFAULT NULL,
  mgr_comment text,
  balance_type varchar(4) NOT NULL DEFAULT 'PTO',
  PRIMARY KEY (id),
  CONSTRAINT user_balance_rule_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE,
  CONSTRAINT user_balance_rule_mgr_id_at_user_id FOREIGN KEY (mgr_id) REFERENCES at_user_revision (id)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

INSERT INTO user_balance_rule (user_id, type, value, frequency, frequency_params, effective_start_date,
  effective_finish_date, mgr_id, mgr_comment)
  SELECT user_id, type, value, frequency, frequency_params, effective_start_date,
  effective_finish_date, mgr_id, mgr_comment FROM user_pto_rule;

DROP TABLE user_pto_rule;


CREATE TABLE user_balance_cap (
  id int(5) unsigned NOT NULL AUTO_INCREMENT,
  user_id int(3) unsigned NOT NULL,
  cap double NULL,
  effective_date date NOT NULL,
  balance_type varchar(4) NOT NULL DEFAULT 'PTO',
  PRIMARY KEY (id),
  CONSTRAINT uk_user_balance_cap_effective_date UNIQUE(user_id, effective_date, balance_type),
  CONSTRAINT user_balance_cap_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

INSERT INTO user_balance_cap (user_id, cap, effective_date)
  SELECT user_id, cap, effective_date FROM user_pto_cap;

DROP TABLE user_pto_cap;


CREATE TABLE user_cur_balance (
  user_id int(3) unsigned NOT NULL,
  balance double NOT NULL,
  expiry datetime NOT NULL,
  balance_type varchar(4) NOT NULL DEFAULT 'PTO',
  PRIMARY KEY (user_id, balance_type),
  CONSTRAINT user_cur_balance_at_user_id FOREIGN KEY (user_id) REFERENCES at_user (id) ON DELETE CASCADE ON UPDATE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

ALTER TABLE user_cur_balance ADD INDEX user_cur_pto_balance_user_id_expiry (user_id, expiry);

INSERT INTO user_cur_balance (user_id, balance, expiry)
  SELECT user_id, balance, expiry
  FROM user_cur_pto_balance;

DROP TABLE user_cur_pto_balance;


CREATE TABLE user_balance_stats (
  user_id int(3) unsigned NOT NULL,
  approved_balance double NOT NULL,
  not_approved_balance double NOT NULL,
  balance_type varchar(4) NOT NULL DEFAULT 'PTO',
  primary key (user_id, balance_type)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

INSERT INTO user_balance_stats (user_id, approved_balance, not_approved_balance)
  SELECT user_id, approved_pto, not_approved_pto FROM user_pto_stats;

DROP TABLE user_pto_stats;

ALTER TABLE at_user
  add is_sick tinyint(1) NOT NULL DEFAULT 0,
  add def_sick_rules tinyint(1) NOT NULL DEFAULT 1,
  add KEY is_sick (is_sick);

update at_user set is_sick=false, def_sick_rules=true;

alter table leave_type add column balance_type varchar(255);
update leave_type set balance_type = 'PTO' where is_pto = 1;
alter table leave_type drop column is_pto;

update leave_type set balance_type = 'SICK' where name = 'Sick Leave' and balance_type is null;

ALTER TABLE leave_type ADD COLUMN balance_coeff double(10,2) unsigned NOT NULL DEFAULT 1.00;
UPDATE leave_type SET balance_coeff = pto_coeff;
ALTER TABLE leave_type DROP COLUMN pto_coeff;

update access_right set name = 'role.leave_balance_manager' where id = 22;
update access_right set name = 'system.manage_pto_and_sick' where id = 14;

-- -----------------------------------------------------------------------------------------------------------

insert
into patch_history (patch, pack_version, patch_timestamp)
values (32, 0, now());

commit;