CREATE TABLE tt_record_backup
(
  user_id INTEGER,
  task_id INTEGER,
  record_date DATETIME,
  actuals INTEGER,

  backup_timestamp DATETIME
);

CREATE TABLE task_backup
(
  id INTEGER,
  customer_id INTEGER,
  project_id INTEGER,
  create_timestamp DATETIME,
  completion_date DATETIME,
  name VARCHAR(191),
  name_lower VARCHAR(191),
  description MEMO,
  deadline_date DATETIME,
  billing_type_id INTEGER,
  budget INTEGER,
  parent_id INTEGER,

  backup_timestamp DATETIME
);

CREATE TABLE project_backup
(
  id INTEGER,
  customer_id INTEGER,
  create_timestamp DATETIME,
  name VARCHAR(191),
  name_lower VARCHAR(191),
  description MEMO,
  archiving_timestamp DATETIME,

  backup_timestamp DATETIME
);

CREATE TABLE customer_backup
(
  id INTEGER,
  create_timestamp DATETIME,
  name VARCHAR(191),
  name_lower VARCHAR(191),
  description MEMO,
  archiving_timestamp DATETIME,

  backup_timestamp DATETIME
);

CREATE TABLE user_project_backup
(
  user_id INTEGER,
  project_id INTEGER,

  backup_timestamp DATETIME
);

CREATE TABLE user_task_comment_backup
(
  user_id INTEGER,
  task_id INTEGER,
  comment_date DATETIME,
  comments MEMO,

  backup_timestamp DATETIME
);

-- Lock revisions

create table tt_lock_revision
(
  locked_user_id integer not null,
  `date` date not null,
  revision counter not null,
  constraint uk_tt_lock_revision unique(revision)
);

-- see comment in mysql patch
insert into tt_lock_revision(`date`, locked_user_id)
select distinct d.selected_date, tt_lock.locked_user_id from tt_lock,
(select DateAdd("d", t4.i*10000 + t3.i*1000 + t2.i*100 + t1.i*10 + t0.i, #1970/01/01#) as selected_date from
(select 0 as i from tt_lock union select 1 from tt_lock union select 2 from tt_lock union select 3 from tt_lock union select 4 from tt_lock union select 5 from tt_lock union select 6 from tt_lock union select 7 from tt_lock union select 8 from tt_lock union select 9 from tt_lock) as t0,
(select 0 as i from tt_lock union select 1 from tt_lock union select 2 from tt_lock union select 3 from tt_lock union select 4 from tt_lock union select 5 from tt_lock union select 6 from tt_lock union select 7 from tt_lock union select 8 from tt_lock union select 9 from tt_lock) as t1,
(select 0 as i from tt_lock union select 1 from tt_lock union select 2 from tt_lock union select 3 from tt_lock union select 4 from tt_lock union select 5 from tt_lock union select 6 from tt_lock union select 7 from tt_lock union select 8 from tt_lock union select 9 from tt_lock) as t2,
(select 0 as i from tt_lock union select 1 from tt_lock union select 2 from tt_lock union select 3 from tt_lock union select 4 from tt_lock union select 5 from tt_lock union select 6 from tt_lock union select 7 from tt_lock union select 8 from tt_lock union select 9 from tt_lock) as t3,
(select 0 as i from tt_lock union select 1 from tt_lock union select 2 from tt_lock) as t4) as d
where d.selected_date between tt_lock.date_from and tt_lock.date_to and not date_from is null;

alter table tt_lock_revision alter column revision long not null;
alter table tt_lock_revision add constraint pk_tt_lock_revision primary key (locked_user_id, `date`);

-- Week approval status revisions

create table week_approval_status_revision (
  user_id int not null,
  week_date date not null,
  revision counter not null,
  constraint uk_week_approval_status_revision unique(revision)
);

insert into week_approval_status_revision (user_id, week_date) select user_id, week_date from week_approval_current_status;

alter table week_approval_status_revision alter column revision long not null;
alter table week_approval_status_revision add constraint pk_week_approval_status_revision primary key (user_id, week_date);

alter table week_approval_current_status add constraint fk_week_approval_status_revision foreign key (user_id, week_date) references week_approval_status_revision (user_id, week_date) on delete no action;

-- Overtime revisions

create table overtime_revision (
  user_id int not null,
  overtime_date date not null,
  revision counter not null,
  constraint uk_overtime_revision unique(revision)
);

insert into overtime_revision (user_id, overtime_date) select user_id, overtime_date from overtime;

alter table overtime_revision alter column revision long not null;
alter table overtime_revision add constraint pk_overtime_revision primary key (user_id, overtime_date);

alter table overtime add constraint fk_overtime_revision foreign key (user_id, overtime_date) references overtime_revision (user_id, overtime_date) on delete no action;


alter table at_user drop column fax;
alter table at_user drop column phone;
alter table at_user drop column other_contact;
alter table at_user drop column mobile;


CREATE TABLE TMP_USER_MIN_DATE (
  user_id int NOT NULL,
  first_date date NOT NULL,
  PRIMARY KEY (user_id)
);

INSERT INTO TMP_USER_MIN_DATE(user_id, first_date)
  select user_id, min(first_date) from at_user as u,
    (select user_id, min(record_date) as first_date from tt_record group by user_id UNION
     select user_id, min(comment_date) as first_date from user_task_comment group by user_id UNION
     select user_id, min(leave_date) as first_date  from user_date_leave group by user_id) as tr
  where u.id=tr.user_id and tr.first_date < u.hire_date
  group by user_id;

update at_user inner join TMP_USER_MIN_DATE on at_user.id = TMP_USER_MIN_DATE.user_id
  set at_user.hire_date = TMP_USER_MIN_DATE.first_date;

DROP TABLE TMP_USER_MIN_DATE;



-- IMPORT_LICENSE_KEY_PLACEHOLDER
-- -----------------------------------------------------------------------------------------------------------

INSERT
INTO patch_history (patch, pack_version, patch_timestamp)
VALUES ( 39, 0, now( ) );