CREATE TABLE tt_record_backup
(
  user_id INT(3) unsigned,
  task_id INT(5),
  record_date DATE,
  actuals INT(11),

  backup_timestamp DATETIME
) ENGINE = InnoDB;

CREATE TABLE task_backup
(
  id INT(5),
  customer_id INT(3),
  project_id INT(3),
  create_timestamp DATETIME,
  completion_date DATE,
  name VARCHAR(191),
  name_lower VARCHAR(191),
  description TEXT,
  deadline_date DATE,
  billing_type_id INT(3) unsigned,
  budget INT(10) unsigned,
  parent_id INT(10),

  backup_timestamp DATETIME
) ENGINE = InnoDB;

CREATE TABLE project_backup
(
  id INT(3),
  customer_id INT(11),
  create_timestamp DATETIME,
  name VARCHAR(191),
  name_lower VARCHAR(191),
  description TEXT,
  archiving_timestamp DATETIME,

  backup_timestamp DATETIME
) ENGINE = InnoDB;

CREATE TABLE customer_backup
(
  id INT(3),
  create_timestamp DATETIME,
  name VARCHAR(191),
  name_lower VARCHAR(191),
  description TEXT,
  archiving_timestamp DATETIME,

  backup_timestamp DATETIME
) ENGINE = InnoDB;

CREATE TABLE user_project_backup
(
  user_id INT(6) unsigned,
  project_id INT(6),

  backup_timestamp DATETIME
);

CREATE TABLE user_task_comment_backup
(
  user_id INT(3) unsigned,
  task_id INT(5),
  comment_date DATE,
  comments TEXT,

  backup_timestamp DATETIME
);


-- Lock revisions

create table tt_lock_revision (
  locked_user_id int(3) unsigned not null,
  `date` date not null,
  revision int unsigned not null auto_increment,
  constraint uk_tt_lock_revision unique key(revision)
) ENGINE=InnoDB;

-- Here we need to convert date ranges to dates. We generate 30000 rows by joining a combination of each digit together (00000, 00001, ... 29999).
-- Then we add these numbers to the date '1970-01-01' and get 30000 dates.
create table temp_date (
  `date` date not null
) ENGINE=InnoDB;

insert into temp_date (`date`)
(select adddate('1970-01-01', t4.i*10000 + t3.i*1000 + t2.i*100 + t1.i*10 + t0.i) from
(select 0 i union select 1 union select 2 union select 3 union select 4 union select 5 union select 6 union select 7 union select 8 union select 9) t0,
(select 0 i union select 1 union select 2 union select 3 union select 4 union select 5 union select 6 union select 7 union select 8 union select 9) t1,
(select 0 i union select 1 union select 2 union select 3 union select 4 union select 5 union select 6 union select 7 union select 8 union select 9) t2,
(select 0 i union select 1 union select 2 union select 3 union select 4 union select 5 union select 6 union select 7 union select 8 union select 9) t3,
(select 0 i union select 1 union select 2) t4);

insert into tt_lock_revision (`date`, locked_user_id)
select distinct d.selected_date, tt_lock.locked_user_id from tt_lock,
(select `date` selected_date from temp_date) d
where d.selected_date between tt_lock.date_from and tt_lock.date_to and not date_from is null;

drop table temp_date;

alter table tt_lock_revision modify revision bigint unsigned not null;
alter table tt_lock_revision add constraint pk_tt_lock_revision primary key (locked_user_id, date);

-- Week approval status revisions

create table week_approval_status_revision (
  user_id int(3) unsigned not null,
  week_date date not null,
  revision int unsigned not null auto_increment,
  constraint uk_week_approval_status_revision unique key(revision)
) ENGINE=InnoDB;

insert into week_approval_status_revision (user_id, week_date) select user_id, week_date from week_approval_current_status;

alter table week_approval_status_revision modify revision bigint unsigned not null;
alter table week_approval_status_revision add constraint pk_week_approval_status_revision primary key (user_id, week_date);

alter table week_approval_current_status add constraint fk_week_approval_status_revision
foreign key (user_id, week_date) references week_approval_status_revision (user_id, week_date) on delete no action;

-- Overtime revisions

create table overtime_revision (
  user_id int(3) unsigned not null,
  overtime_date date not null,
  revision int unsigned not null auto_increment,
  constraint uk_overtime_revision unique key(revision)
) ENGINE=InnoDB;

insert into overtime_revision (user_id, overtime_date) select user_id, overtime_date from overtime;

alter table overtime_revision modify revision bigint unsigned not null;
alter table overtime_revision add constraint pk_overtime_revision primary key (user_id, overtime_date);

alter table overtime add constraint fk_overtime_revision
foreign key (user_id, overtime_date) references overtime_revision (user_id, overtime_date) on delete no action;


alter table at_user drop column fax,
                    drop column phone,
                    drop column other_contact,
                    drop column mobile;

-- user_leave_time_request does`t has index for user_id, first_date
-- If we use select instead of update, then we get query for finding users with wrong hire_date
update at_user u,
  (select user_id, min(first_date) first_date from at_user as u,
    (select user_id, min(record_date) as first_date from tt_record group by user_id UNION
      select user_id, min(comment_date) as first_date from user_task_comment group by user_id UNION
      select user_id, min(leave_date) as first_date  from user_date_leave group by user_id UNION
      select user_id, min(first_date) as first_date  from user_leave_time_request group by user_id UNION
      select user_id, min(status_date) as first_date  from ap_leave_status group by user_id) as tr
    where u.id=tr.user_id and tr.first_date < u.hire_date
    group by user_id) tr
  set u.hire_date = tr.first_date
  where u.id = tr.user_id;


-- IMPORT_LICENSE_KEY_PLACEHOLDER
-- -----------------------------------------------------------------------------------------------------------

INSERT
INTO patch_history (patch, pack_version, patch_timestamp)
VALUES ( 39, 0, now( ) );

COMMIT;