drop table tt_lock_revision;

create table tt_lock_revision (
  locked_user_id int(3) unsigned not null,
  `date` date not null,
  revision int unsigned not null auto_increment,
  constraint uk_tt_lock_revision unique key(revision)
) ENGINE=InnoDB;

-- Here we need to convert date ranges to dates. We generate 30000 rows by joining a combination of each digit together (00000, 00001, ... 29999).
-- Then we add these numbers to the date '1970-01-01' and get 30000 dates.
create table temp_date (
  `date` date not null
) ENGINE=InnoDB;

insert into temp_date (`date`)
(select adddate('1970-01-01', t4.i*10000 + t3.i*1000 + t2.i*100 + t1.i*10 + t0.i) from
(select 0 i union select 1 union select 2 union select 3 union select 4 union select 5 union select 6 union select 7 union select 8 union select 9) t0,
(select 0 i union select 1 union select 2 union select 3 union select 4 union select 5 union select 6 union select 7 union select 8 union select 9) t1,
(select 0 i union select 1 union select 2 union select 3 union select 4 union select 5 union select 6 union select 7 union select 8 union select 9) t2,
(select 0 i union select 1 union select 2 union select 3 union select 4 union select 5 union select 6 union select 7 union select 8 union select 9) t3,
(select 0 i union select 1 union select 2) t4);

insert into tt_lock_revision (`date`, locked_user_id)
select distinct d.selected_date, tt_lock.locked_user_id from tt_lock,
(select `date` selected_date from temp_date) d
where d.selected_date between tt_lock.date_from and tt_lock.date_to and not date_from is null;

drop table temp_date;

alter table tt_lock_revision modify revision bigint unsigned not null;
alter table tt_lock_revision add constraint pk_tt_lock_revision primary key (locked_user_id, date);

-- update max revision for next_revision function

REPLACE INTO revision_seq SELECT max(revision) FROM tt_lock_revision;

-- IMPORTANT! manually look at duplicates in "user_leave_time_request" tables on hosting before running this patch
delete ultr1 from user_leave_time_request ultr1 inner join user_leave_time_request ultr2 on (ultr1.user_id = ultr2.user_id and ultr1.first_date = ultr2.first_date) where ultr1.id > ultr2.id;
alter table user_leave_time_request add constraint uk_user_leave_time_request_user_id_first_date unique(user_id, first_date);


-- update leave time csv report to html
update report_configs set format_id = 1 where format_id = 2 and report_form = 10;

-- remove my_leave_time_history user settings
delete from user_settings where param_category='my_leave_time_history';

-- IMPORT_LICENSE_KEY_PLACEHOLDER
-- -----------------------------------------------------------------------------------------------------------

INSERT
INTO patch_history (patch, pack_version, patch_timestamp)
VALUES (41, 0, now());

COMMIT;