-- Creates "null" record for registration information if registration information does not exist (new installation)
-- If registration information already exists (version update) it is not modified by the statement
INSERT INTO registration_info (registrationId, companyName)
    SELECT '', '' FROM (SELECT count(*) AS record_count FROM registration_info) temp
    WHERE NOT EXISTS (SELECT * FROM registration_info);

-- Table to hold versions of CRM site resources
CREATE TABLE crm_resource(

    id      INT NOT NULL,
    version VARCHAR(80) NOT NULL,

    CONSTRAINT pk_crm_resource PRIMARY KEY(id)
);

-- Table to hold information about new version of crm resource for particular user
-- Presence of table entry indicates that new version of particular resource is available for particular user
CREATE TABLE user_new_crm_resource(

    user_id     INT NOT NULL,
    resource_id INT NOT NULL,

    CONSTRAINT pk_user_new_crm_resource PRIMARY KEY (user_id, resource_id),
    CONSTRAINT fk_uncr_user FOREIGN KEY (user_id) REFERENCES at_user(id) ON DELETE CASCADE,
    CONSTRAINT fk_uncr_resource FOREIGN KEY (resource_id) REFERENCES crm_resource(id) ON DELETE CASCADE
);

-- Fix of bug #20746 - unnamed incorrect constraint in "user_work_schedule_calendar" table

-- Copy calendar configurations to temporary table
SELECT user_id, month_count INTO schedule_calendar_temp FROM user_work_schedule_calendar;

-- Drop original table
DROP TABLE user_work_schedule_calendar;

-- Recreate table but with correct constraint definitions
CREATE TABLE user_work_schedule_calendar(

    user_id     int not null,
    month_count tinyint not null,

    CONSTRAINT pk_user_ws_calendar PRIMARY KEY(user_id),
    CONSTRAINT fk_user_ws_calendar_user FOREIGN KEY (user_id) REFERENCES at_user(id) ON DELETE CASCADE
);

-- Copy calendar configurations from temporary table
INSERT INTO user_work_schedule_calendar (user_id, month_count) SELECT user_id, month_count FROM schedule_calendar_temp;

-- Drop temporary table
DROP TABLE schedule_calendar_temp;

insert into patch_history(patch, patch_timestamp)
    values(9, now());