ALTER TABLE access_right CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci;
ALTER TABLE at_user CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci;
ALTER TABLE billing_type CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci;
ALTER TABLE customer CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci;
ALTER TABLE project CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci;
ALTER TABLE registration_info CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci;
ALTER TABLE system_settings CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci;
ALTER TABLE task CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci;
ALTER TABLE user_task_comment CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci;

ALTER DATABASE DEFAULT CHARACTER SET utf8 DEFAULT COLLATE utf8_unicode_ci;

-- Creates "null" record for registration information if registration information does not exist (new installation)
-- If registration information already exists (version update) it is not modified by the statement
INSERT INTO registration_info (registrationId, companyName)
    SELECT '', '' FROM (SELECT count(*) AS record_count FROM registration_info) temp
    WHERE NOT EXISTS (SELECT * FROM registration_info);

-- Indexes to facilitate quick access to time-tracks and comments
ALTER TABLE tt_record ADD INDEX (user_id, record_date);
ALTER TABLE user_task_comment ADD INDEX (user_id, comment_date);

-- Table to hold versions of CRM site resources
CREATE TABLE crm_resource(

    id      INT(3) NOT NULL,
    version VARCHAR(80) NOT NULL,

    CONSTRAINT pk_crm_resource PRIMARY KEY(id)
) ENGINE=InnoDB COMMENT='CRM site resource information';

-- Table to hold information about new version of crm resource for particular user
-- Presence of table entry indicates that new version of particular resource is available for particular user
CREATE TABLE user_new_crm_resource(

    user_id     INT(3) UNSIGNED NOT NULL,
    resource_id INT(3) NOT NULL,

    CONSTRAINT pk_user_new_crm_resource PRIMARY KEY (user_id, resource_id),
    CONSTRAINT fk_uncr_user FOREIGN KEY (user_id) REFERENCES at_user(id) ON DELETE CASCADE,
    CONSTRAINT fk_uncr_resource FOREIGN KEY (resource_id) REFERENCES crm_resource(id) ON DELETE CASCADE
) ENGINE=InnoDB COMMENT='CRM resources not visited by users';

-- "ON DELETE CASCADE" is missing for foreign key constraints for user_id in user work shedule calendar
-- configuration table. So if it is attempted to delete user who already has the configurations error occurs
ALTER TABLE user_work_schedule_calendar DROP FOREIGN KEY fk_user_ws_calendar_user_id;
ALTER TABLE user_work_schedule_calendar ENGINE = InnoDB;
ALTER TABLE user_work_schedule_calendar
    ADD CONSTRAINT fk_user_ws_calendar_user_id FOREIGN KEY (user_id) REFERENCES at_user(id) ON DELETE CASCADE;

insert into patch_history(patch, patch_timestamp)
    values(9, now());

commit;