/**
 * Support request controller; extends from cross domain controller
 */
function SupportController(){

    this.crossDomainController();
    this.registrationId = '';
}

SupportController.prototype = new CrossDomainController();
SupportController.prototype.crossDomainController = CrossDomainController;

/**
 * @return registration id
 */
SupportController.prototype.getRegistrationId = function(){

    return this.registrationId;
};

/**
 * Sets registration id
 */
SupportController.prototype.setRegistrationId = function(registrationId){

    this.registrationId = registrationId;
};

/**
 * Starts execution of this controller
 */
SupportController.prototype.startExecution = function(){
    
    // Begin controller execution with "Registration" state
    this.setState(new CrossDomainControllerState_Registration(this));
    this.crossDomainController.prototype.startExecution.call(this);
};

/**
 * This method is called when cross domain request fails
 */
SupportController.prototype.requestFailed = function(){

    this.crossDomainController.prototype.requestFailed.call(this); // Call parent method

    if(jsonrpc) {
        jsonrpc.abortAllRequests(); // Abort all pending requests
    }
};

SupportController.prototype.showElement = function(elementId){

    this.setElementDisplayProperty(elementId, "");
};

SupportController.prototype.hideElement = function(elementId){

    this.setElementDisplayProperty(elementId, "none");
};

SupportController.prototype.setElementDisplayProperty = function(elementId, propertyValue){

    var element = document.getElementById(elementId);
    element.style.display = propertyValue;
};

/**
 *  State in which controller registrates program
 */
function CrossDomainControllerState_Registration(controller){

    this.parent = CrossDomainControllerState;
    this.parent(controller);

    // Create registrator
    this.registrator = new ProgramRegistrator(this.controller.createStepCompletionHandler());
    // Create "registrator" global reference; since it is referenced in "registration frame"
    registrator = this.registrator;
}

CrossDomainControllerState_Registration.prototype = new CrossDomainControllerState();

CrossDomainControllerState_Registration.prototype.startLoading = function(){

    this.registrator.queryRegistrationId();
};

CrossDomainControllerState_Registration.prototype.loadingFinished = function(){

    this.controller.setRegistrationId(this.registrator.getRegistrationId());
    this.controller.switchToNextState(new CrossDomainControllerState_QueryRequestId(this.controller));
};

