/**
 * Creates iframe with properties specified as parameters
 *
 * @param frameId   frame id
 * @param frameSrc  frame source
 * @param onLoadHandler  "onload" handler
 */
function createWorkFrame(frameId, frameSrc, onLoadHandler){

    var iframe = document.getElementById(frameId);

    // If frame already exists remove it
    if(iframe){
        document.body.removeChild(iframe);
    }

    iframe = document.createElement("IFRAME");  // Create iframe
    iframe.id = frameId;    // Set frame id
    iframe.src = frameSrc;  // Set frame source
    iframe.style.visibility = 'hidden'; // Not visible
    iframe.style.display = 'block'; // Do not reserve space
    iframe.frameBorder = 0; // No border and margins, takes no space
    iframe.height = 0;
    iframe.width = 0;
    iframe.marginHeight = 0;
    iframe.marginWidth = 0;
    document.body.appendChild(iframe);  // Add iframe to document body

    addOnloadHandler(iframe, onLoadHandler); // Add onload event listener
    return iframe;
}

/**
 * Adds onload evenet handler to element
 * @param element element to add handler to
 * @param onLoadHandler handler method
 */
function addOnloadHandler(element, onLoadHandler){

    // If no handler specified simply exit
    if(onLoadHandler == null){

        return;
    }

    // If FireFox
    if(element.addEventListener){

        element.addEventListener("load", onLoadHandler, false);
    }
    else{
        // Otherwise IE assumed
        element.attachEvent("onload", onLoadHandler);
    }
}