// require task_list.js

function TaskListTableController()
{
    this.constructor = TaskListTableController;
}

TaskListTableController.sortOrderClasses = [];
TaskListTableController.sortOrderClasses[ SubmitTTConstants.TASK_LIST_ORDER_BY_TASK_NAME ] = "sortByTaskName";
TaskListTableController.sortOrderClasses[ SubmitTTConstants.TASK_LIST_ORDER_BY_PROJECT_NAME ] = "sortByProjectName";
TaskListTableController.sortOrderClasses[ SubmitTTConstants.TASK_LIST_ORDER_BY_CUSTOMER_NAME ] = "sortByCustomerName";

TaskListTableController.prototype.init = function ( submitTTSettings )
{
    this.currentState = new TaskListTableState().init(
            new TaskListTableOrder().init( submitTTSettings.taskOrder.order, submitTTSettings.taskOrder.asc )
    );
    return this;
};

TaskListTableController.prototype.getCurrentState = function ()
{
    return this.currentState;
};

TaskListTableController.prototype.changeSortOrder = function ( newSortOrder )
{
    this.currentState.taskOrder = new TaskListTableOrder().init( newSortOrder, newSortOrder != this.currentState.taskOrder.order || !this.currentState.taskOrder.asc );
    this.sortTasks();
};

TaskListTableController.prototype.sortTasks = function ()
{
    var tableElement = $( "#tt-table" );
    tableElement.removeClass( "desc " + TaskListTableController.sortOrderClasses.join( ' ' ) );

    var comparator = TaskComparator.getComparator( this.currentState.taskOrder );
    $( "#actualTTRows .taskRow" ).sort(function ( taskRow1, taskRow2 )
                                       {
                                           return comparator.compare(
                                                   SubmitTTData.getTaskById( getTaskIdByRow( taskRow1 ) ),
                                                   SubmitTTData.getTaskById( getTaskIdByRow( taskRow2 ) )
                                           );
                                       } ).appendTo( '#actualTTRows' );
    tableElement.addClass( TaskListTableController.sortOrderClasses[this.currentState.taskOrder.order] + (this.currentState.taskOrder.asc ? ' asc' : ' desc') );
    this.highlightTasksRowsByParity();
    this.separateNotAssignedRows();
};

TaskListTableController.prototype.highlightTasksRowsByParity = function ()
{
    $( "#actualTTRows .taskRow" ).each( function ( index, row )
                                        {
                                            $( row )
                                                    .removeClass( 'odd even' )
                                                    .addClass( (index % 2 == 0) ? 'odd' : 'even' );
                                        } );
};

TaskListTableController.prototype.separateNotAssignedRows = function ()
{
    if (SubmitTTData.notAssignedTasksCount > 0 && SubmitTTData.tasks.size() > 1)
    {
        var $actualTTRows = $("#actualTTRows");
        $actualTTRows.find(".taskRow").removeClass("notAssignedSeparator");
        $actualTTRows.find(".assignedTaskRow").last().addClass("notAssignedSeparator");
    }
};

function TaskListTableState()
{
    this.constructor = TaskListTableState;
}

TaskListTableState.prototype.init = function ( taskOrder )
{
    this.taskOrder = taskOrder;
    return this;
};

function TaskListTableOrder()
{
    this.constructor = TaskListTableState;
}

TaskListTableOrder.prototype.init = function ( order, asc )
{
    this.order = order;
    this.asc = asc;
    return this;
};

function TaskComparator() {}

(function ()
{
    TaskComparator.getComparator = function ( tableOrder )
    {
        return new MultipleComparator( comparators[tableOrder.order], tableOrder.asc );
    };

    var comparators = {};
    comparators[SubmitTTConstants.TASK_LIST_ORDER_BY_TASK_NAME] = [NotAssignedComparator, TaskNameComparator, CustomerNameComparator, ProjectNameComparator];
    comparators[SubmitTTConstants.TASK_LIST_ORDER_BY_PROJECT_NAME] = [NotAssignedComparator, ProjectNameComparator, CustomerNameComparator, TaskNameComparator];
    comparators[SubmitTTConstants.TASK_LIST_ORDER_BY_CUSTOMER_NAME] = [NotAssignedComparator, CustomerNameComparator, ProjectNameComparator, TaskNameComparator];

    function MultipleComparator( comparators, asc )
    {
        this.comparators = comparators;
        this.asc = asc;
    }

    MultipleComparator.prototype.compare = function ( task1, task2 )
    {
        for( var i = 0; i < this.comparators.length; i++ )
        {
            var t = this.comparators[i].compare( task1, task2, this.asc );
            if( t != 0 ) return this.asc ? t : -t;
        }
        return 0;
    };

    function TaskNameComparator() {}

    TaskNameComparator.compare = function ( task1, task2 )
    {
        function getTaskNames( task )
        {
            if( task.hasParentTask() )
            {
                return [task.getParentTask().name, task.name];
            }
            else
            {
                return [task.name, ''];
            }
        }

        var taskNames1 = getTaskNames( task1 );
        var taskNames2 = getTaskNames( task2 );
        return stringComparator( taskNames1[0], taskNames2[0] ) || stringComparator( taskNames1[1], taskNames2[1] );
    };

    function ProjectNameComparator() {}

    ProjectNameComparator.compare = function ( task1, task2 )
    {
        return stringComparator( task1.getProject().name, task2.getProject().name );
    };

    function CustomerNameComparator() {}

    CustomerNameComparator.compare = function ( task1, task2 )
    {
        return stringComparator( task1.getProject().getCustomer().name, task2.getProject().getCustomer().name );
    };

    function NotAssignedComparator() {}

    NotAssignedComparator.compare = function ( task1, task2, asc )
    {
        var result = task1.notAssigned === task2.notAssigned ? 0 : (task1.notAssigned > task2.notAssigned ? 1 : -1);
        if (!asc) result *= -1;
        return result;
    };
})();
