function ReportConfigs ()
{
    this.constructor = ReportConfigs;
}

ReportConfigs.prototype.init = function ( reports, users, userGroups, timeZoneGroups, cpList, leaveTypes )
{
    this.reports = reports;
    this.users = users;
    this.userGroups = userGroups;
    this.timeZoneGroups = timeZoneGroups;
    this.cpList = cpList;
    this.leaveTypes = leaveTypes;

    this.reportParamsStringBuilder = new ReportParamsStringBuilder().init( {
                                                                               users: this.users,
                                                                               timeZoneGroups: this.timeZoneGroups,
                                                                               userGroups: this.userGroups,
                                                                               cpList: this.cpList,
                                                                               leaveTypes: this.leaveTypes
                                                                           }, SystemSettings.dateFormat.replace(/MMMM/, 'MMM') );

    var self = this;

    var titleStrings = self.buildTitlesStrings();
    var paramStrings = self.buildParamsStringsForReportConfigs();
    var dateRangeStrings = self.buildDateParamStringsForReportConfigs();

    for (var reportId in paramStrings) {
        if (paramStrings.hasOwnProperty(reportId)) {
            $("#" + reportId + " .type-title" ).html(titleStrings[reportId]);
            $("#" + reportId + " .otherParams" ).html(paramStrings[reportId]);
            $("#" + reportId + " .date-filter" ).html(dateRangeStrings[reportId]);
        }
    }
};

ReportConfigs.prototype.buildTitlesStrings = function(){
    var self = this;
    var reportTitlesStrings = [];
    var nodeIdBase = "reportConfig_";
    for(var i = 0; i < self.reports.length; i++) {
        reportTitlesStrings[nodeIdBase + self.reports[i].id] = self.getTitleString(self.reports[i]);
    }
    return reportTitlesStrings;
};

ReportConfigs.prototype.getTitleString = function(reportConfig){
    var title = messageResource.getMessage(ReportChartNames[reportConfig.type]);
    if (reportConfig.isChart){
        var self = this;
        title += " " + messageResource.getMessage("saved_reports.charts_title.part2",
                                                  [messageResource.getEscapedMessage(this.getByTextForCharts(self.getParameters(reportConfig)))]);
    }
    return title;
};

ReportConfigs.prototype.getByTextForCharts = function(chartParams){
    switch (chartParams.groupingLevel){
        case ReportGroupingLevel.USER: return "report.common.users";
        case ReportGroupingLevel.USER_GROUP :  return "common.User_Groups";
        case ReportGroupingLevel.TIME_ZONE_GROUP :  return "common.Time_Zone_Groups";
        case ReportGroupingLevel.CUSTOMER : return "common.Customers";
        case ReportGroupingLevel.PROJECT : return "common.Projects";
        case ReportGroupingLevel.BILLING_TYPE : return "report.common.billing_types";
        case ReportGroupingLevel.DATE : return "report.common.dates";
        case ReportGroupingLevel.WEEK : return "report.common.weeks";
        case ReportGroupingLevel.MONTH : return "report.common.months";
    };
};

ReportConfigs.prototype.buildParamsStringsForReportConfigs = function( ) {
    var self = this;
    var nodeIdBase = "reportConfig_";
    var reportParamsStrings = [];
    for(var i = 0; i < self.reports.length; i++) {
        reportParamsStrings[nodeIdBase + self.reports[i].id] = self.getParamsString(self.reports[i]);
    }
    return reportParamsStrings;
};

ReportConfigs.prototype.buildDateParamStringsForReportConfigs = function ()
{
    var self = this;
    var nodeIdBase = "reportConfig_";
    var reportDateRangeStrings = [];
    for( var i = 0; i < self.reports.length; i++ )
    {
        if( self.reports[i].type != "ESTIMATED_VS_ACTUAL" && self.reports[i].type != "PTO_BALANCE" && self.reports[i].type != "SICK_DAYS_BALANCE" )
            reportDateRangeStrings[nodeIdBase + self.reports[i].id] = self.formatHTML(
                    self.reportParamsStringBuilder.getDateRangeParamString( self.getParameters(self.reports[i]).dateRangeSelection ),
                    "dateRange",
                    self.reportParamsStringBuilder.getDateRangeDetailsString( self.getParameters(self.reports[i]).dateRangeSelection ) );
        else
            reportDateRangeStrings[nodeIdBase + self.reports[i].id] = self.formatHTML(
                    self.reportParamsStringBuilder.getEvaDateParamString( self.getParameters(self.reports[i]).dateSelection ),
                    "dateRange",
                    self.reportParamsStringBuilder.getEvaDateDetailsString(self.getParameters(self.reports[i]).dateSelection ) );
    }
    return reportDateRangeStrings;
};

ReportConfigs.prototype.getParameters = function(report){

    if(report.isChart){
        return report.params.dataParameters;
    }
    return report.params;
};


ReportConfigs.prototype.getParamsString = function( reportConfig ){
    var DELIMITER = ";";

    if (reportConfig.type == "STAFF_PERFORMANCE" ||
        reportConfig.type == "TIME_TRACK_IN_DETAIL" ||
        reportConfig.type == "COST_OF_WORK" ||
        reportConfig.type == "TOTAL_WORKING_HOURS" ||
        reportConfig.type == "APPROVED_WORKING_HOURS" )
    {
        return this.getUserSelectionParamString(this.getParameters(reportConfig).userSelection, DELIMITER) +
               this.getCPSelectionParamString(this.getParameters(reportConfig).customersProjectsSelection);
    }

    if( reportConfig.type == "ESTIMATED_VS_ACTUAL" ||
        reportConfig.type == "PROFIT_LOSS" ||
        reportConfig.type == "INVOICING" ||
        reportConfig.type == "BILLING_SUMMARY" ||
        reportConfig.type == "COST_OF_WORK" ||
        reportConfig.type == "BILLABLE_AMOUNTS"  )
    {
        return this.getCPSelectionParamString( this.getParameters(reportConfig).customersProjectsSelection );
    }

    if(  reportConfig.type == "OVERTIME" ||
         reportConfig.type == "OVERTIME_HOURS" )
    {
        var userSelection = this.getFilteredOvertimeUserSelection( this.getParameters(reportConfig).userSelection) ;
        return this.getUserSelectionParamString(userSelection);
    }
    if( reportConfig.type == "PTO_BALANCE" ||
        reportConfig.type == "SICK_DAYS_BALANCE" ) {
        return this.getUserSelectionParamString( this.getParameters(reportConfig).userSelection );
    }

    if( reportConfig.type == "LEAVE_TIME" ||
        reportConfig.type == "LEAVE_HOURS")
    {
        return this.getUserSelectionParamString(this.getParameters(reportConfig).userSelection, DELIMITER) +
                this.getLeaveTypeSelectionParamString(this.getParameters(reportConfig).leaveTypeSelection);
    }
};

ReportConfigs.prototype.formatHTML = function( string, className, alt ) {
    var result = "<span class=\"";
    result += className + "\"";
    if (alt)
        result += " title=\"" + escapeHtmlSymbols( alt ) + "\"";
    result += ">" + escapeHtmlSymbols(string) + "</span>";
    return result.toString();
};


ReportConfigs.prototype.getUserSelectionParamString = function ( userSelection, delimiter )
{
    delimiter = !!delimiter ? delimiter : "";
    return this.formatHTML(
            this.reportParamsStringBuilder.getUserSelectionParamString( userSelection ) + delimiter,
            this.reportParamsStringBuilder.getUserSelectionClass( userSelection ),
            this.reportParamsStringBuilder.getUserSelectionDetailsString( userSelection ) );
};

ReportConfigs.prototype.getCPSelectionParamString = function( cpSelection ) {
    var self = this;
    var CLASS_NAME = "cp-selector";
    var CUSTOMERS_CLASS = "customers-filter";
    var PROJECTS_CLASS = "projects-filter";
    var ERROR_CLASS_NAME = "error-cp-selector";
    var AND_DELIMITER = this.formatHTML("&", "delimiter");

    var customerFormatter = {
        format: function(customer) {
            return customer.name;
        }
    };

    var projectFormatter = {
        format: function(project) {
            return project.name;
        }
    };

    if ( cpSelection.qualifier == CustomersProjectsSelectionQualifier.ALL )
        return this.formatHTML(messageResource.getMessage("saved_reports.cp_selector.all_projects"), CLASS_NAME);

    if( cpSelection.qualifier == CustomersProjectsSelectionQualifier.ALL_ACTIVE )
        return this.formatHTML( messageResource.getMessage("saved_reports.cp_selector.all_active_projects"), CLASS_NAME );
    else {
        var customers = this.reportParamsStringBuilder.getFilteredCustomers(cpSelection.customers);
        var projects = this.reportParamsStringBuilder.getFilteredProjects(cpSelection.projects);

        if( customers.length == 0 && projects.length == 0 )
            return this.formatHTML( messageResource.getMessage( "saved_reports.cp_selector.emty_selection" ),
                                    CLASS_NAME + " " + ERROR_CLASS_NAME );

        var message, alt, key, params;

        if( customers.length == 1 && projects.length == 0 )
        {
            key = "saved_reports.cp_selector.single_element.customer";
            params = [customerFormatter.format( customers[0] )];

            message = messageResource.getMessage(key, params);
            alt = messageResource.getMessage(key, params);

            return this.formatHTML( message, CUSTOMERS_CLASS, alt );
        }

        if( projects.length == 1 && customers.length == 0 )
        {
            key = "saved_reports.cp_selector.single_element.project";
            params = [projectFormatter.format( projects[0] )];

            message = messageResource.getMessage(key, params);
            alt = messageResource.getMessage(key, params);

            return this.formatHTML( message, PROJECTS_CLASS, alt );
        }


        var getCustomersPart = function (customers, availableSuffix, activeProjectsSuffix)
        {
            return self.formatHTML( messageResource.getMessage( "saved_reports.cp_selector.multiple_customer" + ( customers.length == 1 ? "" : "s" ), [customers.length.toString() ,
                                                                       " " + availableSuffix, activeProjectsSuffix] ),
                               CUSTOMERS_CLASS, self.reportParamsStringBuilder.getTitle( messageResource.getMessage( "saved_reports.cp_selector.title_header.customers" ), customers, customerFormatter ) );

        };

        var getProjectsPart = function (projects)
        {
            return self.formatHTML(messageResource.getMessage( "saved_reports.cp_selector.multiple_project" + ( projects.length == 1 ? "" : "s" ), [projects.length.toString()] ),
                              PROJECTS_CLASS, self.reportParamsStringBuilder.getTitle(messageResource.getMessage( "saved_reports.cp_selector.title_header.projects"), projects, projectFormatter));
        };

        var customersPart = "", projectsPart = "";
        if (customers.length > 0)
        {
            customersPart = getCustomersPart(customers, "", "");
        }

        if (projects.length > 0)
        {
            projectsPart = getProjectsPart( projects );
        }
        return customersPart + (customersPart.length > 0 && projectsPart.length > 0 ? AND_DELIMITER : "") + projectsPart;
    }
};

ReportConfigs.prototype.getLeaveTypeSelectionParamString = function ( leaveTypeSelection )
{
    return this.formatHTML(
            this.reportParamsStringBuilder.getLeaveTypeSelectionParamString( leaveTypeSelection ),
            this.reportParamsStringBuilder.getLeaveTypeSelectionClass( leaveTypeSelection ) ,
            this.reportParamsStringBuilder.getLeaveTypeSelectionDetailsString( leaveTypeSelection ) );
};

ReportConfigs.prototype.getFilteredOvertimeUserSelection = function ( userSelection )
{
    var self = this;

    var selectedUsersIds = $.map( $.grep( this.users,
                                   function ( e )
                                   {
                                       return $.inArray( e.id, userSelection.selectedUsers ) != -1 && e.isOvertimeEnabled();
                                   } ), function ( user ) { return user.id; } );
    var selectedGroupIds = [];
    var showUserGroups = EnabledFeatures.UGR && userSelection.groupType == GroupType.USER_GROUP;
    var showTimeZoneGroups = EnabledFeatures.TZG && userSelection.groupType == GroupType.TIME_ZONE_GROUP;
    if ( showUserGroups)
    {
        selectedGroupIds = $.map( $.grep( this.userGroups,
                                          function ( e )
                                          {
                                              var groupId = e.id;
                                              return $.inArray( e.id, userSelection.selectedGroups ) != -1 &&
                                                     $.grep( self.users, function ( u ) {return u.isOvertimeEnabled() && u.userGroupId == groupId} ).length;
                                          } ), function ( group ) { return group.id; } )
    }
    if ( showTimeZoneGroups){
        selectedGroupIds = $.map( $.grep( this.timeZoneGroups,
                                          function ( e )
                                          {
                                              var groupId = e.id;
                                              return $.inArray( e.id, userSelection.selectedGroups ) != -1 &&
                                                     $.grep( self.users, function ( u ) {return u.isOvertimeEnabled() && u.timeZoneGroupId == groupId} ).length;
                                          } ), function ( group ) { return group.id; } )

    }
    if(!showTimeZoneGroups && !showUserGroups)
    {
        $.each(
            userSelection.selectedGroups,
            function( i, groupId ) {
                selectedUsersIds = $.merge(
                    selectedUsersIds,
                    $.map(
                        $.grep(
                            self.users,
                            function ( u ) {
                                return u.isOvertimeEnabled() && u[userSelection.groupType == GroupType.TIME_ZONE_GROUP ?"timeZoneGroupId" : "userGroupId"] == groupId && (u.active || userSelection.showDisabled)
                            }
                        ),
                        function( u ) {
                            return u.id;
                        }
                    )
                );
            }
        );
    }

    userSelection.selectedUsers = selectedUsersIds;
    userSelection.selectedGroups = selectedGroupIds;
    return userSelection;
};